Package: qtl
Version: 1.74
Date: 2025-12-08
Title: Tools for Analyzing QTL Experiments
Author: Karl W Broman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4914-6671>),
  Hao Wu [aut],
  Gary Churchill [ctb] (ORCID: <https://orcid.org/0000-0001-9190-9284>),
  Saunak Sen [ctb] (ORCID: <https://orcid.org/0000-0003-4519-6361>),
  Danny Arends [ctb] (ORCID: <https://orcid.org/0000-0001-8738-0162>),
  Robert Corty [ctb],
  Timothee Flutre [ctb],
  Ritsert Jansen [ctb],
  Pjotr Prins [ctb] (ORCID: <https://orcid.org/0000-0002-8021-9162>),
  Lars Ronnegard [ctb],
  Rohan Shah [ctb],
  Laura Shannon [ctb],
  Quoc Tran [ctb],
  Aaron Wolen [ctb],
  Brian Yandell [ctb] (ORCID: <https://orcid.org/0000-0002-8774-9377>),
  R Core Team [ctb]
Authors@R: c(person(given = c("Karl", "W"),
                    family = "Broman",
                    role = c("aut", "cre"),
                    email = "broman@wisc.edu",
                    comment=c(ORCID = "0000-0002-4914-6671")),
             person(given = "Hao",
                    family = "Wu",
                    role = "aut"),
             person(given = c("Gary"),
                    family = "Churchill",
                    role = "ctb",
                    comment=c(ORCID = "0000-0001-9190-9284")),
             person(given = "Saunak",
                    family = "Sen",
                    role = "ctb",
                    comment=c(ORCID = "0000-0003-4519-6361")),
             person(given = "Danny",
                    family = "Arends",
                    role = "ctb",
                    comment=c(ORCID = "0000-0001-8738-0162")),
             person(given = "Robert",
                    family = "Corty",
                    role = "ctb"),
             person(given = "Timothee",
                    family = "Flutre",
                    role = "ctb"),
             person(given = "Ritsert",
                    family = "Jansen",
                    role = "ctb"),
             person(given = "Pjotr",
                    family = "Prins",
                    role = "ctb",
                    comment=c(ORCID = "0000-0002-8021-9162")),
             person(given = "Lars",
                    family = "Ronnegard",
                    role = "ctb"),
             person(given = "Rohan",
                    family = "Shah",
                    role = "ctb"),
             person(given = "Laura",
                    family = "Shannon",
                    role = "ctb"),
             person(given = "Quoc",
                    family = "Tran",
                    role = "ctb"),
             person(given = "Aaron",
                    family = "Wolen",
                    role = "ctb"),
             person(given = "Brian",
                    family = "Yandell",
                    role = "ctb",
                    comment=c(ORCID = "0000-0002-8774-9377")),
             person("R Core Team",
                    role = "ctb"))
Maintainer: Karl W Broman <broman@wisc.edu>
Description: Analysis of experimental crosses to identify genes
  (called quantitative trait loci, QTLs) contributing to variation in
  quantitative traits.
  Broman et al. (2003) <doi:10.1093/bioinformatics/btg112>.
Copyright: Code for Brent's method for finding the root of a univariate
        function was taken from R 2.15.1 (Copyright 1999, 2001 The R
        Core Team)
Depends: R (>= 2.14.0)
Imports: parallel, graphics, stats, utils, grDevices
Suggests: testthat
License: GPL-3
URL: https://rqtl.org, https://github.com/kbroman/qtl
BugReports: https://github.com/kbroman/qtl/issues
Encoding: UTF-8
ByteCompile: true
NeedsCompilation: yes
Packaged: 2025-12-08 13:03:51 UTC; kbroman
Repository: CRAN
Date/Publication: 2025-12-08 17:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:14:05 UTC; unix
Archs: qtl.so.dSYM

useDynLib(qtl, .registration=TRUE)

# Import package dependencies
importFrom(graphics,par)
importFrom(graphics,abline)
importFrom(graphics,points)
importFrom(graphics,lines)
importFrom(graphics,text)
importFrom(graphics,mtext)
importFrom(graphics,axis)
importFrom(graphics,title)
importFrom(graphics,arrows)
importFrom(graphics,barplot)
importFrom(graphics,contour)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,persp)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,rug)
importFrom(graphics,segments)
importFrom(graphics,xspline)

importFrom(stats,Gamma)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,chisq.test)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,hatvalues)
importFrom(stats,heatmap)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats,weighted.mean)

importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(utils,packageVersion)
importFrom(utils,packageDescription)

importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(parallel,clusterApply)
importFrom(parallel,mclapply)
importFrom(parallel,clusterApplyLB)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,nextRNGStream)
importFrom(parallel,parLapply)

importFrom(grDevices,topo.colors)
importFrom(grDevices,terrain.colors)
importFrom(grDevices,heat.colors)
importFrom(grDevices,cm.colors)
importFrom(grDevices,gray)
importFrom(grDevices,rgb)
importFrom(grDevices,rainbow)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)

# export functions for users
export(find_large_intervals)
export(table2map)
export(flip.order)
export(map2table)
export(scanonevar)
export(scanonevar.meanperm)
export(scanonevar.varperm)
export(reduce2grid)
export(formMarkerCovar)
export(nqtl)
export(addmarker)
export(inferFounderHap)
export(pull.genoprob, pull.argmaxgeno, pull.draws)
export(nullmarkers)

export(scantwopermhk)

export(convert2bcsft)
export(add.cim.covar, add.threshold, addcovarint, addint)
export(addloctocross, addmarkerstointervalmap, addpair, addqtl)
export(addtoqtl, allchrsplits, argmax.geno, bayesint)
export(calc.errorlod, calc.genoprob, calc.penalties, calc.plod)
export(checkAlleles, checkcovar, chrlen, chrnames)
export(cim, cleanGeno, comparecrosses, comparegeno)
export(compareorder, convert2riself, convert2risib, convert2sa)
export(countXO, create.map, drop.dupmarkers, drop.markers)
export(drop.nullmarkers, dropfromqtl, droponemarker)
export(effectplot, effectscan, est.map, est.rf)
export(fill.geno, find.flanking, find.marker, find.markerindex)
export(find.markerpos, find.pheno, find.pseudomarker, find.pseudomarkerpos)
export(findDupMarkers, fitqtl, fitstahl, flipcross, formLinkageGroups)
export(genAllPartitions, geno.crosstab, geno.image, geno.table)
export(getgenonames, getid, getsex, groupclusteredheatmap)
export(imf.cf, imf.h, imf.k, imf.m, imf.stahl)
export(inferredpartitions, interpPositions, jittermap, locatemarker, locateXO)
export(lodint, makeqtl, markerlrt, markernames)
export(mf.cf, mf.h, mf.k, mf.m, mf.stahl)
export(movemarker, mqm_version, mqmaugment)
export(mqmautocofactors, mqmextractmarkers)
export(mqmfind.marker, mqmgetmodel, mqmpermutation)
export(mqmplot.circle, mqmplot.cistrans, mqmplot.clusteredheatmap, mqmplot.cofactors)
export(mqmplot.directedqtl, mqmplot.heatmap, mqmplot.multitrait, mqmplot.permutations)
export(mqmplot.singletrait, mqmprocesspermutation, mqmscan, mqmscanall)
export(mqmscanfdr, mqmsetcofactors, mqmtestnormal)
export(nchr, nind, nmar, nmissing, nphe, nqrank, ntyped, orderMarkers)
export(phenames, pickMarkerSubset, plotErrorlod, plotGeno, plotInfo, plot.map, plotMap)
export(plotMissing, plotPheno, plotPXG, plotRF, plotLodProfile)
export(plotModel, polyplot, pull.geno, pull.map, pull.markers, pull.pheno, pull.rf)
export(qtlversion, read.cross, readMWril, refineqtl, reorderqtl, replace.map)
export(replacemap, replaceqtl, rescalemap, ripple)
export(scanall, scanone, scanoneboot, scanPhyloQTL, scanqtl, scantwo)
export(shiftmap, sim.cross, sim.geno, sim.map, simFounderSnps, simPhyloQTL)
export(simulatemissingdata, stepwiseqtl, strip.partials, subrousummaryscantwo)
export(summaryMap, summary.map, switch.order, switchAlleles, top.errorlod)
export(totmar, transformPheno, tryallpositions, typingGap, write.cross, xaxisloc.scanone)
export(clean, condense, convert)
export(summaryScantwoOld)
export(reviseXdata)


# used by funqtl
export(checkStepwiseqtlStart)
export(checkformula)
export(countqtlterms)
export(deparseQTLformula)
export(fitqtlengine)
export(matchchr)
export(parseformula)
export(reviseqtlnuminformula)


# S3 methods
S3method("-", scanone)
S3method("-", scanoneperm)
S3method("-", scantwo)
S3method("-", scantwoperm)
S3method("+", scanone)
S3method("+", scanoneperm)
S3method("+", scantwo)
S3method("+", scantwoperm)
S3method("[", cross)
S3method("[", map)
S3method("[", scanoneperm)
S3method("[", scantwoperm)
S3method(c, cross)
S3method(c, scanone)
S3method(c, scanoneperm)
S3method(c, scantwo)
S3method(c, scantwoperm)
S3method(cbind, scanone)
S3method(cbind, scanoneperm)
S3method(cbind, scantwo)
S3method(cbind, scantwoperm)
S3method(clean, cross)
S3method(clean, scantwo)
S3method(condense, scantwo)
S3method(convert, scanone)
S3method(convert, scantwo)
S3method(convert, map)
S3method(max, scanone)
S3method(max, scanPhyloQTL)
S3method(max, scantwo)
S3method(max, scantwocondensed)
S3method(plot, cross)
S3method(plot, map)
S3method(plot, qtl)
S3method(plot, rfmatrix)
S3method(plot, scanone)
S3method(plot, scanoneboot)
S3method(plot, scanoneperm)
S3method(plot, scanPhyloQTL)
S3method(plot, scantwo)
S3method(plot, scantwoperm)
S3method(print, addcovarint)
S3method(print, addint)
S3method(print, compactqtl)
S3method(print, cross)
S3method(print, map)
S3method(print, qtl)
S3method(print, scanoneboot)
S3method(print, scantwo)
S3method(print, summary.addpair)
S3method(print, summary.compactqtl)
S3method(print, summary.cross)
S3method(print, summary.fitqtl)
S3method(print, summary.map)
S3method(print, summary.qtl)
S3method(print, summary.ripple)
S3method(print, summary.scanone)
S3method(print, summary.scanoneperm)
S3method(print, summary.scantwo)
S3method(print, summary.scantwo.old)
S3method(print, summary.scantwoperm)
S3method(rbind, scanoneperm)
S3method(rbind, scantwoperm)
S3method(replacemap, cross)
S3method(replacemap, scanone)
S3method(replacemap, scantwo)
S3method(subset, cross)
S3method(subset, map)
S3method(subset, scanone)
S3method(subset, scanoneperm)
S3method(subset, scantwo)
S3method(subset, scantwoperm)
S3method(summary, addcovarint)
S3method(summary, addint)
S3method(summary, compactqtl)
S3method(summary, cross)
S3method(summary, fitqtl)
S3method(summary, map)
S3method(summary, qtl)
S3method(summary, ripple)
S3method(summary, scanone)
S3method(summary, scanoneboot)
S3method(summary, scanoneperm)
S3method(summary, scanPhyloQTL)
S3method(summary, scantwo)
S3method(summary, scantwocondensed)
S3method(summary, scantwoperm)
S3method(plot, comparegeno)
S3method(summary, comparegeno)
S3method(print, summary.comparegeno)



# Hidden functions
#   asnumericwithdec
#   adjust.rf.ri
#   calc.genoprob.special
#   calc.pairprob
#   calcPermPval
#   calculatedensity
#   charround
#   checkdistances
#   checkformula
#   checkPhyloCrosses
#   checkPhyloPartition
#   checkStepwiseqtlStart
#   circlelocations
#   convertMWril
#   countqtlterms
#   deparseQTLformula
#   discan
#   drawcirculargenome
#   drawspline
#   dropfromqtlformula
#   dropXcol
#   effectplot.calmeanse
#   effectplot.getmark
#   estimatemarkerlod
#   expandf2covar
#   fitqtlengine
#   fitstahl.este
#   fitstahl.este.sub
#   fitstahl.estp
#   fitstahl.estp.sub
#   fitstahl.estpe
#   fitstahl.estpe.sub
#   fixX4write
#   fixXgeno.bc
#   fixXgeno.f2
#   getChr
#   getchromosomelength
#   getgenomelength
#   getThird
#   grab.arg.names
#   interpmap
#   interpmap4scantwo
#   genotab.em
#   LikePheVector
#   locationtocircle
#   loopthroughmulti
#   makeSSmap
#   markerforwsel
#   markerforwself2
#   markerloglik
#   matchchr
#   mqmaugment_on_cofactors
#   mqmextractpseudomarkers
#   orderMarkers.sub
#   ourline
#   ourstop
#   parseformula
#   mybinaryrep
#   plot.effectscan
#   printQTLformulanicely
#   qtlByPartition
#   qtlformulasymmetric
#   read.cro.qtlcart
#   read.cross.csv
#   read.cross.csvs
#   read.cross.gary
#   read.cross.karl
#   read.cross.mm
#   read.cross.qtlcart
#   read.cross.qtx
#   read.map.qtlcart
#   read.maps.mm
#   reorgRIargmax
#   reorgRIdraws
#   reorgRIgenoprob
#   reorgRIpairprob
#   revisecovar
#   reviseqtlnuminformula
#   revisescantwodf
#   reviseXdata
#   ripple.perm.sub
#   ripple.perm1
#   ripple.perm2
#   rippleSnowCountxo
#   rippleSnowLik
#   roundqtlpos
#   scanone.perm
#   scanone.perm.engine
#   scanoneXnull
#   scantwo.perm
#   scantwo.perm.engine
#   scantwoperm2scanoneperm
#   scoremissingmarkers
#   sim.cross.4way
#   sim.cross.bc
#   sim.cross.f2
#   sim.ril
#   snowCoreALL
#   snowCoreBOOT
#   sortPhyloPartitions
#   sim.bcg
#   stepsize
#   stringPhenoToInt
#   testchr
#   vbscan
#   write.cross.csv
#   write.cross.gary
#   write.cross.mm
#   write.cross.qtlcart

## R/qtl: A QTL mapping environment

[![R-CMD-check](https://github.com/kbroman/qtl/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kbroman/qtl/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/qtl)](https://cran.r-project.org/package=qtl)
[![zenodo DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.3688461.svg)](https://doi.org/10.5281/zenodo.3688461)

**Authors**: Karl W Broman and Hao Wu, with ideas from Gary Churchill
and &#346;aunak Sen and contributions from Danny Arends,
Robert Corty, Timoth&eacute;e Flutre, Ritsert Jansen, Pjotr Prins, Lars
R&ouml;nneg&aring;rd, Rohan Shah, Laura Shannon, Quoc Tran, Aaron
Wolen, and Brian Yandell

[R/qtl](https://rqtl.org) is an extensible, interactive environment for mapping
quantitative trait loci (QTL) in experimental crosses. It is
implemented as an add-on package for the freely available and widely
used statistical language/software [R](https://www.r-project.org).
The development of this software as an add-on to R allows us to take
advantage of the basic mathematical and statistical functions, and
powerful graphics capabilities, that are provided with R. Further, the
user will benefit by the seamless integration of the QTL mapping
software into a general statistical analysis program. Our goal is to
make complex QTL mapping methods widely accessible and allow users to
focus on modeling rather than computing.

A key component of computational methods for QTL mapping is the hidden
Markov model (HMM) technology for dealing with missing genotype
data. We have implemented the main HMM algorithms, with allowance for
the presence of genotyping errors, for backcrosses, intercrosses, and
phase-known four-way crosses.

The current version of R/qtl includes facilities for estimating
genetic maps, identifying genotyping errors, and performing single-QTL
genome scans and two-QTL, two-dimensional genome scans, by interval
mapping (with the EM algorithm), Haley-Knott regression, and multiple
imputation. All of this may be done in the presence of covariates
(such as sex, age or treatment). One may also fit higher-order QTL
models by multiple imputation and Haley-Knott regression.

### License

The R/qtl package is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License,
version 3, as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.  See the GNU
General Public License for more details.

A copy of the GNU General Public License, version 3, is available at
<https://www.r-project.org/Licenses/GPL-3>

Sample data for the R/qtl package

These files contain sample data in several formats, so that the user
may better understand how data may be formatted for import via the
read.cross function.  These are the same as the "listeria" data
set included with the R/qtl package.  

Note: Replace the "..." in the directory string to the appropriate
location of the sampledata directory (for example,
"/usr/local/lib/R/library/qtl" or "c:/R/rw1081/library/qtl").


1. "csv" format

    File:

        listeria.csv

    Data import:

        listeria.a <- read.cross("csv", ".../sampledata", "listeria.csv")

2. "csvr" format (rotated "csv")

   File:

        listeria_rot.csv

   Data import:

        listeria.a2 <- read.cross("csvr", ".../sampledata", "listeria_rot.csv")


3. "csvs" format (like "csv", but with separate files for phenotype
   and genotype data)

   Files:

        listeria_gen.csv         Genotype data
        listeria_phe.csv         Phenotype data

   Data import:

        listeria.a3 <- read.cross("csvs", ".../sampledata",
                                  "listeria_gen.csv", "listeria_phe.csv")


4. "csvsr" format (like "csvr", but both files are rotated)

   Files:

        listeria_gen_rot.csv     Genotype data
        listeria_phe_rot.csv     Phenotype data

   Data import:

        listeria.a4 <- read.cross("csvsr", ".../sampledata",
                                  "listeria_gen_rot.csv", 
                                  "listeria_phe_rot.csv")


5. "mm" (mapmaker) format

   Files:

       listeria_raw.txt   "raw" file (with genotype and phenotype data)
       listeria_map.txt    Genetic map information (markers must be in
                           order; map positions are not required)
       listeria_maps.txt   Genetic map information, as produced by 
                           Mapmaker/exp

   Data import:

       listeria.b <- read.cross("mm", ".../sampledata",
                                "listeria_raw.txt",,"listeria_map.txt")

       listeria.bb <- read.cross("mm", ".../sampledata",
                                 "listeria_raw.txt",,"listeria_maps.txt")


6. "qtx" (Mapmanager QTX) format

   File:

       listeria.qtx

   Data import:

       listeria.c <- read.cross("qtx", ".../sampledata", "listeria.qtx")


7. "qtlcart" (QTL Cartographer) format

   Files:

        listeria_qc_cro.txt    Genotype/phenotype data
        listeria_qc_map.txt    Genetic map information

   Data import:

        listeria.d <- read.cross("qtlcart", ".../sampledata", 
                                 "listeria_qc_cro.txt", "listeria_qc_map.txt")


8. "karl" format

    Files:

        gen.txt    Genotype data
        phe.txt    Phenotype data
        map.txt    Genetic map information

    Data import:

        listeria.e <- read.cross("karl", ".../sampledata",
                                 genfile="gen.txt", phefile="phe.txt",
                                 mapfile="map.txt")

    or just the following:  

        listeria.e <- read.cross("karl", ".../sampledata")

This directory contains standalone binaries of R/qtl (currently MQM
implentations).

run:

  cd ./contrib/bin
  cmake .
	make 
	make test   
  (the binary is 'mqm').

(you will need the standard GNU gcc set, as well as cmake)

To run regression testing from the default R interpreter:

  rm CMakeCache.txt ; cmake -DTEST_R=TRUE 
  make testR

To test all

  make testall

To run a single test you can use regular expressions (man ctest), e.g.

  ctest -R TestMQMF2_test0 

Both build and install the 'R/qtl' package in its default location before
running the test scripts. You need both R and lapack, on Debian/Ubuntu:

  apt-get install r-base r-base-dev liblapack-dev

This directory contains information for creating a shared library that can be
linked against C and SWIG mappings for Perl, Ruby etc.

See the BioLib project at http://biolib.open-bio.org/ for more information.

######################################################################
#
# Regression test
#
# copyright (c) 2009 Pjotr Prins
# first written July 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Some basic regression/integration testing for some of the QTL mapping routines
#
# You can run it with:
#
#   R --no-save --no-restore --no-readline --slave < ./tests/test_qtl.R 

######################################################################

script='mqm_listeria1'

library(qtl)

data(listeria)

augmentedcross <- mqmaugment(listeria, minprob=1.0)
result <- mqmscan(augmentedcross)
sink(paste('regression/',script,'.rnew',sep=''))
result
sink()

cat(script,'successful')

##############################################################
# R code for "A shorter tour of R/qtl"
#
# Karl W Broman, broman@wisc.edu
# University of Wisconsin Madison
#
# https://rqtl.org
#
# 26 November 2012
##############################################################

############################################################
# Preliminaries
############################################################
library(qtl)

ls()

help(read.cross)
?read.cross

############################################################
# Data import
############################################################
sug <- read.cross("csv", "https://rqtl.org", "sug.csv",
                  genotypes=c("CC", "CB", "BB"), alleles=c("C", "B"))

############################################################
# Summaries
############################################################
summary(sug)

nind(sug)
nchr(sug)
totmar(sug)
nmar(sug)
nphe(sug)

plot(sug)

plotMissing(sug)
plotMap(sug)
plotPheno(sug, pheno.col=1)
plotPheno(sug, pheno.col=2)
plotPheno(sug, pheno.col=3)
plotPheno(sug, pheno.col=4)
plotPheno(sug, pheno.col=5)
plotPheno(sug, pheno.col=6)

############################################################
# Single-QTL analysis
############################################################
sug <- calc.genoprob(sug, step=1)

out.em <- scanone(sug)

summary(out.em)

summary(out.em, threshold=3)

plot(out.em)

out.hk <- scanone(sug, method="hk")

plot(out.em, out.hk, col=c("blue", "red"))

plot(out.em, col="blue")
plot(out.hk, col="red", add=TRUE)

plot(out.hk - out.em, ylim=c(-0.3, 0.3), ylab="LOD(HK)-LOD(EM)")

sug <- sim.geno(sug, step=1, n.draws=64)
out.imp <- scanone(sug, method="imp")

plot(out.em, out.hk, out.imp, col=c("blue", "red", "green"))

plot(out.em, out.hk, out.imp, col=c("blue", "red", "green"), chr=c(7,15))

plot(out.imp - out.em, out.hk - out.em, col=c("green", "red"), ylim=c(-1,1))

############################################################
# Permutation tests
############################################################
load(url("https://rqtl.org/various.RData"))

operm <- scanone(sug, method="hk", n.perm=1000)

plot(operm)

summary(operm)
summary(operm, alpha=c(0.05, 0.2))

summary(out.hk, perms=operm, alpha=0.2, pvalues=TRUE)

############################################################
# Interval estimates of QTL location
############################################################
lodint(out.hk, chr=7)
bayesint(out.hk, chr=7)

lodint(out.hk, chr=7, expandtomarkers=TRUE)
bayesint(out.hk, chr=7, expandtomarkers=TRUE)

lodint(out.hk, chr=7, drop=2)
bayesint(out.hk, chr=7, prob=0.99)

lodint(out.hk, chr=15)
bayesint(out.hk, chr=15)

############################################################
# QTL effects
############################################################
max(out.hk)
mar <- find.marker(sug, chr=7, pos=47.7)
plotPXG(sug, marker=mar)

effectplot(sug, mname1=mar)

effectplot(sug, mname1="7@47.7")

max(out.hk, chr=15)
mar2 <- find.marker(sug, chr=15, pos=12)
plotPXG(sug, marker=mar2)
effectplot(sug, mname1="15@12")

plotPXG(sug, marker=c(mar, mar2))
plotPXG(sug, marker=c(mar2, mar))

effectplot(sug, mname1="7@47.7", mname2="15@12")
effectplot(sug, mname2="7@47.7", mname1="15@12")

############################################################
# Other phenotypes
############################################################
out.hr <- scanone(sug, pheno.col=2, method="hk")

out.bw <- scanone(sug, pheno.col="bw", method="hk")

out.logbw <- scanone(sug, pheno.col=log(sug$pheno$bw), method="hk")

out.all <- scanone(sug, pheno.col=1:4, method="hk")

summary(out.all, threshold=3)

summary(out.all, threshold=3, lodcolumn=4)

summary(out.all, threshold=3, format="allpeaks")

summary(out.all, threshold=3, format="allpheno")

summary(out.all, threshold=3, format="tabByCol")
summary(out.all, threshold=3, format="tabByChr")

############################################################
# Two-dimensional, two-QTL scans
############################################################
sug <- calc.genoprob(sug, step=2)

out2 <- scantwo(sug, method="hk")

plot(out2)

plot(out2, lower="fv1")

plot(out2, lower="fv1", upper="av1")

operm2 <- scantwo(sug, method="hk", n.perm=5)

summary(out2, perms=operm2, alpha=0.2, pvalues=TRUE)

############################################################
# Multiple-QTL analyses
############################################################
sug <- calc.genoprob(sug, step=1)

qtl <- makeqtl(sug, chr=c(7,15), pos=c(47.7, 12), what="prob")

out.fq <- fitqtl(sug, qtl=qtl, method="hk")
summary(out.fq)

summary(fitqtl(sug, qtl=qtl, method="hk", get.ests=TRUE, dropone=FALSE))

out.fqi <- fitqtl(sug, qtl=qtl, method="hk", formula=y~Q1*Q2)
out.fqi <- fitqtl(sug, qtl=qtl, method="hk", formula=y~Q1+Q2+Q1:Q2)
summary(out.fqi)

addint(sug, qtl=qtl, method="hk")

rqtl <- refineqtl(sug, qtl=qtl, method="hk")
rqtl

summary(out.fqr <- fitqtl(sug, qtl=rqtl, method="hk"))

plotLodProfile(rqtl)

plot(out.hk, chr=c(7,15), col="red", add=TRUE)

out.aq <- addqtl(sug, qtl=rqtl, method="hk")

plot(out.aq)

print(pen <- calc.penalties(operm2))

out.sq <- stepwiseqtl(sug, max.qtl=5, penalties=pen, method="hk", verbose=2)
out.sq

# end of rqtltour2.R

##############################################################
# R code for "A brief tour of R/qtl"
#
# Karl W Broman, broman@wisc.edu
# University of Wisconsin Madison
#
# https://rqtl.org
#
# 21 March 2012
##############################################################

save.image()

library(qtl)

ls()

help(read.cross)
?read.cross

############################################################
# Example 1: Hypertension
############################################################
data(hyper)
ls()
?hyper

summary(hyper)

nind(hyper)
nphe(hyper)
nchr(hyper)
totmar(hyper)
nmar(hyper)

plot(hyper)

plotMissing(hyper)
plotMap(hyper)
plotPheno(hyper, pheno.col=1)

plotMap(hyper, chr=c(1, 4, 6, 7, 15), show.marker.names=TRUE)

plotMissing(hyper, reorder=TRUE)

hyper <- drop.nullmarkers(hyper)
totmar(hyper)

hyper <- est.rf(hyper)
plotRF(hyper)
plotRF(hyper, chr=c(1,4))

plotRF(hyper, chr=6)
plotMissing(hyper, chr=6)

newmap <- est.map(hyper, error.prob=0.01)
plotMap(hyper, newmap)

hyper <- replace.map(hyper, newmap)

hyper <- calc.errorlod(hyper, error.prob=0.01)

top.errorlod(hyper)

plotGeno(hyper, chr=16, ind=c(24:34, 71:81))

plotInfo(hyper)
plotInfo(hyper, chr=c(1,4,15))
plotInfo(hyper, chr=c(1,4,15), method="entropy")
plotInfo(hyper, chr=c(1,4,15), method="variance")

hyper <- calc.genoprob(hyper, step=1, error.prob=0.01)

out.em <- scanone(hyper)
out.hk <- scanone(hyper, method="hk")

hyper <- sim.geno(hyper, step=2, n.draws=16, error.prob=0.01)
out.imp <- scanone(hyper, method="imp")

summary(out.em)
summary(out.em, threshold=3)
summary(out.hk, threshold=3)
summary(out.imp, threshold=3)

max(out.em)
max(out.hk)
max(out.imp)

plot(out.em, chr=c(1,4,15))
plot(out.em, out.hk, out.imp, chr=c(1,4,15))
plot(out.em, chr=c(1,4,15))
plot(out.hk, chr=c(1,4,15), col="blue", add=TRUE)
plot(out.imp, chr=c(1,4,15), col="red", add=TRUE)

operm.hk <- scanone(hyper, method="hk", n.perm=1000)

summary(operm.hk, alpha=0.05)

summary(out.hk, perms=operm.hk, alpha=0.05, pvalues=TRUE)

save.image()

hyper <- calc.genoprob(hyper, step=5, error.prob=0.01)

out2.hk <- scantwo(hyper, method="hk")

summary(out2.hk, thresholds=c(6.0, 4.7, 4.4, 4.7, 2.6))

summary(out2.hk, thresholds=c(6.0, 4.7, Inf, 4.7, 2.6))

plot(out2.hk)
plot(out2.hk, chr=c(1,4,6,15))

max(out2.hk)

operm2.hk <- scantwo(hyper, method="hk", n.perm=100)

summary(operm2.hk)

summary(out2.hk, perms=operm2.hk, pvalues=TRUE,
        alphas=c(0.05, 0.05, 0, 0.05, 0.05))

chr <- c(1, 1, 4, 6, 15)
pos <- c(50, 76, 30, 70, 20)
qtl <- makeqtl(hyper, chr, pos)

my.formula <- y ~ Q1 + Q2 + Q3 + Q4 + Q5 + Q4:Q5
out.fitqtl <- fitqtl(hyper, qtl=qtl, formula=my.formula)
summary(out.fitqtl)

ls()
rm(list=ls())

############################################################
# Example 2: Genetic mapping
############################################################
data(badorder)
summary(badorder)
plot(badorder)

badorder <- est.rf(badorder)
plotRF(badorder)

plotRF(badorder, chr=1)

newmap <- est.map(badorder, verbose=TRUE)
plotMap(badorder, newmap)

plotRF(badorder, chr=2:3)

pull.map(badorder, chr=2)
pull.map(badorder, chr=3)

badorder <- movemarker(badorder, "D2M937", 3, 48)
badorder <- movemarker(badorder, "D3M160", 2, 28.8)

plotRF(badorder, chr=2:3)

rip1 <- ripple(badorder, chr=1, window=6)
summary(rip1)

rip2 <- ripple(badorder, chr=1, window=3, err=0.01, method="likelihood")
summary(rip2)

badorder.rev <- switch.order(badorder, 1, rip1[2,])
rip1r <- ripple(badorder.rev, chr=1, window=6)
summary(rip1r)

badorder.rev <- switch.order(badorder.rev, 1, rip1r[2,])
rip2r <- ripple(badorder.rev, chr=1, window=3, err=0.01)
summary(rip2r)

badorder.rev <- est.rf(badorder.rev)
plotRF(badorder.rev, 1)

############################################################
# Example 3: Listeria susceptibility
############################################################
data(listeria)
summary(listeria)
plot(listeria)
plotMissing(listeria)

listeria$pheno$logSurv <- log(listeria$pheno[,1])
plot(listeria)

listeria <- est.rf(listeria)
plotRF(listeria)
plotRF(listeria, chr=c(5,13))

newmap <- est.map(listeria, error.prob=0.01)
plotMap(listeria, newmap)
listeria <- replace.map(listeria, newmap)

listeria <- calc.errorlod(listeria, error.prob=0.01)
top.errorlod(listeria)
top.errorlod(listeria, cutoff=3.5)
plotGeno(listeria, chr=13, ind=61:70, cutoff=3.5)

listeria <- calc.genoprob(listeria, step=2)
out.2p <- scanone(listeria, pheno.col=3, model="2part", upper=TRUE)

summary(out.2p)
summary(out.2p, threshold=4.5)

summary(out.2p, format="allpeaks", threshold=3)
summary(out.2p, format="allpeaks", threshold=c(4.5,3,3))

plot(out.2p)
plot(out.2p, lodcolumn=2)
plot(out.2p, lodcolumn=1:3, chr=c(1,5,13,15))

operm.2p <- scanone(listeria, model="2part", pheno.col=3,
                    upper=TRUE, n.perm=25)
summary(operm.2p, alpha=0.05)

summary(out.2p, format="allpeaks", perms=operm.2p,
        alpha=0.05, pvalues=TRUE)

y <- listeria$pheno$logSurv
my <- max(y, na.rm=TRUE)
z <- as.numeric(y==my)
y[y==my] <- NA
listeria$pheno$logSurv2 <- y
listeria$pheno$binary <- z
plot(listeria)

out.mu <- scanone(listeria, pheno.col=4)
plot(out.mu, out.2p, lodcolumn=c(1,3), chr=c(1,5,13,15), col=c("blue","red"))

out.p <- scanone(listeria, pheno.col=5, model="binary")
plot(out.p, out.2p, lodcolumn=c(1,2), chr=c(1,5,13,15), col=c("blue","red"))

out.p.alt <- scanone(listeria, pheno.col=as.numeric(listeria$pheno$T264==264),
                     model="binary")

out.np1 <- scanone(listeria, model="np", ties.random=TRUE)
out.np2 <- scanone(listeria, model="np", ties.random=FALSE)

plot(out.np1, out.np2, col=c("blue","red"))
plot(out.2p, out.np1, out.np2, chr=c(1,5,13,15))

############################################################
# Example 4: Covariates in QTL mapping
############################################################
data(fake.bc)
summary(fake.bc)
plot(fake.bc)

fake.bc <- calc.genoprob(fake.bc, step=2.5)
out.nocovar <- scanone(fake.bc, pheno.col=1:2)

sex <- fake.bc$pheno$sex
out.acovar <- scanone(fake.bc, pheno.col=1:2, addcovar=sex)

summary(out.nocovar, threshold=3, format="allpeaks")
summary(out.acovar, threshold=3, format="allpeaks")

plot(out.nocovar, out.acovar, chr=c(2, 5))
plot(out.nocovar, out.acovar, chr=c(2, 5), lodcolumn=2)

out.icovar <- scanone(fake.bc, pheno.col=1:2, addcovar=sex, intcovar=sex)

summary(out.icovar, threshold=3, format="allpeaks")

plot(out.acovar, out.icovar, chr=c(2,5), col=c("blue", "red"))
plot(out.acovar, out.icovar, chr=c(2,5), lodcolumn=2,
     col=c("blue", "red"))

out.sexint <- out.icovar - out.acovar
plot(out.sexint, lodcolumn=1:2, chr=c(2,5), col=c("green", "purple"))

seed <- ceiling(runif(1, 0, 10^8))
set.seed(seed)
operm.acovar <- scanone(fake.bc, pheno.col=1:2, addcovar=sex,
                        method="hk", n.perm=100)
set.seed(seed)
operm.icovar <- scanone(fake.bc, pheno.col=1:2, addcovar=sex,
                        intcovar=sex, method="hk", n.perm=100)

operm.sexint <- operm.icovar - operm.acovar

summary(operm.sexint, alpha=c(0.05, 0.20))

summary(out.sexint, perms=operm.sexint, alpha=0.1,
        format="allpeaks", pvalues=TRUE)

############################################################
# Example 5: Multiple QTL mapping
############################################################
rm(list=ls())
data(hyper)

hyper <- sim.geno(hyper, step=2.5, n.draws=16, err=0.01)

out1 <- scanone(hyper, method="imp")
plot(out1)

max(out1)

find.marker(hyper, 4, 29.5)

g <- pull.geno(hyper)[,"D4Mit164"]
mean(is.na(g))

g <- pull.geno(fill.geno(hyper))[,"D4Mit164"]

out1.c4 <- scanone(hyper, method="imp", addcovar=g)

plot(out1, out1.c4, col=c("blue", "red"))

plot(out1.c4 - out1, ylim=c(-3,3))
abline(h=0, lty=2, col="gray")

out1.c4i <- scanone(hyper, method="imp", addcovar=g, intcovar=g)

plot(out1.c4i - out1.c4)

out2 <- scantwo(hyper, method="imp")

summary(out2, thr=c(6.0, 4.7, Inf, 4.7, 2.6))

summary( subset(out2, chr=1) )

summary( subset(out2, chr=c(7,15)) )

plot(out2, chr=c(1,4,6,7,15))

plot(out2, chr=1, lower="cond-add")
plot(out2, chr=c(6,15), lower="cond-int")
plot(out2, chr=c(7,15), lower="cond-int")

out2.c4 <- scantwo(hyper, method="imp", addcovar=g, chr=c(1,6,7,15))

summary(out2.c4, thr=c(6.0, 4.7, Inf, 4.7, 2.6))
summary( subset(out2.c4, chr=1) )
summary( subset(out2.c4, chr=c(7,15)) )

plot(out2.c4)
plot(out2.c4, chr=1, lower="cond-int")
plot(out2.c4, chr=c(6,15), lower="cond-int")
plot(out2.c4, chr=c(7,15), lower="cond-int")

out2sub <- subset(out2, chr=c(1,6,7,15))
plot(out2.c4 - out2sub, allow.neg=TRUE, lower="cond-int")

qc <- c(1, 1, 4, 6, 15)
qp <- c(43.3, 78.3, 30.0, 62.5, 18.0)
qtl <- makeqtl(hyper, chr=qc, pos=qp)

myformula <- y ~ Q1+Q2+Q3+Q4+Q5 + Q4:Q5

out.fq <- fitqtl(hyper, qtl=qtl, formula = myformula)
summary(out.fq)

out.fq <- fitqtl(hyper, qtl=qtl, formula = myformula, drop=FALSE, get.ests=TRUE)
summary(out.fq)

revqtl <- refineqtl(hyper, qtl=qtl, formula = myformula)

revqtl

plot(revqtl)

out.fq2 <- fitqtl(hyper, qtl=revqtl, formula=myformula)
summary(out.fq2)

out1.c4r <- addqtl(hyper, qtl=revqtl, formula=y~Q3)

plot(out1.c4, out1.c4r, col=c("blue", "red"))

plot(out1.c4r - out1.c4, ylim=c(-1.7, 1.7))
abline(h=0, lty=2, col="gray")

out2.c4r <- addpair(hyper, qtl=revqtl, formula=y~Q3, chr=c(1,6,7,15))

plot(out2.c4r - out2.c4, lower="cond-int", allow.neg=TRUE)

out.1more <- addqtl(hyper, qtl=revqtl, formula=myformula)
plot(out.1more)

out.iw4 <- addqtl(hyper, qtl=revqtl, formula=y~Q1+Q2+Q3+Q4+Q5+Q4:Q5+Q6+Q5:Q6)
plot(out.iw4)

out.2more <- addpair(hyper, qtl=revqtl, formula=myformula, chr=c(2,5,7,15))

plot(out.2more, lower="cond-int")

out.ai <- addint(hyper, qtl=revqtl, formula=myformula)
out.ai

qtl2 <- addtoqtl(hyper, revqtl, 7, 53.6)
qtl2

qtl3 <- dropfromqtl(qtl2, index=2)
qtl3

qtl4 <- replaceqtl(hyper, qtl3, index=1, chr=1, pos=50)
qtl4

qtl5 <- reorderqtl(qtl4, c(1:3,5,4))
qtl5

stepout.a <- stepwiseqtl(hyper, additive.only=TRUE, max.qtl=6)
stepout.a

stepout.i <- stepwiseqtl(hyper, max.qtl=6)
stepout.i

############################################################
# Example 6: Internal data structure
############################################################
data(fake.bc)

class(fake.bc)

names(fake.bc)

fake.bc$pheno[1:10,]

names(fake.bc$geno)
sapply(fake.bc$geno, class)

names(fake.bc$geno[[3]])
fake.bc$geno[[3]]$data[1:5,]
fake.bc$geno[[3]]$map

names(fake.bc$geno[[3]])
fake.bc <- calc.genoprob(fake.bc, step=10, err=0.01)
names(fake.bc$geno[[3]])
fake.bc <- sim.geno(fake.bc, step=10, n.draws=8, err=0.01)
names(fake.bc$geno[[3]])
fake.bc <- argmax.geno(fake.bc, step=10, err=0.01)
names(fake.bc$geno[[3]])
fake.bc <- calc.errorlod(fake.bc, err=0.01)
names(fake.bc$geno[[3]])

names(fake.bc)
fake.bc <- est.rf(fake.bc)
names(fake.bc)

# end of rqtltour.R

library(qtl)

set.seed(1000)              

version = mqm_version()
cat("R/qtl=",version$RQTL)
cat("R-MQM=",version$RMQM)
cat("MQM=",version$MQM)

testaugmentation <- function(cross, ...){
  crossML <- mqmaugment(cross, ...)
  crossIMP <- mqmaugment(cross,strategy="impute", ...)
  crossDROP <- mqmaugment(cross,strategy="drop", ...)

  res1 <- mqmscan(crossML)
  res2 <- mqmscan(crossIMP)
  res3 <- mqmscan(crossDROP)

  plot(res1,res2,res3,lty=c(1,2,3),col=c("black","blue","red"))
  legend("topleft",c("MinProb","Imputation","Drop"),lty=c(1,2,3),col=c("black","blue","red"))
  list(res1,res2,res3)
}

stabilitytest <- function(cross, ...){
  crossML1 <- mqmaugment(cross, minprob=0.5,maxaugind=16,...)
  crossML2 <- mqmaugment(cross, minprob=0.05,maxaugind=16,...)
  crossML3 <- mqmaugment(cross, minprob=0.005,maxaugind=16,...)
  res1 <- mqmscan(crossML1)
  res2 <- mqmscan(crossML2)
  res3 <- mqmscan(crossML3)

  plot(res1,res2,res3,lty=c(1,2,3),col=c("black","blue","red"))
  legend("topleft",c("minprob 0.5","minprob 0.05","minprob 0.005"),lty=c(1,2,3),col=c("black","blue","red"))
  list(res1,res2,res3)
}

data(multitrait)
multimissing <- simulatemissingdata(multitrait,25)
#r <- stabilitytest(multimissing,verbose=TRUE)
r <- testaugmentation(multimissing)
if(!round(r[[1]][3,3],3)==0.764) stop("Multitrait ML dataaugmentation error")
if(!round(r[[1]][3,3],3)==round(r[[2]][3,3],3)) stop("Multitrait ML compared versus IMP error")
if(!round(r[[3]][3,3],3)==0.844) stop("Multitrait DROP dataaugmentation error")

data(hyper)
r <- testaugmentation(hyper,maxaugind=32,minprob=0.75)
if(!round(r[[1]][3,3],3)==0.568) stop("Hyper ML dataaugmentation error")
if(!round(r[[1]][3,3],3)==round(r[[2]][3,3],3)) stop("Hyper ML compared versus IMP error")
if(!round(r[[3]][3,3],3)==0.252) stop("Hyper DROP dataaugmentation error")


data(listeria)
r <- testaugmentation(listeria)
if(!round(r[[1]][3,3],3)==0.352) stop("Listeria ML dataaugmentation error")
if(!round(r[[1]][3,3],3)==round(r[[2]][3,3],3)) stop("Listeria ML compared versus IMP error")
if(!round(r[[3]][3,3],3)==0.034) stop("Listeria DROP dataaugmentation error")


cat("testaugmentation.R, tests succesfully run!")

# find large intervals in a map
#
#
# input: map = list of chromosomes that are vectors of marker positions
#              [can also be a cross object, in which case pull.map() is used]
#        min_length = minimum distance between markers to be flagged
#
# example use:
#   library(qtl)
#   data(hyper)
#   find_large_intervals(hyper, 20)

find_large_intervals <-
    function(map, min_length=35)
{
    output <- NULL

    # if input is a cross, pull out the map
    if("cross" %in% class(map)) {
        map <- pull.map(map)
    }

    # make sure there are names
    if(is.null(names(map))) {
        names(map) <- seq_along(map)
    }

    for(i in names(map)) {

        d <- diff(map[[i]])
        big <- which(d > min_length)
        n_big <- length(big)
        mar <- names(map[[i]])

        if(n_big > 0) {
            this <- data.frame(chr=rep(i, n_big),
                               left=mar[big],
                               right=mar[big+1],
                               interval_size=d[big],
                               stringsAsFactors=FALSE)

            if(is.null(output)) output <- this
            else output <- rbind(output, this)
        }

    }

    if(!is.null(output)) rownames(output) <- 1:nrow(output)
    output
}

######################################################################
# stepwiseqtlX.R
#
# copyright (c) 2013-2021, Karl W Broman and Quoc Tran
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
#
# Functions related to stepwise QTL analysis but with separate
# penalties for autosome and X chromosome
#
# Contains: countqtltermsX, calc.penalties.X, stepwiseqtlX, calc.plod.X
######################################################################

######################################################################
# count terms in a model, for use by plod, Quoc editted
######################################################################
countqtltermsX <-
    function(formula, qtl, ignore.covar=TRUE) # qtl is used to extract a logical vector: on "X" chr == 1
{
    if(is.character(formula)) formula <- as.formula(formula)
    factors <- attr(terms(formula), "factors")[-1,,drop=FALSE]
    if(any(factors > 1))  {
        warning("some formula terms > 1; may be a problem with the formula:\n    ", deparseQTLformula(formula))
        factors[factors > 1] <- 1
    }
    nterm <- apply(factors, 2, sum)

    if(any(nterm>2))
        stop("Can't deal with higher-order interactions\n")

    # need to check for QTL x covariate interactions in here!

    if(ignore.covar) {
        cn <- colnames(factors)
        wh <- c(grep("^[Qq][0-9]+$", cn),
                grep("^[Qq][0-9]+:[Qq][0-9]+$", cn))
        rn <- rownames(factors)
        wh2 <- c(grep("^[Qq][0-9]+$", rn),
                 grep("^[Qq][0-9]+:[Qq][0-9]+$", rn))
        factors <- factors[wh2,wh, drop=FALSE]
    }
    nterm <- apply(factors, 2, sum)

    nmain <- sum(nterm==1)

    qtltype <- qtl$chrtype
    names(qtltype) <- qtl$altname
    factors.main <- factors[,nterm==1, drop=FALSE] # Get first order factors
    nmainA <- sum(qtltype[colnames(factors.main)]=="A")
    nmainX <- sum(qtltype[colnames(factors.main)]=="X")


    if(all(nterm==1))
        return(c(mainA=nmainA, mainX=nmainX, intH=0, intL=0, intAX=0, intXX=0, inttot=0))

    n.int <- sum(nterm==2)

    if(n.int <=0) # 0 interactions, so no need to figure them out
        return(c(mainA=nmainA, mainX=nmainX, intH=0, intL=n.int, intAX=0, intXX=0, inttot=n.int))

    factors <- factors[, nterm==2, drop=FALSE] # Get second order factors

    wh <- apply(factors, 2, function(a) which(a==1)) # Get the location of interaction, check for X here.

    if(n.int ==1){ # 1 interaction, check if it is AA AX or XX
        int.type <- sum(qtltype[rownames(factors)[wh[,1]]]=="X") #0 is AA, 1 is AX, 2 is XX
        if (int.type == 0) return(c(mainA=nmainA, mainX=nmainX, intH=0, intL=1, intAX=0, intXX=0, inttot=n.int))
        else
            if (int.type == 1) return(c(mainA=nmainA, mainX=nmainX, intH=0, intL=0, intAX=1, intXX=0, inttot=n.int))
            else return(c(mainA=nmainA, mainX=nmainX, intH=0, intL=0, intAX=0, intXX=1, inttot=n.int))
    }

    u <- sort(unique(as.numeric(wh))) # Unique nodes with interactions, in increasing order; does not contain isolated nodes
    grp <- rep(NA, length(u)) # Group member, length = number of node
    names(grp) <- u

    ngrp <- 0 # Number of group
    nintAA <- NULL # Number of AA interaction of specific group
    nintAX <- NULL # Number of AX interaction of specific group
    nintXX <- NULL # Number of XX interaction of specific group


    for(i in 1:ncol(wh)) {
        thegrp <- grp[as.character(wh[,i])]
        int.type <- sum(qtltype[rownames(factors)[wh[,i]]]=="X") #0 is AA, 1 is AX, 2 is XX
        if(all(!is.na(thegrp))) { # Merge 2 groups
            nintAA[as.character(thegrp[1])] <-
                sum(nintAA[unique(as.character(thegrp))])
            nintAX[as.character(thegrp[1])] <-
                sum(nintAX[unique(as.character(thegrp))])
            nintXX[as.character(thegrp[1])] <-
                sum(nintXX[unique(as.character(thegrp))])
            if (int.type==0) nintAA[as.character(thegrp[1])] <- nintAA[as.character(thegrp[1])] + 1
            if (int.type==1) nintAX[as.character(thegrp[1])] <- nintAX[as.character(thegrp[1])] + 1
            if (int.type==2) nintXX[as.character(thegrp[1])] <- nintXX[as.character(thegrp[1])] + 1
            grp[grp==thegrp[1] | grp==thegrp[2]] <- thegrp[1]
        } # two connected group become one group and has the name of the first group, number of interaction is updated
        else if(any(!is.na(thegrp))) { # add 1 more interaction to current group
            grp[as.character(wh[,i])] <- thegrp[!is.na(thegrp)]
            if (int.type==0) nintAA[as.character(thegrp[!is.na(thegrp)])] <-
                nintAA[as.character(thegrp[!is.na(thegrp)])] + 1
            if (int.type==1) nintAX[as.character(thegrp[!is.na(thegrp)])] <-
                nintAX[as.character(thegrp[!is.na(thegrp)])] + 1
            if (int.type==2) nintXX[as.character(thegrp[!is.na(thegrp)])] <-
                nintXX[as.character(thegrp[!is.na(thegrp)])] + 1
        }
        else { # introduce new group
            ngrp <- ngrp+1
            grp[as.character(wh[,i])] <- ngrp
            # Initialize nint
            nintAA[as.character(ngrp)] <- 0
            nintAX[as.character(ngrp)] <- 0
            nintXX[as.character(ngrp)] <- 0
            if (int.type==0) nintAA[as.character(ngrp)] <- 1
            if (int.type==1) nintAX[as.character(ngrp)] <- 1
            if (int.type==2) nintXX[as.character(ngrp)] <- 1
        }
    }

    nintAA <- nintAA[as.character(unique(grp))]
    nintAX <- nintAX[as.character(unique(grp))]
    nintXX <- nintXX[as.character(unique(grp))]
    nL <- sum(nintAA>0)
    nH <- sum(nintAA)-nL
    c(mainA=nmainA, mainX=nmainX, intH=nH, intL=nL,
      intAX=sum(nintAX), intXX=sum(nintXX),
      inttot=nH+nL+sum(nintAX)+sum(nintXX))
}

######################################################################
# calculate penalties for pLOD using scantwo permutation results,  Quoc editted
######################################################################
calc.penalties.X <-
    function(perms, alpha=0.05, lodcolumn)
{
    if(missing(perms) || !("scantwoperm" %in% class(perms)))
        stop("You must include permutation results from scantwo.")

    if(!("AA" %in% names(perms)))
        stop("perms need to be X-chr-specific")

    if(length(alpha) > 1) {
        alpha <- alpha[1]
        warning("alpha needs to be a single value; only the first will be used.")
    }

    if(missing(lodcolumn) || is.null(lodcolumn)) {
        if(is.matrix(perms[[1]][[1]]) && ncol(perms[[1]][[1]]) > 1)
            lodcolumn <- 1:ncol(perms[[1]][[1]])
        else lodcolumn <- 1
    }

    if(length(lodcolumn)>1) {
        result <- NULL
        for(i in seq(along=lodcolumn)) {
            temp <- calc.penalties.X(perms, alpha, lodcolumn[i])
            result <- rbind(result, temp)
        }
        dimnames(result) <- list(colnames(perms[[1]][[1]])[lodcolumn], names(temp))
        return(result)
    }

    if(is.matrix(perms[[1]][[1]]) && ncol(perms[[1]][[1]]) >1) {
        if(lodcolumn < 1 || lodcolumn > ncol(perms[[1]][[1]]))
            stop("lodcolumn misspecified")
        for(j in 1:3) {
            for(i in seq(along=perms[[j]]))
                perms[[j]][[i]] <- perms[[j]][[i]][,lodcolumn,drop=FALSE]
        }
    }

    qu <- summary(perms, alpha=alpha)

    c(mainA=qu$AA$one,
      mainX=qu$XX$one,
      intH=qu$AA$int,
      intL=qu$AA$fv1 - qu$AA$one,
      intAX=qu$AX$int,
      intXX=qu$XX$int)
}

######################################################################
# stepwiseqtlX, work for data with both A and X chromosomes,
# Need a check for X chromosome to switch back to standard stepwiseqtl
# This is derived from stepwiseqtl (R/qtl v1.25.14)
# 2 more variables than stepwiseqtl:
#   stop.rule (default at 0), 0 - no early stopping rule,
#                             1 - early stoping rule; plod<-k_f*mainA,
#                             2 - early stoping rule; plod<bestplod-k_f*mainA.
#   k_f (default at 3) number of main penalty plod can go below.
######################################################################
stepwiseqtlX <-
    function(cross, chr, pheno.col=1, qtl, formula, max.qtl=10, k_f=3, stop.rule=0, covar=NULL,
             method=c("imp", "hk"), model=c("normal", "binary"), incl.markers=TRUE, refine.locations=TRUE,
             additive.only=FALSE, penalties,
             keeplodprofile=TRUE, keeptrace=FALSE, verbose=TRUE,
             tol=1e-4, maxit=1000, require.fullrank=TRUE)
{
    if(verbose) message("Running stepwiseqtlX")

    if(!("cross" %in% class(cross)))
        stop("Input should have class \"cross\".")

    if(missing(penalties))
        stop("penalties must be provided")

    if(missing(qtl)) qtl <- NULL
    if(missing(formula)) formula <- NULL

    if(!missing(chr)) cross <- subset(cross, chr)

    method <- match.arg(method)
    model <- match.arg(model)

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    chr_type <- sapply(cross$geno, chrtype)
    if(any(chr_type=="X")) {
        Xadjustment <- scanoneXnull(class(cross)[1], getsex(cross))
        forceXcovar <- Xadjustment$adjustX
        Xcovar <- Xadjustment$sexpgmcovar
    } else forceXcovar <- FALSE

    if(!is.null(qtl)) { # start f.s. at somewhere other than the null
        if( !("qtl" %in% class(qtl)) )
            stop("The qtl argument must be an object of class \"qtl\".")

        # check that chromosomes were retained, otherwise give error
        m <- is.na(match(qtl$chr, names(cross$geno)))
        if(any(m)) {
            wh <- qtl$chr[m]
            if(length(wh) > 1)
                stop("Chromosomes ", paste(wh, collapse=", "), " (in QTL object) not in cross object.")
            else
                stop("Chromosome ", wh, " (in QTL object) not in cross object.")
        }
        if(is.null(formula)) { # create a formula with all covariates and all QTL add've
            if(!is.null(covar))
                formula <- paste("y ~ ", paste(names(covar), collapse="+"), "+")
            else
                formula <- "y ~ "
            formula <- paste(formula, paste(paste("Q", 1:length(qtl$chr), sep=""), collapse="+"))
        }
        else {
            temp <- checkStepwiseqtlStart(qtl, formula, covar)
            qtl <- temp$qtl
            formula <- temp$formula
        }
        startatnull <- FALSE
    }
    else {
        if(!is.null(formula))
            warning("formula ignored if qtl is not provided.")
        startatnull <- TRUE
    }

    # revise names in qtl object
    if(!startatnull)
        qtl$name <- qtl$altname

    # check that we have the right stuff for the selected method
    if(method=="imp") {
        if(!("draws" %in% names(cross$geno[[1]]))) {
            if("prob" %in% names(cross$geno[[1]])) {
                warning("The cross doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("You need to first run sim.geno.")
        }
    }
    else {
        if(!("prob" %in% names(cross$geno[[1]]))) {
            if("draws" %in% names(cross$geno[[1]])) {
                warning("The cross doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("You need to first run calc.genoprob.")
        }
    }
    if(method=="imp") qtlmethod <- "draws"
    else qtlmethod <- "prob"

    if(!is.null(qtl) && qtl$n.ind != nind(cross)) {
        warning("No. individuals in qtl object doesn't match that in the input cross; re-creating qtl object.")
        if(method=="imp")
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        else
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="prob")
    }

    if(!is.null(qtl) && method=="imp" && dim(qtl$geno)[3] != dim(cross$geno[[1]]$draws)[3])  {
        warning("No. imputations in qtl object doesn't match that in the input cross; re-creating qtl object.")
        qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
    }

    # check that qtl object matches the method
    if(!startatnull) {
        if(method=="imp" && !("geno" %in% names(qtl)))
            stop("The qtl object doesn't contain imputations; re-run makeqtl with what=\"draws\".")
        else if(method=="hk" && !("prob" %in% names(qtl)))
            stop("The qtl object doesn't contain QTL genotype probabilities; re-run makeqtl with what=\"prob\".")
    }

    # check phenotypes and covariates; drop ind'ls with missing values
    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("stepwiseqtl can take just one phenotype; only the first will be used")
    }
    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }
    if(any(pheno.col < 1 | pheno.col > nphe(cross)))
        stop("pheno.col values should be between 1 and the no. phenotypes")
    pheno <- cross$pheno[,pheno.col]
    if(!is.null(covar)) phcovar <- cbind(pheno, covar)
    else phcovar <- as.data.frame(pheno, stringsAsFactors=TRUE)
    hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
    if(all(hasmissing))
        stop("All individuals are missing phenotypes or covariates.")
    if(any(hasmissing)) {
        pheno <- pheno[!hasmissing]
        cross <- subset(cross, ind=!hasmissing)
        if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]

        if(!startatnull) {
            if(method=="imp")
                qtl$geno <- qtl$geno[!hasmissing,,,drop=FALSE]
            else {
                for(i in seq(along=qtl$prob))
                    qtl$prob[[i]] <- qtl$prob[[i]][!hasmissing,,drop=FALSE]
            }
            qtl$n.ind <- sum(!hasmissing)
        }
    }

    if(max.qtl < 1)
        stop("Need max.qtl > 0 if we are to scan for qtl")

    if(is.null(covar)) {
        lod0 <- 0
        if(startatnull)
            firstformula <- y~Q1
        else firstformula <- formula
    }
    else {
        nullformula <- as.formula(paste("y~", paste(names(covar), collapse="+")))
        tempqtl <- makeqtl(cross, chrnames(cross)[1], 0, what=ifelse(method=="imp", "draws", "prob"))
        fit <- fitqtl(cross, pheno.col, tempqtl, covar=covar, formula=nullformula,
                      method=method, model=model, dropone=FALSE, get.ests=FALSE,
                      run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
        lod0 <- fit$result.full[1,4]
        if(startatnull)
            firstformula <- as.formula(paste("y~", paste(names(covar), collapse="+"), "+", "Q1"))
        else firstformula <- formula
    }


    cross.type <- class(cross)[1]
    if(verbose > 2) verbose.scan <- TRUE
    else verbose.scan <- FALSE

    curbest <- NULL
    curbestplod <- 0

    # initial scan : either 1d or 2d
    if(verbose) cat(" -Initial scan\n")
    if(startatnull) { # Quoc: changed

        if(forceXcovar) {
            if(is.null(covar)) covar.w.X <- Xcovar
            else covar.w.X <- cbind(covar, Xcovar)
        }  else covar.w.X <- covar

        suppressWarnings(out.scanone <- scanone(cross, pheno.col=pheno.col, method=method, model=model,
                                                addcovar=covar.w.X))
        out.A <- subset(out.scanone, chr='-X')
        out.X <- subset(out.scanone, chr='X')
        formula <- firstformula
        n.qtl <- 1
        # Calculate max plod in Autosome
        lod.A <- max(out.A[,3], na.rm=TRUE)

        wh <- which(!is.na(out.A[,3]) & out.A[,3]==lod.A)
        if(length(wh) > 1) wh <- sample(wh, 1)

        qtl.A <- makeqtl(cross, as.character(out.A[wh,1]), out.A[wh,2], "Q1",
                         what=qtlmethod)

        # update Autosome plod after choosing formula and qtl
        curplod.A <- calc.plod.X(lod.A, formula=firstformula, qtl=qtl.A, penalties=penalties)
        # Calculate max plod in X chr
        lod.X <- max(out.X[,3], na.rm=TRUE)

        wh <- which(!is.na(out.X[,3]) & out.X[,3]==lod.X)
        if(length(wh) > 1) wh <- sample(wh, 1)

        qtl.X <- makeqtl(cross, as.character(out.X[wh,1]), out.X[wh,2], "Q1",
                         what=qtlmethod)

        # update X chr plod after choosing formula and qtl
        curplod.X <- calc.plod.X(lod.X, formula=firstformula, qtl=qtl.X, penalties=penalties)
        # Compare and choose plod between Autosome and X Chr
        if (curplod.X > curplod.A) {
            curplod <- curplod.X
            qtl <- qtl.X
            lod <- lod.X
        } else {
            curplod <- curplod.A
            qtl <- qtl.A
            lod <- lod.A
        }
        if(verbose) cat("initial lod: ", lod, "\n")
    } # start at null, Quoc: changed
    else {
        if(verbose) cat(" ---Starting at a model with", length(qtl$chr), "QTL\n")
        if(refine.locations) {
            if(verbose) cat(" ---Refining positions\n")
            rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                              covar=covar, formula=formula, method=method,
                              verbose=verbose.scan, incl.markers=incl.markers,
                              keeplodprofile=FALSE, forceXcovar=forceXcovar)
            if(any(rqtl$pos != qtl$pos)) { # updated positions
                if(verbose) cat(" ---  Moved a bit\n")
            }
            qtl <- rqtl
        }
        fit <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                      method=method, model=model, dropone=FALSE, get.ests=FALSE,
                      run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
        lod <- fit$result.full[1,4] - lod0
        if(require.fullrank && attr(fit, "matrix.rank") < attr(fit, "matrix.ncol")) lod <- 0
        curplod <- calc.plod.X(lod, formula=formula, qtl=qtl,
                               penalties=penalties) # Quoc changed
        attr(qtl, "pLOD") <- curplod
        n.qtl <- length(qtl$chr)
    }

    attr(qtl, "formula") <- deparseQTLformula(formula)
    attr(qtl, "pLOD") <- curplod

    if(curplod > 0) {
        curbest <- qtl
        curbestplod <- curplod

        if(verbose)
            cat("** new best ** (pLOD increased by ", round(curplod, 4), ")\n", sep="")
    }

    if(keeptrace) {
        temp <- list(chr=qtl$chr, pos=qtl$pos)
        attr(temp, "formula") <- deparseQTLformula(formula)
        attr(temp, "pLOD") <- curplod
        attr(temp, "LOD") <- lod # check LOD
        class(temp) <- c("compactqtl", "list")
        thetrace <- list("0"=temp)
    }

    if(verbose)
        cat("    no.qtl = ", n.qtl, "  pLOD =", curplod, "  formula:",
            deparseQTLformula(formula), "\n")
    if(verbose > 1)
        cat("         qtl:", paste(qtl$chr, round(qtl$pos,1), sep="@"), "\n")

    # start stepwise search
    i <- 0
    while(n.qtl < max.qtl) {
        i <- i+1

        if(verbose) {
            cat(" -Step", i, "\n")
            cat(" ---Scanning for additive qtl\n")
        }
        curformula <- as.formula(paste(deparseQTLformula(formula), "+Q", n.qtl+1, sep=""))
        # Add QTL in X chr
        out.X <- addqtl(cross, chr="X", pheno.col=pheno.col, qtl=qtl, covar=covar,
                        formula=formula, method=method, incl.markers=incl.markers,
                        verbose=verbose.scan, forceXcovar=forceXcovar,
                        require.fullrank=require.fullrank)

        curlod.X <- max(out.X[,3], na.rm=TRUE)
        wh <- which(!is.na(out.X[,3]) & out.X[,3]==curlod.X)
        if(length(wh) > 1) wh <- sample(wh,1)
        qtl.X <- addtoqtl(cross, qtl, as.character(out.X[wh,1]), out.X[wh,2],
                          paste("Q", n.qtl+1, sep=""))
        curlod.X <- curlod.X+lod
        plod.X <- calc.plod.X(curlod.X, formula=curformula, qtl=qtl.X,
                              penalties=penalties)
        # Add QTL in Autosome chr
        out.A <- addqtl(cross, chr="-X", pheno.col=pheno.col, qtl=qtl, covar=covar,
                        formula=formula, method=method, incl.markers=incl.markers,
                        verbose=verbose.scan, forceXcovar=forceXcovar,
                        require.fullrank=require.fullrank)

        curlod.A <- max(out.A[,3], na.rm=TRUE)
        wh <- which(!is.na(out.A[,3]) & out.A[,3]==curlod.A)
        if(length(wh) > 1) wh <- sample(wh,1)
        qtl.A <- addtoqtl(cross, qtl, as.character(out.A[wh,1]), out.A[wh,2],
                          paste("Q", n.qtl+1, sep=""))
        curlod.A <-curlod.A+lod
        plod.A <- calc.plod.X(curlod.A, formula=curformula, qtl=qtl.A,
                              penalties=penalties)
        if (plod.X > plod.A) {
            curplod <- plod.X
            curqtl <- qtl.X
            curlod <- curlod.X
        } else {
            curplod <- plod.A
            curqtl <- qtl.A
            curlod <- curlod.A
        }
        if(verbose) cat("        plod =", curplod, "\n")

        curnqtl <- n.qtl+1
        if(!additive.only) {
            for(j in 1:n.qtl) {

                if(verbose)
                    cat(" ---Scanning for QTL interacting with Q", j, "\n", sep="")

                thisformula <- as.formula(paste(deparseQTLformula(formula), "+Q", n.qtl+1,
                                                "+Q", j, ":Q", n.qtl+1, sep=""))
                # Scanning for QTL interacting with Qj in Autosome
                if(verbose)
                    cat(" ---Scanning for QTL in Autosome interacting with Q", j, "\n", sep="")

                out <- addqtl(cross, chr="-X", pheno.col=pheno.col, qtl=qtl, covar=covar,
                              formula=thisformula, method=method, incl.markers=incl.markers,
                              verbose=verbose.scan, forceXcovar=forceXcovar,
                              require.fullrank=require.fullrank)
                thislod <- max(out[,3], na.rm=TRUE)

                wh <- which(!is.na(out[,3]) & out[,3]==thislod)
                if(length(wh) > 1) wh <- sample(wh,1)
                thisqtl <- addtoqtl(cross, qtl, as.character(out[wh,1]), out[wh,2],
                                    paste("Q", n.qtl+1, sep=""))

                thislod <- thislod + lod
                thisplod <- calc.plod.X(thislod, formula=thisformula, qtl=thisqtl,
                                        penalties=penalties)
                if(verbose) cat("        plod =", thisplod, "\n")

                if(thisplod > curplod) {
                    curformula <- thisformula
                    curplod <- thisplod
                    curlod <- thislod
                    curqtl <- thisqtl

                    curnqtl <- n.qtl+1
                }
                # End: Scanning for QTL interacting with Qj in Autosome

                # Scanning for QTL interacting with Qj in X chromosome
                if(verbose)
                    cat(" ---Scanning for QTL in X chromosome interacting with Q", j, "\n", sep="")

                out <- addqtl(cross, chr="X", pheno.col=pheno.col, qtl=qtl, covar=covar,
                              formula=thisformula, method=method, incl.markers=incl.markers,
                              verbose=verbose.scan, forceXcovar=forceXcovar,
                              require.fullrank=require.fullrank)
                thislod <- max(out[,3], na.rm=TRUE)

                wh <- which(!is.na(out[,3]) & out[,3]==thislod)
                if(length(wh) > 1) wh <- sample(wh,1)
                thisqtl <- addtoqtl(cross, qtl, as.character(out[wh,1]), out[wh,2],
                                    paste("Q", n.qtl+1, sep=""))

                thislod <- thislod + lod
                thisplod <- calc.plod.X(thislod, formula=thisformula, qtl=thisqtl,
                                        penalties=penalties)
                if(verbose) cat("        plod =", thisplod, "\n")

                if(thisplod > curplod) {
                    curformula <- thisformula
                    curplod <- thisplod
                    curlod <- thislod
                    curqtl <- thisqtl

                    curnqtl <- n.qtl+1
                }
                # End: Scanning for QTL interacting with Qj in X chromosome
            }

            if(n.qtl > 1) {
                if(verbose)
                    cat(" ---Look for additional interactions\n")
                temp <- addint(cross, pheno.col, qtl, covar=covar, formula=formula,
                               method=method, qtl.only=TRUE, verbose=verbose.scan,
                               require.fullrank=require.fullrank)
                if(!is.null(temp)) {

                    formula.addint <- paste(deparseQTLformula(formula), "+", rownames(temp)) # formula of addint model
                    thelod <- temp[,3]
                    thelod <- thelod + lod
                    #             browser()
                    plod.addint <- mapply(FUN=calc.plod.X, lod=thelod, formula=formula.addint, MoreArgs=list(qtl=qtl,penalties=penalties))
                    thisplod <- max(plod.addint, na.rm=TRUE)
                    wh <- which(!is.na(thelod) & plod.addint==thisplod)
                    if(length(wh) > 1) wh <- sample(wh, 1)
                    thisformula <- as.formula(formula.addint[wh])
                    thislod <- thelod[wh]

                    if(verbose) cat("        plod =", thisplod, "\n")
                    if(thisplod > curplod) {
                        curformula <- thisformula
                        curplod <- thisplod
                        curlod <- thislod
                        curqtl <- qtl
                        curnqtl <- n.qtl
                    }
                }
            }
        }

        qtl <- curqtl
        n.qtl <- curnqtl
        attr(qtl, "formula") <- deparseQTLformula(curformula)
        attr(qtl, "pLOD") <- curplod
        formula <- curformula
        lod <- curlod

        if(refine.locations) {
            if(verbose) cat(" ---Refining positions\n")
            rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                              covar=covar, formula=formula, method=method,
                              verbose=verbose.scan, incl.markers=incl.markers,
                              keeplodprofile=FALSE, forceXcovar=forceXcovar)
            if(any(rqtl$pos != qtl$pos)) { # updated positions
                if(verbose) cat(" ---  Moved a bit\n")
                qtl <- rqtl
                fit <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                              method=method, model=model, dropone=FALSE, get.ests=FALSE,
                              run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
                lod <- fit$result.full[1,4] - lod0
                if(require.fullrank && attr(fit, "matrix.rank") < attr(fit, "matrix.ncol")) lod <- 0
                curplod <- calc.plod.X(lod, formula=formula, qtl=qtl,
                                       penalties=penalties)
                attr(qtl, "pLOD") <- curplod
            }

        }

        if(verbose)
            cat("    no.qtl = ", n.qtl, "  pLOD =", curplod, "  formula:",
                deparseQTLformula(formula), "\n")
        if(verbose > 1)
            cat("         qtl:", paste(qtl$chr, round(qtl$pos,1), sep="@"), "\n")


        if(curplod > curbestplod) {
            if(verbose)
                cat("** new best ** (pLOD increased by ", round(curplod - curbestplod, 4),
                    ")\n", sep="")

            curbest <- qtl
            curbestplod <- curplod
        }

        if(keeptrace) {
            temp <- list(chr=qtl$chr, pos=qtl$pos)
            attr(temp, "formula") <- deparseQTLformula(formula)
            attr(temp, "pLOD") <- curplod
            attr(temp, "LOD") <- lod # check LOD
            class(temp) <- c("compactqtl", "list")
            temp <- list(temp)
            names(temp) <- i
            thetrace <- c(thetrace, temp)
        }

        if(n.qtl >= max.qtl) break
        if (stop.rule==1) if(curplod < -k_f*penalties[1]) break # Add 1 Early stoping rule; plod<-k_f*mainA; k_f default at 3.
        if (stop.rule==2) if(curplod < (curbestplod-k_f*penalties[1])) break # Add 1 Early stoping rule; plod<bestplod-k_f*mainA k_f default at 3.
        # stop.rule==0 mean no early stop.
    }

    if(verbose) cat(" -Starting backward deletion\n")

    while(n.qtl > 1) {
        i <- i+1
        out <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                      method=method, model=model, dropone=TRUE, get.ests=FALSE,
                      run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)$result.drop

        formulas <- attr(out, "formulas") # Extract formula of dropone model
        #       lods <- attr(out, "lods")

        rn <- rownames(out)
        # ignore things with covariates
        wh <- c(grep("^[Qq][0-9]+$", rn),
                grep("^[Qq][0-9]+:[Qq][0-9]+$", rn))
        out <- out[wh,,drop=FALSE]
        formulas <- formulas[wh,drop=FALSE]
        #       lods <- lods[wh,drop=FALSE]
        thelod <- out[,3]
        plod.dropone <- mapply(FUN=calc.plod.X, lod=lod-thelod, formula=formulas, MoreArgs=list(qtl=qtl,penalties=penalties))
        maxplod <- max(plod.dropone, na.rm=TRUE)

        wh <- which(!is.na(thelod) & plod.dropone==maxplod)
        if(length(wh) > 1) wh <- sample(wh, 1)

        lod <- lod - thelod[wh] # End of Quoc change, below code is unchanged.
        todrop <- rownames(out)[wh]

        if(verbose) cat(" ---Dropping", todrop, "\n")

        if(length(grep(":", todrop)) > 0) { # dropping an interaction
            theterms <- attr(terms(formula), "factors")
            wh <- colnames(theterms)==todrop
            if(!any(wh)) stop("Confusion about what interation to drop!")
            theterms <- colnames(theterms)[!wh]
            formula <- as.formula(paste("y~", paste(theterms, collapse="+")))
        }
        else {
            numtodrop <- as.numeric(substr(todrop, 2, nchar(todrop)))

            theterms <- attr(terms(formula), "factors")
            cn <- colnames(theterms)
            g <- c(grep(paste("^[Qq]", numtodrop, "$", sep=""), cn),
                   grep(paste("^[Qq]", numtodrop, ":", sep=""), cn),
                   grep(paste(":[Qq]", numtodrop, "$", sep=""), cn))
            cn <- cn[-g]
            formula <- as.formula(paste("y~", paste(cn, collapse="+")))

            if(n.qtl > numtodrop) {
                for(j in (numtodrop+1):n.qtl)
                    formula <- reviseqtlnuminformula(formula, j, j-1)
            }

            qtl <- dropfromqtl(qtl, index=numtodrop)
            qtl$name <- qtl$altname <- paste("Q", 1:qtl$n.qtl, sep="")
            n.qtl <- n.qtl - 1
        }

        curplod <- calc.plod.X(lod, formula=formula, qtl=qtl,
                               penalties=penalties)
        #       browser()
        if(verbose)
            cat("    no.qtl = ", n.qtl, "  pLOD =", curplod, "  formula:",
                deparseQTLformula(formula), "\n")
        if(verbose > 1)
            cat("         qtl:", paste(qtl$chr, round(qtl$pos,1), sep=":"), "\n")

        attr(qtl, "formula") <- deparseQTLformula(formula)
        attr(qtl, "pLOD") <- curplod

        if(refine.locations) {
            if(verbose) cat(" ---Refining positions\n")
            if(!is.null(qtl)) {
                rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                                  covar=covar, formula=formula, method=method,
                                  verbose=verbose.scan, incl.markers=incl.markers,
                                  keeplodprofile=FALSE, forceXcovar=forceXcovar)
                if(any(rqtl$pos != qtl$pos)) { # updated positions
                    if(verbose) cat(" ---  Moved a bit\n")
                    qtl <- rqtl
                    fit <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                                  method=method, model=model, dropone=FALSE, get.ests=FALSE,
                                  run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
                    lod <- fit$result.full[1,4] - lod0
                    if(require.fullrank && attr(fit, "matrix.rank") < attr(fit, "matrix.ncol")) lod <- 0
                    curplod <- calc.plod.X(lod, formula=formula, qtl=qtl,
                                           penalties=penalties)
                    attr(qtl, "pLOD") <- curplod
                }
            }
        }

        if(curplod > curbestplod) {
            if(verbose)
                cat("** new best ** (pLOD increased by ", round(curplod - curbestplod, 4),
                    ")\n", sep="")
            #         browser()
            curbestplod <- curplod
            curbest <- qtl
        }

        if(keeptrace) {
            temp <- list(chr=qtl$chr, pos=qtl$pos)
            attr(temp, "formula") <- deparseQTLformula(formula)
            attr(temp, "pLOD") <- curplod
            attr(temp, "LOD") <- lod # check LOD
            class(temp) <- c("compactqtl", "list")
            temp <- list(temp)
            names(temp) <- i
            thetrace <- c(thetrace, temp)
        }
    }

    # re-form the qtl
    if(!is.null(curbest)) {
        chr <- curbest$chr
        pos <- curbest$pos
        o <- order(factor(chr, levels=names(cross$geno)), pos)

        qtl <- makeqtl(cross, chr[o], pos[o], what=qtlmethod)

        # need to redo numbering in formula
        formula <- as.formula(attr(curbest, "formula"))

        if(length(chr) > 1) {
            n.qtl <- length(chr)
            for(i in 1:n.qtl)
                formula <- reviseqtlnuminformula(formula, i, n.qtl+i)
            for(i in 1:n.qtl)
                formula <- reviseqtlnuminformula(formula, n.qtl+o[i], i)
        }

        if(keeplodprofile) {
            if(verbose) cat(" ---One last pass through refineqtl\n")
            qtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                             covar=covar, formula=formula, method=method,
                             verbose=verbose.scan, incl.markers=incl.markers,
                             keeplodprofile=TRUE, forceXcovar=forceXcovar)
        }
        attr(qtl, "formula") <- deparseQTLformula(formula)
        attr(qtl, "pLOD") <- attr(curbest, "pLOD")
        curbest <- qtl
    }
    else {
        curbest <- numeric(0)
        class(curbest) <- "qtl"
        attr(curbest,"pLOD") <- 0
    }

    if(keeptrace)
        attr(curbest, "trace") <- thetrace

    attr(curbest, "formula") <- deparseQTLformula(attr(curbest, "formula"), TRUE)
    attr(curbest, "penalties") <- penalties

    curbest
}


######################################################################
# penalized LOD score, Quoc changed, now central function for pLOD
######################################################################
calc.plod.X <-
    function(lod, nterms, type=c("f2","bc"), penalties, formula, qtl)
{
    if (missing(nterms) && (!missing(formula) && !is.null(formula)) && !missing(qtl))
        nterms <- countqtltermsX(formula, qtl)
    nterms <- nterms[1:6]
    if(any(penalties==Inf & nterms > 0)) return(-Inf)

    as.numeric(lod - sum((nterms*penalties)[nterms > 0]))
}

# end of stepwiseqtlX.R

######################################################################
#
# scanqtl.R
#
# copyright (c) 2002-2019, Hao Wu and Karl W. Broman
# last modified Dec, 2019
# first written Apr, 2002
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: scanqtl
#
######################################################################

scanqtl <-
    function(cross, pheno.col=1, chr, pos, covar=NULL, formula,
             method=c("imp", "hk"), model=c("normal", "binary"),
             incl.markers=FALSE, verbose=TRUE, tol=1e-4, maxit=1000,
             forceXcovar=FALSE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(!is.null(covar) && !is.data.frame(covar)) {
        if(is.matrix(covar) && is.numeric(covar))
            covar <- as.data.frame(covar, stringsAsFactors=TRUE)
        else stop("covar should be a data.frame")
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("scanqtl can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 | pheno.col > nphe(cross))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    pheno <- cross$pheno[,pheno.col]
    if(!is.null(covar) && nrow(covar) != length(pheno))
        stop("nrow(covar) != no. individuals in cross.")

    method <- match.arg(method)
    model <- match.arg(model)

    # allow formula to be a character string
    if(!missing(formula) && is.character(formula))
        formula <- as.formula(formula)

    if(method=="imp") {
        if(!("draws" %in% names(cross$geno[[1]]))) {
            if("prob" %in% names(cross$geno[[1]])) {
                warning("The cross doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("You need to first run sim.geno.")
        }
    }
    else {
        if(!("prob" %in% names(cross$geno[[1]]))) {
            if("draws" %in% names(cross$geno[[1]])) {
                warning("The cross doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("You need to first run calc.genoprob.")
        }
    }

    if(method=="imp") {
        if("stepwidth" %in% names(attributes(cross$geno[[1]]$draws)) &&
           attr(cross$geno[[1]]$draws, "stepwidth") != "fixed") {
            stepwidth.var <- TRUE
            incl.markers <- TRUE
        }
        else stepwidth.var <- FALSE
    }
    else {
        if("stepwidth" %in% names(attributes(cross$geno[[1]]$prob)) &&
           attr(cross$geno[[1]]$prob, "stepwidth") != "fixed") {
            stepwidth.var <- TRUE
            incl.markers <- TRUE
        }
        else stepwidth.var <- FALSE
    }

    type <- crosstype(cross)
    chr_type <- sapply(cross$geno, chrtype)

    # input data checking
    if( length(chr) != length(pos))
        stop("Input chr and pos must have the same length")
    # note that input chr is a vector and pos is a list

    method <- match.arg(method)

    ichr <- match(chr, names(cross$geno))
    if(any(is.na(ichr)))
        stop("There's no chromosome number ", chr[is.na(ichr)],
             " in input cross object")


    # if formula is missing, make one.
    # All QTLs and covariates will be additive by default
    n.qtl <- length(chr)
    n.covar <- length(covar)
    if(missing(formula)) {
        tmp.Q <- paste("Q", 1:n.qtl, sep="") # QTL term names
        formula <- "y~Q1"
        if(n.qtl > 1)
            for (i in 2:n.qtl)
                formula <- paste(formula, tmp.Q[i], sep="+")
        if (n.covar) { # if covariate is not empty
            tmp.C <- names(covar) # covariate term names
            for(i in 1:n.covar)
                formula <- paste(formula, tmp.C[i], sep="+")
        }
        formula <- as.formula(formula)
    }
    else {
        # include all input QTLs and covariates in the formula additively
        formula.str <- deparseQTLformula(formula) # deparse formula as a string
        for(i in 1:n.qtl) { # loop thru the QTLs
            qtl.term <- paste("Q", i, sep="")
            if( length(grep(qtl.term, formula.str, ignore.case=TRUE))==0 )
                # this term is not in the formula
                # add it to the formula
                formula.str <- paste(formula.str, qtl.term, sep="+")
        }
        if(n.covar) { # covariate is not empty
            for(i in 1:n.covar) {
                covar.term <- names(covar)[i]
                if( length(grep(covar.term, formula.str, ignore.case=TRUE))==0 )
                    # this term is not in the formula
                    # add it to the formula
                    formula.str <- paste(formula.str, covar.term, sep="+")
            }
        }
        formula <- as.formula(formula.str)
    }

    # check the formula
    formula <- checkformula(formula, paste("Q", 1:length(chr), sep=""),
                            colnames(covar))

    # drop covariates that are not in the formula
    if(!is.null(covar)) {
        theterms <- rownames(attr(terms(formula), "factors"))
        m <- match(colnames(covar), theterms)
        if(all(is.na(m))) covar <- NULL
        else covar <- covar[,!is.na(m),drop=FALSE]
    }

    # check phenotypes and covariates; drop ind'ls with missing values
    if(!is.null(covar)) phcovar <- cbind(pheno, covar)
    else phcovar <- cbind(pheno)
    if(any(is.na(phcovar))) {
        if(ncol(phcovar)==1) hasmissing <- is.na(phcovar)
        else hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
        if(all(hasmissing))
            stop("All individuals are missing phenotypes or covariates.")
        if(any(hasmissing)) {
            warning("Dropping ", sum(hasmissing), " individuals with missing phenotypes.\n")
            cross <- subset(cross, ind=!hasmissing)
            pheno <- pheno[!hasmissing]
            if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]
        }
    }
    sexpgm <- getsex(cross)

    # find the chromosome with multiple QTLs
    # indices for chromosomes with multiple QTLs
    idx.varied <- NULL
    indices <- pos  ## added by Karl 8/23/05
    for(i in 1:length(pos)) {
        l <- length(pos[[i]] )
        if( l >= 2 ) {
            # if there're more than two elements in pos, issue warning message
            if(l > 2) {
                msg <- "There are more than two elements in "
                msg <- paste(msg, i, "th input pos.")
                msg <- paste(msg, "The first two are taken as starting and ending position.")
                warning(msg)
            }

            # user specified a range
            # find all markers in this range
            idx.varied <- c(idx.varied, i)
            # make the genetic map on this chromosome

            # make genetic map
            if(method=="imp") {
                if("map" %in% names(attributes(cross$geno[[ichr[i]]]$draws)))
                    map <- attr(cross$geno[[ichr[i]]]$draws,"map")
                else {
                    stp <- attr(cross$geno[[ichr[i]]]$draws, "step")
                    oe <- attr(cross$geno[[ichr[i]]]$draws, "off.end")

                    if("stepwidth" %in% names(attributes(cross$geno[[ichr[i]]]$draws)))
                        stpw <- attr(cross$geno[[ichr[i]]]$draws, "stepwidth")
                    else stpw <- "fixed"
                    map <- create.map(cross$geno[[ichr[i]]]$map,stp,oe,stpw)
                }
            }
            else {
                if("map" %in% names(attributes(cross$geno[[ichr[i]]]$prob)))
                    map <- attr(cross$geno[[ichr[i]]]$prob,"map")
                else {
                    stp <- attr(cross$geno[[ichr[i]]]$prob, "step")
                    oe <- attr(cross$geno[[ichr[i]]]$prob, "off.end")

                    if("stepwidth" %in% names(attributes(cross$geno[[ichr[i]]]$prob)))
                        stpw <- attr(cross$geno[[ichr[i]]]$prob, "stepwidth")
                    else stpw <- "fixed"
                    map <- create.map(cross$geno[[ichr[i]]]$map,stp,oe,stpw)
                }

            }
            # pull out the female map if there are sex-specific maps
            if(is.matrix(map)) map <- map[1,]

            indices[[i]] <- seq(along=map)
            if(method=="imp")
                step <- attr(cross$geno[[ichr[i]]]$draws,"step")
            else
                step <- attr(cross$geno[[ichr[i]]]$prob,"step")

            if(!incl.markers && step>0) { # equally spaced positions
                eq.sp.pos <- seq(min(map), max(map), by=step)
                wh.eq.pos <- match(eq.sp.pos, map)
                map <- map[wh.eq.pos]
                indices[[i]] <- indices[[i]][wh.eq.pos]
            }

            # locate the markers given starting and ending postion
            # we should do this before or after incl.markers?
            start <- pos[[i]][1]
            end <- pos[[i]][2]
            # replace pos[[i]] (a range) by the marker positions within the range
            # extend the position to the nearest markers outside the ranges
            tmp <- which( (map - start)<=0 )
            if(length(tmp) != 0) # starting position is after the first marker
                start <- map[max(tmp)]
            tmp <- which( (end-map) <= 0 )
            if(length(tmp) != 0) # ending position is before the last marker
                end <- map[min(tmp)]
            pos[[i]] <- as.vector( map[(map>=start)&(map<=end)] )
            indices[[i]] <- indices[[i]][(map>=start)&(map<=end)]
        }
    }
    # Now, pos contains all the marker positions for all chromosomes

    #########################
    # Now start general scan
    #########################
    # There might be several chromosomes with multiple QTLs
    # Use one loop

    sexpgm <- getsex(cross)
    cross.attr <- attributes(cross)

    # number of chromosomes with multiple positions to be scanned
    n.idx.varied <- length(idx.varied)
    n.loop <- 1 # total number of loops
    if(n.idx.varied != 0) { # there IS some chromosomes with multiple QTL
        # vector to indicate the positions indices for those chromosomes
        idx.pos <- rep(0, n.idx.varied)
        l.varied <- NULL
        for(i in 1:n.idx.varied) {
            l.varied[i] <- length(pos[[idx.varied[i]]])
            n.loop <- n.loop * l.varied[i]
        }
        # initialize output variable
        result <- array(rep(0, n.loop), rev(l.varied))
        matrix.rank <- matrix.ncol <- array(rep(0, n.loop), rev(l.varied))
    }
    else { # fixed QTL model (no scanning)
        if(method=="imp")
            qtl <- makeqtl(cross, chr=chr, pos=unlist(pos), what="draws")
        else
            qtl <- makeqtl(cross, chr=chr, pos=unlist(pos), what="prob")

        result <- fitqtlengine(pheno=pheno, qtl=qtl, covar=covar,
                               formula=formula, method=method, model=model, dropone=FALSE,
                               get.ests=FALSE, run.checks=FALSE, cross.attr=cross.attr,
                               crosstype=crosstype(cross),
                               sexpgm=sexpgm, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
        matrix.rank <- attr(result, "matrix.rank")
        matrix.ncol <- attr(result, "matrix.ncol")

        result <- result[[1]][1,4]
        names(result) <- "LOD"
        class(result) <- "scanqtl"
        attr(result, "method") <- method
        attr(result, "formula") <- deparseQTLformula(formula)
        attr(result, "matrix.rank") <- matrix.rank
        attr(result, "matrix.ncol") <- matrix.ncol
        return(result)
    }

    # loop thru all varied QTLs
    if(verbose) {
        cat(" ",n.loop, "models to fit\n")
        n.prnt <- floor(n.loop/20)
        if(n.prnt < 1) n.prnt <- 1
    }
    current.pos <- NULL ## added by Karl 8/23/05
    for(i in 1:n.loop) {
        # find the indices for positions
        remain <- i
        if(n.idx.varied > 1) {
            for(j in 1:(n.idx.varied-1)) {
                ns <- 1
                for( k in (j+1):n.idx.varied )
                    ns <- ns * length(pos[[idx.varied[k]]])
                idx.pos[j] <- floor(remain / ns) + 1
                remain <- remain - (idx.pos[j]-1) * ns
                # remain cannot be zero
                if(remain == 0) {
                    idx.pos[j] <- idx.pos[j] - 1
                    remain <- remain + ns
                }
            }
        }
        idx.pos[n.idx.varied] <- remain

        # make an QTL object
        pos.tmp <- NULL
        for(j in 1:length(pos)) {
            if(j %in% idx.varied) {
                idx.tmp <- which(j==idx.varied)
                pos.tmp <- c(pos.tmp, pos[[j]][idx.pos[idx.tmp]])
            }
            else
                pos.tmp <- c(pos.tmp, pos[[j]])
        }

        # this bit revised by Karl 8/23/05; now we make the qtl object
        #     once, and copy stuff over otherwise
        if(is.null(current.pos)) {
            if(method=="imp")
                qtl.obj <- makeqtl(cross, chr, pos.tmp, what="draws")
            else
                qtl.obj <- makeqtl(cross, chr, pos.tmp, what="prob")
            current.pos <- pos.tmp
        }
        else {
            thew <- rep(NA, length(pos.tmp)) ####
            for(kk in seq(along=pos.tmp)) {
                if(pos.tmp[kk] != current.pos[kk]) {
                    u <- abs(pos.tmp[kk]-pos[[kk]])
                    w <- indices[[kk]][u==min(u)]
                    if(length(w) > 1) {
                        warning("Confused about QTL positions.  You should probably run jittermap to ensure that no two markers conincide.")
                        w <- sample(w, 1)
                    }
                    if(method=="imp")
                        qtl.obj$geno[,kk,] <- cross$geno[[ichr[kk]]]$draws[,w,]
                    else
                        qtl.obj$prob[[kk]] <- cross$geno[[ichr[kk]]]$prob[,w,]
                    thew[kk] <- w ####

                    if(chr_type[ichr[kk]]=="X" && (type=="bc" || type=="f2")) {
                        if(method=="imp")
                            qtl.obj$geno[,kk,] <-
                                reviseXdata(type,"full",sexpgm,draws=qtl.obj$geno[,kk,,drop=FALSE],
                                            cross.attr=attributes(cross))
                        else {
                            temp <- qtl.obj$prob[[kk]]
                            temp <- array(temp, dim=c(nrow(temp),1,ncol(temp)))
                            dimnames(temp) <- list(NULL,"loc", 1:ncol(qtl.obj$prob[[kk]]))
                            qtl.obj$prob[[kk]] <- reviseXdata(type,"full",sexpgm,prob=temp,
                                                              cross.attr=attributes(cross))[,1,]
                        }
                    }

                    current.pos[kk] <- pos.tmp[kk]
                }
            }
        }
        # end of Karl's 8/23/05 addition

        # fit QTL, don't do drop one at a time
        fit <- fitqtlengine(pheno=pheno, qtl=qtl.obj, covar=covar,
                            formula=formula, method=method, model=model, dropone=FALSE,
                            get.ests=FALSE, run.checks=FALSE,
                            cross.attr=cross.attr, crosstype=crosstype(cross),
                            sexpgm=sexpgm, tol=tol, maxit=maxit,
                            forceXcovar=forceXcovar)

        matrix.rank[i] <- attr(fit, "matrix.rank")
        matrix.ncol[i] <- attr(fit, "matrix.ncol")

        if(verbose && ((i-1) %% n.prnt) == 0)
            cat("    ", i,"/", n.loop, "\n")

        # assign to result matrix
        #     Note: [[1]][1,4] picks out the LOD score
        result[i] <- fit[[1]][1,4]
    }

    # make the row and column names for the result matrix
    dnames <- list(NULL)
    for(i in 1:n.idx.varied) {
        i.chr <- chr[idx.varied[n.idx.varied-i+1]]
        i.pos <- pos[[idx.varied[n.idx.varied-i+1]]]
        dnames[[i]] <- paste( paste("Chr", i.chr,sep=""),
                             i.pos, sep="@")
    }
    dimnames(result) <- dnames

    class(result) <- "scanqtl"
    attr(result, "method") <- method
    attr(result, "formula") <- deparseQTLformula(formula)
    attr(result, "matrix.rank") <- matrix.rank
    attr(result, "matrix.ncol") <- matrix.ncol

    result
}


#summary.scanqtl <- function(object, ...)
#{
#}

#print.summary.qtl <- function(x, ...)
#{
#}

# end of scanqtl.R

library(qtl)
data(map10)
map10 <- map10[1:2]
set.seed(8789993)
simcross <- sim.cross(map10, n.ind=125, type="bc",
                      model=rbind(c(1, 50, 1.5), c(2, 50, 0)))
simcross$pheno[,1] <- simcross$pheno[,1] + rnorm(nind(simcross), 0, 2*simcross$qtlgeno[,2])
simcross <- calc.genoprob(simcross)
out <- scanonevar(simcross,
                  tol=0.01)
summary(out, format="allpeaks")

####

data(fake.bc)
fake.bc <- fake.bc[1:2,1:150] # only chr 1 and 2, and first 100 individuals
fake.bc <- calc.genoprob(fake.bc, step=5)
out <- scanonevar(fake.bc,
                  tol=0.01)
summary(out, format="allpeaks")
covar <- fake.bc$pheno[,c("sex", "age")]
out <- scanonevar(fake.bc, mean_covar=covar, var_covar=covar,
                  tol=0.01)
summary(out, format="allpeaks")

#########Simulate a vQTL on Chromosome 1########

chromo=1
qtl.position=14 # 50 cM
N=nind(fake.bc)
a1<-fake.bc$geno[[chromo]]$prob[,,1]
y <- fake.bc$pheno$pheno1
y <- y + rnorm(N,0,exp(a1[,qtl.position]))
out <- scanonevar(fake.bc, y, mean_covar=covar, var_covar=covar)
summary(out, format="allpeaks")

out <- scanonevar(fake.bc, y, mean_covar=covar,
                  tol=0.01)
summary(out, format="allpeaks")

out <- scanonevar(fake.bc, y, var_covar=covar,
                  tol=0.01)
summary(out, format="allpeaks")

out <- scanonevar(fake.bc, y,
                  tol=0.01)
summary(out, format="allpeaks")

######################################################################
#
# read.cross.gary.R
#
# copyright (c) 2000-2019, Karl W Broman
# last modified Dec, 2019
# first written Aug, 2000
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.gary
#           [See read.cross.R for the main read.cross function.]
#
######################################################################

######################################################################
#
# read.cross.gary
#
# read data in Gary's format
#
######################################################################

read.cross.gary <-
    function(dir,genfile,mnamesfile,chridfile,phefile,pnamesfile,mapfile,
             estimate.map,na.strings)
{
    # create file names
    if(missing(genfile)) genfile <- "geno.dat"
    if(missing(mnamesfile)) mnamesfile <- "mnames.txt"
    if(missing(chridfile)) chridfile <- "chrid.dat"
    if(missing(phefile)) phefile <- "pheno.dat"
    if(missing(pnamesfile)) pnamesfile <- "pnames.txt"
    if(missing(mapfile)) mapfile <- "markerpos.txt"

    if(!missing(dir) && dir != "") {
        genfile <- file.path(dir, genfile)
        mnamesfile <- file.path(dir, mnamesfile)
        chridfile <- file.path(dir, chridfile)
        phefile <- file.path(dir, phefile)
        if(!is.null(pnamesfile)) pnamesfile <- file.path(dir, pnamesfile)
        if(!is.null(mapfile)) mapfile <- file.path(dir, mapfile)
    }

    # read data
    allgeno <- as.matrix(read.table(genfile,na.strings="9"))+1
    pheno <- as.matrix(read.table(phefile,na.strings=na.strings,header=FALSE))
    chr <- scan(chridfile,what=character(),quiet=TRUE)
    mnames <- scan(mnamesfile,what=character(),quiet=TRUE)

    if(!is.null(mapfile)) {
        map <- read.table(mapfile,row.names=1)
        map <- map[mnames,1]
        map.included <- TRUE
    }
    else {
        map <- seq(0,by=5,len=length(mnames))
        map.included <- FALSE
    }

    if(!is.null(pnamesfile)) pnames <- scan(pnamesfile,what=character(),quiet=TRUE)
    else pnames <- paste("pheno",1:ncol(pheno),sep="")


    # fix up map information
    # number of chromosomes
    uchr <- unique(chr)
    n.chr <- length(uchr)
    geno <- vector("list",n.chr)
    names(geno) <- uchr
    min.mar <- 1
    for(i in 1:n.chr) { # loop over chromosomes
        # create map
        temp.map <- map[chr==uchr[i]]

        # deal with any markers that didn't appear in the marker pos file
        if(any(is.na(temp.map))) {
            o <- (seq(along=temp.map))[is.na(temp.map)]
            for(j in o) {
                if(j==1 || all(is.na(temp.map[1:(j-1)]))) {
                    z <- min((seq(along=temp.map))[-o])
                    temp.map[j] <- min(temp.map,na.rm=TRUE)-(z-j+1)
                }
                else if(j==length(temp.map) || all(is.na(temp.map[-(1:j)]))) {
                    z <- max((seq(along=temp.map))[-o])
                    temp.map[j] <- max(temp.map,na.rm=TRUE)+(j-z+1)
                }
                else {
                    temp.map[j] <- (min(temp.map[-(1:j)],na.rm=TRUE)+
                                    max(temp.map[1:(j-1)],na.rm=TRUE))/2
                }
            }
        }

        names(temp.map) <- mnames[chr==uchr[i]]

        # pull out appropriate portion of genotype data
        data <- allgeno[,min.mar:(length(temp.map)+min.mar-1),drop=FALSE]
        min.mar <- min.mar + length(temp.map)
        colnames(data) <- names(temp.map)

        geno[[i]] <- list(data=data,map=temp.map)
        if(uchr[i] == "X" || uchr[i] == "x")
            class(geno[[i]]) <- "X"
        else class(geno[[i]]) <- "A"
    }
    colnames(pheno) <- pnames

    # fix up phenotype data: make things numeric that look numeric
    sw2numeric_gary <-
        function(x) {
            pattern <- "^[ \t]*-*[0-9]*[.]*[0-9]*[ \t]*$"
            n <- sum(!is.na(x))
            if(length(grep(pattern,as.character(x[!is.na(x)])))==n)
                return(as.numeric(as.character(x)))
            else return(x)
        }
    pheno <- data.frame(lapply(as.data.frame(pheno), sw2numeric_gary), stringsAsFactors=TRUE)

    # check that data dimensions match
    n.mar1 <- sapply(geno,function(a) ncol(a$data))
    n.mar2 <- sapply(geno,function(a) length(a$map))
    n.phe <- ncol(pheno)
    n.ind1 <- nrow(pheno)
    n.ind2 <- sapply(geno,function(a) nrow(a$data))
    if(any(n.ind1 != n.ind2)) {
        cat(n.ind1,n.ind2,"\n")
        stop("Number of individuals in genotypes and phenotypes do not match.");
    }
    if(any(n.mar1 != n.mar2)) {
        cat(n.mar1,n.mar2,"\n")
        stop("Numbers of markers in genotypes and marker names files do not match.");
    }

    # print some information about the amount of data read
    cat(" --Read the following data:\n");
    cat("\t", n.ind1, " individuals\n");
    cat("\t", sum(n.mar1), " markers\n");
    cat("\t", n.phe, " phenotypes\n");

    # determine map type: f2 or bc or 4way?
    if(max(allgeno[!is.na(allgeno)])<=2) type <- "bc"
    else type <- "f2"
    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c(type,"cross")

    # check that nothing is strange in the genotype data
    if(type=="f2") max.gen <- 5
    else max.gen <- 2

    u <- unique(allgeno)
    if(any(!is.na(u) & (u > max.gen | u < 1)))
        stop("There are stange values in the genotype data : ",
             paste(sort(u),collapse=":"), ".")

    cross$pheno <- as.data.frame(cross$pheno, stringsAsFactors=TRUE)

    # if map wasn't included, go through each chromosome and
    # make first marker at 0 cM.
    if(!map.included) {
        for(i in 1:nchr(cross))
            cross$geno[[i]]$map <- cross$geno[[i]]$map - min(cross$geno[[i]]$map)
    }

    # return cross + indicator of whether to run est.map
    # [run est.map if map not included and estimate.map == TRUE]
    list(cross, (!map.included && estimate.map) )
}

# end of read.cross.gary.R

#####################################################################
#
# xchr.R
#
# copyright (c) 2004-2018, Karl W Broman
# last modified Mar, 2018
# first written Apr, 2004
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: Utilities for dealing with the X chromosome.
#           getsex, getgenonames, reviseXdata, scanoneXnull
#           revisecovar, dropXcol
#           [See also fixXgeno.bc & fixXgeno.f2 in read.cross.R]
#
######################################################################

# get sex and pgm columns from phenotype data
getsex <-
    function(cross)
{
    type <- crosstype(cross)
    if(type != "bc" && type != "f2" && type != "4way") return(list(sex=NULL, pgm=NULL))

    phe.names <- names(cross$pheno)

    sex.column <- grep("^[Ss][Ee][Xx]$", phe.names)
    pgm.column <- grep("^[Pp][Gg][Mm]$", phe.names)

    if(length(sex.column)==0) { # no sex included
        sex <- NULL
    }
    else {
        if(length(sex.column)>1)
            warning("'sex' included multiple times.  Using the first one.")
        temp <- cross$pheno[,sex.column[1]]
        if(is.numeric(temp)) {
            if(any(!is.na(temp) & temp != 0 & temp != 1)) {
                warning("Sex column should be coded as 0=female 1=male; sex ignored.")
                sex <- NULL
            }
            else sex <- temp
        }
        else {
            if(!is.factor(temp)) temp <- as.factor(temp)

            if(length(levels(temp)) == 1) {
                if(levels(temp) == "F" || levels(temp)=="f" ||
                   toupper(levels(temp)) == "FEMALE") sex <- rep(0,nind(cross))
                else if(levels(temp) == "M" || levels(temp)=="m" ||
                        toupper(levels(temp)) == "MALE") sex <- rep(1,nind(cross))
                else
                    warning("Sex column should be coded as 0=female 1=male; sex ignored.")
            }
            else if(length(levels(temp)) > 2) {
                warning("Sex column should be coded as a two-level factor; sex ignored.")
                sex <- NULL
            }
            else { # is a factor with two levels
                lev <- levels(temp)
                if(length(grep("^[Ff]",lev))>0 &&
                   length(males <- grep("^[Mm]",lev))>0) {
                    temp <- as.character(temp)
                    sex <- rep(0,length(temp))
                    sex[is.na(temp)] <- NA
                    sex[!is.na(temp) & temp==lev[males]] <- 1
                }
                else
                    warning("Don't understand levels in sex column; sex ignored.")
            }
        }
    }

    if(length(pgm.column)==0 || type=="4way") { # no pgm included
        pgm <- NULL
    }
    else {
        if(length(pgm.column)>1)
            warning("'pgm' included multiple times.  Using the first one.")
        temp <- cross$pheno[,pgm.column[1]]
        if(!is.numeric(temp))
            temp <- as.numeric(temp)-1
        if(any(!is.na(temp) & temp != 0 & temp != 1)) {
            warning("pgm column should be coded as 0/1; pgm ignored.")
            pgm <- NULL
        }
        else pgm <- temp
    }

    if(!is.null(sex) && any(is.na(sex))) {
        if(all(sex[!is.na(sex)]==1)) {
            warning(sum(is.na(sex)), " individuals with missing sex; assuming they're male like the others")
            sex[is.na(sex)] <- 1
        }
        else if(all(sex[!is.na(sex)]==0)) {
            warning(sum(is.na(sex)), " individuals with missing sex; assuming they're female like the others")
            sex[is.na(sex)] <- 0
        }
        else {
            warning(sum(is.na(sex)), " individuals with missing sex; assuming they're female")
            sex[is.na(sex)] <- 0
        }
    }

    if(!is.null(pgm) && any(is.na(pgm))) {
        if(all(pgm[!is.na(pgm)]==1)) {
            warning(sum(is.na(pgm)), " individuals with missing pgm; assuming pgm==1 like the others")
            pgm[is.na(pgm)] <- 1
        }
        else if(all(pgm[!is.na(pgm)]==0)) {
            warning(sum(is.na(pgm)), " individuals with missing pgm; assuming pgm==0 like the others")
            pgm[is.na(pgm)] <- 0
        }
        else {
            warning(sum(is.na(pgm)), " individuals with missing pgm; assuming pgm==0")
            pgm[is.na(pgm)] <- 0
        }
    }

    list(sex=sex,pgm=pgm)
}



# get names of genotypes
# used in discan, effectplot, plotPXG, scanone, scantwo, vbscan, reviseXdata
# cross.attr gives the cross attributes
getgenonames <-
    function(type=c("f2","bc","riself","risib","4way","dh","haploid","special","bcsft"),
             chrtype=c("A","X"), expandX=c("simple","standard","full"),
             sexpgm, cross.attr)
{
    type <- match.arg(type)
    chrtype <- match.arg(chrtype)
    expandX <- match.arg(expandX)

    ## Treat bcsft as bc if no intercross generations; otherwise as f2.
    if(type == "bcsft") {
        if(cross.attr$scheme[2] == 0)
            type <- "bc"
        else
            type <- "f2"
    }

    if(chrtype=="X") {
        sex <- sexpgm$sex
        pgm <- sexpgm$pgm
    }

    if(type=="special") return(cross.attr$genotypes)

    if(missing(cross.attr) || !("alleles" %in% names(cross.attr))) {
        if(type == "4way") alleles <- LETTERS[1:4]
        else alleles <- LETTERS[1:2]
    }
    else {
        alleles <- cross.attr$alleles
    }
    if(type=="4way") { # ensure that we have enough allele codes
        if(length(alleles) < 4) alleles <- LETTERS[1:4]
    } else {
        if(length(alleles) < 2) alleles <- LETTERS[1:2]
    }

    tempgn <- c(paste(rep(alleles[1],2),collapse=""),
                paste(alleles,collapse=""),
                paste(rep(alleles[2],2),collapse=""),
                paste(alleles[1],"Y",sep=""),
                paste(alleles[2],"Y",sep=""))

    # get rid of missing sex and pgm values, if there are any
    if(chrtype=="X") {
        if(length(sex)>0) sex <- sex[!is.na(sex)]
        if(length(pgm)>0) pgm <- pgm[!is.na(pgm)]
    }

    if(type=="riself" || type=="risib" || type=="dh")
        gen.names <- tempgn[c(1,3)]

    else if(type=="haploid")
        gen.names <- alleles

    else if(type == "4way") {
        if(chrtype=="A")
            gen.names <- c(paste(alleles[1],alleles[3],sep=""),
                           paste(alleles[2],alleles[3],sep=""),
                           paste(alleles[1],alleles[4],sep=""),
                           paste(alleles[2],alleles[4],sep=""))
        else
            gen.names <- c(paste(alleles[1],alleles[3],sep=""),
                           paste(alleles[2],alleles[3],sep=""),
                           paste(alleles[1],"Y",sep=""),
                           paste(alleles[2],"Y",sep=""))
    }

    else if(type == "bc") {

        if(chrtype=="A") # autosome
            gen.names <- tempgn[1:2] # AA AB

        else { # X chromosome

            #                 simple     standard       full
            #   -both sexes   A-/AB/BY   AA/AB/AY/BY    same as std
            #   -all females  AA/AB      same           same
            #   -all males    AY/BY      same           same

            if(length(sex)==0 || all(sex==0)) # all females
                gen.names <- tempgn[1:2] # AA AB
            else if(all(sex==1)) # all males
                gen.names <- tempgn[4:5] # AY BY
            else { # some of each
                if(expandX == "simple")
                    gen.names <- c(paste(alleles[1], "-", sep=""),
                                   tempgn[c(2,5)]) # A-, AB, BY
                else gen.names <- tempgn[c(1,2,4,5)]  # AA,AB,AY,BY
            }
        }
    }

    else { # intercross
        if(chrtype == "A")  # autosomal
            gen.names <- tempgn[1:3]
        else { # X chromsome

            # both crosses     simple     standard         full
            #   -both sexes   A-/AB/B-    AA/AB/BB/AY/BY   AA/AB1/AB2/BB/AY/BY
            #   -all females  AA/AB/BB    same as simple   AA/AB1/AB2/BB
            #   -all males    AY/BY       same             same
            # forw cross
            #   -both sexes   A-/AB/BY    AA/AB/AY/BY      same as std
            #   -all females  AA/AB       same             same
            #   -all males    AY/BY       same             same
            # backw cross
            #   -both sexes   B-/AB/AY    BB/AB/AY/BY      same as std
            #   -all females  BB/AB       same             same
            #   -all males    AY/BY       same             same

            if(length(sex)==0 || all(sex==0)) { # all females
                if(length(pgm)==0 || all(pgm==0)) # all forw dir
                    gen.names <- tempgn[1:2] # AA AB
                else if(all(pgm==1))  # all backw dir
                    gen.names <- tempgn[3:2] # BB AB
                else { # some of each direction
                    if(expandX=="full")
                        gen.names <- c(tempgn[1],
                                       paste(tempgn[2],c("f","r"), sep=""),
                                       tempgn[3])
                    else gen.names <- tempgn[1:3]
                }
            }
            else if(all(sex==1))  # all males
                gen.names <- tempgn[4:5]
            else { # some of each sex
                if(length(pgm)==0 || all(pgm==0)) { # all forw
                    if(expandX=="simple")
                        gen.names <- c(paste(alleles[1],"-", sep=""),
                                       tempgn[c(2,5)])
                    else gen.names <- tempgn[c(1,2,4,5)]
                }
                else if (all(pgm==1)) { # all backw
                    if(expandX=="simple")
                        gen.names <- c(paste(alleles[2], "-",sep=""),
                                       tempgn[c(2,4)])
                    else gen.names <- tempgn[c(3,2,4,5)]
                }
                else { # some of each dir
                    if(expandX=="simple")
                        gen.names <- c(paste(alleles[1],"-",sep=""),
                                       tempgn[2],
                                       paste(alleles[2],"-",sep=""))
                    else if(expandX=="standard")
                        gen.names <- tempgn
                    else
                        gen.names <- c(tempgn[1],
                                       paste(tempgn[2],c("f","r"),sep=""),
                                       tempgn[3:5])
                }
            }
        }
    }

    gen.names
}

# revise genotype data, probabilities or imputations for the X chromosome
reviseXdata <-
    function(type=c("f2","bc","bcsft"), expandX=c("simple","standard","full"),
             sexpgm, geno, prob, draws, pairprob, cross.attr, force=FALSE)
{
    type <- match.arg(type)
    expandX <- match.arg(expandX)

    ## Treat bcsft as bc if no intercross generations; otherwise as f2.
    if(type == "bcsft") {
        if(cross.attr$scheme[2] == 0)
            type <- "bc"
        else
            type <- "f2"
    }

    sex <- sexpgm$sex
    pgm <- sexpgm$pgm

    notmissing <- (!missing(geno)) + (!missing(prob)) + (!missing(draws)) +
        (!missing(pairprob))
    if(notmissing == 0)
        stop("Provide one of geno, prob, draws, pairprob.")
    if(notmissing > 1)
        stop("Provide just one of geno, prob, draws, pairprob.")

    # get genonames
    genonames <- getgenonames(type, "X", expandX, sexpgm, cross.attr)

    if(type == "bc") { # backcross

        if(length(sex)==0 || ((all(sex==0) || all(sex==1)) && !force)) { # all one sex
            # no changes necessary
            if(!missing(geno)) return(geno)
            else if(!missing(prob)) {
                dimnames(prob)[[3]] <- genonames
                return(prob)
            }
            else if(!missing(draws))
                return(draws)
            else # pairprob
                return(pairprob)
        }

        else { # both sexes

            if(!missing(geno)) {
                gmale <- geno[sex==1,]
                if(expandX=="simple")
                    gmale[!is.na(gmale) & gmale==2] <- 3
                else {
                    gmale[!is.na(gmale) & gmale==1] <- 3
                    gmale[!is.na(gmale) & gmale==2] <- 4
                }
                geno[sex==1,] <- gmale
                return(geno)
            }

            else if(!missing(draws)) {
                gmale <- draws[sex==1,,]
                if(expandX=="simple")
                    gmale[gmale==2] <- 3
                else {
                    gmale[gmale==1] <- 3
                    gmale[gmale==2] <- 4
                }
                draws[sex==1,,] <- gmale
                return(draws)
            }

            else if(!missing(prob)) {
                dimprob <- dim(prob)
                dimprob[3] <- length(genonames)
                newprob <- array(0,dim=dimprob)
                dimnames(newprob) <- c(dimnames(prob)[1:2],list(genonames))
                newprob[sex==0,,1:2] <- prob[sex==0,,1:2]

                if(expandX=="simple") {
                    newprob[sex==1,,1] <- prob[sex==1,,1]
                    newprob[sex==1,,3] <- prob[sex==1,,2]
                }
                else {
                    newprob[sex==1,,3] <- prob[sex==1,,1]
                    newprob[sex==1,,4] <- prob[sex==1,,2]
                }
                return(newprob)
            }

            else { # pairprob
                dimpairprob <- dim(pairprob)
                dimpairprob[3] <- dimpairprob[4] <- length(genonames)
                newpairprob <- array(0,dim=dimpairprob)
                newpairprob[sex==0,,1:2,1:2] <- pairprob[sex==0,,,]

                if(expandX=="simple") {
                    newpairprob[sex==1,,1,1] <- pairprob[sex==1,,1,1]
                    newpairprob[sex==1,,1,3] <- pairprob[sex==1,,1,2]
                    newpairprob[sex==1,,3,1] <- pairprob[sex==1,,2,1]
                    newpairprob[sex==1,,3,3] <- pairprob[sex==1,,2,2]
                }
                else {
                    newpairprob[sex==1,,3,3] <- pairprob[sex==1,,1,1]
                    newpairprob[sex==1,,3,4] <- pairprob[sex==1,,1,2]
                    newpairprob[sex==1,,4,3] <- pairprob[sex==1,,2,1]
                    newpairprob[sex==1,,4,4] <- pairprob[sex==1,,2,2]
                }
                return(newpairprob)
            }

        } # end of "both sexes" / backcross

    } # end of backcross

    else { # intercross

        if(length(sex)==0 || all(sex==0)) { # all females

            if(length(pgm)==0 || ((all(pgm==0) || all(pgm==1)) && !force)) { # one dir, females
                if(!missing(geno)) return(geno)
                else if(!missing(draws)) return(draws)
                else if(!missing(pairprob)) return(pairprob)
                else {
                    dimnames(prob)[[3]] <- genonames
                    return(prob)
                }
            }

            else { # both dir, females
                if(!missing(geno)) {
                    gback <- geno[pgm==1,]
                    if(expandX!="full") {
                        gback[!is.na(gback) & gback==1] <- 3
                        geno[pgm==1,] <- gback
                    }
                    else {
                        gback[!is.na(gback) & gback==1] <- 4
                        gback[!is.na(gback) & gback==2] <- 3
                        geno[pgm==1,] <- gback
                    }
                    return(geno)
                }
                else if(!missing(draws)) {
                    gback <- draws[pgm==1,,]
                    if(expandX!="full") {
                        gback[!is.na(gback) & gback==1] <- 3
                    }
                    else {
                        gback[!is.na(gback) & gback==1] <- 4
                        gback[!is.na(gback) & gback==2] <- 3
                    }
                    draws[pgm==1,,] <- gback
                    return(draws)
                }
                else if(!missing(prob)) {
                    dimprob <- dim(prob)
                    dimprob[3] <- length(genonames)
                    newprob <- array(0,dim=dimprob)
                    dimnames(newprob) <- c(dimnames(prob)[1:2],list(genonames))
                    newprob[pgm==0,,1:2] <- prob[pgm==0,,1:2]

                    if(expandX!="full") { # simple/standard
                        newprob[pgm==1,,3] <- prob[pgm==1,,1]
                        newprob[pgm==1,,2] <- prob[pgm==1,,2]
                    }
                    else {
                        newprob[pgm==1,,4] <- prob[pgm==1,,1]
                        newprob[pgm==1,,3] <- prob[pgm==1,,2]
                    }
                    return(newprob)
                }
                else { # pairprob
                    dimpairprob <- dim(pairprob)
                    dimpairprob[3] <- dimpairprob[4] <- length(genonames)
                    newpairprob <- array(0,dim=dimpairprob)
                    newpairprob[pgm==0,,1:2,1:2] <- pairprob[pgm==0,,,]

                    if(expandX!="full") { # simple/standard
                        newpairprob[pgm==1,,3,3] <- pairprob[pgm==1,,1,1]
                        newpairprob[pgm==1,,3,2] <- pairprob[pgm==1,,1,2]
                        newpairprob[pgm==1,,2,3] <- pairprob[pgm==1,,2,1]
                        newpairprob[pgm==1,,2,2] <- pairprob[pgm==1,,2,2]
                    }
                    else {
                        newpairprob[pgm==1,,4,4] <- pairprob[pgm==1,,1,1]
                        newpairprob[pgm==1,,4,3] <- pairprob[pgm==1,,1,2]
                        newpairprob[pgm==1,,3,4] <- pairprob[pgm==1,,2,1]
                        newpairprob[pgm==1,,3,3] <- pairprob[pgm==1,,2,2]
                    }
                    return(newpairprob)
                }
            }
        }
        else if(all(sex==1) && !force)  { # all males
            if(!missing(geno)) return(geno)
            else if(!missing(draws)) return(draws)
            else if(!missing(pairprob)) return(pairprob)
            else {
                dimnames(prob)[[3]] <- genonames
                return(prob)
            }
        }

        else { # both sexes

            if(length(pgm)==0 || all(pgm==0)) { # both sexes, forw dir
                if(!missing(geno)) {
                    gmale <- geno[sex==1,]
                    if(expandX=="simple")
                        gmale[!is.na(gmale) & gmale==2] <- 3
                    else {
                        gmale[!is.na(gmale) & gmale==1] <- 3
                        gmale[!is.na(gmale) & gmale==2] <- 4
                    }
                    geno[sex==1,] <- gmale
                    return(geno)
                }

                else if(!missing(draws)) {
                    gmale <- draws[sex==1,,]
                    if(expandX=="simple")
                        gmale[gmale==2] <- 3
                    else {
                        gmale[gmale==1] <- 3
                        gmale[gmale==2] <- 4
                    }
                    draws[sex==1,,] <- gmale
                    return(draws)
                }

                else if(!missing(prob)) {
                    dimprob <- dim(prob)
                    dimprob[3] <- length(genonames)
                    newprob <- array(0,dim=dimprob)
                    dimnames(newprob) <- c(dimnames(prob)[1:2],list(genonames))
                    newprob[sex==0,,1:2] <- prob[sex==0,,1:2]

                    if(expandX=="simple") {
                        newprob[sex==1,,1] <- prob[sex==1,,1]
                        newprob[sex==1,,3] <- prob[sex==1,,2]
                    }
                    else {
                        newprob[sex==1,,3] <- prob[sex==1,,1]
                        newprob[sex==1,,4] <- prob[sex==1,,2]
                    }
                    return(newprob)
                }

                else { # pairprob
                    dimpairprob <- dim(pairprob)
                    dimpairprob[3] <- dimpairprob[4] <- length(genonames)
                    newpairprob <- array(0,dim=dimpairprob)
                    newpairprob[sex==0,,1:2,1:2] <- pairprob[sex==0,,,]

                    if(expandX=="simple") {
                        newpairprob[sex==1,,1,1] <- pairprob[sex==1,,1,1]
                        newpairprob[sex==1,,1,3] <- pairprob[sex==1,,1,2]
                        newpairprob[sex==1,,3,1] <- pairprob[sex==1,,2,1]
                        newpairprob[sex==1,,3,3] <- pairprob[sex==1,,2,2]
                    }
                    else {
                        newpairprob[sex==1,,3,3] <- pairprob[sex==1,,1,1]
                        newpairprob[sex==1,,3,4] <- pairprob[sex==1,,1,2]
                        newpairprob[sex==1,,4,3] <- pairprob[sex==1,,2,1]
                        newpairprob[sex==1,,4,4] <- pairprob[sex==1,,2,2]
                    }
                    return(newpairprob)
                }
            } # both sexes, forw dir

            if(all(pgm==1) && !force) { # both sexes, backw dir
                if(!missing(geno)) {
                    gmale <- geno[sex==1,]
                    if(expandX=="simple") {
                        gmale[!is.na(gmale) & gmale==1] <- 3
                        gmale[!is.na(gmale) & gmale==2] <- 1
                    }
                    else {
                        gmale[!is.na(gmale) & gmale==1] <- 3
                        gmale[!is.na(gmale) & gmale==2] <- 4
                    }
                    geno[sex==1,] <- gmale
                    return(geno)
                }

                else if(!missing(draws)) {
                    gmale <- draws[sex==1,,]
                    if(expandX=="simple") {
                        gmale[gmale==1] <- 3
                        gmale[gmale==2] <- 1
                    }
                    else {
                        gmale[gmale==1] <- 3
                        gmale[gmale==2] <- 4
                    }
                    draws[sex==1,,] <- gmale
                    return(draws)
                }

                else if(!missing(prob)) {
                    dimprob <- dim(prob)
                    dimprob[3] <- length(genonames)
                    newprob <- array(0,dim=dimprob)
                    dimnames(newprob) <- c(dimnames(prob)[1:2],list(genonames))
                    newprob[sex==0,,1:2] <- prob[sex==0,,1:2]

                    if(expandX=="simple") {
                        newprob[sex==1,,3] <- prob[sex==1,,1]
                        newprob[sex==1,,1] <- prob[sex==1,,2]
                    }
                    else {
                        newprob[sex==1,,3] <- prob[sex==1,,1]
                        newprob[sex==1,,4] <- prob[sex==1,,2]
                    }
                    return(newprob)
                }

                else { # pairprob
                    dimpairprob <- dim(pairprob)
                    dimpairprob[3] <- dimpairprob[4] <- length(genonames)
                    newpairprob <- array(0,dim=dimpairprob)
                    newpairprob[sex==0,,1:2,1:2] <- pairprob[sex==0,,,]

                    if(expandX=="simple") {
                        newpairprob[sex==1,,3,3] <- pairprob[sex==1,,1,1]
                        newpairprob[sex==1,,1,3] <- pairprob[sex==1,,2,1]
                        newpairprob[sex==1,,3,1] <- pairprob[sex==1,,1,2]
                        newpairprob[sex==1,,1,1] <- pairprob[sex==1,,2,2]
                    }
                    else {
                        newpairprob[sex==1,,3,3] <- pairprob[sex==1,,1,1]
                        newpairprob[sex==1,,3,4] <- pairprob[sex==1,,1,2]
                        newpairprob[sex==1,,4,3] <- pairprob[sex==1,,2,1]
                        newpairprob[sex==1,,4,4] <- pairprob[sex==1,,2,2]
                    }
                    return(newpairprob)
                }
            } # both sexes, backw dir

            else { # both dir, both sexes

                if(!missing(geno)) {
                    gmale <- geno[sex==1,]
                    gfemaler <- geno[sex==0 & pgm==1,]
                    if(expandX=="simple") {
                        gmale[!is.na(gmale) & gmale==2] <- 3
                        gfemaler[!is.na(gfemaler) & gfemaler==1] <- 3
                    }
                    else if(expandX=="standard") {
                        gmale[!is.na(gmale) & gmale==1] <- 4
                        gmale[!is.na(gmale) & gmale==2] <- 5
                        gfemaler[!is.na(gfemaler) & gfemaler==1] <- 3
                    }
                    else {
                        gmale[!is.na(gmale) & gmale==1] <- 5
                        gmale[!is.na(gmale) & gmale==2] <- 6
                        gfemaler[!is.na(gfemaler) & gfemaler==1] <- 4
                        gfemaler[!is.na(gfemaler) & gfemaler==2] <- 3
                    }
                    geno[sex==1,] <- gmale
                    geno[sex==0 & pgm==1,] <- gfemaler
                    return(geno)
                }

                else if(!missing(draws)) {
                    gmale <- draws[sex==1,,]
                    gfemaler <- draws[sex==0 & pgm==1,,]
                    if(expandX=="simple") {
                        gmale[gmale==2] <- 3
                        gfemaler[gfemaler==1] <- 3
                    }
                    else if(expandX=="standard") {
                        gmale[gmale==1] <- 4
                        gmale[gmale==2] <- 5
                        gfemaler[gfemaler==1] <- 3
                    }
                    else {
                        gmale[gmale==1] <- 5
                        gmale[gmale==2] <- 6
                        gfemaler[gfemaler==1] <- 4
                        gfemaler[gfemaler==2] <- 3
                    }
                    draws[sex==1,,] <- gmale
                    draws[sex==0 & pgm==1,,] <- gfemaler
                    return(draws)
                }

                else if(!missing(prob)) {
                    dimprob <- dim(prob)
                    dimprob[3] <- length(genonames)
                    newprob <- array(0,dim=dimprob)
                    dimnames(newprob) <- c(dimnames(prob)[1:2],list(genonames))
                    newprob[sex==0 & pgm==0,,1:2] <- prob[sex==0 & pgm==0,,1:2]

                    if(expandX=="simple") {
                        newprob[sex==1,,1] <- prob[sex==1,,1]
                        newprob[sex==1,,3] <- prob[sex==1,,2]
                        newprob[sex==0 & pgm==1,,3] <- prob[sex==0 & pgm==1,,1]
                        newprob[sex==0 & pgm==1,,2] <- prob[sex==0 & pgm==1,,2]
                    }
                    else if(expandX=="standard") {
                        newprob[sex==1,,4] <- prob[sex==1,,1]
                        newprob[sex==1,,5] <- prob[sex==1,,2]
                        newprob[sex==0 & pgm==1,,3] <- prob[sex==0 & pgm==1,,1]
                        newprob[sex==0 & pgm==1,,2] <- prob[sex==0 & pgm==1,,2]
                    }
                    else {
                        newprob[sex==1,,5] <- prob[sex==1,,1]
                        newprob[sex==1,,6] <- prob[sex==1,,2]
                        newprob[sex==0 & pgm==1,,4] <- prob[sex==0 & pgm==1,,1]
                        newprob[sex==0 & pgm==1,,3] <- prob[sex==0 & pgm==1,,2]
                    }
                    return(newprob)
                }

                else { # pairprob
                    dimpairprob <- dim(pairprob)
                    dimpairprob[3] <- dimpairprob[4] <- length(genonames)
                    newpairprob <- array(0,dim=dimpairprob)
                    newpairprob[sex==0 & pgm==0,,1:2,1:2] <- pairprob[sex==0 & pgm==0,,,]

                    male <- (sex==1)
                    femaler <- (sex==0) & (pgm==1)
                    if(expandX=="simple") {
                        newpairprob[male,,1,1] <- pairprob[male,,1,1]
                        newpairprob[male,,1,3] <- pairprob[male,,1,2]
                        newpairprob[male,,3,1] <- pairprob[male,,2,1]
                        newpairprob[male,,3,3] <- pairprob[male,,2,2]

                        newpairprob[femaler,,3,3] <- pairprob[femaler,,1,1]
                        newpairprob[femaler,,3,2] <- pairprob[femaler,,1,2]
                        newpairprob[femaler,,2,3] <- pairprob[femaler,,2,1]
                        newpairprob[femaler,,2,2] <- pairprob[femaler,,2,2]
                    }
                    else if(expandX=="standard") {
                        newpairprob[male,,4,4] <- pairprob[male,,1,1]
                        newpairprob[male,,4,5] <- pairprob[male,,1,2]
                        newpairprob[male,,5,4] <- pairprob[male,,2,1]
                        newpairprob[male,,5,5] <- pairprob[male,,2,2]

                        newpairprob[femaler,,3,3] <- pairprob[femaler,,1,1]
                        newpairprob[femaler,,3,2] <- pairprob[femaler,,1,2]
                        newpairprob[femaler,,2,3] <- pairprob[femaler,,2,1]
                        newpairprob[femaler,,2,2] <- pairprob[femaler,,2,2]
                    }
                    else {
                        newpairprob[male,,5,5] <- pairprob[male,,1,1]
                        newpairprob[male,,5,6] <- pairprob[male,,1,2]
                        newpairprob[male,,6,5] <- pairprob[male,,2,1]
                        newpairprob[male,,6,6] <- pairprob[male,,2,2]

                        newpairprob[femaler,,4,4] <- pairprob[femaler,,1,1]
                        newpairprob[femaler,,4,3] <- pairprob[femaler,,1,2]
                        newpairprob[femaler,,3,4] <- pairprob[femaler,,2,1]
                        newpairprob[femaler,,3,3] <- pairprob[femaler,,2,2]
                    }
                    return(newpairprob)
                }

            }
        }

    } # end of intercross

}

######################################################################
# scanoneXnull
#
# figure out null hypothesis business for scanone on X chromosome
######################################################################
scanoneXnull <-
    function(type, sexpgm, cross.attr)
{
    sex <- sexpgm$sex
    pgm <- sexpgm$pgm

    if(type == "risib" || type=="riself" || type=="dh" || type=="haploid") type <- "bc"

    if(type == "bcsft") {
        if(cross.attr$scheme[2] == 0)
            type <- "bc"
        else
            type <- "f2"
    }

    ### first figure out sex/pgm pattern

    # sex
    if(length(sex)==0 || all(sex==0)) { # all female
        onesex <- allfemale <- TRUE
    }
    else if(all(sex==1)) { # all male
        onesex <- TRUE
        allfemale <- FALSE
    }
    else { # both sexes
        onesex <- allfemale <- FALSE
    }
    # pgm
    if(length(pgm)==0 || all(pgm==0) || all(pgm==1)) # one direction
        onedir <- TRUE
    else onedir <- FALSE

    allmale <- onesex && !allfemale
    bothsex <- !onesex
    bothdir <- !onedir


    ### now figure out the null hypothesis and pull out appropriate
    ### covariates for the null

    # backcross, one sex
    # OR intercross, one dir and one sex
    # OR intercross, both dir and all male
    if((type=="bc" && onesex) ||
       (type=="f2" && ((onedir && onesex) || (bothdir && allmale)))) {
        adjustX <- FALSE
        parX0 <- 1
        sexpgmcovar <- sexpgmcovar.alt <- NULL
    }

    # backcross, both sexes
    # OR intercross, one direction and both sexes
    else if((type=="bc" && bothsex) ||
            (type=="f2" && onedir && bothsex)) {
        adjustX <- TRUE
        parX0 <- 2
        sexpgmcovar <- cbind(sex)
        sexpgmcovar.alt <- sex+1
    }

    # intercross, both dir and all female
    else if(type=="f2" && bothdir && allfemale) {
        adjustX <- TRUE
        parX0 <- 2
        sexpgmcovar <- cbind(pgm)
        sexpgmcovar.alt <- pgm+1
    }

    # intercross, both dir and both sexes
    else {
        adjustX <- TRUE
        parX0 <- 3
        sexpgmcovar <- cbind(sex,as.numeric(sex==0 & pgm==1))
        sexpgmcovar.alt <- rep(3,length(sex))
        sexpgmcovar.alt[sex==0 & pgm==0] <- 1
        sexpgmcovar.alt[sex==0 & pgm==1] <- 2
    }

    list(adjustX=adjustX, parX0=parX0, sexpgmcovar=sexpgmcovar,
         sexpgmcovar.alt=sexpgmcovar.alt)
}

######################################################################
# revisecovar
#
# Drop sex and pgm and their interxn as covariates for the X chr.
######################################################################
revisecovar <-
    function(sexpgm, covar)
{
    if(is.null(covar) || (is.null(sexpgm$sex) && is.null(sexpgm$pgm))) {
        if(!is.null(covar)) attr(covar, "n.dropped") <- 0
        return(covar)
    }

    covar <- as.matrix(covar)

    sex <- sexpgm$sex
    pgm <- sexpgm$pgm

    if(!is.null(pgm) && length(unique(pgm))==1) pgm <- NULL
    allfemale <- FALSE
    if(is.null(sex)) allfemale <- TRUE
    else {
        if(all(sex==0)) {
            allfemale <- TRUE
            sex <- NULL
        }
        else if(all(sex==1)) {
            allfemale <- FALSE
            sex <- NULL
        }
    }

    if(!is.null(pgm)) { # some of each direction
        if(!is.null(sex)) { # some of each sex
            femf <- as.numeric(pgm==0 & sex==0)
            femr <- as.numeric(pgm==1 & sex==0)
            mal <- sex
            X <- cbind(femf, femr, mal)
        }
        else { # all of one sex
            if(allfemale)
                X <- cbind(1-pgm, pgm)
            else
                X <- cbind(rep(1, nrow(covar)))
        }
    }
    else { # all of one direction
        if(!is.null(sex))  # some of each sex
            X <- cbind(sex, 1-sex)
        else X <- cbind(rep(1, nrow(covar)))
    }

    nc <- ncol(X)

    keep <- rep(TRUE,ncol(covar))
    for(i in 1:ncol(covar)) {
        if(qr(cbind(X,covar[,i]))$rank <= nc)
            keep[i] <- FALSE
    }
    if(!any(keep))
        covar <- numeric(0)
    else
        covar <- covar[,keep,drop=FALSE]

    attr(covar, "n.dropped") <- sum(!keep)
    covar
}

######################################################################
# dropXcol: for use with scantwo() for the X chromosome:
#           figure out what columns to drop...both for the full model
#           and for the additive model.
######################################################################
dropXcol <-
    function(type=c("f2","bc", "riself", "risib", "4way", "dh", "haploid", "special","bcsft"),
             sexpgm, cross.attr)
{
    type <- match.arg(type)

    ## Treat bcsft as bc if no intercross generations; otherwise as f2.
    if(type == "bcsft") {
        if(cross.attr$scheme[2] == 0)
            type <- "bc"
        else
            type <- "f2"
    }

    gn <- getgenonames(type, "X", "full", sexpgm, cross.attr)

    if(length(gn)==2) return(rep(0,4))

    if(length(gn)==4) return( c(0,0,0,0,0,1,0,  0,1,1,1,1,1,1,1,0) )

    if(length(gn)==6) {
        todrop <- c(rep(0,11), rep(1,25))
        todrop[c(8,10)] <- 1
        todrop[11+c(1,13,25)] <- 0
        return(todrop)
    }

    return(rep(0,length(gn)^2))
}


# end of xchr.R

#####################################################################
#
# util.R
#
# copyright (c) 2001-2022, Karl W Broman
#     [find.pheno, find.flanking, and a modification to create.map
#      from Brian Yandell]
# last modified Oct, 2022
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: markernames, c.cross, create.map, reduce2grid,
#           clean, clean.cross, drop.nullmarkers, nullmarkers
#           drop.markers, pull.markers, drop.dupmarkers
#           geno.table, genotab.em
#           mf.k, mf.h, imf.k, imf.h, mf.cf, imf.cf, mf.m, imf.m,
#           mf.stahl, imf.stahl
#           switch.order, flip.order, makeSSmap,
#           subset.cross, fill.geno, checkcovar, find.marker,
#           find.pseudomarker,
#           adjust.rf.ri, lodint, bayesint,
#           comparecrosses, movemarker, summary.map (aka summaryMap),
#           print.summary.map, find.pheno,
#           convert, convert.scanone, convert.scantwo
#           find.flanking, strip.partials,
#           qtlversion, locateXO, jittermap, getid,
#           find.markerpos, geno.crosstab, LikePheVector,
#           matchchr, convert2sa, charround, testchr,
#           scantwoperm2scanoneperm, subset.map, [.map, [.cross,
#           findDupMarkers, convert2riself, convert2risib,
#           switchAlleles, nqrank, cleanGeno, typingGap,
#           calcPermPval, phenames, updateParallelRNG
#
######################################################################

######################################################################
#
# markernames
#
# pull out the marker names for selected chromosomes as one big vector
#
######################################################################

markernames <-
    function(cross, chr)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(!missing(chr)) cross <- subset(cross, chr=chr)
    temp <- unlist(lapply(cross$geno, function(a) colnames(a$data)))
    names(temp) <- NULL
    temp
}

######################################################################
#
# chrnames
#
# pull out the chrnames for a cross
#
######################################################################

chrnames <-
    function(cross)
{
    names(cross$geno)
}

######################################################################
#
# create.map
#
# create a new map with inserted inter-marker locations
#
# Note: map is a vector or a matrix with 2 rows
#
# stepwidth = "fixed" is the original R/qtl version;
# stepwidth="variable" is for Brian Yandell and the qtlbim package
# stepwidth="max" creates the minimal number of inserted pseudomarkers
#                 to have the maximum stepwidth = step
######################################################################
create.map <-
    function(map, step, off.end, stepwidth = c("fixed", "variable", "max"))
{
    stepwidth <- match.arg(stepwidth)
    if(step<0 || off.end<0) stop("step and off.end must be > 0.")

    if(!is.matrix(map)) { # sex-ave map
        if(stepwidth == "variable") {
            if(off.end > 0) {
                tmp <- names(map)
                ## Append and prepend by off.end value (exact here, no roundoff).
                map <- c(map[1] - off.end, map, map[length(map)] + off.end)
                names(map) <- c("loc000", tmp, "loc999")
            }
            if(step == 0)
                return(unclass(map))

            ## Determine differences and expansion vector.
            dif <- diff(map)
            expand <- pmax(1, floor(dif / step))

            ## Create pseudomarker map.
            a <- min(map) + cumsum(c(0, rep(dif / expand, expand)))

            ## Names are marker names or locNNN.
            namesa <- paste("loc", seq(length(a)), sep = "")
            namesa[cumsum(c(1, expand))] <- names(map)
            names(a) <- namesa

            return(unclass(a))
        }
        if(stepwidth == "max") {
            if(off.end > 0) {
                toadd <- c(map[1] - off.end, map[length(map)]+off.end)

                if(step==0) {
                    names(toadd) <- paste("loc", 1:2, sep="")
                    map <- sort(c(map, toadd))
                    return(unclass(map))
                }

                nmap <- c(map[1] - off.end, map, map[length(map)]+off.end)
            }
            else {
                nmap <- map
                toadd <- NULL
            }

            if(step==0 || (length(map)==1 && off.end==0)) return(unclass(map))

            d <- diff(nmap)
            nadd <- ceiling(d/step)-1
            if(sum(nadd) > 0) {
                for(j in 1:(length(nmap)-1)) {
                    if(nadd[j]>0)
                        toadd <- c(toadd, seq(nmap[j], nmap[j+1], len=nadd[j]+2)[-c(1,nadd[j]+2)])
                }
            }
            if(length(toadd) > 0)  {
                names(toadd) <- paste("loc", 1:length(toadd), sep="")
                map <- sort(c(map, toadd))
            }
            return(unclass(map))
        }

        if(length(map) == 1) { # just one marker!
            if(off.end==0) {
                if(step == 0) step <- 1
                nam <- names(map)
                map <- c(map,map+step)
                names(map) <- c(nam,paste("loc",step,sep=""))
            }
            else {
                if(step==0) m <- c(-off.end,off.end)
                else m <- seq(-off.end,off.end,by=step)
                m <- m[m!=0]
                names(m) <- paste("loc",m,sep="")
                map <- sort(c(m+map,map))
            }
            return(map)
        }

        minloc <- min(map)
        map <- map-minloc

        if(step==0 && off.end==0) return(map+minloc)
        else if(step==0 && off.end > 0) {
            a <- c(floor(min(map)-off.end),ceiling(max(map)+off.end))
            names(a) <- paste("loc", a, sep="")
            return(sort(c(a,map))+minloc)
        }
        else if(step>0 && off.end == 0) {
            a <- seq(floor(min(map)),max(map),
                     by = step)
            if(any(is.na(match(a, map)))) {
                a <- a[is.na(match(a,map))]
                names(a) <- paste("loc",a,sep="")
                return(sort(c(a,map))+minloc)
            }
            else return(map+minloc)
        }
        else {
            a <- seq(floor(min(map)-off.end),ceiling(max(map)+off.end+step),
                     by = step)
            a <- a[is.na(match(a,map))]

            # no more than one point above max(map)+off.end
            z <- (seq(along=a))[a >= max(map)+off.end]
            if(length(z) > 1) a <- a[-z[-1]]

            names(a) <- paste("loc",a,sep="")
            return(sort(c(a,map))+minloc)
        }
    } # end sex-ave map
    else { # sex-specific map
        if(stepwidth == "variable") {
            if(off.end > 0) {
                tmp <- colnames(map)
                map <- cbind(map[, 1] - off.end, map, map[, ncol(map)] + off.end)
                dimnames(map) <- list(NULL, c("loc000", tmp, "loc999"))
            }
            if(step == 0)
                return(unclass(map))

            ## Determine differences and expansion vector.
            dif <- diff(map[1, ])
            expand <- pmax(1, floor(dif / step))

            ## Create pseudomarker map.
            a <- min(map[1, ]) + cumsum(c(0, rep(dif / expand, expand)))
            b <- min(map[2, ]) + cumsum(c(0, rep(diff(map[2, ]) / expand, expand)))

            namesa <- paste("loc", seq(length(a)), sep = "")
            namesa[cumsum(c(1, expand))] <- dimnames(map)[[2]]
            map <- rbind(a,b)
            dimnames(map) <- list(NULL, namesa)

            return(unclass(map))
        }
        if(stepwidth == "max") {
            if(step==0 && off.end==0) return(unclass(map))
            if(step==0 && off.end>0) {
                if(ncol(map)==1) { # only one marker; assume equal recomb in sexes
                    L1 <- L2 <- 1
                }
                else {
                    L1 <- diff(range(map[1,]))
                    L2 <- diff(range(map[2,]))
                }

                nam <- colnames(map)
                nmap1 <- c(map[1,1]-off.end, map[1,], map[1,ncol(map)]+off.end)
                nmap2 <- c(map[2,1]-off.end*L2/L1, map[2,], map[2,ncol(map)]+off.end*L2/L1)
                map <- rbind(nmap1, nmap2)
                colnames(map) <- c("loc1", nam, "loc2")
                return(unclass(map))
            }

            if(ncol(map)==1) L1 <- L2 <- 1
            else {
                L1 <- diff(range(map[1,]))
                L2 <- diff(range(map[2,]))
            }

            nam <- colnames(map)

            if(off.end > 0) {
                toadd1 <- c(map[1,1] - off.end, map[1,ncol(map)]+off.end)
                toadd2 <- c(map[2,1] + off.end*L2/L1, map[2,ncol(map)]+off.end*L2/L1)

                neword <- order(c(map[1,], toadd1))
                nmap1 <- c(map[1,], toadd1)[neword]
                nmap2 <- c(map[2,], toadd2)[neword]
            }
            else {
                nmap1 <- map[1,]
                nmap2 <- map[2,]
                toadd1 <- toadd2 <- NULL
            }

            d <- diff(nmap1)
            nadd <- ceiling(d/step)-1
            if(sum(nadd) > 0) {
                for(j in 1:(length(nmap1)-1)) {
                    if(nadd[j]>0) {
                        toadd1 <- c(toadd1, seq(nmap1[j], nmap1[j+1], len=nadd[j]+2)[-c(1,nadd[j]+2)])
                        toadd2 <- c(toadd2, seq(nmap2[j], nmap2[j+1], len=nadd[j]+2)[-c(1,nadd[j]+2)])
                    }
                }
            }
            newnam <- paste("loc", 1:length(toadd1), sep="")

            toadd1 <- sort(toadd1)
            toadd2 <- sort(toadd2)
            neword <- order(c(map[1,], toadd1))
            nmap1 <- c(map[1,], toadd1)[neword]
            nmap2 <- c(map[2,], toadd2)[neword]
            map <- rbind(nmap1, nmap2)
            colnames(map) <- c(nam, newnam)[neword]

            return(unclass(map))
        }

        minloc <- c(min(map[1,]),min(map[2,]))
        map <- unclass(map-minloc)
        markernames <- colnames(map)

        if(step==0 && off.end==0) return(map+minloc)
        else if(step==0 && off.end > 0) {
            map <- map+minloc
            if(ncol(map)==1) { # only one marker; assume equal recomb in sexes
                L1 <- L2 <- 1
            }
            else {
                L1 <- diff(range(map[1,]))
                L2 <- diff(range(map[2,]))
            }

            nam <- colnames(map)
            nmap1 <- c(map[1,1]-off.end, map[1,], map[1,ncol(map)]+off.end)
            nmap2 <- c(map[2,1]-off.end*L2/L1, map[2,], map[2,ncol(map)]+off.end*L2/L1)
            map <- rbind(nmap1, nmap2)
            colnames(map) <- c("loc1", nam, "loc2")
            return(map)
        }
        else if(step>0 && off.end == 0) {

            if(ncol(map)==1) return(map+minloc)

            a <- seq(floor(min(map[1,])),max(map[1,]),
                     by = step)
            a <- a[is.na(match(a,map[1,]))]

            if(length(a)==0) return(map+minloc)

            b <- sapply(a,function(x,y,z) {
                ZZ <- min((seq(along=y))[y > x])
                (x-y[ZZ-1])/(y[ZZ]-y[ZZ-1])*(z[ZZ]-z[ZZ-1])+z[ZZ-1] }, map[1,],map[2,])

            m1 <- c(a,map[1,])
            m2 <- c(b,map[2,])

            names(m1) <- names(m2) <- c(paste("loc",a,sep=""),markernames)
            return(rbind(sort(m1),sort(m2))+minloc)
        }
        else {
            a <- seq(floor(min(map[1,])-off.end),ceiling(max(map[1,])+off.end+step),
                     by = step)
            a <- a[is.na(match(a,map[1,]))]
            # no more than one point above max(map)+off.end
            z <- (seq(along=a))[a >= max(map[1,])+off.end]
            if(length(z) > 1) a <- a[-z[-1]]

            b <- sapply(a,function(x,y,z,ml) {
                if(x < min(y)) {
                    return(min(z) - (min(y)-x)/diff(range(y))*diff(range(z)) - ml)
                }
                else if(x > max(y)) {
                    return(max(z) + (x - max(y))/diff(range(y))*diff(range(z)) - ml)
                }
                else {
                    ZZ <- min((seq(along=y))[y > x])
                    (x-y[ZZ-1])/(y[ZZ]-y[ZZ-1])*(z[ZZ]-z[ZZ-1])+z[ZZ-1]
                }
            }, map[1,],map[2,], minloc[2])
            m1 <- c(a,map[1,])
            m2 <- c(b,map[2,])
            names(m1) <- names(m2) <- c(paste("loc",a,sep=""),markernames)
            return(rbind(sort(m1),sort(m2))+minloc)
        }
    }
}

######################################################################
# reduce2grid
#
# for high-density marker data, rather than run scanone at both the
# markers and at a set of pseudomarkers, we could reduce to just
# a set of evenly-spaced pseudomarkers
#
# first run calc.genoprob (or sim.geno) and then use this.
######################################################################
reduce2grid <-
    function(cross)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    # sample one element from a vector
    sampleone <- function(x) ifelse(length(x)==1, x, sample(x, 1))

    # for a map containing a grid with a given step size,
    # find the grid min(map), min(map)+step, min(map)+2step, ...
    gridindex <- function(map, step) {
        if(is.matrix(map)) stop("reduce2grid isn't working for sex-specific maps")
        grid <- seq(min(map), max(map), by=step)
        index <- match(grid, map)
        if(any(is.na(index)))
            index <- sapply(grid, function(a,b) { d <- abs(a-b); sampleone(which(d == min(d))) }, map)
        index
    }

    attr2fix <- c("error.prob", "step", "off.end", "map.function", "stepwidth")

    reduced <- FALSE
    if("prob" %in% names(cross$geno[[1]])) {
        stepwidth <- attr(cross$geno[[1]]$prob, "stepwidth")
        if(stepwidth != "fixed") {
            warning("You need to have run calc.genoprob with stepwidth=\"fixed\".")
        }
        else {
            step <- attr(cross$geno[[1]]$prob, "step")

            for(i in 1:nchr(cross)) {
                pr <- cross$geno[[i]]$prob
                map <- attr(pr, "map")
                butes <- attributes(pr)

                reduced <- gridindex(map, step)
                pr <- pr[,reduced,,drop=FALSE]
                attr(pr, "map") <- map[reduced]
                for(a in attr2fix)
                    attr(pr, a) <- butes[[a]]

                attr(pr, "reduced2grid") <- TRUE
                cross$geno[[i]]$prob <- pr
            }

            reduced <- TRUE
        }
    }
    if("draws" %in% names(cross$geno[[1]])) {
        stepwidth <- attr(cross$geno[[1]]$draws, "stepwidth")
        if(stepwidth != "fixed") {
            warning("You need to have run sim.geno with stepwidth=\"fixed\".")
        }
        else {
            step <- attr(cross$geno[[1]]$draws, "step")

            for(i in 1:nchr(cross)) {
                dr <- cross$geno[[i]]$draws
                map <- attr(dr, "map")
                butes <- attributes(dr)

                reduced <- gridindex(map, step)
                dr <- dr[,reduced,,drop=FALSE]
                attr(dr, "map") <- map[reduced]
                for(a in attr2fix)
                    attr(dr, a) <- butes[[a]]

                attr(dr, "reduced2grid") <- TRUE
                cross$geno[[i]]$draws <- dr
            }

            reduced <- TRUE
        }
    }

    if(!reduced)
        warning("You first need to run calc.genoprob or sim.geno with stepwidth=\"fixed\".")

    cross
}


######################################################################
# clean functions
######################################################################
clean <-
    function(object, ...)
    UseMethod("clean")

######################################################################
#
# clean.cross
#
# remove all of the extraneous stuff from a cross object, to get back
# to just the data
#
######################################################################

clean.cross <-
    function(object, ...)
{
    if(!inherits(object, "cross"))
        stop("Input should have class \"cross\".")

    cross2 <- list(geno=object$geno,pheno=object$pheno)

    if("cross" %in% names(object))
        cross2$cross <- object$cross

    if("founderGeno" %in% names(object))
        cross2$founderGeno <- object$founderGeno

    if(!is.null(attr(object, "alleles")))
        attr(cross2, "alleles") <- attr(object, "alleles")

    if(!is.null(attr(object, "scheme")))
        attr(cross2, "scheme") <- attr(object, "scheme")

    for(i in 1:length(object$geno)) {
        cross2$geno[[i]] <- list(data=object$geno[[i]]$data,
                                 map=object$geno[[i]]$map)
        class(cross2$geno[[i]]) <- class(object$geno[[i]])
    }

    class(cross2) <- class(object)
    cross2
}


######################################################################
#
# drop.qtlgeno
#
# remove any QTLs from the genotype data and the genetic maps
# from data simulated via sim.cross. (They all have names "QTL*")
#
######################################################################

#drop.qtlgeno <-
#function(cross)
#{
#  n.chr <- nchr(cross)
#  mar.names <- lapply(cross$geno, function(a) {
#    m <- a$map
#    if(is.matrix(m)) return(colnames(m))
#    else return(names(m)) } )
#
#  for(i in 1:n.chr) {
#    o <- grep("^QTL[0-9]+",mar.names[[i]])
#    if(length(o) != 0) {
#      cross$geno[[i]]$data <- cross$geno[[i]]$data[,-o,drop=FALSE]
#      if(is.matrix(cross$geno[[i]]$map))
#        cross$geno[[i]]$map <- cross$geno[[i]]$map[,-o,drop=FALSE]
#      else
#        cross$geno[[i]]$map <- cross$geno[[i]]$map[-o]
#    }
#  }
#  cross
#}

######################################################################
#
# drop.nullmarkers
#
# remove markers that have no genotype data from the data matrix and
# genetic maps
#
######################################################################

drop.nullmarkers <-
    function(cross)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    n.chr <- nchr(cross)

    keep.chr <- rep(TRUE,n.chr)
    for(i in 1:n.chr) {
        o <- !apply(cross$geno[[i]]$data,2,function(a) sum(!is.na(a)))
        if(any(o)) { # remove from genotype data and map
            mn.drop <- colnames(cross$geno[[i]]$data)[o]
            if(length(mn.drop) == ncol(cross$geno[[i]]$data))
                keep.chr[i] <- FALSE # removing all markers from this chromosome

            cross$geno[[i]]$data <- cross$geno[[i]]$data[,!o,drop=FALSE]

            if(is.matrix(cross$geno[[i]]$map))
                cross$geno[[i]]$map <- cross$geno[[i]]$map[,!o,drop=FALSE]
            else
                cross$geno[[i]]$map <- cross$geno[[i]]$map[!o]

            # results of calc.genoprob
            if("prob" %in% names(cross$geno[[i]])) {
                o <- match(mn.drop,colnames(cross$geno[[i]]$prob))
                cross$geno[[i]]$prob <- cross$geno[[i]]$prob[,-o,,drop=FALSE]
            }

            # results of argmax.geno
            if("argmax" %in% names(cross$geno[[i]])) {
                o <- match(mn.drop,colnames(cross$geno[[i]]$argmax))
                cross$geno[[i]]$argmax <- cross$geno[[i]]$argmax[,-o,drop=FALSE]
            }

            # results of sim.geno
            if("draws" %in% names(cross$geno[[i]])) {
                o <- match(mn.drop,colnames(cross$geno[[i]]$draws))
                cross$geno[[i]]$draws <- cross$geno[[i]]$draws[,-o,,drop=FALSE]
            }

            # results of est.rf
            if("rf" %in% names(cross)) {
                attrib <- attributes(cross$rf)

                o <- match(mn.drop,colnames(cross$rf))
                cross$rf <- cross$rf[-o,-o]

                if("onlylod" %in% names(attrib)) # save the onlylod attribute if its there
                    attr(cross$rf, "onlylod") <- attrib$onlylod
            }
        }
    }
    cross$geno <- cross$geno[keep.chr]

    if("founderGeno" %in% names(cross))
        cross$founderGeno <- cross$founderGeno[,markernames(cross)]

    cross
}


######################################################################
#
# nullmarkers
#
# identify markers that have no genotype data from the data matrix and
# genetic maps
#
######################################################################

nullmarkers <-
    function(cross)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    n.chr <- nchr(cross)

    keep.chr <- rep(TRUE,n.chr)
    all2drop <- NULL
    for(i in 1:n.chr) {
        o <- !apply(cross$geno[[i]]$data,2,function(a) sum(!is.na(a)))
        if(any(o)) { # remove from genotype data and map
            mn.drop <- colnames(cross$geno[[i]]$data)[o]
            all2drop <- c(all2drop, mn.drop)
        }
    }

    all2drop
}

######################################################################
#
# drop.markers
#
# remove a vector of markers from the data matrix and genetic maps
#
######################################################################

drop.markers <-
    function(cross, markers)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    n.chr <- nchr(cross)

    keep.chr <- rep(TRUE,n.chr)
    found <- rep(FALSE, length(markers))
    for(i in 1:n.chr) {
        # find markers on this chromosome
        o <- match(markers,colnames(cross$geno[[i]]$data))
        found[!is.na(o)] <- TRUE
        o <- o[!is.na(o)]
        a <- rep(FALSE,ncol(cross$geno[[i]]$data))
        a[o] <- TRUE
        o <- a

        if(any(o)) { # remove from genotype data and map
            mn.drop <- colnames(cross$geno[[i]]$data)[o]
            if(length(mn.drop) == ncol(cross$geno[[i]]$data))
                keep.chr[i] <- FALSE # removing all markers from this chromosome

            cross$geno[[i]]$data <- cross$geno[[i]]$data[,!o,drop=FALSE]

            if(is.matrix(cross$geno[[i]]$map))
                cross$geno[[i]]$map <- cross$geno[[i]]$map[,!o,drop=FALSE]
            else
                cross$geno[[i]]$map <- cross$geno[[i]]$map[!o]

            # results of calc.genoprob
            if("prob" %in% names(cross$geno[[i]])) {
                o <- match(mn.drop,colnames(cross$geno[[i]]$prob))
                cross$geno[[i]]$prob <- cross$geno[[i]]$prob[,-o,,drop=FALSE]
            }

            # results of argmax.geno
            if("argmax" %in% names(cross$geno[[i]])) {
                o <- match(mn.drop,colnames(cross$geno[[i]]$argmax))
                cross$geno[[i]]$argmax <- cross$geno[[i]]$argmax[,-o,drop=FALSE]
            }

            # results of sim.geno
            if("draws" %in% names(cross$geno[[i]])) {
                o <- match(mn.drop,colnames(cross$geno[[i]]$draws))
                cross$geno[[i]]$draws <- cross$geno[[i]]$draws[,-o,,drop=FALSE]
            }

            # results of est.rf
            if("rf" %in% names(cross)) {
                attrib <- attributes(cross$rf)

                o <- match(mn.drop,colnames(cross$rf))
                cross$rf <- cross$rf[-o,-o]

                if("onlylod" %in% names(attrib))
                    attr(cross$rf, "onlylod") <- attrib$onlylod
            }
        }
    }

    if(sum(keep.chr) == 0)
        stop("You're attempting to drop *all* of the markers, which isn't allowed.")

    if(any(!found))
        warning("Markers not found: ", paste(markers[!found],collapse=" "))

    cross$geno <- cross$geno[keep.chr]

    if("founderGeno" %in% names(cross))
        cross$founderGeno <- cross$founderGeno[,markernames(cross)]

    cross
}

######################################################################
# pull.markers
#
# like drop.markers, but retain just those indicated
######################################################################
pull.markers <-
    function(cross, markers)
{
    mn <- markernames(cross)
    m <- match(markers, mn)
    if(any(is.na(m)))
        warning("Some markers couldn't be found: ", paste(markers[is.na(m)], collapse=" "))
    drop.markers(cross, mn[is.na(match(mn, markers))])
}

######################################################################
# drop.dupmarkers
#
# drop duplicate markers, retaining the consensus genotypes
######################################################################
drop.dupmarkers <-
    function(cross, verbose=TRUE)
{
    mn <- markernames(cross)
    tab <- table(mn)
    if(all(tab==1)) {
        if(verbose) cat("No duplicate markers.\n")
        return(cross)
    }

    dup <- names(tab[tab > 1])
    if(verbose) cat(" ", length(dup), "duplicate markers\n")

    # get consensus genotypes
    g <- pull.geno(cross)[,!is.na(match(mn, dup))]
    ng.omitted <- rep(NA, length(dup))
    tot.omitted <- 0
    nmar.omitted <- 0
    for(i in seq(along=dup)) {
        gg <- g[,colnames(g)==dup[i],drop=FALSE]
        res <- apply(gg, 1, function(a)
                 {
                     if(all(is.na(a))) return(c(NA, 0))
                     a <- unique(a[!is.na(a)])
                     if(length(a)==1) return(c(a, 0))
                     return(c(NA, 1))
                 } )
        if(verbose>1) {
            cat("  ", dup[i], ":\t", sum(res[2,]), " genotypes omitted\n", sep="")
            if(sum(res[2,]) > 1) cat("      ", paste(which(res[2,]>0), collapse=" "), "\n")
        }
        tot.omitted <- tot.omitted + sum(res[2,])

        flag <- FALSE
        for(j in seq(along=cross$geno)) {
            mn <- colnames(cross$geno[[j]]$data)
            wh <- mn==dup[i]
            if(!any(wh)) next
            wh <- which(wh)
            if(!flag) {
                flag <- TRUE
                cross$geno[[j]]$data[,wh[1]] <- res[1,]
                if(length(wh)==1) next
                else wh <- wh[-1]
            }
            if(length(wh) > 0) {
                nmar.omitted <- nmar.omitted + length(wh)
                cross$geno[[j]]$data <- cross$geno[[j]]$data[,-wh,drop=FALSE]
                if(is.matrix(cross$geno[[j]]$map))
                    cross$geno[[j]]$map <- cross$geno[[j]]$map[,-wh,drop=FALSE]
                else
                    cross$geno[[j]]$map <- cross$geno[[j]]$map[-wh]
            }
        }
    }
    if(verbose) {
        cat("  Total genotypes omitted:", tot.omitted, "\n")
        cat("  Total markers omitted:  ", nmar.omitted, "\n")
    }

    if("founderGeno" %in% names(cross))
        cross$founderGeno <- cross$founderGeno[,markernames(cross)]

    clean(cross)
}



######################################################################
#
# geno.table
#
# create table showing observed numbers of individuals with each
# genotype at each marker
#
######################################################################

geno.table <-
    function(cross, chr, scanone.output=FALSE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(!missing(chr))
        cross <- subset(cross, chr=chr)

    n.chr <- nchr(cross)

    type <- crosstype(cross)
    is.bcs <- type == "bcsft"
    cross.scheme <- attr(cross, "scheme")
    if(is.bcs)
        is.bcs <- (cross.scheme[2] == 0)

    chr_type <- sapply(cross$geno, chrtype)
    allchrtype <- rep(chr_type, nmar(cross))
    chrname <- names(cross$geno)
    allchrname <- rep(chrname, nmar(cross))

    ## Actually plan to have our own geno.table.bcsft routine.
    if(type == "f2" || (type == "bcsft" && !is.bcs)) {
        n.gen <- 5
        temp <- getgenonames("f2", "A", cross.attr=attributes(cross))
        gen.names <- c(temp, paste("not", temp[c(3,1)]))
    }
    else if(type %in% c("bc", "riself", "risib", "dh", "haploid", "bcsft")) {
        n.gen <- 2
        gen.names <- getgenonames(type, "A", cross.attr=attributes(cross))
    }
    else if(type == "4way") {
        n.gen <- 14
        temp <- getgenonames("4way", "A", cross.attr=attributes(cross))
        gen.names <- c(temp,
                       paste(temp[c(1,3)], collapse="/"),
                       paste(temp[c(2,4)], collapse="/"),
                       paste(temp[c(1,2)], collapse="/"),
                       paste(temp[c(3,4)], collapse="/"),
                       paste(temp[c(1,4)], collapse="/"),
                       paste(temp[c(2,3)], collapse="/"),
                       paste("not", temp[1]),
                       paste("not", temp[2]),
                       paste("not", temp[3]),
                       paste("not", temp[4]))

        gen.names[5:8] <- substr(temp[c(1,2,1,3)], c(1,1,2,2), c(1,1,2,2))
    }
    else stop("Unknown cross type: ",type)

    res <- lapply(cross$geno, function(a,ngen) {
        a <- a$data; a[is.na(a)] <- 0
        apply(a,2,function(b,ngen) table(factor(b,levels=0:ngen)),ngen)
    },n.gen)

    results <- NULL
    for(i in 1:length(res))
        results <- rbind(results,t(res[[i]]))

    colnames(results) <- c("missing",gen.names)
    rownames(results) <- unlist(lapply(cross$geno,function(a) colnames(a$data)))

    pval <- rep(NA,nrow(results))
    if(type %in% c("bc","risib","riself","dh","haploid") || (type=="bcsft" & is.bcs)) {
        sexpgm <- getsex(cross)
        if((type == "bc" || type=="bcsft") && any(chr_type == "X") && !is.null(sexpgm$sex) && any(sexpgm$sex==1)) {
            for(i in which(allchrtype=="A")) {
                x <- results[i,2:3]
                if(sum(x) > 0)
                    pval[i] <- chisq.test(x,p=c(0.5,0.5))$p.value
                else pval[i] <- 1
            }

            gn <- getgenonames("bc","X", "full", sexpgm, attributes(cross))
            wh <- which(is.na(match(gn, colnames(results))))
            temp <- matrix(0, nrow=nrow(results), ncol=length(wh))
            colnames(temp) <- gn[wh]
            results <- cbind(results, temp)

            for(i in which(chr_type=="X")) {
                dat <- reviseXdata("bc", "full", sexpgm, geno=cross$geno[[i]]$data,
                                   cross.attr=attributes(cross))
                dat[is.na(dat)] <- 0
                tab <- t(apply(dat, 2, function(x) table(factor(x, levels=0:length(gn)))))
                colnames(tab) <- c("missing", gn)
                results[allchrname==chrname[i],] <- 0
                results[allchrname==chrname[i],colnames(tab)] <- tab

                for(j in 1:ncol(dat)) {
                    stat <- apply(table(sexpgm$sex, cross$geno[[i]]$data[,j]),1,
                                  function(a) if(length(a) > 1 && sum(a)>0) return(chisq.test(a,p=c(0.5,0.5))$stat)
                                  else return(0))
                    pval[allchrname==chrname[i]][j] <- 1-pchisq(sum(stat),length(stat))
                }
            }
            results <- cbind(results, P.value=pval)
        }
        else {
            for(i in 1:length(pval)) {
                x <- results[i,2:3]
                if(sum(x) > 0)
                    pval[i] <- chisq.test(x,p=c(0.5,0.5))$p.value
                else
                    pval[i] <- 1
            }
            results <- cbind(results, P.value=pval)
        }
    }
    else  if(type == "f2" || (type == "bcsft" && !is.bcs)) {
        sexpgm <- getsex(cross)

        ## F2 with set initial genotype probabilities.
        init.geno <- c(0.25,0.5,0.25,0.75,0.75)
        ## BCsFt initial genotype probabilities need to be computed.
        if(type == "bcsft") {
            ret <- .C("genotab_em_bcsft",
                      as.integer(cross.scheme),
                      init.geno = as.double(init.geno))
            init.geno <- ret$init.geno
        }

        for(i in which(allchrtype=="A")) {
            dat <- results[i,2:6]
            if(sum(dat)==0) pval[i] <- 1
            else if(dat[4]==0 && dat[5]==0)
                pval[i] <- chisq.test(dat[1:3],   p=init.geno[1:3]   )$p.value
            else if(all(dat[2:4]==0))
                pval[i] <- chisq.test(dat[c(1,5)],p=init.geno[c(1,5)])$p.value
            else if(all(dat[c(1,2,5)]==0))
                pval[i] <- chisq.test(dat[3:4],   p=init.geno[3:4]   )$p.value
            else { # this is harder: some dominant and some not
                pval[i] <- genotab.em(dat, init.geno)
            }
        }

        for(i in which(chr_type=="X")) {
            gn <- getgenonames("f2","X","full", getsex(cross), attributes(cross))
            wh <- which(is.na(match(gn, colnames(results))))
            temp <- matrix(0, nrow=nrow(results), ncol=length(wh))
            colnames(temp) <- gn[wh]
            results <- cbind(results, temp)

            dat <- reviseXdata("f2", "full", sexpgm, geno=cross$geno[[i]]$data,
                               cross.attr=attributes(cross))
            dat[is.na(dat)] <- 0
            tab <- t(apply(dat, 2, function(x) table(factor(x, levels=0:length(gn)))))
            colnames(tab) <- c("missing", gn)
            results[allchrname==chrname[i],] <- 0
            results[allchrname==chrname[i],colnames(tab)] <- tab

            cn <- colnames(results)
            f <- grep("f$", cn)
            r <- grep("r$", cn)
            if(length(f)>0 && length(r)>0) {
                results <- results[,c(1:2,f,3,r,(1:ncol(results))[-c(1:3,f,r)])]
                colnames(results) <- cn[c(1:2,f,3,r,(1:ncol(results))[-c(1:3,f,r)])]
            }

            sex <- sexpgm$sex
            pgm <- sexpgm$pgm
            for(j in 1:ncol(dat)) {
                g <- cross$geno[[i]]$data[,j]
                if(!is.null(sex)) {
                    if(!is.null(pgm))
                        g <- matrix(as.numeric(table(g,sex,pgm)),ncol=2,byrow=TRUE)
                    else
                        g <- matrix(as.numeric(table(g,sex)),ncol=2,byrow=TRUE)
                }
                else {
                    if(!is.null(pgm)) {
                        g <- matrix(as.numeric(table(g,pgm)),ncol=2,byrow=TRUE)
                    }
                    else g <- matrix(table(g),ncol=2,byrow=TRUE)
                }
                stat <- apply(g, 1,
                              function(a) if(sum(a)>0) return(chisq.test(a,p=c(0.5,0.5))$stat)
                              else return(0))
                pval[allchrname==chrname[i]][j] <- 1-pchisq(sum(stat),length(stat))
            }
        }

        results <- cbind(results, P.value=pval)
    }
    else if(type == "4way") {
        for(i in 1:length(pval)) {
            x <- results[i,2:5]
            y <- results[i,-(1:5)]
            if(sum(x) > 0 && sum(y)==0)
                pval[i] <- chisq.test(x,p=c(0.25,0.25,0.25,0.25))$p.value
            else {
                if(allchrtype[i] == "A") {
                    res <- results[i,-1]
                    if(all(res==0)) pval[i] <- 1 # entirely missing
                    else if(all(res[-c(1,11)]==0))      # AC/not AC
                        pval[i] <- chisq.test(res[c(1,11)], p=c(0.25, 0.75))$p.value
                    else if(all(res[-c(2,12)]==0)) # BC/not BC
                        pval[i] <- chisq.test(res[c(2,12)], p=c(0.25, 0.75))$p.value
                    else if(all(res[-c(3,13)]==0)) # AD/not AD
                        pval[i] <- chisq.test(res[c(3,13)], p=c(0.25, 0.75))$p.value
                    else if(all(res[-c(4,14)]==0)) # BD/not BD
                        pval[i] <- chisq.test(res[c(4,14)], p=c(0.25, 0.75))$p.value
                    else if(all(res[-c(5,6)]==0)) # A/B
                        pval[i] <- chisq.test(res[c(5,6)], p=c(0.5, 0.5))$p.value
                    else if(all(res[-c(7,8)]==0)) # C/D
                        pval[i] <- chisq.test(res[c(7,8)], p=c(0.5, 0.5))$p.value
                    else if(all(res[-c(9,10)]==0)) # AC/BD or AD/BC
                        pval[i] <- chisq.test(res[c(9,10)], p=c(0.5, 0.5))$p.value
                    else if(all(res[-c(2,4,5)]==0)) # BC/BD/A
                        pval[i] <- chisq.test(res[c(2,4,5)], p=c(0.25, 0.25, 0.5))$p.value
                    else if(all(res[-c(1,3,6)]==0)) # AC/AD/B
                        pval[i] <- chisq.test(res[c(1,3,6)], p=c(0.25, 0.25, 0.5))$p.value
                    else if(all(res[-c(3,4,7)]==0)) # AD/BD/C
                        pval[i] <- chisq.test(res[c(3,4,7)], p=c(0.25, 0.25, 0.5))$p.value
                    else if(all(res[-c(1,2,8)]==0)) # AC/BC/D
                        pval[i] <- chisq.test(res[c(1,2,8)], p=c(0.25, 0.25, 0.5))$p.value
                    else if(all(res[-c(2,3,9)]==0)) # AC/BD or AD or BC
                        pval[i] <- chisq.test(res[c(2,3,9)], p=c(0.25, 0.25, 0.5))$p.value
                    else if(all(res[-c(1,4,10)]==0)) # AD/BC or AC or BD
                        pval[i] <- chisq.test(res[c(1,4,10)], p=c(0.25, 0.25, 0.5))$p.value
                }
            }
        }
        results <- cbind(results, P.value=pval)
    }

    if(!scanone.output)
        return(data.frame(chr=rep(names(cross$geno),nmar(cross)),results, stringsAsFactors=TRUE))

    themap <- pull.map(cross)
    if(is.matrix(themap[[1]]))
        thepos <- unlist(lapply(themap, function(a) a[1,]))
    else thepos <- unlist(themap)

    temp <- results[,1:(ncol(results)-1),drop=FALSE]
    res <- data.frame(chr=rep(names(cross$geno),nmar(cross)),
                      pos=thepos,
                      neglog10P=-log10(results[,ncol(results)]),
                      missing=temp[,1]/apply(temp, 1, sum),
                      temp[,-1]/apply(temp[,-1], 1, sum), stringsAsFactors=TRUE)
    class(res) <- c("scanone", "data.frame")
    rownames(res) <- rownames(results)
    res[,1] <- factor(as.character(res[,1]), levels=unique(as.character(res[,1])))
    res
}



genotab.em <-
    function(dat, init.geno, tol=1e-6, maxit=10000, verbose=FALSE)
{

    genotab.ll <-
        function(dat, gam, init.geno)
        {
            p <- c(init.geno[1]*(1-gam[2]), init.geno[2]*gam[1], init.geno[3]*(1-gam[3]),
                   gam[2]*init.geno[4], gam[3]*init.geno[5])
            if(any(p==0 & dat > 0)) return(-Inf)
            return( sum((dat*log(p))[dat>0 & p>0]) )
        }

    n <- sum(dat)

    gam <- c(sum(dat[1:3]), dat[4], dat[5])/n

    curll <- genotab.ll(dat, gam, init.geno)

    flag <- 0
    if(verbose) cat(0, gam, curll, "\n")
    for(i in 1:maxit) {
        # estep
        zAA <- dat[1]*gam[3]/(gam[1]+gam[3])
        zBB <- dat[3]*gam[2]/(gam[1]+gam[2])

        # mstep
        gamnew <- c(sum(dat[1:3])-zAA-zBB, dat[4]+zBB, dat[5]+zAA)/n

        newll <- genotab.ll(dat, gamnew, init.geno)

        if(verbose) cat(i, gamnew, newll, "\n")

        if(abs(curll-newll) < tol) {
            flag <- 1
            break
        }
        gam <- gamnew
        curll <- newll
    }
    if(!flag) warning("Didn't converge.")

    gam <- gamnew
    p <- c(init.geno[1]*(1-gam[2]), init.geno[2]*gam[1], init.geno[3]*(1-gam[3]),
           gam[2]*init.geno[4], gam[3]*init.geno[5])
    ex <- p*n
    1-pchisq(sum(((dat-ex)^2/ex)[ex>0]), 2)
}

######################################################################
# geno.crosstab
#
# Get a cross-tabulation of the genotypes at two markers,
# with the markers specified by name
######################################################################
geno.crosstab <-
    function(cross, mname1, mname2, eliminate.zeros=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(missing(mname2) && length(mname1)>1) {
        mname2 <- mname1[2]
        mname1 <- mname1[1]
    }

    if(length(mname1) > 1 || length(mname2) > 1)
        stop("mname1 and mname2 should both have lenght 1, or mname1 should have length 2 and mname1 should be missing.")

    if(mname1==mname2)
        stop("You must give two distinct marker names.")

    pos <- find.markerpos(cross, c(mname1, mname2))
    if(any(is.na(pos$chr))) {
        if(all(is.na(pos$chr)))
            stop("Markers ", mname1, " and ", mname2, " not found.")
        else
            stop("Marker ", rownames(pos)[is.na(pos$chr)], " not found.")
    }

    chr_type <- sapply(cross$geno[pos$chr], chrtype)
    crosstype <- crosstype(cross)

    g1 <- pull.geno(cross, pos$chr[1])[,mname1, drop=FALSE]
    g2 <- pull.geno(cross, pos$chr[2])[,mname2, drop=FALSE]

    if(chr_type[1] == "X")
        g1 <- reviseXdata(crosstype, "full", getsex(cross), geno=g1, cross.attr=attributes(cross))

    if(chr_type[2] == "X")
        g2 <- reviseXdata(crosstype, "full", getsex(cross), geno=g2, cross.attr=attributes(cross))

    g1[is.na(g1)] <- 0
    g2[is.na(g2)] <- 0

    g1names <- getgenonames(crosstype, chr_type[1], "full", getsex(cross), attributes(cross))
    g2names <- getgenonames(crosstype, chr_type[2], "full", getsex(cross), attributes(cross))

    if(chr_type[1] != "X") {
        if(crosstype == "f2")
            g1names <- c(g1names, paste("not", g1names[c(3,1)]))
        else if(crosstype == "bc" || crosstype == "risib" || crosstype=="riself" || crosstype=="dh" || crosstype=="haploid") {
        }
        else if(crosstype == "4way") {
            temp <- g1names
            g1names <- c(temp,
                         paste(temp[c(1,3)], collapse="/"),
                         paste(temp[c(2,4)], collapse="/"),
                         paste(temp[c(1,2)], collapse="/"),
                         paste(temp[c(3,4)], collapse="/"),
                         paste(temp[c(1,4)], collapse="/"),
                         paste(temp[c(2,3)], collapse="/"),
                         paste("not", temp[1]),
                         paste("not", temp[2]),
                         paste("not", temp[3]),
                         paste("not", temp[4]))

            g1names[5:8] <- substr(temp[c(1,2,1,3)], c(1,1,2,2), c(1,1,2,2))
        }
        else stop("Unknown cross type: ",crosstype)
    }
    if(chr_type[2] != "X") {
        if(crosstype == "f2")
            g2names <- c(g2names, paste("not", g2names[c(3,1)]))
        else if(crosstype == "bc" || crosstype == "risib" || crosstype=="riself" || crosstype=="dh" || crosstype=="haploid") {
        }
        else if(crosstype == "4way") {
            temp <- g2names
            g2names <- c(temp,
                         paste(temp[c(1,3)], collapse="/"),
                         paste(temp[c(2,4)], collapse="/"),
                         paste(temp[c(1,2)], collapse="/"),
                         paste(temp[c(3,4)], collapse="/"),
                         paste(temp[c(1,4)], collapse="/"),
                         paste(temp[c(2,3)], collapse="/"),
                         paste("not", temp[1]),
                         paste("not", temp[2]),
                         paste("not", temp[3]),
                         paste("not", temp[4]))

            g2names[5:8] <- substr(temp[c(1,2,1,3)], c(1,1,2,2), c(1,1,2,2))
        }
        else stop("Unknown cross type: ",crosstype)
    }
    g1names <- c("-", g1names)
    g2names <- c("-", g2names)

    g1 <- as.character(g1)
    g2 <- as.character(g2)

    for(i in 1:length(g1names)) {
        j <- as.character(i-1)
        g1[g1==j] <- g1names[i]
    }
    g1 <- factor(g1, levels=g1names)

    for(i in 1:length(g2names)) {
        j <- as.character(i-1)
        g2[g2==j] <- g2names[i]
    }
    g2 <- factor(g2, levels=g2names)

    tab <- table(g1, g2)
    names(attributes(tab)$dimnames) <- c(mname1, mname2)

    if(eliminate.zeros) { # eliminate rows and columns with no data (but always include missing data row and column)
        rs <- apply(tab, 1, sum); rs[1] <- 1
        tab <- tab[rs>0,,drop=FALSE]
        cs <- apply(tab, 2, sum); cs[1] <- 1
        tab <- tab[,cs>0,drop=FALSE]
    }

    tab
}


# map functions
mf.k <- function(d) 0.5*tanh(d/50)
mf.h <- function(d) 0.5*(1-exp(-d/50))
imf.k <- function(r) { r[r >= 0.5] <- 0.5-1e-14; 50*atanh(2*r) }
imf.h <- function(r) { r[r >= 0.5] <- 0.5-1e-14; -50*log(1-2*r) }
mf.m <- function(d) sapply(d,function(a) min(a/100,0.5))
imf.m <- function(r) sapply(r,function(a) min(a*100,50))

# carter-falconer: mf.cf, imf.cf
imf.cf <- function(r) { r[r >= 0.5] <- 0.5-1e-14; 12.5*(log(1+2*r)-log(1-2*r))+25*atan(2*r) }

mf.cf <-
    function(d)
{
    d[d >= 300] <- 300

    icf <- function(r,d)
        imf.cf(r)-d

    sapply(d,function(a) {
        if(a==0) return(0)
        uniroot(icf, c(0,0.5-1e-14),d=a,tol=1e-12)$root })
}


######################################################################
#
# switch.order: change the marker order on a given chromosome to some
#               specified order
#
######################################################################

switch.order <-
    function(cross, chr, order, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             maxit=4000, tol=1e-6, sex.sp=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    map.function <- match.arg(map.function)

    # check chromosome argument
    if(missing(chr)) {
        chr <- names(cross$geno)[1]
        warning("Assuming you mean chromosome ", chr)
    }
    else {
        if(length(chr) > 1) {
            chr <- chr[1]
            warning("switch.order can deal with just one chromosome at a time; assuming you want chr ", chr)
        }
        if(!testchr(chr, names(cross$geno)))
            stop("Chr ", chr, " not found.")
    }
    chr <- matchchr(chr, names(cross$geno))

    # check order argument
    n.mar <- nmar(cross)
    if(n.mar[chr] == length(order)-2 || n.mar[chr]==length(order)-1)
        order <- order[1:n.mar[chr]]     # useful for output from ripple()
    if(n.mar[chr] != length(order))
        stop("Incorrect number of markers.")

    # save recombination fractions
    flag <- 0
    if("rf" %in% names(cross)) {
        attrib <- attributes(cross$rf)

        rf <- cross$rf
        # determine column within rec fracs
        whchr <- which(names(cross$geno)==chr)
        oldcols <- cumsum(c(0,n.mar))[whchr]+seq(along=order)
        newcols <- cumsum(c(0,n.mar))[whchr]+order
        rf[oldcols,] <- rf[newcols,]
        rf[,oldcols] <- rf[,newcols]
        colnames(rf)[oldcols] <- colnames(rf)[newcols]
        rownames(rf)[oldcols] <- rownames(rf)[newcols]
        flag <- 1
    }

    # remove any intermediate calculations (except rec fracs),
    #   as they will no longer be meaningful
    cross <- clean(cross)

    if(!is.matrix(cross$geno[[chr]]$map))
        first <- min(cross$geno[[chr]]$map)
    else
        first <- apply(cross$geno[[chr]]$map,1,min)

    # re-order markers
    cross$geno[[chr]]$data <- cross$geno[[chr]]$data[,order,drop=FALSE]
    m <- seq(0,by=5,length=ncol(cross$geno[[chr]]$data))
    names(m) <- colnames(cross$geno[[chr]]$data)
    if(is.matrix(cross$geno[[chr]]$map))
        cross$geno[[chr]]$map <- rbind(m,m)
    else
        cross$geno[[chr]]$map <- m

    # re-estimate rec fracs for re-ordered chromosome
    if(flag==1) {
        temp <- est.rf(subset(cross, chr=chr))$rf
        rf[oldcols,oldcols] <- temp
        cross$rf <- rf
        if("onlylod" %in% names(attrib))
            attr(cross$rf, "onlylod") <- attrib$onlylod
    }

    # re-estimate map
    newmap <- est.map(cross, chr=chr,
                      error.prob=error.prob, map.function=map.function,
                      maxit=maxit, tol=tol, sex.sp=sex.sp)

    if(!is.matrix(newmap[[1]]))
        cross$geno[[chr]]$map <- newmap[[1]] + first
    else {
        cross$geno[[chr]]$map[1,] <- newmap[[1]][1,] + first[1]
        cross$geno[[chr]]$map[2,] <- newmap[[1]][2,] + first[2]
        rownames(cross$geno[[chr]]$map) <- NULL
    }

    cross
}


######################################################################
# flip.order: flip the order of markers on a set of chromosomes
######################################################################

flip.order <-
    function(cross, chr)
{
    # utility function to flip a map
    flipChrOnMap <-
        function(map)
        {
            if(is.matrix(map)) {
                map <- map[,ncol(map):1,drop=FALSE]
                for(j in 1:nrow(map))
                    map[j,] <- max(map[j,]) - map[j,]
            } else {
                map <- max(map) - map[length(map):1]
            }
            map
        }

    # utility function to flip intermediate calc
    flipChrInterCalc <-
        function(object, attr2consider=c("error.prob", "step", "off.end", "map.function", "stepwidth"))
        {
            the_attr <- attributes(object)
            ndim <- length(dim(object))
            if(ndim == 3)
                object <- object[,ncol(object):1,,drop=FALSE]
            else
                object <- object[,ncol(object):1,drop=FALSE]
            for(a in attr2consider) {
                if(a %in% names(the_attr))
                    attr(object, a) <- the_attr[[a]]
            }
            if("map" %in% names(the_attr))
                attr(object, "map") <- flipChrOnMap(the_attr$map)

            object
        }

    chr <- matchchr(chr, names(cross$geno))

    for(i in chr) {
        nc <- ncol(cross$geno[[i]]$data)
        cross$geno[[i]]$data <- cross$geno[[i]]$data[,nc:1,drop=FALSE]
        cross$geno[[i]]$map <- flipChrOnMap(cross$geno[[i]]$map)

        if("prob" %in% names(cross$geno[[i]]))
            cross$geno[[i]]$prob <- flipChrInterCalc(cross$geno[[i]]$prob)
        if("argmax" %in% names(cross$geno[[i]]))
            cross$geno[[i]]$argmax <- flipChrInterCalc(cross$geno[[i]]$argmax)
        if("draws" %in% names(cross$geno[[i]]))
            cross$geno[[i]]$draws <- flipChrInterCalc(cross$geno[[i]]$draws)
        if("errorlod" %in% names(cross$geno[[i]]))
            cross$geno[[i]]$errorlod <- flipChrInterCalc(cross$geno[[i]]$errorlod,
                                                          c("error.prob", "map.function"))
    }

    cross$rf <- NULL

    cross
}


######################################################################
#
# subset.cross: General subsetting function for a cross object
#
######################################################################

subset.cross <-
    function(x, chr, ind, ...)
{
    if(!inherits(x, "cross"))
        stop("Input should have class \"cross\".")

    if(missing(chr) && missing(ind))
        stop("You must specify either chr or ind.")

    n.chr <- nchr(x)
    n.ind <- nind(x)

    # pull out relevant chromosomes
    if(!missing(chr)) {
        chr <- matchchr(chr, names(x$geno))

        if("rf" %in% names(x)) { # pull out part of rec fracs
            if(totmar(x) != ncol(x$rf))
                x <- clean(x)
            else {
                attrib <- attributes(x$rf)

                n.mar <- nmar(x)
                n.chr <- n.chr
                wh <- rbind(c(0,cumsum(n.mar)[-n.chr])+1,cumsum(n.mar))
                dimnames(wh) <- list(NULL, names(n.mar))
                wh <- wh[,chr,drop=FALSE]
                wh <- unlist(apply(wh,2,function(a) a[1]:a[2]))
                x$rf <- x$rf[wh,wh]

                if("onlylod" %in% names(attrib)) # save the onlylod attribute if its there
                    attr(x$rf, "onlylod") <- attrib$onlylod
            }
        }

        x$geno <- x$geno[chr]

        if("founderGeno" %in% names(x))
            x$founderGeno <- x$founderGeno[,unlist(lapply(x$geno, function(a) colnames(a$data)))]
    }

    if(!missing(ind)) {
        theid <- getid(x)

        if(is.logical(ind)) {
            ind[is.na(ind)] <- FALSE
            if(length(ind) != n.ind)
                stop("ind argument has wrong length (", length(ind), "; should be ", n.ind, ")")
            ind <- (1:n.ind)[ind]
        }

        else if(is.numeric(ind)) { # treat as numeric indices; don't match against individual identifiers
            if(all(ind < 0)) { # drop all but these
                ind <- -ind
                if(any(ind > nind(x))) {
                    a <- -ind[ind > nind(x)]
                    if(length(a) > 1) a <- sample(a, 1)
                    stop("Invalid ind values (e.g., ", a, ")")
                }
                ind <- (1:n.ind)[-ind]
            }
            else if(all(ind > 0)) { # keep these
                if(any(ind > nind(x))) {
                    a <- ind[ind > nind(x)]
                    if(length(a) > 1) a <- sample(a, 1)
                    stop("Invalid ind values (e.g., ", a, ")")
                }
            }
            else {
                stop("Need ind to be all > 0 or all < 0.")
            }
        }


        else if(!is.null(theid)) { # cross has individual IDs
            ind <- as.character(ind)
            if(all(substr(ind, 1,1) == "-")) {
                ind <- substr(ind, 2, nchar(ind))
                m <- match(ind, theid)
                if(all(is.na(m)))
                    stop("No matching individuals.")
                if(any(is.na(m)))
                    warning("Individuals not found: ", paste(ind[is.na(m)]))
                ind <- (1:n.ind)[-m[!is.na(m)]]
            }
            else  {
                m <- match(ind, theid)
                if(any(is.na(m)))
                    warning("Individuals not found: ", paste(ind[is.na(m)], collapse=" "))
                ind <- m[!is.na(m)]
            }
            ind <- ind[!is.na(ind)]
        }
        else { # no individual IDs
            stop("In the absense of individual IDs, ind should be logical or numeric.")
        }
        # Note: ind should now be a numeric vector

        if(length(ind) == 0)
            stop("Must retain at least one individual.")
        if(length(ind) == 1)
            warning("Retained only one individual!")

        x$pheno <- x$pheno[ind,,drop=FALSE]

        if("cross" %in% names(x))
            x$cross <- x$cross[ind,,drop=FALSE]

        for(i in 1:nchr(x)) {
            x$geno[[i]]$data <- x$geno[[i]]$data[ind,,drop=FALSE]

            if("prob" %in% names(x$geno[[i]])) {
                temp <- attributes(x$geno[[i]]$prob) # all attributes but dim and dimnames
                x$geno[[i]]$prob <- x$geno[[i]]$prob[ind,,,drop=FALSE]
                # put attributes back in
                for(k in seq(along=temp)) {
                    if(names(temp)[k] != "dim" && names(temp)[k] != "dimnames")
                        attr(x$geno[[i]]$prob, names(temp)[k]) <- temp[[k]]
                }
            }

            if("errorlod" %in% names(x$geno[[i]])) {
                temp <- attributes(x$geno[[i]]$prob) # all attributes but dim and dimnames
                x$geno[[i]]$errorlod <- x$geno[[i]]$errorlod[ind,,drop=FALSE]
                # put attributes back in
                for(k in seq(along=temp)) {
                    if(names(temp)[k] != "dim" && names(temp)[k] != "dimnames")
                        attr(x$geno[[i]]$errorlod, names(temp)[k]) <- temp[[k]]
                }
            }

            if("argmax" %in% names(x$geno[[i]])) {
                temp <- attributes(x$geno[[i]]$argmax) # all attributes but dim and dimnames
                x$geno[[i]]$argmax <- x$geno[[i]]$argmax[ind,,drop=FALSE]
                # put attributes back in
                for(k in seq(along=temp)) {
                    if(names(temp)[k] != "dim" && names(temp)[k] != "dimnames")
                        attr(x$geno[[i]]$argmax, names(temp)[k]) <- temp[[k]]
                }
            }

            if("draws" %in% names(x$geno[[i]])) {
                temp <- attributes(x$geno[[i]]$draws) # all attributes but dim and dimnames
                x$geno[[i]]$draws <- x$geno[[i]]$draws[ind,,,drop=FALSE]
                # put attributes back in
                for(k in seq(along=temp)) {
                    if(names(temp)[k] != "dim" && names(temp)[k] != "dimnames")
                        attr(x$geno[[i]]$draws, names(temp)[k]) <- temp[[k]]
                }
            }
        }

        if("qtlgeno" %in% names(x))
            x$qtlgeno <- x$qtlgeno[ind,,drop=FALSE]
    }
    x
}

#pull.chr <-
#function(cross, chr) {
#  warning("pull.chr is deprecated; use subset.cross.")
#  subset.cross(cross, chr)
#}


######################################################################
#
# c.cross: Combine crosses
#
######################################################################

c.cross <-
    function(...)
{
    args <- list(...)
    n.args <- length(args)

    for(i in seq(along=args)) {
        if(!inherits(args[[i]], "cross"))
            stop("Input should have class \"cross\".")
    }

    # if only one cross, just return it
    if(n.args==1) return(args[[1]])

    if(any(sapply(args, function(a) !inherits(a, "cross"))))
        stop("All arguments must be cross objects.")

    # crosses must be all the same, or must be combination of F2 and BC
    classes <- sapply(args, crosstype)
    if(length(unique(classes))==1) {
        allsame <- TRUE
        type <- classes[1]
    }
    else {
        if(any(classes != "bc" & classes != "f2"))
            stop("Experiments must be either the same type or be bc/f2.")
        allsame <- FALSE
        type <- "f2"
    }

    if(length(unique(sapply(args, nchr))) > 1)
        stop("All arguments must have the same number of chromosomes.")

    x <- args[[1]]
    chr <- names(x$geno)
    n.mar <- nmar(x)
    marnam <- unlist(lapply(x$geno,function(b) colnames(b$data)))
    marpos <- unlist(lapply(x$geno,function(b) b$map))

    map.mismatch <- 0
    for(i in 2:n.args) {
        y <- args[[i]]
        y.marnam <- unlist(lapply(y$geno, function(b) colnames(b$data)))
        y.marpos <- unlist(lapply(y$geno, function(b) b$map))
        if(any(chr != names(y$geno)) || any(n.mar != nmar(y)) ||
           any(marnam != y.marnam) || any(marpos != y.marpos)) {
            map.mismatch <- 1
            break
        }
    }
    if(map.mismatch) { # get the maps to line up
        for(i in 1:nchr(args[[1]])) {
            themap <- NULL

            themaps <- vector("list", n.args)

            for(j in 1:n.args)  {
                if(is.matrix(args[[j]]$map))
                    stop("c.cross() won't work with sex-specific maps.")

                themaps[[j]] <- args[[j]]$geno[[i]]$map
                themap <- c(themap, themaps[[j]])
            }
            themap <- sort(themap)
            mn <- unique(names(themap))

            newmap <- rep(0,length(mn))
            names(newmap) <- mn
            for(j in 1:length(newmap))
                newmap[j] <- mean(themap[names(themap) == mn[j]])

            for(j in 1:n.args) {
                m <- match(names(themaps[[j]]), mn)
                m <- m[!is.na(m)]
                if(any(diff(m)) < 0)
                    stop(" Markers must all be in the same order.")

                if(!all(mn %in% names(themaps[[j]]))) {
                    temp <- matrix(ncol=length(mn), nrow=nind(args[[j]]))
                    colnames(temp) <- mn
                    temp[,names(themaps[[j]])] <- args[[j]]$geno[[i]]$data
                    args[[j]]$geno[[i]]$data <- temp
                }
                args[[j]]$geno[[i]]$map <- newmap
            }

        }

    } # end of map mismatch fix

    x <- args[[1]]
    chr <- names(x$geno)
    n.mar <- nmar(x)
    marnam <- unlist(lapply(x$geno,function(b) colnames(b$data)))


    # create genotype information
    geno <- x$geno
    for(j in 1:nchr(x)) { # drop extraneous stuff
        geno[[j]] <- list(data=geno[[j]]$data, map=geno[[j]]$map)
        class(geno[[j]]) <- class(x$geno[[j]])
    }
    for(i in 2:n.args)
        for(j in 1:nchr(x))
            geno[[j]]$data <- rbind(geno[[j]]$data,args[[i]]$geno[[j]]$data)

    # get all phenotype names
    phenam <- names(x$pheno)
    for(i in 2:n.args)
        phenam <- c(phenam, names(args[[i]]$pheno))
    phenam <- unique(phenam)

    # form big phenotype matrix
    n.ind <- sapply(args,nind)
    pheno <- matrix(nrow=sum(n.ind),ncol=length(phenam))
    colnames(pheno) <- phenam
    pheno <- as.data.frame(pheno, stringsAsFactors=TRUE)

    if(!allsame) {
        crosstype <- factor(rep(c("bc","f2")[match(classes,c("bc","f2"))],n.ind),
                            levels=c("bc","f2"))
        pheno <- cbind(pheno,crosstype=crosstype)
    }

    for(i in 1:length(phenam)) {
        phe <- vector("list",n.args)
        for(j in 1:n.args) {
            o <- match(phenam[i],names(args[[j]]$pheno))
            if(is.na(o)) phe[[j]] <- rep(NA,n.ind[j])
            else phe[[j]] <- args[[j]]$pheno[,o]
        }
        pheno[,i] <- unlist(phe)
    }

    # indicator of which cross
    whichcross <- matrix(0,ncol=n.args,nrow=sum(n.ind))
    colnames(whichcross) <- paste("cross",1:n.args,sep="")
    thecross <- rep(NA, sum(n.ind))
    prev <- 0
    for(i in 1:n.args) {
        wh <- prev + 1:n.ind[i]
        prev <- prev + n.ind[i]
        whichcross[wh,i] <- 1
        thecross[wh] <- i
    }
    pheno <- cbind(pheno,cross=thecross,whichcross)

    x$pheno <- pheno

    if(!map.mismatch) { # keep probs and draws only if we've not re-aligned the maps
        # if probs exist in each and all have the same
        #     set up values, keep them
        wh <- sapply(args, function(a) match("prob",names(a$geno[[1]])))
        step <- sapply(args,function(a) attr(a$geno[[1]]$prob,"step"))
        error.prob <- sapply(args,function(a) attr(a$geno[[1]]$prob,"error.prob"))
        off.end <- sapply(args,function(a) attr(a$geno[[1]]$prob,"off.end"))
        map.function <- sapply(args,function(a) attr(a$geno[[1]]$prob,"map.function"))
        map <- lapply(args,function(a) attr(a$geno[[1]]$prob,"map"))
        if(!any(is.na(wh)) && length(unique(step))==1 &&
           length(unique(error.prob))==1 && length(unique(off.end))==1 &&
           length(unique(map.function))==1) {
            if(allsame) { # all same cross type
                for(j in 1:nchr(x)) {
                    geno[[j]]$prob <- array(dim=c(sum(n.ind),dim(x$geno[[j]]$prob)[-1]))
                    dimnames(geno[[j]]$prob) <- dimnames(x$geno[[j]]$prob)
                    prev <- 0
                    for(i in 1:n.args) {
                        wh <- prev + 1:n.ind[i]
                        prev <- prev + n.ind[i]
                        geno[[j]]$prob[wh,,] <- args[[i]]$geno[[j]]$prob
                    }
                }
            }
            else { # mixed F2 and BC
                for(j in 1:nchr(x)) {
                    wh <- match("f2",classes)
                    geno[[j]]$prob <- array(0,dim=c(sum(n.ind),dim(args[[wh]]$geno[[j]]$prob)[-1]))
                    dimnames(geno[[j]]$prob) <- dimnames(args[[wh]]$geno[[j]]$prob)
                    prev <- 0
                    for(i in 1:n.args) {
                        wh <- prev + 1:n.ind[i]
                        prev <- prev + n.ind[i]

                        if(classes[i]=="f2")
                            geno[[j]]$prob[wh,,] <- args[[i]]$geno[[j]]$prob
                        else # backcross
                            geno[[j]]$prob[wh,,1:2] <- args[[i]]$geno[[j]]$prob
                    }
                }
            }

            for(j in 1:nchr(x)) {
                wh <- sapply(args, function(a) match("prob",names(a$geno[[j]])))
                step <- sapply(args,function(a) attr(a$geno[[j]]$prob,"step"))
                error.prob <- sapply(args,function(a) attr(a$geno[[j]]$prob,"error.prob"))
                off.end <- sapply(args,function(a) attr(a$geno[[j]]$prob,"off.end"))
                map.function <- sapply(args,function(a) attr(a$geno[[j]]$prob,"map.function"))
                map <- lapply(args,function(a) attr(a$geno[[j]]$prob,"map"))

                attr(geno[[j]]$prob,"step") <- step[1]
                attr(geno[[j]]$prob,"error.prob") <- error.prob[1]
                attr(geno[[j]]$prob,"off.end") <- off.end[1]
                attr(geno[[j]]$prob,"map.function") <- map.function[1]
                attr(geno[[j]]$prob,"map") <- map[[1]]
            }
        }

        # if draws exist in each and all have the same
        #     set up values, keep them
        wh <- sapply(args, function(a) match("draws",names(a$geno[[1]])))
        step <- sapply(args,function(a) attr(a$geno[[1]]$draws,"step"))
        error.prob <- sapply(args,function(a) attr(a$geno[[1]]$draws,"error.prob"))
        off.end <- sapply(args,function(a) attr(a$geno[[1]]$draws,"off.end"))
        map.function <- sapply(args,function(a) attr(a$geno[[1]]$draws,"map.function"))
        map <- lapply(args,function(a) attr(a$geno[[1]]$draws,"map"))
        ndraws <- sapply(args,function(a) dim(a$geno[[1]]$draws)[3])
        if(!any(is.na(wh)) && length(unique(step))==1 &&
           length(unique(error.prob))==1 && length(unique(off.end))==1 &&
           length(unique(map.function))==1 && length(unique(ndraws))==1) {
            for(j in 1:nchr(x)) {
                geno[[j]]$draws <- array(0,dim=c(sum(n.ind),dim(x$geno[[j]]$draws)[-1]))
                dimnames(geno[[j]]$draws) <- dimnames(x$geno[[j]]$draws)
                prev <- 0
                for(i in 1:n.args) {
                    wh <- prev + 1:n.ind[i]
                    prev <- prev + n.ind[i]
                    geno[[j]]$draws[wh,,] <- args[[i]]$geno[[j]]$draws
                }

                wh <- sapply(args, function(a) match("draws",names(a$geno[[j]])))
                step <- sapply(args,function(a) attr(a$geno[[j]]$draws,"step"))
                error.prob <- sapply(args,function(a) attr(a$geno[[j]]$draws,"error.prob"))
                off.end <- sapply(args,function(a) attr(a$geno[[j]]$draws,"off.end"))
                map.function <- sapply(args,function(a) attr(a$geno[[j]]$draws,"map.function"))
                map <- lapply(args,function(a) attr(a$geno[[j]]$draws,"map"))

                attr(geno[[j]]$draws,"step") <- step[1]
                attr(geno[[j]]$draws,"error.prob") <- error.prob[1]
                attr(geno[[j]]$draws,"off.end") <- off.end[1]
                attr(geno[[j]]$draws,"map.function") <- map.function[1]
                attr(geno[[j]]$draws,"map") <- map[[1]]
            }
        }
    }

    x <- list(geno=geno, pheno=pheno)
    class(x) <- c(type,"cross")
    x
}

######################################################################
#
# fill.geno: Run argmax.geno or sim.geno and then fill in the
#            genotype data with the results.  This will allow
#            rough genome scans by marker regression without
#            holes.  WE WOULD NOT PLACE ANY TRUST IN THE RESULTS!
#
#   the newer method, "no_dbl_XO", fills in missing genotypes between
#   markers with matching genotypes
#
######################################################################

fill.geno <-
    function(cross, method=c("imp","argmax", "no_dbl_XO", "maxmarginal"),
             error.prob=0.0001, map.function=c("haldane","kosambi","c-f","morgan"),
             min.prob=0.95)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    method <- match.arg(method)

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }

    # remove any extraneous material
    cross <- clean(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)

    if(method=="imp") {
        # do one imputation
        temp <- sim.geno(cross,n.draws=1,step=0,off.end=0,
                         error.prob=error.prob,map.function=map.function)
        # replace the genotype data with the results,
        #     stripping off any attributes
        for(i in 1:n.chr) {
            nam <- colnames(cross$geno[[i]]$data)
            if(n.mar[i] == 1)
                cross$geno[[i]]$data <-
                    matrix(as.numeric(temp$geno[[i]]$draws[,2,1]),ncol=n.mar[i])
            else
                cross$geno[[i]]$data <-
                    matrix(as.numeric(temp$geno[[i]]$draws[,,1]),ncol=n.mar[i])
            colnames(cross$geno[[i]]$data) <- nam
        }
    }
    else if(method=="argmax") {
        # run the Viterbi algorithm
        temp <- argmax.geno(cross,step=0,off.end=0,error.prob=error.prob,
                            map.function=map.function)
        # replace the genotype data with the results,
        #     stripping off any attributes
        for(i in 1:n.chr) {
            nam <- colnames(cross$geno[[i]]$data)
            if(n.mar[i] == 1)
                cross$geno[[i]]$data <-
                    matrix(as.numeric(temp$geno[[i]]$argmax[,2]),ncol=n.mar[i])
            else
                cross$geno[[i]]$data <-
                    matrix(as.numeric(temp$geno[[i]]$argmax),ncol=n.mar[i])
            colnames(cross$geno[[i]]$data) <- nam
        }
    }
    else if(method=="maxmarginal") {
        temp <- calc.genoprob(cross, step=0, off.end=0, error.prob=error.prob,
                              map.function=map.function)

        for(i in 1:n.chr) {
            p <- temp$geno[[i]]$prob
            whmax <- apply(p, 1:2, which.max)
            maxpr <- apply(p, 1:2, max)
            g <- cross$geno[[i]]$data
            g[maxpr >= min.prob] <- whmax[maxpr > min.prob]
            g[maxpr < min.prob] <- NA
            cross$geno[[i]]$data <- g
        }
    }
    else {
        for(i in 1:n.chr) {
            dat <- cross$geno[[i]]$data
            dat[is.na(dat)] <- 0
            nr <- nrow(dat)
            nc <- ncol(dat)
            dn <- dimnames(dat)
            dat <- .C("R_fill_geno_nodblXO",
                      as.integer(nr),
                      as.integer(nc),
                      dat=as.integer(dat),
                      PACKAGE="qtl")$dat
            dat[dat==0] <- NA
            dat <- matrix(dat, ncol=nc, nrow=nr)
            dimnames(dat) <- dn
            cross$geno[[i]]$data <- dat
        }
    }
    cross
}

######################################################################
#
# checkcovar
#
# This is a utility function for scanone and scantwo.  We remove
# individuals with missing phenotypes or covariates and check
# that the covariates are of the appropriate form.
#
######################################################################

checkcovar <-
    function(cross, pheno.col, addcovar, intcovar, perm.strata, ind.noqtl=NULL, weights=NULL, verbose=TRUE)
{
    chr_type <- sapply(cross$geno, chrtype)

    # drop individuals whose sex or pgm is missing if X chr is included
    if(any(chr_type=="X")) {
        sexpgm <- getsex(cross)
        keep <- rep(TRUE,nind(cross))
        flag <- 0
        if(!is.null(sexpgm$sex)) {
            if(any(is.na(sexpgm$sex))) {
                keep[is.na(sexpgm$sex)] <- FALSE
                flag <- 1
            }
        }
        if(!is.null(sexpgm$pgm)) {
            if(any(is.na(sexpgm$pgm))) {
                keep[is.na(sexpgm$pgm)] <- FALSE
                flag <- 1
            }
        }
        if(flag) {
            if(verbose) warning("Dropping ", sum(!keep), " individuals with missing sex or pgm.\n")
            cross <- subset(cross, ind=keep)
            if(!is.null(addcovar)) {
                if(!is.matrix(addcovar)) addcovar <- addcovar[keep]
                else addcovar <- addcovar[keep,]
            }
            if(!is.null(intcovar)) {
                if(!is.matrix(intcovar)) intcovar <- intcovar[keep]
                else intcovar <- intcovar[keep,]
            }
        }
    }

    # check phenotypes - we allow multiple phenotypes here
    if(any(pheno.col < 1 | pheno.col > nphe(cross)))
        stop("Specified phenotype column is invalid.")
    # check if all phenotypes are numeric
    pheno <- cross$pheno[,pheno.col,drop=FALSE]
    idx.nonnum <- which(!apply(pheno,2, is.numeric))
    if(length(idx.nonnum) > 0)
        stop("Following phenotypes are not numeric: ",
             paste(colnames(pheno)[idx.nonnum], collapse=", "))

    orig.n.ind <- nind(cross)

    # drop individuals with missing phenotypes
    if(any(!is.finite(unlist(pheno)))) {
        keep.ind <- as.numeric(which(apply(pheno, 1, function(x) !any(!is.finite(x)))))
        #    keep.ind <- (1:length(pheno))[!is.na(pheno)]
        n.drop <- nind(cross) - length(keep.ind)
        keep.ind.boolean <- rep(FALSE, nind(cross))
        keep.ind.boolean[keep.ind] <- TRUE
        cross <- subset.cross(cross,ind=keep.ind.boolean)
        pheno <- pheno[keep.ind,,drop=FALSE]
        if(verbose) warning("Dropping ", n.drop, " individuals with missing phenotypes.\n")
    }
    else keep.ind <- 1:nind(cross)
    n.ind <- nind(cross)
    n.chr <- nchr(cross)      # number of chromosomes
    type <- crosstype(cross)   # type of cross

    n.addcovar <- n.intcovar <- 0
    if(!is.null(addcovar)) { # for additive covariates
        if(!is.matrix(addcovar)) {
            if(!is.numeric(as.matrix(addcovar)))
                stop("addcovar should be numeric")
            if(is.vector(addcovar) || is.data.frame(addcovar))
                addcovar <- as.matrix(addcovar)
            else stop("addcovar should be a matrix")
        }
        if(!all(apply(addcovar,2,is.numeric)))
            stop("All columns of addcovar must be numeric")
        if( nrow(addcovar) != orig.n.ind ) {
            # the length of additive covariates is incorrect
            stop("Number of rows in additive covariates is incorrect")
        }
        addcovar <- addcovar[keep.ind,,drop=FALSE]
        n.addcovar <- ncol(addcovar)

    }
    if(!is.null(intcovar)) { # interacting covariates
        if(!is.matrix(intcovar)) {
            if(!is.numeric(as.matrix(intcovar)))
                stop("intcovar should be a numeric")
            if(is.vector(intcovar) || is.data.frame(intcovar))
                intcovar <- as.matrix(intcovar)
            else stop("intcovar should be a matrix")
        }
        if(!all(apply(intcovar,2,is.numeric)))
            stop("All columns of intcovar must be numeric")
        if(nrow(intcovar)[1] != orig.n.ind) {
            # the length of interacting covariates is incorrect
            stop("The length of interacting covariates is incorrect!")
        }
        intcovar <- intcovar[keep.ind,,drop=FALSE]
        n.intcovar <- ncol(intcovar)
    }
    if(!is.null(perm.strata)) perm.strata <- perm.strata[keep.ind]
    if(!is.null(ind.noqtl)) ind.noqtl <- ind.noqtl[keep.ind]
    if(!is.null(weights)) weights <- weights[keep.ind]

    # drop individuals missing any covariates
    if(!is.null(addcovar)) { # note that intcovar is contained in addcovar
        wh <- apply(cbind(addcovar,intcovar),1,function(a) any(!is.finite(a)))
        if(any(wh)) {
            cross <- subset.cross(cross,ind=(!wh))
            pheno <- pheno[!wh,,drop=FALSE]
            addcovar <- addcovar[!wh,,drop=FALSE]
            if(!is.null(intcovar)) intcovar <- intcovar[!wh,,drop=FALSE]
            n.ind <- nind(cross)
            if(!is.null(perm.strata)) perm.strata <- perm.strata[!wh]
            if(!is.null(ind.noqtl)) ind.noqtl <- ind.noqtl[!wh]
            if(!is.null(weights)) weights <- weights[!wh]
            if(verbose) warning("Dropping ", sum(wh), " individuals with missing covariates.\n")
        }
    }

    # make sure columns of intcovar are contained in addcovar
    if(!is.null(intcovar)) {
        if(is.null(addcovar)) {
            addcovar <- intcovar
            n.addcovar <- n.intcovar
            if(verbose) warning("addcovar forced to contain all columns of intcovar\n")
        }
        else {
            wh <- 1:n.intcovar
            for(i in 1:n.intcovar) {
                o <- (apply(addcovar,2,function(a,b) max(abs(a-b)),intcovar[,i])<1e-14)
                if(any(o)) wh[i] <- (1:n.addcovar)[o]
                else wh[i] <- NA
            }
            if(any(!is.finite(wh))) {
                addcovar <- cbind(addcovar,intcovar[,!is.finite(wh)])
                n.addcovar <- ncol(addcovar)
                if(verbose) warning("addcovar forced to contain all columns of intcovar")
            }
        }
    }

    if(n.addcovar > 0) { # check rank
        if(qr(cbind(1,addcovar))$rank < n.addcovar+1)
            if(verbose) warning("addcovar appears to be over-specified; consider dropping columns.\n")
    }
    if(n.intcovar > 0) { # check rank
        if(qr(cbind(1,intcovar))$rank < n.intcovar+1)
            if(verbose) warning("intcovar appears to be over-specified; consider dropping columns.\n")
    }

    pheno <- as.matrix(pheno)

    list(cross=cross, pheno=pheno, addcovar=addcovar, intcovar=intcovar,
         n.addcovar=n.addcovar, n.intcovar=n.intcovar, perm.strata=perm.strata,
         ind.noqtl=ind.noqtl, weights=weights)
}

# Find the nearest marker to a particular position
find.marker <-
    function(cross, chr, pos, index)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(missing(pos) && missing(index))
        stop("Give either pos or index.")
    if(!missing(pos) && !missing(index))
        stop("Give just one of pos or index.")

    # if chr has length 1, expand if necessary
    if(length(chr) == 1) {
        if(!missing(pos))
            chr <- rep(chr,length(pos))
        else
            chr <- rep(chr, length(index))
    }
    # otherwise, chr and pos should have same length
    else {
        if(!missing(pos) && length(chr) != length(pos))
            stop("chr and pos must be the same length.")
        if(!missing(index) && length(chr) != length(index))
            stop("chr and index must be the same length.")
    }

    markers <- rep("",length(chr))
    chrnotfound <- NULL
    for(i in 1:length(chr)) {
        # find chromosome
        o <- match(chr[i], names(cross$geno))
        if(is.na(o)) {
            markers[i] <- NA  # chr not matched
            chrnotfound <- c(chrnotfound, chr[i])
        }
        else {
            thismap <- cross$geno[[o]]$map # genetic map
            # sex-specific map; look at female positions
            if(is.matrix(thismap)) thismap <- thismap[1,]

            if(!missing(pos)) {
                # find closest marker
                d <- abs(thismap-pos[i])
                o2 <- (1:length(d))[d==min(d)]
                if(length(o2)==1) markers[i] <- names(thismap)[o2]
                # if multiple markers are equidistant,
                #     choose the one with the most data
                #     or choose among them at random
                else {
                    x <- names(thismap)[o2]
                    n.geno <- apply(cross$geno[[o]]$data[,o2],2,function(a) sum(!is.na(a)))

                    o2 <- o2[n.geno==max(n.geno)]
                    if(length(o2) == 1)
                        markers[i] <- names(thismap)[o2]
                    else
                        markers[i] <- names(thismap)[sample(o2,1)]

                }
            }
            else { # by index
                if(index[i] < 1 || index[i] > length(thismap))
                    stop("Misspecified index ", index[i], " on chr ", chr[i])
                markers[i] <- names(thismap)[index[i]]
            }
        }
    }
    if(length(chrnotfound) > 0) {
        chrnotfound <- sort(unique(chrnotfound))
        if(length(chrnotfound) == 1)
            warning("Chromosome ", paste("\"", chrnotfound, "\"", sep=""), " not found")
        else
            warning("Chromosomes ", paste("\"", chrnotfound, "\"", sep="", collapse=", "), " not found")
    }

    markers
}


### Find the nearest pseudomarker to a particular position
find.pseudomarker <-
    function(cross, chr, pos, where=c("draws","prob"), addchr=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    # if chr has length 1, expand if necessary
    if(length(chr) == 1)
        chr <- rep(chr,length(pos))
    # otherwise, chr and pos should have same length
    else if(length(chr) != length(pos))
        stop("chr and pos must be the same length.")

    markers <- rep("",length(chr))

    where <- match.arg(where)
    if(where=="draws" && !("draws" %in% names(cross$geno[[1]])))
        stop("You'll need to first run sim.geno")
    if(where=="prob" && !("prob" %in% names(cross$geno[[1]])))
        stop("You'll need to first run calc.genoprob")

    for(i in 1:length(chr)) {
        # find chromosome
        o <- match(chr[i], names(cross$geno))
        if(is.na(o)) markers[i] <- NA  # chr not matched
        else {
            if(where=="draws") {
                if("map" %in% names(attributes(cross$geno[[o]]$draws)))
                    thismap <- attr(cross$geno[[o]]$draws, "map")
                else {
                    stp <- attr(cross$geno[[o]]$draws, "step")
                    oe <- attr(cross$geno[[o]]$draws, "off.end")

                    if("stepwidth" %in% names(attributes(cross$geno[[o]]$draws)))
                        stpw <- attr(cross$geno[[o]]$draws, "stepwidth")
                    else stpw <- "fixed"
                    thismap <- create.map(cross$geno[[o]]$map,stp,oe,stpw)
                    attr(cross$geno[[o]]$draws, "map") <- thismap
                }
            }
            else { # prob
                if("map" %in% names(attributes(cross$geno[[o]]$prob)))
                    thismap <- attr(cross$geno[[o]]$prob, "map")
                else {
                    stp <- attr(cross$geno[[o]]$prob, "step")
                    oe <- attr(cross$geno[[o]]$prob, "off.end")

                    if("stepwidth" %in% names(attributes(cross$geno[[o]]$prob)))
                        stpw <- attr(cross$geno[[o]]$prob, "stepwidth")
                    else stpw <- "fixed"
                    thismap <- create.map(cross$geno[[o]]$map,stp,oe,stpw)
                    attr(cross$geno[[o]]$prob, "map") <- thismap
                }
            }

            # sex-specific map; look at female positions
            if(is.matrix(thismap)) thismap <- thismap[1,]

            # find closest marker
            d <- abs(thismap-pos[i])
            o2 <- (1:length(d))[d==min(d)]
            if(length(o2)==1) themarker <- names(thismap)[o2]
            else themarker <- names(thismap)[sample(o2, 1)]

            if(addchr && length(grep("^loc[0-9]+\\.*[0-9]*(\\.[0-9]+)*$", themarker)) > 0)
                themarker <- paste("c", chr[i], ".", themarker, sep="")
            markers[i] <- themarker
        }
    }

    markers
}



# expand recombination fractions for RI lines
adjust.rf.ri <-
    function(r, type=c("self","sib"), chrtype=c("A","X"), expand=TRUE)
{
    # type of RI lines
    type <- match.arg(type)
    chrtype <- match.arg(chrtype)

    if(type=="self") {
        if(expand) return(r*2/(1+2*r))
        else return(r/2/(1-r))
    }
    else {
        if(chrtype=="A") { # autosome / sib mating
            if(expand) return(r*4/(1+6*r))
            else return(r/(4-6*r))
        }
        else { # X chromosome/ sib mating
            if(expand) return(8/3*r/(1+4*r))
            else return(3/8*r/(1-1.5*r))
        }
    }
}

######################################################################
# lodint: function to get lod support interval
######################################################################
lodint <-
    function(results, chr, qtl.index, drop=1.5, lodcolumn=1,
             expandtomarkers=FALSE)
{
    if(!inherits(results, "scanone")) {
        if(!inherits(results, "qtl"))
            stop("Input must have class \"scanone\" or \"qtl\".")
        else {
            if(!("lodprofile" %in% names(attributes(results))))
                stop("qtl object needs to be produced by refineqtl with keeplodprofile=TRUE.")
            else { # qtl object
                if(lodcolumn != 1) {
                    warning("lod column ignored if input is a qtl object.")
                    lodcolumn <- 1
                }
                results <- attr(results, "lodprofile")
                if(missing(qtl.index)) {
                    if(length(results)==1)
                        results <- results[[1]]
                    else
                        stop("You must specify qtl.index.")
                }
                else {
                    if(length(qtl.index)>1) stop("qtl.index should have length 1")
                    if(qtl.index < 1 || qtl.index > length(results)) {
                        stop("qtl.index should be between 1 and ", length(results))
                    }
                    results <- results[[qtl.index]]
                }
                chr <- results[1,1]
            }
        }
    }
    else {
        if(lodcolumn < 1 || lodcolumn +2 > ncol(results))
            stop("Argument lodcolumn should be between 1 and ", ncol(results)-2)

        if(missing(chr)) {
            if(length(unique(results[,1]))>1)
                stop("Give a chromosome ID.")
        }
        else {
            if(length(chr) > 1) stop("chr should have length 1")
            if(is.na(match(chr, results[,1])))
                stop("Chromosome ", chr, " not found.")
            results <- results[results[,1]==chr,]
        }
    }

    if(all(is.na(results[,lodcolumn+2]))) return(NULL)

    maxlod <- max(results[,lodcolumn+2],na.rm=TRUE)
    w <- which(!is.na(results[,lodcolumn+2]) & results[,lodcolumn+2] == maxlod)
    o <- range(which(!is.na(results[,lodcolumn+2]) & results[,lodcolumn+2] > maxlod-drop))

    if(length(o)==0) o <- c(1,nrow(results))

    else {
        if(o[1] > 1) o[1] <- o[1]-1
        if(o[2] < nrow(results)) o[2] <- o[2]+1
    }

    if(expandtomarkers) {
        markerpos <- (1:nrow(results))[-grep("^c.+\\.loc-*[0-9]+(\\.[0-9]+)*$", rownames(results))]
        if(any(markerpos <= o[1]))
            o[1] <- max(markerpos[markerpos <= o[1]])
        if(any(markerpos >= o[2]))
            o[2] <- min(markerpos[markerpos >= o[2]])
    }

    rn <- rownames(results)[c(o[1],w,o[2])]

    # look for duplicate rows
    if(length(w)>1 && rn[length(rn)]==rn[length(rn)-1]) w <- w[-length(w)]
    else if(length(w)>1 && rn[2]==rn[1]) w <- w[-1]
    rn <- rownames(results)[c(o[1],w,o[2])]

    # look for more duplicate rows
    if(any(table(rn)> 1)) {
        tab <- table(rn)
        temp <- which(tab>1)
        for(j in temp) {
            z <- which(rn==names(tab)[j])
            for(k in 2:length(z))
                rn[z[k:length(z)]] <- paste(rn[z[k:length(z)]], " ", sep="")
        }
    }

    results <- results[c(o[1],w,o[2]),]
    rownames(results) <- rn
    class(results) <- c("scanone","data.frame")

    results
}


######################################################################
# bayesint: function to get Bayesian probability interval
######################################################################
bayesint <-
    function(results, chr, qtl.index, prob=0.95, lodcolumn=1, expandtomarkers=FALSE)
{
    if(!inherits(results, "scanone")) {
        if(!inherits(results, "qtl"))
            stop("Input must have class \"scanone\" or \"qtl\".")
        else {
            if(!("lodprofile" %in% names(attributes(results))))
                stop("qtl object needs to be produced by refineqtl with keeplodprofile=TRUE.")
            else { # qtl object
                if(lodcolumn != 1) {
                    warning("lod column ignored if input is a qtl object.")
                    lodcolumn <- 1
                }
                results <- attr(results, "lodprofile")
                if(missing(qtl.index)) {
                    if(length(results)==1)
                        results <- results[[1]]
                    else
                        stop("You must specify qtl.index.")
                }
                else {
                    if(length(qtl.index)>1) stop("qtl.index should have length 1")
                    if(qtl.index < 1 || qtl.index > length(results))
                        stop("qtl.index should be between 1 and ", length(results))
                    results <- results[[qtl.index]]
                }
                chr <- results[1,1]
            }
        }
    }
    else {
        if(lodcolumn < 1 || lodcolumn +2 > ncol(results))
            stop("Argument lodcolumn should be between 1 and ", ncol(results)-2)

        if(missing(chr)) {
            if(length(unique(results[,1]))>1)
                stop("Give a chromosome ID.")
        }
        else {
            if(length(chr) > 1) stop("chr should have length 1")
            if(is.na(match(chr, results[,1])))
                stop("Chromosome ", chr, " not found.")
            results <- results[results[,1]==chr,]
        }
    }

    if(all(is.na(results[,lodcolumn+2]))) return(NULL)

    loc <- results[,2]
    width <- (c(loc[-1], loc[length(loc)]) - c(loc[1], loc[-length(loc)]))/2
    width[c(1, length(width))] <- width[c(1, length(width))]*2 # adjust widths at ends

    area <- 10^results[,lodcolumn+2]*width
    area <- area/sum(area)

    o <- order(results[,lodcolumn+2], decreasing=TRUE)

    cs <- cumsum(area[o])

    wh <- min(which(cs >= prob))
    int <- range(o[1:wh])

    if(expandtomarkers) {
        markerpos <- (1:nrow(results))[-grep("^c.+\\.loc-*[0-9]+(\\.[0-9]+)*$", rownames(results))]
        if(any(markerpos <= int[1]))
            int[1] <- max(markerpos[markerpos <= int[1]])
        if(any(markerpos >= int[2]))
            int[2] <- min(markerpos[markerpos >= int[2]])
    }

    rn <- rownames(results)[c(int[1],o[1],int[2])]
    # look for duplicate rows
    if(any(table(rn)> 1)) {
        rn[2] <- paste(rn[2], "")
        if(rn[1] == rn[3]) rn[3] <- paste(rn[3], " ")
    }

    results <- results[c(int[1],o[1],int[2]),]
    rownames(results) <- rn
    class(results) <- c("scanone", "data.frame")
    results
}


######################################################################
# makeSSmap: convert a genetic map, or the genetic maps in a cross
#            object, to be sex-specific (i.e., 2-row matrices)
######################################################################
makeSSmap <-
    function(cross)
{
    if(!inherits(cross, "map")) {
        # input object is a genetic map
        for(i in 1:length(cross)) {
            if(!is.matrix(cross[[i]]))
                cross[[i]] <- rbind(cross[[i]], cross[[i]])
        }
    }
    else { # input object is assumed to be a "cross" object
        n.chr <- nchr(cross)
        for(i in 1:n.chr) {
            if(!is.matrix(cross$geno[[i]]$map))
                cross$geno[[i]]$map <-
                    rbind(cross$geno[[i]]$map, cross$geno[[i]]$map)
        }
    }

    cross
}

######################################################################
# comparecrosses: verify that two cross objects have identical
#                 classes, chromosomes, markers, genotypes, maps,
#                 and phenotypes
######################################################################
comparecrosses <-
    function(cross1, cross2, tol=1e-5)
{
    if(missing(cross1) || missing(cross2))
        stop("Two crosses must be input.")

    # both are of class "cross"
    if(!inherits(cross1, "cross") ||
       !inherits(cross2, "cross"))
        stop("Input should have class \"cross\".")

    # classes are the same
    if(crosstype(cross1) != crosstype(cross2))
        stop("crosses are not the same type.")

    if(nchr(cross1) != nchr(cross2))
        stop("crosses do not have the same number of chromosomes.")

    if(any(names(cross1$geno) != names(cross2$geno)))
        stop("Chromosome names do not match.")

    if(any(nmar(cross1) != nmar(cross2)))
        stop("Number of markers per chromosome do not match.")

    mnames1 <- unlist(lapply(cross1$geno, function(a) colnames(a$data)))
    mnames2 <- unlist(lapply(cross2$geno, function(a) colnames(a$data)))
    if(any(mnames1 != mnames2)) {
        #    stop("Markers names do not match.")
        for(i in 1:nchr(cross1))
            if(any(colnames(cross1$geno[[i]]$data) != colnames(cross2$geno[[i]]$data)))
                stop("Marker names on chr ", names(cross1$geno)[i], " don't match.")
    }


    chr_type1 <- sapply(cross1$geno, chrtype)
    chr_type2 <- sapply(cross2$geno, chrtype)
    if(any(chr_type1 != chr_type2))
        stop("Chromosome types (autosomal vs X) do not match.")

    for(i in 1:nchr(cross1)) {
        if(any(abs(diff(cross1$geno[[i]]$map) - diff(cross2$geno[[i]]$map)) > tol))
            stop("Genetic maps for chromosome ", names(cross1$geno)[i],
                 " do not match.")

        if(abs(cross1$geno[[i]]$map[1] - cross2$geno[[i]]$map[1]) > tol)
            warning("Initial marker positions for chromosome ", names(cross1$geno)[i],
                    " do not match.")
    }

    if(nind(cross1) != nind(cross2))
        stop("Number of individuals do not match.")

    for(i in 1:nchr(cross1)) {
        g1 <- cross1$geno[[i]]$data
        g2 <- cross2$geno[[i]]$data
        if(any((is.na(g1) & !is.na(g2)) | (!is.na(g1) & is.na(g2)) |
               (!is.na(g1) & !is.na(g2) & g1!=g2)))
            stop("Genotype data for chromosome ", names(cross1$geno)[i],
                 " do not match.")
    }

    if(nphe(cross1) != nphe(cross2))
        stop("Number of phenotypes do not match.")

    if(any(names(cross1$pheno) != names(cross2$pheno)))
        stop("Phenotype names do not match.")

    for(i in 1:nphe(cross1)) {
        phe1 <- cross1$pheno[,i]
        phe2 <- cross2$pheno[,i]
        if(is.numeric(phe1) & is.numeric(phe2)) {
            phe1[phe1 == Inf] <- max(phe1[phe1 < Inf], na.rm=TRUE)+5
            phe2[phe2 == Inf] <- max(phe2[phe2 < Inf], na.rm=TRUE)+5
            phe1[phe1 == -Inf] <- min(phe1[phe1 > -Inf], na.rm=TRUE)-5
            phe2[phe2 == -Inf] <- min(phe2[phe2 > -Inf], na.rm=TRUE)-5
            if(any((is.na(phe1) & !is.na(phe2)) | (!is.na(phe1) & is.na(phe2)) |
                   (!is.na(phe1) & !is.na(phe2) & abs(phe1-phe2) > tol))) {
                stop("Data for phenotype ", names(cross1$pheno)[i],
                     " do not match.")
            }
        }
        else {
            if(any((is.na(phe1) & !is.na(phe2)) | (!is.na(phe1) & is.na(phe2)) |
                   (!is.na(phe1) & !is.na(phe2) &
                    as.character(phe1) != as.character(phe2)))) {
                stop("Data for phenotype ", names(cross1$pheno)[i], " do not match.")
            }
        }
    }

    message("\tCrosses are identical.")
}


######################################################################
# move marker
# Move a marker to a new chromosome...placed at the end
######################################################################
movemarker <-
    function(cross, marker, newchr, newpos)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    mnames <- unlist(lapply(cross$geno,function(a) colnames(a$data)))
    chr <- rep(names(cross$geno),nmar(cross))
    pos <- unlist(lapply(cross$geno,function(a) 1:ncol(a$data)))
    oldindex <- match(marker, mnames)

    # Marker found precisely once?
    if(is.na(oldindex)) stop(marker, " not found.\n")
    if(length(oldindex) > 1) stop(marker, " found multiple times.\n")

    chr <- chr[oldindex]
    pos <- pos[oldindex]

    # pull out genotype data
    g <- cross$geno[[chr]]$data[,pos]

    chr_type <- chrtype(cross$geno[[chr]])
    mapmatrix <- is.matrix(cross$geno[[chr]]$map)

    # delete marker
    if(nmar(cross)[chr] == 1)  { # only marker on that chromosome, so drop the chromosome
        cross$geno <- cross$geno[-match(chr,names(cross$geno))]
        delchr <- TRUE
    }
    else {
        delchr <- FALSE
        cross$geno[[chr]]$data <- cross$geno[[chr]]$data[,-pos,drop=FALSE]
        if(is.matrix(cross$geno[[chr]]$map))
            cross$geno[[chr]]$map <- cross$geno[[chr]]$map[,-pos,drop=FALSE]
        else
            cross$geno[[chr]]$map <- cross$geno[[chr]]$map[-pos]
    }

    if(is.numeric(newchr)) newchr <- as.character(newchr)

    if(!(newchr %in% names(cross$geno))) { # create a new chromosome
        n <- length(cross$geno)
        cross$geno[[n+1]] <- list("data"=as.matrix(g),
                                  "map"=as.numeric(0))
        names(cross$geno)[n+1] <- newchr
        class(cross$geno[[n+1]]) <- chr_type
        colnames(cross$geno[[n+1]]$data) <- marker
        if(mapmatrix) {
            if(missing(newpos)) newpos <- 0
            cross$geno[[n+1]]$map <- matrix(newpos, ncol=1, nrow=2)
            colnames(cross$geno[[n+1]]$map) <- marker
        }
        else {
            if(missing(newpos)) newpos <- 0
            cross$geno[[n+1]]$map <- newpos
            names(cross$geno[[n+1]]$map) <- marker
        }
        return(cross)
    }

    if(missing(newpos)) {
        # add marker to end of new chromosome
        n.mar <- nmar(cross)[newchr]

        cross$geno[[newchr]]$data <- cbind(cross$geno[[newchr]]$data,g)
        colnames(cross$geno[[newchr]]$data)[n.mar+1] <- marker

        if(is.matrix(cross$geno[[newchr]]$map)) {
            cross$geno[[newchr]]$map <- cbind(cross$geno[[newchr]]$map,
                                              cross$geno[[newchr]]$map[,n.mar]+10)
            colnames(cross$geno[[newchr]]$map)[n.mar+1] <- marker
        }
        else {
            cross$geno[[newchr]]$map <- c(cross$geno[[newchr]]$map,
                                          cross$geno[[newchr]]$map[n.mar]+10)
            names(cross$geno[[newchr]]$map)[n.mar+1] <- marker
        }
    }
    else {
        # add marker to the specified position
        dat <- cross$geno[[newchr]]$data
        map <- cross$geno[[newchr]]$map

        if(length(newpos) != 1)
            stop("newpos should be a single number.")

        if(is.matrix(map)) { # sex-specific maps
            wh <- which(map[1,] < newpos)
            if(length(wh) == 0) { # place in first spot
                map <- cbind(c(newpos,map[2,1]-(map[1,1]-newpos)),map)
                colnames(map)[1] <- marker
            }
            else {
                wh <- max(wh)
                if(wh == ncol(map)) { # place at end of chromosome
                    map <- cbind(map,c(newpos,map[2,ncol(map)]+(newpos-map[1,ncol(map)])))
                    colnames(map)[ncol(map)] <- marker
                }
                else {
                    left <- map[,1:wh,drop=FALSE]
                    right <- map[,-(1:wh),drop=FALSE]
                    marleft <- colnames(left)
                    marright <- colnames(right)
                    left <- left[,ncol(left)]
                    right <- right[,1]

                    newpos2 <- (newpos-left[1])/(right[1]-left[1])*(right[2]-left[2])+left[2]
                    map <- cbind(map[,1:wh], c(newpos,newpos2), map[,-(1:wh)])
                    colnames(map) <- c(marleft, marker, marright)
                }
            }
        }
        else {
            wh <- which(map < newpos)
            if(length(wh) == 0) { # place in first position
                map <- c(newpos,map)
                names(map)[1] <- marker
            }
            else {
                wh <- max(wh)
                if(wh == length(map)) { # place in last position
                    map <- c(map,newpos)
                    names(map)[length(map)] <- marker
                }
                else {
                    map <- c(map[1:wh],newpos,map[-(1:wh)])
                    names(map)[wh+1] <- marker
                }
            }
        }
        cross$geno[[newchr]]$map <- map

        if(length(wh)==0) { # place marker in first position
            dat <- cbind(g, dat)
            colnames(dat)[1] <- marker
        }
        else if(wh == ncol(dat)) { # place marker in last position
            dat <- cbind(dat, g)
            colnames(dat)[ncol(dat)] <- marker
        }
        else { # place marker in the middle
            dat <- cbind(dat[,1:wh],g,dat[,-(1:wh)])
            colnames(dat)[wh+1] <- marker
        }
        cross$geno[[newchr]]$data <- dat

        # make sure the marker names for the data and the genetic map match
        if(is.matrix(cross$geno[[newchr]]$map))
            colnames(cross$geno[[newchr]]$data) <- colnames(cross$geno[[newchr]]$map)
        else
            colnames(cross$geno[[newchr]]$data) <- names(cross$geno[[newchr]]$map)
    }

    # update genoprob, errorlod, argmax, draws, rf
    if("rf" %in% names(cross)) {
        # reorder the recombination fractions
        #  -- a bit of pain, 'cause we need LODs in upper triangle
        #     and rec fracs in lower triangle
        newmar <- unlist(lapply(cross$geno,function(a) colnames(a$data)))

        attrib <- attributes(cross$rf)

        rf <- cross$rf
        lods <- rf;lods[lower.tri(rf)] <- t(rf)[lower.tri(rf)]
        rf[upper.tri(rf)] <- t(rf)[upper.tri(rf)]
        lods <- lods[newmar,newmar]
        rf <- rf[newmar,newmar]
        rf[upper.tri(rf)] <- lods[upper.tri(rf)]

        cross$rf <- rf

        if("onlylod" %in% names(attrib)) # save the onlylod attribute if its there
            attr(cross$rf, "onlylod") <- attrib$onlylod
    }


    if(!delchr) thechr <- c(chr,newchr)
    else thechr <- newchr
    for(i in thechr) {
        tempg <- cross$geno[[i]]
        tempx <- subset(cross, chr=i)
        if("prob" %in% names(tempg))
            atp <- attributes(tempg$prob)
        if("draws" %in% names(tempg)) {
            at <- attributes(cross$geno[[1]]$draws)
            tempg$draws <- sim.geno(tempx,
                                    n.draws=at$dim[3],
                                    step=at$step,
                                    off.end=at$off.end,
                                    map.function=at$map.function,
                                    error.prob=at$error.prob)$geno[[1]]$draws
        }
        if("argmax" %in% names(tempg)) {
            at <- attributes(cross$geno[[1]]$argmax)
            tempg$argmax <- argmax.geno(tempx,
                                        step=at$step,
                                        off.end=at$off.end,
                                        map.function=at$map.function,
                                        error.prob=at$error.prob)$geno[[1]]$argmax
        }
        if("errorlod" %in% names(tempg)) {
            at <- attributes(cross$geno[[1]]$errorlod)
            tempg$errorlod <- argmax.geno(tempx,
                                          map.function=at$map.function,
                                          error.prob=at$error.prob)$geno[[1]]$errorlod
        }
        if("prob" %in% names(tempg))
            tempg$prob <- calc.genoprob(tempx,
                                        step=atp$step,
                                        off.end=atp$off.end,
                                        map.function=atp$map.function,
                                        error.prob=atp$error.prob)$geno[[1]]$prob

        cross$geno[[i]] <- tempg
    }

    cross
}

######################################################################
#
# summary.map
#
# Give a short summary of a genetic map object.
#
######################################################################
summaryMap <- summary.map <-
    function(object, ...)
{
    map <- object
    if(inherits(map, "cross")) # a cross object
        map <- pull.map(map)
    if(!inherits(map, "map") && !is.list(map))
        warning("Input should have class \"cross\" or \"map\".")

    n.chr <- length(map)
    chrnames <- names(map)
    if(is.null(chrnames)) chrnames <- 1:length(map)
    if(is.matrix(map[[1]])) { # sex-specific map
        sexsp <- TRUE
        n.mar <- sapply(map,ncol)
        tot.mar <- sum(n.mar)
        fmap <- lapply(map,function(a) a[1,])
        mmap <- lapply(map,function(a) a[2,])

        len.f <- sapply(fmap,function(a) diff(range(a)))
        len.m <- sapply(mmap,function(a) diff(range(a)))
        avesp.f <- sapply(fmap,function(a) {if(length(a)<2) return(NA); mean(diff(a))})
        avesp.m <- sapply(mmap,function(a) {if(length(a)<2) return(NA); mean(diff(a))})
        maxsp.f <- sapply(fmap,function(a) {if(length(a)<2) return(NA); max(diff(a))})
        maxsp.m <- sapply(mmap,function(a) {if(length(a)<2) return(NA); max(diff(a))})
        totlen.f <- sum(len.f)
        totlen.m <- sum(len.m)

        tot.avesp.f <- mean(unlist(lapply(fmap,diff)), na.rm=TRUE)
        tot.avesp.m <- mean(unlist(lapply(mmap,diff)), na.rm=TRUE)
        tot.maxsp.f <- max(maxsp.f,na.rm=TRUE)
        tot.maxsp.m <- max(maxsp.m,na.rm=TRUE)

        output <- rbind(cbind(n.mar,len.f,len.m,avesp.f,avesp.m, maxsp.f, maxsp.m),
                        c(tot.mar,totlen.f,totlen.m,tot.avesp.f,tot.avesp.m,
                          tot.maxsp.f, tot.maxsp.m))
        dimnames(output) <- list(c(chrnames,"overall"),
                                 c("n.mar","length.female","length.male",
                                   "ave.spacing.female","ave.spacing.male",
                                   "max.spacing.female", "max.spacing.male"))
    }
    else {
        sexsp=FALSE
        n.mar <- sapply(map,length)
        len <- sapply(map,function(a) diff(range(a)))
        tot.mar <- sum(n.mar)

        len <- sapply(map,function(a) diff(range(a)))
        avesp <- sapply(map,function(a) {if(length(a)<2) return(NA); mean(diff(a))})
        maxsp <- sapply(map,function(a) {if(length(a)<2) return(NA); max(diff(a))})
        totlen <- sum(len)
        tot.avesp <- mean(unlist(lapply(map,diff)), na.rm=TRUE)
        tot.maxsp <- max(maxsp, na.rm=TRUE)

        output <- rbind(cbind(n.mar,len,avesp, maxsp),
                        c(tot.mar,totlen,tot.avesp, tot.maxsp))
        dimnames(output) <- list(c(chrnames,"overall"),
                                 c("n.mar","length","ave.spacing", "max.spacing"))
    }

    output <- as.data.frame(output)
    attr(output, "sexsp") <- sexsp
    class(output) <- c("summary.map", "data.frame")
    output
}


######################################################################
#
# print.summary.map
#
# Print out the result of summary.map()
#
######################################################################
print.summary.map <-
    function(x, ...)
{
    sexsp <- attr(x, "sexsp")
    if(sexsp) cat("Sex-specific map\n\n")

    x <- apply(x,2,round,1)
    print(x)
}

######################################################################
# convert functions
######################################################################
convert <-
    function(object, ...)
    UseMethod("convert")


######################################################################
#
# convert.scanone
#
# Convert scanone output from the format for R/qtl ver 0.97 to
# that for R/qtl ver 0.98
# (previously, inter-maker locations named loc*.c*; now c*.loc*)
#
######################################################################
convert.scanone <-
    function(object, ...)
{
    if(!inherits(object, "scanone"))
        stop("Input should have class \"scanone\".")

    rn <- rownames(object)
    o <- grep("^loc-*[0-9]+(\\.[0-9]+)*\\.c[0-9A-Za-z]+$", rn)
    if(length(o) > 0) {
        temp <- rn[o]
        temp <- strsplit(temp,"\\.")
        temp <- sapply(temp, function(a)
                       paste(a[c(length(a),1:(length(a)-1))],collapse="."))
        rownames(object)[o] <- temp
    }
    object
}

######################################################################
# convert.scantwo
#
# convert scantwo output from the format used in R/qtl version 1.03
# and earlier to that used in R/qtl version 1.04 and later.
#
# 1.03 and earlier: contained joint and interactive LOD scores
# 1.04 and later:   contains joint LOD scores and LOD scores from the
#                   additive QTL model
######################################################################
convert.scantwo <-
    function(object, ...)
{
    if(!inherits(object, "scantwo"))
        stop("Input should have class \"scantwo\".")

    lod <- object$lod
    if(length(dim(lod)) == 2) {
        u <- upper.tri(lod)
        lod[u] <- t(lod)[u] - lod[u]
    }
    else { # multiple phenotypes
        u <- upper.tri(lod[,,1])
        for(i in 1:dim(lod)[3])
            lod[,,i][u] <- t(lod[,,i])[u] - lod[,,i][u]
    }
    object$lod <- lod
    object
}


######################################################################
# convert.map
#
# convert a genetic map from one map function to another
######################################################################
convert.map <-
    function(object, old.map.function=c("haldane", "kosambi", "c-f", "morgan"),
             new.map.function=c("haldane", "kosambi", "c-f", "morgan"), ...)
{
    old.map.function <- match.arg(old.map.function)
    new.map.function <- match.arg(new.map.function)
    if(!inherits(object, "map"))
        stop("Input should have class \"map\".")

    if(old.map.function==new.map.function) {
        warning("old and new map functions are the same; no change.")
        return(object)
    }

    mf <- switch(old.map.function,
                 "haldane"=mf.h,
                 "kosambi"=mf.k,
                 "c-f"=mf.cf,
                 "morgan"=mf.m)
    imf <- switch(new.map.function,
                  "haldane"=imf.h,
                  "kosambi"=imf.k,
                  "c-f"=imf.cf,
                  "morgan"=imf.m)

    if(is.matrix(object[[1]])) { # sex-specific map
        for(i in seq(along=object)) {
            theclass <- class(object[[i]])
            thenames <- colnames(object[[i]])
            for(j in 1:2)
                object[[i]][j,] <- cumsum(c(object[[i]][j,1], imf(mf(diff(object[[i]][j,])))))

            class(object[[i]]) <- theclass
            colnames(object[[i]]) <- thenames
        }
    }
    else {
        for(i in seq(along=object)) {
            theclass <- class(object[[i]])
            thenames <- names(object[[i]])
            object[[i]] <- cumsum(c(object[[i]][1], imf(mf(diff(object[[i]])))))
            class(object[[i]]) <- theclass
            names(object[[i]]) <- thenames
        }
    }

    object
}


######################################################################
# find.pheno
#
# utility to get pheno number given pheno name
######################################################################
find.pheno <-
    function( cross,  pheno )
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    seq( ncol( cross$pheno ))[match(pheno,names(cross$pheno))]
}

######################################################################
# find.flanking
#
# utility to get flanking and/or closest marker to chr and pos
######################################################################
find.flanking <-
    function( cross, chr, pos)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    map = pull.map(cross)

    if(is.matrix(map[[1]]) && nrow(map[[1]]) > 1)
        stop("This function works only for crosses with sex-averaged maps.")

    if(length(chr) == 1 && length(pos) > 1) {
        chr <- rep(chr,length(pos))
    }
    wh <- match(chr, names(cross$geno))
    if(any(is.na(wh))) {
        stop("Chr ", paste(chr[is.na(wh)], collapse=", "), " not found.")
        wh <- wh[!is.na(wh)]
    }
    chr <- names(cross$geno)[wh]

    marker = NULL
    for (i in seq(length(chr))) {
        tmp = map[[chr[i]]]-pos[i]
        m = names(map[[chr[i]]])
        left = sum(tmp < 0)
        at = sum(tmp == 0)
        right = sum(tmp > 0)
        f <- if (at > 0)
            left+at[c(1,length(at))]
        else {
            if (right > 0)
                c(left,left+at+1)
            else
                c(left,left+at)
        }
        marker = rbind(marker,m[f[c(1:2,order(abs(tmp[f]))[1])]])
    }
    dimnames(marker) <- list(paste(chr,":",pos,sep=""),
                             c("left","right","close"))
    as.data.frame(marker, stringsAsFactors=TRUE)
}

######################################################################
# strip.partials
#
# Removes partially informative genotypes in an intercross.
#
# Input: a cross object; if verbose=TRUE, a statement regarding the
#        number of genotypes removed is printed.
######################################################################
strip.partials <-
    function(cross, verbose=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    type <- crosstype(cross)
    if(type != "f2")
        stop("This is for intercrosses only")

    n.removed <- 0
    for(i in 1:nchr(cross)) {
        g <- cross$geno[[i]]$data
        wh <- !is.na(g) & g>3
        if(any(wh)) {
            g[wh] <- NA
            cross$geno[[i]]$data <- g
            n.removed <- n.removed + sum(wh)
        }
    }
    if(verbose) {
        if(n.removed == 0) cat(" --Didn't remove any genotypes.\n")
        else cat("Removed", n.removed, "genotypes.\n")
    }
    cross
}

######################################################################
# print the installed version of R/qtl
######################################################################
qtlversion <-
    function()
{
    version <- unlist(packageVersion("qtl"))

    if(length(version) == 3) {
        # make it like #.#-#
        return( paste(c(version, ".", "-")[c(1,4,2,5,3)], collapse="") )
    }

    paste(version, collapse=".")
}


######################################################################
#
# locateXO
#
# Locate crossovers on a single chromosome in each individual
# Look at just the first chromosome
#
######################################################################

locateXO <-
    function(cross, chr, full.info=FALSE)
{
    if(!missing(chr)) {
        cross <- subset(cross, chr=chr)
        if(nchr(cross) != 1)
            warning("locateXO works on just one chr; considering chr ", names(cross$geno)[1])
    }

    # individual IDs
    id <- getid(cross)
    if(is.null(id)) id <- as.character(1:nind(cross))

    if(nmar(cross)[1] == 1) { # just one marker; don't need to do anything
        warning("Just one marker.")
        res <- vector("list", nind(cross))
        names(res) <- id
        for(i in seq(along=res)) res[[i]] <- numeric(0)
        return(res)
    }

    geno <- cross$geno[[1]]$data
    geno[is.na(geno)] <- 0
    type <- crosstype(cross)

    if(!(type %in% c("bc", "f2", "riself", "risib", "dh", "haploid", "bcsft")))
        stop("locateXO only working for backcross, intercross, RI strains, or BCsFt.")

    map <- cross$geno[[1]]$map
    if(is.matrix(map)) map <- map[1,]
    #  map <- map - map[1] # shift first marker to 0

    # bc or intercross?  thetype==0 for BC and ==1 for intercross
    if(type=="f2" || type=="bcsft") {
        if(chrtype(geno) == "X") thetype <- 0
        else thetype <- 1
    }
    else thetype <- 0

    n.ind <- nrow(geno)
    n.mar <- ncol(geno)

    z <- .C("R_locate_xo",
            as.integer(n.ind),
            as.integer(n.mar),
            as.integer(thetype),
            as.integer(geno),
            as.double(map),
            location=as.double(rep(0,n.ind*2*(n.mar-1))),
            nseen=as.integer(rep(0,n.ind)),
            ileft=as.integer(rep(0,n.ind*2*(n.mar-1))),
            iright=as.integer(rep(0,n.ind*2*(n.mar-1))),
            left=as.double(rep(0,n.ind*2*(n.mar-1))),
            right=as.double(rep(0,n.ind*2*(n.mar-1))),
            gleft=as.integer(rep(0, n.ind*2*(n.mar-1))),
            gright=as.integer(rep(0, n.ind*2*(n.mar-1))),
            ntype=as.integer(rep(0,n.ind*2*(n.mar-1))),
            as.integer(full.info),
            PACKAGE="qtl")
    location <- t(matrix(z$location, nrow=n.ind))
    nseen <- z$nseen
    if(full.info) {
        ileft <- t(matrix(z$ileft, nrow=n.ind))
        iright <- t(matrix(z$iright, nrow=n.ind))
        left <- t(matrix(z$left, nrow=n.ind))
        right <- t(matrix(z$right, nrow=n.ind))
        gleft <- t(matrix(z$gleft, nrow=n.ind))
        gright <- t(matrix(z$gright, nrow=n.ind))
        ntype <- t(matrix(z$ntype, nrow=n.ind))
    }

    if(!full.info)
        res <- lapply(as.data.frame(rbind(nseen, location), stringsAsFactors=TRUE),
                      function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })
    else {
        location <- lapply(as.data.frame(rbind(nseen, location), stringsAsFactors=TRUE),
                           function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })

        ileft <- lapply(as.data.frame(rbind(nseen, ileft), stringsAsFactors=TRUE),
                        function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })

        iright <- lapply(as.data.frame(rbind(nseen, iright), stringsAsFactors=TRUE),
                         function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })

        left <- lapply(as.data.frame(rbind(nseen, left), stringsAsFactors=TRUE),
                       function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })

        right <- lapply(as.data.frame(rbind(nseen, right), stringsAsFactors=TRUE),
                        function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })

        gleft <- lapply(as.data.frame(rbind(nseen, gleft), stringsAsFactors=TRUE),
                        function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })

        gright <- lapply(as.data.frame(rbind(nseen, gright), stringsAsFactors=TRUE),
                         function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })

        ntype <- lapply(as.data.frame(rbind(nseen, ntype), stringsAsFactors=TRUE),
                        function(a) { if(a[1]==0) return(numeric(0)); a[(1:a[1])+1] })

        res <- location
        for(i in seq(along=res)) {
            if(length(res[[i]])>0) {
                ntype[[i]][length(ntype[[i]])] <- NA
                res[[i]] <- cbind(location=location[[i]],
                                  left=left[[i]],
                                  right=right[[i]],
                                  ileft=ileft[[i]],
                                  iright=iright[[i]],
                                  gleft=gleft[[i]],
                                  gright=gright[[i]],
                                  nTypedBetween=ntype[[i]])
            }
        }
    }

    names(res) <- id
    res
}

# jittermap: make sure no two markers are at precisely the same position
jittermap <-
    function(object, amount=1e-6) # x is either a cross object or a map
{
    if(inherits(object, "cross")) {
        themap <- pull.map(object)
        return.cross <- TRUE
    }
    else {
        if(!inherits(object, "map") && !is.list(object))
            stop("Input must be a cross or a map")
        return.cross <- FALSE
        themap <- object
    }

    for(i in 1:length(themap)) {
        if(is.matrix(themap[[i]])) { # sex-specific maps
            n <- ncol(themap[[i]])
            if(n > 1) {
                for(j in 1:2)
                    themap[[i]][j,] <- themap[[i]][j,] + c(0,cumsum(rep(amount, n-1)))
            }
        }
        else {
            n <- length(themap[[i]])
            if(n > 1)
                themap[[i]] <- themap[[i]] + c(0,cumsum(rep(amount, n-1)))
        }
    }

    if(return.cross)
        return(clean(replace.map(object, themap)))

    themap
}


print.map <-
    function(x, ...)
{
    for(i in seq(along=x))
        attr(x[[i]], "loglik") <- NULL

    if(length(x) == 1)
        print(unclass(x[[1]]))
    else
        print(unclass(lapply(x, unclass)))
}


# map function for Stahl model
mf.stahl <-
    function(d, m=0, p=0)
{
    if(any(d < 0))
        stop("d must be >= 0\n")
    if(m < 0)
        stop("Must have m >= 0\n")
    if(p < 0 || p > 1)
        stop("Must have 0 <= p <= 1\n")

    # handle missing values
    if(any(!is.finite(d))) {
        which.finite <- is.finite(d)
        dsub <- d[which.finite]
        dropped.some <- TRUE
    } else {
        dsub <- d
        dropped.some <- FALSE
    }

    result <- .C("R_mf_stahl",
                 as.integer(length(dsub)),
                 as.double(dsub/100), # convert to Morgans
                 as.integer(m),
                 as.double(p),
                 result=as.double(rep(0,length(dsub))),
                 PACKAGE="qtl")$result

    if(dropped.some) {
        d[!which.finite] <- NA
        d[which.finite] <- result
        result <- d
    }
    result
}

# inverse map function for Stahl model
imf.stahl <-
    function(r, m=0, p=0, tol=1e-12, maxit=1000)
{
    if(any(r < 0 | r > 0.5))
        stop("r must be >= 0 or <= 0.5\n")
    if(m < 0)
        stop("Must have m >= 0\n")
    if(p < 0 || p > 1)
        stop("Must have 0 <= p <= 1\n")

    # handle missing values
    if(any(!is.finite(r))) {
        which.finite <- is.finite(r)
        rsub <- r[which.finite]
        dropped.some <- TRUE
    } else {
        rsub <- r
        dropped.some <- FALSE
    }

    result <- .C("R_imf_stahl",
                 as.integer(length(rsub)),
                 as.double(rsub),
                 as.integer(m),
                 as.double(p),
                 result=as.double(rep(0,length(rsub))),
                 as.double(tol),
                 as.integer(maxit),
                 PACKAGE="qtl")$result*100 # convert to cM

    if(dropped.some) {
        r[!which.finite] <- NA
        r[which.finite] <- result
        result <- r
    }
    result
}

######################################################################
# getid: internal function to pull out the "ID" column from the
#        phenotype data, if there is one
######################################################################
getid <-
    function(cross)
{
    phe <- cross$pheno
    nam <- names(phe)
    if("id" %in% nam) {
        id <- phe$id
        phenam <- "id"
    }
    else if("ID" %in% nam) {
        id <- phe$ID
        phenam <- "ID"
    }
    else if("Id" %in% nam) {
        id <- phe$Id
        phenam <- "Id"
    }
    else if("iD" %in% nam) {
        id <- phe$iD
        phenam <- "iD"
    }
    else {
        id <- NULL
        phenam <- NULL
    }

    if(is.factor(id))
        id <- as.character(id)

    attr(id, "phenam") <- phenam

    id
}

######################################################################
# find the chromosome and position of a vector of markers
######################################################################
find.markerpos <-
    function(cross, marker)
{
    if(length(marker) != length(unique(marker))) {
        warning("Dropping duplicate marker names.")
        marker <- unique(marker)
    }

    output <- data.frame(chr=rep("", length(marker)),
                         pos=rep(NA, length(marker)), stringsAsFactors=TRUE)
    output$chr <- as.character(output$chr)
    rownames(output) <- marker

    map <- pull.map(cross)
    n.mar <- nmar(cross)
    chr <- rep(names(map), n.mar)

    if(!is.matrix(map[[1]])) {
        pos <- unlist(map)
        onemap <- TRUE
    }
    else {
        pos <- unlist(lapply(map, function(a) a[1,]))
        pos2 <- unlist(lapply(map, function(a) a[2,]))
        onemap <- FALSE
        output <- cbind(output, pos2=rep(NA, length(marker)))
        colnames(output)[2:3] <- c("pos.female","pos.male")
    }

    mnam <- markernames(cross)

    for(i in seq(along=marker)) {

        wh <- match(marker[i], mnam)
        if(is.na(wh)) {
            output[i,1] <- NA
            output[i,2] <- NA
        }
        else {
            if(length(wh) > 1) {
                warning("Marker ", marker[i], " found multiple times.")
                wh <- sample(wh, 1)
            }
            output[i,1] <- chr[wh]
            output[i,2] <- pos[wh]
            if(!onemap) output[i,3] <- pos2[wh]
        }
    }

    output
}


######################################################################
# find the chromosome and position of a vector of pseudomarkers
######################################################################
find.pseudomarkerpos <-
    function(cross, marker, where=c("draws","prob"))
{
    if(length(marker) != length(unique(marker))) {
        warning("Dropping duplicate pseudomarker names.")
        marker <- unique(marker)
    }

    output <- data.frame(chr=rep("", length(marker)),
                         pos=rep(NA, length(marker)), stringsAsFactors=TRUE)
    output$chr <- as.character(output$chr)
    rownames(output) <- marker

    where <- match.arg(where)
    if(where=="draws" && !("draws" %in% names(cross$geno[[1]])))
        stop("You'll need to first run sim.geno")
    if(where=="prob" && !("prob" %in% names(cross$geno[[1]])))
        stop("You'll need to first run calc.genoprob")

    themap <- vector("list", nchr(cross))
    names(themap) <- names(cross$geno)
    for(i in 1:nchr(cross)) {
        if(where=="draws")
            themap[[i]] <- attr(cross$geno[[i]]$draws, "map")
        else
            themap[[i]] <- attr(cross$geno[[i]]$prob, "map")
    }

    if(!is.matrix(themap[[1]])) {
        pmar <- unlist(lapply(themap, names))
        pos <- unlist(themap)
        onemap <- TRUE
        chr <- rep(names(themap), sapply(themap, length))
    }
    else {
        pos <- unlist(lapply(themap, function(a) a[1,]))
        pos2 <- unlist(lapply(themap, function(a) a[2,]))
        onemap <- FALSE
        pmar <- unlist(lapply(themap, colnames))
        output <- cbind(output, pos2=rep(NA, length(marker)))
        colnames(output)[2:3] <- c("pos.female","pos.male")
        chr <- rep(names(themap), sapply(themap, ncol))
    }
    whnotmarker <- grep("^loc-*[0-9]*", pmar)
    pmar[whnotmarker] <- paste("c", chr[whnotmarker], ".", pmar[whnotmarker], sep="")

    for(i in seq(along=marker)) {
        wh <- match(marker[i], pmar)
        if(is.na(wh)) {
            output[i,1] <- NA
            output[i,2] <- NA
        }
        else {
            if(length(wh) > 1) {
                warning("Pseudomarker ", marker[i], " found multiple times.")
                wh <- sample(wh, 1)
            }
            output[i,1] <- chr[wh]
            output[i,2] <- pos[wh]
            if(!onemap) output[i,3] <- pos2[wh]
        }
    }

    output
}


######################################################################
# utility function for determining whether pheno.col (as argument
# to scanone etc) can be interpreted as a vector of phenotypes,
# versus a vector of phenotype columns
######################################################################
LikePheVector <-
    function(pheno, n.ind, n.phe)
{
    if(is.numeric(pheno) && length(pheno)==n.ind &&
       any(pheno < 1 | pheno > n.phe | pheno!=round(pheno)))
        return(TRUE)
    FALSE
}

######################################################################
# for matching chromosome names
######################################################################
matchchr <-
    function(selection, thechr)
{
    if(is.factor(thechr)) thechr <- as.character(thechr)
    if(length(thechr) > length(unique(thechr)))
        stop("Duplicate chromosome names.")

    if(is.logical(selection)) {
        if(length(selection) != length(thechr))
            stop("Logical vector to select chromosomes is the wrong length")
        return(thechr[selection])
    }

    if(is.numeric(selection)) selection <- as.character(selection)

    if(length(selection) > length(unique(selection))) {
        warning("Dropping duplicate chromosomes")
        selection <- unique(selection)
    }

    g <- grep("^-", selection)
    if(length(g) > 0 && length(g) < length(selection))
        stop("In selecting chromosomes, all must start with '-' or none should.")
    if(length(g) > 0) {
        selectomit <- TRUE
        selection <- substr(selection, 2, nchar(selection))
    }
    else selectomit <- FALSE

    wh <- match(selection, thechr)
    if(any(is.na(wh))) {
        warning("Chr ", paste(selection[is.na(wh)], collapse=", "), " not found")
        wh <- wh[!is.na(wh)]
        if(length(wh) == 0) return(thechr)
    }

    if(selectomit) return(thechr[-wh])
    thechr[sort(wh)]
}

######################################################################
# check that chromosomes match appropriately
# TRUE = chr okay
# FALSE = problem
######################################################################
testchr <-
    function(selection, thechr)
{
    if(is.factor(thechr)) thechr <- as.character(thechr)
    if(length(thechr) > length(unique(thechr))) {
        #    warning("Duplicate chromosome names.")
        return(FALSE)
    }

    if(is.logical(selection)) {
        if(length(selection) != length(thechr)) {
            #      warning("Logical vector to select chromosomes is the wrong length")
            return(FALSE)
        }
        return(TRUE)
    }

    if(is.numeric(selection)) selection <- as.character(selection)

    if(length(selection) > length(unique(selection))) {
        #    warning("Dropping duplicate chromosomes")
        selection <- unique(selection)
    }

    g <- grep("^-", selection)
    if(length(g) > 0 && length(g) < length(selection)) {
        #    stop("In selecting chromosomes, all must start with '-' or none should.")
        return(FALSE)
    }
    if(length(g) > 0) {
        selectomit <- TRUE
        selection <- substr(selection, 2, nchar(selection))
    }
    else selectomit <- FALSE

    wh <- match(selection, thechr)
    if(any(is.na(wh))) return(FALSE)

    TRUE
}

######################################################################
# convert2sa
#
# convert a sex-specific maps to a sex-averaged one.
# We pull out just the female map, and give a warning if the male and
# female maps are too different
######################################################################
convert2sa <-
    function(map, tol=1e-4)
{
    if(!inherits(map, "map"))
        stop("Input should have class \"map\".")

    if(!is.matrix(map[[1]]))
        stop("Input map doesn't seem to be a sex-specific map.")

    theclass <- sapply(map, chrtype)

    fem <- lapply(map, function(a) a[1,])

    dif <- sapply(map, function(a) { if(ncol(a)==1) return(diff(a))
                                     a <- apply(a, 1, diff);
                                     if(is.matrix(a)) return(max(abs(apply(a, 1, diff))))
                                     abs(diff(a)) })

    if(max(dif) > tol)
        warning("Female and male inter-marker distances differ by as much as ", max(dif), ".")

    for(i in seq(along=theclass)) class(fem[[i]]) <- theclass[i]

    class(fem) <- "map"
    fem
}

# round as character string, ensuring ending 0's are kept.
charround <-
    function(x, digits=1)
{
    if(digits < 1)
        stop("This is intended for the case digits >= 1.")

    y <- as.character(round(x, digits))

    z <- strsplit(y, "\\.")
    sapply(z, function(a, digits)
       {
           if(length(a) == 1)
               b <- paste(a[1], ".", paste(rep("0", digits),collapse=""), sep="")
           else {
               if(nchar(a[2]) == digits)
                   b <- paste(a, collapse=".")
               else
                   b <- paste(a[1], ".", a[2],
                              paste(rep("0", digits - nchar(a[2])), collapse=""),
                              sep="")
           }
       }, digits)
}

######################################################################
# scantwoperm2scanoneperm
#
# pull out the scanone permutations from scantwo permutation results,
# so that one may use the scantwo perms in calls to summary.scanone
######################################################################
scantwoperm2scanoneperm <-
    function(scantwoperms)
{
    if(!inherits(scantwoperms, "scantwoperm"))
        stop("Input must have class \"scantwoperm\".")
    if(!("one" %in% names(scantwoperms)))
        stop("Input doesn't contain scanone permutation results.")
    scanoneperms <- scantwoperms$one
    class(scanoneperms) <- "scanoneperm"
    scanoneperms
}


######################################################################
# subset.map
######################################################################
subset.map <-
    function(x, ...)
{
    cl <- class(x)
    x <- x[...]
    class(x) <- cl
    x
}

`[.map` <-
    function(x, ...)
{
    x <- unclass(x)[...]
    class(x) <- "map"
    x
}


######################################################################
# subset.cross with [ ]
######################################################################
`[.cross` <-
    function(x, chr, ind)
    subset(x, chr, ind)


######################################################################
# findDupMarkers
#
# find markers whose genotype data is identical to another marker
# (which thus might be dropped, as extraneous)
#
# chr          (Optional) set of chromosomes to consider
#
# exact.only   If TRUE, look for markers with the same genotypes
#                       and the same pattern of missing data
#              If FALSE, also identify markers whose observed
#                        genotypes match another marker, with no
#                        observed genotypes for which the other is
#                        missing
#
# adjacent.only   If TRUE, only consider adjacent markers
######################################################################

findDupMarkers <-
    function(cross, chr, exact.only=TRUE, adjacent.only=FALSE)
{
    if(!missing(chr)) cross <- subset(cross, chr=chr)

    g <- pull.geno(cross)
    markers <- colnames(g)
    markerloc <- lapply(nmar(cross), function(a) 1:a)
    if(length(markerloc) > 1) {
        for(j in 2:length(markerloc))
            markerloc[[j]] <- markerloc[[j]] + max(markerloc[[j-1]]) + 10
    }
    markerloc <- unlist(markerloc)

    if(exact.only) {
        g[is.na(g)] <- 0

        # genotype data patterns
        pat <- apply(g, 2, paste, collapse="")

        # table of unique values
        tab <- table(pat)

        # no duplicates; return
        if(all(tab == 1)) return(NULL)

        wh <- which(tab > 1)
        theloc <- themar <- vector("list", length(wh))
        for(i in seq(along=wh)) {
            themar[[i]] <- names(pat)[pat==names(tab)[wh[i]]]
            theloc[[i]] <- markerloc[pat==names(tab)[wh[i]]]
        }

        if(adjacent.only) {
            extraloc <- list()
            extramar <- list()
            for(i in seq(along=theloc)) {
                d <- diff(theloc[[i]]) # look for adjacency
                if(any(d>1)) { # split into adjacent groups
                    temp <- which(d>1)
                    first <- c(1, temp+1)
                    last <- c(temp, length(theloc[[i]]))
                    for(j in 2:length(first)) {
                        extraloc[[length(extraloc)+1]] <- theloc[[i]][first[j]:last[j]]
                        extramar[[length(extramar)+1]] <- themar[[i]][first[j]:last[j]]
                    }
                    themar[[i]] <- themar[[i]][first[1]:last[1]]
                    theloc[[i]] <- theloc[[i]][first[1]:last[1]]
                }
            }
            themar <- c(themar, extramar)
            theloc <- c(theloc, extraloc)

            nm <- sapply(themar, length)
            if(all(nm==1)) return(NULL) # nothing left
            themar <- themar[nm>1]
            theloc <- theloc[nm>1]
        }

        # order by first locus
        o <- order(sapply(theloc, min))
        themar <- themar[o]

        randompics <- sapply(themar, function(a) sample(length(a), 1))
        for(i in seq(along=themar)) {
            names(themar)[i] <- themar[[i]][randompics[i]]
            themar[[i]] <- themar[[i]][-randompics[i]]
        }

    }
    else {
        themar <- NULL

        ntyp <- ntyped(cross, "mar")
        o <- order(ntyp, decreasing=TRUE)

        g[is.na(g)] <- 0
        z <- .C("R_findDupMarkers_notexact",
                as.integer(nrow(g)),
                as.integer(ncol(g)),
                as.integer(g),
                as.integer(o),
                as.integer(markerloc),
                as.integer(adjacent.only),
                result=as.integer(rep(0,length(o))),
                PACKAGE="qtl")
        if(all(z$result==0)) return(NULL)
        u <- unique(z$result[z$result != 0])
        themar <- vector("list", length(u))
        names(themar) <- colnames(g)[u]
        for(i in seq(along=themar))
            themar[[i]] <- colnames(g)[z$result==u[i]]
    }

    themar
}

######################################################################
# convert2riself
######################################################################
convert2riself <-
    function(cross)
{
    if(!inherits(cross, "cross"))
        stop("input must be a cross object.")
    curtype <- crosstype(cross)
    chr_type <- sapply(cross$geno, chrtype)
    whX <- NULL
    if(any(chr_type != "A")) { # there's an X chromosome
        whX <- names(cross$geno)[chr_type != "A"]
        if(length(whX) > 1)
            warning("Converting chromosomes ", paste(whX, collapse=" "), " to autosomal.")
        else
            warning("Converting chromosome ", whX, " to autosomal.")
        for(i in whX) class(cross$geno[[i]]) <- "A"
    }

    gtab <- table(pull.geno(cross))
    usethree <- FALSE
    if(!is.na(gtab["3"])) {
        if(!is.na(gtab["2"]) && gtab["3"] < gtab["2"])
            usethree <- FALSE
        else usethree <- TRUE
    }

    g2omit <- g3omit <- g4omit <- 0
    for(i in 1:nchr(cross)) {
        dat <- cross$geno[[i]]$data
        g1 <- sum(!is.na(dat) & dat==1)
        g2 <- sum(!is.na(dat) & dat==2)
        g3 <- sum(!is.na(dat) & dat==3)
        g4 <- sum(!is.na(dat) & dat>3)
        if(usethree && chr_type[i] == "A") {
            dat[!is.na(dat) & dat!=1 & dat!=3] <- NA
            dat[!is.na(dat) & dat==3] <- 2
            g2omit <- g2omit + g2
            g4omit <- g4omit + g4
        }
        else {
            dat[!is.na(dat) & dat!=1 & dat!=2] <- NA
            g3omit <- g3omit + g3
            g4omit <- g4omit + g4
        }
        cross$geno[[i]]$data <- dat
    }
    if(g2omit > 0)
        warning("Omitting ", g2omit, " genotypes with code==2.")
    if(g3omit > 0)
        warning("Omitting ", g3omit, " genotypes with code==3.")
    if(g4omit > 0)
        warning("Omitting ", g4omit, " genotypes with code>3.")

    class(cross) <- c("riself", "cross")

    cross
}


######################################################################
# convert2risib
######################################################################
convert2risib <-
    function(cross)
{
    if(!inherits(cross, "cross"))
        stop("input must be a cross object.")
    curtype <- crosstype(cross)
    chr_type <- sapply(cross$geno, chrtype)

    gtab <- table(pull.geno(cross))
    usethree <- FALSE
    if(!is.na(gtab["3"])) {
        if(!is.na(gtab["2"]) && gtab["3"] < gtab["2"])
            usethree <- FALSE
        else usethree <- TRUE
    }

    g2omit <- g3omit <- g4omit <- 0
    for(i in 1:nchr(cross)) {
        dat <- cross$geno[[i]]$data
        g1 <- sum(!is.na(dat) & dat==1)
        g2 <- sum(!is.na(dat) & dat==2)
        g3 <- sum(!is.na(dat) & dat==3)
        g4 <- sum(!is.na(dat) & dat>3)
        if(usethree) {
            if(chr_type[i] == "A") {
                dat[!is.na(dat) & dat!=1 & dat!=3] <- NA
                dat[!is.na(dat) & dat==3] <- 2
                g2omit <- g2omit + g2
                g4omit <- g4omit + g4
            }
            else { # X chromosome
                if(g2 >= g3) {
                    dat[!is.na(dat) & dat!=1 & dat!=2] <- NA
                    g3omit <- g3omit + g3
                    g4omit <- g4omit + g4
                }
                else {
                    dat[!is.na(dat) & dat!=1 & dat!=3] <- NA
                    dat[!is.na(dat) & dat==3] <- 2
                    g2omit <- g2omit + g2
                    g4omit <- g4omit + g4
                }
            }
        }
        else {
            dat[!is.na(dat) & dat!=1 & dat!=2] <- NA
            g3omit <- g3omit + g3
            g4omit <- g4omit + g4
        }
        cross$geno[[i]]$data <- dat
    }
    if(g2omit > 0)
        warning("Omitting ", g2omit, " genotypes with code==2.")
    if(g3omit > 0)
        warning("Omitting ", g3omit, " genotypes with code==3.")
    if(g4omit > 0)
        warning("Omitting ", g4omit, " genotypes with code>3.")

    class(cross) <- c("risib", "cross")

    cross
}


rescalemap <-
    function(object, scale=1e-6)
{
    if(inherits(object, "cross")) {
        for(i in 1:nchr(object)) {
            object$geno[[i]]$map <-
                object$geno[[i]]$map * scale
        }
        if(abs(scale - 1) > 1e-6)
            object <- clean(object) # strip off intermediate calculations
    } else if(inherits(object, "map") || is.list(object)) {
        for(i in seq(along=object)) {
            object[[i]] <- object[[i]] * scale
        }
    } else {
        stop("rescalemap works only for objects of class \"cross\" or \"map\".")
    }

    object
}


shiftmap <-
    function(object, offset=0)
{
    if(inherits(object, "cross")) {
        if(length(offset) == 1) offset <- rep(offset, nchr(object))
        else if(length(offset) != nchr(object))
            stop("offset must have length 1 or n.chr (", nchr(object), ")")

        for(i in 1:nchr(object)) {
            if(is.matrix(object$geno[[i]]$map)) {
                for(j in 1:2)
                    object$geno[[i]]$map[j,] <- object$geno[[i]]$map[j,] - object$geno[[i]]$map[j,1] + offset[i]
            } else {
                object$geno[[i]]$map <- object$geno[[i]]$map - object$geno[[i]]$map[1] + offset[i]
            }
        }
    } else if(inherits(object, "map") || is.list(object)) {
        if(length(offset) == 1) offset <- rep(offset, length(object))
        else if(length(offset) != length(object))
            stop("offset must have length 1 or n.chr (", length(object), ")")
        for(i in seq(along=object)) {
            if(is.matrix(object[[i]])) {
                for(j in 1:2)
                    object[[i]][j,] <- object[[i]][j,] - object[[i]][j,1] + offset[i]
            } else {
                object[[i]] <- object[[i]] - object[[i]][1] + offset[i]
            }
        }
    } else
        stop("shiftmap works only for objects of class \"cross\" or \"map\".")

    object
}

######################################################################
# switch alleles in a cross
######################################################################
switchAlleles <-
    function(cross, markers, switch=c("AB","CD","ABCD", "parents"))
{
    type <- crosstype(cross)
    switch <- match.arg(switch)

    if(type %in% c("bc", "risib", "riself", "dh", "haploid")) {
        if(switch != "AB")
            warning("Using switch = \"AB\".")

        found <- rep(FALSE, length(markers))
        for(i in 1:nchr(cross)) {
            cn <- colnames(cross$geno[[i]]$data)
            m <- match(markers, cn)
            if(any(!is.na(m))) {
                found[!is.na(m)] <- TRUE
                for(j in m[!is.na(m)]) {
                    g <- cross$geno[[i]]$data[,j]
                    cross$geno[[i]]$data[!is.na(g) & g==1,j] <- 2
                    cross$geno[[i]]$data[!is.na(g) & g==2,j] <- 1
                }
            }
        }

    }
    else if(type=="f2" || type=="bcsft") {
        if(switch != "AB")
            warning("Using switch = \"AB\".")

        found <- rep(FALSE, length(markers))
        for(i in 1:nchr(cross)) {
            cn <- colnames(cross$geno[[i]]$data)
            m <- match(markers, cn)
            if(any(!is.na(m))) {
                found[!is.na(m)] <- TRUE
                for(j in m[!is.na(m)]) {
                    g <- cross$geno[[i]]$data[,j]
                    cross$geno[[i]]$data[!is.na(g) & g==1,j] <- 3
                    cross$geno[[i]]$data[!is.na(g) & g==3,j] <- 1
                    cross$geno[[i]]$data[!is.na(g) & g==4,j] <- 5
                    cross$geno[[i]]$data[!is.na(g) & g==5,j] <- 4
                }
            }
        }
    }
    else if(type=="4way") {

        if(switch=="AB")
            newg <- c(2,1,4,3,6,5,7,8,10,9,12,11,14,13)
        else if(switch=="CD")
            newg <- c(3,4,1,2,5,6,8,7,10,9,13,14,11,12)
        else if(switch=="ABCD")
            newg <- c(4,3,2,1,6,5,8,7,9,10,14,13,12,11)
        else # switch parents
            newg <- c(1,3,2,4,7,8,5,6,9,10,11,13,12,14)

        found <- rep(FALSE, length(markers))
        for(i in 1:nchr(cross)) {
            cn <- colnames(cross$geno[[i]]$data)
            m <- match(markers, cn)
            if(any(!is.na(m))) {
                found[!is.na(m)] <- TRUE
                for(j in m[!is.na(m)]) {
                    g <- cross$geno[[i]]$data[,j]
                    for(k in 1:14)
                        cross$geno[[i]]$data[!is.na(g) & g==k,j] <- newg[k]
                }
            }
        }

    }
    else {
        stop("Cross type ", type, " not supported by switchAlleles()")
    }

    if(any(!found))
        warning("Some markers not found: ", paste(markers[!found], collapse=" "))

    clean(cross)
}

######################################################################
#
# nqrank: Convert a set of quantitative values to the corresponding
#         normal quantiles (preserving mean and SD)
#
######################################################################

nqrank <-
    function(x, jitter=FALSE)
{
    y <- x[!is.na(x)]
    themean <- mean(y, na.rm=TRUE)
    thesd <- sd(y, na.rm=TRUE)

    y[y == Inf] <- max(y[y<Inf])+10
    y[y == -Inf] <- min(y[y > -Inf]) - 10
    if(jitter)
        y <- rank(y+runif(length(y))/(sd(y)*10^8))
    else y <- rank(y)

    x[!is.na(x)] <- qnorm((y-0.5)/length(y))

    x*thesd/sd(x, na.rm=TRUE)-mean(x,na.rm=TRUE)+themean
}

######################################################################
#
# cleanGeno: omit genotypes that are possibly in error, as indicated
#            by apparent double-crossovers separated by a distance of
#            no more than maxdist and having no more than maxmark
#            interior typed markers
#
######################################################################

cleanGeno <-
    function(cross, chr, maxdist=2.5, maxmark=2, verbose=TRUE)
{
    if(!(crosstype(cross) %in% c("bc", "riself", "risib", "dh", "haploid")))
        stop("This function currently only works for crosses with two genotypes")

    if(!missing(chr)) cleaned <- subset(cross, chr=chr)
    else cleaned <- cross

    thechr <- names(cleaned$geno)
    totdrop <- 0
    maxmaxdist <- max(maxdist)
    for(i in thechr) {
        xoloc <- locateXO(cleaned, chr=i, full.info=TRUE)
        nxo <- sapply(xoloc, function(a) if(is.matrix(a)) return(nrow(a)) else return(0))
        g <- pull.geno(cleaned, chr=i)

        ndrop <- 0
        for(j in which(nxo > 1)) {
            maxd <- xoloc[[j]][-1,"right"] - xoloc[[j]][-nrow(xoloc[[j]]),"left"]
            wh <- maxd <= maxmaxdist
            if(any(wh)) {
                for(k in which(wh)) {
                    nt <- sum(!is.na(g[j,(xoloc[[j]][k,"ileft"]+1):(xoloc[[j]][k+1,"iright"]-1)]))
                    if(nt > 0 && any(nt <= maxmark & maxd[k] < maxdist)) {
                        cleaned$geno[[i]]$data[j,(xoloc[[j]][k,"ileft"]+1):(xoloc[[j]][k+1,"iright"]-1)] <- NA
                        ndrop <- ndrop + nt
                        totdrop <- totdrop + nt
                    }
                }
            }
        }
        if(verbose && ndrop > 0) {
            totgen <- sum(ntyped(subset(cross, chr=i)))
            cat(" ---Dropping ", ndrop, " genotypes (out of ", totgen, ") on chr ", i, "\n", sep="")
        }
    }

    if(verbose && nchr(cleaned)>1 && totdrop > 0) {
        totgen <- sum(ntyped(subset(cross, chr=thechr)))
        cat(" ---Dropped ", totdrop, " genotypes (out of ", totgen, ") in total\n", sep="")
    }

    for(i in names(cleaned$geno))
        cross$geno[[i]] <- cleaned$geno[[i]]

    cross
}

######################################################################
# typingGap: calculate gaps between typed markers
######################################################################

typingGap <-
    function(cross, chr, terminal=FALSE)
{
    if(!missing(chr))
        cross <- subset(cross, chr)

    n.ind <- nind(cross)
    n.chr <- nchr(cross)

    gaps <- matrix(nrow=n.ind, ncol=n.chr)
    colnames(gaps) <- names(cross$geno)

    for(i in 1:n.chr) {
        map <- cross$geno[[i]]$map
        map <- c(map[1], map, map[length(map)])
        if(is.matrix(map)) stop("This function can't currently handle sex-specific maps.")

        if(terminal) # just look at terminal gaps
            gaps[,i] <- apply(cbind(1,cross$geno[[i]]$data,1), 1,
                              function(a,b) {d <- diff(b[!is.na(a)]); max(d[c(1,length(d))]) }, map)
        else
            gaps[,i] <- apply(cbind(1,cross$geno[[i]]$data,1), 1,
                              function(a,b) max(diff(b[!is.na(a)])), map)
    }
    if(n.chr==1) gaps <- as.numeric(gaps)
    gaps
}

######################################################################
# calcPermPval
#
# calculate permutation pvalues for summary.scanone()
######################################################################
calcPermPval <-
    function(peaks, perms)
{
    if(!is.matrix(peaks))
        peaks <- as.matrix(peaks)
    if(!is.matrix(perms))
        perms <- as.matrix(perms)

    ncol.peaks <- ncol(peaks)
    nrow.peaks <- nrow(peaks)
    n.perms <- nrow(perms)

    if(ncol.peaks != ncol(perms))
        stop("ncol(peaks) != ncol(perms)")

    pval <- .C("R_calcPermPval",
               as.double(peaks),
               as.integer(ncol.peaks),
               as.integer(nrow.peaks),
               as.double(perms),
               as.integer(n.perms),
               pval=as.double(rep(0,ncol.peaks*nrow.peaks)),
               PACKAGE="qtl")$pval

    matrix(pval, ncol=ncol.peaks, nrow=nrow.peaks)
}

######################################################################
# phenames: pull out phenotype names
######################################################################
phenames <-
    function(cross)
    colnames(cross$pheno)


######################################################################
# updateParallelRNG
#
# set RNGkind
# advance RNGstream by no. clusters
######################################################################
updateParallelRNG <-
    function(n.cluster=1)
{
    kind <- RNGkind()[1]
    if(kind != "L'Ecuyer-CMRG") RNGkind("L'Ecuyer-CMRG")

    s <- .Random.seed
    if(n.cluster < 1) n.cluster <- 1
    for(i in 1:n.cluster) s <- nextRNGStream(s)
    .GlobalEnv$.Random.seed <- s ## global assign new .Random.seed
}

######################################################################
# formMarkerCovar
#
# cross: cross object
#
# markers: marker names or pseudomarker names (like "c5loc25.1" or "5@25.1")
#
# method: use genotype probabilities or imputated genotypes (imputed with imp or argmax)
#
# ...: passed to fill.geno, if necessary
#
######################################################################
formMarkerCovar <-
    function(cross, markers, method=c("prob", "imp", "argmax"), ...)
{
    method <- match.arg(method)

    if(!is.character(markers)) markers <- as.character(markers)

    # check if the marker names are all like "5@25.1"
    grepresult <- grepl("@\\d", markers)
    if(all(grepresult)) {
        spl <- strsplit(markers, "@")
        chr <- sapply(spl, "[", 1)
        pos <- as.numeric(sapply(spl, "[", 2))
        m <- match(chr, chrnames(cross))
        if(any(is.na(m)))
            stop("Some chr not found: ", paste(unique(chr[m]), collapse=" "))

        if(method=="prob")
            markers <- find.pseudomarker(cross, chr, pos, where="prob")
        else
            markers <- find.marker(cross, chr, pos)

        chr <- unique(chr)
    }
    else {
        if(method=="prob") z <- find.pseudomarkerpos(cross, markers, where="prob")
        else z <- find.markerpos(cross, markers)

        if(any(is.na(z[,1])))
            stop("Some markers not found: ", paste(markers[is.na(z[,1])], collapse=", "))

        chr <- unique(z$chr)
    }

    cross <- subset(cross, chr=chr)
    isXchr <- (sapply(cross$geno, chrtype) == "X")
    crosstype <- crosstype(cross)
    sexpgm <- getsex(cross)
    crossattr <- attributes(cross)

    if(method=="prob") {
        if(any(isXchr) && crosstype %in% c("f2", "bc", "bcsft")) {
            for(i in which(isXchr))
                cross$geno[[i]]$prob <- reviseXdata(crosstype, "full", sexpgm=sexpgm, prob=cross$geno[[i]]$prob,
                                                    cross.attr=crossattr)

        }

        if(any(isXchr) && any(!isXchr)) # some X, some not
            prob <- cbind(pull.genoprob(cross[!isXchr,], omit.first.prob=TRUE),
                          pull.genoprob(cross[isXchr,], omit.first.prob=TRUE))
        else # all X or all not X
            prob <- pull.genoprob(cross, omit.first.prob=TRUE)

        markercols <- sapply(strsplit(colnames(prob), ":"), "[", 1)
        m <- match(markers, markercols)
        if(any(is.na(m)))
            warning("Some markers/pseudomarkers not found: ", paste(unique(markers[is.na(m)]), collapse=" "))
        return(prob[,!is.na(match(markercols, markers)), drop=FALSE])
    }
    else {
        cross <- fill.geno(cross, method=method, ...)

        if(any(isXchr) && crosstype %in% c("f2", "bc", "bcsft")) {
            for(i in which(isXchr))
                cross$geno[[i]]$data <- reviseXdata(crosstype, "full", sexpgm=sexpgm, geno=cross$geno[[i]]$data,
                                                    cross.attr=crossattr)
        }

        geno <- pull.geno(cross)
        markercols <- colnames(geno)
        m <- match(markers, markercols)
        if(any(is.na(m)))
            warning("Some markers not found: ", paste(unique(markers[is.na(m)]), collapse=" "))

        geno <- geno[,!is.na(match(markercols, markers)), drop=FALSE]

        # expand each column
        nalle <- apply(geno, 2, function(a) length(unique(a)))
        g <- matrix(ncol=sum(nalle-1), nrow=nrow(geno))
        colnames(g) <- as.character(1:ncol(g))
        cur <- 0
        for(i in 1:ncol(geno)) {
            if(nalle[i] <= 1) next
            for(j in 2:nalle[i])
                g[,cur+j-1] <- as.numeric(geno[,i] == j)
            colnames(g)[cur - 1 + (2:nalle[i])] <- paste(colnames(geno)[i], 2:nalle[i], sep=".")
            cur <- cur + nalle[i] - 1
        }
        return(g)
    }
}

# omit the X chromosome from a cross
omit_x_chr <-
    function(cross, warn=TRUE)
{
    is_x <- vapply(cross$geno, function(chr) {
        chr_type <- chrtype(chr)
        !is.null(chr_type) && chr_type=="X" }, FALSE)

    if(any(is_x)) {
        if(all(is_x)) stop("Omitting X chromosome, but there are no other chromosomes.")
        if(warn) warning("Omitting X chromosome",
                         ifelse(sum(is_x)>1, "s", ""), " (",
                         paste0(names(cross$geno)[is_x], collapse=", "), ")")
        cross <- cross[!is_x, ]
    }

    cross
}

# determine cross type
crosstype <-
    function(cross)
    {
        type <- class(cross)
        type <- type[type != "cross" & type != "list"]
        if(length(type) > 1) {
            warning("cross has multiple classes")
        }
        type[1]
    }

# determine chromosome type
chrtype <-
    function(object)
    {
        if(inherits(object, "X")) return("X")
        "A"
    }

# end of util.R

context("flip.order")

test_that("flip.order, when applied twice, should get us back to the same thing", {

    data(hyper)

    # reduce size
    set.seed(53307443)
    hyper <- hyper[,sample(nind(hyper), 8)]

    hyper <- calc.genoprob(hyper, step=1)
    hyper <- sim.geno(hyper, step=10, n.draws=2)
    hyper <- argmax.geno(hyper, step=1)
    hyper <- calc.errorlod(hyper)

    hyperfl <- flip.order(hyper, chr=c(1, 4, 6, 15))
    summary(hyperfl)

    hyperfl2 <- flip.order(hyperfl, chr=c(1, 4, 6, 15))
    summary(hyperfl2)

    # having flipped twice, should be back to where we were
    # (except starting locations for each chromosome map)
    expect_null(comparecrosses(shiftmap(hyper), shiftmap(hyperfl2)))

})

test_that("flip.order for 4-way cross", {

    data(fake.4way)

    # reduce size
    set.seed(36461124)
    fake.4way <- fake.4way[,sample(nind(fake.4way), 8)]

    fake.4way <- calc.genoprob(fake.4way, step=1)

    fake.4way.fl <- flip.order(fake.4way, chr=c(1, 4, 6, 15))
    summary(fake.4way.fl)

    fake.4way.fl2 <- flip.order(fake.4way.fl, chr=c(1, 4, 6, 15))
    summary(fake.4way.fl2)

    # having flipped twice, should be back to where we were
    # (except starting locations for each chromosome map)
    expect_null(comparecrosses(shiftmap(fake.4way), shiftmap(fake.4way.fl2)))

})

context("scantwo perms")

test_that("scantwo and scantwopermhk give same results", {

    data(hyper)
    hyper <- calc.genoprob(hyper[c(18,19,"X"),])

    set.seed(92999298)
    out1 <- scantwo(hyper, method="hk", n.perm=3, verbose=FALSE)

    set.seed(92999298)
    out2 <- scantwopermhk(hyper, n.perm=3, verbose=FALSE)

    expect_equivalent(out1, out2)

    # X-chr-specific permutations
    set.seed(92999298)
    out1 <- scantwo(hyper, method="hk", n.perm=3, perm.Xsp=TRUE, verbose=FALSE)

    set.seed(92999298)
    out2 <- scantwopermhk(hyper, n.perm=3, perm.Xsp=TRUE, verbose=FALSE)

    expect_equivalent(out1, out2)

})

test_that("summary.scantwo works with X-chr-specific perms", {

    data(hyper)
    set.seed(23615071)
    hyper <- calc.genoprob(fill.geno(hyper[c(18,19,"X"),])) # selected chr; imputed genotypes
    out2 <- scantwo(hyper, method="hk", verbose=FALSE)

    set.seed(17370120)
    operm1 <- scantwopermhk(hyper, n.perm=100, verbose=FALSE)
    set.seed(17370120)
    operm2 <- scantwopermhk(hyper, n.perm=100, perm.Xsp=TRUE, verbose=FALSE)

    # no significant pairs
    sum1 <- summary(out2, perms=operm1, alpha=0.05)
    sum2 <- summary(out2, perms=operm2, alpha=0.05)
    expect_equal(nrow(sum1), 0)
    expect_equal(nrow(sum2), 0)

    # p-values match expectation; not X-chr-specific
    sum1 <- summary(out2, perms=operm1, pvalues=TRUE)
    lodcol <- grep("^lod", names(sum1))
    expect_equal(lodcol, c(5, 7, 9, 13, 15))
    for(i in 1:5)
        expect_equal(sum1[,lodcol[i]+1], sapply(sum1[,lodcol[i]], function(a) mean(operm1[[i]] >= a)))

    # p-values match expectation; X-chr-specific
    sum2 <- summary(out2, perms=operm2, pvalues=TRUE)
    pairtype <- paste0(ifelse(sum2$chr1=="X", "X", "A"),
                       ifelse(sum2$chr2=="X", "X", "A"))
    pairtype <- match(pairtype, c("AA", "AX", "XX"))
    L <- attr(operm2, "L")
    pow <- sum(L)/L
    lodcol <- grep("^lod", names(sum1))
    expect_equal(lodcol, c(5, 7, 9, 13, 15))
    for(j in 1:nrow(sum2)) {
        for(i in 1:5) {
            lod <- sum2[j,lodcol[i]]
            p <- sum2[j,lodcol[i]+1]
            nominal_p <- mean(operm2[[pairtype[j]]][[i]] >= lod)
            adj_p <- 1 - (1-nominal_p)^pow[pairtype[j]]
            expect_equivalent(p, adj_p)
        }
    }

})

######################################################################
#
# TestIO/input.R
#
# copyright (c) 2002, Karl W Broman
# last modified Feb, 2002
# first written Feb, 2002
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
# 
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
# 
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# This file contains code for testing the cross IO in R/qtl.
#
# Needed input files:
#
#    gen.txt, map.txt, phe.txt    [Karl's format]
#    listeria.raw, listeria.map   [mapmaker format]
#    listeria.raw, listeria2.map  [mapmaker format; no marker pos]
#    listeria.csv                 [csv format] 
#    listeria2.csv                [csv format; no marker pos] 
#
######################################################################

library(qtl)

##############################
# Reading
##############################
# Read CSV format
csv <- read.cross("csv", "", "listeria.csv")
csv2 <- read.cross("csv", "", "listeria2.csv", estimate=FALSE)

# Read mapmaker format
mm <- read.cross("mm", "", "listeria.raw", "listeria.map")
mm2 <- read.cross("mm", "", "listeria.raw", "listeria2.map", estimate=FALSE)

##############################
# Writing
##############################
# Write in CSV format
write.cross(csv, "csv", filestem="junk1")
csv3 <- read.cross("csv", "", "junk1.csv", genotypes=c("AA","AB","BB","not BB","not AA"))
comparecrosses(csv, csv3)

# Write in mapmaker format
write.cross(csv, "mm", filestem="junk2")

# Cleanup
unlink("junk1.csv")
unlink("junk2.raw")
unlink("junk2.prep")

######################################################################
# phyloqtl_scan.R
#
# copyright (c) 2009-2020, Karl W Broman
# last modified Dec, 2020
# first written May, 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Single-QTL scan to map QTL to a phylogenetic tree
#
# Part of the R/qtl package
# Contains: scanPhyloQTL, max.scanPhyloQTL, plot.scanPhyloQTL,
#           inferredpartitions
#
######################################################################

scanPhyloQTL <-
    function(crosses, partitions, chr, pheno.col=1,
             model=c("normal","binary"), method=c("em","imp","hk"),
             addcovar, maxit=4000, tol=0.0001, useAllCrosses=TRUE,
             verbose=FALSE)
{
    if(missing(chr)) chr <- names(crosses[[1]]$geno)
    model <- match.arg(model)
    method <- match.arg(method)

    thecrosses <- names(crosses)
    taxa <- sort(unique(unlist(strsplit(thecrosses, ""))))

    thecrosses <- names(crosses) <- checkPhyloCrosses(thecrosses, taxa)

    if(missing(partitions)) {
        # generate all partitions
        partitions <- genAllPartitions(length(taxa), taxa)
    }
    checkPhyloPartition(partitions, taxa)

    crossmat <- qtlByPartition(thecrosses, partitions)

    dimnames(crossmat) <- list(thecrosses, partitions)

    if(!missing(addcovar)) {
        if(!is.list(addcovar) || length(addcovar) != length(crosses))
            stop("addcovar must be a list with the same length as crosses (", length(crosses), ")")
        n.ind <- sapply(crosses, nind)
        if(is.matrix(addcovar[[1]])) {
            nind.addcovar <- sapply(addcovar, nrow)
            n.addcovar <- sapply(addcovar, ncol)
        }
        else {
            nind.addcovar <- sapply(addcovar, length)
            n.addcovar <- 1
        }
        if(any(nind.addcovar != n.ind)) {
            err <- paste("crosses: ", paste(n.ind, collapse=" "), "\n",
                         "addcovar:", paste(n.addcovar, collapse=" "), "\n")

            stop("Mismatch between no. individuals in addcovar and crosses.\n", err)
        }
        if(length(unique(n.addcovar)) > 1)
            stop("Mismatch in no. add've covariates: ", paste(n.addcovar, collapse=" "))
    }

    # check that the marker maps are all exactly the same
    n.chr <- sapply(crosses, nchr)
    if(!all(n.chr == n.chr[1]))
        stop("Different numbers of chromosomes")
    chrnam1 <- chrnames(crosses[[1]])
    for(j in 2:length(crosses)) {
        chrnam2 <- chrnames(crosses[[j]])
        if(!all(chrnam1 == chrnam2))
            stop("Different chromosome names")
    }
    n.mar1 <- nmar(crosses[[1]])
    for(j in 2:length(crosses)) {
        n.mar2 <- nmar(crosses[[j]])
        if(!all(n.mar1 == n.mar2))
            stop("Different numbers of markers")
    }
    mn1 <- markernames(crosses[[1]])
    for(j in 2:length(crosses)) {
        mn2 <- markernames(crosses[[j]])
        if(!all(mn1 == mn2))
            stop("Different marker names")
    }
    mp1 <- unlist(pull.map(crosses[[1]]))
    for(j in 2:length(crosses)) {
        mp2 <- unlist(pull.map(crosses[[j]]))
        if(!all(mp1 == mp2))
            stop("Different marker positions")
    }

    out <- vector("list", length(partitions))

    for(i in seq(along=partitions)) {
        if(verbose) cat("Partition", i, "of", length(partitions), "\n")
        cm <- crossmat[,i]

        # if all crosses need to be flipped, don't flip any of them
        if(!any(cm > 0)) cm <- -cm

        if(!useAllCrosses) {
            whcm <- which(cm != 0)
            x <- crosses[whcm]
            cm <- cm[whcm]
        }
        else {
            o <- order(abs(cm), decreasing=TRUE)
            cm <- cm[o]
            whcm <- seq(along=cm)
            x <- crosses[o]
            if(!missing(addcovar))
                addcovar <- addcovar[o]
        }

        # flip crosses if necessary
        if(any(cm < 0))
            for(j in which(cm < 0))
                x[[j]] <- flipcross(x[[j]])

        # combine the crosses
        xx <- x[[1]]
        n.phe <- nphe(x[[1]])
        if(length(x) > 1) {
            for(j in 2:length(x)) {
                xx <- c(xx, x[[j]])
                xx$pheno <- xx$pheno[,1:n.phe,drop=FALSE]
            }
        }

        # create cross indicators (as additive covariates)
        if(length(x)==1) alladdcovar<-NULL
        else {
            ni <- sapply(x, nind)
            alladdcovar <- matrix(0, ncol=length(x)-1, nrow=sum(ni))
            thestart <- cumsum(c(1,ni))
            end <- cumsum(ni)
            for(j in 2:length(x))
                alladdcovar[thestart[j]:end[j],j-1] <- 1
        }
        if(!missing(addcovar)) {
            theaddcovar <- as.matrix(addcovar[[whcm[1]]])
            if(length(whcm) > 1)
                for(j in 2:length(whcm))
                    theaddcovar <- rbind(theaddcovar, as.matrix(addcovar[[whcm[j]]]))

            # quick check to be sure that it's not a column with all one value
            if(ncol(theaddcovar)>1 || length(unique(theaddcovar))>1)
                alladdcovar <- cbind(alladdcovar, theaddcovar)
        }

        # ind with no QTL effect
        ind.noqtl <- rep(cm == 0, sapply(x, nind))

        # do the scan
        out[[i]] <- scanone(xx, chr=chr, pheno.col=pheno.col, addcovar=alladdcovar,
                            model=model, method=method, maxit=maxit, tol=tol,
                            ind.noqtl=ind.noqtl)
    }

    # just one partition
    if(length(out) == 1) return(out[[1]])

    # multiple partitions
    result <- out[[1]]
    for(j in 2:length(out))
        result <- c(result, out[[j]])
    colnames(result)[-(1:2)] <- partitions
    class(result) <- c("scanPhyloQTL", "scanone", "data.frame")
    result
}


max.scanPhyloQTL <-
    function(object, chr, format=c("postprob", "lod"), ...)
{
    format <- match.arg(format)

    if(!missing(chr))
        object <- subset.scanone(object, chr=chr)

    mx <- summary.scanPhyloQTL(object, format=format)

    if(nrow(mx) > 1) {
        nc <- ncol(mx)
        mx <- mx[!is.na(mx[,nc]) & mx[,nc]==max(mx[,nc], na.rm=TRUE),,drop=FALSE]
    }

    class(mx) <- c("summary.scanPhyloQTL", "summary.scanone", "data.frame")

    mx
}

summary.scanPhyloQTL <-
    function(object, format=c("postprob", "lod"), threshold, ...)
{
    format <- match.arg(format)

    themax <- apply(object[,-(1:2)], 2, tapply, object[,1], max, na.rm=TRUE)
    if(length(unique(object[,1]))==1) {
        themax <- as.data.frame(matrix(themax, nrow=1), stringsAsFactors=TRUE)
        names(themax) <- colnames(object)[-(1:2)]
    }
    wh <- apply(themax, 1, function(a) { a <- which(a==max(a)); if(length(a) > 1) a <- sample(a, 1); a })
    whpos <- rep(NA, length(wh))
    names(whpos) <- unique(object[,1])
    for(i in seq(along=whpos)) {
        temp <- object[object[,1]==names(whpos)[i],,drop=FALSE]
        temp2 <- which(temp[,wh[i]+2]==max(temp[,wh[i]+2],na.rm=TRUE))
        if(length(temp2) > 1) temp2 <- sample(temp2, 1)
        whpos[i] <- temp[temp2,2]
        names(whpos)[i] <- rownames(temp)[temp2]
    }
    if(format=="lod") {
        out <- data.frame(chr=unique(object[,1]), pos=whpos, themax,
                          loddif=apply(themax, 1, function(a) -diff(sort(a, decreasing=TRUE)[1:2])),
                          inferred=colnames(object)[wh+2],
                          maxlod=apply(themax, 1, max),
                          stringsAsFactors=TRUE)
    }
    else {
        out <- data.frame(chr=unique(object[,1]), pos=whpos, themax,
                          inferred=colnames(object)[wh+2],
                          maxlod=apply(themax, 1, max),
                          stringsAsFactors=TRUE)
        temp <- out[,-c(1:2, ncol(out)-0:1)]
        out[,-c(1:2, ncol(out)-0:1)] <- t(apply(temp, 1, function(a) 10^a/sum(10^a)))
    }
    colnames(out)[(1:ncol(themax))+2] <- colnames(themax)

    rownames(out) <- names(whpos)

    if(!missing(threshold))
        out <- out[out$maxlod >= threshold,,drop=FALSE]

    class(out) <- c("summary.scanPhyloQTL", "summary.scanone", "data.frame")
    out
}


# plot results of scanPhyloQTL
plot.scanPhyloQTL <-
    function(x, chr, incl.markers=TRUE, col, xlim, ylim, lwd=2,
             gap=25, mtick=c("line", "triangle"),
             show.marker.names=FALSE, alternate.chrid=FALSE,
             legend=TRUE, ...)
{
    mtick <- match.arg(mtick)

    if(!missing(chr)) x <- subset(x, chr=chr)

    if(missing(col)) {
        col <- c("black","blue","red","green","orange","brown","gray","cyan","magenta")
        if(ncol(x)-2 > length(col))
            stop("Please give a list of colors")
        col <- col[1:(ncol(x)-2)]
    }

    if(missing(ylim))
        ylim <- c(0, max(apply(x[,-(1:2)], 2, max)))

    dots <- list(...)

    if(missing(xlim)) {
        if("ylab" %in% names(dots))
            plot.scanone(x, incl.markers=incl.markers, col=col[1], lodcolumn=1,
                         ylim=ylim, lwd=lwd, gap=gap, mtick=mtick,
                         show.marker.names=show.marker.names, alternate.chrid=alternate.chrid, ...)
        else
            plot.scanone(x, incl.markers=incl.markers, col=col[1], lodcolumn=1,
                         ylim=ylim, lwd=lwd, gap=gap, mtick=mtick,
                         show.marker.names=show.marker.names, alternate.chrid=alternate.chrid,
                         ylab="LOD score", ...)
    }
    else {
        if("ylab" %in% names(dots))
            plot.scanone(x, incl.markers=incl.markers, col=col[1], lodcolumn=1,
                         ylim=ylim, xlim=xlim, lwd=lwd, gap=gap, mtick=mtick,
                         show.marker.names=show.marker.names, alternate.chrid=alternate.chrid, ...)
        else
            plot.scanone(x, incl.markers=incl.markers, col=col[1], lodcolumn=1,
                         ylim=ylim, xlim=xlim, lwd=lwd, gap=gap, mtick=mtick,
                         show.marker.names=show.marker.names, alternate.chrid=alternate.chrid,
                         ylab="LOD score", ...)
    }

    if(ncol(x) > 3)
        for(i in 2:(ncol(x)-2))
            plot.scanone(x, col=col[i], lodcolumn=i, add=TRUE, ...)

    if(is.character(legend) || legend) {
        if(is.character(legend))
            legend(legend, legend=colnames(x)[-(1:2)], col=col, lwd=lwd)
        else
            legend("topright", legend=colnames(x)[-(1:2)], col=col, lwd=lwd)
    }

    invisible()
}

inferredpartitions <-
    function(output, chr, lodthreshold, probthreshold=0.9)
{
    if(!inherits(output, "scanPhyloQTL"))
        stop("Argument 'output' must be of class \"scanPhyloQTL\", as output by scanPhyloQTL.")

    if(missing(chr)) {
        chr <- output[1,1]
        warning("Missing chromosome; using ", chr)
    }
    else if(!any(output[,1]==chr))
        stop("Chromosome \"", chr, "\" not found.")

    if(missing(lodthreshold)) {
        warning("No lodthreshold given; using lodthreshold=0.")
        lodthreshold=0
    }

    if(probthreshold >= 1 || probthreshold <= 0) {
        stop("probthreshold should be in (0,1)")
    }

    output <- output[output[,1]==chr,]
    output[,1] <- as.factor(as.character(output[,1]))
    output <- summary(output, format="postprob")

    if(output$maxlod < lodthreshold) return("null")

    prob <- sort(unlist(output[,3:(ncol(output)-2)]), decreasing=TRUE)
    cs <- cumsum(as.numeric(prob))
    if(!any(cs >= probthreshold)) {
        warning("No values >= probthreshold")
        return(NULL)
    }
    wh <- min(which(cs >= probthreshold))
    names(prob)[seq_len(wh)]
}

# end of phyloqtl_scan.R

#####################################################################
#
# scanone.R
#
# copyright (c) 2001-2020, Karl W Broman
# last modified Feb, 2020
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Hao Wu (The Jackson Lab) wrote the imputation method
#
# Part of the R/qtl package
# Contains: scanone, scanone.perm, scanone.perm.engine
#
######################################################################

######################################################################
#
# scanone: scan genome, calculating LOD scores with single QTL model
#          (covariates are not allowed for models other than "normal"
#           and "binary")
#
######################################################################

scanone <-
    function(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
             method=c("em","imp","hk","ehk","mr","mr-imp","mr-argmax"),
             addcovar=NULL, intcovar=NULL, weights=NULL,
             use=c("all.obs", "complete.obs"), upper=FALSE,
             ties.random=FALSE, start=NULL, maxit=4000, tol=1e-4,
             n.perm, perm.Xsp=FALSE, perm.strata=NULL, verbose, batchsize=250,
             n.cluster=1, ind.noqtl)
{
    if(batchsize < 1) stop("batchsize must be >= 1.")
    model <- match.arg(model)
    method <- match.arg(method)
    use <- match.arg(use)

    # in RIL, treat X chromomse like an autosome
    chr_type <- sapply(cross$geno, chrtype)
    crosstype <- crosstype(cross)
    if(any(chr_type=="X") && (crosstype == "risib" || crosstype == "riself"))
        for(i in which(chr_type=="X")) class(cross$geno[[i]]) <- "A"

    if(!missing(n.perm) && n.perm > 0 && n.cluster > 1) {
        cat(" -Running permutations via a cluster of", n.cluster, "nodes.\n")
        updateParallelRNG(n.cluster)

        scanonePermInParallel <- function(n.perm, cross, chr, pheno.col, model, method, addcovar,
                                          intcovar, weights, use, upper, ties.random, start, maxit, tol,
                                          perm.Xsp, perm.strata, batchsize)
            scanone(cross=cross, chr=chr, pheno.col=pheno.col, model=model, method=method, addcovar=addcovar,
                    intcovar=intcovar, weights=weights, use=use, upper=upper, ties.random=ties.random, start=start,
                    maxit=maxit, tol=tol, n.perm=n.perm, perm.Xsp=perm.Xsp, perm.strata=perm.strata, batchsize=batchsize,
                    n.cluster=0, verbose=FALSE)

        n.perm <- ceiling(n.perm/n.cluster)
        if(missing(chr)) chr <- names(cross$geno)
        if(Sys.info()[1] == "Windows") { # Windows doesn't support mclapply, but it's faster if available
            cl <- makeCluster(n.cluster)
            on.exit(stopCluster(cl))
            operm <- clusterApply(cl, rep(n.perm, n.cluster), scanonePermInParallel, cross=cross, chr=chr, pheno.col=pheno.col,
                                  model=model, method=method, addcovar=addcovar, intcovar=intcovar, weights=weights, use=use,
                                  upper=upper, ties.random=ties.random, start=start, maxit=maxit, tol=tol, perm.Xsp=perm.Xsp,
                                  perm.strata=perm.strata, batchsize=batchsize)
        }
        else {
            operm <- mclapply(rep(n.perm, n.cluster), scanonePermInParallel, cross=cross, chr=chr, pheno.col=pheno.col,
                              model=model, method=method, addcovar=addcovar, intcovar=intcovar, weights=weights, use=use,
                              upper=upper, ties.random=ties.random, start=start, maxit=maxit, tol=tol, perm.Xsp=perm.Xsp,
                              perm.strata=perm.strata, batchsize=batchsize, mc.cores=n.cluster)
        }
        for(j in 2:length(operm))
            operm[[1]] <- c(operm[[1]], operm[[j]])
        return(operm[[1]])
    }

    # check perm.strat
    if(!missing(perm.strata) && !is.null(perm.strata)) {
        if(length(perm.strata) != nind(cross))
            stop("perm.strata, if given, must have length = nind(cross) [", nind(cross), "]")
    }

    # individuals with no QTL effect
    if(missing(ind.noqtl)) ind.noqtl <- rep(FALSE, nind(cross))
    else {
        if(method %in% c("mr", "mr-imp", "mr-argmax","ehk") ||
           model %in% c("2part", "np")) {
            ind.noqtl <- rep(FALSE, nind(cross))
            warning("ind.noqtl ignored for model=", model, ", method=", method)
        }
        else if(is.null(addcovar) && (!is.logical(ind.noqtl) || any(ind.noqtl))) {
            ind.noqtl <- rep(FALSE, nind(cross))
            warning("ind.noqtl ignored when no additive covariates")
        }
        else if(!is.logical(ind.noqtl) || length(ind.noqtl) != nind(cross))
            stop("ind.noqtl be a logical vector of length n.ind (", nind(cross), ")")
    }

    if(!missing(chr)) cross <- subset(cross, chr)
    if(missing(n.perm)) n.perm <- 0

    if(missing(verbose)) {
        if(!missing(n.perm) && n.perm > 0) verbose <- TRUE
        else verbose <- FALSE
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(any(is.na(num))) {
            if(sum(is.na(num)) > 1)
                stop("Couldn't identify phenotypes ", paste(paste("\"", pheno.col[is.na(num)], "\"", sep=""),
                                                            collapse=" "))
            else
                stop("Couldn't identify phenotype \"", pheno.col[is.na(num)], "\"")
        }
        pheno.col <- num
    }

    if(any(pheno.col < 1 | pheno.col > nphe(cross)))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    if(length(pheno.col)==1 && n.perm>=0) use <- "complete.obs"

    if(n.perm >= 0) {    # not in the midst of a permutation test
        # If use="all.obs", check whether there are individuals missing some
        # phenotypes but not others.  If not, act like "complete.obs".
        if(use=="all.obs" && length(pheno.col) > 1) {
            n.phe <- length(pheno.col)
            temp <- apply(cross$pheno[,pheno.col], 1, function(a) sum(is.na(a)))
            if(all(temp==0 | temp==n.phe)) use <- "complete.obs"
        }

        # If use="complete.obs", drop individuals with any missing phenotypes
        if(use=="complete.obs") {
            temp <- checkcovar(cross, pheno.col, addcovar, intcovar,
                               perm.strata, ind.noqtl, weights, TRUE)
            cross <- temp[[1]]
            pheno <- temp[[2]]
            addcovar <- temp[[3]]
            intcovar <- temp[[4]]
            n.addcovar <- temp[[5]]
            n.intcovar <- temp[[6]]
            perm.strata <- temp[[7]]
            ind.noqtl <- temp[[8]]
            weights <- temp[[9]]
        }
    }

    # use all observations; not in a permutation test; different phenotypes have different sets of missing values
    #   -> want to do in batches, but need to define batches by the pattern of missing data
    if(n.perm <= 0 && use=="all.obs" && length(pheno.col) > 1 && (method=="hk" || method=="imp") && model == "normal") {
        # drop individuals with missing covariates
        cross$pheno <- cbind(cross$pheno, rep(1, nind(cross)))
        temp <- checkcovar(cross, nphe(cross), addcovar, intcovar,
                           perm.strata, ind.noqtl, weights, TRUE)
        cross <- temp[[1]]
        pheno <- cross$pheno[,pheno.col, drop=FALSE]
        addcovar <- temp[[3]]
        intcovar <- temp[[4]]
        n.addcovar <- temp[[5]]
        n.intcovar <- temp[[6]]
        perm.strata <- temp[[7]]
        ind.noqtl <- temp[[8]]
        weights <- temp[[9]]

        # determine the batches (defined by the pattern of missing data)
        patterns <- apply(pheno, 2, function(a) paste(!is.na(a), collapse=":"))
        upat <- unique(patterns)
        m <- match(patterns, upat)
        batches <- vector("list", length(upat))
        upat <- lapply(strsplit(upat, ":"), function(a) as.logical(a))
        for(i in seq(along=batches)) batches[[i]] <- pheno.col[m==i]

        # run scanone for one batch at a time
        out <- NULL
        for(i in seq(along=batches)) {
            if(!is.null(addcovar)) {
                if(!is.matrix(addcovar)) addcovar <- as.matrix(addcovar)
                tempac <- addcovar[upat[[i]],,drop=FALSE]
            }
            else tempac <- addcovar
            if(!is.null(intcovar)) {
                if(!is.matrix(intcovar)) intcovar <- as.matrix(intcovar)
                tempic <- intcovar[upat[[i]],,drop=FALSE]
            }
            else tempic <- intcovar

            temp <- scanone(subset(cross, ind=upat[[i]]), chr=chr, pheno.col=batches[[i]], model=model,
                            method=method, addcovar=tempac, intcovar=tempic,
                            weights=weights[upat[[i]]], use="complete.obs", upper=upper, ties.random=ties.random,
                            start=start, maxit=maxit, tol=tol, n.perm=n.perm, perm.Xsp=perm.Xsp,
                            perm.strata=perm.strata[upat[[i]]], verbose=verbose, batchsize=batchsize,
                            n.cluster=n.cluster)
            if(is.null(out)) out <- temp
            else out <- cbind(out, temp)
        }

        # reorder LOD score columns and make sure that the names are correct
        colnames(out)[-(1:2)] <- colnames(cross$pheno)[unlist(batches)]
        out[,-(1:2)] <- out[,colnames(cross$pheno)[pheno.col]]
        colnames(out)[-(1:2)] <- colnames(cross$pheno)[pheno.col]

        return(out)
    }

    # multiple phenotype for methods except imp or hk with normal model
    if(length(pheno.col)>1 && n.perm <= 0 && (model != "normal" ||
                              (method!="imp" && method != "hk"))) {
        out <- scanone(cross, chr, pheno.col[1], model, method,
                       addcovar, intcovar, weights, use, upper, ties.random,
                       start, maxit, tol, n.perm, perm.Xsp, perm.strata,
                       verbose, batchsize, n.cluster=0)
        nc <- ncol(out)-2
        cn <- colnames(out)[-(1:2)]
        for(i in 2:length(pheno.col))
            out[,ncol(out)+1:nc] <- scanone(cross, chr, pheno.col[i], model,
                                            method, addcovar, intcovar, weights,
                                            use, upper, ties.random, start,
                                            maxit, tol, n.perm, perm.Xsp,
                                            perm.strata, verbose, batchsize, n.cluster=0)[,-(1:2)]

        if(length(cn) > 1)
            colnames(out)[-(1:2)] <- paste(rep(cn,length(pheno.col)),
                                           rep(colnames(cross$pheno)[pheno.col],
                                               rep(nc,length(pheno.col))),
                                           sep=".")
        else
            colnames(out)[-(1:2)] <- colnames(cross$pheno)[pheno.col]

        return(out)
    }

    if(n.perm>0) {
        return(scanone.perm(cross, pheno.col, model, method, addcovar,
                            intcovar, weights, use, upper, ties.random,
                            start, maxit, tol, n.perm, perm.Xsp, perm.strata,
                            verbose, batchsize))
    }

    if(n.perm < 0) { # in the midst of permutations
        if(use=="all.obs") {
            temp <- checkcovar(cross, pheno.col, addcovar, intcovar,
                               perm.strata, ind.noqtl, weights, n.perm==-1)
            cross <- temp[[1]]
            pheno <- temp[[2]]
            addcovar <- temp[[3]]
            intcovar <- temp[[4]]
            n.addcovar <- temp[[5]]
            n.intcovar <- temp[[6]]
            perm.strata <- temp[[7]]
            ind.noqtl <- temp[[8]]
            weights <- temp[[9]]
        }
        else {
            pheno <- as.matrix(cross$pheno[,pheno.col])
            if(is.null(addcovar)) n.addcovar <- 0
            else n.addcovar <- ncol(addcovar)
            if(is.null(intcovar)) n.intcovar <- 0
            else n.intcovar <- ncol(intcovar)
        }
    }

    n.chr <- nchr(cross)
    n.ind <- nind(cross)
    n.phe <- ncol(pheno)
    type <- crosstype(cross)
    is.bcs <- FALSE
    if(type == "bcsft") {
        cross.scheme <- attr(cross, "scheme")
        is.bcs <- (cross.scheme[2] == 0)
    }

    # fill in missing genotypes with imputed values
    if(n.perm==0) { # not in the midst of permutations
        if(method=="mr-argmax")
            cross <- fill.geno(cross,method="argmax")
        if(method=="mr-imp")
            cross <- fill.geno(cross,method="imp")
    }

    # weights for model="normal"
    if(model != "normal") {
        if(!is.null(weights) && !all(weights==1) && n.perm > -2)
            warning("weights used only for normal model.")
    }
    else {
        if(is.null(weights))
            weights <- rep(1, nind(cross))
        else
            if(length(weights) != nind(cross))
                stop("weights should either be NULL or a vector of length n.ind")
        if(any(weights <= 0))
            stop("weights should be entirely positive")
        weights <- sqrt(weights)
    }

    if(model=="binary") {
        if(method=="imp") {
            if(n.perm > -2) warning("Method imp not available for binary model; using em")
            method <- "em"
        }
        return(discan(cross, pheno, method, addcovar, intcovar, maxit, tol, verbose, n.perm > -2, ind.noqtl))
    }
    else if(model=="2part") {
        if((n.addcovar > 0 || n.intcovar > 0) && n.perm > -2)
            warning("Covariates ignored for the two-part model.")
        if(method!="em") {
            if(n.perm > -2) warning("Only em method is available for the two-part model")
            method <- "em"
        }
        return(vbscan(cross, pheno.col, upper, method, maxit, tol))
    }
    else if(model=="np") {
        if((n.addcovar > 0 || n.intcovar > 0) && n.perm > -2)
            warning("Covariates ignored for non-parametric interval mapping.")
        if(method!="em") {
            if(n.perm > -2) warning("Method argument ignored for non-parametric interval mapping.")
            method <- "em"
        }
    }

    # if non-parametric, convert phenotypes to ranks
    if(model=="np") {
        if(ties.random) {
            y <- pheno[!is.na(pheno)]
            y <- rank(y+runif(length(y))/(sd(y)*10^8))
            pheno[!is.na(pheno)] <- y
            correct <- 1
        }
        else {
            ties <- table(pheno)
            if(any(ties > 1)) {
                ties <- ties[ties>1]
                correct <- 1-sum(ties^3-ties)/(n.ind^3-n.ind)
            }
            else correct <- 1
            pheno <- rank(pheno)
        }
    }

    results <- NULL

    # starting points for interval mapping
    if(method=="em" && model=="normal") {
        if(is.null(start)) std.start <- 1
        else if(length(start)==1) std.start <- -1
        else std.start <- 0
    }

    # scan genome one chromosome at a time
    for(i in 1:n.chr) {
        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type=="X") {
            sexpgm <- getsex(cross)
            ac <- revisecovar(sexpgm,addcovar)

            if(!is.null(addcovar) && (nd <- attr(ac, "n.dropped")) > 0 && n.perm > -2)
                warning("Dropped ", nd, " additive covariates on X chromosome.")

            if(length(ac)==0) {
                n.ac <- 0
                ac <- NULL
            }
            else n.ac <- ncol(ac)
            ic <- revisecovar(sexpgm,intcovar)
            if(!is.null(intcovar) && (nd <- attr(ic, "n.dropped")) > 0 && n.perm > -2)
                warning("Dropped ", nd, " interactive covariates on X chromosome.")
            if(length(ic)==0) {
                n.ic <- 0
                ic <- NULL
            }
            else n.ic <- ncol(ic)
        }
        else {
            sexpgm <- NULL
            ac <- addcovar
            n.ac <- n.addcovar
            ic <- intcovar
            n.ic <- n.intcovar
        }

        # get genotype names
        gen.names <- getgenonames(type,chr_type,"full",sexpgm,attributes(cross))
        n.gen <- length(gen.names)

        # starting values for interval mapping
        if(method=="em" && model=="normal") {
            this.start <- rep(0,n.gen+1)
            if(std.start == 0) {
                if(length(start) < n.gen+1)
                    stop("Length of start argument should be 0, 1 or ", n.gen+1)
                this.start <- c(start[1:n.gen],start[length(start)])
            }
        }

        # pull out reconstructed genotypes (mr)
        # or imputations (imp)
        # or genotype probabilities (em or hk)
        if(method=="mr" || method=="mr-imp" || method=="mr-argmax") {
            newgeno <- cross$geno[[i]]$data
            newgeno[is.na(newgeno)] <- 0

            # discard partially informative genotypes
            if(type=="f2" || (type=="bcsft" && !is.bcs)) newgeno[newgeno>3] <- 0
            if(type=="4way") newgeno[newgeno>4] <- 0

            # revise X chromosome genotypes
            if(chr_type=="X" && (type %in% c("bc","f2","bcsft")))
                newgeno <- reviseXdata(type, "full", sexpgm, geno=newgeno,
                                       cross.attr=attributes(cross))

            n.pos <- ncol(newgeno)
            map <- cross$geno[[i]]$map
            if(is.matrix(map)) {
                marnam <- colnames(map)
                map <- map[1,]
            }
            else marnam <- names(map)
        }
        else if(method == "imp") {
            if(!("draws" %in% names(cross$geno[[i]]))) {
                # need to run sim.geno
                if(n.perm > -2) warning("First running sim.geno.")
                cross <- sim.geno(cross)
            }

            draws <- cross$geno[[i]]$draws
            n.pos <- ncol(draws)
            n.draws <- dim(draws)[3]

            # revise X chromosome genotypes
            if(chr_type=="X" && (type %in% c("bc","f2","bcsft")))
                draws <- reviseXdata(type, "full", sexpgm, draws=draws,
                                     cross.attr=attributes(cross))

            if("map" %in% names(attributes(cross$geno[[i]]$draws)))
                map <- attr(cross$geno[[i]]$draws,"map")
            else {
                stp <- attr(cross$geno[[i]]$draws, "step")
                oe <- attr(cross$geno[[i]]$draws, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[i]]$draws)))
                    stpw <- attr(cross$geno[[i]]$draws, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
            }

            if(is.matrix(map)) {
                marnam <- colnames(map)
                map <- map[1,]
            }
            else marnam <- names(map)
        }
        else {
            if(!("prob" %in% names(cross$geno[[i]]))) {
                # need to run calc.genoprob
                if(n.perm > -2) warning("First running calc.genoprob.")
                cross <- calc.genoprob(cross)
            }
            genoprob <- cross$geno[[i]]$prob
            n.pos <- ncol(genoprob)

            # revise X chromosome genotypes
            if(chr_type=="X" && (type %in% c("bc","f2","bcsft")))
                genoprob <- reviseXdata(type, "full", sexpgm, prob=genoprob,
                                        cross.attr=attributes(cross))

            if("map" %in% names(attributes(cross$geno[[i]]$prob)))
                map <- attr(cross$geno[[i]]$prob,"map")
            else {
                stp <- attr(cross$geno[[i]]$prob, "step")
                oe <- attr(cross$geno[[i]]$prob, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                    stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
            }

            if(is.matrix(map)) {
                marnam <- colnames(map)
                map <- map[1,]
            }
            else marnam <- names(map)
        }

        # call the C function
        if(method == "mr" || method=="mr-imp" || method=="mr-argmax")
            z <- .C("R_scanone_mr",
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(n.gen),         # number of possible genotypes
                    as.integer(newgeno),       # genotype data
                    as.double(ac),       # additive covariates
                    as.integer(n.ac),
                    as.double(ic),       # interactive covariates
                    as.integer(n.ic),
                    as.double(pheno),          # phenotype data
                    as.double(weights),        # weights
                    result=as.double(rep(0,n.pos)),
                    PACKAGE="qtl")
        else if(method=="imp") {
            if(n.phe > batchsize) {
                firstcol <- 1
                z <- NULL
                while(firstcol <= n.phe) {
                    if(verbose > 2) cat("chr", names(cross$geno)[i], "phe", firstcol, "...\n")
                    thiscol <- firstcol + 0:(batchsize-1)
                    thiscol <- thiscol[thiscol <= n.phe]
                    thisz <- .C("R_scanone_imp",
                                as.integer(n.ind),
                                as.integer(n.pos),
                                as.integer(n.gen),
                                as.integer(n.draws),
                                as.integer(draws),
                                as.double(ac),
                                as.integer(n.ac),
                                as.double(ic),
                                as.integer(n.ic),
                                as.double(pheno[,thiscol]),
                                as.integer(length(thiscol)), # number of phenotypes
                                as.double(weights),
                                result=as.double(rep(0,length(thiscol)*n.pos)),
                                as.integer(ind.noqtl),       # indicators of ind'l w/o QTL effects
                                PACKAGE="qtl")
                    firstcol <- firstcol + batchsize
                    if(is.null(z)) {
                        z <- thisz
                        z$result <- matrix(ncol=n.phe, nrow=n.pos)
                    }
                    z$result[,thiscol] <- matrix(thisz$result, nrow=n.pos)
                }
            }
            else {
                z <- .C("R_scanone_imp",
                        as.integer(n.ind),
                        as.integer(n.pos),
                        as.integer(n.gen),
                        as.integer(n.draws),
                        as.integer(draws),
                        as.double(ac),
                        as.integer(n.ac),
                        as.double(ic),
                        as.integer(n.ic),
                        as.double(pheno),
                        as.integer(n.phe), # number of phenotypes
                        as.double(weights),
                        result=as.double(rep(0,n.phe*n.pos)),
                        as.integer(ind.noqtl),       # indicators of ind'l w/o QTL effects
                        PACKAGE="qtl")
            }
        }
        else if(method=="hk") { # Haley-Knott regression
            if(n.phe > batchsize) {
                firstcol <- 1
                z <- NULL
                while(firstcol <= n.phe) {
                    if(verbose > 2) cat("chr", names(cross$geno)[i], "phe", firstcol, "...\n")
                    thiscol <- firstcol + 0:(batchsize-1)
                    thiscol <- thiscol[thiscol <= n.phe]
                    thisz <- .C("R_scanone_hk",
                                as.integer(n.ind),         # number of individuals
                                as.integer(n.pos),         # number of markers
                                as.integer(n.gen),         # number of possible genotypes
                                as.double(genoprob),       # genotype probabilities
                                as.double(ac),         # additive covariates
                                as.integer(n.ac),
                                as.double(ic),         # interactive covariates
                                as.integer(n.ic),
                                as.double(pheno[,thiscol]),          # phenotype data
                                as.integer(length(thiscol)), # number of phenotypes
                                as.double(weights),
                                result=as.double(rep(0,length(thiscol)*n.pos)),
                                as.integer(ind.noqtl),       # indicators of ind'l w/o QTL effects
                                PACKAGE="qtl")
                    firstcol <- firstcol + batchsize
                    if(is.null(z)) {
                        z <- thisz
                        z$result <- matrix(ncol=n.phe, nrow=n.pos)
                    }
                    z$result[,thiscol] <- matrix(thisz$result, nrow=n.pos)
                }
            }
            else {
                z <- .C("R_scanone_hk",
                        as.integer(n.ind),         # number of individuals
                        as.integer(n.pos),         # number of markers
                        as.integer(n.gen),         # number of possible genotypes
                        as.double(genoprob),       # genotype probabilities
                        as.double(ac),         # additive covariates
                        as.integer(n.ac),
                        as.double(ic),         # interactive covariates
                        as.integer(n.ic),
                        as.double(pheno),          # phenotype data
                        as.integer(n.phe), # number of phenotypes
                        as.double(weights),
                        result=as.double(rep(0,n.phe*n.pos)),
                        as.integer(ind.noqtl),       # indicators of ind'l w/o QTL effects
                        PACKAGE="qtl")
            }
        }
        else if(method=="ehk")  { # extended Haley-Knott method
            z <- .C("R_scanone_ehk",
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(n.gen),         # number of possible genotypes
                    as.double(genoprob),       # genotype probabilities
                    as.double(ac),         # additive covariates
                    as.integer(n.ac),
                    as.double(ic),         # interactive covariates
                    as.integer(n.ic),
                    as.double(pheno),          # phenotype data
                    as.double(weights),
                    result=as.double(rep(0,n.pos)),
                    as.integer(maxit),
                    as.double(tol),
                    PACKAGE="qtl")
        }

        else if(method=="em" && model=="normal")  # interval mapping
            z <- .C("R_scanone_em",
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(n.gen),         # number of possible genotypes
                    as.double(genoprob),       # genotype probabilities
                    as.double(ac),
                    as.integer(n.ac),
                    as.double(ic),
                    as.integer(n.ic),
                    as.double(pheno),          # phenotype data
                    as.double(weights),
                    result=as.double(rep(0,n.pos)),
                    as.integer(std.start),
                    as.double(this.start),
                    as.integer(maxit),
                    as.double(tol),
                    as.integer(0), # debugging verbose off
                    as.integer(ind.noqtl),       # indicators of ind'l w/o QTL effects
                    PACKAGE="qtl")

        else if(model=="np")  # non-parametric interval mapping
            z <- .C("R_scanone_np",
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(n.gen),         # number of possible genotypes
                    as.double(genoprob),       # genotype probabilities
                    as.double(pheno) ,         # phenotype data
                    result=as.double(rep(0,n.pos)),
                    PACKAGE="qtl")

        else
            stop("Model ", model, " with method ", method, " not available")

        z <- matrix(z$result,nrow=n.pos)

        # interval mapping without covariates:
        #   rescale log likelihood
        if(model == "np" && !ties.random)
            z <- z/correct  # correct for ties

        # setup column names for z
        if(length(pheno.col)==1)
            colnames(z) <- "lod"
        else {
            if(is.null(colnames(pheno)))
                colnames(z) <- paste("lod", pheno.col, sep="")
            else
                colnames(z) <- colnames(pheno)
        }

        # get null log10 likelihood
        if(i==1 & model != "np") {
            if(n.ac > 0)
                resid0 <- lm(pheno ~ ac, weights=weights^2)$resid
            else
                resid0 <- lm(pheno ~ 1, weights=weights^2)$resid

            if(method=="hk")  {
                if(n.phe > 1) {
                    nllik0 <- apply(resid0, 2, function(x)
                                    (n.ind/2)*log10(sum((x*weights)^2)))
                }
                else
                    nllik0 <- (n.ind/2)*log10(sum((resid0*weights)^2))
            }

            else {
                sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
                nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
            }
        }

        # re-scale with null log10 likel for methods em and ehk
        if(method=="em" && model=="normal")
            z <- nllik0 - z

        else if(method=="ehk")
            z <- z/log(10) + nllik0

        else if(method == "hk") {
            if(n.phe > 1) {
                z <- t(nllik0 - t(z))
            }
            else
                z <- nllik0 - z
        }

        # get null log10 likelihood for the X chromosome
        if(chr_type=="X") {

            # determine which covariates belong in null hypothesis
            temp <- scanoneXnull(type, sexpgm, cross.attr=attributes(cross))
            adjustX <- temp$adjustX
            parX0 <- temp$parX0+n.ac
            sexpgmcovar <- temp$sexpgmcovar

            if(adjustX) {
                if(model == "np") {
                    sexpgmcovar <- factor(apply(sexpgmcovar,1,paste,collapse=":"))
                    nllikX <- kruskal.test(pheno ~ sexpgmcovar)$stat/(2*log(10))
                    z <- z - nllikX
                }
                else if(method=="mr") {
                    for(s in 1:ncol(newgeno)) {
                        wh <- newgeno[,s] != 0

                        if(n.ac > 0) {
                            residX <- lm(pheno ~ ac+sexpgmcovar, weights=weights^2,subset=wh)$resid
                            resid0 <- lm(pheno ~ ac, weights=weights^2,subset=wh)$resid
                        }
                        else {
                            residX <- lm(pheno ~ sexpgmcovar, weights=weights^2,subset=wh)$resid
                            resid0 <- lm(pheno ~ 1, weights=weights^2,subset=wh)$resid
                        }
                        nllikX <- (sum(wh)/2)*log10(sum((residX*weights[wh])^2))
                        nllik0 <- (sum(wh)/2)*log10(sum((resid0*weights[wh])^2))

                        # rescale LOD score
                        z[s,] <- z[s,] + nllikX - nllik0
                    }
                }
                else {
                    if(n.ac > 0) {
                        outX <- lm(pheno ~ ac+sexpgmcovar, weights=weights^2)
                        residX <- outX$resid
                        # revise the parX0, if some columns got dropped
                        parX0 <- outX$rank
                    }
                    else
                        residX <- lm(pheno ~ sexpgmcovar, weights=weights^2)$resid

                    if(method=="hk") {
                        if(n.phe==1)
                            nllikX <- (n.ind/2)*log10(sum((residX*weights)^2))
                        else
                            nllikX <- (n.ind/2)*apply(residX, 2, function(a,b)
                                                      log10(sum((a*b)^2)), weights)
                    }
                    else {
                        if(method=="imp") {
                            if(n.ac > 0) {
                                out0 <- lm(pheno ~ ac, weights=weights^2)
                                resid0 <- out0$resid
                            }
                            else {
                                out0 <- lm(pheno ~ 1, weights=weights^2)
                                resid0 <- out0$resid
                            }

                            if(n.phe > 1) {
                                sig0 <- sqrt(apply(resid0, 2, function(a,b) sum((a*b)^2),weights)/n.ind)
                                nllik0 <- sig0
                                for(j in seq(along=nllik0))
                                    nllik0[j] <- -sum(dnorm(resid0[,j],0,sig0[j]/weights,log=TRUE))/log(10)
                            }
                            else {
                                sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
                                nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
                            }
                        }

                        if(n.phe > 1) {
                            sigX <- sqrt(apply(residX, 2, function(a,b) sum((a*b)^2),weights)/n.ind)
                            nllikX <- sigX
                            for(j in seq(along=nllikX))
                                nllikX[j] <- -sum(dnorm(residX[,j],0,sigX[j]/weights,log=TRUE))/log(10)
                        }
                        else {
                            sigX <- sqrt(sum((residX*weights)^2)/n.ind)
                            nllikX <- -sum(dnorm(residX,0,sigX/weights,log=TRUE))/log(10)
                        }
                    }
                }

                if(method != "mr" && model != "np") {
                    # rescale LOD score
                    z <- t(t(z) + nllikX - nllik0)
                }
            }
        }

        # replace missing or negative LODs with 0
        z[is.na(z) | z<0] <- 0

        w <- marnam
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",names(cross$geno)[i],".",w[o],sep="")

        z <- as.data.frame(z, stringsAsFactors=TRUE)
        z <- cbind(chr=factor(rep(names(cross$geno)[i],length(map)),levels=names(cross$geno)),
                   pos=as.numeric(map), z)
        rownames(z) <- w

        results <- rbind(results, z)
    }

    class(results) <- c("scanone","data.frame")
    attr(results,"method") <- method
    attr(results,"type") <- type
    attr(results,"model") <- model

    results
}




######################################################################
#
# scanone.perm: Permutation test of scanone
#
######################################################################

scanone.perm <-
    function(cross, pheno.col=1, model=c("normal","binary","2part","np"),
             method=c("em","imp","hk","ehk","mr","mr-imp","mr-argmax"),
             addcovar=NULL, intcovar=NULL, weights=NULL,
             use=c("all.obs", "complete.obs"), upper=FALSE,
             ties.random=FALSE, start=NULL, maxit=4000, tol=1e-4,
             n.perm=1000, perm.Xsp=FALSE, perm.strata=NULL,
             verbose=TRUE, batchsize=250)
{
    method <- match.arg(method)
    model <- match.arg(model)
    use <- match.arg(use)

    if((model=="2part" || model=="np") && (!is.null(addcovar) || !is.null(intcovar))) {
        warning("Use of covariates not available for model ", model)
        addcovar <- intcovar <- NULL
    }

    chr.type <- sapply(cross$geno, chrtype)
    if((all(chr.type=="X") || all(chr.type=="X")) && perm.Xsp==TRUE)
        warning("All chromosomes of the same type, so X-chr specific permutations not needed.\n")

    if(any(chr.type=="X") && any(chr.type=="A") && perm.Xsp) { # autosome and X-specific perms
        # X chr versus autosomes
        xchr <- chr.type=="X"
        names(xchr) <- names(cross$geno)

        # chromosome lengths
        L <- summary(pull.map(cross))[,2]
        L <- L[-length(L)]
        La <- sum(L[!xchr])
        Lx <- sum(L[xchr])

        n.perm.X <- ceiling(La/Lx*n.perm)

        if(verbose) cat("--Autosome permutations\n")
        resA <- scanone.perm.engine(n.perm, subset(cross, chr=!xchr),
                                    pheno.col, model,
                                    method, addcovar, intcovar, weights, use,
                                    upper, ties.random, start, maxit, tol,
                                    verbose, perm.strata, batchsize)

        if(verbose) cat("--X chromosome permutations\n")
        resX <- scanone.perm.engine(n.perm.X, subset(cross, chr=xchr),
                                    pheno.col, model,
                                    method, addcovar, intcovar, weights, use,
                                    upper, ties.random, start, maxit, tol,
                                    verbose, perm.strata, batchsize)
        res <- list("A"=resA, "X"=resX)
        attr(res, "xchr") <- xchr
        attr(res, "L") <- c("A"=La, "X"=Lx)
    }

    else {
        res <- scanone.perm.engine(n.perm, cross, pheno.col, model,
                                   method, addcovar, intcovar, weights, use,
                                   upper, ties.random, start, maxit, tol,
                                   verbose, perm.strata, batchsize)
    }

    attr(res,"method") <- method
    attr(res,"model") <- model
    attr(res,"type") <- crosstype(cross)

    if(any(chr.type=="X") && any(chr.type=="A") && perm.Xsp)
        class(res) <- c("scanoneperm", "list")
    else
        class(res) <- c("scanoneperm", "matrix")

    res
}

##################################################################
# engine function to permform permutation test
##################################################################
scanone.perm.engine <-
    function(n.perm, cross, pheno.col, model,
             method, addcovar, intcovar, weights, use,
             upper, ties.random, start, maxit, tol,
             verbose, perm.strata, batchsize=250)
{
    ## local variables
    n.phe <- length(pheno.col)
    n.addcov <- ncol(addcovar)
    n.intcovar <- ncol(intcovar)
    n.ind <- dim(cross$pheno)[1]

    if(method=="mr-imp") # save version with missing genotypes
        tempcross <- cross
    if(method=="mr-argmax") # impute genotypes
        cross <- fill.geno(cross,method="argmax")

    ## if there's only one phenotype, no covariate, and method is imp or hk,
    ## generate permuted phenotype as a matrix
    ## we also need one sex and one direction, or that the
    ##     stratification is within those groups
    batch.mode <- FALSE
    if( (n.phe==1) && ((method=="imp") || (method=="hk")) &&
       model == "normal" &&
       is.null(addcovar) && is.null(intcovar) ) {
        chr_type <- sapply(cross$geno, chrtype)
        sexpgm <- getsex(cross)
        sex <- sexpgm$sex
        pgm <- sexpgm$pgm
        if(all(chr_type=="A"))
            batch.mode <- TRUE
        else if((is.null(sex) || length(unique(sex))==1) &&
                (is.null(pgm) || length(unique(pgm))==1))
            batch.mode <- TRUE
        else if(!is.null(perm.strata)) {
            sp <- paste(sex, pgm, sep=":")
            tab <- table(sp, perm.strata)
            if(all(apply(tab, 2, function(a) sum(a != 0))==1))
                batch.mode <- TRUE
        }
    }

    if(batch.mode) {
        if(verbose)
            cat("Doing permutation in batch mode ...\n")
        ## if there's only one phenotype, no covariate, and method is imp or hk,
        ## generate permuted phenotype as a matrix and do permutation
        ## as multiple phenotypes
        ord <- matrix(0, n.ind, n.perm)
        if(!is.null(perm.strata)) {  # stratified permutation test
            if(length(perm.strata) != n.ind)
                stop("perm.strata must be NULL or have length nind(cross).")
            u <- unique(perm.strata)
            theindex <- 1:n.ind
            if(length(u)==n.ind)
                stop("All elements of perm.strata are unique, so there will be no real permutation.")
            if(length(u)==1)
                warning("Just one unique element in perm.strata, so the perms are not stratified.")

            for(iperm in 1:n.perm) {
                for(j in u) {
                    wh <- perm.strata==j
                    if(sum(wh)==1) ord[wh,iperm] <- theindex[wh]
                    else ord[wh,iperm] <- sample(theindex[wh])
                }
            }
        }
        else {
            for(iperm in 1:n.perm)
                ord[,iperm] <- sample(n.ind)
        }

        cross$pheno <- cbind(matrix(cross$pheno[,pheno.col][ord], nrow=n.ind), cross$pheno)

        pheno.col <- 1:n.perm
        tem <- scanone(cross,,pheno.col,model,method,addcovar,
                       intcovar, weights, use, upper,ties.random,start,
                       maxit,tol,n.perm= -1, perm.Xsp=FALSE, perm.strata, verbose=FALSE, batchsize,
                       n.cluster=0)

        res <- matrix(apply(tem[,-(1:2),drop=FALSE], 2, max, na.rm=TRUE), ncol=1)
        colnames(res) <- "lod"
    }
    else { ## all other cases, do one permutation at a time
        ## rnd: how often to print tracing information
        if(verbose > 1) rnd <- 1
        else {
            if(n.perm >= 1000) rnd <- 20
            else if(n.perm >= 100) rnd <- 5
            else rnd <- 1
        }

        addcovarp <- addcovar
        intcovarp <- intcovar
        if(!is.null(addcovar)) addcovarp <- as.matrix(addcovarp)
        if(!is.null(intcovar)) intcovarp <- as.matrix(intcovarp)

        if(model=="2part") res <- matrix(ncol=3*n.phe,nrow=n.perm)
        else res <- matrix(0, n.perm, n.phe)

        for(i in 1:n.perm) {
            if(verbose && i/rnd == floor(i/rnd))
                cat("Permutation", i, "\n")
            # impute genotypes for method "mr-imp"
            if(method=="mr-imp") cross <- fill.geno(tempcross)

            if(!is.null(perm.strata)) {  # stratified permutation test
                if(length(perm.strata) != n.ind)
                    stop("perm.strata must be NULL or have length nind(cross).")
                u <- unique(perm.strata)
                theindex <- 1:n.ind
                if(length(u)==n.ind)
                    stop("All elements of perm.strata are unique, so no real permutations.")
                if(length(u)==1 && i==1)
                    warning("Just one unique element in perm.strata, so the perms are not stratified.")

                o <- 1:n.ind
                for(j in u) {
                    wh <- perm.strata==j
                    if(sum(wh)>1) o[wh] <- sample(o[wh])
                }
            }
            else
                o <- sample(1:n.ind)

            cross$pheno <- cross$pheno[o,,drop=FALSE]
            if(!is.null(addcovar)) addcovarp <- addcovarp[o,,drop=FALSE]
            if(!is.null(intcovar)) intcovarp <- intcovarp[o,,drop=FALSE]
            if(!is.null(weights)) weights <- weights[o]

            tem <- scanone(cross,,pheno.col,model,method,addcovarp,
                           intcovarp,weights,use,upper,ties.random,start,
                           maxit,tol,n.perm= -i, perm.Xsp=FALSE, perm.strata, verbose=FALSE, batchsize,
                           n.cluster=0)

            res[i,] <- apply(tem[,-(1:2),drop=FALSE], 2, max, na.rm=TRUE)

        } # finish permutation

        colnames(res) <- colnames(tem)[-(1:2)]
    }

    ## set row and column names when n.phe>1
    rownames(res) <- 1:n.perm

    ## return
    res

}

# end of scanone.R

# Test augmentation with MQM
#
# Note: the full version of this test has moved to ./contrib/bin/rtest, 
#       as it takes a long time to run. The full version can be run with:
#
#   cd contrib/bin
#   rm CMakeCache.txt ; cmake -DTEST_R=TRUE
#   make testR

library(qtl)

set.seed(1000)              

version = mqm_version()
cat("R/qtl=",version$RQTL)
cat("R-MQM=",version$RMQM)
cat("MQM=",version$MQM)

testaugmentation <- function(cross, ...){
  crossML <- mqmaugment(cross, ...)

  res1 <- mqmscan(crossML,logtransform=TRUE)
  list(res1)
}

data(listeria)
r <- testaugmentation(listeria)
if(!round(r[[1]][3,3],3)==0.307) stop("Listeria ML dataaugmentation error")

cat("testaugmentation.R, tests succesfully run!")

#####################################################################
#
# mqmutil.R
#
# Copyright (c) 2009-2018, Danny Arends
#
# Modified by Pjotr Prins and Karl Broman
#
#
# first written Februari 2009
# last modified Mar 2018
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: ourstop
#           ourline
#           mqmextractmarkers
#           estimatemarkerlod
#           stringPhenoToInt
#           addmarkerstointervalmap
#           mqmtestnormal
#           mqmgetmodel
#
#
#####################################################################


# Returns the effective version of R/qtl and MQM. This is used for testing,
# debugging, and error reporting on MQM itself. We could also R/qtl tagged
# releases, but they may be faster/slower than the development version. Also
# the C libraries may be used outside R. Returns a list with values for
# RQTL, RMQM and MQM.

mqm_version <- function() {
    rqtl_version = qtlversion()
    rmqm_version  = "0.90-pre1"
    mqm_version   = rmqm_version # fetch from C code, later
    list(RQTL=rqtl_version, RMQM=rmqm_version, MQM=mqm_version)
}

groupclusteredheatmap <- function(cross, clusteredheatmapresult, height){
    items <- cut(clusteredheatmapresult$Rowv,h=height)$lower
    phenotypes <- names(pull.pheno(cross))
    groups <- vector(length(items), mode="list")
    cnt <- 1
    for(x in items){
        nam <- labels(x)
        groups[[cnt]] <- which(phenotypes %in% nam)
        cnt <- cnt+1
    }
    groups
}

ourstop <- function(...){
    stop(...)
}

ourline <- function(){
    cat("------------------------------------------------------------------\n")
}

simulatemissingdata <- function(cross,percentage=5){
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }
    for(x in 1:length(cross$geno)){
        numtoDROP <- length(cross$geno[[x]]$data)*(percentage/100)
        toDROP <- sample(length(cross$geno[[x]]$data))[1:numtoDROP]
        cross$geno[[x]]$data[toDROP] <- NA
    }
    cross
}

# Return the real markers in the set (remove fake ones)
mqmextractmarkers <- function(mqmresult){
    if(!inherits(mqmresult, "scanone")){
        stop("Wrong type of result file, please supply a valid scanone (from MQM) object.")
    }
    result <- NULL
    for(x in 1:nrow(mqmresult)){
        # for every marker...
        marker = mqmresult[x,]
        found = grep('.loc',rownames(marker))
        if (length(found)==0) {
            result <- rbind(result,marker)
        }
    }
    class(result) <- class(mqmresult)
    result
}

# Return the fake markers in the set (remove real ones)
mqmextractpseudomarkers <- function(mqmresult){
    if(!inherits(mqmresult, "scanone")){
        stop("Wrong type of result file, please supply a valid scanone (from MQM) object.")
    }
    result <- NULL
    for(x in 1:nrow(mqmresult)){
        # for every marker...
        marker = mqmresult[x,]
        found = grep('.loc',rownames(marker))
        if(length(found)!=0) {
            result <- rbind(result,marker)
        }
    }
    class(result) <- class(mqmresult)
    result
}

stepsize <- function(mqmpseudomarkers){
    step <- as.numeric(strsplit(rownames(mqmpseudomarkers)[2],"loc")[[1]][2])-as.numeric(strsplit(rownames(mqmpseudomarkers)[1],"loc")[[1]][2])
    step
}


estimatemarkerlod <- function(interresults){
    #For an okay return, with all markers filled every REAL marker has to be surrounded by interval markers
    #It does skip markers untill we reach the next pseudomarker. When one of the assumptions fails
    #we return, so there could be markers without a LOD score
    if(all(is.na(interresults[,3]))) return (interresults)
    pY <- interresults[1,3]
    pX <- interresults[1,2]
    if(is.na(pY) || is.na(pX)) return (interresults) #The first marker needs to be a interval marker
    for(x in 2:nrow(interresults)){
        if(is.na(interresults[x,3])){
            y <- x
            while(y <= nrow(interresults) && is.na(interresults[y,3]) && interresults[y,1] == interresults[x,1]){
                y <- y + 1
            }
            nY <- interresults[y,3]
            nX <- interresults[y,2]
            if(is.na(nY) || is.na(nX)) return (interresults) #The next marker also needs to be a interval marker
            distp = interresults[x,2] - pX
            distn = nX - interresults[x,2]
            disttot = distn+distp
            interresults[x,3] <- (((nY-pY)/disttot) * distp) + pY
            interresults[x,4] <- 1
            interresults[x,5] <- interresults[x,3]
        }
        pY <- interresults[x,3]
        pX <- interresults[x,2]
    }
    interresults
}

#Function to go from character phenotypes to numeric column numbers
#Based on the code in the scanone function
stringPhenoToInt <- function(cross,pheno.col){
    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(any(is.na(num))) {
            if(sum(is.na(num)) > 1){
                stop("Couldn't identify phenotypes ", paste(paste("\"", pheno.col[is.na(num)], "\"", sep=""),collapse=" "))
            }else{
                stop("Couldn't identify phenotype \"", pheno.col[is.na(num)], "\"")
            }
        }
        pheno.col <- num
    }
    if(any(pheno.col < 1 | pheno.col > nphe(cross))) stop("pheno.col values should be between 1 and the no. phenotypes")
    pheno.col
}

addmarkerstointervalmap <- function(cross,intervalresult,verbose=FALSE){
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }
    if(!inherits(intervalresult, "scanone")) {
        stop("Wrong type of result file, please supply a valid scanone (from MQM) object.")
    }
    map <- pull.map(cross)
    newres <- NULL
    intervalmaploc <- 1
    n <- NULL
    for(chr in 1:length(map)){
        for(mar in 1:length(map[[chr]])){

            if(verbose) cat(chr,"Placing marker: ",names(map[[chr]])[mar]," at ",map[[chr]][mar],"\t",intervalresult[intervalmaploc,2],"\n")
            if((chrtype(map[[chr]])=="A")){
                while(intervalresult[intervalmaploc,2] < map[[chr]][mar]  || intervalresult[intervalmaploc,1] < chr ){
                    newres <- rbind(newres,intervalresult[intervalmaploc,])
                    n <- c(n,rownames(intervalresult)[intervalmaploc])
                    intervalmaploc <- intervalmaploc+1
                }
                if(intervalresult[intervalmaploc,2] == map[[chr]][mar]){
                    newres <- rbind(newres,c(chr,map[[chr]][mar],intervalresult[intervalmaploc,3],intervalresult[intervalmaploc,4],intervalresult[intervalmaploc,5]))
                    while(intervalresult[intervalmaploc,2] == map[[chr]][mar]){
                        intervalmaploc <- intervalmaploc+1
                    }
                }else{
                    newres <- rbind(newres,c(chr,map[[chr]][mar],NA,NA,NA))
                }

                n <- c(n,names(map[[chr]])[mar])
                colnames(newres) <- colnames(intervalresult)
            }
        }
        while(intervalmaploc < nrow(intervalresult) && intervalresult[intervalmaploc,1] <= chr ){
            if((chrtype(map[[chr]])=="A")){
                newres <- rbind(newres,intervalresult[intervalmaploc,])
                n <- c(n,rownames(intervalresult)[intervalmaploc])
            }
            intervalmaploc <- intervalmaploc+1
        }
    }
    #	if(intervalmaploc <= nrow(intervalresult)){
    #	newres <- rbind(newres,intervalresult[intervalmaploc,])
    #n <- c(n,rownames(intervalresult)[intervalmaploc])
    #		intervalmaploc <- intervalmaploc+1
    #	}
    rownames(newres) <- n
    newres <- estimatemarkerlod(newres)
    class(newres) <- c("scanone",class(newres))
    newres
}

mqmtestnormal <- function(cross, pheno.col=1,significance=0.05, verbose=FALSE){
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }

    # if augmented data, pull out just the unique individuals
    if("mqm" %in% names(cross)) {
        theind <- cross$mqm$augIND
        cross <- subset(cross, ind=match(unique(theind), theind))
    }

    # shapiro.test works only for 3 <= n <= 5000
    if(nind(cross) < 3 || nind(cross) > 5000) {
        warning("Can perform test of normality only if 3 <= n <= 5000")
        return(TRUE)   # I guess this should be NA
    }

    if(significance > 1 || significance <= 0){
        stop("significance should be between 0 and 1")
    }

    # if pheno.col has multiple entries
    if(length(pheno.col) > 1) {
        returnval <- rep(NA, length(pheno.col))
        for(i in seq(along=pheno.col))
            returnval[i] <- mqmtestnormal(cross, pheno.col=pheno.col[i], significance=significance, verbose=verbose)
        names(returnval) <- colnames(cross$pheno)[pheno.col]
        return(returnval)
    }

    returnval <- FALSE
    if(pheno.col <0 || pheno.col > nphe(cross)){
        stop("No such phenotype (pheno.col = ",pheno.col,")")
    }
    if(!is.numeric(cross$pheno[,pheno.col])){
        stop("Please supply a numeric trait (pheno.col = ",pheno.col," is not numeric)")
    }

    if((shapiro.test(cross$pheno[,pheno.col])$p.value)  > significance){
        if(verbose) cat("Trait distribution normal\n")
        returnval<- TRUE
    }else{
        if(verbose) cat("Trait distribution not normal\n")
        returnval<- FALSE
    }
    returnval
}

mqmgetmodel <- function(scanresult){
    if(!is.null(scanresult)){
        model <- attr(scanresult,"mqmmodel")
        model
    }else{
        stop("Please supply a scan result made by using mqm with cofactors")
    }
}

# end of mqmutil.R

######################################################################
#
# markerlrt.R
#
# copyright (c) 2010-2019, Karl W Broman
# last modified Dec, 2019
# first written Jul, 2010
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: markerlrt
#
######################################################################

######################################################################
#
# markerlrt: General likelihood ratio test to assess linkage between
#            all pairs of markers
#
######################################################################

markerlrt <-
    function(cross)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    Geno <- pull.geno(cross)
    Geno[is.na(Geno)] <- 0
    maxg <- max(as.numeric(Geno))
    n.ind <- nrow(Geno)
    n.mar <- ncol(Geno)
    mar.names <- colnames(Geno)

    z <- .C("R_markerlrt",
            as.integer(n.ind),         # number of individuals
            as.integer(n.mar),         # number of markers
            as.integer(Geno),
            as.integer(maxg),          # maximum observed genotype
            lod = as.double(rep(0,n.mar*n.mar)),
            PACKAGE="qtl")

    cross$rf <- matrix(z$lod,ncol=n.mar)
    dimnames(cross$rf) <- list(mar.names,mar.names)
    attr(cross$rf, "onlylod") <- TRUE

    cross
}

# end of markerlrt.R

#####################################################################
#
# addmarker.R
#
# copyright (c) 2013-2019, Karl W Broman
# last modified Dec, 2019
# first written May, 2013
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: addmarker.R
######################################################################

######################################################################
# add a marker to a cross object
#
# cross: the cross object
# genotypes: vector of numeric genotypes
# markername: character string with the marker name
# chr: character string with the chromosome ID
# pos: position of the marker
######################################################################
addmarker <-
    function(cross, genotypes, markername, chr, pos)
{
    if(!inherits(cross, "cross"))
        stop("cross must have class \"cross\"")

    chr <- matchchr(chr, names(cross$geno))
    if(length(chr) != 1)
        stop("Only 1 chromosome should be specified.")
    if(length(pos) != 1)
        stop("length(pos) != 1: length(pos) = ", length(pos))
    if(length(markername) != 1)
        stop("length(markername) != 1: length(markername) = ", length(markername))
    if(length(genotypes) != nind(cross))
        stop("length(genotypes) != nind(cross): length(genotypes) = ", length(genotypes),
             ", nind(cross) = ", nind(cross))

    map <- cross$geno[[chr]]$map
    where <- sum(map < pos)
    g <- cross$geno[[chr]]$data
    if(where == 0) {
        map <- c("tmp"=pos, map)
        names(map)[1] <- markername
        cross$geno[[chr]]$map <- map
        g <- cbind("tmp"=genotypes, g)
        colnames(g)[1] <- markername
        cross$geno[[chr]]$data <- g
    }
    else if(where == length(map)) {
        map <- c(map,"tmp"=pos)
        names(map)[length(map)] <- markername
        cross$geno[[chr]]$map <- map
        g <- cbind(g, "tmp"=genotypes)
        colnames(g)[length(map)] <- markername
        cross$geno[[chr]]$data <- g
    }
    else {
        mnames <- names(map)
        map <- c(map[1:where], "tmp"=pos, map[-(1:where)])
        names(map) <- c(mnames[1:where], markername, mnames[-(1:where)])
        cross$geno[[chr]]$map <- map
        g <- cbind(g[,1:where, drop=FALSE], genotypes, g[,-(1:where), drop=FALSE])
        colnames(g) <- names(map)
        cross$geno[[chr]]$data <- g
    }
    cross
}

# end of addmarker.R

######################################################################
#
# readMWril.R
#
# copyright (c) 2009-2020, Karl W Broman
# last modified Feb, 2020
# first written Apr, 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: readMWril
#
######################################################################

######################################################################
#
# readMWril
#
# read multi-way RIL data in comma-delimited format,
# with a separate file for the founder genotypes, and possible a
# separate file for the phenotype data.
#
######################################################################

readMWril <-
    function(dir, rilfile, founderfile,
             type=c("ri4self", "ri4sib", "ri8self", "ri8selfIRIP1", "ri8sib", "bgmagic16"),
             na.strings=c("-","NA"), rotate=FALSE,
             ...)
{
    # create file names
    if(missing(rilfile) || missing(founderfile))
        stop("Need to specify rilfile and founderfile.")

    if(!missing(dir) && dir != "") {
        rilfile <- file.path(dir, rilfile)
        founderfile <- file.path(dir, founderfile)
    }

    type <- match.arg(type)

    args <- list(...)

    # if user wants to use comma for decimal point, we need
    if(length(args) > 0 && "dec" %in% names(args)) {
        dec <- args[["dec"]]
    }
    else dec <- "."

    # read the data file
    if(length(args) < 1 || !("sep" %in% names(args))) {
        # "sep" not in the "..." argument and so take sep=","
        if(length(args) < 1 || !("comment.char" %in% names(args))) {
            gen <- read.table(rilfile, sep=",", na.strings=na.strings,
                              colClasses="character", fill=TRUE,
                              stringsAsFactors=TRUE,
                              blank.lines.skip=TRUE, comment.char="", ...)
            founder <- read.table(founderfile, sep=",", na.strings=na.strings,
                                  colClasses="character", fill=TRUE,
                                  stringsAsFactors=TRUE,
                                  blank.lines.skip=TRUE, comment.char="", ...)
        }
        else {
            gen <- read.table(rilfile, sep=",", na.strings=na.strings,
                              colClasses="character", fill=TRUE,
                              stringsAsFactors=TRUE,
                              blank.lines.skip=TRUE, ...)
            founder <- read.table(founderfile, sep=",", na.strings=na.strings,
                                  colClasses="character", fill=TRUE,
                                  stringsAsFactors=TRUE,
                                  blank.lines.skip=TRUE, ...)
        }
    }
    else {
        if(length(args) < 1 || !("comment.char" %in% names(args))) {
            gen <- read.table(rilfile, na.strings=na.strings,
                              colClasses="character", fill=TRUE,
                              stringsAsFactors=TRUE,
                              blank.lines.skip=TRUE, comment.char="", ...)
            founder <- read.table(founderfile, na.strings=na.strings,
                                  colClasses="character", fill=TRUE,
                                  stringsAsFactors=TRUE,
                                  blank.lines.skip=TRUE, comment.char="", ...)
        }
        else {
            gen <- read.table(rilfile, na.strings=na.strings,
                              colClasses="character", fill=TRUE,
                              stringsAsFactors=TRUE,
                              blank.lines.skip=TRUE, ...)
            founder <- read.table(founderfile, na.strings=na.strings,
                                  colClasses="character", fill=TRUE,
                                  stringsAsFactors=TRUE,
                                  blank.lines.skip=TRUE, ...)
        }
    }

    if(rotate) {
        gen <- as.data.frame(t(gen), stringsAsFactors=FALSE)
        founder <- as.data.frame(t(founder), stringsAsFactors=FALSE)
    }
    rn <- founder[-1,1]
    cn <- founder[1,-1]
    founder <- founder[-1,-1, drop=FALSE]
    founder <- matrix(as.numeric(unlist(founder)), ncol=length(cn))
    dimnames(founder) <- list(rn, cn)

    # determine number of phenotypes based on initial blanks in row 2
    n <- ncol(gen)
    temp <- rep(FALSE,n)
    for(i in 1:n) {
        temp[i] <- all(gen[2,1:i]=="")
        if(!temp[i]) break
    }

    if(!any(temp)) # no phenotypes!
        stop("You must include at least one phenotype (e.g., an index).")
    n.phe <- max((1:n)[temp])

    # Is map included?  yes if first n.phe columns in row 3 are all blank
    if(all(!is.na(gen[3,1:n.phe]) & gen[3,1:n.phe]=="")) {
        map.included <- TRUE
        map <- asnumericwithdec(unlist(gen[3,-(1:n.phe)]), dec=dec)
        if(any(is.na(map)))
            stop("There are missing marker positions.")
        nondatrow <- 3
    }
    else {
        map.included <- FALSE
        map <- rep(0,ncol(gen)-n.phe)
        nondatrow <- 2 # last non-data row
    }
    pheno <- as.data.frame(gen[-(1:nondatrow),1:n.phe,drop=FALSE], stringsAsFactors=FALSE)
    colnames(pheno) <- as.character(gen[1,1:n.phe])

    # replace empty cells with NA
    gen <- sapply(gen,function(a) { a[!is.na(a) & a==""] <- NA; a })

    # pull apart phenotypes, genotypes and map
    mnames <- gen[1,-(1:n.phe)]
    if(any(is.na(mnames)))  stop("There are missing marker names.")
    chr <- gen[2,-(1:n.phe)]
    if(any(is.na(chr))) stop("There are missing chromosome IDs.")

    gen <- matrix(as.numeric(gen[-(1:nondatrow),-(1:n.phe)]),
                  ncol=ncol(gen)-n.phe)

    pheno <- data.frame(lapply(pheno, sw2numeric, dec=dec), stringsAsFactors=TRUE)

    n.str <- nrow(founder)

    if(type == "ri8selfIRIP1"){
    }
    else if(!("cross" %in% names(pheno))) {
        warning("Need a phenotype named \"cross\"; assuming all come from the cross ",
                paste(LETTERS[1:n.str], collapse="x"))
        crosses <- matrix(1:n.str, ncol=n.str, nrow=nrow(gen), byrow=TRUE)
    }
    else {
        pheno$cross <- as.character(pheno$cross)
        if(any(nchar(pheno$cross) != n.str))
            stop("Mismatches in length of \"cross\" phenotype.")
        thecross <- matrix(unlist(strsplit(pheno$cross, "")), byrow=TRUE, ncol=n.str)
        crosses <- matrix(NA, ncol=n.str, nrow=nrow(gen))
        for(i in 1:nrow(thecross))
            crosses[i,] <- match(thecross[i,], LETTERS[1:n.str])
        if(any(is.na(crosses)))
            stop("Problems in the \"cross \" phenotype.")
    }

    # check founder data matches in dimension
    if(ncol(founder) != ncol(gen))
        stop("Different numbers of markers in RIL and founder files.")
    if(any(colnames(founder) != mnames)) {
        cnf <- colnames(founder)
        if(any(is.na(match(cnf, mnames))) || any(is.na(match(mnames, cnf))))
            stop("Mismatch in markers in RIL and founder files.")
        founder <- founder[,match(mnames, cnf),drop=FALSE]
    }

    wh <- which(is.na(gen))
    missingval <- min(as.numeric(c(gen, founder)), na.rm=TRUE)-1
    gen[wh] <- missingval
    founder[is.na(founder)] <- missingval
    d <- dim(gen)
    if(type == "ri8selfIRIP1")
    {
        gen <-
            .C("R_reviseMWrilNoCross",
               as.integer(d[1]),
               as.integer(d[2]),
               as.integer(n.str),
               as.integer(founder),
               gen=as.integer(gen),
               as.integer(missingval),
               PACKAGE="qtl")$gen
    }
    else
    {
        gen <-
            .C("R_reviseMWril",
               as.integer(d[1]),
               as.integer(d[2]),
               as.integer(n.str),
               as.integer(founder),
               gen=as.integer(gen),
               as.integer(crosses),
               as.integer(missingval),
               PACKAGE="qtl")$gen
    }
    gen[wh] <- NA
    gen <- matrix(gen, nrow=d[1])
    gen[gen==0 | gen==(2^n.str-1)] <- NA


    # re-order the markers by chr and position
    # try to figure out the chr labels
    if(all(chr %in% c(1:999,"X","x"))) { # 1...19 + X
        tempchr <- chr
        tempchr[chr=="X" | chr=="x"] <- 1000
        tempchr <- as.numeric(tempchr)
        if(map.included) neworder <- order(tempchr, map)
        else neworder <- order(tempchr)

        chr <- chr[neworder]
        map <- map[neworder]
        gen <- gen[,neworder,drop=FALSE]
        mnames <- mnames[neworder]
    }

    # fix up dummy map
    if(!map.included) {
        map <- split(rep(0,length(chr)),chr)[unique(chr)]
        map <- unlist(lapply(map,function(a) seq(0,length=length(a),by=5)))
        names(map) <- NULL
    }

    # fix up map information
    # number of chromosomes
    uchr <- unique(chr)
    n.chr <- length(uchr)
    geno <- vector("list",n.chr)
    names(geno) <- uchr
    min.mar <- 1
    for(i in 1:n.chr) { # loop over chromosomes
        # create map
        temp.map <- map[chr==uchr[i]]
        names(temp.map) <- mnames[chr==uchr[i]]

        # pull out appropriate portion of genotype data
        data <- gen[,min.mar:(length(temp.map)+min.mar-1),drop=FALSE]
        min.mar <- min.mar + length(temp.map)
        colnames(data) <- names(temp.map)

        geno[[i]] <- list(data=data,map=temp.map)
        if(uchr[i] == "X" || uchr[i] == "x")
            class(geno[[i]]) <- "X"
        else
            class(geno[[i]]) <- "A"
    }

    cross <- list(geno=geno,pheno=pheno)

    # check that data dimensions match
    n.mar1 <- sapply(geno,function(a) ncol(a$data))
    n.mar2 <- sapply(geno,function(a) length(a$map))
    n.phe <- ncol(pheno)
    n.ind1 <- nrow(pheno)
    n.ind2 <- sapply(geno,function(a) nrow(a$data))
    if(any(n.ind1 != n.ind2)) {
        cat(n.ind1,n.ind2,"\n")
        stop("Number of individuals in genotypes and phenotypes do not match.");
    }
    if(any(n.mar1 != n.mar2)) {
        cat(n.mar1,n.mar2,"\n")
        stop("Numbers of markers in genotypes and marker names files do not match.");
    }

    # print some information about the amount of data read
    cat(" --Read the following data:\n");
    cat("\t", n.ind1, " individuals\n");
    cat("\t", sum(n.mar1), " markers\n");
    cat("\t", n.phe, " phenotypes\n");

    if(all(is.na(gen)))
        warning("There is no genotype data!\n")

    if(type != "ri8selfIRIP1")
    {
        cross$cross <- crosses
    }

    # save founder genotypes in data
    founder[founder==missingval] <- NA
    ua <- apply(founder, 2, function(a) unique(a[!is.na(a)]))
    nua <- sapply(ua, length)
    if(all(nua <= 2)) { # re-code as 0/1
        for(i in 1:ncol(founder)) {
            if(nua[i]==1)
                founder[!is.na(founder[,i]),i] <- 0
            else if(nua[i]==2) {
                founder[!is.na(founder[,i]) & founder[,i]==ua[[i]][1],i] <- 0
                founder[!is.na(founder[,i]) & founder[,i]==ua[[i]][2],i] <- 1
            }
        }
    }
    cross$founderGeno <- founder

    class(cross) <- c(type, "cross")

    # check data
    summary(cross)

    cross
}


# end of readMWril.R

## scantwopermhk.R

scantwopermhk <-
    function(cross, chr, pheno.col=1, addcovar=NULL, weights=NULL, n.perm=1,
             batchsize=1000, perm.strata=NULL, perm.Xsp=NULL, verbose=FALSE,
             assumeCondIndep=FALSE)
{
    if(!missing(chr) && !is.null(chr))
        cross <- subset(cross, chr)

    # in RIL, treat X chromosome like an autosome
    chr.names <- chrnames(cross)
    chr_type <- sapply(cross$geno, chrtype)
    type <- crosstype(cross)
    if(any(chr_type=="X") && (type=="risib" || type=="riself"))
        for(i in which(chr_type=="X"))
            class(cross$geno[[i]]) <- chr_type[i] <- "A"

    if(any(chr_type=="X") && (type=="bc" || type=="f2")) # force stratified permutation test
        perm.strata <- force_sexstrata(cross, perm.strata)

    if(!assumeCondIndep) { # if reduce2grid was used, for assumeCondIndep
        # if reduced2grid, force assumeCondIndep=TRUE
        reduced2grid <- attr(cross$geno[[1]]$prob, "reduced2grid")
        if(!is.null(reduced2grid) && reduced2grid) {
            assumeCondIndep <- TRUE
            warning("Using assumeCondIndep=TRUE, since probabilities reduced to grid")
        }
    }

    if(is.null(perm.Xsp) || !perm.Xsp || !any(chr_type=="X")) { # all autosomes
        result <- .scantwopermhk(cross, pheno.col=pheno.col,
                                 addcovar=addcovar, weights=weights, n.perm=n.perm,
                                 batchsize=batchsize,
                                 perm.strata=perm.strata, verbose=verbose,
                                 assumeCondIndep=assumeCondIndep)
    }
    else { # separate A:A, A:X, and X:X
        # covariates for X chr
        Xcovar <- scanoneXnull(crosstype(cross), getsex(cross), attributes(cross))$sexpgmcovar

        # lengths of autosomes and X chr
        chrL <- sapply(cross$geno, function(a) diff(range(a$map)))
        AL <- sum(chrL[chr_type=="A"])
        XL <- sum(chrL[chr_type=="X"])
        AAL <- AL*AL/2
        XXL <- XL*XL/2
        AXL <- AL*XL
        n.permAA <- n.perm
        n.permXX <- ceiling(n.perm * AAL/XXL)
        n.permAX <- ceiling(n.perm * AAL/AXL)

        # names of autosomes and X chr
        Achr <- chr.names[chr_type=="A"]
        Xchr <- chr.names[chr_type=="X"]


        if(verbose) message("Running ", n.permAA, " A:A permutations")
        AAresult <- .scantwopermhk(cross, chr=Achr, pheno.col=pheno.col,
                                   addcovar=cbind(addcovar, Xcovar), # include X covariates
                                   weights=weights, n.perm=n.permAA,
                                   batchsize=batchsize,
                                   perm.strata=perm.strata, verbose=verbose,
                                   assumeCondIndep=assumeCondIndep)

        if(verbose) message("Running ", n.permXX, " X:X permutations")
        XXresult <- .scantwopermhk(cross, chr=Xchr, pheno.col=pheno.col,
                                   addcovar=addcovar, weights=weights, n.perm=n.permXX,
                                   batchsize=batchsize,
                                   perm.strata=perm.strata, verbose=verbose,
                                   assumeCondIndep=assumeCondIndep)

        if(verbose) message("Running ", n.permAX, " A:X permutations")
        AXresult <- .scantwopermhk(cross, chr=list(Achr, Xchr),
                                   pheno.col=pheno.col,
                                   addcovar=addcovar, weights=weights, n.perm=n.permAX,
                                   batchsize=batchsize,
                                   perm.strata=perm.strata, verbose=verbose,
                                   assumeCondIndep=assumeCondIndep)

        result <- list(AA=AAresult, AX=AXresult, XX=XXresult)
        attr(result, "L") <- c(A=AL, X=XL)
        attr(result, "LL") <- c(AA=AAL, AX=AXL, XX=XXL)
        names(chr_type) <- chr.names
        attr(result, "chrtype") <- chr_type
    }

    class(result) <- "scantwoperm"
    result
}



.scantwopermhk <-
    function(cross, chr, pheno.col=1, addcovar=NULL,
             weights=NULL, n.perm=1, batchsize=1000,
             perm.strata=NULL, verbose=FALSE, assumeCondIndep=FALSE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(n.perm > batchsize) { # run in batches
        if(missing(chr)) chr <- chrnames(cross)

        # determine size of batches: as equal as possible
        n.batches <- ceiling(n.perm/batchsize)
        batchsizes <- rep(ceiling(n.perm/n.batches), n.batches)
        batchsizes[n.batches] <- n.perm - sum(batchsizes[-n.batches])

        result <- NULL
        if(verbose) message(" - perms in ", n.batches, " batches")
        for(i in seq(along=batchsizes)) {
            if(verbose) message("   -- batch ", i)
            thisresult <- .scantwopermhk(cross, chr, pheno.col=pheno.col, addcovar=addcovar,
                                         weights=weights, n.perm=batchsizes[i], batchsize=Inf,
                                         perm.strata=perm.strata, verbose=verbose, assumeCondIndep)
            if(is.null(result)) result <- thisresult
            else {
                for(j in seq(along=result))
                    result[[j]] <- rbind(result[[j]], thisresult[[j]])
            }
        }
        return(result)
    }


    # pull out chromosomes to be scanned
    if(missing(chr)) chr1 <- chr2 <- chr <- names(cross$geno)
    else {
        thechr <- names(cross$geno)
        if(is.list(chr)) {
            # special case: do just specific pairs (each of chr1 vs each of chr2, except when chr2 < chr1)
            chr1 <- matchchr(chr[[1]], thechr)
            chr2 <- matchchr(chr[[2]], thechr)
        }
        else
            chr1 <- chr2 <- matchchr(chr, thechr)
    }

    # subset cross and check chr arguments
    cross <- subset(cross, unique(c(chr1, chr2)))
    thechr <- names(cross$geno)
    nchr1 <- match(chr1, thechr)
    nchr2 <- match(chr2, thechr)
    if(!any(sapply(nchr1, function(a,b) any(a <= b), nchr2)))
        stop("Need some of first chr to be <= some of second chr")

    # in RIL, treat X chromomse like an autosome
    chr_type <- sapply(cross$geno, chrtype)
    type <- crosstype(cross)
    if(any(chr_type=="X") && (type == "risib" || type == "riself"))
        for(i in which(chr_type=="X")) class(cross$geno[[i]]) <- "A"

    # check perm.strat
    if(!missing(perm.strata) && !is.null(perm.strata)) {
        if(length(perm.strata) != nind(cross))
            stop("perm.strata, if given, must have length = nind(cross) [", nind(cross), "]")
        if(any(is.na(perm.strata)))
            stop("perm.strata cannot have missing values")
    }

    # grab phenotypes
    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }
    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(any(is.na(num))) {
            if(sum(is.na(num)) > 1)
                stop("Couldn't identify phenotypes ", paste(paste("\"", pheno.col[is.na(num)], "\"", sep=""),
                                                            collapse=" "))
            else
                stop("Couldn't identify phenotype \"", pheno.col[is.na(num)], "\"")
        }
        pheno.col <- num
    }
    if(length(pheno.col) > 1)
        stop("Only one phenotype column allowed")
    if(pheno.col < 1 || pheno.col > nphe(cross))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    # if stepwidth="variable" or stepwidth=="max" when calling calc.genoprob or sim.geno,
    # we force incl.markers=TRUE; I assume it is the same for all chromosomes
    stepwidth.var <- FALSE
    if("stepwidth" %in% names(attributes(cross$geno[[1]]$prob)) &&
       attr(cross$geno[[1]]$prob, "stepwidth") != "fixed") {
        stepwidth.var <- TRUE
        incl.markers <- TRUE
    }

    # omit individuals with missing phenotypes or covariates
    temp <- checkcovar(cross, pheno.col, addcovar, intcovar=NULL,
                       perm.strata, ind.noqtl=NULL, weights, TRUE)
    cross <- temp[[1]]
    pheno <- temp[[2]]
    addcovar <- temp[[3]]
    n.addcovar <- temp[[5]]
    perm.strata <- temp[[7]]
    weights <- temp[[9]]
    n.ind <- length(pheno)
    n.chr <- nchr(cross)
    if(is.null(weights)) weights <- rep(1, n.ind)

    # null log likelihood
    if(n.addcovar > 0)
        resid0 <- lm(pheno ~ addcovar, weights=weights^2)$resid
    else
        resid0 <- lm(pheno ~ 1, weights=weights^2)$resid
    nllik0X <- nllik0 <- (n.ind/2)*log10(sum((resid0*weights)^2))

    # reorganize perm.strata
    if(!missing(perm.strata) && !is.null(perm.strata)) {
        u <- unique(perm.strata)
        perm.strata <- match(perm.strata, u) # turn into integers in {1, ..., n.strata}
        n.strata <- length(u)
    }
    else {
        perm.strata <- rep(0, n.ind)
        n.strata <- 0
    }

    # X chromosome covariates
    if(any(chr_type=="X")) {
        sexpgm <- getsex(cross)

        # get null loglik for X chr
        Xnullcovar <- scanoneXnull(crosstype(cross), sexpgm, attributes(cross))[[3]]
        adjcovar <- cbind(Xnullcovar, addcovar)
        if(!is.null(adjcovar) && ncol(adjcovar) > 0) {
            resid0 <- lm(pheno ~ adjcovar, weights=weights^2)$resid
            nllik0X <- (n.ind/2)*log10(sum((resid0*weights)^2))
        }

        # covariates to use on X chr
        addcovarX <- revisecovar(sexpgm,addcovar)
        if(!is.null(addcovar) && (nd <- attr(addcovarX, "n.dropped")) > 0 && n.perm > -2)
            warning("Dropped ", nd, " additive covariates on X chromosome.")
        if(length(addcovarX)==0) {
            n.acX <- 0
            addcovarX <- NULL
        }
        else n.acX <- ncol(addcovarX)


        for(i in 1:n.chr) {
            if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft"))) {
                oldXchr <- subset(cross, chr=thechr[i])
                cross$geno[[i]]$prob <-
                    reviseXdata(type, "full", sexpgm, prob=cross$geno[[i]]$prob,
                                cross.attr=attributes(cross))
            }
        }
    }

    # no. genotypes and positions on each chromosome
    n.gen <- n.pos <- rep(0, n.chr)
    for(i in 1:n.chr) {
        n.pos[i] <- ncol(cross$geno[[i]]$prob)
        n.gen[i] <- dim(cross$geno[[i]]$prob)[3]
    }

    # create shuffled indices
    if(n.strata==0)
        permindex <- replicate(n.perm, sample(1:n.ind))
    else { # stratified permutation
        permindex <- replicate(n.perm, 1:n.ind)
        for(i in 1:n.strata)
            permindex[perm.strata==i,] <- apply(permindex[perm.strata==i,], 2, sample)
    }
    permindex <- permindex-1

    # begin loop over pairs of chromosomes
    result <- vector("list", length(nchr1)*length(nchr2))
    k <- 0
    for(i in nchr1) { # loop over the 1st chromosome
        for(j in nchr2) { # loop over the 2nd chromosome
            if(j < i) next
            k <- k + 1

            if(chr_type[i]=="X" || chr_type[j]=="X") {
                ac <- addcovarX
                n.ac <- n.acX
            }
            else {
                ac <- addcovar
                n.ac <- n.addcovar
            }
            if(i==j && chr_type[i]=="X") {
                col2drop <- dropXcol(type, sexpgm, attributes(cross))
                n.col2drop <- sum(col2drop)
                n.col2drop.addmodel <- sum(col2drop[1:(2*n.gen[i]-1)])
            }
            else {
                col2drop <- rep(0,n.gen[i]*n.gen[j])
                n.col2drop <- 0
            }

            # print the current working pair
            if(verbose>1) cat(" (", names(cross$geno)[i], ",",
                            names(cross$geno)[j],")\n",sep="")

            if(i==j) { # same chromosome

                if(verbose>2) cat("  --Calculating joint probs.\n")

                if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft"))) {
                    # calculate joint genotype probabilities for all pairs of positions
                    stp <- attr(oldXchr$geno[[1]]$prob, "step")
                    oe <- attr(oldXchr$geno[[1]]$prob, "off.end")
                    err <- attr(oldXchr$geno[[1]]$prob, "error.prob")
                    mf <- attr(oldXchr$geno[[1]]$prob, "map.function")

                    if("stepwidth" %in% names(attributes(oldXchr$geno[[1]]$prob)))
                        stpw <- attr(oldXchr$geno[[1]]$prob, "stepwidth")
                    else stpw <- "fixed"
                    if("map" %in% names(attributes(oldXchr$geno[[1]]$prob)))
                        tmap <- attr(oldXchr$geno[[1]]$prob,"map")
                    else
                        tmap <- create.map(oldXchr$geno[[1]]$map, stp, oe, stpw)

                    temp <- calc.pairprob(oldXchr,stp,oe,err,mf,tmap,
                                          assumeCondIndep=assumeCondIndep)
                }
                else {
                    # calculate joint genotype probabilities for all pairs of positions
                    stp <- attr(cross$geno[[i]]$prob, "step")
                    oe <- attr(cross$geno[[i]]$prob, "off.end")
                    err <- attr(cross$geno[[i]]$prob, "error.prob")
                    mf <- attr(cross$geno[[i]]$prob, "map.function")

                    if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                        stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
                    else stpw <- "fixed"
                    if("map" %in% names(attributes(cross$geno[[i]]$prob)))
                        tmap <- attr(cross$geno[[i]]$prob,"map")
                    else
                        tmap <- create.map(cross$geno[[i]]$map, stp, oe, stpw)

                    temp <- calc.pairprob(subset(cross,chr=thechr[i]),stp,oe,err,mf,tmap,
                                          assumeCondIndep=assumeCondIndep)
                }

                # revise pair probilities for X chromosome
                if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft")))
                    temp <- reviseXdata(type, "full", sexpgm, pairprob=temp,
                                        cross.attr=attributes(cross))

                if(verbose>2) cat("  --Done.\n")

                thisz <- .C("R_scantwopermhk_1chr",
                            as.integer(n.ind),
                            as.integer(n.pos[i]),
                            as.integer(n.gen[i]),
                            as.double(cross$geno[[i]]$prob),
                            as.double(temp),
                            as.double(ac),
                            as.integer(n.ac),
                            as.double(pheno),
                            as.integer(n.perm),
                            as.integer(permindex),
                            as.double(weights),
                            result=as.double(rep(0,n.perm*6)),
                            as.integer(n.col2drop),
                            as.integer(col2drop),
                            PACKAGE="qtl")
                result[[k]] <- -matrix(thisz$result, nrow=n.perm)
                result[[k]][,c(1,4,6)] <- result[[k]][,c(1,4,6)] +
                    ifelse(chr_type[i]=="X", nllik0X, nllik0)
            } # end same chromosome
            else {
                thisz <- .C("R_scantwopermhk_2chr",
                            as.integer(n.ind),
                            as.integer(n.pos[i]),
                            as.integer(n.pos[j]),
                            as.integer(n.gen[i]),
                            as.integer(n.gen[j]),
                            as.double(cross$geno[[i]]$prob),
                            as.double(cross$geno[[j]]$prob),
                            as.double(ac),
                            as.integer(n.ac),
                            as.double(pheno),
                            as.integer(n.perm),
                            as.integer(permindex),
                            as.double(weights),
                            result=as.double(rep(0,n.perm*6)),
                            PACKAGE="qtl")
                result[[k]] <- -matrix(thisz$result, nrow=n.perm)
                result[[k]][,c(1,4,6)] <- result[[k]][,c(1,4,6)] +
                    ifelse(chr_type[i]=="X" || chr_type[j]=="X", nllik0X, nllik0)
            } # end diff chr
        } # end loop chr 2
    } # end loop chr 1

    result <- apply( array(unlist(result[1:k]), dim=c(n.perm, 6, k)), 1:2, max, na.rm=TRUE)
    result <- as.list(as.data.frame(result))
    phename <- phenames(cross)[pheno.col]
    result <- lapply(result, function(a) { a <- as.matrix(a); colnames(a) <- phename; a })
    names(result) <- c("full", "fv1", "int", "add", "av1", "one")
    class(result) <- c("scantwoperm", "list")
    result
}

# force stratified permutation test for X chr
force_sexstrata <-
function(cross, perm.strata)
{
    type <- crosstype(cross)
    if(type!="bc" && type!="f2")
        return(perm.strata)

    Xcovar <- scanoneXnull(type, getsex(cross), attributes(cross))$sexpgmcovar

    if(!is.null(Xcovar)) {
        pastedcovar <- apply(Xcovar, 1, paste, collapse="")
        upastedcovar <- unique(pastedcovar)
        sexstrata <- match(pastedcovar, upastedcovar)
        if(is.null(perm.strata)) perm.strata <- sexstrata
        else {
            u <- unique(perm.strata)
            perm.strata <- match(perm.strata, u)
            m <- max(perm.strata)
            perm.strata <- perm.strata + m*(sexstrata-1)
        }
    }

    perm.strata
}

######################################################################
#
# scantwo.R
#
# copyright (c) 2001-2020, Karl W Broman and Hao Wu
# last modified Feb, 2020
# first written Nov, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Hao Wu (The Jackson Lab) wrote the initial code for the imputation
# method.
#
# Part of the R/qtl package
# Contains: scantwo, scantwo.perm, scantwo.perm.engine
#
######################################################################

######################################################################
#
# scantwo: Do 2-dimensional genome scan with a two-QTL model,
#          calculating joint LOD scores and LOD scores testing
#          epistasis.
#
######################################################################

scantwo <-
    function(cross, chr, pheno.col=1,
             model=c("normal","binary"),
             method=c("em","imp","hk","mr","mr-imp","mr-argmax"),
             addcovar=NULL, intcovar=NULL, weights=NULL,
             use=c("all.obs", "complete.obs"),
             incl.markers=FALSE, clean.output=FALSE, clean.nmar=1,
             clean.distance=0,
             maxit=4000, tol=1e-4, verbose=TRUE, n.perm,
             perm.Xsp=FALSE, perm.strata=NULL, assumeCondIndep=FALSE,
             batchsize=250, n.cluster=1)
{
    if(batchsize < 1) stop("batchsize must be >= 1.")
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    method <- match.arg(method)
    model <- match.arg(model)
    use <- match.arg(use)

    # pull out chromosomes to be scanned
    if(missing(chr)) chr1 <- chr2 <- chr <- names(cross$geno)
    else {
        thechr <- names(cross$geno)
        if(is.list(chr)) {
            # special case: do just specific pairs (each of chr1 vs each of chr2, except when chr2 < chr1)
            chr1 <- matchchr(chr[[1]], thechr)
            chr2 <- matchchr(chr[[2]], thechr)
        }
        else
            chr1 <- chr2 <- matchchr(chr, thechr)
    }

    cross <- subset(cross, unique(c(chr1, chr2)))
    thechr <- names(cross$geno)
    nchr1 <- match(chr1, thechr)
    nchr2 <- match(chr2, thechr)
    if(!any(sapply(nchr1, function(a,b) any(a <= b), nchr2)))
        stop("Need some of first chr to be <= some of second chr")

    if(missing(n.perm)) n.perm <- 0

    if((method=="hk" || method=="em") && !assumeCondIndep) { # if reduce2grid was used, for assumeCondIndep
        # if reduced2grid, force assumeCondIndep=TRUE
        reduced2grid <- attr(cross$geno[[1]]$prob, "reduced2grid")
        if(!is.null(reduced2grid) && reduced2grid) {
            assumeCondIndep <- TRUE
            warning("Using assumeCondIndep=TRUE, since probabilities reduced to grid")
        }
    }

    # in RIL, treat X chromomse like an autosome
    chr_type <- sapply(cross$geno, chrtype)
    crosstype <- crosstype(cross)
    if(any(chr_type=="X") && (crosstype == "risib" || crosstype == "riself"))
        for(i in which(chr_type=="X")) class(cross$geno[[i]]) <- "A"

    # X-chr-specific perms (actually A:A, A:X, and X:X specific)
    if(perm.Xsp && n.perm > 0 && !(all(chr_type=="X") || all(chr_type=="A"))) { # no need for x-chr-specific perms
        if(length(chr1) != length(chr2) || any(chr1 != chr2))
            stop("With perm.Xsp=TRUE, chr can't be a list")

        chr.names <- chrnames(cross)
        chrL <- sapply(cross$geno, function(a) diff(range(a$map)))
        AL <- sum(chrL[chr_type=="A"])
        XL <- sum(chrL[chr_type=="X"])
        AAL <- AL*AL/2
        XXL <- XL*XL/2
        AXL <- AL*XL
        n.permAA <- n.perm
        n.permXX <- ceiling(n.perm * AAL/XXL)
        n.permAX <- ceiling(n.perm * AAL/AXL)

        # names of autosomes and X chr
        Achr <- chr.names[chr_type=="A"]
        Xchr <- chr.names[chr_type=="X"]

        # X chr covariates
        Xnull <- scanoneXnull(crosstype(cross), getsex(cross), attributes(cross))
        Xcovar <- Xnull$sexpgmcovar

        if(verbose) message("Running ", n.permAA, " A:A permutations")
        AAresult <- scantwo(cross, chr=Achr, pheno.col=pheno.col,
                            model=model, method=method,
                            addcovar=cbind(addcovar, Xcovar), intcovar=intcovar,
                            weights=weights, use=use,
                            incl.markers=incl.markers, clean.output=clean.output,
                            clean.nmar=clean.nmar, clean.distance=clean.distance,
                            maxit=maxit, tol=tol, verbose=verbose, n.perm=n.permAA,
                            perm.Xsp=FALSE, perm.strata=perm.strata,
                            assumeCondIndep=assumeCondIndep,
                            batchsize=batchsize, n.cluster=n.cluster)

        if(verbose) message("Running ", n.permXX, " X:X permutations")
        XXresult <- scantwo(cross, chr=Xchr, pheno.col=pheno.col,
                            model=model, method=method,
                            addcovar=addcovar, intcovar=intcovar,
                            weights=weights, use=use,
                            incl.markers=incl.markers, clean.output=clean.output,
                            clean.nmar=clean.nmar, clean.distance=clean.distance,
                            maxit=maxit, tol=tol, verbose=verbose, n.perm=n.permXX,
                            perm.Xsp=FALSE, perm.strata=perm.strata,
                            assumeCondIndep=assumeCondIndep,
                            batchsize=batchsize, n.cluster=n.cluster)

        if(verbose) message("Running ", n.permAX, " A:X permutations")
        AXresult <- scantwo(cross, chr=list(Achr, Xchr), pheno.col=pheno.col,
                            model=model, method=method,
                            addcovar=addcovar, intcovar=intcovar,
                            weights=weights, use=use,
                            incl.markers=incl.markers, clean.output=clean.output,
                            clean.nmar=clean.nmar, clean.distance=clean.distance,
                            maxit=maxit, tol=tol, verbose=verbose, n.perm=n.permAX,
                            perm.Xsp=FALSE, perm.strata=perm.strata,
                            assumeCondIndep=assumeCondIndep,
                            batchsize=batchsize, n.cluster=n.cluster)

        result <- list(AA=AAresult, AX=AXresult, XX=XXresult)
        attr(result, "L") <- c(A=AL, X=XL)
        attr(result, "LL") <- c(AA=AAL, AX=AXL, XX=XXL)
        names(chr_type) <- chr.names
        attr(result, "chrtype") <- chr_type
        class(result) <- c("scantwoperm", "list")
        return(result)
    }

    if(!missing(n.perm) && n.perm > 0 && n.cluster > 1) {
        cat(" -Running permutations via a cluster of", n.cluster, "nodes.\n")
        updateParallelRNG(n.cluster)
        n.perm <- ceiling(n.perm/n.cluster)
        scantwoPermInParallel <- function(n.perm, cross, chr, pheno.col, model, method, addcovar, intcovar, weights,
                                          incl.markers, clean.output, clean.nmar, clean.distance, maxit, tol,
                                          perm.strata, assumeCondIndep, batchsize)
            scantwo(cross=cross, chr=chr, pheno.col=pheno.col, model=model, method=method, addcovar=addcovar,
                    intcovar=intcovar, weights=weights, incl.markers=incl.markers, clean.output=clean.output,
                    clean.nmar=clean.nmar, clean.distance=clean.distance, maxit=maxit, tol=tol, perm.strata=perm.strata,
                    assumeCondIndep=assumeCondIndep, batchsize=batchsize, n.cluster=0, verbose=FALSE, n.perm=n.perm)

        if(Sys.info()[1] == "Windows") { # Windows doesn't support mclapply, but it's faster if available
            cl <- makeCluster(n.cluster)
            on.exit(stopCluster(cl))
            operm <- clusterApply(cl, rep(n.perm, n.cluster), scantwoPermInParallel, cross=cross, chr=chr, pheno.col=pheno.col,
                                  model=model, method=method, addcovar=addcovar, intcovar=intcovar,
                                  weights=weights, incl.markers=incl.markers, clean.output=clean.output,
                                  clean.nmar=clean.nmar, clean.distance=clean.distance,
                                  maxit=maxit, tol=tol, perm.strata=perm.strata,
                                  assumeCondIndep=assumeCondIndep, batchsize=batchsize)
        }
        else {
            operm <- mclapply(rep(n.perm, n.cluster), scantwoPermInParallel, cross=cross, chr=chr, pheno.col=pheno.col,
                              model=model, method=method, addcovar=addcovar, intcovar=intcovar,
                              weights=weights, incl.markers=incl.markers, clean.output=clean.output,
                              clean.nmar=clean.nmar, clean.distance=clean.distance,
                              maxit=maxit, tol=tol, perm.strata=perm.strata,
                              assumeCondIndep=assumeCondIndep, batchsize=batchsize, mc.cores=n.cluster)
        }
        for(j in 2:length(operm))
            operm[[1]] <- c(operm[[1]], operm[[j]])
        return(operm[[1]])
    }

    # check perm.strat
    if(!missing(perm.strata) && !is.null(perm.strata)) {
        if(length(perm.strata) != nind(cross))
            stop("perm.strata, if given, must have length = nind(cross) [", nind(cross), "]")
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    origcross <- cross

    fullmap <- pull.map(cross)

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(any(is.na(num))) {
            if(sum(is.na(num)) > 1)
                stop("Couldn't identify phenotypes ", paste(paste("\"", pheno.col[is.na(num)], "\"", sep=""),
                                                            collapse=" "))
            else
                stop("Couldn't identify phenotype \"", pheno.col[is.na(num)], "\"")
        }
        pheno.col <- num
    }

    if(any(pheno.col < 1 | pheno.col > nphe(cross)))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    # if stepwidth="variable" or stepwidth=="max" when calling calc.genoprob or sim.geno,
    # we force incl.markers=TRUE; I assume it is the same for all chromosomes
    stepwidth.var <- FALSE
    if(method=="em" || method=="hk") {
        if("stepwidth" %in% names(attributes(cross$geno[[1]]$prob)) &&
           attr(cross$geno[[1]]$prob, "stepwidth") != "fixed") {
            stepwidth.var <- TRUE
            incl.markers <- TRUE
        }
    }
    else if(method=="imp") {
        if("stepwidth" %in% names(attributes(cross$geno[[1]]$draws)) &&
           attr(cross$geno[[1]]$draws, "stepwidth") != "fixed") {
            stepwidth.var <- TRUE
            incl.markers <- TRUE
        }
    }

    if(length(pheno.col)==1 && n.perm>=0) use <- "complete.obs"

    if(n.perm >= 0) {    # not in the midst of a permutation test
        # If use="all.obs", check whether there are individuals missing some
        # phenotypes but not others.  If not, act like "complete.obs".
        if(use=="all.obs" && length(pheno.col) > 1) {
            n.phe <- length(pheno.col)
            temp <- apply(cross$pheno[,pheno.col], 1, function(a) sum(is.na(a)))
            if(all(temp==0 | temp==n.phe)) use <- "complete.obs"
        }

        # If use="complete.obs", drop individuals with any missing phenotypes
        if(use=="complete.obs") {
            temp <- checkcovar(cross, pheno.col, addcovar, intcovar,
                               perm.strata, ind.noqtl=NULL, weights, TRUE)
            cross <- temp[[1]]
            pheno <- temp[[2]]
            addcovar <- temp[[3]]
            intcovar <- temp[[4]]
            n.addcovar <- temp[[5]]
            n.intcovar <- temp[[6]]
            perm.strata <- temp[[7]]
            weights <- temp[[9]]
        }
    }

    # use all observations; not in a permutation test; different phenotypes have different sets of missing values
    #   -> want to do in batches, but need to define batches by the pattern of missing data
    if(n.perm <= 0 && use=="all.obs" && length(pheno.col) > 1 && (method=="hk" || method=="imp")) {
        # drop individuals with missing covariates
        cross$pheno <- cbind(cross$pheno, rep(1, nind(cross)))
        temp <- checkcovar(cross, nphe(cross), addcovar, intcovar,
                           perm.strata, ind.noqtl=NULL, weights, TRUE)
        cross <- temp[[1]]
        pheno <- cross$pheno[,pheno.col, drop=FALSE]
        addcovar <- temp[[3]]
        intcovar <- temp[[4]]
        n.addcovar <- temp[[5]]
        n.intcovar <- temp[[6]]
        perm.strata <- temp[[7]]
        weights <- temp[[9]]

        # determine the batches (defined by the pattern of missing data)
        patterns <- apply(pheno, 2, function(a) paste(!is.na(a), collapse=":"))
        upat <- unique(patterns)
        m <- match(patterns, upat)
        batches <- vector("list", length(upat))
        upat <- lapply(strsplit(upat, ":"), function(a) as.logical(a))
        for(i in seq(along=batches)) batches[[i]] <- pheno.col[m==i]

        # run scanone for one batch at a time
        out <- NULL
        for(i in seq(along=batches)) {
            if(!is.null(addcovar)) {
                if(!is.matrix(addcovar)) addcovar <- as.matrix(addcovar)
                tempac <- addcovar[upat[[i]],,drop=FALSE]
            }
            else tempac <- addcovar
            if(!is.null(intcovar)) {
                if(!is.matrix(intcovar)) intcovar <- as.matrix(intcovar)
                tempic <- intcovar[upat[[i]],,drop=FALSE]
            }
            else tempic <- intcovar

            temp <- scantwo(subset(cross, ind=upat[[i]]), chr=chr, pheno.col=batches[[i]], model=model,
                            method=method, addcovar=tempac, intcovar=tempic,
                            weights=weights[upat[[i]]], use=use, incl.markers=incl.markers, clean.output=clean.output,
                            clean.nmar=clean.nmar, clean.distance=clean.distance,
                            maxit=maxit, tol=tol, verbose=verbose, n.perm=n.perm,
                            perm.strata=perm.strata[upat[[i]]], assumeCondIndep=assumeCondIndep,
                            batchsize=batchsize, n.cluster=n.cluster)
            if(is.null(out)) out <- temp
            else out <- cbind(out, temp)
        }

        # reorder LOD score columns and make sure that the names are correct
        dimnames(out$lod) <- list(NULL, NULL, colnames(cross$pheno)[unlist(batches)])
        out$lod <- out$lod[,,colnames(cross$pheno)[pheno.col]]
        dimnames(out$lod)[[3]] <- colnames(cross$pheno)[pheno.col]
        attr(out,"phenotypes") <- colnames(cross$pheno)[pheno.col]

        return(out)
    }

    # multiple phenotype for methods other than imp and hk
    if(length(pheno.col)>1 && n.perm <= 0 && (model != "normal" ||
                              (method!="imp" && method != "hk"))) {
        n.phe <- length(pheno.col)
        if(verbose) cat(" -Phenotype 1\n")
        output <- scantwo(cross, pheno.col=pheno.col[1], model=model,
                          method=method, addcovar=addcovar,
                          intcovar=intcovar, weights=weights, use=use,
                          incl.markers=incl.markers,
                          clean.output=clean.output, clean.nmar=clean.nmar,
                          clean.distance=clean.distance,
                          maxit=maxit, tol=tol, verbose=verbose, n.perm=n.perm,
                          perm.strata=perm.strata,
                          assumeCondIndep=assumeCondIndep,
                          batchsize=batchsize, n.cluster=0)
        temp <- array(dim=c(nrow(output$lod), ncol(output$lod), n.phe))
        temp[,,1] <- output$lod
        output$lod <- temp
        for(i in 2:n.phe) {
            if(verbose) cat(" -Phenotype ", i, "\n")
            temp <- scantwo(cross, pheno.col=pheno.col[i], model=model, method=method,
                            addcovar=addcovar, intcovar=intcovar, weights=weights,
                            use=use, incl.markers=incl.markers,
                            clean.output=clean.output, clean.nmar=clean.nmar,
                            clean.distance=clean.distance,
                            maxit=maxit, tol=tol, verbose=verbose, n.perm=n.perm,
                            perm.strata=perm.strata,
                            assumeCondIndep=assumeCondIndep,
                            batchsize=batchsize, n.cluster=0)
            output$lod[,,i] <- temp$lod
            if(!is.null(output$scanoneX))
                output$scanoneX <- cbind(output$scanoneX, temp$scanoneX)
        }
        attr(output,"fullmap") <- fullmap
        attr(output,"phenotypes") <- colnames(cross$pheno)[pheno.col]
        names(output$map)[2] <- "pos"
        dimnames(output$lod) <- list(NULL, NULL, colnames(cross$pheno)[pheno.col])
        return(output)
    }

    # if n.perm specified, do a permutation test
    if(n.perm>0) {
        return(scantwo.perm(cross, pheno.col=pheno.col, model=model, method=method,
                            addcovar=addcovar, intcovar=intcovar, weights=weights,
                            use=use, incl.markers=incl.markers, clean.output=clean.output,
                            clean.nmar=clean.nmar,
                            clean.distance=clean.distance,
                            maxit=maxit,
                            tol=tol, verbose=verbose, n.perm=n.perm,
                            perm.strata=perm.strata, assumeCondIndep=assumeCondIndep,
                            batchsize=batchsize, chr=chr))
    }


    if(n.perm < 0) { # in the midst of permutations
        if(use=="all.obs") {
            temp <- checkcovar(cross, pheno.col, addcovar, intcovar,
                               perm.strata, ind.noqtl=NULL, weights, n.perm==-1)
            cross <- temp[[1]]
            pheno <- temp[[2]]
            addcovar <- temp[[3]]
            intcovar <- temp[[4]]
            n.addcovar <- temp[[5]]
            n.intcovar <- temp[[6]]
            perm.strata <- temp[[7]]
            weights <- temp[[9]]
        }
        else {
            pheno <- as.matrix(cross$pheno[,pheno.col])
            if(is.null(addcovar)) n.addcovar <- 0
            else n.addcovar <- ncol(addcovar)
            if(is.null(intcovar)) n.intcovar <- 0
            else n.intcovar <- ncol(intcovar)
        }
    }

    n.chr <- nchr(cross)
    n.ind <- nind(cross)
    n.phe <- length(pheno.col)
    type <- crosstype(cross)
    chr_type <- sapply(cross$geno,chrtype)
    is.bcs <- FALSE
    if(type == "bcsft") {
        cross.scheme <- attr(cross, "scheme")
        is.bcs <- (cross.scheme[2] == 0)
    }

    if(any(chr_type=="X")) {
        sexpgm <- getsex(cross)

        addcovarX <- revisecovar(sexpgm,addcovar)
        if(!is.null(addcovar) && (nd <- attr(addcovarX, "n.dropped")) > 0 && n.perm > -2)
            warning("Dropped ", nd, " additive covariates on X chromosome.")
        if(length(addcovarX)==0) {
            n.acX <- 0
            addcovarX <- NULL
        }
        else n.acX <- ncol(addcovarX)

        intcovarX <- revisecovar(sexpgm,intcovar)
        if(!is.null(intcovar) && (nd <- attr(intcovarX, "n.dropped")) > 0 && n.perm > -2)
            warning("Dropped ", nd, " interactive covariates on X chromosome.")
        if(length(intcovarX)==0) {
            n.icX <- 0
            intcovarX <- NULL
        }
        else n.icX <- ncol(intcovarX)
    }

    if(model=="binary") {
        if(method != "em" && method != "hk") {
            method <- "em"
            if(n.perm > -2) warning("Only EM algorithm and Haley-Knott regression coded for binary traits; using EM")
        }
        if(!is.null(weights)) {
            weights <- NULL
            if(n.perm > -2) warning("weights ignored for binary traits.")
        }

        u <- unique(pheno)
        if(any(u!=0 & u!=1))
            stop("Phenotypes must be either 0 or 1.")
    }

    if(n.perm == 0) { # not in the midst of permutations
        if(method=="mr-argmax")
            cross <- fill.geno(cross,method="argmax")
        if(method=="mr-imp")
            cross <- fill.geno(cross,method="imp")
    }

    # weights of individuals
    if(model == "normal") {
        if(is.null(weights))
            weights <- rep(1, nind(cross))
        if(length(weights) != nind(cross))
            stop("weights should either be NULL or a vector of length n.ind")
        if(any(weights <= 0))
            stop("weights should be entirely positive")
        weights <- sqrt(weights)
    }

    if(verbose) cat(" --Running scanone\n")
    temp <- scanone(cross, pheno.col=pheno.col, model=model, method=method,
                    addcovar=addcovar, intcovar=intcovar, weights=weights,
                    use=use, maxit=maxit, tol=tol, verbose=FALSE)
    out.scanone <- temp[,-(1:2),drop=FALSE]
    if(verbose) cat(" --Running scantwo\n")

    if(method=="mr" || method=="mr-imp" || method=="mr-argmax") { # marker regression
        # number of genotypes on each chromosome,
        #     combine the genetic maps for all chromosomes
        map <- unlist(pull.map(cross))
        names(map) <- unlist(lapply(pull.map(cross),names))
        n.pos <- nmar(cross)
        gmap <- data.frame(chr=rep(names(cross$geno),n.pos),
                           pos=map,
                           eq.spacing=rep(1,sum(n.pos)),
                           xchr=rep(sapply(cross$geno,chrtype)=="X",nmar(cross)), stringsAsFactors=TRUE)

        # number of possible genotypes for each chromosome
        n.gen <- 1:n.chr
        for(i in 1:n.chr) {
            gen.names <- getgenonames(type, chr_type[i], "full", sexpgm, attributes(cross))
            n.gen[i] <- length(gen.names)
        }
    } # end of if(method=="mr")

    else { # all methods except "mr"
        # check for genotype probabilities or simulated genotypes
        steps <- rep(0,n.chr) # step length on each chromosome
        if(method=="imp") {
            for(i in 1:n.chr) {
                if(!("draws" %in% names(cross$geno[[i]]))) {
                    # need to run sim.geno
                    if(n.perm > -2) warning("First running sim.geno.")
                    cross <- sim.geno(cross)
                }
                steps[i] <- attr(cross$geno[[i]]$draws,"step")
            }

            # make sure all chromosomes have the same number of imputations
            n.draws <- sapply(cross$geno, function(a) dim(a$draws)[3])
            if(length(unique(n.draws)) > 1) {
                if(n.perm > -2) warning("Re-running sim.geno to have a fixed number of imputations.")
                cross <- sim.geno(cross, n.draws=max(n.draws),
                                  step=attr(cross$geno[[1]]$draws,"step"),
                                  off.end=attr(cross$geno[[1]]$draws,"off.end"))
            }
            n.draws <- max(n.draws)
        }
        else { # H-K or EM
            for(i in 1:n.chr) {
                if(!("prob" %in% names(cross$geno[[i]]))) {
                    # need to run calc.genoprob
                    if(n.perm > -2) warning("First running calc.genoprob.")
                    cross <- calc.genoprob(cross)
                }
                steps[i] <- attr(cross$geno[[i]]$prob,"step")
            }
        }

        # number of genotypes on each chromosome,
        #     construct the genetic map for all chromosomes
        #     and possibly drop marker positions
        gmap <- NULL
        n.pos <- n.gen <- rep(0,n.chr)
        keep.pos <- vector("list",n.chr)
        some.dropped <- rep(FALSE,n.chr)

        for(i in 1:n.chr) {
            gen.names <- getgenonames(type, chr_type[i], "full", sexpgm, attributes(cross))
            n.gen[i] <- length(gen.names)

            # construct the genetic map for this chromesome
            if(method=="imp") {
                if("map" %in% names(attributes(cross$geno[[i]]$draws)))
                    map <- attr(cross$geno[[i]]$draws,"map")
                else {
                    stp <- attr(cross$geno[[i]]$draws, "step")
                    oe <- attr(cross$geno[[i]]$draws, "off.end")

                    if("stepwidth" %in% names(attributes(cross$geno[[i]]$draws)))
                        stpw <- attr(cross$geno[[i]]$draws, "stepwidth")
                    else stpw <- "fixed"
                    map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
                }
            }
            else {
                if("map" %in% names(attributes(cross$geno[[i]]$prob)))
                    map <- attr(cross$geno[[i]]$prob,"map")
                else {
                    stp <- attr(cross$geno[[i]]$prob, "step")
                    oe <- attr(cross$geno[[i]]$prob, "off.end")

                    if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                        stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
                    else stpw <- "fixed"
                    map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
                }
            }

            if(is.matrix(map)) map <- map[1,] # in case of sex-specific map

            w <- names(map)
            o <- grep("^loc-*[0-9]+",w)

            if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
                w[o] <- paste("c",names(cross$geno)[i],".",w[o],sep="")
            map <- cbind(chr=factor(rep(names(cross$geno)[i],length(map)),levels=names(cross$geno)),
                         pos=as.data.frame(as.numeric(map), stringsAsFactors=TRUE) )
            rownames(map) <- w

            # equally spaced positions
            if(steps[i]==0 || stepwidth.var)  # just use markers
                eq.sp.pos <- rep(1,nrow(map))
            else {
                eq.sp.pos <- seq(min(map[,2]),max(map[,2]),by=steps[i])
                wh.eq.sp <- match(eq.sp.pos,map[,2])
                if(any(is.na(wh.eq.sp))) { # this shouldn't happen
                    if(n.perm > -2) warning("Possible error in determining the equally spaced positions.")
                    wh.eq.sp <- wh.eq.sp[!is.na(wh.eq.sp)]
                }
                eq.sp.pos <- rep(0,nrow(map))
                eq.sp.pos[wh.eq.sp] <- 1
            }
            if(!incl.markers && any(eq.sp.pos==0)) {
                keep.pos[[i]] <- (seq(along=eq.sp.pos))[eq.sp.pos==1]
                map <- map[eq.sp.pos==1,]
                eq.sp.pos <- eq.sp.pos[eq.sp.pos==1]
                some.dropped[i] <- TRUE # indicates some positions were dropped
            }
            else keep.pos[[i]] <- seq(along=eq.sp.pos)
            gmap <- rbind(gmap, cbind(map,eq.spacing=eq.sp.pos,
                                      xchr=(chrtype(cross$geno[[i]])=="X")))
            n.pos[i] <- length(keep.pos[[i]])

            # Revise X chromosome genotype probabilities or imputations
            if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft"))) {
                if(method=="imp")
                    cross$geno[[i]]$draws <-
                        reviseXdata(type, "full", sexpgm, draws=cross$geno[[i]]$draws,
                                    cross.attr=attributes(cross))
                else if(method=="hk" || method=="em") {
                    oldXchr <- subset(cross, chr=thechr[i])
                    cross$geno[[i]]$prob <-
                        reviseXdata(type, "full", sexpgm, prob=cross$geno[[i]]$prob,
                                    cross.attr=attributes(cross))
                }
                else
                    cross$geno[[i]]$data <-
                        reviseXdata(type, "full", sexpgm, geno=cross$geno[[i]]$data,
                                    cross.attr=attributes(cross))
            }

        } # end loop over chromosomes
    } # end of if/else for method="mr" vs other

    # columns in result matrix for each chromosome
    wh.col <- vector("list",n.chr)
    first.pos <- cumsum(c(1,n.pos))
    for(i in 1:n.chr)
        wh.col[[i]] <- seq(first.pos[i],by=1,length=n.pos[i])

    # initialize the results matrix
    if(n.phe > 1)
        results <- array(NA,dim=c(sum(n.pos),sum(n.pos), n.phe))
    else
        results <- matrix(NA,sum(n.pos),sum(n.pos))

    # do the 2-dimensional genome scan
    do.nllik0 <- TRUE
    for(i in nchr1) { # loop over the 1st chromosome
        for(j in nchr2) { # loop over the 2nd chromosome
            if(j < i) next

            if(chr_type[i]=="X" || chr_type[j]=="X") {
                ac <- addcovarX
                n.ac <- n.acX
                ic <- intcovarX
                n.ic <- n.icX
            }
            else {
                ac <- addcovar
                n.ac <- n.addcovar
                ic <- intcovar
                n.ic <- n.intcovar
            }
            if(i==j && chr_type[i]=="X") {
                col2drop <- dropXcol(type, sexpgm, attributes(cross))
                n.col2drop <- sum(col2drop)
                n.col2drop.addmodel <- sum(col2drop[1:(2*n.gen[i]-1)])
            }
            else {
                col2drop <- rep(0,n.gen[i]*n.gen[j])
                n.col2drop <- 0
            }

            # print the current working pair
            if(verbose) cat(" (", names(cross$geno)[i], ",",
                            names(cross$geno)[j],")\n",sep="")

            if(method=="imp") {
                if(n.phe > batchsize) {
                    firstcol <- 1
                    z <- NULL
                    while(firstcol <= n.phe) {
                        thiscol <- firstcol + 0:(batchsize-1)
                        thiscol <- thiscol[thiscol <= n.phe]
                        thisz <- .C("R_scantwo_imp",
                                    as.integer(n.ind),
                                    as.integer(i==j),
                                    as.integer(n.pos[i]),
                                    as.integer(n.pos[j]),
                                    as.integer(n.gen[i]),
                                    as.integer(n.gen[j]),
                                    as.integer(n.draws),
                                    as.integer(cross$geno[[i]]$draws[,keep.pos[[i]],]),
                                    as.integer(cross$geno[[j]]$draws[,keep.pos[[j]],]),
                                    as.double(ac),
                                    as.integer(n.ac),
                                    as.double(ic),
                                    as.integer(n.ic),
                                    as.double(pheno[,thiscol]),
                                    as.integer(length(thiscol)),
                                    as.double(weights),
                                    result=as.double(rep(0,2*n.pos[i]*n.pos[j]*length(thiscol))),
                                    as.integer(n.col2drop),
                                    as.integer(col2drop),
                                    PACKAGE="qtl")
                        firstcol <- firstcol + batchsize
                        if(is.null(z))
                            z <- array(NA, dim=c(n.pos[i], n.pos[j], 2*n.phe))
                        thisz$result <- array(thisz$result, dim=c(n.pos[i], n.pos[j], 2*length(thiscol)))
                        z[,,thiscol] <- thisz$result[,,1:length(thiscol)]
                        z[,,n.phe+thiscol] <- thisz$result[,,length(thiscol)+1:length(thiscol)]
                    }
                }
                else {
                    z <- .C("R_scantwo_imp",
                            as.integer(n.ind),
                            as.integer(i==j),
                            as.integer(n.pos[i]),
                            as.integer(n.pos[j]),
                            as.integer(n.gen[i]),
                            as.integer(n.gen[j]),
                            as.integer(n.draws),
                            as.integer(cross$geno[[i]]$draws[,keep.pos[[i]],]),
                            as.integer(cross$geno[[j]]$draws[,keep.pos[[j]],]),
                            as.double(ac),
                            as.integer(n.ac),
                            as.double(ic),
                            as.integer(n.ic),
                            as.double(pheno),
                            as.integer(n.phe),
                            as.double(weights),
                            result=as.double(rep(0,2*n.pos[i]*n.pos[j]*n.phe)),
                            as.integer(n.col2drop),
                            as.integer(col2drop),
                            PACKAGE="qtl")
                    z <- array(z$result,dim=c(n.pos[i], n.pos[j], 2*n.phe)) # rearrange the result
                }

                # do this just once: do null model and get neg log10 likelihood
                if(do.nllik0) {
                    do.nllik0 <- FALSE
                    if(n.ac > 0)
                        resid0 <- lm(pheno ~ ac, weights=weights^2)$resid
                    else
                        resid0 <- lm(pheno ~ 1, weights=weights^2)$resid
                    sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
                    nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
                }

                # update the final result matrix
                if(i == j) { # on same chromosome
                    if(n.phe > 1)
                        results[wh.col[[i]],wh.col[[j]],] <- z[,,1:n.phe]
                    else
                        results[wh.col[[i]],wh.col[[j]]] <- z[,,1]

                }
                else { # on different chromosomes
                    if(n.phe > 1) {
                        # full lod
                        results[wh.col[[i]],wh.col[[j]],] <- z[,,1:n.phe]
                        # epistasis lod - need to reshape the matrix
                        results[wh.col[[j]],wh.col[[i]],] <- array(z[,,1:n.phe+n.phe],
                                                                   c(n.pos[j],n.pos[i], n.phe))
                    }
                    else { # only one phenotype, result is a matrix
                        # full lod
                        results[wh.col[[i]],wh.col[[j]]] <- z[,,1]
                        # epistasis - need to reshape the matrix
                        results[wh.col[[j]],wh.col[[i]]] <- matrix(z[,,2],n.pos[j],n.pos[i])
                    }
                }
            }
            else if(model=="normal" && (method=="hk" || method=="em")) {
                if(do.nllik0) { # first time! do null model and get neg log10 likelihood
                    do.nllik0 <- FALSE
                    if(n.ac > 0)
                        resid0 <- lm(pheno ~ ac, weights=weights^2)$resid
                    else
                        resid0 <- lm(pheno ~ 1, weights=weights^2)$resid
                    if(method=="hk") {
                        if(n.phe == 1)
                            nllik0 <- (n.ind/2)*log10(sum((resid0*weights)^2))
                        else # multiple phenotypes
                            nllik0 <- apply(resid0, 2, function(x)
                                            (n.ind/2)*log10(sum((x*weights)^2)))
                    }
                    else {
                        sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
                        nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
                    }
                }

                if(i==j) { # same chromosome


                    if(verbose>1) cat("  --Calculating joint probs.\n")

                    if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft"))) {
                        # calculate joint genotype probabilities for all pairs of positions
                        stp <- attr(oldXchr$geno[[1]]$prob, "step")
                        oe <- attr(oldXchr$geno[[1]]$prob, "off.end")
                        err <- attr(oldXchr$geno[[1]]$prob, "error.prob")
                        mf <- attr(oldXchr$geno[[1]]$prob, "map.function")

                        if("stepwidth" %in% names(attributes(oldXchr$geno[[1]]$prob)))
                            stpw <- attr(oldXchr$geno[[1]]$prob, "stepwidth")
                        else stpw <- "fixed"
                        if("map" %in% names(attributes(oldXchr$geno[[1]]$prob)))
                            tmap <- attr(oldXchr$geno[[1]]$prob,"map")
                        else
                            tmap <- create.map(oldXchr$geno[[1]]$map, stp, oe, stpw)

                        temp <- calc.pairprob(oldXchr,stp,oe,err,mf,tmap,
                                              assumeCondIndep=assumeCondIndep)
                    }
                    else {
                        # calculate joint genotype probabilities for all pairs of positions
                        stp <- attr(cross$geno[[i]]$prob, "step")
                        oe <- attr(cross$geno[[i]]$prob, "off.end")
                        err <- attr(cross$geno[[i]]$prob, "error.prob")
                        mf <- attr(cross$geno[[i]]$prob, "map.function")

                        if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                            stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
                        else stpw <- "fixed"
                        if("map" %in% names(attributes(cross$geno[[i]]$prob)))
                            tmap <- attr(cross$geno[[i]]$prob,"map")
                        else
                            tmap <- create.map(cross$geno[[i]]$map, stp, oe, stpw)

                        temp <- calc.pairprob(subset(cross,chr=thechr[i]),stp,oe,err,mf,tmap,
                                              assumeCondIndep=assumeCondIndep)
                    }

                    # pull out positions from genotype probs
                    if(some.dropped[i]) {
                        # figure out pos'ns corresponding to columns of temp
                        nc <- ncol(cross$geno[[i]]$prob)
                        ind <- matrix(rep(1:nc,nc),ncol=nc)
                        w <- lower.tri(ind)
                        ind <- cbind(first=t(ind)[w],second=ind[w])

                        # which part to keep
                        keep <- apply(ind,1,function(a,b) all(a %in% b),
                                      keep.pos[[i]])
                        temp <- temp[,keep,,]
                    }

                    # revise pair probilities for X chromosome
                    if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft")))
                        temp <- reviseXdata(type, "full", sexpgm, pairprob=temp,
                                            cross.attr=attributes(cross))

                    if(verbose>1) cat("  --Done.\n")

                    if(method=="hk") {
                        if(n.phe > batchsize) {
                            firstcol <- 1
                            z <- NULL
                            while(firstcol <= n.phe) {
                                thiscol <- firstcol + 0:(batchsize-1)
                                thiscol <- thiscol[thiscol <= n.phe]

                                thisz <- .C("R_scantwo_1chr_hk",
                                            as.integer(n.ind),
                                            as.integer(n.pos[i]),
                                            as.integer(n.gen[i]),
                                            as.double(cross$geno[[i]]$prob[,keep.pos[[i]],]),
                                            as.double(temp),
                                            as.double(ac),
                                            as.integer(n.ac),
                                            as.double(ic),
                                            as.integer(n.ic),
                                            as.double(pheno[,thiscol]),
                                            as.integer(length(thiscol)),
                                            as.double(weights),
                                            result=as.double(rep(0,n.pos[i]^2*length(thiscol))),
                                            as.integer(n.col2drop),
                                            as.integer(col2drop),
                                            PACKAGE="qtl")

                                firstcol <- firstcol + batchsize
                                if(is.null(z)) {
                                    z <- thisz
                                    z$result <- array(NA, dim=c(n.pos[i], n.pos[i], n.phe))
                                }
                                z$result[,,thiscol] <- array(thisz$result, dim=c(n.pos[i], n.pos[i], length(thiscol)))
                            }
                        }
                        else {
                            z <- .C("R_scantwo_1chr_hk",
                                    as.integer(n.ind),
                                    as.integer(n.pos[i]),
                                    as.integer(n.gen[i]),
                                    as.double(cross$geno[[i]]$prob[,keep.pos[[i]],]),
                                    as.double(temp),
                                    as.double(ac),
                                    as.integer(n.ac),
                                    as.double(ic),
                                    as.integer(n.ic),
                                    as.double(pheno),
                                    as.integer(n.phe),
                                    as.double(weights),
                                    result=as.double(rep(0,n.pos[i]^2*n.phe)),
                                    as.integer(n.col2drop),
                                    as.integer(col2drop),
                                    PACKAGE="qtl")
                        }

                        ## fill results matrix
                        if(n.phe == 1)
                            results[wh.col[[i]],wh.col[[i]]] <-
                                matrix(z$result,ncol=n.pos[i])
                        else # multiple phenotypes
                            results[wh.col[[i]],wh.col[[i]],] <-
                                array(z$result,c(n.pos[i],n.pos[i], n.phe))
                        z <- 0
                    }
                    else {
                        z <- .C("R_scantwo_1chr_em",
                                as.integer(n.ind),
                                as.integer(n.pos[i]),
                                as.integer(n.gen[i]),
                                as.double(temp),
                                as.double(ac),
                                as.integer(n.ac),
                                as.double(ic),
                                as.integer(n.ic),
                                as.double(pheno),
                                as.double(weights),
                                result=as.double(rep(0,n.pos[i]^2)),
                                as.integer(maxit),
                                as.double(tol),
                                as.integer(verbose),
                                as.integer(n.col2drop),
                                as.integer(col2drop),
                                PACKAGE="qtl")
                        # re-organize results
                        results[wh.col[[i]],wh.col[[i]]] <-
                            matrix(z$result,ncol=n.pos[i])
                    }

                    z <- 0
                    temp <- 0 # remove the joint genotype probabilities
                } # end same chromosome

                else {
                    if(method=="hk") {
                        if(n.phe > batchsize) {
                            firstcol <- 1
                            z <- NULL
                            while(firstcol <= n.phe) {
                                thiscol <- firstcol + 0:(batchsize-1)
                                thiscol <- thiscol[thiscol <= n.phe]

                                thisz <- .C("R_scantwo_2chr_hk",
                                            as.integer(n.ind),
                                            as.integer(n.pos[i]),
                                            as.integer(n.pos[j]),
                                            as.integer(n.gen[i]),
                                            as.integer(n.gen[j]),
                                            as.double(cross$geno[[i]]$prob[,keep.pos[[i]],]),
                                            as.double(cross$geno[[j]]$prob[,keep.pos[[j]],]),
                                            as.double(ac),
                                            as.integer(n.ac),
                                            as.double(ic),
                                            as.integer(n.ic),
                                            as.double(pheno[,thiscol]),
                                            as.integer(length(thiscol)),
                                            as.double(weights),
                                            full=as.double(rep(0,n.pos[i]*n.pos[j]*length(thiscol))),
                                            int=as.double(rep(0,n.pos[i]*n.pos[j]*length(thiscol))),
                                            PACKAGE="qtl")

                                firstcol <- firstcol + batchsize
                                if(is.null(z)) {
                                    z <- thisz
                                    z$full <- z$int <- array(NA, dim=c(n.pos[j], n.pos[i], n.phe))
                                }
                                z$full[,,thiscol] <- array(thisz$full, dim=c(n.pos[j], n.pos[i], length(thiscol)))
                                z$int[,,thiscol] <- array(thisz$int, dim=c(n.pos[j], n.pos[i], length(thiscol)))
                            }
                        }
                        else {
                            z <- .C("R_scantwo_2chr_hk",
                                    as.integer(n.ind),
                                    as.integer(n.pos[i]),
                                    as.integer(n.pos[j]),
                                    as.integer(n.gen[i]),
                                    as.integer(n.gen[j]),
                                    as.double(cross$geno[[i]]$prob[,keep.pos[[i]],]),
                                    as.double(cross$geno[[j]]$prob[,keep.pos[[j]],]),
                                    as.double(ac),
                                    as.integer(n.ac),
                                    as.double(ic),
                                    as.integer(n.ic),
                                    as.double(pheno),
                                    as.integer(n.phe),
                                    as.double(weights),
                                    full=as.double(rep(0,n.pos[i]*n.pos[j]*n.phe)),
                                    int=as.double(rep(0,n.pos[i]*n.pos[j]*n.phe)),
                                    PACKAGE="qtl")
                        }
                        ## reorgnize results
                        if(n.phe == 1) {
                            results[wh.col[[j]],wh.col[[i]]] <-
                                matrix(z$full,ncol=n.pos[j])
                            results[wh.col[[i]],wh.col[[j]]] <-
                                matrix(z$int,ncol=n.pos[j])
                        }
                        else { # multiple phenotypes
                            results[wh.col[[j]],wh.col[[i]],] <-
                                array(z$full,c(n.pos[j], n.pos[i], n.phe))
                            results[wh.col[[i]],wh.col[[j]],] <-
                                array(z$int,c(n.pos[j], n.pos[i], n.phe))
                        }
                        z <- 0
                    }
                    else  {
                        z <- .C("R_scantwo_2chr_em",
                                as.integer(n.ind),
                                as.integer(n.pos[i]),
                                as.integer(n.pos[j]),
                                as.integer(n.gen[i]),
                                as.integer(n.gen[j]),
                                as.double(cross$geno[[i]]$prob[,keep.pos[[i]],]),
                                as.double(cross$geno[[j]]$prob[,keep.pos[[j]],]),
                                as.double(ac),
                                as.integer(n.ac),
                                as.double(ic),
                                as.integer(n.ic),
                                as.double(pheno),
                                as.double(weights),
                                full=as.double(rep(0,n.pos[i]*n.pos[j])),
                                int=as.double(rep(0,n.pos[i]*n.pos[j])),
                                as.integer(maxit),
                                as.double(tol),
                                as.integer(verbose),
                                PACKAGE="qtl")

                        results[wh.col[[j]],wh.col[[i]]] <-
                            t(matrix(z$full,ncol=n.pos[j]))
                        results[wh.col[[i]],wh.col[[j]]] <-
                            matrix(z$int,ncol=n.pos[j])
                        z <- 0
                    }
                } # end different chromosome
            }
            else if(model=="binary") {
                if(do.nllik0) { # first time! do null model and get neg log10 likelihood
                    do.nllik0 <- FALSE
                    if(n.ac > 0)
                        nullfit <- glm(pheno ~ ac, family=binomial(link="logit"))
                    else
                        nullfit <- glm(pheno ~ 1, family=binomial(link="logit"))
                    fitted <- nullfit$fitted
                    nullcoef <- nullfit$coef
                    nllik0 <- -sum(pheno*log10(fitted) + (1-pheno)*log10(1-fitted))
                    if(verbose > 1) cat("null log lik: ", nllik0, "\n")
                }

                if(i==j) { # same chromosome

                    start <- c(rep(nullcoef[1],n.gen[i]),rep(0,n.gen[i]-1),
                               nullcoef[-1],rep(0,n.gen[i]*n.gen[i]+
                                                (n.gen[i]*(n.gen[i]-1)*n.ic)))

                    if(n.col2drop)
                        start <- c(start[!col2drop], rep(0,sum(col2drop)))

                    if(verbose>1) cat("  --Calculating joint probs.\n")

                    if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft"))) {
                        # calculate joint genotype probabilities for all pairs of positions
                        stp <- attr(oldXchr$geno[[1]]$prob, "step")
                        oe <- attr(oldXchr$geno[[1]]$prob, "off.end")
                        err <- attr(oldXchr$geno[[1]]$prob, "error.prob")
                        mf <- attr(oldXchr$geno[[1]]$prob, "map.function")

                        if("stepwidth" %in% names(attributes(oldXchr$geno[[1]]$prob)))
                            stpw <- attr(oldXchr$geno[[1]]$prob, "stepwidth")
                        else stpw <- "fixed"
                        if("map" %in% names(attributes(oldXchr$geno[[1]]$prob)))
                            tmap <- attr(oldXchr$geno[[1]]$prob,"map")
                        else
                            tmap <- create.map(oldXchr$geno[[1]]$map, stp, oe, stpw)

                        temp <- calc.pairprob(oldXchr,stp,oe,err,mf,tmap,
                                              assumeCondIndep=assumeCondIndep)
                    }
                    else {
                        # calculate joint genotype probabilities for all pairs of positions
                        stp <- attr(cross$geno[[i]]$prob, "step")
                        oe <- attr(cross$geno[[i]]$prob, "off.end")
                        err <- attr(cross$geno[[i]]$prob, "error.prob")
                        mf <- attr(cross$geno[[i]]$prob, "map.function")

                        if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                            stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
                        else stpw <- "fixed"
                        if("map" %in% names(attributes(cross$geno[[i]]$prob)))
                            tmap <- attr(cross$geno[[i]]$prob,"map")
                        else
                            tmap <- create.map(cross$geno[[i]]$map, stp, oe, stpw)

                        temp <- calc.pairprob(subset(cross,chr=thechr[i]),stp,oe,err,mf,tmap,
                                              assumeCondIndep=assumeCondIndep)
                    }

                    # pull out positions from genotype probs
                    if(some.dropped[i]) {
                        # figure out pos'ns corresponding to columns of temp
                        nc <- ncol(cross$geno[[i]]$prob)
                        ind <- matrix(rep(1:nc,nc),ncol=nc)
                        w <- lower.tri(ind)
                        ind <- cbind(first=t(ind)[w],second=ind[w])

                        # which part to keep
                        keep <- apply(ind,1,function(a,b) all(a %in% b),
                                      keep.pos[[i]])
                        temp <- temp[,keep,,]
                    }

                    # revise pair probilities for X chromosome
                    if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft")))
                        temp <- reviseXdata(type, "full", sexpgm, pairprob=temp,
                                            cross.attr=attributes(cross))

                    if(verbose>1) cat("  --Done.\n")

                    if(method=="em")
                        z <- .C("R_scantwo_1chr_binary_em",
                                as.integer(n.ind),
                                as.integer(n.pos[i]),
                                as.integer(n.gen[i]),
                                as.double(temp),
                                as.double(ac),
                                as.integer(n.ac),
                                as.double(ic),
                                as.integer(n.ic),
                                as.integer(pheno),
                                as.double(start),
                                result=as.double(rep(0,n.pos[i]^2)),
                                as.integer(maxit),
                                as.double(tol),
                                as.integer(verbose),
                                as.integer(n.col2drop),
                                as.integer(col2drop),
                                PACKAGE="qtl")
                    else # h-k regression
                        z <- .C("R_scantwo_1chr_binary_hk",
                                as.integer(n.ind),
                                as.integer(n.pos[i]),
                                as.integer(n.gen[i]),
                                as.double(cross$geno[[i]]$prob[,keep.pos[[i]],]),
                                as.double(temp),
                                as.double(ac),
                                as.integer(n.ac),
                                as.double(ic),
                                as.integer(n.ic),
                                as.double(pheno),
                                result=as.double(rep(0,n.pos[i]^2)),
                                as.integer(n.col2drop),
                                as.integer(col2drop),
                                as.double(tol),
                                as.integer(maxit),
                                as.integer(verbose),
                                PACKAGE="qtl")


                    # re-organize results
                    results[wh.col[[i]],wh.col[[i]]] <-
                        matrix(z$result,ncol=n.pos[i])

                    z <- 0
                    temp <- 0 # remove the joint genotype probabilities
                } # end same chromosome
                else {
                    start <- c(rep(nullcoef[1],n.gen[i]),rep(0,n.gen[j]-1),
                               nullcoef[-1],rep(0,n.gen[i]*n.gen[j]+
                                                (n.gen[i]*(n.gen[j]-1)*n.intcovar)));

                    if(method=="em")
                        z <- .C("R_scantwo_2chr_binary_em",
                                as.integer(n.ind),
                                as.integer(n.pos[i]),
                                as.integer(n.pos[j]),
                                as.integer(n.gen[i]),
                                as.integer(n.gen[j]),
                                as.double(cross$geno[[i]]$prob[,keep.pos[[i]],]),
                                as.double(cross$geno[[j]]$prob[,keep.pos[[j]],]),
                                as.double(ac),
                                as.integer(n.ac),
                                as.double(ic),
                                as.integer(n.ic),
                                as.integer(pheno),
                                as.double(start),
                                full=as.double(rep(0,n.pos[i]*n.pos[j])),
                                int=as.double(rep(0,n.pos[i]*n.pos[j])),
                                as.integer(maxit),
                                as.double(tol),
                                as.integer(verbose),
                                PACKAGE="qtl")
                    else # h-k regression
                        z <- .C("R_scantwo_2chr_binary_hk",
                                as.integer(n.ind),
                                as.integer(n.pos[i]),
                                as.integer(n.pos[j]),
                                as.integer(n.gen[i]),
                                as.integer(n.gen[j]),
                                as.double(cross$geno[[i]]$prob[,keep.pos[[i]],]),
                                as.double(cross$geno[[j]]$prob[,keep.pos[[j]],]),
                                as.double(ac),
                                as.integer(n.ac),
                                as.double(ic),
                                as.integer(n.ic),
                                as.double(pheno),
                                full=as.double(rep(0,n.pos[i]*n.pos[j])),
                                int=as.double(rep(0,n.pos[i]*n.pos[j])),
                                as.double(tol),
                                as.integer(maxit),
                                as.integer(verbose),
                                PACKAGE="qtl")

                    results[wh.col[[j]],wh.col[[i]]] <-
                        t(matrix(z$full,ncol=n.pos[j]))
                    results[wh.col[[i]],wh.col[[j]]] <-
                        matrix(z$int,ncol=n.pos[j])
                    z <- 0
                } # end same chromosome
            }
            else { # marker regression
                # replace missing and partially informative genotypes with 0's
                datai <- cross$geno[[i]]$data
                datai[is.na(datai)] <- 0

                if(type=="f2" || (type=="bcsft" & !is.bcs)) datai[datai>3] <- 0
                else if(type=="4way") datai[datai>4] <- 0

                if(chr_type[i]=="X" && (type %in% c("bc","f2","bcsft")))
                    datai <- reviseXdata(type, "full", sexpgm, geno=datai,
                                         cross.attr=attributes(cross))

                if(i==j) { # same chromosome

                    z <- .C("R_scantwo_1chr_mr",
                            as.integer(n.ind),
                            as.integer(n.pos[i]),
                            as.integer(n.gen[i]),
                            as.integer(datai),
                            as.double(ac),
                            as.integer(n.ac),
                            as.double(ic),
                            as.integer(n.ic),
                            as.double(pheno),
                            as.double(weights),
                            result=as.double(rep(0,n.pos[i]^2)),
                            as.integer(n.col2drop),
                            as.integer(col2drop),
                            PACKAGE="qtl")

                    # re-organize results
                    results[wh.col[[i]],wh.col[[i]]] <-
                        matrix(z$result,ncol=n.pos[i])
                    z <- 0
                } # end same chromosome
                else {

                    # replace missing and partially informative genotypes with 0's
                    dataj <- cross$geno[[j]]$data
                    dataj[is.na(dataj)] <- 0
                    if(type=="f2" || (type=="bcsft" && !is.bcs)) dataj[dataj>3] <- 0
                    else if(type=="4way") dataj[dataj>4] <- 0

                    if(chr_type[j]=="X" && (type %in% c("bc","f2","bcsft")))
                        dataj <- reviseXdata(type, "full", sexpgm, geno=dataj,
                                             cross.attr=attributes(cross))

                    z <- .C("R_scantwo_2chr_mr",
                            as.integer(n.ind),
                            as.integer(n.pos[i]),
                            as.integer(n.pos[j]),
                            as.integer(n.gen[i]),
                            as.integer(n.gen[j]),
                            as.integer(datai),
                            as.integer(dataj),
                            as.double(ac),
                            as.integer(n.ac),
                            as.double(ic),
                            as.integer(n.ic),
                            as.double(pheno),
                            as.double(weights),
                            full=as.double(rep(0,n.pos[i]*n.pos[j])),
                            int=as.double(rep(0,n.pos[i]*n.pos[j])),
                            PACKAGE="qtl")

                    results[wh.col[[j]],wh.col[[i]]] <-
                        t(matrix(z$full,ncol=n.pos[j]))
                    results[wh.col[[i]],wh.col[[j]]] <-
                        matrix(z$int,ncol=n.pos[j])

                    z <- 0
                }
            }

        } # end loop over second chr
    } # end loop over first chromosome

    # subtract null neg log lik from lower tri
    if(method=="em") {
        offdiag <- lower.tri(results) | upper.tri(results)
        results[offdiag] <- nllik0 - results[offdiag]
    }
    else if(method=="hk") {
        if(n.phe == 1) {
            offdiag <- lower.tri(results) | upper.tri(results)
            results[offdiag] <- nllik0 - results[offdiag]
        }
        else { # multiple phenotypes
            offdiag <- lower.tri(results[,,1]) | upper.tri(results[,,1])
            for(itmp in 1:n.phe) {
                # I'm doing a loop here. I should put null model back to C function
                results[,,itmp][offdiag] <- nllik0[itmp] - results[,,itmp][offdiag]
            }
        }
    }

    # If the X chromosome was included, need to do an adjustment...
    scanoneX <- NULL
    if(any(gmap[,4])) { # the X chromosome was included

        # determine which covariates belong in null hypothesis
        temp <- scanoneXnull(type, sexpgm, cross.attr=attributes(cross))
        adjustX <- temp$adjustX
        parX0 <- temp$parX0
        sexpgmcovar <- temp$sexpgmcovar

        if(adjustX) {
            if(method=="mr" && any(is.na(pull.geno(cross))))
                if(n.perm > -2) warning("Scantwo with the X chr doesn't work quite right when method=\"mr\"\n",
                                        "    when there is missing genotype data.")

            if(model=="binary") {
                if(n.ac > 0) {
                    nullfitX <- glm(pheno ~ ac+sexpgmcovar, family=binomial(link="logit"))
                    parX0 <- lm(pheno ~ ac+sexpgmcovar)$rank
                }
                else {
                    nullfitX <- glm(pheno ~ sexpgmcovar, family=binomial(link="logit"))
                    parX0 <- ncol(sexpgmcovar)
                }
                fittedX <- nullfitX$fitted
                nullcoefX <- nullfitX$coef
                nllikX <- -sum(pheno*log10(fittedX) + (1-pheno)*log10(1-fittedX))
                if(verbose > 1) cat("X chr null log lik: ", nllikX, "\n")
            }
            else {
                if(n.ac > 0) {
                    outX <- lm(pheno ~ ac+sexpgmcovar, weights=weights^2)
                    residX <- outX$resid
                    parX0 <- outX$rank
                }
                else
                    residX <- lm(pheno ~ sexpgmcovar, weights=weights^2)$resid

                if(method=="hk") {
                    if(n.phe==1)
                        nllikX <- (n.ind/2)*log10(sum((residX*weights)^2))
                    else
                        nllikX <- (n.ind/2)*apply(residX, 2, function(a,b)
                                                  log10(sum((a*b)^2)), weights)
                }
                else {
                    if(method=="imp" || method=="mr") {
                        if(n.ac > 0) {
                            out0 <- lm(pheno ~ ac, weights=weights^2)
                            resid0 <- out0$resid
                        }
                        else {
                            out0 <- lm(pheno ~ 1, weights=weights^2)
                            resid0 <- out0$resid
                        }

                        if(n.phe > 1) {
                            sig0 <- sqrt(apply(resid0, 2, function(a,b) sum((a*b)^2),weights)/n.ind)
                            nllik0 <- sig0
                            for(i in seq(along=nllik0))
                                nllik0[i] <- -sum(dnorm(resid0[,i],0,sig0[i]/weights,log=TRUE))/log(10)
                        }
                        else {
                            sig0 <- sqrt(sum((resid0*weights)^2)/n.ind)
                            nllik0 <- -sum(dnorm(resid0,0,sig0/weights,log=TRUE))/log(10)
                        }
                    }

                    if(n.phe > 1) {
                        sigX <- sqrt(apply(residX, 2, function(a,b) sum((a*b)^2),weights)/n.ind)
                        nllikX <- sigX
                        for(i in seq(along=nllikX))
                            nllikX[i] <- -sum(dnorm(residX[,i],0,sigX[i]/weights,log=TRUE))/log(10)
                    }
                    else {
                        sigX <- sqrt(sum((residX*weights)^2)/n.ind)
                        nllikX <- -sum(dnorm(residX,0,sigX/weights,log=TRUE))/log(10)
                    }
                }
            }

            if(n.phe > 1)
                wh <- ((gmap[row(results[,,1]),4] | gmap[col(results[,,1]),4]) &
                       (lower.tri(results[,,1]) | upper.tri(results[,,1])))
            else
                wh <- ((gmap[row(results),4] | gmap[col(results),4]) &
                       (lower.tri(results) | upper.tri(results)))

            if(n.phe > 1) {
                for(i in 1:n.phe)
                    results[,,i][wh] <- results[,,i][wh] + nllikX[i] - nllik0[i]
            }
            else
                results[wh] <- results[wh] + nllikX - nllik0

            notxchr <- names(cross$geno)[sapply(cross$geno,chrtype)!="X"]
            if(length(notxchr) > 0) {
                if(verbose) cat(" --Running scanone with special X chr covariates\n")
                temp <- scanone(subset(cross,chr=notxchr),
                                pheno.col=pheno.col,
                                model=model, method=method,
                                addcovar=cbind(ac,sexpgmcovar),
                                intcovar=ic, weights=weights, use=use,
                                maxit=maxit, tol=tol, verbose=FALSE)

                scanoneX <- temp[,-(1:2),drop=FALSE]
                scanoneX <- rbind(scanoneX,
                                  out.scanone[rownames(gmap),,drop=FALSE][gmap[,4],,drop=FALSE])

                scanoneX <- scanoneX[rownames(gmap),,drop=FALSE]
            }
            else {
                scanoneX <- out.scanone[rownames(gmap),,drop=FALSE][gmap[,4],,drop=FALSE]
                scanoneX <- scanoneX[rownames(gmap),,drop=FALSE]
            }
        }

    }

    #  if(any(is.na(results)) && n.perm > -2)
    #    warning("Some LOD scores NA")
    #  if(any(!is.na(results) & results < 0) && n.perm > -2)
    #    warning("Some LOD scores < 0")
    #  if(any(!is.na(results) & (results == Inf | results == -Inf)) && n.perm > -2)
    #    warning("Some LOD scores = Inf or -Inf")

    if(!is.null(scanoneX)) scanoneX <- as.matrix(scanoneX)

    # output has 2 fields, lod and map
    out <- list(lod=results,map=gmap,scanoneX=scanoneX)

    # fill in scanone result
    if(n.phe == 1)
        diag(out$lod) <- out.scanone[rownames(out$map),]
    else {
        for(iphe in 1:n.phe) {
            if(nrow(out$lod)==1)
                out$lod[1,1,iphe] <- out.scanone[rownames(out$map),iphe]
            else
                diag(out$lod[,,iphe]) <- out.scanone[rownames(out$map),iphe]
        }
    }

    attr(out,"method") <- method
    attr(out,"type") <- type
    attr(out, "fullmap") <- fullmap
    class(out) <- "scantwo"

    if(clean.output) # remove NA, 0 out positions between markers
        out <- clean(out, clean.nmar, clean.distance)

    attr(out, "phenotypes") <- colnames(pheno)
    if(length(colnames(pheno)) > 1)
        dimnames(out$lod) <- list(NULL, NULL, colnames(pheno))
    names(out$map)[2] <- "pos"
    out
}

######################################################################
#
# scantwo.perm: Permutation test of scantwo
#
######################################################################

scantwo.perm <-
    function(cross, pheno.col=1, model=c("normal","binary"),
             method=c("em","imp","hk","mr","mr-imp","mr-argmax"),
             addcovar=NULL, intcovar=NULL, weights=NULL,
             use=c("all.obs", "complete.obs"),
             incl.markers=FALSE, clean.output=FALSE,
             clean.nmar=1, clean.distance=0,
             maxit=4000, tol=1e-4, verbose=FALSE,
             n.perm=1000, perm.strata,
             assumeCondIndep=FALSE, batchsize=250, chr)
{
    method <- match.arg(method)
    model <- match.arg(model)
    use <- match.arg(use)
    if(missing(chr)) chr <- names(chr$geno)

    scantwo.perm.engine(n.perm, cross=cross, pheno.col=pheno.col,
                        model=model, method=method, addcovar=addcovar,
                        intcovar=intcovar, weights=weights, use=use,
                        incl.markers=incl.markers, clean.output=clean.output,
                        clean.nmar=clean.nmar,
                        clean.distance=clean.distance,
                        maxit=maxit, tol=tol, verbose=verbose,
                        perm.strata=perm.strata,
                        assumeCondIndep=assumeCondIndep, batchsize=batchsize,
                        chr=chr)

}


######################################################################
#
# Engine function to scantwo permutation
#
######################################################################
scantwo.perm.engine <-
    function(n.perm, cross, pheno.col, model,
             method, addcovar, intcovar, weights, use,
             incl.markers, clean.output, clean.nmar=1, clean.distance=0,
             maxit, tol, verbose, perm.strata,
             assumeCondIndep=FALSE, batchsize=250, chr)
{
    if(missing(chr)) chr <- names(chr$geno)

    ## local variables
    n.phe <- length(pheno.col)
    n.ind <- dim(cross$pheno)[1]
    pn <- colnames(cross$pheno)[pheno.col]

    ## if there's only one phenotype, no covariate, and method is imp or hk,
    ## generate permuted phenotype as a matrix and do permutation
    ## as multiple phenotypes
    ## we also need one sex and one direction, or that the
    ##     stratification is within those groups
    batch.mode <- FALSE
    if( (n.phe==1) && ((method=="imp") || (method=="hk")) &&
       model=="normal" &&
       is.null(addcovar) && is.null(intcovar) ) {
        chr_type <- sapply(cross$geno, chrtype)
        sexpgm <- getsex(cross)
        sex <- sexpgm$sex
        pgm <- sexpgm$pgm
        if(all(chr_type=="A"))
            batch.mode <- TRUE
        else if((is.null(sex) || length(unique(sex))==1) &&
                (is.null(pgm) || length(unique(pgm))==1))
            batch.mode <- TRUE
        else if(!is.null(perm.strata)) {
            sp <- paste(sex, pgm, sep=":")
            tab <- table(sp, perm.strata)
            if(all(apply(tab, 2, function(a) sum(a != 0))==1))
                batch.mode <- TRUE
        }
    }

    if(batch.mode) {
        if(verbose)
            cat("Doing permutation in batch mode ...\n")
        ord <- matrix(0, n.ind, n.perm)
        if(!is.null(perm.strata)) {  # stratified permutation test
            if(length(perm.strata) != n.ind)
                stop("perm.strata must be NULL or have length nind(cross).")
            u <- unique(perm.strata)
            theindex <- 1:n.ind
            if(length(u)==n.ind)
                stop("All elements of perm.strata are unique, so there will be no real permutation.")
            if(length(u)==1)
                warning("Just one unique element in perm.strata, so the perms are not stratified.")

            for(iperm in 1:n.perm) {
                for(j in u) {
                    wh <- perm.strata==j
                    if(sum(wh)==1) ord[wh,iperm] <- theindex[wh]
                    else ord[wh,iperm] <- sample(theindex[wh])
                }
            }
        }
        else {
            for(iperm in 1:n.perm)
                ord[,iperm] <- sample(n.ind)
        }

        cross$pheno <- cbind(matrix(cross$pheno[,pheno.col][ord], nrow=n.ind), cross$pheno)

        pheno.col <- 1:n.perm

        if(is.list(chr)) {
            chr1 <- chr[[1]]
            chr2 <- chr[[2]]
        }
        else chr1 <- chr2 <- chr

        thechr <- names(cross$geno)
        nchr1 <- match(chr1, thechr)
        nchr2 <- match(chr2, thechr)

        perm.result <- NULL
        for(i in nchr1) {
            for(j in nchr2) {
                if(j < i) next

                tem <- scantwo(cross, pheno.col=pheno.col, model=model, method=method,
                               addcovar=addcovar, intcovar=intcovar, weights=weights,
                               use=use, incl.markers=incl.markers,
                               clean.output=clean.output, clean.nmar=clean.nmar,
                               clean.distance=clean.distance,
                               maxit=maxit, tol=tol,verbose=FALSE, n.perm=-1,
                               perm.strata=perm.strata,
                               assumeCondIndep=assumeCondIndep,
                               batchsize=batchsize, n.cluster=0, chr=list(thechr[i],thechr[j]))

                if(clean.output) tem <- clean(tem, clean.nmar, clean.distance)

                ## find the maximum LOD on each permutation
                if(is.null(perm.result)) {
                    perm.result <- lapply(subrousummaryscantwo(tem,for.perm=TRUE), as.matrix)
                }
                else {
                    tem <- lapply(subrousummaryscantwo(tem,for.perm=TRUE), as.matrix)
                    for(k in seq(along=perm.result))
                        perm.result[[k]] <- as.matrix(apply(cbind(perm.result[[k]], tem[[k]]), 1, max, na.rm=TRUE))
                }


            }
        }
    }
    else { ## all other cases, do one permutation at a time
        if(method=="mr-imp") # save version with missing genotypes
            tempcross <- cross
        if(method=="mr-argmax") # impute genotypes
            cross <- fill.geno(cross,method="argmax")
        if(!is.null(addcovar)) addcovar <- as.matrix(addcovar)
        if(!is.null(intcovar)) intcovar <- as.matrix(intcovar)
        addcovarp <- addcovar
        intcovarp <- intcovar


        ## initialize result
        temp <- matrix(ncol=n.phe, nrow=n.perm)
        perm.result <- list("full"=temp,
                            "fv1"=temp,
                            "int"=temp,
                            "add"=temp,
                            "av1"=temp,
                            "one"=temp)

        if(is.list(chr)) {
            chr1 <- chr[[1]]
            chr2 <- chr[[2]]
        }
        else chr1 <- chr2 <- chr

        thechr <- names(cross$geno)
        nchr1 <- match(chr1, thechr)
        nchr2 <- match(chr2, thechr)

        ## permutation loop
        for(i in 1:n.perm) {
            if(verbose) cat("Permutation", i, "\n")
            ## impute genotypes for method "mr-imp"
            if(method=="mr-imp") cross <- fill.geno(tempcross)

            if(!is.null(perm.strata)) {  # stratified permutation test
                if(length(perm.strata) != n.ind)
                    stop("perm.strata must be NULL or have length nind(cross).")
                u <- unique(perm.strata)
                theindex <- 1:n.ind
                if(length(u)==n.ind)
                    stop("All elements of perm.strata are unique, so no real permutations.")
                if(length(u)==1 && i==1)
                    warning("Just one unique element in perm.strata, so the perms are not stratified.")

                o <- 1:n.ind
                for(j in u) {
                    wh <- perm.strata==j
                    if(sum(wh)>1) o[wh] <- sample(o[wh])
                }
            }
            else
                o <- sample(1:n.ind)

            cross$pheno <- cross$pheno[o,,drop=FALSE]
            if(!is.null(addcovar)) addcovarp <- addcovarp[o,,drop=FALSE]
            if(!is.null(intcovar)) intcovarp <- intcovarp[o,,drop=FALSE]

            temp <- NULL
            for(ii in nchr1) {
                for(jj in nchr2) {
                    if(jj < ii) next

                    tem <- scantwo(cross, pheno.col=pheno.col, model=model, method=method,
                                   addcovar=addcovar, intcovar=intcovar, weights=weights,
                                   use=use, incl.markers=incl.markers,
                                   clean.output=clean.output, clean.nmar=clean.nmar,
                                   clean.distance=clean.distance,
                                   maxit=maxit, tol=tol,verbose=FALSE, n.perm=-i,
                                   perm.strata=perm.strata,
                                   assumeCondIndep=assumeCondIndep,
                                   batchsize=batchsize, n.cluster=0, chr=list(thechr[ii],thechr[jj]))

                    if(clean.output) tem <- clean(tem, clean.nmar, clean.distance)

                    ## find the maximum LOD on each permutation
                    if(is.null(temp)) {
                        temp <- lapply(subrousummaryscantwo(tem,for.perm=TRUE), as.matrix)
                    }
                    else {
                        tem <- lapply(subrousummaryscantwo(tem,for.perm=TRUE), as.matrix)
                        for(k in seq(along=temp))
                            temp[[k]] <- as.matrix(apply(cbind(temp[[k]], tem[[k]]), 1, max, na.rm=TRUE))
                    }

                }
            }


            # maxima
            for(j in 1:6) perm.result[[j]][i,] <- temp[[j]]

        }
    }

    ## make result
    attr(perm.result,"method") <- method
    class(perm.result) <- c("scantwoperm", "list")

    ## add column names
    for(i in 1:length(perm.result))
        colnames(perm.result[[i]]) <- pn

    perm.result
}


# end of scantwo.R

######################################################################
#
# write.cross.R
#
# copyright (c) 2001-2019, Karl W Broman and Hao Wu
# last modified Dec, 2019
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: write.cross, write.cross.mm, write.cross.csv,
#           write.cross.gary, write.cross.tidy, fixX4write
#           [See qtlcart_io.R for write.cross.qtlcart]
#           [write.cross.qtab in write.cross.qtab.R]
#
######################################################################


######################################################################
#
# write.cross: Wrapper for the other write.cross functions
#
######################################################################

write.cross <-
    function(cross, format=c("csv", "csvr", "csvs", "csvsr", "mm", "qtlcart",
                    "gary", "qtab", "mapqtl", "tidy"),
             filestem="data", chr, digits=NULL, descr)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    format <- match.arg(format)
    if(!missing(chr)) cross <- subset(cross,chr=chr)

    # revise X data
    chr_type <- sapply(cross$geno,chrtype)
    crosstype <- crosstype(cross)
    if((crosstype=="bc" || crosstype=="f2") &&
       any(chr_type=="X")) {
        sexpgm <- getsex(cross)
        sex <- sexpgm$sex
        pgm <- sexpgm$pgm
        for(i in which(chr_type=="X"))
            cross$geno[[i]]$data <- fixX4write(cross$geno[[i]]$data,sex,pgm,crosstype)
    }

    if(crosstype == "bcsft") # convert BCsFt to intercross for writing
        class(cross) <- c("f2", "cross")

    if(format=="csv") write.cross.csv(cross,filestem,digits,FALSE,FALSE)
    else if(format=="csvr") write.cross.csv(cross,filestem,digits,TRUE,FALSE)
    else if(format=="csvs") write.cross.csv(cross,filestem,digits,FALSE,TRUE)
    else if(format=="csvsr") write.cross.csv(cross,filestem,digits,TRUE,TRUE)
    else if(format=="mm") write.cross.mm(cross,filestem,digits)
    else if(format=="qtlcart") write.cross.qtlcart(cross, filestem)
    else if(format=="gary") write.cross.gary(cross, digits)
    else if(format=="tidy") write.cross.tidy(cross, filestem, digits)
    else if(format=="qtab") {
        if(missing(descr)) descr <- paste(deparse(substitute(cross)), "from R/qtl")
        write.cross.qtab(cross, filestem, descr, verbose=FALSE)
    } else if(format == "mapqtl")
        write.cross.mq(cross, filestem, digits)
}




######################################################################
#
# write.cross.mm: Write data for an experimental cross in Mapmaker
#                 format
#
#           creates two files: "raw" file with geno & pheno data
#                              "prep" file with map information
#
######################################################################

write.cross.mm <-
    function(cross, filestem="data", digits=NULL)
{
    n.ind <- nind(cross)
    tot.mar <- totmar(cross)
    n.phe <- nphe(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)

    type <- crosstype(cross)
    if(type=="riself" || type=="risib" || type=="dh" || type=="haploid") type <- "bc"
    if(type != "f2" && type != "bc")
        stop("write.cross.mm only works for intercross, backcross, doubled haploid and RI data.")

    # write genotype and phenotype data
    file <- paste(filestem, ".raw", sep="")

    # write experiment type
    if(type == "f2")
        write("data type f2 intercross", file, append=FALSE)
    else
        write("data type f2 backcross", file, append=FALSE)

    # write numbers of progeny, markers and phenotypes
    write(paste(n.ind, tot.mar, n.phe), file, append=TRUE)

    # max length of marker name
    mlmn <- max(nchar(unlist(lapply(cross$geno,function(a) colnames(a$data)))))+1

    # write marker data
    for(i in 1:n.chr) {
        for(j in 1:n.mar[i]) {
            mn <- paste("*", colnames(cross$geno[[i]]$data)[j], sep="")
            if(nchar(mn) < mlmn)
                mn <- paste(mn,paste(rep(" ", mlmn-nchar(mn)),collapse=""),sep="")
            g <- cross$geno[[i]]$data[,j]

            x <- rep("", n.ind)
            x[is.na(g)] <- "-"
            x[!is.na(g) & g==1] <- "A"
            x[!is.na(g) & g==2] <- "H"
            if(type == "f2") {
                x[!is.na(g) & g==3] <- "B"
                x[!is.na(g) & g==4] <- "D"
                x[!is.na(g) & g==5] <- "C"
            }

            if(n.ind < 60)
                write(paste(mn, paste(x,collapse="")), file, append=TRUE)
            else {
                lo <- seq(1,n.ind-1,by=60)
                hi <- c(lo[-1]-1,n.ind)
                for(k in seq(along=lo)) {
                    if(k==1) write(paste(mn,paste(x[lo[k]:hi[k]],collapse="")),file,append=TRUE)
                    else write(paste(paste(rep(" ", mlmn),collapse=""),
                                     paste(x[lo[k]:hi[k]],collapse="")),file,append=TRUE)
                }
            }

        }
    } # end writing marker data

    # max length of phenotype name
    mlpn <- max(nchar(colnames(cross$pheno)))+1

    # write phenotypes
    for(i in 1:n.phe) {
        pn <- paste("*",colnames(cross$pheno)[i],sep="")
        if(nchar(pn) < mlpn)
            pn <- paste(pn, paste(rep(" ", mlpn-nchar(pn)),collapse=""),sep="")

        if(!is.factor(cross$pheno[,i])) {
            if(is.null(digits)) x <- as.character(cross$pheno[,i])
            else x <- as.character(round(cross$pheno[,i],digits))
        }
        else
            x <- as.character(cross$pheno[,i])
        x[is.na(x)] <- "-"

        if(n.ind < 10)
            write(paste(pn, paste(x,collapse="")), file, append=TRUE)
        else {
            lo <- seq(1,n.ind-1,by=10)
            hi <- c(lo[-1]-1,n.ind)
            for(k in seq(along=lo)) {
                if(k==1) write(paste(pn,paste(x[lo[k]:hi[k]],collapse=" ")),file,append=TRUE)
                else write(paste(paste(rep(" ", mlpn),collapse=""),
                                 paste(x[lo[k]:hi[k]],collapse=" ")),file,append=TRUE)
            }
        }
    }


    # make "prep" file with map information
    file <- paste(filestem, ".prep", sep="")

    for(i in 1:n.chr) {
        cname <- paste("chr", names(cross$geno)[i], sep="")
        line <- paste("make chromosome", cname)
        if(i==1) write(line, file, append=FALSE)
        else write(line, file, append=TRUE)

        mn <- names(cross$geno[[i]]$map)
        #    dis <- round(diff(cross$geno[[i]]$map),2)
        #    dis <- paste("=", dis, sep="")
        #    write(paste(paste("sequence", mn[1]), paste(dis,mn[-1],collapse=" ")),
        #          file, append=TRUE)
        write(paste(paste("sequence", mn[1]), paste(mn[-1],collapse=" ")),
              file, append=TRUE)

        write(paste("anchor", cname), file, append=TRUE)
        write(paste("framework", cname), file, append=TRUE)
    }

}

######################################################################
#
# write.cross.csv: Write data for an experimental cross in
#                  comma-delimited format (the same format as is read
#                  by read.cross.csv)
#
######################################################################

write.cross.csv <-
    function(cross, filestem="data", digits=NULL, rotate=FALSE, split=FALSE)
{
    type <- crosstype(cross)
    if(type != "f2" && type != "bc" && type != "riself" && type != "risib" && type != "dh" && type != "haploid")
        stop("write.cross.csv only works for intercross, backcross, RI, doubled haploid, and haploid data.")

    if(!split)
        file <- paste(filestem, ".csv", sep="")
    else {
        genfile <- paste(filestem, "_gen.csv", sep="")
        phefile <- paste(filestem, "_phe.csv", sep="")
    }

    if(split) { # split files; need individual IDs
        id <- getid(cross)
        if(is.null(id)) {
            cross$pheno$id <- 1:nind(cross)
            id <- getid(cross)
        }
        id.col <- which(colnames(cross$pheno)==attr(id,"phenam"))
    }

    n.ind <- nind(cross)
    tot.mar <- totmar(cross)
    n.phe <- nphe(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)

    geno <- matrix(ncol=tot.mar,nrow=n.ind)

    # allele codes to use
    if("alleles" %in% names(attributes(cross))) {
        alle <- attr(cross, "alleles")
        alleles <- c(paste(alle[1],alle[1],sep=""),
                     paste(alle[1],alle[2],sep=""),
                     paste(alle[2],alle[2],sep=""),
                     paste("not ", alle[2],alle[2],sep=""),
                     paste("not ", alle[1],alle[1],sep=""))
    }
    else alleles <- c("A","H","B","D","C")

    if(type=="dh" || type=="riself" || type=="risib") alleles[2:3] <- alleles[3:2]
    else if(type=="haploid") alleles <- alle

    firstmar <- 1
    for(i in 1:n.chr) {
        # replace allele numbers with
        geno[,firstmar:(firstmar+n.mar[i]-1)] <-
            alleles[match(cross$geno[[i]]$data,1:5)]
        firstmar <- firstmar + n.mar[i]
    }
    if(any(is.na(geno))) geno[is.na(geno)] <- "-"
    pheno <- cross$pheno
    for(i in 1:nphe(cross)) {
        if(is.factor(pheno[,i])) pheno[,i] <- as.character(pheno[,i])
        else if(is.numeric(pheno[,i])) {
            if(!is.null(digits)) pheno[,i] <- round(pheno[,i], digits)
            pheno[,i] <- as.character(pheno[,i])
        }
    }
    pheno <- matrix(unlist(pheno), nrow=n.ind)

    if(any(is.na(pheno))) pheno[is.na(pheno)] <- "-"
    thedata <- cbind(pheno,geno)
    colnames(thedata) <- c(colnames(cross$pheno),
                           unlist(lapply(cross$geno, function(a) colnames(a$data))))
    chr <- rep(names(cross$geno),n.mar)
    pos <- unlist(lapply(cross$geno,function(a) a$map))
    chr <- c(rep("",n.phe),chr)
    if(!is.null(digits))
        pos <- c(rep("",n.phe),as.character(round(pos,digits)))
    else
        pos <- c(rep("",n.phe),as.character(pos))

    # put it all together
    thenames <- colnames(thedata)
    thedata <- matrix(as.character(thedata), ncol=ncol(thedata))
    thedata <- rbind(thenames, chr, pos, thedata)

    if(!split) {
        if(!rotate)
            write.table(thedata, file, quote=FALSE, sep=",",
                        row.names=FALSE, col.names=FALSE)
        else
            write.table(t(thedata), file, quote=FALSE, sep=",",
                        row.names=FALSE, col.names=FALSE)
    }
    else { # split files: one for phenotypes and one for genotypes
        n.phe <- nphe(cross)
        phe <- thedata[-(2:3),1:n.phe]
        gen <- cbind(thedata[,id.col], thedata[,-(1:n.phe)])

        if(!rotate) {
            write.table(gen, genfile, quote=FALSE, sep=",",
                        row.names=FALSE, col.names=FALSE)
            write.table(phe, phefile, quote=FALSE, sep=",",
                        row.names=FALSE, col.names=FALSE)
        }
        else {
            write.table(t(gen), genfile, quote=FALSE, sep=",",
                        row.names=FALSE, col.names=FALSE)
            write.table(t(phe), phefile, quote=FALSE, sep=",",
                        row.names=FALSE, col.names=FALSE)
        }

    }


}


######################################################################
#
# write.cross.gary: Write data for an experimental cross in
# Gary's format. There will be 6 output files, they are:
#    chrid.dat - chromosome ids
#    markerpos.txt - marker position
#    mnames.txt - marker names
#    geno.data - genotypes
#    pheno.data - phenotypes
#    pnames.txt - phenotype names
#
######################################################################

write.cross.gary <-
    function(cross, digits=NULL)
{
    # local variables
    n.ind <- nind(cross)
    tot.mar <- totmar(cross)
    n.phe <- nphe(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)

    # chromosome ids
    chrid <- NULL
    for(i in 1:n.chr) {
        # the name for this chromosome
        chrname <- names(cross$geno[i])
        # convert to number (why?)
        #    if(chrname=="X") chrname <- 20
        #    else chrname <- as.numeric(chrname)
        chrid <- c(chrid, rep(chrname, n.mar[i]))
    }
    write.table(chrid, file="chrid.dat", quote=FALSE, row.names=FALSE,
                col.names=FALSE)

    # marker position file
    markpos <- NULL
    for(i in 1:n.chr)
        markpos <- c(markpos, cross$geno[[i]]$map)
    write.table(markpos, file="markerpos.txt", quote=FALSE, sep="\t",
                row.names=TRUE, col.names=FALSE)

    # marker names
    mnames <- names(markpos)
    write.table(mnames, file="mnames.txt", quote=FALSE, row.names=FALSE,
                col.names=FALSE)

    # genotype
    geno <- NULL
    for(i in 1:n.chr)
        geno <- cbind(geno, cross$geno[[i]]$data)
    # note that gary's format codes genotype from 0
    # and 9 is for NA
    geno <- geno - 1 # note NA will still be NA
    write.table(geno, file="geno.dat", quote=FALSE, row.names=FALSE,
                col.names=FALSE, sep="\t", na="9")

    # phenotype
    pheno <- cross$pheno
    for(i in 1:nphe(cross)) {
        if(is.factor(pheno[,i])) pheno[,i] <- as.character(pheno[,i])
        else if(is.numeric(pheno[,i])) {
            if(is.null(digits)) pheno[,i] <- as.character(pheno[,i])
            else pheno[,i] <- as.character(round(pheno[,i], digits))
        }
    }
    pheno <- matrix(unlist(pheno), nrow=nrow(pheno))

    write.table(pheno, file="pheno.dat", quote=FALSE, row.names=FALSE,
                col.names=FALSE, sep="\t", na="-999")
    # phenotype names
    write.table(names(cross$pheno), file="pnames.txt", quote=FALSE,
                row.names=FALSE, col.names=FALSE, sep="\t", na="-999")

}


######################################################################
#
# write.cross.tidy: Write data for an experimental cross in tidy
# format. There will be 3 output files, they are:
#    geno.csv
#    pheno.csv
#    map.csv
#
######################################################################

write.cross.tidy <-
    function(cross, filestem="data", digits=NULL)
{
    genfile <- paste(filestem, "_gen.csv", sep="")
    phefile <- paste(filestem, "_phe.csv", sep="")
    mapfile <- paste(filestem, "_map.csv", sep = "")

    type <- crosstype(cross)

    id <- getid(cross)

    if(is.null(id)) {
        cross$pheno$id <- 1:nind(cross)
        id <- getid(cross)
    }
    id.col <- which(colnames(cross$pheno) == attr(id,"phenam"))

    # allele codes to use
    if("alleles" %in% names(attributes(cross))) {
        alle <- attr(cross, "alleles")
        alleles <- c(paste(alle[1],alle[1],sep=""),
                     paste(alle[1],alle[2],sep=""),
                     paste(alle[2],alle[2],sep=""),
                     paste("not ", alle[2],alle[2],sep=""),
                     paste("not ", alle[1],alle[1],sep=""))
    }
    else alleles <- c("A","H","B","D","C")

    if (type %in% c("dh", "riself", "risib")) alleles[2:3] <- alleles[3:2]
    else if (type == "haploid") alleles <- alle

    geno <- pull.geno(cross)
    geno <- matrix(alleles[geno], ncol = nind(cross), byrow = TRUE,
                   dimnames = list(markernames(cross), make.names(id)))

    if(any(is.na(geno))) geno[is.na(geno)] <- "-"

    pheno <- cross$pheno[-id.col]
    for(i in 1:ncol(pheno)) {
        if(is.factor(pheno[,i])) pheno[,i] <- as.character(pheno[,i])
        else if(is.numeric(pheno[,i])) {
            if(!is.null(digits)) pheno[,i] <- round(pheno[,i], digits)
            pheno[,i] <- as.character(pheno[,i])
        }
    }

    pheno <- matrix(unlist(pheno), nrow=nind(cross),
                    dimnames = list(make.names(id), phenames(cross)[-id.col]))
    pheno <- t(pheno)

    if(any(is.na(pheno))) pheno[is.na(pheno)] <- "-"

    map <- pull.map(cross, as.table = TRUE)

    if(!is.null(digits))
        map$pos <- as.character(round(map$pos, digits))
    else
        map$pos <- as.character(map$pos)

    write.table(geno,  genfile, quote = FALSE, sep = ",", col.names = NA)
    write.table(pheno, phefile, quote = FALSE, sep = ",", col.names = NA)
    write.table(map,   mapfile, quote = FALSE, sep = ",", col.names = NA)
}


######################################################################
# fixX4write
######################################################################

fixX4write <-
    function(geno,sex,pgm,crosstype)
{
    # males
    if(!is.null(sex) & any(sex==1)) {
        temp <- geno[sex==1,,drop=FALSE]
        temp[temp==2] <- 3
        geno[sex==1,] <- temp
    }

    if(crosstype == "f2") {
        # females
        if(!is.null(pgm)) {
            if(!is.null(sex)) {
                if(any(pgm==1 & sex==0)) {
                    temp <- geno[sex==0 & pgm==1,,drop=FALSE]
                    temp[temp==1] <- 3
                    geno[sex==0 & pgm==1,] <- temp
                }
            }
            else { # assume all females
                if(any(pgm==1)) {
                    temp <- geno[pgm==1,,drop=FALSE]
                    temp[temp==1] <- 3
                    geno[pgm==1,] <- temp
                }
            }
        }

    }

    geno
}

# end of write.cross.R

######################################################################
#
# calc.pairprob.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Nov, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: calc.pairprob
#
######################################################################

######################################################################
#
# calc.pairprob: calculate joint genotype probabilities for all pairs
#                of putative QTLs, conditional on the observed marker
#                data
#
# This is an *internal* function, not to be called by the user.
#
# The input argument cross is assumed to have just one chromosome.
#
######################################################################

calc.pairprob <-
    function(cross, step=0, off.end=0, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             map, assumeCondIndep=FALSE)
{
    # which type of cross is this?
    type <- crosstype(cross)

    if(assumeCondIndep) { # assume conditional independence of QTL given markers
        if(!("prob" %in% names(cross$geno[[1]]))) {
            cross <- calc.genoprob(subset(cross, chr=1), step=step, off.end=off.end,
                                   error.prob=error.prob, map.function=map.function)
        }
        prob <- cross$geno[[1]]$prob
        n.ind <- dim(prob)[1]
        n.pos <- dim(prob)[2]
        n.gen <- dim(prob)[3]

        if(n.pos < 2) return(NULL)

        z <- .C("R_calc_pairprob_condindep",
                as.integer(n.ind),
                as.integer(n.pos),
                as.integer(n.gen),
                as.double(prob),
                pairprob=as.double(rep(0,n.ind*choose(n.pos, 2)*n.gen*n.gen)),
                PACKAGE="qtl")

        pairprob <- array(z$pairprob, dim=c(n.ind,n.pos*(n.pos-1)/2,n.gen,n.gen))

        return(pairprob)
    }

    if(step==0 && off.end > 0) step <- off.end*2

    # map function
    map.function <- match.arg(map.function)
    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }
    n.ind <- nind(cross)
    n.chr <- nchr(cross)

    # type of chromosome?
    chrtype <- chrtype(cross$geno[[1]])
    if(chrtype=="X") xchr <- TRUE
    else xchr <- FALSE

    if(type == "f2") {
        one.map <- TRUE
        if(!xchr) { # autosome
            cfunc <- "calc_pairprob_f2"
            n.gen <- 3
            gen.names <- getgenonames("f2", "A", cross.attr=attributes(cross))
        }
        else {                             # X chromsome
            cfunc <- "calc_pairprob_bc"
            n.gen <- 2
            gen.names <- c("g1","g2")
        }
    }
    else if(type == "bc") {
        cfunc <- "calc_pairprob_bc"
        n.gen <- 2
        if(!xchr) # autosome
            gen.names <- getgenonames("bc", "A", cross.attr=attributes(cross))
        else gen.names <- c("g1","g2")
        one.map <- TRUE
    }
    else if(type == "riself" || type=="risib" || type=="dh" || type=="haploid") {
        cfunc <- "calc_pairprob_bc"
        n.gen <- 2
        gen.names <- getgenonames(type, "A", cross.attr=attributes(cross))
        one.map <- TRUE
    }
    else if(type == "4way") {
        cfunc <- "calc_pairprob_4way"
        n.gen <- 4
        one.map <- FALSE
        gen.names <- getgenonames(type, "A", cross.attr=attributes(cross))
    }
    else if(type == "ri4self" || type=="ri4sib" || type=="ri8self" || type=="ri8sib" || type=="magic16") {
        cfunc <- paste("calc_pairprob_", type, sep="")
        if(type=="magic16") n.gen <- 16
        else n.gen <- as.numeric(substr(type, 3, 3))
        one.map <- TRUE
        gen.names <- LETTERS[1:n.gen]
        if(xchr)
            warning("calc.pairprob not working properly for the X chromosome for 4- or 8-way RIL.")
    }
    else if(type == "bcsft") {
        one.map <- TRUE
        cfunc <- "calc_pairprob_bcsft"
        cross.scheme <- attr(cross, "scheme") ## c(s,t) for BC(s)F(t)
        if(!xchr) { # autosome
            gen.names <- getgenonames("bcsft", "A", cross.attr=attributes(cross))
            n.gen <- 2 + (cross.scheme[2] > 0)
        }
        else { ## X chromsome
            cross.scheme[1] <- cross.scheme[1] + cross.scheme[2] - (cross.scheme[1] == 0)
            cross.scheme[2] <- 0
            gen.names <- c("g1","g2")
            n.gen <- 2
        }
    }
    else
        stop("calc.pairprob not available for cross type ", type, ".")

    # genotype data
    gen <- cross$geno[[1]]$data
    gen[is.na(gen)] <- 0

    # get recombination fractions
    if(one.map) {
        #    map <- create.map(cross$geno[[1]]$map,step,off.end)
        rf <- mf(diff(map))
        if(type=="risib" || type=="riself")
            rf <- adjust.rf.ri(rf,substr(type,3,nchar(type)),chrtype(cross$geno[[1]]))
        rf[rf < 1e-14] <- 1e-14

        # new genotype matrix with pseudomarkers filled in
        newgen <- matrix(ncol=length(map),nrow=nrow(gen))
        colnames(newgen) <- names(map)
        newgen[,colnames(gen)] <- gen
        newgen[is.na(newgen)] <- 0
        n.pos <- ncol(newgen)
        marnames <- names(map)
    }
    else {
        #    map <- create.map(cross$geno[[1]]$map,step,off.end)
        rf <- mf(diff(map[1,]))
        rf[rf < 1e-14] <- 1e-14
        rf2 <- mf(diff(map[2,]))
        rf2[rf2 < 1e-14] <- 1e-14

        # new genotype matrix with pseudomarkers filled in
        newgen <- matrix(ncol=ncol(map),nrow=nrow(gen))
        colnames(newgen) <- colnames(map)
        newgen[,colnames(gen)] <- gen
        newgen[is.na(newgen)] <- 0
        n.pos <- ncol(newgen)
        marnames <- colnames(map)
    }

    if(n.pos < 2) return(NULL)

    # below: at least two positions
    # call the C function
    if(one.map) {
        ## Hide cross scheme in genoprob to pass to routine. BY
        temp <- as.double(rep(0,n.gen*n.ind*n.pos))
        if(type == "bcsft")
            temp[1:2] <- cross.scheme

        z <- .C(cfunc,
                as.integer(n.ind),         # number of individuals
                as.integer(n.pos),         # number of markers
                as.integer(newgen),        # genotype data
                as.double(rf),             # recombination fractions
                as.double(error.prob),     #
                as.double(temp),
                pairprob=as.double(rep(0,n.ind*n.pos*(n.pos-1)/2*n.gen^2)),
                PACKAGE="qtl")
    }
    else {
        z <- .C(cfunc,
                as.integer(n.ind),         # number of individuals
                as.integer(n.pos),         # number of markers
                as.integer(newgen),        # genotype data
                as.double(rf),             # recombination fractions
                as.double(rf2),            # recombination fractions
                as.double(error.prob),     #
                as.double(rep(0,n.gen*n.ind*n.pos)),
                pairprob=as.double(rep(0,n.ind*n.pos*(n.pos-1)/2*n.gen^2)),
                PACKAGE="qtl")
    }

    pairprob <- array(z$pairprob, dim=c(n.ind,n.pos*(n.pos-1)/2,n.gen,n.gen))

    # 4- and 8-way RIL: reorganize the results
    if(type=="ri4self" || type=="ri4sib" || type=="ri8self" || type=="ri8sib" || type=="bgmagic16")
        pairprob <- reorgRIpairprob(cross, pairprob)

    pairprob
}

# end of calc.pairprob.R

######################################################################
#
# effectscan.R
#
# copyright (c) 2003-2020, Karl W. Broman
# [completely re-written in Sep, 2007, based partly on code from Hao Wu]
# last modified Feb, 2020
# first written Jan, 2003
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: effectscan
#
######################################################################

effectscan <-
    function(cross, pheno.col=1, chr, get.se=FALSE, draw=TRUE,
             gap=25, ylim, mtick=c("line","triangle"),
             add.legend=TRUE, alternate.chrid=FALSE, ...)
{
    type <- crosstype(cross)
    mtick <- match.arg(mtick)
    if(type == "4way")
        stop("effect scan not working for 4-way cross yet.")

    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("effectscan can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 | pheno.col > nphe(cross))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    if(!is.numeric(cross$pheno[,pheno.col]))
        stop("phenotype \"", colnames(cross$pheno)[pheno.col], "\" is not numeric.")

    pheno <- cross$pheno[,pheno.col]
    wh <- is.na(pheno)
    if(any(wh)) {
        pheno <- pheno[!wh]
        cross <- subset.cross(cross, ind=(!wh))
    }

    if(!missing(chr)) cross <- subset.cross(cross, chr=chr)

    chr_type <- sapply(cross$geno, chrtype)

    n.ind <- length(pheno)

    results <- NULL
    for(i in 1:nchr(cross)) {
        if(!("draws" %in% names(cross$geno[[i]])))
            stop("You must first run sim.geno.")

        draws <- cross$geno[[i]]$draws

        # create map of positions
        if("map" %in% names(attributes(cross$geno[[i]]$draws)))
            map <- attr(cross$geno[[i]]$draws,"map")
        else {
            stp <- attr(cross$geno[[i]]$draws, "step")
            oe <- attr(cross$geno[[i]]$draws, "off.end")

            if("stepwidth" %in% names(attributes(cross$geno[[i]]$draws)))
                stpw <- attr(cross$geno[[i]]$draws, "stepwidth")
            else stpw <- "fixed"
            map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
        }

        if(is.matrix(map)) {
            marnam <- colnames(map)
            map <- map[1,]
        }
        else marnam <- names(map)

        if(type == "risib" || type=="riself" || type=="dh" || type=="haploid") {
            mapping <- rbind(c(+1, -1),
                             c(+1, +1))
            colnames(mapping) <- c("intercept","a")
            dropcol <- 1
        }
        else if(type=="bc") {
            if(chr_type[i] == "X") {
                sexpgm <- getsex(cross)
                draws <- reviseXdata(type, "full", sexpgm, draws=draws,
                                     cross.attr=attributes(cross))

                if(is.null(sexpgm$sex) || all(sexpgm$sex==0) || all(sexpgm$sex==1)) { # all one sex
                    mapping <- rbind(c(+1, -0.5),
                                     c(+1, +0.5))
                    colnames(mapping) <- c("intercept", "a")
                    dropcol <- 1
                }
                else { # some of each sex
                    mapping <- rbind(c(+1, 0,-0.5,   0),
                                     c(+1, 0,+0.5,   0),
                                     c(+1,+1, 0,  -0.5),
                                     c(+1,+1, 0,  +0.5))
                    colnames(mapping) <- c("intercept", "sex", "a.female", "a.male")
                    dropcol <- 1:2
                }
            } # end bc X chr
            else {
                mapping <- rbind(c(+1, -0.5),
                                 c(+1, +0.5))
                colnames(mapping) <- c("intercept", "a")
                dropcol <- 1
            } # end bc autosome
        } # end bc
        else { # intercross
            if(chr_type[i] == "X") {
                sexpgm <- getsex(cross)
                draws <- reviseXdata(type, "full", sexpgm, draws=draws,
                                     cross.attr=attributes(cross))


                if(is.null(sexpgm$pgm) || all(sexpgm$pgm==0) || all(sexpgm$pgm==1)) { # all one direction

                    if(is.null(sexpgm$sex) || all(sexpgm$sex==0) || all(sexpgm$sex==1)) { # all one sex
                        mapping <- rbind(c(+1, -0.5),
                                         c(+1, +0.5))
                        colnames(mapping) <- c("intercept", "a")
                        dropcol <- 1
                    }
                    else {
                        mapping <- rbind(c(+1, 0,-0.5,   0),
                                         c(+1, 0,+0.5,   0),
                                         c(+1,+1, 0,  -0.5),
                                         c(+1,+1, 0,  +0.5))
                        colnames(mapping) <- c("intercept", "sex", "a.female", "a.male")
                        dropcol <- 1:2
                    }
                }
                else { # some of each direction
                    if(is.null(sexpgm$sex) || all(sexpgm$sex==0)) { # all female
                        mapping <- rbind(c(+1, 0,-0.5,  0),
                                         c(+1, 0,+0.5,  0),
                                         c(+1,+1,   0,-0.5),
                                         c(+1,+1,   0,+0.5))
                        colnames(mapping) <- c("intercept","dir","a.forw","a.rev")
                        dropcol <- 1:2
                    }
                    else if(all(sexpgm$sex==1)) { # all male
                        mapping <- rbind(c(+1, -0.5),
                                         c(+1, +0.5))
                        colnames(mapping) <- c("intercept", "a")
                        dropcol <- 1
                    }
                    else { # some of each sex
                        mapping <- rbind(c(+1, 0, 0, -0.5,  0,   0),
                                         c(+1, 0, 0, +0.5,  0,   0),
                                         c(+1,+1, 0,   0,-0.5,   0),
                                         c(+1,+1, 0,   0,+0.5,   0),
                                         c(+1, 0,+1,   0,   0,-0.5),
                                         c(+1, 0,+1,   0,   0,+0.5))
                        colnames(mapping) <- c("intercept","dir","sex","a.femaleforw","a.femalerev","a.male")
                        dropcol <- 1:3
                    }
                }
            } # end f2 X chr
            else {
                mapping <- rbind(c(+1, -1,  0),
                                 c(+1,  0, +1),
                                 c(+1, +1,  0))
                colnames(mapping) <- c("intercept","a","d")
                dropcol <- 1
            } # f2 autosome
        } # end f2

        n.gen <- ncol(mapping)
        n.pos <- ncol(draws)
        n.imp <- dim(draws)[3]

        z <- .C("R_effectscan",
                as.integer(n.ind),
                as.integer(n.gen),
                as.integer(n.imp),
                as.integer(n.pos),
                as.integer(draws-1),
                as.double(pheno),
                as.double(mapping),
                beta=as.double(rep(0,n.pos*n.gen)),
                se=as.double(rep(0,n.pos*n.gen)),
                as.integer(get.se),
                PACKAGE="qtl")


        beta <- t(matrix(z$beta, ncol=n.pos))
        colnames(beta) <- colnames(mapping)

        if(get.se) {
            se <- t(matrix(z$se, ncol=n.pos))
            colnames(se) <- paste("se", colnames(mapping), sep=".")
            beta <- cbind(beta, se[,-dropcol,drop=FALSE])
        }
        z <- beta[,-dropcol,drop=FALSE]

        w <- marnam
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",names(cross$geno)[i],".",w[o],sep="")
        rownames(z) <- w

        z <- as.data.frame(z, stringsAsFactors=TRUE)
        z <- cbind(chr=factor(rep(names(cross$geno)[i],length(map)),levels=names(cross$geno)),
                   pos=as.numeric(map), z)
        rownames(z) <- w

        if(i==1) results <- z
        else {
            w <- match(colnames(z), colnames(results))

            if(any(is.na(w))) {
                curnam <- colnames(results)
                for(j in which(is.na(w)))
                    results <- cbind(results, rep(NA, nrow(results)))
                colnames(results) <- c(curnam, colnames(z)[is.na(w)])
            }

            w <- match(colnames(results), colnames(z))
            if(any(is.na(w))) {
                curnam <- colnames(z)
                for(j in which(is.na(w)))
                    z <- cbind(z, rep(NA, nrow(z)))
                colnames(z) <- c(curnam, colnames(results)[is.na(w)])
            }

            results <- rbind(results, z)

        }
    } # end  loop over chromosomes

    class(results) <- c("effectscan", "scanone", "data.frame")

    if(draw) { # make the figure
        if(missing(ylim))
            plot.effectscan(results, gap=gap, mtick=mtick, add.legend=add.legend,
                            alternate.chrid=alternate.chrid, ...)
        else
            plot.effectscan(results, gap=gap, mtick=mtick, add.legend=add.legend,
                            ylim=ylim, alternate.chrid=alternate.chrid, ...)
    }

    invisible(results)
}

# function to make the effectscan plot
plot.effectscan <-
    function(x, gap=25, ylim, mtick=c("line","triangle"),
             add.legend=TRUE, alternate.chrid=FALSE, ...)
{
    col <- c("blue","red","darkorange","darkgreen","purple")
    lightcol <- c("lightblue", "pink", "peachpuff1", "palegreen1", "thistle1")

    results <- x
    eff <- 3:ncol(results)

    if(length(grep("^se", colnames(results)))>0) get.se <- TRUE
    else get.se <- FALSE

    if(get.se) {
        se <- grep("^se", colnames(results)[eff])
        eff <- eff[-se]
        se <- se + 2

        lo <- as.matrix(results[,eff]) - as.matrix(results[,se])
        hi <- as.matrix(results[,eff]) + as.matrix(results[,se])

        yl <- range(c(lo,hi), na.rm=TRUE)
    }
    else yl <- range(results[,eff], na.rm=TRUE)

    if(!missing(ylim)) yl <- ylim

    plot.scanone(results, lodcolumn=1, ylim=yl, gap=gap, mtick=mtick, alternate.chrid=alternate.chrid,
                 col=col[1], ...)

    if(get.se) {
        begend <- matrix(unlist(tapply(results[,2],results[,1],range)),ncol=2,byrow=TRUE)
        rownames(begend) <- unique(results[,1])
        chr <- unique(as.character(results[,1]))

        begend <- begend[as.character(chr),,drop=FALSE]
        len <- begend[,2]-begend[,1]

        if(length(len)>1) start <- c(0,cumsum(len+gap))-c(begend[,1],0)
        else start <- 0

        x <- results[,2]
        for(i in seq(along=chr))
            x[results[,1]==chr[i]] <- results[results[,1]==chr[i],2]+start[i]

        for(i in seq(along=chr)) {
            wh <- results[,1]==chr[i]

            for(j in 1:ncol(lo)) {
                if(any(!is.na(lo[wh,j]))) {
                    xx <- c(x[wh], rev(x[wh]))
                    yy <- c(lo[wh,j], rev(hi[wh,j]))
                    polygon(xx, yy, col=lightcol[j], border=lightcol[j])
                }
            }
        }

        # go back and add lines at edges
        for(i in seq(along=chr)) {
            wh <- results[,1]==chr[i]

            for(j in 1:ncol(lo)) {
                if(any(!is.na(lo[wh,j]))) {
                    xx <- c(x[wh], rev(x[wh]))
                    yy <- c(lo[wh,j], rev(hi[wh,j]))
                    lines(xx, yy, col=lightcol[j])
                }
            }
        }

        plot.scanone(results, lodcolumn=1, add=TRUE, col=col[1])
    }

    if(length(eff) > 1) {
        for(i in seq(along=eff)[-1])
            plot.scanone(results, lodcolumn=eff[i]-2, gap=gap, add=TRUE, col=col[i])
    }

    if(add.legend)
        legend("top", legend=names(results)[eff], col=col[1:length(eff)], lwd=2)

    abline(h=0, lty=2)
}



# end of effectscan.R

######################################################################
#
# read.cross.tidy.R
#
# copyright (c) 2005-2019, Karl W Broman
# last modified Dec, 2019
# first written Aug, 2014
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.tidy
#           [See read.cross.R for the main read.cross function.]
#
######################################################################

######################################################################
#
# read.cross.tidy
#
# read data in comma-delimited format, with separate files for phenotype,
# genotype, and map data
#
######################################################################

read.cross.tidy <-
    function(dir, genfile, phefile, mapfile, na.strings=c("-","NA"),
             genotypes=c("A","H","B","D","C"), ...)
{
    # create file names
    if(missing(genfile)) genfile <- "gen.csv"
    if(missing(phefile)) phefile <- "phe.csv"
    if(missing(mapfile)) mapfile <- "map.csv"

    if(!missing(dir) && dir != "") {
        genfile <- file.path(dir, genfile)
        phefile <- file.path(dir, phefile)
        mapfile <- file.path(dir, mapfile)
    }

    args <- list(...)

    if("" %in% na.strings) {
        na.strings <- na.strings[na.strings != ""]
        warning("Including \"\" in na.strings will cause problems; omitted.")
    }

    # if user wants to use comma for decimal point, we need
    dec <- ifelse("dec" %in% names(args), args[["dec"]], ".")

    # "sep" not in the "..." argument and so take sep=","
    sep <- ifelse("sep" %in% names(args), args[["sep"]], ",")

    # read the data files
    args <- c(args, list(sep = sep, na.strings = na.strings,
                         row.names = 1, header = TRUE, stringsAsFactors = FALSE))

    gen   <- do.call("read.table", c(args, list(file = genfile)))
    pheno <- do.call("read.table", c(args, list(file = phefile)))
    map   <- do.call("read.table", c(args, list(file = mapfile)))

    # Check individual IDs
    mp <- setdiff(colnames(gen), colnames(pheno))
    if (length(mp) > 0) {
        warning(length(mp), " individuals with genotypes but no phenotypes\n    ", paste(mp, collapse="|"), "\n")
        pheno[mp] <- NA
    }

    mg <- setdiff(colnames(pheno), colnames(gen))
    if (length(mg) > 0) {
        warning(length(mg), " individuals with phenotypes but no genotypes\n    ", paste(mg, collapse="|"), "\n")
        gen[mg] <- NA
    }

    # ensure individual order is consistent
    ids   <- colnames(pheno)
    pheno <- pheno[, ids]
    gen   <- gen[,   ids]

    # Check markers
    genm   <- rownames(gen)
    mapm   <- rownames(map)
    mnames   <- intersect(genm, mapm)

    mg <- setdiff(mapm, genm)
    if (length(mg) > 0) warning("Removing ", length(mg),
                                " genotyped markers with missing positions\n")

    mm <- setdiff(genm, mapm)
    if (length(mm) > 0) warning("Removing", length(mm),
                                " mapped markers with no genotypes\n")

    map[[2]] <- asnumericwithdec(map[[2]], dec)

    # replace empty cells with NA
    add_na <- function(x) replace(x, !is.na(x) & x == "", NA)
    pheno <- add_na(pheno)
    gen   <- add_na(gen)


    # check chromosomes
    chr <- map[[1]]

    if(any(is.na(chr))) {
        stop("There are missing chromosome IDs. Check row(s) ",
             paste(which(is.na(chr)), collapse=","), sep = "")
    }

    # look for strange entries in the genotype data
    if(length(genotypes) > 0) {
        temp <- Filter(Negate(is.na), unique(unlist(gen)))
        wh <- !(temp %in% genotypes)
        if(any(wh)) {
            warn <- "The following unexpected genotype codes were treated as missing.\n    "
            ge <- paste("|", paste(temp[wh],collapse="|"),"|",sep="")
            warn <- paste(warn,ge,"\n",sep="")
            warning(warn)
        }

        # convert genotype data
        gen <- as.matrix(gen)
        allgeno <- matrix(match(gen, genotypes), ncol = ncol(gen), dimnames = dimnames(gen))
    } else {
        genotypes <- Filter(Negate(is.na), unique(unlist(gen)))
        gen <- as.data.frame(lapply(gen, factor, levels = genotypes), rownames(gen))
        allgeno <- data.matrix(gen)
    }

    # convert phenotype data
    # pheno must be rotated to allow for numeric and factor variables
    pheno <- data.frame(t(pheno))
    pheno <- data.frame(lapply(pheno, sw2numeric, dec = dec),
                        row.names = NULL, stringsAsFactors = TRUE)

    # add id column if informative identifiers are provided
    default.ids <- make.names(seq_len(nrow(pheno)))
    if (!all(ids %in% default.ids)) pheno$id <- ids

    # re-order the markers by chr and position
    # try to figure out the chr labels
    if(all(chr %in% c(1:999,"X","x"))) { # 1...19 + X
        tempchr <- chr
        tempchr[chr=="X" | chr=="x"] <- 1000
        tempchr <- as.numeric(tempchr)
        neworder <- order(tempchr, map[[2]])
    } else {
        # prevent reordering of chromosomes
        tempchr <- factor(chr, levels=unique(chr))

        neworder <- order(tempchr, map[[2]])
    }
    chr <- chr[neworder]
    map <- map[neworder, ]
    allgeno <- allgeno[rownames(map), , drop = FALSE]
    mnames <- mnames[neworder]

    # fix up map information
    # number of chromosomes
    uchr <- unique(chr)
    n.chr <- length(uchr)
    geno <- vector("list", n.chr)
    names(geno) <- uchr
    min.mar <- 1
    allautogeno <- NULL

    for(i in 1:n.chr) { # loop over chromosomes
        # create map
        temp.map <- map[[2]][chr==uchr[i]]
        names(temp.map) <- mnames[chr==uchr[i]]

        # pull out appropriate portion of genotype data
        data <- t(allgeno[names(temp.map), ])

        # drop genotype rownames
        rownames(data) <- NULL

        geno[[i]] <- list(data=data, map=temp.map)
        if(uchr[i] == "X" || uchr[i] == "x")
            class(geno[[i]]) <- "X"
        else {
            class(geno[[i]]) <- "A"
            if(is.null(allautogeno)) allautogeno <- data
            else allautogeno <- cbind(allautogeno,data)
        }
    }

    if(is.null(allautogeno)) allautogeno <- allgeno

    # check that data dimensions match
    n.mar1 <- sapply(geno,function(a) ncol(a$data))
    n.mar2 <- sapply(geno,function(a) length(a$map))
    n.phe <- ncol(pheno)
    n.ind1 <- nrow(pheno)
    n.ind2 <- sapply(geno,function(a) nrow(a$data))
    if(any(n.ind1 != n.ind2)) {
        cat(n.ind1,n.ind2,"\n")
        stop("Number of individuals in genotypes and phenotypes do not match.");
    }
    if(any(n.mar1 != n.mar2)) {
        cat(n.mar1,n.mar2,"\n")
        stop("Numbers of markers in genotypes and marker names files do not match.");
    }

    # print some information about the amount of data read
    cat(" --Read the following data:\n");
    cat("\t", n.ind1, " individuals\n");
    cat("\t", sum(n.mar1), " markers\n");
    cat("\t", n.phe, " phenotypes\n");

    if(all(is.na(allgeno)))
        warning("There is no genotype data!\n")

    # determine map type: f2 or bc or 4way?
    if(all(is.na(allgeno))) warning("There is no genotype data!\n")
    if(all(is.na(allautogeno)) || max(allautogeno,na.rm=TRUE)<=2) type <- "bc"
    else if(max(allautogeno,na.rm=TRUE)<=5) type <- "f2"
    else type <- "4way"
    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c(type,"cross")

    # check that nothing is strange in the genotype data
    if(type=="f2") max.gen <- 5
    else if(type=="bc") max.gen <- 2
    else max.gen <- 14

    # check that markers are in proper order
    #     if not, fix up the order
    for(i in 1:n.chr) {
        if(any(diff(cross$geno[[i]]$map)<0)) {
            o <- order(cross$geno[[i]]$map)
            cross$geno[[i]]$map <- cross$geno[[i]]$map[o]
            cross$geno[[i]]$data <- cross$geno[[i]]$data[,o,drop=FALSE]
        }
    }

    # return cross + indicator of whether to run est.map
    list(cross,FALSE)
}

# end of read.cross.tidy.R

######################################################################
#
# add_threshold.R
#
# copyright (c) 2006-2019, Karl W Broman
# last modified Dec, 2019
# first written Dec, 2006
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Contains: add.threshold, xaxisloc.scanone
#
######################################################################

######################################################################
# add.threshold: function to add threshold lines to a plot
# created by plot.scanone()
#
# out: scanone output used to create the plot
# chr: chromosomes that were plotted
#
# perms: scanone permutation results
# alpha: significance level (a single number)
#
# gap:   the gap between chromosomes, as specified in the call to plot.scanone
#
# ...:   extra arguments passed to abline or segments to control line types/widths/colors
#        (e.g., specify lty=2 to give dashed lines)
######################################################################

add.threshold <-
    function(out, chr, perms, alpha=0.05, lodcolumn=1, gap=25, ...)
{
    if(missing(out))
        stop("You must provide scanone output, so we can get chromosome lengths.")
    if(!inherits(out, "scanone"))
        stop("out should have class \"scanone\".")
    if(!missing(chr))
        out <- subset(out, chr=chr)

    if(missing(perms))
        stop("You must specify permutation results, to get the thresholds")
    if(length(alpha) > 1 || alpha<0 || alpha>1)
        stop("alpha should have length 1 and be between 0 and 1.")

    thr <- summary(perms, alpha=alpha)


    if(!is.list(thr)) {
        if(any(lodcolumn < 1 | lodcolumn > length(thr)))
            stop("lodcolumn should be between 1 and ", length(thr))
        abline(h=thr[lodcolumn], ...)
    }
    else {
        if(any(lodcolumn < 1 | lodcolumn > length(thr$A)))
            stop("lodcolumn should be between 1 and ", length(thr$A))
        a <- thr$A[lodcolumn]
        x <- thr$X[lodcolumn]
        noX <- FALSE
        xchr <- attr(perms, "xchr")

        L <- tapply(out[,2], out[,1], function(a) diff(range(a)))
        L <- L[!is.na(L)]
        xchr <- xchr[match(names(L), names(xchr))]

        if(all(xchr))
            abline(h=x, ...)
        else if(all(!xchr))
            abline(h=a, ...)
        else {

            start <- c(0,cumsum(L+gap))
            end <- start+ c(L,0)
            wh <- which(!xchr)
            if(length(wh)==1 || all(diff(wh)==1))
                segments(start[min(wh)], a, end[max(wh)], a, ...)
            else
                segments(start[wh], a, start[wh+1], a, ...)
            wh <- which(xchr)
            if(length(wh)==1 || all(diff(wh)==1))
                segments(start[min(wh)], x, end[max(wh)], x, ...)
            else
                segments(start[wh], x, start[wh+1], x, ...)
        }
    }
    invisible()
}

# xaxisloc.scanone
# find x-axis locations for a plot of scanone output
xaxisloc.scanone <-
    function(out, thechr, thepos, chr, gap=25)
{
    if(missing(out))
        stop("You must provide scanone output, so we can get chromosome lengths.")
    if(!inherits(out, "scanone"))
        stop("out should have class \"scanone\".")
    if(!missing(chr))
        out <- subset(out, chr=chr)
    chr <- unique(out[,1])

    if(length(thechr) != 1) {
        if(length(thepos)==1)
            thepos <- rep(thepos, length(thechr))
        else if(length(thechr) != length(thepos))
            stop("If thechr and thepos have length>1 they must both have the same length")
    }
    else {
        if(length(thepos)!=1)
            thechr <- rep(thechr, length(thepos))
    }

    if(length(thechr) > 1) {
        res <- rep(NA, length(thechr))
        for(i in seq(along=thechr))
            res[i] <- xaxisloc.scanone(out, thechr[i], thepos[i], chr, gap)
        return(res)
    }

    L <- tapply(out[,2], out[,1], function(a) diff(range(a, na.rm=TRUE)))
    Lmin <- tapply(out[,2], out[,1], min, na.rm=TRUE)
    start <- c(0,cumsum(L+gap))

    start[which(as.character(thechr)==chr)]+(thepos - Lmin[as.character(thechr)==chr])
}

# end of add_threshold.R

#####################################################################
#
# sim_ril.R
#
# copyright (c) 2004-2022, Karl W Broman
# last modified June, 2022
# first written May, 2004
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: sim.ril, simulateFounderSnps, convertMWril
#
######################################################################

######################################################################
#
# sim.ril
#
# Simulate RILs by selfing or sibling mating from 2, 4, or 8
# parental strains
# map = map in the usual R/qtl map format
# m = interference parameter (0 is no interference)
######################################################################
sim.ril <-
    function(map, n.ril=1, type=c("sibmating", "selfing"),
             n.str=c("2","4","8"), m=0, p=0,
             error.prob=0, missing.prob=0, random.cross=TRUE)
{
    type <- match.arg(type)
    if(type=="sibmating") selfing <- 0
    else selfing <- 1
    if(is.numeric(n.str)) n.str <- as.character(n.str)
    n.str <- as.numeric(match.arg(n.str))
    n.chr <- length(map)
    n.mar <- sapply(map,length)
    tot.mar <- sum(n.mar)

    if(m < 0) stop("Must have m >= 0.")
    if(p < 0 || p > 1) stop("Must have 0 <= p <= 1.")
    if(p == 1) {
        p <- 0
        m <- 0
    }

    omap <- map
    map <- lapply(map, function(a) a-min(a))

    if(!selfing && inherits(omap[[length(omap)]], "X"))
        include.x <- TRUE
    else {
        for(i in seq(along=omap)) class(omap[[i]]) <- "A"
        include.x <- FALSE
    }

    if(n.str==2) random.cross <- FALSE

    x <- .C("R_sim_ril",
            as.integer(n.chr),
            as.integer(n.mar),
            as.integer(n.ril),
            as.double(unlist(map)),
            as.integer(n.str),
            as.integer(m),
            as.double(p),
            as.integer(include.x),
            as.integer(random.cross),
            as.integer(selfing),
            cross=as.integer(rep(0,n.ril*n.str)),
            res=as.integer(rep(0,tot.mar*n.ril)),
            orig=as.integer(rep(0,tot.mar*n.ril)),
            as.double(error.prob),
            as.double(missing.prob),
            err=as.integer(rep(0,tot.mar*n.ril)),
            PACKAGE="qtl")

    cross <- t(matrix(x$cross,ncol=n.ril,nrow=n.str))
    err <- t(matrix(x$err,nrow=tot.mar,ncol=n.ril))
    truegeno <- t(matrix(x$orig, nrow=tot.mar, ncol=n.ril))
    x <- t(matrix(x$res,nrow=tot.mar,ncol=n.ril))
    x[x==0] <- NA

    geno <- vector("list", n.chr)
    names(geno) <- names(map)
    cur <- 0
    for(i in 1:n.chr) {
        geno[[i]]$data <- x[,cur + 1:n.mar[i],drop=FALSE]
        colnames(geno[[i]]$data) <- names(map[[i]])
        geno[[i]]$map <- omap[[i]]
        if(missing.prob > 0 || (error.prob>0 && n.str==2))
            geno[[i]]$truegeno <- truegeno[,cur+1:n.mar[i],drop=FALSE]
        if(error.prob > 0 && n.str==2)
            geno[[i]]$errors <- err[,cur+1:n.mar[i],drop=FALSE]

        cur <- cur + n.mar[i]
        class(geno[[i]]) <- class(omap[[i]])

    }
    pheno <- data.frame(line=1:n.ril, stringsAsFactors=TRUE)
    x <- list(geno=geno,pheno=pheno,cross=cross)

    # ri[n][sib/self]un: un = genotypes not yet transformed
    if(type=="sibmating") {
        if(n.str=="2")
            class(x) <- c("risib","cross")
        else
            class(x) <- c(paste("ri", n.str, "sibun",sep=""),"cross")
    }
    else {
        if(n.str=="2")
            class(x) <- c("riself","cross")
        else
            class(x) <- c(paste("ri", n.str, "selfun",sep=""),"cross")
    }

    x
}

######################################################################
# simFounderSnps
#
# Simulate founder snp genotypes for a multiple-strain RIL
#
# map = genetic map of markers (used just to get no. markers per chr)
#
# n.str = number of founder strains (4 or 8)
#
# pat.freq = frequency of SNP genotype patterns (length n.str/2 + 1)
#            (monoallelic, snp unique to a founder,
#             snp present in 2 founder,
#             [for 8 founders: snp in 3/8, snp in 4/8] )
######################################################################
simFounderSnps <-
    function(map, n.str=c("4","8"), pat.freq)
{
    if(is.numeric(n.str)) n.str <- as.character(n.str)
    n.str <- as.numeric(match.arg(n.str))

    if(missing(pat.freq)) {
        if(n.str==8) pat.freq <- c(0, 0.4, 0.3, 0.2, 0.1)
        else pat.freq <- c(0, 0.7, 0.3)
    }

    if(length(pat.freq) < n.str/2+1)
        pat.freq <- c(pat.freq, rep(0, n.str/2+1 - length(pat.freq)))
    else pat.freq <- pat.freq[1:(n.str/2+1)]
    pat.freq <- pat.freq/sum(pat.freq)

    n.mar <- sapply(map, length)
    output <- vector("list", length(map))
    names(output) <- names(map)
    for(i in seq(along=map)) {
        thepat <- sample(seq(length(pat.freq))-1, n.mar[i], prob=pat.freq, replace=TRUE)
        output[[i]] <- matrix(0, ncol=n.str, nrow=n.mar[i])
        for(j in seq(along=thepat))
            output[[i]][j,sample(1:n.str, thepat[j])] <- 1
    }

    output
}

######################################################################
# convertMWril: Convert multiple-strain RIL genotypes using parental data
#
# parents = Parental genotype data, with genetic map
#           list with elements being chromosomes
#           each chromosome is a matrix n.mar x n.str,
######################################################################
convertMWril <-
    function(cross, parents, error.prob=0)
{
    crosstype <- crosstype(cross)
    n.str.by.crosstype <- as.numeric(substr(crosstype, 3, 3))
    if(!grepl("un$", crosstype)) {
        stop("cross appears to have already been converted.")
    }
    class(cross) <- c(sub("un$", "", crosstype), "cross")

    n.ril <- nind(cross)
    thecrosses <- cross$cross
    n.str <- ncol(thecrosses)
    if(n.str != ncol(parents[[1]]))
        stop("Different numbers of founders in cross and parents.")
    if(n.str != n.str.by.crosstype)
        stop("Confusion regarding no. founders within cross.")
    if(length(parents) != nchr(cross))
        stop("Different numbers of chromosomes in cross and parents.")

    n.mar <- nmar(cross)
    n.mar2 <- sapply(parents, nrow)
    if(any(n.mar != n.mar2))
        stop("Different numbers of markers in cross and parents.")

    pg <- unlist(parents)
    if(any(is.na(pg)))
        stop("Missing parental data not allowed.")

    # if positive error prob, check whether all parental data are snps
    if(error.prob > 0) {
        if(all(pg==0 | pg==1)) all.snps <- TRUE
        else {
            if(all(pg==1 | pg==2)) { # convert to 0/1
                parents <- lapply(parents, function(a) a - 1)
                all.snps <- TRUE
            }
            else all.snps <- FALSE
        }
    }
    else all.snps <- FALSE


    for(i in 1:nchr(cross)) {
        dat <- cross$geno[[i]]$data
        dat[is.na(dat)] <- 0

        results <-
            .C("R_convertMWril",
               as.integer(n.ril),        # no. ril
               as.integer(n.mar[i]),     # no. markers
               as.integer(n.str),        # no. founders
               as.integer(parents[[i]]), # SNP data on parents (n.mar x n.str)
               g=as.integer(dat), # SNP data on RIL (n.ril x n.mar)
               as.integer(thecrosses),   # the crosses (n.ril x n.str)
               as.integer(all.snps),
               as.double(error.prob),
               err=as.integer(rep(0,n.mar[i]*n.ril)),
               PACKAGE="qtl")

        # replace 0's with missing values
        newgeno <- results$g
        newgeno[newgeno==0] <- NA
        newgeno <- matrix(newgeno, n.ril, n.mar[i])
        colnames(newgeno) <- colnames(cross$geno[[i]]$data)
        cross$geno[[i]]$data <- newgeno
        if(error.prob > 0)
            cross$geno[[i]]$errors <- matrix(results$err, n.ril, n.mar[i])
    }

    cross
}

# end of sim_ril.R

######################################################################
#
# est.rf.R
#
# copyright (c) 2001-2022, Karl W Broman
# last modified Jun, 2022
# first written Apr, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: est.rf, plotRF, checkAlleles, pull.rf, plot.rfmatrix
#
######################################################################

######################################################################
#
# est.rf: Estimate sex-averaged recombination fractions between
#         all pairs of markers
#
######################################################################

est.rf <-
    function(cross, maxit=10000, tol=1e-6)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    n.chr <- nchr(cross)
    n.mar <- totmar(cross)
    n.ind <- nind(cross)
    mar.names <- unlist(lapply(cross$geno,function(a) colnames(a$data)))

    type <- crosstype(cross)
    chr_type <- sapply(cross$geno, chrtype)

    is.bcsft <- (type == "bcsft")
    if(is.bcsft) {
        cross.scheme <- attr(cross, "scheme") ## c(s,t) for BC(s)F(t)
        is.bcsft <- cross.scheme[2] > 0 ## used for fixX only
    }

    xchrcol <- NULL
    fixX <- FALSE
    Geno <- NULL
    # create full genotype matrix
    for(i in 1:n.chr) {
        temp <- cross$geno[[i]]$data

        # treat X chromosome specially in an intercross or BCsFt with t>0.
        if((type=="f2" || is.bcsft) && chr_type[i]=="X") {
            fixX <- TRUE
            if(i != 1) xchrcol <- c(xchrcol,ncol(Geno)+(1:ncol(cross$geno[[i]]$data)))
            else xchrcol <- 1:ncol(cross$geno[[i]]$data)
            xchr <- temp
            xchr[is.na(xchr)] <- 0
            temp <- reviseXdata("f2","simple",getsex(cross),geno=temp,
                                cross.attr=attributes(cross))
        }
        Geno <- cbind(Geno,temp)
    }

    # which type of cross is this?
    if(type == "f2")
        cfunc <- "est_rf_f2"
    else if(type == "bc" || type=="risib" || type=="riself" || type=="dh" || type=="haploid")
        cfunc <- "est_rf_bc"
    else if(type == "4way")
        cfunc <- "est_rf_4way"
    else if(type=="ri8sib" || type=="ri8self" || type=="ri4sib" || type=="ri4self") {
        cfunc <- paste("est_rf_", type, sep="")
        if(any(chr_type == "X"))
            warning("est.rf not working properly for the X chromosome for 4- or 8-way RIL.")
    }
    else if(type == "bcsft")
        cfunc <- "est_rf_bcsft"
    else
        stop("est.rf not available for cross type ", type, ".")

    Geno[is.na(Geno)] <- 0

    if(type=="bc" || type=="risib" || type=="riself" || type=="dh" || type=="haploid")
        z <- .C(cfunc,
                as.integer(n.ind),         # number of individuals
                as.integer(n.mar),         # number of markers
                as.integer(Geno),
                rf = as.double(rep(0,n.mar*n.mar)),
                PACKAGE="qtl")
    else {
        ## Hide cross scheme in genoprob to pass to routine. BY
        temp <- as.double(rep(0,n.mar*n.mar))
        if(type == "bcsft")
            temp[1:2] <- cross.scheme

        z <- .C(cfunc,
                as.integer(n.ind),         # number of individuals
                as.integer(n.mar),         # number of markers
                as.integer(Geno),
                rf = as.double(temp),
                as.integer(maxit),
                as.double(tol),
                PACKAGE="qtl")
    }

    cross$rf <- matrix(z$rf,ncol=n.mar)
    dimnames(cross$rf) <- list(mar.names,mar.names)

    if(fixX) {
        temp <- as.double(rep(0, ncol(xchr) ^ 2))
        if(type == "bcsft") {
            temp[1] <- cross.scheme[1] + cross.scheme[2] - (cross.scheme[1] == 0)

            zz <- .C(cfunc,
                     as.integer(n.ind),         # number of individuals
                     as.integer(ncol(xchr)),   # number of markers on X chr.
                     as.integer(xchr),
                     rf = as.double(temp),
                     as.integer(maxit),
                     as.double(tol),
                     PACKAGE="qtl")
        }
        else {
            zz <- .C("est_rf_bc",
                     as.integer(n.ind),
                     as.integer(ncol(xchr)),
                     as.integer(xchr),
                     rf=as.double(temp),
                     PACKAGE="qtl")
        }
        zz <- matrix(zz$rf,ncol=ncol(xchr))
        cross$rf[xchrcol,xchrcol] <- zz
    }

    # check for alleles switches
    if(type == "risib" || type=="riself" || type=="f2" || type=="bc" || type=="dh" || type=="haploid") {
        out <- checkAlleles(cross, 5, FALSE)
        if(!is.null(out)) {
            out <- as.character(out[,1])
            warning("Alleles potentially switched at markers \n  ",
                    paste(out, collapse=" "))
        }
    }

    cross
}



plotRF <-
    function(x, chr, what=c("both","lod","rf"),
             alternate.chrid=FALSE, zmax=12,
             mark.diagonal=FALSE,
             col.scheme=c("viridis", "redblue"),
             ...)
{
    if(!inherits(x, "cross"))
        stop("Input should have class \"cross\".")

    what <- match.arg(what)
    if("onlylod" %in% names(attributes(x$rf)) && attr(x$rf, "onlylod")) {
        onlylod <- TRUE
        what <- "lod"
    }
    else onlylod <- FALSE

    if(!missing(chr)) x <- subset(x,chr=chr)

    if(!("rf" %in% names(x))) {
        warning("Running est.rf.")
        x <- est.rf(x)
    }
    g <- x$rf

    old.xpd <- par("xpd")
    old.las <- par("las")
    par(xpd=TRUE,las=1)
    on.exit(par(xpd=old.xpd,las=old.las))

    if(!onlylod) {
        # if any of the rf's are NA (ie no data), put NAs in corresponding LODs
        if(any(is.na(g))) g[is.na(t(g))] <- NA

        # convert rf to -2*(log2(rf)+1); place zmax's on the diagonal;
        #    anything above zmax replaced by zmax;
        #    NA's replaced by -1
        g[row(g) > col(g) & g > 0.5] <- 0.5
        g[row(g) > col(g)] <- -4*(log2(g[row(g) > col(g)])+1)/12*zmax
    }
    diag(g) <- zmax
    g[!is.na(g) & g>zmax] <- zmax

    g[is.na(g)] <- -1

    if(what=="lod" && !onlylod) { # plot LOD scores
        # copy upper triangle (LODs) to lower triangle (rec fracs)
        g[row(g) > col(g)] <- t(g)[row(g) > col(g)]
    }
    else if(what=="rf") { # plot recombination fractions
        # copy lower triangle (rec fracs) to upper triangle (LODs)
        g[row(g) < col(g)] <- t(g)[row(g) < col(g)]
    }
    br <- c(-1, seq(-1e-6, zmax, length=257))

    col.scheme <- match.arg(col.scheme)
    if(col.scheme=="redblue") {
        # convert colors using gamma=0.6 (which will no longer be available in R)
        thecol <- rev(rainbow(256, start=0, end=2/3))
        rgbval <- (col2rgb(thecol)/255)^0.6
        thecol <- rgb(rgbval[1,], rgbval[2,], rgbval[3,])
    } else {
        thecol <- viridis_qtl(256) # the new default
    }

    image(1:ncol(g),1:nrow(g),t(g),ylab="Markers",xlab="Markers",breaks=br,
          col=c("lightgray",thecol))

    if(mark.diagonal) {
        for(i in 1:ncol(g))
            segments(i+c(-0.5, -0.5, -0.5, +0.5), i+c(-0.5, +0.5, -0.5, -0.5),
                     i+c(-0.5, +0.5, +0.5, +0.5), i+c(+0.5, +0.5, -0.5, +0.5))
    }

    # plot lines at the chromosome boundaries
    n.mar <- nmar(x)
    n.chr <- nchr(x)
    a <- c(0.5,cumsum(n.mar)+0.5)
    abline(v=a,xpd=FALSE,col="white")
    abline(h=a,xpd=FALSE,col="white")

    # this line adds a line above the image
    #     (the image function leaves it out)
    abline(h=0.5+c(0,nrow(g)),xpd=FALSE)
    abline(v=0.5+c(0,nrow(g)),xpd=FALSE)

    # add chromosome numbers
    a <- par("usr")
    wh <- cumsum(c(0.5,n.mar))
    chrnam <- names(x$geno)
    chrpos <- (wh[-1] + wh[-length(wh)])/2
    if(!alternate.chrid || length(chrnam) < 2) {
        for(i in seq(along=chrpos)) {
            axis(side=3, at=chrpos[i], labels=chrnam[i], tick=FALSE, line=-0.8)
            axis(side=4, at=chrpos[i], labels=chrnam[i], tick=FALSE, line=-0.8)
        }
    }
    else {
        odd <- seq(1, length(chrpos), by=2)
        even <- seq(2, length(chrpos), by=2)
        for(i in odd) {
            axis(side=3, at=chrpos[i], labels=chrnam[i], line=-0.8, tick=FALSE)
            axis(side=4, at=chrpos[i], labels=chrnam[i], line=-0.8, tick=FALSE)
        }
        for(i in even) {
            axis(side=3, at=chrpos[i], labels=chrnam[i], line=0, tick=FALSE)
            axis(side=4, at=chrpos[i], labels=chrnam[i], line=0, tick=FALSE)
        }

    }


    dots <- list(...)
    if("main" %in% names(dots))
        title(main=dots$main)
    else {
        if(what=="lod") title(main="Pairwise LOD scores")
        else if(what=="rf") title(main="Recombination fractions")
        else title("Pairwise recombination fractions and LOD scores")
    }

    invisible()
}

######################################################################
# check for apparent errors in the recombination fractions
######################################################################
#checkrf <-
#function(cross, threshold=5)
#{
#  rf <- cross$rf
#  n.mar <- nmar(cross)
#  map <- pull.map(cross)
#  n <- ncol(rf)
#  mnam <- colnames(rf)
#  whpos <- unlist(lapply(map,function(a) 1:length(a)))
#  whchr <- rep(names(map),sapply(map,length))
#
#  # first check whether a locus has "significant" pairwise recombination
#  #     with rf > 0.5
#  for(i in 1:n) {
#    if(i == 1) {
#      lod <- rf[1,-1]
#      r <- rf[-1,1]
#    }
#    else if(i == n) {
#      lod <- rf[-n,n]
#      r <- rf[n,-n]
#    }
#    else {
#      lod <- c(rf[1:(i-1),i],rf[i,(i+1):n])
#      r <- c(rf[i,1:(i-1)],rf[(i+1):n,i])
#    }
#
#    # if rf > 1/2 and LOD > threshold for more than two other markers
#    if(sum(!is.na(lod) & !is.na(r) & lod > threshold & r > 0.5) >= 2)
#      warning("Genotypes potentially switched for marker ", mnam[i],
#          paste(" (",whpos[i],")",sep=""), " on chr ", whchr[i], "\n")
#
#  }
#
#}


######################################################################
# checkAlleles()
#
# Function to find markers that may have alleles miscoded;
# we go through each marker, one at a time, swap alleles and
# then see what it does to pairwise linkage against all other
# markers
######################################################################

checkAlleles <-
    function(cross, threshold=3, verbose=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("checkAlleles() only works for cross objects.")

    type <- crosstype(cross)
    if(type != "f2" && type != "bc" &&
       type != "risib" && type != "riself" && type != "dh" && type!="haploid")
        stop("checkAlleles not available for cross type ", type, ".")

    # drop X chromosome
    chr_type <- sapply(cross$geno, chrtype)
    if(all(chr_type=="X")) {
        if(verbose) cat("checkAlleles() only works for autosomal data.\n")
        return(NULL)
    }

    cross <- subset(cross, chr = (chr_type != "X"))

    n.mar <- nmar(cross)
    mar.names <- unlist(lapply(cross$geno,function(a) colnames(a$data)))

    if(!("rf" %in% names(cross))) {
        warning("First running est.rf.")
        cross <- est.rf(cross)
    }
    onlylod <- attr(cross$rf, "onlylod")
    if(!is.null(onlylod) && onlylod) { # need to re-run est.rf()
        warning("First running est.rf.")
        cross <- est.rf(cross)
    }

    diag(cross$rf) <- 0
    lod <- rf <- cross$rf
    lod[lower.tri(lod)] <- t(lod)[lower.tri(lod)]
    rf[upper.tri(rf)] <- t(rf)[upper.tri(rf)]

    orig.lod <- rev.lod <- lod
    orig.lod[rf > 0.5] <- 0
    rev.lod[rf < 0.5] <- 0

    dif <- apply(rev.lod, 2, max, na.rm=TRUE) -
        apply(orig.lod, 2, max, na.rm=TRUE)

    results <- data.frame(marker=mar.names,
                          chr=rep(names(cross$geno), n.mar),
                          index=unlist(lapply(n.mar, function(a) 1:a)),
                          "diff in max LOD" = dif, stringsAsFactors=TRUE)
    rownames(results) <- 1:nrow(results)

    if(all(results[,4] < threshold)) {
        if(verbose) cat("No apparent problems.\n")
        return(invisible(NULL))
    }

    results[results[,4] >= threshold,]
}

######################################################################
# pull.rf
#
# pull out the pairwise marker recombination fraction estimates,
# with class "rfmatrix"
######################################################################

pull.rf <-
    function(cross, what=c("rf", "lod"), chr)
{
    if(!inherits(cross, "cross"))
        stop("Input must have class \"cross\".")

    if(!missing(chr)) cross <- subset(cross, chr=chr)

    if(!("rf" %in% names(cross))) {
        warning(" -Running est.rf")
        cross <- est.rf(cross)
    }
    rf <- cross$rf
    if(nrow(rf) != totmar(cross) || ncol(rf) != totmar(cross) ||
       any(rownames(rf) != markernames(cross)) ||
       any(colnames(rf) != markernames(cross))) {
        warning(" -Rec. frac. estimates seem corrupted; re-running est.rf")
        cross <- est.rf(cross)
    }
    rf <- cross$rf

    diag(rf) <- NA
    what <- match.arg(what)
    if(what=="rf")
        rf[upper.tri(rf)] <- t(rf)[upper.tri(rf)]
    else
        rf[lower.tri(rf)] <- t(rf)[lower.tri(rf)]

    attr(rf, "map") <- pull.map(cross, as.table=TRUE)
    attr(rf, "what") <- what
    class(rf) <- c("rfmatrix", "matrix")
    rf
}

######################################################################
# plot.rfmatrix:
#
# plot a slice through the matrix (that is, for one marker)
######################################################################

plot.rfmatrix <-
    function(x, marker, ...)
{
    if(!inherits(x, "rfmatrix"))
        stop("Input must have class \"rfmatrix\".")

    if(missing(marker))
        stop("You must provide a marker name.")

    if(length(marker) > 1) {
        warning("Ignoring all but the first marker, ", marker[1])
        marker <- marker[1]
    }

    if(!(marker %in% rownames(x)))
        stop("Marker ", marker, " not found.")

    what <- attr(x, "what")
    x <- cbind(attr(x, "map"), x[marker,])
    x$chr <- factor(x$chr, levels=unique(x$chr))

    # fill in hole
    wh <- which(rownames(x)==marker)
    if(wh > 1 && wh < nrow(x) && x[wh-1,1] == x[wh,1] && x[wh+1,1] == x[wh,1]) {
        xl <- x[wh-1,2]
        xm <- x[wh,2]
        xr <- x[wh+1,2]
        yl <- x[wh-1,3]
        yr <- x[wh+1,3]
        x[wh,3] <- yl + (xm-xl)*(yr-yl)/(xr-xl)
    }

    colnames(x)[3] <- what
    class(x) <- c("scanone", "data.frame")

    dots <- list(...)
    if("main" %in% names(dots))
        plot(x, ...)
    else
        plot(x, main=marker, ...)

    invisible(x)
}


# end of est.rf.R

######################################################################
#
# est.map.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Apr, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: est.map
#
######################################################################

######################################################################
#
# est.map: re-estimate the genetic map for an experimental cross
#
######################################################################

est.map <-
    function(cross, chr, error.prob=0.0001, map.function=c("haldane","kosambi","c-f","morgan"),
             m=0, p=0, maxit=10000, tol=1e-6, sex.sp=TRUE, verbose=FALSE,
             omit.noninformative=TRUE, offset, n.cluster=1)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(!missing(chr))
        cross <- subset(cross, chr=chr)

    type <- crosstype(cross)

    if(!missing(offset)) {
        if(length(offset)==1) offset <- rep(offset, nchr(cross))
        else if(length(offset) != nchr(cross))
            stop("offset must have length 1 or n.chr (", nchr(cross), ")")
    }

    if(m < 0 || p < 0 || p > 1)
        stop("Must have m >=0 and 0 <= p <= 1")

    if(m > 0 && p < 1 && type != "bc" && type != "f2") {
        warning("m and p currently used only for backcrosses and intercrosses.")
        m <- p <- 0
    }
    if(m > 0 && p < 1 && !missing(map.function))
        warning("Map function not used with interference model.")
    if(m > 0 && p < 1) interf.model <- TRUE
    else interf.model <- FALSE

    # map function
    map.function <- match.arg(map.function)
    if(map.function=="kosambi") {
        mf <- mf.k; imf <- imf.k
    }
    else if(map.function=="c-f") {
        mf <- mf.cf; imf <- imf.cf
    }
    else if(map.function=="morgan") {
        mf <- mf.m; imf <- imf.m
    }
    else {
        mf <- mf.h; imf <- imf.h
    }

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }

    n.ind <- nind(cross)
    n.mar <- nmar(cross)
    n.chr <- nchr(cross)

    newmap <- vector("list",n.chr)
    names(newmap) <- names(cross$geno)
    chr_type <- sapply(cross$geno, chrtype)

    if(n.cluster > 1 && nchr(cross) > 1) {
        cat(" -Running est.map via a cluster of", n.cluster, "nodes.\n")
        cl <- makeCluster(n.cluster)
        clusterStopped <- FALSE
        on.exit(if(!clusterStopped) stopCluster(cl))
        clusterEvalQ(cl, library(qtl, quietly=TRUE))

        chr <- names(cross$geno)

        # temporary definition of est.map
        temp.est.map <- function(chr, cross, error.prob, map.function, m, p, maxit, tol,
                                 sex.sp, omit.noninformative)
            est.map(cross=cross, chr=chr, error.prob=error.prob, map.function=map.function,
                    m=m, p=p, maxit=maxit, tol=tol, sex.sp=sex.sp, omit.noninformative=omit.noninformative,
                    verbose=FALSE)#, n.cluster=1)

        newmap <- clusterApplyLB(cl, chr, temp.est.map, cross, error.prob, map.function, m, p,
                                 maxit, tol, sex.sp, omit.noninformative)
        for(i in seq(along=newmap)) {
            newmap[[i]] <- newmap[[i]][[1]]
            class(newmap[[i]]) <- class(cross$geno[[i]])
        }

        names(newmap) <- chr

        if(!missing(offset)) {  # shift map start positions
            for(i in seq(along=newmap))
                if(is.matrix(newmap[[i]])) {
                    for(j in 1:2)
                        newmap[[i]][j,] <- newmap[[i]][j,] - newmap[[i]][j,1] + offset[i]
                } else {
                    newmap[[i]] <- newmap[[i]] - newmap[[i]][1] + offset[i]
                }
        }

        class(newmap) <- "map"
        return(newmap)
    }

    # calculate genotype probabilities one chromosome at a time
    for(i in 1:n.chr) {

        if(n.mar[i] < 2) {
            newmap[[i]] <- cross$geno[[i]]$map
            next
        }

        # which type of cross is this?
        if(type == "f2") {
            one.map <- TRUE
            if(chr_type[i] != "X") # autosomal
                cfunc <- "est_map_f2"
            else                              # X chromsome
                cfunc <- "est_map_bc"
        }
        else if(type == "bc" || type=="riself" || type=="risib" || type=="dh" || type=="haploid") {
            one.map <- TRUE
            cfunc <- "est_map_bc"
        }
        else if(type == "4way") {
            one.map <- FALSE
            cfunc <- "est_map_4way"
        }
        else if(type=="ri8sib" || type=="ri4sib" || type=="ri8self" || type=="ri4self" || type=="bgmagic16") {
            cfunc <- paste("est_map_", type, sep="")
            one.map <- TRUE
            if(chr_type[i] == "X")
                warning("est.map not working properly for the X chromosome for 4- or 8-way RIL.")
        }
        else if(type == "bcsft") {
            one.map <- TRUE
            interf.model <- FALSE
            cfunc <- "est_map_bcsft"
            cross.scheme <- attr(cross, "scheme") ## c(s,t) for BC(s)F(t)
            if(chr_type[i] == "X") { # X chromsome
                cross.scheme[1] <- cross.scheme[1] + cross.scheme[2] - (cross.scheme[1] == 0)
                cross.scheme[2] <- 0
            }
            ## Tolerance: need two values.
            if(length(tol) == 1) {
                tol[2] <- 1e-6
            }
        }
        else
            stop("est.map not available for cross type ", type, ".")

        # genotype data
        gen <- cross$geno[[i]]$data
        gen[is.na(gen)] <- 0

        # remove individuals that have less than two typed markers
        if(omit.noninformative) {
            o <- apply(gen,1,function(a) sum(a!=0)>1)
            gen <- gen[o,,drop=FALSE]
        }

        # recombination fractions
        if(one.map) {
            # recombination fractions
            rf <- mf(diff(cross$geno[[i]]$map))
            if(type=="risib" || type=="riself")
                rf <- adjust.rf.ri(rf, sub("^ri", "", type), chr_type[i])
            rf[rf < 1e-14] <- 1e-14
        }
        else {
            orig <- cross$geno[[i]]$map
            # randomize the maps a bit [we no longer do this]
            #      cross$geno[[i]]$map <- cross$geno[[i]]$map +
            #        runif(length(cross$geno[[i]]$map), -0.2, 0.2)

            rf <- mf(diff(cross$geno[[i]]$map[1,]))
            rf[rf < 1e-14] <- 1e-14
            rf2 <- mf(diff(cross$geno[[i]]$map[2,]))
            rf2[rf2 < 1e-14] <- 1e-14
            if(!sex.sp && chr_type[i]=="X")
                temp.sex.sp <- TRUE
            else temp.sex.sp <- sex.sp
        }

        if(interf.model)
            d <- diff(cross$geno[[i]]$map)

        if(verbose) cat(paste("Chr ", names(cross$geno)[i], ":\n",sep=""))

        # call the C function
        if(one.map && !interf.model) {
            ## Hide cross scheme in genoprob to pass to routine. BY
            temp <- 0
            if(type == "bcsft")
                temp[1] <- cross.scheme[1] * 1000 + cross.scheme[2]

            z <- .C(cfunc,
                    as.integer(nrow(gen)),         # number of individuals
                    as.integer(n.mar[i]),      # number of markers
                    as.integer(gen),           # genotype data
                    rf=as.double(rf),          # recombination fractions
                    as.double(error.prob),
                    loglik=as.double(temp),       # log likelihood
                    as.integer(maxit),
                    as.double(tol),
                    as.integer(verbose),
                    PACKAGE="qtl")

            z$rf[z$rf < 1e-14] <- 1e-14
            if(type=="riself" || type=="risib")
                z$rf <- adjust.rf.ri(z$rf, substr(type, 3, nchar(type)),
                                     chr_type[i], expand=FALSE)
            newmap[[i]] <- cumsum(c(min(cross$geno[[i]]$map),imf(z$rf)))
            names(newmap[[i]]) <- names(cross$geno[[i]]$map)
            attr(newmap[[i]],"loglik") <- z$loglik
        }
        else if(interf.model) { # Chi-square / Stahl model
            if(type=="bc" || (type=="f2" && chr_type[i]=="X")) {
                z <- .C("R_est_map_bci",
                        as.integer(nrow(gen)),         # number of individuals
                        as.integer(n.mar[i]),      # number of markers
                        as.integer(gen),           # genotype data
                        d=as.double(d),          # cM distances
                        as.integer(m),
                        as.double(p),
                        as.double(error.prob),
                        loglik=as.double(0),       # log likelihood
                        as.integer(maxit),
                        as.double(tol),
                        as.integer(verbose),
                        PACKAGE="qtl")
            } else {
                z <- .C("R_est_map_f2i",
                        as.integer(nrow(gen)),         # number of individuals
                        as.integer(n.mar[i]),      # number of markers
                        as.integer(gen),           # genotype data
                        d=as.double(d),          # cM distances
                        as.integer(m),
                        as.double(p),
                        as.double(error.prob),
                        loglik=as.double(0),       # log likelihood
                        as.integer(maxit),
                        as.double(tol),
                        as.integer(verbose),
                        PACKAGE="qtl")
            }
            z$d[z$d < 1e-14] <- 1e-14
            newmap[[i]] <- cumsum(c(min(cross$geno[[i]]$map),z$d))
            names(newmap[[i]]) <- names(cross$geno[[i]]$map)
            attr(newmap[[i]], "loglik") <- z$loglik
            attr(newmap[[i]], "m") <- m
            attr(newmap[[i]], "p") <- p
        }
        else {
            z <- .C(cfunc,
                    as.integer(nrow(gen)),         # number of individuals
                    as.integer(n.mar[i]),      # number of markers
                    as.integer(gen),           # genotype data
                    rf=as.double(rf),          # recombination fractions
                    rf2=as.double(rf2),        # recombination fractions
                    as.double(error.prob),
                    loglik=as.double(0),       # log likelihood
                    as.integer(maxit),
                    as.double(tol),
                    as.integer(temp.sex.sp),
                    as.integer(verbose),
                    PACKAGE="qtl")

            z$rf[z$rf<1e-14] <- 1e-14
            z$rf2[z$rf2<1e-14] <- 1e-14

            if(!temp.sex.sp) z$rf2 <- z$rf

            newmap[[i]] <- rbind(cumsum(c(min(orig[1,]),imf(z$rf))),
                                 cumsum(c(min(orig[2,]),imf(z$rf2))))
            dimnames(newmap[[i]]) <- dimnames(cross$geno[[i]]$map)
            attr(newmap[[i]],"loglik") <- z$loglik
        }

        class(newmap[[i]]) <- chr_type[i]
    } # end loop over chromosomes


    if(!missing(offset)) {  # shift map start positions
        for(i in seq(along=newmap))
            if(is.matrix(newmap[[i]])) {
                for(j in 1:2)
                    newmap[[i]][j,] <- newmap[[i]][j,] - newmap[[i]][j,1] + offset[i]
            } else {
                newmap[[i]] <- newmap[[i]] - newmap[[i]][1] + offset[i]
            }
    }


    class(newmap) <- "map"
    newmap
}

# end of est.map.R

#####################################################################
#
# plot.scanone.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: plot.scanone,
#
######################################################################

######################################################################
#
# plot.scanone: plot output from scanone
#
######################################################################

plot.scanone <-
    function(x,x2,x3,chr,lodcolumn=1,incl.markers=TRUE,xlim, ylim,
             lty=1,col=c("black","blue","red"),lwd=2,add=FALSE,gap=25,
             mtick=c("line", "triangle"), show.marker.names=FALSE,
             alternate.chrid=FALSE, bandcol=NULL, type="l", cex=1,
             pch=1, bg="transparent", bgrect=NULL, ...)
{
    if(!inherits(x, "scanone") ||
       (!missing(x2) && !inherits(x2, "scanone")) ||
       (!missing(x3) && !inherits(x3, "scanone")))
        stop("Input should have class \"scanone\".")

    if(!is.factor(x$chr)) x$chr <- factor(x$chr, levels=unique(x$chr))

    dots <- list(...)

    # handle special arguments to be used in lines()
    if(length(type)==1) type <- rep(type, 3)
    if(length(cex)==1) cex <- rep(cex, 3)
    if(length(pch)==1) pch <- rep(pch, 3)
    if(length(bg)==1) bg <- rep(bg, 3)

    mtick <- match.arg(mtick)

    if(length(dim(x))!=2)
        stop("Argument x must be a matrix or data.frame.")
    if(!missing(x2) && length(dim(x2))!=2)
        stop("Argument x2 must be a matrix or data.frame.")
    if(!missing(x3) && length(dim(x3))!=2)
        stop("Argument x3 must be a matrix or data.frame.")

    if(length(lodcolumn)==1)
        lodcolumn <- rep(lodcolumn,3)[1:3]
    else if(length(lodcolumn)==2) {
        if(missing(x2)) x2 <- x
        lodcolumn <- lodcolumn[c(1,2,3)]
    }
    else {
        if(missing(x2)) x2 <- x
        if(missing(x3)) x3 <- x
    }
    lodcolumn <- lodcolumn+2

    second <- third <- TRUE
    if(missing(x2) && missing(x3))
        second <- third <- FALSE
    if(missing(x3))
        third <- FALSE
    if(missing(x2))
        second <- FALSE

    # rename things and turn into data frames
    if(lodcolumn[1] > ncol(x) ||
       (second && lodcolumn[2] > ncol(x2)) ||
       (third && lodcolumn[3] > ncol(x3)))
        stop("Argument lodcolumn misspecified.")

    out <- x[,c(1:2,lodcolumn[1])]
    if(second) out2 <- x2[,c(1:2,lodcolumn[2])]
    if(third) out3 <- x3[,c(1:2,lodcolumn[3])]
    if(length(lty)==1) lty <- rep(lty,3)
    if(length(lwd)==1) lwd <- rep(lwd,3)
    if(length(col)==1) col <- rep(col,3)

    # pull out desired chromosomes
    if(missing(chr) || length(chr)==0)
        chr <- unique(as.character(out[,1]))
    else chr <- matchchr(chr, unique(out[,1]))

    out <- out[!is.na(match(out[,1],chr)),]
    if(second) out2 <- out2[!is.na(match(out2[,1],chr)),]
    if(third) out3 <- out3[!is.na(match(out3[,1],chr)),]

    onechr <- FALSE
    if(length(chr) == 1) {
        gap <- 0
        onechr <- TRUE
    }

    # beginning and end of chromosomes
    temp <- out
    begend <- matrix(unlist(tapply(temp[,2],temp[,1],range)),ncol=2,byrow=TRUE)
    rownames(begend) <- unique(out[,1])
    begend <- begend[as.character(chr),,drop=FALSE]
    len <- begend[,2]-begend[,1]

    # locations to plot start of each chromosome
    if(!onechr)
        start <- c(0,cumsum(len+gap))-c(begend[,1],0)
    else start <- 0

    maxx <- sum(len+gap)-gap

    # replace +/- Inf with NA
    out[!is.finite(out[,3]),3] <- NA
    if(second) out2[!is.finite(out2[,3]),3] <- NA
    if(third) out3[!is.finite(out3[,3]),3] <- NA

    # get max y-axis value
    if(all(is.na(out[,3]))) maxy <- 1
    else maxy <- max(out[,3],na.rm=TRUE)
    if(second) maxy <- max(c(maxy,out2[,3]),na.rm=TRUE)
    if(third) maxy <- max(c(maxy,out3[,3]),na.rm=TRUE)

    # graphics parameters
    old.xpd <- par("xpd")
    old.las <- par("las")
    par(xpd=FALSE,las=1)
    on.exit(par(xpd=old.xpd,las=old.las))

    # make frame of plot
    if(missing(ylim)) ylim <- c(0,maxy)
    if(missing(xlim)) {
        if(onechr) xlim <- c(0,max(out[,2]))
        else xlim <- c(-gap/2,maxx+gap/2)
    }

    if(!add) {
        if(onechr) {

            if("ylab" %in% names(dots)) {
                if("xlab" %in% names(dots)) {
                    plot(0,0,ylim=ylim,xlim=xlim,type="n",...)
                }
                else {
                    plot(0,0,ylim=ylim,xlim=xlim,type="n",
                         xlab="Map position (cM)",...)
                }
            }
            else {
                if("xlab" %in% names(dots)) {
                    plot(0,0,ylim=ylim,xlim=xlim,type="n",
                         ylab=dimnames(out)[[2]][3],
                         ...)
                }
                else {
                    plot(0,0,ylim=ylim,xlim=xlim,type="n",
                         xlab="Map position (cM)",ylab=dimnames(out)[[2]][3],
                         ...)
                }
            }
        }
        else {
            if("ylab" %in% names(dots)) {
                if("xlab" %in% names(dots)) {
                    plot(0,0,ylim=ylim,xlim=xlim,type="n",xaxt="n",
                         xaxs="i", ...)
                }
                else {
                    plot(0,0,ylim=ylim,xlim=xlim,type="n",xaxt="n",
                         xlab="Chromosome", xaxs="i", ...)
                }
            }
            else {
                if("xlab" %in% names(dots)) {
                    plot(0,0,ylim=ylim,xlim=xlim,type="n",xaxt="n",
                         ylab=dimnames(out)[[2]][3], xaxs="i",
                         ...)
                }
                else {
                    plot(0,0,ylim=ylim,xlim=xlim,type="n",xaxt="n",
                         xlab="Chromosome",ylab=dimnames(out)[[2]][3], xaxs="i",
                         ...)
                }
            }
        }
    }

    if(!add && !is.null(bgrect)) {
        u <- par("usr")
        rect(u[1], u[3], u[2], u[4], col=bgrect)
    }
    if(!add && !onechr && !is.null(bandcol)) {
        u <- par("usr")
        for(i in seq(2, by=2, length(chr))) {
            rect(min(out[out[,1]==chr[i],2]) + start[i]-gap/2, u[3],
                 max(out[out[,1]==chr[i],2]) + start[i]+gap/2, u[4], border=bandcol, col=bandcol)
        }
        abline(h=u[3:4])
    }

    # initialize xtick and xtickmark
    xtick <- NULL
    xticklabel <- NULL
    for(i in 1:length(chr)) {
        # plot first out
        x <- out[out[,1]==chr[i],2]+start[i]
        y <- out[out[,1]==chr[i],3]
        if(length(x)==1) {
            g <- max(gap/10,2)
            x <- c(x-g,x,x+g)
            y <- rep(y,3)
        }
        lines(x,y,lwd=lwd[1],lty=lty[1],col=col[1], type=type[1], cex=cex[1], pch=pch[1], bg=bg[1])
        # plot chromosome number
        if(!add && !onechr) {
            tloc <- mean(c(min(x),max(x)))
            xtick <- c(xtick, tloc)
            xticklabel <- c(xticklabel, as.character(chr[i]))
        }

        # plot second out
        if(second) {
            x <- out2[out2[,1]==chr[i],2]+start[i]
            y <- out2[out2[,1]==chr[i],3]
            if(length(x)==1) {
                g <- max(gap/10,2)
                x <- c(x-g,x,x+g)
                y <- rep(y,3)
            }
            lines(x,y,lty=lty[2],col=col[2],lwd=lwd[2], type=type[2], cex=cex[2], pch=pch[2], bg=bg[2])
        }

        if(third) {
            x <- out3[out3[,1]==chr[i],2]+start[i]
            y <- out3[out3[,1]==chr[i],3]
            if(length(x)==1) {
                g <- max(gap/10,2)
                x <- c(x-g,x,x+g)
                y <- rep(y,3)
            }
            lines(x,y,lty=lty[3],col=col[3],lwd=lwd[3], type=type[3], cex=cex[3], pch=pch[3], bg=bg[3])
        }

        # plot lines or triangles at marker positions
        if(!add) {
            nam <- dimnames(out)[[1]][out[,1]==chr[i]]
            wh.genoprob <- grep("^c.+\\.loc-*[0-9]+", nam)
            if(length(wh.genoprob)==0) wh.genoprob <- seq(along=nam)
            else wh.genoprob <- (seq(along=nam))[-wh.genoprob]
            pos <- out[out[,1]==chr[i],2][wh.genoprob]+start[i]

            if(incl.markers) {
                if(mtick=="line")
                    rug(pos, 0.02, quiet=TRUE)
                else {
                    a <- par("usr")
                    points(pos, rep(a[3]+diff(a[3:4])*0.01, length(pos)), pch=17, cex=1.5)
                }
            }

            if(show.marker.names) {
                a <- par("usr")
                text(pos, rep(a[3]+diff(a[3:4])*0.03, length(pos)), nam[wh.genoprob],
                     srt=90, adj=c(0,0.5))
            }
        }
    }

    # draw the axis
    if(!add && !onechr) {
        if(!alternate.chrid || length(xtick) < 2) {
            for(i in seq(along=xtick))
                axis(side=1, at=xtick[i], labels=xticklabel[i])
        }
        else {
            odd <- seq(1, length(xtick), by=2)
            even <- seq(2, length(xtick), by=2)
            for(i in odd) {
                axis(side=1, at=xtick[i], labels="")
                axis(side=1, at=xtick[i], labels=xticklabel[i], line=-0.4, tick=FALSE)
            }
            for(i in even) {
                axis(side=1, at=xtick[i], labels="")
                axis(side=1, at=xtick[i], labels=xticklabel[i], line=+0.4, tick=FALSE)
            }
        }
    }

    if(!add && !is.null(bgrect)) {
        u <- par("usr")
        rect(u[1], u[3], u[2], u[4])
    }

    invisible()
}

# end of plot.scanone.R

######################################################################
#
# ripple.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Oct, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: ripple, summary.ripple, print.summary.ripple
#           ripple.perm1, ripple.perm2, ripple.perm.sub
#
######################################################################

######################################################################
#
# ripple: Check marker orders for a given chromosome, comparing all
#         possible permutations of a sliding window of markers
#
######################################################################

ripple <-
    function(cross, chr, window=4, method=c("countxo","likelihood"),
             error.prob=0.0001, map.function=c("haldane","kosambi","c-f","morgan"),
             maxit=4000, tol=1e-6, sex.sp=TRUE, verbose=TRUE, n.cluster=1)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    # pull out relevant chromosome
    if(missing(chr)) {
        chr <- names(cross$geno)[1]
        warning("chr argument not provided; assuming you want chr ", chr)
    }
    else {
        if(length(chr) > 1)
            stop("ripple only works for one chromosome at a time.")
        if(!testchr(chr, names(cross$geno)))
            stop("Chr ", chr, " not found.")
    }
    cross <- subset(cross,chr=chr)
    chr.name <- names(cross$geno)[1]

    if(nmar(cross)[1] < 3) {
        warning("Less than three markers.")
        return(NULL)
    }

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }

    # make sure window is an integer >= 2
    if(window < 2) {
        warning("The window argument must be > 1; using window=2.")
        window <- 2
    }
    window <- round(window)

    method <- match.arg(method)
    map.function <- match.arg(map.function)

    # get marker orders to test
    n.mar <- totmar(cross)
    if(n.mar <= window) # look at all possible orders
        orders <- ripple.perm2(n.mar)
    else {
        temp <- ripple.perm1(window)
        n <- nrow(temp)
        orders <- cbind(temp,matrix(rep((window+1):n.mar,n),
                                    byrow=TRUE,ncol=n.mar-window))
        for(i in 2:(n.mar-window+1)) {
            left <- matrix(rep(1:(i-1),n),byrow=TRUE,ncol=i-1)
            if(i < n.mar-window+1)
                right <- matrix(rep((i+window):n.mar,n),byrow=TRUE,ncol=n.mar-window-i+1)
            else
                right <- NULL
            orders <- rbind(orders,cbind(left,temp+i-1,right))
        }
        # keep only distinct orders
        orders <- as.numeric(unlist(strsplit(unique(apply(orders,1,paste,collapse=":")),":")))
        orders <- matrix(orders,ncol=n.mar,byrow=TRUE)
    }
    n.orders <- nrow(orders)


    # how often to print information about current order being considered
    if(n.orders > 49) print.by <- 10
    else if(n.orders > 14) print.by <- 5
    else print.by <- 2

    if(method=="likelihood") {
        # calculate log likelihoods (and est'd chr length) for each marker order
        loglik <- 1:n.orders
        chrlen <- 1:n.orders

        # create temporary cross
        m <- seq(0,by=5,length=n.mar)
        temcross <- cross
        if(is.matrix(cross$geno[[1]]$map))
            temcross$geno[[1]]$map <- rbind(m,m)
        else temcross$geno[[1]]$map <- m

        if(verbose) cat("  ", n.orders,"total orders\n")
        if(n.cluster > 1) {
            # parallelize
            if(n.orders <= n.cluster) n.cluster <- n.orders
            cl <- makeCluster(n.cluster)
            clusterStopped <- FALSE
            on.exit(if(!clusterStopped) stopCluster(cl))
            if(verbose) cat("   Running in", n.cluster, "clusters\n")
            clusterEvalQ(cl, library(qtl, quietly=TRUE))

            whclust <- sort(rep(1:n.cluster, ceiling(n.orders/n.cluster))[1:n.orders])
            order.split <- vector("list", n.cluster)
            for(i in 1:n.cluster)
                order.split[[i]] <- orders[whclust==i,,drop=FALSE]
            result <- parLapply(cl, order.split, rippleSnowLik, cross=temcross,
                                error.prob=error.prob, map.function=map.function, maxit=maxit, tol=tol,
                                sex.sp=sex.sp)
            loglik <- unlist(lapply(result, function(a) a$loglik))
            chrlen <- unlist(lapply(result, function(a) a$chrlen))
        }
        else {
            for(i in 1:n.orders) {
                if(verbose && (i %/% print.by)*print.by == i) cat("    --Order", i, "\n")
                temcross$geno[[1]]$data <- cross$geno[[1]]$data[,orders[i,]]
                newmap <- est.map(temcross, error.prob=error.prob, map.function=map.function,
                                  m=0, p=0, maxit=maxit, tol=tol, sex.sp=sex.sp, verbose=FALSE)
                loglik[i] <- attr(newmap[[1]],"loglik")
                chrlen[i] <- diff(range(newmap[[1]]))
            }
        }

        # re-scale log likelihoods and convert to lods
        loglik <- (loglik - loglik[1])/log(10)

        # sort orders by lod
        o <- order(loglik[-1], decreasing=TRUE)+1

        # create output
        orders <- cbind(orders,LOD=loglik,chrlen)[c(1,o),]
    }
    else { # count obligate crossovers for each order
        # which type of cross is this?
        type <- crosstype(cross)
        is.bcs <- type == "bcsft"
        if(is.bcs)
            is.bcs <- (attr(cross, "scheme")[2] == 0)

        if(type == "f2" || (type == "bcsft" && !is.bcs)) {
            if(chrtype(cross$geno[[1]]) == "A") # autosomal
                func <- "R_ripple_f2"
            else func <- "R_ripple_bc"        # X chromsome
        }
        else if(type %in% c("bc", "riself", "risib", "dh", "haploid", "bcsft")) func <- "R_ripple_bc"
        else if(type == "4way") func <- "R_ripple_4way"
        else if(type=="ri4self" || type=="ri8self" || type=="ri4sib" || type=="ri8sib" || type=="bgmagic16")
            func <- "R_ripple_ril48"
        else
            stop("ripple not available for cross ", type)

        # data to be input
        genodat <- cross$geno[[1]]$data
        genodat[is.na(genodat)] <- 0
        n.ind <- nind(cross)

        if(verbose) cat("  ", n.orders,"total orders\n")
        if(n.cluster > 1) {
            # parallelize
            if(n.orders <= n.cluster) n.cluster <- n.orders

            cl <- makeCluster(n.cluster)
            clusterStopped <- FALSE
            on.exit(if(!clusterStopped) stopCluster(cl))
            if(verbose) cat("   Running in", n.cluster, "clusters\n")
            clusterEvalQ(cl, library(qtl, quietly=TRUE))

            whclust <- sort(rep(1:n.cluster, ceiling(n.orders/n.cluster))[1:n.orders])
            order.split <- vector("list", n.cluster)
            for(i in 1:n.cluster)
                order.split[[i]] <- orders[whclust==i,,drop=FALSE]
            oblxo <- unlist(parLapply(cl, order.split, rippleSnowCountxo, genodat=genodat, func=func))
            stopCluster(cl)
            clusterStopped <- TRUE
        }
        else {
            z <- .C(func,
                    as.integer(n.ind),
                    as.integer(n.mar),
                    as.integer(genodat),
                    as.integer(n.orders),
                    as.integer(orders-1),
                    oblxo=as.integer(rep(0,n.orders)),
                    as.integer(print.by),
                    PACKAGE="qtl")
            oblxo <- z$oblxo
        }
        # sort orders by lod
        o <- order(oblxo[-1])+1

        # create output
        orders <- cbind(orders,obligXO=oblxo)[c(1,o),]
    }

    rownames(orders) <- c("Initial", paste(1:(nrow(orders)-1)))
    class(orders) <- c("ripple","matrix")
    attr(orders,"chr") <- chr.name
    attr(orders,"window") <- window
    attr(orders,"error.prob") <- error.prob
    attr(orders,"method") <- method

    # make sure, for each order considered, that the proximal marker
    # (in the original order) is to the left of the distal marker
    # (in the original order)
    orders[,1:n.mar] <- t(apply(orders[,1:n.mar,drop=FALSE],1,
                                function(a) {
                                    n <- length(a)
                                    if((1:n)[a==1] > (1:n)[a==n]) return(rev(a))
                                    else return(a) }))

    orders
}

######################################################################
# function for method="likelihood", for parallel processing (formerly with snow pkg)
######################################################################
rippleSnowLik <-
    function(orders, cross, error.prob, map.function, maxit, tol, sex.sp)
{
    n.orders <- nrow(orders)
    temcross <- cross
    loglik <- chrlen <- rep(NA, n.orders)
    for(i in 1:n.orders) {

        temcross$geno[[1]]$data <- cross$geno[[1]]$data[,orders[i,]]
        newmap <- est.map(temcross, error.prob=error.prob, map.function=map.function,
                          m=0, p=0, maxit=maxit, tol=tol, sex.sp=sex.sp, verbose=FALSE)
        loglik[i] <- attr(newmap[[1]],"loglik")
        chrlen[i] <- diff(range(newmap[[1]]))
    }
    list(loglik=loglik, chrlen=chrlen)
}



######################################################################
# function for method="countxo", for parallel processing (formerly with snow pkg)
######################################################################
rippleSnowCountxo <-
    function(orders, genodat, func)
{
    func <- func # this avoids a Note from R CMD check
    .C(func,
       as.integer(nrow(genodat)),
       as.integer(ncol(genodat)),
       as.integer(genodat),
       as.integer(nrow(orders)),
       as.integer(orders-1),
       oblxo=as.integer(rep(0, nrow(orders))),
       as.integer(0),
       PACKAGE="qtl")$oblxo
}


######################################################################
#
# summary.ripple: print top results from ripple().  We do this so
#                 that we can return *all* results but allow easy
#                 view of only the important ones
#
######################################################################

summary.ripple <-
    function(object, lod.cutoff = -1, ...)
{
    if(!inherits(object, "ripple"))
        stop("Input should have class \"ripple\".")

    n <- ncol(object)

    if("obligXO" %in% colnames(object)) # counts of crossovers
        o <- (object[-1,n] <= (object[1,n] - lod.cutoff*2))
    else o <- (object[-1,n-1] >= lod.cutoff) # likelihood analysis

    if(!any(o)) object <- object[1:2,,drop=FALSE]
    else  # make sure first row is included
        object <- object[c(TRUE,o),,drop=FALSE]

    rownames(object) <- c("Initial ", paste(1:(nrow(object)-1)))
    class(object) <- c("summary.ripple","matrix")
    object
}

######################################################################
#
# print.summary.ripple
#
######################################################################

print.summary.ripple <-
    function(x, ...)
{
    n <- ncol(x)
    x <- round(x,1)
    max.row <- 6

    if(!("obligXO" %in% colnames(x)))
        colnames(x)[n-1] <- "    LOD"

    class(x) <- "matrix"


    if(nrow(x) > max.row) {
        print(x[1:max.row,])
        cat("... [", nrow(x)-max.row, " additional rows] ...\n")
    }
    else print(x)
}

######################################################################
#
# ripple.perm1: Utility function for ripple().  Returns all possible
#               permutations of {1, 2, ..., n}
#
######################################################################

ripple.perm1 <-
    function(n)
{
    if(n == 1) return(rbind(1))
    o <- rbind(c(n-1,n),c(n,n-1))
    if(n > 2)
        for(i in (n-2):1)
            o <- ripple.perm.sub(i,o)
    dimnames(o) <- NULL
    o
}

######################################################################
#
# ripple.perm2: Utility function for ripple().  Returns all possible
#               permutations of {1, 2, ..., n}, up to orientation of
#               the entire group
#
######################################################################

ripple.perm2 <-
    function(n)
{
    if(n < 3) return(rbind(1:n))
    o <- rbind(c(n-2,n-1,n),c(n-1,n-2,n),c(n-1,n,n-2))
    if(n > 3)
        for(i in (n-3):1)
            o <- ripple.perm.sub(i,o)
    dimnames(o) <- NULL
    o
}

######################################################################
#
# ripple.perm.sub: Subroutine used for ripple().  I'm too tired to
#                  explain.
#
######################################################################

ripple.perm.sub <-
    function(x,mat)
{
    res <- cbind(x,mat)
    if(ncol(mat) > 1) {
        for(i in 1:ncol(mat))
            res <- rbind(res,cbind(mat[,1:i],x,mat[,-(1:i)]))
    }
    res
}

# end of ripple.R

######################################################################
#
# fitqtl.R
#
# copyright (c) 2002-2019, Hao Wu and Karl W. Broman
# last modified Dec, 2019
# first written Apr, 2002
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: fitqtl, fitqtlengine, parseformula, summary.fitqtl,
#           print.summary.fitqtl, mybinaryrep, deparseQTLformula
#           printQTLformulanicely
#
######################################################################

######################################################################
#
# This is the function to fit a model and generate some tables
#
# Only Haley-Knott regression and multiple imputation are implemented.
#
######################################################################

fitqtl <-
    function(cross, pheno.col=1, qtl, covar=NULL, formula, method=c("imp", "hk"),
             model=c("normal", "binary"), dropone=TRUE, get.ests=FALSE,
             run.checks=TRUE, tol=1e-4, maxit=1000, forceXcovar=FALSE)
{
    # some input checking stuff in here
    if( !inherits(cross, "cross") )
        stop("The cross argument must be an object of class \"cross\".")

    if( !inherits(qtl, "qtl") )
        stop("The qtl argument must be an object of class \"qtl\".")

    if(!is.null(covar) && !is.data.frame(covar)) {
        if(is.matrix(covar) && is.numeric(covar))
            covar <- as.data.frame(covar, stringsAsFactors=TRUE)
        else stop("covar should be a data.frame")
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("fitqtl can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 | pheno.col > nphe(cross))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    pheno <- cross$pheno[,pheno.col]
    if(!is.null(covar) && nrow(covar) != length(pheno))
        stop("nrow(covar) != no. individuals in cross.")

    method <- match.arg(method)
    model <- match.arg(model)

    if(model=="binary" && any(!is.na(pheno) & pheno != 0 & pheno != 1))
        stop("For model=\"binary\", phenotypes must by 0 or 1.")

    # allow formula to be a character string
    if(!missing(formula) && is.character(formula))
        formula <- as.formula(formula)

    if(method=="imp") {
        if(!("geno" %in% names(qtl))) {
            if("prob" %in% names(qtl)) {
                warning("The qtl object doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"draws\".")
        }
    }
    else {
        if(!("prob" %in% names(qtl))) {
            if("geno" %in% names(qtl)) {
                warning("The qtl object doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"prob\".")
        }
    }

    if(qtl$n.ind != nind(cross)) {
        warning("No. individuals in qtl object doesn't match that in the input cross; re-creating qtl object.")
        if(method=="imp")
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        else
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="prob")
    }
    if(method=="imp" && dim(qtl$geno)[3] != dim(cross$geno[[1]]$draws)[3])  {
        warning("No. imputations in qtl object doesn't match that in the input cross; re-creating qtl object.")
        qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
    }

    fitqtlengine(pheno=pheno, qtl=qtl, covar=covar, formula=formula,
                 method=method, model=model, dropone=dropone, get.ests=get.ests,
                 run.checks=run.checks, cross.attr=attributes(cross), crosstype=crosstype(cross),
                 sexpgm=getsex(cross), tol=tol, maxit=maxit, forceXcovar=forceXcovar)
}


fitqtlengine <-
    function(pheno, qtl, covar=NULL, formula, method=c("imp", "hk"),
             model=c("normal", "binary"),
             dropone=TRUE, get.ests=FALSE, run.checks=TRUE, cross.attr,
             crosstype, sexpgm, tol, maxit, forceXcovar=FALSE)
{
    model <- match.arg(model)
    method <- match.arg(method)

    # local variables
    n.ind <- qtl$n.ind # number of individuals
    n.qtl <- qtl$n.qtl # number of selected markers
    n.gen <- qtl$n.gen # number of genotypes
    if(method=="imp")
        n.draws <- dim(qtl$geno)[3] # number of draws

    if( is.null(covar) )  # number of covariates
        n.covar <- 0
    else
        n.covar <- ncol(covar)

    # if formula is missing, build one
    # all QTLs and covariates will be additive by default
    if(missing(formula)) {
        tmp.Q <- paste("Q", 1:n.qtl, sep="") # QTL term names
        formula <- "y~Q1"
        if(n.qtl > 1)
            for (i in 2:n.qtl)
                formula <- paste(formula, tmp.Q[i], sep="+")
        if (n.covar) { # if covariate is not empty
            tmp.C <- colnames(covar) # covariate term names
            for(i in 1:n.covar)
                formula <- paste(formula, tmp.C[i], sep="+")
        }
        formula <- as.formula(formula)
    }

    # check input formula
    if(run.checks) {
        formula <- checkformula(formula, qtl$altname, colnames(covar))

        if(qtl$n.ind != length(pheno))
            stop("No. individuals in qtl object doesn't match length of input phenotype.")

        # drop covariates that are not in the formula
        if(!is.null(covar)) {
            theterms <- rownames(attr(terms(formula), "factors"))
            m <- match(colnames(covar), theterms)
            if(all(is.na(m))) covar <- NULL
            else covar <- covar[,!is.na(m),drop=FALSE]
        }

        # check phenotypes and covariates; drop ind'ls with missing values
        if(!is.null(covar)) phcovar <- cbind(pheno, covar)
        else phcovar <- as.data.frame(pheno, stringsAsFactors=TRUE)
        if(any(is.na(phcovar))) {
            if(ncol(phcovar)==1) hasmissing <- is.na(phcovar)
            else hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
            if(all(hasmissing))
                stop("All individuals are missing phenotypes or covariates.")
            if(any(hasmissing)) {
                warning("Dropping ", sum(hasmissing), " individuals with missing phenotypes.\n")
                pheno <- pheno[!hasmissing]
                n.ind <- qtl$n.ind <- sum(!hasmissing)
                if(method=="imp")
                    qtl$geno <- qtl$geno[!hasmissing,,,drop=FALSE]
                else
                    qtl$prob <- lapply(qtl$prob, function(a) a[!hasmissing,,drop=FALSE])

                if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]

                # subset sexpgm
                for(i in seq(along=sexpgm))
                    if(!is.null(sexpgm[[i]]))
                        sexpgm[[i]] <- sexpgm[[i]][!hasmissing]
            }
        }
    }

    # parse the input formula
    p <- parseformula(formula, qtl$altname, colnames(covar))
    # make an array n.gen.QC to represent the genotype numbers
    # for all input QTLs and covariates. For covariates the
    # number of genotyps is 1. This makes programming easier
    n.gen.QC <- c(n.gen[p$idx.qtl]-1, rep(1, p$n.covar))

    # covariates to be passed to C function
    # This is done in case of that user input covar but has no covar in formula
    covar.C <- NULL
    if(!is.null(p$idx.covar))
        covar.C <- as.matrix(covar[,p$idx.covar,drop=FALSE])

    sizefull <- 1+sum(n.gen.QC)
    if(p$n.int > 0) {
        form <- p$formula.intmtx*n.gen.QC
        if(!is.matrix(form)) {
            sizefull <- sizefull + prod(form[form!=0])
        }
        else {
            form <- apply(form,2,function(a) prod(a[a != 0]))
            sizefull <- sizefull + sum(form)
        }
    }

    if(method != "imp") {
        # form genotype probabilities as a matrix
        prob <- matrix(ncol=sum(qtl$n.gen[p$idx.qtl]), nrow=n.ind)

        curcol <- 0
        for(i in p$idx.qtl) {
            prob[,curcol+1:n.gen[i]] <- qtl$prob[[i]]
            curcol <- curcol + n.gen[i]
        }
    }

    Xadjustment <- scanoneXnull(crosstype, sexpgm, cross.attr)
    n.origcovar <- p$n.covar
    if((sum(qtl$chrtype[p$idx.qtl]=="X") >= 1 || forceXcovar) && Xadjustment$adjustX) { # need to include X chromosome covariates
        adjustX <- TRUE

        n.newcovar <- ncol(Xadjustment$sexpgmcovar)
        n.gen.QC <- c(n.gen.QC, rep(1, n.newcovar))
        p$n.covar <- p$n.covar + n.newcovar
        covar.C <- cbind(covar.C, Xadjustment$sexpgmcovar)
        sizefull <- sizefull + n.newcovar

        if(p$n.int==1)
            p$formula.intmtx <- c(p$formula.intmtx, rep(0,n.newcovar))
        if(p$n.int>1) {
            for(i in 1:n.newcovar)
                p$formula.intmtx <- rbind(p$formula.intmtx, rep(0,p$n.int))
        }
    }
    else adjustX <- FALSE

    # call C function to do the genome scan
    if(model=="normal") {
        if(method == "imp") {
            z <- .C("R_fitqtl_imp",
                    as.integer(n.ind), # number of individuals
                    as.integer(p$n.qtl), # number of qtls
                    as.integer(n.gen.QC), # number of genotypes QTLs and covariates
                    as.integer(n.draws), # number of draws
                    as.integer(qtl$geno[,p$idx.qtl,]), # genotypes for selected marker
                    as.integer(p$n.covar), # number of covariate
                    as.double(covar.C), # covariate
                    as.integer(p$formula.intmtx),  # formula matrix for interactive terms
                    as.integer(p$n.int), # number of interactions in the formula
                    as.double(pheno), # phenotype
                    as.integer(get.ests), # get estimates?
                    # return variables
                    lod=as.double(0), # LOD score
                    df=as.integer(0), # degree of freedom
                    ests=as.double(rep(0,sizefull)),
                    ests.cov=as.double(rep(0,sizefull*sizefull)),
                    design.mat=as.double(rep(0,sizefull*n.ind)),
                    matrix.rank=as.integer(0), # on return, minimum of matrix rank across imputations
                    resid=as.double(rep(0,n.ind)), # on return, the average residuals across imputations
                    PACKAGE="qtl")
        }
        else {
            z <- .C("R_fitqtl_hk",
                    as.integer(n.ind), # number of individuals
                    as.integer(p$n.qtl), # number of qtls
                    as.integer(n.gen.QC), # number of genotypes QTLs and covariates
                    as.double(prob),      # QTL genotype probabilities
                    as.integer(p$n.covar), # number of covariate
                    as.double(covar.C), # covariates
                    as.integer(p$formula.intmtx),  # formula matrix for interactive terms
                    as.integer(p$n.int), # number of interactions in the formula
                    as.double(pheno), # phenotype
                    as.integer(get.ests), # get estimates?
                    # return variables
                    lod=as.double(0), # LOD score
                    df=as.integer(0), # degree of freedom
                    ests=as.double(rep(0,sizefull)),
                    ests.cov=as.double(rep(0,sizefull*sizefull)),
                    design.mat=as.double(rep(0,sizefull*n.ind)),
                    matrix.rank=as.integer(0), # on return, rank of matrix
                    resid=as.double(rep(0,n.ind)), # on return, residuals from the fit
                    PACKAGE="qtl")
        }
    }
    else {
        if(method=="imp") {
            z <- .C("R_fitqtl_imp_binary",
                    as.integer(n.ind), # number of individuals
                    as.integer(p$n.qtl), # number of qtls
                    as.integer(n.gen.QC), # number of genotypes QTLs and covariates
                    as.integer(n.draws), # number of draws
                    as.integer(qtl$geno[,p$idx.qtl,]), # genotypes for selected marker
                    as.integer(p$n.covar), # number of covariate
                    as.double(covar.C), # covariate
                    as.integer(p$formula.intmtx),  # formula matrix for interactive terms
                    as.integer(p$n.int), # number of interactions in the formula
                    as.double(pheno), # phenotype
                    as.integer(get.ests), # get estimates?
                    # return variables
                    lod=as.double(0), # LOD score
                    df=as.integer(0), # degree of freedom
                    ests=as.double(rep(0,sizefull)),
                    ests.cov=as.double(rep(0,sizefull*sizefull)),
                    design.mat=as.double(rep(0,sizefull*n.ind)),
                    as.double(tol),
                    as.integer(maxit),
                    matrix.rank=as.integer(0), # on return, minimum of matrix rank across imputations
                    PACKAGE="qtl")
        }
        else {
            z <- .C("R_fitqtl_hk_binary",
                    as.integer(n.ind), # number of individuals
                    as.integer(p$n.qtl), # number of qtls
                    as.integer(n.gen.QC), # number of genotypes QTLs and covariates
                    as.double(prob),      # QTL genotype probabilities
                    as.integer(p$n.covar), # number of covariate
                    as.double(covar.C), # covariates
                    as.integer(p$formula.intmtx),  # formula matrix for interactive terms
                    as.integer(p$n.int), # number of interactions in the formula
                    as.double(pheno), # phenotype
                    as.integer(get.ests), # get estimates?
                    # return variables
                    lod=as.double(0), # LOD score
                    df=as.integer(0), # degree of freedom
                    ests=as.double(rep(0,sizefull)),
                    ests.cov=as.double(rep(0,sizefull*sizefull)),
                    design.mat=as.double(rep(0,sizefull*n.ind)),
                    # convergence
                    as.double(tol),
                    as.integer(maxit),
                    matrix.rank=as.integer(0), # on return, rank of matrix
                    PACKAGE="qtl")
        }

    }
    matrix.rank <- z$matrix.rank
    matrix.ncol <- sizefull
    residuals <- z$resid

    if(get.ests) {
        # first, construct the new design matrix
        #  X = the matrix used in the C coe
        #  Z = the matrix we want
        thenames <- qtl$name[p$idx.qtl]
        if(n.covar > 0) thenames <- c(thenames,names(covar)[p$idx.covar])

        ests <- z$ests
        ests.cov <- matrix(z$ests.cov,ncol=sizefull)

        if(adjustX) {
            keep <- 1:(length(ests)-n.newcovar)
            ests <- ests[keep]
            ests.cov <- ests.cov[keep,keep]
        }

        if(any(qtl$n.gen[p$idx.qtl]>=4)) {
            type <- crosstype
            if(type == "4way")
                genotypes <- c("AC","BC","AD","BD")
            else {
                genotypes <- cross.attr$genotypes
                if(is.null(genotypes))
                    genotypes <- as.character(1:max(qtl$n.gen))
            }

            # just attach rownames for this case
            thenames <- "Intercept"

            if(length(p$idx.qtl) > 0) { # qtl names
                qtlnames <- vector("list", length(p$idx.qtl))
                names(qtlnames) <- paste("Q", 1:length(p$idx.qtl), sep="")
                for(i in seq(along=p$idx.qtl)) {
                    qtlnames[[i]] <- paste(qtl$name[p$idx.qtl[i]], genotypes[2:qtl$n.gen[p$idx.qtl[i]]], sep=".")
                    thenames <- c(thenames, qtlnames[[i]])
                }
            }

            if(p$n.covar > 0) {
                covnames <- colnames(covar)[p$idx.covar]
                thenames <- c(thenames, covnames)
            }

            if(p$n.int > 0) { # interactions
                if(!is.matrix(p$formula.intmtx)) {
                    nam <- names(p$formula.intmtx)
                    p$formula.intmtx <- matrix(p$formula.intmtx, nrow=p$n.int)
                    colnames(p$formula.intmtx) <- nam
                }

                for(i in 1:p$n.int) {
                    wh <- which(p$formula.intmtx[i,]==1)
                    nam <- colnames(p$formula.intmtx)[wh]

                    if(length(grep("^Q[0-9]+$", nam[1])) > 0)
                        curnam <- qtlnames[[nam[1]]]
                    else
                        curnam <- nam[1]

                    for(j in 2:length(nam)) {
                        if(length(grep("Q[0-9]+", nam[j])) > 0)
                            curnam <- paste(curnam, qtlnames[[nam[j]]], sep=".")
                        else
                            curnam <- paste(curnam, nam[j], sep=".")
                    }
                    thenames <- c(thenames, curnam)
                }
            }

            if(length(thenames) ==length(ests)) {
                names(ests) <- thenames
                dimnames(ests.cov) <- list(thenames, thenames)
            }
            else
                warning("Estimated QTL effects not yet made meaningful for this case.\n   ")

        }
        else {
            X <- matrix(z$design.mat,ncol=sizefull)
            Z <- matrix(0,nrow=n.ind,ncol=sizefull)
            colnames(Z) <- rep("",sizefull)

            # mean column
            Z[,1] <- 1
            colnames(Z)[1] <- "Intercept"

            # ZZ stores the main effects matrices, for creating the interactions
            ZZ <- vector("list",p$n.qtl+p$n.covar)
            curcol <- 1
            # covariates
            if(p$n.covar > 0) {
                for(j in 1:p$n.covar) {
                    Z[,curcol+j] <- ZZ[[p$n.qtl+j]] <- as.matrix(covar[,p$idx.covar[j],drop=FALSE])
                    colnames(Z)[curcol+j] <- colnames(ZZ[[p$n.qtl+j]]) <- names(covar)[p$idx.covar[j]]
                }
                curcol <- curcol + p$n.covar
            }
            # QTL main effects
            for(i in seq(along=p$idx.qtl)) {
                if(n.gen[i]==2) {
                    if(method=="imp") {
                        if(crosstype == "bc") {
                            Z[qtl$geno[,p$idx.qtl[i],1]==1,curcol+1] <- -0.5
                            Z[qtl$geno[,p$idx.qtl[i],1]==2,curcol+1] <- 0.5
                        } else {
                            Z[qtl$geno[,p$idx.qtl[i],1]==1,curcol+1] <- -1
                            Z[qtl$geno[,p$idx.qtl[i],1]==2,curcol+1] <- 1
                        }
                    }
                    else
                        if(crosstype == "bc") {
                            Z[,curcol+1] <- (qtl$prob[[p$idx.qtl[i]]][,2] - qtl$prob[[p$idx.qtl[i]]][,1])/2
                        } else {
                            Z[,curcol+1] <- (qtl$prob[[p$idx.qtl[i]]][,2] - qtl$prob[[p$idx.qtl[i]]][,1])
                        }
                    colnames(Z)[curcol+1] <- thenames[i]
                }
                else { # 3 genotypes
                    if(method=="imp") {
                        Z[qtl$geno[,p$idx.qtl[i],1]==1,curcol+1] <- -1
                        Z[qtl$geno[,p$idx.qtl[i],1]==3,curcol+1] <- 1
                        Z[qtl$geno[,p$idx.qtl[i],1]==2,curcol+2] <- 0.5
                        Z[qtl$geno[,p$idx.qtl[i],1]!=2,curcol+2] <- -0.5
                    }
                    else {
                        Z[,curcol+1] <- qtl$prob[[p$idx.qtl[i]]][,3] - qtl$prob[[p$idx.qtl[i]]][,1]
                        Z[,curcol+2] <- (qtl$prob[[p$idx.qtl[i]]][,2] - qtl$prob[[p$idx.qtl[i]]][,1] -
                                         qtl$prob[[p$idx.qtl[i]]][,3])/2
                    }
                    colnames(Z)[curcol+1:2] <- paste(thenames[i],c("a","d"),sep="")
                }
                ZZ[[i]] <- Z[,curcol+1:(n.gen[i]-1),drop=FALSE]
                curcol <- curcol + n.gen[i]-1
            }

            if(p$n.int>0) {
                for(i in 1:p$n.int) {
                    if(p$n.int==1)
                        intform <- p$formula.intmtx
                    else
                        intform <- p$formula.intmtx[,i]
                    tempZ <- matrix(1,ncol=1,nrow=nrow(Z))
                    colnames(tempZ) <- ""
                    for(j in seq(along=intform)) {
                        if(intform[j]==1) {
                            tZ <- NULL
                            for(k in 1:ncol(ZZ[[j]])) {
                                tZZ <- tempZ * ZZ[[j]][,k]
                                if(all(colnames(tempZ) == ""))
                                    colnames(tZZ) <- colnames(ZZ[[j]])[k]
                                else
                                    colnames(tZZ) <- paste(colnames(tempZ),colnames(ZZ[[j]])[k],sep=":")
                                tZ <- cbind(tZ,tZZ)
                            }
                            tempZ <- tZ
                        }
                    }
                    Z[,curcol+1:ncol(tempZ)] <- tempZ
                    colnames(Z)[curcol+1:ncol(tempZ)] <- colnames(tempZ)
                    curcol <- curcol + ncol(tempZ)
                }
            }

            b <- solve(t(Z) %*% Z, t(Z) %*% X)
            ests <- as.numeric(b %*% ests)
            ests.cov <- b %*% ests.cov %*% t(b)
            names(ests) <- colnames(Z)
            dimnames(ests.cov) <- list(colnames(Z),colnames(Z))
        }
    }

    ##### output ANOVA table for full model #####
    result.full <- matrix(NA, 3, 7)
    colnames(result.full) <- c("df", "SS", "MS", "LOD", "%var", "Pvalue(Chi2)",
                               "Pvalue(F)")
    rownames(result.full) <- c("Model", "Error", "Total")
    result.full[1,1] <- z$df # model degree of freedom

    if(model=="normal") {
        # compute the SS for total

        if(adjustX) {
            mpheno <- mean(pheno)
            Rss0 <- sum( (pheno-mpheno)^2 )

            Rss0adj <- Rss0x <- sum( lm(pheno ~ Xadjustment$sexpgmcovar)$resid^2 )
            OrigModellod <- z$lod
            Modellod <- z$lod + length(pheno)/2 * (log10(Rss0x) - log10(Rss0))
        } else {
            mpheno <- mean(pheno)
            Rss0adj <- Rss0 <- sum( (pheno-mpheno)^2 )
            OrigModellod <- Modellod <- z$lod
        }

        # third row, for Total
        result.full[3,1] <- length(pheno) - 1 # total degree of freedom
        result.full[3,2] <- Rss0adj # total sum of squares

        # first row, for Model
        result.full[1,1] <- z$df # df for Model
        # Variance explained by model
        result.full[1,5] <- 100 * (1 - exp(-2*Modellod*log(10)/n.ind))
        result.full[1,2] <- Rss0adj * result.full[1,5]/100  # SS for model
        result.full[1,3] <- result.full[1,2]/z$df # MS for model
        result.full[1,4] <- Modellod # Model LOD score

        # Second row, for Error
        # df
        result.full[2,1] <- result.full[3,1] - result.full[1,1]
        # SS
        result.full[2,2] <- result.full[3,2] - result.full[1,2]
        # MS
        result.full[2,3] <- result.full[2,2] / result.full[2,1]

        # first row, P values
        # P value (chi2) for model
        result.full[1,6] <- 1 - pchisq(2*log(10)*Modellod, z$df)
        # P value (F statistics) for model
        df0 <- result.full[3,1]; df1 <- result.full[2,1];
        Rss1 <- result.full[2,2]
        Fstat <- ((Rss0adj-Rss1)/(df0-df1)) / (Rss1/df1)
        result.full[1,7] <- 1 - pf(Fstat, df0-df1, df1)
    }
    else {
        # third row, for Total
        result.full[3,1] <- length(pheno) - 1 # total degree of freedom
        result.full[3,2] <- NA # total sum of squares

        # first row, for Model
        result.full[1,1] <- z$df # df for Model
        # Variance explained by model
        result.full[1,5] <- 100 * (1 - exp(-2*z$lod*log(10)/n.ind))
        result.full[1,2] <- NA  # SS for model
        result.full[1,3] <- NA # MS for model
        result.full[1,4] <- z$lod # Model LOD score
        OrigModellod <- Modellod <- z$lod

        # Second row, for Error
        # df
        result.full[2,1] <- result.full[3,1] - result.full[1,1]
        # SS
        result.full[2,2] <- NA
        # MS
        result.full[2,3] <- NA

        # first row, P values
        # P value (chi2) for model
        result.full[1,6] <- 1 - pchisq(2*log(10)*z$lod, z$df)
        # P value (F statistics) for model
        result.full[1,7] <- NA
    }
    ############# Finish ANOVA table for full model


    # initialize output object
    output <- NULL
    output$result.full <- result.full

    # drop one at a time?
    if(dropone && (p$n.qtl+n.origcovar)>1) {
        # user wants to do drop one term at a time and output anova table

        # get the terms etc. for input formula
        f.terms <- terms(formula)
        f.order <- attr(f.terms, "order")
        f.label <- attr(f.terms, "term.labels")

        # initialize output matrix
        # ANOVA table will have five columns, e.g., df,Type III SS,
        # LOD, %var, Pvalue for each dropping term
        # Full model result will not be in this table
        result <- matrix(0, length(f.order), 7)
        colnames(result) <- c("df", "Type III SS", "LOD", "%var", "F value",
                              "Pvalue(Chi2)", "Pvalue(F)")
        rownames(result) <- rep("",length(f.order))

        drop.term.name <- NULL
        formulas <- rep("", length(f.order))
        lods <- rep(NA, length(f.order))

        for( i in (1:length(f.order)) ) {
            # loop thru all terms in formula, from the highest order
            # the label of the term to be droped
            label.term.drop <- f.label[i]

            ### find the corresponding QTL name for this term ###
            # This is used for output ANOVA table
            if(f.order[i] == 1) {
                # this is a first order term
                # if the term label is like Q(q)1, Q(q)2, etc., then it's a QTL
                if( length(grep("Q[0-9]", label.term.drop, ignore.case=TRUE)) != 0) {
                    idx.qtlname <- as.integer(substr(label.term.drop, 2, nchar(label.term.drop)))
                    drop.term.name[i] <- qtl$name[idx.qtlname]
                }
                else { # this is a covariate
                    drop.term.name[i] <- label.term.drop
                }
            }
            else {
                # this is a 2nd (or higher)order and the term is a string like "Q2:Q3:C1"
                # I use strsplit to split it to a string vector "Q2" "Q3" "C1".
                # then take out 2 and 3 as integer. Then find out the
                # QTL name from the input QTL object and concatenate them
                tmp.str <- strsplit(label.term.drop,":")[[1]]
                for(j in 1:length(tmp.str)) {
                    if( length(grep("Q[0-9]", tmp.str[j], ignore.case=TRUE)) != 0 ) {
                        # this is a QTL
                        idx.qtlname <- as.integer(substr(tmp.str[j], 2, nchar(tmp.str[j])))
                        tmp.str[j] <- qtl$name[idx.qtlname]
                    }
                    if(j == 1) # first term
                        drop.term.name[i] <- tmp.str[j]
                    else # not the first term
                        drop.term.name[i] <- paste(drop.term.name[i], tmp.str[j], sep=":")
                }
            }
            ### Finish QTL name ###

            # find the indices of the term(s) to be dropped
            # All terms contain label.term.drop will be dropped
            idx.term.drop <- NULL
            tmp.str.drop <- tolower(strsplit(label.term.drop,":")[[1]])
            for(j in 1:length(f.label)) {
                tmp.str.label <- tolower(strsplit(f.label[j], ":")[[1]])
                if(all(tmp.str.drop %in% tmp.str.label))
                    idx.term.drop <- c(idx.term.drop, j)
            }

            # the indices of term(s) to be kept
            idx.term.kept <- setdiff(1:length(f.order), idx.term.drop)

            #### regenerate a formula with the kept terms additive ###
            if(length(idx.term.kept) == 0) # nothing left after drop label.term.drop
                stop("There will be nothing left if drop ", drop.term.name[i])
            else {
                # All terms for idx.term.kept will be additive
                formula.new <- as.formula(paste("y~", paste(f.label[idx.term.kept], collapse="+"), sep=""))
            }

            ### Start fitting model again
            # parse the input formula
            p.new <- parseformula(formula.new, qtl$altname, colnames(covar))
            n.gen.QC <- c(n.gen[p.new$idx.qtl]-1, rep(1, p.new$n.covar))

            formulas[i] <- deparseQTLformula(formula.new)

            # covariate to be passed to C function
            covar.C <- NULL
            if(!is.null(p.new$idx.covar))
                covar.C <- as.matrix(covar[,p.new$idx.covar,drop=FALSE])

            if(method != "imp") {
                # form genotype probabilities as a matrix
                prob <- matrix(ncol=sum(qtl$n.gen[p.new$idx.qtl]), nrow=n.ind)

                curcol <- 0
                for(z in p.new$idx.qtl) {
                    prob[,curcol+1:n.gen[z]] <- qtl$prob[[z]]
                    curcol <- curcol + n.gen[z]
                }
            }

            if(adjustX)  { # need to include X chromosome covariates
                n.newcovar <- ncol(Xadjustment$sexpgmcovar)
                n.gen.QC <- c(n.gen.QC, rep(1, n.newcovar))
                p.new$n.covar <- p.new$n.covar + n.newcovar
                covar.C <- cbind(covar.C, Xadjustment$sexpgmcovar)
                sizefull <- sizefull + n.newcovar

                if(p.new$n.int==1)
                    p.new$formula.intmtx <- c(p.new$formula.intmtx, rep(0,n.newcovar))
                if(p.new$n.int>1) {
                    for(i2 in 1:n.newcovar)
                        p.new$formula.intmtx <- rbind(p.new$formula.intmtx, rep(0,p.new$n.int))
                }
            }

            # call C function fit model
            if(model=="normal") {
                if(method == "imp") {
                    z <- .C("R_fitqtl_imp",
                            as.integer(n.ind), # number of individuals
                            as.integer(p.new$n.qtl), # number of qtls
                            as.integer(n.gen.QC), # number of genotypes QTLs and covariates
                            as.integer(n.draws), # number of draws
                            as.integer(qtl$geno[,p.new$idx.qtl,]), # genotypes for selected marker
                            as.integer(p.new$n.covar), # number of covariate
                            as.double(covar.C), # covariate
                            as.integer(p.new$formula.intmtx),  # formula matrix for interactive terms
                            as.integer(p.new$n.int), # number of interactions in the formula
                            as.double(pheno), # phenotype
                            as.integer(0),
                            # return variables
                            lod=as.double(0), # LOD score
                            df=as.integer(0), # degree of freedom
                            as.double(rep(0,sizefull)),
                            as.double(rep(0,sizefull*sizefull)),
                            as.double(rep(0,n.ind*sizefull)),
                            matrix.rank=as.integer(0),
                            resid=as.double(rep(0,n.ind)), # on return, the average residuals across imputations
                            PACKAGE="qtl")
                }

                else {
                    z <- .C("R_fitqtl_hk",
                            as.integer(n.ind), # number of individuals
                            as.integer(p.new$n.qtl), # number of qtls
                            as.integer(n.gen.QC), # number of genotypes QTLs and covariates
                            as.double(prob),
                            as.integer(p.new$n.covar), # number of covariate
                            as.double(covar.C), # covariate
                            as.integer(p.new$formula.intmtx),  # formula matrix for interactive terms
                            as.integer(p.new$n.int), # number of interactions in the formula
                            as.double(pheno), # phenotype
                            as.integer(0),
                            # return variables
                            lod=as.double(0), # LOD score
                            df=as.integer(0), # degree of freedom
                            as.double(rep(0,sizefull)),
                            as.double(rep(0,sizefull*sizefull)),
                            as.double(rep(0,n.ind*sizefull)),
                            matrix.rank=as.integer(0),
                            resid=as.double(rep(0,n.ind)), # on return, the residuals
                            PACKAGE="qtl")
                }
            }
            else { # binary trait
                if(method=="imp") {
                    z <- .C("R_fitqtl_imp_binary",
                            as.integer(n.ind), # number of individuals
                            as.integer(p.new$n.qtl), # number of qtls
                            as.integer(n.gen.QC), # number of genotypes QTLs and covariates
                            as.integer(n.draws), # number of draws
                            as.integer(qtl$geno[,p.new$idx.qtl,]), # genotypes for selected marker
                            as.integer(p.new$n.covar), # number of covariate
                            as.double(covar.C), # covariate
                            as.integer(p.new$formula.intmtx),  # formula matrix for interactive terms
                            as.integer(p.new$n.int), # number of interactions in the formula
                            as.double(pheno), # phenotype
                            as.integer(0),
                            # return variables
                            lod=as.double(0), # LOD score
                            df=as.integer(0), # degree of freedom
                            as.double(rep(0,sizefull)),
                            as.double(rep(0,sizefull*sizefull)),
                            as.double(rep(0,n.ind*sizefull)),
                            as.double(tol),
                            as.integer(maxit),
                            matrix.rank=as.integer(0),
                            PACKAGE="qtl")
                }
                else {
                    z <- .C("R_fitqtl_hk_binary",
                            as.integer(n.ind), # number of individuals
                            as.integer(p.new$n.qtl), # number of qtls
                            as.integer(n.gen.QC), # number of genotypes QTLs and covariates
                            as.double(prob),
                            as.integer(p.new$n.covar), # number of covariate
                            as.double(covar.C), # covariate
                            as.integer(p.new$formula.intmtx),  # formula matrix for interactive terms
                            as.integer(p.new$n.int), # number of interactions in the formula
                            as.double(pheno), # phenotype
                            as.integer(0),
                            # return variables
                            lod=as.double(0), # LOD score
                            df=as.integer(0), # degree of freedom
                            as.double(rep(0,sizefull)),
                            as.double(rep(0,sizefull*sizefull)),
                            as.double(rep(0,n.ind*sizefull)),
                            # convergence
                            as.double(tol),
                            as.integer(maxit),
                            matrix.rank = as.integer(0),
                            PACKAGE="qtl")
                }
            }

            if(model=="normal" && adjustX) # adjust for X chromosome covariates
                z$lod <- z$lod + length(pheno)/2 * (log10(Rss0x) - log10(Rss0))

            # record the result for dropping this term
            # df
            result[i,1] <- result.full[1,1] - z$df
            # LOD score
            result[i,3] <- Modellod - z$lod
            # % variance explained
            result[i,4] <- result.full[1,5] - 100*(1 - 10^(-2*z$lod/n.ind))

            # lod score for reduced model
            lods[i] <- z$lod

            # Type III SS for this term - computed from %var
            if(model=="normal")
                result[i,2] <- result.full[3,2] * result[i,4] / 100
            else
                result[i,2] <- NA
            # F value
            if(model=="normal") {
                df0 <- length(pheno) - z$df - 1; df1 <- result.full[2,1];
                Rss0p <- result.full[2,2] + result[i,2];
                Rss1p <- result.full[2,2]
                Fstat <- ((Rss0p-Rss1p)/(df0-df1)) / (Rss1/df1)
                result[i,5] <- Fstat
                # P value (F)
                result[i,7] <- 1 - pf(Fstat, df0-df1, df1)
            }
            else # ignore F stat for binary trait
                result[i,c(5,7)] <- NA
            # P value (chi2)
            result[i,6] <- 1 - pchisq(2*log(10)*result[i,3], result[i,1])
            # assign row name
            rownames(result)[i] <- drop.term.name[i]
        } # finish dropping terms loop

        attr(result, "formulas") <- formulas
        attr(result, "lods") <- lods

        # assign output object
        output$result.drop <- result

    }  ## if(dropone)

    if(get.ests)
        output$ests <- list(ests=ests, covar=ests.cov)

    output$lod <- output$result.full[1,4]

    class(output) <- "fitqtl"
    attr(output, "method") <- method
    attr(output, "model") <- model
    attr(output, "formula") <- deparseQTLformula(formula)
    attr(output, "type") <- qtl$type
    attr(output, "nind") <- length(pheno)

    attr(output, "matrix.rank") <- matrix.rank
    attr(output, "matrix.ncol") <- matrix.ncol

    if(!is.null(residuals))
        attr(output, "residuals") <- residuals

    output
}


######################################################################
# checkformula
#
# check that input formula satisfies our imposed hiearchy: that
# main effects for terms in any interactions are also included
######################################################################
checkformula <-
    function(formula, qtl.name, covar.name)
{
    factors <- attr(terms(formula), "factors")
    altform <- deparseQTLformula(formula)

    if(sum(factors[1,])==0) factors <- factors[-1,,drop=FALSE]

    rn <- rownames(factors)
    # if mentions of "q1" or such, convert to "Q1" and such
    g <- grep("^[Qq][0-9]+$", rn)
    todrop <- NULL
    if(length(g) >= 1) {
        rownames(factors)[g] <- rn[g] <- toupper(rn[g])
        if(any(table(rn) > 1)) { # now there are some duplicates
            urn <- unique(rn)
            for(i in urn) {
                wh <- which(rn == i)
                if(length(wh) > 1) {
                    factors[wh[1],] <- apply(factors[wh,], 2, sum)
                    todrop <- c(todrop, wh[-1])
                    rownames(factors)[wh[1]] <- rn[wh[1]]
                }
            }
        }
    }
    if(length(todrop) > 0) factors <- factors[-todrop,]
    rn <- rownames(factors)

    if(!missing(qtl.name) || !missing(covar.name)) {
        m <- match(rn, c(qtl.name, covar.name))
        if(any(is.na(m)))
            warning("Terms ", paste(rn[is.na(m)], collapse=" "), " not understood.")
    }

    # paste rows together
    zo <- factors
    zo[zo>1] <- 1
    pzo <- apply(zo, 2, paste, collapse="")
    nt <- apply(zo, 2, sum)

    # form binary representations
    maxnt <- max(nt)
    v <- vector("list", maxnt)
    for(i in 2:maxnt) {
        v[[i]] <- mybinaryrep(i)
        v[[i]] <- v[[i]][,-ncol(v[[i]])+c(0,1)]
    }

    # for each higher-order column, form all lower-order terms
    for(i in which(nt > 1)) {
        cur <- zo[,i]
        wh <- which(cur==1)
        z <- v[[nt[i]]]
        zz <- matrix(0, ncol=ncol(z), nrow=length(cur))
        for(j in seq(along=wh))
            zz[wh[j],] <- z[j,]
        pzo <- unique(c(pzo, apply(zz, 2, paste, collapse="")))
    }

    zo <- matrix(as.numeric(unlist(strsplit(pzo, ""))), ncol=length(pzo))
    nt <- apply(zo, 2, sum)
    zo <- zo[,order(nt, apply(1-zo, 2, paste, collapse="")), drop=FALSE]
    rownames(zo) <- rn

    # form column names
    theterms <- apply(zo, 2, function(a, b) paste(b[as.logical(a)], collapse=":"),
                      rownames(zo))

    as.formula(paste("y ~ ", paste(theterms, collapse=" + ")))
}

#####################################################################
#
# parseformula
#
# Function to be called by fitqtl. It's used to
# parse the input formula
#
# This is the internal function and not supposed to be used by user
#
#####################################################################

parseformula <- function(formula, qtl.dimname, covar.dimname)
{
    # The terms for input formula
    f.formula <- terms(formula)
    order.term <- attr(f.formula, "order") # get the order of the terms
    idx.term <- which(order.term==1) # get the first order terms
    label.term <- attr(f.formula, "term.labels")[idx.term]
    formula.mtx <- attr(f.formula, "factors") # formula matrix

    idx.qtl <- NULL
    idx.covar <- NULL

    # loop thru all terms and find out how many QTLs and covariates
    # are there in the formula. Construct idx.qtl and idx.covar at the same time
    termisqtl <- rep(0, length(idx.term))
    for (i in 1:length(idx.term)) {
        # find out if there term is a QTL or a covariate
        # ignore the case for QTLs, e.g., Q1 is equivalent to q1
        idx.tmp <- grep(paste(label.term[i],"$", sep=""),
                        qtl.dimname, ignore.case=TRUE)
        if( length(idx.tmp) ) { # it's a QTL
            idx.qtl <- c(idx.qtl, idx.tmp)
            termisqtl[i] <- 1
        }
        else if(label.term[i] %in% covar.dimname) # it's a covariate
            idx.covar <- c(idx.covar, which(label.term[i]==covar.dimname))
        else
            stop("Unrecognized term ", label.term[i], " in formula")
    }
    n.qtl <- length(idx.qtl) # number of QTLs in formula
    n.covar <- length(idx.covar) # number of covariates in formula
    # now idx.qtl and idx.covar are the indices for genotype
    # and covariate matrices according to input formula

    # loop thru all terms again and reorganize formula.mtx
    formula.idx <- NULL
    ii <- 1
    jj <- 1
    for (i in 1:length(idx.term)) {
        #    if(label.term[i] %in% qtl.dimname) {  # it's a QTL
        if(termisqtl[i]) {
            formula.idx <- c(formula.idx, ii)
            ii <- ii+1
        }
        else { # it's a covariate
            formula.idx <- c(formula.idx, jj+n.qtl)
            jj <- jj+1
        }
    }

    # reorganize formula.mtx according to formula.idx
    # remove the first row (for y)
    formula.mtx <- formula.mtx[2:nrow(formula.mtx),]
    # rearrange the rows according to formula.idx if there's more than one row
    if(length(formula.idx) > 1)
        formula.mtx <- formula.mtx[order(formula.idx),]
    # take out only part of the matrix for interactions and pass to C function
    # all the input QTLs and covariates for C function will be additive
    n.int <- length(order.term) - length(idx.term) # number of interactions
    if(n.int != 0)
        formula.intmtx <- formula.mtx[,(length(idx.term)+1):length(order.term)]
    else # no interaction terms
        formula.intmtx <- NULL

    # return object
    result <- NULL
    result$idx.qtl <- idx.qtl
    result$n.qtl <- n.qtl
    result$idx.covar <- idx.covar
    result$n.covar <- n.covar
    result$formula.intmtx <- formula.intmtx
    result$n.int <- n.int

    result

}


#####################################################################
#
# summary.fitqtl
#
#####################################################################
summary.fitqtl <-
    function(object, pvalues=TRUE, simple=FALSE, ...)
{
    if(!inherits(object, "fitqtl"))
        stop("Input should have class \"fitqtl\".")

    # this is just an interface.
    if("ests" %in% names(object)) {
        ests <- object$ests$ests
        se <- sqrt(diag(object$ests$covar))
        object$ests <- cbind(est=ests, SE=se, t=ests/se)
    }
    class(object) <- "summary.fitqtl"
    if(simple) pvalues <- FALSE
    attr(object, "pvalues") <- pvalues
    attr(object, "simple") <- simple
    object
}


#####################################################################
#
# print.summary.fitqtl
#
#####################################################################
print.summary.fitqtl <- function(x, ...)
{
    cat("\n")
    cat("\t\tfitqtl summary\n\n")
    meth <- attr(x, "method")
    mod <- attr(x, "model")
    if(is.null(mod)) mod <- "normal"
    if(meth=="imp") meth <- "multiple imputation"
    else if(meth=="hk") meth <- "Haley-Knott regression"
    cat("Method:", meth, "\n")
    cat("Model: ", mod, "phenotype\n")
    cat("Number of observations :", attr(x, "nind"), "\n\n")

    # print ANOVA table for full model
    cat("Full model result\n")
    cat("----------------------------------  \n")
    cat("Model formula:")
    w <- options("width")[[1]]
    printQTLformulanicely(attr(x, "formula"), "                   ", w+5, w)
    cat("\n")

    pval <- attr(x, "pvalues")
    simple <- attr(x, "simple")
    if(!is.null(pval) && !pval)
        x$result.full <- x$result.full[,-ncol(x$result.full)+(0:1)]
    if(mod=="binary" || (!is.null(simple) && simple))
        x$result.full <- x$result.full[1,-c(2:3,7),drop=FALSE]

    print(x$result.full, quote=FALSE, na.print="")
    cat("\n")

    # print ANOVA table for dropping one at a time analysis (if any)
    if("result.drop" %in% names(x)) {
        cat("\n")
        cat("Drop one QTL at a time ANOVA table: \n")
        cat("----------------------------------  \n")
        # use printCoefmat instead of print.data.frame
        # make sure the last column is P value
        if(!is.null(pval) && !pval)
            x$result.drop <- x$result.drop[,-ncol(x$result.drop)+(0:1)]
        if(mod=="binary" || (!is.null(simple) && simple))
            x$result.drop <- x$result.drop[,-c(2,5,7),drop=FALSE]

        printCoefmat(x$result.drop, digits=4, cs.ind=1, P.values=TRUE, has.Pvalue=TRUE)
        cat("\n")
    }

    if("ests" %in% names(x)) {
        cat("\n")
        cat("Estimated effects:\n")
        cat("-----------------\n")
        printCoefmat(x$ests,digits=4)
        cat("\n")
    }

}

######################################################################
# binary repreentation of the numbers 1...2^n;
# used in checkformula
######################################################################
mybinaryrep <-
    function(n)
{
    lx <- 2^n
    x <- 1:lx
    ans <- 0:(n-1)
    x <- matrix(rep(x,rep(n, lx)), ncol=lx)
    (x %/% 2^ans) %% 2
}

######################################################################
# deparseQTLformula: turn QTL formula into a string
######################################################################
deparseQTLformula <-
    function(formula, reorderterms=FALSE)
{
    if(is.null(formula)) return(NULL)
    if(reorderterms) {
        if(is.character(formula)) formula <- as.formula(formula)
        factors <- colnames(attr(terms(formula), "factors"))
        wh <- grep("^[Qq][0-9]+$", factors)
        if(length(wh)>0)
            factors[wh] <- paste("Q", sort(as.numeric(substr(factors[wh], 2, nchar(factors[wh])))), sep="")
        wh <- grep(":", factors)
        if(length(wh)>0) {
            temp <- strsplit(factors[wh], ":")
            temp <- sapply(temp, function(a) {
                wh <- grep("^[Qq][0-9]+$", a)
                if(any(wh)) a[wh] <- paste("Q", sort(as.numeric(substr(a[wh], 2, nchar(a[wh])))), sep="")
                paste(a[order(is.na(match(seq(along=a),wh)))], collapse=":")
            })
            factors[wh] <- temp
        }
        return(paste("y ~ ", paste(factors, collapse=" + "), sep=""))
    }


    if(is.character(formula)) return(formula)
    paste(as.character(formula)[c(2,1,3)], collapse=" ")
}

printQTLformulanicely <-
    function(formula, header, width, width2, sep=" ")
{
    if(!is.character(formula)) formula <- deparseQTLformula(formula)
    thetext <- unlist(strsplit(formula, " "))

    if(missing(width2)) width2 <- width
    nleft <- width - nchar(header)
    nsep <- nchar(sep)
    if(length(thetext) < 2) cat("", thetext, "\n", sep=sep)
    else {
        z <- paste("", thetext[1], sep=sep, collapse=sep)
        for(j in 2:length(thetext)) {
            if(nchar(z) + nsep + nchar(thetext[j]) > nleft) {
                cat(z, "\n")
                nleft <- width2
                z <- paste(header, thetext[j], sep=sep)
            }
            else {
                z <- paste(z, thetext[j], sep=sep)
            }
        }
        cat(z, "\n")
    }
}

# end of fitqtl.R

######################################################################
#
# simulate.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Apr, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: sim.map, sim.cross, sim.cross.bc, sim.cross.f2,
#           sim.cross.4way, sim.bcg
#
######################################################################

######################################################################
#
# sim.map: simulate a genetic map
#
######################################################################

sim.map <-
    function(len=rep(100,20), n.mar=10, anchor.tel=TRUE, include.x=TRUE,
             sex.sp=FALSE, eq.spacing=FALSE)
{
    if(length(len)!=length(n.mar) && length(len)!=1 && length(n.mar)!=1)
        stop("Lengths of vectors len and n.mar do not conform.")

    # make vectors the same length
    if(length(len) == 1) len <- rep(len,length(n.mar))
    else if(length(n.mar) == 1) n.mar <- rep(n.mar,length(len))

    n.chr <- length(n.mar)

    map <- vector("list",n.chr)
    names(map) <- as.character(1:n.chr)
    if(include.x) names(map)[n.chr] <- "X"

    for(i in 1:n.chr) {
        if(anchor.tel) {
            if(n.mar[i] < 2) n.mar[i] <- 2
            map[[i]] <- c(0,len[i])
            if(n.mar[i] > 2) {
                if(!eq.spacing)
                    map[[i]] <- sort(c(map[[i]],runif(n.mar[i]-2,0,len[i])))
                else # equal spacing
                    map[[i]] <- seq(0,len[i],length=n.mar[i])
            }
        }
        else {
            if(!eq.spacing) {
                map[[i]] <- sort(runif(n.mar[i],0,len[i]))
                map[[i]] <- map[[i]] - min(map[[i]])
            }
            else {  # equal spacing
                map[[i]] <- seq(0,len[i],length=n.mar[i]+1)
                map[[i]] <- map[[i]][-1] - map[[i]][2]/2
            }
        }
        names(map[[i]]) <- paste("D", names(map)[i], "M", 1:n.mar[i], sep="")
        class(map[[i]]) <- "A"
    }

    if(sex.sp) {
        for(i in 1:n.chr) {
            if(eq.spacing) tempmap <- map[[i]]
            else {
                if(anchor.tel) {
                    if(n.mar[i] < 2) n.mar[i] <- 2
                    tempmap <- c(0,len[i])
                    if(n.mar[i] > 2)
                        tempmap <- sort(c(tempmap,runif(n.mar[i]-2,0,len[i])))
                }
                else {
                    tempmap <- sort(runif(n.mar[i],0,len[i]))
                    tempmap <- tempmap - min(tempmap)
                }
            }
            map[[i]] <- rbind(map[[i]],tempmap)
            dimnames(map[[i]]) <- list(NULL,paste("D", names(map)[i], "M", 1:n.mar[i], sep=""))
            class(map[[i]]) <- "A"

            if(include.x && i==n.chr)  # if X chromosome, force no recombination in male
                map[[i]][2,] <- rep(0,ncol(map[[i]]))
        }
    }

    if(include.x) class(map[[n.chr]]) <- "X"

    class(map) <- "map"
    map
}

######################################################################
#
# sim.cross: Simulate an experimental cross
#
# Note: These functions are a bit of a mess.  I was in the "get it to
#       work without worrying about efficiency" mode while writing it.
#       Sorry!
#
######################################################################

sim.cross <-
    function(map, model=NULL, n.ind=100,
             type=c("f2", "bc", "4way", "risib", "riself",
             "ri4sib", "ri4self", "ri8sib", "ri8self","bcsft"),
             error.prob=0, missing.prob=0, partial.missing.prob=0,
             keep.qtlgeno=TRUE, keep.errorind=TRUE, m=0, p=0,
             map.function=c("haldane","kosambi","c-f","morgan"),
             founderGeno, random.cross=TRUE, ...)
{
    type <- match.arg(type)
    map.function <- match.arg(map.function)

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }

    # 2-way RIL by sibmating or selfing
    if(type=="risib" || type=="riself") {
        if(!is.null(model))
            warning('"model" argument currently ignored in simulating RILs')

        if(type=="risib") type <- "sibmating"
        else type <- "selfing"
        cross <- sim.ril(map, n.ind, type, "2", m=m, p=p,
                         error.prob=error.prob, missing.prob=missing.prob)
        cross$cross <- NULL

        return(cross)
    }
    # 4- or 8-way RIL by sibmating or selfing
    if(type=="ri4sib" || type=="ri4self" || type=="ri8sib" || type=="ri8self") {
        if(!is.null(model))
            warning('"model" argument currently ignored in simulating RILs')

        if(substr(type, 4, nchar(type))=="self") crosstype <- "selfing"
        else crosstype <- "sibmating"
        n.str <- substr(type, 3, 3)
        cross <- sim.ril(map, n.ind, crosstype, n.str, m=m, p=p,
                         random.cross=random.cross,
                         error.prob=0,
                         missing.prob=missing.prob)

        rcross <- convertMWril(cross, founderGeno, error.prob=error.prob)
        for(i in names(cross$geno))
            if(!("truegeno" %in% names(rcross$geno[[i]])))
                rcross$geno[[i]]$truegeno <- cross$geno[[i]]$data

        # remove "un" from cross type
        class(rcross) <- c(sub("un$", "", crosstype(cross)), "cross")

        fg <- t(founderGeno[[1]])
        if(length(founderGeno)>1)
            for(i in 2:length(founderGeno))
                fg <- cbind(fg, t(founderGeno[[i]]))
        colnames(fg) <- markernames(rcross)
        rcross$founderGeno <- fg
        return(rcross)
    }

    # sort the model matrix
    if(!is.null(model) && is.matrix(model))
        model <- model[order(model[,1],model[,2]),]

    if(type=="bc")
        cross <- sim.cross.bc(map,model,n.ind,error.prob,missing.prob,
                              keep.errorind,m,p,map.function)
    else if(type=="f2")
        cross <- sim.cross.f2(map,model,n.ind,error.prob,missing.prob,
                              partial.missing.prob,keep.errorind,
                              m,p,map.function)
    else if(type=="bcsft")
        cross <- sim.cross.bcsft(map,model,n.ind,error.prob,missing.prob,
                                 partial.missing.prob,keep.errorind,
                                 m,p,map.function, ...)
    else
        cross <- sim.cross.4way(map,model,n.ind,error.prob,missing.prob,
                                partial.missing.prob,keep.errorind,
                                m,p,map.function)

    # remove QTL genotypes from data and, if keep.qtlgeno=TRUE,
    #     place them in cross$qtlgeno
    qtlgeno <- NULL
    for(i in 1:nchr(cross)) {
        o <- grep("^QTL[0-9]+", colnames(cross$geno[[i]]$data))
        if(length(o) != 0) {
            qtlgeno <- cbind(qtlgeno, cross$geno[[i]]$data[,o,drop=FALSE])
            cross$geno[[i]]$data <- cross$geno[[i]]$data[,-o,drop=FALSE]
            if(is.matrix(cross$geno[[i]]$map))
                cross$geno[[i]]$map <- cross$geno[[i]]$map[,-o,drop=FALSE]
            else
                cross$geno[[i]]$map <- cross$geno[[i]]$map[-o]
        }
    }
    if(keep.qtlgeno) cross$qtlgeno <- qtlgeno

    # store genotype data as integers
    for(i in 1:nchr(cross))
        storage.mode(cross$geno[[i]]$data) <- "integer"

    if(is.null(names(cross$geno)))
        names(cross$geno) <- 1:length(cross$geno)

    cross
}


######################################################################
#
# sim.cross.bc
#
######################################################################

sim.cross.bc <-
    function(map,model,n.ind,error.prob,missing.prob,
             keep.errorind,m,p,map.function)
{
    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    if(any(sapply(map,is.matrix)))
        stop("Map must not be sex-specific.")

    chr.type <- sapply(map, chrtype)

    n.chr <- length(map)

    if(is.null(model)) n.qtl <- 0
    else {
        if(!((!is.matrix(model) && length(model) == 3) ||
             (is.matrix(model) && ncol(model) == 3)))
            stop("Model must be a matrix with 3 columns (chr, pos and effect).")
        if(!is.matrix(model)) model <- rbind(model)
        n.qtl <- nrow(model)
        if(any(model[,1] < 0 | model[,1] > n.chr))
            stop("Chromosome indicators in model matrix out of range.")
        model[,2] <- model[,2]+1e-14 # so QTL not on top of marker
    }

    # if any QTLs, place qtls on map
    if(n.qtl > 0) {
        for(i in 1:n.qtl) {
            temp <- map[[model[i,1]]]
            if(model[i,2] < min(temp)) {
                temp <- c(model[i,2],temp)
                names(temp)[1] <- paste("QTL",i,sep="")
            }
            else if(model[i,2] > max(temp)) {
                temp <- c(temp,model[i,2])
                names(temp)[length(temp)] <- paste("QTL",i,sep="")
            }
            else {
                j <- max((seq(along=temp))[temp < model[i,2]])
                temp <- c(temp[1:j],model[i,2],temp[(j+1):length(temp)])
                names(temp)[j+1] <- paste("QTL",i,sep="")
            }
            map[[model[i,1]]] <- temp
        }
    }

    geno <- vector("list", n.chr)
    names(geno) <- names(map)
    n.mar <- sapply(map,length)
    mar.names <- lapply(map,names)

    for(i in 1:n.chr) {
        # simulate genotype data
        thedata <- sim.bcg(n.ind, map[[i]], m, p, map.function)
        dimnames(thedata) <- list(NULL,mar.names[[i]])

        geno[[i]] <- list(data = thedata, map = map[[i]])
        class(geno[[i]]) <- chr.type[i]
        class(geno[[i]]$map) <- NULL

    } # end loop over chromosomes

    # simulate phenotypes
    pheno <- rnorm(n.ind,0,1)

    if(n.qtl > 0) {
        # find QTL positions in genotype data
        QTL.chr <- QTL.loc <- NULL
        for(i in 1:n.chr) {
            o <- grep("^QTL[0-9]+",mar.names[[i]])
            if(length(o)>0) {
                QTL.chr <- c(QTL.chr,rep(i,length(o)))
                QTL.loc <- c(QTL.loc,o)
            }
        }

        # incorporate QTL effects
        for(i in 1:n.qtl) {
            QTL.geno <- geno[[QTL.chr[i]]]$data[,QTL.loc[i]]
            pheno[QTL.geno==2] <- pheno[QTL.geno==2] + model[i,3]
        }

    } # end simulate phenotype

    n.mar <- sapply(geno, function(a) length(a$map))

    # add errors
    if(error.prob > 0) {
        for(i in 1:n.chr) {
            a <- sample(0:1,n.mar[i]*n.ind,replace=TRUE,
                        prob=c(1-error.prob,error.prob))
            geno[[i]]$data[a == 1] <- 3 - geno[[i]]$data[a == 1]
            if(keep.errorind) {
                errors <- matrix(0,n.ind,n.mar[i])
                errors[a==1] <- 1
                colnames(errors) <- colnames(geno[[i]]$data)
                geno[[i]]$errors <- errors
            }
        }
    }

    # add missing
    if(missing.prob > 0) {
        for(i in 1:n.chr) {
            o <- grep("^QTL[0-9]+",mar.names[[i]])
            if(length(o)>0)
                x <- geno[[i]]$data[,o]
            geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,replace=TRUE,
                                  prob=c(missing.prob,1-missing.prob))] <- NA
            if(length(o)>0)
                geno[[i]]$data[,o] <- x
        }
    }

    pheno <- data.frame(phenotype=pheno, stringsAsFactors=TRUE)

    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c("bc","cross")

    cross
}

######################################################################
#
# sim.cross.f2
#
######################################################################

sim.cross.f2 <-
    function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
             keep.errorind,m,p,map.function)
{
    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    if(any(sapply(map,is.matrix)))
        stop("Map must not be sex-specific.")

    # chromosome types
    chr.type <- sapply(map, chrtype)

    n.chr <- length(map)
    if(is.null(model)) n.qtl <- 0
    else {
        if(!((!is.matrix(model) && length(model) == 4) ||
             (is.matrix(model) && ncol(model) == 4))) {
            stop("Model must be a matrix with 4 columns (chr, pos and effects).")
        }
        if(!is.matrix(model)) model <- rbind(model)
        n.qtl <- nrow(model)
        if(any(model[,1] < 0 | model[,1] > n.chr))
            stop("Chromosome indicators in model matrix out of range.")
        model[,2] <- model[,2]+1e-14 # so QTL not on top of marker
    }

    # if any QTLs, place qtls on map
    if(n.qtl > 0) {
        for(i in 1:n.qtl) {
            temp <- map[[model[i,1]]]
            if(model[i,2] < min(temp)) {
                temp <- c(model[i,2],temp)
                names(temp)[1] <- paste("QTL",i,sep="")
            }
            else if(model[i,2] > max(temp)) {
                temp <- c(temp,model[i,2])
                names(temp)[length(temp)] <- paste("QTL",i,sep="")
            }
            else {
                j <- max((seq(along=temp))[temp < model[i,2]])
                temp <- c(temp[1:j],model[i,2],temp[(j+1):length(temp)])
                names(temp)[j+1] <- paste("QTL",i,sep="")
            }
            map[[model[i,1]]] <- temp
        }
    }

    geno <- vector("list", n.chr)
    names(geno) <- names(map)
    n.mar <- sapply(map,length)
    mar.names <- lapply(map,names)

    for(i in 1:n.chr) {

        # simulate genotype data
        thedata <- sim.bcg(n.ind, map[[i]], m, p, map.function)
        dimnames(thedata) <- list(NULL,mar.names[[i]])

        if(chr.type[i] != "X")
            thedata <- thedata + sim.bcg(n.ind, map[[i]], m, p, map.function) - 1

        geno[[i]] <- list(data = thedata, map = map[[i]])
        class(geno[[i]]) <- chr.type[i]
        class(geno[[i]]$map) <- NULL

    } # end loop over chromosomes

    # simulate phenotypes
    pheno <- rnorm(n.ind,0,1)

    if(n.qtl > 0) {
        # find QTL positions in genotype data
        QTL.chr <- QTL.loc <- NULL
        for(i in 1:n.chr) {
            o <- grep("^QTL[0-9]+",mar.names[[i]])
            if(length(o)>0) {
                QTL.chr <- c(QTL.chr,rep(i,length(o)))
                QTL.loc <- c(QTL.loc,o)
            }
        }

        # incorporate QTL effects
        for(i in 1:n.qtl) {
            QTL.geno <- geno[[QTL.chr[i]]]$data[,QTL.loc[i]]
            pheno[QTL.geno==1] <- pheno[QTL.geno==1] - model[i,3]
            pheno[QTL.geno==2] <- pheno[QTL.geno==2] + model[i,4]
            pheno[QTL.geno==3] <- pheno[QTL.geno==3] + model[i,3]
        }

    } # end simulate phenotype

    n.mar <- sapply(geno, function(a) length(a$map))

    # add errors
    if(error.prob > 0) {
        for(i in 1:n.chr) {
            if(chr.type[i]=="X") {
                a <- sample(0:1,n.mar[i]*n.ind,replace=TRUE,
                            prob=c(1-error.prob,error.prob))
                geno[[i]]$data[a == 1] <- 3 - geno[[i]]$data[a == 1]
            }
            else {
                a <- sample(0:2,n.mar[i]*n.ind,replace=TRUE,
                            prob=c(1-error.prob,error.prob/2,error.prob/2))
                if(any(a>0 & geno[[i]]$data==1))
                    geno[[i]]$data[a>0 & geno[[i]]$data==1] <-
                        (geno[[i]]$data+a)[a>0 & geno[[i]]$data==1]
                if(any(a>0 & geno[[i]]$data==2)) {
                    geno[[i]]$data[a>0 & geno[[i]]$data==2] <-
                        (geno[[i]]$data+a)[a>0 & geno[[i]]$data==2]
                    geno[[i]]$data[geno[[i]]$data>3] <- 1
                }
                if(any(a>0 & geno[[i]]$data==3))
                    geno[[i]]$data[a>0 & geno[[i]]$data==3] <-
                        (geno[[i]]$data-a)[a>0 & geno[[i]]$data==3]
            }

            if(keep.errorind) {
                errors <- matrix(0,n.ind,n.mar[i])
                errors[a>0] <- 1
                colnames(errors) <- colnames(geno[[i]]$data)
                geno[[i]]$errors <- errors
            }

        } # end loop over chromosomes
    } # end simulate genotyping errors

    # add partial missing
    if(partial.missing.prob > 0) {
        for(i in 1:n.chr) {
            if(chr.type[i] != "X") {
                o <- sample(c(TRUE,FALSE),n.mar[i],replace=TRUE,
                            prob=c(partial.missing.prob,1-partial.missing.prob))
                if(any(o)) {
                    o2 <- grep("^QTL[0-9]+",mar.names[[i]])
                    if(length(o2)>0)
                        x <- geno[[i]]$data[,o2]
                    m <- (1:n.mar[i])[o]
                    for(j in m) {
                        if(runif(1) < 0.5)
                            geno[[i]]$data[geno[[i]]$data[,j]==1 | geno[[i]]$data[,j]==2,j] <- 4
                        else
                            geno[[i]]$data[geno[[i]]$data[,j]==3 | geno[[i]]$data[,j]==2,j] <- 5
                    }
                    if(length(o2)>0)
                        geno[[i]]$data[,o2] <- x
                }
            }

        } # end loop over chromosomes
    } # end simulate partially missing data

    # add missing
    if(missing.prob > 0) {
        for(i in 1:n.chr) {
            o <- grep("^QTL[0-9]+",mar.names[[i]])
            if(length(o)>0)
                x <- geno[[i]]$data[,o]
            geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,replace=TRUE,
                                  prob=c(missing.prob,1-missing.prob))] <- NA
            if(length(o)>0)
                geno[[i]]$data[,o] <- x
        }
    }

    pheno <- data.frame(phenotype=pheno, stringsAsFactors=TRUE)

    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c("f2","cross")

    cross
}

######################################################################
#
# sim.cross.4way
#
######################################################################

sim.cross.4way <-
    function(map,model,n.ind,error.prob,missing.prob,partial.missing.prob,
             keep.errorind,m,p,map.function)
{
    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    if(!all(sapply(map,is.matrix)))
        stop("Map must be sex-specific.")

    n.chr <- length(map)
    if(is.null(model)) n.qtl <- 0
    else {
        if(!((!is.matrix(model) && length(model) == 5) ||
             (is.matrix(model) && ncol(model) == 5))) {
            stop("Model must be a matrix with 5 columns (chr, pos and effects).")
        }
        if(!is.matrix(model)) model <- rbind(model)
        n.qtl <- nrow(model)
        if(any(model[,1] < 0 | model[,1] > n.chr))
            stop("Chromosome indicators in model matrix out of range.")
        model[,2] <- model[,2]+1e-14 # so QTL not on top of marker
    }

    chr.type <- sapply(map, chrtype)

    # if any QTLs, place qtls on map
    if(n.qtl > 0) {
        for(i in 1:n.qtl) {
            temp <- map[[model[i,1]]]
            temp1 <- temp[1,]
            temp2 <- temp[2,]
            qtlloc <- model[i,2]

            if(qtlloc < min(temp1)) {
                temp1 <- c(qtlloc,temp1)
                temp2 <- min(temp2) - (min(temp1)-qtlloc)/diff(range(temp1))*diff(range(temp2))
                temp1 <- temp1-min(temp1)
                temp2 <- temp2-min(temp2)
                n <- c(paste("QTL",i,sep=""),colnames(temp))
            }
            else if(qtlloc > max(temp1)) {
                temp1 <- c(temp1,qtlloc)
                temp2 <- (qtlloc-max(temp1))/diff(range(temp1))*diff(range(temp2))+max(temp2)
                n <- c(colnames(temp),paste("QTL",i,sep=""))
            }
            else {
                temp1 <- c(temp1,qtlloc)
                o <- order(temp1)
                wh <- (seq(along=temp1))[order(temp1)==length(temp1)]
                temp2 <- c(temp2[1:(wh-1)],NA,temp2[-(1:(wh-1))])
                temp2[wh] <- temp2[wh-1] + (temp1[wh]-temp1[wh-1])/(temp1[wh+1]-temp1[wh-1]) *
                    (temp2[wh+1]-temp2[wh-1])
                temp1 <- sort(temp1)
                n <- c(colnames(temp),paste("QTL",i,sep=""))[o]
            }
            map[[model[i,1]]] <- rbind(temp1,temp2)
            dimnames(map[[model[i,1]]]) <- list(NULL, n)
        }
    }

    geno <- vector("list", n.chr)
    names(geno) <- names(map)
    n.mar <- sapply(map,ncol)
    mar.names <- lapply(map,function(a) colnames(a))

    for(i in 1:n.chr) {

        # simulate sex
        sex <- NULL
        if(chr.type[i]=="X")
            sex <- rep(0,n.ind)

        # simulate genotype data
        thedata <- sim.bcg(n.ind, map[[i]], m, p, map.function)
        dimnames(thedata) <- list(NULL,mar.names[[i]])

        if(chr.type[i] != "X")
            thedata <- thedata + 2*sim.bcg(n.ind, map[[i]][2:1,], m, p, map.function) - 2

        dimnames(thedata) <- list(NULL,mar.names[[i]])

        geno[[i]] <- list(data = thedata, map = map[[i]])

        class(geno[[i]]) <- chr.type[i]
        class(geno[[i]]$map) <- NULL
    } # end loop over chromosomes

    # simulate phenotypes
    pheno <- rnorm(n.ind,0,1)

    if(n.qtl > 0) {
        # find QTL positions
        QTL.chr <- QTL.loc <- NULL
        for(i in 1:n.chr) {
            o <- grep("^QTL[0-9]+",mar.names[[i]])
            if(length(o)>0) {
                QTL.chr <- c(QTL.chr,rep(i,length(o)))
                QTL.loc <- c(QTL.loc,o)
            }
        }

        # incorporate QTL effects
        for(i in 1:n.qtl) {
            QTL.geno <- geno[[QTL.chr[i]]]$data[,QTL.loc[i]]
            pheno[QTL.geno==1] <- pheno[QTL.geno==1] + model[i,3]
            pheno[QTL.geno==2] <- pheno[QTL.geno==2] + model[i,4]
            pheno[QTL.geno==3] <- pheno[QTL.geno==3] + model[i,5]
        }

    } # end simulate phenotype

    n.mar <- sapply(geno, function(a) ncol(a$map))

    # add errors
    if(error.prob > 0) {
        for(i in 1:n.chr) {
            if(chr.type[i] != "X") { # 4-way cross; autosomal
                a <- sample(0:3,n.mar[i]*n.ind,replace=TRUE,
                            prob=c(1-error.prob,rep(error.prob/3,3)))
                if(any(a>0 & geno[[i]]$data==1))
                    geno[[i]]$data[a>0 & geno[[i]]$data==1] <-
                        geno[[i]]$data[a>0 & geno[[i]]$data==1] + a[a>0 & geno[[i]]$data==1]
                if(any(a>0 & geno[[i]]$data==2))
                    geno[[i]]$data[a>0 & geno[[i]]$data==2] <-
                        geno[[i]]$data[a>0 & geno[[i]]$data==2] + c(-1,1,2)[a[a>0 & geno[[i]]$data==2]]
                if(any(a>0 & geno[[i]]$data==3))
                    geno[[i]]$data[a>0 & geno[[i]]$data==3] <-
                        geno[[i]]$data[a>0 & geno[[i]]$data==3] + c(-2,-1,1)[a[a>0 & geno[[i]]$data==3]]
                if(any(a>0 & geno[[i]]$data==4))
                    geno[[i]]$data[a>0 & geno[[i]]$data==4] <-
                        geno[[i]]$data[a>0 & geno[[i]]$data==4] - a[a>0 & geno[[i]]$data==4]
            }
            else {
                a <- sample(0:1,n.mar[i]*n.ind,replace=TRUE,
                            prob=c(1-error.prob,error.prob))
                if(any(a>0 & geno[[i]]$data==1))
                    geno[[i]]$data[a>0 & geno[[i]]$data==1] <-
                        geno[[i]]$data[a>0 & geno[[i]]$data==1] + 1
                if(any(a>0 & geno[[i]]$data==2))
                    geno[[i]]$data[a>0 & geno[[i]]$data==2] <-
                        geno[[i]]$data[a>0 & geno[[i]]$data==2] - 1
                if(any(a>0 & geno[[i]]$data==3))
                    geno[[i]]$data[a>0 & geno[[i]]$data==3] <-
                        geno[[i]]$data[a>0 & geno[[i]]$data==3] + 1
                if(any(a>0 & geno[[i]]$data==4))
                    geno[[i]]$data[a>0 & geno[[i]]$data==4] <-
                        geno[[i]]$data[a>0 & geno[[i]]$data==4] - 1
            }

            if(keep.errorind) {
                errors <- matrix(0,n.ind,n.mar[i])
                errors[a>0] <- 1
                colnames(errors) <- colnames(geno[[i]]$data)
                geno[[i]]$errors <- errors
            }

        } # end loop over chromosomes
    } # end simulate genotyping errors

    # add partial missing
    if(partial.missing.prob > 0) {
        for(i in 1:n.chr) {
            if(chr.type[i] != "X") {
                o <- sample(c(TRUE,FALSE),n.mar[i],replace=TRUE,
                            prob=c(partial.missing.prob,1-partial.missing.prob))

                if(any(o)) {
                    o2 <- grep("^QTL[0-9]+",mar.names[[i]])
                    if(length(o2)>0)
                        x <- geno[[i]]$data[,o2]
                    m <- (1:n.mar[i])[o]
                    for(j in m) {
                        a <- sample(1:4,1)
                        if(a==1) { # AB:AA marker
                            geno[[i]]$data[geno[[i]]$data[,j]==1 | geno[[i]]$data[,j]==3,j] <- 5
                            geno[[i]]$data[geno[[i]]$data[,j]==2 | geno[[i]]$data[,j]==4,j] <- 6
                        }
                        else if(a==2) { # AA:AB marker
                            geno[[i]]$data[geno[[i]]$data[,j]==1 | geno[[i]]$data[,j]==2,j] <- 7
                            geno[[i]]$data[geno[[i]]$data[,j]==3 | geno[[i]]$data[,j]==4,j] <- 8
                        }
                        else if(a==3)  # AB:AB marker
                            geno[[i]]$data[geno[[i]]$data[,j]==2 | geno[[i]]$data[,j]==3,j] <- 10
                        else  # AB:BA marker
                            geno[[i]]$data[geno[[i]]$data[,j]==1 | geno[[i]]$data[,j]==4,j] <- 9
                    }
                    if(length(o2) > 0)
                        geno[[i]]$data[,o2] <- x
                }
            }

        } # end loop over chromosomes
    } # end simulate partially missing data

    # add missing
    if(missing.prob > 0) {
        for(i in 1:n.chr) {
            o <- grep("^QTL[0-9]+",mar.names[[i]])
            if(length(o)>0)
                x <- geno[[i]]$data[,o]
            geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,replace=TRUE,
                                  prob=c(missing.prob,1-missing.prob))] <- NA
            if(length(o)>0)
                geno[[i]]$data[,o] <- x
        }
    }

    if(!is.null(sex)) {
        pheno <- cbind(pheno,sex)
        dimnames(pheno) <- list(NULL, c("phenotype", "sex"))
    }
    else {
        pheno <- cbind(pheno)
        dimnames(pheno) <- list(NULL, "phenotype")
    }

    pheno <- as.data.frame(pheno, stringsAsFactors=TRUE)
    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c("4way","cross")

    cross
}


######################################################################
# sim.bcg
#
# simulate backcross genotype data for a single chromosome;
# output is a matrix of 1's and 0's
######################################################################

sim.bcg <-
    function(n.ind, map, m, p,
             map.function=c("haldane","kosambi","c-f","morgan"))
{
    map.function <- match.arg(map.function)

    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    if(m < 0 || p < 0 || p > 1)
        stop("Must have m >= 0 and 0 <= p <= 1")

    if(is.matrix(map)) map <- map[1,]
    map <- map-map[1]
    n.mar <- length(map)

    if(m==0 || p==1) { # no interference
        g <- .C("R_sim_bc_ni",
                as.integer(n.mar),
                as.integer(n.ind),
                as.double(mf(diff(map))),
                g=as.integer(rep(0, n.mar*n.ind)),
                PACKAGE="qtl")$g
    }
    else {
        g <- .C("R_sim_bc",
                as.integer(n.mar),
                as.integer(n.ind),
                as.double(map),
                as.integer(m),
                as.double(p),
                g=as.integer(rep(0, n.mar*n.ind)),
                PACKAGE="qtl")$g
    }
    matrix(g, ncol=n.mar)
}



# end of simulate.R

######################################################################
#
# sim.geno.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: sim.geno
#
######################################################################

######################################################################
#
# sim.geno: simulate from the joint distribution Pr(g | O)
#
######################################################################

sim.geno <-
    function(cross, n.draws=16, step=0, off.end=0, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             stepwidth=c("fixed", "variable", "max"))
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    # map function
    map.function <- match.arg(map.function)
    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    stepwidth <- match.arg(stepwidth)

    # don't let error.prob be exactly zero, just in case
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }

    n.ind <- nind(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)
    type <- crosstype(cross)

    # calculate genotype probabilities one chromosome at a time
    for(i in 1:n.chr) {
        if(n.mar[i]==1) temp.offend <- max(c(off.end,5))
        else temp.offend <- off.end

        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type=="X") xchr <- TRUE
        else xchr <- FALSE

        # which type of cross is this?
        if(type == "f2") {
            n.gen <- 3
            one.map <- TRUE
            if(!xchr) # autosomal
                cfunc <- "sim_geno_f2"
            else                              # X chromsome
                cfunc <- "sim_geno_bc"
        }
        else if(type == "bc" || type=="riself" || type=="risib" || type=="dh" || type=="haploid") {
            cfunc <- "sim_geno_bc"
            n.gen <- 2
            one.map <- TRUE
        }
        else if(type == "4way") {
            n.gen <- 4
            cfunc <- "sim_geno_4way"
            one.map <- FALSE
        }
        else if(type=="ri8sib" || type=="ri4sib" || type=="ri8self" || type=="ri4self" || type=="bgmagic16") {
            cfunc <- paste("sim_geno_", type, sep="")
            if(type=="bgmagic16") n.gen <- 16
            else n.gen <- as.numeric(substr(type, 3, 3))
            one.map <- TRUE
            if(xchr)
                warning("sim.geno not working properly for the X chromosome for 4- or 8-way RIL.")
        }
        else if(type == "bcsft") {
            one.map <- TRUE
            cfunc <- "sim_geno_bcsft"
            cross.scheme <- attr(cross, "scheme") ## c(s,t) for BC(s)F(t)
            if(!xchr) {# autosomal
                n.gen <- 3 - (cross.scheme[1] == 0)
            }
            else {                             # X chromsome
                cross.scheme[1] <- cross.scheme[1] + cross.scheme[2] - (cross.scheme[1] == 0)
                cross.scheme[2] <- 0
                n.gen <- 2
            }
        }
        else
            stop("sim_geno not available for cross type ", type, ".")

        # genotype data
        gen <- cross$geno[[i]]$data
        gen[is.na(gen)] <- 0

        # recombination fractions
        if(one.map) {
            # recombination fractions
            map <- create.map(cross$geno[[i]]$map,step,temp.offend,stepwidth)
            rf <- mf(diff(map))
            if(type=="risib" || type=="riself")
                rf <- adjust.rf.ri(rf, sub("^ri", "", type), chr_type)
            rf[rf < 1e-14] <- 1e-14

            # new genotype matrix with pseudomarkers filled in
            newgen <- matrix(ncol=length(map),nrow=nrow(gen))
            dimnames(newgen) <- list(NULL,names(map))
            newgen[,colnames(gen)] <- gen
            newgen[is.na(newgen)] <- 0
            n.pos <- ncol(newgen)
        }
        else {
            map <- create.map(cross$geno[[i]]$map,step,temp.offend,stepwidth)
            rf <- mf(diff(map[1,]))
            rf[rf < 1e-14] <- 1e-14
            rf2 <- mf(diff(map[1,]))
            rf2[rf2 < 1e-14] <- 1e-14

            # new genotype matrix with pseudomarkers filled in
            newgen <- matrix(ncol=ncol(map),nrow=nrow(gen))
            dimnames(newgen) <- list(NULL,colnames(map))
            newgen[,colnames(gen)] <- gen
            newgen[is.na(newgen)] <- 0
            n.pos <- ncol(newgen)
        }


        # call C function
        if(one.map) {
            ## Hide cross scheme in genoprob to pass to routine. BY
            temp <- as.integer(rep(0,n.draws*n.ind*n.pos))
            if(type == "bcsft")
                temp[1:2] <- cross.scheme

            z <- .C(cfunc,
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(n.draws),       # number of simulation replicates
                    as.integer(newgen),        # genotype data
                    as.double(rf),             # recombination fractions
                    as.double(error.prob),     #
                    draws=as.integer(temp),
                    PACKAGE="qtl")

            cross$geno[[i]]$draws <- array(z$draws,dim=c(n.ind,n.pos,n.draws))
            dimnames(cross$geno[[i]]$draws) <- list(NULL, names(map), NULL)
        }
        else {
            z <- .C(cfunc,
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(n.draws),       # number of simulation replicates
                    as.integer(newgen),        # genotype data
                    as.double(rf),             # recombination fractions
                    as.double(rf2),            # recombination fractions
                    as.double(error.prob),     #
                    draws=as.integer(rep(0,n.draws*n.ind*n.pos)),
                    PACKAGE="qtl")

            cross$geno[[i]]$draws <- array(z$draws,dim=c(n.ind,n.pos,n.draws))
            dimnames(cross$geno[[i]]$draws) <- list(NULL, colnames(map), NULL)

        }

        # attribute set to the error.prob value used, for later
        #     reference
        attr(cross$geno[[i]]$draws, "map") <- map
        attr(cross$geno[[i]]$draws,"error.prob") <- error.prob
        attr(cross$geno[[i]]$draws,"step") <- step
        attr(cross$geno[[i]]$draws,"off.end") <- temp.offend
        attr(cross$geno[[i]]$draws,"map.function") <- map.function
        attr(cross$geno[[i]]$draws,"stepwidth") <- stepwidth
    }

    # store simulated genotypes as integers
    for(i in 1:nchr(cross))
        storage.mode(cross$geno[[i]]$draws) <- "integer"

    return(cross)

    # 4- and 8-way RIL: reorganize the results
    if(type=="ri4self" || type=="ri4sib" || type=="ri8self" || type=="ri8sib" || type=="bgmagic16")
        cross <- reorgRIdraws(cross)

    cross
}

# end of sim.geno.R

#####################################################################
#
# mqmprepare.R
#
# Copyright (c) 2009-2010, Danny Arends
#
# Modified by Pjotr Prins and Karl Broman
#
#
# first written Februari 2009
# last modified April 2010
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: mqmfind.marker
#
#
#####################################################################

mqmfind.marker <- function(cross,mqmscan=NULL,perm=NULL,alpha=0.05,verbose=FALSE){
    thesum <- summary(mqmscan[,1:3],alpha=alpha,perms=perm,pvalues=FALSE)
    chr <- thesum$'chr'
    pos <- thesum$'pos'
    if(verbose) cat("INFO: Found",length(chr),"markers with alpha <",alpha,".\n")
    ret <- NULL
    for(i in 1:length(chr)){
        ret <- rbind(ret,cbind(find.marker(cross,chr=chr[i],pos=pos[i]),as.integer(chr[i]),as.double(pos[i])))
    }
    colnames(ret) <- c("marker","chr","pos (cM)")
    ret
}



# end of mqmprepare.R

######################################################################
#
# test_scanon_mr.R
#
# copyright (c) 2009 Pjotr Prins
# first written July 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Some basic regression/integration testing for some of the QTL mapping routines
#
# You can run it with:
#
#   R --no-save --no-restore --no-readline --slave < ./tests/test_qtl.R 

######################################################################

library(qtl)

data(listeria)

mr = scanone(listeria,method='mr')

sink('regression/scanone_mr.rnew')
mr
sink()

cat("test_qtl.R tests succesfully run!")

######################################################################
# plotModel.R
#
# copyright (c) 2008-9, Karl W Broman
# last modified January, 2009
# first written Apr, 2008
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: plotModel
#
######################################################################

######################################################################
# plotModel
#
# plot a QTL model
######################################################################
plotModel <-
    function(qtl, formula, circrad.rel=0.25, circrad.abs,
             cex.name=1, chronly=FALSE, order, ...)
{
    if(missing(qtl))
        stop("Must provide qtl object or a vector of QTL names")

    if(is.character(qtl)) { # qtl names
        if(missing(formula)) formula <- NULL
    }
    else {
        if(missing(formula)) {
            if("formula" %in% names(attributes(qtl)))
                formula <- attr(qtl, "formula")
            else {
                if("formula" %in% names(qtl))
                    formula <- qtl$formula
                else formula <- NULL
            }
        }
        if(length(qtl)>0) {

            if(chronly) qtl <- qtl$chr
            else {
                if("name" %in% names(qtl))
                    qtl <- qtl$name
                else {
                    if("pos" %in% names(qtl))
                        qtl <- paste(qtl$chr, roundqtlpos(qtl$pos, 1), sep="@")
                    else qtl <- qtl$chr
                }
            }
        }
    }

    nqtl <- length(qtl)

    if(!is.null(formula)) {
        if(is.character(formula))
            formula <- as.formula(formula)

        theterms <- attr(terms(formula), "factors")[-1,]
        rn <- rownames(theterms)
        g <- grep("^[Qq][0-9]+$", rn)
        qtlnum <- as.numeric(substr(rn[g], 2, nchar(rn[g])))
        if(any(qtlnum < 1 | qtlnum > nqtl))
            stop("QTL in formula must be numbered between 1 and ", nqtl)

        cn <- colnames(theterms)
        intxn <- cn[grep("^[Qq][0-9]+:[Qq][0-9]+$", cn)]
    }
    else intxn <- NULL

    plot(0,0,type="n", xlab="", ylab="", xaxt="n", yaxt="n",
         xaxs="i", yaxs="i", xlim=c(-1.5,1.5), ylim=c(-1.5,1.5),
         ...)

    if(!is.null(qtl) && length(qtl) > 0) {
        radloc <- rev(seq(pi/2, 2.5*pi, len=nqtl+1)[-1])
        xloc <- cos(radloc)
        yloc <- sin(radloc)

        rad <- seq(0, 2*pi, length=100)
        if(missing(circrad.abs)) {
            if(length(xloc) == 1)
                circrad.abs <- circrad.rel*2
            else
                circrad.abs <- circrad.rel * sqrt(diff(xloc[1:2])^2 + diff(yloc[1:2])^2)
            if(circrad.abs > 0.45) circrad.abs <- 0.45
        }
        # use the smallest of the two relative lengths
        u <- par("usr")
        pin <- par("pin")
        pin <- pin/c(diff(u[1:2]), diff(u[3:4]))
        circrad.abs<- circrad.abs/(pin/min(pin))

        if(!missing(order)) {
            if(length(order) != length(qtl))
                stop("order should have length ", nqtl)
            if(!all(sort(order) == seq(along=qtl)))
                stop("order should be a permutaton of 1,2,...,", nqtl)
            xloc <- xloc[order(order)]
            yloc <- yloc[order(order)]
        }

        if(length(intxn) > 0) {
            for(i in intxn) {
                thisint <- strsplit(i, ":")[[1]]
                theseqtl <- as.numeric(substr(thisint, 2, nchar(thisint)))
                lines(xloc[theseqtl], yloc[theseqtl], lwd=2)
            }
        }

        for(i in seq(along=xloc))
            polygon(xloc[i]+cos(rad)*circrad.abs[1], yloc[i]+sin(rad)*circrad.abs[2], col="white", border="black", lwd=2)

        text(xloc, yloc, qtl, cex=cex.name)
    }

    invisible()
}

roundqtlpos <-
    function (x, digits = 1)
{
    if (digits < 1)
        stop("This is intended for the case digits >= 1.")
    y <- as.character(round(x, digits))
    z <- strsplit(y, "\\.")
    sapply(z, function(a, digits) {
        if (length(a) == 1)
            b <- paste(a[1], ".", paste(rep("0", digits), collapse = ""),
                       sep = "")
        else {
            if (nchar(a[2]) == digits)
                b <- paste(a, collapse = ".")
            else b <- paste(a[1], ".", a[2], paste(rep("0", digits -
                                                       nchar(a[2])), collapse = "."), sep = "")
        }
    }, digits)
}

# end of plotModel.R

######################################################################
#
# read.cross.mm.R
#
# copyright (c) 2000-2020, Karl W Broman
# last modified Feb, 2020
# first written Aug, 2000
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.mm, read.maps.mm
#           [See read.cross.R for the main read.cross function.]
#
######################################################################

######################################################################
#
# read.cross.mm: read data from an experimental cross in mapmaker
#                format.
#
# We need two files: a "raw" file containing the genotype and
# phenotype data and a "map" file containing the chromosomes
# assignments and (optionally) map positions.
#
# The map file contains two or three columns, separated by white
# space, with the chromosome number, marker name (with markers in
# order along the chromosomes) and (optionally) the map position.
#
######################################################################

read.cross.mm <-
    function(dir,rawfile,mapfile,estimate.map=TRUE)
{
    # create file names
    if(missing(mapfile)) stop("Missing mapfile.")
    if(missing(rawfile)) stop("Missing rawfile.")
    if(!missing(dir)  && dir != "") {
        mapfile <- file.path(dir, mapfile)
        rawfile <- file.path(dir, rawfile)
    }

    # count lines in rawfile
    n.lines <- length(scan(rawfile, what=character(), skip=0, nlines=0,
                           blank.lines.skip=FALSE,quiet=TRUE,sep="\n"))

    # read map file
    map <- read.table(mapfile,header=FALSE,colClasses="character",blank.lines.skip=FALSE,stringsAsFactors=TRUE)
    fixmap <- TRUE
    if(ncol(map) == 1)
        stop("Map file should contain the markers' chromosome IDs.")

    if(ncol(map) > 3) { # special maps format
        maps <- read.maps.mm(mapfile)
        chr <- rep(names(maps),sapply(maps,length))
        markers <- unlist(lapply(maps,names))
        includes.pos <- TRUE
        fixmap <- FALSE
    }

    if(fixmap) { # my map format: 2 or 3 column table
        # remove any rows lacking a chromosome ID
        o <- (1:nrow(map))[map[,1]==""]
        if(length(o) > 0) map <- map[-o,]

        # remove any leading *'s from the marker names
        g <- grep("^\\*",map[,2])
        if(length(g) > 0)
            map[g,2] <- substr(map[g,2],2,nchar(map[g,2]))
    }

    # begin reading/parsing the genotype data
    cur.mar <- 0
    cur.phe <- 0
    NEW.symb <- c("1","2","3","4","5","0")
    OLD.symb <- c("A","H","B","D","C","-")

    flag <- 0
    #  rawdata <- scan(rawfile,what=character(),sep="\n",
    #                  blank.lines.skip=TRUE,quiet=TRUE)
    #
    for(i in 1:n.lines) {
        a <- scan(rawfile,what=character(),skip=i-1,nlines=1,
                  blank.lines.skip=TRUE,quiet=TRUE)

        if(length(a) == 0) next
        if(length(grep("#", a[1])) != 0) next

        if(flag == 0) {
            flag <- 1
            if(!is.na(match("intercross", a))) type <- "f2"
            else if(!is.na(match("backcross", a)) || !is.na(match("bc", a))) type <- "bc"
            else
                stop("File indicates invalid cross type: ", a[length(a)], ".")
        }
        else if(flag == 1) {
            flag <- 2
            n.ind <- as.numeric(a[1])
            n.mar <- as.numeric(a[2])
            n.phe <- as.numeric(a[3])
            cat(" --Read the following data:\n")
            cat("\tType of cross:         ", type, "\n")
            cat("\tNumber of individuals: ", n.ind, "\n")
            cat("\tNumber of markers:     ", n.mar, "\n")
            cat("\tNumber of phenotypes:  ", n.phe, "\n")

            # if there's a set of "symbols" for non-standard symbols in
            #     the file, use them.
            if(length(a) > 3 && ("symbols" %in% a)) {
                o <- match("symbols",a)
                b <- a[-(1:o)]
                infile.symb <- substring(b,1,1)
                std.symb <- substring(b,3,3)

                wh <- rep(0,length(std.symb))
                fixed <- rep(0,length(OLD.symb))
                for(j in 1:length(std.symb))
                    if(std.symb[j] %in% OLD.symb)
                        wh[j] <- match(std.symb[j],OLD.symb)
                for(j in 1:length(std.symb))
                    if(wh[j] != 0) {
                        OLD.symb[wh[j]] <- infile.symb[j]
                        fixed[wh[j]] <- 1
                    }

                temp <- table(OLD.symb)
                if(any(temp>1)) {
                    for(j in names(temp)[temp>1]) {
                        o <- OLD.symb==j & fixed==0
                        if(any(o)) OLD.symb[o] <- paste(OLD.symb[o],"   ")
                    }
                }
            }

            marnames <- rep("", n.mar)
            geno <- matrix(0,ncol=n.mar,nrow=n.ind)
            if(n.phe == 0) {
                pheno <- matrix(1:n.ind,ncol=1)
                phenames <- c("number")
            }
            else {
                pheno <- matrix(0,ncol=n.phe,nrow=n.ind)
                phenames <- rep("", n.phe)
            }

        }
        else {
            if(substring(a[1],1,1) == "*") {
                cur.mar <- cur.mar+1
                cur.row <- 1

                if(cur.mar > n.mar) { # now reading phenotypes
                    cur.phe <- cur.phe+1
                    if(cur.phe > n.phe) next
                    phenames[cur.phe] <- substring(a[1],2)
                    if(length(a) > 1) {
                        p <- a[-1]
                        p[p=="-"] <- NA
                        n <- length(p)
                        oldna <- is.na(p)
                        numerp <- suppressWarnings(as.numeric(p))
                        newna <- is.na(numerp)
                        wh <- !oldna & newna
                        if(any(wh)) {
                            droppedasmissing <- unique(p[wh])
                            if(length(droppedasmissing) > 1) {
                                themessage <- paste("The values", paste("\"", droppedasmissing, "\"", sep="", collapse=" "))
                                themessage <- paste(themessage, " for phenotype \"", phenames[cur.phe], "\" were", sep="")
                            }
                            else {
                                themessage <- paste("The value \"", droppedasmissing, "\" ", sep="")
                                themessage <- paste(themessage, " for phenotype \"", phenames[cur.phe], "\" was", sep="")
                            }
                            themessage <- paste(themessage, "interpreted as missing.")
                            warning(themessage)
                        }

                        pheno[cur.row+(0:(n-1)),cur.phe] <- numerp
                    }
                    else n <- 0 ## ?
                    cur.row <- cur.row + n
                }

                else { # reading genotypes
                    marnames[cur.mar] <- substring(a[1],2)
                    if(length(a) > 1) {
                        g <- paste(a[-1],collapse="")
                        h <- g <- unlist(strsplit(g,""))
                        for(j in seq(along=NEW.symb)) {
                            if(any(h==OLD.symb[j]))
                                g[h==OLD.symb[j]] <- NEW.symb[j]
                        }

                        n <- length(g)

                        geno[cur.row+(0:(n-1)),cur.mar] <- as.numeric(g)
                    }
                    else n <- 0
                    cur.row <- cur.row + n
                }

            }
            else { # continuation lines
                if(cur.mar > n.mar) { # now reading phenotypes
                    a[a=="-"] <- NA
                    n <- length(a)
                    pheno[cur.row+(0:(n-1)),cur.phe] <- as.numeric(a)
                    cur.row <- cur.row + n
                }
                else {
                    g <- paste(a,collapse="")
                    h <- g <- unlist(strsplit(g,""))
                    for(j in seq(along=NEW.symb)) {
                        if(any(h==OLD.symb[j]))
                            g[h==OLD.symb[j]] <- NEW.symb[j]
                    }
                    n <- length(g)
                    geno[cur.row+(0:(n-1)),cur.mar] <- as.numeric(g)
                    cur.row <- cur.row + n
                }
            } # end continuation line
        } # end non-intro line
    }
    dimnames(pheno) <- list(NULL, phenames)
    # done reading the raw file

    if(fixmap) { # my map format: 2 or 3 column table
        # parse map file
        if(ncol(map) == 3) {
            includes.pos <- TRUE
            # make positions numeric
            pos <- as.numeric(map[,3])
        }
        else includes.pos <- FALSE

        chr <- as.character(map[,1])
        markers <- map[,2]

        # reorder markers?
        if(all(chr %in% c(1:999,"X","x"))) { # 1...19 + X
            tempchr <- chr
            tempchr[chr=="X" | chr=="x"] <- 1000
            tempchr <- as.numeric(tempchr)
            if(includes.pos) neworder <- order(tempchr, pos)
            else neworder <- order(tempchr)
        } else {
            # prevent reordering of chromosomes
            tempchr <- factor(chr, levels=unique(chr))

            if(includes.pos) neworder <- order(tempchr, pos)
            else neworder <- order(tempchr)
        }
        chr <- chr[neworder]
        if(includes.pos) pos <- pos[neworder]
        markers <- markers[neworder]
    }

    Geno <- vector("list",length(unique(chr)))
    names(Geno) <- unique(chr)

    for(i in unique(chr)) {
        mar <- markers[chr == i]

        if(fixmap) { # my map format: 2 or 3 column table
            # create map
            if(includes.pos) {
                map <- pos[chr == i]

                # reorder markers?
                if(any(diff(map)<0)) {
                    o <- order(map)
                    map <- map[o]
                    mar <- mar[o]
                }
            }
            else map <- seq(0,by=5,length=length(mar))
            names(map) <- mar
        }
        else map <- maps[[i]]

        # pull out genotype data
        o <- match(mar,marnames)
        if(any(is.na(o)))
            stop("Cannot find markers in genotype data: ",
                 paste(mar[is.na(o)],collapse=" "), ".",sep="")

        if(length(o)==1) data <- matrix(geno[,o],ncol=1)
        else data <- geno[,o]
        # add marker names to data
        colnames(data) <- mar
        # changes 0's to NA's
        data[!is.na(data) & data==0] <- NA

        Geno[[i]] <- list(data=data,map=map)
        if(i=="X" || i=="x") class(Geno[[i]]) <- "X"
        else class(Geno[[i]]) <- "A"
    }

    cross <- list(geno=Geno,pheno=pheno)
    class(cross) <- c(type,"cross")

    if(estimate.map && !includes.pos) estmap <- TRUE
    else estmap <- FALSE

    cross$pheno <- as.data.frame(cross$pheno, stringsAsFactors=TRUE)

    # return cross + indicator of whether to run est.map
    list(cross,estmap)
}

######################################################################
#
# read.maps.mm: Read genetic map for a special Mapmaker format
# Written by Brian S Yandell; modified by Karl W Broman
#
######################################################################
read.maps.mm <-
    function( mapsfile )
{
    if (missing(mapsfile)) stop("Missing mapsfile.")

    ## find where everything is
    f <- scan(mapsfile, what = "", blank.lines.skip = FALSE, sep = "\n",
              quiet = TRUE)
    start <- pmatch( paste( "*", c("OrderInfo","Classes","Chromosomes",
                                   "Assignments and Placements" ), ":", sep = "" ), f )

    ## marker names
    f <- scan( mapsfile, what = c("",rep(0,9)), skip = start[1],
              nlines = start[2] - start[1] - 1,
              blank.lines.skip = FALSE, quiet = TRUE)
    markers <- substring( f[ seq( 1, length( f ), by = 10 ) ], 2 )

    ## distances
    f <- scan( mapsfile, what = "", skip = start[3],
              nlines = start[4] - start[3] - 1,
              blank.lines.skip = FALSE, quiet = TRUE)
    chr <- grep( "^\\*", f)
    chrom <- substring( f[chr], 2 )
    nmark <- as.integer( f[ 1 + chr ] )
    chr <- c( chr[-1], 1 + length( f ))
    lo <- chr - 2 * nmark + 2
    hi <- chr - nmark
    map <- list()
    imark <- c( 0, cumsum( nmark ))
    for( i in seq( along = chrom )) {
        tmp <- cumsum( c(0,imf.h(as.numeric( f[ lo[i]:hi[i] ] ))))
        names( tmp ) <- markers[ imark[i] + seq( nmark[i] ) ]
        map[[ chrom[i] ]] <- tmp
    }
    map
}

# end of read.cross.mm.R

######################################################################
#
# plot.scantwo.R
#
# copyright (c) 2001-2019, Karl W Broman, Hao Wu and Brian Yandell
# last modified Dec, 2019
# first written Nov, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Hao Wu (The Jackson Lab) wrote the initial code
#
# Part of the R/qtl package
# Contains: plot.scantwo
#
######################################################################

plot.scantwo <-
    function(x, chr, incl.markers = FALSE, zlim, lodcolumn=1,
             lower = c("full", "add", "cond-int", "cond-add", "int"),
             upper = c("int", "cond-add", "cond-int", "add", "full"),
             nodiag = TRUE,
             contours = FALSE, main, zscale = TRUE, point.at.max=FALSE,
             col.scheme = c("viridis", "redblue","cm","gray","heat","terrain","topo"),
             gamma=0.6, allow.neg=FALSE, alternate.chrid=FALSE, ...)
{
    if(!inherits(x, "scantwo"))
        stop("Input should have class \"scantwo\".")

    col.scheme <- match.arg(col.scheme)

    if(length(dim(x$lod)) > 2) { # results from multiple phenotypes
        if(length(lodcolumn) > 1) {
            warning("Argument lodcolumn should be of length 1.")
            lodcolumn <- lodcolumn[1]
        }

        if(lodcolumn < 0 || lodcolumn > dim(x$lod)[3])
            stop("Argument lodcolumn misspecified.")
        x$lod <- x$lod[,,lodcolumn]
    }

    if(!missing(chr)) x <- subset(x, chr=chr)
    if(nrow(x$lod)==0) {
        warning("Empty scantwo object.")

        return(invisible(NULL))
    }


    chr <- as.character(unique(x$map[,1]))

    addpair <- attr(x, "addpair")
    if(!is.null(addpair) && addpair) {
        lower <- "full"
        upper <- "add"
        if(missing(zlim)) {
            if(allow.neg)
                zlim <- rep(max(abs(x$lod), na.rm=TRUE), 2)
            else
                zlim <- rep(max(x$lod, na.rm=TRUE), 2)
        }
        addpair <- TRUE
    }
    else addpair <- FALSE

    if(length(lower)==1 && lower == "fv1") lower <- "cond-int"
    if(length(lower)==1 && lower == "av1") lower <- "cond-add"
    if(length(upper)==1 && upper == "fv1") upper <- "cond-int"
    if(length(upper)==1 && upper == "av1") upper <- "cond-add"

    lower <- match.arg(lower)
    upper <- match.arg(upper)
    if(!any(class(x) == "scantwo"))
        stop("Input variable is not an object of class scantwo!")
    lod <- x$lod
    map <- x$map

    # backward compatibility for previous version of R/qtl
    if(!("scanoneX" %in% names(x))) {
        warning("It would be best to re-run scantwo() with the R/qtl version 0.98 or later.\n")
        scanoneX <- NULL
    }
    else scanoneX <- x$scanoneX

    # if incl.markers is FALSE, drop positions
    #     for which third column of map is 0
    if(!incl.markers && any(map[, 3] == 0)) {
        o <- (map[, 3] == 1)
        lod <- lod[o, o]
        map <- map[o, ]
        if(!is.null(scanoneX)) scanoneX <- scanoneX[o]
    }

    if(all(diag(lod) < 1e-14) && (lower == "cond-int" || lower=="cond-add") )
        stop("Need to run scantwo with run.scanone=TRUE.")

    oldlod <- lod

    lo <- lower.tri(lod)
    up <- upper.tri(lod)

    # grab the interaction LOD scores
    if(upper=="int")
        lod[up] <- t(oldlod)[up] - oldlod[up]
    if(lower=="int")
        lod[lo] <- oldlod[lo] - t(oldlod)[lo]

    if(lower=="add")
        lod[lo] <- t(oldlod)[lo]

    if(upper=="full")
        lod[up] <- t(oldlod)[up]

    # get conditional LOD scores
    if(lower=="cond-int" || lower=="cond-add") {
        if(lower=="cond-add")
            lod[lo] <- t(oldlod)[lo]

        thechr <- map$chr
        uchr <- unique(thechr)
        thechr <- factor(as.character(thechr), levels=as.character(uchr))
        uchr <- factor(as.character(uchr), levels=levels(thechr))
        xchr <- tapply(map$xchr, thechr, function(a) a[1])

        maxo <- tapply(diag(lod), thechr, max, na.rm=TRUE)
        if(any(xchr) && !is.null(scanoneX)) {
            maxox <- tapply(scanoneX, thechr, max, na.rm=TRUE)
            maxo[xchr] <- maxox[xchr]
        }

        n.chr <- length(chr)
        for(i in 1:n.chr) {
            pi <- which(thechr==uchr[i])
            for(j in i:n.chr) {
                pj <- which(thechr==uchr[j])
                temp <- lod[pj,pi] - max(maxo[c(i,j)])
                temp[!is.na(temp) & temp<0] <- 0
                if(i==j) lod[pj,pi][lower.tri(temp)] <- temp[lower.tri(temp)]
                else lod[pj,pi] <- temp
            }
        }
    }

    if(upper=="cond-int" || upper=="cond-add") {
        if(upper=="cond-int")
            lod[up] <- t(oldlod)[up]

        thechr <- map$chr
        uchr <- unique(thechr)
        thechr <- factor(as.character(thechr), levels=as.character(uchr))
        uchr <- factor(as.character(uchr), levels=levels(thechr))
        xchr <- tapply(map$xchr, thechr, function(a) a[1])

        maxo <- tapply(diag(lod), thechr, max, na.rm=TRUE)
        if(any(xchr) && !is.null(scanoneX)) {
            maxox <- tapply(scanoneX, thechr, max, na.rm=TRUE)
            maxo[xchr] <- maxox[xchr]
        }

        n.chr <- length(chr)
        for(i in 1:n.chr) {
            pi <- which(thechr==uchr[i])
            for(j in i:n.chr) {
                pj <- which(thechr==uchr[j])
                temp <- lod[pi,pj] - max(maxo[c(i,j)])
                temp[!is.na(temp) & temp<0] <- 0
                if(i==j) lod[pi,pj][upper.tri(temp)] <- temp[upper.tri(temp)]
                else lod[pi,pj] <- temp
            }
        }
    }


    if(nodiag) diag(lod) <- 0

    # deal with bad LOD score values
    if(any(is.na(lod))) {
        u <- is.na(lod)
        n <- sum(u)
        warning(n, " LOD scores NA, set to 0")
        lod[u] <- 0
    }
    if(!allow.neg && any(!is.na(lod) & lod < -1e-6)) {
        u <- !is.na(lod) & lod < 0
        n <- sum(u)
        warning(n, " LOD scores <0, set to 0")
        lod[u] <- 0
    }
    if(any(!is.na(lod) & lod == Inf)) {
        u <- !is.na(lod) & lod == Inf
        n <- sum(u)
        warning(n, " LOD scores =Inf, set to maximum observed value")
        lod[u] <- max(lod[!is.na(lod) & lod < Inf])
    }

    if(missing(zlim)) { # no given zlim
        # calculate the zlim for interactive and full LODs
        if(allow.neg) {
            zlim.int <- max(abs(lod[row(lod) < col(lod)]))
            zlim.jnt <- max(abs(lod[row(lod) >= col(lod)]))
        }
        else {
            zlim.int <- max(lod[row(lod) < col(lod)])
            zlim.jnt <- max(lod[row(lod) >= col(lod)])
        }
    }
    else {
        zlim.jnt <- zlim[1]
        if(length(zlim) < 2)
            zlim.int <- zlim[1]
        else
            zlim.int <- zlim[2]
    }

    # rescale the data in upper triangle based on zlims.jnt
    lod[row(lod) < col(lod)] <- lod[row(lod) < col(lod)] * zlim.jnt/zlim.int

    # make sure LOD values are below (0,zlim.jnt) or update zlim.jnt
    #  if(max(lod) > zlim.jnt) {
    #    warning("LOD values out of range; updating zlim.")
    #    temp <- max(lod)
    #    zlim.int <- zlim.int * temp/zlim.jnt
    #    zlim.jnt <- temp
    #  }

    # save old par parameters, to restore them on exit
    if(zscale) {
        old.mar <- par("mar")
        old.mfrow <- par("mfrow")
        old.las <- par("las")
        on.exit(par(las = old.las, mar = old.mar, mfrow = old.mfrow))
    }
    else {
        old.las <- par("las")
        on.exit(par(las = old.las))
    }
    par(las = 1)
    dots <- list(...)
    if(zscale) {
        if("layout" %in% names(dots))
            layout(dots[["layout"]][[1]],dots[["layout"]][[2]])
        else
            layout(cbind(1, 2), c(6, 1))

        if("mar1" %in% names(dots))
            par(mar=dots[["mar1"]])
        else
            par(mar = c(5, 4, 4, 2) + 0.1)
    }
    if( gamma < 0 && col.scheme == "redblue")
        stop( "gamma must be non-negative" )
    cols <- switch(col.scheme,
                   gray = if( gamma <= 0) rev(gray(seq(0,1,len=256)))
                   else rev(gray(log(seq(1,exp(gamma),len=256))/gamma)),
                   heat = heat.colors(256),
                   terrain = terrain.colors(256),
                   topo = topo.colors(256),
                   cm = cm.colors(256),
                   redblue = rev(rainbow(256, start = 0, end = 2/3)),
                   viridis = viridis_qtl(256)
                   )
    if(col.scheme=="redblue") {
        # convert colors using gamma=0.6 (which will no longer be available in R)
        rgbval <- (col2rgb(cols)/255)^0.6
        cols <- rgb(rgbval[1,], rgbval[2,], rgbval[3,])
    }

    if(allow.neg) {
        lo <- -zlim.jnt
        lo.int <- -zlim.int
    }
    else lo.int <- lo <- 0

    if("xlab" %in% names(dots)) {
        xlab <- dots$xlab
        if("ylab" %in% names(dots))
            ylab <- dots$ylab
        else ylab <- xlab
    }
    else {
        if("ylab" %in% names(dots))
            xlab <- ylab <- dots$ylab
        else {
            if(length(chr) > 1)
                xlab <- ylab <- "Chromosome"
            else
                xlab <- ylab <- "Location (cM)"
        }
    }

    if(length(chr) > 1)
        image(1:ncol(lod), 1:nrow(lod), lod, ylab = ylab,
              xlab = xlab, zlim = c(lo, zlim.jnt), col = cols,
              xaxt = "n", yaxt = "n")
    else
        image(map[,2], map[,2], lod, ylab = ylab,
              xlab = xlab, zlim = c(lo, zlim.jnt), col = cols)

    # plot point at maximum, if requested
    if(point.at.max) {
        temp <- lod
        temp[upper.tri(temp)] <- -50
        temp[diag(temp)] <- -50
        wh <- which(temp == max(temp), arr.ind=TRUE)
        if(length(chr) > 1)
            points(wh,rev(wh),pch=4,lwd=2)
        else {
            points(map[wh[,1],2],map[wh[,2],2],pch=4,lwd=2,col="blue")
            points(map[wh[,2],2],map[wh[,1],2],pch=4,lwd=2,col="blue")
        }
    }

    # add contours if requested
    if(any(contours > 0)) {
        if(is.logical(contours))
            contours = 1.5
        tmp = lod
        tmp[row(lod) < col(lod)] <- NA
        if(length(chr) > 1)
            thepos <- 1:ncol(lod)
        else thepos <- map[,2]
        contour(thepos, thepos, tmp, add = TRUE,drawlabels=FALSE,
                levels = max(tmp,na.rm=TRUE) - contours, col = "blue",
                lwd = 2)
        tmp = lod
        tmp[row(lod) > col(lod)] <- NA
        contour(thepos, thepos, tmp, add = TRUE,drawlabels=FALSE,
                levels = max(tmp,na.rm=TRUE) - contours * zlim.jnt/zlim.int,
                col = "blue", lwd = 2)
    }

    if(length(chr) > 1) {
        # calculate how many markers in each chromesome
        n.mar <- NULL
        for(i in 1:length(chr)) n.mar[i] <- sum(map[, 1] == chr[i])

        # plot lines at the chromosome boundaries
        if(length(chr) > 1)
            wh <- c(0.5, cumsum(n.mar) + 0.5)
        abline(v = wh, xpd = FALSE)
        abline(h = wh, xpd = FALSE)

        # add chromesome numbers
        a <- par("usr")
        placement <- (wh[-1] + wh[-length(wh)])/2
        if(!alternate.chrid || length(chr)<2) {
            for(i in 1:length(chr)) {
                axis(side=1, at=placement[i], labels=chr[i])
                axis(side=2, at=placement[i], labels=chr[i])
            }
        }
        else {
            odd <- seq(1, length(chr), by=2)
            even <- seq(2, length(chr), by=2)
            for(i in odd) {
                axis(side=1, at=placement[i], labels="")
                axis(side=2, at=placement[i], labels="")
                axis(side=1, at=placement[i], labels=chr[i], line=-0.4, tick=FALSE)
                axis(side=2, at=placement[i], labels=chr[i], line=-0.4, tick=FALSE)
            }

            for(i in even) {
                axis(side=1, at=placement[i], labels="")
                axis(side=2, at=placement[i], labels="")
                axis(side=1, at=placement[i], labels=chr[i], line=+0.4, tick=FALSE)
                axis(side=2, at=placement[i], labels=chr[i], line=+0.4, tick=FALSE)
            }
        }
    }
    else {
        u <- par("usr")
        abline(v=u[1:2], h=u[3:4])
    }

    # add title
    if(!missing(main))
        title(main = main)

    if(zscale) {
        # plot the colormap
        dots <- list(...)
        if("mar2" %in% names(dots))
            par(mar=dots[["mar2"]])
        else
            par(mar = c(5, 2, 4, 2) + 0.1)

        colorstep <- (zlim.jnt-lo)/255
        image(x = 1:1, y = seq(lo, zlim.jnt, colorstep), z = matrix(1:256, 1, 256),
              zlim = c(1, 256), ylab = "", xlab = "",
              xaxt = "n", yaxt = "n", col = cols)

        # make sure there's a box around it
        u <- par("usr")
        abline(v = u[1:2], xpd = FALSE)
        abline(h = u[3:4], xpd = FALSE)
        if(any(contours) > 0) {
            for(i in seq(length(contours))) {
                segments(mean(u[1:2]),
                         max(lod[row(lod) > col(lod)]) - contours[i],
                         u[2], max(lod[row(lod) > col(lod)]) - contours[i],
                         xpd = FALSE, col = "blue", lwd = 2)
                segments(u[1], max(lod[row(lod) < col(lod)]) - contours[i] *
                         zlim.jnt/zlim.int, mean(u[1:2]),
                         max(lod[row(lod) < col(lod)]) - contours[i] * zlim.jnt / zlim.int,
                         xpd = FALSE, col = "blue", lwd = 2)
            }
        }

        # figure out how big the axis labels should be
        fin <- par("fin")[1] # figure width in inches
        pin <- par("pin")[1] # plot width in inches
        mai <- par("mai")[2] # margin width in inches
        # note: pin + 2*mai = fin
        xlen.mar <- mai/pin * diff(u[1:2])

        # axis for full LODs
        yloc <- pretty(c(lo, zlim.jnt), 4)
        yloc <- yloc[yloc >= u[3] & yloc <= u[4]]
        #    segments(u[2], yloc, u[2] + xlen.mar/4, yloc, xpd = TRUE)
        #    text(u[2] + xlen.mar/3, yloc, as.character(yloc), xpd = TRUE, adj = 0)
        axis(side=4, at=yloc, labels=yloc)

        # axis for int've LODs
        yloc <- pretty(c(lo.int, zlim.int), 4)
        yloc.rev <- yloc * zlim.jnt/zlim.int
        yloc <- yloc[yloc.rev >= u[3] & yloc.rev <= u[4]]
        yloc.rev <- yloc.rev[yloc.rev >= u[3] & yloc.rev <= u[4]]
        #    segments(u[1], yloc.rev, u[1] - xlen.mar/4, yloc.rev, xpd = TRUE)
        #    text(u[1] - xlen.mar/3, yloc.rev, as.character(yloc), xpd = TRUE, adj = 1)
        if(!addpair)
            axis(side=2, at=yloc.rev, labels=yloc)
    }

    invisible()
}


# end of plot.scantwo.R

######################################################################
#
# summary.scantwo.R
#
# copyright (c) 2001-2020, Karl W Broman, Hao Wu, and Brian Yandell
# last modified Feb, 2020
# first written Nov, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: summary.scantwo, print.summary.scantwo,
#           max.scantwo, clean.scantwo, print.scantwo, subset.scantwo
#           summary.scantwoperm, print.summary.scantwoperm
#           condense.scantwo, summary.scantwocondensed
#           max.scantwocondensed, print.summary.addpair
#           rbind.scantwoperm, c.scantwoperm, subset.scantwoperm
#           [.scantwoperm
#
######################################################################
#
# summarize the result from scantwo
#
######################################################################
summary.scantwo <-
    function(object, thresholds,
             what=c("best", "full", "add", "int"),
             perms, alphas, lodcolumn=1, pvalues=FALSE,
             allpairs=TRUE, ...)
{
    if(!inherits(object, "scantwo") &&
       !inherits(object, "scantwocondensed"))
        stop("Input should have class \"scantwo\".")

    addpair <- attr(object, "addpair")
    if(!is.null(addpair) && addpair) { # results from addpair() that need special treatment
        if("lod.minus1" %in% names(attributes(object))) { # asymmetric formula
            attr(object, "addpair") <- NULL
            x <- summary.scantwo(object, allpairs=allpairs)
            class(x) <- "data.frame"
            x <- x[,c(1,2,8,9,10,3,4,5),drop=FALSE]

            mlod.minus1 <- tapply(attr(object, "lod.minus1"), object$map$chr, max, na.rm=TRUE)
            mlod.minus2 <- tapply(attr(object, "lod.minus2"), object$map$chr, max, na.rm=TRUE)

            w <- which(x[,1] == x[,2])
            for(i in seq(along=w))
                if(x[w[i],5] < x[w[i],8]) x[w[i],3:5] <- x[w[i],c(7,6,8)]

            if(any(x[,1] != x[,2])) {
                w <- which(x[,1] != x[,2])
                y <- x[w,,drop=FALSE]
                for(i in 1:nrow(y))
                    y[,1:5] <- y[,c(2,1,7,6,8)]

                neworder <- cbind(1:nrow(x), rep(NA, nrow(x)))
                neworder[w,2] <- nrow(x) + 1:nrow(y)
                neworder <- as.numeric(t(neworder))

                x <- rbind(x, y)[neworder[!is.na(neworder)],,drop=FALSE]
            }

            x <- cbind(x[,1:5], lod.2v1b=rep(NA,nrow(x)), lod.2v1a=rep(NA,nrow(x)))
            names(x)[5] <- "lod.2v0"

            x[,6] <- x[,5] - mlod.minus1[as.character(x[,2])]
            x[,7] <- x[,5] - mlod.minus2[as.character(x[,1])]

            if(!missing(thresholds)) {
                if(length(thresholds) > 2)
                    warning("Only the first two values in thresholds are used.")

                if(length(thresholds) == 1) thresholds <- c(thresholds, 0)
                x <- x[!is.na(x[,5]) & x[,5] >= thresholds[1] &
                       ((!is.na(x[,6]) & x[,6] >= thresholds[2]) |
                        (!is.na(x[,7]) & x[,7] >= thresholds[2])),,drop=FALSE]
            }

            class(x) <- c("summary.addpair", "data.frame")
            return(x)
        }
        else { # symmetric formula
            attr(object, "addpair") <- NULL
            if(missing(thresholds))
                x <- summary.scantwo(object, allpairs=allpairs)
            else
                x <- summary.scantwo(object, thresholds=thresholds, allpairs=allpairs)

            x <- x[,1:6]
            colnames(x)[5:6] <- c("lod.2v0", "lod.2v1")

            if(!missing(thresholds)) {
                if(length(thresholds) > 2)
                    warning("Only the first two values in thresholds are used.")

                if(length(thresholds) == 1) thresholds <- c(thresholds, 0)
                x <- x[!is.na(x[,5]) & x[,5] >= thresholds[1] &
                       !is.na(x[,6]) & x[,6] >= thresholds[2],,drop=FALSE]
            }

            class(x) <- c("summary.addpair", "data.frame")
            return(x)
        }
    }

    what <- match.arg(what)

    if(!missing(thresholds)) {
        if(length(thresholds) != 5)
            stop("If thresholds are given, there must be 5 of them.")
        else if(!is.numeric(thresholds))
            stop("thresholds should be a numeric vector")
    }

    if(!missing(alphas)) {
        if(length(alphas) != 5) {
            if(length(alphas)==1)
                alphas <- rep(alphas, 5)
            else
                stop("If alphas are given, there must be 5 of them.")
        }
        else if(!is.numeric(alphas))
            stop("alphas should be a numeric vector")
    }

    if(!missing(perms) && !inherits(perms, "scantwoperm"))
        stop("perms must be in scantwoperm format.")

    # subset object and permutations, if necessary
    if(inherits(object, "scantwo")) {
        d <- dim(object$lod)
        if(length(d)==3) {
            if(!missing(perms)) {
                if("AA" %in% names(perms)) { # contains X-specific results
                    ncp <- sapply(perms$AA, ncol)
                }
                else {
                    ncp <- sapply(perms, ncol)
                }

                if(all(ncp==1)) onepermcol <- TRUE
                else onepermcol <- FALSE

                if(any(ncp != d[3])) {
                    if(onepermcol)  {
                        if(lodcolumn > 1)
                            warning("Just one column of permutation results; assuming they apply to all LOD score columns.")
                    }
                    else
                        stop("perms have different numbers of columns as object input.\n")
                }
            }

            if(lodcolumn < 1 || lodcolumn > d[3])
                stop("lodcolumn must be between 1 and ", d[3])

            object$lod <- object$lod[,,lodcolumn]
            if(!missing(perms) && !onepermcol) {
                if("AA" %in% names(perms)) {
                    for(i in seq(along=perms))
                        perms[[i]] <- lapply(perms[[i]], function(a, b) a[,b,drop=FALSE], lodcolumn)
                }
                else {
                    perms <- lapply(perms, function(a, b) a[,b,drop=FALSE], lodcolumn)
                }
            }
        }
    }
    else { # condensed version
        if(is.matrix(object$pos1.jnt)) {
            d <- ncol(object$pos1.jnt)
            if(!missing(perms)) {
                ncp <- sapply(perms, ncol)
                if(all(ncp==1)) onepermcol <- TRUE
                else onepermcol <- FALSE
                if(any(ncp != d[3])) {
                    if(onepermcol)
                        warning("Just one column of permutation results; reusing for all LOD score columns.")
                    else
                        stop("perms have different numbers of columns as object input.\n")
                }
            }

            if(lodcolumn < 1 || lodcolumn > d)
                stop("lodcolumn must be between 1 and ", d)

            for(i in 3:length(object))
                object[[i]] <- object[[i]][,lodcolumn]

            if(!missing(perms) && !onepermcol)
                perms <- lapply(perms, function(a, b) a[,b,drop=FALSE], lodcolumn)
        }
    }

    # check input
    if(missing(perms) && !missing(alphas))
        stop("If alphas are to be used, permutation results must be provided.")
    if(!missing(thresholds) && !missing(alphas))
        stop("Only one of threshold and alpha should be specified.")
    if(pvalues && what != "best") {
        pvalues <- FALSE
        warning("pvalues shown only with what=\"best\".")
    }
    if(pvalues && missing(perms)) {
        pvalues <- FALSE
        warning("p-values may be calculated only if perms are provided.")
    }

    if(inherits(object, "scantwo"))
        out <- subrousummaryscantwo(object, for.perm=FALSE)
    else
        out <- as.data.frame(object, stringsAsFactors=TRUE)

    if(!allpairs) # only look at self-self cases
        out <- out[out$chr1==out$chr2,]

    if(what=="best") {
        p1.f <- out$pos1.jnt
        p2.f <- out$pos2.jnt
        p1.a <- out$pos1.add
        p2.a <- out$pos2.add

        lf <- out$jnt.lod.full
        li <- out$jnt.lod.full - out$add.lod.add
        lfv1 <- out$jnt.lod.full - out$lod.1qtl
        la <- out$add.lod.add
        lav1 <- out$add.lod.add - out$lod.1qtl
    }
    else if(what=="full") {
        p1.f <- p1.a <- out$pos1.jnt
        p2.f <- p2.a <- out$pos2.jnt

        lf <- out$jnt.lod.full
        li <- out$jnt.lod.full - out$jnt.lod.add
        lfv1 <- out$jnt.lod.full - out$lod.1qtl
        la <- out$jnt.lod.add
        lav1 <- out$jnt.lod.add - out$lod.1qtl
    }
    else if(what=="add") {
        p1.f <- p1.a <- out$pos1.add
        p2.f <- p2.a <- out$pos2.add

        lf <- out$add.lod.full
        li <- out$add.lod.full - out$add.lod.add
        lfv1 <- out$add.lod.full - out$lod.1qtl
        la <- out$add.lod.add
        lav1 <- out$add.lod.add - out$lod.1qtl
    }
    else { # what == "int"
        p1.f <- p1.a <- out$pos1.int
        p2.f <- p2.a <- out$pos2.int

        lf <- out$int.lod.full
        li <- out$int.lod.full - out$int.lod.add
        lfv1 <- out$int.lod.full - out$lod.1qtl
        la <- out$int.lod.add
        lav1 <- out$int.lod.add - out$lod.1qtl
    }

    out <- data.frame(chr1=out$chr1, chr2=out$chr2,
                      pos1f=p1.f,
                      pos2f=p2.f,
                      lod.full=lf, lod.fv1=lfv1, lod.int=li,
                      pos1a=p1.a,
                      pos2a=p2.a,
                      lod.add=la, lod.av1=lav1, stringsAsFactors=TRUE)

    if(what != "best") {
        out <- out[,-(8:9)]
        names(out)[3:4] <- c("pos1", "pos2")
    }

    if(!missing(perms) && "AA" %in% names(perms)) {
        xchr_specific <- TRUE
        chr_type <- attr(perms, "chrtype")
        chrpair_type <- paste0(chr_type[out$chr1], chr_type[out$chr2])
        if(all(chrpair_type=="AA")) {
            perms <- perms$AA
            xchr_specific <- FALSE
        }
        else if(all(chrpair_type=="XX")) {
            perms <- perms$XX
            xchr_specific <- FALSE
        }
        else if(all(chrpair_type=="AX")) {
            perms <- perms$AX
            xchr_specific <- FALSE
        }
    }
    else xchr_specific <- FALSE

    if(!missing(alphas)) { # get thresholds
        if(!xchr_specific) {
            thresholds <- rep(0,5)
            for(i in 1:5)
                thresholds[i] <- quantile(perms[[i]], 1-alphas[i])
            thresholds[alphas==1] <- 0
            thresholds[alphas==0] <- Inf
        }
        else { # x-chr-specific
            thresholds <- matrix(nrow=3, ncol=5)
            for(j in 1:3) {
                for(i in 1:5)
                    thresholds[j,i] <- quantile(perms[[j]][[i]], 1-alphas[i])
            }
            thresholds[alphas==1] <- 0
            thresholds[alphas==0] <- Inf
        }
    }

    if(!missing(thresholds) || !missing(alphas)) { # apply thresholds
        if(!xchr_specific) {
            out <- out[(out$lod.full >= thresholds[1] & (out$lod.fv1 >= thresholds[2] |
                        out$lod.int >= thresholds[3])) |
                       (out$lod.add >= thresholds[4] & out$lod.av1 >= thresholds[5]),,drop=FALSE]
        }
        else {
            tokeep <- NULL
            for(i in 1:nrow(out)) {
                this_chrpair_type <- match(chrpair_type[i], c("AA", "AX", "XX"))
                if((out$lod.full[i] >= thresholds[this_chrpair_type, 1] &
                    (out$lod.fv1[i] >= thresholds[this_chrpair_type, 2] |
                     out$lod.int[i] >= thresholds[this_chrpair_type, 3])) |
                   (out$lod.add[i] >= thresholds[this_chrpair_type, 4] &
                    out$lod.av1[i] >= thresholds[this_chrpair_type, 5]))
                    tokeep <- c(tokeep, i)
            }
            out <- out[tokeep, , drop=FALSE]
        }
    }

    if(pvalues && nrow(out) > 0) {
        result <- as.data.frame(matrix(ncol=11+5, nrow=nrow(out)), stringsAsFactors=TRUE)
        wh <- c(1,2,3,4,5,7,9,11,12,13,15)
        wh2 <- (1:16)[-wh]
        result[,wh] <- out
        names(result)[wh] <- names(out)
        colnames(result)[wh2] <- rep("pval",5)

        if(!xchr_specific) {
            for(i in 1:5) {
                for(j in 1:nrow(out))
                    result[j,wh2[i]] <- mean(perms[[i]] >= result[j,wh2[i]-1], na.rm=TRUE)
            }
        }
        else { # X-chr-specific
            L <- attr(perms, "L")
            sumL <- sum(L)
            for(j in 1:nrow(out)) {
                this_chrpair_type <- match(chrpair_type[j], c("AA", "AX", "XX"))
                pow <- sumL/L[this_chrpair_type]
                for(i in 1:5) {
                    nominal_p <- mean(perms[[this_chrpair_type]][[i]] >= result[j,wh2[i]-1], na.rm=TRUE)
                    result[j,wh2[i]] <- 1 - (1-nominal_p)^pow # adjusted P-value
                }
            }
        }
        out <- result
    }

    class(out) <- c("summary.scantwo", "data.frame")

    out
}




# subroutine for summary.scantwo; pulls out the key info
#     on each pair of chromosomes
subrousummaryscantwo <-
    function(object, for.perm=FALSE)
{
    lod <- object$lod
    lod[is.na(lod) | lod == Inf | lod == -Inf] <- 0
    map <- object$map

    pos <- map[,2]
    chr <- as.factor(map[,1])

    tchr <- as.numeric(chr)
    n.chr <- max(tchr)
    xchr <- tapply(map[,4], map[,1], function(a) a[1])
    xchr <- xchr[!is.na(xchr)]
    n.phe <- 1
    if(length(dim(lod)) == 3) n.phe <- dim(lod)[3]

    if(!("scanoneX" %in% names(object)) ||
       is.null(object$scanoneX) || length(object$scanoneX)==0) {
        if(n.phe==1) scanoneX <- diag(lod)
        else {
            if(nrow(lod)==1) scanoneX <- lod[1,1,1]
            else scanoneX <- diag(lod[,,1])
            for(i in 2:n.phe) {
                if(nrow(lod)==1) scanoneX <- cbind(scanoneX, lod[1,1,i])
                else scanoneX <- cbind(scanoneX, diag(lod[,,i]))
            }
        }
    }
    else scanoneX <- object$scanoneX

    if((is.matrix(scanoneX) && nrow(scanoneX) != nrow(lod)) ||
       (!is.matrix(scanoneX) && length(scanoneX) != nrow(lod)))
        stop("scanoneX component has length ", length(scanoneX), " but should have length ", nrow(lod))

    n.chrpair <- n.chr*(n.chr+1)/2

    fill <- matrix(0, nrow=n.chrpair, ncol=n.phe)

    out <- .C("R_summary_scantwo",
              as.integer(nrow(map)),
              as.integer(n.phe),
              as.double(lod),
              as.integer(n.chr),
              as.integer(tchr),
              as.double(pos),
              as.integer(xchr),
              as.double(scanoneX),
              as.integer(n.chrpair),
              chr1=as.integer(rep(0,n.chrpair)),
              chr2=as.integer(rep(0,n.chrpair)),
              as.integer(rep(0,n.chr*n.chr)),
              pos1.jnt=as.double(fill),
              pos2.jnt=as.double(fill),
              pos1.add=as.double(fill),
              pos2.add=as.double(fill),
              pos1.int=as.double(fill),
              pos2.int=as.double(fill),
              jnt.lod.full=as.double(fill),
              jnt.lod.add=as.double(fill),
              add.lod.full=as.double(fill),
              add.lod.add=as.double(fill),
              int.lod.full=as.double(fill),
              int.lod.add=as.double(fill),
              lod.1qtl=as.double(fill),
              PACKAGE="qtl")

    chr1 <- factor(levels(chr)[out$chr1+1], levels=levels(chr))
    chr2 <- factor(levels(chr)[out$chr2+1], levels=levels(chr))

    if(n.phe == 1) {
        out <- data.frame(chr1=chr1, chr2=chr2,
                          pos1.jnt=out$pos1.jnt,
                          pos2.jnt=out$pos2.jnt,
                          jnt.lod.full=out$jnt.lod.full,
                          jnt.lod.add=out$jnt.lod.add,
                          pos1.add=out$pos1.add,
                          pos2.add=out$pos2.add,
                          add.lod.full=out$add.lod.full,
                          add.lod.add=out$add.lod.add,
                          pos1.int=out$pos1.int,
                          pos2.int=out$pos2.int,
                          int.lod.full=out$int.lod.full,
                          int.lod.add=out$int.lod.add,
                          lod.1qtl=out$lod.1qtl,
                          stringsAsFactors=TRUE)
    }
    else
        out <- list(chr1=chr1, chr2=chr2,
                    pos1.jnt=matrix(out$pos1.jnt, ncol=n.phe),
                    pos2.jnt=matrix(out$pos2.jnt, ncol=n.phe),
                    jnt.lod.full=matrix(out$jnt.lod.full, ncol=n.phe),
                    jnt.lod.add=matrix(out$jnt.lod.add, ncol=n.phe),
                    pos1.add=matrix(out$pos1.add, ncol=n.phe),
                    pos2.add=matrix(out$pos2.add, ncol=n.phe),
                    add.lod.full=matrix(out$add.lod.full, ncol=n.phe),
                    add.lod.add=matrix(out$add.lod.add, ncol=n.phe),
                    pos1.int=matrix(out$pos1.int, ncol=n.phe),
                    pos2.int=matrix(out$pos2.int, ncol=n.phe),
                    int.lod.full=matrix(out$int.lod.full, ncol=n.phe),
                    int.lod.add=matrix(out$int.lod.add, ncol=n.phe),
                    lod.1qtl=matrix(out$lod.1qtl, ncol=n.phe))

    if(for.perm) {
        if(n.phe==1) {
            out <- c("full"=max(out$jnt.lod.full,na.rm=TRUE),
                     "fv1"=max(out$jnt.lod.full - out$lod.1qtl, na.rm=TRUE),
                     "int"=max(out$jnt.lod.full - out$add.lod.add, na.rm=TRUE),
                     "add"=max(out$add.lod.add, na.rm=TRUE),
                     "av1"=max(out$add.lod.add - out$lod.1qtl, na.rm=TRUE),
                     "one"=max(out$lod.1qtl, na.rm=TRUE))
        }
        else {
            out <- list("full"=apply(out$jnt.lod.full, 2, max, na.rm=TRUE),
                        "fv1"=apply(out$jnt.lod.full - out$lod.1qtl, 2, max, na.rm=TRUE),
                        "int"=apply(out$jnt.lod.full - out$add.lod.add, 2, max, na.rm=TRUE),
                        "add"=apply(out$add.lod.add, 2, max, na.rm=TRUE),
                        "av1"=apply(out$add.lod.add - out$lod.1qtl, 2, max, na.rm=TRUE),
                        "one"=apply(out$lod.1qtl, 2, max, na.rm=TRUE))
        }
    }

    out
}


print.summary.scantwo <-
    function(x, ...)
{
    if(nrow(x)==0) {
        cat("    There were no pairs of loci meeting the criteria.\n")
        return(invisible(NULL))
    }

    z <- as.character(unlist(x[,1]))

    if(max(nchar(z)) == 1)
        rownames(x) <- apply(x[,1:2], 1, function(a)
                             paste("c", a, collapse=":", sep=""))
    else
        rownames(x) <- apply(x[,1:2], 1, function(a)
                             paste(sprintf("c%-2s", a), collapse=":"))

    x <- x[,-(1:2)]

    cn <- colnames(x)
    if(any(cn=="pos1a"))
        cn[cn=="pos1a"] <- "    pos1a"

    wh <- grep("^pval", cn)
    if(length(wh) > 0)
        cn[wh] <- "pval"
    colnames(x) <- cn

    print.data.frame(x, digits=3)
}


print.summary.addpair <-
    function(x, ...)
{
    if(nrow(x)==0) {
        cat("    There were no pairs of loci meeting the criteria.\n")
        return(invisible(NULL))
    }

    z <- as.character(unlist(x[,1]))

    if(max(nchar(z)) == 1)
        rownames(x) <- apply(x[,1:2], 1, function(a)
                             paste("c", a, collapse=":", sep=""))
    else
        rownames(x) <- apply(x[,1:2], 1, function(a)
                             paste(sprintf("c%-2s", a), collapse=":"))

    x <- x[,-(1:2), drop=FALSE]

    print.data.frame(x, digits=3)
}



print.scantwo <-
    function(x, ...)
{
    d <- dim(x$lod)
    dn <- dimnames(x$lod)[[3]]
    if(is.null(dn)) dn <- attr(x, "phenotypes")


    if(nrow(x$lod) == 0)
        cat("Empty scantwo object.\n")
    else {

        if(length(d)==2)
            print(summary(x))
        else {
            for(i in 1:d[3]) {
                if(is.null(dn)) cat("Phenotype", i, "\n")
                else cat(dn[i], "\n")
                print(summary(x, lod=i))
            }
        }
    }
}


######################################################################
#
# max.scantwo:  Give pair of chromosome with maximum 2-locus LOD score
#
######################################################################

max.scantwo <-
    function(object, lodcolumn=1,
             what=c("best", "full", "add", "int"),
             na.rm=TRUE, ...)
{
    if(!inherits(object, "scantwo") &&
       !inherits(object, "scantwocondensed"))
        stop("Input must have class \"scantwo\".")

    addpair <- attr(object, "addpair")
    if(!is.null(addpair) && addpair) { # special treatment for output for addpair
        temp <- summary(object)
        mx <- max(temp[,5],na.rm=TRUE)
        return(temp[!is.na(temp[,5]) & temp[,5]==mx,])
    }

    what <- match.arg(what)
    if(inherits(object, "scantwo")) {
        d <- dim(object$lod)
        if(length(d)==3) {
            if(lodcolumn < 1 || lodcolumn > d[3])
                stop("lodcolumn must be between 1 and ", d[3])
            object$lod <- object$lod[,,lodcolumn]
        }

        out <- subrousummaryscantwo(object, for.perm=FALSE)
    }
    else { # condensed version
        if(is.matrix(object$pos1.jnt)) {
            d <- ncol(object$pos1.jnt)
            if(lodcolumn < 1 || lodcolumn > d)
                stop("lodcolumn must be between 1 and ", d)

            for(i in 3:length(object))
                object[[i]] <- object[[i]][,lodcolumn]
        }
        out <- as.data.frame(object, stringsAsFactors=TRUE)
    }

    if(what=="best") {
        wh <- which(!is.na(out$jnt.lod.full) & out$jnt.lod.full==max(out$jnt.lod.full, na.rm=TRUE))
        p1.f <- out$pos1.jnt[wh]
        p2.f <- out$pos2.jnt[wh]
        p1.a <- out$pos1.add[wh]
        p2.a <- out$pos2.add[wh]

        lf <- out$jnt.lod.full[wh]
        li <- out$jnt.lod.full[wh] - out$add.lod.add[wh]
        lfv1 <- out$jnt.lod.full[wh] - out$lod.1qtl[wh]
        la <- out$add.lod.add[wh]
        lav1 <- out$add.lod.add[wh] - out$lod.1qtl[wh]
    }
    else if(what=="full") {
        wh <- which(!is.na(out$jnt.lod.full) & out$jnt.lod.full==max(out$jnt.lod.full, na.rm=TRUE))
        p1.f <- p1.a <- out$pos1.jnt[wh]
        p2.f <- p2.a <- out$pos2.jnt[wh]

        lf <- out$jnt.lod.full[wh]
        li <- out$jnt.lod.full[wh] - out$jnt.lod.add[wh]
        lfv1 <- out$jnt.lod.full[wh] - out$lod.1qtl[wh]
        la <- out$jnt.lod.add[wh]
        lav1 <- out$jnt.lod.add[wh] - out$lod.1qtl[wh]
    }
    else if(what=="add") {
        wh <- which(!is.na(out$add.lod.add) & out$add.lod.add==max(out$add.lod.add, na.rm=TRUE))
        p1.f <- p1.a <- out$pos1.add[wh]
        p2.f <- p2.a <- out$pos2.add[wh]

        lf <- out$add.lod.full[wh]
        li <- out$add.lod.full[wh] - out$add.lod.add[wh]
        lfv1 <- out$add.lod.full[wh] - out$lod.1qtl[wh]
        la <- out$add.lod.add[wh]
        lav1 <- out$add.lod.add[wh] - out$lod.1qtl[wh]
    }
    else { # what == "int"
        lod.int <- out$int.lod.full - out$int.lod.add
        wh <- which(!is.na(lod.int) & lod.int == max(lod.int, na.rm=TRUE))
        p1.f <- p1.a <- out$pos1.int[wh]
        p2.f <- p2.a <- out$pos2.int[wh]

        lf <- out$int.lod.full[wh]
        li <- out$int.lod.full[wh] - out$int.lod.add[wh]
        lfv1 <- out$int.lod.full[wh] - out$lod.1qtl[wh]
        la <- out$int.lod.add[wh]
        lav1 <- out$int.lod.add[wh] - out$lod.1qtl[wh]
    }

    out <- data.frame(chr1=out$chr1[wh], chr2=out$chr2[wh],
                      pos1f=p1.f,
                      pos2f=p2.f,
                      lod.full=lf, lod.fv1=lfv1, lod.int=li,
                      pos1a=p1.a,
                      pos2a=p2.a,
                      lod.add=la, lod.av1=lav1, stringsAsFactors=TRUE)

    if(what != "best") {
        out <- out[,-(8:9)]
        names(out)[3:4] <- c("pos1", "pos2")
    }

    class(out) <- c("summary.scantwo", "data.frame")
    rownames(out) <- what

    out
}



######################################################################
# clean.scantwo
#
# sets LOD scores that are missing or < 0 to 0
# If full LOD < add've LOD, set full = add've
# sets LOD scores, for pairs of positions that are not separated
#     by n.mar markers and distance cM, to 0
######################################################################

clean.scantwo <-
    function(object, n.mar=1, distance=0, ...)
{
    if(!inherits(object, "scantwo"))
        stop("Input should have class \"scantwo\".")

    addpair <- attr(object, "addpair")
    if(is.null(addpair)) addpair <- FALSE

    lod <- object$lod
    map <- object$map
    if(!("fullmap" %in% names(attributes(object))))
        stop("clean.scantwo only works on scantwo objects created with R/qtl ver >= 1.04-38.\n")
    fmap <- attr(object, "fullmap")

    lod[is.na(lod) | lod < 0] <- 0

    subrou <- function(x,y,z)
    {
        out <- x
        for(i in seq(along=x))
            out[i] <- sum((z > x[i] & z < y[i]) | (z < x[i] & z > y[i])) +
                (x[i] == y[i])
        out
    }

    last <- 0
    for(i in seq(along=fmap)) {
        m <- map[map[,1]==names(fmap)[i],2]
        idx <- 1:length(m)+last
        last <- last + length(m)

        toclean <- (outer(m, m, subrou, fmap[[i]]) < n.mar) | (abs(outer(m, m, "-")) <distance)
        diag(toclean) <- FALSE

        if(length(dim(lod))==3) # case of multiple phenotypes
            lod[idx,idx,][toclean] <- 0
        else
            lod[idx,idx][toclean] <- 0
    }

    # if full LOD < add've LOD, set full = add've
    if(!addpair) {
        if(length(dim(lod)) == 2) {
            lo <- lower.tri(lod)
            wh <- (lod[lo] < t(lod)[lo])
            if(any(wh))
                lod[lo][wh] <- t(lod)[lo][wh]
        }
        else {
            lo <- lower.tri(lod[,,1])
            for(i in 1:dim(lod)[3]) {
                wh <- (lod[,,i][lo] < t(lod[,,i])[lo])
                if(any(wh))
                    lod[,,i][lo][wh] <- t(lod[,,i])[lo][wh]
            }
        }
    }

    object$lod <- lod

    object
}

######################################################################
#
# subset.scantwo
#
######################################################################
subset.scantwo <-
    function(x, chr, lodcolumn, ...)
{
    if(!inherits(x, "scantwo"))
        stop("Input should have class \"scantwo\".")

    if((missing(chr) || length(chr)==0) && missing(lodcolumn)) return(x)

    if(!missing(lodcolumn)) {
        if(any(lodcolumn>0) && any(lodcolumn<0))
            stop("lodcolumn values can't be both >0 and <0.")

        if(any(lodcolumn<0) || is.logical(lodcolumn))
            lodcolumn <- (1:(dim(x$lod)[3]))[lodcolumn]
        if(length(lodcolumn)==0)
            stop("You must retain at least one LOD column.")
        if(any(lodcolumn < 1 || lodcolumn > dim(x$lod)[3]))
            stop("lodcolumn values must be >=1 and <=",dim(x$lod)[3])

        x$lod <- x$lod[,,lodcolumn]
        if("scanoneX" %in% names(x))
            x$scanoneX <- x$scanoneX[,lodcolumn]
    }

    if(!missing(chr)) {
        chr <- matchchr(chr, unique(x$map[,1]))

        wh <- x$map[,1] %in% chr

        x$map <- x$map[wh,,drop=FALSE]
        x$map[,1] <- droplevels(x$map[,1])

        if(length(dim(x$lod))==2)
            x$lod <- x$lod[wh,wh,drop=FALSE]
        else
            x$lod <- x$lod[wh,wh,,drop=FALSE]

        if(!is.null(x$scanoneX))
            x$scanoneX <- x$scanoneX[wh]

        if("fullmap" %in% names(attributes(x))) {
            fmap <- attr(x, "fullmap")
            fmap <- fmap[chr]
            attr(x, "fullmap") <- fmap
        }

    }

    x
}

######################################################################
# summary.scantwoperm
#
# Give genome-wide LOD thresholds on the basis of results of
# scantwo permutation test (from scantwo with n.perm > 0)
######################################################################
summary.scantwoperm <-
    function(object, alpha=c(0.05, 0.10), ...)
{
    if(!inherits(object, "scantwoperm"))
        stop("Input should have class \"scantwoperm\".")

    if("AA" %in% names(object)) { # X-chr-specific version
        # get region-specific (nominal) signif levels
        L <- attr(object, "L")
        LL <- attr(object, "LL")
        if(is.null(LL)) stop("LL attribute not found in input object")
        if(length(alpha)==1) {
            tmp <- L/sum(L); tmp <- c(tmp[1], 1, tmp[2])
            one_minus_alpha_onechr <- cbind((1-alpha)^tmp)
            one_minus_alpha <- cbind((1-alpha)^(LL/sum(LL)))
        }
        else {
            tmp <- L/sum(L); tmp <- c(tmp[1], 1, tmp[2])
            one_minus_alpha_onechr <- vapply(alpha, function(a,b) (1-a)^b, rep(0, length(tmp)), tmp)
            one_minus_alpha <- vapply(alpha, function(a,b) (1-a)^b, rep(0, length(LL)), LL/sum(LL))
        }
        # get quantiles
        out <- vector("list", length(object))
        names(out) <- names(object)
        for(i in seq(along=out)) {
            f <- function(a, qu) {
                b <- apply(a, 2, quantile, qu)
                if(!is.matrix(b)) {
                    nam <- names(b)
                    b <- matrix(b, nrow=1)
                    colnames(b) <- nam
                }
                rownames(b) <- paste0(alpha*100, "%")
                b
            }
            out[[i]] <- lapply(unclass(object[[i]])[1:5], f, one_minus_alpha[i,,drop=FALSE])
            qu_one <- f(object[[i]][[6]], one_minus_alpha_onechr[i,,drop=FALSE])
            out[[i]] <- c(out[[i]], "one"=list(qu_one))
        }

        attr(out, "n.perm") <- vapply(object, function(a) nrow(a[[1]]), 0)
        class(out) <- c("summary.scantwoperm", "list")

        return(out)
    }

    out <- lapply(object, apply, 2, quantile, 1-alpha)

    for(i in 1:length(out)) {
        if(!is.matrix(out[[i]]))
            out[[i]] <- matrix(out[[i]], nrow=length(alpha))
        rownames(out[[i]]) <- paste0(alpha*100, "%")
    }

    attr(out, "n.perm") <- nrow(object[[1]])
    class(out) <- c("summary.scantwoperm", "list")
    out
}

print.summary.scantwoperm <-
    function(x, ...)
{
    if("AA" %in% names(x)) { # X-sp perms
        nperm <- attr(x, "n.perm")
        rn <- rownames(x[[1]][[1]])
        nc <- rapply(x, ncol)
        if(length(unique(nc)) != 1)
            stop("The components shouldn't have varying numbers of columns.\n")
        nc <- nc[1]
        phe <- colnames(x[[1]][[1]])

        convert2matrix <-
            function(a, which_col=1) {
                a <- lapply(a, "[", , which_col, drop=FALSE)
                b <- matrix(unlist(a), ncol=6)
                rownames(b) <- rn
                colnames(b) <- names(a)
                b
            }

        for(i in seq(along=phe)) {
            cat(phe[i], ":\n", sep="")
            y <- lapply(x, convert2matrix, i)
            lab <- c(AA="A:A", AX="A:X", XX="X:X")
            for(j in c("AA", "AX", "XX")) {
                cat("  ", lab[j], " (", nperm[j], " permutations)\n", sep="")
                print(y[[j]], digits=3)
                cat("\n")
            }
            if(i != length(phe)) cat("\n")
        }
        invisible(return(x))
    }

    nam <- names(x)
    rn <- rownames(x[[1]])
    n.perm <- attr(x, "n.perm")

    nc <- sapply(x, ncol)
    if(length(unique(nc)) != 1)
        stop("The components shouldn't have varying numbers of columns.\n")
    nc <- nc[1]

    if(nc==1) {
        phe <- colnames(x[[1]])
        if(is.null(phe)) phe <- ""

        x <- matrix(unlist(x), nrow=length(rn))
        dimnames(x) <- list(rn, nam)

        if(is.null(phe))
            cat("(", n.perm, " permutations)\n", sep="")
        else
            cat(phe, " (", n.perm, " permutations)\n", sep="")
        print(x, digits=3)
    }
    else {
        phe <- colnames(x[[1]])
        if(is.null(phe)) phe <- paste("pheno", 1:nc)

        for(i in 1:nc) {
            y <- matrix(ncol=length(x), nrow=nrow(x[[1]]))
            for(j in 1:length(x))
                y[,j] <- x[[j]][,i]
            dimnames(y) <- list(rn, nam)

            cat(phe[i], " (", n.perm, " permutations)\n", sep="")
            print(y, digits=3)
            if(i != nc) cat("\n")
        }
    }
}

######################################################################
# combine scantwo results ... paste the phenotype columns together
######################################################################
cbind.scantwo <- c.scantwo <-
    function(...)
{
    dots <- list(...)

    if(length(dots)==1 && is.list(dots[[1]])) dots <- dots[[1]]

    if(length(dots)==1) return(dots[[1]])
    for(i in seq(along=dots)) {
        if(!inherits(dots[[i]], "scantwo"))
            stop("Input should have class \"scantwo\".")
    }

    # check dimensions of LODs
    nr <- sapply(dots, function(a) nrow(a$lod))
    nc <- sapply(dots, function(a) ncol(a$lod))
    nd3 <- sapply(dots, function(a) dim(a$lod)[3])
    if(any(nr[-1]!=nr[1]) || any(nc[-1] != nc[1]))
        stop("Input objects are not the same dimensions.")

    # check maps
    map1 <- dots$map[[1]]
    for(i in 2:length(dots)) {
        if(!all(dots$map[[i]] != map1))
            stop("Maps are not all the same.")
    }

    output <- dots[[1]]
    nd3[is.na(nd3)] <- 1
    output$lod <- array(dim=c(nr[1], nc[1], sum(nd3)))
    start <- cumsum(c(0,nd3))[-length(nd3)-1]
    end <- start+nd3

    for(i in seq(along=dots))
        output$lod[,,(start[i]+1):end[i]] <- dots[[i]]$lod
    attr(output, "phenotypes") <- unlist(lapply(dots, attr, "phenotypes"))
    dimnames(output$lod) <- list(NULL, NULL, attr(output, "phenotypes"))

    # check scanoneX
    if(is.null(dots[[1]]$scanoneX)) {
        for(i in 2:length(dots)) {
            if(!is.null(dots[[i]]$scanoneX))
                stop("Some but not all input objects have null scanoneX.")
        }
    }
    else {
        nrx <- sapply(dots, function(a) nrow(a$scanoneX))
        if(any(nrx[-1]!=nrx[1]))
            stop("Mismatch in scanoneX dimensions.")
        for(i in 2:length(dots))
            output$scanoneX <- cbind(output$scanoneX, dots[[i]]$scanoneX)
        colnames(output$scanoneX) <- attr(output, "phenotypes")
    }

    methods <- sapply(dots, attr, "method")
    if(length(unique(methods)) != 1)
        attr(output, "method") <- rep(sapply(dots, attr, "method"), nd3)

    fm <- lapply(dots, attr, "fullmap")
    for(i in 2:length(fm)) {
        if(length(fm[[1]]) != length(fm[[i]]) || !all(names(fm[[1]]) == names(fm[[i]])))
            stop("Mismatch in \"fullmap\" attributes (1).")
        for(j in 1:length(fm[[1]])) {
            if(length(fm[[1]][[j]]) != length(fm[[i]][[j]]) ||
               !all(names(fm[[1]][[j]]) == names(fm[[i]][[j]])) ||
               max(abs(fm[[1]][[j]] - fm[[i]][[j]])) > 0.001)
                stop("Mismatch in \"fullmap\" attributes (2).")
        }
    }

    output
}


######################################################################
# combine scantwoperm results ... paste the rows together
######################################################################
rbind.scantwoperm <- c.scantwoperm <-
    function(...)
{
    dots <- list(...)

    if(length(dots)==1 && is.list(dots[[1]])) dots <- dots[[1]]
    if(length(dots)==1) return(dots[[1]])

    xchrsp <- vapply(dots, function(a) "AA" %in% names(a), TRUE)
    if(any(xchrsp)) {
        if(!all(xchrsp)) stop("Some but not all inputs are X-chr specific")

        for(i in 2:length(dots)) {
            for(j in seq(along=dots[[1]])) {
                for(k in seq(along=dots[[1]][[j]])) {
                    if(ncol(dots[[1]][[j]][[k]]) != ncol(dots[[i]][[j]][[k]]))
                        stop("Mismatch in no. columns")
                    if(any(colnames(dots[[1]][[j]][[k]]) != colnames(dots[[1]][[j]][[k]])))
                        warning("Mismatch in column names")
                    dots[[1]][[j]][[k]] <- rbind(dots[[1]][[j]][[k]], dots[[i]][[j]][[k]])
                }
            }
        }
        return(dots[[1]])
    }

    for(i in seq(along=dots)) {
        if(!inherits(dots[[i]], "scantwoperm"))
            stop("Input should have class \"scantwoperm\".")
    }

    nc <- sapply(dots, function(a) ncol(a[[1]]))
    if(length(unique(nc)) != 1)
        stop("Number of LOD columns in the input objects must be constant.\n")

    flag <- 0
    for(j in 1:length(dots[[1]])) {
        for(i in 2:length(dots)) {
            dots[[1]][[j]] <- rbind(dots[[1]][[j]], dots[[i]][[j]])
            if(any(colnames(dots[[i]][[j]]) != colnames(dots[[1]][[j]]))) flag <- 1
        }
    }
    if(flag) warning("Mismatch in column names; input may not be consistent.\n")

    dots[[1]]
}

# paste columns together
cbind.scantwoperm <-
function(...)
{
    dots <- list(...)
    if(length(dots)==1 && is.list(dots[[1]])) dots <- dots[[1]]

    if(length(dots)==1) return(dots)

    xchrsp <- vapply(dots, function(a) "AA" %in% names(a), TRUE)
    if(any(xchrsp)) {
        if(!all(xchrsp)) stop("Some but not all inputs are X-chr specific")

        for(i in 2:length(dots)) {
            for(j in seq(along=dots[[1]])) {
                for(k in seq(along=dots[[1]][[j]])) {
                    if(nrow(dots[[1]][[j]][[k]]) != nrow(dots[[i]][[j]][[k]]))
                        stop("Mismatch in no. permutations")
                    dots[[1]][[j]][[k]] <- cbind(dots[[1]][[j]][[k]], dots[[i]][[j]][[k]])
                }
            }
        }
        return(dots[[1]])
    }

    for(i in 2:length(dots)) {
        for(j in seq(along=dots[[1]])) {
            if(nrow(dots[[1]][[j]]) != nrow(dots[[i]][[j]]))
                stop("Mismatch in no. permutations")
            dots[[1]][[j]] <- cbind(dots[[1]][[j]], dots[[i]][[j]])
        }
    }

    dots[[1]]
}



######################################################################
# condensed scantwo output

condense <-
    function(object)
    UseMethod("condense")

condense.scantwo <-
    function(object)
{
    out <- subrousummaryscantwo(object, for.perm=FALSE)
    class(out) <- c("scantwocondensed", "list")
    out
}

summary.scantwocondensed <- summary.scantwo
max.scantwocondensed <- max.scantwo


##############################
# subset.scantwoperm: pull out a set of lodcolumns
##############################
subset.scantwoperm <-
    function(x, repl, lodcolumn, ...)
{
    if("AA" %in% names(x)) { # x chr specific
        for(j in seq(along=x)) {
            if(missing(lodcolumn)) lodcolumn <- 1:ncol(x[[j]][[1]])
            else if(!check_colindex(lodcolumn, x[[j]][[1]]))
                stop("lodcolumn misspecified.")

            repl <- 1:nrow(x[[j]][[1]])

            x[[j]] <- lapply(x[[j]], function(a,b,d) unclass(a)[b,d,drop=FALSE], repl, lodcolumn)
        }
        return(x)
    }

    att <- attributes(x)

    if(any(!sapply(x, is.matrix)))
        x <- lapply(x, as.matrix)

    if(missing(lodcolumn)) lodcolumn <- 1:ncol(x[[1]])
    else if(!check_colindex(lodcolumn, x[[1]]))
        stop("lodcolumn misspecified.")

    if(missing(repl)) repl <- 1:nrow(x[[1]])
    else if(!check_rowindex(repl, x[[1]]))
        stop("repl misspecified.")

    cl <- class(x)
    x <- lapply(x, function(a,b,d) unclass(a)[b,d,drop=FALSE], repl, lodcolumn)
    class(x) <- cl

    for(i in seq(along=att)) {
        if(names(att)[i] == "dim" || length(grep("names", names(att)[i]))>0) next
        attr(x, names(att)[i]) <- att[[i]]
    }

    x
}

# subset.scantwoperm using [,]
`[.scantwoperm` <-
    function(x, repl, lodcolumn)
    subset.scantwoperm(x, repl, lodcolumn)

# end of summary.scantwo.R

######################################################################
#
# map_construction.R
#
# copyright (c) 2008-2019, Karl W Broman
# last modified Dec, 2019
# first written Oct, 2008
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: formLinkageGroups, orderMarkers, orderMarkers.sub
#
######################################################################

######################################################################
# formLinkageGroups
#
# Use the estimated recombination fractions between pairs of markers
# (and LOD scores for a test of rf = 1/2) to partition the markers
# into a set of linkage groups.
#
# Two markers are placed in the same linkage group if rf <= max.rf
# and LOD >= min.lod.  The transitive property (if A is linked to B
# and B is linked to C then A is linked to C) is used to close the
# groups.
#
# If reorgMarkers=FALSE, the output is a data frame with two columns:
# the initial chromosome assignments and the linkage group assigments
# determined from the pairwise recombination fractions.
#
# If reorgMarkers=TRUE, the output is an experimental cross object,
# with the data reorganized according to the inferred linkage groups.
#
# The linkage groups are sorted by the number of markers they contain
# (from largest to smallest).
#
# If verbose=TRUE, tracing information is printed.
######################################################################
formLinkageGroups <-
    function(cross, max.rf=0.25, min.lod=3, reorgMarkers=FALSE,
             verbose=FALSE)
{
    if(!("rf" %in% names(cross))) {
        warning("Running est.rf.")
        cross <- est.rf(cross)
    }
    n.mar <- nmar(cross)
    tot.mar <- totmar(cross)
    rf <- cross$rf
    diagrf <- diag(rf)
    if(ncol(rf) != tot.mar)
        stop("dimension of recombination fractions inconsistent with no. markers in cross.")

    onlylod <- attr(cross$rf, "onlylod")
    if(!is.null(onlylod) && onlylod) { # results of markerlrt()
        if(!missing(max.rf))
            warning("max.rf ignored, as markerlrt() was used.")
        max.rf <- Inf
    }

    marnam <- colnames(rf)
    chrstart <- rep(names(cross$geno), n.mar)

    lod <- rf
    lod[lower.tri(rf)] <- t(rf)[lower.tri(rf)]
    rf[upper.tri(rf)] <- t(rf)[upper.tri(rf)]
    diag(rf) <- 1
    diag(lod) <- 0

    ingrp <- 1:tot.mar
    for(i in 1:tot.mar) {
        if(verbose) {
            if(tot.mar > 100)
                if(i==round(i,-2)) cat(i,"of", tot.mar, "\n")
                else
                    if(i==round(i,-1)) cat(i,"of", tot.mar, "\n")
        }
        wh <- (rf[,i]<=max.rf & lod[,i] > min.lod)

        if(any(wh) && length(unique(ingrp[c(i, which(wh))]))>1) {
            oldgrp <- ingrp[wh]
            ingrp[wh] <- ingrp[i]
            u <- unique(oldgrp[oldgrp != ingrp[i]])
            ingrp[!is.na(match(ingrp, u))] <- ingrp[i]
        }
    }
    tab <- sort(table(ingrp), decreasing=TRUE)
    u <- names(tab)

    revgrp <- ingrp
    for(i in seq(along=u))
        revgrp[ingrp==u[i]] <- i

    if(reorgMarkers) {
        cross <- clean(cross)
        chr_type <- rep(sapply(cross$geno, chrtype), n.mar)
        crosstype <- crosstype(cross)
        g <- pull.geno(cross)

        cross$geno <- vector("list", max(revgrp))
        names(cross$geno) <- 1:max(revgrp)

        for(i in 1:max(revgrp)) {
            cross$geno[[i]]$data <- g[,revgrp==i,drop=FALSE]

            cross$geno[[i]]$map <- seq(0, by=10, length=tab[i])
            if(crosstype=="4way") {
                cross$geno[[i]]$map <- rbind(cross$geno[[i]]$map,
                                             cross$geno[[i]]$map)
                colnames(cross$geno[[i]]$map) <- colnames(cross$geno[[i]]$data)
            }
            else
                names(cross$geno[[i]]$map) <- colnames(cross$geno[[i]]$data)

            thechrtype <- unique(chr_type[revgrp==i])
            if(length(thechrtype) > 1)
                warning("Problem with linkage group ", i, ": A or X?\n", paste(thechrtype, collapse=" "))
            else
                class(cross$geno[[i]]) <- thechrtype

        }

        mname <- markernames(cross)
        m <- match(mname, marnam)
        rf <- rf[m,m]
        lod <- lod[m,m]
        rf[upper.tri(rf)] <- lod[upper.tri(lod)]
        diag(rf) <- diagrf[m]
        cross$rf <- rf

        return(cross)
    }
    else {
        result <- data.frame(origchr=factor(chrstart, levels=names(cross$geno)),
                             LG=factor(revgrp, levels=1:max(revgrp)), stringsAsFactors=TRUE)
        rownames(result) <- marnam
        return(result)
    }
}

######################################################################
# orderMarkers
#
# For each of the selected chromosomes, construct a new genetic map
# from scratch.  Marker order is determined in by an expedient and not
# necessarily good algorithm, with orders compared by the number of
# obligate crossovers.
######################################################################
orderMarkers <-
    function(cross, chr, window=7, use.ripple=TRUE, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             maxit=4000, tol=1e-4, sex.sp=TRUE, verbose=FALSE)
{
    map.function <- match.arg(map.function)
    if(!missing(chr)) chr <- matchchr(chr, names(cross$geno))
    else chr <- names(cross$geno)

    n.mar <- nmar(cross)

    if(verbose > 1) verbose.sub <- TRUE else verbose.sub <- FALSE

    for(i in chr) {
        if(verbose && length(chr) > 1) cat(" - Chr", i,"\n")
        if(n.mar[i] > 2) {
            neworder <- orderMarkers.sub(cross, i, window=window, use.ripple=use.ripple,
                                         verbose=verbose.sub)
            cross <- switch.order(cross, i, neworder, error.prob=error.prob,
                                  map.function=map.function, maxit=maxit, tol=tol,
                                  sex.sp=sex.sp)
        }
    }

    cross
}

######################################################################
# orderMarkers.sub
# For the markers on a chromosome, use a greedy algorithm to order
# the markers de novo, possibly running ripple() to refine the order.
######################################################################
orderMarkers.sub <-
    function(cross, chr, window=7, use.ripple=TRUE, verbose=FALSE)
{
    if(missing(chr)) chr <- names(cross$geno)[1]
    if(length(chr) > 1) {
        if(length(grep("^-", chr)) > 0)
            stop("Need to give a single chromosome name.")
        warning("Need to give a single chromosome name; using just the first")
        chr <- chr[1]
    }

    if(length(matchchr(chr, names(cross$geno)))>1)
        stop("Chr ", chr, " not found.")

    cross <- subset(cross, chr=chr)
    names(cross$geno)[1] <- "1"

    n.mar <- totmar(cross)
    if(n.mar < 3) return(1:n.mar)

    if(use.ripple && n.mar <= window) { # just use ripple
        rip <- summary(ripple(cross, chr=1, window=window, verbose=FALSE))

        nxo <- rip[1:2,ncol(rip)]
        if(nxo[1] <= nxo[2]) return(1:n.mar)
        else return(rip[2,1:(ncol(rip)-1)])
    }

    nt <- ntyped(cross, "mar")

    # start with the most typed markers and move down
    themar <- order(nt, decreasing=TRUE)
    marnam <- markernames(cross)
    if(n.mar > 3) {
        for(i in 4:n.mar)
            cross <- movemarker(cross, marnam[themar[i]], 2)
    }

    # create matrix of orders to test
    makeorders <-
        function(n)
        {
            orders <- matrix(ncol=n, nrow=n)
            for(k in 1:n) { orders[k,n-k+1] <- n; orders[k,-(n-k+1)] <- 1:(n-1) }
            orders
        }

    # simple switch of marker order on chr 1
    simpleswitch <-
        function(cross, neworder)
        {
            cross$geno[[1]]$data <- cross$geno[[1]]$data[,neworder]
            if(is.matrix(cross$geno[[1]]$map))
                cross$geno[[1]]$map <- cross$geno[[1]]$map[,neworder]
            else
                cross$geno[[1]]$map <- cross$geno[[1]]$map[neworder]
            cross
        }

    # work on marker 3
    if(verbose) cat(" --- Adding marker 3 of", n.mar, "\n")
    orders <- makeorders(3)
    nxo <- rep(NA, nrow(orders))
    nxo[1] <- sum(countXO(cross, 1))
    temp <- cross
    for(kk in 2:nrow(orders)) {
        temp$geno[[1]]$data <- temp$geno[[1]]$data[,orders[kk,]]
        nxo[kk] <- sum(countXO(cross, 1))
    }
    wh <- which(nxo==min(nxo))
    if(length(wh) > 1) wh <- sample(wh, 1)
    if(wh > 1)
        cross <- simpleswitch(cross, orders[wh,])

    # rest of the markers
    if(n.mar > 3) {
        for(k in 4:n.mar) {
            if(verbose) cat(" --- Adding marker", k, "of", n.mar, "\n")
            cross <- movemarker(cross, marnam[themar[k]], 1)
            orders <- makeorders(k)
            nxo <- rep(NA, nrow(orders))
            nxo[1] <- sum(countXO(cross, 1))
            temp <- cross
            for(kk in 2:nrow(orders)) {
                temp$geno[[1]]$data <- cross$geno[[1]]$data[,orders[kk,]]
                nxo[kk] <- sum(countXO(temp, 1))
            }

            wh <- which(nxo==min(nxo))
            if(length(wh) > 1) wh <- sample(wh, 1)
            if(wh > 1)
                cross <- simpleswitch(cross, orders[wh,])
        }
    }

    if(use.ripple) {
        dif <- -8
        while(dif < 0) {
            rip <- summary(ripple(cross, chr=1, window=window, verbose=FALSE))
            dif <- diff(rip[1:2, ncol(rip)])
            if(dif < 0)
                cross <- simpleswitch(cross, rip[2,1:(ncol(rip)-1)])
        }
    }

    match(colnames(cross$geno[[1]]$data), marnam)
}


# end of map_construction.R

######################################################################
#
# read.cross.csvs.R
#
# copyright (c) 2005-2020, Karl W Broman
# last modified Feb, 2020
# first written Oct, 2005
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.csvs
#           [See read.cross.R for the main read.cross function.]
#
######################################################################

######################################################################
#
# read.cross.csvs
#
# read data in comma-delimited format, with separate files for phenotype
# and genotype data
#
######################################################################

read.cross.csvs <-
    function(dir, genfile, phefile, na.strings=c("-","NA"),
             genotypes=c("A","H","B","D","C"),
             estimate.map=TRUE, rotate=FALSE, ...)
{
    # create file names
    if(missing(genfile)) genfile <- "gen.csv"
    if(missing(phefile)) phefile <- "phe.csv"

    if(!missing(dir) && dir != "") {
        genfile <- file.path(dir, genfile)
        phefile <- file.path(dir, phefile)
    }

    args <- list(...)

    if("" %in% na.strings) {
        na.strings <- na.strings[na.strings != ""]
        warning("Including \"\" in na.strings will cause problems; omitted.")
    }

    # if user wants to use comma for decimal point, we need
    if(length(args) > 0 && "dec" %in% names(args)) {
        dec <- args[["dec"]]
    }
    else dec <- "."

    # read the data file
    if(length(args) < 1 || !("sep" %in% names(args))) {
        # "sep" not in the "..." argument and so take sep=","
        if(length(args) < 1 || !("comment.char" %in% names(args))) {
            gen <- read.table(genfile, sep=",", na.strings=na.strings,
                              colClasses="character", fill=TRUE,
                              stringsAsFactors=TRUE,
                              blank.lines.skip=TRUE, comment.char="", ...)
            pheno <- read.table(phefile, sep=",", na.strings=na.strings,
                                colClasses="character", fill=TRUE,
                                stringsAsFactors=TRUE,
                                blank.lines.skip=TRUE, comment.char="", ...)
        }
        else {
            gen <- read.table(genfile, sep=",", na.strings=na.strings,
                              colClasses="character", fill=TRUE,
                              stringsAsFactors=TRUE,
                              blank.lines.skip=TRUE, ...)
            pheno <- read.table(phefile, sep=",", na.strings=na.strings,
                                colClasses="character", fill=TRUE,
                                stringsAsFactors=TRUE,
                                blank.lines.skip=TRUE, ...)
        }
    }
    else {
        if(length(args) < 1 || !("comment.char" %in% names(args))) {
            gen <- read.table(genfile, na.strings=na.strings,
                              colClasses="character", fill=TRUE,
                              stringsAsFactors=TRUE,
                              blank.lines.skip=TRUE, comment.char="", ...)
            pheno <- read.table(phefile, na.strings=na.strings,
                                colClasses="character", fill=TRUE,
                                stringsAsFactors=TRUE,
                                blank.lines.skip=TRUE, comment.char="", ...)
        }
        else {
            gen <- read.table(genfile, na.strings=na.strings,
                              colClasses="character", fill=TRUE,
                              stringsAsFactors=TRUE,
                              blank.lines.skip=TRUE, ...)
            pheno <- read.table(phefile, na.strings=na.strings,
                                colClasses="character", fill=TRUE,
                                stringsAsFactors=TRUE,
                                blank.lines.skip=TRUE, ...)
        }
    }

    if(rotate) {
        gen <- as.data.frame(t(gen), stringsAsFactors=FALSE)
        pheno <- as.data.frame(t(pheno), stringsAsFactors=FALSE)
    }

    # We must make the first column have the individual IDs
    indname <- gen[1,1]
    if(gen[3,1] == "") {
        genind <- gen[-(1:3),1]
        map.included <- TRUE
        nondatrow <- 3 # last non-data row
    }
    else {
        genind <- gen[-(1:2),1]
        map.included <- FALSE
        nondatrow <- 2 # last non-data row
    }
    gen <- gen[,-1,drop=FALSE]

    wh <- which(pheno[1,] == indname)
    if(length(wh) < 1)
        stop("Can't find the individual ID column (expected '", indname,
             "') in the phenotype file.")

    pheind <- pheno[-1,wh[1]]

    if(length(genind) == length(pheind) && all(genind == pheind)) {
        if(length(genind) != length(unique(genind)))
            warning("Duplicate individual IDs")
    }
    else {

        if(any(is.na(genind)) && any(is.na(pheind)))
            stop("There are missing genotype and phenotype IDs")
        else if(any(is.na(genind)))
            stop("There are missing genotype IDs")
        else if(any(is.na(pheind)))
            stop("There are missing phenotype IDs")

        if(length(genind) != length(unique(genind)) && length(pheind) != length(unique(pheind)))
            stop("There are duplicate genotype and phenotype IDs, and they don't all line up.")
        else if(length(genind) != length(unique(genind)))
            stop("There are duplicate genotype IDs, and the genotype and phenotype IDs don't all line up.")
        else if(length(pheind) != length(unique(pheind)))
            stop("There are duplicate phenotype IDs, and the genotype and phenotype IDs don't all line up.")

        mgp <- match(genind, pheind)

        if(any(is.na(mgp))) { # individuals with genotypes but no phenotypes
            n.add <- sum(is.na(mgp))
            pheind <- c(pheind, genind[is.na(mgp)])
            pheno <- rbind(pheno, matrix(rep(NA, n.add*ncol(pheno)), ncol=ncol(pheno)))
            pheno[-1, wh[1]] <- pheind

            warning(n.add, " individuals with genotypes but no phenotypes\n    ", paste(genind[is.na(mgp)], collapse="|"), "\n")
        }

        mpg <- match(pheind, genind)
        if(any(is.na(mpg))) {
            n.add <- sum(is.na(mpg))
            genind <- c(genind, pheind[is.na(mpg)])

            genadd <- matrix(rep(NA, n.add*ncol(gen)), ncol=ncol(gen))
            colnames(genadd) <- colnames(gen)
            gen <- rbind(gen, genadd)

            warning(n.add, " individuals with phenotypes but no genotypes\n    ", paste(pheind[is.na(mpg)], collapse="|"), "\n")
        }

        mgp <- match(genind, pheind)
        pheind <- pheind[mgp]
        pheno <- pheno[1+c(0,mgp),,drop=FALSE]
    }
    n.phe <- ncol(pheno)

    if(map.included) {
        map <- asnumericwithdec(unlist(gen[3,]), dec=dec)
        if(any(is.na(map))) {
            temp <- unique(unlist(gen[3,])[is.na(map)])
            stop("There are missing marker positions.\n",
                 "   In particular, we see these value(s): ",
                 paste("\"",paste(temp,collapse="\",\"",sep=""),"\"",collapse=" ",sep=""),
                 " at position(s): ",
                 paste(which(is.na(map)),colapse=",",sep=""),sep="")
        }
    }
    else
        map <- rep(0,ncol(gen))

    colnames(pheno) <- unlist(pheno[1,])
    pheno <- apply(pheno, 2, function(a) { a[!is.na(a) & a==""] <- NA; a })
    pheno <- as.data.frame(pheno[-1,], stringsAsFactors=TRUE)

    # replace empty cells with NA
    gen <- sapply(gen,function(a) { a[!is.na(a) & a==""] <- NA; a })

    # pull apart phenotypes, genotypes and map
    mnames <- unlist(gen[1,])
    if(any(is.na(mnames)))
        stop("There are missing marker names. Check column(s) ",paste(which(is.na(mnames))+1+n.phe,collapse=","),sep="")
    chr <- unlist(gen[2,])
    if(any(is.na(chr)))
        stop("There are missing chromosome IDs. Check column(s) ",paste(which(is.na(chr))+1+n.phe,collapse=","),sep="")

    if(any(is.na(chr))) {
        na.positions <- which(is.na(chr))
        na.positions.str <- ""
        if (length(na.positions)<10) {
            na.positions.str <- paste(" at position(s) ",
                                      paste(na.positions,collapse=",",sep=""),sep="")
        }
        stop("There are ", length(na.positions), " missing chromosome IDs",
             na.positions.str, ".")
    }

    # look for strange entries in the genotype data
    if(length(genotypes) > 0) {
        temp <- unique(as.character(gen[-(1:nondatrow),,drop=FALSE]))
        temp <- temp[!is.na(temp)]
        wh <- !(temp %in% genotypes)
        if(any(wh)) {
            warn <- "The following unexpected genotype codes were treated as missing.\n    "
            ge <- paste("|", paste(temp[wh],collapse="|"),"|",sep="")
            warn <- paste(warn,ge,"\n",sep="")
            warning(warn)
        }

        # convert genotype data
        allgeno <- matrix(match(gen[-(1:nondatrow),,drop=FALSE],genotypes),
                          ncol=ncol(gen))
    }
    else
        allgeno <- matrix(as.numeric(gen[-(1:nondatrow),,drop=FALSE]),
                          ncol=ncol(gen))

    pheno <- data.frame(lapply(pheno, sw2numeric, dec=dec), stringsAsFactors=TRUE)

    # re-order the markers by chr and position
    # try to figure out the chr labels
    if(all(chr %in% c(1:999,"X","x"))) { # 1...19 + X
        tempchr <- chr
        tempchr[chr=="X" | chr=="x"] <- 1000
        tempchr <- as.numeric(tempchr)
        if(map.included) neworder <- order(tempchr, map)
        else neworder <- order(tempchr)
    }
    else {
        # don't let it reorder the chromosomes
        tempchr <- factor(chr, levels=unique(chr))

        if(map.included) neworder <- order(tempchr, map)
        else neworder <- order(tempchr)
    }
    chr <- chr[neworder]
    map <- map[neworder]
    allgeno <- allgeno[,neworder,drop=FALSE]
    mnames <- mnames[neworder]

    # fix up dummy map
    if(!map.included) {
        map <- split(rep(0,length(chr)),chr)[unique(chr)]
        map <- unlist(lapply(map,function(a) seq(0,length=length(a),by=5)))
        names(map) <- NULL
    }

    # fix up map information
    # number of chromosomes
    uchr <- unique(chr)
    n.chr <- length(uchr)
    geno <- vector("list",n.chr)
    names(geno) <- uchr
    min.mar <- 1
    allautogeno <- NULL
    for(i in 1:n.chr) { # loop over chromosomes
        # create map
        temp.map <- map[chr==uchr[i]]
        names(temp.map) <- mnames[chr==uchr[i]]

        # pull out appropriate portion of genotype data
        data <- allgeno[,min.mar:(length(temp.map)+min.mar-1),drop=FALSE]
        min.mar <- min.mar + length(temp.map)
        colnames(data) <- names(temp.map)

        geno[[i]] <- list(data=data,map=temp.map)
        if(uchr[i] == "X" || uchr[i] == "x")
            class(geno[[i]]) <- "X"
        else {
            class(geno[[i]]) <- "A"
            if(is.null(allautogeno)) allautogeno <- data
            else allautogeno <- cbind(allautogeno,data)
        }
    }

    if(is.null(allautogeno)) allautogeno <- allgeno

    # check that data dimensions match
    n.mar1 <- sapply(geno,function(a) ncol(a$data))
    n.mar2 <- sapply(geno,function(a) length(a$map))
    n.phe <- ncol(pheno)
    n.ind1 <- nrow(pheno)
    n.ind2 <- sapply(geno,function(a) nrow(a$data))
    if(any(n.ind1 != n.ind2)) {
        cat(n.ind1,n.ind2,"\n")
        stop("Number of individuals in genotypes and phenotypes do not match.");
    }
    if(any(n.mar1 != n.mar2)) {
        cat(n.mar1,n.mar2,"\n")
        stop("Numbers of markers in genotypes and marker names files do not match.");
    }

    # print some information about the amount of data read
    cat(" --Read the following data:\n");
    cat("\t", n.ind1, " individuals\n");
    cat("\t", sum(n.mar1), " markers\n");
    cat("\t", n.phe, " phenotypes\n");

    if(all(is.na(allgeno)))
        warning("There is no genotype data!\n")

    # determine map type: f2 or bc or 4way?
    if(all(is.na(allgeno))) warning("There is no genotype data!\n")
    if(all(is.na(allautogeno)) || max(allautogeno,na.rm=TRUE)<=2) type <- "bc"
    else if(max(allautogeno,na.rm=TRUE)<=5) type <- "f2"
    else type <- "4way"
    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c(type,"cross")

    # check that nothing is strange in the genotype data
    if(type=="f2") max.gen <- 5
    else if(type=="bc") max.gen <- 2
    else max.gen <- 14

    # check that markers are in proper order
    #     if not, fix up the order
    for(i in 1:n.chr) {
        if(any(diff(cross$geno[[i]]$map)<0)) {
            o <- order(cross$geno[[i]]$map)
            cross$geno[[i]]$map <- cross$geno[[i]]$map[o]
            cross$geno[[i]]$data <- cross$geno[[i]]$data[,o,drop=FALSE]
        }
    }

    # estimate genetic map
    if(estimate.map && !map.included) estmap <- TRUE
    else estmap <- FALSE

    # return cross + indicator of whether to run est.map
    list(cross,estmap)
}

# end of read.cross.csvs.R

#####################################################################
#
# mqmcircleplot.R
#
# Copyright (c) 2009-2019, Danny Arends
#
# Modified by Pjotr Prins and Karl Broman
#
#
# first written Februari 2009
# last modified December 2019
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: mqmplot_c
#           mqmplotcircle
#           mqmplot_circle
#           circlelocations
#           drawspline
#           getchromosomelength
#           getgenomelength
#           locationtocircle
#           drawcirculargenome
#           loopthroughmulti
#
#
#####################################################################

mqmplot.circle <- function(cross, result, highlight=0, spacing=25, interactstrength=2, axis.legend = TRUE, col.legend=FALSE, verbose=FALSE, transparency=FALSE){
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }
    retresults <- NULL
    lod<-FALSE
    if(inherits(result, "mqmmulti")){
        if(highlight > 0){
            templateresult <- mqmextractmarkers(result[[highlight]])
            lod<-TRUE
        }else{
            templateresult <- mqmextractmarkers(result[[1]])
            lod<-FALSE
        }
    }else{
        templateresult <- mqmextractmarkers(result)
        lod<-TRUE
    }
    if(!inherits(templateresult, "scanone")) {
        stop("Wrong type of result file, please supply a valid scanone object.")
    }
    if(transparency){
        colorz <- rainbow(length(result), alpha=0.8)
    }else{
        colorz <- rainbow(length(result))
    }
    if(!col.legend){
        totallength <- getgenomelength(templateresult)
        nchr <- length(unique(templateresult[,1]))
        cvalues <- circlelocations(totallength+(nchr*spacing))

        drawcirculargenome(templateresult,spacing=spacing,lodmarkers=lod)

        if(inherits(result, "mqmmulti")){
            #multiple scan results, so plot em all
            todo <- 1:length(result)
            if(highlight!=0){
                #Unless we highlight only one of them
                todo <- highlight
            }
            for(x in todo){
                model <- mqmgetmodel(result[[x]])
                if(!is.null(model)){
                    if(verbose) cat("Model found for trait ",x,"\n")
                    if(!is.null(cross$locations)){
                        if(verbose) cat("Locations of traits available\n")
                        location <- cross$locations[[x]]
                        traitl <- locationtocircle(templateresult,location[[1]],location[[2]],spacing=spacing)
                    }else{
                        if(verbose) cat("Locations of traits not available\n")
                        traitl <- t(c(0,0+0.25*(0.5-(x/length(result)))))
                    }
                    if(highlight==0){
                        col=colorz[x]
                    }else{
                        col=rgb(0.1, 0.1, 0.1, 0.1)
                        if(highlight==x) title(main = paste("Circleplot of:",colnames(cross$pheno)[x]))
                    }
                    for(y in 1:length(model[[4]])){
                        qtll <- locationtocircle(templateresult,model[[4]][y],model[[5]][y],spacing=spacing)
                        if(highlight==x){
                            for(z in y:length(model[[4]])){
                                if(!z==y){
                                    cross <- sim.geno(cross)
                                    eff <- effectplot(cross,pheno.col=x,mname1=model$name[y],mname2=model$name[z],draw=FALSE)
                                    changeA <- (eff$Means[1,2]-eff$Means[1,1])
                                    changeB <- (eff$Means[2,2]-eff$Means[2,1])
                                    #interaction
                                    qtl2 <- locationtocircle(templateresult,model[[4]][z],model[[5]][z],spacing=spacing)
                                    if(!is.na(changeA) && !is.na(changeB) && !any(is.na(eff$SEs))){
                                        col <- "blue"
                                        if(changeA/abs(changeA) <  changeB/abs(changeB)) col <- "green"
                                        if(abs(abs(changeA)-abs(changeB)) > interactstrength*mean(eff$SEs)){
                                            retresults <- rbind(retresults,c(model$name[y],model$name[z],changeA,changeB,mean(eff$SEs)))
                                            drawspline(qtll,qtl2,lwd=2,col=col)
                                        }
                                    }
                                }
                            }
                        }
                        if(highlight==0){
                            points(traitl,col=col,pch=24,cex=1)
                            drawspline(traitl,qtll,col=col)
                            points(qtll*(1+0.1*((x/length(result)))),col=col,pch=19,cex=1)
                        }
                    }
                }else{
                    if(verbose) cat("Trait ",x," has no model\n")
                }
            }
            if(axis.legend) legend("topleft",c("Trait","QTL"),col=c("black","black"),pch=c(24,19),cex=1)
        }
        if(inherits(result, "scanone") || highlight > 0){
            #single scan result or highlighting one of the multiple
            if(!inherits(result, "scanone")){
                #Just highlight the template result
                result <- templateresult
            }
            model <- mqmgetmodel(result)
            if(!is.null(model)){
                for(y in 1:length(model[[4]])){
                    qtll <- locationtocircle(templateresult,model[[4]][y],model[[5]][y],spacing=spacing)
                    points(qtll,col="red",pch=19,cex=1)
                    text(qtll*1.15,model[[2]][y],col="red",cex=0.7)
                    if(!is.null(cross$locations)){
                        location <- cross$locations[[highlight]]
                        traitl <- locationtocircle(templateresult,location[[1]],location[[2]],spacing=spacing)
                        points(traitl,col="red",lwd=2,pch=24,cex=1.5)
                    }else{
                        traitl <- c(0,0)
                    }
                    if(!(highlight>0))drawspline(traitl,qtll,col="red")
                }
            }
            if(axis.legend) legend("topright",c("Selected cofactor(s)","Epistasis (+)","Epistasis (-)"), col=c("red", "blue", "green"), pch=19, lwd=c(0,1,2), cex=0.75)
            if(axis.legend) legend("bottomright",c("LOD 3","LOD 6","LOD 9","LOD 12"), pch=19, lwd=0, pt.cex = c(1, 2, 3, 4), cex=0.75)
            if(highlight==0) title(sub = "Single trait")
        }
    }else{
        plot(c(-1,1), c(-1, 1), type = "n", axes = FALSE, xlab = "", ylab = "")
        title(main = "Legend to circular genome plot")
        legend("center", paste(colnames(cross$pheno)), col=colorz, pch=19, cex=0.75)
    }
    if(!is.null(retresults)){
        colnames(retresults) <- c("Marker","Marker","Change","Change","SEs")
        retresults <- as.data.frame(retresults, stringsAsFactors=TRUE)
        return(invisible(retresults))
    }
}

circlelocations <- function(nt){
    medpoints <- matrix(nrow = nt, ncol = 2)
    phi <- seq(0, 2 * pi, length = (nt + 1))
    complex.circle <- complex(modulus = 1, argument = phi)
    for (j in 1:nt) {
        medpoints[j, ] <- c(Im(complex.circle[j]), Re(complex.circle[j]))
    }
    medpoints
}

drawspline <- function (cn1, cn2, lwd = 1,col="blue",...){
    x <- cbind(cn1[1],0,cn2[1])
    y <- cbind(cn1[2],0,cn2[2])
    r <- xspline(x, y, lty=1, shape=1, lwd=lwd, border=col,...)
}

getchromosomelength <- function(result, chr){
    l <- ceiling(max(result[which(result[,1]==chr),2]))
    l
}

getgenomelength <- function(result){
    l <- 1
    for(x in unique(result[,1])){
        l <- l + getchromosomelength(result,x)
    }
    l
}

locationtocircle <- function(result, chr, loc, spacing=50, fixoutofbounds=TRUE, verbose=FALSE){
    templateresult <- result
    totallength <- getgenomelength(result)
    nchr <- length(unique(templateresult[,1]))
    cvalues <- circlelocations(totallength+(nchr*spacing))
    l <- 1
    for(x in unique(templateresult[,1])){
        if(x==chr){
            if(loc < getchromosomelength(result,x)){
                return(t(cvalues[(l+loc),]))
            }else{
                if(verbose) cat("Location out of chromosome bounds",loc," ",getchromosomelength(result,x),"\n")
                if(fixoutofbounds) return(t(cvalues[(l+getchromosomelength(result,x)),]))
                stop(paste("Location out of chromosome bounds",loc," ",getchromosomelength(result,x),"\n"))
            }
        }
        l <- l + getchromosomelength(result,x) + spacing
    }
    stop("No such chromosome")
}

drawcirculargenome <- function(result,lodmarkers=FALSE,spacing=50){
    result <- mqmextractmarkers(result)
    plot(c(-1.1, 1.1), c(-1.1, 1.1), type = "n", axes = FALSE, xlab = "", ylab = "")
    totallength <- getgenomelength(result)
    nchr <- length(unique(result[,1]))
    cvalues <- circlelocations(totallength+(nchr*spacing))
    l <- 1
    for(x in unique(result[,1])){
        #Draw chromosomes
        nl <- l+getchromosomelength(result,x)
        lines(cvalues[l:nl,],cex=0.01)
        l <- nl + spacing
    }
    for(x in 1:nrow(result)){
        #Draw markers
        if(lodmarkers){
            points(locationtocircle(result,result[x,1],result[x,2],spacing=spacing),pch=20,cex=min(c((result[x,3]),4)))
        }else{
            points(locationtocircle(result,result[x,1],result[x,2],spacing=spacing),pch=20)
        }
    }
    for(x in unique(result$chr)) {
        chrnumberloc <- locationtocircle(result,x,getchromosomelength(result,x)/2,spacing=spacing)
        points(t(c(-1.1, -1.15)))
        points(t(c(-0.9, -1.15)))
        points(t(c(-0.7, -1.15)))
        text(t(c(-0.9, -1.0)),paste("Distances in cM"),cex=0.8)
        text(t(c(-1.1, -1.1)),paste("0 cM"),cex=0.7)
        text(t(c(-0.9, -1.1)),paste(round((totallength+(nchr*spacing))*(0.2/(2*pi)),digits=1),"cM"),cex=0.7)
        text(t(c(-0.7, -1.1)),paste(round((totallength+(nchr*spacing))*(0.4/(2*pi)),digits=1),"cM"),cex=0.7)
        text(0.9*chrnumberloc,paste("Chr",x),cex=0.8)

    }
}

loopthroughmulti <- function(cross,result,save=FALSE,spacing=100){
    n <- 1
    while(n <= length(result)){
        if(save) png(filename=paste("circleplotT",n,".png",sep=""),width=1024,height=768)
        mqmplot.circle(cross,result,spacing=spacing,highlight=n)
        if(save) dev.off()
        n <- n+1
    }
}

######################################################################
#
# makeqtl.R
#
# copyright (c) 2002-2019, Hao Wu and Karl W. Broman
# last modified Dec, 2019
# first written Apr, 2002
#
# Modified by Danny Arends
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: makeqtl, replaceqtl, addtoqtl, dropfromqtl, locatemarker
#           print.qtl, summary.qtl, print.summary.qtl, reorderqtl
#           plot.qtl
#           print.compactqtl, summary.compactqtl, print.summary.compactqtl
#
######################################################################

######################################################################
#
# This is the function to construct an object of class "qtl"
# The phenotype data and genotype data for a given list of
# chromosome and locations will be extracted from the input
# "cross" object
#
######################################################################
makeqtl <-
    function(cross, chr, pos, qtl.name, what=c("draws", "prob"))
{
    if( !inherits(cross, "cross") )
        stop("The first input variable must be an object of class cross")

    # cross type
    type <- crosstype(cross)
    chr_type <- sapply(cross$geno, chrtype)
    names(chr_type) <- names(cross$geno)
    sexpgm <- getsex(cross)

    what <- match.arg(what)

    themap <- pull.map(cross)

    # try to interpret chr argument
    if(!is.character(chr))
        chr <- as.character(chr)

    # chr, pos and qtl.name must have the same length
    if(length(chr) != length(pos))
        stop("Input chr and pos must have the same length.")
    else if( !missing(qtl.name) )
        if( length(chr) != length(qtl.name) )
            stop("Input chr and qtl.name must have the same length.")

    # local variables
    n.ind <- nrow(cross$pheno) # number of individuals
    n.pos <- length(chr) # number of selected markers
    n.gen <- NULL

    # initialize output object
    qtl <- NULL

    # take out the imputed genotypes and/or genoprobs for the
    # selected markers (if there are there)
    if(what == "draws") { # pull out draws
        if(!("draws" %in% names(cross$geno[[1]])))
            stop("You must first run sim.geno.")

        # take out imputed genotype data
        n.draws <- dim(cross$geno[[1]]$draws)[3] # number of draws

        # initialize geno matrix for selected markers
        geno <- array(rep(0, n.ind*n.pos*n.draws),
                      dim=c(n.ind, n.pos, n.draws))

        for(i in 1:n.pos) {
            # get the index for this chromosome
            i.chr <- which(chr[i]==names(cross$geno))
            if(length(i.chr) == 0) # no this chromosome in cross
                stop("There's no chromosome number ", chr[i], " in input cross object")
            i.pos <- pos[i] # marker position

            # make the genetic map for this chromosome
            if("map" %in% names(attributes(cross$geno[[i.chr]]$draws)))
                map <- attr(cross$geno[[i.chr]]$draws,"map")
            else {
                stp <- attr(cross$geno[[i.chr]]$draws, "step")
                oe <- attr(cross$geno[[i.chr]]$draws, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[i.chr]]$draws)))
                    stpw <- attr(cross$geno[[i.chr]]$draws, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[i.chr]]$map,stp,oe,stpw)
            }

            # pull out the female map if there are sex-specific maps
            if(is.matrix(map)) map <- map[1,]

            # locate this marker (given chromosome and position)
            marker.idx <- locatemarker(map, i.pos, i.chr, flag="draws")
            if(length(marker.idx) > 1)
                stop("Multiple markers at the same position; run jittermap.")

            # if everything is all right, take the genotype
            geno[,i,] <- cross$geno[[i.chr]]$draws[,marker.idx,]
            pos[i] <- map[marker.idx]

            # no. genotypes
            n.gen[i] <- length(getgenonames(type,chr_type[i.chr],"full",sexpgm, attributes(cross)))

            # Fix up X chromsome here
            if(chr_type[i.chr]=="X" && (type=="bc" || type=="f2"))
                geno[,i,] <- reviseXdata(type,"full",sexpgm,draws=geno[,i,,drop=FALSE],
                                         cross.attr=attributes(cross))
        }
        # give geno dimension names
        # the 2nd dimension called "Q1", "Q2", etc.
        dimnames(geno) <- list(NULL, paste("Q", 1:n.pos, sep=""), NULL)
        # output
        qtl$geno <- geno
    }
    else { # pull out probs

        if(!("prob" %in% names(cross$geno[[1]])))
            stop("You must first run calc.genoprob.")

        # initialize prob matrix
        prob <- vector("list",n.pos)

        # locate the marker
        for(i in 1:n.pos) {

            # get the index for this chromosome
            i.chr <- which(chr[i]==names(cross$geno))
            if(length(i.chr) == 0) # no this chromosome in cross
                stop("There's no chromosome number ", chr[i], " in input cross object")
            i.pos <- pos[i] # marker position


            if("map" %in% names(attributes(cross$geno[[i.chr]]$prob)))
                map <- attr(cross$geno[[i.chr]]$prob,"map")
            else {
                stp <- attr(cross$geno[[i.chr]]$prob, "step")
                oe <- attr(cross$geno[[i.chr]]$prob, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[i.chr]]$prob)))
                    stpw <- attr(cross$geno[[i.chr]]$prob, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[i.chr]]$map,stp,oe,stpw)
            }

            # pull out the female map if there are sex-specific maps
            if(is.matrix(map)) map <- map[1,]

            # locate this marker (given chromosome and position)
            marker.idx <- locatemarker(map, i.pos, i.chr, flag="prob")
            if(length(marker.idx) > 1)
                stop("Multiple markers at the same position; run jittermap.")

            # take genoprob
            if(chr_type[i.chr]=="X" && (type=="bc" || type=="f2")) { # fix X chromosome probs
                prob[[i]] <- reviseXdata(type, "full", sexpgm,
                                         prob=cross$geno[[i.chr]]$prob[,marker.idx,,drop=FALSE],
                                         cross.attr=attributes(cross))[,1,]
            }
            else
                prob[[i]] <- cross$geno[[i.chr]]$prob[,marker.idx,]

            pos[i] <- map[marker.idx]

            # no. genotypes
            n.gen[i] <- ncol(prob[[i]])
        }
        qtl$prob <- prob
    }

    if(missing(qtl.name))  { # no given qtl names
        dig <- 1

        if(what=="draws")
            step <- attr(cross$geno[[i.chr]]$draws, "step")
        else
            step <- attr(cross$geno[[i.chr]]$prob, "step")

        if(!is.null(step)) {
            if(step > 0) dig <- max(dig, -floor(log10(step)))
        }
        else {
            if(what=="draws")
                stepw <- attr(cross$geno[[i.chr]]$draws, "stepwidth")
            else
                stepw <- attr(cross$geno[[i.chr]]$prob, "stepwidth")

            if(!is.null(stepw) && stepw > 0) dig <- max(dig, -floor(log10(stepw)))
        }

        # make qtl names
        qtl.name <- paste( paste(chr,sep=""), charround(pos,dig), sep="@")
    }

    # output object
    qtl$name <- qtl.name
    qtl$altname <- paste("Q", 1:n.pos, sep="")
    qtl$chr <- chr
    qtl$pos <- pos
    qtl$n.qtl <- n.pos
    qtl$n.ind <- nind(cross)
    qtl$n.gen <- n.gen
    qtl$chrtype <- chr_type[qtl$chr]
    names(qtl$chrtype) <- NULL

    class(qtl) <- "qtl"
    attr(qtl, "map") <- themap

    qtl
}



######################################################################
#
# This is the function to replace one QTL by another.
#
######################################################################
replaceqtl <-
    function(cross, qtl, index, chr, pos, qtl.name, drop.lod.profile=TRUE)
{
    if(!inherits(qtl, "qtl"))
        stop("qtl should have class \"qtl\".")

    if(any(index < 1 | index > qtl$n.qtl))
        stop("index should be between 1 and ", qtl$n.qtl)

    if(length(index) != length(chr) || length(index) != length(pos))
        stop("index, chr, and pos should all have the same length.")
    if(!missing(qtl.name) && length(index) != length(qtl.name))
        stop("index and qtl.name should have the same length.")

    if("geno" %in% names(qtl)) what <- "draws"
    else what <- "prob"

    if(missing(qtl.name))
        newqtl <- makeqtl(cross, chr, pos, what=what)
    else
        newqtl <- makeqtl(cross, chr, pos, qtl.name=qtl.name, what=what)

    if(what=="draws") {
        qtl$geno[,index,] <- newqtl$geno
    }
    else {
        qtl$prob[index] <- newqtl$prob
    }

    qtl$name[index] <- newqtl$name
    qtl$chr[index] <- newqtl$chr
    qtl$pos[index] <- newqtl$pos
    qtl$chrtype[index] <- newqtl$chrtype

    if(qtl$n.ind != newqtl$n.ind) stop("Mismatch in no. individuals")

    qtl$n.gen[index] <- newqtl$n.gen

    if(drop.lod.profile)
        attr(qtl, "lodprofile") <- NULL

    qtl
}


######################################################################
#
# This is the function to add a QTL to given qtl object
#
######################################################################

addtoqtl <-
    function(cross, qtl, chr, pos, qtl.name, drop.lod.profile=TRUE)
{
    if(!inherits(qtl, "qtl"))
        stop("qtl should have class \"qtl\".")

    if("geno" %in% names(qtl)) what <- "draws"
    else what <- "prob"

    if(missing(qtl.name))
        newqtl <- makeqtl(cross, chr, pos, what=what)
    else
        newqtl <- makeqtl(cross, chr, pos, qtl.name=qtl.name, what=what)

    if(what=="draws") {
        do <- dim(qtl$geno)
        dn <- dim(newqtl$geno)
        if(do[1] != dn[1] || do[3] != dn[3])
            stop("Mismatch in number of individuals or number of imputations.")

        temp <- array(dim=c(do[1], do[2]+dn[2], do[3]))
        temp[,1:ncol(qtl$geno),] <- qtl$geno
        temp[,-(1:ncol(qtl$geno)),] <- newqtl$geno
        colnames(temp) <- paste("Q", 1:ncol(temp), sep="")
        qtl$geno <- temp
    }
    else {
        qtl$prob <- c(qtl$prob, newqtl$prob)
    }

    qtl$name <- c(qtl$name, newqtl$name)
    qtl$chr <- c(qtl$chr, newqtl$chr)
    qtl$pos <- c(qtl$pos, newqtl$pos)
    qtl$n.qtl <- qtl$n.qtl + newqtl$n.qtl
    qtl$altname <- paste("Q", 1:qtl$n.qtl, sep="")
    qtl$chrtype <- c(qtl$chrtype, newqtl$chrtype)
    if(qtl$n.ind != newqtl$n.ind)
        stop("Mismatch in no. individuals")
    qtl$n.gen <- c(qtl$n.gen, newqtl$n.gen)

    attr(qtl, "formula") <- NULL
    attr(qtl, "pLOD") <- NULL

    if(drop.lod.profile)
        attr(qtl, "lodprofile") <- NULL

    qtl
}

######################################################################
#
# This is the function to drop a QTL from a given qtl object
#
######################################################################
dropfromqtl <-
    function(qtl, index, chr, pos, qtl.name, drop.lod.profile=TRUE)
{
    if(!inherits(qtl, "qtl"))
        stop("qtl should have class \"qtl\".")

    if(!missing(chr) || !missing(pos)) {
        if(missing(chr) || missing(pos))
            stop("Give both chr and pos, or give name, or give a numeric index")
        if(!missing(qtl.name) || !missing(index))
            stop("Give chr and pos or qtl.name or numeric index, but not multiple of these.")
        if(length(chr) != length(pos))
            stop("chr and pos must have the same lengths.")

        todrop <- NULL
        for(i in seq(along=chr)) {
            m <- which(qtl$chr == chr[i])

            if(length(m) < 1)
                stop("No QTL on chr ", chr[i], " in input qtl object.")

            for(j in seq(along=m)) {
                d <- abs(qtl$pos[m[j]] - pos[i])
                if(min(d) > 10) stop("No qtl near position ", pos[i], " on chr ", chr[i])

                wh <- m[d==min(d)]
                if(length(wh) > 1)
                    stop("Multiple QTL matching chr ", chr[i], " at pos ", pos[i])

                if(min(d) > 1)
                    warning("No QTL on chr ", chr[i], " exactly at ", pos[i],
                            "; dropping that at ", qtl$pos[wh])

                todrop <- c(todrop, wh)
            }
        }

        todrop <- unique(todrop)
    }
    else if(!missing(qtl.name)) {
        if(!missing(index))
            stop("Give chr and pos or qtl.name or numeric index, but not multiple of these.")

        m <- match(qtl.name, qtl$name)
        if(all(is.na(m))) # if no matches, try "altname"
            m <- match(qtl.name, qtl$altname)

        if(any(is.na(m)))
            warning("Didn't match QTL ", qtl.name[is.na(m)])

        todrop <- m[!is.na(m)]
    }
    else {
        if(missing(index))
            stop("Give chr and pos or qtl.name or numeric index, but not multiple of these.")
        if(any(index < 1 | index > qtl$n.qtl))
            stop("index should be between 1 and ", qtl$n.qtl)

        todrop <- index
    }

    # input drop is an integer index
    # get the index for exclusing drop QTL
    idx <- setdiff(1:qtl$n.qtl, todrop)

    # result object
    qtl$name <- qtl$name[idx]
    qtl$chr <- qtl$chr[idx]
    qtl$chrtype <- qtl$chrtype[idx]
    qtl$pos <- qtl$pos[idx]
    qtl$n.qtl <- qtl$n.qtl - length(todrop)
    qtl$altname <- paste("Q", 1:qtl$n.qtl, sep="")
    qtl$n.ind <- qtl$n.ind
    qtl$n.gen <- qtl$n.gen[idx]
    if("geno" %in% names(qtl)) {
        qtl$geno <- qtl$geno[,idx,,drop=FALSE]
        colnames(qtl$geno) <- paste("Q", 1:ncol(qtl$geno), sep="")
    }
    if("prob" %in% names(qtl))
        qtl$prob <- qtl$prob[idx]

    attr(qtl, "formula") <- NULL
    attr(qtl, "pLOD") <- NULL

    if(drop.lod.profile)
        attr(qtl, "lodprofile") <- NULL

    qtl
}


##################################################################
#
# locate the marker on a genetic map. Choose the nearest
# one if there's no marker or pseudomarker one the given
# location
#
# This is the internal function and not supposed to be used by user
#
###################################################################

locatemarker <-
    function(map, pos, chr, flag)
{
    marker.idx <- which(map == pos)
    if( length(marker.idx)==0 ) {
        # there's no this marker, take the nearest marker instead
        # if there's a tie, take the first nearst one
        m.tmp <- abs(pos-map)
        marker.idx <- which(m.tmp==min(m.tmp))[[1]]
    }

    if(length(marker.idx) > 1)
        marker.idx <- marker.idx[sample(length(marker.idx))]
    marker.idx
}

# print QTL object
print.qtl <-
    function(x, ...)
{
    print(summary(x))
}

# summary of QTL object
summary.qtl <-
    function(object, ...)
{
    if(is.null(object) || length(object) == 0 || length(object$chr)==0) {
        object <- numeric(0)
        class(object) <- "summary.qtl"
        return(object)
    }

    if("geno" %in% names(object)) {
        type <- "draws"
        n.draws <- dim(object$geno)[3]
    }
    else type <- "prob"

    output <- data.frame(name=object$name, chr=object$chr, pos=object$pos, n.gen=object$n.gen, stringsAsFactors=TRUE)
    rownames(output) <- object$altname

    attr(output, "type") <- type
    if(!is.null(attr(object,"mqm"))) attr(output, "mqm") <- attr(object,"mqm")
    if(type=="draws") attr(output, "n.draws") <- n.draws
    class(output) <- c("summary.qtl", "data.frame")

    if("formula" %in% names(attributes(object)))
        attr(output, "formula") <- attr(object, "formula")
    if("pLOD" %in% names(attributes(object)))
        attr(output, "pLOD") <- attr(object, "pLOD")

    output
}

# print summary of QTL object
print.summary.qtl <-
    function(x, ...)
{
    if(is.null(x) || length(x) == 0) {
        cat("  Null QTL model\n")
    }
    else {
        type <- attr(x, "type")
        if(type=="draws")
            thetext <- paste("imputed genotypes, with", attr(x, "n.draws"), "imputations.")
        else thetext <- "genotype probabilities."
        if(!is.null(attr(x,"mqm"))) thetext <- paste("model created by mqmscan")
        cat("  QTL object containing", thetext, "\n\n")

        print.data.frame(x, digits=5)
    }

    if("formula" %in% names(attributes(x))) {
        form <- attr(x, "formula")
        if(!is.character(form)) form <- deparseQTLformula(form)
        cat("\n  Formula:")
        w <- options("width")[[1]]
        printQTLformulanicely(form, "               ", w+5, w)
    }

    if("pLOD" %in% names(attributes(x)))
        cat("\n  pLOD: ", round(attr(x, "pLOD"),3), "\n")
}

######################################################################
# plot locations of QTLs on the genetic map
######################################################################
plot.qtl <-
    function(x, chr, horizontal=FALSE, shift=TRUE,
             show.marker.names=FALSE,  alternate.chrid=FALSE,
             justdots=FALSE, col="red", ...)
{
    if(!inherits(x, "qtl"))
        stop("input should be a qtl object")

    if(length(x) == 0)
        stop("  There are no QTL to plot.")

    map <- attr(x, "map")
    if(is.null(map))
        stop("qtl object doesn't contain a genetic map.")

    if(missing(chr))
        chr <- names(map)
    else {
        chr <- matchchr(chr, names(map))
        map <- map[chr]
        class(map) <- "map"
    }

    if(horizontal)
        plotMap(map, horizontal=horizontal, shift=shift,
                show.marker.names=show.marker.names, alternate.chrid=alternate.chrid,
                ylim=c(length(map)+0.5, 0), ...)
    else
        plotMap(map, horizontal=horizontal, shift=shift,
                show.marker.names=show.marker.names, alternate.chrid=alternate.chrid,
                xlim=c(0.5,length(map)+1), ...)

    whchr <- match(x$chr, names(map))
    thepos <- x$pos
    thepos[is.na(whchr)] <- NA

    if(any(!is.na(thepos))) {
        whchr <- whchr[!is.na(whchr)]

        if(shift) thepos <- thepos - sapply(map[whchr], min)

        if(is.matrix(map[[1]])) whchr <- whchr - 0.3

        if(length(grep("^.+@[0-9\\.]+$", x$name)) == length(x$name))
            x$name <- x$altname

        if(!justdots) {
            if(horizontal) {
                arrows(thepos, whchr - 0.35, thepos, whchr, lwd=2, col=col, length=0.05)
                text(thepos, whchr-0.4, x$name, col=col, adj=c(0.5,0))
            }
            else {
                arrows(whchr + 0.35, thepos, whchr, thepos, lwd=2, col=col, length=0.05)
                text(whchr+0.4, thepos, x$name, col=col, adj=c(0,0.5))
            }
        }
        else {
            if(horizontal)
                points(thepos, whchr, pch=16, col=col)
            else {
                points(whchr, thepos, pch=16, col=col)
            }
        }
    }

    invisible()
}

######################################################################
#
# This is the function to reorder the QTL within a QTL object
#
######################################################################
reorderqtl <-
    function(qtl, neworder)
{
    if(!inherits(qtl, "qtl"))
        stop("qtl should have class \"qtl\".")

    if(missing(neworder)) {
        if(!("map" %in% names(attributes(qtl))))
            stop("No map in the qtl object; you must provide argument 'neworder'.")
        chr <- names(attr(qtl, "map"))
        thechr <- match(qtl$chr, chr)
        if(any(is.na(thechr)))
            stop("Chr ", paste(qtl$chr[is.na(thechr)], " "), " not found.")
        neworder <- order(thechr, qtl$pos)
    }

    curorder <- seq(qtl$n.qtl)
    if(length(neworder) != qtl$n.qtl ||
       !all(curorder == sort(neworder)))
        stop("neworder should be an ordering of the integers from 1 to ", qtl$n.qtl)

    if(qtl$n.qtl == 1)
        stop("Nothing to do; just one qtl.")


    if("geno" %in% names(qtl))
        qtl$geno <- qtl$geno[,neworder,]
    else
        qtl$prob <- qtl$prob[neworder]

    qtl$name <- qtl$name[neworder]
    qtl$chr <- qtl$chr[neworder]
    qtl$pos <- qtl$pos[neworder]
    qtl$n.gen <- qtl$n.gen[neworder]
    qtl$chrtype <- qtl$chrtype[neworder]

    attr(qtl, "formula") <- NULL
    attr(qtl, "pLOD") <- NULL

    if("lodprofile" %in% names(attributes(qtl))) {
        lodprof <- attr(qtl, "lodprofile")
        if(length(lodprof) == length(neworder))
            attr(qtl, "lodprofile") <- lodprof[neworder]
    }

    qtl
}

# print compact version of QTL object
print.compactqtl <-
    function(x, ...)
{
    print(summary(x))
}

summary.compactqtl <-
    function(object, ...)
{
    class(object) <- c("summary.compactqtl", "list")
    object
}

print.summary.compactqtl <-
    function(x, ...)
{
    if(is.null(x) || length(x) == 0)
        cat("Null QTL model\n")
    else {
        temp <- as.data.frame(x, stringsAsFactors=TRUE)
        rownames(temp) <- paste("Q", 1:nrow(temp), sep="")
        print.data.frame(temp)
    }
    if("formula" %in% names(attributes(x))) {
        form <- attr(x, "formula")
        if(!is.character(form)) form <- deparseQTLformula(form)
        cat("  Formula:")
        w <- options("width")[[1]]
        printQTLformulanicely(form, "               ", w+5, w)
    }

    if("pLOD" %in% names(attributes(x)))
        cat("  pLOD: ", round(attr(x, "pLOD"),3), "\n")
}

# nqtl: number of qtl
nqtl <- function(qtl) ifelse(length(qtl)==0,0,length(qtl$chr))

# end of makeqtl.R

#####################################################################
#
# qtlcart_io.R
#
# copyright (c) 2002-2019, Brian S. Yandell
#          [with some modifications by Karl W. Broman and Hao Wu]
# last modified Dec, 2019
# first written Jun, 2002
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.qtlcart, read.cro.qtlcart, read.map.qtlcart,
#           write.cross.qtlcart
#           [See read.cross.R for the main read.cross function, and
#            write.cross.R for the main write.cross function.]
#
##############################################################################

######################################################################
# read.cross.qtlcart
#
# read QTL cross object in QTL cartographer format
######################################################################
read.cross.qtlcart <-
    function (dir, crofile, mapfile)
{
    if (missing(mapfile)) stop("Missing mapfile.")
    if (missing(crofile)) stop("Missing crofile.")

    if(!missing(dir) && dir != "") {
        mapfile <- file.path(dir, mapfile)
        crofile <- file.path(dir, crofile)
    }
    map <- read.map.qtlcart( mapfile )
    cro <- read.cro.qtlcart( crofile )

    cat(" --Read the following data:\n")
    cat("       Type of cross:         ", cro$cross.class, "\n")
    cat("       Number of individuals: ", nrow( cro$markers ), "\n")
    cat("       Number of markers:     ", ncol( cro$markers ), "\n")
    cat("       Number of phenotypes:  ", ncol( cro$traits ), "\n")

    maplen <- unlist(lapply(map,length))
    markers <- split( as.data.frame( t( cro$markers ), stringsAsFactors=TRUE),
                     ordered( rep(names( maplen ), maplen )))

    Geno <- list()
    for( i in names( map )) {
        name.markers <- names( map[[i]] )
        markers[[i]] <- t( markers[[i]] )
        colnames( markers[[i]] ) <- name.markers
        tmp <- list( data = markers[[i]], map = map[[i]] )

        # determine whether autosomal chromosome or X chromosome
        #     using the chromosome name
        class(tmp) <- ifelse(length(grep("[Xx]", i)), "X", "A")
        Geno[[i]] <- tmp
    }
    cross <- list(geno = Geno, pheno = cro$traits )
    cross$pheno <- as.data.frame(cross$pheno, stringsAsFactors=TRUE)
    class(cross) <- c( cro$cross.class, "cross")
    if(cro$cross.class == "bcsft")
        attr(cross, "scheme") <- cro$cross.scheme

    list(cross,FALSE)
}

######################################################################
# read.map.qtlcart
#
# read QTL Cartographer map file
######################################################################
read.map.qtlcart <-
    function (file)
{
    # only interested in chromosomes, marker IDs and positions
    f <- scan(file, what = "", blank.lines.skip = FALSE, sep = "\n",
              quiet = TRUE)
    ctrl <- seq(f)[substring(f, 1, 1) == "-"]
    getvalue <- function(s, f, ctrl) {
        tmp <- unlist(strsplit(f[ctrl[substring(f[ctrl], 2, 3) ==
                                      s]], " "))
        as.numeric(tmp["" != tmp][2])
    }
    nchrom <- getvalue("c ", f, ctrl)
    nmarkers <- getvalue("i ", f, ctrl)

    # marker positions
    tmp <- range(seq(f)[substring(f, 1, 3) == "-l "])
    s <- strsplit(f[tmp[1]], "")[[1]]
    b <- grep("\\|", s)
    s <- grep("0", s)
    s <- ceiling((s[length(s)] - b - 1)/nchrom)

    position <- scan(file, what=character(), sep="\n",
                     skip = tmp[1] - 1, n = tmp[2], quiet=TRUE)


    tmp <- grep("-b", f)
    if(length(tmp) < 1) stop("Marker names not found in map file\n")
    markers <- scan(file, list(1, 2, ""), skip = tmp[1], nlines = nmarkers,
                    blank.lines.skip = FALSE, quiet = TRUE)

    if(length(tmp) < 2) {
        warning("Chromosome names not found in map file\n")
        chroms <- as.character(1:nchrom)
    }
    else
        chroms <- scan(file, list(1, ""), skip = tmp[2], nlines = nchrom,
                       blank.lines.skip = FALSE, quiet = TRUE)[[2]]

    map <- list()
    n.markers <- table(markers[[1]])

    # make sense of "position"
    position <- strsplit(position, "\\s+")
    pos <- matrix(ncol=nchrom,nrow=max(n.markers))
    for(i in 1:max(n.markers)) {
        z <- position[[i]][-(1:3)]
        pos[i,which(n.markers >= i-1)] <- as.numeric(z)
    }

    for (i in seq(nchrom)) {
        tmp <- cumsum(pos[1:n.markers[i],i])
        names(tmp) <- markers[[3]][i == markers[[1]]]
        map[[chroms[i]]] <- tmp
    }
    map
}

######################################################################
# read.cro.qtlcart
#
# read QTL cartographer CRO file
######################################################################
read.cro.qtlcart <-
    function (file)
{
    # translation from cro to R/qtl (see read.cross)
    # -1  NA      missing data
    #  0  1       AA
    #  1  2       AB
    #  2  3       BB
    # 10  4       AA or AB
    # 12  5       AB or BB
    #
    f <- scan(file, what = "", blank.lines.skip = FALSE, sep = "\n",
              quiet = TRUE)
    ctrl <- seq(f)[substring(f, 1, 1) == "-"]
    s <- strsplit(f[ctrl], " ")
    ns <- character(length(ctrl))
    for (i in seq(ctrl)) {
        ns[i] <- substring(s[[i]][1], 2)
        s[[i]] <- s[[i]]["" != s[[i]]][-1]
    }
    names(s) <- ns
    size <- as.numeric(s$n[1])
    nmarkers <- as.numeric(s$p[1]) - 1
    ntraits <- as.numeric(s$traits[1])

    # cross.scheme (used for bcsft only)
    cross.scheme <- c(0,0)

    # cross type
    fix.bc1 <- fix.ridh <- FALSE # indicator of whether to fix genotypes
    cross <- tolower(s$cross[1])
    if (cross == "ri1" || cross=="riself") {
        cross <- "riself"
        fix.ridh <- TRUE
    }
    else if (cross == "ri2" || cross=="risib") {
        cross <- "risib"
        fix.ridh <- TRUE
    }
    else if (cross == "ri0" || cross=="dh") {
        cross <- "dh" # doubled haploid
        fix.ridh <- TRUE
    }
    else if (cross == "b1" || cross == "b2") {
        fix.bc1 <- cross == "b1"
        cross <- "bc"
    }
    else if (cross == "sf2" || cross == "rf2")
        cross <- "f2"
    else if (cross == "sf3") {
        cross <- "bcsft"
        cross.scheme <- c(0,3)
    }

    if(!cross %in% c("f2","bc","risib","riself","bcsft", "dh"))
        stop("Cross type ", cross, " not supported.")

    notraits <- as.numeric(s$otraits[1])
    skip <- ctrl["s" == ns]
    nlines <- ctrl["e" == ns] - skip - 1
    trait.names <- f[ctrl["Names" == ns][1] + 1:ntraits]
    if(notraits)
        trait.names <- c(trait.names, f[ctrl["Names" == ns][2] + 1:notraits] )
    ns <- strsplit(trait.names, " ")
    for (i in seq(ns)) ns[[i]] <- ns[[i]][length(ns[[i]])]
    trait.names <- unlist(ns)
    # kludge to handle factor phenos
    f <- matrix(scan(file, "", skip = skip, nlines = nlines, na.strings = ".",
                     blank.lines.skip = FALSE, quiet = TRUE), ncol = size)
    traits <- t(f[-(1:(2 + nmarkers)), ])
    traits = as.data.frame(traits, stringsAsFactors=TRUE)
    if (nrow(traits) == 1)
        traits <- as.data.frame(t(traits), stringsAsFactors=TRUE)
    colnames(traits) <- trait.names

    tmp = options(warn=-1)
    for(i in names(traits)){
        tmp1 = as.numeric(as.character(traits[[i]]))
        if(!all(is.na(tmp1))) traits[[i]] = tmp1
    }
    options(tmp)
    f <- t(f[3:(2 + nmarkers), ])

    # here is the translation
    f = array(as.numeric(f),dim(f))

    # omit negative genotypes (treat as missing)
    f[f<0] <- NA

    f[!is.na(f)] <- c(NA, 1:3, rep(NA, 7), 4, NA, 5)[2 + f[!is.na(f)]]
    if (fix.ridh && all(is.na(f) | f == 1 | f == 3))
        f[!is.na(f) & f == 3] <- 2
    if (fix.bc1) {
        f[!is.na(f) & f == 5] <- NA
        f[!is.na(f) & f == 2] <- 1
        f[!is.na(f) & f == 3] <- 2
    }
    list(traits = traits, markers = f, cross.class = cross, cross.scheme = cross.scheme)
}


######################################################################
# write.cross.qtlcart
#
# write a QTL cross object to files in QTL Cartographer format
######################################################################
write.cross.qtlcart <-
    function( cross, filestem="data")
{
    n.ind <- nind(cross)
    tot.mar <- totmar(cross)
    n.phe <- nphe(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)

    type <- crosstype(cross)
    if(type=="bc") {
        type <- paste("B", 1 + match(1, names(table(c(pull.geno(cross)))),
                                     nomatch = 0), sep = "")
    }
    else if(type=="f2") type <- "RF2"
    else if(type=="riself") type <- "RI1"
    else if(type=="risib") type <- "RI2"
    else {
        warn <- paste("Cross type", type, "may not work with QTL Cartographer.")
        warning(warn)
    }

    # RIL data: convert genotypes to 1/3; later will be converted to 0/2
    if(type=="RI1" || type=="RI2") {
        for(i in 1:n.chr) {
            g <- cross$geno[[i]]$data
            g[!is.na(g) & g==2] <- 3
            cross$geno[[i]]$data <- g
        }
    }

    # write genotype and phenotype data
    file <- paste(filestem, ".cro", sep="")
    if( file.exists( file )) {
        warning( paste( "previous file", file, "moved to *.bak" ))
        file.rename( file, paste( file, "bak", sep = "." ))
    }
    write("#  123456789 -filetype Rcross.out", file, append=FALSE)

    # write numbers of progeny, markers and phenotypes
    write( paste( "-n   ", n.ind ), file, append=TRUE)
    write( paste( "-p   ", 1 + tot.mar ), file, append=TRUE)
    # write experiment type
    write( paste( "-cross", type ), file, append=TRUE)

    # write numbers of progeny, markers and phenotypes
    write( paste( "-traits   ", n.phe ), file, append=TRUE)
    write( "-Names of the traits...", file, append=TRUE)
    phe <- names( cross$pheno )
    for( i in seq( phe ))
        write( paste( i, phe[i] ), file, append=TRUE)
    write( paste( "-otraits   ", 0 ), file, append=TRUE)

    # write genotype and phenotype data by individual
    write( "-s", file, append=TRUE)
    for( ind in 1:n.ind ) {
        write( paste( ind, 1 ), file, append=TRUE)
        for(i in 1:n.chr) {
            g <- unlist( cross$geno[[i]]$data[ind,] )
            g[ is.na( g ) ] <- 0
            g <- c(-1,0,1,2,10,12)[ 1 + g ]

            #      if( length( g ) <= 40)
            write(paste( "      ", paste( g, collapse = " " )), file, append=TRUE)
            #      else {
            #        lo <- seq( 1, length(g), by=40)
            #        hi <- c( lo[-1]-1, length( g ))
            #        for(k in seq(along=lo)) {
            #          write( paste( "      ", paste( g[lo[k]:hi[k]], collapse = " " )),
            #                file, append=TRUE)
            #        }
            #      }
        } # end writing marker data
        p <- c( cross$pheno[ind,])
        tmp <- format( p )
        tmp[ is.na( p ) ] <- "."
        write( paste( "       ", tmp ), file, append = TRUE )
        # end of writing phenotype data
    }
    write( "-e", file, append = TRUE )
    write( "-q", file, append = TRUE )

    # make "prep" file with map information
    file <- paste(filestem, ".map", sep="")
    if( file.exists( file )) {
        warning( paste( "previous file", file, "moved to *.bak" ))
        file.rename( file, paste( file, "bak", sep = "." ))
    }
    write("#  123456789 -filetype Rmap.out", file, append=FALSE)

    # write numbers of progeny, markers and phenotypes
    write( "-s", file, append=TRUE)
    write( "-f 1", file, append=TRUE)
    write( "-p 0.0000", file, append=TRUE)
    write( "-u c", file, append=TRUE)
    write( "#", file, append=TRUE)

    write( paste( "-c", n.chr ), file, append=TRUE)
    write( paste( "-i", tot.mar ), file, append=TRUE)

    map <- lapply( cross$geno, function( x ) x$map )
    maplen <- unlist( lapply( map, length ))

    # mean and SD of number of markers
    write( paste( "-m", round( mean( maplen ), 3 )), file, append=TRUE)
    write( paste( "-vm", round( sqrt( var( maplen )), 3 )), file, append=TRUE)

    mapdif <- lapply( map, diff )
    # mean and SD of intermarker distances
    write( paste( "-d", round( mean( unlist( mapdif )), 3 )), file, append=TRUE)
    write( paste( "-vd", round( sqrt( var( unlist( mapdif ))), 3 )), file, append=TRUE)
    write( "-t 0.0000", file, append=TRUE)
    write( "#", file, append=TRUE)
    write( "          |   Chromosome----> ", file, append=TRUE)
    write( "--------------------------------------", file, append=TRUE)
    mapmat <- matrix( NA, 1 + max( maplen ), n.chr )
    mapmat[ 1, ] <- 0
    for( i in seq( along = maplen )) {
        tmp <- c( mapdif[[i]],0)
        mapmat[1 + seq( along = tmp ), i ] <- tmp
    }
    mapmat <- format(round(mapmat, 6))
    ncmap <- nchar( mapmat[1] )
    mapmat[ grep( "NA", mapmat ) ] <- paste( rep( " ", ncmap ), collapse = "" )
    tmp <- format( seq( n.chr ))
    write( paste( "Marker    |  ",
                 paste( paste(" ", tmp, sep=""), collapse =
                       paste( rep( " ", max( 1, ncmap - nchar( tmp ))), collapse = "" ))),
          file, append=TRUE)
    write( "--------------------------------------", file, append=TRUE)
    for( i in seq( nrow( mapmat ))) {
        j <- 5-nchar(i-1)
        if(j < 1) j <- 1
        write( paste( "-l", paste(rep(" ", j), collapse=""),
                     i - 1, "|",
                     paste( mapmat[i,], collapse = " " )),
              file, append=TRUE)
    }

    write( "---------------------------------------", file, append=TRUE)
    write( paste( "-Number   |", paste( maplen, collapse = "  " )),
          file, append=TRUE)

    write( "Names and positions of the markers", file, append=TRUE)
    write( "Chrom  Mark  Name", file, append=TRUE)
    write( "-b MarkerNames", file, append=TRUE)
    for( i in 1:n.chr )
        for( j in seq( along = map[[i]] ))
            write( paste( i, j, names( map[[i]] )[j] ), file, append=TRUE)
    write( "-e MarkerNames", file, append=TRUE)
    write( "Names of the Chromosomes", file, append=TRUE)
    write( "-b ChromosomeNames", file, append=TRUE)
    for( i in 1:n.chr )
        write( paste( i, names( map )[i] ), file, append=TRUE)
    write( "-e ChromosomeNames", file, append=TRUE)
}

# end of qtlcart_io.R

######################################################################
# comparegeno
######################################################################
comparegeno <-
    function(cross, what=c("proportion","number", "both"))
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    what <- match.arg(what)
    g <- pull.geno(cross)
    g[is.na(g)] <- 0
    n.ind <- nrow(g)
    n.mar <- ncol(g)
    z <- .C("R_comparegeno",
            as.integer(g),
            as.integer(n.ind),
            as.integer(n.mar),
            n.match=as.integer(rep(0,n.ind^2)),
            n.missing=as.integer(rep(0,n.ind^2)),
            PACKAGE="qtl")

    if(what=="number") {
        z <- matrix(z$n.match,n.ind,n.ind)
    }
    else {
        if(what=="proportion") {
            z <- matrix(z$n.match/(n.mar-z$n.missing),n.ind,n.ind)
            diag(z) <- NA
        }
        else {
            prop <- matrix(z$n.match/(n.mar-z$n.missing),n.ind,n.ind)
            z <- matrix(z$n.match,n.ind,n.ind)
            z[lower.tri(z)] <- prop[lower.tri(z)]
        }
    }

    id <- getid(cross)
    if(is.null(id)) id <- as.character(seq_len(nind(cross)))

    dimnames(z) <- list(id, id)
    class(z) <- c("comparegeno", class(z))
    attr(z, "what") <- what

    z
}

# report which pairs of individuals have nearly-matching genotypes
summary.comparegeno <-
    function(object, thresh=0.9, ...)
{
    what <- attr(object, "what")
    if(is.null(what)) what <- "proportion"

    if(what=="number" && thresh < 1) thresh <- thresh*max(diag(object), na.rm=TRUE)

    wh <- which(!is.na(object) & object >= thresh & row(object) > col(object), arr.ind=TRUE)
    if(length(wh)==0) {
        result <- data.frame(ind1=character(0),
                             ind2=character(0),
                             prop_match=numeric(0),
                             stringsAsFactors=FALSE)
        class(result) <- c("summary.comparegeno", "data.frame")
        return(result)
    }

    id <- rownames(object)
    if(is.null(id)) id <- as.character(seq_len(nrow(object)))


    # create results object
    result <- data.frame(ind1=id[wh[,2]],
                         ind2=id[wh[,1]],
                         prop_match=rep(0, nrow(wh)),
                         stringsAsFactors=FALSE)

    # fill in values with lower triangle
    for(i in seq_len(nrow(wh))) {
        result[i,3] <- object[wh[i,1], wh[i,2]]
    }


    # if both count and proportion is provided, determine number of markers
    if(what=="both") {
        result <- cbind(result,
                        n_markers=rep(0, nrow(wh)))

        for(i in seq_len(nrow(wh))) {
            result[i,4] <- round(object[wh[i,2], wh[i,1]]/object[wh[i,1], wh[i,2]])
        }
    }

    if(what=="number") { # change column name
        colnames(result)[3] <- "number_match"
    }

    # sort by decreasing percent matching
    result <- result[order(result[,3], decreasing=TRUE),,drop=FALSE]
    rownames(result) <- 1:nrow(result)

    class(result) <- c("summary.comparegeno", "data.frame")

    result

}


print.summary.comparegeno <-
    function(x, ...)
{
    if(nrow(x)==0) {
        cat("No pairs above threshold.\n")
    } else {
        print.data.frame(x, digits=3)
    }
}


plot.comparegeno <-
    function(x, breaks=NULL, main="", xlab="Proportion matching genotypes", ...)
{
    vals <- x[lower.tri(x)]

    if(is.null(breaks)) breaks <- 2*sqrt(length(vals))

    if(attr(x, "what")=="number" && missing(xlab)) {
        xlab <- "Number matching genotypes"
    }

    hist(vals, breaks=breaks, main=main, xlab=xlab, ...)
    rug(vals)
    invisible()
}

#####################################################################
#
# mqmscanall.R
#
# Copyright (c) 2009-2019, Danny Arends
#
# Modified by Pjotr Prins and Karl Broman
#
# first written Februari 2009
# last modified Dec 2019
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: mqmscanall
#           scanall
#
#
#####################################################################

mqmscanall <- function(cross, multicore=TRUE, n.clusters=1, batchsize=10,cofactors=NULL, ...) {
    scanall(cross=cross, multicore=multicore, n.clusters=n.clusters, batchsize=batchsize,cofactors=cofactors, ..., scanfunction=mqmscan)
}

scanall <- function(cross, scanfunction=scanone, multicore=TRUE, n.clusters=1, batchsize=10, FF=0,cofactors=NULL, ..., plot=FALSE, verbose=FALSE)
{
    if(missing(cross)){
        ourstop("No cross file. Please supply a valid cross object.")
    }
    crosstype <- crosstype(cross)
    if(!(crosstype == "f2" || crosstype == "bc" || crosstype == "riself"))
        stop("Currently only F2, BC, and selfed RIL crosses can be analyzed by MQM.")

    cross <- omit_x_chr(cross)

    start <- proc.time()
    n.pheno <- nphe(cross)
    if(verbose) {
        ourline()
        cat("Starting R/QTL multitrait analysis\n")
        cat("Number of phenotypes:",n.pheno,"\n")
        cat("Batchsize:",batchsize," & n.clusters:",n.clusters,"\n")
        ourline()
    }

    result <- NULL      #BATCH result variable
    res <- NULL			#GLOBAL result variable
    all.data <- cross

    bootstraps <- 1:n.pheno
    batches <- length(bootstraps) %/% batchsize
    last.batch.num <- length(bootstraps) %% batchsize

    if(last.batch.num > 0){
        batches = batches+1
    }

    #INIT TIME VARS
    SUM <- 0
    AVG <- 0
    LEFT <- 0

    #TEST FOR SNOW CAPABILITIES
    if(multicore && n.clusters >1) {
        updateParallelRNG(n.clusters)

        if(verbose) cat("INFO: Using ",n.clusters," Cores/CPU's/PC's for calculation.\n")
        for(x in 1:(batches)){
            start <- proc.time()
            if(verbose) cat("INFO: Starting with batch",x,"/",batches,"\n")
            if(x==batches && last.batch.num > 0){
                boots <- bootstraps[((batchsize*(x-1))+1):((batchsize*(x-1))+last.batch.num)]
            }else{
                boots <- bootstraps[((batchsize*(x-1))+1):(batchsize*(x-1)+batchsize)]
            }

            if(Sys.info()[1] == "Windows") { # Windows doesn't support mclapply, but it's faster if available
                cl <- makeCluster(n.clusters)
                on.exit(stopCluster(cl))
                result <- clusterApply(cl, boots, snowCoreALL, all.data=all.data, scanfunction=scanfunction, cofactors=cofactors,
                                       verbose=verbose, ...)
            }
            else {
                result <- mclapply(boots, snowCoreALL, all.data=all.data, scanfunction=scanfunction, cofactors=cofactors,
                                   verbose=verbose, mc.cores=n.clusters, ...)
            }

            if(plot){
                temp <- result
                class(temp) <- c(class(temp),"mqmmulti")
                mqmplot.multitrait(temp)
            }
            res <- c(res,result)
            end <- proc.time()
            SUM <- SUM + (end-start)[3]
            AVG <- SUM/x
            LEFT <- AVG*(batches-x)
            if(verbose) {
                cat("INFO: Done with batch",x,"/",batches,"\n")
                cat("INFO: Calculation of batch",x,"took:",round((end-start)[3], digits=3),"seconds\n")
                cat("INFO: Elapsed time:",(SUM%/%3600),":",(SUM%%3600)%/%60,":",round(SUM%%60, digits=0),"(Hour:Min:Sec)\n")
                cat("INFO: Average time per batch:",round((AVG), digits=3)," per trait:",round((AVG %/% batchsize), digits=3),"seconds\n")
                cat("INFO: Estimated time left:",LEFT%/%3600,":",(LEFT%%3600)%/%60,":",round(LEFT%%60,digits=0),"(Hour:Min:Sec)\n")
                ourline()
            }
        }
    }else{
        if(verbose) cat("INFO: Going into singlemode.\n")
        for(x in 1:(batches)){
            start <- proc.time()
            if(verbose) cat("INFO: Starting with batch",x,"/",batches,"\n")
            if(x==batches && last.batch.num > 0){
                boots <- bootstraps[((batchsize*(x-1))+1):((batchsize*(x-1))+last.batch.num)]
            }else{
                boots <- bootstraps[((batchsize*(x-1))+1):(batchsize*(x-1)+batchsize)]
            }
            result <- lapply(boots, FUN=snowCoreALL,all.data=all.data,scanfunction=scanfunction,cofactors=cofactors,verbose=verbose,...)
            if(plot){
                temp <- result
                class(temp) <- c(class(temp),"mqmmulti")
                mqmplot.multitrait(temp)
            }
            res <- c(res,result)
            end <- proc.time()
            SUM <- SUM + (end-start)[3]
            AVG <- SUM/x
            LEFT <- AVG*(batches-x)
            if(verbose) {
                cat("INFO: Done with batch",x,"/",batches,"\n")
                cat("INFO: Calculation of batch",x,"took:",round((end-start)[3], digits=3),"seconds\n")
                cat("INFO: Elapsed time:",(SUM%/%3600),":",(SUM%%3600)%/%60,":",round(SUM%%60, digits=0),"(Hour:Min:Sec)\n")
                cat("INFO: Average time per batch:",round((AVG), digits=3)," per trait:",round((AVG %/% batchsize), digits=3),"seconds\n")
                cat("INFO: Estimated time left:",LEFT%/%3600,":",(LEFT%%3600)%/%60,":",round(LEFT%%60,digits=0),"(Hour:Min:Sec)\n")
                ourline()
            }
        }
    }
    if(FF){
        if(verbose) cat(rownames(res[[1]]),"\n",res[[1]][,1],"\n",res[[1]][,2],"\n",file="out.frank")
        for(i in 1:length(res)){
            if(verbose) cat("INFO: Saving trait",i,"in frankformat\n")
            qtl <- res[[i]]
            if(verbose) cat(colnames(qtl)[3],qtl[,3],"\n",file="out.frank",append = TRUE)
        }
    }
    #Return the results
    if(length(res) > 1){
        class(res) <- c(class(res),"mqmmulti")
    }else{
        class(res) <- c(class(res),"scanone")
    }
    #All done now plot the results
    end <- proc.time()
    SUM <- SUM + (end-start)[3]
    AVG <- SUM/n.pheno
    if(verbose) {
        cat("------------------------------------------------------------------\n")
        cat("INFO: Elapsed time:",(SUM%/%3600),":",(SUM%%3600)%/%60,":",round(SUM%%60, digits=0),"(Hour:Min:Sec)\n")
        cat("INFO: Average time per trait:",round(AVG, digits=3),"seconds\n")
        cat("------------------------------------------------------------------\n")
    }
    res
}


# end of scanall.R

######################################################################
#
# fitstahl.R
#
# copyright (c) 2006-2019, Karl W Broman
# last modified Dec, 2019
# first written Aug, 2006
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: fitstahl, fitstahl.estp, fitstahl.este, fitstahl.estpe,
#           fitstahl.estp.sub, fitstahl.este.sub, fitstahl.estpe.sub
#
######################################################################

######################################################################
# fitstahl
#
# Fit the Stahl model for crossover interference (or the chi-square
# model, which is a special case)
#
# cross: the cross object
#
# chr:   Chromosome(s) to use; if unspecified, pooled estimates for
#        all chromosomes are obtained
#
# m:     Interference parameter (a non-negative integer); if unspecified,
#        this is estimated
#
# p:     The proportion of chiasmata coming from the no interference
#        mechanism in the Stahl model (0 <= p <= 1).  p=0 gives the
#        chi-square model.  If unspecified, this is estimated
#
# error.prob  The genotyping error probability.  If = NULL, it is
#             estimated
#
# NOTE: If m, p, error.prob are all specified, they can be vectors
#       or have length 1; any with length > 1 should all have the
#       same length
#
# maxit  Maximum number of iterations
#
# tol    Tolerance for convergence
#
# maxm   Maximum value of m to consider, if m is unspecified.
#
######################################################################

fitstahl <-
    function(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
             maxm=15, verbose=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    type <- crosstype(cross)
    if(type != "bc" && type != "f2")
        stop("fitstahl only working for backcrosses and intercrosses.")

    if(!missing(chr)) cross <- subset(cross, chr)

    if(!missing(error.prob) && any(error.prob > 0.5))
        warning("You probably want to use error.prob <= 0.5.")

    if(!missing(m)) { # m was specified
        if(!missing(p)) { # p was specified

            # m, p, error.prob all specified
            if(!is.null(error.prob)) {

                n <- c(length(m), length(p), length(error.prob))
                mn <- max(n)
                if(mn > 1) {
                    if(any(n > 1 & n < mn))
                        stop("Any m, p, error.prob with length > 1 must have same length")
                    if(length(m) == 1) m <- rep(m, mn)
                    if(length(p) == 1) p <- rep(p, mn)
                    if(length(error.prob) == 1) error.prob <- rep(error.prob, mn)
                }

                result <- matrix(ncol=4, nrow=mn)
                colnames(result) <- c("m", "p", "error.prob", "loglik")
                for(i in 1:mn) {
                    temp <- fitstahl.estp.sub(p[i], cross, error.prob[i],
                                              m[i], tol, maxit)
                    result[i,] <- c(m[i], p[i], error.prob[i], as.numeric(temp))
                    cross <- replace.map(cross, attr(temp, "map"))
                    if(verbose) cat(i,result[i,], "\n")
                }
            }

            # m,p specified but error.prob wasn't
            else {
                if(length(m) == 1)
                    m <- rep(m, length(p))
                else {
                    if(length(p) == 1)
                        p <- rep(p, length(m))
                    else if(length(m) != length(p))
                        stop("Any m, p, error.prob with length > 1 must have same length")
                }
                mn <- length(m)

                result <- matrix(ncol=4, nrow=mn)
                colnames(result) <- c("m", "p", "error.prob", "loglik")
                for(i in 1:mn) {
                    temp <- fitstahl.este(cross, m[i], p[i], tol, maxit)
                    cross <- replace.map(cross, attr(temp, "map"))
                    result[i,] <- c(m[i], p[i], temp$est, temp$loglik)
                    if(verbose) cat(i, result[i,], "\n")
                }
            }
        }
        else {
            # m, error.prob specified; p unspecified
            if(!is.null(error.prob)) {
                if(length(m) == 1)
                    m <- rep(m, length(error.prob))
                else {
                    if(length(error.prob) == 1)
                        error.prob <- rep(error.prob, length(m))
                    else if(length(m) != length(error.prob))
                        stop("Any m, p, error.prob with length > 1 must have same length")
                }
                mn <- length(m)

                result <- matrix(ncol=4, nrow=mn)
                colnames(result) <- c("m", "p", "error.prob", "loglik")
                for(i in 1:mn) {
                    temp <- fitstahl.estp(cross, error.prob[i], m[i], tol, maxit)
                    cross <- replace.map(cross, attr(temp, "map"))
                    result[i,] <- c(m[i], temp$est, error.prob[i], temp$loglik)
                    if(verbose) cat(i, result[i,], "\n")
                }
            }

            # only m specified
            else {
                result <- matrix(ncol=4, nrow=length(m))
                colnames(result) <- c("m", "p", "error.prob", "loglik")
                for(i in 1:length(m)) {
                    temp <- fitstahl.estpe(cross, m[i], tol, maxit)
                    cross <- replace.map(cross, attr(temp, "map"))
                    result[i,] <- c(m[i], temp$est, temp$loglik)
                    if(verbose) cat(i, result[i,], "\n")
                }
            }

        }
    }
    else { # m unspecified
        if(!missing(p)) { # p was specified

            # p, e specified
            if(!is.null(error.prob)) {
                if(length(p) == 1)
                    p <- rep(p, length(error.prob))
                else {
                    if(length(error.prob) == 1)
                        error.prob <- rep(error.prob, length(p))
                    else if(length(p) != length(error.prob))
                        stop("Any m, p, error.prob with length > 1 must have same length")
                }
                mn <- length(p)

                result <- matrix(ncol=4, nrow=mn)
                colnames(result) <- c("m", "p", "error.prob", "loglik")
                for(i in 1:mn) {
                    # fit the case m=0
                    maxll <- fitstahl.estp.sub(p[i], cross, error.prob[i], 0, tol, maxit)
                    cross <- replace.map(cross, attr(maxll, "map"))
                    themax <- 0
                    if(verbose) cat(i, 0, maxll, "\n")
                    for(j in 1:maxm) {
                        curll <- fitstahl.estp.sub(p[i], cross, error.prob[i], j, tol, maxit)
                        cross <- replace.map(cross, attr(curll, "map"))
                        if(verbose) cat(i, j, curll, "\n")
                        if(curll < maxll) break
                        if(curll > maxll) {
                            maxll <- curll
                            themax <- j
                        }
                    }
                    result[i,] <- c(themax, p[i], error.prob[i], maxll)
                    if(verbose) cat(i, result[i,], "\n")
                }
            }

            # only p specified
            else {
                mn <- length(p)

                result <- matrix(ncol=4, nrow=mn)
                colnames(result) <- c("m", "p", "error.prob", "loglik")
                for(i in 1:mn) {
                    # fit the case m=0
                    temp <- fitstahl.este(cross, 0, p[i], tol, maxit)
                    cross <- replace.map(cross, attr(temp, "map"))
                    maxll <- temp$loglik
                    themax <- 0
                    themaxe <- temp$est
                    if(verbose) cat(i, 0, 0, temp$est, maxll, "\n")
                    for(j in 1:maxm) {
                        temp <- fitstahl.este(cross, j, p[i], tol, maxit)
                        cross <- replace.map(cross, attr(temp, "map"))
                        curll <- temp$loglik
                        if(verbose) cat(i, j, p[i], temp$est, curll, "\n")
                        if(curll < maxll) break
                        if(curll > maxll) {
                            maxll <- curll
                            themax <- j
                            themaxe <- temp$est
                        }
                    }
                    result[i,] <- c(themax, p[i], themaxe, maxll)
                    if(verbose) cat(i, result[i,], "\n")
                }
            }
        }
        else {
            if(!is.null(error.prob)) { # error.prob specified; p unspecified
                mn <- length(error.prob)

                result <- matrix(ncol=4, nrow=mn)
                colnames(result) <- c("m", "p", "error.prob", "loglik")
                for(i in 1:mn) {
                    # fit the case m=0 (in which case p doesn't matter)
                    maxll <- fitstahl.estp.sub(0, cross, error.prob[i], 0, tol, maxit)
                    cross <- replace.map(cross, attr(maxll, "map"))
                    themax <- 0
                    themaxp <- 0
                    if(verbose) cat(i, 0, 0, error.prob[i], maxll, "\n")
                    for(j in 1:maxm) {
                        temp <- fitstahl.estp(cross, error.prob[i], j, tol, maxit)
                        cross <- replace.map(cross, attr(temp, "map"))
                        curll <- temp$loglik
                        if(verbose) cat(i, j, temp$est, error.prob[i], curll, "\n")
                        if(curll < maxll) break
                        if(curll > maxll) {
                            maxll <- curll
                            themax <- j
                            themaxp <- temp$est
                        }
                    }
                    result[i,] <- c(themax, themaxp, error.prob[i], maxll)
                    if(verbose) cat(i, result[i,], "\n")
                }
            }

            else { # nothing specified

                result <- matrix(ncol=4, nrow=1)
                colnames(result) <- c("m", "p", "error.prob", "loglik")
                # fit the case m=0 (in which case p doesn't matter)
                temp <- fitstahl.este(cross, 0, 0, tol, maxit)
                cross <- replace.map(cross, attr(temp, "map"))
                maxll <- temp$loglik
                themax <- 0
                themaxest <- c(0, temp$est)
                if(verbose) cat(0, 0, temp$est, maxll, "\n")
                for(j in 1:maxm) {
                    temp <- fitstahl.estpe(cross, j, tol, maxit)
                    cross <- replace.map(cross, attr(temp, "map"))
                    curll <- temp$loglik
                    if(verbose) cat(j, temp$est, curll, "\n")
                    if(curll < maxll) break
                    if(curll > maxll) {
                        maxll <- curll
                        themax <- j
                        themaxest <- temp$est
                    }
                }
                result[1,] <- c(themax, themaxest, maxll)
                if(verbose) cat(1, result[1,], "\n")
            }
        }
    }
    as.data.frame(result, stringsAsFactors=TRUE)
}



######################################################################
# fitstahl.estp:  estimate p for fixed m and error.prob
######################################################################
fitstahl.estp <-
    function(cross, error.prob=0.0001, m=0, tol=1e-4, maxit=4000)
{
    out <- optimize(fitstahl.estp.sub, interval=c(0,1), maximum=TRUE,
                    cross=cross, m=m, error.prob=error.prob, thetol=tol,
                    maxit=maxit)

    # make sure we fit p=0
    temp <- fitstahl.estp.sub(0, cross, error.prob, m, tol, maxit)

    if(temp >= out[[2]]) {
        est <- 0
        loglik <- temp
        themap <- attr(temp, "map")
    }
    else {
        est <- out[[1]]
        loglik <- out[[2]]
        themap <- attr(out[[2]], "map")
    }
    out <- list(est=est, loglik=loglik)
    attr(out, "map") <- themap
    out
}

fitstahl.estp.sub <-
    function(p, cross, error.prob=0.0001, m=0, thetol=1e-4, maxit=4000)
{
    newmap <- est.map(cross, error.prob=error.prob, m=m, p=p, tol=thetol,
                      maxit=maxit)
    out <- sum(sapply(newmap, function(a) attr(a, "loglik")))

    attr(out, "map") <- newmap
    out
}

######################################################################
# fitstahl.este: estimate error.prob for fixed m and p
######################################################################
fitstahl.este <-
    function(cross, m=0, p=0, tol=1e-4, maxit=4000)
{
    out <- optimize(fitstahl.este.sub, interval=c(0,0.5), maximum=TRUE,
                    cross=cross, m=m, p=p, thetol=tol, maxit=maxit)

    # make sure we fit error.prob=0
    temp <- fitstahl.este.sub(0, cross, m, p, tol, maxit)
    if(temp >= out[[2]]) {
        est <- 0
        loglik <- temp
        themap <- attr(temp, "map")
    }
    else {
        est <- out[[1]]
        loglik <- out[[2]]
        themap <- attr(out[[2]], "map")
    }

    out <- list(est=est, loglik=loglik)
    attr(out, "map") <- themap
    out
}

fitstahl.este.sub <-
    function(error.prob, cross, m=0, p=0, thetol=1e-4, maxit=4000)
{
    newmap <- est.map(cross, error.prob=error.prob, m=m, p=p, tol=thetol,
                      maxit=maxit)
    out <- sum(sapply(newmap, function(a) attr(a, "loglik")))
    attr(out, "map") <- newmap
    out
}


######################################################################
# fitstahl.estpe: estimate p and error.prob for fixed m
######################################################################
fitstahl.estpe <-
    function(cross, m=0, tol=1e-4, maxit=4000)
{
    if(m==0) { # don't need to estimate p
        out <- fitstahl.este(cross, 0, 0, tol, maxit)
        out$est <- c(0, out$est)
        return(out)
    }

    out <- optim(c(0.1,0.01), fitstahl.estpe.sub, method="L-BFGS-B",
                 lower=c(0,0), upper=c(1,0.5),
                 control=list(fnscale=-1, maxit=maxit, factr=tol/1e-15),
                 cross=cross, m=m, thetol=tol, maxit=maxit)

    if(out$convergence !=0)
        warning(" Didn't converge.")

    # make sure we fit the case p=0
    temp <- fitstahl.este(cross, m, 0, tol, maxit)
    templl <- temp$loglik
    if(templl >= out$value) {
        est <- c(0, temp$est)
        loglik <- templl
        themap <- attr(temp, "map")
    }
    else {
        est <- out$par
        loglik <- out$value
        themap <- est.map(cross, error.prob=out$par[2], m=m, p=out$par[1],
                          maxit=maxit, tol=tol)
    }
    names(est) <- c("p", "error.prob")
    out <- list(est=est, loglik=loglik)
    attr(out, "map") <- themap
    out
}

fitstahl.estpe.sub <-
    function(x, cross, m=0, thetol=1e-4, maxit=4000)
{
    newmap <- est.map(cross, error.prob=x[2], m=m, p=x[1], tol=thetol,
                      maxit=maxit)
    out <- sum(sapply(newmap, function(a) attr(a, "loglik")))
    attr(out, "map") <- newmap
    out
}

# end of fitstahl.R

######################################################################
#
# read.cross.qtx.R
#
# copyright (c) 2000-2011, Karl W Broman
# last modified May, 2011
# first written Aug, 2000
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.qtx
#           [See read.cross.R for the main read.cross function.]
#
######################################################################

######################################################################
#
# read.cross.qtx
#
# read data in Map Manager QTX format
#
######################################################################

read.cross.qtx <-
    function(dir, file, estimate.map=TRUE)
{
    if(!missing(dir) && dir != "") {
        file <- file.path(dir, file)
    }


    # This is a revised version of match which gives *all* matches
    # of x within the table
    mymatch <-
        function(x, table)
        {
            if(length(x) > 1) x <- x[1] # ignore any but the first element of x

            if(!any(x==table)) return(NA)
            seq(along=table)[x==table]
        }

    # read file into a big vector, each item one line
    cat(" --Read the following data:\n")
    x <- scan(file,what=character(0),sep="\n",quiet=TRUE)

    genoabbrev <- unlist(strsplit(x[9],""))
    if(length(genoabbrev) < 8)  # just in case, fill out to 8 chars
        genoabbrev <- c(genoabbrev,rep("H",8-length(genoabbrev)))
    myabbrev <- c(0,1,3,2,5,4,2,2)
    ugeno <- NULL

    # individuals
    ind.beg <- match("{pgy", x) # there should be just one
    ind.end <- match("}pgy", x)
    n.ind <- as.numeric(x[ind.beg+1])
    ind <- x[(ind.beg+2):(ind.end-1)]
    if(length(ind) != n.ind)
        stop("Problem with individual IDs ({pgy}).")
    cat("\t", n.ind, " individuals\n", sep="")

    # determine if individuals can be viewed as numbers
    g <- grep("^[0-9\\.]+$", ind)
    if(length(g) == n.ind)
        ind <- as.numeric(as.character(ind))

    # phenotypes
    phe.beg <- mymatch("{trt",x)
    phe.end <- mymatch("}trt",x)
    pheno <- NULL
    if(!is.na(phe.beg[1])) { # at least one phenotype
        pheno <- vector("list",length(phe.beg))
        names(pheno) <- paste(phe.beg)
        for(i in 1:length(phe.beg)) {
            z <- x[phe.beg[i]:phe.end[i]]
            names(pheno)[i] <- z[2]
            vals.beg <- match("{tvl", z)+1 # there should be just one match
            vals.end <- match("}tvl", z)-1

            # "X" or "x" is a missing phenotype
            temp <- unlist(strsplit(z[vals.beg[1]:vals.end[1]]," "))
            temp[temp=="X" | temp=="x"] <- NA
            pheno[[i]] <- as.numeric(temp)
        }
        pheno <- cbind(as.data.frame(pheno, stringsAsFactors=TRUE),ind=ind)
        cat("\t", length(pheno), " phenotypes\n",sep="")
    }
    else {
        pheno <- data.frame(ind=ind, stringsAsFactors=TRUE)
        cat("\t", 0, "  phenotypes\n",sep="")
    }

    # chromosomes
    chr.beg <- mymatch("{chx",x)
    chr.end <- mymatch("}chx",x)

    if(is.na(chr.beg[1])) # no genotype data
        stop("There appears to be no genotype data!")
    geno <- vector("list", length(chr.beg))
    names(geno) <- paste(chr.beg)
    has.loci <- rep(TRUE,length(chr.beg))
    map.offset <- rep(0,length(chr.beg))

    cat("\t", length(chr.beg), "  chromosomes\n",sep="")

    for(i in 1:length(chr.beg)) {
        z <- x[chr.beg[i]:chr.end[i]]
        names(geno)[i] <- z[2]
        map.offset <- as.numeric(z[5])

        # loci
        loc.beg <- mymatch("{lox",z)
        loc.end <- mymatch("}lox",z)
        if(all(is.na(loc.beg))) {
            has.loci[i] <- FALSE
            next
        }
        data <- matrix(ncol=length(loc.beg),nrow=n.ind)
        loctype <- rep(NA,length(loc.beg)) ####
        colnames(data) <- paste(loc.beg)
        has.geno <- rep(TRUE,length(loc.beg))
        for(j in 1:length(loc.beg)) {
            zz <- z[loc.beg[j]:loc.end[j]]
            colnames(data)[j] <- zz[2]
            loctype[j] <- zz[5] ####
            geno.beg <- match("{sdp",zz)+1 # should be just one match
            geno.end <- match("}sdp",zz)-1
            if(all(is.na(geno.beg))) { # no genotype data
                has.geno[j] <- FALSE
                next
            }
            dat <- unlist(strsplit(paste(zz[geno.beg[1]:geno.end[1]],collapse=""),""))

            data[,j] <- myabbrev[match(dat,genoabbrev)]
        } # end loop over loci

        # check that all loci have the same code
        if(all(loctype == loctype[1]))
            loctype <- loctype[1]
        # 0 = unknown
        # 1 = backcross codominant maternal unique
        # 2 = backcross codominant paternal unique
        # 3 = backcross maternal dominant
        # 4 = backcross paternal dominant
        # 5 = f2 codominant
        # 6 = f2 maternal dominant
        # 7 = f2 paternal dominant
        # 8 = doubled haploid
        # 9 = selfed RI
        # 10 = sib-mated RI
        # 11 = advanced backcross codominant maternal unique
        # 12 = advanced backcross codominant paternal udnique
        # 13 = advanced backcross maternal dominant
        # 14 = advanced backcross paternal dominant
        # 15 = AIL codominant
        # 16 = AIL maternal dominant
        # 17 = AIL paternal dominant
        # 18 = radiation hybrid data
        # 19 = radiation hybrid data
        # 20 = selfed RIX
        # 21 = sib-mated RIX

        # replace 0's with NA's
        data[!is.na(data) & data==0] <- NA

        # remove columns with no data
        data <- data[,has.geno,drop=FALSE]

        # temporary map
        map <- seq(0,length=ncol(data),by=5)+map.offset
        names(map) <- colnames(data)
        geno[[i]] <- list(data=data,map=map)
        if(length(grep("[Xx]", names(geno)[i]))>0) # X chromosome
            class(geno[[i]]) <- "X"
        else class(geno[[i]]) <- "A"
    } # end loop over chromosomes

    # unique genotypes
    for(i in 1:length(geno)) {
        ugeno <- unique(c(ugeno,unique(geno[[i]]$data)))
        ugeno <- ugeno[!is.na(ugeno)]
    }

    if(length(ugeno)==2) { # backcross
        # Fix if coded as A:B rather than A:H (RI lines)
        if(all(ugeno==1 | ugeno==3)) {
            for(i in 1:length(geno))
                geno[[i]]$data[geno[[i]]$data == 3] <- 2
        }
        # Fix if coded as H:B rather than A:H (other backcross)
        else if(all(ugeno==2 | ugeno==3)) {
            for(i in 1:length(geno))
                geno[[i]]$data[geno[[i]]$data == 3] <- 1
        }

        type <- "bc"
        for(i in 1:length(geno))
            geno[[i]]$data[geno[[i]]$data > 2] <- 1
    }
    else type <- "f2"

    totmar <- sum(sapply(geno,function(a) ncol(a$data)))
    cat("\t", totmar, "  total markers\n",sep="")

    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c(type,"cross")

    if(estimate.map) estmap <- TRUE
    else estmap <- FALSE

    # return cross + indicator of whether to run est.map
    list(cross,estmap)
}

# end of read.cross.qtx.R

######################################################################
#
# effectplot.R
#
# copyright (c) 2002-2019, Hao Wu and Karl W. Broman
# Last modified Dec, 2019
# first written Jul, 2002
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Modified by Hao Wu Feb 2005 for the following:
# 1. function will take marker, pseudomarker or phenotype as input;
# 2. separate functions to extract marker genodata given marker names
# and calculate means and ses;
#
# Part of the R/qtl package
# Contains: effectplot, effectplot.getmark, effectplot.calmeanse
#
######################################################################

effectplot <-
    function (cross, pheno.col = 1, mname1, mark1, geno1, mname2,
              mark2, geno2, main, ylim, xlab, ylab, col, add.legend = TRUE,
              legend.lab, draw=TRUE, var.flag=c("pooled","group"))
{
    if(!inherits(cross, "cross"))
        stop("The first input variable must be an object of class cross")

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    # if mname2 is given but not mname1, switch it around
    if((missing(mname1) && missing(mark1) && missing(geno1)) &&
       any(!missing(mname2) || missing(mark2) || missing(geno2))) {
        if(!missing(mname2)) { mname1 <- mname2; mname2 <- NULL }
        if(!missing(mark2)) { mark1 <- mark2; mark2 <- NULL }
        if(!missing(geno2)) { geno1 <- geno2; geno2 <- NULL }
    }
    if(missing(mark2)) mark2 <- NULL
    if(missing(mname2)) mname2 <- NULL

    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("effectplot can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 | pheno.col > nphe(cross))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    if(!is.numeric(cross$pheno[,pheno.col]))
        stop("phenotype \"", colnames(cross$pheno)[pheno.col], "\" is not numeric.")

    var.flag <- match.arg(var.flag)

    # local variables
    n.ind <- nind(cross)
    pheno <- cross$pheno[, pheno.col]
    type <- crosstype(cross)
    chr_type1 <- chr_type2 <- "A"
    gennames1 <- gennames2 <- NULL

    # If imputations are not available, create them
    if(!("draws" %in% names(cross$geno[[1]]))) {
        warning(" -Running sim.geno.")
        cross <- sim.geno(cross, n.draws=16)
    }

    ####################################################
    # get genotype data for markers given marker name
    # if marker genodata were given, this will be skipped
    ####################################################

    # used for alternative print name for pseudomarkers
    pm.pattern <- "^c.*\\.loc.*$" # pseudomarker names will be like "c1.loc10"
    dig <- 1
    step <- attr(cross$geno[[1]]$draws, "step")
    if(!is.null(step)) {
        if(step > 0) dig <- max(dig, -floor(log10(step)))
    }
    else {
        stepw <- attr(cross$geno[[1]]$draws, "stepwidth")
        if(!is.null(stepw) && stepw > 0) dig <- max(dig, -floor(log10(stepw)))
    }

    printname1 <- printname2 <- NULL
    # Get marker 1 genotype data
    if(missing(mark1)) { # no data given
        if(missing(mname1)) # no marker data or marker name, have to stop
            stop("Either mname1 or mark1 must be specified.")
        # get marker data according to marker name
        tmp <- effectplot.getmark(cross, mname1)
        tmptmp <- attr(tmp, "mname")
        if(!is.null(tmptmp)) mname1 <- tmptmp
        mark1 <- tmp$mark
        gennames1 <- tmp$genname

        # perhaps alternative print name
        if(length(grep(pm.pattern, mname1))>0) {
            tmp <- find.pseudomarkerpos(cross, mname1, "draws")
            printname1 <- paste(tmp[1,1], charround(tmp[1,2],dig), sep="@")
        }
    }
    else {
        # make mark1 a matrix if it's not
        if(!is.matrix(mark1))
            mark1 <- matrix(mark1, ncol=1)
        if(dim(mark1)[1] != n.ind)
            stop("Marker 1 data has the wrong dimension")
        if(missing(mname1))
            mname1 <- "Marker 1"
    }
    if(is.null(printname1)) printname1 <- mname1

    # Deal with marker 2
    if(!is.null(mname2) || !is.null(mark2)) {
        if(is.null(mark2)) {
            # get marker data according to marker name
            tmp <- effectplot.getmark(cross, mname2)
            tmptmp <- attr(tmp, "mname")
            if(!is.null(tmptmp)) mname2 <- tmptmp
            mark2 <- tmp$mark
            gennames2 <- tmp$genname

            # perhaps alternative print name
            if(length(grep(pm.pattern, mname2))>0) {
                tmp <- find.pseudomarkerpos(cross, mname2, "draws")
                printname2 <- paste(tmp[1,1], charround(tmp[1,2],dig), sep="@")
            }
        }
        else { # mark2 data is given
            # make mark2 a matrix if it's not
            if(!is.matrix(mark2))
                mark2 <- matrix(mark2, ncol=1)
            if(dim(mark2)[1] != n.ind)
                stop("Marker 2 data has the wrong dimension")
            if(is.null(mname2))
                mname2 <- "Marker 2"
        }
        if(is.null(printname2)) printname2 <- mname2
    }
    else {
        mark2 <- NULL
        geno2 <- NULL
    }
    ### till now, mark1 and mark2 are genotype data in matrix

    ########################################################
    # deal with the data - if one of them is a pseudomarker,
    # make the other one the same number of draws
    ########################################################
    # determine number of draws - this part of codes works even if mark2 is NULL
    ndraws1 <- dim(mark1)[2]
    if(is.null(mark2))
        ndraws2 <- 1
    else
        ndraws2 <- dim(mark2)[2]

    # make them the same number of draws
    if( (ndraws1>1) && (ndraws2>1) ) {
        # two pseudomarkers, they must have the same number of draws
        if(ndraws1 != ndraws2)
            stop("Input two pseudomarkers have different number of draws.")
        else
            ndraws <- ndraws1
    }
    else if( (ndraws1>1) && (ndraws2==1) ) {
        # one pm and one typed marker
        if(!is.null(mark2))
            mark2 <- matrix(rep(mark2,ndraws1), ncol=ndraws1)
        ndraws <- ndraws1
    }
    else if( (ndraws1==1) && (ndraws2>1) ) {
        # one pm and one typed marker
        mark1 <- matrix(rep(mark1,ndraws2), ncol=ndraws2)
        ndraws <- ndraws2
    }
    else # they are all real markers
        ndraws <- 1

    # drop data for individuals with missing phenotypes or genotypes
    keepind <- !is.na(pheno)
    if(!is.null(mark1))
        keepind <- keepind & apply(mark1, 1, function(a) all(!is.na(a)))
    if(!is.null(mark2))
        keepind <- keepind & apply(mark2, 1, function(a) all(!is.na(a)))

    mark1 <- mark1[keepind,]
    mark2 <- mark2[keepind,]
    pheno <- pheno[keepind]

    ########################################################
    # 1. get level names - this part will be executed when
    # user only input mark without mname and geno
    # 2. adjust marker data if the input is not numeric
    ########################################################
    tmpf <- factor(mark1)
    if(!missing(geno1)) { # geno1 is given
        # check if it has the correct length
        if(length(geno1) < length(levels(tmpf)))
            stop("geno1 is too short.")
    }
    else {
        # geno1 is not given
        if(!is.null(gennames1)) # get it from genname1
            geno1 <- gennames1
        else if(is.factor(mark1)) { # or if it's factor, get it from level
            geno1 <- levels(mark1)
            mark1 <- as.numeric(mark1)
        }
        else if(!is.numeric(mark1)) {
            # if it's neither factor nor numeric, it must be a string vector
            # such like c("F","M","F")...
            geno1 <- levels(tmpf)
        }
        else { # otherwise, generate a standard one
            geno1 <- getgenonames(type, "A", cross.attr=attributes(cross))
            if(length(levels(tmpf)) > length(geno1))
                geno1 <- c(geno1, rep("?", length(levels(tmpf)) -
                                      length(geno1)))
        }
    }
    # adjust marker data - if the input is not numeric, convert them into numeric
    if(!is.numeric(mark1))
        mark1 <- matrix(as.numeric(tmpf, levels=sort(levels(tmpf))), ncol=ndraws)

    # Now work on mark2
    if(!is.null(mark2)) {
        tmpf <- factor(mark2)
        if(!missing(geno2)) { # geno2 is given
            # check if it has the correct length
            if(length(geno2) < length(levels(tmpf)))
                stop("geno2 is too short.")
        }
        else {
            # geno2 is not given
            if(!is.null(gennames2)) # get it from genname2
                geno2 <- gennames2
            else if(is.factor(mark2)) { # or if it's factor, get it from level
                geno2 <- levels(mark2)
                mark2 <- as.numeric(mark2)
            }
            else if(!is.numeric(mark2)) {
                # if it's neither factor nor numberic, it must be a string vector
                # such like c("F","M","F")...
                geno2 <- levels(tmpf)
            }
            else { # otherwise, generate a standard one
                geno2 <- getgenonames(type, "A", cross.attr=attributes(cross))
                if(length(levels(tmpf)) > length(geno2))
                    geno2 <- c(geno2, rep("?", length(levels(tmpf)) -
                                          length(geno2)))
            }
        }
        # adjust marker data - if the input is not numeric, convert them into numeric
        if(!is.numeric(mark2))
            mark2 <- matrix(as.numeric(tmpf, levels=sort(levels(tmpf))), ncol=ndraws)
    }

    # number of genotypes
    ngen1 <- length(geno1)
    if(!is.null(mark2))
        ngen2 <- length(geno2)

    # calculate means and ses
    # and make output object
    # the output will be a data frame. For two-marker case,
    # the rows corresponding to the first marker and the columns
    # corresponding to the second marker
    result <- effectplot.calmeanse(pheno, mark1, mark2, geno1, geno2, ndraws, var.flag)
    means <- result$Means
    ses <- result$SEs

    # assign column and row names
    if(is.null(mark2)) {
        if(length(means) != length(geno1)) {
            warning("Number of genotypes is different than length(geno1).")
            if(length(means) < length(geno1))
                geno1 <- geno1[1:length(means)]
            else geno1 <- c(geno1, rep("?", length(means) - length(geno1)))
            ngen1 <- length(geno1)
        }
        names(result$Means) <- paste(printname1, geno1, sep = ".")
        names(result$SEs) <- paste(printname1, geno1, sep = ".")
    }
    else {
        if(nrow(means) != length(geno1)) {
            warning("Number of genotypes in marker 1 is different than length(geno1).")
            if(nrow(means) < length(geno1))
                geno1 <- geno1[1:nrow(means)]
            else geno1 <- c(geno1, rep("?", nrow(means) - length(geno1)))
            ngen1 <- length(geno1)
        }
        if(ncol(means) != length(geno2)) {
            warning("Number of genotypes in marker 2 is different than length(geno2).")
            if(ncol(means) < length(geno2))
                geno2 <- geno2[1:ncol(means)]
            else geno2 <- c(geno2, rep("?", ncol(means) - length(geno2)))
            ngen2 <- length(geno2)
        }
        rownames(result$Means) <- paste(printname1, geno1, sep = ".")
        colnames(result$Means) <- paste(printname2, geno2, sep = ".")
        rownames(result$SEs) <- paste(printname1, geno1, sep = ".")
        colnames(result$SEs) <- paste(printname2, geno2, sep = ".")
    }

    # calculate lo's and hi's for plot
    lo <- means - ses
    hi <- means + ses

    ######### Draw the figure if requested ############
    if(draw) {
        # graphics parameters
        old.xpd <- par("xpd")
        old.las <- par("las")
        par(xpd = FALSE, las = 1)
        on.exit(par(xpd = old.xpd, las = old.las))

        # colors (for case of two markers)
        if(missing(col)) {
            if(ngen1 <= 5) {
                if(ngen1 == 1) int.color <- "black"
                else if(ngen1 == 2) int.color <- c("red", "blue")
                else int.color <- c("black", "red", "blue", "orange", "green")[1:ngen1]
            }
            else
                int.color <- c("black", rainbow(ngen1-1, start=0, end=2/3))
        }
        else int.color <- col

        # plot title
        if(missing(main)) {
            if(is.null(mark2))
                main <- paste("Effect plot for", printname1)
            else main <- paste("Interaction plot for", printname1, "and",
                               printname2)
        }

        # y axis limits
        if(missing(ylim)) {
            ylimits <- range(c(lo, means, hi), na.rm = TRUE)
            ylimits[2] <- ylimits[2] + diff(ylimits) * 0.1
        }
        else ylimits <- ylim

        # x axis limits
        if(is.null(mark2)) { # one marker
            u <- seq(along=geno1)
            d <- diff(u[1:2])
            xlimits <- c(min(u) - d/4, max(u) + d/4)
        }
        else { # two markers
            u <- seq(along=geno2)
            d <- diff(u[1:2])
            xlimits <- c(min(u) - d/4, max(u) + d/4)
        }

        ## fix of x limits
        d <- 1
        xlimits <- c(1 - d/4, length(u) + d/4)

        if(is.null(mark2)) { # single marker
            if(missing(xlab)) xlab <- printname1
            if(missing(ylab)) ylab <- names(cross$pheno)[pheno.col]
            if(missing(col)) col <- "black"

            # plot the means
            plot(1:ngen1, means, main = main, xlab = xlab, ylab = ylab,
                 pch = 1, col = col[1], ylim = ylimits, xaxt = "n",
                 type = "b", xlim = xlimits)
            # confidence limits
            for(i in 1:ngen1) {
                if(!is.na(lo[i]) && !is.na(hi[i]))
                    lines(c(i, i), c(lo[i], hi[i]), pch = 3, col = col[1],
                          type = "b", lty = 3)
            }

            # X-axis ticks
            a <- par("usr")
            ystart <- a[3]
            yend <- ystart - diff(a[3:4]) * 0.02
            ytext <- ystart - diff(a[3:4]) * 0.05
            #      for(i in 1:ngen1) {
            #        lines(x = c(i, i), y = c(ystart, yend), xpd = TRUE)
            #        text(i, ytext, geno1[i], xpd = TRUE)
            #      }
            axis(side=1, at=1:ngen1, labels=geno1)
        }
        else { # two markers
            if(missing(xlab)) xlab <- printname2
            if(missing(ylab)) ylab <- names(cross$pheno)[pheno.col]
            # plot the first genotype of marker 1
            plot(1:ngen2, means[1, ], main = main, xlab = xlab,
                 ylab = ylab, pch = 1, col = int.color[1],
                 ylim = ylimits, xaxt = "n", type = "b", xlim = xlimits)
            # confidence limits
            for(i in 1:ngen2) {
                if(!is.na(lo[1, i]) && !is.na(hi[1, i]))
                    lines(c(i, i), c(lo[1, i], hi[1, i]), pch = 3,
                          col = int.color[1], type = "b", lty = 3)
            }
            for(j in 2:ngen1) { # for the rest of genotypes for Marker 1
                lines(1:ngen2, means[j, ], col = int.color[j], pch = 1,
                      type = "b")
                # confidence limits
                for(i in 1:ngen2) {
                    if(!is.na(lo[j, i]) && !is.na(hi[j, i]))
                        lines(c(i, i), c(lo[j, i], hi[j, i]), pch = 3,
                              col = int.color[j], type = "b", lty = 3)
                }
            }

            # draw X-axis ticks
            a <- par("usr")
            ystart <- a[3]
            yend <- ystart - diff(a[3:4]) * 0.02
            ytext <- ystart - diff(a[3:4]) * 0.05
            #      for(i in 1:ngen2) {
            #        lines(x = c(i, i), y = c(ystart, yend), xpd = TRUE)
            #        text(i, ytext, geno2[i], xpd = TRUE)
            #      }
            axis(side=1, at=1:ngen2, labels=geno2)

            # add legend
            if(add.legend) {
                col <- int.color[1:ngen1]
                # legend position
                x.leg <- a[1]*0.25+a[2]*0.75
                y.leg <- a[4] - diff(a[3:4]) * 0.05
                y.leg2 <- a[4] - diff(a[3:4]) * 0.03
                legend(x.leg, y.leg, geno1, lty = 1, pch = 1, col = col,
                       cex = 1, xjust = 0.5)
                if(missing(legend.lab)) legend.lab <- printname1
                text(x.leg, y.leg2, legend.lab)
            }
        }
    }

    return(invisible(result))
}

##############################################
# function to get genotype data for a marker
# given marker name
##############################################

effectplot.getmark <-
    function (cross, mname)
{
    # cross type
    type <- crosstype(cross)
    # return variables
    mark <- NULL
    gennames <- NULL

    # for pseudomarkers refered to as "1@10.5":
    #    - check that it is not a phenotype or marker name
    #    - otherwise convert to the usual name via find.pseudomarker
    pmalt.pattern <- "@-*[0-9]+" # alternate way to refer to a pseudomarker ("1@10.5")
    if(length(grep(pmalt.pattern, mname))>0 &&
       !(mname %in% names(cross$pheno) || mname %in% colnames(pull.geno(cross)))) {
        ss <- unlist(strsplit(mname, "@"))
        if(!(ss[1] %in% names(cross$geno)))
            stop("Don't understand the marker name ", mname)
        mname <- find.pseudomarker(cross, ss[1], as.numeric(ss[2]), "draws")
    }

    # determine marker type - it could be a marker, a pseudomarker or a phenotype
    mar.type <- "none"
    # regular expression pattern for a pseudomarker names
    pm.pattern <- "^c.*\\.loc.*$" # pseudomarker names will be like "c1.loc10"
    if(mname %in% names(cross$pheno)) { # this is a phenotype
        mar.type <- "pheno"
        idx.pos <- which(mname==names(cross$pheno))
    }
    else if(length(grep(pm.pattern, mname)) > 0) { # like "c1.loc10", this is a pseudomarker
        # note that the column names for draws is like "loc10",
        # so I need to take the part after "." in mname
        tmp <- unlist(strsplit(mname, "loc"))
        chr <- substr(tmp[1],2,nchar(tmp[1])-1) # this will be like 1 or "X"
        if( !(chr %in% names(cross$geno)) )
            stop("Couldn't find marker ", mname)
        mar.type <- "pm"
        chr_type <- chrtype(cross$geno[[chr]])
        pm.name <- paste("loc", tmp[2],sep="") # this will be like loc10
        idx.pos <- which(pm.name==colnames(cross$geno[[chr]]$draws))
        if(length(idx.pos) == 0)
            stop("Couldn't find marker ", mname)
        else if(length(idx.pos)>1) # take the first one for multiple markers with the same name
            idx.pos <- idx.pos[1]
    }
    else { # this is a real marker name but it could be a observed or imputed
        for(i in 1:length(cross$geno)) {
            if(mname %in% colnames(cross$geno[[i]]$draws)) { # this is a pseudomarker
                mar.type <- "pm"
                chr <- i
                chr_type <- chrtype(cross$geno[[chr]])
                idx.pos <- which(mname == colnames(cross$geno[[i]]$draws))
                break
            }
            else if(mname %in% colnames(cross$geno[[i]]$data)) { # this is a typed marker
                mar.type <- "marker"
                chr <- i
                chr_type <- chrtype(cross$geno[[i]])
                idx.pos <- which(mname == colnames(cross$geno[[i]]$data))
                break
            }
        }
    }

    # if didn't find this marker
    if(mar.type == "none")
        stop("Marker ", mname, " not found")

    # get data from typed marker, pseudomarker or phenotype
    if(mar.type == "pheno") { # this is a phenotype
        mark <- cross$pheno[,idx.pos]
        # the phenotype need to be categorical
        if(length(unique(mark)) > 5) { # I'm using arbitrary number here
            stop("The input phenotype ", mname, " is not a categorical trait")
        }
        gennames <- sort(unique(mark))
    }
    else if(mar.type=="marker") { # this is a real marker
        mark <- cross$geno[[chr]]$data[, idx.pos]
        # if X chr and backcross or intercross, get sex/dir data + revise data
        if(chr_type == "X" && (type %in% c("bc","f2","bcsft"))) {
            sexpgm <- getsex(cross)
            mark <- as.numeric(reviseXdata(type, "full", sexpgm,
                                           geno = as.matrix(mark),
                                           cross.attr=attributes(cross)))
            gennames <- getgenonames(type, chr_type, "full", sexpgm, attributes(cross))
        }
    }

    else if(mar.type=="pm") { # this is a pseudomarker
        # get the imputed genotype data for this marker
        mark <- cross$geno[[chr]]$draws[,idx.pos,,drop=FALSE]

        # if X chr and backcross or intercross, get sex/dir data + revise data
        if(chr_type == "X" && (type %in% c("bc","f2","bcsft"))) {
            sexpgm <- getsex(cross)
            mark <- reviseXdata(type, "full", sexpgm, draws=mark,
                                cross.attr=attributes(cross))[,1,]
            gennames <- getgenonames(type, chr_type, "full", sexpgm, attributes(cross))
        }
        else mark <- mark[,1,]
    }

    else  # none of the above
        stop("Couldn't find marker ", mname)

    # make mark a matrix if it's not one
    if(!is.matrix(mark))
        mark <- matrix(mark, ncol=1)

    # return
    ret <- list(mark=mark, gennames=gennames)
    attr(ret, "mname") <- mname
    ret
}

##############################################
# function to calculate the means and ses
# if ndraws is 1, it's easy
# if ndraws > 1 (has pseudomarker),
# loop thru the draws
##############################################
effectplot.calmeanse <-
    function(pheno, mark1, mark2, geno1, geno2, ndraws, var.flag=c("pooled","group"))
{
    # local variables
    nind <- length(pheno)
    # method to calculate variances for estimated QTL effects
    var.flag <- match.arg(var.flag)

    result <- NULL
    nind <- sum(!is.na(pheno)) # number of individuals

    if(is.null(mark2)) { # if mark2 is missing
        if(ndraws > 1) { # more than one draws
            mark1.level <- seq(along=geno1) # level for mark1
            # init
            means.all <- matrix(NA, nrow=ndraws, ncol=length(mark1.level))
            colnames(means.all) <- mark1.level
            vars.all <- matrix(NA, nrow=ndraws, ncol=length(mark1.level))
            colnames(vars.all) <- mark1.level
            weight <- rep(0, ndraws) # weight for draws
            # loop thru draws
            for(i in 1:ndraws) {
                mark1.tmp <- mark1[,i] # data for current draw
                # fit a regression - this is used to calculate the weights
                mark1.factor <- factor(mark1.tmp, mark1.level)
                lm.tmp <- lm(pheno~mark1.factor-1)
                rss <- sum(lm.tmp$residuals^2)
                # compute the weight
                weight[i] <- (-nind/2)*log(rss)
                # group means
                means.tmp <- tapply(pheno, mark1.tmp, mean, na.rm = TRUE)
                # calculate group means and variances
                if(var.flag=="group") { # use variance in each group
                    vars.tmp <- tapply(pheno, mark1.tmp, function(a) var(a,na.rm = TRUE)/length(a))
                }
                else { # use pooled variance
                    vars.tmp <- tapply(mark1.tmp, mark1.tmp, function(a) rss/nind/length(a))
                }
                # note that there could be missing categories in draws
                means.all[i, names(means.tmp)] <- means.tmp
                vars.all[i, names(vars.tmp)] <- vars.tmp
            }
            # average across draws - for vars, it should be
            # mean of variance plus variance of means
            weight <- exp(weight-max(weight))
            means <- apply(means.all, 2, function(a) weighted.mean(a,weight,na.rm=TRUE))
            meanvar <- apply(vars.all, 2, function(a) weighted.mean(a,weight,na.rm=TRUE)) # mean of vars
            varmean <- apply(means.all, 2, function(a) weighted.mean((a-mean(a,na.rm=TRUE))^2,weight,na.rm=TRUE)) # var of means
            measured <- apply(means.all, 2, function(a) sum(!is.na(a)))
            means[measured==0] <- meanvar[measured==0] <- varmean[measured==0] <- NA
            # standard error
            ses <- sqrt(meanvar+varmean)
        }
        else { # ndraws is 1
            u <- sort(unique(mark1))
            if(any(!(u %in% seq(along=geno1)))) {
                newmark1 <- mark1
                for(i in seq(along=u))
                    newmark1[mark1==u[i]] <- i
                mark1 <- newmark1
            }
            means <- tapply(pheno, mark1, mean, na.rm = TRUE)

            if(var.flag == "group") { # use group variance
                ses <- tapply(pheno, mark1, function(a) sd(a, na.rm = TRUE)/sqrt(sum(!is.na(a))))
            }
            else { # use pooled variance
                mark1.factor <- factor(mark1, seq(along=geno1))
                lm.tmp <- lm(pheno~mark1.factor-1)
                rss <- sum(lm.tmp$residuals^2)
                ses <- tapply(mark1, mark1, function(a) sqrt(rss/nind/length(a)))
            }
        }
    }

    else { # with mark2
        if(ndraws > 1) {
            mark1.level <- seq(along=geno1) # level for mark1
            mark2.level <- seq(along=geno2) # level for mark2

            u <- sort(unique(as.numeric(mark1)))
            if(any(!(u %in% seq(along=geno1)))) {
                newmark1 <- mark1
                for(i in seq(along=u))
                    for(j in 1:ncol(mark2))
                        newmark1[mark1[,j]==u[i],j] <- i
                mark1 <- newmark1
            }

            u <- sort(unique(as.numeric(mark2)))
            if(any(!(u %in% seq(along=geno2)))) {
                newmark2 <- mark2
                for(i in seq(along=u))
                    for(j in 1:ncol(mark2))
                        newmark2[mark2[,j]==u[i],j] <- i
                mark2 <- newmark2
            }

            # init
            means.all <- array(NA, c(length(mark1.level), length(mark2.level), ndraws))
            dimnames(means.all) <- list(mark1.level, mark2.level, NULL)
            vars.all <- array(NA, c(length(mark1.level), length(mark2.level), ndraws))
            dimnames(vars.all) <- list(mark1.level, mark2.level, NULL)
            weight <- rep(0, ndraws) # weight for draws
            # loop thru draws
            for(i in 1:ndraws) {
                mark1.tmp <- mark1[,i] # data for current draw
                mark2.tmp <- mark2[,i]
                # fit a regression - this is used to calculate the weights
                mark1.factor <- factor(mark1.tmp, mark1.level)
                mark2.factor <- factor(mark2.tmp, mark2.level)
                lm.tmp <- lm(pheno~mark1.factor+mark2.factor+1)
                rss <- sum(lm.tmp$residuals^2)
                # compute the weight
                weight[i] <- (-nind/2)*log(rss)
                # group means
                means.tmp <- tapply(pheno, list(mark1.tmp, mark2.tmp), mean, na.rm = TRUE)
                # calculate group means and variances
                if(var.flag=="group") { # use variance in each group
                    vars.tmp <- tapply(pheno, list(mark1.tmp,mark2.tmp),
                                       function(a) var(a,na.rm = TRUE)/length(a))
                }
                else { # use pooled variance
                    vars.tmp <- tapply(mark1.tmp, list(mark1.tmp,mark2.tmp),
                                       function(a) rss/nind/length(a))
                }
                # note that there could be missing categories in draws
                means.all[dimnames(means.tmp)[[1]], dimnames(means.tmp)[[2]],i] <- means.tmp
                vars.all[dimnames(vars.tmp)[[1]], dimnames(vars.tmp)[[2]], i] <- vars.tmp
            }
            # average across draws - for vars, it should be
            # mean of variance plus variance of means
            weight <- exp(weight-max(weight))
            means <- apply(means.all, c(1,2), function(a) weighted.mean(a,weight,na.rm=TRUE))
            meanvar <- apply(vars.all, c(1,2), function(a) weighted.mean(a,weight,na.rm=TRUE))
            varmean <- apply(means.all, c(1,2), function(a) weighted.mean((a-mean(a,na.rm=TRUE))^2,weight,na.rm=TRUE)) # var of means
            measured <- apply(means.all, c(1,2), function(a) sum(!is.na(a)))
            means[measured==0] <- meanvar[measured==0] <- varmean[measured==0] <- NA
            # standard error
            ses <- sqrt(meanvar+varmean)
        }
        else { # ndraws is 1
            u <- sort(unique(mark1))
            if(any(!(u %in% seq(along=geno1)))) {
                newmark1 <- mark1
                for(i in seq(along=u))
                    newmark1[mark1==u[i]] <- i
                mark1 <- newmark1
            }

            u <- sort(unique(mark2))
            if(any(!(u %in% seq(along=geno2)))) {
                newmark2 <- mark2
                for(i in seq(along=u))
                    newmark2[mark2==u[i]] <- i
                mark2 <- newmark2
            }
            mark1 <- factor(mark1, seq(along=geno1))
            mark2 <- factor(mark2, seq(along=geno2))

            means <- tapply(pheno, list(mark1, mark2), mean, na.rm = TRUE)
            if(var.flag=="group") { # use group variance
                ses <- tapply(pheno, list(mark1, mark2), function(a) sd(a, na.rm = TRUE)/sqrt(sum(!is.na(a))))
            }
            else {# use pooled variance
                lm.tmp <- lm(pheno~mark1+mark2-1)
                rss <- sum(lm.tmp$residuals^2)
                ses <- tapply(mark1, list(mark1, mark2), function(a) sqrt(rss/nind/length(a)))
            }
        }
    }

    # result
    result$Means <- means
    result$SEs <- ses
    result
}



# end of effectplot.R

######################################################################
#
# calc.genoprob.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: calc.genoprob
#
######################################################################

######################################################################
#
# calc.genoprob: calculate genotype probabilities conditional on
#                observed marker genotypes
#
######################################################################

calc.genoprob <-
    function(cross, step=0, off.end=0, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             stepwidth=c("fixed", "variable", "max"))
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    # map function
    map.function <- match.arg(map.function)
    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    stepwidth <- match.arg(stepwidth)

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }

    n.ind <- nind(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)

    type <- crosstype(cross)

    # calculate genotype probabilities one chromosome at a time
    for(i in 1:n.chr) {
        if(n.mar[i]==1) temp.offend <- max(c(off.end,5))
        else temp.offend <- off.end

        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type=="X") xchr <- TRUE
        else xchr <- FALSE

        # which type of cross is this?
        if(type == "f2") {
            one.map <- TRUE
            if(!xchr) { # autosomal
                cfunc <- "calc_genoprob_f2"
                n.gen <- 3
                gen.names <- getgenonames("f2", "A", cross.attr=attributes(cross))
            }
            else {                             # X chromsome
                cfunc <- "calc_genoprob_bc"
                n.gen <- 2
                gen.names <- c("g1","g2")
            }
        }
        else if(type == "bc") {
            cfunc <- "calc_genoprob_bc"
            n.gen <- 2
            if(!xchr)
                gen.names <- getgenonames("bc", "A", cross.attr=attributes(cross))
            else gen.names <- c("g1","g2")
            one.map <- TRUE
        }
        else if(type == "riself" || type=="risib" || type=="dh" || type=="haploid") {
            cfunc <- "calc_genoprob_bc"
            n.gen <- 2
            gen.names <- getgenonames(type, "A", cross.attr=attributes(cross))
            one.map <- TRUE
        }
        else if(type == "4way") {
            cfunc <- "calc_genoprob_4way"
            n.gen <- 4
            one.map <- FALSE
            gen.names <- getgenonames(type, "A", cross.attr=attributes(cross))
        }
        else if(type=="ri8sib" || type=="ri4sib" || type=="ri8self" || type=="ri4self" || type=="bgmagic16") {
            cfunc <- paste("calc_genoprob_", type, sep="")
            if(type=="bgmagic16") n.gen <- 16
            else n.gen <- as.numeric(substr(type, 3, 3))
            one.map <- TRUE
            gen.names <- LETTERS[1:n.gen]
            if(xchr)
                warning("calc.genoprob not working properly for the X chromosome for 4- or 8-way RIL.")
        }
        else if(type == "bcsft") {
            one.map <- TRUE
            cfunc <- "calc_genoprob_bcsft"
            cross.scheme <- attr(cross, "scheme") ## c(s,t) for BC(s)F(t)
            if(!xchr) { # autosomal
                gen.names <- getgenonames("bcsft", "A", cross.attr=attributes(cross))
                n.gen <- 2 + (cross.scheme[2] > 0)
            }
            else { ## X chr
                cross.scheme[1] <- cross.scheme[1] + cross.scheme[2] - (cross.scheme[1] == 0)
                cross.scheme[2] <- 0
                gen.names <- c("g1","g2")
                n.gen <- 2
            }
        }
        else if(type == "ri8selfIRIP1") {
            one.map <- TRUE
            cfunc <- "calc_genoprob_ri8selfIRIP1"
            n.gen <- 8
            gen.names <- LETTERS[1:n.gen]
        }
        else
            stop("calc.genoprob not available for cross type ", type, ".")

        # genotype data
        gen <- cross$geno[[i]]$data
        gen[is.na(gen)] <- 0

        # recombination fractions
        if(one.map) {
            # recombination fractions
            map <- create.map(cross$geno[[i]]$map,step,temp.offend,stepwidth)
            rf <- mf(diff(map))
            if(type=="risib" || type=="riself")
                rf <- adjust.rf.ri(rf, sub("^ri", "", type), chr_type)
            rf[rf < 1e-14] <- 1e-14

            # new genotype matrix with pseudomarkers filled in
            newgen <- matrix(ncol=length(map),nrow=nrow(gen))
            dimnames(newgen) <- list(NULL,names(map))
            newgen[,colnames(gen)] <- gen
            newgen[is.na(newgen)] <- 0
            n.pos <- ncol(newgen)
            marnames <- names(map)
        }
        else {
            map <- create.map(cross$geno[[i]]$map,step,temp.offend,stepwidth)
            rf <- mf(diff(map[1,]))
            rf[rf < 1e-14] <- 1e-14
            rf2 <- mf(diff(map[2,]))
            rf2[rf2 < 1e-14] <- 1e-14

            # new genotype matrix with pseudomarkers filled in
            newgen <- matrix(ncol=ncol(map),nrow=nrow(gen))
            dimnames(newgen) <- list(NULL,dimnames(map)[[2]])
            newgen[,colnames(gen)] <- gen
            newgen[is.na(newgen)] <- 0
            n.pos <- ncol(newgen)
            marnames <- colnames(map)
        }
        ## Cross scheme being added for Ft and BCs.
        ## cross_scheme = c(BC = s, F = t).
        ## BC has cross_scheme = c(1,0)
        ## F2 has cross_scheme = c(0,2)
        ## BCsFt has cross_scheme = c(s,t)
        ## Other designs such as Ft with test cross don't quite fit in (yet).
        ## *** Need to change all the C routines!!

        # call the C function
        if(one.map) {
            ## Hide cross scheme in genoprob to pass to routine. BY
            temp <- as.double(rep(0,n.gen*n.ind*n.pos))
            if(type == "bcsft")
                temp[1:2] <- cross.scheme

            z <- .C(cfunc,
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(newgen),        # genotype data
                    as.double(rf),             # recombination fractions
                    as.double(error.prob),     #
                    genoprob=as.double(temp),
                    PACKAGE="qtl")
        }
        else {
            z <- .C(cfunc,
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(newgen),        # genotype data
                    as.double(rf),             # recombination fractions
                    as.double(rf2),            # recombination fractions
                    as.double(error.prob),     #
                    genoprob=as.double(rep(0,n.gen*n.ind*n.pos)),
                    PACKAGE="qtl")
        }

        # re-arrange marginal probabilites
        cross$geno[[i]]$prob <- array(z$genoprob,dim=c(n.ind,n.pos,n.gen))
        dimnames(cross$geno[[i]]$prob) <- list(NULL, marnames, gen.names)
        # attribute set to the error.prob value used, for later
        #     reference, especially by calc.errorlod()

        attr(cross$geno[[i]]$prob, "map") <- map
        attr(cross$geno[[i]]$prob,"error.prob") <- error.prob
        attr(cross$geno[[i]]$prob,"step") <- step
        attr(cross$geno[[i]]$prob,"off.end") <- temp.offend
        attr(cross$geno[[i]]$prob,"map.function") <- map.function
        attr(cross$geno[[i]]$prob,"stepwidth") <- stepwidth
    } # end loop over chromosomes

    # 4- and 8-way RIL: reorganize the results
    if(type=="ri4self" || type=="ri4sib" || type=="ri8self" || type=="ri8sib" || type=="bgmagic16")
        cross <- reorgRIgenoprob(cross)

    cross
}

######################################################################
#
# calc.genoprob.special:
#    special version used by calc.errorlod
#    for each individual and marker, calculate probabilities allowing
#    that genotype to be in error but assuming that all other genotypes
#    are correct
#
######################################################################

calc.genoprob.special <-
    function(cross, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"))
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    step <- 0
    off.end <- 0
    stepwidth <- "fixed"

    # map function
    map.function <- match.arg(map.function)
    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }

    n.ind <- nind(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)

    type <- crosstype(cross)

    # calculate genotype probabilities one chromosome at a time
    for(i in 1:n.chr) {
        if(n.mar[i]==1) temp.offend <- max(c(off.end,5))
        else temp.offend <- off.end

        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type=="X") xchr <- TRUE
        else xchr <- FALSE

        # which type of cross is this?
        if(type == "f2") {
            one.map <- TRUE
            if(!xchr) { # autosomal
                cfunc <- "calc_genoprob_special_f2"
                n.gen <- 3
                gen.names <- getgenonames("f2", "A", cross.attr=attributes(cross))
            }
            else {                             # X chromsome
                cfunc <- "calc_genoprob_special_bc"
                n.gen <- 2
                gen.names <- c("g1","g2")
            }
        }
        else if(type == "bc") {
            cfunc <- "calc_genoprob_special_bc"
            n.gen <- 2
            if(!xchr)
                gen.names <- getgenonames("bc", "A", cross.attr=attributes(cross))
            else gen.names <- c("g1","g2")
            one.map <- TRUE
        }
        else if(type == "riself" || type=="risib" || type=="dh" || type=="haploid") {
            cfunc <- "calc_genoprob_special_bc"
            n.gen <- 2
            gen.names <- getgenonames(type, "A", cross.attr=attributes(cross))
            one.map <- TRUE
        }
        else if(type == "4way") {
            cfunc <- "calc_genoprob_special_4way"
            n.gen <- 4
            one.map <- FALSE
            gen.names <- getgenonames(type, "A", cross.attr=attributes(cross))
        }
        else if(type=="ri8sib" || type=="ri4sib" || type=="ri8self" || type=="ri4self" || type=="bgmagic16") {
            cfunc <- paste("calc_genoprob_special_", type, sep="")
            if(type=="bgmagic16") n.gen <- 16
            else n.gen <- as.numeric(substr(type, 3, 3))
            one.map <- TRUE
            gen.names <- LETTERS[1:n.gen]
            if(xchr)
                warning("calc.genoprob.special not working properly for the X chromosome for 4- or 8-way RIL.")
        }
        else if(type == "bcsft") {
            one.map <- TRUE
            cfunc <- "calc_genoprob_bcsft"
            cross.scheme <- attr(cross, "scheme") ## c(s,t) for BC(s)F(t)
            if(!xchr) { # autosomal
                if(cross.scheme[2] == 0) {
                    gen.names <- getgenonames("bc", "A", cross.attr=attributes(cross))
                    n.gen <- 2
                }
                else {
                    gen.names <- getgenonames("f2", "A", cross.attr=attributes(cross))
                    n.gen <- 3
                }
            }
            else { ## X chr
                cross.scheme[1] <- cross.scheme[1] + cross.scheme[2] - (cross.scheme[1] == 0)
                cross.scheme[2] <- 0
                gen.names <- c("g1","g2")
                n.gen <- 2
            }
        }
        else
            stop("calc.genoprob.special not available for cross type ", type, ".")

        # genotype data
        gen <- cross$geno[[i]]$data
        gen[is.na(gen)] <- 0

        # recombination fractions
        if(one.map) {
            # recombination fractions
            map <- create.map(cross$geno[[i]]$map,step,temp.offend,stepwidth)
            rf <- mf(diff(map))
            if(type=="risib" || type=="riself")
                rf <- adjust.rf.ri(rf,sub("^ri", "", type), chr_type)
            rf[rf < 1e-14] <- 1e-14

            # new genotype matrix with pseudomarkers filled in
            newgen <- matrix(ncol=length(map),nrow=nrow(gen))
            dimnames(newgen) <- list(NULL,names(map))
            newgen[,colnames(gen)] <- gen
            newgen[is.na(newgen)] <- 0
            n.pos <- ncol(newgen)
            marnames <- names(map)
        }
        else {
            map <- create.map(cross$geno[[i]]$map,step,temp.offend,stepwidth)
            rf <- mf(diff(map[1,]))
            rf[rf < 1e-14] <- 1e-14
            rf2 <- mf(diff(map[2,]))
            rf2[rf2 < 1e-14] <- 1e-14

            # new genotype matrix with pseudomarkers filled in
            newgen <- matrix(ncol=ncol(map),nrow=nrow(gen))
            dimnames(newgen) <- list(NULL,dimnames(map)[[2]])
            newgen[,colnames(gen)] <- gen
            newgen[is.na(newgen)] <- 0
            n.pos <- ncol(newgen)
            marnames <- colnames(map)
        }

        # call the C function
        if(one.map) {
            ## Hide cross scheme in genoprob to pass to routine. BY
            temp <- as.double(rep(0,n.gen*n.ind*n.pos))
            if(type == "bcsft")
                temp[1:2] <- cross.scheme

            z <- .C(cfunc,
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(newgen),        # genotype data
                    as.double(rf),             # recombination fractions
                    as.double(error.prob),     #
                    genoprob=as.double(temp),
                    PACKAGE="qtl")
        }
        else {
            z <- .C(cfunc,
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(newgen),        # genotype data
                    as.double(rf),             # recombination fractions
                    as.double(rf2),            # recombination fractions
                    as.double(error.prob),     #
                    genoprob=as.double(rep(0,n.gen*n.ind*n.pos)),
                    PACKAGE="qtl")
        }

        # re-arrange marginal probabilites
        cross$geno[[i]]$prob <- array(z$genoprob,dim=c(n.ind,n.pos,n.gen))
        dimnames(cross$geno[[i]]$prob) <- list(NULL, marnames, gen.names)
        # attribute set to the error.prob value used, for later
        #     reference, especially by calc.errorlod()

        attr(cross$geno[[i]]$prob, "map") <- map
        attr(cross$geno[[i]]$prob,"error.prob") <- error.prob
        attr(cross$geno[[i]]$prob,"step") <- step
        attr(cross$geno[[i]]$prob,"off.end") <- temp.offend
        attr(cross$geno[[i]]$prob,"map.function") <- map.function
        attr(cross$geno[[i]]$prob,"stepwidth") <- stepwidth
    } # end loop over chromosomes

    cross
}

# end of calc.genoprob.R

#####################################################################
#
# mqmsnow.R
#
# Copyright (c) 2009-2010, Danny Arends
#
# Modified by Karl Broman and Pjotr Prins
#
#
# first written Februari 2009
# last modified April 2010
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: snowCoreALL
#           snowCoreBOOT
#
#
#####################################################################






snowCoreALL <- function(x,all.data,scanfunction,cofactors,verbose=FALSE,...){
    b <- proc.time()
    result <- NULL
    num.traits <- nphe(all.data)
    if(verbose) {
        cat("------------------------------------------------------------------\n")
        cat("INFO: Starting analysis of trait (",x,"/",num.traits,")\n")
        cat("------------------------------------------------------------------\n")
    }
    if(!is.null(cofactors)) {
        result <- scanfunction(cross=all.data,cofactors=cofactors,pheno.col=x,verbose=verbose,...)
    }
    else{
        result <- scanfunction(cross=all.data,pheno.col=x,...)
    }
    colnames(result)[3] <- paste("LOD",names(all.data$pheno)[x])
    e <- proc.time()
    if(verbose) {
        cat("------------------------------------------------------------------\n")
        cat("INFO: Done with the analysis of trait (",x,"/",num.traits,")\n")
        cat("INFO: Calculation of trait",x,"took:",round((e-b)[3], digits=3)," seconds\n")
        cat("------------------------------------------------------------------\n")
    }
    result
}


snowCoreBOOT <- function(x,all.data,scanfunction,bootmethod,cofactors,verbose=FALSE,...){
    b <- proc.time()
    result <- NULL
    if(!bootmethod){
        #random permutation
        neworder <- sample(nind(all.data))
        all.data$pheno[[1]] <- all.data$pheno[[1]][neworder]
    }else{
        #parametric permutation
        all.data$pheno[[1]] <- rnorm(nind(all.data))
    }
    if("cofactors" %in% names(formals(scanfunction))){
        if(!is.null(cofactors)){
            result <- scanfunction(cross=all.data,cofactors=cofactors,pheno.col=1,verbose=FALSE,...)
        }else{
            result <- scanfunction(cross=all.data,pheno.col=1,verbose=FALSE,...)
        }
    }else{
        if("plot" %in% names(formals(scanfunction))){
            result <- scanfunction(cross=all.data,pheno.col=1,...)
        }else{
            result <- scanfunction(cross=all.data,pheno.col=1,...)
        }
    }
    e <- proc.time()
    if(verbose) {
        cat("------------------------------------------------------------------\n")
        cat("INFO: Done with bootstrap\n")
        cat("INFO: Calculation took:",round((e-b)[3], digits=3)," seconds\n")
        cat("------------------------------------------------------------------\n")
    }
    result
}

# end of mqmsnow.R

######################################################################
#
# tryallpositions.R
#
# copyright (c) 2007-2019, Karl W Broman
# last modified Dec, 2019
# first written Oct, 2007
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: tryallpositions, markerloglik, allchrsplits
#
######################################################################

######################################################################
# tryallpositions
#
# Place a given marker in all possible positions on a selected set of
# chromosomes, keeping the positions of all other markers fixed, and
# evaluate the likelihood and estimate the chromosome length
######################################################################

tryallpositions <-
    function(cross, marker, chr, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             m=0, p=0, maxit=4000, tol=1e-6, sex.sp=TRUE, verbose=TRUE)
{
    map.function <- match.arg(map.function)
    if(missing(chr)) chr <- names(cross$geno)
    else chr <- matchchr(chr, names(cross$geno))

    thechr <- find.markerpos(cross, marker)[1,1]
    if(is.na(thechr))
        stop("Marker ", marker, " not found.")

    markerll <- markerloglik(cross, marker, error.prob)

    allchr <- names(subset(cross, chr=chr)$geno)

    results <- NULL
    for(i in allchr) {
        if(i == thechr) { # marker already on this chromosome
            pos <- cross$geno[[i]]$map
            if(is.matrix(pos)) {
                pos <- pos[1,]
                matrixmap <- TRUE
            }
            else matrixmap <- FALSE

            n.mar <- ncol(cross$geno[[i]]$data)
            if(n.mar == 1) { # this is the only marker
                pos <- 0
                length <- length.male <- 0
                llik <- 0
                if(verbose) cat(i, pos, llik/log(10), "\n")
                interval <- "---"
            } # just this marker
            else if(n.mar == 2) { # just two markers
                pos <- 0
                themar <- colnames(cross$geno[[i]]$data)
                initialloglik <- sum(sapply(themar, function(a, b, c) markerloglik(b, a, c), cross, error.prob))

                nm <- est.map(cross, chr=i, error.prob=error.prob,
                              map.function=map.function, m=m, p=p, maxit=maxit,
                              tol=tol, sex.sp=sex.sp, verbose=FALSE,
                              omit.noninformative=FALSE)[[1]]
                llik <- attr(nm, "loglik") - initialloglik
                if(verbose) cat(i, pos, llik/log(10), "\n")

                if(is.matrix(nm)) {
                    length <- diff(range(nm[1,]))
                    length.male <- diff(range(nm[2,]))
                }
                else length <- diff(range(nm))
                interval <- "---"
            } # just two markers
            else { # >2 markers
                temp <- drop.markers(cross, marker)

                pos <- temp$geno[[i]]$map
                if(is.matrix(pos)) {
                    pos <- pos[1,]
                    matrixmap <- TRUE
                }
                else matrixmap <- FALSE

                pos <- c(min(pos)-10, pos, max(pos)+10)
                pos <- (pos[-1] + pos[-length(pos)])/2

                themarkers <- colnames(temp$geno[[i]]$data)
                int2 <- c("*", match(themarkers, colnames(cross$geno[[i]]$data)), "*")
                themarkers <- c("pter", themarkers, "qter")
                interval <- paste(themarkers[-length(themarkers)], themarkers[-1],
                                  sep="-")
                int2 <- paste("(", int2[-length(int2)], "-", int2[-1], ")", sep="")
                interval <- paste(interval, int2)

                initialmap <- est.map(temp, chr=i,
                                      error.prob=error.prob,
                                      map.function=map.function, m=m, p=p, maxit=maxit,
                                      tol=tol, sex.sp=sex.sp, verbose=FALSE,
                                      omit.noninformative=FALSE)[[1]]
                initialloglik <- attr(initialmap, "loglik") + markerll

                llik <- length <- length.male <- rep(NA, length(pos))
                for(j in seq(along=pos)) {
                    temp <- movemarker(cross, marker, i, pos[j])
                    nm <- est.map(temp, chr=i, error.prob=error.prob,
                                  map.function=map.function, m=m, p=p, maxit=maxit,
                                  tol=tol, sex.sp=sex.sp, verbose=FALSE,
                                  omit.noninformative=FALSE)[[1]]
                    llik[j] <- attr(nm, "loglik") - initialloglik
                    if(verbose) cat(i, pos[j], llik[j]/log(10), "\n")

                    if(is.matrix(nm)) {
                        length[j] <- diff(range(nm[1,]))
                        length.male[j] <- diff(range(nm[2,]))
                    }
                    else length[j] <- diff(range(nm))
                }
            } # >2 markers

        }

        else { # marker not on this chromosome

            pos <- cross$geno[[i]]$map
            if(is.matrix(pos)) {
                pos <- pos[1,]
                matrixmap <- TRUE
            }
            else matrixmap <- FALSE

            n.mar <- ncol(cross$geno[[i]]$data)

            if(n.mar > 1) {
                pos <- c(min(pos)-10, pos, max(pos)+10)
                pos <- (pos[-1] + pos[-length(pos)])/2

                themarkers <- colnames(cross$geno[[i]]$data)
                int2 <- c("*", 1:length(themarkers), "*")
                themarkers <- c("pter", themarkers, "qter")
                interval <- paste(themarkers[-length(themarkers)], themarkers[-1],
                                  sep="-")
                int2 <- paste("(", int2[-length(int2)], "-", int2[-1], ")", sep="")
                interval <- paste(interval, int2)

                initialmap <- est.map(cross, chr=i, error.prob=error.prob,
                                      map.function=map.function, m=m, p=p, maxit=maxit,
                                      tol=tol, sex.sp=sex.sp, verbose=FALSE,
                                      omit.noninformative=FALSE)[[1]]
                initialloglik <- attr(initialmap, "loglik") + markerll

                llik <- length <- length.male <- rep(NA, length(pos))
                for(j in seq(along=pos)) {
                    temp <- movemarker(cross, marker, i, pos[j])

                    nm <- est.map(temp, chr=i, error.prob=error.prob,
                                  map.function=map.function, m=m, p=p, maxit=maxit,
                                  tol=tol, sex.sp=sex.sp, verbose=FALSE,
                                  omit.noninformative=FALSE)[[1]]
                    llik[j] <- attr(nm, "loglik") - initialloglik
                    if(verbose) cat(i, pos[j], llik[j]/log(10), "\n")

                    if(is.matrix(nm)) {
                        length[j] <- diff(range(nm[1,]))
                        length.male[j] <- diff(range(nm[2,]))
                    }
                    else length[j] <- diff(range(nm))
                }
            } # >1 marker on chromosome
            else {
                initialloglik <- markerloglik(cross, colnames(cross$geno[[i]]$data), error.prob) + markerll

                pos <- pos+10
                interval <- "---"

                temp <- movemarker(cross, marker, i, pos)

                nm <- est.map(temp, chr=i, error.prob=error.prob,
                              map.function=map.function, m=m, p=p, maxit=maxit,
                              tol=tol, sex.sp=sex.sp, verbose=FALSE,
                              omit.noninformative=FALSE)[[1]]
                llik <- attr(nm, "loglik") - initialloglik
                if(verbose) cat(i, pos, llik/log(10), "\n")

                if(is.matrix(nm)) {
                    length <- diff(range(nm[1,]))
                    length.male <- diff(range(nm[2,]))
                }
                else length <- diff(range(nm))

            } # one marker on chromosome
        } # marker not on this chr

        if(matrixmap && sex.sp)
            tempres <- data.frame(chr=rep(i, length(pos)),
                                  pos=pos,
                                  lod=llik/log(10),
                                  length.female=length,
                                  length.male=length.male,
                                  interval=interval,
                                  stringsAsFactors=FALSE)
        else
            tempres <- data.frame(chr=rep(i, length(pos)),
                                  pos=pos,
                                  lod=llik/log(10),
                                  length=length,
                                  interval=interval,
                                  stringsAsFactors=FALSE)


        results <- rbind(results, tempres)
    } # loop over chromosomes

    rownames(results) <- results$interval
    results <- results[,-ncol(results)]
    class(results) <- c("scanone", "data.frame")

    results
}

######################################################################
#
# markerloglik: Calculate log likelihood for a given marker
#
######################################################################

markerloglik <-
    function(cross, marker, error.prob=0.0001)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    type <- crosstype(cross)

    if(length(marker) > 1) {
        ll <- sapply(marker, function(a,b,d) markerloglik(b, a, d), cross, error.prob)
        names(ll) <- marker
        return(ll)
    }

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 1) {
        error.prob <- 1-1e-50
        warning("error.prob shouldn't be > 1!")
    }

    n.ind <- nind(cross)

    thechr <- find.markerpos(cross, marker)[1,1]
    if(is.na(thechr))
        stop("Marker ", marker, " not found.")

    chr_type <- chrtype(cross$geno[[thechr]])

    g <- pull.geno(cross, chr=thechr)
    m <- match(marker, colnames(g))
    if(is.na(m))
        stop("Marker ", marker, " not found.")
    g <- g[,m]
    g[is.na(g)] <- 0

    # which type of cross is this?
    if(type == "f2") {
        if(chr_type == "A") # autosomal
            cfunc <- "marker_loglik_f2"
        else                  # X chromsome
            cfunc <- "marker_loglik_bc"
    }
    else if(type == "bc" || type=="riself" || type=="risib" || type=="dh" || type=="haploid") {
        cfunc <- "marker_loglik_bc"
    }
    else if(type == "4way") {
        cfunc <- "marker_loglik_4way"
    }
    else if(type=="ri4sib" || type=="ri4self" || type=="ri8sib" || type=="ri8self" || type=="bgmagic16") {
        cfunc <- paste("marker_loglik_", type, sep="")
        if(chr_type=="X")
            warning("markerloglik not working properly for the X chromosome for 4- or 8-way RIL.")
    }
    else if(type == "bcsft") {
        cfunc <- "marker_loglik_bcsft"
        cross.scheme <- attr(cross, "scheme") ## c(s,t) for BC(s)F(t)
        if(chr_type != "A") { ## X chromosome
            cross.scheme[1] <- cross.scheme[1] + cross.scheme[2] - (cross.scheme[1] == 0)
            cross.scheme[2] <- 0
        }
    }
    else
        stop("markerloglik not available for cross type ", type, ".")

    ## Hide cross scheme in genoprob to pass to routine. BY
    temp <- 0
    if(type == "bcsft")
        temp[1] <- cross.scheme[1] * 1000 + cross.scheme[2]

    # call the C function
    z <- .C(cfunc,
            as.integer(n.ind),       # number of individuals
            as.integer(g),           # genotype data
            as.double(error.prob),
            loglik=as.double(temp),     # log likelihood
            PACKAGE="qtl")

    z$loglik
}


######################################################################
# allchrsplits
#
# get LOD scores for each possible split of each chromosome into
# two pieces
#
######################################################################

allchrsplits <-
    function(cross, chr, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             m=0, p=0, maxit=4000, tol=1e-6, sex.sp=TRUE, verbose=TRUE)
{
    map.function <- match.arg(map.function)
    if(!missing(chr)) cross <- subset(cross, chr=chr)

    biggap <- imf.h(0.5 - 1e-14)

    n.mar <- nmar(cross)
    chrnam <- names(cross$geno)

    result <- NULL

    for(i in seq(along=cross$geno)) {
        if(n.mar[i] == 1) {
            #      temp <- data.frame(chr=chrnam[i], pos=pos, lod=NA, gap=0)
            #      rownames(temp) <- themarkers
            #      result <- cbind(result, temp)
            next
        }
        thischr <- subset(cross, chr=chrnam[i])
        if(verbose) cat("Chr ", chrnam[i], " (", n.mar[i], " markers)\n", sep="")
        pos <- cross$geno[[i]]$map
        themarkers <- colnames(cross$geno[[i]]$data)
        if(is.matrix(pos)) {
            pos <- pos[1,]
            matrixmap <- TRUE
        }
        else matrixmap <- FALSE


        gap <- (pos[-1] - pos[-length(pos)])
        pos <- (pos[-1] + pos[-length(pos)])/2
        int2 <- match(themarkers, colnames(cross$geno[[i]]$data))
        interval <- paste(themarkers[-length(themarkers)], themarkers[-1],
                          sep="-")
        int2 <- paste("(", int2[-length(int2)], "-", int2[-1], ")", sep="")
        interval <- paste(interval, int2)

        initialmap <- est.map(thischr, error.prob=error.prob,
                              map.function=map.function, m=m, p=p, maxit=maxit, tol=tol, sex.sp=sex.sp)
        thischr$geno[[1]]$map <- initialmap[[1]]
        initialloglik <- attr(initialmap[[1]], "loglik")

        if(n.mar[i] == 2) { # 2 markers
            mmll <- markerloglik(thischr, markernames(thischr), error.prob=error.prob)
            temp <- data.frame(chr=chrnam[i], pos=pos,
                               lod=(initialloglik - sum(mmll))/log(10), gap=gap, stringsAsFactors=TRUE)
            rownames(temp) <- interval
        }
        else { # >2 markers
            mn <- markernames(thischr)
            mmll <- markerloglik(thischr, mn[c(1,length(mn))], error.prob=error.prob)
            lod <- rep(NA, length(mn)-1)

            if(verbose) cat("  interval 1\n")
            # first interval
            lod[1] <- initialloglik - mmll[1] -
                attr(est.map(drop.markers(thischr, mn[1]), error.prob=error.prob,
                             map.function=map.function, m=m, p=p, maxit=maxit, tol=tol, sex.sp=sex.sp)[[1]], "loglik")

            if(n.mar[i] > 3) {
                for(j in 2:(n.mar[i]-2)) {
                    if(verbose) cat("  interval", j, "\n")
                    temp1 <- est.map(pull.markers(thischr, mn[1:j]), error.prob=error.prob,
                                     map.function=map.function, m=m, p=p, maxit=maxit, tol=tol, sex.sp=sex.sp)[[1]]
                    temp2 <- est.map(drop.markers(thischr, mn[1:j]), error.prob=error.prob,
                                     map.function=map.function, m=m, p=p, maxit=maxit, tol=tol, sex.sp=sex.sp)[[1]]
                    #          lod[j] <- initialloglik - attr(temp1, "loglik") - attr(temp2, "loglik")

                    if(any(is.na(temp1)) || any(is.na(temp2)))
                        stop("Missing values in estimated map on chr ", chrnam[i], " with split at interval ", j, "\n")

                    # the likelihoods aren't adding properly, so I'll use the following kluge:
                    temp3 <- thischr
                    if(is.matrix(temp1))
                        temp3$geno[[1]]$map <- cbind(temp1, biggap+temp2)
                    else
                        temp3$geno[[1]]$map <- c(temp1, biggap+temp2)

                    temp3 <- est.map(temp3, error.prob=error.prob, map.function=map.function, m=m, p=p, maxit=0, tol=tol,
                                     sex.sp=sex.sp)[[1]]
                    lod[j] <- initialloglik - attr(temp3, "loglik")
                }
            }

            if(verbose) cat("  interval", n.mar[i]-1, "\n")
            # last interval
            lod[length(mn)-1] <- initialloglik - mmll[2] -
                attr(est.map(drop.markers(thischr, mn[length(mn)]), error.prob=error.prob,
                             map.function=map.function, m=m, p=p, maxit=maxit, tol=tol, sex.sp=sex.sp)[[1]], "loglik")

            temp <- data.frame(chr=rep(chrnam[i], length(interval)), pos=pos, lod=lod/log(10), gap=gap, stringsAsFactors=TRUE)
            rownames(temp) <- interval
        }
        result <- rbind(result, temp)
    }
    class(result) <- c("scanone", "data.frame")
    result
}

# end of tryallpositions.R

######################################################################
# phyloqtl_sim.R
#
# copyright (c) 2009, Karl W Broman
# last modified May, 2009
# first written May, 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Simulating multiple crosses with multiple taxa with diallelic QTL
# located on a phylogenetic tree
#
# Part of the R/qtl package
# Contains: simPhyloQTL
#
######################################################################

######################################################################
# simPhyloQTL
#
# function for simulating data on multiple intercrosses, for a single
# diallelic QTL
#
# n.taxa =
# partition = character string of the form "AB|CD" or "A|BCD"
#             if missing, simulate under the null
# crosses = set of two-character strings indicating the crosses to do
#             (of the form "AB", AC", etc.)
#            these will be sorted and then only unique ones used
#            if missing, we'll do all crosses
######################################################################

simPhyloQTL <-
    function(n.taxa=3, partition, crosses, map, n.ind=100, model,
             error.prob=0, missing.prob=0, partial.missing.prob=0,
             keep.qtlgeno=FALSE, keep.errorind=TRUE, m=0, p=0,
             map.function=c("haldane", "kosambi", "c-f", "morgan"))
{
    if(n.taxa < 3) stop("Should have 3 or more taxa.")
    if(n.taxa > 26) stop("We can only deal with 3-26 taxa.")

    map.function <- match.arg(map.function)
    if(!missing(model)) {
        if((is.matrix(model) && ncol(model)!=4) ||
           (!is.matrix(model) && length(model) != 4))
            stop("model should have QTL chr, pos, and additive and dominance effects.")
        if(is.matrix(model)) n.qtl <- nrow(model)
        else {
            n.qtl <- 1
            model <- rbind(model)
        }
    }
    else {
        model <- NULL
        n.qtl <- 0
    }

    taxa <- LETTERS[1:n.taxa]

    if(missing(partition) || is.null(partition) || partition=="") {
        partition <- NULL
        if(!is.null(model))
            warning("partition is NULL, so model is ignored and data are simulated with no QTL.")

    }
    else {
        if(is.null(model)) {
            warning("model is NULL, so partition is ignored and data are simulated with no QTL.")
            partition <- NULL
        }
        else {
            if(length(partition)==1 && n.qtl > 1)
                partition <- rep(partition, n.qtl)
            if(length(partition)>1 && n.qtl == 1)
                stop("Model indicates just one QTL, so partition should have length 1.")
            if(length(partition)>1 && n.qtl != length(partition))
                stop("No. QTL in model should match the length of partition.")

            for(i in unique(partition))
                checkPhyloPartition(i, taxa)
        }
    }

    if(missing(crosses) || is.null(crosses)) { # use all possible crosses
        crosses <- NULL
        for(i in 1:(n.taxa-1))
            for(j in (i+1):n.taxa)
                crosses <- c(crosses, paste(taxa[i], taxa[j], sep=""))
    }
    else  # check that the crosses are correct and sufficient
        crosses <- checkPhyloCrosses(crosses, taxa)

    if(length(n.ind) > 1 && length(n.ind) != length(crosses))
        stop("n.ind should have length 1 or have length equal to the number of crosses.")
    if(length(n.ind)==1) n.ind <- rep(n.ind, length(crosses))

    if(n.qtl > 0) {
        # for each partition, determine which crosses have the QTL
        crossmat <- qtlByPartition(crosses, partition)

        models <- vector("list", length(crosses))
        for(i in seq(along=crosses)) {
            models[[i]] <- model
            models[[i]][crossmat[i,] <0,3] <- -models[[i]][crossmat[i,] <0,3]
            models[[i]] <- models[[i]][crossmat[i,]!=0,,drop=FALSE]
        }
    }
    else models <- vector("list", length(crosses))

    thedata <- vector("list", length(crosses))
    names(thedata) <- crosses
    if(missing(map)) stop("Must provide a genetic map")

    for(i in seq(along=crosses))
        thedata[[i]] <- sim.cross(map, models[[i]], n.ind=n.ind[i], type="f2",
                                  error.prob=error.prob, missing.prob=missing.prob,
                                  partial.missing.prob=partial.missing.prob,
                                  keep.qtlgeno = keep.qtlgeno, keep.errorind=keep.errorind,
                                  m=m, p=p, map.function=map.function)
    thedata
}

# end of phyloqtl_sim.R

#####################################################################
#
# plotperm.R
#
# copyright (c) 2007-2014, Karl W Broman
# last modified Oct, 2014
# first written Dec, 2007
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: plot.scanoneperm, plot.scantwoperm, plot.scanoneboot
#
######################################################################

############################################################
# plot.scanoneboot
#
# plot histogram of the results from scanoneboot
############################################################
plot.scanoneboot <-
    function(x, ...)
{
    results <- attr(x, "results")
    markerpos <- results[-grep("^c.+\\.loc-*[0-9]+(\\.[0-9]+)*$", rownames(results)),2]

    hideplot.scanoneboot <-
        function(x, breaks, xlim, main, xlab, ...)
        {
            if(missing(breaks)) breaks <- ceiling(2*sqrt(length(x)))
            if(missing(xlim)) xlim <- range(results[,2])
            if(missing(main)) main <- ""
            if(missing(xlab)) xlab <- "QTL position (cM)"

            hist(x, xlim=xlim, xlab=xlab, main=main, breaks=breaks, ...)
        }

    hideplot.scanoneboot(x, ...)
    rug(markerpos)
}

############################################################
# plot.scanoneperm
#
# plot histogram of the permutation results from scanone
############################################################
plot.scanoneperm <-
    function(x, lodcolumn=1, ...)
{
    # subroutine for hiding arguments in ...
    hideplot.scanoneperm <-
        function(A, X, breaks, xlab, xlim, main, ...)
        {
            if(missing(xlab)) xlab <- "maximum LOD score"

            if(missing(X)) {
                if(missing(breaks)) breaks <- ceiling(2*sqrt(length(A)))
                if(missing(xlim)) xlim <- c(0, max(A))
                if(missing(main)) main <- ""

                hist(A, breaks=breaks, xlab=xlab, xlim=xlim, main=main, ...)
            }
            else {
                mfrow <- par("mfrow")
                on.exit(par(mfrow=mfrow))
                par(mfrow=c(2,1))

                if(missing(breaks)) {
                    breaks.missing <- TRUE
                    breaks <- seq(0, max(c(A,X)), length=2*sqrt(length(A)))
                }
                else breaks.missing <- FALSE

                if(missing(xlim)) xlim <- c(0, max(c(A,X)))
                if(missing(main)) {
                    main <- "Autosomes"
                    main.missing <- TRUE
                }
                else {
                    main.missing <- FALSE
                    if(length(main)>1) { main2 <- main[2]; main <- main[1] }
                    else main2 <- main
                }

                hist(A, xlim=xlim, breaks=breaks, xlab=xlab, main=main, ...)
                rug(A)

                if(breaks.missing)
                    breaks <- seq(0, max(c(A,X)), length=2*sqrt(length(X)))
                if(main.missing) main <- "X chromosome"
                else main <- main2
                hist(X, xlim=xlim, breaks=breaks, xlab=xlab, main=main, ...)
                rug(X)
            }
        }

    # now to the actual code
    if(is.list(x)) { # separate X chr results
        if(lodcolumn < 1 || lodcolumn > ncol(x$A))
            stop("lodcolumn should be between 1 and ", ncol(x$A))
        A <- as.numeric(x$A[,lodcolumn])
        X <- as.numeric(x$X[,lodcolumn])
        hideplot.scanoneperm(A, X, ...)
    }
    else {
        if(lodcolumn < 1 || lodcolumn > ncol(x))
            stop("lodcolumn should be between 1 and ", ncol(x$A))
        A <- as.numeric(x[,lodcolumn])
        hideplot.scanoneperm(A, ...)
    }
}


############################################################
# plot.scantwoperm
#
# plot histogram of the permutation results from scantwo
############################################################
plot.scantwoperm <-
    function(x, lodcolumn=1, include_rug=TRUE, ...)
{
    hideplot.scantwoperm <-
        function(x, include_rug=TRUE, xlim, breaks, xlab, main, ...)
        {
            if(missing(xlim)) xlim <- c(0, max(unlist(x)))
            if(missing(xlab)) xlab <- "maximum LOD score"
            if(missing(main)) main.missing <- TRUE
            else { main.missing <- FALSE; main.input <- main }


            mfcol <- par("mfcol")
            on.exit(par(mfcol=mfcol))

            if("AA" %in% names(x)) {

                if(missing(breaks))
                    breaks <- seq(0, max(unlist(x)), len=ceiling(4*sqrt(length(x[[1]][[1]]))+1))

                par(mfrow=c(3,6))

                for(i in seq(along=x)) {
                    for(j in seq(along=x[[i]])) {
                        if(main.missing) main <- paste(names(x)[i], names(x[[i]])[j])
                        else {
                            if(length(main.input) >= i) main <- main.input[i]
                            else if(length(main.input) == 1) main <- main.input
                            else main <- ""
                        }

                        hist(x[[i]][[j]], xlim=xlim, breaks=breaks, xlab=xlab, main=main,...)
                        if(include_rug) rug(x[[i]][[j]])
                    }
                }
                invisible(return(NULL))
            }

            if(missing(breaks))
                breaks <- seq(0, max(unlist(x)), len=ceiling(4*sqrt(length(x[[1]]))+1))


            par(mfcol=c(3,2))

            for(i in seq(along=x)) {
                if(main.missing) main <- names(x)[i]
                else {
                    if(length(main.input) >= i) main <- main.input[i]
                    else if(length(main.input) == 1) main <- main.input
                    else main <- ""
                }

                hist(x[[i]], xlim=xlim, breaks=breaks, xlab=xlab, main=main,...)
                if(include_rug) rug(x[[i]])
            }
        }

    if(length(lodcolumn) > 1)
        stop("Select just one lod column")
    x <- x[,lodcolumn]
    hideplot.scantwoperm(x, include_rug=include_rug, ...)
}


# end of plotperm.R

######################################################################
#
# droponemarker.R
#
# copyright (c) 2019, Karl W Broman
# last modified Dec, 2019
# first written Oct, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: droponemarker
#
######################################################################

######################################################################
#
# droponemarker: Drop one marker at a time from a genetic map, to
#                evaluate the change in log likelihood and in map
#                length, in order to identify problematic markers
#
######################################################################

droponemarker <-
    function(cross, chr, error.prob=0.0001,
             map.function=c("haldane", "kosambi", "c-f", "morgan"),
             m=0, p=0, maxit=4000, tol=1e-6, sex.sp=TRUE, verbose=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("Input must have class \"cross\".")

    if(!missing(chr)) cross <- subset(cross, chr=chr)
    if(any(nmar(cross) < 3)) {
        if(all(nmar(cross) < 3))
            stop("No chromosomes with at least three markers\n")
        todrop <- names(cross$geno)[nmar(cross) < 3]
        tokeep <- names(cross$geno)[nmar(cross) > 2]
        warning("Dropping chr with <3 markers: ", paste(todrop, collapse=", "))
        cross <- subset(cross, chr=tokeep)
    }

    map.function <- match.arg(map.function)
    if(verbose) cat(" -Re-estimating map\n")
    origmap <- est.map(cross, error.prob=error.prob, map.function=map.function,
                       m=m, p=p, maxit=maxit, tol=tol, sex.sp=sex.sp)

    cross <- replace.map(cross, origmap)
    origmaptab <- pull.map(cross, as.table=TRUE)
    origmaptab <- cbind(origmaptab, LOD=rep(NA, nrow(origmaptab)))
    if(is.matrix(origmap[[1]])) {
        origmaptab <- cbind(origmaptab, Ldiff.female=rep(NA, nrow(origmaptab)),
                            Ldiff.male=rep(NA, nrow(origmaptab)))
        sexsp <- TRUE
    }
    else {
        origmaptab <- cbind(origmaptab, Ldiff=rep(NA, nrow(origmaptab)))
        sexsp <- FALSE
    }

    for(i in names(cross$geno)) {
        if(sexsp) {
            Lf <- diff(range(origmap[[i]][1,]))
            Lm <- diff(range(origmap[[i]][2,]))
        }
        else L <- diff(range(origmap[[i]]))

        if(verbose) cat(" -Chromosome", i, "\n")
        mnames <- markernames(cross, chr=i)
        temp <- subset(cross, chr=i)
        for(j in seq(along=mnames)){
            if(verbose > 1) cat(" ---Marker", j, "of", length(mnames), "\n")
            markerll <- markerloglik(cross, mnames[j], error.prob)

            newmap <- est.map(drop.markers(temp, mnames[j]), error.prob=error.prob,
                              map.function=map.function, m=m, p=p, maxit=maxit, tol=tol,
                              sex.sp=sex.sp)

            if(sexsp) {
                origmaptab[mnames[j],4] <- -(attr(origmap[[i]], "loglik") - markerll - attr(newmap[[1]], "loglik"))/log(10)
                origmaptab[mnames[j],5] <- Lf - diff(range(newmap[[1]][1,]))
                origmaptab[mnames[j],6] <- Lm - diff(range(newmap[[1]][2,]))
            }
            else {
                origmaptab[mnames[j],3] <- -(attr(origmap[[i]], "loglik") - markerll - attr(newmap[[1]], "loglik"))/log(10)
                origmaptab[mnames[j],4] <- L - diff(range(newmap[[1]]))
            }
        }
    }

    class(origmaptab) <- c("scanone", "data.frame")
    origmaptab$chr <- factor(origmaptab$chr, levels=unique(origmaptab$chr))
    origmaptab
}


# end of droponemarker.R

#####################################################################
#
# pull_stuff.R
#
# copyright (c) 2001-2020, Karl W Broman
#     [find.pheno, find.flanking, and a modification to create.map
#      from Brian Yandell]
# last modified Dec, 2020
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: pull.map, pull.geno, pull.pheno
#           pull.genoprob, pull.argmaxgeno
#
######################################################################

######################################################################
#
# pull.map
#
# pull out the map portion of a cross object, as a list
#
######################################################################

pull.map <-
    function(cross, chr, as.table=FALSE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(!missing(chr)) cross <- subset(cross, chr=chr)
    if(!as.table) {
        result <- lapply(cross$geno,function(a) {
            b <- a$map
            class(b) <- chrtype(a)
            b })
        class(result) <- "map"
        return(result)
    } else {
        return(map2table(pull.map(cross, as.table=FALSE)))
    }
}

map2table <-
    function(map, chr)
{
    if(!missing(chr)) {
        chr <- matchchr(chr, names(map))
        map <- map[chr]
    }

    if(is.matrix(map[[1]])) {
        map1 <- unlist(lapply(map, function(a) a[1,]))
        map2 <- unlist(lapply(map, function(a) a[2,]))
        result <- data.frame(chr=rep(names(map), vapply(map, ncol, 0)),
                             pos.female=map1, pos.male=map2, stringsAsFactors=FALSE)
        rownames(result) <- make_unique(unlist(lapply(map, colnames)), "marker names")
    } else {
        result <- data.frame(chr=rep(names(map), vapply(map, length, 0)),
                             pos=unlist(map), stringsAsFactors=FALSE)
        rownames(result) <- make_unique(unlist(lapply(map, names)), "marker names")
    }
    result[,1] <- factor(result[,1], levels=unique(result[,1]))

    result
}

# force a string of names to be unique
# 'label' used if there are mismatches
make_unique <-
    function(nam, label="names")
{
    tab <- table(nam)
    if(any(tab > 1)) {
        warning(label, " are not all distinct; output names adjusted to make them distinct")

        for(marker in names(tab)[tab > 1]) {
            wh <- (nam==marker)
            nam[wh] <- paste0(marker, "_", seq_len(sum(wh)))
        }
    }

    nam
}


######################################################################
# pull.geno
######################################################################
pull.geno <-
    function(cross, chr)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(!missing(chr))
        cross <- subset(cross, chr=chr)

    X <- cross$geno[[1]]$data
    if(nchr(cross) > 1)
        for(i in 2:nchr(cross))
            X <- cbind(X, cross$geno[[i]]$data)
    X
}

######################################################################
# pull.pheno
######################################################################
pull.pheno <-
    function(cross, pheno.col)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    pheno <- cross$pheno

    if(!missing(pheno.col)) {

        if(is.character(pheno.col)) {
            m <- match(pheno.col, names(pheno))
            if(any(is.na(m))) {
                if(sum(is.na(m)) > 1)
                    warning("Phenotypes ", paste("\"", pheno.col[is.na(m)], "\"", sep="", collapse=" "), " not found.")
                else
                    warning("Phenotype ", paste("\"", pheno.col[is.na(m)], "\"", sep="", collapse=" "), " not found.")
            }
            if(all(is.na(m))) return(NULL)

            m <- m[!is.na(m)]
            pheno <- pheno[,m]
        }
        else if(is.logical(pheno.col)) {
            if(length(pheno.col) != ncol(pheno))
                stop("If pheno.col is logical, it should have length ", ncol(pheno))
            pheno <- pheno[,pheno.col]
        }
        else if(is.numeric(pheno.col)) {
            if(any(pheno.col > 0) && any(pheno.col < 0))
                stop("If pheno.col is numeric, values should be all > 0 or all < 0")
            if(any(pheno.col > 0) && (any(pheno.col < 1) || any(pheno.col > ncol(pheno))))
                stop("pheno.col values should be >= 1 and <= ", ncol(pheno))
            if(any(pheno.col < 0) && (any(pheno.col > -1) || any(pheno.col < -ncol(pheno))))
                stop("With negative pheno.col values, they should be between -", ncol(pheno), " and -1")
            pheno <- pheno[,pheno.col]
        }
    }

    if(is.data.frame(pheno) && ncol(pheno) == 1) pheno <- pheno[,1]

    pheno
}

######################################################################
# pull.genoprob
######################################################################
pull.genoprob <-
    function(cross, chr, omit.first.prob=FALSE, include.pos.info=FALSE, rotate=FALSE)
{
    if(!missing(chr))
        cross <- subset(cross, chr=chr)

    if(!("prob" %in% names(cross$geno[[1]])))
        stop("You must first run calc.genoprob.")

    if(include.pos.info && !rotate) {
        warning("If include.pos.info=TRUE, we assume rotate=TRUE as well.")
        rotate <- TRUE
    }

    pr <- lapply(cross$geno, function(a) a$prob)
    chrnames <- names(cross$geno)
    for(i in seq(along=pr)) {
        w <- colnames(pr[[i]])
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",chrnames[i],".",w[o],sep="")
        colnames(pr[[i]]) <- w
    }

    if(omit.first.prob)
        fullncol <- sum(sapply(pr, ncol))*(dim(pr[[1]])[3]-1)
    else
        fullncol <- sum(sapply(pr, ncol))*dim(pr[[1]])[3]

    fullpr <- matrix(nrow=nrow(pr[[1]]), ncol=fullncol)
    colnames(fullpr) <- 1:fullncol
    curcol <- 0
    thegen <- themarker <- rep(NA, fullncol)
    if(include.pos.info)
        thechr <- thepos <- rep(NA, fullncol)
    for(i in seq(along=pr)) {
        dim3 <- 1:dim(pr[[i]])[3]
        if(omit.first.prob) dim3 <- dim3[-1]
        for(j in seq(along=dim3)) {
            thecol <- curcol + ((1:ncol(pr[[i]]))-1)*length(dim3) + j
            fullpr[,thecol] <- pr[[i]][,,dim3[j]]
            thisgen <- dimnames(pr[[i]])[[3]][dim3[j]]
            thegen[thecol] <- rep(thisgen, length(thecol))
            themarker[thecol] <- colnames(pr[[i]])
            colnames(fullpr)[thecol] <- paste(colnames(pr[[i]]), thisgen, sep=":")
            if(include.pos.info) {
                thechr[thecol] <- rep(names(cross$geno)[i], length(thecol))
                map <- attr(pr[[i]], "map")
                if(is.matrix(map)) map <- map[1,] # sex-specific map; take female positions
                thepos[thecol] <- map
            }
        }
        curcol <- curcol + ncol(pr[[i]])*length(dim3)
    }

    id <- getid(cross)
    if(is.null(id)) id <- paste("ind", 1:nrow(fullpr), sep="")
    rownames(fullpr) <- id

    if(rotate) {
        fullpr <- as.data.frame(t(fullpr))
        if(include.pos.info) {
            thechr <- factor(thechr, names(cross$geno))
            fullpr <- cbind(marker=themarker, gen=thegen, chr=thechr, pos=thepos, fullpr, stringsAsFactors=FALSE)
        }
    }

    fullpr
}


######################################################################
# pull.argmaxgeno
######################################################################
pull.argmaxgeno <-
    function(cross, chr, include.pos.info=FALSE, rotate=FALSE)
{
    if(!missing(chr))
        cross <- subset(cross, chr=chr)

    if(!("argmax" %in% names(cross$geno[[1]])))
        stop("You must first run argmax.geno.")

    if(include.pos.info && !rotate) {
        warning("If include.pos.info=TRUE, we assume rotate=TRUE as well.")
        rotate <- TRUE
    }

    am <- lapply(cross$geno, function(a) a$argmax)
    chrnames <- names(cross$geno)
    for(i in seq(along=am)) {
        w <- colnames(am[[i]])
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",chrnames[i],".",w[o],sep="")
        colnames(am[[i]]) <- w
    }

    fullncol <- sum(sapply(am, ncol))

    fullam <- matrix(nrow=nrow(am[[1]]), ncol=fullncol)
    colnames(fullam) <- 1:fullncol
    curcol <- 0
    if(include.pos.info)
        thechr <- thepos <- rep(NA, fullncol)
    for(i in seq(along=am)) {
        thecol <- curcol + 1:ncol(am[[i]])
        fullam[,thecol] <- am[[i]]
        colnames(fullam)[thecol] <- colnames(am[[i]])
        if(include.pos.info) {
            thechr[thecol] <- rep(names(cross$geno)[i], length(thecol))
            map <- attr(am[[i]], "map")
            if(is.matrix(map)) map <- map[1,] # sex-specific map; take female positions
            thepos[thecol] <- map
        }
        curcol <- curcol + length(thecol)
    }

    id <- getid(cross)
    if(is.null(id)) id <- paste("ind", 1:nrow(fullam), sep="")
    rownames(fullam) <- id

    if(rotate) {
        fullam <- as.data.frame(t(fullam))
        if(include.pos.info) {
            thechr <- factor(thechr, names(cross$geno))
            fullam <- cbind(marker=rownames(fullam), chr=thechr, pos=thepos, fullam, stringsAsFactors=FALSE)
        }
    }

    fullam
}

######################################################################
# pull.draws
######################################################################
pull.draws <-
    function(cross, chr)
{
    if(!missing(chr))
        cross <- subset(cross, chr=chr)

    if(!("draws" %in% names(cross$geno[[1]])))
        stop("You must first run sim.geno.")

    dr <- lapply(cross$geno, function(a) a$draws)
    chrnames <- names(cross$geno)
    for(i in seq(along=dr)) {
        w <- colnames(dr[[i]])
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",chrnames[i],".",w[o],sep="")
        colnames(dr[[i]]) <- w
    }

    fullncol <- sum(sapply(dr, ncol))
    d <- dim(dr[[1]])

    fulldr <- array(dim=c(d[1], fullncol, d[3]))
    colnames(fulldr) <- 1:fullncol
    curcol <- 0

    for(i in seq(along=dr)) {
        thecol <- curcol + 1:ncol(dr[[i]])
        fulldr[,thecol,] <- dr[[i]]
        colnames(fulldr)[thecol] <- colnames(dr[[i]])
        curcol <- curcol + length(thecol)
    }

    id <- getid(cross)
    if(is.null(id)) id <- paste("ind", 1:nrow(fulldr), sep="")
    rownames(fulldr) <- id

    fulldr
}

##############################
# table2map: create map object from a table
#
# rownames should be marker names
# first column chromosome
# second column position
##############################
table2map <-
    function(tab)
{
    mar <- rownames(tab)
    if(is.null(mar)) stop("marker names should be the row names")
    chr <- factor(tab[,1], levels=unique(tab[,1]))
    pos <- tab[,2]

    map <- split(pos, chr)
    mar <- split(mar, chr)
    for(i in seq(along=map))
        names(map[[i]]) <- mar[[i]]

    if(all(names(map) %in% c(1:20,"X"))) { # names are as in mouse
        for(i in seq(along=map))
            class(map[[i]]) <- ifelse(names(map)[i]=="X", "X", "A")
    }

    class(map) <- "map"
    map
}


# end of pull_stuff.R

######################################################################
#
# vbscan.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written May, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: vbscan
#
######################################################################

######################################################################
#
# vbscan: scan genome for a quantitative phenotype for which some
# individuals' phenotype is undefined (for example, the size of a
# lesion, where some individuals have no lesion).
#
######################################################################

vbscan <-
    function(cross, pheno.col=1, upper=FALSE, method="em", maxit=4000,
             tol=1e-4)
{
    method <- match.arg(method)
    type <- crosstype(cross)

    # check arguments are okay
    if(length(pheno.col) > 1) pheno.col <- pheno.col[1]
    if(pheno.col > nphe(cross))
        stop("Specified phenotype column exceeds the number of phenotypes")
    y <- cross$pheno[,pheno.col]

    # modify phenotypes
    if(upper) {
        if(!any(y == Inf)) y[y==max(y)] <- Inf
    }
    else {
        if(!any(y == -Inf)) y[y==min(y)] <- -Inf
    }
    survived <- rep(0,length(y))
    survived[y == -Inf | y == Inf] <- 1

    # The following line is included since .C() doesn't accept Infs
    y[y == -Inf | y == Inf] <- 99999

    n.chr <- nchr(cross)
    results <- NULL

    for(i in 1:n.chr) {
        # make sure inferred genotypes or genotype probabilities are available
        if(!("prob" %in% names(cross$geno[[i]]))) {
            cat(" -Calculating genotype probabilities\n")
            cross <- calc.genoprob(cross)
        }

        genoprob <- cross$geno[[i]]$prob
        n.pos <- dim(genoprob)[2]
        n.ind <- length(y)

        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type=="X") sexpgm <- getsex(cross)
        else sexpgm <- NULL

        gen.names <- getgenonames(type,chr_type,"full", sexpgm, attributes(cross))
        n.gen <- length(gen.names)

        # revise X chromosome genotypes
        if(chr_type=="X" && (type=="f2" || type=="bc"))
            genoprob <- reviseXdata(type, "full", sexpgm, prob=genoprob,
                                    cross.attr=attributes(cross))

        z <- .C("R_vbscan",
                as.integer(n.pos),
                as.integer(n.ind),
                as.integer(n.gen),
                as.double(genoprob),
                as.double(y),
                as.integer(survived),
                lod=as.double(rep(0, n.pos*3)),
                as.integer(maxit),
                as.double(tol),
                PACKAGE="qtl")

        if("map" %in% names(attributes(cross$geno[[i]]$prob)))
            map <- attr(cross$geno[[i]]$prob,"map")
        else {
            stp <- attr(cross$geno[[i]]$prob, "step")
            oe <- attr(cross$geno[[i]]$prob, "off.end")

            if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
            else stpw <- "fixed"
            map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
        }

        if(is.matrix(map)) map <- map[1,]

        res <- data.frame(chr=rep(names(cross$geno)[i],length(map)),
                          pos = as.numeric(map),
                          matrix(z$lod,nrow=n.pos,byrow=TRUE), stringsAsFactors=TRUE)

        colnames(res)[-(1:2)] <- c("lod.p.mu","lod.p","lod.mu")

        w <- names(map)
        o <- grep("^loc-*[0-9]+",w)

        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",names(cross$geno)[i],".",w[o],sep="")
        rownames(res) <- w

        z <- res

        # get null log10 likelihood for the X chromosome
        if(chr_type=="X") {

            # determine which covariates belong in null hypothesis
            temp <- scanoneXnull(type, sexpgm, cross.attr=attributes(cross))
            adjustX <- temp$adjustX
            parX0 <- temp$parX0
            sexpgmcovar <- temp$sexpgmcovar
            sexpgmcovar.alt <- temp$sexpgmcovar.alt

            if(adjustX) { # get LOD-score adjustment
                n.gen <- ncol(sexpgmcovar)+1
                genoprob <- matrix(0,nrow=n.ind,ncol=n.gen)
                for(i in 1:n.gen)
                    genoprob[sexpgmcovar.alt==i,i] <- 1

                nullz <- .C("R_vbscan",
                            as.integer(1),
                            as.integer(n.ind),
                            as.integer(n.gen),
                            as.double(genoprob),
                            as.double(y),
                            as.integer(survived),
                            lod=as.double(rep(0,(4+2*n.gen))),
                            as.integer(maxit),
                            as.double(tol),
                            PACKAGE="qtl")

                # adjust LOD curve
                for(i in 1:3) z[,i+2] <- z[,i+2] - nullz$lod[i]
            }
        }

        results <- rbind(results, z)
    }

    class(results) <- c("scanone","data.frame")
    attr(results,"method") <- method
    attr(results,"type") <- crosstype(cross)
    attr(results,"model") <- "twopart"

    results
}

# end of vbscan.R

######################################################################
# phyloqtl_util.R
#
# copyright (c) 2009-2019, Karl W Broman
# last modified Dec, 2019
# first written May, 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Utility functions for the phylo/qtl analyses
#
# Part of the R/qtl package
# Contains: checkPhyloPartition, checkPhyloCrosses, qtlByPartition,
#           flipcross, genAllPartitions, sortPhyloPartitions
#
######################################################################

# check that 'partition' is appropriate for a given set of taxa
checkPhyloPartition <-
    function(partition, taxa)
{
    n.taxa <- length(taxa)

    if(length(partition) > 1) {
        for(i in partition) checkPhyloPartition(i, taxa)
        return(TRUE)
    }

    # check that all the taxa are there and that there is just one "|"
    temp <- unlist(strsplit(partition, ""))
    if(length(temp) != n.taxa+1 || any(is.na(match(c(taxa, "|"), temp))))
        stop("partition is mis-specified; should be a character string with all taxa and one vertical bar (|).")

    # check that "|" is not at one end or the other
    ss <- unlist(strsplit(partition, "\\|"))
    if(length(ss) != 2 || any(nchar(ss)==0))
        stop("partition is mis-specified; vertical bar (|) should be somewhere in the middle.")

    TRUE
}

# check that the crosses are correct
checkPhyloCrosses <-
    function(crosses, taxa, nostop=FALSE)
{
    temp <- strsplit(crosses, "")
    if(any(is.na(match(unlist(temp), taxa)))) {
        temp <- unlist(temp)
        extras <- unique(temp[is.na(match(temp, taxa))])
        if(nostop) return(FALSE)
        stop("Crosses mis-specified; have extra taxa, ", paste(extras, collapse=" "))
    }
    crosses <- sapply(temp, paste, collapse="")
    if(length(crosses) != length(unique(crosses))) {
        warning("Some crosses given multiple times; these are omitted.")
        crosses <- unique(crosses)
        temp <- strsplit(crosses, "")
    }

    # all taxa included in the crosses?
    m <- is.na(match(taxa, unique(unlist(temp))))
    if(any(m)) {
        temp <- paste(taxa[m], collapse=" ")
        if(nostop) return(FALSE)
        stop("Some taxa missing from the crosses: ", temp)
    }

    # check that the crosses connect all n taxa
    thetaxa <- as.list(taxa)
    for(i in seq(along=temp)) {
        wh1 <- which(sapply(thetaxa, function(a,b) any(a==b), temp[[i]][1]))
        wh2 <- which(sapply(thetaxa, function(a,b) any(a==b), temp[[i]][2]))
        if(wh1 != wh2) {
            thetaxa[[wh1]] <- c(thetaxa[[wh1]], thetaxa[[wh2]])
            thetaxa <- thetaxa[-wh2]
        }
    }
    if(length(thetaxa) > 1) {
        if(nostop) return(FALSE)
        stop("The crosses are insufficient; they should connect all taxa.")
    }

    if(nostop) return(TRUE)
    crosses
}


######################################################################
# qtlByPartition
#
# for each cross and each partition, determine which crosses have a
# QTL and whether the alleles need to be swapped
######################################################################
qtlByPartition <-
    function(crosses, partition)
{
    # check for multiple crosses (of the form "AB:AC:AD")
    if(length(grep(":", crosses)) > 0) {
        result <- lapply(strsplit(crosses, ":"), qtlByPartition, partition)
        names(result) <- crosses
        return(result)
    }

    # for each partition, determine which crosses have the QTL
    crossmat <- matrix(NA, ncol=length(partition), nrow=length(crosses))
    crosssplit <- strsplit(crosses, "")
    partitionsplit <- lapply(strsplit(partition, "\\|"), strsplit, "")
    for(i in seq(along=crosses)) {
        for(j in seq(along=partition)) {
            v <- vector("list", 2)
            for(k in 1:2)
                v[[k]] <- sapply(partitionsplit[[j]], function(a,b) match(b, a), crosssplit[[i]][k])
            crossmat[i,j] <- diff(sapply(v, function(a) which(!is.na(a))))
        }
    }
    dimnames(crossmat) <- list(crosses, partition)
    crossmat
}


######################################################################
# flipcross
#
# The goal of this function is to take a QTL cross object (for R/qtl)
# and flip the alleles A <-> B.
#
# For the case of an intercross, the allele codes (and corresponding
# QTL genotype probabilities and/or imputated genotypes) are switched
# as follows:
#
# genotype   old code    new code
#   AA          1           3
#   AB          2           2
#   BB          3           1
#  not BB       4           5
#  not AA       5           4
######################################################################

flipcross <-
    function(cross)
{
    if(!inherits(cross, "cross"))
        stop("The input should have class 'cross'")
    allowed_crosses <- c("f2", "riself", "risib", "dh", "haploid")
    crosstype <- crosstype(cross)
    if(!(crosstype %in% allowed_crosses))
        stop("The function is not working for cross type ", crosstype)

    chr_type <- sapply(cross$geno, chrtype)

    # omit X chr
    if(any(chr_type=="X")) {
        cross <- subset(cross, chr = (chr_type != "X"))
        warning("flipcross is not yet working for the X chromosome; X chr omitted from output.")
    }


    if(crosstype == "f2") {

        for(i in seq(along=cross$geno)) {
            nd <- d <- cross$geno[[i]]$data
            nd[!is.na(d) & d==1] <- 3
            nd[!is.na(d) & d==3] <- 1
            nd[!is.na(d) & d==4] <- 5
            nd[!is.na(d) & d==5] <- 4
            cross$geno[[i]]$data <- nd

            if("prob" %in% names(cross$geno[[i]])) {
                theattr <- attributes(cross$geno[[i]]$prob)
                cross$geno[[i]]$prob <- cross$geno[[i]]$prob[,,3:1,drop=FALSE]
                attr(cross$geno[[i]]$prob,"map") <- theattr$map
                attr(cross$geno[[i]]$prob,"error.prob") <- theattr$error.prob
                attr(cross$geno[[i]]$prob,"step") <- theattr$step
                attr(cross$geno[[i]]$prob,"off.end") <- theattr$off.end
                attr(cross$geno[[i]]$prob,"map.function") <- theattr$map.function
                attr(cross$geno[[i]]$prob,"stepwidth") <- theattr$stepwidth
            }

            if("draws" %in% names(cross$geno[[i]])) {
                nd <- d <- cross$geno[[i]]$draws
                nd[d==3] <- 1
                nd[d==1] <- 3
                cross$geno[[i]]$draws <- nd
            }
        }
    }
    else { # riself/risib/dh/haploid

        for(i in seq(along=cross$geno)) {
            nd <- d <- cross$geno[[i]]$data
            nd[!is.na(d) & d==1] <- 2
            nd[!is.na(d) & d==2] <- 1

            if("prob" %in% names(cross$geno[[i]])) {
                theattr <- attributes(cross$geno[[i]]$prob)
                cross$geno[[i]]$prob <- cross$geno[[i]]$prob[,,2:1,drop=FALSE]
                attr(cross$geno[[i]]$prob,"map") <- theattr$map
                attr(cross$geno[[i]]$prob,"error.prob") <- theattr$error.prob
                attr(cross$geno[[i]]$prob,"step") <- theattr$step
                attr(cross$geno[[i]]$prob,"off.end") <- theattr$off.end
                attr(cross$geno[[i]]$prob,"map.function") <- theattr$map.function
                attr(cross$geno[[i]]$prob,"stepwidth") <- theattr$stepwidth
            }

            if("draws" %in% names(cross$geno[[i]])) {
                nd <- d <- cross$geno[[i]]$draws
                nd[d==2] <- 1
                nd[d==1] <- 2
                cross$geno[[i]]$draws <- nd
            }
        }
    }

    if("alleles" %in% names(attributes(cross)))
        attr(cross, "alleles") <- rev(attr(cross, "alleles"))

    cross
}

# generate all possible partitions (except the null)
genAllPartitions <-
    function(n.taxa, taxa)
{

    # Utility function
    #     returns binary representation of 1:(2^n)
    binary.v <-
        function(n)
        {
            x <- 1:(2^n)
            mx <- max(x)
            digits <- floor(log2(mx))
            ans <- 0:(digits-1); lx <- length(x)
            x <- matrix(rep(x,rep(digits, lx)),ncol=lx)
            (x %/% 2^ans) %% 2
        }

    if(missing(taxa))
        taxa <- LETTERS[1:n.taxa]
    else {
        if(missing(n.taxa)) n.taxa <- length(taxa)
        else {
            if(n.taxa != length(taxa))
                stop("n.taxa != length(taxa)")
        }
    }

    mat <- binary.v(n.taxa)
    colsum <- apply(mat, 2, sum)
    mat <- mat[,colsum > 0 & colsum <= floor(n.taxa/2)]

    result <- unique(apply(mat, 2, function(a,b) {
        x <- c(paste(b[a==1],collapse=""), paste(b[a==0],collapse=""))
        if(diff(nchar(x)) == 0) x <- sort(x)
        paste(x, collapse="|")}, taxa))

    sortPhyloPartitions(result, taxa)
}


sortPhyloPartitions <-
    function(partitions, taxa)
{
    if(missing(taxa)) taxa <- LETTERS[1:26]

    thesplit <- strsplit(partitions, "\\|")
    n1 <- sapply(thesplit, function(a) nchar(a[1]))
    c1 <- sapply(thesplit, function(a) a[1])
    c1 <- strsplit(c1, "")
    for(i in seq(along=c1))
        c1[[i]] <- as.numeric(paste(match(c1[[i]], taxa), collapse=""))
    c1 <- unlist(c1)
    partitions[order(n1, c1)]
}

# end of phyloqtl_util.R

#####################################################################
#
# mqmscan.R
#
# Copyright (c) 2009-2019, Danny Arends
#
# Modified by Pjotr Prins and Karl Broman
#
#
# first written Februari 2009
# last modified Dec 2019
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: mqmscan
#
#
#####################################################################

######################################################################
#
# mqmscan: main scanning function to the mqmpackage
#
######################################################################

mqmscan <- function(cross,cofactors=NULL,pheno.col=1,model=c("additive","dominance"),forceML=FALSE,
                    cofactor.significance=0.02,em.iter=1000,window.size=25.0,step.size=5.0,logtransform = FALSE,
                    estimate.map = FALSE,plot=FALSE,verbose=FALSE, outputmarkers=TRUE, multicore=TRUE, batchsize=10, n.clusters=1,
                    test.normality=FALSE,off.end=0)
{

    start <- proc.time()
    model <- match.arg(model)

    # omit X chromosome
    cross <- omit_x_chr(cross)

    #Because iirc we cannot pass booleans from R to C
    if(forceML){
        forceML <- 1           #1 -> Maximum Likelyhood
    }else{
        forceML <- 0           #0 -> Restricted Maximum Likelyhood
    }
    dominance <- 0            #We code :0 -> Additive model (no_dominance)
    if(model=="dominance"){
        dominance <- 1          #and 1 -> Dominance model
    }
    if(estimate.map){
        estimate.map <- 1
    }else{
        estimate.map <- 0
    }
    n.run <- 0
    if(is.null(cross)){
        stop("No cross file. Please supply a valid cross object.")
    }
    crosstype <- crosstype(cross)
    if(crosstype == "f2" || crosstype == "bc" || crosstype == "riself"){
        if(crosstype == "f2"){
            ctype = 1
        }
        if(crosstype == "bc" || crosstype=="dh" || crosstype=="haploid"){
            ctype = 2
        }
        if(crosstype == "riself") {
            ctype = 3

            # check genotypes
            g <- as.numeric(pull.geno(cross))
            g <- sort(unique(g[!is.na(g)]))
            if(max(g)==2) { # convert genotypes from 1/2 to 1/3
                for(i in seq(along=cross$geno))
                    cross$geno[[i]]$data[!is.na(cross$geno[[i]]$data) & cross$geno[[i]]$data==2] <- 3
            }
        }
        n.ind <- nind(cross)
        n.chr <- nchr(cross)
        if(verbose) {
            cat("INFO: Received a valid cross file type:",crosstype,".\n")
            cat("INFO: Number of individuals: ",n.ind,"\n")
            cat("INFO: Number of chromosomes: ",n.chr,"\n")
        }
        savecross <- cross
        geno <- NULL
        chr <- NULL
        dist <- NULL
        newcmbase <- NULL
        out.qtl <- NULL
        for(i in 1:n.chr) {
            #We always shift the marker positions to 0
            geno <- cbind(geno,cross$geno[[i]]$data)
            chr <- c(chr,rep(i,dim(cross$geno[[i]]$data)[2]))
            newcmbase = c(newcmbase,min(cross$geno[[i]]$map))
            cross$geno[[i]]$map <- cross$geno[[i]]$map-(min(cross$geno[[i]]$map))
            dist <- c(dist,cross$geno[[i]]$map)
        }
        if(cofactor.significance <=0 || cofactor.significance >= 1){
            stop("cofactor.significance must be between 0 and 1.\n")
        }
        if(any(is.na(geno))){
            stop("Missing genotype information, please estimate unknown data, before running mqmscan.\n")
        }
        if(missing(cofactors)) cofactors <- rep(0,sum(nmar(cross)))
        numcofold <- sum(cofactors)
        cofactors <- checkdistances(cross,cofactors,1)
        numcofnew <- sum(cofactors)
        if(numcofold!=numcofnew){
            cat("INFO: Removed ",numcofold-numcofnew," cofactors that were close to eachother\n")
        }
        #Convert phenotypes from character to numeric
        pheno.col = stringPhenoToInt(cross,pheno.col)

        if (length(pheno.col) > 1){
            cross$pheno <- cross$pheno[,pheno.col]   #Scale down the triats
            result <- mqmscanall( cross,cofactors=cofactors,forceML=forceML,model=model,
                                 cofactor.significance=cofactor.significance,step.size=step.size,window.size=window.size,
                                 logtransform=logtransform, estimate.map = estimate.map,plot=plot, verbose=verbose,n.clusters=n.clusters,batchsize=batchsize)
            return(result)
        }
        if(pheno.col != 1){
            if(verbose) {
                cat("INFO: Selected phenotype ",pheno.col,".\n")
                cat("INFO: Number of phenotypes in object ",nphe(cross),".\n")
            }
            if(nphe(cross) < pheno.col || pheno.col < 1){
                stop("No such phenotype in cross object.\n")
            }
        }
        if(test.normality && !mqmtestnormal(cross, pheno.col, 0.01, FALSE)){
            warning("Trait might not be normal (Shapiro normality test)\n")
        }
        pheno <- cross$pheno[,pheno.col]
        phenovar = var(pheno,na.rm = TRUE)
        if(phenovar > 1000){
            if(!logtransform){
                if(verbose) cat("INFO: Before LOG transformation Mean:",mean(pheno,na.rm = TRUE),"variation:",var(pheno,na.rm = TRUE),".\n")
                #warning(paste("WARNING: Set needs Log-transformation? (var=",phenovar,"), use mqmscan with logtransform=TRUE"))
            }
        }
        if(logtransform){
            #transform the cross file
            cross <- transformPheno(cross,pheno.col,transf=log)
            pheno <- cross$pheno[,pheno.col]
        }
        n.mark <- ncol(geno)
        if(verbose) cat("INFO: Number of markers:",n.mark,"\n")

        #check for missing phenotypes
        dropped <- NULL
        droppedIND <- NULL
        for(i in 1:length(pheno)) {
            if(is.na(pheno[i]) || is.infinite(pheno[i])){
                if(verbose) cat("INFO: Dropped individual ",i," with missing phenotype.\n")
                dropped <- c(dropped,i)
                if(!is.null(cross$mqm)){
                    droppedIND <- c(droppedIND,cross$mqm$augIND[i])
                }
                n.ind = n.ind-1
            }
        }
        #Throw out missing phenotypes from phenotype vector and genotype matrix
        if(!is.null(dropped)){
            geno <- geno[-dropped,]
            pheno <- pheno[-dropped]
        }

        #CHECK for previously augmented dataset
        if(!is.null(cross$mqm)){
            augmentedNind <- cross$mqm$Nind
            augmentedInd <- cross$mqm$augIND
            if(verbose){
                cat("n.ind:",n.ind,"\n")
                cat("augmentedNind:",augmentedNind,"\n")
                cat("length(augmentedInd):",length(augmentedInd),"\n")
            }
            if(!is.null(dropped)){
                augmentedInd <- cross$mqm$augIND[-dropped]
                augmentedInd[1] <- 0
                for(x in 1:(length(augmentedInd)-1)){
                    if(augmentedInd[x+1] - 1 > augmentedInd[x] ){
                        for(y in (x+1):length(augmentedInd)){
                            augmentedInd[y] <- augmentedInd[y] - ((augmentedInd[x+1] - augmentedInd[x])-1)
                        }
                    }
                }
            }
            augmentedNind <- length(unique(augmentedInd))
            if(verbose){
                cat("New augmentedNind:",augmentedNind,"\n")
                cat("New length(augmentedInd):",length(augmentedInd),"\n")
            }
        }else{
            #No augmentation
            augmentedNind <- n.ind
            augmentedInd <- 0:n.ind
        }

        #CHECK if we have cofactors, so we can do backward elimination
        backward <- 0;
        if(missing(cofactors)){
            if(verbose) cat("INFO: No cofactors, setting cofactors to 0\n")
            cofactors = rep(0,n.mark)
        }else{
            if(length(cofactors) != n.mark){
                if(verbose) cat("ERROR: # Cofactors != # Markers\n")
                }else{
                    if(verbose) cat("INFO:",sum(cofactors!=0),"Cofactors received to be analyzed\n")
                    if((sum(cofactors) > n.ind-10 && dominance==0)){
                        stop("Cofactors don't look okay for use without dominance\n")
                    }
                    if((sum(cofactors)*2 > n.ind-10 && dominance==1)){
                        stop("Cofactors don't look okay for use with dominance\n")
                    }
                    if(sum(cofactors) > 0){
                        if(verbose) cat("INFO: Doing backward elimination of selected cofactors.\n")
                        backward <- 1;
                        n.run <- 0;
                    }else{
                        backward <- 0;
                        cofactors = rep(0,n.mark)
                    }
                }
        }
        step.min = -off.end;
        step.max = max(dist)+step.size+off.end;
        step.max <- as.integer(ceiling((step.max+step.size)/step.size)*step.size)
        if((step.min+step.size) > step.max){
            stop("step.max needs to be >= step.min + step.size")
        }
        #    cat("Step.min:",step.min," Step.max:",step.max,"\n")
        if(step.size < 1){
            stop("step.size needs to be >= 1")
        }
        qtlAchromo <- length(seq(step.min,step.max,step.size))
        if(verbose) cat("INFO: Number of locations per chromosome: ",qtlAchromo, "\n")
        end.1 <- proc.time()
        result <- .C("R_mqmscan",
                     as.integer(n.ind),
                     as.integer(n.mark),
                     as.integer(1),    # 1 phenotype
                     as.integer(geno),
                     as.integer(chr),
                     DIST=as.double(dist),
                     as.double(pheno),
                     COF=as.integer(cofactors),
                     as.integer(backward),
                     as.integer(forceML),
                     as.double(cofactor.significance),
                     as.integer(em.iter),
                     as.double(window.size),
                     as.double(step.size),
                     as.double(step.min),
                     as.double(step.max),
                     as.integer(n.run),
                     as.integer(augmentedNind),
                     as.integer(augmentedInd),
                     QTL=as.double(rep(0,2*n.chr*qtlAchromo)),
                     as.integer(estimate.map),
                     as.integer(ctype),
                     as.integer(dominance),
                     as.integer(verbose),
                     PACKAGE="qtl")
        end.2 <- proc.time()
        # initialize output object
        qtl <- NULL
        info <- NULL
        names <- NULL
        for(i in 1:(n.chr*qtlAchromo)) {
            #Store the result in the qtl object
            qtl <- rbind(qtl,c(ceiling(i/qtlAchromo),rep(seq(step.min,step.max,step.size),n.chr)[i],result$QTL[i]))
            info <- rbind(info,result$QTL[(n.chr*qtlAchromo)+i])
            #make names in the form: cX.locXX
            names <- c(names,paste("c",ceiling(i/qtlAchromo),".loc",rep(seq(step.min,step.max,step.size),n.chr)[i],sep=""))
        }
        if(estimate.map){
            new.map <- pull.map(cross)
            chrmarkers <- nmar(cross)
            sum <- 1
            for(i in 1:length(chrmarkers)) {
                for(j in 1:chrmarkers[[i]]) {
                    new.map[[i]][j] <- result$DIST[sum]
                    sum <- sum+1
                }
            }
        }
        if(plot){
            if(estimate.map && backward){
                op <- par(mfrow = c(3,1))
            }else{
                if(estimate.map || backward){
                    op <- par(mfrow = c(2,1))
                }else{
                    op <- par(mfrow = c(1,1))
                }
            }
            if(estimate.map){
                if(verbose) cat("INFO: Viewing the user supplied map versus genetic map used during analysis.\n")
                plotMap(pull.map(cross), new.map,main="Supplied map versus re-estimated map")
            }
        }
        if(backward){
            if(!estimate.map){
                new.map <- pull.map(cross)
            }
            chrmarkers <- nmar(cross)
            mapnames <- NULL
            for(x in 1:nchr(cross)){
                mapnames <- c(mapnames,names(pull.map(cross)[[x]]))
            }
            sum <- 1
            model.present <- 0
            qc <- NULL
            qp <- NULL
            qn <- NULL
            for(i in 1:length(chrmarkers)) {
                for(j in 1:chrmarkers[[i]]) {
                    #cat("INFO ",sum," ResultCOF:",result$COF[sum],"\n")
                    if(result$COF[sum] != 48){
                        if(verbose) cat("MODEL: Marker",sum,"named:", strsplit(names(unlist(new.map)),".",fixed=TRUE)[[sum]][2],"from model found, CHR=",i,",POSITION=",as.double(unlist(new.map)[sum])," cM\n")
                        qc <- c(qc, as.character(names(cross$geno)[i]))
                        qp <- c(qp, as.double(unlist(new.map)[sum]))
                        qn <- c(qn, mapnames[sum])
                        model.present <- 1
                    }
                    sum <- sum+1
                }
            }
            if(!is.null(qc) && model.present){
                why <- sim.geno(savecross,n.draws=1)
                QTLmodel <- makeqtl(why, qc, qp, qn, what="draws")
                attr(QTLmodel,"mqm") <- 1
                if(plot) plot(QTLmodel)
            }
        }
        rownames(qtl) <- names

        qtl <- cbind(qtl,1/(min(info))*(info-min(info)))
        qtl <- cbind(qtl,1/(min(info))*(info-min(info))*qtl[,3])
        colnames(qtl) = c("chr","pos (cM)",paste("LOD",colnames(cross$pheno)[pheno.col]),"info","LOD*info")
        #Convert to data/frame and scanone object so we can use the standard plotting routines
        qtl <- as.data.frame(qtl, stringsAsFactors=TRUE)
        if(backward && !is.null(qc) && model.present){
            attr(qtl,"mqmmodel") <- QTLmodel
            cimcovar <- as.data.frame(cbind(as.numeric(attr(qtl,"mqmmodel")[[4]]),as.data.frame(attr(qtl,"mqmmodel")[[5]])), stringsAsFactors=TRUE)
            rownames(cimcovar) <- attr(qtl,"mqmmodel")[[2]]
            colnames(cimcovar) <- c("chr","pos")
            attr(qtl, "marker.covar.pos") <- cimcovar
        }
        class(qtl) <- c("scanone",class(qtl))
        if(outputmarkers){
            #Remove pseudomarkers from the dataset and scale to the chromosome
            #Somewhat longer then off-end to be able to put back the original markers
            for( x in 1:nchr(cross)){
                to.remove <- NULL
                chr.length <- max(cross$geno[[x]]$map)
                markers.on.chr <- which(qtl[,1]==x)
                to.remove <- markers.on.chr[which(qtl[markers.on.chr,2] > chr.length+off.end+(2*step.size))]
                to.remove <- c(to.remove,markers.on.chr[which(qtl[markers.on.chr,2] < -off.end)])
                if(length(to.remove) > 0) qtl <- qtl[-to.remove,]
            }
            qtl <- addmarkerstointervalmap(cross,qtl)
            qtl <- as.data.frame(qtl, stringsAsFactors=TRUE)
            if(backward && !is.null(qc) && model.present){
                attr(qtl,"mqmmodel") <- QTLmodel
                cimcovar <- as.data.frame(cbind(as.numeric(attr(qtl,"mqmmodel")[[4]]),as.data.frame(attr(qtl,"mqmmodel")[[5]])), stringsAsFactors=TRUE)
                rownames(cimcovar) <- attr(qtl,"mqmmodel")[[2]]
                colnames(cimcovar) <- c("chr","pos")
                attr(qtl, "marker.covar.pos") <- cimcovar
            }
            class(qtl) <- c("scanone",class(qtl))
        }
        #Now we handle the off-end and any shifts comming from that (newcmbase)
        #We didn't thow away the old names, and could thus get duplicates
        qtlnew <- qtl
        rownames(qtlnew) <- paste0("qtlnew_X", 1:nrow(qtl))
        oldnames <- rownames(qtl)
        for(x in 1:n.chr){
            #Do per chromosome
            markers.on.chr <- which(qtl[,1]==x)
            if(newcmbase[x] !=0 && nrow(qtl[markers.on.chr,]) > 0){
                qtl[markers.on.chr,2] <- qtl[markers.on.chr,2]+newcmbase[x]
                #We end up with 3 posibilities
                for(n in 1:nrow(qtl[markers.on.chr,])){
                    name <- rownames(qtl[markers.on.chr,])[n]
                    id <- which(name==oldnames)
                    if(!is.na(name) && grepl(".loc",name,fixed=TRUE)){
                        #a marker we need to shift
                        rownames(qtlnew)[id] <- paste(strsplit(name,".loc",fixed=TRUE)[[1]][1],".loc",qtlnew[id,2],sep="")
                    }else{
                        #a marker with a user defined name, no need to shift
                        rownames(qtlnew)[id] <- name
                    }
                }
            }else{
                #No shift for the chromosome so we don't have to worry about shifting
                rownames(qtlnew)[markers.on.chr] <- rownames(qtl)[markers.on.chr]
            }
        }
        qtl <- qtlnew

        for( x in 1:nchr(cross)){
            #Remove the off ends that we left hanging a step before
           if(chrtype(cross$geno[[x]])!="X"){
                to.remove <- NULL
                chr.length <- max(cross$geno[[x]]$map)
                markers.on.chr <- which(qtl[,1]==x)
                to.remove <- markers.on.chr[which(qtl[markers.on.chr,2] > chr.length+off.end)]
                to.remove <- c(to.remove,markers.on.chr[which(qtl[markers.on.chr,2] < -off.end)])
                qtl <- qtl[-to.remove,]
            }
        }

        qtl[,1] <- factor(names(cross$geno)[qtl[,1]], levels=names(cross$geno))

        #Plot the results if the user asked for it
        if(plot){
            info.c <- qtl
            #Check for errors in the information content IF err we can't do a second plot
            e <- 0
            for(i in 1:ncol(qtl)){
                if(is.na(info.c[i,5])){
                    e<- 1
                }
                if(is.infinite(info.c[i,5])){
                    e<- 1
                }
                if(is.null(info.c[i,5])){
                    e<- 1
                }
            }
            #No error do plot 2
            if(!e){
                mqmplot.singletrait(qtl,main=paste(colnames(cross$pheno)[pheno.col],"at significance=",cofactor.significance))
            }else{
                plot(qtl,main=paste(colnames(cross$pheno)[pheno.col],"at significance=",cofactor.significance),lwd=1)
                grid(max(qtl$chr),5)
                labels <- paste("QTL",colnames(cross$pheno)[pheno.col])
                legend("topright", labels,col=c("black"),lty=c(1))
            }
            op <- par(mfrow = c(1,1))
        }

        end.3 <- proc.time()
        if(verbose) cat("INFO: Calculation time (R->C,C,C-R): (",round((end.1-start)[3], digits=3), ",",round((end.2-end.1)[3], digits=3),",",round((end.3-end.2)[3], digits=3),") (in seconds)\n")

        qtl
    }else{
        stop("Currently only F2, BC, and selfed RIL crosses can be analyzed by MQM.")
    }
}

# end of mqmscan.R

######################################################################
#
# cim.R
#
# copyright (c) 2007-2021, Karl W Broman
# last modified Jun, 2021
# first written Jan, 2007
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: cim, markerforwsel, markerforwself2, expandf2covar
#
######################################################################

######################################################################
# CIM by first doing forward selection at the markers (with filled-in
# data) to a fixed number of markers, followed by interval mapping with
# the selected markers as covariates, dropping marker covariates if
# they are within some fixed window of the location under test.
######################################################################
cim <-
    function(cross, pheno.col=1, n.marcovar=3, window=10,
             method=c("em", "imp", "hk", "ehk"),
             imp.method=c("imp", "argmax"), error.prob=0.0001,
             map.function=c("haldane", "kosambi", "c-v", "morgan"),
             addcovar=NULL, n.perm)
{
    method <- match.arg(method)
    imp.method <- match.arg(imp.method)
    map.function <- match.arg(map.function)

    if(!is.null(addcovar)) {
        if(!is.matrix(addcovar)) addcovar <- as.matrix(addcovar)
        if(nrow(addcovar) != nind(cross)) {
            stop("nrow(addcovar) != no. individuals in cross")
        }
    }

    type <- crosstype(cross)
    if(type=="4way")
        stop("cim() has not been implemented for 4-way crosses")

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(any(is.na(num))) {
            if(sum(is.na(num)) > 1)
                stop("Couldn't identify phenotypes ", paste(paste("\"", pheno.col[is.na(num)], "\"", sep=""),
                                                            collapse=" "))
            else
                stop("Couldn't identify phenotype \"", pheno.col[is.na(num)], "\"")
        }
        pheno.col <- num
    }

    if(any(pheno.col < 1 | pheno.col > nphe(cross)))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    y <- cross$pheno[,pheno.col]
    if(any(is.na(y))) {
        cross <- subset(cross, ind=(!is.na(y)))
        y <- y[!is.na(y)]
    }

    if(!missing(n.perm) && n.perm > 0) {
        results <- matrix(ncol=1, nrow=n.perm)
        for(i in 1:n.perm) {
            o <- sample(length(y))
            y <- y[o]
            cross$pheno[,pheno.col] <- y
            temp <- cim(cross, pheno.col=pheno.col, n.marcovar=n.marcovar,
                        window=window, method=method, imp.method=imp.method,
                        error.prob=error.prob, map.function=map.function,
                        addcovar=addcovar)
            results[i,1] <- max(temp[,3], na.rm=TRUE)
        }
        class(results) <- c("scanoneperm", "matrix")
        return(results)
    }

    window <- window/2 # window specifies twice the distance between marker and test position

    g <- pull.geno(cross)

    if(any(is.na(g)))
        g <- pull.geno(fill.geno(cross, method=imp.method, error.prob=error.prob,
                                 map.function=map.function))

    if(type=="f2") out.forw <- markerforwself2(g, y, n.marcovar)
    else out.forw <- markerforwsel(g, y, n.marcovar)

    mar <- colnames(g)[out.forw]
    chrpos <- find.markerpos(cross, mar)

    ac <- g[,mar,drop=FALSE]

    if(type=="f2") useac <- expandf2covar(ac)
    else useac <- ac

    firstscan <- scanone(cross, pheno.col=pheno.col, addcovar=cbind(useac,addcovar),
                         method=method)

    # scan again, dropping one marker covariate at a time
    for(i in seq(along=mar)) {
        if(type=="f2") useac <- expandf2covar(ac[,-i])
        else useac <- ac[,-i]

        temp <- scanone(cross, pheno.col=pheno.col, addcovar=cbind(useac,addcovar),
                        method=method, chr=chrpos[i,1])
        wh1 <- (firstscan[,1]==chrpos[i,1] & firstscan[,2] >= chrpos[i,2]-window &
                firstscan[,2] <= chrpos[i,2]+window)
        wh2 <- (temp[,2] >= chrpos[i,2]-window & temp[,2] <= chrpos[i,2] + window)

        firstscan[wh1,3] <- temp[wh2,3]
    }

    attr(firstscan, "marker.covar") <- mar
    attr(firstscan, "marker.covar.pos") <- chrpos

    u <- table(chrpos[,1])
    if(any(u>1)) { # chromosomes with multiple marker covariates
        u <- names(u)[u>1]

        for(j in u) {
            wh <- which(chrpos[,1]==j)
            pos <- chrpos[wh,2] # positions of the covariates on this chromosome

            scanpos <- firstscan[firstscan[,1]==j,2] # positions at which the genome scan is performed

            # matrix indicating, for each position, whether marker covariates need to be dropped
            need2drop <- t(sapply(scanpos, function(a,b,d) as.numeric(abs(a-b) <= d), pos, window))
            n2drop <- apply(need2drop, 1, sum)
            if(any(n2drop > 1)) {
                pat2drop <- apply(need2drop, 1, paste, collapse="")
                thepat <- unique(pat2drop[n2drop > 1])

                for(k in thepat) {
                    whpos <- which(pat2drop==k)
                    whpos2 <- (firstscan[,1]==j & !is.na(match(firstscan[,2], scanpos[whpos])))

                    tempac <- ac[,-wh[as.logical(need2drop[whpos[1],])]]

                    if(type=="f2") useac <- expandf2covar(tempac)
                    else useac <- tempac
                    temp <- scanone(cross, pheno.col=pheno.col, addcovar=cbind(useac,addcovar),
                                    method=method, chr=j)

                    firstscan[whpos2,3] <- temp[whpos,3]
                }
            }

        }
    }
    firstscan
}

######################################################################
# Simple forward selection to a fixed number of covariates
#
# x = matrix of covariates
# y = outcome
# maxsize = maximum size of model
#
# output: indices of chosen covariates [1, 2, ..., ncol(x)]
######################################################################
markerforwsel <-
    function(x, y, maxsize=7)
{
    if(length(y) != nrow(x))
        stop("Need length(y) == nrow(x).")

    if(maxsize < 0 || maxsize > ncol(x))
        stop("Need maxsize between 1 and ncol(x).")

    out <- .C("R_markerforwsel",
              as.integer(nrow(x)),
              as.integer(ncol(x)),
              as.double(x),
              as.double(y),
              as.integer(maxsize),
              chosen=as.integer(rep(0,maxsize)),
              rss=as.double(rep(0,maxsize)),
              PACKAGE="qtl")

    #  out$chosen+1
    temp <- out$chosen+1
    attr(temp, "rss") <- out$rss
    temp
}

######################################################################
# The same as markerforwsel, but for an intercross, in which
# we need to expand each marker to two columns and do selection
# with those pairs of columns
######################################################################
markerforwself2 <-
    function(x, y, maxsize=7)
{
    if(length(y) != nrow(x))
        stop("Need length(y) == nrow(x).")

    if(maxsize < 0 || maxsize > ncol(x))
        stop("Need maxsize between 1 and ncol(x).")

    out <- .C("R_markerforwself2",
              as.integer(nrow(x)),
              as.integer(ncol(x)),
              as.integer(x),
              as.double(y),
              as.integer(maxsize),
              chosen=as.integer(rep(0,maxsize)),
              rss=as.double(rep(0,maxsize)),
              PACKAGE="qtl")

    #  out$chosen+1
    temp <- out$chosen+1
    attr(temp, "rss") <- out$rss
    temp
}

######################################################################
# expand covariates for F2
######################################################################
expandf2covar <-
    function(thecovar)
{
    if(is.null(thecovar) || (is.matrix(thecovar) && ncol(thecovar)==0)) return(NULL)
    if(!is.matrix(thecovar))
        return(cbind((thecovar==3) - (thecovar==1),
                     as.numeric(thecovar==2)))

    revcovar <- matrix(ncol=ncol(thecovar)*2, nrow=nrow(thecovar))
    for(i in 1:ncol(thecovar)) {
        revcovar[,i*2-1] <- (thecovar[,i]==3) - (thecovar[,i]==1)
        revcovar[,i*2] <- as.numeric(thecovar[,i]==2)
    }

    revcovar
}

# end of cim.R

######################################################################
#
# read.cross.karl.R
#
# copyright (c) 2000-2019, Karl W Broman
# last modified Dec, 2019
# first written Aug, 2000
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.karl
#           [See read.cross.R for the main read.cross function.]
#
######################################################################

######################################################################
#
# read.cross.karl
#
# read data in Karl's format
#
######################################################################

read.cross.karl <-
    function(dir,genfile,mapfile,phefile)
{
    # create file names
    if(missing(genfile)) genfile <- "gen.txt"
    if(missing(mapfile)) mapfile <- "map.txt"
    if(missing(phefile)) phefile <- "phe.txt"

    if(!missing(dir) && dir != "") {
        genfile <- file.path(dir, genfile)
        mapfile <- file.path(dir, mapfile)
        phefile <- file.path(dir, phefile)
    }

    # read data
    geno <- as.matrix(read.table(genfile,na.strings="0"))
    pheno <- as.matrix(read.table(phefile,na.strings="-",header=TRUE))
    tempmap <- scan(mapfile, what=character(),quiet=TRUE)

    # fix up map information
    # number of chromosomes
    n.chr <- as.numeric(tempmap[1])
    n.mar <- 1:n.chr
    g <- map <- geno.data <- vector("list", n.chr)
    cur <- 2
    min.mar <- 1
    names(g) <- as.character(1:n.chr)
    for(i in 1:n.chr) { # loop over chromosomes
        # number of markers
        n.mar[i] <- as.numeric(tempmap[cur])
        cur <- cur+1

        # pull out appropriate portion of genotype data
        geno.data[[i]] <- geno[,min.mar:(min.mar+n.mar[i]-1)]
        min.mar <- min.mar + n.mar[i]

        # recombination fractions
        r <- as.numeric(tempmap[cur:(cur+n.mar[i]-2)])

        # convert to cM distances (w/ Kosambi map function)
        d <- 0.25*log((1+2*r)/(1-2*r))*100

        # convert to locations
        map[[i]] <- round(c(0,cumsum(d)),2)
        cur <- cur+n.mar[i]-1

        # marker names
        names(map[[i]]) <- tempmap[cur:(cur+n.mar[i]-1)]
        dimnames(geno.data[[i]]) <- list(NULL, names(map[[i]]))
        cur <- cur+n.mar[i]

        g[[i]] <- list(data=geno.data[[i]],map=map[[i]])

        # attempt to pull out chromosome number
        mar.names <- names(map[[i]])
        twodig <- grep("[Dd][1-9][0-9][Mm]", mar.names)
        onedig <- grep("[Dd][1-9][Mm]", mar.names)
        xchr <- grep("[Dd][Xx][Mm]", mar.names)

        chr.num <- NULL
        if(length(twodig) > 0)
            chr.num <- c(chr.num,substr(mar.names[twodig],2,3))
        if(length(onedig) > 0)
            chr.num <- c(chr.num,substr(mar.names[onedig],2,2))
        if(length(xchr) > 0)
            chr.num <- c(chr.num,rep("X",length(xchr)))

        # no marker names of the form above
        if(is.null(chr.num)) {
            chr.num <- length(mar.names)
            names(chr.num) <- "1"
        }
        else {
            chr.num <- table(chr.num)
        }

        m <- max(chr.num)
        if(m > sum(chr.num)/2 && m > 1)
            names(g)[i] <- names(chr.num)[chr.num==m][1]

        if(names(g)[i] == "X" || names(g)[i] == "x") class(g[[i]]) <- "X"
        else class(g[[i]]) <- "A"
    }

    # check that data dimensions match
    n.mar1 <- sapply(g,function(a) ncol(a$data))
    n.mar2 <- sapply(g,function(a) length(a$map))
    n.phe <- ncol(pheno)
    n.ind1 <- nrow(pheno)
    n.ind2 <- sapply(g,function(a) nrow(a$data))
    if(any(n.ind1 != n.ind2)) {
        print(c(n.ind1,n.ind2))
        stop("Number of individuals in genotypes and phenotypes do not match.");
    }
    if(any(n.mar1 != n.mar2)) {
        print(c(n.mar,n.mar2))
        stop("Numbers of markers in genotypes and marker names files do not match.");
    }

    # print some information about the amount of data read
    cat(" --Read the following data:\n");
    cat("\t", n.ind1, " individuals\n");
    cat("\t", sum(n.mar1), " markers\n");
    cat("\t", n.phe, " phenotypes\n");

    # add phenotype names, if missing
    if(is.null(colnames(pheno)))
        dimnames(pheno) <- list(NULL, paste("phenotype", 1:n.phe,sep=""))

    # determine map type: f2 or bc or 4way?
    if(max(geno[!is.na(geno)])<=2) type <- "bc"
    else if(max(geno[!is.na(geno)])<=5) type <- "f2"
    else type <- "4way"
    cross <- list(geno=g,pheno=pheno)
    class(cross) <- c(type,"cross")

    # check that nothing is strange in the genotype data
    if(type=="f2") max.gen <- 5
    else if(type=="bc") max.gen <- 2
    else max.gen <- 14

    u <- unique(geno)
    if(any(!is.na(u) & (u > max.gen | u < 1)))
        stop("There are stange values in the genotype data : ",
             paste(u,collapse=":"), ".")

    cross$pheno <- as.data.frame(cross$pheno, stringsAsFactors=TRUE)

    # return cross + indicator of whether to run est.map
    list(cross,FALSE)
}

# end of read.cross.karl.R

######################################################################
#
# summaryScantwoOld.R
#
# copyright (c) 2001-2019, Karl W Broman, Hao Wu, and Brian Yandell
# last modified Dec, 2019
# first written Nov, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Hao Wu (The Jackson Lab) wrote the initial code for summary.scantwo
# function.  Brian Yandell made further modifications/enhancements to
# summary.scantwo, but Karl re-wrote most of it later.
#
# Part of the R/qtl package
# Contains: summaryScantwoOld, print.summary.scantwo.old
#
######################################################################

summaryScantwoOld <-
    function (object, thresholds = c(0, 0, 0), lodcolumn=1,
              type = c("joint","interaction"), ...)
{
    warning("This function is provided solely for continuity of the software;\n",
            "it is not recommended.\n")
    if(!inherits(object, "scantwo"))
        stop("Input should have class \"scantwo\".")

    type <- match.arg(type)

    if(length(dim(object$lod)) > 2) { # results from multiple phenotypes
        if(length(lodcolumn) > 1) {
            warning("Argument lodcolumn should be of length 1.")
            lodcolumn <- lodcolumn[1]
        }

        if(lodcolumn < 0 || lodcolumn > dim(object$lod)[3])
            stop("Argument lodcolumn misspecified.")
        object$lod <- object$lod[,,lodcolumn]
    }

    if(length(thresholds) < 3) {
        if(length(thresholds) == 1) thresholds <- c(thresholds, 0, 0)
        else stop("You must give three thresholds: full, interaction and main\n")
    }

    thrfull <- thresholds[1]
    thrint <- thresholds[2]
    thrcond <- thresholds[3]

    lod <- object$lod
    map <- object$map

    # backward compatibility for previous version of R/qtl
    if(is.na(match("scanoneX",names(object)))) {
        warning("It would be best to re-run scantwo() with the R/qtl version 0.98 or later.")
        scanoneX <- NULL
    }
    else scanoneX <- object$scanoneX

    # deal with bad LOD score values
    if(any(is.na(lod) | lod < -1e-06 | lod == Inf))
        warning("Some LOD scores NA, Inf or < 0; set to 0")
    lod[is.na(lod) | lod < 0 | lod == Inf] <- 0

    # if there's no mainscan result, ignore the thresholds
    #     and don't include the 4 conditional LOD columns
    if(all(is.na(diag(lod)) | diag(lod) < 1e-10))
        includes.scanone <- FALSE
    else includes.scanone <- TRUE

    # change lod scores to old version
    u <- upper.tri(lod)
    lod[u] <- t(lod)[u] - lod[u]

    # If scanone results available, calculate conditional LOD scores
    if(includes.scanone) {
        d <- diag(lod)
        q1 <- matrix(rep(d,length(d)),ncol=length(d))
        q2 <- matrix(rep(d,length(d)),ncol=length(d),byrow=TRUE)

        if(!is.null(scanoneX) && any(map[,4])) {
            d <- scanoneX
            q1X <- matrix(rep(d,length(d)),ncol=length(d))
            q2X <- matrix(rep(d,length(d)),ncol=length(d),byrow=TRUE)
            q1[map[,4],] <- q1X[map[,4],]
            q2[,map[,4]] <- q2X[,map[,4]]
        }

        q1[lower.tri(q1)] <- t(q2)[lower.tri(q2)]
        condlod <- abs(lod - t(lod)) - q1
        diag(condlod) <- 0
    }
    else condlod <- NULL

    # Negative thresholds are interpreted relative to the maximum LOD score
    if(thrfull < 0)
        thrfull <- max(0,max(lod[lower.tri(lod)]) + thrfull)
    if(thrint < 0)
        thrint <- max(0,max(lod[upper.tri(lod)]) + thrint)
    if(thrcond < 0 && includes.scanone)
        thrcond <- max(0,max(condlod) + thrcond)

    crosstype <- attr(object, "type")
    if(is.null(crosstype)) {
        warning("No type attribute in input data; assuming backcross.")
        crosstype <- "bc"
    }

    # calculate the degree of freedom
    if(crosstype == "bc" || crosstype == "riself" || crosstype ==
       "risib" || crosstype=="dh") {
        df.int <- 1
        df.add <- 1
    }
    else if(crosstype == "f2") {
        df.int <- 4
        df.add <- 2
    }
    else if(crosstype == "4way") {
        df.int <- 9
        df.add <- 3
    }
    else {
        stop("Don't know what to do with cross type ", crosstype)
    }

    # chromsomes in the result
    chr <- unique(map[, 1])
    n.chr <- length(chr)

    # calculate the locations of each chromosome within the LOD matrix
    wh.index <- vector("list", n.chr)
    n <- nrow(map)
    for(i in 1:n.chr)
        wh.index[[i]] <- which(map[, 1] == chr[i])

    results <- NULL

    # go through each pair of chromosomes
    for(i in 1:n.chr) {
        for(j in i:n.chr) {
            tmplod1 <- lod[wh.index[[j]], wh.index[[i]],drop=FALSE]
            if(!is.null(condlod)) {
                if(i==j) tmpcondlod <- condlod[wh.index[[i]],wh.index[[i]],drop=FALSE]
                else {
                    tmpcondlod1 <- condlod[wh.index[[j]],wh.index[[i]],drop=FALSE]
                    tmpcondlod2 <- condlod[wh.index[[i]],wh.index[[j]],drop=FALSE]
                }
            }

            if(i != j) tmplod2 <- lod[wh.index[[i]], wh.index[[j]],drop=FALSE]
            else tmplod2 <- tmplod1


            if(type == "joint") {
                if(i == j) {
                    tri <- lower.tri(tmplod1)
                    lod.joint <- max(tmplod1[tri])
                    idx <- which(tmplod1 == lod.joint & tri, arr.ind=TRUE)
                    if(!is.matrix(idx)) {
                        cat("problem\n")
                        return(tmplod1)
                    }
                }
                else {
                    lod.joint <- max(tmplod1)
                    idx <- which(tmplod1 == lod.joint, arr.ind=TRUE)
                    if(!is.matrix(idx)) {
                        cat("problem\n")
                        return(tmplod1)
                    }
                }
                if(nrow(idx)>1) idx <- idx[sample(nrow(idx),1),,drop=FALSE]
                idx.row <- idx[1]
                idx.col <- idx[2]

                lod.int <- tmplod2[idx.col, idx.row,drop=FALSE]
            }
            else { # interaction lod
                if(i == j) {
                    tri <- upper.tri(tmplod2)
                    lod.int <- max(tmplod2[tri])
                    idx <- which(tmplod2 == lod.int & tri, arr.ind=TRUE)
                }
                else {
                    lod.int <- max(tmplod2)
                    idx <- which(tmplod2 == lod.int)
                }
                if(nrow(idx)>1) idx <- idx[sample(nrow(idx),1),,drop=FALSE]
                idx.row <- idx[2]
                idx.col <- idx[1]

                lod.joint <- tmplod1[idx.row, idx.col,drop=FALSE]
            }

            full.idx.row <- idx.row + wh.index[[j]][1] - 1
            full.idx.col <- idx.col + wh.index[[i]][1] - 1

            flag <- FALSE # a flag to indicate whether there's any peak on this pair
            if(lod.joint >= thrfull) {
                if(includes.scanone) {
                    if(i==j) {
                        lod.q1 <- tmpcondlod[idx.row,idx.col,drop=FALSE]
                        lod.q2 <- tmpcondlod[idx.col,idx.row,drop=FALSE]
                    }
                    else {
                        lod.q1 <- tmpcondlod1[idx.row,idx.col,drop=FALSE]
                        lod.q2 <- tmpcondlod2[idx.col,idx.row,drop=FALSE]
                    }

                    if(lod.int >= thrint || min(c(lod.q1, lod.q2)) >= thrcond) {
                        flag <- TRUE
                        i.pos <- map[full.idx.col, 2]
                        j.pos <- map[full.idx.row, 2]
                        results <- rbind(results,
                                         data.frame(chr[i], chr[j], i.pos, j.pos,
                                                    lod.joint, 1 - pchisq(2 * log(10) * lod.joint,
                                                                          df.int + 2 * df.add),
                                                    lod.int, 1 - pchisq(2 * log(10) * lod.int, df.int),
                                                    lod.q1, 1 - pchisq(2 * log(10) * lod.q1, df.add),
                                                    lod.q2, 1 - pchisq(2 * log(10) * lod.q2, df.add), stringsAsFactors=TRUE)
                                         )
                    }
                }
                else { # no scanone output
                    flag <- TRUE
                    i.pos <- map[full.idx.col, 2]
                    j.pos <- map[full.idx.row, 2]
                    results <- rbind(results,
                                     data.frame(chr[i], chr[j], i.pos, j.pos,
                                                lod.joint, 1 - pchisq(2 * log(10) * lod.joint,
                                                                      df.int + 2 * df.add),
                                                lod.int, 1 - pchisq(2 * log(10) * lod.int, df.int), stringsAsFactors=TRUE)
                                     )
                }
                # give the new row (if any) a name
                if(flag) {
                    mname <- rownames(map)
                    rownames(results)[nrow(results)] <- paste(mname[full.idx.col], ":",
                                                              mname[full.idx.row], sep="")
                }
            } # lod joint above threshold

        } # end loop over chromosomes
    }

    if(is.null(results)) {
        results <- numeric(0)
    }
    else {
        if(includes.scanone)
            colnames(results) <- c("chr1", "chr2", "pos1", "pos2",
                                   "lod.joint", "p.joint", "lod.int", "p.int", "lod.q1",
                                   "p.q1", "lod.q2", "p.q2")
        else colnames(results) <- c("chr1", "chr2", "pos1", "pos2",
                                    "lod.joint", "p.joint", "lod.int", "p.int")
        results <- as.data.frame(results, stringsAsFactors=TRUE)
    }
    class(results) <- c("summary.scantwo.old", "data.frame")
    results
}


print.summary.scantwo.old <-
    function(x,...)
{
    if(length(x)==0) {
        cat("    There were no pairs of loci meeting the criteria.\n")
        return(invisible(NULL))
    }

    # column names
    cnames <- c("pos1", "pos2", "  LODjnt", "-logP",
                "  LODint", "-logP", "  LODq1", "-logP",
                "  LODq2", "-logP")

    # chr names
    chr1 <- paste("c",x[,1],sep="")
    chr2 <- paste("c",x[,2],sep="")

    # pad chr names with spaces; this isn't really necessary
    nchar.c1 <- nchar(chr1); max.nchar.c1 <- max(nchar.c1)
    nchar.c2 <- nchar(chr2); max.nchar.c2 <- max(nchar.c2)
    if(any(nchar.c1 < max.nchar.c1 | nchar.c2 < max.nchar.c2)) {
        for(i in 1:length(nchar.c2)) {
            if(nchar.c1[i] < max.nchar.c1)
                chr1[i] <- paste(paste(rep(" ", max.nchar.c1-nchar.c1[i]),collapse=""),
                                 chr1[i],sep="")
            if(nchar.c2[i] < max.nchar.c2)
                chr2[i] <- paste(paste(rep(" ", max.nchar.c2-nchar.c2[i]),collapse=""),
                                 chr2[i],sep="")
        }
    }
    chr <- paste(chr1,chr2,sep=":")

    # round the rest; take -log10(P-values)
    for(j in 3:ncol(x)) {
        if(j<5)
            x[,j] <- round(x[,j])
        else if(j %% 2)  # odd
            x[,j] <- round(x[,j],2)
        else
            x[,j] <- -round(log10(x[,j]),1)
    }

    res <- as.data.frame(x[,-(1:2)], stringsAsFactors=TRUE)
    names(res) <- cnames[1:ncol(res)]
    rownames(res) <- chr

    cat("\n")
    print.data.frame(res)
    cat("\n")
}

# end of summary.scantwo.old.R

######################################################################
#
# compareorder.R
#
# copyright (c) 2007-2011, Karl W Broman
# last modified May, 2011
# first written Oct, 2007
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: compareorder
#
######################################################################

######################################################################
# Calculate likelihood for a fixed order of markers on a given
# chromosome versus the current one
######################################################################
compareorder <-
    function(cross, chr, order, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             maxit=4000, tol=1e-6, sex.sp=TRUE)
{
    if(missing(chr)) chr <- names(cross$geno)[1]
    if(length(chr) > 1) {
        chr <- chr[1]
        warning("compareorder works on a single chromosome.")
    }

    map.function <- match.arg(map.function)
    cross <- subset(cross, chr)

    if(length(order) != totmar(cross)) {
        if(length(order) == totmar(cross)+1 || length(order) == totmar(cross)+2)
            order <- order[1:totmar(cross)]
        else
            stop("Argument 'order' should have length ", totmar(cross))
    }
    if(any(is.na(match(1:totmar(cross), order))))
        stop("order should be a permutation of the numbers 1, 2, ..., ", totmar(cross))

    orig <- est.map(cross, error.prob=error.prob, map.function=map.function,
                    maxit=maxit, tol=tol, sex.sp=sex.sp, verbose=FALSE)

    cross$geno[[1]]$data <- cross$geno[[1]]$data[,order]

    new <- est.map(cross, error.prob=error.prob, map.function=map.function,
                   maxit=maxit, tol=tol, sex.sp=sex.sp, verbose=FALSE)

    result <- matrix(0, ncol=2, nrow=2)
    dimnames(result) <- list(c("orig","new"), c("LOD", "length"))
    result[2,1] <- (attr(new[[1]], "loglik") - attr(orig[[1]], "loglik"))/log(10)

    if(is.matrix(orig[[1]])) {
        result[,2] <- c(diff(range(orig[[1]][1,])),
                        diff(range(new[[1]][1,])))
        if(sex.sp) {
            result <- cbind(result, c(diff(range(orig[[1]][2,])),
                                      diff(range(new[[1]][2,]))))
            colnames(result)[2:3] <- c("femaleLength","maleLength")
        }
    }
    else
        result[,2] <- c(diff(range(orig[[1]])),
                        diff(range(new[[1]])))

    as.data.frame(result, stringsAsFactors=TRUE)
}

# end of compareorder.R

######################################################################
#
# read.cross.R
#
# copyright (c) 2000-2020, Karl W Broman
# last modified Dec, 2020
# first written Aug, 2000
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross, fixXgeno.bc, fixXgeno.f2, asnumericwithdec
#           [See read.cross.csv.R, read.cross.mm.R, read.cross.qtx.R,
#            qtlcart_io.R, read.cross.gary.R, and read.cross.karl.R
#            for the format-specific functions.]
#
######################################################################

######################################################################
#
# read.cross: read data from an experimental cross
#
######################################################################

read.cross <-
    function(format=c("csv", "csvr", "csvs", "csvsr", "mm", "qtx",
             "qtlcart", "gary", "karl", "mapqtl", "tidy"),
             dir="", file, genfile, mapfile, phefile, chridfile, mnamesfile, pnamesfile,
             na.strings=c("-","NA"), genotypes=c("A","H","B","D","C"),
             alleles=c("A","B"), estimate.map=FALSE, convertXdata=TRUE,
             error.prob=0.0001, map.function=c("haldane", "kosambi", "c-f", "morgan"),
             BC.gen = 0, F.gen = 0, crosstype=NULL, ...)
{
    if(format == "csvrs") {
        format <- "csvsr"
        warning("Assuming you mean 'csvsr' rather than 'csvrs'.\n")
    }
    format <- match.arg(format)

    if(format=="csv" || format=="csvr") { # comma-delimited format
        cross <- read.cross.csv(dir, file, na.strings, genotypes,
                                estimate.map, rotate=(format=="csvr"),
                                ...)
    }
    else if(format=="csvs" || format=="csvsr") { # comma-delimited format

        # allow easier input of filenames into function arguments
        if(missing(phefile) && !missing(file) && !missing(genfile)) {
            # read.cross("format", "dir", "genfile", "phefile")
            phefile <- genfile
            genfile <- file
        }
        else if(missing(genfile) && !missing(file) && !missing(phefile)) {
            # read.cross("format", "dir", "genfile", phefile="phefile")
            genfile <- file
        }

        cross <- read.cross.csvs(dir, genfile, phefile, na.strings, genotypes,
                                 estimate.map, rotate=(format=="csvsr"),
                                 ...)
    }
    else if(format=="qtx") { # Mapmanager QTX format
        cross <- read.cross.qtx(dir,file,estimate.map)
    }
    else if(format=="qtlcart") { # QTL Cartographer format
        # if missing mapfile but genfile is specified,
        #     use genfile as the map file.
        if(missing(mapfile) && !missing(genfile))
            mapfile <- genfile

        cross <- read.cross.qtlcart(dir, file, mapfile)
    }
    else if(format=="karl") { # karl's format
        # if missing file names, use standard ones
        if(missing(genfile)) genfile <- "gen.txt"
        if(missing(mapfile)) mapfile <- "map.txt"
        if(missing(phefile)) phefile <- "phe.txt"

        cross <- read.cross.karl(dir,genfile,mapfile,phefile)
    }
    else if(format=="mm") { # mapmaker format
        # if missing mapfile but genfile is specified,
        #     use genfile as the map file.
        if(missing(mapfile) && !missing(genfile))
            mapfile <- genfile

        cross <- read.cross.mm(dir,file,mapfile,estimate.map)
    }
    else if(format=="gary") { # gary's format
        # if missing file names, use the standard ones
        if(missing(genfile)) genfile <- "geno.dat"
        if(missing(mnamesfile)) mnamesfile <- "mnames.txt"
        if(missing(chridfile)) chridfile <- "chrid.dat"
        if(missing(phefile)) phefile <- "pheno.dat"
        if(missing(pnamesfile)) pnamesfile <- "pnames.txt"
        if(missing(mapfile)) mapfile <- "markerpos.txt"

        cross <- read.cross.gary(dir,genfile,mnamesfile,chridfile,
                                 phefile,pnamesfile,mapfile,estimate.map,na.strings)
    }
    else if(format == "mapqtl") { # MapQTL format (same as JoinMap)
        cross <- read.cross.mq(dir=dir, locfile=genfile, mapfile=mapfile,
                               quafile=phefile)
    }
    else if (format == "tidy") { # tidy format
        if(!missing(file) && !missing(genfile) && !missing(mapfile) && missing(phefile)) {
            phefile <- mapfile
            mapfile <- genfile
            genfile <- file
        }

        if(missing(genfile)) genfile <- "gen.csv"
        if(missing(phefile)) phefile <- "phe.csv"
        if(missing(mapfile)) mapfile <- "map.csv"

        cross <- read.cross.tidy(dir=dir,
                                 genfile=genfile, phefile=phefile, mapfile=mapfile,
                                 na.strings=na.strings, genotypes=genotypes)
    }

    estimate.map <- cross[[2]]
    cross <- cross[[1]]

    if(is.null(crosstype)) crosstype <- crosstype(cross)
    else class(cross) <- c(crosstype, "cross")

    # if chr names all start with "chr" or "Chr", remove that part
    chrnam <- names(cross$geno)
    if(all(regexpr("^[Cc][Hh][Rr]",chrnam)>0)){
        chrnam <- substr(chrnam,4,nchar(chrnam))
        if(all(regexpr("^[Oo][Mm][Oo][Ss][Oo][Mm][Ee]",chrnam)>0))
            chrnam <- substr(chrnam,8,nchar(chrnam))
    }
    # if chr named "x" make it "X"
    if(sum(chrnam=="x")>0) chrnam[chrnam=="x"] <- "X"
    names(cross$geno) <- chrnam
    # make sure the class of chromosomes named "X" is "X"
    for(i in 1:length(cross$geno))
        if(names(cross$geno)[i] == "X")
            class(cross$geno[[i]]) <- "X"

    # Fix up the X chromosome data for a backcross or intercross
    chr_type <- sapply(cross$geno,chrtype)
    if(any(chr_type=="X") && convertXdata) {
        if(crosstype(cross)=="bc")
            cross <- fixXgeno.bc(cross)
        if(crosstype(cross)=="f2") {
            if(missing(alleles)) alleles <- c("A","B")
            cross <- fixXgeno.f2(cross, alleles)
        }
    }

    ## Pass through read.cross.bcsft for BCsFt (convert if appropriate).
    cross <- read.cross.bcsft(cross = cross, BC.gen = BC.gen, F.gen = F.gen, ...)

    if(crosstype=="risib") cross <- convert2risib(cross)
    else if(crosstype=="riself") cross <- convert2riself(cross)

    # store genotype data as integers
    for(i in 1:nchr(cross))
        storage.mode(cross$geno[[i]]$data) <- "integer"

    # check alleles
    if(crosstype(cross) != "4way") {
        if(length(alleles) > 2) {
            warning("length of arg alleles should be 2")
            alleles <- alleles[1:2]
        }
        if(length(alleles) < 2)
            stop("length of arg alleles should be 2")
    }
    else { # 4-way cross
        if(missing(alleles))
            alleles <- c("A","B","C","D")

        if(length(alleles) > 4) {
            warning("length of arg alleles should be 4 for a 4-way cross")
            alleles <- alleles[1:4]
        }
        if(length(alleles) < 4)
            stop("length of arg alleles should be 4 for a 4-way cross")
    }
    if(any(nchar(alleles)) != 1) {
        warning("Each item in arg alleles should be a single character")
        alleles <- substr(alleles, 1, 1)
    }

    attr(cross, "alleles") <- alleles

    # if 4-way cross, make the maps matrices
    if(crosstype=="4way") {
        for(i in seq_along(cross$geno)) {
            if(!is.matrix(cross$geno[[i]]$map) || nrow(cross$geno[[i]]$map) < 2) {
                cross$geno[[i]]$map <- rbind(cross$geno[[i]]$map, cross$geno[[i]]$map)
            }
        }
    }

    # re-estimate map?
    if(estimate.map) {
        cat(" --Estimating genetic map\n")
        map.function <- match.arg(map.function)
        newmap <- est.map(cross, error.prob=error.prob, map.function=map.function)
        cross <- replace.map(cross, newmap)
    }

    # run checks
    summary(cross)

    cat(" --Cross type:", crosstype(cross), "\n")

    cross
}




##############################
# fixXgeno.bc: fix up the X chromosome genotype data for backcross
##############################
fixXgeno.bc <-
    function(cross)
{
    omitX <- FALSE

    # pull out X chr genotype data
    chr_type <- sapply(cross$geno,chrtype)
    xchr <- which(chr_type=="X")
    Xgeno <- cross$geno[[xchr]]$data

    # find "sex" and "pgm" in the phenotype data
    sexpgm <- getsex(cross)

    if(!is.null(sexpgm$sex)) {     # "sex" is provided
        malegeno <- Xgeno[sexpgm$sex==1,]
        if(any(!is.na(malegeno) & malegeno==2)) {
            n.omit <- sum(!is.na(malegeno) & malegeno==2)
            warning(" --Omitting ", n.omit, " male heterozygote genotypes on the X chromosome.")
            malegeno[!is.na(malegeno) & malegeno==2] <- NA
        }
        malegeno[!is.na(malegeno) & malegeno==3] <- 2

        femalegeno <- Xgeno[sexpgm$sex==0,]
        if(any(!is.na(femalegeno) & femalegeno==3)) {
            n.omit <- sum(!is.na(femalegeno) & femalegeno==3)
            warning(" --Omitting ", n.omit, " BB genotypes from females on the X chromosome.")
            femalegeno[!is.na(femalegeno) & femalegeno==3] <- NA
        }

        Xgeno[sexpgm$sex==1,] <- malegeno
        Xgeno[sexpgm$sex==0,] <- femalegeno

    }
    else {
        # "sex" not provided

        if(all(is.na(Xgeno) | Xgeno==1 | Xgeno==3)) { # look like all males
            warning(" --Assuming that all individuals are male.\n")
            Xgeno[!is.na(Xgeno) & Xgeno==3] <- 2
            cross$pheno$sex <- factor(rep("m",nind(cross)),levels=c("f","m"))
        }
        else if(all(is.na(Xgeno) | Xgeno==1 | Xgeno==2)) { # look like females A:H
            warning(" --Assuming that all individuals are female.\n")
            cross$pheno$sex <- factor(rep("f",nind(cross)),levels=c("f","m"))
        }
        else { # have some of each of the three genotypes
            warning(" --Can't figure out the X chromosome genotypes.\n   You need to provide phenotypes \"sex\"\n   See the help file for read.cross() for details.\n   Omitting the X chr for now.\n  ")
            omitX <- TRUE
        }
    }

    if(!omitX) {
        wh <- !is.na(Xgeno) & Xgeno!=1 & Xgeno!=2
        if(any(wh)) {
            Xgeno[wh] <- NA
            n.omit <- sum(wh)
            warning(" --Omitted ", n.omit, " additional X chr genotype(s).")
        }

        cross$geno[[xchr]]$data <- Xgeno
    }
    else cross <- subset(cross,chr= -xchr) # <- omit the X chr completely

    cross
}


##############################
# fixXgeno.f2: fix up the X chromosome genotype data for intercross
##############################
fixXgeno.f2 <-
    function(cross, alleles)
{
    omitX <- FALSE

    # pull out X chr genotype data
    chr_type <- sapply(cross$geno,chrtype)
    xchr <- which(chr_type=="X")
    Xgeno <- cross$geno[[xchr]]$data

    # find "sex" and "pgm" in the phenotype data
    sexpgm <- getsex(cross)

    AA <- paste(rep(alleles[1], 2), collapse="")
    AB <- paste(alleles, collapse="")
    BB <- paste(rep(alleles[2], 2), collapse="")
    cross0 <- paste("(", alleles[1], "x", alleles[2], ")x(",
                    alleles[1], "x", alleles[2], ")", sep="")
    cross1 <- paste("(", alleles[2], "x", alleles[1], ")x(",
                    alleles[2], "x", alleles[1], ")", sep="")

    if(!is.null(sexpgm$sex) && !is.null(sexpgm$pgm)) {
        # both "sex" and "pgm" are provided

        if(any(sexpgm$sex == 1)) { # there are males
            malegeno <- Xgeno[sexpgm$sex==1,]
            if(any(!is.na(malegeno) & malegeno==2)) {
                n.omit <- sum(!is.na(malegeno) & malegeno==2)
                warning(" --Omitting ", n.omit, " male heterozygote genotypes on the X chromosome.")
                malegeno[!is.na(malegeno) & malegeno==2] <- NA
            }
            malegeno[!is.na(malegeno) & malegeno==3] <- 2
            Xgeno[sexpgm$sex==1,] <- malegeno
        }

        if(any(sexpgm$sex==0)) { # there are females
            femalegeno0 <- Xgeno[sexpgm$sex==0 & sexpgm$pgm==0,]
            femalegeno1 <- Xgeno[sexpgm$sex==0 & sexpgm$pgm==1,]

            if((any(!is.na(femalegeno0) & femalegeno0==3) ||
                any(!is.na(femalegeno1) & femalegeno1==1)) &&
               !any(!is.na(femalegeno0) & femalegeno0==1) &&
               !any(!is.na(femalegeno1) & femalegeno1==3)) {
                # appear to switched the "pgm" values
                warning(" --The 0/1 values for \"pgm\" appear to be switched; switching back.")
                sexpgm$pgm[sexpgm$pgm==1] <- 2
                sexpgm$pgm[sexpgm$pgm==0] <- 1
                sexpgm$pgm[sexpgm$pgm==2] <- 0
                cross$pheno$pgm[cross$pheno$pgm==1] <- 2
                cross$pheno$pgm[cross$pheno$pgm==0] <- 1
                cross$pheno$pgm[cross$pheno$pgm==2] <- 0

                temp <- femalegeno0
                femalegeno0 <- femalegeno1
                femalegeno1 <- temp
            }
            if(any(!is.na(femalegeno0) & femalegeno0==3)) {
                n.omit <- sum(!is.na(femalegeno0) & femalegeno0==3)
                warning(" --Omitting ", n.omit, " ", BB, " genotypes from females from cross ",
                        cross0, " on the X chr.\n")
                femalegeno0[!is.na(femalegeno0) & femalegeno0==3] <- NA
            }
            if(any(!is.na(femalegeno1) & femalegeno1==1)) {
                n.omit <- sum(!is.na(femalegeno1) & femalegeno1==1)
                warning(" --Omitting ", n.omit, " ", AA, " genotypes from females from cross ",
                        cross1, " on the X chr.\n")
                femalegeno1[!is.na(femalegeno1) & femalegeno1==1] <- NA
            }
            femalegeno1[!is.na(femalegeno1) & femalegeno1==3] <- 1
            Xgeno[sexpgm$sex==0 & sexpgm$pgm==0,] <- femalegeno0
            Xgeno[sexpgm$sex==0 & sexpgm$pgm==1,] <- femalegeno1
        }

    }

    else if(!is.null(sexpgm$sex) && is.null(sexpgm$pgm)) {
        # "sex" is provided but not "pgm"

        if(any(sexpgm$sex == 1)) { # there are males
            malegeno <- Xgeno[sexpgm$sex==1,]
            if(any(!is.na(malegeno) & malegeno==2)) {
                n.omit <- sum(!is.na(malegeno) & malegeno==2)
                warning(" --Omitting ", n.omit, " male heterozygote genotypes on the X chromosome.")
                malegeno[!is.na(malegeno) & malegeno==2] <- NA
            }
            malegeno[!is.na(malegeno) & malegeno==3] <- 2
            Xgeno[sexpgm$sex==1,] <- malegeno
        }

        if(any(sexpgm$sex==0)) { # there are females
            femalegeno <- Xgeno[sexpgm$sex==0,]

            if(any(!is.na(femalegeno) & femalegeno==3) &
               !any(!is.na(femalegeno) & femalegeno==1)) { # looks like (BxA)x(BxA)
                cross$pheno$pgm <- rep(1,nind(cross))
                femalegeno[!is.na(femalegeno) & femalegeno==3] <- 1
            }
            else if(any(!is.na(femalegeno) & femalegeno==1) &
                    !any(!is.na(femalegeno) & femalegeno==3)) { # looks like (AxB)x(AxB)
                cross$pheno$pgm <- rep(0,nind(cross))
            }
            else { # we have some 1's and some 3's
                warning(" --There appear to be some individuals of each cross direction, but \"pgm\" is not provided.\n   Check the X chr genotype data and include a \"pgm\" column in the phenotype data.\n   \"pgm\" was inferred (probably poorly).\n   ")

                cross$pheno$pgm <- rep(0,nind(cross))

                # females with no 3's -> assumed to be from (AxB)x(AxB)
                # females with both 3's and 1's -> assumed to be from (AxB)x(AxB); 3's tossed
                wh.have3 <- apply(femalegeno, 1, function(a) any(!is.na(a) & a==3))
                cross$pheno$pgm[sexpgm$sex==0][wh.have3] <- 1

                temp <- femalegeno[wh.have3,]
                temp[!is.na(temp) & temp==1] <- NA
                temp[!is.na(temp) & temp==3] <- 1
                femalegeno[wh.have3,] <- temp

            }

            Xgeno[sexpgm$sex==0,] <- femalegeno
        }
    }
    else if(is.null(sexpgm$sex) && !is.null(sexpgm$pgm)) {
        # "pgm" is provided but not "sex"

        if(all(is.na(Xgeno) | Xgeno==1 | Xgeno==3)) { # look like all males
            cross$pheno$sex <- factor(rep("m",nind(cross)),levels=c("f","m"))
            Xgeno[!is.na(Xgeno) & Xgeno==3] <- 2
        }
        else { # assume all females
            cross$pheno$sex <- factor(rep("f",nind(cross)),levels=c("f","m"))
            Xgeno.pgm0 <- Xgeno[sexpgm$pgm==0,]
            Xgeno.pgm1 <- Xgeno[sexpgm$pgm==1,]
            if(all(is.na(Xgeno.pgm0) | Xgeno.pgm0==2 | Xgeno.pgm0==3) &&
               all(is.na(Xgeno.pgm1) | Xgeno.pgm1==1 | Xgeno.pgm1==2)) {
                cross$pheno$pgm <- 1 - sexpgm$pgm
                temp <- Xgeno.pgm0
                Xgeno.pgm0 <- Xgeno.pgm1
                Xgeno.pgm1 <- temp
            }
            Xgeno.pgm1[!is.na(Xgeno.pgm1) & Xgeno.pgm1==1] <- NA
            Xgeno.pgm1[!is.na(Xgeno.pgm1) & Xgeno.pgm1==3] <- 1
            Xgeno.pgm0[!is.na(Xgeno.pgm0) & Xgeno.pgm0==3] <- NA
            Xgeno[sexpgm$pgm==0,] <- Xgeno.pgm0
            Xgeno[sexpgm$pgm==1,] <- Xgeno.pgm1
        }
    }


    else {
        # Neither "sex" and "pgm" provided

        if(all(is.na(Xgeno) | Xgeno==1 | Xgeno==3)) { # look like all males
            warning(" --Assuming that all individuals are male.\n")
            Xgeno[!is.na(Xgeno) & Xgeno==3] <- 2
            cross$pheno$sex <- factor(rep("m",nind(cross)),levels=c("f","m"))
            cross$pheno$pgm <- rep(0,nind(cross))
        }
        else if(all(is.na(Xgeno) | Xgeno==2 | Xgeno==3)) { # look like females H:B
            warning(" --Assuming that all individuals are female.\n")
            Xgeno[!is.na(Xgeno) & Xgeno==3] <- 1
            cross$pheno$sex <- factor(rep("f",nind(cross)),levels=c("f","m"))
            cross$pheno$pgm <- rep(1,nind(cross))
        }
        else if(all(is.na(Xgeno) | Xgeno==2 | Xgeno==1)) { # looks like females A:H
            warning(" --Assuming that all individuals are female.\n")
            cross$pheno$sex <- factor(rep("f",nind(cross)),levels=c("f","m"))
            cross$pheno$pgm <- rep(0,nind(cross))
        }
        else { # have some of each of the three genotypes
            warning(" --Can't figure out the X chromosome genotypes.\n   You need to provide phenotypes \"sex\" and/or \"pgm\"\n   See the help file for read.cross() for details.\n   Omitting the X chr for now.\n  ")

            omitX <- TRUE

        }
    }


    if(!omitX) {
        wh <- !is.na(Xgeno) & Xgeno!=1 & Xgeno!=2
        if(any(wh)) {
            Xgeno[wh] <- NA
            n.omit <- sum(wh)
            warning(" --Omitted ", n.omit, " additional X chr genotype(s).")
        }

        cross$geno[[xchr]]$data <- Xgeno
    }
    else cross <- subset(cross,chr= -xchr) # <- omit the X chr completely

    cross
}


######################################################################
# convert character to numeric, using dec as the decimal point
######################################################################
asnumericwithdec <-
    function(x, dec=".")
{
    if(dec!=".") x <- gsub(paste0("\\", dec), ".", x)
    as.numeric(x)
}

# Fix up phenotypes
sw2numeric <-
    function(x, dec) {
        x[x == ""] <- NA
        wh1 <- is.na(x)
        n <- sum(!is.na(x))
        y <- suppressWarnings(asnumericwithdec(as.character(x), dec))
        wh2 <- is.na(y)
        m <- sum(!is.na(y))
        if(n==m || (n-m) < 2 || (n-m) < n*0.05) {
            if(sum(!wh1 & wh2) > 0) {
                u <- unique(as.character(x[!wh1 & wh2]))
                if(length(u) > 1) {
                    themessage <- paste("The phenotype values", paste("\"", u, "\"", sep="", collapse=" "))
                    themessage <- paste(themessage, " were", sep="")
                }
                else {
                    themessage <- paste("The phenotype value \"", u, "\" ", sep="")
                    themessage <- paste(themessage, " was", sep="")
                }
                themessage <- paste(themessage, "interpreted as missing.")
                warning(themessage)

            }
            return(y)
        }
        else return(x)
    }

# end of read.cross.R

#####################################################################
#
# replacemap.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: replace.map, replacemap, replacemap.cross,
#           replacemap.scanone, replacemap.scantwo
######################################################################

######################################################################
#
# replace.map
#
# replace the map portion of a cross object with a list defining a map
#
######################################################################

replace.map <- function(cross, map) replacemap.cross(cross, map)

replacemap.cross <-
    function(object, map)
{
    cross <- object
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    chr.names <- names(cross$geno)
    chr.names2 <- names(map)

    m <- match(chr.names2, chr.names)
    if(any(is.na(m))) { # some extra chr in map
        extra <- chr.names2[is.na(m)]
        map <- map[!is.na(m)]
        chr.names2 <- names(map)
        warning("Extra chr in map: ", paste(extra, collapse=" "))
    }

    cross.sexsp <- sapply(cross$geno, function(a) is.matrix(a$map))
    map.sexsp <- sapply(map, is.matrix)
    if(all(cross.sexsp)) cross.sexsp <- TRUE
    else if(all(!cross.sexsp)) cross.sexsp <- FALSE
    else stop("In cross, some maps sex-specific, some not.")

    if(all(map.sexsp)) map.sexsp <- TRUE
    else if(all(!map.sexsp)) map.sexsp <- FALSE
    else stop("In map, some chrsex-specific, some not.")

    m <- match(chr.names, chr.names2)
    if(any(is.na(m))) {
        for(i in chr.names2) {
            if(cross.sexsp) mnames <- colnames(cross$geno[[i]]$map)
            else mnames <- names(cross$geno[[i]]$map)

            if(map.sexsp) mnames2 <- colnames(map[[i]])
            else mnames2 <- names(map[[i]])

            if(length(mnames) != length(mnames2))
                stop("Different numbers of markers on chr ", i)
            if(!all(mnames==mnames2))
                stop("Different marker names on chr ", i)

            cross$geno[[i]]$map <- map[[i]]
        }
    }
    else { # the same chromosomes
        n.chr <- nchr(cross)
        n.mar <- nmar(cross)

        n.chr2 <- length(map)

        if(cross.sexsp) mnames <- unlist(lapply(cross$geno, function(a) colnames(a$map)))
        else mnames <- unlist(lapply(cross$geno, function(a) names(a$map)))

        if(map.sexsp) {
            mnames2 <- unlist(lapply(map, colnames))
            n.mar2 <- sapply(map, ncol)
        }
        else {
            mnames2 <- unlist(lapply(map, names))
            n.mar2 <- sapply(map, length)
        }

        # check that things line up properly
        if(n.chr != n.chr2)
            stop("Numbers of chromosomes don't match.")
        if(any(names(cross$geno) != names(map)))
            stop("Chromosome names don't match.")
        if(any(n.mar != n.mar2))
            stop("Number of markers don't match.")
        if(any(mnames != mnames2))
            stop("Marker names don't match.")

        # proceed if no errors
        for(i in 1:length(cross$geno))
            cross$geno[[i]]$map <- map[[i]]
    }

    #### < BUG > ####
    # the next two things don't work for sex-sp maps (4-way cross)
    #################

    # maps in geno prob
    if("prob" %in% names(cross$geno[[1]])) {
        for(i in names(cross$geno)) {
            if("map" %in% names(attributes(cross$geno[[i]]$prob))) {
                temp <- attr(cross$geno[[i]]$prob, "map")
                tempr <- interpmap(data.frame(chr=rep(i, length(temp)), pos=temp, stringsAsFactors=TRUE), map)[,2]
                names(tempr) <- names(temp)
                attr(cross$geno[[i]]$prob, "map") <- tempr
            }
        }
    }

    # maps in draws
    if("draws" %in% names(cross$geno[[1]])) {
        for(i in names(cross$geno)) {
            if("map" %in% names(attributes(cross$geno[[i]]$draws))) {
                temp <- attr(cross$geno[[i]]$draws, "map")
                tempr <- interpmap(data.frame(chr=rep(i, length(temp)), pos=temp, stringsAsFactors=TRUE), map)[,2]
                names(tempr) <- names(temp)
                attr(cross$geno[[i]]$draws, "map") <- tempr
            }
        }
    }

    # maps in argmax
    if("argmax" %in% names(cross$geno[[1]])) {
        for(i in names(cross$geno)) {
            if("map" %in% names(attributes(cross$geno[[i]]$argmax))) {
                temp <- attr(cross$geno[[i]]$argmax, "map")
                tempr <- interpmap(data.frame(chr=rep(i, length(temp)), pos=temp, stringsAsFactors=TRUE), map)[,2]
                names(tempr) <- names(temp)
                attr(cross$geno[[i]]$argmax, "map") <- tempr
            }
        }
    }

    cross
}

# generic function
replacemap <- function(object, map) UseMethod("replacemap")

replacemap.scanone <-
    function(object, map)
{
    object[,2] <- interpmap(object[,1:2], map)[,2]
    object
}


replacemap.scantwo <-
    function(object, map)
{
    object$map[,2] <- interpmap4scantwo(object, map)[,2]
    object
}


######################################################################
# interpolate map positions from one to another
######################################################################
interpmap <-
    function(oldmap, newmap)
{
    returnasmap <- TRUE
    if(is.data.frame(oldmap)) {
        origmap <- oldmap
        pos <- oldmap[,2]
        names(pos) <- rownames(oldmap)
        oldmap <- split(pos, oldmap[,1])
        returnasmap <- FALSE
    }

    ochrnam <- names(oldmap)
    nchrnam <- names(newmap)
    m <- match(ochrnam, nchrnam)
    if(any(is.na(m))) {
        u <- ochrnam[is.na(m)]
        stop("Chr ", paste(u, collapse=" "), " not found in new map")
    }
    for(i in seq(along=ochrnam)) {
        omap <- oldmap[[i]]
        nmap <- newmap[[m[i]]]

        mm <- match(names(omap), names(nmap))
        wh <- is.na(mm)

        if(sum(!wh) > 1 && any(diff(mm[!wh])<0))
            stop("Need old and new maps to have markers in the same order.")

        if(sum(wh)==0) {
            oldmap[[i]] <- nmap
            next
        }

        if(sum(!wh) < 2)
            stop("Need at least two markers per chromosome in both old and new maps")

        nnmap <- omap
        nnmap[!wh] <- nmap[mm[!wh]]

        nL <- diff(range(nmap[mm[!wh]]))
        oL <- diff(range(omap[!wh]))

        onmap <- which(!wh)
        first <- onmap[1]
        last <- max(onmap)
        notonmap <- which(wh)

        for(j in notonmap) {
            if(!any(onmap < j))  # before first marker
                nnmap[j] <- nnmap[first] - (omap[first] - omap[j])*nL/oL
            else if(!any(onmap > j))  # after last marker
                nnmap[j] <- nnmap[last] + (omap[j] - omap[last])*nL/oL
            else {
                left <- max(onmap[onmap < j])
                right <- min(onmap[onmap > j])
                nnmap[j] <- nnmap[left] + (omap[j]-omap[left])*(nnmap[right]-nnmap[left])/
                    (omap[right]-omap[left])
            }
        }
        oldmap[[i]] <- nnmap
    }

    if(!returnasmap) {
        origmap[,2] <- unlist(oldmap)
        return(origmap)
    }
    oldmap
}

# like interpmap, but special for scantwo to deal with
# the case that scantwo was run without incl.markers=TRUE,
# so we need to add all marker positions to the map thing
interpmap4scantwo <-
    function(output, newmap)
{
    themap <- output$map[,1:2]

    pos <- themap[,2]
    names(pos) <- rownames(themap)
    themapalt <- split(pos, themap[,1])

    markermap <- attr(output, "fullmap")

    omapnam <- names(themapalt)
    nmapnam <- names(markermap)

    m <- match(omapnam, nmapnam)
    if(any(is.na(m))) {
        u <- omapnam[is.na(m)]
        stop("Chr ", paste(u, collapse=" "), " not found in new map")
    }

    flag <- FALSE
    for(i in seq(along=m)) {
        omap <- themapalt[[i]]
        nmap <- markermap[[m[i]]]

        mm <- match(names(nmap), names(omap))
        if(any(is.na(mm))) {
            flag <- TRUE
            themapalt[[i]] <- sort(c(omap, nmap[is.na(mm)]))
        }
    }

    if(flag) {
        revmap <- data.frame(chr=factor(rep(names(themapalt), sapply(themapalt, length)),
                             levels=names(themapalt)),
                             pos=unlist(themapalt), stringsAsFactors=TRUE)
        rownames(revmap) <- unlist(lapply(themapalt, names))
        revmap[,2] <- interpmap(revmap, newmap)[,2]
        themap <- revmap[rownames(themap),]
    }
    else
        themap[,2] <- interpmap(themap, newmap)[,2]

    themap
}

# end of replacemap.R

######################################################################
#
# add.cim.covar.R
#
# copyright (c) 2007-8, Karl W Broman
# last modified Aug, 2008
# first written Mar, 2007
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: add.cim.covar
#
######################################################################

######################################################################
# Indicate the locations of the selected marker covariates
# in a plot of CIM results (as obtained by plot.scanone)
#
# The chr and gap arguments must be identical to those used in the
# call to plot.scanone.
######################################################################
add.cim.covar <-
    function(cimresult, chr, gap=25, ...)
{
    cimcovar <- attr(cimresult, "marker.covar.pos")

    if(!missing(chr)) cimresult <- subset(cimresult, chr=chr)
    if(nrow(cimcovar) == 0) return(invisible(NULL))
    chr <- as.character(unique(cimresult[,1]))

    dots <- list(...)
    ndots <- names(dots)

    u <- par("usr")
    if(length(chr)==1) {
        if(!("col" %in% ndots) && !("pch" %in% ndots))
            points(cimcovar[,2], u[3], xpd=TRUE, col="red", pch=16, ...)
        else if(!("col" %in% ndots))
            points(cimcovar[,2], u[3], xpd=TRUE, col="red", ...)
        else if(!("pch" %in% ndots))
            points(cimcovar[,2], u[3], xpd=TRUE, pch=16, ...)
        else
            points(cimcovar[,2], u[3], xpd=TRUE, ...)
    }
    else {
        begend <- matrix(unlist(tapply(cimresult[,2],cimresult[,1],range)),ncol=2,byrow=TRUE)
        rownames(begend) <- chr
        begend <- begend[as.character(chr),,drop=FALSE]
        len <- begend[,2]-begend[,1]
        start <- c(0,cumsum(len+gap))-c(begend[,1],0)
        start <- start[-length(start)]
        names(start) <- chr

        for(i in 1:nrow(cimcovar)) {
            x <- start[cimcovar[i,1]] + cimcovar[i,2]
            if(!("col" %in% ndots) && !("pch" %in% ndots))
                points(x, u[3], xpd=TRUE, col="red", pch=16, ...)
            else if(!("col" %in% ndots))
                points(x, u[3], xpd=TRUE, col="red", ...)
            else if(!("pch" %in% ndots))
                points(x, u[3], xpd=TRUE, pch=16, ...)
            else
                points(x, u[3], xpd=TRUE, ...)
        }
    }

    invisible(cimcovar)
}

# end of add.cim.covar.R

######################################################################
#
# test_qtl.R
#
# copyright (c) 2009, Karl W Broman, Pjotr Prins
# first written July 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Some basic regression/integration testing for some of the QTL mapping routines
#
# You can run it with:
#
#   R --no-save --no-restore --no-readline --slave < ./tests/test_qtl.R 

######################################################################

library(qtl)

version = mqm_version()
cat("R/qtl=",version$RQTL)
cat("R-MQM=",version$RMQM)
cat("MQM=",version$MQM)


data(listeria)
if (nind(listeria)!=120) stop("Number of individuals incorrect")

# ---- a quick test of standard R/qtl scanone
mr = scanone(listeria, method='mr')
test = round(mr[15,]$lod*1000)
cat(mr[15,]$lod,test)
if (test != 966) stop("scanone_mr gives an incorrect result")

# ---- a quick test of MQM for R/qtl
augmentedcross <- mqmaugment(listeria, minprob=1.0, verbose=TRUE)
nind = nind(augmentedcross)
if (nind!=120) stop("Number of individuals incorrect: ",nind)
result <- mqmscan(augmentedcross, logtransform=TRUE, outputmarkers = FALSE,off.end=0)
test1 = round(result[5,5]*1000)
test2 = round(max(result[,5]*1000))
cat("test1 = ",test1,"\n")
cat("test2 = ",test2,"\n")
if (test1 != 76) stop("MQM gives an unexpected result (1)")
if (test2 != 5384) stop("MQM gives an unexpected result (2)")

# ---- Test for negative markerlocations
data(hyper)
hyper <- fill.geno(hyper)
#Mess up the markers by shifting
temp <- shiftmap(hyper, offset=10^7)
out.temp <- mqmscan(temp,verb=TRUE,off.end=10)
if(!(rownames(out.temp)[3]=="D1Mit296")) stop("MQM something wrong with positive shifts in location")
#Mess up the dataset by moving 1 marker infront of the chromosome
hyper$geno[[1]]$map[1] <- -10
res <- mqmscan(hyper,verbose=T,off.end=100)
if(any(is.na(res[,3]))) stop("MQM failed to handle negative locations correctly")
if(!(rownames(res)[2]=="c1.loc-95")) stop("MQM something wrong with negative locations")   #to -15 because off.end defaults to 10


cat("Version information:\n")
cat("R/qtl = ",version$RQTL,"\n")
cat("R-MQM = ",version$RMQM,"\n")
cat("MQM   = ",version$MQM,"\n\n")

cat("test_qtl.R tests succesfully run!")

#####################################################################
#
# scanoneboot.R
#
# copyright (c) 2007-2011, Karl W Broman
# last modified Mar, 2011
# first written Apr, 2007
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: scanoneboot, summary.scanoneboot, print.scanoneboot
#
######################################################################

######################################################################
# scanoneboot: function to get bootstrap-based confidence interval for
#              QTL location
######################################################################
scanoneboot <-
    function(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
             method=c("em","imp","hk","ehk","mr","mr-imp","mr-argmax"),
             addcovar=NULL, intcovar=NULL, weights=NULL,
             use=c("all.obs", "complete.obs"), upper=FALSE,
             ties.random=FALSE, start=NULL, maxit=4000, tol=1e-4,
             n.boot=1000, verbose=FALSE)
{
    if(!missing(chr)) cross <- subset(cross, chr)
    if(nchr(cross) != 1) { # scan just one chromosome
        warning("Considering just the first chromosome (", names(cross$geno)[1], ").")
        cross <- subset(cross, names(cross$geno)[1])
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(length(pheno.col) > 1)
        stop("pheno.col should indicate a single phenotype")
    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }
    if(pheno.col < 1 || pheno.col > nphe(cross))
        stop("pheno.col should be between 1 and ", nphe(cross))

    # do scan with actual data
    out <- scanone(cross, pheno.col=pheno.col, model=model, method=method,
                   addcovar=addcovar, intcovar=intcovar, weights=weights,
                   use=use, upper=upper, ties.random=ties.random, start=start,
                   maxit=maxit, tol=tol)

    maxlod <- max(out[,3],na.rm=TRUE)
    w <- which(!is.na(out[,3]) & out[,3] == maxlod)

    results <- rep(NA, n.boot)
    n.ind <- nind(cross)
    n.prnt <- floor(n.boot/20)

    for(i in 1:n.boot) {
        temp <- subset(cross, ind=sample(n.ind, replace=TRUE))
        out <- scanone(temp, pheno.col=pheno.col, model=model, method=method,
                       addcovar=addcovar, intcovar=intcovar, weights=weights,
                       use=use, upper=upper, ties.random=ties.random, start=start,
                       maxit=maxit, tol=tol)
        mx <- max(out[,3],na.rm=TRUE)
        w <- out[!is.na(out[,3]) & out[,3]==mx,2]
        if(length(w) > 1) w <- sample(w,1)
        results[i] <- w

        if(verbose && ((i-1) %% n.prnt) == 0)
            cat("replicate", i, "\n")
    }

    attr(results, "results") <- out
    class(results) <- "scanoneboot"
    results
}


# summary function for scanoneboot output
summary.scanoneboot <-
    function(object, prob=0.95, expandtomarkers=FALSE, ...)
{
    lo <- (1-prob)/2
    results <- attr(object, "results")
    o <- max(results)
    qu <- quantile(object, c(lo, 1-lo))

    wh1 <- which(results[,2] <= qu[1])
    wh1 <- wh1[length(wh1)]
    wh2 <- which(results[,2] >= qu[2])
    wh2 <- wh2[1]

    if(expandtomarkers) {
        markerpos <- (1:nrow(results))[-grep("^c.+\\.loc-*[0-9]+(\\.[0-9]+)*$", rownames(results))]
        if(any(markerpos <= wh1))
            wh1 <- max(markerpos[markerpos <= wh1])
        if(any(markerpos >= wh2))
            wh2 <- min(markerpos[markerpos >= wh2])
    }

    rbind(results[wh1,], o, results[wh2,])
}


# print function for scanoneboot output
print.scanoneboot <-
    function(x, ...)
{
    print(as.numeric(x))
}

# end of scanoneboot.R

#####################################################################
#
# interpPositions.R
#
# copyright (c) 2011-2012, Karl W Broman
# last modified Mar, 2012
# first written Nov, 2011
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Takes an "oldmap" (e.g., a physical map in bp or Mbp)
#     and a "newmap" (e.g., a genetic map in cM)
# take additional positions in the "oldmap" scale and estimate
# the corresponding positions (by interpolation or extrapolation)
# in the "newmap" scale
######################################################################

# oldmap and newmap in "map" format (list of vectors of positions)
# oldpositions as dataframe with $chr and $pos
interpPositions <-
    function(oldpositions, oldmap, newmap)
{
    orig.rownames <- rownames(oldpositions)
    # not sure why this is necessary, but it avoids a bug
    if(is.null(rownames(oldpositions)))
        rownames(oldpositions) <- paste("temprn", 1:nrow(oldpositions), sep="")
    else
        rownames(oldpositions) <- paste("temprn", rownames(oldpositions), sep="")

    oldchrnum <- match(oldpositions$chr, names(oldmap))
    newchrnum <- match(oldpositions$chr, names(newmap))

    missingchr <- is.na(oldchrnum) | is.na(newchrnum)
    if(any(missingchr))
        warning("Chromosomes ", paste(sort(unique(oldpositions$chr[missingchr])), collapse=", "), " not found")

    newpositions <- cbind(oldpositions, newpos=rep(NA, nrow(oldpositions)))
    u <- unique(oldchrnum)

    for(i in seq(along=u)) { # loop over chromosomes
        chrnam <- names(oldmap)[u[i]] # name of chromosome

        # the positions to be interpolated
        wholdpositions <- !missingchr & oldchrnum==u[i]
        theposnames <- rownames(oldpositions)[wholdpositions]
        if(!any(wholdpositions)) next

        # data frame with oldmap positions for this chromosome
        tempoldmap <- oldmap[[u[i]]]
        tempoldmap.df <- data.frame(chr=rep(chrnam, length(tempoldmap)),
                                    pos=as.numeric(tempoldmap))
        rownames(tempoldmap.df) <- names(tempoldmap)

        # add the positions to be interpolated
        tempoldmap.df <- rbind(tempoldmap.df, oldpositions[wholdpositions,,drop=FALSE])
        tempoldmap.df <- tempoldmap.df[order(tempoldmap.df$pos),,drop=FALSE]
        tempoldmap.df$chr <- as.character(tempoldmap.df$chr)

        # do the interpolation
        result <- interpmap(tempoldmap.df, newmap[chrnam])

        # paste in the interpolated positions
        newpositions[theposnames, "newpos"] <- result[theposnames, "pos"]
    }

    rownames(newpositions) <- orig.rownames
    newpositions
}

# end of interpPositions.R

#####################################################################
#
# mqmplots.R
#
# Copyright (c) 2009-2020, Danny Arends
# Copyright polyplot routine (c) 2009, Rutger Brouwer
#
# Modified by Pjotr Prins and Karl Broman
#
#
# first written Februari 2009
# last modified Feb 2020
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: mqmplot.directedqtl
#           mqmplot.cistrans
#           addloctocross
#           polyplot
#           getThird
#           getChr
#           mqmplot.multitrait
#           mqmplot.permutations
#           mqmplot.singletrait
#
#
#
#####################################################################

mqmplot.directedqtl <- function(cross, mqmresult, pheno.col=1, draw = TRUE)
{
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }
    if(!is.null(cross$mqm$Nind)){
        stop("Augmented crossobject. Please supply the original unaugmented dataset.")
    }
    if(is.null(mqmresult)){
        stop("No mqmresult object. Please supply a valid scanone object.")
    }
    if(!inherits(mqmresult, "scanone")){
        stop("No mqmresult object. Please supply a valid scanone object.")
    }
    onlymarkers <- mqmextractmarkers(mqmresult)
    eff <- effectscan(sim.geno(cross),pheno.col=pheno.col,draw=FALSE)
    if(any(eff[,1]=="X")){
        eff <- eff[-which(eff[,1]=="X"),]
    }
    onlymarkers[,3] <- onlymarkers[,3]*(eff[,3]/abs(eff[,3]))
    if(draw) plot(ylim=c((min(onlymarkers[,3])*1.1),(max(onlymarkers[,3])*1.1)),onlymarkers)
    class(onlymarkers) <- c("scanone",class(onlymarkers))
    if(!is.null(attr(mqmresult,"mqmmodel"))) attr(onlymarkers,"mqmmodel") <- attr(mqmresult,"mqmmodel")
    invisible(onlymarkers)
}

mqmplot.heatmap <- function(cross, result, directed=TRUE, legend=FALSE, breaks = c(-100,-10,-3,0,3,10,100), col = c("darkblue","blue","lightblue","yellow","orange","red"), ...)
{
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }
    if(directed && !is.null(cross$mqm$Nind)){
        stop("Augmented crossobject. Please supply the original unaugmented dataset.")
    }
    if(is.null(result)){
        stop("No result object. Please supply a valid scanone object.")
    }
    if(!inherits(result,"mqmmulti")){
        stop("Not a mqmmulti object. Please supply a valid scanone object.")
    }
    cross <- sim.geno(cross)
    names <- NULL
    for(x in 1:nphe(cross)){
        result[[x]] <- mqmextractpseudomarkers(result[[x]])
        if(directed){
            effect <- effectscan(sim.geno(cross,step=stepsize(result[[x]])), pheno.col=x, draw=FALSE)
            for(y in 1:nrow(result[[x]])){
                effectid <- which(rownames(effect)==rownames(result[[x]])[y])
                if(!is.na(effectid&&1)){
                    result[[x]][y,3]  <- result[[x]][y,3] *(effect[effectid,3]/abs(effect[effectid,3]))
                }
            }
        }
        names <- c(names,substring(colnames(result[[x]])[3],5))
    }
    chrs <- unique(lapply(result,getChr))
    data <- NULL
    for(x in 1:length(result)){
        data <- rbind(data,result[[x]][,3])
    }
    rownames(data) <- names
    if(nphe(cross) < 100){
        image(seq(0,nrow(result[[1]])),seq(0,nphe(cross)),t(data),xlab="Markers",ylab="Traits",breaks=breaks,col=col,yaxt="n",...)
        axis(2,at=seq(1,nphe(cross)),labels=colnames(pull.pheno(cross)),las=1)
    }else{
        image(seq(0,nrow(result[[1]])),seq(0,nphe(cross)),t(data),xlab="Markers",ylab="Traits",breaks=breaks,col=col,...)
    }
    abline(v=0)
    for(x in unique(chrs[[1]])){
        abline(v=sum(as.numeric(chrs[[1]])<=x))
    }
    for(x in 1:nphe(cross)){
        abline(h=x)
    }
    if(legend){
        leg <- NULL
        for(x in 2:length(breaks)){
            leg <- c(leg,paste("LOD",breaks[x-1],"to",breaks[x]))
        }
        legend("bottom",leg,col=col,lty=1,bg="white")
    }
    invisible(data)
}

mqmplot.clusteredheatmap <- function(cross, mqmresult, directed=TRUE, legend=FALSE, Colv=NA, scale="none", verbose=FALSE, breaks = c(-100,-10,-3,0,3,10,100), col = c("darkblue","blue","lightblue","yellow","orange","red"), ...)
{
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }
    if(directed && !is.null(cross$mqm$Nind)){
        stop("Augmented crossobject. Please supply the original unaugmented dataset.")
    }
    if(is.null(mqmresult)){
        stop("No mqmresult object. Please supply a valid mqmmulti object.")
    }
    if(!inherits(mqmresult,"mqmmulti")){
        stop("Not a mqmmulti object. Please supply a valid mqmmulti object.")
    }
    cross <- sim.geno(cross)
    names <- NULL
    for(x in 1:nphe(cross)){
        mqmresult[[x]] <- mqmextractpseudomarkers(mqmresult[[x]])
        if(directed){
            effect <- effectscan(sim.geno(cross,step=stepsize(mqmresult[[x]])), pheno.col=x, draw=FALSE)
            if(verbose) cat(".")
            for(y in 1:nrow(mqmresult[[x]])){
                effectid <- which(rownames(effect)==rownames(mqmresult[[x]])[y])
                if(!is.na(effectid&&1)){
                    mqmresult[[x]][y,3]  <- mqmresult[[x]][y,3] *(effect[effectid,3]/abs(effect[effectid,3]))
                }
            }
        }
        names <- c(names,substring(colnames(mqmresult[[x]])[3],5))
    }
    if(verbose && directed) cat("\n")
    chrs <- unique(lapply(mqmresult,getChr))
    data <- NULL
    for(x in 1:length(mqmresult)){
        data <- rbind(data,mqmresult[[x]][,3])
    }
    colnames(data) <- rownames(mqmresult[[1]])
    rownames(data) <- names
    if(length(names) < 100){
        retresults <- heatmap(data,Colv=Colv,scale=scale, xlab="Markers",main="Clustered heatmap",breaks=breaks,col=col,keep.dendro = TRUE, ...)
    }else{
        retresults <- heatmap(data,Colv=Colv,scale=scale, xlab="Markers",main="Clustered heatmap",breaks=breaks,col=col,keep.dendro = TRUE,labRow=1:length(names), ...)
    }
    if(legend){
        leg <- NULL
        for(x in 2:length(breaks)){
            leg <- c(leg,paste("LOD",breaks[x-1],"to",breaks[x]))
        }
        legend("bottom",leg,col=col,lty=1,bg="white")
    }
    invisible(retresults)
}

mqmplot.cistrans <- function(result,cross,threshold=5,onlyPEAK=TRUE,highPEAK=FALSE,cisarea=10,pch=22,cex=0.5, verbose=FALSE, ...)
{
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }
    if(is.null(cross$locations)){
        stop("Please add trait locations to the cross file\n")
    }
    if(inherits(result, "mqmmulti")){
        sum.map <- 0
        chr.breaks <- NULL
        for(j in 1:nchr(cross)){
            l.chr <- max(result[[1]][result[[1]][,1]==j,2])
            chr.breaks <- c(chr.breaks,sum.map)
            sum.map <- sum.map+l.chr
        }
        sum.map <- ceiling(sum.map)
        if(verbose) cat("Total maplength:",sum.map," cM in ",nchr(cross),"Chromosomes\nThe lengths are:",chr.breaks,"\n")
        locations <-  do.call(rbind,cross$locations)
        QTLs <- do.call(rbind,lapply(FUN=getThird,result))
        colnames(QTLs) <- rownames(result[[1]])
        bmatrix <- QTLs>threshold
        pmatrix <- NULL
        for(j in 1:nrow(QTLs)){
            if(verbose && (j%%1000 == 0)) cat("QTL row:",j,"\n")
            temp <- as.vector(bmatrix[j,])
            tempv <- QTLs[j,]
            value = 0
            index = -1
            for(l in 1:length(temp)){
                if(temp[l]){
                    if( tempv[l] > value){
                        #New highest marker set the old index to false
                        if(index != -1){
                            temp[index] <- FALSE
                        }
                        #and store the new one
                        value = tempv[l]
                        index <- l
                    }else{
                        #Lower marker
                        temp[l] <- FALSE
                    }
                }else{
                    index = -1
                    value = 0
                }
            }
            if(onlyPEAK){
                bmatrix[j,] <- temp
            }
            if(highPEAK){
                pmatrix <- rbind(pmatrix,temp)
            }
        }

        locz <- NULL
        for(marker in 1:ncol(QTLs)){
            if(verbose && (j%%1000 == 0)) cat("QTL col:",marker,"\n")
            pos <- find.markerpos(cross, colnames(QTLs)[marker])
            if(!is.na(pos[1,1])){
                locz <- c(locz,round(chr.breaks[as.numeric(pos[[1]])] + as.numeric(pos[[2]])))
            }else{
                mark <- colnames(QTLs)[marker]
                mchr <- substr(mark,sum(regexpr("c",mark)+attr(regexpr("c",mark),"match.length")),regexpr(".loc",mark)-1)
                mpos <- as.numeric(substr(mark,sum(regexpr("loc",mark)+attr(regexpr("loc",mark),"match.length")),nchar(mark)))
                locz <- c(locz,round(chr.breaks[as.numeric(mchr)] + as.numeric(mpos)))
            }
        }

        axi <- 1:sum.map
        plot(x=axi,y=axi,type="n",main=paste("Cis/Trans QTL plot at LOD",threshold),xlab="Markers (in cM)",ylab="Location of traits (in cM)",xaxt="n",yaxt="n")
        trait.locz <- NULL
        for(j in 1:nrow(QTLs)){
            if(verbose && (j%%10 == 0)) cat("QTL row:",j,"\n")
            values <- rep(NA,sum.map)
            aa <- locz[bmatrix[j,]]
            trait.locz <- c(trait.locz,chr.breaks[locations[j,1]] + locations[j,2])
            values[aa] = chr.breaks[locations[j,1]] + locations[j,2]
            if(!highPEAK){
                points(values,pch=pch,cex=cex)
            }else{
                points(values,pch=24,cex=1.25*cex,col="black",bg="red")
            }
        }
        points(axi,type="l")
        points(axi+(cisarea/2),type="l",col="green")
        points(axi-(cisarea/2),type="l",col="green")
        chr.breaks <- c(chr.breaks,sum.map)
        axis(1,at=chr.breaks,labels=FALSE)
        axis(2,at=chr.breaks,labels=FALSE)
        axis(1,at=locz,line=1,pch=24)
        axis(2,at=seq(0,sum.map,25),line=1)
    }else{
        stop("invalid object supplied\n")
    }
}

addloctocross <- function(cross,locations=NULL,locfile="locations.txt", verbose=FALSE)
{
    if(is.null(cross)){
        stop("No cross object. Please supply a valid cross object.")
    }
    if(is.null(locations)){
        locations <- read.table(locfile,row.names=1,header=TRUE, stringsAsFactors=TRUE)
    }
    if(verbose) {
        cat("Phenotypes in cross:",nphe(cross),"\n")
        cat("Phenotypes in file:",nrow(locations),"\n")
    }
    if(max(as.numeric(rownames(locations))) != nphe(cross)){
        stop("ID's of traits in file are larger than # of traits in crossfile.")
    }
    if(nphe(cross)==nrow(locations)){
        locs <- vector(mode = "list", length = nphe(cross))
        for(x in as.numeric(rownames(locations))){
            if(names(cross$pheno)[x] == locations[x,1]){
                locs[[x]] <- locations[x,2:3]
                rownames(locs[[x]]) <- locations[x,1]
            }else{
                warning("Mismatch between name of trait in cross & file.\n")
            }
        }
    }else{
        stop("Number of traits in cross & file don't match.")
    }
    cross$locations <- locs
    cross
}


polyplot <- function( x, type='b', legend=TRUE,legendloc=0, labels=NULL, cex = par("cex"), pch = 19, gpch = 21, bg = par("bg"), color = par("fg"), col=NULL, ylim=range(x[is.finite(x)]), xlim = NULL,
                     main = NULL, xlab = NULL, ylab = NULL, add=FALSE, ... )
{
    #Addition by Danny Arends
    if(legend){
        if(legendloc){
            op <- par(mfrow = c(1,2))
        }else{
            op <- par(mfrow = c(1,1))
        }
    }else{
        op <- par(mfrow = c(1,1))
    }
    #End of addition
    if ( is.vector(x) ) {
        x <- t( as.matrix(x) )
        if (is.null(labels) ) {
            rownames(x) <- c("unspecified gene")
        } else {
            rownames(x) <- labels
        }
    } else {
        x <- as.matrix(x)
    }
    if (is.null(labels) ) labels = rownames( x )
    if (is.null(col) )  col = rainbow( nrow(x),alpha=0.35 )
    if (is.null(xlab) ) xlab="Markers"
    if (is.null(ylab) ) ylab="QTL (LOD)"

    timepoints <- as.numeric( colnames(x) )
    tps        <- sort( unique( timepoints ) )

    if(is.null(xlim)) xlim = c(min(tps),max(tps))
    plot.new()															# make a new plot
    plot.window(xlim=xlim, ylim=ylim, log="")							# add the plot windows size
    #grid()
    for( k in 1:nrow( x ) ) {
        max.p  <- NULL			# the expression of the maximum
        min.p  <- NULL			#
        med.p  <- NULL			#
        for( i in 1:length(tps)) {	#
            idx <- ( 1:length( timepoints ) )[timepoints==tps[i] ]	# get the indeces of the
            work  <- x[k, idx]
            pmax  <- max(work[is.finite(work)], na.rm=TRUE)
            pmin  <- min(work[is.finite(work)], na.rm=TRUE)
            pmed  <- median(work[is.finite(work)], na.rm=TRUE)

            max.p <- append(max.p, pmax)	#
            min.p <- append(min.p, pmin)	#
            med.p <- append(med.p, pmed)	#
        }
        lines( x=tps, y=max.p, type='l', col=col[k] )		# add the lines if requested
        lines( x=tps, y=min.p, type='l', col=col[k] )		# add the lines if requested
        xp <- append(tps, rev(tps))
        yp <- append(max.p, rev(min.p) )

        polygon(xp, y=yp, col=col[k], border=FALSE)
        lines( x=tps, y=med.p, type='l', col=col[k] )		# add the lines if requested
    }


    axis(1)																# add the x axis
    axis(2)																# add the y axis

    title(main=main, xlab=xlab, ylab=ylab, ...)							# add the title and axis labels
    #Addition by Danny Arends
    if (legend){
        if(legendloc){
            plot.new()
        }
        legend("topright", labels, col=col, pch=pch)			# add a legend if requested
    }
    #End of addition
    op <- par(mfrow = c(1,1))
    invisible()															# return the plot
}

getThird <- function(x){
    x[,3]
}

getChr <- function(x){
    x[,1]
}

mqmplot.multitrait <- function(result, type=c("lines","image","contour","3Dplot"), group=NULL, meanprofile=c("none","mean","median"), theta=30, phi=15, ...)
{
    #Helperfunction to plot mqmmulti objects made by doing multiple mqmscan runs (in a LIST)
    type <- match.arg(type)
    meanprofile <- match.arg(meanprofile)
    if(!inherits(result, "mqmmulti")){
        stop("Wrong type of result file, please supply a valid mqmmulti object.")
    }
    n.pheno <- length(result)
    temp <- lapply(result,getThird)
    chrs <- unique(lapply(result,getChr))
    qtldata <- do.call("rbind",temp)
    if(!is.null(group)){
        qtldata <- qtldata[group,]
        colors <- rep("blue",n.pheno)
    }else{
        group <- 1:n.pheno
        colors <- rainbow(n.pheno)
    }
    qtldata <- t(qtldata)
    if(type=="contour"){
        #Countour plot
        contour(x=seq(1,dim(qtldata)[1]),
                y=seq(1,dim(qtldata)[2]),
                qtldata,
                xlab="Markers",ylab="Trait", ...)
        for(x in unique(chrs[[1]])){
            abline(v=sum(as.numeric(chrs[[1]])<=x))
        }
    }
    if(type=="image"){
        #Image plot
        image(x=1:dim(qtldata)[1],y=1:dim(qtldata)[2],qtldata,xlab="Markers",ylab="Trait",...)
        for(x in unique(chrs[[1]])){
            abline(v=sum(as.numeric(chrs[[1]])<=x))
        }
    }
    if(type=="3Dplot"){
        #3D perspective plot
        persp(x=1:dim(qtldata)[1],y=1:dim(qtldata)[2],qtldata,
              theta = theta, phi = phi, expand = 1,
              col="gray", xlab = "Markers", ylab = "Traits", zlab = "LOD score")
    }
    if(type=="lines"){
        #Standard plotting option, Lineplot
        first <- TRUE
        for(i in group) {
            if(first){
                plot(result[[i]],ylim=c(0,max(qtldata)),col=colors[i],lwd=1,ylab="LOD score",xlab="Markers",main="Multiple profiles", ...)
                first <- FALSE
            }else{
                plot(result[[i]],add=TRUE,col=colors[i],lwd=1,...)
            }
        }
        if(meanprofile != "none"){
            temp <- result[[1]]
            if(meanprofile=="median"){
                temp[,3] <- apply(qtldata,1,median)
                legend("topright",c("QTL profiles","Median profile"),col=c("blue","black"),lwd=c(1,3))
            }
            if(meanprofile=="mean"){
                temp[,3] <- rowMeans(qtldata)
                legend("topright",c("QTL profiles","Mean profile"),col=c("blue","black"),lwd=c(1,3))
            }
            plot(temp,add=TRUE,col="black",lwd=3,...)
        }
    }
}

mqmplot.permutations <- function(permutationresult, ...)
{
    #Helperfunction to show mqmmulti objects made by doing multiple mqmscan runs (in a LIST)
    #This function should only be used for bootstrapped data
    matrix <- NULL
    row1 <- NULL
    row2 <- NULL
    i <- 1
    if(!inherits(permutationresult, "mqmmulti"))
        ourstop("Wrong type of result file, please supply a valid mqmmulti object.")

    for( j in 1:length( permutationresult[[i]][,3] ) ) {
        row1 <- NULL
        row2 <- NULL
        for(i in 1:length( permutationresult ) ) {
            if(i==1){
                row1 <- c(row1,rep(permutationresult[[i]][,3][j],(length( permutationresult )-1)))
                names(row1) <- rep(j,(length( permutationresult )-1))
            }else{
                row2 <- c(row2,permutationresult[[i]][,3][j])
            }
        }
        names(row2) <- rep(j,(length( permutationresult )-1))
        matrix <- cbind(matrix,rbind(row1,row2))
    }

    rownames(matrix) <- c("QTL trait",paste("# of bootstraps:",length(permutationresult)-1))

    #Because bootstrap only has 2 rows of data we can use black n blue
    polyplot(matrix,col=c(rgb(0,0,0,1),rgb(0,0,1,0.35)),...)
    #PLot some lines so we know what is significant
    perm.temp <- mqmprocesspermutation(permutationresult)			#Create a permutation object
    numresults <- dim(permutationresult[[1]])[1]
    lines(x=1:numresults,y=rep(summary(perm.temp)[1,1],numresults),col="green",lwd=2,lty=2)
    lines(x=1:numresults,y=rep(summary(perm.temp)[2,1],numresults),col="blue",lwd=2,lty=2)
    chrs <- unique(lapply(permutationresult,getChr))
    for(x in unique(chrs[[1]])){
        abline(v=sum(as.numeric(chrs[[1]])<=x),lty="dashed",col="gray",lwd=1)
    }
}

mqmplot.singletrait <- function(result, extended=0,...)
{
    #Helperfunction to show scanone objects made by doing mqmscan runs
    if(!inherits(result, "scanone")){
        stop("Wrong type of result file, please supply a valid scanone (from MQM) object.")
    }
    if(is.null(result$"info")){
        stop("Wrong type of result file, please supply a valid scanone (from MQM) object.")
    }
    if(is.null(attr(result,"mqmmodel"))){
        op <- par(mfrow = c(1,1))
    }else{
        op <- par(mfrow = c(2,1))
    }
    info.l <- result
    info.l[,3] <- result[,4]
    if(extended){
        if(!is.null(attr(result,"mqmmodel"))){
            plot(attr(result,"mqmmodel"))
        }
        plot(result,lwd=1,col=c("black"),ylab="QTL (LOD)",...)
        par(new=TRUE)
        plot(info.l,lwd=1,col=c("red"),ylab="QTL (LOD)",yaxt="n",lty=1,...)
        grid(length(result$chr),5)
        labels <- c(colnames(result)[3],"Information Content")
        mtext("Information Content",side=4,col="red",line=4)
        axis(4, ylim=c(0,1), col="red",col.axis="red",las=1)

        legend("right", labels,col=c("black","red"),lty=c(1,1,1))
    }else{
        if(!is.null(attr(result,"mqmmodel"))){
            plot(attr(result,"mqmmodel"))
        }
        plot(result,lwd=1,ylab="QTL (LOD)",...)
        grid(length(result$chr),5)
        labels <- c(colnames(result)[3])
        legend("right", labels,col=c("black","blue"),lty=c(1,1))
    }
    op <- par(mfrow = c(1,1))
}

# end of mqmplots.R

convert2bcsft <- function(cross, BC.gen = 0, F.gen = 0, estimate.map = TRUE,
                          error.prob=0.0001, map.function=c("haldane","kosambi","c-f","morgan"),
                          verbose=FALSE)
{
    cross.class <- crosstype(cross)
    if(cross.class=="bcsft") cross.class <- "f2"

    if((cross.class %in% c("bc","f2"))) {
        class(cross) <- c("bcsft", "cross")
        ## If BC.gen = 0 and F.gen = 0, then set to BC1F0 (bc) or BC0F2 (f2).
        if(cross.class == "bc" & F.gen > 0) {
            stop("input cross has only 2 genotypes--cannot have F.gen > 0")
            if(BC.gen == 0)
                BC.gen <- 1
        }
        if(cross.class == "f2") {
            if(F.gen == 0) {
                if(BC.gen == 0)
                    F.gen <- 2
                else
                    stop("input cross has 3 genotypes--cannot have F.gen = 0")
            }
        }
        if(BC.gen < 0 | F.gen < 0)
            stop("BC.gen and F.gen cannot be negative")

        attr(cross, "scheme") <- c(BC.gen, F.gen)
        cross
    }
    else stop("cross object has to be of class bc or f2 to be converted to bcsft")

    # re-estimate map?
    if(estimate.map) {
        cat(" --Estimating genetic map\n")
        newmap <- est.map(cross, error.prob=error.prob, map.function=map.function, verbose=verbose)
        cross <- replace.map(cross, newmap)
    }

    cross
}

read.cross.bcsft <- function(..., BC.gen = 0, F.gen = 0, cross = NULL, force.bcsft = FALSE,
                             estimate.map=TRUE)
{
    ## Must specify s = BC.gen and t = F.gen.
    ## Later: Could import in clever way from qtlcart? See qtlcart_io.R and their software.

    ## Make sure we only estimate map once!
    if(is.null(cross)) # Estimate map at end of this routine (called read.cross.bcsft directly).
        cross <- read.cross(..., estimate.map = FALSE)
    else # Estimate map in parent read.cross() call (read.cross.bcsft is pass-through from read.cross).
        estimate.map <- FALSE

    force.bcsft <- force.bcsft | (BC.gen > 0 | F.gen > 0)
    if((crosstype(cross) %in% c("bc","f2","bcsft")) && force.bcsft) {

        # deal with ... args
        dots <- list(...)

        if("verbose" %in% names(dots)) verbose <- dots$verbose
        else verbose <- TRUE

        if("error.prob" %in% names(dots)) error.prob <- dots$error.prob
        else error.prob <- 0.0001

        if("map.function" %in% names(dots)) map.function <- dots$map.function
        else map.function <- "haldane"

        cross <- convert2bcsft(cross, BC.gen, F.gen, estimate.map = estimate.map,
                               error.prob=error.prob, map.function=map.function,
                               verbose=verbose)
    }

    cross
}

######################################################################

sim.cross.bcsft <- function(map,model,n.ind,error.prob,missing.prob,
                            partial.missing.prob,keep.errorind,
                            m,p,map.function,
                            cross.scheme)
{
    if(map.function=="kosambi") mf <- mf.k
    else if(map.function=="c-f") mf <- mf.cf
    else if(map.function=="morgan") mf <- mf.m
    else mf <- mf.h

    if(any(sapply(map,is.matrix)))
        stop("Map must not be sex-specific.")

    ## cross.scheme = c(s,t) for bcsft.
    if(missing(cross.scheme))
        stop("must specify cross.scheme for bcsft")
    if(length(cross.scheme) != 2)
        stop("cross.scheme for bcsft must have 2 values")
    cross.scheme <- round(cross.scheme)
    if(min(cross.scheme) < 0)
        stop("cross.scheme for bcsft must have 2 non-negative integers")
    n.eff <- 3 + (cross.scheme[2] > 0)

    ## chromosome types
    chr.type <- sapply(map,chrtype)

    n.chr <- length(map)
    if(is.null(model)) n.qtl <- 0
    else {
        if(!((!is.matrix(model) && length(model) == n.eff) ||
             (is.matrix(model) && ncol(model) == n.eff))) {
            stop(paste("Model must be a matrix with ", n.eff, " columns (chr, pos and effect",
                       ifelse(n.eff == 4, "s", ""), ").", sep = ""))
        }
        if(!is.matrix(model)) model <- rbind(model)
        n.qtl <- nrow(model)
        if(any(model[,1] < 0 | model[,1] > n.chr))
            stop("Chromosome indicators in model matrix out of range.")
        model[,2] <- model[,2]+1e-14 # so QTL not on top of marker
    }

    # if any QTLs, place qtls on map
    if(n.qtl > 0) {
        for(i in 1:n.qtl) {
            temp <- map[[model[i,1]]]
            if(model[i,2] < min(temp)) {
                temp <- c(model[i,2],temp)
                names(temp)[1] <- paste("QTL",i,sep="")
            }
            else if(model[i,2] > max(temp)) {
                temp <- c(temp,model[i,2])
                names(temp)[length(temp)] <- paste("QTL",i,sep="")
            }
            else {
                j <- max((seq(along=temp))[temp < model[i,2]])
                temp <- c(temp[1:j],model[i,2],temp[(j+1):length(temp)])
                names(temp)[j+1] <- paste("QTL",i,sep="")
            }
            map[[model[i,1]]] <- temp
        }
    }

    geno <- vector("list", n.chr)
    names(geno) <- names(map)
    n.mar <- sapply(map,length)
    mar.names <- lapply(map,names)
    BC.gen <- cross.scheme[1]
    F.gen <- cross.scheme[2] - (BC.gen == 0)

    for(i in 1:n.chr) {

        # simulate genotype data
        bcallele1 <- sim.bcg(n.ind, map[[i]], m, p, map.function) - 1
        ## BCs: multiply independent instances of meiosis together.
        if(BC.gen > 0) {
            if(BC.gen > 1) for(j in seq(2, BC.gen))
                bcallele1 <- bcallele1 * (sim.bcg(n.ind, map[[i]], m, p, map.function) - 1)
        }

        if(F.gen == 0) ## BCs only.
            thedata <- bcallele1 + 1
        else {
            if(chr.type[i] == "X") {
                if(F.gen > 1) for(j in seq(F.gen))
                    bcallele1 <- bcallele1 * (sim.bcg(n.ind, map[[i]], m, p, map.function) - 1)

                thedata <- bcallele1 + 1
            }
            else { ## chr.type[i] != "X"
                if(BC.gen > 0) { ## Two unique alleles from BC(s).
                    bcallele2 <- bcallele1 * (sim.bcg(n.ind, map[[i]], m, p, map.function) - 1)
                    bcallele1 <- bcallele1 * (sim.bcg(n.ind, map[[i]], m, p, map.function) - 1)
                }
                else ## Starting from F(1) with two unique alleles.
                    bcallele2 <- sim.bcg(n.ind, map[[i]], m, p, map.function) - 1

                if(F.gen > 1) for(j in seq(2, F.gen)) {
                    ## need two instances.
                    allelemask1 <- sim.bcg(n.ind, map[[i]], m, p, map.function) - 1
                    allelemask2 <- sim.bcg(n.ind, map[[i]], m, p, map.function) - 1
                    bcallele1 <- bcallele1 * allelemask1 + bcallele2 * (1 - allelemask1)
                    bcallele2 <- bcallele2 * allelemask2 + bcallele1 * (1 - allelemask2)
                }
                thedata <- bcallele1 + bcallele2 + 1
            }
        }

        dimnames(thedata) <- list(NULL,mar.names[[i]])

        geno[[i]] <- list(data = thedata, map = map[[i]])
        class(geno[[i]]) <- chr.type[i]
        class(geno[[i]]$map) <- NULL

    } # end loop over chromosomes

    # simulate phenotypes
    pheno <- rnorm(n.ind,0,1)

    if(n.qtl > 0) {
        # find QTL positions in genotype data
        QTL.chr <- QTL.loc <- NULL
        for(i in 1:n.chr) {
            o <- grep("^QTL[0-9]+",mar.names[[i]])
            if(length(o)>0) {
                QTL.chr <- c(QTL.chr,rep(i,length(o)))
                QTL.loc <- c(QTL.loc,o)
            }
        }

        # incorporate QTL effects
        for(i in 1:n.qtl) {
            QTL.geno <- geno[[QTL.chr[i]]]$data[,QTL.loc[i]]
            pheno[QTL.geno==2] <- pheno[QTL.geno==2] + model[i,n.eff]
            if(n.eff == 4) {
                pheno[QTL.geno==1] <- pheno[QTL.geno==1] - model[i,3]
                pheno[QTL.geno==3] <- pheno[QTL.geno==3] + model[i,3]
            }
        }

    } # end simulate phenotype

    n.mar <- sapply(geno, function(a) length(a$map))

    # add errors
    if(error.prob > 0) {
        for(i in 1:n.chr) {
            if(chr.type[i]=="X") {
                a <- sample(0:1,n.mar[i]*n.ind,replace=TRUE,
                            prob=c(1-error.prob,error.prob))
                geno[[i]]$data[a == 1] <- 3 - geno[[i]]$data[a == 1]
            }
            else {
                a <- sample(0:2,n.mar[i]*n.ind,replace=TRUE,
                            prob=c(1-error.prob,error.prob/2,error.prob/2))
                if(any(a>0 & geno[[i]]$data==1))
                    geno[[i]]$data[a>0 & geno[[i]]$data==1] <-
                        (geno[[i]]$data+a)[a>0 & geno[[i]]$data==1]
                if(any(a>0 & geno[[i]]$data==2)) {
                    geno[[i]]$data[a>0 & geno[[i]]$data==2] <-
                        (geno[[i]]$data+a)[a>0 & geno[[i]]$data==2]
                    geno[[i]]$data[geno[[i]]$data>3] <- 1
                }
                if(any(a>0 & geno[[i]]$data==3))
                    geno[[i]]$data[a>0 & geno[[i]]$data==3] <-
                        (geno[[i]]$data-a)[a>0 & geno[[i]]$data==3]
            }

            if(keep.errorind) {
                errors <- matrix(0,n.ind,n.mar[i])
                errors[a>0] <- 1
                colnames(errors) <- colnames(geno[[i]]$data)
                geno[[i]]$errors <- errors
            }

        } # end loop over chromosomes
    } # end simulate genotyping errors

    ## add partial missing
    if(partial.missing.prob > 0) {
        for(i in 1:n.chr) {
            if(chr.type[i] != "X") {
                o <- sample(c(TRUE,FALSE),n.mar[i],replace=TRUE,
                            prob=c(partial.missing.prob,1-partial.missing.prob))
                if(any(o)) {
                    o2 <- grep("^QTL[0-9]+",mar.names[[i]])
                    if(length(o2)>0)
                        x <- geno[[i]]$data[,o2]
                    m <- (1:n.mar[i])[o]
                    for(j in m) {
                        if(runif(1) < 0.5)
                            geno[[i]]$data[geno[[i]]$data[,j]==1 | geno[[i]]$data[,j]==2,j] <- 4
                        else
                            geno[[i]]$data[geno[[i]]$data[,j]==3 | geno[[i]]$data[,j]==2,j] <- 5
                    }
                    if(length(o2)>0)
                        geno[[i]]$data[,o2] <- x
                }
            }

        } # end loop over chromosomes
    } # end simulate partially missing data

    # add missing
    if(missing.prob > 0) {
        for(i in 1:n.chr) {
            o <- grep("^QTL[0-9]+",mar.names[[i]])
            if(length(o)>0)
                x <- geno[[i]]$data[,o]
            geno[[i]]$data[sample(c(TRUE,FALSE),n.mar[i]*n.ind,replace=TRUE,
                                  prob=c(missing.prob,1-missing.prob))] <- NA
            if(length(o)>0)
                geno[[i]]$data[,o] <- x
        }
    }

    pheno <- data.frame(phenotype=pheno)

    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c("bcsft","cross")
    attr(cross, "scheme") <- cross.scheme

    cross
}

## End bcsft.R

######################################################################
#
# read.cross.csv.R
#
# copyright (c) 2000-2020, Karl W Broman
# last modified Feb, 2020
# first written Aug, 2000
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.csv
#           [See read.cross.R for the main read.cross function.]
#
######################################################################

######################################################################
#
# read.cross.csv
#
# read data in comma-delimited format
#
######################################################################

read.cross.csv <-
    function(dir, file, na.strings=c("-","NA"),
             genotypes=c("A","H","B","D","C"),
             estimate.map=TRUE, rotate=FALSE, ...)
{
    # create file names
    if(missing(file)) file <- "data.csv"

    if(!missing(dir) && dir != "") {
        file <- file.path(dir, file)
    }

    args <- list(...)

    if("" %in% na.strings) {
        na.strings <- na.strings[na.strings != ""]
        warning("Including \"\" in na.strings will cause problems; omitted.")
    }

    # if user wants to use comma for decimal point, we need
    if(length(args) > 0 && "dec" %in% names(args)) {
        dec <- args[["dec"]]
    }
    else dec <- "."

    # read the data file
    if(length(args) < 1 || !("sep" %in% names(args))) {
        # "sep" not in the "..." argument and so take sep=","
        if(length(args) < 1 || !("comment.char" %in% names(args)))
            data <- read.table(file, sep=",", na.strings=na.strings,
                               colClasses="character", fill=TRUE,
                               stringsAsFactors=TRUE,
                               blank.lines.skip=TRUE, comment.char="", ...)
        else
            data <- read.table(file, sep=",", na.strings=na.strings,
                               colClasses="character", fill=TRUE,
                               stringsAsFactors=TRUE,
                               blank.lines.skip=TRUE, ...)
    }
    else {
        if(length(args) < 1 || !("comment.char" %in% names(args)))
            data <- read.table(file, na.strings=na.strings,
                               colClasses="character", fill=TRUE,
                               stringsAsFactors=TRUE,
                               blank.lines.skip=TRUE, comment.char="", ...)
        else
            data <- read.table(file, na.strings=na.strings,
                               colClasses="character", fill=TRUE,
                               stringsAsFactors=TRUE,
                               blank.lines.skip=TRUE, ...)
    }

    if(rotate)
        data <- as.data.frame(t(data), stringsAsFactors=FALSE)

    empty <- grep("^\\s*$", data[2, ])

    if( ! 1 %in% empty)
        stop("You must include at least one phenotype (e.g., an index). ",
             "There was this value in the first column of the second row '",
             data[2,1],"' where was supposed to be nothing.",sep="")

    # determine number of phenotypes based on initial blanks in row 2
    if(length(empty)==ncol(data))
        stop("Second row has all blank cells; you need to include chromosome IDs for the markers.")
    n.phe <- min((1:ncol(data))[-empty])-1
    empty <- rep(FALSE, n.phe)
    empty[grep("^\\s*$", data[3,1:n.phe])] <- TRUE

    # Is map included?  yes if first n.phe columns in row 3 are all blank
    if(all(empty)) {
        map.included <- TRUE
        map <- asnumericwithdec(unlist(data[3,-(1:n.phe)]), dec=dec)
        if(any(is.na(map))) {
            temp <- unique(unlist(data[3,-(1:n.phe)])[is.na(map)])
            stop("There are missing marker positions.\n",
                 "   In particular, we see these value(s): ",
                 paste("\"",paste(temp,collapse="\",\"",sep=""),"\"",collapse=" ",sep=""),
                 " at position(s): ",
                 paste(which(is.na(map)),colapse=",",sep=""),sep="")
        }
        nondatrow <- 3
    }
    else {
        map.included <- FALSE
        map <- rep(0,ncol(data)-n.phe)
        nondatrow <- 2 # last non-data row
    }

    # replace empty cells with NA
    data <- sapply(data,function(a) { a[!is.na(a) & a==""] <- NA; a })

    pheno <- as.data.frame(data[-(1:nondatrow),1:n.phe,drop=FALSE], stringsAsFactors=TRUE)
    colnames(pheno) <- data[1,1:n.phe]

    # pull apart phenotypes, genotypes and map
    mnames <- data[1,-(1:n.phe)]
    if(any(is.na(mnames)))
        stop("There are missing marker names. Check column(s) ",paste(which(is.na(mnames))+1+n.phe,collapse=","),sep="")
    chr <- data[2,-(1:n.phe)]
    if(any(is.na(chr)))
        stop("There are missing chromosome IDs. Check column(s) ",paste(which(is.na(chr))+1+n.phe,collapse=","),sep="")

    if(length(genotypes) > 0) {
        # look for strange entries in the genotype data
        temp <- unique(as.character(data[-(1:nondatrow),-(1:n.phe),drop=FALSE]))

        temp <- temp[!is.na(temp)]
        wh <- !(temp %in% genotypes)
        if(any(wh)) {
            warn <- "The following unexpected genotype codes were treated as missing.\n    "
            ge <- paste("|", paste(temp[wh],collapse="|"),"|",sep="")
            warn <- paste(warn,ge,"\n",sep="")
            warning(warn)
        }

        # convert genotype data
        allgeno <- matrix(match(data[-(1:nondatrow),-(1:n.phe)],genotypes),
                          ncol=ncol(data)-n.phe)
    }
    else
        allgeno <- matrix(as.numeric(data[-(1:nondatrow),-(1:n.phe)]),
                          ncol=ncol(data)-n.phe)

    oldpheno <- pheno

    pheno <- data.frame(lapply(pheno, sw2numeric, dec=dec), stringsAsFactors=TRUE)

    # re-order the markers by chr and position
    # try to figure out the chr labels
    if(all(chr %in% c(1:999,"X","x"))) { # 1...19 + X
        tempchr <- chr
        tempchr[chr=="X" | chr=="x"] <- 1000
        tempchr <- as.numeric(tempchr)
        if(map.included) neworder <- order(tempchr, map)
        else neworder <- order(tempchr)
    }
    else {
        # don't let it reorder the chromosomes
        tempchr <- factor(chr, levels=unique(chr))

        if(map.included) neworder <- order(tempchr, map)
        else neworder <- order(tempchr)
    }
    chr <- chr[neworder]
    map <- map[neworder]
    allgeno <- allgeno[,neworder,drop=FALSE]
    mnames <- mnames[neworder]

    # fix up dummy map
    if(!map.included) {
        map <- split(rep(0,length(chr)),chr)[unique(chr)]
        map <- unlist(lapply(map,function(a) seq(0,length=length(a),by=5)))
        names(map) <- NULL
    }

    # fix up map information
    # number of chromosomes
    uchr <- unique(chr)
    n.chr <- length(uchr)
    geno <- vector("list",n.chr)
    names(geno) <- uchr
    min.mar <- 1
    allautogeno <- NULL
    for(i in 1:n.chr) { # loop over chromosomes
        # create map
        temp.map <- map[chr==uchr[i]]
        names(temp.map) <- mnames[chr==uchr[i]]

        # pull out appropriate portion of genotype data
        data <- allgeno[,min.mar:(length(temp.map)+min.mar-1),drop=FALSE]
        min.mar <- min.mar + length(temp.map)
        colnames(data) <- names(temp.map)

        geno[[i]] <- list(data=data,map=temp.map)
        if(uchr[i] == "X" || uchr[i] == "x")
            class(geno[[i]]) <- "X"
        else {
            class(geno[[i]]) <- "A"
            if(is.null(allautogeno)) allautogeno <- data
            else allautogeno <- cbind(allautogeno,data)
        }
    }

    if(is.null(allautogeno)) allautogeno <- allgeno

    # check that data dimensions match
    n.mar1 <- sapply(geno,function(a) ncol(a$data))
    n.mar2 <- sapply(geno,function(a) length(a$map))
    n.phe <- ncol(pheno)
    n.ind1 <- nrow(pheno)
    n.ind2 <- sapply(geno,function(a) nrow(a$data))
    if(any(n.ind1 != n.ind2)) {
        cat(n.ind1,n.ind2,"\n")
        stop("Number of individuals in genotypes and phenotypes do not match.")
    }
    if(any(n.mar1 != n.mar2)) {
        cat(n.mar1,n.mar2,"\n")
        stop("Numbers of markers in genotypes and marker names files do not match.")
    }

    # print some information about the amount of data read
    cat(" --Read the following data:\n")
    cat("\t",n.ind1," individuals\n")
    cat("\t",sum(n.mar1)," markers\n")
    cat("\t",n.phe," phenotypes\n")

    # determine map type: f2 or bc or 4way?
    if(all(is.na(allgeno))) warning("There is no genotype data!\n")
    if(all(is.na(allautogeno)) || max(allautogeno,na.rm=TRUE)<=2) type <- "bc"
    else if(max(allautogeno,na.rm=TRUE)<=5) type <- "f2"
    else type <- "4way"
    cross <- list(geno=geno,pheno=pheno)
    class(cross) <- c(type,"cross")

    # check that nothing is strange in the genotype data
    if(type=="f2") max.gen <- 5
    else if(type=="bc") max.gen <- 2
    else max.gen <- 14

    # check that markers are in proper order
    #     if not, fix up the order
    for(i in 1:n.chr) {
        if(any(diff(cross$geno[[i]]$map)<0)) {
            o <- order(cross$geno[[i]]$map)
            cross$geno[[i]]$map <- cross$geno[[i]]$map[o]
            cross$geno[[i]]$data <- cross$geno[[i]]$data[,o,drop=FALSE]
        }
    }

    # estimate genetic map
    if(estimate.map && !map.included) estmap <- TRUE
    else estmap <- FALSE

    # return cross + indicator of whether to run est.map
    list(cross,estmap)
}

# end of read.cross.csv.R

######################################################################
#
# discan.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Oct, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: discan
#
######################################################################

######################################################################
#
# discan: scan genome, calculating LOD scores with single QTL model
#         for a dichotomous trait
#
######################################################################

discan <-
    function(cross, pheno, method=c("em","hk","mr"),
             addcovar=NULL, intcovar=NULL, maxit=4000, tol=1e-4,
             verbose=FALSE, give.warnings=TRUE, ind.noqtl)
{
    method <- match.arg(method)

    n.ind <- nind(cross)
    n.chr <- nchr(cross)
    type <- crosstype(cross)
    if(is.null(addcovar)) n.addcovar <- 0
    else n.addcovar <- ncol(addcovar)
    if(is.null(intcovar)) n.intcovar <- 0
    else n.intcovar <- ncol(intcovar)

    # individuals with no QTL effect
    if(missing(ind.noqtl)) ind.noqtl <- rep(FALSE, nind(cross))
    else {
        if(!is.logical(ind.noqtl) || length(ind.noqtl) != nind(cross))
            stop("ind.noqtl be a logical vector of length n.ind (", nind(cross), ")")

        if(sum(ind.noqtl) > 1) {
            if(method == "mr") {
                ind.noqtl <- rep(FALSE, nind(cross))
                warning("ind.noqtl ignored for method=", method, ", model=binary")
            }
            else if(is.null(addcovar) && (!is.logical(ind.noqtl) || any(ind.noqtl))) {
                ind.noqtl <- rep(FALSE, nind(cross))
                warning("ind.noqtl ignored when no additive covariates")
            }
        }
    }

    if(method=="mr" && n.addcovar+n.intcovar>0)  {
        if(give.warnings) warning("Covariates ignored with method=\"mr\"; use \"em\" instead")
        n.addcovar <- n.intcovar <- addcovar <- intcovar <- 0
    }

    u <- unique(pheno)
    if(any(u != 0 & u != 1))
        stop("Phenotypes must be either 0 or 1.")

    results <- NULL

    llik0 <- c("A"=NA,"X"=NA)
    nullcoef <- list("A"=NA,"X"=NA)

    for(i in 1:n.chr) {

        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type=="X") {
            sexpgm <- getsex(cross)

            ac <- revisecovar(sexpgm,addcovar)
            if(!is.null(addcovar) && (nd <- attr(ac, "n.dropped")) > 0 && give.warnings)
                warning("Dropped ", nd, " additive covariates on X chromosome.")
            if(length(ac)==0) {
                n.ac <- 0
                ac <- NULL
            }
            else n.ac <- ncol(ac)
            ic <- revisecovar(sexpgm,intcovar)
            if(!is.null(intcovar) && (nd <- attr(ic, "n.dropped")) > 0 && give.warnings)
                warning("Dropped ", nd, " interactive covariates on X chromosome.")
            if(length(ic)==0) {
                n.ic <- 0
                ic <- NULL
            }
            else n.ic <- ncol(ic)
        }
        else {
            sexpgm <- NULL
            ac <- addcovar
            n.ac <- n.addcovar
            ic <- intcovar
            n.ic <- n.intcovar
        }

        # get null log liklihood
        if(is.na(llik0[chr_type])) {
            if(n.ac > 0)
                nullfit <- glm(pheno ~ ac, family=binomial(link="logit"))
            else if(i==1)
                nullfit <- glm(pheno ~ 1, family=binomial(link="logit"))
            fitted <- nullfit$fitted
            nullcoef[[chr_type]] <- nullfit$coef
            llik0[chr_type] <- sum(pheno*log10(fitted) + (1-pheno)*log10(1-fitted))
        }

        # get genotype names
        gen.names <- getgenonames(type,chr_type,"full",sexpgm,attributes(cross))
        n.gen <- length(gen.names)

        # pull out genotype data (mr)
        # or genotype probabilities (em)
        if(method == "mr") {
            newgeno <- cross$geno[[i]]$data
            newgeno[is.na(newgeno)] <- 0

            # discard partially informative genotypes
            if(type=="f2") newgeno[newgeno>3] <- 0
            if(type=="4way") newgeno[newgeno>4] <- 0

            # revise X chromosome genotypes
            if(chr_type=="X" && (type=="bc" || type=="f2"))
                newgeno <- reviseXdata(type, "full", sexpgm, geno=newgeno,
                                       cross.attr=attributes(cross))

            n.pos <- ncol(newgeno)
            map <- cross$geno[[i]]$map
            if(is.matrix(map)) map <- map[1,]

            z <- .C("R_discan_mr",
                    as.integer(n.ind),         # number of individuals
                    as.integer(n.pos),         # number of markers
                    as.integer(n.gen),         # number of possible genotypes
                    as.integer(newgeno),       # genotype data
                    as.integer(pheno),          # phenotype data
                    result=as.double(rep(0,n.pos*(n.gen+1))),
                    PACKAGE="qtl")

        }
        else {
            if(!("prob" %in% names(cross$geno[[i]]))) { # need to run calc.genoprob
                if(give.warnings) warning("First running calc.genoprob.")
                cross <- calc.genoprob(cross)
            }
            genoprob <- cross$geno[[i]]$prob
            n.pos <- ncol(genoprob)

            # revise X chromosome genotypes
            if(chr_type=="X" && (type=="bc" || type=="f2"))
                genoprob <- reviseXdata(type, "full", sexpgm, prob=genoprob,
                                        cross.attr=attributes(cross))

            if("map" %in% names(attributes(cross$geno[[i]]$prob)))
                map <- attr(cross$geno[[i]]$prob,"map")
            else {
                stp <- attr(cross$geno[[i]]$prob, "step")
                oe <- attr(cross$geno[[i]]$prob, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                    stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
            }

            if(is.matrix(map)) map <- map[1,]

            if(method=="hk") {
                z <- .C("R_scanone_hk_binary",
                        as.integer(n.ind),         # number of individuals
                        as.integer(n.pos),         # number of markers
                        as.integer(n.gen),         # number of possible genotypes
                        as.double(genoprob),       # genotype probabilities
                        as.double(ac),
                        as.integer(n.ac),
                        as.double(ic),
                        as.integer(n.ic),
                        as.double(pheno),          # phenotype data
                        result=as.double(rep(0,n.pos)),
                        as.double(tol),
                        as.integer(maxit),
                        as.integer(verbose),
                        as.integer(ind.noqtl),
                        PACKAGE="qtl")
            }
            else if(n.ac + n.ic > 0) {

                start <- rep(nullcoef[[chr_type]][1],n.gen)
                if(n.ac > 0)
                    start <- c(start, nullcoef[[chr_type]][-1])
                if(n.ic > 0)
                    start <- c(start, rep(0, n.ic*(n.gen-1)))

                z <- .C("R_discan_covar",
                        as.integer(n.ind),         # number of individuals
                        as.integer(n.pos),         # number of markers
                        as.integer(n.gen),         # number of possible genotypes
                        as.double(genoprob),       # genotype probabilities
                        as.double(ac),
                        as.integer(n.ac),
                        as.double(ic),
                        as.integer(n.ic),
                        as.integer(pheno),          # phenotype data
                        as.double(start),
                        result=as.double(rep(0,n.pos)),
                        as.integer(maxit),
                        as.double(tol),
                        as.integer(verbose),
                        as.integer(ind.noqtl),
                        PACKAGE="qtl")
            }
            else {
                z <- .C("R_discan_im",
                        as.integer(n.ind),         # number of individuals
                        as.integer(n.pos),         # number of markers
                        as.integer(n.gen),         # number of possible genotypes
                        as.double(genoprob),       # genotype probabilities
                        as.integer(pheno),          # phenotype data
                        result=as.double(rep(0,n.pos)),
                        as.integer(maxit),
                        as.double(tol),
                        PACKAGE="qtl")
            }

        }
        z <- matrix(z$result,nrow=n.pos)

        if(method != "mr") z[,1] <- z[,1] - llik0[chr_type]
        z[is.na(z[,1]),1] <- 0

        z <- z[,1,drop=FALSE]

        colnames(z)[1] <- "lod"

        # get null log10 likelihood for the X chromosome
        adjustX <- FALSE
        if(chr_type=="X") {
            # determine which covariates belong in null hypothesis
            temp <- scanoneXnull(type, sexpgm, cross.attr=attributes(cross))
            adjustX <- temp$adjustX
            parX0 <- temp$parX0+n.ac
            sexpgmcovar <- temp$sexpgmcovar
            sexpgmcovar.alt <- temp$sexpgmcovar.alt

            if(adjustX) { # get LOD-score adjustment
                if(n.ac > 0) {
                    nullfitX <- glm(pheno ~ ac+sexpgmcovar,
                                    family=binomial(link="logit"))
                    parX0 <- lm(pheno~ac+sexpgmcovar)$rank
                }
                else
                    nullfitX <- glm(pheno ~ sexpgmcovar,
                                    family=binomial(link="logit"))
                fittedX <- nullfitX$fitted
                llik0X <- sum(pheno*log10(fittedX) + (1-pheno)*log10(1-fittedX))

                # adjust LOD curve
                z <- z - (llik0X - llik0["X"])
            }

        }

        w <- names(map)
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",names(cross$geno)[i],".",w[o],sep="")

        z <- data.frame(chr=rep(names(cross$geno)[i],length(map)),
                        pos=as.numeric(map), z, stringsAsFactors=TRUE)
        rownames(z) <- w

        results <- rbind(results, z)
    } # loop over chromosomes

    class(results) <- c("scanone","data.frame")
    attr(results,"method") <- method
    attr(results,"type") <- type
    attr(results,"model") <- "binary"
    attr(results,"null.log10.lik") <- llik0["A"]
    if(adjustX)
        attr(results,"null.log10.lik.X") <- llik0X

    results
}

# end of discan.R

# scanonevar
# single-QTL genome scan for QTL affecting variance
# with code from Lars Ronnegard

scanonevar <-
function(cross, pheno.col=1, mean_covar = NULL, var_covar = NULL,
         maxit = 25 , tol=1e-6, quiet=TRUE)
{
    # check input
    crosstype <- crosstype(cross)
    if(!(crosstype %in% c("bc", "dh", "f2", "haploid", "risib", "riself")))
       stop('scanonevar not implemented for cross type "', crosstype, '"')
    chr_type <- sapply(cross$geno, chrtype)
    if(any(chr_type=="X")) {
        warning("Analysis of X chromosome not implemented for scanonevar; omitted.")
        cross <- subset(cross, chr=(chr_type != "X"))
    }

    # grab phenotype
    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }
    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(any(is.na(num))) {
            if(sum(is.na(num)) > 1)
                stop("Couldn't identify phenotypes ", paste(paste("\"", pheno.col[is.na(num)], "\"", sep=""),
                                                            collapse=" "))
            else
                stop("Couldn't identify phenotype \"", pheno.col[is.na(num)], "\"")
        }
        pheno.col <- num
    }
    if(any(pheno.col < 1 | pheno.col > nphe(cross)))
        stop("pheno.col values should be between 1 and the no. phenotypes")
    pheno <- cross$pheno[,pheno.col]
    if(is.matrix(pheno) && ncol(pheno) > 1) {
        pheno <- pheno[,1]
        warning('scanonevar requires a single phenotype; all but "', phenames(cross)[pheno.col[1]], '" omitted.')
    }

    N <- length(pheno) # No. individuals
    n.chr <- nchr(cross) #No. chromosomes
    chr.names <- chrnames(cross)

    # need to run calc.genoprob?
    if(!("prob" %in% names(cross$geno[[1]]))) {
        warning("First running calc.genoprob")
        cross <- calc.genoprob(cross)
    }

    scan.logPm <- scan.logPd <- chr.names.out <- NULL

    # set up data and formulas
    X <- cbind(pheno=pheno, add=rep(0, length(pheno)))
    mean_formula <- var_formula <- "pheno ~ add"
    if(!is.null(mean_covar)) {
        ncolX <- ncol(X)
        X <- cbind(X, mean_covar)
        meancovarnames <- paste0("meancov", 1:(ncol(X)-ncolX))
        colnames(X)[-(1:ncolX)] <- meancovarnames
        mean_formula <- paste(mean_formula, "+",
                              paste(meancovarnames, collapse="+"))
    }
    if(!is.null(var_covar)) {
        ncolX <- ncol(X)
        X <- cbind(X, var_covar)
        varcovarnames <- paste0("varcov", 1:(ncol(X)-ncolX))
        colnames(X)[-(1:ncolX)] <- varcovarnames
        var_formula <- paste(var_formula, "+",
                             paste(varcovarnames, collapse="+"))
    }
    X <- as.data.frame(X)
    mean_formula <- as.formula(mean_formula)
    var_formula <- as.formula(var_formula)

    result <- NULL
    for(j in seq(along=cross$geno)) { # loop over chromosomes
        if(!quiet) message(" - Chr ", chr.names[j])

        if (crosstype=="f2") {
            g11 <- cross$geno[[j]]$prob[,,1]
            g12 <- cross$geno[[j]]$prob[,,2]
            g13 <- cross$geno[[j]]$prob[,,3]
            a1  <- g11 + g12/2
            d1 <-  g12 - (g11+g13)/2
        }
        else {
            a1 <- cross$geno[[j]]$prob[,,1]
        }

        n.loci <- dim(a1)[2]

        logP.m <- logP.d <- numeric(n.loci)

        for(i in 1:n.loci) { # loop over positions within chromosome
            # fill in genotype probs for this locus
            X[,2] <- a1[,i]

            d.fit <- DGLM_norm(m.form=mean_formula, d.form=var_formula, indata=X,
                                maxiter=maxit, conv=tol)

            p.mean <- summary(d.fit$mean)$coef[2,4]
            p.disp<- summary(d.fit$disp)$coef[2,4]
            if (d.fit$iter < maxit) {
                logP.m[i]<- -log10(p.mean)
                logP.d[i]<- -log10(p.disp)
            }
            else {
                logP.m[i]<- -log10(p.mean)
                logP.d[i]<- 0
                warning("dglm did not converge on chr", chr.names[j], " position ", i)
            }
        }

        # set up the output
        map <- attr(cross$geno[[j]]$prob,"map")
        w <- names(map)
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",chr.names[j],".",w[o],sep="")
        thischr <- data.frame(chr=rep(chr.names[j], length(w)),
                              pos=unclass(map),
                              neglogP_mean=logP.m,
                              neglogP_disp=logP.d, stringsAsFactors=FALSE)
        rownames(thischr) <- w

        if(is.null(result)) result <- thischr
        else result <- rbind(result, thischr)
    }

    class(result) <- c("scanone", "data.frame")
    attr(result, "method") <- "scanonevar"

    result
}

DGLM_norm <-
    function(m.form, d.form, indata, maxiter=20, conv=1e-6)
{
    X.mean <- model.matrix(m.form, data = indata)
    X.disp <- model.matrix(d.form, data = indata)
    y.name <- all.vars(m.form)[1]
    y <- indata[,y.name]
    w <- rep(1, nrow(indata))
    convergence <- 1
    iter <- 0
    while (convergence > conv & iter < maxiter) {
        iter <- iter +1
        w.old <- w
        glm1 <- lm(y~.-1, weights=w, data=data.frame(X.mean))
        res <- resid(glm1)
        q <- hatvalues(glm1)
        y2 <- res^2/(1-q)
        glm2 <- glm(y2~.-1, family=Gamma(link=log), weights=(1-q)/2, data=data.frame(X.disp))
        w <- 1/fitted(glm2)
        convergence <- (max(abs(w.old-w)) + (summary(glm1)$sigma-1) )
    }
    return(list(mean=glm1, disp=glm2, iter=iter))
}

#####################################################################
#
# mqmpermutation.R
#
# Copyright (c) 2009-2019, Danny Arends
#
# Modified by Pjotr Prins and Karl Broman
#
#
# first written Februari 2009
# last modified Dec 2019
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: mqmscanfdr
#           mqmpermutation
#           mqmprocesspermutation
#
#
#####################################################################

mqmscanfdr <- function(cross, scanfunction=mqmscanall, thresholds=c(1,2,3,4,5,7,10,15,20), n.perm = 10, verbose=FALSE, ...){
    if(verbose){cat("Calculation of FDR estimate of threshold in multitrait analysis.\n")}
    results <- NULL
    above.in.real.res <- NULL
    res <- scanfunction(cross,...)
    for(threshold in thresholds){
        above.in.real <- 0
        for(x in 1:nphe(cross)){
            above.in.real = above.in.real + sum(res[[x]][,3] > threshold)
        }
        above.in.real.res <- c(above.in.real.res,above.in.real)
    }
    perm <- cross
    if(verbose){cat("QTL's above threshold:",above.in.real,"\n")}
    above.in.perm.res <- rep(0,length(thresholds))
    for(x in 1:n.perm){
        if(verbose){cat("Starting permutation",x,"\n")}
        perm.res <- NULL

        neworder <- sample(nind(cross))
        for(chr in 1:nchr(cross)){
            perm$geno[[chr]]$data <- perm$geno[[chr]]$data[neworder,]
        }
        res <- scanfunction(perm, ...)
        for(threshold in thresholds){
            above.in.perm <- 0
            for(y in 1:nphe(cross)){
                above.in.perm = above.in.perm + sum(res[[y]][,3] > threshold)
            }
            perm.res <- c(perm.res,above.in.perm)
            #if(verbose){cat("Permutation",x,"QTL's above threshold:",above.in.perm,"\n")}
        }
        above.in.perm.res <- above.in.perm.res+perm.res
    }
    above.in.perm.res <- above.in.perm.res/n.perm
    results <- cbind(above.in.real.res,above.in.perm.res,above.in.perm.res/above.in.real.res)
    rownames(results) <- thresholds
    results
}
######################################################################
#
# mqmpermutation: Shuffles phenotype or does parametric bootstrapping of mqmscan
#
######################################################################

mqmpermutation <- function(cross,scanfunction=scanone,pheno.col=1,multicore=TRUE,n.perm=10,file="MQM_output.txt",n.cluster=1,method=c("permutation","simulation"),cofactors=NULL,plot=FALSE,verbose=FALSE,...)
{
    bootmethod <- 0

    supported <- c("permutation","simulation")
    bootmethod <- pmatch(method, supported)[1]-1
    if(missing(cross))
        stop("No cross file. Please supply a valid cross object.")

    crosstype <- crosstype(cross)
    if(crosstype == "f2" || crosstype == "bc" || crosstype == "riself"){
        #Echo back the cross type
        if(verbose) {
            cat("------------------------------------------------------------------\n")
            cat("Starting permutation analysis\n")
            cat("Number of permutations:",n.perm,"\n")
            cat("n.cluster:",n.cluster,"\n")
            cat("------------------------------------------------------------------\n")
            cat("INFO: Received a valid cross file type:",crosstype,".\n")
        }
        b <- proc.time()
        if(!bootmethod){
            if(verbose) cat("INFO: Shuffleling traits between individuals.\n")
        }else{
            if(verbose) cat("INFO: Parametric permutation\nINFO: Calculating new traits for each individual.\n")
        }

        #Set the Phenotype under interest as the first
        cross$pheno[,1] <- cross$pheno[,pheno.col]
        names(cross$pheno)[1] <- names(cross$pheno)[pheno.col]

        #Scan the original
        #cross <- fill.geno(cross) # <- this should be done outside of this function
        res0 <- lapply(1, FUN=snowCoreALL,all.data=cross,scanfunction=scanfunction,verbose=verbose,cofactors=cofactors,...)

        #Setup bootstraps by generating a list of random numbers to set as seed for each bootstrap
        batchsize <- n.perm
        bootstraps <- runif(n.perm)
        batches <- length(bootstraps) %/% batchsize
        last.batch.num <- length(bootstraps) %% batchsize
        results <- NULL
        if(last.batch.num > 0){
            batches = batches+1
        }
        SUM <- 0
        AVG <- 0
        LEFT <- 0
        if(multicore && n.cluster >1) {
            updateParallelRNG(n.cluster)
            if(verbose) cat("INFO: Using ",n.cluster," Cores/CPU's/PC's for calculation.\n")
            for(x in 1:(batches)){
                start <- proc.time()
                if(verbose) {
                    ourline()
                    cat("INFO: Starting with batch",x,"/",batches,"\n")
                    ourline()
                }
                if(x==batches && last.batch.num > 0){
                    boots <- bootstraps[((batchsize*(x-1))+1):((batchsize*(x-1))+last.batch.num)]
                }else{
                    boots <- bootstraps[((batchsize*(x-1))+1):(batchsize*(x-1)+batchsize)]
                }
                if(Sys.info()[1] == "Windows") { # Windows doesn't support mclapply, but it's faster if available
                    cl <- makeCluster(n.cluster)
                    on.exit(stopCluster(cl))
                    res <- clusterApply(cl, boots, snowCoreBOOT, all.data=cross, scanfunction=scanfunction, bootmethod=bootmethod,
                                        cofactors=cofactors, verbose=verbose, ...)
                }
                else {
                    res <- mclapply(boots, snowCoreBOOT, all.data=cross, scanfunction=scanfunction, bootmethod=bootmethod,
                                    cofactors=cofactors, verbose=verbose, mc.cores=n.cluster, ...)
                }
                results <- c(results,res)
                if(plot){
                    temp <- c(res0,results)
                    class(temp) <- c(class(temp),"mqmmulti")
                    mqmplot.permutations(temp)
                }
                end <- proc.time()
                SUM <- SUM + (end-start)[3]
                AVG <- SUM/x
                LEFT <- AVG*(batches-x)
                if(verbose) {
                    cat("INFO: Done with batch",x,"/",batches,"\n")
                    cat("INFO: Calculation of batch",x,"took:",round((end-start)[3], digits=3),"seconds\n")
                    cat("INFO: Elapsed time:",(SUM%/%3600),":",(SUM%%3600)%/%60,":",round(SUM%%60, digits=0),"(Hour:Min:Sec)\n")
                    cat("INFO: Average time per batch:",round((AVG), digits=3)," per trait:",round((AVG %/% batchsize), digits=3),"seconds\n")
                    cat("INFO: Estimated time left:",LEFT%/%3600,":",(LEFT%%3600)%/%60,":",round(LEFT%%60,digits=0),"(Hour:Min:Sec)\n")
                    ourline()
                }
            }
        }else{
            if(verbose) cat("INFO: Going into singlemode.\n")
            for(x in 1:(batches)){
                start <- proc.time()
                if(verbose) {
                    ourline()
                    cat("INFO: Starting with batch",x,"/",batches,"\n")
                    ourline()
                }
                if(x==batches && last.batch.num > 0){
                    boots <- bootstraps[((batchsize*(x-1))+1):((batchsize*(x-1))+last.batch.num)]
                }else{
                    boots <- bootstraps[((batchsize*(x-1))+1):(batchsize*(x-1)+batchsize)]
                }
                res <- lapply(boots, FUN=snowCoreBOOT,all.data=cross,scanfunction=scanfunction,bootmethod=bootmethod,cofactors=cofactors,verbose=verbose,...)
                results <- c(results,res)
                if(plot){
                    temp <- c(res0,results)
                    class(temp) <- c(class(temp),"mqmmulti")
                    mqmplot.permutations(temp)
                }
                end <- proc.time()
                SUM <- SUM + (end-start)[3]
                AVG <- SUM/x
                LEFT <- AVG*(batches-x)
                if(verbose) {
                    cat("INFO: Done with batch",x,"/",batches,"\n")
                    cat("INFO: Calculation of batch",x,"took:",round((end-start)[3], digits=3),"seconds\n")
                    cat("INFO: Elapsed time:",(SUM%/%3600),":",(SUM%%3600)%/%60,":",round(SUM%%60, digits=0),"(Hour:Min:Sec)\n")
                    cat("INFO: Average time per batch:",round((AVG), digits=3),",per run:",round((AVG %/% batchsize), digits=3),"seconds\n")
                    cat("INFO: Estimated time left:",LEFT%/%3600,":",(LEFT%%3600)%/%60,":",round(LEFT%%60,digits=0),"(Hour:Min:Sec)\n")
                    ourline()
                }
            }
        }
        res <- c(res0,results)
        #Set the class of the result to mqmmulti (so we can use our plotting routines)
        class(res) <- c(class(res),"mqmmulti")
        e <- proc.time()
        SUM <- (e-b)[3]
        AVG <- SUM/(n.perm+1)
        if(verbose) {
            cat("INFO: Done with MQM permutation analysis\n")
            cat("------------------------------------------------------------------\n")
            cat("INFO: Elapsed time:",(SUM%/%3600),":",(SUM%%3600)%/%60,":",round(SUM%%60, digits=0),"(Hour:Min:Sec)\n")
            cat("INFO: Average time per trait:",round(AVG, digits=3),"seconds\n")
            cat("------------------------------------------------------------------\n")
        }
        res
    }else{
        stop("Currently only F2, BC, and selfed RIL crosses can be analyzed by MQM.")
    }
}

mqmprocesspermutation <- function(mqmpermutationresult = NULL){
    if(!is.null(mqmpermutationresult) && inherits(mqmpermutationresult, "mqmmulti")){
        result <- NULL
        result <- sapply(mqmpermutationresult[-1], function(a) max(a[,3], na.rm=TRUE))
        result <- as.matrix(result)
        colnames(result) <- colnames(mqmpermutationresult[[2]])[3]
        rownames(result) <- 1:(length(mqmpermutationresult)-1)
        class(result) <- c("scanoneperm",class(result))
        result
    }else{
        stop("Please supply a valid resultobject (mqmmulti).")
    }
}

# end of mqmpermutation.R

######################################################################
#
# plot.R
#
# copyright (c) 2000-2019, Karl W Broman
#       [modifications of plot.cross from Brian Yandell]
# last modified Dec, 2019
# first written Mar, 2000
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: plotMissing, plotMap, plot.cross, plotGeno, plotInfo,
#           plotPXG, plotPheno
#
######################################################################

plotMissing <-
    function(x, chr, reorder=FALSE, main="Missing genotypes",
             alternate.chrid=FALSE, ...)
{
    cross <- x
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")
    if(!missing(chr)) cross <- subset(cross,chr=chr)

    # get full genotype data into one matrix
    Geno <- pull.geno(cross)

    # reorder the individuals according to their phenotype
    o <- 1:nrow(Geno)
    if(reorder) {
        # if reorder is a number, use the corresponding phenotype
        if(is.numeric(reorder)) {
            if(reorder < 1 || reorder > nphe(cross))
                stop("reorder should be TRUE, FALSE, or an integer between 1 and", nphe(cross))

            o <- order(cross$pheno[,reorder])
        }

        # otherwise, order according to the sum of the numeric phenotypes
        else {
            wh <- sapply(cross$pheno, is.numeric)
            o <- order(apply(cross$pheno[,wh,drop=FALSE],1,sum))
        }
    }

    # make matrix with  0 where genotype data is missing
    #                   1 where data is not missing
    #                 0.5 where data is partially missing
    type <- crosstype(cross)
    g <- t(Geno[o,])
    g[is.na(g)] <- 0
    if(type == "bc" || type=="risib" || type=="riself" || type=="bc")
        g[g > 0] <- 1
    else if(type=="f2") {
        g[g > 0 & g < 4] <- 1
        g[g > 3] <- 0.5
    }
    else if(type=="4way") {
        g[g > 0 & g < 5] <- 1
        g[g > 4] <- 0.5
    }
    else {
        g[g > 0] <- 1
    }

    old.xpd <- par("xpd")
    old.las <- par("las")
    par(xpd=TRUE,las=1)
    on.exit(par(xpd=old.xpd,las=old.las))

    colors <- c("#000000", "gray80", "#FFFFFF")

    # plot grid with black pixels where there is missing data
    image(1:nrow(g),1:ncol(g),g,ylab="Individuals",xlab="Markers",col=colors,zlim=c(0,1))

    # plot lines at the chromosome boundaries
    n.mar <- nmar(cross)
    n.chr <- nchr(cross)
    a <- c(0.5,cumsum(n.mar)+0.5)

    # the following makes the lines go slightly above the plotting region
    b <- par("usr")
    segments(a,b[3],a,b[4]+diff(b[3:4])*0.02)

    # this line adds a line above the image
    #     (the image function seems to leave it out)
    abline(h=0.5+c(0,ncol(g)),xpd=FALSE)

    # add chromosome numbers
    a <- par("usr")
    wh <- cumsum(c(0.5,n.mar))
    x <- 1:n.chr
    for(i in 1:n.chr)
        x[i] <- mean(wh[i+c(0,1)])

    thechr <- names(cross$geno)
    if(!alternate.chrid || length(thechr) < 2) {
        for(i in seq(along=x))
            axis(side=3, at=x[i], thechr[i], tick=FALSE, line=-0.5)
    }
    else {
        odd <- seq(1, length(x), by=2)
        even <- seq(2, length(x), by=2)
        for(i in odd) {
            axis(side=3, at=x[i], labels=thechr[i], line=-0.75, tick=FALSE)
        }
        for(i in even) {
            axis(side=3, at=x[i], labels=thechr[i], line=+0, tick=FALSE)
        }
    }

    title(main=main)
    invisible()
}

geno.image <-
    function(x, chr, reorder=FALSE, main="Genotype data",
             alternate.chrid=FALSE, col=NULL, ...)
{
    cross <- x
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")
    if(!missing(chr)) cross <- subset(cross,chr=chr)

    type <- crosstype(cross)

    # revise X chromosome data
    if(type=="bc" || type=="f2") {
        chr_type <- sapply(cross$geno, chrtype)
        if(any(chr_type=="X")) {
            for(i in which(chr_type=="X"))
                cross$geno[[i]]$data <- reviseXdata(type, "simple", getsex(cross),
                                                    geno=cross$geno[[i]]$data,
                                                    cross.attr=attributes(cross))
        }
    }

    # get full genotype data into one matrix
    Geno <- pull.geno(cross)

    # colors to use
    maxgeno <- max(Geno, na.rm=TRUE)
    if(type != "4way") {
        thecolors <- c("white", "#E41A1C", "#377EB8", "#4DAF4A", "#984EA3", "#FF7F00")
        thebreaks <- seq(-0.5, 5.5, by=1)
    }
    else {
        if(maxgeno <= 5) {
            thecolors <- c("white", "#E41A1C", "#377EB8", "#4DAF4A", "#984EA3", "#FF7F00")
            thebreaks <- seq(-0.5, 5.5, by=1)
        }
        else {
            thecolors <- c("white",  "#8DD3C7", "#FFFFB3", "#BEBADA", "#FB8072",
                           "#80B1D3", "#FDB462", "#B3DE69", "#FCCDE5", "#D9D9D9", "#BC80BD")
            thebreaks <- seq(-0.5, 10.5, by=1)
        }
    }
    thecolors <- thecolors[1:(maxgeno+1)]
    thebreaks <- thebreaks[1:(maxgeno+2)]
    if(!is.null(col)) { # colors provided
        if(length(col) < length(thecolors)) {
            warning("col should have length ", maxgeno+1)
            thecolors[seq_along(col)] <- col
        }
        else {
            thecolors <- col[1:(maxgeno+1)]
        }
    }

    # reorder the individuals according to their phenotype
    o <- 1:nrow(Geno)
    if(reorder) {
        # if reorder is a number, use the corresponding phenotype
        if(is.numeric(reorder)) {
            if(reorder < 1 || reorder > nphe(cross))
                stop("reorder should be TRUE, FALSE, or an integer between 1 and ", nphe(cross))

            o <- order(cross$pheno[,reorder])
        }

        # otherwise, order according to the sum of the numeric phenotypes
        else {
            wh <- sapply(cross$pheno, is.numeric)
            o <- order(apply(cross$pheno[,wh,drop=FALSE],1,sum))
        }
    }

    g <- t(Geno[o,])
    g[is.na(g)] <- 0

    # make matrix with  0 where genotype data is missing
    #                   1 where data is not missing
    #                 0.5 where data is partially missing
    old.xpd <- par("xpd")
    old.las <- par("las")
    par(xpd=TRUE,las=1)
    on.exit(par(xpd=old.xpd,las=old.las))

    # plot grid with black pixels where there is missing data
    plot_image_sub <-
        function(g, ylab="Individuals",xlab="Markers", col=thecolors, ...)
        {
            if(length(thebreaks) != length(col)+1)
                stop("Must have one more break than color\n",
                     "length(breaks) = ", length(thebreaks),
                     "\nlength(col) = ", length(col))
            image(1:nrow(g),1:ncol(g), g, col=col, xlab=xlab, ylab=ylab,
                  breaks=thebreaks, ...)
        }
    plot_image_sub(g, ...)

    # plot lines at the chromosome boundaries
    n.mar <- nmar(cross)
    n.chr <- nchr(cross)
    a <- c(0.5,cumsum(n.mar)+0.5)

    # the following makes the lines go slightly above the plotting region
    b <- par("usr")
    segments(a,b[3],a,b[4]+diff(b[3:4])*0.02)

    # this line adds a line above the image
    #     (the image function seems to leave it out)
    abline(h=0.5+c(0,ncol(g)),xpd=FALSE)

    # add chromosome numbers
    a <- par("usr")
    wh <- cumsum(c(0.5,n.mar))
    x <- 1:n.chr
    for(i in 1:n.chr)
        x[i] <- mean(wh[i+c(0,1)])
    thechr <- names(cross$geno)
    if(!alternate.chrid || length(thechr) < 2) {
        for(i in seq(along=x))
            axis(side=3, at=x[i], thechr[i], tick=FALSE, line=-0.5)
    }
    else {
        odd <- seq(1, length(x), by=2)
        even <- seq(2, length(x), by=2)
        for(i in odd)
            axis(side=3, at=x[i], labels=thechr[i], line=-0.75, tick=FALSE)
        for(i in even)
            axis(side=3, at=x[i], labels=thechr[i], line=+0, tick=FALSE)
    }

    title(main=main)
    invisible()
}

plotMap <- plot.map <-
    function(x, map2, chr, horizontal=FALSE, shift=TRUE,
             show.marker.names=FALSE, alternate.chrid=FALSE, ...)
{
    dots <- list(...)
    if("main" %in% names(dots)) {
        themain <- dots$main
        usemaindefault <- FALSE
    }
    else usemaindefault <- TRUE

    if("xlim" %in% names(dots)) {
        xlim <- dots$xlim
        usexlimdefault <- FALSE
    }
    else usexlimdefault <- TRUE

    if("ylim" %in% names(dots)) {
        ylim <- dots$ylim
        useylimdefault <- FALSE
    }
    else useylimdefault <- TRUE

    if("xlab" %in% names(dots))
        xlab <- dots$xlab
    else {
        if(horizontal)
            xlab <- "Location (cM)"
        else
            xlab <- "Chromosome"
    }

    if("ylab" %in% names(dots))
        ylab <- dots$ylab
    else {
        if(horizontal)
            ylab <- "Chromosome"
        else
            ylab <- "Location (cM)"
    }

    map <- x
    # figure out if the input is a cross (containing a map)
    #    or is the map itself
    if(inherits(map, "cross"))
        map <- pull.map(map)
    if(!missing(map2) && inherits(map2, "cross"))
        map2 <- pull.map(map2)

    if(!inherits(map, "map")  || (!missing(map2) && !inherits(map2, "map")))
        warning("Input should have class \"cross\" or \"map\".")

    if(!missing(map2) && is.matrix(map[[1]]) != is.matrix(map2[[1]]))
        stop("Maps must be both sex-specific or neither sex-specific.")

    if(!missing(chr)) {
        map <- map[matchchr(chr, names(map))]
        if(!missing(map2)) map2 <- map2[matchchr(chr, names(map2))]
    }

    sex.sp <- FALSE

    if(is.matrix(map[[1]])) { # sex-specific map
        one.map <- FALSE
        sex.sp <- TRUE
        if(!missing(map2)) {
            if(is.logical(map2)) {
                horizontal <- map2
                map2 <- lapply(map,function(a) a[2,])
                map <- lapply(map,function(a) a[1,])
            }
            else {
                Map1 <- lapply(map,function(a) a[1,,drop=TRUE])
                Map2 <- lapply(map,function(a) a[2,,drop=TRUE])
                Map3 <- lapply(map2,function(a) a[1,,drop=TRUE])
                Map4 <- lapply(map2,function(a) a[2,,drop=TRUE])
                old.mfrow <- par("mfrow")
                on.exit(par(mfrow=old.mfrow))

                par(mfrow=c(2,1))
                class(Map1) <- class(Map2) <- class(Map3) <- class(Map4) <- "map"
                plotMap(Map1,Map3,horizontal=horizontal,shift=shift,
                        show.marker.names=show.marker.names,alternate.chrid=alternate.chrid)
                plotMap(Map2,Map4,horizontal=horizontal,shift=shift,
                        show.marker.names=show.marker.names,alternate.chrid=alternate.chrid)
                return(invisible(NULL))
            }
        }
        else {
            map2 <- lapply(map,function(a) a[2,])
            map <- lapply(map,function(a) a[1,])
        }
    }
    else { # single map
        # determine whether a second map was given
        if(!missing(map2))
            one.map <- FALSE
        else one.map <- TRUE
    }

    if(one.map) {

        n.chr <- length(map)
        if(!show.marker.names) {  # locations of chromosomes
            chrpos <- 1:n.chr
            thelim <- range(chrpos)+c(-0.5, 0.5)
        }
        else {
            chrpos <- seq(1, n.chr*2, by=2)
            thelim <- range(chrpos)+c(-0.35, 2.35)
        }

        if(shift) map <- lapply(map, function(a) a-a[1])
        maxlen <- max(unlist(lapply(map,max)))

        if(horizontal) {
            old.xpd <- par("xpd")
            old.las <- par("las")
            par(xpd=TRUE, las=1)
            on.exit(par(xpd=old.xpd,las=old.las))

            if(usexlimdefault) xlim <- c(0,maxlen)
            if(useylimdefault) ylim <- rev(thelim)
            plot(0,0,type="n",xlim=xlim, ylim=ylim,yaxs="i",
                 xlab=xlab, ylab=ylab, yaxt="n")
            a <- par("usr")

            for(i in 1:n.chr) {
                segments(min(map[[i]]), chrpos[i], max(map[[i]]), chrpos[i])
                segments(map[[i]], chrpos[i]-0.25, map[[i]], chrpos[i]+0.25)

                if(show.marker.names)
                    text(map[[i]], chrpos[i]+0.35, names(map[[i]]), srt=90, adj=c(1,0.5))
            }

            # add chromosome labels
            if(!alternate.chrid || length(chrpos) < 2) {
                for(i in seq(along=chrpos))
                    axis(side=2, at=chrpos[i], labels=names(map)[i])
            }
            else {
                odd <- seq(1, length(chrpos), by=2)
                even <- seq(2, length(chrpos), by=2)
                for(i in odd) {
                    axis(side=2, at=chrpos[i], labels="")
                    axis(side=2, at=chrpos[i], labels=names(map)[i], line=-0.4, tick=FALSE)
                }
                for(i in even) {
                    axis(side=2, at=chrpos[i], labels="")
                    axis(side=2, at=chrpos[i], labels=names(map)[i], line=+0.4, tick=FALSE)
                }
            }
        }
        else {
            old.xpd <- par("xpd")
            old.las <- par("las")
            par(xpd=TRUE,las=1)
            on.exit(par(xpd=old.xpd,las=old.las))

            if(usexlimdefault) xlim <- thelim
            if(useylimdefault) ylim <- c(maxlen, 0)
            plot(0,0,type="n",ylim=ylim,xlim=xlim,xaxs="i",
                 xlab=xlab, ylab=ylab, xaxt="n")

            a <- par("usr")

            for(i in 1:n.chr) {
                segments(chrpos[i], min(map[[i]]), chrpos[i], max(map[[i]]))
                segments(chrpos[i]-0.25, map[[i]], chrpos[i]+0.25, map[[i]])

                if(show.marker.names)
                    text(chrpos[i]+0.35, map[[i]], names(map[[i]]), adj=c(0,0.5))

            }
            # add chromosome labels
            if(!alternate.chrid || length(chrpos) < 2) {
                for(i in seq(along=chrpos))
                    axis(side=1, at=chrpos[i], labels=names(map)[i])
            }
            else {
                odd <- seq(1, length(chrpos), by=2)
                even <- seq(2, length(chrpos), by=2)
                for(i in odd) {
                    axis(side=1, at=chrpos[i], labels="")
                    axis(side=1, at=chrpos[i], labels=names(map)[i], line=-0.4, tick=FALSE)
                }
                for(i in even) {
                    axis(side=1, at=chrpos[i], labels="")
                    axis(side=1, at=chrpos[i], labels=names(map)[i], line=+0.4, tick=FALSE)
                }
            }
        }
        if(usemaindefault)
            title(main="Genetic map")
        else if(themain != "")
            title(main=themain)

    }
    else {
        map1 <- map

        # check that maps conform
        if(is.matrix(map2[[1]]))
            stop("Second map appears to be a sex-specific map.")
        if(length(map1) != length(map2))
            stop("Maps have different numbers of chromosomes.")
        if(any(names(map1) != names(map2))) {
            cat("Map1: ", names(map1), "\n")
            cat("Map2: ", names(map2), "\n")
            stop("Maps have different chromosome names.")
        }

        if(shift) {
            map1 <- lapply(map1,function(a) a-a[1])
            map2 <- lapply(map2,function(a) a-a[1])
        }

        n.mar1 <- sapply(map1, length)
        n.mar2 <- sapply(map2, length)
        markernames1 <- lapply(map1, names)
        markernames2 <- lapply(map2, names)
        if(any(n.mar1 != n.mar2)) {
            if(show.marker.names) {
                warning("Can't show marker names because of different numbers of markers.")
                show.marker.names <- FALSE
            }
        }
        else if(any(unlist(markernames1) != unlist(markernames2))) {
            if(show.marker.names) {
                warning("Can't show marker names because markers in different orders.")
                show.marker.names <- FALSE
            }
        }

        n.chr <- length(map1)
        maxloc <- max(c(unlist(lapply(map1,max)),unlist(lapply(map2,max))))

        if(!show.marker.names) {  # locations of chromosomes
            chrpos <- 1:n.chr
            thelim <- range(chrpos)+c(-0.5, 0.5)
        }
        else {
            chrpos <- seq(1, n.chr*2, by=2)
            thelim <- range(chrpos)+c(-0.4, 2.4)
        }

        if(!horizontal) {
            old.xpd <- par("xpd")
            old.las <- par("las")
            par(xpd=TRUE,las=1)
            on.exit(par(xpd=old.xpd,las=old.las))

            if(usexlimdefault) xlim <- thelim
            if(useylimdefault) ylim <- c(maxloc, 0)
            plot(0,0,type="n",ylim=ylim,xlim=xlim, xaxs="i",
                 xlab=xlab, ylab=ylab, xaxt="n")

            a <- par("usr")

            for(i in 1:n.chr) {
                if(max(map2[[i]]) < max(map1[[i]]))
                    map2[[i]] <- map2[[i]] + (max(map1[[i]])-max(map2[[i]]))/2

                else
                    map1[[i]] <- map1[[i]] + (max(map2[[i]])-max(map1[[i]]))/2

                segments(chrpos[i]-0.3, min(map1[[i]]), chrpos[i]-0.3, max(map1[[i]]))
                segments(chrpos[i]+0.3, min(map2[[i]]), chrpos[i]+0.3, max(map2[[i]]))

                # lines between markers
                wh <- match(markernames1[[i]], markernames2[[i]])
                for(j in which(!is.na(wh)))
                    segments(chrpos[i]-0.3, map1[[i]][j], chrpos[i]+0.3, map2[[i]][wh[j]])
                if(any(is.na(wh)))
                    segments(chrpos[i]-0.4, map1[[i]][is.na(wh)], chrpos[i]-0.2, map1[[i]][is.na(wh)])
                wh <- match(markernames2[[i]], markernames1[[i]])
                if(any(is.na(wh)))
                    segments(chrpos[i]+0.4, map2[[i]][is.na(wh)], chrpos[i]+0.2, map2[[i]][is.na(wh)])

                if(show.marker.names)
                    text(chrpos[i]+0.35, map2[[i]], names(map2[[i]]), adj=c(0,0.5))

            }
            # add chromosome labels
            if(!alternate.chrid || length(chrpos) < 2) {
                for(i in seq(along=chrpos))
                    axis(side=1, at=chrpos[i], labels=names(map1)[i])
            }
            else {
                odd <- seq(1, length(chrpos), by=2)
                even <- seq(2, length(chrpos), by=2)
                for(i in odd) {
                    axis(side=1, at=chrpos[i], labels="")
                    axis(side=1, at=chrpos[i], labels=names(map1)[i], line=-0.4, tick=FALSE)
                }
                for(i in even) {
                    axis(side=1, at=chrpos[i], labels="")
                    axis(side=1, at=chrpos[i], labels=names(map1)[i], line=+0.4, tick=FALSE)
                }
            }
        }
        else {
            old.xpd <- par("xpd")
            old.las <- par("las")
            par(xpd=TRUE,las=1)
            on.exit(par(xpd=old.xpd,las=old.las))

            if(usexlimdefault) xlim <- c(0,maxloc)
            if(useylimdefault) ylim <- rev(thelim)
            plot(0,0,type="n",xlim=xlim,ylim=ylim,
                 xlab=xlab, ylab=ylab, yaxt="n", yaxs="i")

            a <- par("usr")

            for(i in 1:n.chr) {

                if(max(map2[[i]]) < max(map1[[i]]))
                    map2[[i]] <- map2[[i]] + (max(map1[[i]])-max(map2[[i]]))/2
                else
                    map1[[i]] <- map1[[i]] + (max(map2[[i]])-max(map1[[i]]))/2

                segments(min(map1[[i]]), chrpos[i]-0.3, max(map1[[i]]), chrpos[[i]]-0.3)
                segments(min(map2[[i]]), chrpos[i]+0.3, max(map2[[i]]), chrpos[[i]]+0.3)

                # lines between markers
                wh <- match(markernames1[[i]], markernames2[[i]])
                for(j in which(!is.na(wh)))
                    segments(map1[[i]][j], chrpos[i]-0.3, map2[[i]][wh[j]], chrpos[i]+0.3)
                if(any(is.na(wh)))
                    segments(map1[[i]][is.na(wh)], chrpos[i]-0.4, map1[[i]][is.na(wh)], chrpos[i]-0.2)
                wh <- match(markernames2[[i]], markernames1[[i]])
                if(any(is.na(wh)))
                    segments(map2[[i]][is.na(wh)], chrpos[i]+0.4, map2[[i]][is.na(wh)], chrpos[i]+0.2)

                if(show.marker.names)
                    text(map2[[i]], chrpos[i]+0.35, names(map2[[i]]), srt=90, adj=c(1,0.5))

            }
            # add chromosome labels
            if(!alternate.chrid || length(chrpos) < 2) {
                for(i in seq(along=chrpos))
                    axis(side=2, at=chrpos[i], labels=names(map1)[i])
            }
            else {
                odd <- seq(1, length(chrpos), by=2)
                even <- seq(2, length(chrpos), by=2)
                for(i in odd) {
                    axis(side=2, at=chrpos[i], labels="")
                    axis(side=2, at=chrpos[i], labels=names(map1)[i], line=-0.4, tick=FALSE)
                }
                for(i in even) {
                    axis(side=2, at=chrpos[i], labels="")
                    axis(side=2, at=chrpos[i], labels=names(map1)[i], line=+0.4, tick=FALSE)
                }
            }

        }
        if(usemaindefault) {
            if(!sex.sp) title(main="Comparison of genetic maps")
            else title(main="Genetic map")
        }
        else if(themain != "")
            title(main=themain)
    }
    invisible()
}


plot.cross <-
    function (x, auto.layout = TRUE, pheno.col,
              alternate.chrid=TRUE, ...)
{
    # look to see whether this should really be shipped to plotMap
    if(inherits(auto.layout, "map") &&
       (inherits(x, "map") || inherits(x, "cross"))) {
        plotMap(x, auto.layout, alternate.chrid=alternate.chrid, ...)
        return(invisible())
    }

    # make sure this is a cross
    if(!inherits(x, "cross"))
        stop("Input should have class \"cross\".")

    old.yaxt <- par("yaxt")
    old.mfrow <- par("mfrow")
    on.exit(par(yaxt = old.yaxt, mfrow = old.mfrow))

    n.phe <- nphe(x)
    if(missing(pheno.col)) pheno.col <- 1:n.phe
    if(is.character(pheno.col)) {
        temp <- match(pheno.col, names(x$pheno))
        if(any(is.na(temp)))
            warning("Some phenotypes not found:",
                    paste(pheno.col[is.na(temp)], collapse=" "))
        pheno.col <- temp[!is.na(temp)]
    }
    else pheno.col <- (1:nphe(x))[pheno.col]

    n.plot = length(pheno.col) + 2

    # automatically choose row/column structure for the plots
    if(auto.layout) {
        nr <- ceiling(sqrt(n.plot))
        nc <- ceiling((n.plot)/nr)
        par(mfrow = c(nr, nc))
    }

    plotMissing(x,alternate.chrid=alternate.chrid)
    plotMap(x,alternate.chrid=alternate.chrid)

    for(i in pheno.col) plotPheno(x, pheno.col=i)

    invisible()
}


##################################################r####################
#
# plotGeno: Plot genotypes for a specified chromosome, with likely
#           genotyping errors indicated.
#
######################################################################

plotGeno <-
    function(x, chr, ind, include.xo=TRUE, horizontal=TRUE,
             cutoff=4, min.sep=2, cex=1.2, ...)
{
    cross <- x
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(missing(chr)) chr <- names(cross$geno)[1]
    cross <- subset(cross,chr=chr)
    if(nchr(cross) > 1)
        cross <- subset(cross,chr=names(cross$geno)[1])

    if(!missing(ind)) {
        if(is.null(getid(cross))) cross$pheno$id <- 1:nind(cross)
        if(!is.logical(ind)) ind <- unique(ind)
        cross <- subset(cross, ind=ind)
    }
    id <- getid(cross)
    if(is.null(id)) id <- 1:nind(cross)
    use.id <- TRUE

    type <- crosstype(cross)

    old.las <- par("las")
    on.exit(par(las=old.las))
    par(las=1)

    if(!("errorlod" %in% names(cross$geno[[1]]))) {
        warning("First running calc.errorlod.")
        cross <- calc.errorlod(cross,error.prob=0.01)
    }

    # indicators for apparent errors
    errors <- matrix(0,ncol=ncol(cross$geno[[1]]$data),
                     nrow=nrow(cross$geno[[1]]$data))
    dimnames(errors) <- dimnames(cross$geno[[1]]$data)

    top <- top.errorlod(cross,names(cross$geno)[1],cutoff,FALSE)
    if(length(top) > 0)
        for(i in 1:nrow(top))
            errors[match(top[i,2],id),as.character(top[i,3])] <- 1

    # map, data, errors
    map <- cross$geno[[1]]$map
    if(is.matrix(map)) map <- map[1,] # if sex-specific map
    L <- diff(range(map))
    min.d <- L*min.sep/100
    d <- diff(map)
    d[d < min.d] <- min.d
    map <- cumsum(c(0,d))
    cross$geno[[1]]$map <- map

    n.ind <- nrow(errors)

    color <- c("white","gray60","black","green","orange","red")

    # revise X chr data for backcross/intercross
    data <- cross$geno[[1]]$data
    chr_type <- chrtype(cross$geno[[1]])
    if(chr_type=="X" && (type=="f2" || type=="bc"))
        data <- reviseXdata(type, sexpgm=getsex(cross), geno=data, cross.attr=attributes(cross), force=TRUE)

    if(include.xo) {
        if(type != "4way") { # find crossover locations
            xoloc <- locateXO(cross)
            xoloc <- data.frame(ind=rep(1:length(xoloc),sapply(xoloc,length)),
                                loc=unlist(xoloc), stringsAsFactors=TRUE)
        }
        else { # 4-way cross
            mcross <- dcross <- cross
            class(mcross) <- class(dcross) <- c("bc", "cross")
            mcross$geno[[1]]$data[!is.na(data) & data==1 | data==3 | data==5] <- 1
            mcross$geno[[1]]$data[!is.na(data) & data==2 | data==4 | data==6] <- 2
            mcross$geno[[1]]$data[!is.na(data) & data==7 | data==8 | data==9 | data==10] <- NA

            dcross$geno[[1]]$data[!is.na(data) & data==1 | data==2 | data==7] <- 1
            dcross$geno[[1]]$data[!is.na(data) & data==3 | data==4 | data==8] <- 2
            dcross$geno[[1]]$data[!is.na(data) & data==5 | data==6 | data==9 | data==10] <- NA

            mxoloc <- locateXO(mcross)
            mxoloc <- data.frame(ind=rep(1:length(mxoloc),sapply(mxoloc,length)),
                                 loc=unlist(mxoloc), stringsAsFactors=TRUE)

            dxoloc <- locateXO(dcross)
            dxoloc <- data.frame(ind=rep(1:length(dxoloc),sapply(dxoloc,length)),
                                 loc=unlist(dxoloc), stringsAsFactors=TRUE)
        }
    }

    # check for 'main' in the ...
    args <- list(...)
    if("main" %in% names(args))
        themain <- args$main
    else
        themain <- paste("Chromosome",names(cross$geno)[1])

    # check for 'xlim' and 'ylim'
    if("xlim" %in% names(args)) thexlim <- args$xlim
    else thexlim <- NULL
    if("ylim" %in% names(args)) theylim <- args$ylim
    else theylim <- NULL

    if(type=="4way") {
        jit <- 0.15
        mdata <- data
        ddata <- data

        # mom's allele
        mdata[!is.na(data) & (data==1 | data==3 | data==5)] <- 1
        mdata[!is.na(data) & (data==2 | data==4 | data==6)] <- 2
        mdata[!is.na(data) & (data==7 | data==8)] <- NA

        # dad's allele
        ddata[!is.na(data) & (data==1 | data==2 | data==7)] <- 1
        ddata[!is.na(data) & (data==3 | data==4 | data==8)] <- 2
        ddata[!is.na(data) & (data==5 | data==6)] <- NA

        if(horizontal) {
            if(is.null(thexlim)) thexlim <- c(0, max(map))
            if(is.null(theylim)) theylim <- c(n.ind+1, 0)
            plot(0,0,type="n",xlab="Location (cM)",ylab="Individual",
                 main=themain,
                 ylim=theylim,xlim=thexlim, yaxt="n", yaxs="i")
            segments(0, 1:n.ind-jit, max(map), 1:n.ind-jit)
            segments(0, 1:n.ind+jit, max(map), 1:n.ind+jit)

            if(use.id) axis(side=2, at=1:n.ind, labels=id)
            else axis(side=2, at=1:n.ind)

            # A alleles
            tind <- rep(1:n.ind,length(map));tind[is.na(mdata)] <- NA
            ind <- tind; ind[!is.na(mdata) & mdata!=1] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind-jit,pch=21,col="black", bg=color[1],cex=cex)

            # B alleles
            tind <- rep(1:n.ind,length(map));tind[is.na(mdata)] <- NA
            ind <- tind; ind[!is.na(mdata) & mdata!=2] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind-jit,pch=21,col="black", bg=color[3],cex=cex)

            # 9/10 genotypes
            tind <- rep(1:n.ind,length(map));tind[is.na(mdata)] <- NA
            ind <- tind; ind[!is.na(mdata) & mdata!=9] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind-jit,pch=21,col="black", bg=color[4],cex=cex)

            tind <- rep(1:n.ind,length(map));tind[is.na(mdata)] <- NA
            ind <- tind; ind[!is.na(mdata) & mdata!=10] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind-jit,pch=21,col="black", bg=color[5],cex=cex)

            # C alleles
            tind <- rep(1:n.ind,length(map));tind[is.na(ddata)] <- NA
            ind <- tind; ind[!is.na(ddata) & ddata!=1] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind+jit,pch=21,col="black", bg=color[1],cex=cex)

            # D alleles
            tind <- rep(1:n.ind,length(map));tind[is.na(ddata)] <- NA
            ind <- tind; ind[!is.na(ddata) & ddata!=2] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind+jit,pch=21,col="black", bg=color[3],cex=cex)

            # 9/10 genotypes
            tind <- rep(1:n.ind,length(map));tind[is.na(ddata)] <- NA
            ind <- tind; ind[!is.na(ddata) & ddata!=9] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind+jit,pch=21,col="black", bg=color[4],cex=cex)

            tind <- rep(1:n.ind,length(map));tind[is.na(ddata)] <- NA
            ind <- tind; ind[!is.na(ddata) & ddata!=10] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind+jit,pch=21,col="black", bg=color[5],cex=cex)

            # plot map
            u <- par("usr")
            segments(map,u[3],map,u[3]-1/2)
            segments(map,u[4],map,u[4]+1/2)

            if(any(errors != 0)) {
                ind <- rep(1:n.ind,length(map));ind[errors!=1]<-NA
                points(x,ind-jit,pch=0,col=color[6],cex=cex+0.4,lwd=2)
                points(x,ind+jit,pch=0,col=color[6],cex=cex+0.4,lwd=2)
            }

            if(include.xo) {
                points(mxoloc$loc,mxoloc$ind-jit,pch=4,col="blue",lwd=2)
                points(dxoloc$loc,dxoloc$ind+jit,pch=4,col="blue",lwd=2)
            }
        }
        else {
            if(is.null(theylim)) theylim <- c(max(map), 0)
            if(is.null(thexlim)) thexlim <- c(0, n.ind+1)

            plot(0,0,type="n",ylab="Location (cM)",xlab="Individual",
                 main=themain,
                 xlim=thexlim,ylim=theylim, xaxt="n", xaxs="i")

            segments(1:n.ind-jit, 0, 1:n.ind-jit, max(map))
            segments(1:n.ind+jit, 0, 1:n.ind+jit, max(map))


            if(use.id) axis(side=1, at=1:n.ind, labels=id)
            else axis(side=1, at=1:n.ind)

            # A alleles
            tind <- rep(1:n.ind,length(map));tind[is.na(mdata)] <- NA
            ind <- tind; ind[!is.na(mdata) & mdata!=1] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind-jit,y,pch=21,col="black",bg=color[1],cex=cex)

            # B alleles
            tind <- rep(1:n.ind,length(map));tind[is.na(mdata)] <- NA
            ind <- tind; ind[!is.na(mdata) & mdata!=2] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind-jit,y,pch=21,col="black",bg=color[3],cex=cex)

            # 9/10 genotypes
            tind <- rep(1:n.ind,length(map));tind[is.na(mdata)] <- NA
            ind <- tind; ind[!is.na(mdata) & mdata!=9] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind-jit,y,pch=21,col="black", bg=color[4],cex=cex)

            tind <- rep(1:n.ind,length(map));tind[is.na(mdata)] <- NA
            ind <- tind; ind[!is.na(mdata) & mdata!=10] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind-jit,y,pch=21,col="black", bg=color[5],cex=cex)

            # C alleles
            tind <- rep(1:n.ind,length(map));tind[is.na(ddata)] <- NA
            ind <- tind; ind[!is.na(ddata) & ddata!=1] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind+jit,y,pch=21,col="black", bg=color[1],cex=cex)

            # D alleles
            tind <- rep(1:n.ind,length(map));tind[is.na(ddata)] <- NA
            ind <- tind; ind[!is.na(ddata) & ddata!=2] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind+jit,y,pch=21,col="black", bg=color[3],cex=cex)

            # 9/10 genotypes
            tind <- rep(1:n.ind,length(map));tind[is.na(ddata)] <- NA
            ind <- tind; ind[!is.na(ddata) & ddata!=9] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind+jit,y,pch=21,col="black", bg=color[4],cex=cex)

            tind <- rep(1:n.ind,length(map));tind[is.na(ddata)] <- NA
            ind <- tind; ind[!is.na(ddata) & ddata!=10] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind+jit,y,pch=21,col="black", bg=color[5],cex=cex)

            # plot map
            u <- par("usr")
            segments(u[1],map,(u[1]+1)/2,map)
            segments(u[2],map,(n.ind+u[2])/2,map)

            if(any(errors != 0)) {
                ind <- rep(1:n.ind,length(map));ind[errors!=1]<-NA
                points(ind-jit,y,pch=0,col=color[6],cex=cex+0.4,lwd=2)
                points(ind+jit,y,pch=0,col=color[6],cex=cex+0.4,lwd=2)
            }

            if(include.xo) {
                points(mxoloc$ind-jit,mxoloc$loc,pch=4,col="blue",lwd=2)
                points(dxoloc$ind+jit,dxoloc$loc,pch=4,col="blue",lwd=2)
            }

        }


    }
    else {

        if(horizontal) {
            if(is.null(thexlim)) thexlim <- c(0, max(map))
            if(is.null(theylim)) theylim <- c(n.ind+0.5,0.5)

            plot(0,0,type="n",xlab="Location (cM)",ylab="Individual",
                 main=themain,
                 ylim=theylim,xlim=thexlim, yaxt="n")
            segments(0, 1:n.ind, max(map), 1:n.ind)
            if(use.id) axis(side=2, at=1:n.ind, labels=id)
            else axis(side=2)

            # AA genotypes
            tind <- rep(1:n.ind,length(map));tind[is.na(data)] <- NA
            ind <- tind; ind[!is.na(data) & data!=1] <- NA
            x <- rep(map,rep(n.ind,length(map)))
            points(x,ind,pch=21,col="black", bg=color[1],cex=cex)

            # AB genotypes
            ind <- tind; ind[!is.na(data) & data!=2] <- NA
            if(type=="f2" || (type=="bc" && chr_type=="X"))
                points(x,ind,pch=21,col="black", bg=color[2],cex=cex)
            else points(x,ind,pch=21,col="black", bg=color[3],cex=cex)

            if(type=="f2" || (type=="bc" && chr_type=="X")) {
                # BB genotypes
                ind <- tind; ind[!is.na(data) & data!=3] <- NA
                points(x,ind,pch=21,col="black", bg=color[3],cex=cex)
            }

            if(type=="f2") {
                # not BB (D in mapmaker/qtl) genotypes
                ind <- tind; ind[!is.na(data) & data!=4] <- NA
                points(x,ind,pch=21,col="black", bg=color[4],cex=cex)

                # not AA (C in mapmaker/qtl) genotypes
                ind <- tind; ind[!is.na(data) & data!=5] <- NA
                points(x,ind,pch=21,col="black", bg=color[5],cex=cex)
            }

            # plot map
            u <- par("usr")
            segments(map,u[3],map,u[3]-1/2)
            segments(map,u[4],map,u[4]+1/2)

            if(any(errors != 0)) {
                ind <- rep(1:n.ind,length(map));ind[errors!=1]<-NA
                points(x,ind,pch=0,col=color[6],cex=cex+0.4,lwd=2)
            }

            if(include.xo) points(xoloc$loc,xoloc$ind,pch=4,col="blue",lwd=2)
        }
        else {
            if(is.null(theylim)) theylim <- c(max(map), 0)
            if(is.null(thexlim)) thexlim <- c(0.5,n.ind+0.5)
            plot(0,0,type="n",ylab="Location (cM)",xlab="Individual",
                 main=themain,
                 xlim=thexlim,ylim=theylim, xaxt="n")
            segments(1:n.ind,0,1:n.ind,max(map))
            if(use.id) axis(side=1, at=1:n.ind, labels=id)
            else axis(side=1)

            # AA genotypes
            tind <- rep(1:n.ind,length(map));tind[is.na(data)] <- NA
            ind <- tind; ind[!is.na(data) & data!=1] <- NA
            y <- rep(map,rep(n.ind,length(map)))
            points(ind,y,pch=21,col="black", bg="white",cex=cex)

            # AB genotypes
            ind <- tind; ind[!is.na(data) & data!=2] <- NA
            if(type=="f2" || (type=="bc" && chr_type=="X"))
                points(ind,y,pch=21,col="black", bg=color[2],cex=cex)
            else points(ind,y,pch=21,col="black", bg=color[3],cex=cex)

            if(type=="f2" || (type=="bc" && chr_type=="X")) {
                # BB genotypes
                ind <- tind; ind[!is.na(data) & data!=3] <- NA
                points(ind,y,pch=21,col="black", bg=color[3],cex=cex)
            }

            if(type=="f2") {
                # not BB genotypes
                ind <- tind; ind[!is.na(data) & data!=4] <- NA
                points(ind,y,pch=21,col="black", bg=color[4],cex=cex)

                # not AA genotypes
                ind <- tind; ind[!is.na(data) & data!=5] <- NA
                points(ind,y,pch=21,col="black", bg=color[5],cex=cex)
            }

            # plot map
            u <- par("usr")
            segments(u[1],map,(u[1]+1)/2,map)
            segments(u[2],map,(n.ind+u[2])/2,map)

            if(any(errors != 0)) {
                ind <- rep(1:n.ind,length(map));ind[errors!=1]<-NA
                points(ind,y,pch=0,col=color[6],cex=cex+0.4,lwd=2)
            }

            if(include.xo) points(xoloc$ind,xoloc$loc,pch=4,col="blue",lwd=2)

        }
    }
    invisible()
}

######################################################################
#
# plotInfo: Plot the proportion of missing information in the
#            genotype data.
#
######################################################################
plotInfo <-
    function(x,chr,method=c("entropy","variance","both"), step=1,
             off.end=0, error.prob=0.001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             alternate.chrid=FALSE, fourwaycross=c("all", "AB", "CD"),
             include.genofreq=FALSE, ...)
{
    cross <- x
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    method <- match(match.arg(method),c("entropy","variance","both"))-1
    map.function <- match.arg(map.function)

    if(!missing(chr)) cross <- subset(cross,chr=chr)

    n.chr <- nchr(cross)
    results <- NULL

    cross <- calc.genoprob(cross, step=step, error.prob=error.prob,
                           off.end=off.end, map.function=map.function)

    gap <- 25

    # 4-way cross: can consider just A/B or just C/D
    fourwaycross <- match.arg(fourwaycross)

    n.ind <- nind(cross)
    if(include.genofreq) theprob <- NULL
    for(i in 1:n.chr) {

        n.gen <- dim(cross$geno[[i]]$prob)[3]
        n.pos <- ncol(cross$geno[[i]]$prob)

        # 4-way cross: can consider just A/B or just C/D
        if(n.gen==4 && (fourwaycross!="all")) {
            att <- attributes(cross$geno[[i]]$prob)
            if(fourwaycross == "AB") {
                cross$geno[[i]]$prob[,,1] <- cross$geno[[i]]$prob[,,1] + cross$geno[[i]]$prob[,,3]
                cross$geno[[i]]$prob[,,2] <- cross$geno[[i]]$prob[,,2] + cross$geno[[i]]$prob[,,4]
            }
            else {
                cross$geno[[i]]$prob[,,1] <- cross$geno[[i]]$prob[,,1] + cross$geno[[i]]$prob[,,2]
                cross$geno[[i]]$prob[,,2] <- cross$geno[[i]]$prob[,,3] + cross$geno[[i]]$prob[,,4]
            }
            cross$geno[[i]]$prob <- cross$geno[[i]]$prob[,,1:2]
            n.gen <- 2

            for(j in seq(along=att))
                if(names(att)[j] != "dim" && names(att)[j] != "dimnames")
                    attr(cross$geno[[i]]$prob, names(att)[j]) <- att[[j]]
        }

        # calculate information (between 0 and 1)
        info <- .C("R_info",
                   as.integer(n.ind),
                   as.integer(n.pos),
                   as.integer(n.gen),
                   as.double(cross$geno[[i]]$prob),
                   info1=as.double(rep(0,n.pos)),
                   info2=as.double(rep(0,n.pos)),
                   as.integer(method),
                   PACKAGE="qtl")

        if(method != 1) { # rescale entropy version
            if(n.gen==3) maxent <- 1.5*log(2)
            else maxent <- log(n.gen)
            info$info1 <- -info$info1/maxent
        }
        if(method != 0) { # rescale variance version
            maxvar <- c(0.25,0.5,1.25)[n.gen-1]
            info$info2 <- info$info2/maxvar
        }

        # reconstruct map
        if("map" %in% names(attributes(cross$geno[[i]]$prob)))
            map <- attr(cross$geno[[i]]$prob,"map")
        else {
            stp <- attr(cross$geno[[i]]$prob, "step")
            oe <- attr(cross$geno[[i]]$prob, "off.end")

            if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
            else stpw <- "fixed"
            map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
        }

        if(is.matrix(map)) map <- map[1,]

        z <- data.frame(chr=rep(names(cross$geno)[i],length(map)),
                        pos=as.numeric(map),
                        "Missing information"=info$info1,
                        "Missing information"=info$info2, stringsAsFactors=TRUE)

        w <- names(map)
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",names(cross$geno)[i],".",w[o],sep="")
        rownames(z) <- w
        results <- rbind(results, z)

        if(include.genofreq) {
            p <- cross$geno[[i]]$prob
            if(chrtype(cross$geno[[i]])=="X")
                p <- reviseXdata(crosstype(cross), expandX="full", sexpgm=getsex(cross), prob=p,
                                 cross.attr=attributes(cross))
            p <- apply(p, 2:3, mean, na.rm=TRUE)
            if(is.null(theprob))
                theprob <- p
            else {
                m1 <- match(colnames(p), colnames(theprob))
                m2 <- match(colnames(theprob), colnames(p))
                if(!any(is.na(m1)) && !any(is.na(m2))) {
                    theprob <- rbind(theprob, p[,colnames(theprob)])
                }
                else { # some differences in column names
                    if(all(!is.na(m1))) { # need to add some new columns to p
                        noldcol <- ncol(p)
                        nnewcol <- sum(is.na(m2))
                        for(i in 1:nnewcol)
                            p <- cbind(p, 0)
                        colnames(p)[-(1:noldcol)] <- colnames(theprob)[is.na(m2)]
                        theprob <- rbind(theprob, p[,colnames(theprob)])
                    }
                    else if(all(!is.na(m2))) { # need to add some new columns to theprob
                        noldcol <- ncol(theprob)
                        nnewcol <- sum(is.na(m1))
                        for(i in 1:nnewcol)
                            theprob <- cbind(theprob, 0)
                        colnames(theprob)[-(1:noldcol)] <- colnames(p)[is.na(m1)]
                        theprob <- rbind(theprob, p[,colnames(theprob)])
                    }
                    else { # need to add some new columns to each
                        noldcol <- ncol(theprob)
                        nnewcol <- sum(is.na(m1))
                        oldcolnam <- colnames(theprob)
                        for(i in 1:nnewcol)
                            theprob <- cbind(theprob, 0)
                        colnames(theprob)[-(1:noldcol)] <- colnames(p)[is.na(m1)]

                        noldcol <- ncol(p)
                        nnewcol <- sum(is.na(m2))
                        for(i in 1:nnewcol)
                            p <- cbind(p, 0)
                        colnames(p)[-(1:noldcol)] <- oldcolnam[is.na(m2)]

                        theprob <- rbind(theprob, p[,colnames(theprob)])
                    }
                }
            }
        } # end of if(include.genofreq)

    }
    colnames(results)[3:4] <- c("misinfo.entropy","misinfo.variance")

    if(method==0) results <- results[,-4]
    if(method==1) results <- results[,-3]

    if(include.genofreq)
        results <- cbind(results, theprob)

    class(results) <- c("scanone","data.frame")

    # check whether main was included as an argument
    args <- list(...)
    if("main" %in% names(args)) hasmain <- TRUE
    else hasmain <- FALSE

    # check whether gap was included as an argument
    if(!("gap" %in% names(args))) {
        if(method==0) {
            if(hasmain)
                plot.scanone(results,ylim=c(0,1),gap=gap,
                             alternate.chrid=alternate.chrid,...)
            else
                plot.scanone(results,ylim=c(0,1),gap=gap,
                             main="Missing information",
                             alternate.chrid=alternate.chrid,...)
        }
        else if(method==1) {
            if(hasmain)
                plot.scanone(results,ylim=c(0,1),gap=gap,
                             alternate.chrid=alternate.chrid,...)
            else
                plot.scanone(results,ylim=c(0,1),gap=gap,
                             main="Missing information",
                             alternate.chrid=alternate.chrid,...)
        }
        else if(method==2) {
            if(hasmain)
                plot.scanone(results,results,lodcolumn=1:2,ylim=c(0,1),gap=gap,
                             alternate.chrid=alternate.chrid,...)
            else
                plot.scanone(results,results,lodcolumn=1:2,ylim=c(0,1),gap=gap,
                             main="Missing information",
                             alternate.chrid=alternate.chrid,...)
        }
    }
    else { # gap was included in ...
        if(method==0) {
            if(hasmain)
                plot.scanone(results,ylim=c(0,1),
                             alternate.chrid=alternate.chrid,...)
            else
                plot.scanone(results,ylim=c(0,1),
                             main="Missing information",
                             alternate.chrid=alternate.chrid,...)
        }
        else if(method==1) {
            if(hasmain)
                plot.scanone(results,ylim=c(0,1),
                             alternate.chrid=alternate.chrid,...)
            else
                plot.scanone(results,ylim=c(0,1),
                             main="Missing information",
                             alternate.chrid=alternate.chrid,...)
        }
        else if(method==2) {
            if(hasmain)
                plot.scanone(results,results,lodcolumn=1:2,ylim=c(0,1),
                             alternate.chrid=alternate.chrid,...)
            else
                plot.scanone(results,results,lodcolumn=1:2,ylim=c(0,1),
                             main="Missing information",
                             alternate.chrid=alternate.chrid,...)
        }
    }

    invisible(results)
}


# plot phenotypes against one or more markers
plotPXG <-
    function(x, marker, pheno.col = 1, jitter = 1, infer = TRUE,
             pch, ylab, main, col, ...)
{
    cross <- x
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")
    type <- crosstype(cross)

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("plotPXG can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 | pheno.col > nphe(cross))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    if(!is.numeric(cross$pheno[,pheno.col]))
        stop("phenotype \"", colnames(cross$pheno)[pheno.col], "\" is not numeric.")

    if(missing(pch)) pch <- par("pch")
    if(missing(ylab)) ylab <-  colnames(cross$pheno)[pheno.col]

    oldlas <- par("las")
    on.exit(par(las = oldlas))
    par(las = 1)

    # find chromosomes containing the markers
    o <- sapply(cross$geno, function(a, b) b %in% colnames(a$data),
                marker)
    if(length(marker)==1) o <- matrix(o,nrow=1)
    if(!all(apply(o,1,any))) {
        oo <- apply(o,1,any)
        stop("Marker ", marker[!oo], " not found")
    }
    n.mark <- length(marker)
    o <- apply(o, 1, which)
    chr <- names(cross$geno)[o]
    uchr <- unique(chr)

    cross <- subset(cross, chr=uchr)
    map <- pull.map(cross)
    pos <- NULL
    for(i in seq(length(chr))) pos[i] <- map[[chr[i]]][marker[i]]
    chr_type <- sapply(cross$geno, chrtype)
    names(chr_type) <- names(cross$geno)
    chr_type <- chr_type[chr]

    # if X chromosome and backcross or intercross, get sex/direction data
    if(any(chr_type == "X") && (type == "bc" || type == "f2"))
        sexpgm <- getsex(cross)
    else sexpgm <- NULL

    # number of possible genotypes
    gen.names <- list()
    for(i in seq(length(chr)))
        gen.names[[i]] <- getgenonames(type, chr_type[i], "full", sexpgm, attributes(cross))
    n.gen <- sapply(gen.names, length)

    jitter <- jitter/10
    if(any(n.gen == 2)) jitter <- jitter * 0.75

    # function to determine whether genotype is fully known
    tempf <-
        function(x, type)
        {
            tmp <- is.na(x)
            if(type=="f2") tmp[!is.na(x) & x>3] <- TRUE
            if(type=="4way") tmp[!is.na(x) & x>4] <- TRUE
            tmp
        }

    # if infer=TRUE, fill in genotype data by a single imputation
    if(infer) {
        which.missing <- tempf(cross$geno[[chr[1]]]$data[, marker[1]],type)
        if(n.mark > 1)
            for(i in 2:n.mark)
                which.missing <- which.missing | tempf(cross$geno[[chr[i]]]$data[,marker[i]],type)
        which.missing <- as.numeric(which.missing)

        cross <- fill.geno(cross, method = "imp")
    }
    else which.missing <- rep(1,nind(cross))

    # data to plot
    x <- cross$geno[[chr[1]]]$data[, marker[1]]
    if(n.mark > 1)
        for(i in 2:n.mark)
            x <- cbind(x, cross$geno[[chr[i]]]$data[, marker[i]])
    else x <- as.matrix(x)
    y <- cross$pheno[, pheno.col]

    if(!infer) { # replace partially informative genotypes with NAs
        if(type == "f2") x[x > 3] <- NA
        if(type == "4way") x[x > 4] <- NA
        if(sum(!is.na(x)) == 0)
            stop("Can't use infer=FALSE as there are no fully informative genotypes")
    }

    # in case of X chromosome, recode some genotypes
    if(any(chr_type == "X") && (type == "bc" || type == "f2")) {
        ix = seq(n.mark)[chr_type == "X"]
        for(i in ix)
            x[, i] <- as.numeric(reviseXdata(type, "full", sexpgm,
                                             geno = as.matrix(x[, i]),
                                             cross.attr=attributes(cross)))
    }

    # save all of the data, returned invisibly
    data <- as.data.frame(x, stringsAsFactors=TRUE)
    names(data) <- marker
    for(i in marker) data[[i]] <- ordered(data[[i]])
    data$pheno <- y
    data$inferred <- which.missing

    # re-code the multi-marker genotypes
    if(n.mark > 1) {
        for(i in 2:n.mark)
            x[, 1] <- n.gen[i] * (x[, 1] - 1) + x[, i]
    }
    x <- x[, 1]

    observed <- sort(unique(x))

    # amount of jitter
    u <- runif(nind(cross), -jitter, jitter)
    r <- (1 - 2 * jitter)/2

    # genotype names
    if(n.mark == 1)
        gnames <- gen.names[[1]]
    else {
        gnames <- array(gen.names[[n.mark]], c(prod(n.gen), n.mark))
        for(i in (n.mark - 1):1) {
            tmpi <- rep(gen.names[[i]], rep(prod(n.gen[(i + 1):n.mark]),
                                            n.gen[i]))
            if(i > 1)
                tmpi <- rep(tmpi, prod(n.gen[1:(i - 1)]))
            gnames[, i] <- tmpi
        }
        gnames <- apply(gnames, 1, function(x) paste(x, collapse = "\n"))
    }

    # create plot
    plot(x + u, y, xlab = "Genotype", ylab = ylab, type = "n",
         main = "", xlim = c(1 - r + jitter, length(gnames) + r + jitter), xaxt = "n",
         ...)

    # marker names at top
    if(missing(main))
        mtext(paste(marker, collapse = "\n"),
              line=0.5, cex = ifelse(n.mark==1, 1.2, 0.8))
    else
        title(main=main)

    abline(v = 1:prod(n.gen), col = "gray", lty = 3)

    if(length(pch) == 1)
        pch = rep(pch, length(x))
    if(infer) {
        points((x + u)[which.missing == 1], y[which.missing ==
                       1], col = "red", pch = pch[which.missing == 1])
        points((x + u)[which.missing == 0], y[which.missing ==
                       0], pch = pch[which.missing == 0])
    }
    else points(x + u, y, pch = pch)
    sux = sort(unique(x))

    # add confidence intervals
    me <- se <- array(NA, length(gnames))
    me[sux] <- tapply(y, x, mean, na.rm = TRUE)
    se[sux] <- tapply(y, x, function(a) sd(a, na.rm = TRUE)/sqrt(sum(!is.na(a))))
    thecolors <- c("black", "blue", "red", "purple", "green", "orange")
    if(missing(col)) {
        col <- thecolors[1:n.gen[n.mark]]
        if(n.gen[n.mark] == 3)
            col <- c("blue", "purple", "red")
        else if(n.gen[n.mark] == 2)
            col <- c("blue", "red")
    }

    ng <- length(gnames)
    segments(seq(ng) + jitter * 2, me, seq(ng) +
             jitter * 4, me, lwd = 2, col = col)
    segments(seq(ng) + jitter * 3, me - se, seq(ng) +
             jitter * 3, me + se, lwd = 2, col = col)
    segments(seq(ng) + jitter * 2.5, me - se, seq(ng) +
             jitter * 3.5, me - se, lwd = 2, col = col)
    segments(seq(ng) + jitter * 2.5, me + se, seq(ng) +
             jitter * 3.5, me + se, lwd = 2, col = col)

    # add genotypes below
    u <- par("usr")
    cxaxis <- par("cex.axis")

    segments(seq(along=gnames), u[3], seq(along=gnames), u[3] - diff(u[3:4]) *
             0.015, xpd = TRUE)
    axis(side=1, at=seq(along=gnames), labels=gnames,
         cex=ifelse(n.mark==1, cxaxis, cxaxis*0.8),
         tick=FALSE, line = (length(marker)-1)/2)

    invisible(data)
}

plotPheno <-
    function(x, pheno.col=1, ...)
{
    if(!inherits(x, "cross"))
        stop("Input should have class \"cross\".")

    if(LikePheVector(pheno.col, nind(x), nphe(x))) {
        x$pheno <- cbind(pheno.col, x$pheno)
        pheno.col <- 1
    }

    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("Ignoring all but the first element in pheno.col.")
    }
    if(is.character(pheno.col)) {
        num <- find.pheno(x, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 | pheno.col > nphe(x))
        warning("pheno.col should be between 1 and ", nphe(x))

    phe <- x$pheno[,pheno.col]
    u <- length(unique(phe))
    if(u==2 || (u < 10  && nind(x) > 50))
        phe <- as.factor(phe)

    plot_pheno_sub <-
        function(phe, xlab=paste("phe", pheno.col),
                 main=colnames(x$pheno)[pheno.col], col="white",
                 breaks=ceiling(2*sqrt(nind(x))),
                 las=1,
                 ...)
        {
            if(is.factor(phe)) {
                barplot(table(phe), xlab=xlab, main=main, col=col, las=las, ...)
            }
            else {
                phe <- as.numeric(phe)[1:nind(x)]
                hist(phe, breaks = breaks,
                     xlab = xlab, main = main, las=las, ...)
            }
        }

    plot_pheno_sub(phe, ...)
}

# end of plot.R

#####################################################################
#
# mqmcofactors.R
#
# Copyright (c) 2009-2010, Danny Arends
#
# Modified by Karl Broman and Pjotr Prins
#
#
# first written Februari 2009
# last modified April 2010
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: find.markerindex
#           mqmsetcofactors
#           mqmautocofactors
#           scoremissingmarkers
#           calculatedensity
#           mqmplot.cofactors
#           checkdistances
#
#####################################################################

######################################################################
#
# find.markerindex: Extracts the number of the marker when viewing the markers lineair
# mqmcofactors: Prepares a cofactor list to use with mqmscan
# mqmsetcofactors: Prepares a cofactor list to use with mqmscan
#
######################################################################


find.markerindex <- function(cross, name) {
    match(name, markernames(cross))
}

mqmsetcofactors <- function(cross,each = NULL,cofactors=NULL,sexfactors=NULL,verbose=FALSE){
    if(is.null(each) && is.null(cofactors))
        stop("Please set either the each parameter or the cofactors")
    if(missing(cross))
        stop("No cross file. Please supply a valid cross object.")
    individuals <- nind(cross)
    n.chr <- nchr(cross)
    n.mark <- sum(nmar(cross))
    cofactorlist <- rep(0,n.mark)
    if(!is.null(each)  && each > n.mark)
        stop("Not enough markers to place cofactors at this wide an interval.")


    if(verbose) {
        cat("INFO: Found",individuals,"individuals in the cross object.\n")
        cat("INFO: Mamimum amount of cofactors",(individuals-15)," (each =",ceiling(sum(n.mark)/(individuals-15)),") leaves 15 Degrees of Freedom (no Dominance).\n")
        cat("INFO: Mamimum amount of cofactors",(individuals-15)/2," (each =",ceiling(sum(n.mark)/(individuals-15))*2,") leaves 15 Degrees of Freedom (Dominance).\n")
    }

    if(is.null(cofactors)){
        cofactorlist <- rep(c(rep(0,each-1),1),(2*n.mark)/each)
        cofactorlist <- cofactorlist[1:n.mark]
    }else{
        if(max(cofactors) > n.mark)
            stop("Trying to set a non-existent marker as a cofactor.")
        if(min(cofactors) <= 0)
            stop("Trying to set a non-existent marker as a cofactor.")

        cofactorlist[cofactors]=1
        if(!is.null(sexfactors)){
            cofactorlist[sexfactors]=2
        }
    }
    if(sum(cofactorlist) > (individuals-15)){
        warning("Trying to set: ",ceiling(sum(n.mark)/each)," markers as cofactor. This leaves less than 15 Degrees of Freedom.\n")
    }
    cofactorlist
}

scoremissingmarkers <- function(cross){
    genotype <- pull.geno(cross)
    nind <- dim(genotype)[1]
    missing <- NULL
    for(x in 1:dim(genotype)[2]){
        missing <- c(missing,sum(is.na(genotype[,x]))/nind)
    }
    missing
}

calculatedensity <- function(cross,distance=30){
    genotype <- pull.geno(cross)
    densities <- NULL
    for(chr in 1:nchr(cross)){
        map <- pull.map(cross)[[chr]]
        for(x in 1:length(map)){
            densities <- c(densities,sum(map[which(map > map[x]-distance)] < map[x]+distance))
        }
    }
    densities
}

mqmautocofactors <- function(cross, num=50, distance=5,dominance=FALSE,plot=FALSE,verbose=FALSE){
    if(num > (nind(cross)-15) && !dominance){
        stop("Trying to set: ",num," markers as cofactor. This leaves less than 15 Degrees of Freedom.\n")
    }
    if(num > ((nind(cross)-15)/2) && dominance){
        stop("Trying to set: ",num," markers as cofactor. This leaves less than 15 Degrees of Freedom.\n")
    }
    if(distance < 0.1){
        distance <- 0.1
    }
    #  r <- scanone(cross)
    cofactors <- rep(0,sum(nmar(cross)))
    missing <- scoremissingmarkers(cross)
    densities <- calculatedensity(cross,distance*2)*missing
    cnt <- 0
    while(sum(cofactors) < num && cnt < num){
        lefttoset <- num - sum(cofactors)
        if(verbose) cat("Cofactors left",lefttoset,"/",num,"\n")
        possible <- which(max(densities)==densities)
        if(length(possible) > lefttoset){
            possible <- sample(possible,lefttoset)
        }
        cofactors[possible] <- 1
        densities[which(cofactors==1)] <- 0
        cofactors <- checkdistances(cross,cofactors,distance)
        cnt <- cnt+1
    }
    if(cnt==num && verbose) cat("Solution by iteration, there might be less cofactors then requested\n")
    if(plot) mqmplot.cofactors(cross,cofactors)
    cofactors
}

checkdistances <- function(cross,cofactors,dist=5){
    map <- unlist(pull.map(cross))
    newcofactors <- cofactors
    cnt_dropped <- 0
    for(x in which(cofactors==1)){
        for(y in which(cofactors==1)){
            if(x != y){
                chr_x <- strsplit(names(map[x]),'.',fixed=TRUE)[[1]][1]
                loc_x <- as.double(map[x])
                chr_y <- strsplit(names(map[y]),'.',fixed=TRUE)[[1]][1]
                loc_y <- as.double(map[y])
                if(chr_x==chr_y && abs(loc_x-loc_y) < dist){
                    newcofactors[y] <- 0
                    cnt_dropped <- cnt_dropped+1
                }
            }
        }
    }
    #cat("Dropped ",cnt_dropped," cofactors due to conflicting locations\n")
    newcofactors
}

mqmplot.cofactors <- function(cross,cofactors,...){
    map <- pull.map(cross)
    qc <- NULL
    qn <- NULL
    qp <- NULL
    mapnames <- NULL
    for(x in 1:nchr(cross)){
        mapnames <- c(mapnames,names(pull.map(cross)[[x]]))
    }
    chr <- 1
    genotype <- pull.geno(cross)
    for(x in 1:length(cofactors)){
        if(x > sum(nmar(cross)[1:chr])){
            chr <- chr+1
        }
        if(cofactors[x]>0){
            qn <- c(qn, mapnames[x])
            qc <- c(qc, as.character(names(map)[chr]))
            qp <- c(qp, as.double(unlist(map)[x]))
        }
    }
    plot(makeqtl(sim.geno(cross),qc,qp,qn),...)
}

# end of mqmcofactors.R

library(qtl)
data(hyper)

# write to tidy format
write.cross(hyper, "tidy", "hyper_tidy")

# read back in
x <- read.cross("tidy", "", genfile="hyper_tidy_gen.csv",
                mapfile="hyper_tidy_map.csv", phefile="hyper_tidy_phe.csv",
                genotypes=c("BB", "BA", "AA"))

# compare results
comparecrosses(x, hyper)

######################################################################
#
# addqtl.R
#
# copyright (c) 2007-2021, Karl W. Broman
# last modified Mar, 2021
# first written Nov, 2007
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: addint, print.addint, addqtl, addpair,
#           reviseqtlnuminformula, qtlformulasymmetric
#           dropfromqtlformula
#           addcovarint, print.addcovarint, summary.addcovarint
#
######################################################################

######################################################################
# addint
#
# Try adding each possible QTL:QTL interaction (that is not
# already in the formula), and give results similar to the drop-one
# analysis.
######################################################################
addint <-
    function(cross, pheno.col=1, qtl, covar=NULL, formula,
             method=c("imp","hk"), model=c("normal", "binary"),
             qtl.only=FALSE, verbose=TRUE, pvalues=TRUE, simple=FALSE,
             tol=1e-4, maxit=1000, require.fullrank=FALSE)
{
    if( !inherits(cross, "cross") )
        stop("The cross argument must be an object of class \"cross\".")

    if( !inherits(qtl, "qtl") )
        stop("The qtl argument must be an object of class \"qtl\".")

    if(!is.null(covar) && !is.data.frame(covar)) {
        if(is.matrix(covar) && is.numeric(covar))
            covar <- as.data.frame(covar, stringsAsFactors=TRUE)
        else stop("covar should be a data.frame")
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("addint can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 | pheno.col > nphe(cross))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    pheno <- cross$pheno[,pheno.col]
    if(!is.null(covar) && nrow(covar) != length(pheno))
        stop("nrow(covar) != no. individuals in cross.")

    method <- match.arg(method)
    model <- match.arg(model)

    # allow formula to be a character string
    if(!missing(formula) && is.character(formula))
        formula <- as.formula(formula)

    if(method=="imp") {
        if(!("geno" %in% names(qtl))) {
            if("prob" %in% names(qtl)) {
                warning("The qtl object doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"draws\".")
        }
    }
    else {
        if(!("prob" %in% names(qtl))) {
            if("geno" %in% names(qtl)) {
                warning("The qtl object doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"prob\".")
        }
    }

    if(qtl$n.ind != nind(cross)) {
        warning("No. individuals in qtl object doesn't match that in the input cross; re-creating qtl object.")
        if(method=="imp")
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        else
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="prob")
    }
    if(method=="imp" && dim(qtl$geno)[3] != dim(cross$geno[[1]]$draws)[3])  {
        warning("No. imputations in qtl object doesn't match that in the input cross; re-creating qtl object.")
        qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
    }

    # check phenotypes and covariates; drop ind'ls with missing values
    if(!is.null(covar)) phcovar <- cbind(pheno, covar)
    else phcovar <- as.data.frame(pheno, stringsAsFactors=TRUE)
    if(any(is.na(phcovar))) {
        if(ncol(phcovar)==1) hasmissing <- is.na(phcovar)
        else hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
        if(all(hasmissing))
            stop("All individuals are missing phenotypes or covariates.")
        if(any(hasmissing)) {
            warning("Dropping ", sum(hasmissing), " individuals with missing phenotypes.\n")
            pheno <- pheno[!hasmissing]
            qtl$n.ind <- sum(!hasmissing)
            if(method=="imp")
                qtl$geno <- qtl$geno[!hasmissing,,,drop=FALSE]
            else
                qtl$prob <- lapply(qtl$prob, function(a) a[!hasmissing,,drop=FALSE])

            if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]
        }
    }

    # number of covariates
    if( is.null(covar) ) n.covar <- 0
    else n.covar <- ncol(covar)

    # if formula is missing, build one
    # all QTLs and covarariates will be additive by default
    n.qtl <- qtl$n.qtl
    if(missing(formula)) {
        tmp.Q <- paste("Q", 1:n.qtl, sep="") # QTL term names
        formula <- "y~Q1"
        if(n.qtl > 1)
            for (i in 2:n.qtl)
                formula <- paste(formula, tmp.Q[i], sep="+")
        if (n.covar) { # if covarariate is not empty
            tmp.C <- colnames(covar) # covarariate term names
            for(i in 1:n.covar)
                formula <- paste(formula, tmp.C[i], sep="+")
        }
        formula <- as.formula(formula)
    }

    # check input formula
    formula <- checkformula(formula, qtl$altname, colnames(covar))

    # look for interactions that haven't been added
    factors <- attr(terms(formula), "factors")
    if(sum(factors[1,])==0) factors <- factors[-1,]

    # replace QTL altnames (Q1 etc) with real names (chr1@20 etc)
    fn <- fn.alt <- rownames(factors)
    qan <- qtl$altname
    qn <- qtl$name
    m <- match(fn, qan)
    fn.alt[!is.na(m)] <- qn[m[!is.na(m)]]

    # all possible interactions
    int2test <- int2test.alt <- NULL
    for(i in 1:(nrow(factors)-1)) {
        for(j in (i+1):nrow(factors)) {
            temp <- rep(0, nrow(factors))
            temp[c(i,j)] <- 1
            if(!any(apply(factors, 2, function(a, b) all(a==b), temp))) {
                int2test <- c(int2test, paste(fn[i], fn[j], sep=":"))
                int2test.alt <- c(int2test.alt, paste(fn.alt[i], fn.alt[j], sep=":"))
            }
        }
    }

    if(qtl.only && length(int2test) > 0) {
        z <- matrix(unlist(strsplit(int2test, ":")), ncol=2, byrow=TRUE)
        wh <- apply(z, 1, function(a)
                    length(grep("^[Qq][0-9]+$", a)) )
        int2test <- int2test[wh==2]
        int2test.alt <- int2test.alt[wh==2]
    }

    n2test <- length(int2test)

    if(n2test == 0) {
        if(verbose) cat("No pairwise interactions to add.\n")
        return(NULL)
    }

    sexpgm <- getsex(cross)
    cross.attr <- attributes(cross)

    # fit base model
    thefit0 <- fitqtlengine(pheno=pheno, qtl=qtl, covar=covar, formula=formula,
                            method=method, model=model, dropone=FALSE, get.ests=FALSE,
                            run.checks=FALSE, cross.attr=cross.attr,
                            crosstype=crosstype(cross), sexpgm=sexpgm, tol=tol, maxit=maxit)

    matrix0.rank <- attr(thefit0, "matrix.rank")
    matrix0.ncol <- attr(thefit0, "matrix.ncol")

    results <- matrix(ncol=7, nrow=n2test)
    dimnames(results) <- list(int2test.alt, c("df", "Type III SS", "LOD", "%var",
                                              "F value", "Pvalue(Chi2)", "Pvalue(F)"))

    matrix1.rank <- matrix1.ncol <- rep(0, n2test)

    for(k in seq(along=int2test)) {
        thefit1 <- fitqtlengine(pheno=pheno, qtl=qtl, covar=covar,
                                formula=as.formula(paste(deparseQTLformula(formula), int2test[k], sep="+")),
                                method=method, model=model, dropone=FALSE, get.ests=FALSE,
                                run.checks=FALSE, cross.attr=cross.attr,
                                crosstype=crosstype(cross), sexpgm=sexpgm,
                                tol=tol, maxit=maxit)

        results[k,1] <- thefit1$result.full[1,1] - thefit0$result.full[1,1]
        results[k,2] <- thefit1$result.full[1,2] - thefit0$result.full[1,2]
        results[k,3] <- thefit1$result.full[1,4] - thefit0$result.full[1,4]

        results[k,4] <- 100*(1-10^(-2*thefit1$result.full[1,4]/qtl$n.ind)) -
            100*(1-10^(-2*thefit0$result.full[1,4]/qtl$n.ind))

        results[k,5] <- (results[k,2]/results[k,1])/thefit1$result.full[2,3]
        results[k,6] <- pchisq(results[k,3]*2*log(10), results[k,1], lower.tail=FALSE)
        results[k,7] <- pf(results[k,5], results[k,1], thefit1$result.full[3,1], lower.tail=FALSE)

        matrix1.rank[k] <- attr(thefit1, "matrix.rank")
        matrix1.ncol[k] <- attr(thefit1, "matrix.ncol")
    }
    matrix.fullrank <- (matrix1.rank - matrix0.rank == matrix1.ncol - matrix0.ncol)

    results <- as.data.frame(results, stringsAsFactors=TRUE)
    class(results) <- c("addint", "data.frame")
    attr(results, "method") <- method
    attr(results, "model") <- model
    attr(results, "formula") <- deparseQTLformula(formula)
    if(simple) pvalues <- FALSE
    attr(results, "pvalues") <- pvalues
    attr(results, "simple") <- simple
    attr(results, "matrix.fullrank") <- matrix.fullrank

    if(require.fullrank) results[!matrix.fullrank,3] <- 0

    results
}

print.addint <-
    function(x, ...)
{
    meth <- attr(x, "method")
    mod <- attr(x, "model")
    simp <- attr(x, "simple")
    if(is.null(mod)) mod <- "normal"
    if(is.null(meth)) meth <- "unknown"
    if(mod=="binary" || simp) attr(x, "pvalues") <- FALSE
    if(meth=="imp") meth <- "multiple imputation"
    else if(meth=="hk") meth <- "Haley-Knott regression"
    cat("Method:", meth, "\n")
    cat("Model: ", mod, "phenotype\n")

    cat("Model formula:")
    w <- options("width")[[1]]
    printQTLformulanicely(attr(x, "formula"), "                   ", w+5, w)
    cat("\n")

    cat("Add one pairwise interaction at a time table:\n")
    cat("--------------------------------------------\n")
    pval <- attr(x, "pvalues")
    if(!is.null(pval) && !pval)
        x <- x[,-ncol(x)+(0:1)]

    if(mod == "binary" || simp) x <- x[,c(1,3,4), drop=FALSE]

    printCoefmat(x, digits=4, cs.ind=1, P.values=pval, has.Pvalue=pval)

    cat("\n")
}

summary.addint <- function(object, ...) object


######################################################################
# addqtl
#
# scan for an additional QTL in the context of a multiple-QTL model
#
# If the formula includes one more QTL than in the QTL object, we
# use it as given; otherwise, a main effect for the additional QTL
# is added
#
# the output is like scanone
######################################################################
addqtl <-
    function(cross, chr, pheno.col=1, qtl, covar=NULL, formula,
             method=c("imp","hk"), model=c("normal", "binary"),
             incl.markers=TRUE, verbose=FALSE, tol=1e-4, maxit=1000,
             forceXcovar=FALSE, require.fullrank=FALSE)
{
    method <- match.arg(method)
    model <- match.arg(model)

    if( !inherits(cross, "cross") )
        stop("The cross argument must be an object of class \"cross\".")

    if( !inherits(qtl, "qtl") )
        stop("The qtl argument must be an object of class \"qtl\".")

    # allow formula to be a character string
    if(!missing(formula) && is.character(formula))
        formula <- as.formula(formula)

    if(!is.null(covar) && !is.data.frame(covar)) {
        if(is.matrix(covar) && is.numeric(covar))
            covar <- as.data.frame(covar, stringsAsFactors=TRUE)
        else stop("covar should be a data.frame")
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(method=="imp") {
        if(!("geno" %in% names(qtl))) {
            if("prob" %in% names(qtl)) {
                warning("The qtl object doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"draws\".")
        }
    }
    else {
        if(!("prob" %in% names(qtl))) {
            if("geno" %in% names(qtl)) {
                warning("The qtl object doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"prob\".")
        }
    }

    if(verbose > 1) {
        verbose <- TRUE
        verbose.scanqtl <- TRUE
    }
    else verbose.scanqtl <- FALSE

    n.qtl <- qtl$n.qtl
    qtlchr <- qtl$chr
    qtlpos <- qtl$pos

    if(qtl$n.ind != nind(cross)) {
        warning("No. individuals in qtl object doesn't match that in the input cross; re-creating qtl object.")
        if(method=="imp")
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        else
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="prob")
    }
    if(method=="imp" && dim(qtl$geno)[3] != dim(cross$geno[[1]]$draws)[3])  {
        warning("No. imputations in qtl object doesn't match that in the input cross; re-creating qtl object.")
        qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
    }

    if(method=="imp") {
        if("stepwidth" %in% names(attributes(cross$geno[[1]]$draws)) &&
           attr(cross$geno[[1]]$draws, "stepwidth") != "fixed") {
            stepwidth.var <- TRUE
            incl.markers <- TRUE
        }
        else stepwidth.var <- FALSE
    }
    else {
        if("stepwidth" %in% names(attributes(cross$geno[[1]]$prob)) &&
           attr(cross$geno[[1]]$prob, "stepwidth") != "fixed") {
            stepwidth.var <- TRUE
            incl.markers <- TRUE
        }
        else stepwidth.var <- FALSE
    }

    # look for the chr
    if(missing(chr)) chr <- names(cross$geno)
    else chr <- matchchr(chr, names(cross$geno))

    # if formula is missing, make one.
    # All QTLs and covariates will be additive by default
    if(is.null(covar)) n.covar <- 0
    else n.covar <- ncol(covar)

    if(missing(formula)) {
        tmp.Q <- paste("Q", 1:n.qtl, sep="") # QTL term names
        formula <- "y~Q1"
        if(n.qtl > 1)
            for (i in 2:n.qtl)
                formula <- paste(formula, tmp.Q[i], sep="+")
        if(n.covar) { # if covariate is not empty
            tmp.C <- names(covar) # covariate term names
            for(i in 1:n.covar)
                formula <- paste(formula, tmp.C[i], sep="+")
        }
        newformula <- as.formula(paste(formula, "+Q", n.qtl+1, sep=""))
        formula <- as.formula(formula)
    }
    else { # formula given
        newqtl <- paste("Q", n.qtl+1, sep="")

        # check the formula
        formula <- checkformula(formula, c(qtl$altname, newqtl), colnames(covar))

        theterms <- rownames(attr(terms(formula), "factors"))

        # is new QTL in the formula?
        g <- grep(paste("^[Qq]", n.qtl+1, "$", sep=""), theterms)
        if(length(g) == 0)  { # no; add to formula
            newformula <- as.formula(paste(deparseQTLformula(formula), "+ Q", n.qtl+1, sep=""))
        }
        else { # need a version without it
            newformula <- formula

            theterms <- colnames(attr(terms(formula), "factors"))
            g <- unique(c(grep(paste("^[Qq]", n.qtl+1, "$", sep=""), theterms),
                          grep(paste("^[Qq]", n.qtl+1, " *:", sep=""), theterms),
                          grep(paste(": +[Qq]", n.qtl+1, " *:", sep=""), theterms),
                          grep(paste(": +[Qq]", n.qtl+1, "$", sep=""), theterms)))

            if(length(g) > 0) {
                theterms <- theterms[-g]
                formula <- as.formula(paste("y ~ ", paste(theterms, collapse=" + "), sep=""))
            }
        }
    }

    # drop qtl that are not in the formula
    thefactors <- rownames(attr(terms(formula), "factors"))
    todrop <- NULL
    for(i in 1:n.qtl) {
        if(length(grep(paste("^[Qq]", i, "$", sep=""), thefactors))==0)
            todrop <- c(todrop, i)
    }

    if(length(todrop) > 0) {
        newqtlnum <- n.qtl+1

        notdropped <- (1:n.qtl)[-todrop]
        newnum <- 1:length(notdropped)

        qtl <- dropfromqtl(qtl, index=todrop)
        qtlchr <- qtlchr[-todrop]
        qtlpos <- qtlpos[-todrop]
        n.qtl <- n.qtl - length(todrop)
        revnewqtlnum <- n.qtl+1

        formula <- reviseqtlnuminformula(formula, notdropped, newnum)
        newformula <- reviseqtlnuminformula(newformula, c(notdropped,newqtlnum),
                                            c(newnum, revnewqtlnum))
    }

    # drop covariates that are not in the formula
    if(!is.null(covar)) {
        theterms <- rownames(attr(terms(formula), "factors"))
        m <- match(colnames(covar), theterms)
        if(all(is.na(m))) covar <- NULL
        else covar <- covar[,!is.na(m),drop=FALSE]
    }

    # phenotype column
    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("addqtl can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 || pheno.col > nphe(cross))
        stop("pheno.col should be between 1 and ", nphe(cross))
    pheno <- cross$pheno[,pheno.col]
    if(!is.null(covar) && nrow(covar) != length(pheno))
        stop("nrow(covar) != no. individuals in cross.")

    # check phenotypes and covariates; drop ind'ls with missing values
    if(!is.null(covar)) phcovar <- cbind(pheno, covar)
    else phcovar <- as.data.frame(pheno, stringsAsFactors=TRUE)
    if(any(is.na(phcovar))) {
        if(ncol(phcovar)==1) hasmissing <- is.na(phcovar)
        else hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
        if(all(hasmissing))
            stop("All individuals are missing phenotypes or covariates.")
        if(any(hasmissing)) {
            warning("Dropping ", sum(hasmissing), " individuals with missing phenotypes.\n")
            pheno <- pheno[!hasmissing]
            qtl$n.ind <- sum(!hasmissing)
            if(method=="imp")
                qtl$geno <- qtl$geno[!hasmissing,,,drop=FALSE]
            else
                qtl$prob <- lapply(qtl$prob, function(a) a[!hasmissing,,drop=FALSE])

            if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]
            cross <- subset(cross, ind=!hasmissing)
        }
    }

    sexpgm <- getsex(cross)
    cross.attr <- attributes(cross)

    # fit the base model
    fit0 <- fitqtlengine(pheno=pheno, qtl=qtl, covar=covar, formula=formula,
                         method=method, model=model, dropone=FALSE, get.ests=FALSE,
                         run.checks=FALSE, cross.attr=cross.attr,
                         crosstype=crosstype(cross), sexpgm=sexpgm,
                         tol=tol, maxit=maxit, forceXcovar=forceXcovar)
    lod0 <- fit0$result.full[1,4]
    matrix0.rank <- attr(fit0, "matrix.rank")
    matrix0.ncol <- attr(fit0, "matrix.ncol")

    results <- matrix1.rank <- matrix1.ncol <- NULL
    for(i in chr) {
        if(verbose) cat("Scanning chr", i, "\n")
        thechr <- c(qtlchr, i)
        thepos <- c(as.list(qtlpos), list(c(-Inf,Inf)))

        sqout <- scanqtl(cross, pheno.col=pheno.col, chr=thechr, pos=thepos,
                         covar=covar, formula=newformula, method=method, model=model,
                         incl.markers=incl.markers, verbose=verbose.scanqtl,
                         tol=tol, maxit=maxit, forceXcovar=forceXcovar)

        matrix1.rank <- c(matrix1.rank, attr(sqout, "matrix.rank"))
        matrix1.ncol <- c(matrix1.ncol, attr(sqout, "matrix.ncol"))


        # get map of positions
        if(method=="imp") {
            if("map" %in% names(attributes(cross$geno[[i]]$draws)))
                map <- attr(cross$geno[[i]]$draws,"map")
            else {
                stp <- attr(cross$geno[[i]]$draws, "step")
                oe <- attr(cross$geno[[i]]$draws, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[i]]$draws)))
                    stpw <- attr(cross$geno[[i]]$draws, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
            }
        }
        else {
            if("map" %in% names(attributes(cross$geno[[i]]$prob)))
                map <- attr(cross$geno[[i]]$prob,"map")
            else {
                stp <- attr(cross$geno[[i]]$prob, "step")
                oe <- attr(cross$geno[[i]]$prob, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[i]]$prob)))
                    stpw <- attr(cross$geno[[i]]$prob, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[i]]$map,stp,oe,stpw)
            }
        }
        # pull out the female map if there are sex-specific maps
        if(is.matrix(map)) map <- map[1,]

        if(method=="imp")
            step <- attr(cross$geno[[i]]$draws,"step")
        else
            step <- attr(cross$geno[[i]]$prob,"step")

        if(!incl.markers && step>0) { # equally spaced positions
            eq.sp.pos <- seq(min(map), max(map), by=step)
            wh.eq.pos <- match(eq.sp.pos, map)
            map <- map[wh.eq.pos]
        }

        w <- names(map)
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",i,".",w[o],sep="")

        z <- data.frame(lod=as.numeric(sqout)-lod0, stringsAsFactors=TRUE)
        z <- cbind(chr=factor(rep(i,length(map)),levels=i),
                   pos=as.numeric(map), z)
        rownames(z) <- w

        results <- rbind(results, z)
    }
    matrix.fullrank <- (matrix1.rank - matrix0.rank == matrix1.ncol - matrix0.ncol)

    class(results) <- c("scanone","data.frame")
    attr(results,"method") <- method
    attr(results,"formula") <- deparseQTLformula(newformula)

    attr(results, "matrix.fullrank") <- matrix.fullrank
    if(require.fullrank) results[!matrix.fullrank,3] <- 0

    results
}


######################################################################
# addpair
#
# scan for an additional pair of QTL in the context of a multiple-QTL
# model
#
# If the formula includes one more QTL than in the QTL object, we
# use it as given (perhaps adding the second, additively);
# otherwise, we do as in scantwo, performing both additive and
# interactive models, plus a single-QTL scan
#
# The output is like scantwo.  If we didn't do the scantwo type format,
# the results are placed where the full LODs usually are, and everything
# else is NA
######################################################################
addpair <-
    function(cross, chr, pheno.col=1, qtl, covar=NULL, formula,
             method=c("imp","hk"), model=c("normal", "binary"),
             incl.markers=FALSE, verbose=TRUE, tol=1e-4, maxit=1000,
             forceXcovar=FALSE)
{
    method <- match.arg(method)
    model <- match.arg(model)

    if( !inherits(cross, "cross") )
        stop("The cross argument must be an object of class \"cross\".")

    if( !inherits(qtl, "qtl") )
        stop("The qtl argument must be an object of class \"qtl\".")

    # allow formula to be a character string
    if(!missing(formula) && is.character(formula))
        formula <- as.formula(formula)

    if(!is.null(covar) && !is.data.frame(covar)) {
        if(is.matrix(covar) && is.numeric(covar))
            covar <- as.data.frame(covar, stringsAsFactors=TRUE)
        else stop("covar should be a data.frame")
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(method=="imp") {
        if(!("geno" %in% names(qtl))) {
            if("prob" %in% names(qtl)) {
                warning("The qtl object doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"draws\".")
        }
    }
    else {
        if(!("prob" %in% names(qtl))) {
            if("geno" %in% names(qtl)) {
                warning("The qtl object doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"prob\".")
        }
    }

    if(verbose > 1) {
        verbose <- TRUE
        verbose.scanqtl <- TRUE
    }
    else verbose.scanqtl <- FALSE

    n.qtl <- qtl$n.qtl
    qtlchr <- qtl$chr
    qtlpos <- qtl$pos
    if(qtl$n.ind != nind(cross)) {
        warning("No. individuals in qtl object doesn't match that in the input cross; re-creating qtl object.")
        if(method=="imp")
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        else
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="prob")
    }
    if(method=="imp" && dim(qtl$geno)[3] != dim(cross$geno[[1]]$draws)[3]) {
        warning("No. imputations in qtl object doesn't match that in the input cross; re-creating qtl object.")
        qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
    }

    if(method=="imp") {
        if("stepwidth" %in% names(attributes(cross$geno[[1]]$draws)) &&
           attr(cross$geno[[1]]$draws, "stepwidth") != "fixed") {
            stepwidth.var <- TRUE
            incl.markers <- TRUE
        }
        else stepwidth.var <- FALSE
    }
    else {
        if("stepwidth" %in% names(attributes(cross$geno[[1]]$prob)) &&
           attr(cross$geno[[1]]$prob, "stepwidth") != "fixed") {
            stepwidth.var <- TRUE
            incl.markers <- TRUE
        }
        else stepwidth.var <- FALSE
    }

    # look for the chr
    if(missing(chr)) chr <- names(cross$geno)
    else chr <- matchchr(chr, names(cross$geno))

    fullmap <- pull.map(cross, chr)

    # if formula is missing, make one.
    # All QTLs and covariates will be additive by default
    if(is.null(covar)) n.covar <- 0
    else n.covar <- ncol(covar)

    if(missing(formula)) {
        tmp.Q <- paste("Q", 1:n.qtl, sep="") # QTL term names
        formula <- "y~Q1"
        if(n.qtl > 1)
            for (i in 2:n.qtl)
                formula <- paste(formula, tmp.Q[i], sep="+")
        if(n.covar) { # if covariate is not empty
            tmp.C <- names(covar) # covariate term names
            for(i in 1:n.covar)
                formula <- paste(formula, tmp.C[i], sep="+")
        }
        newformula1 <- as.formula(paste(formula, " + Q", n.qtl+1,
                                        " + Q", n.qtl+2, " + Q", n.qtl+1,
                                        ":Q", n.qtl+2, sep=""))
        newformula2 <- as.formula(paste(formula, " + Q", n.qtl+1,
                                        " + Q", n.qtl+2, sep=""))
        formula <- as.formula(formula)
        scanbothways <- FALSE
    }
    else { # formula given
        newqtl <- paste("Q", n.qtl+1:2, sep="")

        # check the formula
        formula <- checkformula(formula, c(qtl$altname, newqtl), colnames(covar))

        theterms <- rownames(attr(terms(formula), "factors"))

        # are either of the new QTL in the formula?
        g <- c(grep(paste("^[Qq]", n.qtl+1, "$", sep=""), theterms),
               grep(paste("^[Qq]", n.qtl+2, "$", sep=""), theterms))
        g1 <- grep(paste("^[Qq]", n.qtl+1, "$", sep=""), theterms)
        g2 <- grep(paste("^[Qq]", n.qtl+2, "$", sep=""), theterms)
        if(length(g) == 0)  { # no; add to formula
            newformula1 <- as.formula(paste(deparseQTLformula(formula), "+ Q", n.qtl+1,
                                            " + Q", n.qtl+2, " + Q", n.qtl+1,
                                            ":Q", n.qtl+2, sep=""))
            newformula2 <- as.formula(paste(deparseQTLformula(formula), "+ Q", n.qtl+1,
                                            " + Q", n.qtl+2, sep=""))
            scanbothways <- FALSE
        }
        else { # need a version without them
            # first make sure that *both* terms are in the formula
            if(length(g1)==0) # add Q1
                formula <- as.formula(paste(deparseQTLformula(formula), "+ Q", n.qtl+1, sep=""))
            if(length(g2)==0) # add Q2
                formula <- as.formula(paste(deparseQTLformula(formula), "+ Q", n.qtl+2, sep=""))

            newformula1 <- formula
            newformula2 <- NULL

            theterms <- colnames(attr(terms(formula), "factors"))
            g <- unique(c(grep(paste("^[Qq]", n.qtl+1, "$", sep=""), theterms),
                          grep(paste("^[Qq]", n.qtl+1, " *:", sep=""), theterms),
                          grep(paste(": *[Qq]", n.qtl+1, " *:", sep=""), theterms),
                          grep(paste(": *[Qq]", n.qtl+1, "$", sep=""), theterms),
                          grep(paste("^[Qq]", n.qtl+2, "$", sep=""), theterms),
                          grep(paste("^[Qq]", n.qtl+2, " *:", sep=""), theterms),
                          grep(paste(": *[Qq]", n.qtl+2, " *:", sep=""), theterms),
                          grep(paste(": *[Qq]", n.qtl+2, "$", sep=""), theterms)))
            if(length(g) > 0) {
                theterms <- theterms[-g]
                formula <- as.formula(paste("y ~ ", paste(theterms, collapse=" + "), sep=""))
            }

            # if the QTL formula is symmetric in the two new QTL, need scan only for i<j
            # if not symmetric, we also need to scan with j<i
            scanbothways <- !qtlformulasymmetric(newformula1, n.qtl+1, n.qtl+2)

            if(scanbothways) {
                newformula1.minus1 <- dropfromqtlformula(newformula1, n.qtl+1)
                newformula1.minus2 <- dropfromqtlformula(newformula1, n.qtl+2)
                newformula1.minus1 <- reviseqtlnuminformula(newformula1.minus1, n.qtl+2, n.qtl+1)
            }
        }
    }

    # drop qtl that are not in the formula
    thefactors <- rownames(attr(terms(formula), "factors"))
    todrop <- NULL
    for(i in 1:n.qtl) {
        if(length(grep(paste("^[Qq]", i, "$", sep=""), thefactors))==0)
            todrop <- c(todrop, i)
    }
    if(length(todrop) > 0) {
        newqtlnum1 <- n.qtl+1
        newqtlnum2 <- n.qtl+2

        notdropped <- (1:n.qtl)[-todrop]
        newnum <- 1:length(notdropped)

        qtl <- dropfromqtl(qtl, index=todrop)
        qtlchr <- qtlchr[-todrop]
        qtlpos <- qtlpos[-todrop]
        n.qtl <- n.qtl - length(todrop)
        revnewqtlnum1 <- n.qtl+1
        revnewqtlnum2 <- n.qtl+2

        formula <- reviseqtlnuminformula(formula, notdropped, newnum)
        newformula1 <- reviseqtlnuminformula(newformula1, c(notdropped, newqtlnum1, newqtlnum2),
                                             c(newnum, revnewqtlnum1, revnewqtlnum2))
        if(!is.null(newformula2))
            newformula2 <- reviseqtlnuminformula(newformula2, c(notdropped, newqtlnum1, newqtlnum2),
                                                 c(newnum, revnewqtlnum1, revnewqtlnum2))
        if(scanbothways) {
            newformula1.minus1 <- reviseqtlnuminformula(newformula1.minus1,
                                                        c(notdropped, newqtlnum1),
                                                        c(newnum, revnewqtlnum1))

            newformula1.minus2 <- reviseqtlnuminformula(newformula1.minus2,
                                                        c(notdropped, newqtlnum1),
                                                        c(newnum, revnewqtlnum1))
        }
    }

    # drop covariates that are not in the formula
    if(!is.null(covar)) {
        theterms <- rownames(attr(terms(formula), "factors"))
        m <- match(colnames(covar), theterms)
        if(all(is.na(m))) covar <- NULL
        else covar <- covar[,!is.na(m),drop=FALSE]
    }

    # phenotype column
    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("addpair can take just one phenotype; only the first will be used")
    }
    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }
    if(pheno.col < 1 || pheno.col > nphe(cross))
        stop("pheno.col should be between 1 and ", nphe(cross))
    pheno <- cross$pheno[,pheno.col]
    if(!is.null(covar) && nrow(covar) != length(pheno))
        stop("nrow(covar) != no. individuals in cross.")

    # check phenotypes and covariates; drop ind'ls with missing values
    if(!is.null(covar)) phcovar <- cbind(pheno, covar)
    else phcovar <- as.data.frame(pheno, stringsAsFactors=TRUE)
    if(any(is.na(phcovar))) {
        if(ncol(phcovar)==1) hasmissing <- is.na(phcovar)
        else hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
        if(all(hasmissing))
            stop("All individuals are missing phenotypes or covariates.")
        if(any(hasmissing)) {
            warning("Dropping ", sum(hasmissing), " individuals with missing phenotypes.\n")
            pheno <- pheno[!hasmissing]
            qtl$n.ind <- sum(!hasmissing)
            if(method=="imp")
                qtl$geno <- qtl$geno[!hasmissing,,,drop=FALSE]
            else
                qtl$prob <- lapply(qtl$prob, function(a) a[!hasmissing,,drop=FALSE])

            if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]
            cross <- subset(cross, ind=!hasmissing)
        }
    }

    sexpgm <- getsex(cross)
    cross.attr <- attributes(cross)

    # fit the base model
    fit0 <- fitqtlengine(pheno=pheno, qtl=qtl, covar=covar, formula=formula,
                         method=method, model=model, dropone=FALSE, get.ests=FALSE,
                         run.checks=FALSE, cross.attr=cross.attr,
                         crosstype=crosstype(cross), sexpgm=sexpgm,
                         tol=tol, maxit=maxit, forceXcovar=forceXcovar)
    lod0 <- fit0$result.full[1,4]

    gmap <- NULL

    # form map
    for(i in 1:length(chr)) {
        ci <- chr[i]
        # get map of positions
        if(method=="imp") {
            if("map" %in% names(attributes(cross$geno[[ci]]$draws)))
                map <- attr(cross$geno[[ci]]$draws,"map")
            else {
                stp <- attr(cross$geno[[ci]]$draws, "step")
                oe <- attr(cross$geno[[ci]]$draws, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[ci]]$draws)))
                    stpw <- attr(cross$geno[[ci]]$draws, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[ci]]$map,stp,oe,stpw)
            }
        }
        else {
            if("map" %in% names(attributes(cross$geno[[ci]]$prob)))
                map <- attr(cross$geno[[ci]]$prob,"map")
            else {
                stp <- attr(cross$geno[[ci]]$prob, "step")
                oe <- attr(cross$geno[[ci]]$prob, "off.end")

                if("stepwidth" %in% names(attributes(cross$geno[[ci]]$prob)))
                    stpw <- attr(cross$geno[[ci]]$prob, "stepwidth")
                else stpw <- "fixed"
                map <- create.map(cross$geno[[ci]]$map,stp,oe,stpw)
            }
        }
        # pull out the female map if there are sex-specific maps
        if(is.matrix(map)) map <- map[1,]

        w <- names(map)
        o <- grep("^loc-*[0-9]+",w)
        if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
            w[o] <- paste("c",ci,".",w[o],sep="")
        map <- data.frame(chr=factor(rep(ci,length(map)),levels=ci),
                          pos=as.numeric(map), stringsAsFactors=TRUE)
        rownames(map) <- w

        if(method=="imp")
            step <- attr(cross$geno[[ci]]$draws,"step")
        else
            step <- attr(cross$geno[[ci]]$prob,"step")

        if(step==0 || stepwidth.var)  # just use markers
            eq.sp.pos <- rep(1,nrow(map))
        else {
            eq.sp.pos <- seq(min(map[,2]),max(map[,2]),by=step)
            wh.eq.sp <- match(eq.sp.pos,map[,2])
            if(any(is.na(wh.eq.sp))) { # this shouldn't happen
                warning("Possible error in determining the equally spaced positions.")
                wh.eq.sp <- wh.eq.sp[!is.na(wh.eq.sp)]
            }
            eq.sp.pos <- rep(0,nrow(map))
            eq.sp.pos[wh.eq.sp] <- 1
        }
        if(!incl.markers && any(eq.sp.pos==0)) {
            map <- map[eq.sp.pos==1,]
            eq.sp.pos <- eq.sp.pos[eq.sp.pos==1]
        }
        gmap <- rbind(gmap, cbind(map,
                                  eq.spacing=eq.sp.pos,
                                  xchr=inherits(cross$geno[[i]], "X")))
    }

    lod <- matrix(ncol=nrow(gmap), nrow=nrow(gmap))
    if(scanbothways) lod.m1 <- lod.m2 <- rep(NA, nrow(gmap))

    for(i in 1:length(chr)) {
        ci <- chr[i]
        whi <- which(gmap[,1]==ci)
        for(j in i:length(chr)) {
            cj <- chr[j]
            whj <- which(gmap[,1]==cj)

            if(verbose) {
                if(is.null(newformula2))
                    cat("Scanning chr", ci, "and", cj, "\n")
                else
                    cat("Scanning full model for chr", ci, "and", cj, "\n")
            }

            thechr <- c(qtlchr, ci, cj)
            thepos <- c(as.list(qtlpos), list(c(-Inf, Inf)), list(c(-Inf, Inf)))

            temp1 <- scanqtl(cross, pheno.col=pheno.col, chr=thechr, pos=thepos,
                             covar=covar, formula=newformula1, method=method, model=model,
                             incl.markers=incl.markers, verbose=verbose.scanqtl,
                             tol=tol, maxit=maxit, forceXcovar=forceXcovar) - lod0

            if(!is.null(newformula2)) {
                if(verbose)
                    cat("Scanning add've model for chr", ci, "and", cj, "\n")

                temp2 <- scanqtl(cross, pheno.col=pheno.col, chr=thechr, pos=thepos,
                                 covar=covar, formula=newformula2, method=method, model=model,
                                 incl.markers=incl.markers, verbose=verbose.scanqtl,
                                 tol=tol, maxit=maxit, forceXcovar=forceXcovar) - lod0
            }
            else {
                if(i != j && scanbothways) {
                    if(verbose) cat("Scanning chr", cj, "and", ci, "\n")
                    thechr <- c(qtlchr, cj, ci)
                    temp1r <- scanqtl(cross, pheno.col=pheno.col, chr=thechr, pos=thepos,
                                      covar=covar, formula=newformula1, method=method, model=model,
                                      incl.markers=incl.markers, verbose=verbose.scanqtl,
                                      tol=tol, maxit=maxit, forceXcovar=forceXcovar) - lod0
                }
            }

            if(i==j) {
                if(!is.null(newformula2))
                    temp1[upper.tri(temp1)] <- temp2[upper.tri(temp1)]
                else temp1 <- t(temp1)

                lod[whi,whi] <- temp1
            }
            else {
                if(!is.null(newformula2)) {
                    lod[whi,whj] <- t(temp2)
                    lod[whj,whi] <- temp1
                }
                else {
                    lod[whi,whj] <- t(temp1)
                    if(scanbothways)
                        lod[whj,whi] <- t(temp1r)
                    else
                        lod[whj,whi] <- temp1
                }
            }

        }

        if(scanbothways) {
            if(verbose) cat("Scanning chr", ci, "alone\n")

            thechr <- c(qtlchr, ci)
            thepos <- c(as.list(qtlpos), list(c(-Inf, Inf)))

            lod.m1[whi] <- scanqtl(cross, pheno.col=pheno.col, chr=thechr, pos=thepos,
                                   covar=covar, formula=newformula1.minus1, method=method,
                                   model=model, incl.markers=incl.markers,
                                   verbose=verbose.scanqtl, tol=tol, maxit=maxit, forceXcovar=forceXcovar) - lod0
            lod.m2[whi] <- scanqtl(cross, pheno.col=pheno.col, chr=thechr, pos=thepos,
                                   covar=covar, formula=newformula1.minus2, method=method,
                                   model=model, incl.markers=incl.markers,
                                   verbose=verbose.scanqtl, tol=tol, maxit=maxit, forceXcovar=forceXcovar) - lod0

        }

    }

    result <- list(lod=lod, map=gmap, scanoneX=NULL)
    class(result) <- "scantwo"
    attr(result, "fullmap") <- fullmap
    attr(result,"method") <- method
    attr(result,"formula") <- deparseQTLformula(newformula1)

    if(scanbothways) {
        attr(result, "lod.minus1") <- lod.m1
        attr(result, "lod.minus2") <- lod.m2
    }

    if(is.null(newformula2)) attr(result, "addpair") <- TRUE

    result
}



######################################################################
# consider, e.g., oldnum=5 and newnum = 3
# This functions replaces any instances of "Q5" or "q5" in the
# formula with "Q3".
######################################################################
reviseqtlnuminformula <-
    function(formula, oldnum, newnum)
{
    if(is.character(formula))
        formula <- as.formula(formula)

    if(length(oldnum) != length(newnum))
        stop("oldnum and newnum must be the same length.")

    newterms <- theterms <- colnames(attr(terms(formula), "factors"))

    for(i in seq(along=oldnum)) {
        g <- grep(paste("^[Qq]", oldnum[i], "$", sep=""), theterms)
        if(length(g) > 0)
            newterms[g] <- paste("Q", newnum[i], sep="")
    }

    intxn <- grep(":", theterms)
    if(length(intxn) > 0) {
        temp <- strsplit(theterms[intxn], ":")
        for(i in seq(along=temp)) {
            for(j in seq(along=oldnum)) {
                g <- grep(paste("^[Qq]", oldnum[j], "$", sep=""), temp[[i]])
                if(length(g) > 0)
                    temp[[i]][g] <- paste("Q", newnum[j], sep="")
            }
        }
        temp <- sapply(temp, paste, collapse=":")
        newterms[intxn] <- temp
    }
    as.formula(paste("y ~ ", paste(newterms, collapse=" + "), sep=""))
}

######################################################################
# return TRUE or FALSE according to whether the formula is symmetric
# in QTL qtlnum1 and qtlnum2
######################################################################
qtlformulasymmetric <-
    function(formula, qtlnum1, qtlnum2)
{
    theterms <- attr(terms(formula), "factors")
    rn <- rownames(theterms)
    wh1 <- grep(paste("^[Qq]", qtlnum1, "$", sep=""), rn)
    wh2 <- grep(paste("^[Qq]", qtlnum2, "$", sep=""), rn)
    cn <- colnames(theterms)

    if(length(wh1)==0 && length(wh2)==0) return(TRUE)
    if(length(wh1)==0 || length(wh2)==0) return(FALSE)

    revterms <- theterms
    revterms[c(wh1,wh2),] <- revterms[c(wh2, wh1),]

    theterms <- sort(apply(theterms, 2, paste, collapse=""))
    revterms <- sort(apply(revterms, 2, paste, collapse=""))

    all(theterms==revterms)
}


######################################################################
# If qtlnum=5, drop any terms containing Q5 or q5
######################################################################
dropfromqtlformula <-
    function(formula, qtlnum)
{
    theterms <- colnames(attr(terms(formula), "factors"))

    todrop <- NULL

    for(i in seq(along=qtlnum)) {
        g <- grep(paste("^[Qq]", qtlnum[i], "$", sep=""), theterms)
        if(length(g) > 0) todrop <- c(todrop, g)
    }

    intxn <- grep(":", theterms)
    if(length(intxn) > 0) {
        temp <- strsplit(theterms[intxn], ":")
        for(i in seq(along=temp)) {
            for(j in seq(along=qtlnum)) {
                g <- grep(paste("^[Qq]", qtlnum[j], "$", sep=""), temp[[i]])
                if(length(g) > 0) todrop <- c(todrop, intxn[i])
            }
        }
    }

    todrop <- unique(todrop)

    as.formula(paste("y ~ ", paste(theterms[-todrop], collapse=" + "), sep=""))
}



######################################################################
# addcovarint
#
# Try adding each QTL x covariate interaction (that is not
# already in the formula), and give results similar to the drop-one
# analysis.
######################################################################
addcovarint <-
    function(cross, pheno.col=1, qtl, covar=NULL, icovar, formula,
             method=c("imp","hk"), model=c("normal", "binary"),
             verbose=TRUE, pvalues=TRUE, simple=FALSE, tol=1e-4, maxit=1000,
             require.fullrank=FALSE)
{
    if( !inherits(cross, "cross"))
        stop("The cross argument must be an object of class \"cross\".")

    if( !inherits(qtl, "qtl"))
        stop("The qtl argument must be an object of class \"qtl\".")

    if(missing(covar) || is.null(covar))
        stop("Must include covariate data frame.")
    if(!is.data.frame(covar)) {
        if(is.matrix(covar) && is.numeric(covar))
            covar <- as.data.frame(covar, stringsAsFactors=TRUE)
        else stop("covar should be a data.frame")
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("addcovarint can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 | pheno.col > nphe(cross))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    pheno <- cross$pheno[,pheno.col]
    if(nrow(covar) != length(pheno))
        stop("nrow(covar) != no. individuals in cross.")

    if(missing(icovar))
        stop("Must include icovar (the covariate to consider in interactions)")

    if(!is.character(icovar) || any(is.na(match(icovar, colnames(covar)))))
        stop("icovar must be a vector of character strings corresonding to columns in covar.")

    method <- match.arg(method)
    model <- match.arg(model)

    # allow formula to be a character string
    if(!missing(formula) && is.character(formula))
        formula <- as.formula(formula)

    if(method=="imp") {
        if(!("geno" %in% names(qtl))) {
            if("prob" %in% names(qtl)) {
                warning("The qtl object doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"draws\".")
        }
    }
    else {
        if(!("prob" %in% names(qtl))) {
            if("geno" %in% names(qtl)) {
                warning("The qtl object doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"prob\".")
        }
    }

    if(qtl$n.ind != nind(cross)) {
        warning("No. individuals in qtl object doesn't match that in the input cross; re-creating qtl object.")
        if(method=="imp")
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        else
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="prob")
    }
    if(method=="imp" && dim(qtl$geno)[3] != dim(cross$geno[[1]]$draws)[3])  {
        warning("No. imputations in qtl object doesn't match that in the input cross; re-creating qtl object.")
        qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
    }

    # check phenotypes and covariates; drop ind'ls with missing values
    phcovar <- cbind(pheno, covar)
    if(any(is.na(phcovar))) {
        if(ncol(phcovar)==1) hasmissing <- is.na(phcovar)
        else hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
        if(all(hasmissing))
            stop("All individuals are missing phenotypes or covariates.")
        if(any(hasmissing)) {
            warning("Dropping ", sum(hasmissing), " individuals with missing phenotypes.\n")
            pheno <- pheno[!hasmissing]
            qtl$n.ind <- sum(!hasmissing)
            if(method=="imp")
                qtl$geno <- qtl$geno[!hasmissing,,,drop=FALSE]
            else
                qtl$prob <- lapply(qtl$prob, function(a) a[!hasmissing,,drop=FALSE])

            covar <- covar[!hasmissing,,drop=FALSE]
        }
    }

    # number of covariates
    n.covar <- ncol(covar)

    # if formula is missing, build one
    # all QTLs and covarariates will be additive by default
    n.qtl <- qtl$n.qtl
    if(missing(formula)) {
        tmp.Q <- paste("Q", 1:n.qtl, sep="") # QTL term names
        formula <- "y~Q1"
        if(n.qtl > 1)
            for (i in 2:n.qtl)
                formula <- paste(formula, tmp.Q[i], sep="+")
        if (n.covar) { # if covarariate is not empty
            tmp.C <- colnames(covar) # covarariate term names
            for(i in 1:n.covar)
                formula <- paste(formula, tmp.C[i], sep="+")
        }
        formula <- as.formula(formula)
    }

    # check input formula
    formula <- checkformula(formula, qtl$altname, colnames(covar))

    # make sure icovar is in the formula
    m <- is.na(match(icovar, rownames(attr(terms(formula), "factors"))))
    if(any(m))
        formula <- as.formula(paste(deparseQTLformula(formula), "+",
                                    paste(icovar[m], collapse="+"), sep=""))

    # look for interactions that haven't been added
    factors <- attr(terms(formula), "factors")
    if(sum(factors[1,])==0) factors <- factors[-1,]

    # replace QTL altnames (Q1 etc) with real names (chr1@20 etc)
    fn <- fn.alt <- rownames(factors)
    qan <- qtl$altname
    qn <- qtl$name
    m <- match(fn, qan)
    fn.alt[!is.na(m)] <- qn[m[!is.na(m)]]

    theqtl <- fn[fn != fn.alt]
    theqtl.alt <- fn.alt[fn != fn.alt]

    theint <- theint.alt <- NULL
    for(i in icovar) {
        theint <- c(theint, paste(theqtl, ":", i, sep=""))
        theint.alt <- c(theint.alt, paste(theqtl.alt, ":", i, sep=""))
    }

    wh <- match(theint, colnames(factors))
    theint <- theint[is.na(wh)]
    theint.alt <- theint.alt[is.na(wh)]

    n2test <- length(theint)

    if(n2test == 0) {
        if(verbose) cat("No QTL x covariate interactions to add.\n")
        return(NULL)
    }

    sexpgm <- getsex(cross)
    cross.attr <- attributes(cross)

    # fit base model
    thefit0 <- fitqtlengine(pheno=pheno, qtl=qtl, covar=covar, formula=formula,
                            method=method, model=model, dropone=FALSE, get.ests=FALSE,
                            run.checks=FALSE, cross.attr=cross.attr,
                            crosstype=crosstype(cross), sexpgm=sexpgm,
                            tol=tol, maxit=maxit)
    matrix0.rank <- attr(thefit0, "matrix.rank")
    matrix0.ncol <- attr(thefit0, "matrix.ncol")

    results <- matrix(ncol=7, nrow=n2test)
    dimnames(results) <- list(theint.alt, c("df", "Type III SS", "LOD", "%var",
                                            "F value", "Pvalue(Chi2)", "Pvalue(F)"))

    matrix1.rank <- matrix1.ncol <- rep(0, n2test)
    for(k in seq(along=theint)) {
        thefit1 <- fitqtlengine(pheno=pheno, qtl=qtl, covar=covar,
                                formula=as.formula(paste(deparseQTLformula(formula), theint[k], sep="+")),
                                method=method, model=model, dropone=FALSE, get.ests=FALSE,
                                run.checks=FALSE, cross.attr=cross.attr,
                                crosstype=crosstype(cross), sexpgm=sexpgm,
                                tol=tol, maxit=maxit)

        results[k,1] <- thefit1$result.full[1,1] - thefit0$result.full[1,1]
        results[k,2] <- thefit1$result.full[1,2] - thefit0$result.full[1,2]
        results[k,3] <- thefit1$result.full[1,4] - thefit0$result.full[1,4]

        results[k,4] <- 100*(1-10^(-2*thefit1$result.full[1,4]/qtl$n.ind)) -
            100*(1-10^(-2*thefit0$result.full[1,4]/qtl$n.ind))

        results[k,5] <- (results[k,2]/results[k,1])/thefit1$result.full[2,3]
        results[k,6] <- pchisq(results[k,3]*2*log(10), results[k,1], lower.tail=FALSE)
        results[k,7] <- pf(results[k,5], results[k,1], thefit1$result.full[3,1], lower.tail=FALSE)
        matrix1.rank[k] <- attr(thefit1, "matrix.rank")
        matrix1.ncol[k] <- attr(thefit1, "matrix.ncol")
    }
    matrix.fullrank <- (matrix1.rank - matrix0.rank == matrix1.ncol - matrix0.ncol)

    results <- as.data.frame(results, stringsAsFactors=TRUE)
    class(results) <- c("addcovarint", "data.frame")
    attr(results, "model") <- model
    attr(results, "method") <- method
    attr(results, "formula") <- deparseQTLformula(formula)
    if(simple) pvalues <- FALSE
    attr(results, "pvalues") <- pvalues
    attr(results, "simple") <- simple

    attr(results, "matrix.fullrank") <- matrix.fullrank
    if(require.fullrank) results[!matrix.fullrank,3] <- 0

    results
}

print.addcovarint <-
    function(x, ...)
{
    meth <- attr(x, "method")
    mod <- attr(x, "model")
    simp <- attr(x, "simple")
    if(is.null(mod)) mod <- "normal"
    if(is.null(meth)) meth <- "unknown"
    if(mod=="binary" || simp) attr(x, "pvalues") <- FALSE
    if(meth=="imp") meth <- "multiple imputation"
    else if(meth=="hk") meth <- "Haley-Knott regression"
    cat("Method:", meth, "\n")
    cat("Model: ", mod, "phenotype\n")

    cat("Model formula:")
    w <- options("width")[[1]]
    printQTLformulanicely(attr(x, "formula"), "                   ", w+5, w)
    cat("\n")

    cat("Add one QTL x covar interaction at a time table:\n")
    cat("--------------------------------------------\n")
    pval <- attr(x, "pvalues")
    if(!is.null(pval) && !pval)
        x <- x[,-ncol(x)+(0:1)]

    if(mod == "binary" || simp) x <- x[,c(1,3,4), drop=FALSE]

    printCoefmat(x, digits=4, cs.ind=1, P.values=pval, has.Pvalue=pval)

    cat("\n")
}

summary.addcovarint <- function(object, ...) object

# end of addqtl.R

#####################################################################
#
# mqmaugment.R
#
# Copyright (c) 2009-2019, Danny Arends
#
# Modified by Pjotr Prins; slight modification by Karl Broman
#
#
# first written Februari 2009
# last modified Dec 2019
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: mqmaugment
#
#
#####################################################################

######################################################################
#
# mqmaugment_on_cofactors: Data Augmentation routine for MQM only using the cofactors, the other markers are filled by fill.geno()
#
######################################################################

mqmaugment_on_cofactors <- function(cross, cofactors, maxaugind=82, minprob=0.1, strategy=c("default","impute","drop"), verbose=FALSE, ...){
    markernames <- as.character(unlist(lapply(pull.map(cross),names)))
    todrop <- markernames[which(cofactors==0)]
    toaugment <- drop.markers(cross,todrop)
    cross <- fill.geno(cross, ...)
    augmented <- mqmaugment(toaugment,maxaugind,minprob,strategy,verbose)
    newgenomatrix <- NULL
    for(marker in markernames){
        if(marker %in% todrop){
            newgenomatrix <- cbind(newgenomatrix,pull.geno(cross)[augmented$mqm[[3]],marker])
        }else{
            newgenomatrix <- cbind(newgenomatrix,pull.geno(augmented)[,marker])
        }
    }
    colnames(newgenomatrix) <- markernames
    augmented$geno <- vector("list",nchr(cross))
    for(chr in 1:length(cross$geno)){
        cat(chr,"\n")
        augmented$geno[[chr]]$map <- cross$geno[[chr]]$map
        augmented$geno[[chr]]$data <- newgenomatrix[,names(cross$geno[[chr]]$map)]
        names(augmented$geno) <- names(cross$geno)
        class(augmented$geno[[chr]]) <- chrtype(cross$geno[[chr]])
    }
    augmented
}

######################################################################
#
# mqmaugment: dataaugmentation routine for MQM
#
######################################################################

mqmaugment <- function(cross,maxaugind=82, minprob=0.1, strategy=c("default","impute","drop"), verbose=FALSE) {
    # ---- check input supplied by user, start a timer
    if(minprob <= 0 || minprob > 1){ stop("Error minprob should be a value between 0 and 1.") }

    starttime <- proc.time()
    maxaug    <- nind(cross) * maxaugind   # maxaug is the maximum of individuals to augment to
    supported <- c("default","impute","drop")
    strategy  <- pmatch(strategy, supported)

    # ---- check for supported crosses and set ctype

    isF2       = 1
    isBC       = 2
    isRIL      = 3

    isAA       = 1
    isAB       = 2
    isH        = 2
    isBB       = 3
    isNOTBB    = 4
    isNOTAA    = 5
    isMISSING  = 9

    crosstype <- crosstype(cross)

    if (crosstype == "f2") {
        ctype = isF2
    }
    else if (crosstype == "bc" || crosstype == "dh" || crosstype=="haploid") {
        ctype = isBC
    }
    else if (crosstype == "riself") {
        ctype = isRIL
    }
    else {
        stop("Currently only F2, BC, and selfed RIL crosses can be analyzed by MQM.")
    }

    if (verbose) cat("INFO: Received a valid cross file type:", crosstype,".\n")

    # ---- Check sex chromosome
    # check whether the X chromosome should be dropped
    # (backcross with one sex should be fine)
    chr_type <- sapply(cross$geno, chrtype)
    # Drop the X chromosome in F2 and related crosses
    if (any(chr_type == "X") && (ctype == isF2 || length(getgenonames(crosstype, "X", "full", getsex(cross), attributes(cross))) != 2)) {
        warning("MQM not yet available for the X chromosome; omitting chr ",
                paste(names(cross$geno)[chr_type == "X"], collapse=" "))
        cross <- subset(cross, chr=(chr_type != "X"))
    }

    # ---- Count
    n.ind <- nind(cross)
    n.chr <- nchr(cross)
    n.aug <- maxaug
    if (verbose) {
        cat("INFO: Number of individuals:",n.ind,".\n")
        cat("INFO: Number of chr:",n.chr,".\n")
    }

    # ---- Genotype
    geno <- pull.geno(cross)
    chr <- rep(1:nchr(cross), nmar(cross))
    dist <- unlist(pull.map(cross))
    # Create a fake phenotype for augmentation
    pheno <- rep(1:n.ind)
    n.mark <- ncol(geno)
    if (verbose) cat("INFO: Number of markers:",n.mark,".\n")

    # Check for NA genotypes and replace them with a 9
    geno[is.na(geno)] <- isMISSING
    if (ctype==isRIL) {
        nH = sum(geno==isH)
        if (nH>0) {
            #warning("RIL dataset contains ", nH," heterozygous genotypes")
            if (any(geno==isBB)) { # have 3/BB's, so replace 2/H's with missing values
                geno[geno==isH] <- isMISSING
                #warning("Removed heterozygous genotypes from RIL set")
            } else {
                #warning("Converting heterozygous genotypes to BB from RIL set")
                geno[geno==isH] <- isBB
            }
        }
    } # end if(RIL)

    # ---- Call data augmentation
    result <- .C("R_mqmaugment",
                 as.integer(geno),
                 as.double(dist),
                 as.double(pheno),
                 augGeno=as.integer(rep(0,n.mark*maxaug)),
                 augPheno=as.double(rep(0,maxaug)),
                 augIND=as.integer(rep(0,maxaugind*n.ind)),
                 nind=as.integer(n.ind),
                 naug=as.integer(n.aug),
                 as.integer(n.mark),
                 as.integer(1),    # 1 phenotype
                 as.integer(maxaug),
                 as.integer(maxaugind),
                 as.double(minprob),
                 as.integer(chr),
                 as.integer(ctype),
                 as.integer(strategy),
                 as.integer(verbose),
                 PACKAGE="qtl")

    n.indold = n.ind
    n.ind = result$nind
    n.aug = result$naug
    markONchr <- 0
    markdone <- 0
    pheno <- NULL
    pnames <- phenames(cross)
    oldpheno <- pull.pheno(cross)
    result$augIND <- result$augIND+1
    for(x in result$augPheno[1:n.aug]){
        if(nphe(cross)>1){
            pheno <- rbind(pheno,oldpheno[x,])
        }else{
            pheno <- c(pheno,oldpheno[x])
        }
    }
    for(c in 1:n.chr){
        #print(paste("Cromosome",c,"\n",sep=""))
        matri <- NULL
        markONchr <- dim(cross$geno[[c]]$data)[2]
        #print(paste("# markers",markONchr,"\n",sep=""))
        for(j in markdone:(markdone+markONchr-1)){
            #print(paste("Start",markdone,":End",(markdone+markONchr-1),"\n",sep=""))
            ind2 <- NULL
            ind2 <- result$augGeno[(1+(j*maxaug)):(n.aug+(j*maxaug))]
            matri <- rbind(matri,ind2)
        }
        matri <- t(matri)
        if(markdone==0){
            colnames(matri) <- colnames(geno)[markdone:(markdone+markONchr)]
        } else {
            #print(paste("Markdone",markdone,"End",(markdone+markONchr-1)))
            colnames(matri) <- colnames(geno)[(markdone+1):(markdone+markONchr)]
        }
        cross$geno[[c]]$data <- matri
        markdone <- (markdone+markONchr)
    }
    if(nphe(cross) > 1){ colnames(pheno) <- colnames(cross$pheno) }   # Add the phenotype names if we have multiple phenotypes
    cross$pheno <- as.data.frame(pheno, stringsAsFactors=TRUE)
    # Store extra information (needed by the MQM algorithm) which individual was which original etc..
    cross$mqm$Nind <- n.ind
    cross$mqm$Naug <- n.aug
    result$augIND <- result$augIND-1
    cross$mqm$augIND <- result$augIND[1:n.aug]
    # ---- RESULTS
    endtime <- proc.time()
    if(n.ind != n.indold){
        if(verbose) warning("SERIOUS WARNING: Dropped ",abs(n.ind - n.indold)," original individuals.\n  Information lost, please increase minprob.")
    }
    if(verbose) cat("INFO: DATA-Augmentation took: ",round((endtime-starttime)[3], digits=3)," seconds\n")

    colnames(cross$pheno) <- pnames # fix up phenotype names problem

    cross  # return cross type
}

# test input/output in mapqtl format

library(qtl)
data(fake.4way)

write.cross(fake.4way, "mapqtl", "fake_4way_mapqtl")

x <- read.cross("mapqtl", "", genfile="fake_4way_mapqtl.loc",
                phefile="fake_4way_mapqtl.qua",
                mapfile="fake_4way_mapqtl_female.map")

x <- replace.map(x, pull.map(fake.4way))

comparecrosses(x, fake.4way)

######################################################################
#
# countXO.R
#
# copyright (c) 2008-2022, Karl W Broman
# last modified Oct, 2022
# first written Feb, 2008
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: countXO
#
######################################################################

######################################################################
#
# countXO: Count number of obligate crossovers for each individual
#          on individual chromosomes or overall
#
# if bychr=TRUE, return matrix with no. obligate crossovers for each
#                individual on each chromosome
#        =FALSE, return vector with total no. crossovers across the
#                selected chromosomes
######################################################################

countXO <-
    function(cross, chr, bychr=FALSE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    # pull out relevant chromosome
    if(!missing(chr)) cross <- subset(cross,chr=chr)
    chr.name <- names(cross$geno)

    type <- crosstype(cross)
    n.ind <- nind(cross)
    n.chr <- nchr(cross)
    nxo <- matrix(0, ncol=n.chr, nrow=n.ind)
    id <- getid(cross)
    if(is.null(id)) id <- 1:n.ind
    dimnames(nxo) <- list(id, chr.name)

    for(i in 1:n.chr) {

        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type=="X") xchr <- TRUE
        else xchr <- FALSE

        # which type of cross is this?
        if(type == "f2" || type=="bcsft") {
            if(!xchr) # autosomal
                func <- "R_countXO_f2"
            else func <- "R_countXO_bc"        # X chromsome
        }
        else if(type == "bc" || type=="riself" || type=="risib" || type=="dh" || type=="haploid") func <- "R_countXO_bc"
        else if(type == "4way") func <- "R_countXO_4way"
        else if(type=="ri4self" || type=="ri4sib" || type=="ri8self" || type=="ri8sib" || type=="bgmagic16") {
            func <- "R_countXO_ril48"
            if(xchr)
                warning("countXO not working properly for the X chromosome for 4- or 8-way RIL.")
        }
        else
            stop("ripple not available for cross ", type)

        # data to be input
        genodat <- cross$geno[[i]]$data
        genodat[is.na(genodat)] <- 0
        n.mar <- ncol(genodat)

        if(n.mar > 1) {
            z <- .C(func,
                    as.integer(n.ind),
                    as.integer(n.mar),
                    as.integer(genodat),
                    oblxo=as.integer(rep(0,n.ind)),
                    PACKAGE="qtl")

            nxo[,i] <- z$oblxo
        }
    }

    if(!bychr) nxo <- apply(nxo, 1, sum)

    nxo
}

# end of countXO.R

######################################################################
# write.cross.qtab.R
#
# copyright (c) 2012-2019, Karl W Broman and Danny Arends
# last modified Dec, 2019
# first written Jul, 2012
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: write.cross.qtab
#       rqtl.to.qtab.* where * = symbols, location, genotypes, phenotypes, founder
#       get.qtlHD.ID, get.indID.for.qtab, getgenonames.for.qtab
#       get.qtab.geno.symbols, get.phenotype.type
#
######################################################################


# write cross in a set of qtab-format files
write.cross.qtab <-
    function(cross, filestem="data", descr, verbose=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(missing(descr)) descr <- paste(deparse(substitute(cross)), "from R/qtl")

    # for now, omit X chromosome
    chr_type <- sapply(cross$geno, chrtype)
    if(any(chr_type == "X")) {
        cross <- subset(cross, chr=names(chr_type)[chr_type != "X"])
        warning("Omitting X chromosome.")
    }

    if(verbose) cat("Writing symbols\n")
    rqtl.to.qtab.symbols(cross, paste(filestem,"_symbols.qtab",sep=""), descr=descr)

    if(verbose) cat("Writing founder file\n")
    rqtl.to.qtab.founder(cross, paste(filestem,"_founder.qtab",sep=""), descr=descr)

    if(verbose) cat("Writing genetic map\n")
    rqtl.to.qtab.location(cross, paste(filestem,"_location.qtab",sep=""), descr=descr)

    if(verbose) cat("Writing genotypes\n")
    rqtl.to.qtab.genotypes(cross, paste(filestem,"_genotypes.qtab",sep=""), descr=descr)

    if(verbose) cat("Writing phenotypes\n")
    rqtl.to.qtab.phenotypes(cross, paste(filestem,"_phenotypes.qtab",sep=""), descr=descr)
}


# version number for qtlHD
get.qtlHD.ID <-
    function(){
        VER <- "0.1"
        ID <- paste("qtlHD-in-", VER, sep="")
        ID
    }

# individual IDs for qtab files
get.indID.for.qtab <-
    function(cross)
{
    id <- getid(cross)
    if(is.null(id)) id <- 1:nind(cross)
    paste("ID_", id, sep="")
}

# genotype codes for qtab files
getgenonames.for.qtab <-
    function(cross)
{
    gnames <- getgenonames(crosstype(cross), "A", "full", getsex(cross), attributes(cross))
    if(crosstype(cross) == "f2") {
        gnames <- c(gnames, paste(gnames[1], "or", gnames[2], sep=""), paste(gnames[2], "or", gnames[3], sep=""))
    }

    c("-", gnames)
}

# qtab genotypes symbols
get.qtab.geno.symbols <-
    function(cross)
{
    crtype <- crosstype(cross)
    if(crtype == "bc") {
        return(c("None", "0,0", "0,1"))
    }
    if(crtype == "riself" || crtype == "risib") {
        return(c("None", "0,0", "1,1"))
    }
    if(crtype == "f2") {
        return(c("None", "0,0", "0,1 1,0", "1,1", "0,0 0,1 1,0", "0,1 1,0 1,1"))
    }
    stop("cross type \"", crtype, "\" not yet supported for qtab.")
}

# write qtab symbols
rqtl.to.qtab.symbols <-
    function(cross, filename="symbols.qtab",descr)
{
    if(missing(descr)) descr <- paste(deparse(substitute(cross)), "from R/qtl")

    cat(file=filename, "# --- ",get.qtlHD.ID()," Symbol ",descr, "\n",sep="");
    cat(file=filename, "# --- Genotype Symbol begin\n", append=TRUE)

    gnames <- getgenonames.for.qtab(cross)
    symbols <- get.qtab.geno.symbols(cross)
    for(i in seq(along=gnames)) {
        cat(file=filename, gnames[i], " as ", symbols[i], "\n", sep="", append=TRUE)
    }

    cat(file=filename, "# --- Genotype Symbol end\n", append=TRUE)
}

# write qtab marker map
rqtl.to.qtab.location <-
    function(cross, filename="locations.qtab", descr)
{
    if(missing(descr)) descr <- paste(deparse(substitute(cross)), "from R/qtl")

    cat(file=filename, "# --- ",get.qtlHD.ID()," Location ", descr, "\n", sep="")
    cat(file=filename, "# --- Data Location begin\n", append=TRUE)
    cat(file=filename, "#\tChr\tPos\n", append=TRUE)
    map <- pull.map(cross, as.table=TRUE)
    map <- cbind(rownames(map), map)
    write.table(map, file=filename, append=TRUE, quote=FALSE, sep="\t", na="-", row.names=FALSE, col.names=FALSE)
    cat(file=filename, "# --- Data Location end\n", append=TRUE)
}

rqtl.to.qtab.genotypes <-
    function(cross, filename="genotypes.qtab", descr)
{
    if(missing(descr)) descr <- paste(deparse(substitute(cross)), "from R/qtl")

    cat(file=filename, "# --- ",get.qtlHD.ID()," Genotype ", descr, "\n", sep="")
    cat(file=filename, "# --- Data Genotype begin\n", append=TRUE)

    # pull out genotypes data; convert to strings
    genotypes <- pull.geno(cross)
    genotypes[is.na(genotypes)] <- 0
    gnames <- getgenonames.for.qtab(cross)
    gstr <- matrix(rep("", prod(dim(genotypes))), ncol=ncol(genotypes))
    for(i in seq(along=gnames))
        gstr[genotypes == (i-1)] <- gnames[i]

    # add column with individual IDs
    id <- get.indID.for.qtab(cross)
    gstr <- cbind(id, gstr)

    # marker names
    cat(file=filename, "#", paste(colnames(genotypes), collapse="\t"), "\n", sep="", append=TRUE)

    # genotypes
    write.table(gstr, file=filename, append=TRUE, quote=FALSE, sep="\t", na="-", row.names=FALSE, col.names=FALSE)

    cat(file=filename, "# --- Data Genotype end\n", append=TRUE)
}

rqtl.to.qtab.phenotypes <-
    function(cross, filename="phenotypes.qtab", descr)
{
    if(missing(descr)) descr <- paste(deparse(substitute(cross)), "from R/qtl")

    cat(file=filename, "# --- ",get.qtlHD.ID()," Phenotype ", descr, "\n", sep="")
    cat(file=filename, "# --- Type Phenotype begin\n", append=TRUE)

    for(phename in colnames(cross$pheno)) {
        cat(file=filename, phename, "\t", get.phenotype.type(cross,phename), "\n", sep="", append=TRUE)
    }

    cat(file=filename, "# --- Type Phenotype end", "\n", sep="", append=TRUE)

    cat(file=filename, "# --- Data Phenotype begin", "\n", sep="", append=TRUE)

    cat(file=filename, "#", paste(colnames(cross$pheno), collapse="\t"), "\n", sep="", append=TRUE)

    # add column with individual IDs
    id <- get.indID.for.qtab(cross)
    phe <- cbind(id, cross$pheno)

    write.table(phe, file=filename, append=TRUE, quote=FALSE, sep="\t", na="-", row.names=FALSE, col.names=FALSE)

    cat(file=filename, "# --- Data Phenotype end", "\n", sep="", append=TRUE)
}

get.phenotype.type <-
    function(cross, phenotype)
{
    if(is.numeric(cross$pheno[,phenotype])) return("Float")
    if(is.character(cross$pheno[,phenotype])) return("Char")
    if(is.factor(cross$pheno[,phenotype])) return("Char")
    return("Float")
}

rqtl.to.qtab.founder <-
    function(cross, filename="founder.qtab", descr)
{
    if(missing(descr)) descr <- paste(deparse(substitute(cross)), "from R/qtl")

    cat(file=filename, "# --- ",get.qtlHD.ID()," Founder ", descr, "\n", sep="")
    cat(file=filename, "# --- Set Founder begin\n", append=TRUE)
    cat(file=filename, "Cross\t", toupper(crosstype(cross)), "\n", sep="", append=TRUE)
    cat(file=filename, "# --- Set Founder end\n", append=TRUE)
}

# end of write.cross.qtab.R

######################################################################
#
# refineqtl.R
#
# copyright (c) 2006-2019, Karl W. Broman
# last modified Dec, 2019
# first written Jun, 2006
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: refineqtl, plotLodProfile
#
######################################################################

######################################################################
# this is like scanqtl, though the positions are all fixed loci;
# it calls scanqtl iteratively, trying to find the best positions for
# each QTL.
#
# the method is like that of that described by Zeng and colleagues
# regarding MIM; each QTL is slid from one end of the chromosome to
# the other, or to the next flanking QTLs, if there are linked ones
#
# maxit is the maximum number of iterations (going through all QTLs
# in each iteration) to perform
######################################################################
refineqtl <-
    function(cross, pheno.col=1, qtl, chr, pos, qtl.name, covar=NULL, formula,
             method=c("imp", "hk"), model=c("normal", "binary"), verbose=TRUE, maxit=10,
             incl.markers=TRUE, keeplodprofile=TRUE, tol=1e-4, maxit.fitqtl=1000,
             forceXcovar=FALSE)
{
    method <- match.arg(method)
    model <- match.arg(model)

    if( !inherits(cross, "cross") )
        stop("The cross argument must be an object of class \"cross\".")

    # allow formula to be a character string
    if(!missing(formula) && is.character(formula))
        formula <- as.formula(formula)

    if(!is.null(covar) && !is.data.frame(covar)) {
        if(is.matrix(covar) && is.numeric(covar))
            covar <- as.data.frame(covar, stringsAsFactors=TRUE)
        else stop("covar should be a data.frame")
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    if(!missing(qtl) && (!missing(chr) || !missing(pos) || !missing(qtl.name)))
        warning("qtl argument is provided, and so chr, pos and qtl.name are ignored.")

    if(missing(qtl) && (missing(chr) || missing(pos)))
        stop("Provide either qtl or both chr and pos.")

    if(!missing(qtl)) {
        chr <- qtl$chr
        pos <- qtl$pos
    }
    else { # chr and pos provided
        if(missing(qtl.name)) {
            if(method=="imp")
                qtl <- makeqtl(cross, chr=chr, pos=pos, what="draws")
            else
                qtl <- makeqtl(cross, chr=chr, pos=pos, what="prob")
        }
        else {
            if(method=="imp")
                qtl <- makeqtl(cross, chr=chr, pos=pos, qtl.name=qtl.name, what="draws")
            else
                qtl <- makeqtl(cross, chr=chr, pos=pos, qtl.name=qtl.name, what="prob")
        }
    }

    if(method=="imp") {
        if(!("geno" %in% names(qtl))) {
            if("prob" %in% names(qtl)) {
                warning("The qtl object doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"draws\".")
        }
    }
    else {
        if(!("prob" %in% names(qtl))) {
            if("geno" %in% names(qtl)) {
                warning("The qtl object doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("The qtl object needs to be created with makeqtl with what=\"prob\".")
        }
    }


    if(!all(chr %in% names(cross$geno)))
        stop("Chr ", paste(unique(chr[!(chr %in% cross$geno)]), sep=" "),
             " not found in cross.")

    if(verbose > 1) scanqtl.verbose <- TRUE
    else scanqtl.verbose <- FALSE

    cross <- subset(cross, chr=as.character(unique(chr))) # pull out just those chromosomes

    # save map in the object
    map <- attr(qtl, "map")

    if(qtl$n.ind != nind(cross)) {
        warning("No. individuals in qtl object doesn't match that in the input cross; re-creating qtl object.")
        if(method=="imp")
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        else
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="prob")
        attr(qtl, "map") <- map
    }
    if(method=="imp" && dim(qtl$geno)[3] != dim(cross$geno[[1]]$draws)[3])  {
        warning("No. imputations in qtl object doesn't match that in the input cross; re-creating qtl object.")
        qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        attr(qtl, "map") <- map
    }

    # minimum distance between pseudomarkers
    if(is.null(map))
        stop("Input qtl object should contain the genetic map.")
    mind <- min(unlist(lapply(map, function(a) { if(is.matrix(a)) a <- a[1,];
        if(length(a) <= 1) return(NULL) # deal with case of a single marker
        min(diff(a)) })))/2
    if(mind <= 0) mind <- 1e-6

    # check phenotypes and covariates; drop ind'ls with missing values
    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("refineqtl can take just one phenotype; only the first will be used")
    }

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }

    if(pheno.col < 1 || pheno.col > nphe(cross))
        stop("pheno.col should be between 1 and ", nphe(cross))
    pheno <- cross$pheno[,pheno.col]
    if(!is.null(covar) && nrow(covar) != length(pheno))
        stop("nrow(covar) != no. individuals in cross.")
    if(!is.null(covar)) phcovar <- cbind(pheno, covar)
    else phcovar <- as.data.frame(pheno, stringsAsFactors=TRUE)
    hasmissing <- apply(phcovar, 1, function(a) any(is.na(a)))
    if(all(hasmissing))
        stop("All individuals are missing phenotypes or covariates.")

    if(any(hasmissing)) {
        origcross <- cross
        origqtl <- qtl

        cross <- subset(cross, ind=!hasmissing)
        pheno <- pheno[!hasmissing]
        if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]

        if(method=="imp")
            qtl$geno <- qtl$geno[!hasmissing,,,drop=FALSE]
        else
            qtl$prob <- lapply(qtl$prob, function(a) a[!hasmissing,,drop=FALSE])

        qtl$n.ind <- sum(!hasmissing)
    }

    # if missing formula, include the additive QTL plus all covariates
    if(missing(formula)) {
        formula <- paste("y ~", paste(qtl$altname, collapse="+"))
        if(!is.null(covar))
            formula <- paste(formula, "+", paste(colnames(covar), collapse="+"))
        formula <- as.formula(formula)
    }

    # drop covariates that are not in the formula
    if(!is.null(covar)) {
        theterms <- rownames(attr(terms(formula), "factors"))
        m <- match(colnames(covar), theterms)
        if(all(is.na(m))) covar <- NULL
        else covar <- covar[,!is.na(m),drop=FALSE]
    }

    formula <- checkformula(formula, qtl$altname, colnames(covar))

    # identify which QTL are in the model formula
    tovary <- sort(parseformula(formula, qtl$altname, colnames(covar))$idx.qtl)
    if(length(tovary) != qtl$n.qtl)
        reducedqtl <- dropfromqtl(qtl, index=(1:qtl$n.qtl)[-tovary])
    else reducedqtl <- qtl

    # if a QTL is missing from the formula, we need to revise the formula, moving
    # everything over, for use in scanqtl
    if(any(1:length(tovary) != tovary)) {
        tempform <- strsplit(deparseQTLformula(formula), " *~ *")[[1]][2]
        terms <- strsplit(tempform, " *\\+ *")[[1]]
        for(j in seq(along=terms)) {
            if(length(grep(":", terms[j])) > 0) { # interaction
                temp <- strsplit(terms[j], " *: *")[[1]]

                for(k in seq(along=temp)) {
                    g <- grep("^[Qq][0-9]+$", temp[k])
                    if(length(g) > 0) {
                        num <- as.numeric(substr(temp[k], 2, nchar(temp[k])))
                        temp[k] <- paste("Q", which(tovary == num), sep="")
                    }
                }
                terms[j] <- paste(temp, collapse=":")
            }
            else {
                g <- grep("^[Qq][0-9]+$", terms[j])
                if(length(g) > 0) {
                    num <- as.numeric(substr(terms[j], 2, nchar(terms[j])))
                    terms[j] <- paste("Q", which(tovary == num), sep="")
                }
            }
        }
        formula <- as.formula(paste("y ~", paste(terms, collapse=" + ")))
    }

    curpos <- pos[tovary]
    chrnam <- chr[tovary]
    if(verbose) cat("pos:", curpos, "\n")
    converged <- FALSE

    oldo <- NULL
    lc <- length(chrnam)

    lastout <- vector("list", length(curpos))
    names(lastout) <- qtl$name[tovary]

    sexpgm <- getsex(cross)
    cross.attr <- attributes(cross)

    for(i in 1:maxit) {
        if(keeplodprofile) # do drop-one analysis
            basefit <- fitqtlengine(pheno=pheno, qtl=reducedqtl, covar=covar, formula=formula,
                                    method=method, model=model, dropone=TRUE, get.ests=FALSE,
                                    run.checks=FALSE, cross.attr=cross.attr,
                                    crosstype=crosstype(cross), sexpgm=sexpgm,
                                    tol=tol, maxit=maxit.fitqtl, forceXcovar=forceXcovar)
        else
            basefit <- fitqtlengine(pheno=pheno, qtl=reducedqtl, covar=covar, formula=formula,
                                    method=method, model=model, dropone=FALSE, get.ests=FALSE,
                                    run.checks=FALSE, cross.attr=cross.attr,
                                    crosstype=crosstype(cross), sexpgm=sexpgm,
                                    tol=tol, maxit=maxit.fitqtl, forceXcovar=forceXcovar)

        if(i==1) {
            origlod <- curlod <- thisitlod <- basefit$result.full[1,4]
            origpos <- curpos
        }

        if(verbose) cat("Iteration", i, "\n")
        o <- sample(lc)

        # make sure the first here was not the last before
        if(!is.null(oldo))
            while(o[1] != oldo[lc])
                o <- sample(lc)
        oldo <- o

        newpos <- curpos
        for(j in o) {

            otherchr <- chrnam[-j]
            otherpos <- newpos[-j]

            thispos <- as.list(newpos)
            if(any(otherchr == chrnam[j])) { # linked QTLs
                linkedpos <- otherpos[otherchr==chr[j]]
                if(any(linkedpos < newpos[j]))
                    low <- max(linkedpos[linkedpos < newpos[j]])
                else low <- -Inf
                if(any(linkedpos > newpos[j]))
                    high <- min(linkedpos[linkedpos > newpos[j]])
                else high <- Inf

                thispos[[j]] <- c(low, high)
            }
            else
                thispos[[j]] <- c(-Inf, Inf)

            out <- scanqtl(cross=cross, pheno.col=pheno.col, chr=chrnam, pos=thispos,
                           covar=covar, formula=formula, method=method, model=model,
                           incl.markers=incl.markers,
                           verbose=scanqtl.verbose, tol=tol, maxit=maxit.fitqtl,
                           forceXcovar=forceXcovar)

            lastout[[j]] <- out

            newpos[j] <- as.numeric(strsplit(names(out)[!is.na(out) & out==max(out,na.rm=TRUE)],"@")[[1]][2])

            if(verbose) {
                cat(" Q", j, " pos: ", curpos[j], " -> ", newpos[j], "\n", sep="")
                cat("    LOD increase: ", round(max(out, na.rm=TRUE) - curlod, 3), "\n")
            }
            curlod <- max(out, na.rm=TRUE)
        }

        if(verbose) {
            cat("all pos:", curpos, "->", newpos, "\n")
            cat("LOD increase at this iteration: ", round(curlod - thisitlod, 3), "\n")
        }
        thisitlod <- curlod

        if(max(abs(curpos - newpos)) < mind) {
            converged <- TRUE
            break
        }
        curpos <- newpos

        reducedqtl <- replaceqtl(cross, reducedqtl, seq(length(curpos)),
                                 reducedqtl$chr, curpos, reducedqtl$name)
    }

    if(verbose) {
        cat("overall pos:", origpos, "->", newpos, "\n")
        cat("LOD increase overall: ", round(curlod - origlod, 3), "\n")
    }

    if(!converged) warning("Didn't converge.")

    # do the qtl have custom names?
    g <- grep("^.+@[0-9\\.]+$", qtl$name)
    if(length(g) == length(qtl$name)) thenames <- NULL
    else thenames <- qtl$name

    if(any(hasmissing)) {
        qtl <- origqtl
        cross <- origcross
    }
    for(j in seq(along=tovary))
        qtl <- replaceqtl(cross, qtl, tovary[j], chrnam[j], newpos[j])

    if(!is.null(thenames)) qtl$name <- thenames

    if(keeplodprofile) {
        # subtract off the results from the drop-one analysis from the LOD profiles
        dropresult <- basefit$result.drop
        if(is.null(dropresult)) {
            if(length(lastout)==1) {
                dropresult <- rbind(c(NA,NA, basefit$result.full[1,4]))
                rownames(dropresult) <- names(lastout)
            }
            else
                stop("There's a problem: need dropresult, but didn't obtain one.")
        }

        rn <- rownames(dropresult)
        qn <- names(lastout)

        for(i in seq(along=lastout)) {
            if(sum(rn==qn[i])>1) # ack! multiple QTL at same position
                warning("Multiple QTL at the same location.")
            lastout[[i]] <- lastout[[i]] - (max(lastout[[i]], na.rm=TRUE) - max(dropresult[rn==qn[i],3]))
            pos <- as.numeric(matrix(unlist(strsplit(names(lastout[[i]]), "@")),byrow=TRUE,ncol=2)[,2])
            chr <- rep(qtl$chr[tovary][i], length(pos))
            lastout[[i]] <- data.frame(chr=chr, pos=pos, lod=as.numeric(lastout[[i]]), stringsAsFactors=TRUE)
        }
        names(lastout) <- qtl$name[tovary]

        # make the profiles scanone objects
        for(i in seq(along=lastout)) {
            class(lastout[[i]]) <- c("scanone", "data.frame")
            thechr <- qtl$chr[i]
            if(method=="imp")
                detailedmap <- attr(cross$geno[[thechr]]$draws,"map")
            else
                detailedmap <- attr(cross$geno[[thechr]]$prob,"map")

            if(is.matrix(detailedmap)) detailedmap <- detailedmap[1,]

            r <- range(lastout[[i]][,2])+c(-1e-5, 1e-5)
            rn <- names(detailedmap)[detailedmap>=r[1] & detailedmap<=r[2]]
            o <- grep("^loc-*[0-9]+",rn)
            if(length(o) > 0) # inter-marker locations cited as "c*.loc*"
                rn[o] <- paste("c",thechr,".",rn[o],sep="")
            #      if(length(rn) != nrow(lastout[[i]])) return(list(lastout[[i]], rn, detailedmap))
            if(length(rn) == nrow(lastout[[i]])) rownames(lastout[[i]]) <- rn
        }

        attr(qtl, "lodprofile") <- lastout
    }

    # if there's a pLOD attribute, revise it
    if("pLOD" %in% names(attributes(qtl)) && curlod > origlod)
        attr(qtl,"pLOD") <- attr(qtl,"pLOD") + curlod - origlod

    qtl
}

######################################################################
# plotLodProfile
#
# This is for creating a plot of 1-d LOD profiles calculated within
# refineqtl.
######################################################################

plotLodProfile <-
    function(qtl, chr, incl.markers=TRUE, gap=25, lwd=2, lty=1, col="black",
             qtl.labels=TRUE, mtick=c("line", "triangle"),
             show.marker.names=FALSE, alternate.chrid=FALSE, add=FALSE,
             showallchr=FALSE, labelsep=5, ...)
{
    if(!inherits(qtl, "qtl"))
        stop("Input qtl is not a qtl object")

    if(nqtl(qtl) == 0)
        stop("Null QTL model; there are no profiles to plot")

    lodprof <- attr(qtl, "lodprofile")
    if(is.null(lodprof))
        stop("You must first run refineqtl, using keeplodprofile=TRUE")

    m <- match(qtl$name, names(lodprof))
    if(any(is.na(m)))
        qtl <- dropfromqtl(qtl, index=which(is.na(m)), drop.lod.profile=FALSE)

    # reorder qtl by position
    if(qtl$n.qtl > 1) {
        chrindex <- match(qtl$chr, names(attr(qtl, "map")))
        if(any(is.na(chrindex)))
            stop("Cannot find chr ", qtl$chr[is.na(chrindex)])
        neworder <- order(chrindex, qtl$pos)
        if(any(neworder != seq(qtl$n.qtl))) {
            qtl <- reorderqtl(qtl, neworder)
            lodprof <- attr(qtl, "lodprofile")
        }
    }

    n.qtl <- length(lodprof)
    if(length(lwd) == 1) lwd <- rep(lwd, n.qtl)
    else {
        if(length(lwd) != n.qtl) {
            warning("lwd should have length 1 or ", n.qtl)
            lwd <- rep(lwd[1], n.qtl)
        }
        else lwd <- lwd[neworder]
    }


    if(length(lty) == 1) lty <- rep(lty, n.qtl)
    else {
        if(length(lty) != n.qtl) {
            warning("lty should have length 1 or ", n.qtl)
            lty <- rep(lty[1], n.qtl)
        }
        else lty <- lty[neworder]
    }
    if(length(col) == 1) col <- rep(col, n.qtl)
    else {
        if(length(col) != n.qtl) {
            warning("col should have length 1 or ", n.qtl)
            if(length(col) < n.qtl)
                col <- rep(col, n.qtl)[1:n.qtl]
            else
                col <- col[1:n.qtl]
        }
        else col <- col[neworder]
    }

    map <- attr(qtl, "map")
    if(is.null(map))
        stop("Input qtl object should contain the genetic map.")

    thechr <- unique(qtl$chr)
    orderedchr <- names(map)
    if(showallchr) chr2keep <- seq(along=orderedchr)
    else chr2keep <- which(!is.na(match(orderedchr, thechr)))

    tempscan <- NULL
    for(i in chr2keep) {
        temp <- data.frame(chr=orderedchr[i],
                           pos=as.numeric(map[[i]]),
                           lod=NA, stringsAsFactors=TRUE)
        rownames(temp) <- names(map[[i]])
        tempscan <- rbind(tempscan, temp)
    }
    class(tempscan) <- c("scanone", "data.frame")

    if(missing(chr)) {
        if(showallchr) chr <- orderedchr
        else chr <- thechr
    }

    dontskip <- which(!is.na(match(qtl$chr, chr)))
    if(length(dontskip)==0)
        stop("Nothing to plot.")

    ymax <- max(sapply(lodprof[dontskip], function(a) max(a[,3], na.rm=TRUE)))
    begend <- matrix(unlist(tapply(tempscan[,2],tempscan[,1],range)),ncol=2,byrow=TRUE)
    rownames(begend) <- unique(tempscan[,1])
    begend <- begend[as.character(chr),,drop=FALSE]
    len <- begend[,2]-begend[,1]
    if(length(chr)==1) start <- 0
    else start <- c(0,cumsum(len+gap))-c(begend[,1],0)
    names(start) <- chr

    dots <- list(...)

    if(!add) {
        if("ylim" %in% names(dots)) {
            plot.scanone(tempscan, chr=chr, incl.markers=incl.markers, gap=gap,
                         mtick=mtick, show.marker.names=show.marker.names,
                         alternate.chrid=alternate.chrid, type="n", ...)
        }
        else {
            if(qtl.labels)
                ylim <- c(0, ymax+1)
            else
                ylim <- c(0, ymax)

            plot.scanone(tempscan, chr=chr, incl.markers=incl.markers, gap=gap,
                         mtick=mtick, show.marker.names=show.marker.names,
                         alternate.chrid=alternate.chrid, type="n", ylim=ylim,
                         ...)
        }
    }

    for(i in dontskip) {
        temp <- rbind(tempscan[tempscan[,1] != qtl$chr[i] |
                               (tempscan[,1] == qtl$chr[i] & (tempscan[,2] < min(lodprof[[i]][,2]) |
                                        tempscan[,2] > max(lodprof[[i]][,2], na.rm=TRUE))),],
                      lodprof[[i]])

        temp <- temp[order(match(temp[,1], orderedchr), temp[,2]),]
        class(temp) <- c("scanone", "data.frame")

        plot.scanone(temp, chr=chr, incl.markers=FALSE, gap=gap, add=TRUE,
                     col=col[i], lwd=lwd[i], lty=lty[i], ...)

        if(qtl.labels) {
            maxlod <- max(temp[,3], na.rm=TRUE)
            maxpos <- median(temp[!is.na(temp[,3]) & temp[,3]==maxlod,2] + start[qtl$chr[i]])
            d <- diff(par("usr")[3:4]*labelsep/100)

            text(maxpos, maxlod + d, names(lodprof)[i], col=col[i], font=(lwd[i]>1)+1, ...)
        }
    }

    invisible()
}

# end of refineqtl.R

######################################################################
#
# errorlod.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Apr, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: calc.errorlod, plotErrorlod, top.errorlod
#
######################################################################

######################################################################
#
# calc.errorlod: Calculate LOD scores indicating likely genotyping
#                errors.
#
######################################################################

calc.errorlod <-
    function(cross, error.prob=0.01,
             map.function=c("haldane","kosambi","c-f","morgan"),
             version=c("new","old"))
{
    version <- match.arg(version)

    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    origcross <- cross

    # don't let error.prob be exactly zero (or >1)
    if(error.prob < 1e-50) error.prob <- 1e-50
    if(error.prob > 0.5) {
        error.prob <- 0.5
        warning("error.prob shouldn't be > 0.5.")
    }

    # map function
    map.function <- match.arg(map.function)

    n.ind <- nind(cross)
    n.chr <- nchr(cross)
    n.mar <- nmar(cross)
    type <- crosstype(cross)

    # calculate genotype probabilities one chromosome at a time
    for(i in 1:n.chr) {

        chr.type <- chrtype(cross$geno[[i]])
        if(type=="bc" || type=="risib" || type=="riself" || type=="dh" || type=="haploid")
            cfunc <- "calc_errorlod_bc"
        else if(type=="f2" || type=="bcsft") {
            if(chr.type!="X") cfunc <- "calc_errorlod_f2"
            else cfunc <- "calc_errorlod_bc"
        }
        else if(type=="4way") cfunc <- "calc_errorlod_4way"
        else if(type=="ri4self" || type=="ri4sib" || type=="ri8self" || type=="ri8sib" || type=="bgmagic16")
            cfunc <- paste("calc_errorlod_", type, sep="")
        else
            stop("calc.errorlod not available for cross type ", type, ".")

        # skip chromosomes with only 1 marker
        if(n.mar[i] < 2) next

        if(version=="old") {
            if((!("prob" %in% names(cross$geno[[i]])) ||
                abs(attr(cross$geno[[i]]$prob,"error.prob")
                    - error.prob) > 1e-9)) {
                # need to run calc.genoprob
                cross <- calc.genoprob(cross,error.prob=error.prob,
                                       map.function=map.function)
            }

            Pr <- cross$geno[[i]]$prob
            u <- grep("^loc-*[0-9]+",colnames(Pr))
            if(length(u) > 0) Pr <- Pr[,-u,]
        }
        else { # new version
            cross <- calc.genoprob.special(cross,error.prob=error.prob,
                                           map.function=map.function)
            Pr <- cross$geno[[i]]$prob
        }

        nm <- dim(Pr)[2]
        dat <- cross$geno[[i]]$data
        dat[is.na(dat)] <- 0

        z <- .C(cfunc,
                as.integer(n.ind),
                as.integer(nm),
                as.integer(dat),
                as.double(error.prob),
                as.double(Pr),
                errlod=as.double(rep(0,n.ind*nm)),
                PACKAGE="qtl")

        errlod <- array(z$errlod, dim=dim(Pr)[1:2])
        if(version=="new") errlod[dat==0] <- -99

        dimnames(errlod) <- list(NULL,colnames(cross$geno[[i]]$data))
        origcross$geno[[i]]$errorlod <- errlod

        # attribute set to the error.prob value used, for later
        #     reference.
        attr(origcross$geno[[i]]$errorlod,"error.prob") <- error.prob
        attr(origcross$geno[[i]]$errorlod,"map.function") <- map.function
    }

    origcross
}





######################################################################
#
# plotErrorlod
#
######################################################################

plotErrorlod <-
    function(x, chr, ind, breaks=c(-Inf,2,3,4.5,Inf),
             col=c("white","gray85","hotpink","purple3"),
             alternate.chrid=FALSE, ...)
{
    if(!inherits(x, "cross"))
        stop("Input should have class \"cross\".")

    if(length(breaks) != length(col)+1)
        stop("Length of breaks should be length(col)+1.")
    if(length(breaks) != length(col)+1)
        col <- col[1:(length(breaks)+1)]

    cross <- x
    if(!missing(chr)) cross <- subset(cross,chr=chr)

    use.id <- FALSE
    if(!missing(ind)) {
        if(is.null(getid(cross))) cross$pheno$id <- 1:nind(cross)
        cross <- subset(cross,ind=ind)
        use.id <- TRUE
    }

    # remove chromosomes with < 2 markers
    n.mar <- nmar(cross)
    cross <- subset(cross,chr=names(n.mar)[n.mar >= 2])
    n.chr <- nchr(cross)

    errlod <- NULL
    for(i in 1:n.chr) {
        if(!("errorlod" %in% names(cross$geno[[i]]))) { # need to run calc.errorlod
            warning("First running calc.errorlod.")
            cross <- calc.errorlod(cross,error.prob=0.01,map.function="haldane")
        }
        errlod <- cbind(errlod,cross$geno[[i]]$errorlod)
    }

    errlod <- t(errlod)

    old.xpd <- par("xpd")
    old.las <- par("las")
    par(xpd=TRUE,las=1)
    on.exit(par(xpd=old.xpd,las=old.las))

    # plot grid
    breaks[breaks==Inf] <- max(errlod)
    breaks[breaks==-Inf] <- min(errlod)
    image(1:nrow(errlod),1:ncol(errlod),errlod,
          ylab="Individuals",xlab="Markers",col=col,
          breaks=breaks, yaxt="n")
    if(use.id)
        axis(side=2, at=1:nind(cross), labels=getid(cross))
    else axis(side=2)

    # plot lines at the chromosome boundaries
    n.mar <- nmar(cross)
    n.chr <- nchr(cross)
    chr.names <- names(cross$geno)
    a <- c(0.5,cumsum(n.mar)+0.5)

    # the following makes the lines go slightly above the plotting region
    b <- par("usr")
    segments(a,b[3],a,b[4]+diff(b[3:4])*0.02)

    # this line adds a line above and below the image
    #     (the image function seems to leave these out)
    abline(h=0.5+c(0,ncol(errlod)),xpd=FALSE)

    # add chromosome numbers
    a <- par("usr")
    wh <- cumsum(c(0.5,n.mar))
    chrpos <- (wh[-1]+wh[-length(wh)])/2

    if(!alternate.chrid || length(chrpos) < 2) {
        for(i in seq(along=chrpos))
            axis(side=3, at=chrpos[i], labels=chr.names[i])
    }
    else {
        odd <- seq(1, length(chrpos), by=2)
        even <- seq(2, length(chrpos), by=2)
        for(i in odd) {
            axis(side=3, at=chrpos[i], labels="")
            axis(side=3, at=chrpos[i], labels=chr.names[i], line=-0.4, tick=FALSE)
        }
        for(i in even) {
            axis(side=3, at=chrpos[i], labels="")
            axis(side=3, at=chrpos[i], labels=chr.names[i], line=+0.4, tick=FALSE)
        }
    }

    title(main="Genotyping error LOD scores")

    invisible()
}


######################################################################
#
# top.errorlod
#
# Picks out the genotypes having errorlod values above some cutoff
#
######################################################################

top.errorlod <-
    function(cross, chr, cutoff=4, msg=TRUE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(!missing(chr)) cross <- subset(cross,chr=chr)

    id <- getid(cross)
    if(is.null(id)) id <- 1:nind(cross)

    mar <- ind <- lod <- chr <- NULL

    # remove chromosomes with < 2 markers
    n.mar <- nmar(cross)
    cross <- subset(cross,chr=names(n.mar)[n.mar >= 2])

    flag <- 0
    for(i in 1:nchr(cross)) {

        if(!("errorlod" %in% names(cross$geno[[i]])))
            stop("You first need to run calc.errorlod.")

        el <- cross$geno[[i]]$errorlod

        if(any(el > cutoff)) {
            o <- (el > cutoff)
            mar <- c(mar,colnames(el)[col(el)[o]])
            ind <- c(ind,as.character(id[row(el)][o]))
            lod <- c(lod,el[o])
            chr <- c(chr,rep(names(cross$geno)[i],sum(o)))
            flag <- 1
        }
    }
    if(!flag) {
        if(msg) cat("\tNo errorlods above cutoff.\n")
        return(invisible(NULL))
    }

    suppressWarnings(asnum <- as.numeric(ind))
    if(!any(is.na(asnum)) && all(ind==asnum)) ind <- asnum

    o <- data.frame(chr=chr,id=ind,marker=mar,errorlod=lod,stringsAsFactors=FALSE)[order(-lod,ind),]
    rownames(o) <- 1:nrow(o)
    o
}



# end of errorlod.R

######################################################################
#
# read.cross.mq.R
#
# copyright (c) 2014, INRA (author: Timothee Flutre)
#                     (Some revisions by Karl Broman)
# last modified Dec, 2018
# first written May, 2014
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 3, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but without any warranty; without even the implied warranty of
# merchantability or fitness for a particular purpose. See the GNU
# General Public License, version 3, for more details.
#
# A copy of the GNU General Public License, version 3, is available
# at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: read.cross.mq, read.cross.mq.loc, read.cross.mq.map,
#           read.cross.mq.qua, mq.rmv.comment
# [See read.cross.R for the main read.cross function.]
#
######################################################################

######################################################################
#
# read.cross.mq: read data from an experimental cross in MapQTL (and
# JoinMap) format.
#
# We need three files: a "loc" file containing the genotype data, a
# "map" file containing the linkage group assignments and map
# positions, and a "qua" file containing the phenotypes.
#
# File formats are described in the MapQTL manual available online at
# http://www.kyazma.nl/docs/MQ6Manual.pdf
# For the loc-file, each marker should be on a single line.
# Only 4-way crosses are supported ("CP" type in MapQTL/JoinMap).
#
######################################################################

read.cross.mq <-
function(dir, locfile, mapfile, quafile)
{
    if(! missing(dir) && dir != "") {
        if(length(grep("/$", dir)) > 0) # strip off ending /
            dir <- substr(dir, 1, nchar(dir)-1)

        locfile <- file.path(dir, locfile)
        mapfile <- file.path(dir, mapfile)
        quafile <- file.path(dir, quafile)
    }

    loc <- read.cross.mq.loc(locfile)
    map <- read.cross.mq.map(mapfile)
    pheno <- read.cross.mq.qua(quafile)

    type <- loc$pop.type # only "4way" for the moment
    n.ind <- nrow(loc$genotypes)
    n.mar <- ncol(loc$genotypes)
    n.phe <- ncol(pheno)
    if(nrow(pheno) < n.ind){
        msg <- paste("qua-file should have at least the same number of",
                     " individuals than loc-file")
        stop(msg, call.=FALSE)
    }
    if(nrow(pheno) > n.ind){
        pheno <- pheno[-((n.ind+1):nrow(pheno)),]
    }

    cat(" --Read the following data:\n")
    cat("\tNumber of individuals: ", n.ind, "\n")
    cat("\tNumber of markers: ", n.mar, "\n")
    cat("\tNumber of phenotypes: ", n.phe, "\n")

    geno <- list()
    for(chr in levels(map$chr)){
        geno[[chr]] <- list(data=loc$genotypes[,map$marker[map$chr == chr]],
                            map=map$pos[map$chr == chr])
        names(geno[[chr]]$map) <- map$marker[map$chr == chr]
        if(type == "4way")
            geno[[chr]]$map <- rbind(geno[[chr]]$map,
                                     map$pos[map$chr == chr])
        if(chr %in% c("x", "X")){
            class(geno[[chr]]) <- "X"
        } else
            class(geno[[chr]]) <- "A"
    }

    cross <- list(geno=geno, pheno=pheno)
    class(cross) <- c(type, "cross")
    list(cross, FALSE) # FALSE for "don't estimate map"
}

mq_grab_param <-
    function(lines, param, longname, filetype)
{
    # stuff for error message
    if(missing(filetype)) filetype <- ""
    else filetype <- paste(" in", filetype, "file")
    if(missing(longname)) longname <- param

    g <- grep(param, lines)
    if(length(g) == 0)
        stop("Cannot find ", longname, " in ", filetype)

    # remove white space
    line <- gsub("\\s+", "", lines[g])

    result <- strsplit(line, "=")[[1]][2]

    return(list(result, g)) # g is the line number
}


## each marker should be on a single line
## only 4-way crosses (CP type) are handled
read.cross.mq.loc <-
function(locfile){
    pop.name <- NULL
    pop.type <- NULL
    nb.loci <- NULL
    nb.inds <- NULL
    seg <- NULL
    phase <- NULL
    classif <- NULL
    genotypes <- NULL

    lines <- readLines(locfile, warn=FALSE)

    # drop comments
    lines <- vapply(strsplit(lines, ";"), "[", "", 1)

    # drop empty lines
    lines <- lines[!is.na(lines)]
    blank <- grep("^\\s*$", lines)
    if(length(blank) > 0)
        lines <- lines[-blank]

    ## extract the population name
    res <- mq_grab_param(lines, "name", "population name", "loc")
    pop.name <- res[[1]]
    todrop <- res[[2]]

    ## extract the population type
    res <- mq_grab_param(lines, "popt", "population type", "loc")
    pop.type <- res[[1]]
    todrop <- c(todrop, res[[2]])
    pop.types <- c("BC1","F2","RIx","DH","DH1","DH2","HAP","HAP1","CP","BCpxFy",
                   "IMxFy")
    if(! pop.type %in% pop.types){
        msg <- paste("unknown population type", pop.type)
        stop(msg, call.=FALSE)
    }
    if(pop.type == "CP"){
        pop.type <- "4way"
    } else{
        msg <- paste("population type", pop.type,
                     "is not supported (yet)")
        stop(msg, call.=FALSE)
    }

    ## extract the number of loci
    res <- mq_grab_param(lines, "nloc", "Number of loci", "loc")
    nb.loci <- as.numeric(res[[1]])
    todrop <- c(todrop, res[[2]])
    seg <- rep(NA, nb.loci)
    phase <- rep(NA, nb.loci)
    classif <- rep(NA, nb.loci)

    res <- mq_grab_param(lines, "nind", "Number of individuals", "loc")
    nb.inds <- as.numeric(res[[1]])
    todrop <- c(todrop, res[[2]])
    genotypes <- matrix(NA, nrow=nb.loci, ncol=nb.inds)
    rownames(genotypes) <- paste("loc", 1:nb.loci, sep=".")
    ## colnames(genotypes) <- paste("ind", 1:nb.inds, sep=".")

    lines <- lines[-todrop]

    spl <- strsplit(lines, "\\s+")
    if(length(spl) != nb.loci)
      stop("nloc=", nb.loci, " but genotypes are found at ", length(spl), " markers")
    spl.lengths <- vapply(spl, length, 1)
    if(any(spl.lengths > nb.inds+4))
        stop("lines should have no more than ", nb.inds+4, " columns\n",
             "Problems in lines", seq(along=spl.lengths)[spl.lengths > nb.inds+4])

    rn <- vapply(spl, "[", "", 1)
    if(nrow(genotypes) != length(rn))
        stop(paste0("nloc = ", nrow(genotypes), ", but .loc file contains ", length(rn), " genotype rows"))
    rownames(genotypes) <- rn

    if(length(spl) > nb.loci + 1){
        msg <- paste("there seems to be more loci (", locus.id-1,
                             ") than indicated in the header (", nb.loci, ")")
        stop(msg, call.=FALSE)
    }

    nb.fields <- rep(NA, length(lines))
    for(line.id in 1:length(lines)){
        tokens <- spl[[line.id]]
        nb.fields[line.id] <- length(tokens)

        if(length(tokens) > nb.inds + 1){
            for(i in 2:(length(tokens)-nb.inds)){
                if(length(grep(pattern="\\{", x=tokens[i])) > 0){
                    phase[line.id] <- tokens[i]
                } else if(length(grep(pattern="\\(", x=tokens[i])) > 0){
                    classif[line.id] <- tokens[i]
                } else
                    seg[line.id] <- tokens[i] # only for "4way" type
            }
        }
        genotypes[line.id,] <- tokens[(length(tokens)-nb.inds+1):length(tokens)]
    }
    if(length(unique(nb.fields)) > 1)
        stop("some markers have more fields than others")

    genotypes <- t(genotypes) # individuals in rows, markers in columns

    ## convert all missing data to NA
    for(i in 1:length(genotypes)){
        if(grepl(pattern="\\.", x=genotypes[i]))
            genotypes[i] <- gsub(pattern="\\.", replacement="-",
                                 x=genotypes[i])
        if(grepl(pattern="u", x=genotypes[i]))
            genotypes[i] <- gsub(pattern="u", replacement="-",
                                 x=genotypes[i])
    }
    genotypes[genotypes == "--"] <- NA

    ## convert segregation types and genotypes to new MapQtl format
    convert.seg <- FALSE
    new.seg.types <- c("<abxcd>", "<efxeg>", "<hkxhk>", "<lmxll>", "<nnxnp>")
    for(seg.type in unique(seg))
        if(! seg.type %in% new.seg.types){ # e.g. <abxac>
            convert.seg <- TRUE
            break
        }
    if(convert.seg){
        if(pop.type != "4way"){
            msg <- paste("can't convert genotypes from old to new format",
                         "for 4way cross (yet)")
            stop(msg, call.=FALSE)
        }
        for(locus.id in 1:nb.loci){
            if(seg[locus.id] %in% new.seg.types){
                next
            } else if(seg[locus.id] == "<abxac>"){
                seg[locus.id] <- "<efxeg>"
                tmp <- which(genotypes[,locus.id] == "aa")
                genotypes[tmp, locus.id] <- "ee"
                tmp <- which(genotypes[,locus.id] == "ab")
                genotypes[tmp, locus.id] <- "ef"
                tmp <- which(genotypes[,locus.id] == "ac")
                genotypes[tmp, locus.id] <- "eg"
                tmp <- which(genotypes[,locus.id] == "bc")
                genotypes[tmp, locus.id] <- "fg"
            } else if(seg[locus.id] == "<abxab>"){
                seg[locus.id] <- "<hkxhk>"
                tmp <- which(genotypes[,locus.id] == "aa")
                genotypes[tmp, locus.id] <- "hh"
                tmp <- which(genotypes[,locus.id] == "ab")
                genotypes[tmp, locus.id] <- "hk"
                tmp <- which(genotypes[,locus.id] == "bb")
                genotypes[tmp, locus.id] <- "kk"
                tmp <- which(genotypes[,locus.id] == "a-")
                genotypes[tmp, locus.id] <- "h-"
                tmp <- which(genotypes[,locus.id] == "b-")
                genotypes[tmp, locus.id] <- "k-"
            } else if(seg[locus.id] == "<abxaa>"){
                seg[locus.id] <- "<lmxll>"
                tmp <- which(genotypes[,locus.id] == "aa")
                genotypes[tmp, locus.id] <- "ll"
                tmp <- which(genotypes[,locus.id] == "ab")
                genotypes[tmp, locus.id] <- "lm"
            } else if(seg[locus.id] == "<aaxab>"){
                seg[locus.id] <- "<nnxnp>"
                tmp <- which(genotypes[,locus.id] == "aa")
                genotypes[tmp, locus.id] <- "nn"
                tmp <- which(genotypes[,locus.id] == "ab")
                genotypes[tmp, locus.id] <- "np"
            } else{
                msg <- paste("unrecognized segregation type", seg[locus.id],
                             "at locus", locus.id)
                stop(msg, call.=FALSE)
            }
        }
    }

    ## check genotypes
    proper.genotypes <- c(NA, "ac", "ca", "ad", "da", "bc", "cb", "bd", "db",
                          "ee", "ef", "fe", "eg", "ge", "fg", "gf",
                          "hh", "hk", "kh", "kk", "h-", "k-",
                          "ll", "lm", "ml",
                          "nn", "np", "pn")
    for(genotype in unique(genotypes))
        if(! genotype %in% proper.genotypes){
            msg <- paste("unrecognized genotype", genotype)
            stop(msg, call.=FALSE)
        }

    ## replace all "ca" by "ac", etc -> speed-up next step?
    ## TODO

    ## convert genotypes to R/qtl code (mother=AB x father=CD)
    for(locus.id in 1:nb.loci){
        if(phase[locus.id] == "{0-}"){
            if(seg[locus.id] == "<lmxll>"){ # AB=lm ; CD=ll
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] == "ll"){ # AC or AD
                        genotypes[ind.id,locus.id] <- 5
                    } else if(genotypes[ind.id,locus.id] %in% c("lm","ml")){ # BC or BD
                        genotypes[ind.id,locus.id] <- 6
                    }
                }
            } else{
                msg <- paste("unrecognized segregation ", seg[locus.id],
                             "at locus", locus.id, "with phase",
                             phase[locus.id], "(should be <lmxll>)")
                stop(msg, call.=FALSE)
            }
        } else if(phase[locus.id] == "{1-}"){
            if(seg[locus.id] == "<lmxll>"){ # AB=ml ; CD=ll
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] == "ll"){ # BC or BD
                        genotypes[ind.id,locus.id] <- 6
                    } else if(genotypes[ind.id,locus.id] %in% c("lm","ml")){ # AC or AD
                        genotypes[ind.id,locus.id] <- 5
                    }
                }
            } else{
                msg <- paste("unrecognized segregation ", seg[locus.id],
                             "at locus", locus.id, "with phase",
                             phase[locus.id], "(should be <lmxll>)")
                stop(msg, call.=FALSE)
            }
        } else if(phase[locus.id] == "{-0}"){
            if(seg[locus.id] == "<nnxnp>"){ # AB=nn ; CD=np
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] == "nn"){ # AC or BC
                        genotypes[ind.id,locus.id] <- 7
                    } else if(genotypes[ind.id,locus.id] %in% c("np","pn")){ # AD or BD
                        genotypes[ind.id,locus.id] <- 8
                    }
                }
            } else{
                msg <- paste("unrecognized segregation ", seg[locus.id],
                             "at locus", locus.id, "with phase",
                             phase[locus.id], "(should be <nnxnp>)")
                stop(msg, call.=FALSE)
            }
        } else if(phase[locus.id] == "{-1}"){
            if(seg[locus.id] == "<nnxnp>"){ # AB=nn ; CD=pn
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] == "nn"){ # AD or BD
                        genotypes[ind.id,locus.id] <- 8
                    } else if(genotypes[ind.id,locus.id] %in% c("np","pn")){ # AC or BC
                        genotypes[ind.id,locus.id] <- 7
                    }
                }
            } else{
                msg <- paste("unrecognized segregation ", seg[locus.id],
                             "at locus", locus.id, "with phase",
                             phase[locus.id], "(should be <nnxnp>)")
                stop(msg, call.=FALSE)
            }
        } else if(phase[locus.id] == "{00}"){
            if(seg[locus.id] == "<abxcd>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("ac","ca")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("ad","da")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("bc","cb")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("bd","db")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    }
                }
            } else if(seg[locus.id] == "<efxeg>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("ee")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("eg","ge")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("fe","ef")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("fg","gf")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    }
                }
            } else if(seg[locus.id] == "<hkxhk>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("hh")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("hk","kh")){ # AD or BC
                        genotypes[ind.id,locus.id] <- 10
                    } else if(genotypes[ind.id,locus.id] %in% c("kk")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    } else if(genotypes[ind.id,locus.id] %in% c("h-","-h")){ # not BD
                        genotypes[ind.id,locus.id] <- 14
                    } else if(genotypes[ind.id,locus.id] %in% c("k-","-k")){ # not AC
                        genotypes[ind.id,locus.id] <- 11
                    }
                }
            } else{
                msg <- paste("unrecognized segregation ", seg[locus.id],
                             "at locus", locus.id, "with phase",
                             phase[locus.id])
                stop(msg, call.=FALSE)
            }
        } else if(phase[locus.id] == "{01}"){
            if(seg[locus.id] == "<abxcd>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("ad","da")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("ac","ca")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("bd","db")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("bc","cb")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    }
                }
            } else if(seg[locus.id] == "<efxeg>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("eg","ge")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("ee")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("fg","gf")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("fe","ef")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    }
                }
            } else if(seg[locus.id] == "<hkxhk>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("hk","kh")){ # AC or BD
                        genotypes[ind.id,locus.id] <- 9
                    } else if(genotypes[ind.id,locus.id] %in% c("hh")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("kk")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("h-","-h")){ # not BC
                        genotypes[ind.id,locus.id] <- 12
                    } else if(genotypes[ind.id,locus.id] %in% c("k-","-k")){ # not AD
                        genotypes[ind.id,locus.id] <- 13
                    }
                }
            } else{
                msg <- paste("unrecognized segregation ", seg[locus.id],
                             "at locus", locus.id, "with phase",
                             phase[locus.id])
                stop(msg, call.=FALSE)
            }
        } else if(phase[locus.id] == "{10}"){
            if(seg[locus.id] == "<abxcd>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("bc","cb")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("bd","db")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("ac","ca")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("ad","da")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    }
                }
            } else if(seg[locus.id] == "<efxeg>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("fe","ef")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("fg","gf")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("ee")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("eg","ge")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    }
                }
            } else if(seg[locus.id] == "<hkxhk>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("kh","hk")){ # AC or BD
                        genotypes[ind.id,locus.id] <- 9
                    } else if(genotypes[ind.id,locus.id] %in% c("kk")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("hh")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("h-","-h")){ # not AD
                        genotypes[ind.id,locus.id] <- 13
                    } else if(genotypes[ind.id,locus.id] %in% c("k-","-k")){ # not BC
                        genotypes[ind.id,locus.id] <- 12
                    }
                }
            } else{
                msg <- paste("unrecognized segregation ", seg[locus.id],
                             "at locus", locus.id, "with phase",
                             phase[locus.id])
                stop(msg, call.=FALSE)
            }
        } else if(phase[locus.id] == "{11}"){
            if(seg[locus.id] == "<abxcd>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("bd","db")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("bc","cb")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("ad","da")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("ac","ca")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    }
                }
            } else if(seg[locus.id] == "<efxeg>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("fg","gf")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("fe","ef")){ # AD
                        genotypes[ind.id,locus.id] <- 3
                    } else if(genotypes[ind.id,locus.id] %in% c("eg","ge")){ # BC
                        genotypes[ind.id,locus.id] <- 2
                    } else if(genotypes[ind.id,locus.id] %in% c("ee")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    }
                }
            } else if(seg[locus.id] == "<hkxhk>"){
                for(ind.id in 1:nb.inds){
                    if(is.na(genotypes[ind.id,locus.id])){
                        next
                    } else if(genotypes[ind.id,locus.id] %in% c("kk")){ # AC
                        genotypes[ind.id,locus.id] <- 1
                    } else if(genotypes[ind.id,locus.id] %in% c("kh","hk")){ # AD or BC
                        genotypes[ind.id,locus.id] <- 10
                    } else if(genotypes[ind.id,locus.id] %in% c("hh")){ # BD
                        genotypes[ind.id,locus.id] <- 4
                    } else if(genotypes[ind.id,locus.id] %in% c("h-","-h")){ # not AC
                        genotypes[ind.id,locus.id] <- 11
                    } else if(genotypes[ind.id,locus.id] %in% c("k-","-k")){ # not BD
                        genotypes[ind.id,locus.id] <- 14
                    }
                }
            } else{
                msg <- paste("unrecognized segregation ", seg[locus.id],
                             "at locus", locus.id, "with phase",
                             phase[locus.id])
                stop(msg, call.=FALSE)
            }
        } else{
            msg <- paste("unrecognized phase", phase[locus.id],
                         "at locus", locus.id)
            stop(msg, call.=FALSE)
        }
    }
    storage.mode(genotypes) <- "numeric"

    list(pop.name=pop.name, pop.type=pop.type, genotypes=genotypes,
         seg=seg, phase=phase, classif=classif)
}

## returns a data.frame with 3 columns: chr (factor), marker (char), pos (num)
read.cross.mq.map <-
function(mapfile){

    # read all lines
    lines <- readLines(mapfile, warn=FALSE)

    # remove comments
    lines <- vapply(strsplit(lines, ";"), "[", "", 1)

    # drop empty lines
    lines <- lines[!is.na(lines)]
    blank <- grep("^\\s*$", lines)
    if(length(blank) > 0)
        lines <- lines[-blank]

    # find groups
    grouplines <- grep("group", lines)

    # add lg name to end of each line
    for(i in seq(along=grouplines)) {
        # linkage group name
        groupname <- strsplit(lines[grouplines[i]], "\\s+")[[1]]
        groupname <- groupname[length(groupname)]

        first <- grouplines[i]+1
        if(i==length(grouplines))
            last <- length(lines)
        else last <- grouplines[i+1]-1

        lines[first:last] <- paste(lines[first:last], groupname)
    }

    # drop initial lines
    if(grouplines[1] > 1)
        todrop <- 1:(grouplines[1]-1)
    else todrop <- NULL
    # also drop the group lines
    todrop <- c(todrop, grouplines)
    # now the actual dropping
    lines <- lines[-todrop]

    # split at white space
    spl <- strsplit(lines, "\\s+")

    # combine into a data frame
    genmap <- data.frame(chr=vapply(spl, "[", "", 3),
                         marker=vapply(spl, "[", "", 1),
                         pos=as.numeric(vapply(spl, "[", "", 2)),
                         stringsAsFactors=FALSE)

    # make chr as factor
    genmap[,1] <- factor(genmap[,1], levels=unique(genmap[,1]))

    genmap
}

read.cross.mq.qua <-
function(quafile){
    nb.traits <- NULL
    nb.inds <- NULL
    miss <- NULL
    phenotypes <- NULL

    lines <- readLines(quafile, warn=FALSE)

    # remove comments
    lines <- vapply(strsplit(lines, ";"), "[", "", 1)

    # drop empty lines
    lines <- lines[!is.na(lines)]
    blank <- grep("^\\s*$", lines)
    if(length(blank) > 0)
        lines <- lines[-blank]

    ## extract the number of traits
    res <- mq_grab_param(lines, "ntrt", "Number of traits", "qua")
    nb.traits <- as.numeric(res[[1]])
    todrop <- res[[2]]

    ## extract the number of individuals
    res <- mq_grab_param(lines, "nind", "Number of individuals", "qua")
    nb.inds <- as.numeric(res[[1]])
    todrop <- c(todrop, res[[2]])

    ## extract the symbol for missing values
    res <- mq_grab_param(lines, "miss", "Missing value code", "qua")
    miss <- res[[1]]
    todrop <- c(todrop, res[[2]])

    lines <- lines[-todrop]

    spl <- strsplit(lines, "\\s+")

    ind.id <- 1
    trait.id <- 1
    trait.names <- c()
    for(line.id in 1:length(lines)){

        tokens <- spl[[line.id]]

        if(trait.id <= nb.traits){
            if(length(tokens) == 1){ # one trait name per line
                trait.names <- c(trait.names, tokens[1])
                trait.id <- trait.id + 1
            } else{ # all trait names on the same line
                trait.names <- tokens
                trait.id <- nb.traits + 1
            }
            next
        } else if(trait.id > nb.traits && is.null(phenotypes)){
            if(length(trait.names) != nb.traits){
                msg <- paste("there seems to be fewer trait names (",
                             length(trait.names),
                             ") than indicated in the header (",
                             nb.traits, ")", sep="")
                stop(msg, call.=FALSE)
            }
            phenotypes <- matrix(NA, nrow=nb.inds, ncol=nb.traits)
        }

        if(length(tokens) != nb.traits){
            msg <- paste0("line ", line.id, " should have ", nb.traits,
                          " column", ifelse(nb.traits > 1, "s", ""),
                          " separated by spaces or tabs")
            stop(msg, call.=FALSE)
        }
        phenotypes[ind.id,] <- tokens
        ind.id <- ind.id + 1
    }

    phenotypes[which(phenotypes == miss)] <- NA
    phenotypes <- as.data.frame(phenotypes)

    for(j in 1:ncol(phenotypes))
        phenotypes[,j] <- tryCatch(expr=as.numeric(as.character(phenotypes[,j])),
                                   warning=function(w)
                                   as.factor(phenotypes[,j]))
    colnames(phenotypes) <- trait.names

    phenotypes
}

# end of read.cross.mq.R

#####################################################################
#
# pickMarkerSubset.R
#
# copyright (c) 2011-2013, Karl W Broman
# last modified Apr, 2013
# first written Nov, 2011
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
######################################################################

pickMarkerSubset <-
    function(locations, min.distance, weights)
{
    n.loc <- length(locations)
    if(n.loc==1) return(names(locations)) # just one marker

    if(missing(weights))
        weights <- rep(1, n.loc)
    else {
        if(n.loc != length(weights))
            stop("length(locations) != length(weights) [", n.loc, " != ", length(weights), "]")
    }
    if(is.null(names(locations)))
        names(locations) <- 1:n.loc

    if(any(diff(locations) < 0)) {
        o <- order(locations)
        weights <- weights[o]
        locations <- locations[o]
        warning("Markers are not in order; sorting them.")
    }

    z <- .C("R_pickMarkerSubset",
            as.double(locations),
            as.integer(n.loc),
            as.double(weights),
            as.double(min.distance),
            path=as.integer(rep(0, n.loc)),
            n.path=as.integer(0),
            PACKAGE="qtl")

    path <- rev(z$path[1:z$n.path]+1) # reverse and add 1

    return(names(locations)[path])
}

# end of pickMarkerSubset.R

######################################################################
#
# summary.cross.R
#
# copyright (c) 2001-2022, Karl W Broman
# last modified Oct, 2022
# first written Feb, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: summary.cross, print.summary.cross, nind, nchr, nmar,
#           totmar, nphe, nmissing, ntyped, print.cross, chrlen
#
######################################################################

summary.cross <-
    function(object,...)
{
    if(!inherits(object, "cross"))
        stop("Input should have class \"cross\".")

    n.ind <- nind(object)
    tot.mar <- totmar(object)
    n.phe <- nphe(object)
    n.chr <- nchr(object)
    n.mar <- nmar(object)
    type <- crosstype(object)

    if(!(type %in% c("f2", "bc", "4way", "riself", "risib", "dh", "haploid",
                     "ri4self", "ri4sib", "ri8self", "ri8selfIRIP1", "ri8sib", "bcsft", "bgmagic16")))
        stop("Cross type ", type, " is not supported.")

    # combine genotype data into one big matrix
    Geno <- pull.geno(object)

    # A and X-specific genotypes?
    chr_type <- sapply(object$geno, chrtype)
    if(any(chr_type=="A")) {
        GenoA <- pull.geno(object, chr=chrnames(object)[chr_type=="A"])
    } else {
        GenoA <- NULL
    }
    if(any(chr_type=="X") && type %in% c("f2", "bc", "bcsft")) {
        GenoX <- pull.geno(object, chr=chrnames(object)[chr_type=="X"])
        GenoX <- reviseXdata(type, "full", getsex(object), geno=GenoX, cross.attr=attributes(object))
    } else{
        GenoX <- NULL
    }

    # proportion of missing genotype data
    missing.gen <- mean(is.na(Geno))

    # Get cross scheme for BCsFt.
    if(type == "bcsft") {
        cross.scheme <- attr(object, "scheme")
        is.bcs <- (cross.scheme[2] == 0)
    }
    else {
        cross.scheme <- rep(0,2)
        is.bcs <- FALSE
    }

    # table of genotype values
    typings <- typingsA <- typingsX <- NULL
    if(type %in% c("f2", "bcsft") && !is.bcs) {
        if(is.null(GenoX)) {
            typings <- table(factor(Geno[!is.na(Geno)], levels=1:5))
            temp <- getgenonames("f2", "A", cross.attr=attributes(object))
            names(typings) <- c(temp, paste("not", temp[c(3,1)]))
        } else {
            if(!is.null(GenoA)) {
                typingsA <- table(factor(GenoA[!is.na(GenoA)], levels=1:5))
                temp <- getgenonames("f2", "A", cross.attr=attributes(object))
                names(typingsA) <- c(temp, paste("not", temp[c(3,1)]))
            }
            if(!is.null(GenoX)) {
                gnames <- getgenonames("f2", "X", "full", getsex(object), cross.attr=attributes(object))
                typingsX <- table(factor(GenoX[!is.na(GenoX)], levels=1:length(gnames)))
                names(typingsX) <- gnames
            }
        }
    }
    else if(type %in% c("bc", "riself", "risib", "dh", "haploid", "bcsft")) {
        if(is.null(GenoX)) {
            typings <- table(factor(Geno[!is.na(Geno)], levels=1:2))
            temp <- getgenonames(type, "A", cross.attr=attributes(object))
            names(typings) <- temp
        } else {
            if(!is.null(GenoA)) {
                typingsA <- table(factor(GenoA[!is.na(GenoA)], levels=1:2))
                temp <- getgenonames(type, "A", cross.attr=attributes(object))
                names(typingsA) <- temp
            }
            if(!is.null(GenoX)) {
                gnames <- getgenonames(type, "X", "full", getsex(object), cross.attr=attributes(object))
                typingsX <- table(factor(GenoX[!is.na(GenoX)], levels=1:length(gnames)))
                names(typingsX) <- gnames
            }
        }
    }
    else if(type=="4way") {
        typings <- table(factor(Geno[!is.na(Geno)], levels=1:14))

        temp <- getgenonames("4way", "A", cross.attr=attributes(object))
        names(typings) <- c(temp,
                            paste(temp[c(1,3)], collapse="/"),
                            paste(temp[c(2,4)], collapse="/"),
                            paste(temp[c(1,2)], collapse="/"),
                            paste(temp[c(3,4)], collapse="/"),
                            paste(temp[c(1,4)], collapse="/"),
                            paste(temp[c(2,3)], collapse="/"),
                            paste("not", temp[1]),
                            paste("not", temp[2]),
                            paste("not", temp[3]),
                            paste("not", temp[4]))
    }
    else typings <- table(factor(Geno[!is.na(Geno)]))

    # turn into fractions
    if(!is.null(typings)) typings <- typings/sum(typings)
    if(!is.null(typingsA)) typingsA <- typingsA/sum(typingsA)
    if(!is.null(typingsX)) typingsX <- typingsX/sum(typingsX)


    # amount of missing phenotype data
    if(ncol(object$pheno) <= 30)
        missing.phe <- as.numeric(cbind(apply(object$pheno,2,function(a) mean(is.na(a)))))
    else
        missing.phe <- mean(as.numeric(is.na(object$pheno)))

    # check that, in the case of a "4way" cross, the genetic
    #     maps are matrices with 2 rows, and that for other crosses,
    #     the genetic maps are numeric vectors
    if(type=="4way") {
        if(any(!sapply(object$geno, function(a) (is.matrix(a$map) && nrow(a$map)==2))))
            warning("The genetic maps should all be matrices with two rows.")
    }
    else {
        if(any(sapply(object$geno, function(a) is.matrix(a$map))))
            warning("The genetic maps should all be numeric vectors rather than matrices.")
    }

    # check that object$geno[[i]]$data has colnames and that they match
    #     the names in object$geno[[i]]$map
    jitterwarning <- NULL
    for(i in 1:n.chr) {
        nam1 <- colnames(object$geno[[i]]$data)
        map <- object$geno[[i]]$map
        if(is.matrix(map)) nam2 <- colnames(map)
        else nam2 <- names(map)
        chr <- names(object$geno)[[i]]
        if(is.null(nam1)) {
            warn <- paste("The data matrix for chr", chr,
                          "lacks column names")
            warning(warn)
        }
        if(is.null(nam2)) {
            warn <- paste("The genetic map for chr", chr,
                          "lacks column names")
            warning(warn)
        }
        if(any(nam1 != nam2))
            stop("Marker names in the data matrix and genetic map\n",
                 "for chr ", chr, " do not match.")

        if((is.matrix(map) && (any(diff(map[1,])<0) || any(diff(map[2,])<0))) ||
           (!is.matrix(map) && any(diff(map)<0)))
            stop("Markers out of order on chr ", chr)


        # check that no two markers are on top of each other
        if(is.matrix(map)) { # sex-specific maps
            n <- ncol(map)
            if(n > 1) {
                d1 <- diff(map[1,])
                d2 <- diff(map[2,])
                if(any(d1 < 1e-14 & d2 < 1e-14)) {
                    if (is.null(jitterwarning)) jitterwarning<-list()
                    jitterwarning[[names(object$geno)[i]]]<-which(d1 < 1e-14 & d2 < 1e-14)
                }
            }
        }
        else {
            n <- length(map)
            if(n > 1) {
                d <- diff(map)
                if(any(d < 1e-14)) {
                    if (is.null(jitterwarning)) jitterwarning<-list()
                    jitterwarning[[names(object$geno)[i]]]<-which(d < 1e-14)
                }
            }
        }

    }

    if (!is.null(jitterwarning))
        warning("Some markers at the same position on chr ",
                paste(names(jitterwarning),collapse=",",sep=""),"; use jittermap().")

    if(!is.data.frame(object$pheno))
        warning("Phenotypes should be a data.frame.")

    if(is.null(colnames(object$pheno)))
        stop("Phenotype data needs column names")

    x <- table(colnames(object$pheno))

    if(any(x > 1))
        warning("Some phenotypes have the same name:\n",
                paste(names(x)[x>1], collapse="  "))

    # check genotype data
    if(type %in% c("bc", "riself", "risib", "dh", "haploid") | (type == "bcsft" & is.bcs)) {
        # Invalid genotypes?
        if(any(!is.na(Geno) & Geno != 1 & Geno != 2)) {
            u <- unique(as.numeric(Geno))
            u <- sort(u[!is.na(u)])
            warn <- paste("Invalid genotypes.",
                          "\n    Observed genotypes:",
                          paste(u, collapse=" "))
            warning(warn)
        }

        # Missing genotype category on autosomes?
        if(sum(!is.na(Geno) & Geno==2) == 0 ||
           sum(!is.na(Geno) & Geno==1) == 0) {
            warn <- paste("Strange genotype pattern on chr ", chr, ".", sep="")
            warning(warn)
        }
    }
    else if(type %in% c("f2","bcsft") & !is.bcs) {
        # invalid genotypes
        if(any(!is.na(Geno) & Geno!=1 & Geno!=2 & Geno!=3 &
               Geno!=4 & Geno!=5)) {
            u <- unique(as.numeric(Geno))
            u <- sort(u[!is.na(u)])
            warn <- paste("Invalid genotypes on chr", chr, ".",
                          "\n    Observed genotypes:",
                          paste(u, collapse=" "))
            warning(warn)
        }

        # X chromosome
        for(i in 1:n.chr) {
            if(chrtype(object$geno[[i]]) == "X") {
                dat <- object$geno[[i]]$data
                if(any(!is.na(dat) & dat!=1 & dat!=2)) {
                    u <- unique(as.numeric(dat))
                    u <- sort(u[!is.na(u)])
                    warn <- paste("Invalid genotypes on X chromosome:",
                                  "\n    Observed genotypes:",
                                  paste(u, collapse=" "))
                    warning(warn)
                }
            }
        }

        # Missing genotype category on autosomes?
        dat <- NULL; flag <- 0
        for(i in 1:n.chr) {
            if(chrtype(object$geno[[i]]) != "X") {
                dat <- cbind(dat,object$geno[[i]]$data)
                flag <- 1
            }
        }
        if(flag && (sum(!is.na(dat) & dat==2) == 0 ||
                    sum(!is.na(dat) & dat==1) == 0 ||
                    sum(!is.na(dat) & dat==3) == 0))
            warning("Strange genotype pattern.")
    }
    else if(type=="4way") {
        # Invalid genotypes?
        if(any(!is.na(Geno) & (Geno != round(Geno) | Geno < 1 | Geno > 14))) {
            u <- unique(as.numeric(Geno))
            u <- sort(u[!is.na(u)])
            warn <- paste("Invalid genotypes.",
                          "\n    Observed genotypes:",
                          paste(u, collapse=" "))
            warning(warn)
        }
    }
    else if(type %in% c("ri4sib", "ri4self", "ri8sib", "ri8self", "ri8selfIRIP1", "bgmagic16")) {
        if(type=="bgmagic16") n.str <- 16
        else n.str <- as.numeric(substr(type, 3, 3))
        if(any(!is.na(Geno) & (Geno != round(Geno) | Geno < 1 | Geno > 2^n.str-1))) {
            u <- unique(as.numeric(Geno))
            u <- sort(u[!is.na(u)])
            warn <- paste("Invalid genotypes.",
                          "\n    Observed genotypes:",
                          paste(u, collapse=" "))
            warning(warn)
        }
    }

    # Look for duplicate marker names
    mnames <- NULL
    for(i in 1:nchr(object))
        mnames <- c(mnames,colnames(object$geno[[i]]$data))

    o <- table(mnames)
    if(any(o > 1))
        warning("Duplicate markers [", paste(names(o)[o>1], collapse=", "), "]")

    # make sure the genotype data are matrices rather than data frames
    if(any(sapply(object$geno, function(a) is.data.frame(a$data))))
        warning("The $data objects should be simple matrices, not data frames.")

    # make sure each chromosome has class "A" or "X"
    chr.class <- sapply(object$geno, chrtype)
    if(!all(chr.class == "A" | chr.class == "X"))
        warning("Each chromosome should have class \"A\" or \"X\".")
    chr.nam <- names(object$geno)
    if(is.null(chr.nam)) {
        warning("The chromosome names are missing.")
        chr.nam <- as.character(1:length(chr.class))
    }

    if(type != "riself" && any(chr.class=="A" & (chr.nam=="X" | chr.nam=="x"))) {
        wh <- which(chr.nam=="X" | chr.nam=="x")
        warning("Chromosome \"", chr.nam[wh], "\" has class \"A\" but probably ",
                "should have class \"X\".")
    }

    if(length(chr.nam) > length(unique(chr.nam))) {
        tab <- table(chr.nam)
        dups <- names(tab[tab > 1])
        warning("Duplicate chromosome names: ", paste(dups,sep=", "))
    }
    g <- grep("^-", chr.nam)
    if(length(g) > 0)
        warning("Chromosome names shouldn't start with '-': ", paste(chr.nam[g], sep=", "))

    # if more than one X chromosome, print a warning
    if(sum(chr.class=="X") > 1)
        warning("More than one X chromosome: [",
                paste(chr.nam[chr.class == "X"], collapse=", "), "]")

    if(any(chr.class=="A"))
        autosomes <- chr.nam[chr.class == "A"]
    else autosomes <- NULL
    if(any(chr.class=="X"))
        Xchr <- chr.nam[chr.class == "X"]
    else Xchr <- NULL

    # check individual IDs
    id <- getid(object)
    if(!is.null(id) && length(id) != length(unique(id)))
        warning("The individual IDs are not unique.")

    # check that chromosomes aren't too long
    mapsum <- summaryMap(object)
    if(ncol(mapsum)==7)  # sex-specific map
        maxlen <- max(mapsum[1:(nrow(mapsum)-1),2:3])
    else
        maxlen <- max(mapsum[1:(nrow(mapsum)-1),2])
    if(maxlen > 1000)
        warning(paste("Some chromosomes > 1000 cM in length; there may",
                      "be a problem with the genetic map.\n  (Perhaps it is in basepairs?)"))

    cross.summary <- list(type=type, n.ind = n.ind, n.phe=n.phe,
                          n.chr=n.chr, n.mar=n.mar,
                          missing.gen=missing.gen,
                          typing.freq=typings, typing.freq.A=typingsA, typing.freq.X=typingsX,
                          missing.phe=missing.phe,
                          autosomes=autosomes, Xchr=Xchr, cross.scheme=cross.scheme)
    class(cross.summary) <- "summary.cross"
    cross.summary

}


print.summary.cross <-
    function(x,...)
{
    print.genotypes <- TRUE

    if(x$type=="f2") cat("    F2 intercross\n\n")
    else if(x$type=="bc") cat("    Backcross\n\n")
    else if(x$type=="4way") cat("    4-way cross\n\n")
    else if(x$type=="riself") cat("    RI strains via selfing\n\n")
    else if(x$type=="risib") cat("    RI strains via sib matings\n\n")
    else if(x$type=="dh") cat("    Doubled haploids\n\n")
    else if(x$type=="haploid") cat("    Haploids\n\n")
    else if(x$type %in% c("ri4self", "ri4sib", "ri8self", "ri8selfIRIP1", "ri8sib")) {
        n.str <- substr(x$type, 3, 3)
        if(substr(x$type, 4, min(6, nchar(x$type)))=="sib") crosstype <- "sib-mating"
        else crosstype <- "selfing"
        print.genotypes <- FALSE
        cat("    ", n.str, "-way RIL by ", crosstype, "\n\n", sep="")
    }
    else if(x$type=="bcsft") cat(paste("    BC(", x$cross.scheme[1], ")F(", x$cross.scheme[2], ") cross\n\n", sep = ""))
    else if(x$type %in% c("bgmagic16")) {
        n.str <- 16
        print.genotypes <- FALSE
        cat("    ", n.str, "-way Biogemma MAGIC lines\n\n", sep="")
    }
    else cat("    cross", x$type, "\n\n",sep=" ")

    cat("    No. individuals:   ", x$n.ind,"\n\n")
    cat("    No. phenotypes:    ", x$n.phe,"\n")

    header <- "                       "
    width <- options("width")$width

    cat("    Percent phenotyped:")

    ######################################################################
    # function to print things nicely
    printnicely <-
        function(thetext, header, width, sep=" ")
        {
            nleft <- width - nchar(header)
            nsep <- nchar(sep)
            if(length(thetext) < 2) cat("", thetext, "\n", sep=sep)
            else {
                z <- paste("", thetext[1], sep=sep, collapse=sep)
                for(j in 2:length(thetext)) {
                    if(nchar(z) + nsep + nchar(thetext[j]) > nleft) {
                        cat(z, "\n")
                        nleft <- width
                        z <- paste(header, thetext[j], sep=sep)
                    }
                    else {
                        z <- paste(z, thetext[j], sep=sep)
                    }
                }
                cat(z, "\n")
            }
        }
    ######################################################################
    # function to pre-pad with spaces
    pad_w_spaces <-
        function(x, width, pre=TRUE)
        {
            padding <- vapply(nchar(x), function(n) paste(rep(" ", width-n), collapse=""), "")
            if(pre) return(paste0(padding, x))
            paste0(x, padding)
        }
    ######################################################################

    printnicely(round((1-x$missing.phe)*100,1), header, width)
    cat("\n")

    cat("    No. chromosomes:   ", x$n.chr,"\n")
    if(!is.null(x$autosomes)) {
        cat("        Autosomes:     ")
        printnicely(x$autosomes, header, width)
    }
    if(!is.null(x$Xchr)) {
        cat("        X chr:         ")
        printnicely(x$Xchr, header, width)
    }
    cat("\n")
    cat("    Total markers:     ", sum(x$n.mar), "\n")
    cat("    No. markers:       ")
    printnicely(x$n.mar, header, width)
    cat("    Percent genotyped: ", round((1-x$missing.gen)*100,1), "\n")
    if(print.genotypes) {
        cat("    Genotypes (%):    ")
        header <- "                      "
        if(!is.null(x$typing.freq.X)) {
            # contortions to line things up
            roundedX <- sprintf("%.1f", x$typing.freq.X*100)

            genoX <- paste(names(x$typing.freq.X),roundedX,sep=":")
            if(!is.null(x$typing.freq.A)) {
                roundedA <- sprintf("%.1f", x$typing.freq.A*100)
                genoA <- paste(names(x$typing.freq.A),roundedA,sep=":")

                # line up values
                maxchar <- max(nchar(c(roundedA, roundedX)))
                roundedA <- pad_w_spaces(roundedA, maxchar, pre=FALSE)
                roundedX <- pad_w_spaces(roundedX, maxchar, pre=FALSE)
                genoX <- paste(names(x$typing.freq.X),roundedX,sep=":")
                genoA <- paste(names(x$typing.freq.A),roundedA,sep=":")

                # line things up again
                maxchar <- max(nchar(c(genoA, genoX)))
                genoA <- pad_w_spaces(genoA, maxchar)
                genoX <- pad_w_spaces(genoX, maxchar)

                cat("\n          Autosomes:  ")
                printnicely(genoA, header, width, "  ")
                cat("       X chromosome:  ")
            }
            printnicely(genoX, header, width, "  ")
        } else {
            if(!is.null(x$typing.freq.A) && !is.null(x$typing.freq))
                x$typing.freq <- x$typing.freq.A
            geno <- paste(names(x$typing.freq),sprintf("%.1f", x$typing.freq*100), sep=":")
            printnicely(geno, header, width, "  ")
        }
    }
}



nind <-
    function(object)
{
    if(!inherits(object, "cross"))
        stop("Input should have class \"cross\".")

    n.ind1 <- nrow(object$pheno)
    n.ind2 <- sapply(object$geno,function(x) nrow(x$data))
    if(any(n.ind2 != n.ind1))
        stop("Different numbers of individuals in genotypes and phenotypes.")
    n.ind1
}

nchr <-
    function(object)
{
    if(inherits(object, "map")) {
        return(length(object))
    } else if(inherits(object, "cross")) {
        return(length(object$geno))
    } else {
        # neither cross nor map object
        stop("Input should have class \"cross\" or \"map\".")
    }
}

nmar <-
    function(object)
{
    if(inherits(object, "map")) {
        if(is.matrix(object[[1]]))
            return(sapply(object, function(x) ncol(x)))
        else return(sapply(object, function(x) length(x)))
    }
    else if(!inherits(object, "cross")) {
        stop("Input should have class \"cross\" or \"map\".")
    }


    if(length(object$geno) == 0)
        stop("There is no genotype data.")

    if(!is.matrix(object$geno[[1]]$map))
        n.mar1 <- sapply(object$geno, function(x) length(x$map))
    else # sex-specific maps
        n.mar1 <- sapply(object$geno, function(x) ncol(x$map))

    n.mar2 <- sapply(object$geno, function(x) ncol(x$data))
    if(any(n.mar1 != n.mar2))
        stop("Different numbers of markers in genotypes and maps.")
    n.mar1
}

totmar <-
    function(object)
{
    if(inherits(object, "map")) {
        if(is.matrix(object[[1]]))
            return(sum(sapply(object, function(x) ncol(x))))
        else return(sum(sapply(object, function(x) length(x))))
    } else if(!inherits(object, "cross")) {
        stop("Input should have class \"cross\" or \"map\".")
    }

    if(length(object$geno) == 0)
        stop("There is no genotype data.")

    if(!is.matrix(object$geno[[1]]$map))
        totmar1 <- sum(sapply(object$geno, function(x) length(x$map)))
    else # sex-specific maps
        totmar1 <- sum(sapply(object$geno, function(x) ncol(x$map)))
    totmar2 <- sum(sapply(object$geno, function(x) ncol(x$data)))
    if(totmar1 != totmar2)
        stop("Different numbers of markers in genotypes and maps.")
    totmar1
}

nphe <-
    function(object)
{
    if(!inherits(object, "cross"))
        stop("Input should have class \"cross\".")

    ncol(object$pheno)
}

# count number of missing genotypes for each individual or each marker
nmissing <-
    function(cross,what=c("ind","mar"))
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    what <- match.arg(what)

    if(what=="ind") {
        n.missing <- rep(0,nind(cross))
        for(i in 1:nchr(cross))
            n.missing <- n.missing +
                apply(cross$geno[[i]]$data,1,function(a) sum(is.na(a)))

        # individual IDs
        id <- getid(cross)
        if(!is.null(id)) names(n.missing) <- id
    }
    else {
        n.missing <- NULL
        for(i in 1:nchr(cross))
            n.missing <- c(n.missing,
                           apply(cross$geno[[i]]$data,2,function(a) sum(is.na(a))))
    }

    n.missing
}


# like nmissing, but for the opposite value
ntyped <-
    function(cross, what=c("ind","mar"))
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    what <- match.arg(what)

    if(what=="ind") n <- totmar(cross)
    else n <- nind(cross)

    n - nmissing(cross, what)
}

# "print" method for cross object
#
# to avoid ever printing the entire object, print just a little
#     warning message and then the summary
print.cross <-
    function(x, ...)
{
    cat("  This is an object of class \"cross\".\n")
    cat("  It is too complex to print, so we provide just this summary.\n")
    print(summary(x))
    return(summary(x))
}


# get chromosome lengths
chrlen <-
    function(object)
{
    if(!inherits(object, "map") && !inherits(object, "cross"))
        stop("Input should have class \"map\" or \"cross\".")

    if(!inherits(object, "map"))
        x <- pull.map(object)
    else x <- object

    if(is.matrix(x[[1]]))
        return(sapply(x, apply, 1, function(a) diff(range(a))))

    sapply(x, function(a) diff(range(a)))
}


# end of summary.cross.R

#####################################################################
#
# ril48_reorg.R
#
# copyright (c) 2009-2019, Karl W Broman
# last modified Dec, 2019
# first written Apr, 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: reorgRIgenoprob, reorgRIdraws, reorgRIimp, reorgRIpairprob
#
######################################################################

######################################################################
# reorgRIgenoprob
#
# For 4- and 8-way RIL, reorganize the QTL genotype probabilities
# using the information on the order of the founder strains in each
# cross.
######################################################################

reorgRIgenoprob <-
    function(cross)
{
    crosses <- cross$cross
    flag <- 0
    for(i in 1:ncol(crosses)) {
        if(any(crosses[,i] != i)) {
            flag <- 1
            break
        }
    }
    if(!flag) return(cross) # no need to reorder

    crosstype <- crosstype(cross)
    if(crosstype != "ri4sib" && crosstype != "ri4self" &&
       crosstype != "ri8sib" && crosstype != "ri8self" && crosstype != "bgmagic16")
        stop("reorgRIgenoprob not appropriate for cross type ", crosstype)

    if(crosstype=="bgmagic16") n.str <- 16
    else n.str <- as.numeric(substr(crosstype, 3, 3))
    n.ind <- nind(cross)
    for(i in names(cross$geno)) { # loop over chromosomes
        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type == "X")
            warning("reorgRIgenoprob not working properly for the X chromosome.")

        if(!("prob" %in% names(cross$geno[[i]]))) {
            warning("No QTL genotype probabilities within cross.")
            return(cross)
        }

        prob <- cross$geno[[i]]$prob
        att <- attributes(prob)
        n.mar <- dim(prob)[2]
        if(dim(prob)[1] != n.ind)
            stop("Mismatch between no. individuals in cross and in genoprobs.")
        if(dim(crosses)[2] != n.str)
            stop("Invalid no. of founder strains specified")
        if(dim(prob)[3] != n.str) {
            warning("Odd no. columns in genoprobs for chromosome ", i)
            next
        }

        prob <- .C("R_reorgRIgenoprob",
                   as.integer(n.ind),
                   as.integer(n.mar),
                   as.integer(n.str),
                   prob=as.double(prob),
                   as.integer(crosses),
                   PACKAGE="qtl")$prob
        prob <- array(prob, dim=c(n.ind, n.mar, n.str))
        for(j in names(att))
            attr(prob, j) <- att[[j]]
        cross$geno[[i]]$prob <- prob
    }

    cross
}


######################################################################
# reorgRIdraws
#
# For 4- and 8-way RIL, reorganize the imputed QTL genotypes
# using the information on the order of the founder strains in each
# cross.
######################################################################

reorgRIdraws <-
    function(cross)
{
    crosses <- cross$cross
    flag <- 0
    for(i in 1:ncol(crosses)) {
        if(any(crosses[,i] != i)) {
            flag <- 1
            break
        }
    }
    if(!flag) return(cross) # no need to reorder

    crosstype <- crosstype(cross)
    if(crosstype != "ri4sib" && crosstype != "ri4self" &&
       crosstype != "ri8sib" && crosstype != "ri8self" && crosstype != "bgmagic16")
        stop("reorgRIdraws not appropriate for cross type ", crosstype)

    if(crosstype=="bgmagic16") n.str <- 16
    else n.str <- as.numeric(substr(crosstype, 3, 3))
    n.ind <- nind(cross)
    for(i in names(cross$geno)) { # loop over chromosomes
        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type == "X")
            warning("reorgRIdraws not working properly for the X chromosome.")

        if(!("draws" %in% names(cross$geno[[i]]))) {
            warning("No imputations within cross.")
            return(cross)
        }

        draws <- cross$geno[[i]]$draws
        att <- attributes(draws)
        n.mar <- dim(draws)[2]
        n.imp <- dim(draws)[3]
        if(dim(draws)[1] != n.ind)
            stop("Mismatch between no. individuals in cross and in draws.")
        if(dim(crosses)[2] != n.str)
            stop("Invalid no. of founder strains specified")

        draws <- .C("R_reorgRIdraws",
                    as.integer(n.ind),
                    as.integer(n.mar),
                    as.integer(n.str),
                    as.integer(n.imp),
                    draws=as.integer(draws),
                    as.integer(crosses),
                    PACKAGE="qtl")$draws
        draws <- array(draws, dim=c(n.ind, n.mar, n.imp))
        for(j in names(att))
            attr(draws, j) <- att[[j]]
        cross$geno[[i]]$draws <- draws
    }

    cross
}

######################################################################
# reorgRIargmax
#
# For 4- and 8-way RIL, reorganize the results of argmax.geno
# using the information on the order of the founder strains in each
# cross.
######################################################################

reorgRIargmax <-
    function(cross)
{
    crosses <- cross$cross
    flag <- 0
    for(i in 1:ncol(crosses)) {
        if(any(crosses[,i] != i)) {
            flag <- 1
            break
        }
    }
    if(!flag) return(cross) # no need to reorder

    crosstype <- crosstype(cross)
    if(crosstype != "ri4sib" && crosstype != "ri4self" &&
       crosstype != "ri8sib" && crosstype != "ri8self" && crosstype != "bgmagic16")
        stop("reorgRIargmax not appropriate for cross type ", crosstype)

    if(crosstype=="bgmagic16") n.str <- 16
    else n.str <- as.numeric(substr(crosstype, 3, 3))
    n.ind <- nind(cross)
    for(i in names(cross$geno)) { # loop over chromosomes
        chr_type <- chrtype(cross$geno[[i]])
        if(chr_type == "X")
            warning("reorgRIargmax not working properly for the X chromosome.")

        if(!("argmax" %in% names(cross$geno[[i]]))) {
            warning("No argmax.geno results within cross.")
            return(cross)
        }

        argmax <- cross$geno[[i]]$argmax
        att <- attributes(argmax)
        n.mar <- dim(argmax)[2]
        if(dim(argmax)[1] != n.ind)
            stop("Mismatch between no. individuals in cross and in argmax.")
        if(dim(crosses)[2] != n.str)
            stop("Invalid no. of founder strains specified")

        argmax <- .C("R_reorgRIdraws",
                     as.integer(n.ind),
                     as.integer(n.mar),
                     as.integer(n.str),
                     as.integer(1),
                     argmax=as.integer(argmax),
                     as.integer(crosses),
                     PACKAGE="qtl")$argmax
        argmax <- matrix(argmax, nrow=n.ind, ncol=n.mar)
        for(j in names(att))
            attr(argmax, j) <- att[[j]]
        cross$geno[[i]]$argmax <- argmax
    }

    cross
}

######################################################################
# reorgRIpairprob
#
# For 4- and 8-way RIL, reorganize the results of calc.pairprob
# using the information on the order of the founder strains in each
# cross.
######################################################################

reorgRIpairprob <-
    function(cross, pairprob)
{
    crosses <- cross$cross
    flag <- 0
    for(i in 1:ncol(crosses)) {
        if(any(crosses[,i] != i)) {
            flag <- 1
            break
        }
    }
    if(!flag) return(pairprob) # no need to reorder

    crosstype <- crosstype(cross)
    if(crosstype != "ri4sib" && crosstype != "ri4self" &&
       crosstype != "ri8sib" && crosstype != "ri8self" && crosstype != "bgmagic16")
        stop("reorgRIargmax not appropriate for cross type ", crosstype)

    if(crosstype=="bgmagic16") n.str <- 16
    else n.str <- as.numeric(substr(crosstype, 3, 3))
    n.ind <- nind(cross)
    thedim <- dim(pairprob)

    chr_type <- chrtype(cross$geno[[1]])
    if(chr_type == "X")
        warning("reorgRIpairprob not working properly for the X chromosome.")

    att <- attributes(pairprob)
    if(thedim[1] != n.ind)
        stop("Mismatch between no. individuals in cross and in pairprob.")

    if(thedim[3] != n.str || thedim[4] != n.str)
        stop("Mismatch between no. founder strains in cross and in pairprob.")

    if(dim(crosses)[2] != n.str)
        stop("Invalid no. of founder strains specified")
    n.mar <- nmar(cross)[1]
    if(n.mar*(n.mar-1)/2 != thedim[2])
        stop("Mismatch between no. markers in cross and in pairprob.")

    pairprob <- .C("R_reorgRIpairprob",
                   as.integer(n.ind),
                   as.integer(n.mar), # no. prob
                   as.integer(n.str),
                   pairprob=as.double(pairprob),
                   as.integer(crosses),
                   PACKAGE="qtl")$pairprob
    pairprob <- array(pairprob, dim=thedim)
    for(j in names(att))
        attr(pairprob, j) <- att[[j]]

    pairprob
}

# end of ril48_reorg.R

#####################################################################
#
# transformPheno.R
#
# Copyright (c) 2009, Danny Arends
#
# Modified by Karl Broman
#
#
# first written Februari 2009
# last modified April 2009
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: transformPheno
#
#####################################################################


transformPheno <- function(cross, pheno.col=1, transf=log, ...)
{
    #Helperfunction to transform a specific phenotype specified by the pheno.col parameter
    # by default, a log transformation is used, though one may use any function

    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(any(is.na(num))) {
            if(sum(is.na(num))>1)
                stop("Couldn't identify phenotypes ", paste(paste("\"", pheno.col[is.na(num)], "\"", sep=""),
                                                            collapse=" "))
            else
                stop("Couldn't identify phenotype \"", pheno.col[is.na(num)], "\"")
        }
        pheno.col <- num
    }

    if(any(pheno.col < 1 | pheno.col > nphe(cross)))
        stop("pheno.col values should be between 1 and the no. phenotypes")

    for(i in pheno.col)
        cross$pheno[,i] <- transf(cross$pheno[,i], ...)

    cross
}

# end of transformPheno.R

#####################################################################
#
# inferFounderHap.R
#
# copyright (c) 2011, Karl W Broman
# last modified Dec, 2011
# first written Dec, 2011
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: inferFounderHap, restoreMWrilGeno
#
# This is for reconstructing the founder haplotypes in inbred lines
# by a crude method using groups of adjacent SNPs
#
######################################################################

inferFounderHap <-
    function(cross, chr, max.n.markers=15)
{
    if(!missing(chr))
        cross <- subset(cross, chr=chr)
    if(nchr(cross) > 1) {
        thechr <- names(cross$geno)[1]
        cross <- subset(cross, chr=thechr)
        warning("inferFounderHap is only for one chromosome; considering ", thechr)
    }

    # pull out genotypes for RIL and founders
    offspringGen <- restoreMWrilGeno(cross)
    founderGen <- cross$founderGeno

    # drop markers with any missing data in the founders
    nomissing <- apply(founderGen, 2, function(a) !any(is.na(a)))
    names(nomissing) <- colnames(offspringGen)
    if(!any(nomissing))
        stop("No markers with complete founder genotypes")
    offspringGen <- offspringGen[,nomissing,drop=FALSE]
    founderGen <- founderGen[,nomissing,drop=FALSE]

    longbits <- .Machine$sizeof.long*8
    if(max.n.markers > longbits-1) {
        max.n.markers <- longbits-1
        warning("We can't use max.n.markers > ", longbits-1,
                ", so we're taking max.n.markers = ", longbits-1)
    }
    n.mar <- ncol(offspringGen)
    if(max.n.markers > n.mar) max.n.markers <- n.mar
    max.offset <- ceiling((max.n.markers-1)/2)

    n.ind <- nrow(offspringGen)
    n.founders <- nrow(founderGen)
    if(n.mar != ncol(founderGen))
        stop("ncol(offspringGen) != ncol(founderGen)")
    if(any(!is.na(offspringGen) & offspringGen != 0 & offspringGen != 1))
        stop("offspringGen should be NA, 0 or 1")
    if(any(!is.na(founderGen) & founderGen != 0 & founderGen != 1))
        stop("founderGen should be NA, 0 or 1")

    offspringGen[is.na(offspringGen)] <- -1

    z <- .C("R_inferFounderHap",
            as.integer(n.mar),
            as.integer(n.founders),
            as.integer(n.ind),
            as.integer(founderGen),
            as.integer(offspringGen),
            as.integer(max.offset),
            hap=as.integer(rep(0,n.mar * n.ind)),
            PACKAGE="qtl")
    z$hap[z$hap <= 0] <- NA

    fullhap <- matrix(ncol=length(nomissing), nrow=n.ind)
    fullhap[,nomissing] <- matrix(z$hap, ncol=n.mar, nrow=n.ind)
    colnames(fullhap) <- names(nomissing)

    fullhap
}


restoreMWrilGeno <-
    function(cross)
{
    g <- pull.geno(cross)
    f <- cross$founderGeno
    uf <- unique(as.numeric(f[!is.na(f)]))
    f[is.na(f)] <- missingval <- min(uf)-1
    g[is.na(g)] <- 0

    n.mar <- ncol(g)
    n.ind <- nrow(g)
    n.str <- nrow(f)
    if(n.mar != ncol(f))
        stop("no. genotypes inconsistent between offspring and founders.")

    crosses <- cross$cross
    if(ncol(crosses) != n.str || nrow(crosses) != n.ind)
        stop("Incompatiability in cross$cross dimension.")

    gen <-
        .C("R_restoreMWrilGeno",
           as.integer(n.ind),
           as.integer(n.mar),
           as.integer(n.str),
           as.integer(f),
           gen=as.integer(g),
           as.integer(crosses),
           as.integer(missingval),
           PACKAGE="qtl")$gen
    gen[gen==missingval] <- NA
    gen <- matrix(gen, nrow=n.ind, ncol=n.mar)
    colnames(gen) <- colnames(g)
    gen
}


# end of inferFounderHap.R

######################################################################
#
# summary.scanone.R
#
# copyright (c) 2001-2019, Karl W Broman
# last modified Dec, 2019
# first written Sep, 2001
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: summary.scanone, print.summary.scanone,
#           max.scanone, c.scanone, subset.scanone,
#           summary.scanoneperm, print.summary.scanoneperm
#           c.scanoneperm, rbind.scanoneperm, cbind.scanoneperm
#           grab.arg.names, subset.scanoneperm, [.scanoneperm
#
######################################################################

##################################################################
# summarize scanone results
##################################################################
summary.scanone <-
    function(object, threshold, format=c("onepheno", "allpheno", "allpeaks", "tabByCol", "tabByChr"),
             perms, alpha, lodcolumn=1, pvalues=FALSE,
             ci.function=c("lodint", "bayesint"), ...)
{
    if(!inherits(object, "scanone"))
        stop("Input should have class \"scanone\".")

    format <- match.arg(format)
    ncol.object <- ncol(object)-2
    cn.object <- colnames(object)[-(1:2)]

    if(ncol.object==1 && (format == "allpeaks" || format == "allpheno")) {
        warning("With just one LOD column, format=\"onepheno\" used.")
        format <- "onepheno"
    }

    if(format != "onepheno" && !missing(lodcolumn))
        warning("lodcolumn ignored except when format=\"onepheno\".")

    if(!missing(perms)) {
        if(inherits(perms, "scantwoperm"))
            perms <- scantwoperm2scanoneperm(perms)
        else if(!inherits(perms, "scanoneperm"))
            warning("perms need to be in scanoneperm format.")
    }

    # check input
    if(missing(perms) && !missing(alpha))
        stop("If alpha is to be used, permutation results must be provided.")
    if(!missing(threshold) && !missing(alpha))
        stop("Only one of threshold and alpha should be specified.")

    if(format == "onepheno") {
        if(!missing(lodcolumn) && length(lodcolumn) > 1) {
            warning("With format=\"onepheno\", lodcolumn should have length 1.")
            lodcolumn <- lodcolumn[1]
        }
        if(lodcolumn < 1 || lodcolumn > ncol.object)
            stop("lodcolumn should be between 1 and no. LOD columns.")
    }

    if(!missing(alpha) && length(alpha) > 1) {
        warning("alpha should have length 1.")
        alpha <- alpha[1]
    }

    if(!missing(perms)) {
        if("xchr" %in% names(attributes(perms))) {
            ncol.perms <- ncol(perms$A)
            cn.perms <- colnames(perms$A)
        }
        else {
            ncol.perms <- ncol(perms)
            cn.perms <- colnames(perms)
        }

        if(ncol.object != ncol.perms) {
            if(ncol.perms==1) { # reuse the multiple columns
                origperms <- perms
                if("xchr" %in% names(attributes(perms))) {
                    for(j in 2:ncol.object) {
                        perms$A <- cbind(perms$A, origperms$A)
                        perms$X <- cbind(perms$X, origperms$X)
                    }
                    cn.perms <- colnames(perms$A) <- colnames(perms$X) <- cn.object
                }
                else {
                    for(j in 2:ncol.object)
                        perms <- cbind(perms, origperms)
                    cn.perms <- colnames(perms) <- cn.object
                }
                warning("Just one column of permutation results; reusing for all LOD score columns.")
            }
            else {
                if(ncol.object == 1) {
                    warning("Using just the first column in the perms input")
                    if("xchr" %in% names(attributes(perms))) {
                        perms$A <- perms$A[,1,drop=FALSE]
                        perms$X <- perms$X[,1,drop=FALSE]
                    }
                    else {
                        clp <- class(perms)
                        perms <- perms[,1,drop=FALSE]
                        class(perms) <- clp
                    }
                }
                else
                    stop("scanone input has different number of LOD columns as perms input.")
            }
        }
        if(!all(cn.object == cn.perms))
            warning("Column names in scanone input do not match those in perms input.")
    }

    if(format != "onepheno") {
        if(!missing(threshold)) {
            if(length(threshold)==1)
                threshold <- rep(threshold, ncol.object)
            else if(length(threshold) != ncol.object)
                stop("threshold should have length 1 or match number LOD scores in scanone input.")
        }
    }

    if(missing(perms) && pvalues) {
        warning("Can show p-values only if perms are provided.")
        pvalues <- FALSE
    }
    # end of check of input

    # chromosome IDs as a character string
    chr <- as.character(object[,1])

    if(format=="onepheno") {
        lodcolumn <- lodcolumn+2

        # pull out max on each chromosome
        wh <- NULL
        for(i in unique(chr)) {
            if(any(!is.na(object[chr==i,lodcolumn]))) {
                mx <- max(object[chr==i,lodcolumn],na.rm=TRUE)
                tmp <- which(chr==i & object[,lodcolumn]==mx)
                if(length(tmp) > 1) tmp <- sample(tmp, 1) # if multiple, pick at random
                wh <- c(wh, tmp)
            }
        }
        thechr <- as.character(object[wh,1])

        if(!missing(threshold))
            wh <- wh[object[wh,lodcolumn] > threshold]
        else if(!missing(alpha)) {
            thr <- summary(perms, alpha)
            if("xchr" %in% names(attributes(perms))) {
                thr <- sapply(thr, function(a,b) a[,b], lodcolumn-2)
                xchr <- attr(perms, "xchr")
                xchr <- names(xchr)[xchr]
                xchr <- thechr %in% xchr
                wh <- wh[(!xchr & object[wh,lodcolumn] > thr[1]) |
                         (xchr & object[wh,lodcolumn] > thr[2])]
            }
            else {
                thr <- thr[,lodcolumn-2]
                wh <- wh[object[wh,lodcolumn] > thr]
            }
        }

        result <- object[wh,]
    } # end of "onepheno" format

    else if(format=="allpheno") {
        # pull out max on each chromosome
        wh <- vector("list", ncol.object)
        for(lodcolumn in 1:ncol.object+2) {
            for(i in unique(chr)) {
                if(any(!is.na(object[chr==i,lodcolumn]))) {
                    mx <- max(object[chr==i,lodcolumn],na.rm=TRUE)
                    tmp <- which(chr==i & object[,lodcolumn]==mx)
                    if(length(tmp) > 1) tmp <- sample(tmp, 1)
                    wh[[lodcolumn-2]] <- c(wh[[lodcolumn-2]], tmp)
                }
            }
        }

        if(!missing(threshold)) { # rows with at least one LOD > threshold
            for(lodcolumn in 1:ncol.object) {
                temp <- wh[[lodcolumn]]
                wh[[lodcolumn]] <- temp[object[temp,lodcolumn+2] > threshold[lodcolumn]]
            }
        }
        else if(!missing(alpha)) {
            thr <- summary(perms, alpha)
            if("xchr" %in% names(attributes(perms))) {
                xchr <- attr(perms, "xchr")
                xchr <- names(xchr)[xchr]

                for(lodcolumn in 1:ncol.object) {
                    temp <- wh[[lodcolumn]]
                    thechr <- as.character(object[temp,1])
                    xchr <- thechr %in% xchr
                    wh[[lodcolumn]] <- temp[(!xchr & object[temp,lodcolumn+2] > thr$A[lodcolumn]) |
                                            (xchr & object[temp,lodcolumn+2] > thr$X[lodcolumn])]
                }
            }
            else {
                for(lodcolumn in 1:ncol.object) {
                    temp <- wh[[lodcolumn]]
                    wh[[lodcolumn]] <- temp[object[temp,lodcolumn+2] > thr[lodcolumn]]
                }
            }
        }

        wh <- sort(unique(unlist(wh)))
        result <- object[wh,]
    }  # end of format=="allpheno"

    else if(format=="allpeaks") {
        # pull out max on each chromosome
        wh <- vector("list", ncol.object)

        for(lodcolumn in (1:ncol.object)+2) {
            for(i in unique(chr)) {
                if(any(!is.na(object[chr==i,lodcolumn]))) {
                    mx <- max(object[chr==i,lodcolumn],na.rm=TRUE)
                    temp <- which(chr==i & object[,lodcolumn]==mx)
                    if(length(temp)>1) temp <- sample(temp, 1)
                    wh[[lodcolumn-2]] <- c(wh[[lodcolumn-2]], temp)
                }
                else
                    wh[[lodcolumn-2]] <- c(wh, NA)
            }
        }

        pos <- sapply(wh, function(a,b) b[a], object[,2])
        if(!is.matrix(pos)) pos <- as.matrix(pos)

        lod <- pos

        for(i in 1:ncol(pos))
            lod[,i] <- object[wh[[i]],i+2]
        thechr <- as.character(unique(object[,1]))

        if(!missing(threshold)) { # rows with at least one LOD > threshold
            keep <- NULL
            for(i in seq(along=thechr))
                if(any(lod[i,] > threshold)) keep <- c(keep, i)
        }
        else if(!missing(alpha)) {
            keep <- NULL
            thr <- summary(perms, alpha)
            if("xchr" %in% names(attributes(perms))) {
                xchr <- attr(perms, "xchr")
                xchr <- names(xchr)[xchr]
                xchr <- thechr %in% xchr

                for(i in seq(along=thechr)) {
                    if((xchr[i] && any(lod[i,] > thr$X)) ||
                       (!xchr[i] && any(lod[i,] > thr$A)))
                        keep <- c(keep, i)
                }
            }
            else {
                for(i in seq(along=thechr)) {
                    if(any(lod[i,] > thr))
                        keep <- c(keep, i)
                }
            }
        }
        else keep <- seq(along=thechr)

        if(is.null(keep))
            result <- object[NULL,,drop=FALSE]
        else {
            pos <- pos[keep,,drop=FALSE]
            lod <- lod[keep,,drop=FALSE]
            thechr <- thechr[keep]
            result <- as.data.frame(matrix(ncol=ncol.object*2+1,nrow=length(keep)), stringsAsFactors=TRUE)
            names(result)[1] <- "chr"
            names(result)[(1:ncol.object)*2] <- "pos"
            names(result)[(1:ncol.object)*2+1] <- names(object)[-(1:2)]
            result[,1] <- thechr
            result[,(1:ncol.object)*2] <- pos
            result[,(1:ncol.object)*2+1] <- lod
        }
    }
    else { # format=="tabByChr" or =="tabByCol"
        result <- vector("list", ncol.object)
        names(result) <- names(object)[-(1:2)]

        # pull out max on each chromosome
        wh <- vector("list", ncol.object)

        for(lodcolumn in (1:ncol.object)+2) {
            for(i in unique(chr)) {
                if(any(!is.na(object[chr==i,lodcolumn]))) {
                    mx <- max(object[chr==i,lodcolumn],na.rm=TRUE)
                    temp <- which(chr==i & object[,lodcolumn]==mx)
                    if(length(temp)>1) temp <- sample(temp, 1)
                    wh[[lodcolumn-2]] <- c(wh[[lodcolumn-2]], temp)
                }
                else
                    wh[[lodcolumn-2]] <- c(wh, NA)
            }
        }

        pos <- sapply(wh, function(a,b) b[a], object[,2])
        if(!is.matrix(pos)) pos <- as.matrix(pos)

        lod <- pos

        for(i in 1:ncol(pos))
            lod[,i] <- object[wh[[i]],i+2]
        thechr <- as.character(unique(object[,1]))

        for(i in 1:ncol.object)
            result[[i]] <- object[wh[[i]],c(1,2,i+2)]
    }


    if(pvalues) {
        if(format != "tabByCol" && format != "tabByChr") {
            if(nrow(result) > 0) { # get p-values and add to the results
                rn <- rownames(result)

                if("xchr" %in% names(attributes(perms))) {
                    xchr <- attr(perms, "xchr")
                    xchr <- names(xchr)[xchr]
                    xchr <- as.character(result[,1]) %in% xchr
                    L <- attr(perms, "L")
                    Lt <- sum(L)

                    if(format=="allpeaks")
                        thecol <- (1:ncol.object)*2+1
                    else thecol <- (1:ncol.object)+2

                    if(any(xchr)) {
                        tempX <- calcPermPval(result[xchr,thecol,drop=FALSE], perms$X)
                        tempX <- as.data.frame(1-(1-tempX)^(Lt/L[2]), stringsAsFactors=TRUE)
                    }
                    else tempX <- NULL
                    if(any(!xchr)) {
                        tempA <- calcPermPval(result[!xchr,thecol,drop=FALSE], perms$A)
                        tempA <- as.data.frame(1-(1-tempA)^(Lt/L[1]), stringsAsFactors=TRUE)
                    }
                    else tempA <- NULL
                    pval <- rbind(tempA, tempX)
                    if(any(xchr)) pval[xchr,] <- tempX
                    if(any(!xchr)) pval[!xchr,] <- tempA
                }
                else {
                    if(format=="allpeaks") thecol <- (1:ncol.object)*2+1
                    else thecol <- (1:ncol.object)+2

                    pval <- as.data.frame(calcPermPval(result[,thecol,drop=FALSE], perms), stringsAsFactors=TRUE)
                }

                if(format == "allpeaks") {
                    temp <- as.data.frame(matrix(nrow=nrow(result), ncol=ncol.object*3+1), stringsAsFactors=TRUE)

                    names(temp)[1] <- names(result)[1]
                    temp[,1] <- result[,1]

                    for(i in 1:ncol.object) {
                        names(temp)[i*3+(-1:1)] <- c(names(result)[i*2+(0:1)], "pval")
                        temp[,i*3-1:0] <- result[,i*2+(0:1)]
                        temp[,i*3+1] <- pval[[i]]
                    }
                }
                else if(format != "tabByCol" && format != "tabByChr") {
                    temp <- as.data.frame(matrix(nrow=nrow(result), ncol=ncol.object*2+2), stringsAsFactors=TRUE)

                    names(temp)[1:2] <- names(result)[1:2]
                    temp[,1:2] <- result[,1:2]
                    for(i in 1:ncol.object) {
                        names(temp)[i*2+1:2] <- c(names(result)[i+2], "pval")
                        temp[,i*2+1] <- result[,i+2]
                        temp[,i*2+2] <- pval[[i]]
                    }
                }
                result <- temp
                rownames(result) <- rn

            }
        }
        else { # format=="tabByCol" || format=="tabByChr"
            peaks <- as.data.frame(lapply(result, function(a) a[,3]), stringsAsFactors=TRUE)

            if("xchr" %in% names(attributes(perms))) {
                xchr <- attr(perms, "xchr")
                xchr <- names(xchr)[xchr]
                xchr <- as.character(result[[1]][,1]) %in% xchr
                L <- attr(perms, "L")
                Lt <- sum(L)

                if(any(xchr)) {
                    tempX <- as.data.frame(calcPermPval(peaks[xchr,,drop=FALSE], perms$X), stringsAsFactors=TRUE)
                    tempX <- 1-(1-tempX)^(Lt/L[2])
                }
                else tempX <- NULL
                if(any(!xchr)) {
                    tempA <- as.data.frame(calcPermPval(peaks[!xchr,,drop=FALSE], perms$A), stringsAsFactors=TRUE)
                    tempA <- 1-(1-tempA)^(Lt/L[1])
                }
                else tempA <- NULL

                pval <- rbind(tempA, tempX)
                if(any(xchr)) pval[xchr,] <- tempX
                if(any(!xchr)) pval[!xchr,] <- tempA
            }
            else
                pval <- as.data.frame(calcPermPval(peaks, perms), stringsAsFactors=TRUE)

            for(i in seq(along=result))
                result[[i]] <- cbind(as.data.frame(result[[i]]), pval=pval[,i], stringsAsFactors=TRUE)
        }
    }

    if(format=="tabByCol" || format=="tabByChr") {

        # drop insignificant peaks
        if(!missing(threshold)) { # rows with at least one LOD > threshold
            for(i in seq(along=result))
                result[[i]] <- result[[i]][lod[,i] > threshold[i],,drop=FALSE]
        }
        else if(!missing(alpha)) {
            keep <- NULL
            thr <- summary(perms, alpha)

            if("xchr" %in% names(attributes(perms))) {
                xchr <- attr(perms, "xchr")
                xchr <- names(xchr)[xchr]
                xchr <- thechr %in% xchr

                for(i in seq(along=result))
                    result[[i]] <- result[[i]][(lod[,i] > thr$A[i] & !xchr) |
                                               (lod[,i] > thr$X[i] & xchr), , drop=FALSE]
            }
            else {
                for(i in seq(along=result))
                    result[[i]] <- result[[i]][lod[,i] > thr[i],,drop=FALSE]
            }
        }

        # add intervals
        ci.function <- match.arg(ci.function)
        if(ci.function=="lodint") cif <- lodint
        else cif <- bayesint

        for(i in seq(along=result)) {
            if(nrow(result[[i]]) == 0) next

            lo <- hi <- rep(NA, nrow(result[[i]]))
            for(j in 1:nrow(result[[i]])) {
                temp <- cif(object, chr=as.character(result[[i]][j,1]), lodcolumn=i, ...)
                lo[j] <- temp[1,2]
                hi[j] <- temp[nrow(temp),2]
            }
            result[[i]] <- cbind(as.data.frame(result[[i]]), ci.low=lo, ci.high=hi, stringsAsFactors=TRUE)
            colnames(result[[i]])[3] <- "lod"
        }

        if(format=="tabByChr" && length(result)==1)
            format <- "tabByCol"     # no need to do by chr in this case

        if(format=="tabByChr") {
            temp <- vector("list", length(thechr))
            names(temp) <- thechr

            for(i in seq(along=result)) {
                if(nrow(result[[i]])==0) next
                rownames(result[[i]]) <- paste(names(result)[i], rownames(result[[i]]), sep=" : ")
                for(j in 1:nrow(result[[i]])) {
                    thischr <- match(result[[i]][j,1], thechr)
                    if(length(temp[[thischr]])==0)
                        temp[[thischr]] <- result[[i]][j,,drop=FALSE]
                    else
                        temp[[thischr]] <- rbind(temp[[thischr]], result[[i]][j,,drop=FALSE])
                }
            }
            result <- temp
        }

        # move CI to before the lod score
        for(i in seq(along=result)) {
            if(is.null(result[[i]]) || nrow(result[[i]])==0) next
            nc <- ncol(result[[i]])
            result[[i]] <- result[[i]][,c(1,2,nc-1,nc,3:(nc-2)),drop=FALSE]
        }

        attr(result, "tab") <- format
    }

    if(format=="allpeaks") rownames(result) <- as.character(result$chr)

    if(format=="tabByCol" || format=="tabByChr")
        class(result) <- c("summary.scanone", "list")
    else
        class(result) <- c("summary.scanone", "data.frame")
    result
}

# print output of summary.scanone
print.summary.scanone <-
    function(x, ...)
{
    tab <- attr(x, "tab")

    if(is.null(tab) && nrow(x) == 0) {
        cat("    There were no LOD peaks above the threshold.\n")
        return(invisible(NULL))
    }

    flag <- FALSE
    if(is.null(tab)) {
        print.data.frame(x,digits=3)
        flag <- TRUE
    }
    else if(tab=="tabByChr") {
        for(i in seq(along=x)) {
            if(is.null(x[[i]])) next
            else {
                flag <- TRUE
                cat("Chr ", names(x)[i], ":\n", sep="")
                print(x[[i]], digits=3)
                cat("\n")
            }
        }
    }
    else if(tab=="tabByCol") {
        for(i in seq(along=x)) {
            if(nrow(x[[i]])==0) next
            else {
                flag <- TRUE
                if(length(x) > 1) cat(names(x)[i], ":\n", sep="")
                print(x[[i]], digits=3)
                if(length(x) > 1) cat("\n")
            }
        }
    }
    if(!flag)
        cat("    There were no LOD peaks above the threshold.\n")
}

# pull out maximum LOD peak, genome-wide
max.scanone <-
    function(object, chr, lodcolumn=1, na.rm=TRUE, ...)
{
    if(!inherits(object, "scanone"))
        stop("Input must have class \"scanone\".")

    if(lodcolumn < 1 || lodcolumn+2 > ncol(object))
        stop("Argument lodcolumn should be between 1 and ", ncol(object)-2)

    if(!missing(chr)) object <- subset(object, chr=chr)

    maxlod <- max(object[,lodcolumn+2],na.rm=TRUE)
    wh <- which(!is.na(object[,lodcolumn+2]) & object[,lodcolumn+2]==maxlod)
    if(length(wh) > 1) wh <- sample(wh, 1)
    object <- object[wh,]

    object[,1] <- factor(as.character(unique(object[,1])))

    summary.scanone(object,threshold=0,lodcolumn=lodcolumn)
}

######################################################################
# subset.scanone
######################################################################
subset.scanone <-
    function(x, chr, lodcolumn, ...)
{
    if(!inherits(x, "scanone"))
        stop("Input should have class \"scanone\".")

    if(missing(chr) && missing(lodcolumn))
        stop("You must specify either chr or lodcolumn.")

    y <- x

    if(!missing(chr)) {
        chr <- matchchr(chr, unique(x[,1]))
        x <- x[!is.na(match(x[,1],chr)), ,drop=FALSE]
        thechr <- as.character(x[,1])
        x[,1] <- factor(thechr, levels=unique(thechr))
    }

    if(!missing(lodcolumn)) {
        if(any(lodcolumn>0) && any(lodcolumn<0))
            stop("lodcolumn values can't be both >0 and <0.")
        if(any(lodcolumn<0) || is.logical(lodcolumn))
            lodcolumn <- (1:(ncol(x)-2))[lodcolumn]
        if(length(lodcolumn)==0)
            stop("You must retain at least one LOD column.")
        if(any(lodcolumn < 1 || lodcolumn > ncol(x)-2))
            stop("lodcolumn values must be >=1 and <=",ncol(x)-2)
        x <- x[,c(1,2,lodcolumn+2)]
    }

    class(x) <- class(y)
    nam <- names(attributes(y))
    if("method" %in% nam)
        attr(x, "method") <- attr(y,"method")
    if("type" %in% nam)
        attr(x, "type") <- attr(y,"type")
    if("model" %in% nam)
        attr(x, "model") <- attr(y,"model")

    x
}

######################################################################
# c.scanone
#
# Combine the results of multiple runs of scanone into single object
# (pasting the columns together).
######################################################################
c.scanone <-
    function(..., labels)
{
    dots <- list(...)
    if(length(dots)==1 && is.list(dots[[1]])) dots <- dots[[1]]

    if(length(dots)==1) return(dots[[1]])
    for(i in seq(along=dots)) {
        if(!inherits(dots[[i]], "scanone"))
            stop("Input should have class \"scanone\".")
    }

    if(!missing(labels)) {
        if(length(labels)==1)
            labels <- rep(labels, length(dots))
        if(length(labels) != length(dots))
            stop("labels needs to be the same length as the number of objects input.")
        gavelabels <- TRUE
    }
    else {
        labels <- grab.arg.names(...)
        gavelabels <- FALSE
    }

    nr <- sapply(dots, nrow)
    if(length(unique(nr)) != 1)
        stop("The input must all have the same number of rows.")

    chr <- lapply(dots, function(a) a$chr)
    pos <- lapply(dots, function(a) a$pos)
    for(i in 2:length(dots)) {
        if(any(chr[[1]] != chr[[i]]) || any(pos[[1]] != pos[[i]])) {
            cat("The input must conform exactly (same chr and positions\n")
            stop("(That is, calc.genoprob and/or sim.geno must have used the same step and off.end)\n")
        }
    }
    cl <- class(dots[[1]])

    thenam <- unlist(lapply(dots, function(a) colnames(a)[-(1:2)]))
    if(length(unique(thenam)) == length(thenam))
        repeats <- FALSE
    else repeats <- TRUE

    if(repeats || gavelabels) {
        for(i in 1:length(dots)) {
            colnames(dots[[i]])[-(1:2)] <- paste(colnames(dots[[i]])[-(1:2)], labels[i], sep=".")
            dots[[i]] <- as.data.frame(dots[[i]], stringsAsFactors=TRUE)
        }
    }

    result <- dots[[1]]

    for(i in 2:length(dots))
        result <- cbind(as.data.frame(result, stringsAsFactors=TRUE),
                        as.data.frame(dots[[i]][,-(1:2),drop=FALSE], stringsAsFactors=TRUE))

    class(result) <- cl
    result
}

cbind.scanone <- c.scanone

grab.arg.names <-
    function(...)
{
    # pull out the names from the input
    temp <- deparse(substitute(c(...)))
    temp <- unlist(strsplit(temp, ","))
    for(i in seq(along=temp))
        temp[i] <-  paste(unlist(strsplit(temp[i]," ")),collapse="")
    temp[1] <- substr(temp[1], 3, nchar(temp[1]))
    temp[length(temp)] <- substr(temp[length(temp)], 1, nchar(temp[length(temp)])-1)

    temp
}


######################################################################
# summary.scanoneperm
#
# Give genome-wide LOD thresholds on the basis of the results of
# scanone permutation test (from scanone with n.perm > 0)
######################################################################
summary.scanoneperm <-
    function(object, alpha=c(0.05, 0.10), controlAcrossCol=FALSE, ...)
{
    if(!inherits(object, "scanoneperm"))
        stop("Input should have class \"scanoneperm\".")

    if(any(alpha < 0 | alpha > 1))
        stop("alpha should be between 0 and 1.")

    if("xchr" %in% names(attributes(object))) { # X-chromosome-specific results
        L <- attr(object, "L")
        thealpha <- cbind(1 - (1-alpha)^(L[1]/sum(L)), 1 - (1-alpha)^(L[2]/sum(L)))
        v <- c("A","X")

        quant <- vector("list", 2)
        names(quant) <- c("A","X")
        for(k in 1:2) {
            if(!is.matrix(object[[v[k]]])) object[[v[k]]] <- as.matrix(object[[v[k]]])

            if(controlAcrossCol) {
                if(any(is.na(object[[v[k]]])))
                    object[[v[k]]] <- object[[v[k]]][apply(object[[v[k]]],1,function(a) !any(is.na(a))),,drop=FALSE]

                r <- apply(object[[v[k]]], 2, rank, ties.method="random", na.last=FALSE)
                print(is.matrix(r))
                rmax <- apply(r, 1, max)
                rqu <- quantile(rmax, 1-thealpha[,k], na.rm=TRUE)
                qu <- matrix(nrow=length(thealpha[,k]), ncol=ncol(object[[v[k]]]))
                object.sort <- apply(object[[v[k]]], 2, sort, na.last=FALSE)

                for(i in seq(along=rqu)) {
                    if(fl==ce) # exact
                        qu[i,] <- object.sort[rqu[i],]
                    else # need to interpolate
                        qu[i,] <- object.sort[fl,]*(1-(ce-fl)) + object.sort[ce,]*(ce-fl)
                }
                colnames(qu) <- colnames(object[[v[k]]])
            }
            else
                qu <- apply(object[[v[k]]], 2, quantile, 1-thealpha[,k], na.rm=TRUE)

            if(!is.matrix(qu)) {
                nam <- names(qu)
                qu <- matrix(qu, nrow=length(alpha))
                dimnames(qu) <- list(paste(100*alpha,"%", sep=""), nam)
            }
            else rownames(qu) <- paste(100*alpha, "%", sep="")

            quant[[k]] <- qu
        }

        attr(quant, "n.perm") <- c("A"=nrow(object$A), "X"=nrow(object$X))
        class(quant) <- "summary.scanoneperm"
    }
    else {
        if(!is.matrix(object)) object <- as.matrix(object)

        if(controlAcrossCol) {
            if(any(is.na(object)))
                object <- object[apply(object,1,function(a) !any(is.na(a))),,drop=FALSE]

            r <- apply(object, 2, rank, ties.method="random", na.last=FALSE)
            rmax <- apply(r, 1, max)
            rqu <- quantile(rmax, 1-alpha, na.rm=TRUE)
            quant <- matrix(nrow=length(alpha), ncol=ncol(object))
            object.sort <- apply(object, 2, sort, na.last=FALSE)
            for(i in seq(along=rqu)) {
                fl <- floor(rqu[i])
                ce <- ceiling(rqu[i])
                if(fl==ce) # exact
                    quant[i,] <- object.sort[rqu[i],]
                else # need to interpolate
                    quant[i,] <- object.sort[fl,]*(1-(ce-fl)) + object.sort[ce,]*(ce-fl)
            }
            colnames(quant) <- colnames(object)
        }
        else
            quant <- apply(object, 2, quantile, 1-alpha, na.rm=TRUE)

        if(!is.matrix(quant)) {
            nam <- names(quant)
            quant <- matrix(quant, nrow=length(alpha))
            dimnames(quant) <- list(paste(100*alpha,"%", sep=""), nam)
        }
        else rownames(quant) <- paste(100*alpha, "%", sep="")

        attr(quant, "n.perm") <- nrow(object)
        class(quant) <- "summary.scanoneperm"
    }
    quant
}

print.summary.scanoneperm <-
    function(x, ...)
{
    n.perm <- attr(x, "n.perm")
    if(length(n.perm)==2) {
        cat("Autosome LOD thresholds (", n.perm[1], " permutations)\n", sep="")
        x$A <- x$A[1:nrow(x$A),,drop=FALSE]
        print(x$A, digits=3)

        cat("\nX chromosome LOD thresholds (", n.perm[2], " permutations)\n", sep="")
        x$X <- x$X[1:nrow(x$X),,drop=FALSE]
        print(x$X, digits=3)
    }
    else {
        cat("LOD thresholds (", n.perm, " permutations)\n", sep="")
        x <- x[1:nrow(x),,drop=FALSE]
        print(x, digits=3)
    }
}

######################################################################
# combine scanoneperm results ... paste the rows together
######################################################################
rbind.scanoneperm <- c.scanoneperm <-
    function(...)
{
    dots <- list(...)

    if(length(dots)==1 && is.list(dots[[1]])) dots <- dots[[1]]

    if(length(dots)==1) return(dots[[1]])
    for(i in seq(along=dots)) {
        if(!inherits(dots[[i]], "scanoneperm"))
            stop("Input should have class \"scanoneperm\".")
    }

    if("xchr" %in% names(attributes(dots[[1]]))) {
        xchr <- lapply(dots, attr, "xchr")
        L <- lapply(dots, attr, "L")
        for(i in 2:length(dots)) {
            if(length(xchr[[1]]) != length(xchr[[i]]) ||
               any(xchr[[1]] != xchr[[i]]))
                stop("xchr attributes in the input must be consistent.")
            if(length(L[[1]]) != length(L[[i]]) ||
               any(L[[1]] != L[[i]]))
                stop("L attributes in the input must be consistent.")
        }
        for(i in 1:length(dots)) dots[[i]] <- unclass(dots[[i]])

        ncA <- sapply(dots, function(a) ncol(a$A))
        ncX <- sapply(dots, function(a) ncol(a$X))
        if(length(unique(ncA)) != 1 || length(unique(ncX)) != 1)
            stop("The input must all have the same number of columns.")
        result <- dots[[1]]
        for(i in 2:length(dots)) {
            result$A <- rbind(result$A, dots[[i]]$A)
            result$X <- rbind(result$X, dots[[i]]$X)
        }
        class(result) <- "scanoneperm"
        attr(result, "xchr") <- xchr[[1]]
        attr(result, "L") <- L[[1]]
    }
    else {
        nc <- sapply(dots, ncol)
        if(length(unique(nc)) != 1)
            stop("The input must all have the same number of columns.")
        for(i in 1:length(dots)) dots[[i]] <- unclass(dots[[i]])
        result <- dots[[1]]
        for(i in 2:length(dots))
            result <- rbind(result, dots[[i]])
        class(result) <- "scanoneperm"
    }
    result
}

######################################################################
# combine scanoneperm results ... paste the columns together
######################################################################
cbind.scanoneperm <-
    function(..., labels)
{
    dots <- list(...)
    if(length(dots)==1) return(dots[[1]])

    for(i in seq(along=dots)) {
        if(!inherits(dots[[i]], "scanoneperm"))
            stop("Input should have class \"scanoneperm\".")
    }

    if(!missing(labels)) {
        if(length(labels)==1)
            labels <- rep(labels, length(dots))
        if(length(labels) != length(dots))
            stop("labels needs to be the same length as the number of objects input.")
        gavelabels <- TRUE
    }
    else {
        labels <- grab.arg.names(...)
        gavelabels <- FALSE
    }


    if("xchr" %in% names(attributes(dots[[1]]))) {
        xchr <- lapply(dots, attr, "xchr")
        L <- lapply(dots, attr, "L")
        for(i in 2:length(dots)) {
            if(length(xchr[[1]]) != length(xchr[[i]]) ||
               any(xchr[[1]] != xchr[[i]]))
                stop("xchr attributes in the input must be consistent.")
            if(length(L[[1]]) != length(L[[i]]) ||
               any(L[[1]] != L[[i]]))
                stop("L attributes in the input must be consistent.")
        }
        for(i in 1:length(dots)) dots[[i]] <- unclass(dots[[i]])

        nr <- sapply(dots, function(a) nrow(a$A))
        mnr <- max(nr)
        if(any(nr < mnr)) { # pad with NAs
            for(i in which(nr < mnr))
                dots[[i]]$A <- rbind(dots[[i]]$A, matrix(NA, ncol=ncol(dots[[i]]$A),
                                                         nrow=mnr-nr[i]))
        }

        nr <- sapply(dots, function(a) nrow(a$X))
        mnr <- max(nr)
        if(any(nr < mnr)) { # pad with NAs
            for(i in which(nr < mnr))
                dots[[i]]$X <- rbind(dots[[i]]$X, matrix(NA, ncol=ncol(dots[[i]]$X),
                                                         nrow=mnr-nr[i]))
        }

        thenamA <- unlist(lapply(dots, function(a) colnames(a$A)))
        thenamX <- unlist(lapply(dots, function(a) colnames(a$X)))
        if(length(unique(thenamA)) == length(thenamA) &&
           length(unique(thenamX)) == length(thenamX))
            repeats <- FALSE
        else repeats <- TRUE

        if(repeats || gavelabels) {
            colnames(dots[[1]]$A) <- paste(colnames(dots[[1]]$A),labels[1],sep=".")
            colnames(dots[[1]]$X) <- paste(colnames(dots[[1]]$X),labels[1],sep=".")
            for(i in 2:length(dots)) {
                colnames(dots[[i]]$A) <- paste(colnames(dots[[i]]$A),labels[i],sep=".")
                colnames(dots[[i]]$X) <- paste(colnames(dots[[i]]$X),labels[i],sep=".")
            }
        }

        result <- dots[[1]]
        for(i in 2:length(dots)) {
            result$A <- cbind(result$A, dots[[i]]$A)
            result$X <- cbind(result$X, dots[[i]]$X)
        }
        class(result) <- "scanoneperm"
        attr(result, "xchr") <- xchr[[1]]
        attr(result, "L") <- L[[1]]
    }
    else {
        for(i in 1:length(dots)) dots[[i]] <- unclass(dots[[i]])

        nr <- sapply(dots, nrow)
        mnr <- max(nr)
        if(any(nr < mnr)) { # pad with NAs
            for(i in which(nr < mnr))
                dots[[i]] <- rbind(dots[[i]], matrix(NA, ncol=ncol(dots[[i]]),
                                                     nrow=mnr-nr[i]))
        }

        thenam <- unlist(lapply(dots, colnames))
        if(length(unique(thenam)) == length(thenam))
            repeats <- FALSE
        else repeats <- TRUE

        if(repeats || gavelabels) {
            colnames(dots[[1]]) <- paste(colnames(dots[[1]]),labels[1],sep=".")
            for(i in 2:length(dots))
                colnames(dots[[i]]) <- paste(colnames(dots[[i]]), labels[i], sep=".")
        }
        result <- dots[[1]]
        for(i in 2:length(dots))
            result <- cbind(result, dots[[i]])

        class(result) <- "scanoneperm"
    }
    result
}


##############################
# subset.scanoneperm: pull out a set of lodcolumns
##############################
subset.scanoneperm <-
    function(x, repl, lodcolumn, ...)
{
    att <- attributes(x)

    if(is.list(x)) {
        if(any(!sapply(x, is.matrix)))
            x <- lapply(x, as.matrix)

        if(missing(lodcolumn)) lodcolumn <- 1:ncol(x[[1]])
        else if(!check_colindex(lodcolumn, x[[1]]))
            stop("lodcolumn misspecified.")

        if(missing(repl)) repl <- 1:nrow(x[[1]])
        else if(!check_rowindex(repl, x[[1]]))
            stop("repl misspecified.")

        cl <- class(x)
        x <- lapply(x, function(a,b,d) unclass(a)[b,d,drop=FALSE], repl, lodcolumn)
        class(x) <- cl
    }
    else {
        if(!is.matrix(x)) x <- as.matrix(x)

        if(missing(lodcolumn)) lodcolumn <- 1:ncol(x)
        else if(!check_colindex(lodcolumn, x))
            stop("lodcolumn misspecified.")

        if(missing(repl)) repl <- 1:nrow(x)
        else if(!check_rowindex(repl, x))
            stop("repl misspecified.")

        cl <- class(x)
        x <- unclass(x)[repl,lodcolumn,drop=FALSE]
        class(x) <- cl
    }

    for(i in seq(along=att)) {
        if(names(att)[i] == "dim" || length(grep("names", names(att)[i]))>0) next
        attr(x, names(att)[i]) <- att[[i]]
    }

    x
}

# subset.scanoneperm using [,]
`[.scanoneperm` <-
    function(x, repl, lodcolumn)
    subset.scanoneperm(x, repl, lodcolumn)

# function to check if column indices are okay
check_colindex <-
    function(index, mat)
{
    if(any(is.na(index))) return(FALSE)

    n <- ncol(mat)

    if(is.logical(index) && length(index) != n)
        return(FALSE)
    if(is.numeric(index)) {
        if(any(index <= 0) && !all(index < 0)) return(FALSE) # don't mix pos and neg
        if(all(index < 0) && any(index < -n)) return(FALSE) # out of bounds
        if(all(index > 0) && any(index > n)) return(FALSE) # out of bounds
    }
    if(is.character(index) && !all(index %in% colnames(mat))) return(FALSE)

    TRUE
}

# function to check if row indices are okay
check_rowindex <-
    function(index, mat)
{
    if(any(is.na(index))) return(FALSE)

    n <- nrow(mat)

    if(is.logical(index) && length(index) != n)
        return(FALSE)
    if(is.numeric(index)) {
        if(any(index <= 0) && !all(index < 0)) return(FALSE) # don't mix pos and neg
        if(all(index < 0) && any(index < -n)) return(FALSE) # out of bounds
        if(all(index > 0) && any(index > n)) return(FALSE) # out of bounds
    }
    if(is.character(index) && !all(index %in% rownames(mat))) return(FALSE)

    TRUE
}

# end of summary.scanone.R

######################################################################
# write.cross.mq.R
#
# copyright (c) 2014, 2015, INRA (author: Timothee Flutre)
# last modified March, 2015
# first written May, 2014
#
# small modification by Karl Broman, 2019
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 3, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but without any warranty; without even the implied warranty of
# merchantability or fitness for a particular purpose.  See the GNU
# General Public License, version 3, for more details.
#
# A copy of the GNU General Public License, version 3, is available
# at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: write.cross.mq, write.cross.mq.loc, write.cross.mq.map,
# and write.cross.mq.qua
#
######################################################################

######################################################################
#
# write.cross.mq: write data from an experimental cross in MapQTL (and
# JoinMap) format.
#
# Three files are written: a "loc" file containing the genotype data,
# a "map" file containing the linkage group assignments and map
# positions, and a "qua" file containing the phenotypes.
#
# File formats are described in the MapQTL manual available online at
# http://www.kyazma.nl/docs/MQ6Manual.pdf
# Only 4-way crosses are supported ("CP" type in MapQTL/JoinMap).
#
######################################################################

write.cross.mq <-
    function(cross, filestem, digits)
{
    if(crosstype(cross) != "4way"){
        msg <- paste("population type", crosstype(cross),
                     "is not supported for writing in MapQTL format (yet)")
        stop(msg, call.=FALSE)
    }
    write.cross.mq.loc(cross, paste0(filestem, ".loc"))
    write.cross.mq.map(cross, paste0(filestem, ".map"), digits)
    write.cross.mq.qua(cross, paste0(filestem, ".qua"), digits)
}

write.cross.mq.loc <-
    function(cross, locfile)
{
    sink(locfile)

    cat(paste0("name = cross", "\n"))
    if(crosstype(cross) == "4way"){
        cat(paste0("popt = ", "CP", "\n"))
    } else{
        sink()
        msg <- paste("population type", crosstype(cross),
                     "is not supported (yet)")
        stop(msg, call.=FALSE)
    }
    cat(paste0("nloc = ", totmar(cross), "\n"))
    cat(paste0("nind = ", nind(cross), "\n"))
    cat("\n")

    for(chr in names(cross$geno)){
        for(marker in colnames(cross$geno[[chr]]$data)){
            cat(marker)
            tmp <- sort(names(table(cross$geno[[chr]]$data[,marker])))
            if(length(tmp) == 4 && all(tmp == c("1","2","3","4"))){
                cat("\t<abxcd>") # arbitrary choice
                cat("\t{00}") # arbitrary choice, could also be {01}, {10}, {11}
                for(i in 1:nind(cross)){
                    if(is.na(cross$geno[[chr]]$data[i, marker])){
                        cat("\t--")
                    } else if(cross$geno[[chr]]$data[i, marker] == 1){
                        cat("\tac")
                    } else if(cross$geno[[chr]]$data[i, marker] == 3){
                        cat("\tad")
                    } else if(cross$geno[[chr]]$data[i, marker] == 2){
                        cat("\tbc")
                    } else if(cross$geno[[chr]]$data[i, marker] == 4){
                        cat("\tbd")
                    }
                }
                cat("\n")
            } else if((length(tmp) == 3 && all(tmp == c("1","10","4"))) ||
                      (length(tmp) == 5 && all(tmp == c("1","10","11","14","4")))){
                cat("\t<hkxhk>")
                cat("\t{00}") # arbitrary choice, could also be {11}
                for(i in 1:nind(cross)){
                    if(is.na(cross$geno[[chr]]$data[i, marker])){
                        cat("\t--")
                    } else if(cross$geno[[chr]]$data[i, marker] == 1){
                        cat("\thh")
                    } else if(cross$geno[[chr]]$data[i, marker] == 10){
                        cat("\thk")
                    } else if(cross$geno[[chr]]$data[i, marker] == 4){
                        cat("\tkk")
                    } else if(cross$geno[[chr]]$data[i, marker] == 14){
                        cat("\th-")
                    } else if(cross$geno[[chr]]$data[i, marker] == 11){
                        cat("\tk-")
                    }
                }
                cat("\n")
            } else if((length(tmp) == 3 && all(tmp == c("2","3","9"))) ||
                      (length(tmp) == 5 && all(tmp == c("12","13","2","3","9")))){
                cat("\t<hkxhk>")
                cat("\t{01}") # arbitrary choice, could also be {10}
                for(i in 1:nind(cross)){
                    if(is.na(cross$geno[[chr]]$data[i, marker])){
                        cat("\t--")
                    } else if(cross$geno[[chr]]$data[i, marker] == 9){
                        cat("\thk")
                    } else if(cross$geno[[chr]]$data[i, marker] == 3){
                        cat("\thh")
                    } else if(cross$geno[[chr]]$data[i, marker] == 2){
                        cat("\tkk")
                    } else if(cross$geno[[chr]]$data[i, marker] == 12){
                        cat("\th-")
                    } else if(cross$geno[[chr]]$data[i, marker] == 13){
                        cat("\tk-")
                    }
                }
                cat("\n")
            } else if(length(tmp) == 2 && all(tmp == c("5","6"))){
                cat("\t<lmxll>")
                cat("\t{0-}") # arbitrary choice, could also be {1-}
                for(i in 1:nind(cross)){
                    if(is.na(cross$geno[[chr]]$data[i, marker])){
                        cat("\t--")
                    } else if(cross$geno[[chr]]$data[i, marker] == 5){
                        cat("\tll")
                    } else if(cross$geno[[chr]]$data[i, marker] == 6){
                        cat("\tlm")
                    }
                }
                cat("\n")
            } else if(length(tmp) == 2 && all(tmp == c("7","8"))){
                cat("\t<nnxnp>")
                cat("\t{-0}") # arbitrary choice, could also be {-1}
                for(i in 1:nind(cross)){
                    if(is.na(cross$geno[[chr]]$data[i, marker])){
                        cat("\t--")
                    } else if(cross$geno[[chr]]$data[i, marker] == 7){
                        cat("\tnn")
                    } else if(cross$geno[[chr]]$data[i, marker] == 8){
                        cat("\tnp")
                    }
                }
                cat("\n")
            } else{
                sink()
                msg <- paste("unrecognized segregation type at marker",
                             marker, "on chromosome", chr)
                stop(msg, call.=FALSE)
            }
        }
    }

    sink()
}

write.cross.mq.map <-
    function(cross, mapfile, digits=NULL)
{
    if(is.matrix(cross$geno[[1]]$map)){
        mapfile.female <- sub(pattern="\\.map",
                              replacement="_female.map",
                              x=mapfile)
        sink(mapfile.female)
        for(chr in names(cross$geno)){
            cat(paste0("group ", chr, "\n"))
            for(m in 1:ncol(cross$geno[[chr]]$map)){
                cat(paste0(colnames(cross$geno[[chr]]$map)[m],
                           "\t",
                           ifelse(is.null(digits),
                                  cross$geno[[chr]]$map[1,m],
                                  round(cross$geno[[chr]]$map[1,m], digits)),
                           "\n"))
            }
            cat("\n")
        }
        sink()
        mapfile.male <- sub(pattern="\\.map",
                            replacement="_male.map",
                            x=mapfile)
        sink(mapfile.male)
        for(chr in names(cross$geno)){
            cat(paste0("group ", chr, "\n"))
            for(m in 1:ncol(cross$geno[[chr]]$map)){
                cat(paste0(colnames(cross$geno[[chr]]$map)[m],
                           "\t",
                           ifelse(is.null(digits),
                                  cross$geno[[chr]]$map[2,m],
                                  round(cross$geno[[chr]]$map[2,m], digits)),
                           "\n"))
            }
            cat("\n")
        }
        sink()
    } else{
        sink(mapfile)
        for(chr in names(cross$geno)){
            cat(paste0("group ", chr, "\n"))
            for(m in 1:length(cross$geno[[chr]]$map)){
                cat(paste0(names(cross$geno[[chr]]$map)[m],
                           "\t",
                           ifelse(is.null(digits),
                                  cross$geno[[chr]]$map[m],
                                  round(cross$geno[[chr]]$map[m], digits)),
                           "\n"))
            }
            cat("\n")
        }
        sink()
    }
}

write.cross.mq.qua <-
    function(cross, quafile, digits=NULL)
{
    sink(quafile)

    cat(paste0("ntrt = ", nphe(cross), "\n"))
    cat(paste0("nind = ", nind(cross), "\n"))
    cat(paste0("miss = ", NA, "\n"))
    cat("\n")

    colnames(cross$pheno) <- gsub(pattern=" ", replacement="_",
                                  x=colnames(cross$pheno))
    if(any(nchar(colnames(cross$pheno)) > 20)){
        sink()
        msg <- paste("phenotype name",
                     colnames(cross$pheno)[which(nchar(colnames(cross$pheno)) > 20)[1]],
                     "is longer than 20 characters")
        stop(msg, call.=FALSE)
    }
    cat(colnames(cross$pheno)[1])
    if(ncol(cross$pheno) > 1)
        for(j in 2:ncol(cross$pheno))
            cat(paste0("\t", colnames(cross$pheno)[j]))
    cat("\n")
    cat("\n")

    if(is.null(digits)){
        write.table(x=cross$pheno, file=quafile, append=TRUE,
                    quote=FALSE, sep="\t", row.names=FALSE, col.names=FALSE)
    } else
        write.table(x=format(x=cross$pheno, digits=digits, trim=TRUE),
                    file=quafile, append=TRUE, quote=FALSE, sep="\t",
                    row.names=FALSE, col.names=FALSE)

    sink()
}

# end of write.cross.mq.R

######################################################################
# stepwiseqtl.R
#
# copyright (c) 2007-2021, Karl W Broman
# last modified Mar, 2021
# first written Nov, 2007
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: stepwiseqtl, calc.plod, countqtlterms, calc.penalties,
#           checkStepwiseqtlStart
#
######################################################################

######################################################################
# stepwiseqtl
#
# perform forward and backward selection to identify multiple QTL
#
# cross:        cross object
# chr:          chromosomes to consider
# pheno.col:    phenotype column
# qtl           (Optional) If given, qtl object used at start of
#               forward selection.  If missing, we start at the null
#               model.
# formula       If given, formula used with the qtl object for the model
#               at the start of forward selection
# max.qtl:      maximum no. QTL in forward selection
# covar:        data.frame with covariates (strictly additive at this point)
# method:       imputation or Haley-Knott regression
# incl.markers: If TRUE, include marker positions in scan; if FALSE,
#               just use the grid
# refine.locations:  If TRUE, refine the QTL positions at each step
# additive.only: If TRUE, don't scan for interactions
# scan.pairs:    If TRUE, do a pairscan at each step
# penalties:     Vector with 3 values: the penalties on main effects
#                followed by the heavy and light interaction penalties.
#                (if missing, we use default values derived via
#                simulation)
# keeplodprofile If TRUE, perform one last pass of refineqtl and save
#                the LOD profiles.
# keeptrace      If TRUE, retain the QTL locations, model formula and pLOD
#                for the best model from each step of forward and backward
#                selection as an attribute in the output
# verbose:    If TRUE, print a bunch of tracing information
######################################################################
stepwiseqtl <-
    function(cross, chr, pheno.col=1, qtl, formula, max.qtl=10, covar=NULL,
             method=c("imp", "hk"), model=c("normal", "binary"), incl.markers=TRUE, refine.locations=TRUE,
             additive.only=FALSE, scan.pairs=FALSE, penalties,
             keeplodprofile=TRUE, keeptrace=FALSE, verbose=TRUE,
             tol=1e-4, maxit=1000, require.fullrank=FALSE)
{
    if(!inherits(cross, "cross"))
        stop("Input should have class \"cross\".")

    if(!missing(chr)) cross <- subset(cross, chr)
    if(missing(qtl)) qtl <- NULL
    if(missing(formula)) formula <- NULL

    method <- match.arg(method)
    model <- match.arg(model)

    # force covar to be a data frame
    if(!is.null(covar) && !is.data.frame(covar)) {
        if(is.matrix(covar) && is.numeric(covar))
            covar <- as.data.frame(covar, stringsAsFactors=TRUE)
        else stop("covar should be a data.frame")
    }

    if(!missing(penalties)) {
        if(is.matrix(penalties)) {
            penalties <- penalties[1,]
            warning("penalties should be a vector; only the first row will be used")
        }
        if(length(penalties)==6) { # X-chr-specific penalties
            chr_type <- vapply(cross$geno, chrtype, "")
            if(!all(chr_type=="A")) {
                if(scan.pairs)
                    warning("scan.pairs=TRUE not implemented X-chr specific penalties; ignored.")
                return(stepwiseqtlX(cross, chrnames(cross), pheno.col=pheno.col, qtl=qtl,
                                     formula=formula, max.qtl=max.qtl, k_f=3, stop.rule=0,
                                     covar=covar, method=method, model=model, incl.markers=incl.markers,
                                     refine.locations=refine.locations, additive.only=additive.only,
                                     penalties=penalties, keeplodprofile=keeplodprofile, keeptrace=keeptrace,
                                     verbose=verbose, tol=tol, maxit=maxit, require.fullrank=require.fullrank))
            }
            penalties <- penalties[c(1,3,4)] # just the autosomal penalties
        }
    }

    if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
        cross$pheno <- cbind(pheno.col, cross$pheno)
        pheno.col <- 1
    }

    chr_type <- sapply(cross$geno, chrtype)
    if(any(chr_type=="X")) {
        Xadjustment <- scanoneXnull(crosstype(cross), getsex(cross), attributes(cross))
        forceXcovar <- Xadjustment$adjustX
        Xcovar <- Xadjustment$sexpgmcovar
    }
    else forceXcovar <- FALSE

    if(!is.null(qtl)) { # start f.s. at somewhere other than the null
        if( !inherits(qtl, "qtl") )
            stop("The qtl argument must be an object of class \"qtl\".")

        # check that chromosomes were retained, otherwise give error
        m <- is.na(match(qtl$chr, names(cross$geno)))
        if(any(m)) {
            wh <- qtl$chr[m]
            if(length(wh) > 1)
                stop("Chromosomes ", paste(wh, collapse=", "), " (in QTL object) not in cross object.")
            else
                stop("Chromosome ", wh, " (in QTL object) not in cross object.")
        }
        if(is.null(formula)) { # create a formula with all covariates and all QTL add've
            if(!is.null(covar))
                formula <- paste("y ~ ", paste(names(covar), collapse="+"), "+")
            else
                formula <- "y ~ "
            formula <- paste(formula, paste(paste("Q", 1:length(qtl$chr), sep=""), collapse="+"))
        }
        else {
            temp <- checkStepwiseqtlStart(qtl, formula, covar)
            qtl <- temp$qtl
            formula <- temp$formula
        }
        startatnull <- FALSE
    }
    else {
        if(!is.null(formula))
            warning("formula ignored if qtl is not provided.")
        startatnull <- TRUE
    }

    # revise names in qtl object
    if(!startatnull)
        qtl$name <- qtl$altname

    # check that we have the right stuff for the selected method
    if(method=="imp") {
        if(!("draws" %in% names(cross$geno[[1]]))) {
            if("prob" %in% names(cross$geno[[1]])) {
                warning("The cross doesn't contain imputations; using method=\"hk\".")
                method <- "hk"
            }
            else
                stop("You need to first run sim.geno.")
        }
    }
    else {
        if(!("prob" %in% names(cross$geno[[1]]))) {
            if("draws" %in% names(cross$geno[[1]])) {
                warning("The cross doesn't contain QTL genotype probabilities; using method=\"imp\".")
                method <- "imp"
            }
            else
                stop("You need to first run calc.genoprob.")
        }
    }
    if(method=="imp") qtlmethod <- "draws"
    else qtlmethod <- "prob"

    if(!is.null(qtl) && qtl$n.ind != nind(cross)) {
        map <- attr(qtl, "map") # save map
        warning("No. individuals in qtl object doesn't match that in the input cross; re-creating qtl object.")
        if(method=="imp")
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        else
            qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="prob")
        attr(qtl, "map") <- map
    }

    if(!is.null(qtl) && method=="imp" && dim(qtl$geno)[3] != dim(cross$geno[[1]]$draws)[3])  {
        map <- attr(qtl, "map") # save map
        warning("No. imputations in qtl object doesn't match that in the input cross; re-creating qtl object.")
        qtl <- makeqtl(cross, qtl$chr, qtl$pos, qtl$name, what="draws")
        attr(qtl, "map") <- map
    }

    # check that qtl object matches the method
    if(!startatnull) {
        if(method=="imp" && !("geno" %in% names(qtl)))
            stop("The qtl object doesn't contain imputations; re-run makeqtl with what=\"draws\".")
        else if(method=="hk" && !("prob" %in% names(qtl)))
            stop("The qtl object doesn't contain QTL genotype probabilities; re-run makeqtl with what=\"prob\".")
    }

    # check phenotypes and covariates; drop ind'ls with missing values
    if(length(pheno.col) > 1) {
        pheno.col <- pheno.col[1]
        warning("stepwiseqtl can take just one phenotype; only the first will be used")
    }
    if(is.character(pheno.col)) {
        num <- find.pheno(cross, pheno.col)
        if(is.na(num))
            stop("Couldn't identify phenotype \"", pheno.col, "\"")
        pheno.col <- num
    }
    if(any(pheno.col < 1 | pheno.col > nphe(cross)))
        stop("pheno.col values should be between 1 and the no. phenotypes")
    pheno <- cross$pheno[,pheno.col]
    if(!is.null(covar)) phcovar <- cbind(pheno, covar)
    else phcovar <- as.data.frame(pheno, stringsAsFactors=TRUE)
    hasmissing <- rowSums(is.na(phcovar)) > 0
    if(all(hasmissing))
        stop("All individuals are missing phenotypes or covariates.")
    if(any(hasmissing)) {
        pheno <- pheno[!hasmissing]
        cross <- subset(cross, ind=!hasmissing)
        if(!is.null(covar)) covar <- covar[!hasmissing,,drop=FALSE]
        if(forceXcovar) Xcovar <- Xcovar[!hasmissing,,drop=FALSE]

        if(!startatnull) {
            if(method=="imp")
                qtl$geno <- qtl$geno[!hasmissing,,,drop=FALSE]
            else {
                for(i in seq(along=qtl$prob))
                    qtl$prob[[i]] <- qtl$prob[[i]][!hasmissing,,drop=FALSE]
            }
            qtl$n.ind <- sum(!hasmissing)
        }
    }

    if(max.qtl < 1)
        stop("Need max.qtl > 0 if we are to scan for qtl")

    if(is.null(covar)) {
        lod0 <- 0
        if(startatnull)
            firstformula <- y~Q1
        else firstformula <- formula
    }
    else {
        nullformula <- as.formula(paste("y~", paste(names(covar), collapse="+")))
        tempqtl <- makeqtl(cross, chrnames(cross)[1], 0, what=ifelse(method=="imp", "draws", "prob"))
        fit <- fitqtl(cross, pheno.col, tempqtl, covar=covar, formula=nullformula,
                      method=method, model=model, dropone=FALSE, get.ests=FALSE,
                      run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
        lod0 <- fit$result.full[1,4]
        if(startatnull)
            firstformula <- as.formula(paste("y~", paste(names(covar), collapse="+"), "+", "Q1"))
        else firstformula <- formula
    }


    # penalties
    cross.type <- crosstype(cross)
    if(missing(penalties)) {
        if(cross.type=="f2") {
            penalties <-  c(3.52, 4.28, 2.69)
        }
        else if(cross.type=="bc") {
            penalties <-  c(2.69, 2.62, 1.19)
        }
        else
            stop("No default penalties available for cross type ", cross.type)
    }
    else if(length(penalties) != 3) {
        if(length(penalties)==1) {
            if(additive.only)
                penalties <- c(penalties,Inf,Inf)
            else
                stop("You must include a penalty for interaction terms.")
        }
        else {
            if(length(penalties)==2)
                penalties <- penalties[c(1,2,2)]
            else {
                warning("penalties should have length 3")
                penalties <- penalties[1:3]
            }
        }
    }

    if(verbose > 2) verbose.scan <- TRUE
    else verbose.scan <- FALSE

    curbest <- NULL
    curbestplod <- 0

    # initial scan : either 1d or 2d
    if(verbose) cat(" -Initial scan\n")
    if(startatnull) {

        if(forceXcovar) {
            if(is.null(covar)) covar.w.X <- Xcovar
            else covar.w.X <- cbind(covar, Xcovar)
        }
        else covar.w.X <- covar

        if(additive.only || max.qtl == 1 || !scan.pairs) {
            suppressWarnings(out <- scanone(cross, pheno.col=pheno.col, method=method, model=model,
                                            addcovar=covar.w.X))
            lod <- max(out[,3], na.rm=TRUE)
            if(verbose) cat("initial lod: ", lod, "\n")

            curplod <- calc.plod(lod, c(1,0,0), penalties=penalties)
            wh <- which(!is.na(out[,3]) & out[,3]==lod)
            if(length(wh) > 1) wh <- sample(wh, 1)

            qtl <- makeqtl(cross, as.character(out[wh,1]), out[wh,2], "Q1",
                           what=qtlmethod)
            formula <- firstformula
            n.qtl <- 1
        }
        else {
            suppressWarnings(out <- scantwo(cross, pheno.col=pheno.col, method=method, model=model,
                                            incl.markers=incl.markers, addcovar=covar.w.X, verbose=verbose.scan))
            lod <- out$lod

            lod1 <- max(diag(lod), na.rm=TRUE)
            plod1 <- calc.plod(lod1, c(1,0,0), penalties=penalties)
            loda <- max(lod[upper.tri(lod)], na.rm=TRUE)
            ploda <- calc.plod(loda, c(2,0,0),
                               penalties=penalties)
            lodf <- max(lod[lower.tri(lod)], na.rm=TRUE)
            plodf <- calc.plod(lodf, c(2,0,1),
                               penalties=penalties)

            if(plod1 > ploda && plod1 > plodf) {
                wh <- which(!is.na(diag(lod)) & diag(lod) == lod1)
                if(length(wh) > 1) wh <- sample(wh, 1)
                m <- out$map[wh,]
                qtl <- makeqtl(cross, as.character(m[1,1]), m[1,2], "Q1", what=qtlmethod)
                formula <- firstformula
                n.qtl <- 1
                lod <- lod1
                curplod <- plod1
            }
            else if(ploda > plodf) {
                temp <- max(out, what="add")
                if(nrow(temp) > 1)
                    temp <- temp[sample(1:nrow(temp),1),]
                qtl <- makeqtl(cross, c(as.character(temp[1,1]), as.character(temp[1,2])),
                               c(temp[1,3], temp[1,4]), c("Q1","Q2"), what=qtlmethod)
                formula <- as.formula(paste(deparseQTLformula(firstformula), "+Q2", sep=""))
                curplod <- ploda
                lod <- loda
                n.qtl <- 2
            }
            else {
                temp <- max(out, what="full")
                if(nrow(temp) > 1)
                    temp <- temp[sample(1:nrow(temp),1),]
                qtl <- makeqtl(cross, c(as.character(temp[1,1]), as.character(temp[1,2])),
                               c(temp[1,3], temp[1,4]), c("Q1","Q2"), what=qtlmethod)
                formula <- as.formula(paste(deparseQTLformula(firstformula), "+Q2+Q1:Q2", sep=""))
                curplod <- plodf
                lod <- lodf
                n.qtl <- 2
            }
        }
    } # start at null
    else {
        if(verbose) cat(" ---Starting at a model with", length(qtl$chr), "QTL\n")
        if(refine.locations) {
            if(verbose) cat(" ---Refining positions\n")
            rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                              covar=covar, formula=formula, method=method,
                              model=model,
                              verbose=verbose.scan, incl.markers=incl.markers,
                              keeplodprofile=FALSE, forceXcovar=forceXcovar)
            if(any(rqtl$pos != qtl$pos)) { # updated positions
                if(verbose) cat(" ---  Moved a bit\n")
            }
            qtl <- rqtl
        }
        fit <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                      method=method, model=model, dropone=FALSE, get.ests=FALSE,
                      run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
        lod <- fit$result.full[1,4] - lod0

        if(require.fullrank && attr(fit, "matrix.rank") < attr(fit, "matrix.ncol")) lod <- 0
        curplod <- calc.plod(lod, countqtlterms(formula, ignore.covar=TRUE),
                             penalties=penalties)
        attr(qtl, "pLOD") <- curplod
        n.qtl <- length(qtl$chr)
    }

    attr(qtl, "formula") <- deparseQTLformula(formula)
    attr(qtl, "pLOD") <- curplod

    if(curplod > 0) {
        curbest <- qtl
        curbestplod <- curplod

        if(verbose)
            cat("** new best ** (pLOD increased by ", round(curplod, 4), ")\n", sep="")
    }

    if(keeptrace) {
        temp <- list(chr=qtl$chr, pos=qtl$pos)
        attr(temp, "formula") <- deparseQTLformula(formula)
        attr(temp, "pLOD") <- curplod
        class(temp) <- c("compactqtl", "list")
        thetrace <- list("0"=temp)
    }

    if(verbose)
        cat("    no.qtl = ", n.qtl, "  pLOD =", curplod, "  formula:",
            deparseQTLformula(formula), "\n")
    if(verbose > 1)
        cat("         qtl:", paste(qtl$chr, round(qtl$pos,1), sep="@"), "\n")

    # start stepwise search
    i <- 0
    while(n.qtl < max.qtl) {
        i <- i+1

        if(verbose) {
            cat(" -Step", i, "\n")
            cat(" ---Scanning for additive qtl\n")
        }

        out <- addqtl(cross, pheno.col=pheno.col, qtl=qtl, covar=covar,
                      formula=formula, method=method, model=model, incl.markers=incl.markers,
                      verbose=verbose.scan, forceXcovar=forceXcovar,
                      require.fullrank=require.fullrank)

        curlod <- max(out[,3], na.rm=TRUE)
        wh <- which(!is.na(out[,3]) & out[,3]==curlod)
        if(length(wh) > 1) wh <- sample(wh,1)
        curqtl <- addtoqtl(cross, qtl, as.character(out[wh,1]), out[wh,2],
                           paste("Q", n.qtl+1, sep=""))
        curformula <- as.formula(paste(deparseQTLformula(formula), "+Q", n.qtl+1, sep=""))
        curlod <- curlod + lod
        curplod <- calc.plod(curlod, countqtlterms(curformula, ignore.covar=TRUE),
                             penalties=penalties)
        if(verbose) cat("        plod =", curplod, "\n")

        curnqtl <- n.qtl+1

        if(!additive.only) {
            for(j in 1:n.qtl) {

                if(verbose)
                    cat(" ---Scanning for QTL interacting with Q", j, "\n", sep="")

                thisformula <- as.formula(paste(deparseQTLformula(formula), "+Q", n.qtl+1,
                                                "+Q", j, ":Q", n.qtl+1, sep=""))
                out <- addqtl(cross, pheno.col=pheno.col, qtl=qtl, covar=covar,
                              formula=thisformula, method=method, model=model, incl.markers=incl.markers,
                              verbose=verbose.scan, forceXcovar=forceXcovar,
                              require.fullrank=require.fullrank)
                thislod <- max(out[,3], na.rm=TRUE)

                wh <- which(!is.na(out[,3]) & out[,3]==thislod)
                if(length(wh) > 1) wh <- sample(wh,1)
                thisqtl <- addtoqtl(cross, qtl, as.character(out[wh,1]), out[wh,2],
                                    paste("Q", n.qtl+1, sep=""))

                thislod <- thislod + lod
                thisplod <- calc.plod(thislod, countqtlterms(thisformula, ignore.covar=TRUE),
                                      penalties=penalties)
                if(verbose) cat("        plod =", thisplod, "\n")

                if(thisplod > curplod) {
                    curformula <- thisformula
                    curplod <- thisplod
                    curlod <- thislod
                    curqtl <- thisqtl

                    curnqtl <- n.qtl+1
                }
            }

            if(n.qtl > 1) {
                if(verbose)
                    cat(" ---Look for additional interactions\n")
                temp <- addint(cross, pheno.col, qtl, covar=covar, formula=formula,
                               method=method, model=model, qtl.only=TRUE, verbose=verbose.scan,
                               require.fullrank=require.fullrank)
                if(!is.null(temp)) {
                    thislod <- max(temp[,3], na.rm=TRUE)
                    wh <- which(!is.na(temp[,3]) & temp[,3] == thislod)
                    if(length(wh) > 1) wh <- sample(wh, 1)
                    thisformula <- as.formula(paste(deparseQTLformula(formula), "+", rownames(temp)[wh]))
                    thislod <- thislod + lod
                    thisplod <- calc.plod(thislod, countqtlterms(thisformula, ignore.covar=TRUE),
                                          penalties=penalties)
                    if(verbose) cat("        plod =", thisplod, "\n")
                    if(thisplod > curplod) {
                        curformula <- thisformula
                        curplod <- thisplod
                        curlod <- thislod
                        curqtl <- qtl
                        curnqtl <- n.qtl
                    }
                }
            }

            if(scan.pairs) {
                if(verbose)
                    cat(" ---Scan for an additional pair\n")
                out <- addpair(cross, pheno.col=pheno.col, qtl=qtl, covar=covar,
                               formula=formula, method=method, model=model, incl.markers=incl.markers,
                               verbose=verbose.scan, forceXcovar=forceXcovar)
                thelod <- out$lod

                loda <- max(thelod[upper.tri(thelod)], na.rm=TRUE)
                ploda <- calc.plod(loda+lod, c(2,0,0,0)+countqtlterms(formula, ignore.covar=TRUE),
                                   penalties=penalties)
                lodf <- max(thelod[lower.tri(thelod)], na.rm=TRUE)
                plodf <- calc.plod(lodf+lod, c(2,0,1,1)+countqtlterms(formula, ignore.covar=TRUE),
                                   penalties=penalties)

                if(verbose) {
                    cat("        ploda =", ploda, "\n")
                    cat("        plodf =", plodf, "\n")
                }

                if(ploda > curplod && loda > plodf) {
                    temp <- max(out, what="add")
                    if(nrow(temp) > 1)
                        temp <- temp[sample(1:nrow(temp),1),]
                    curqtl <- addtoqtl(cross, qtl, c(as.character(temp[1,1]), as.character(temp[1,2])),
                                       c(temp[1,3], temp[1,4]), paste("Q", n.qtl+1:2, sep=""))
                    curformula <- as.formula(paste(deparseQTLformula(formula), "+Q", n.qtl+1, "+Q",
                                                   n.qtl+2, sep=""))
                    curplod <- ploda
                    lod <- loda+lod
                    curnqtl <- n.qtl+2
                }
                else if(plodf > curplod) {
                    temp <- max(out, what="full")
                    if(nrow(temp) > 1)
                        temp <- temp[sample(1:nrow(temp),1),]

                    curqtl <- addtoqtl(cross, qtl, c(as.character(temp[1,1]), as.character(temp[1,2])),
                                       c(temp[1,3], temp[1,4]), paste("Q", n.qtl+1:2, sep=""))
                    curformula <- as.formula(paste(deparseQTLformula(formula), "+Q", n.qtl+1, "+Q",
                                                   n.qtl+2, "+Q", n.qtl+1, ":Q", n.qtl+2,
                                                   sep=""))
                    curplod <- plodf
                    lod <- lodf+lod
                    curnqtl <- n.qtl+2
                }
            }

        }

        qtl <- curqtl
        n.qtl <- curnqtl
        attr(qtl, "formula") <- deparseQTLformula(curformula)
        attr(qtl, "pLOD") <- curplod
        formula <- curformula
        lod <- curlod

        if(refine.locations) {
            if(verbose) cat(" ---Refining positions\n")
            rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                              covar=covar, formula=formula, method=method, model=model,
                              verbose=verbose.scan, incl.markers=incl.markers,
                              keeplodprofile=FALSE, forceXcovar=forceXcovar)
            if(any(rqtl$pos != qtl$pos)) { # updated positions
                if(verbose) cat(" ---  Moved a bit\n")
                qtl <- rqtl
                fit <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                              method=method, model=model, dropone=FALSE, get.ests=FALSE,
                              run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
                lod <- fit$result.full[1,4] - lod0
                if(require.fullrank && attr(fit, "matrix.rank") < attr(fit, "matrix.ncol")) lod <- 0
                curplod <- calc.plod(lod, countqtlterms(formula, ignore.covar=TRUE),
                                     penalties=penalties)
                attr(qtl, "pLOD") <- curplod
            }

        }

        if(verbose)
            cat("    no.qtl = ", n.qtl, "  pLOD =", curplod, "  formula:",
                deparseQTLformula(formula), "\n")
        if(verbose > 1)
            cat("         qtl:", paste(qtl$chr, round(qtl$pos,1), sep="@"), "\n")


        if(curplod > curbestplod) {
            if(verbose)
                cat("** new best ** (pLOD increased by ", round(curplod - curbestplod, 4),
                    ")\n", sep="")

            curbest <- qtl
            curbestplod <- curplod
        }

        if(keeptrace) {
            temp <- list(chr=qtl$chr, pos=qtl$pos)
            attr(temp, "formula") <- deparseQTLformula(formula)
            attr(temp, "pLOD") <- curplod
            class(temp) <- c("compactqtl", "list")
            temp <- list(temp)
            names(temp) <- i
            thetrace <- c(thetrace, temp)
        }

        if(n.qtl >= max.qtl) break
    }

    if(verbose) cat(" -Starting backward deletion\n")

    while(n.qtl > 1) {
        i <- i+1
        out <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                      method=method, model=model, dropone=TRUE, get.ests=FALSE,
                      run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)$result.drop

        formulas <- attr(out, "formulas")
        lods <- attr(out, "lods")

        rn <- rownames(out)
        # ignore things with covariates
        wh <- c(grep("^[Qq][0-9]+$", rn),
                grep("^[Qq][0-9]+:[Qq][0-9]+$", rn))
        out <- out[wh,,drop=FALSE]
        formulas <- formulas[wh]
        lods <- lods[wh]

        # need to calculate penalized LOD scores here
        plod <- rep(NA, length(lods))
        for(modi in seq(along=plod))
            plod[modi] <- calc.plod(lods[modi], countqtlterms(formulas[modi], ignore.covar=TRUE),
                                    penalties=penalties)

        maxplod <- max(plod, na.rm=TRUE)

        wh <- which(!is.na(plod) & plod==maxplod)
        if(length(wh) > 1) wh <- sample(wh, 1)

        todrop <- rownames(out)[wh]

        if(verbose) cat(" ---Dropping", todrop, "\n")

        if(length(grep(":", todrop)) > 0) { # dropping an interaction
            theterms <- attr(terms(formula), "factors")
            wh <- colnames(theterms)==todrop
            if(!any(wh)) stop("Confusion about what interation to drop!")
            theterms <- colnames(theterms)[!wh]
            formula <- as.formula(paste("y~", paste(theterms, collapse="+")))
        }
        else {
            numtodrop <- as.numeric(substr(todrop, 2, nchar(todrop)))

            theterms <- attr(terms(formula), "factors")
            cn <- colnames(theterms)
            g <- c(grep(paste("^[Qq]", numtodrop, "$", sep=""), cn),
                   grep(paste("^[Qq]", numtodrop, ":", sep=""), cn),
                   grep(paste(":[Qq]", numtodrop, "$", sep=""), cn))
            cn <- cn[-g]
            formula <- as.formula(paste("y~", paste(cn, collapse="+")))

            if(n.qtl > numtodrop) {
                for(j in (numtodrop+1):n.qtl)
                    formula <- reviseqtlnuminformula(formula, j, j-1)
            }

            qtl <- dropfromqtl(qtl, index=numtodrop)
            qtl$name <- qtl$altname <- paste("Q", 1:qtl$n.qtl, sep="")
            n.qtl <- n.qtl - 1
        }

        # call fitqtl again, just in case
        fit <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                      method=method, model=model, dropone=FALSE, get.ests=FALSE,
                      run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
        lod <- fit$result.full[1,4] - lod0
        if(require.fullrank && attr(fit, "matrix.rank") < attr(fit, "matrix.ncol")) lod <- 0

        curplod <- calc.plod(lod, countqtlterms(formula, ignore.covar=TRUE),
                             penalties=penalties)

        if(verbose)
            cat("    no.qtl = ", n.qtl, "  pLOD =", curplod, "  formula:",
                deparseQTLformula(formula), "\n")
        if(verbose > 1)
            cat("         qtl:", paste(qtl$chr, round(qtl$pos,1), sep=":"), "\n")

        attr(qtl, "formula") <- deparseQTLformula(formula)
        attr(qtl, "pLOD") <- curplod

        if(refine.locations) {
            if(verbose) cat(" ---Refining positions\n")
            if(!is.null(qtl)) {
                rqtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                                  covar=covar, formula=formula, method=method, model=model,
                                  verbose=verbose.scan, incl.markers=incl.markers,
                                  keeplodprofile=FALSE, forceXcovar=forceXcovar)
                if(any(rqtl$pos != qtl$pos)) { # updated positions
                    if(verbose) cat(" ---  Moved a bit\n")
                    qtl <- rqtl
                    fit <- fitqtl(cross, pheno.col, qtl, covar=covar, formula=formula,
                                  method=method, model=model, dropone=FALSE, get.ests=FALSE,
                                  run.checks=FALSE, tol=tol, maxit=maxit, forceXcovar=forceXcovar)
                    lod <- fit$result.full[1,4] - lod0
                    if(require.fullrank && attr(fit, "matrix.rank") < attr(fit, "matrix.ncol")) lod <- 0
                    curplod <- calc.plod(lod, countqtlterms(formula, ignore.covar=TRUE),
                                         penalties=penalties)
                    attr(qtl, "pLOD") <- curplod
                }
            }
        }

        if(curplod > curbestplod) {
            if(verbose)
                cat("** new best ** (pLOD increased by ", round(curplod - curbestplod, 4),
                    ")\n", sep="")

            curbestplod <- curplod
            curbest <- qtl
        }

        if(keeptrace) {
            temp <- list(chr=qtl$chr, pos=qtl$pos)
            attr(temp, "formula") <- deparseQTLformula(formula)
            attr(temp, "pLOD") <- curplod
            class(temp) <- c("compactqtl", "list")
            temp <- list(temp)
            names(temp) <- i
            thetrace <- c(thetrace, temp)
        }
    }

    # re-form the qtl
    if(!is.null(curbest)) {
        chr <- curbest$chr
        pos <- curbest$pos
        o <- order(factor(chr, levels=names(cross$geno)), pos)

        qtl <- makeqtl(cross, chr[o], pos[o], what=qtlmethod)

        # need to redo numbering in formula
        formula <- as.formula(attr(curbest, "formula"))

        if(length(chr) > 1) {
            n.qtl <- length(chr)
            for(i in 1:n.qtl)
                formula <- reviseqtlnuminformula(formula, i, n.qtl+i)
            for(i in 1:n.qtl)
                formula <- reviseqtlnuminformula(formula, n.qtl+o[i], i)
        }

        if(keeplodprofile) {
            if(verbose) cat(" ---One last pass through refineqtl\n")
            qtl <- refineqtl(cross, pheno.col=pheno.col, qtl=qtl,
                             covar=covar, formula=formula, method=method, model=model,
                             verbose=verbose.scan, incl.markers=incl.markers,
                             keeplodprofile=TRUE, forceXcovar=forceXcovar)
        }
        attr(qtl, "formula") <- deparseQTLformula(formula)
        attr(qtl, "pLOD") <- attr(curbest, "pLOD")
        curbest <- qtl
    }
    else {
        curbest <- numeric(0)
        class(curbest) <- "qtl"
        attr(curbest,"pLOD") <- 0
    }

    if(keeptrace)
        attr(curbest, "trace") <- thetrace

    attr(curbest, "formula") <- deparseQTLformula(attr(curbest, "formula"), TRUE)
    attr(curbest, "penalties") <- penalties

    curbest
}


######################################################################
# check initial qtl model for appropriateness
######################################################################
checkStepwiseqtlStart <-
    function(qtl, formula, covar=NULL)
{
    if(is.character(formula)) formula <- as.formula(formula)

    formula <- checkformula(formula, qtl$altname, colnames(covar))
    theterms <- attr(terms(formula), "factors")[-1,,drop=FALSE]
    rn <- rownames(theterms)

    # make sure that all covariates in covar exist in the formula
    if(!is.null(covar)) {
        covarnam <- colnames(covar)
        m <- is.na(match(covarnam, rn))
        if(any(m)) {
            toadd <- covarnam[m]
            warning("Adding ", paste(toadd, collapse="+"), " to formula")
            formula <- as.formula(paste(deparseQTLformula(formula), "+", paste(toadd, collapse="+"), sep=""))
            theterms <- attr(terms(formula), "factors")[-1,,drop=FALSE]
            rn <- rownames(theterms)
        }

        # make sure there are no QTL:covariate interactions
        theqtl <- grep("^Q[0-9]+$", rn)
        thecovar <- seq(along=rn)[-theqtl]

        if(any(apply(theterms[thecovar,,drop=FALSE], 1, sum)>1))
            stop("We can't yet handle QTL:covariate or covariate:covariate interactions")
    }

    # make sure that any QTL in formula exist in object
    theqtl <- grep("^Q[0-9]+$", rn)
    thecovar <- seq(along=rn)[-theqtl]
    qtlindex <- as.numeric(substr(rn[theqtl], 2, nchar(rn[theqtl])))
    wh <- qtlindex < 0 | qtlindex > length(qtl$chr)
    if(any(wh))
        stop("QTL ", paste(rn[theqtl][wh], collapse=" "), " not in qtl object")


    # make sure that there are not any extraneous terms
    if(length(thecovar) > 0) {
        if(is.null(covar))
            stop("Extraneous terms in formula: ", paste(rn[thecovar], collapse=" "))
        else {
            wh <- is.na(match(rn[thecovar], colnames(covar)))
            if(any(wh))
                stop("Extraneous terms in formula: ", paste(rn[thecovar][wh], collapse=" "))
        }
    }

    # if any QTL not referred to in formula, drop them from the QTL object
    todrop <- seq(along=qtl$chr)[-qtlindex]
    if(length(todrop) > 0) {
        oldnum <- seq(along=qtl$chr)[-todrop]
        newnum <- order(oldnum)

        formula <- reviseqtlnuminformula(formula, oldnum, newnum)
        qtl <- dropfromqtl(qtl, todrop)
    }

    return(list(qtl=qtl, formula=as.formula(formula)))
}


######################################################################
# penalized LOD score
######################################################################
calc.plod <-
    function(lod, nterms, type=c("f2","bc"), penalties) {
        nterms <- nterms[1:3]
        if(any(penalties==Inf & nterms > 0)) return(-Inf)

        as.numeric(lod - sum((nterms*penalties)[nterms > 0]))
    }

######################################################################
# count terms in a model, for use by plod
######################################################################
countqtlterms <-
    function(formula, ignore.covar=TRUE)
{
    if(is.character(formula)) formula <- as.formula(formula)
    factors <- attr(terms(formula), "factors")[-1,,drop=FALSE]
    if(any(factors > 1))  {
        warning("some formula terms > 1; may be a problem with the formula:\n    ", deparseQTLformula(formula))
        factors[factors > 1] <- 1
    }
    nterm <- apply(factors, 2, sum)

    if(any(nterm>2))
        stop("Can't deal with higher-order interactions\n")

    # need to check for QTL x covariate interactions in here!

    if(ignore.covar) {
        cn <- colnames(factors)
        wh <- c(grep("^[Qq][0-9]+$", cn),
                grep("^[Qq][0-9]+:[Qq][0-9]+$", cn))
        rn <- rownames(factors)
        wh2 <- c(grep("^[Qq][0-9]+$", rn),
                 grep("^[Qq][0-9]+:[Qq][0-9]+$", rn))
        factors <- factors[wh2,wh, drop=FALSE]
    }
    nterm <- apply(factors, 2, sum)

    nmain <- sum(nterm==1)

    if(all(nterm==1))
        return(c(main=nmain, intH=0, intL=0, inttot=0))

    n.int <- sum(nterm==2)

    if(n.int <=1) # 0 or 1 interactions, so no need to figure them out
        return(c(main=nmain, intH=0, intL=n.int, inttot=n.int))

    factors <- factors[,nterm==2, drop=FALSE]

    wh <- apply(factors, 2, function(a) which(a==1))

    u <- sort(unique(as.numeric(wh)))
    grp <- rep(NA, length(u))
    names(grp) <- u

    ngrp <- 0
    nint <- NULL

    for(i in 1:ncol(wh)) {
        thegrp <- grp[as.character(wh[,i])]
        if(all(!is.na(thegrp))) {
            nint[as.character(thegrp[1])] <-
                sum(nint[unique(as.character(thegrp))]) + 1
            grp[grp==thegrp[1] | grp==thegrp[2]] <- thegrp[1]
        }
        else if(any(!is.na(thegrp))) {
            grp[as.character(wh[,i])] <- thegrp[!is.na(thegrp)]
            nint[as.character(thegrp[!is.na(thegrp)])] <-
                nint[as.character(thegrp[!is.na(thegrp)])] + 1
        }
        else {
            ngrp <- ngrp+1
            grp[as.character(wh[,i])] <- ngrp
            nint[as.character(ngrp)] <- 1
        }
    }

    nint <- nint[as.character(unique(grp))]
    nL <- sum(nint>0)
    nH <- sum(nint)-nL
    c(main=nmain, intH=nH, intL=nL, inttot=n.int)
}

######################################################################
# calculate penalties for pLOD using scantwo permutation results.
######################################################################
calc.penalties <-
    function(perms, alpha=0.05, lodcolumn)
{
    if(missing(perms) || !inherits(perms, "scantwoperm"))
        stop("You must include permutation results from scantwo.")

    if("AA" %in% names(perms)) { # X-chr-specific penalties
        if(missing(lodcolumn)) lodcolumn <- NULL
        return(calc.penalties.X(perms, alpha, lodcolumn))
    }

    if(missing(lodcolumn) || is.null(lodcolumn)) {
        if(is.matrix(perms[[1]]) && ncol(perms[[1]]) > 1)
            lodcolumn <- 1:ncol(perms[[1]])
        else lodcolumn <- 1
    }

    if(length(lodcolumn)>1) {
        result <- NULL
        for(i in seq(along=lodcolumn)) {
            temp <- calc.penalties(perms, alpha, lodcolumn[i])
            result <- rbind(result, temp)
        }
        dimnames(result) <- list(colnames(perms[[1]])[lodcolumn], names(temp))
        return(result)
    }

    if(is.matrix(perms[[1]]) && ncol(perms[[1]]) >1) {
        if(lodcolumn < 1 || lodcolumn > ncol(perms[[1]]))
            stop("lodcolumn misspecified")
        for(i in seq(along=perms))
            perms[[i]] <- perms[[i]][,lodcolumn,drop=FALSE]
    }

    qu <- summary(perms, alpha=alpha)
    if(!("one" %in% names(qu)))
        stop("You need to re-run scantwo permutations with R/qtl version >= 1.09.")

    if(length(alpha)>1) {
        penalties <- cbind(qu[["one"]], qu[["int"]], qu[["fv1"]]-qu[["one"]])
        colnames(penalties) <- c("main","heavy", "light")
    }
    else {
        penalties <- c(qu[["one"]], qu[["int"]], qu[["fv1"]]-qu[["one"]])
        names(penalties) <- c("main","heavy", "light")
    }
    penalties
}

# end of stepwiseqtl.R

context("stepwiseqtl")

test_that("stepwiseqtl works with X-chr-specific perms", {

    data(fake.f2)
    fake.f2 <- calc.genoprob(fake.f2)

    set.seed(17370120)
    operm1 <- scantwopermhk(fake.f2, n.perm=10, verbose=FALSE)
    set.seed(17370120)
    operm2 <- scantwopermhk(fake.f2, n.perm=10, perm.Xsp=TRUE, verbose=FALSE)

    pen1 <- calc.penalties(operm1)
    pen2 <- calc.penalties(operm2)

    out.sq1 <- stepwiseqtl(fake.f2, max.qtl=4, penalties=pen1, method="hk", verbose=FALSE)
    expect_equal(out.sq1$chr, c("1", "8", "13", "X"))
    expect_equal(out.sq1$pos, c(37.11, 61.20, 24.03, 14.20))

    out.sq2 <- stepwiseqtl(fake.f2, max.qtl=4, penalties=pen2, method="hk", verbose=FALSE)
    expect_equal(out.sq2$chr, c("1", "13", "X"))
    expect_equal(out.sq2$pos, c(37.11, 24.03, 14.20))

    out.sq3 <- stepwiseqtl(fake.f2, chr=1:19, max.qtl=4, penalties=pen2, method="hk", verbose=FALSE)
    expect_equal(out.sq3$chr, c("1", "13"))
    expect_equal(out.sq3$pos, c(37.11, 24.03))

    pen2b <- calc.penalties(operm2, alpha=0.2)
    out.sq2b <- stepwiseqtl(fake.f2, max.qtl=6, penalties=pen2b, method="hk", verbose=FALSE)
    expect_equal(out.sq2b$chr, c("1", "8", "13", "X"))
    expect_equal(out.sq2b$pos, c(37.11, 61.20, 24.03, 14.20))

})

#####################################################################
#
# arithscan.R
#
# copyright (c) 2005-2019, Karl W Broman
# last modified Dec, 2019
# first written Mar, 2005
#
#     This program is free software; you can redistribute it and/or
#     modify it under the terms of the GNU General Public License,
#     version 3, as published by the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful,
#     but without any warranty; without even the implied warranty of
#     merchantability or fitness for a particular purpose.  See the GNU
#     General Public License, version 3, for more details.
#
#     A copy of the GNU General Public License, version 3, is available
#     at http://www.r-project.org/Licenses/GPL-3
#
# Part of the R/qtl package
# Contains: +.scanone, -.scanone, +.scanoneperm, -.scanoneperm
#           +.scantwo, -.scantwo, +.scantwoperm, -.scantwoperm
#
######################################################################


"-.scanone" <-
    function(e1,e2)
{
    if(!inherits(e1, "scanone"))
        stop("Input should have class \"scanone\".")

    if(missing(e2)) {
        class(e1) <- "data.frame"
        e1[,-(1:2)] <- -e1[,-(1:2)]
        class(e1) <- c("scanone","data.frame")

        return(e1)
    }
    if(!inherits(e2, "scanone"))
        stop("Input should have class \"scanone\".")

    class(e1) <- class(e2) <- "data.frame"
    if(nrow(e1) != nrow(e2)) {
        u1 <- levels(e1[,1])
        u2 <- levels(e2[,2])
        u <- unique(c(u1,u2))
        if(length(u) == 0) stop("Can't subtract; no chromosomes in common.")
        e1 <- e1[!is.na(match(e1[,1],u)),]
        e2 <- e2[!is.na(match(e2[,1],u)),]
        if(nrow(e1) != nrow(e2) || any(e1[,1] != e2[,1]) || max(abs(e1[,2]-e2[,2])) < 0.01)
            stop("Can't subtract; arguments not compatible")
    }
    nc1 <- ncol(e1)
    nc2 <- ncol(e2)
    nc <- min(c(nc1,nc2))
    e1 <- e1[,1:nc]
    e1[,3:nc] <- e1[,3:nc] - e2[,3:nc]

    # zero out small stuff
    temp <- e1[,3:nc]
    temp[!is.na(temp) & abs(temp) < 1e-6] <- 0
    e1[,3:nc] <- temp

    class(e1) <- c("scanone","data.frame")

    e1
}

"+.scanone" <-
    function(e1,e2)
{
    if(!inherits(e1, "scanone"))
        stop("Input should have class \"scanone\".")

    if(missing(e2)) return(e1)
    if(!inherits(e2, "scanone"))
        stop("Input should have class \"scanone\".")

    class(e1) <- class(e2) <- "data.frame"
    if(nrow(e1) != nrow(e2)) {
        u1 <- levels(e1[,1])
        u2 <- levels(e2[,2])
        u <- unique(c(u1,u2))
        if(length(u) == 0) stop("Can't add; no chromosomes in common.")
        e1 <- e1[!is.na(match(e1[,1],u)),]
        e2 <- e2[!is.na(match(e2[,1],u)),]
        if(nrow(e1) != nrow(e2) || any(e1[,1] != e2[,1]) || max(abs(e1[,2]-e2[,2])) < 0.01)
            stop("Can't add; arguments not compatible")
    }
    nc1 <- ncol(e1)
    nc2 <- ncol(e2)
    nc <- min(c(nc1,nc2))
    e1 <- e1[,1:nc]
    e1[,3:nc] <- e1[,3:nc] + e2[,3:nc]
    class(e1) <- c("scanone","data.frame")

    e1
}

"-.scanoneperm" <-
    function(e1, e2)
{
    if(!inherits(e1, "scanoneperm"))
        stop("Input should have class \"scanoneperm\".")

    if(missing(e2)) {
        e1.x <- ("xchr" %in% names(attributes(e1)))
        if(e1.x) {
            e1$A <- -e1$A
            e1$X <- -e1$X
        }
        else {
            theclass <- class(e1)
            e1 <- -unclass(e1)
            class(e1) <- theclass
        }

        return(e1)
    }
    if(!inherits(e2, "scanoneperm"))
        stop("Input should have class \"scanoneperm\".")

    # check input
    e1.x <- ("xchr" %in% names(attributes(e1)))
    e2.x <- ("xchr" %in% names(attributes(e2)))

    if(e1.x != e2.x)
        stop("Need both or neither input to be X-chr specific.\n")

    if((e1.x && (any(dim(e1$A)!=dim(e2$A)) || any(dim(e1$X)!=dim(e2$X)))) ||
       (!e1.x && any(dim(e1) != dim(e2))))
        stop("Need input to concern the same phenotypes and no. permutations.\n")

    if(e1.x) {
        e1$A <- e1$A - e2$A
        e1$X <- e1$X - e2$X

        # zero out small stuff
        e1$A[!is.na(e1$A) & abs(e1$A) < 1e-6] <- 0
        e1$X[!is.na(e1$X) & abs(e1$X) < 1e-6] <- 0
    }
    else {
        theclass <- class(e1)
        e1 <- unclass(e1) - unclass(e2)

        # zero out small stuff
        e1[!is.na(e1) & abs(e1) < 1e-6] <- 0

        class(e1) <- theclass
    }

    e1
}

"+.scanoneperm" <-
    function(e1, e2)
{
    if(!inherits(e1, "scanoneperm"))
        stop("Input should have class \"scanoneperm\".")
    if(missing(e2)) return(e1)
    if(!inherits(e2, "scanoneperm"))
        stop("Input should have class \"scanoneperm\".")

    # check input
    e1.x <- ("xchr" %in% names(attributes(e1)))
    e2.x <- ("xchr" %in% names(attributes(e2)))

    if(e1.x != e2.x)
        stop("Need both or neither input to be X-chr specific.\n")

    if((e1.x && (any(dim(e1$A)!=dim(e2$A)) || any(dim(e1$X)!=dim(e2$X)))) ||
       (!e1.x && any(dim(e1) != dim(e2))))
        stop("Need input to concern the same phenotypes and no. permutations.\n")

    if(e1.x) {
        e1$A <- e1$A + e2$A
        e1$X <- e1$X + e2$X
    }
    else {
        theclass <- class(e1)
        e1 <- unclass(e1) + unclass(e2)
        class(e1) <- theclass
    }

    e1
}

######################################################################
# -.scantwo: subtract LOD scores in two scantwo results
######################################################################
"-.scantwo" <-
    function(e1, e2)
{
    if(!inherits(e1, "scantwo"))
        stop("Input should have class \"scantwo\".")

    if(missing(e2)) {
        e1$lod <- -e1$lod
        if("scanoneX" %in% names(e1))
            e1$scanoneX <- -e1$scanoneX

        return(e1)
    }
    if(!inherits(e2, "scantwo"))
        stop("Input should have class \"scantwo\".")

    e1x <- "scanoneX" %in% names(e1)
    e2x <- "scanoneX" %in% names(e2)

    if(any(dim(e1$map) != dim(e2$map)) ||
       length(dim(e1$lod)) != length(dim(e2$lod)) ||
       any(dim(e1$lod) != dim(e2$lod)) || e1x != e2x)
        stop("input arguments do not conform.")

    e1$lod <- e1$lod - e2$lod
    if(e1x) {
        if(!is.null(e1$scanoneX) && !is.null(e2$scanoneX))
            e1$scanoneX <- e1$scanoneX - e2$scanoneX
    }

    e1
}


######################################################################
# +.scantwo: add LOD scores in two scantwo results
######################################################################
"+.scantwo" <-
    function(e1, e2)
{
    if(!inherits(e1, "scantwo"))
        stop("Input should have class \"scantwo\".")
    if(missing(e2)) return(e1)
    if(!inherits(e2, "scantwo"))
        stop("Input should have class \"scantwo\".")

    e1x <- "scanoneX" %in% names(e1)
    e2x <- "scanoneX" %in% names(e2)

    if(any(dim(e1$map) != dim(e2$map)) ||
       length(dim(e1$lod)) != length(dim(e2$lod)) ||
       any(dim(e1$lod) != dim(e2$lod)) || e1x != e2x)
        stop("input arguments do not conform.")

    e1$lod <- e1$lod + e2$lod
    if(e1x) {
        if(!is.null(e1$scanoneX) && !is.null(e2$scanoneX))
            e1$scanoneX <- e1$scanoneX + e2$scanoneX
    }

    e1
}


######################################################################
# -.scantwoperm: subtract LOD scores in two scantwo permutation results
######################################################################
"-.scantwoperm" <-
    function(e1, e2)
{
    if(!inherits(e1, "scantwoperm"))
        stop("Input should have class \"scantwoperm\".")

    if(missing(e2)) {
        # x-chr-specific
        if("AA" %in% names(e1)) {
            for(i in seq(along=e1))
                for(j in seq(along=e1[[i]]))
                    e1[[i]][[j]] <- -e1[[i]][[j]]
            return(e1)
        }

        for(i in 1:length(e1))
            e1[[i]] <- -e1[[i]]

        return(e1)
    }

    if(!inherits(e2, "scantwoperm"))
        stop("Input should have class \"scantwoperm\".")

    # x-chr-specific
    if("AA" %in% names(e1) || "AA" %in% names(e2)) {
        if(!("AA" %in% names(e1) && "AA" %in% names(e2)))
            stop("Input must both be Xchr-specific, or neither")
        for(i in seq(along=e1)) {
            for(j in seq(along=e1[[i]])) {
                if(any(dim(e1[[i]][[j]]) != dim(e1[[i]][[j]])))
                    stop("dimensions do not match")
                e1[[i]][[j]] <- e1[[i]][[j]] - e2[[i]][[j]]
            }
        }
        return(e1)
    }

    dim1 <- sapply(e1, dim)
    dim2 <- sapply(e2, dim)
    if(any(dim1 != dim2))
        stop("Need input to concern the same phenotypes and no. permutations.\n")

    for(i in 1:length(e1))
        e1[[i]] <- e1[[i]] - e2[[i]]

    e1
}


######################################################################
# +.scantwoperm: add LOD scores in two scantwo permutation results
######################################################################
"+.scantwoperm" <-
    function(e1, e2)
{
    if(!inherits(e1, "scantwoperm"))
        stop("Input should have class \"scantwoperm\".")
    if(missing(e2)) return(e1)
    if(!inherits(e2, "scantwoperm"))
        stop("Input should have class \"scantwoperm\".")

    # x-chr-specific
    if("AA" %in% names(e1) || "AA" %in% names(e2)) {
        if(!("AA" %in% names(e1) && "AA" %in% names(e2)))
            stop("Input must both be Xchr-specific, or neither")
        for(i in seq(along=e1)) {
            for(j in seq(along=e1[[i]])) {
                if(any(dim(e1[[i]][[j]]) != dim(e1[[i]][[j]])))
                    stop("dimensions do not match")
                e1[[i]][[j]] <- e1[[i]][[j]] + e2[[i]][[j]]
            }
        }
        return(e1)
    }

    dim1 <- sapply(e1, dim)
    dim2 <- sapply(e2, dim)
    if(any(dim1 != dim2))
        stop("Need input to concern the same phenotypes and no. permutations.\n")

    for(i in 1:length(e1))
        e1[[i]] <- e1[[i]] + e2[[i]]

    e1
}


# end of arithscan.R

#! /bin/sh
#
# Usage: contrib/scripts/install_rqtl.

path=$1

if [ ! -z $path -a -d $path ]; then
  cd $path
fi
echo -n "Using: "
pwd
if [ ! -d "contrib" ]; then
  echo "Incorrect path for R/qtl source"
  exit 1
fi
cwd=`pwd`

sh contrib/scripts/cleanup.sh

rqtl_version=`grep Version DESCRIPTION | awk '{ print $2; }'`

echo "* Run R CMD check"
cd $cwd
cd ..
R CMD build $cwd
R CMD INSTALL qtl_${rqtl_version}.tar.gz
rm -v qtl_${rqtl_version}

cd $cwd
time R --no-save --no-restore --no-readline --slave < ./tests/test_qtl.R 

echo "Starting regression tests"
cd test
dir
PATH=%PATH%;c:\Program Files\R\R-2.9.1\bin
# valgrind ../src/mqm -T=0 -V > ../test/MQM_test0.txt
..\mqm -T=0 -V > MQM_test0.txt

echo "Finalized regression tests"

#! /bin/sh
#
# Usage: contrib/scripts/check_rqtl.sh

path=$1

echo "* Run all R CMD check for R/qtl on $path"
if [ ! -z $path -a -d $path ]; then
  cd $path
fi
echo -n "Using: "
pwd
if [ ! -d "contrib" ]; then
  echo "Incorrect path for R/qtl source"
  exit 1
fi
cwd=`pwd`


echo "* Run R CMD check"
cd $cwd
sh contrib/scripts/cleanup.sh
R CMD check .
if [ "$?" -ne "0" ]; then
  echo "Test 'R CMD check' failed"
  exit 1
fi

#! /bin/bash
#
# Create a diff from another repostitory (this one should be standalone 
# and the other a master).

if [ ! -d .git ]; then
  echo Should be in root of repo
  exit 1
fi

git checkout standalone
cd src
ls --color=never *.cpp *.h > standalone.lst
git checkout master
cat standalone.lst | grep -v main | xargs git diff standalone > standalone.patch

#! /bin/sh

if [ ! -d R ] ; then
  echo Only run cleanup from base dir
fi

rm -rvf ..Rcheck/
rm inst/tests/junk*
for x in cmake_install.cmake CTestTestfile.cmake CMakeCache.txt Makefile
do
  rm -vf $x
  find . -name $x -exec rm -v \{\} \;
done

find . -name CMakeFiles -exec rm -rvf \{\} \;
find . -name Testing -exec rm -rvf \{\} \;
find . -name *.so -exec rm -v \{\} \;
find . -name *.o -exec rm -v \{\} \;
find . -name *.a -exec rm -v \{\} \;
find . -name *dump -exec rm -v \{\} \;
find . -name *.aux -exec rm -v \{\} \;
find . -name *.dvi -exec rm -v \{\} \;
find . -name *.log -exec rm -v \{\} \;
find . -name *.orig -exec rm -v \{\} \;
find . -name *.eps -exec rm -v \{\} \;
find . -name *~ -exec rm -v \{\} \;
find . -name Rplots.pdf -exec rm -v \{\} \;
find . -name *.Rout -exec rm -v \{\} \;
find src/ -name *.dll -exec rm -v \{\} \;

rm inst/doc/Sources/MQM/MQM-tour.tex
mv inst/doc/Sources/MQM/MQM-tour.R inst/doc/
mv inst/doc/Sources/MQM/MQM-tour.pdf inst/doc/

rm -rvf src-i386/
rm -rvf src-x86_64/

echo "Regressiontests for standalone MQM"
echo "Dataset 1 F2"
mqm.exe -v -pTest/std/phenotypes1.txt -gTest/std/genotypes1.txt -mTest/std/markers1.txt -sTest/std/settings1.txt --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=n --maugment=10000 --miaugment=250 --neglect=1 > test/t11out.txt
mqm.exe -v -pTest/std/phenotypes1.txt -gTest/std/genotypes1.txt -mTest/std/markers1.txt -sTest/std/settings1.txt -cTest/t12/cofactors.txt --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=n --maugment=10000 --miaugment=250 --neglect=1 > test/t12out.txt

echo "Dataset 2 BC=~ Hyper (genotypes NA -> 0)"
mqm.exe -v -pTest/std/phenotypes2.txt -gTest/std/genotypes2.txt -mTest/std/markers2.txt -sTest/std/settings2.txt  --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=n --maugment=10000 --miaugment=250 --neglect=1 > test/t21out.txt
mqm.exe -v -pTest/std/phenotypes2.txt -gTest/std/genotypes2.txt -mTest/std/markers2.txt -sTest/std/settings2.txt -cTest/t22/cofactors.txt  --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=n --maugment=10000 --miaugment=250 --neglect=1 > test/t22out.txt
mqm.exe -v -pTest/std/phenotypes2.txt -gTest/std/genotypes2.txt -mTest/std/markers2.txt -sTest/std/settings2.txt -cTest/t23/cofactors.txt  --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=n --maugment=10000 --miaugment=250 --neglect=1 > test/t23out.txt
mqm.exe -v -pTest/std/phenotypes2.txt -gTest/std/genotypes2m.txt -mTest/std/markers2.txt -sTest/std/settings2.txt  --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=y --maugment=10000 --miaugment=250 --neglect=1 > test/t24out.txt
mqm.exe -v -pTest/std/phenotypes2.txt -gTest/std/genotypes2m.txt -mTest/std/markers2.txt -sTest/std/settings2.txt  --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=y --maugment=10000 --miaugment=250 --neglect=1 > test/t25out.txt

echo "Dataset 3 F2=~ Listeria"
mqm.exe -v -pTest/std/phenotypes3.txt -gTest/std/genotypes3.txt -mTest/std/markers3.txt -sTest/std/settings3.txt  --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=n --maugment=10000 --miaugment=250 --neglect=1> test/t31out.txt
mqm.exe -v -pTest/std/phenotypes3.txt -gTest/std/genotypes3.txt -mTest/std/markers3m.txt -sTest/std/settings3.txt --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=y --maugment=10000 --miaugment=250 --neglect=1 > test/t32out.txt
mqm.exe -v -pTest/std/phenotypes3.txt -gTest/std/genotypes3.txt -mTest/std/markers3.txt -sTest/std/settings3.txt -cTest/t33/cofactors.txt  --smin=0 --smax=200 --sstep=2 --alpha=0.02 --window=10 --maxiter=1000 --estmap=n --maugment=10000 --miaugment=250 --neglect=1 > test/t33out.txt

echo "Output done"

#! /bin/sh

echo "* Starting SWEAVE to generate MQM-tour (MQM)"
rm -vf MQM-tour.tex
which R
R --version

R CMD BATCH SweaveIt.R
if [ ! -e MQM-tour.tex ]; then
  cat SweaveIt.Rout 
  exit 2
fi
latex MQM-tour.tex
# run twice for references
latex MQM-tour.tex 
dvipdfm MQM-tour.dvi

#! /bin/bash

for x in cmake_install.cmake CTestTestfile.cmake CMakeCache.txt Makefile
do
  rm -vf $x
  find . -name $x -exec rm -v \{\} \;
done

find . -name CMakeFiles -exec rm -rvf \{\} \;
find . -name Testing -exec rm -rvf \{\} \;
find . -name *.so -exec rm -v \{\} \;
find . -name *.dll -exec rm -v \{\} \;
find . -name *.o -exec rm -v \{\} \;
find . -name *.a -exec rm -v \{\} \;
find . -name *dump -exec rm -v \{\} \;

#! /bin/sh
#
# Usage: inst/contrib/scripts/run_all_tests.sh . [options]
#
# Example:
#
#    ./inst/contrib/scripts/run_all_tests.sh . --library=/my/libs

path=$1
Roptions=$2

echo "* Run all functional/regression/unit/check tests for R/qtl on $path"
if [ ! -z $path -a -d $path ]; then
  cd $path
fi
echo -n "Using: "
pwd
if [ ! -d "inst/contrib" ]; then
  echo "Incorrect path for R/qtl source"
  exit 1
fi
cwd=`pwd`

sh inst/contrib/scripts/cleanup.sh

echo "* Run the standard MQM regression tests - without R install"
cd $cwd
cd inst/contrib/bin
rm CMakeCache.txt
cmake .
make clean
make
make test
if [ "$?" -ne "0" ]; then
  echo "Test 'standalone' failed"
  exit 1
fi

echo "* Run R CMD check $Roptions $cwd"
cd $cwd
sh inst/contrib/scripts/cleanup.sh
R CMD check $Roptions .
if [ "$?" -ne "0" ]; then
  echo "Test 'R CMD check' failed"
  exit 1
fi

echo "* Run the R regression tests - with R install from CMakeLists.txt"
cd $cwd
sh inst/contrib/scripts/cleanup.sh
cd inst/contrib/bin
rm CMakeCache.txt
cmake -DTEST_R=TRUE .
make clean
make
make testR
if [ "$?" -ne "0" ]; then
  echo "Test 'R regression tests' failed"
  exit 1
fi

echo "* Generate PDF's"
echo "== skipped =="

\documentclass[11pt]{article}
\setlength{\topmargin}{-.5in}
\setlength{\textheight}{23.5cm}
\setlength{\textwidth}{17.0cm}
\setlength{\oddsidemargin}{.025in}
\setlength{\evensidemargin}{.025in}
\setlength{\textwidth}{6.25in}
\usepackage{amsmath}
\usepackage{graphicx}
\usepackage{verbatim}   % useful for program listings
\usepackage{color}      % use if color is used in text
\usepackage{subfigure}  % use for side-by-side figures
\usepackage{float}
\usepackage{Sweave}
\usepackage{url}

\newcommand{\mqm}{\emph{MQM}}
\newcommand{\MQM}{\mqm}
\newcommand{\qtl}{QTL}
\newcommand{\QTL}{\qtl}
\newcommand{\xqtl}{\emph{x}QTL}
\newcommand{\mqtl}{\emph{m}QTL}
\newcommand{\eqtl}{\emph{e}QTL}
\newcommand{\lod}{LOD}
\newcommand{\cM}{cM}
\newcommand{\rqtl}{\emph{R/qtl}}
\newcommand{\cim}{\emph{CIM}}
\newcommand{\At}{\emph{Arabidopsis thaliana}}
\newcommand{\FIXME}{({\bf FIXME!})}
\newcommand{\CHECK}{({\bf CHECK!})}
\newcommand{\NOTE}[1]{({\tt NOTE: #1 })}
\newcommand{\intro}[1]{\vspace{0.15in}#1:}
\newcommand{\code}{\texttt}
\newcommand{\etal}{\emph{et al.}}

\newcommand{\Atintro}{\At\ RIL mQTL dataset (multitrait) with 24 metabolites as phenotypes \cite{Keurentjes2006}}
\newcommand{\Atintrocolors}{\Atintro\ comparing \mqm\ (\code{mqmscan} in green) and
single \qtl\ mapping (\code{scanone} in black)}

\title { Tutorial - Multiple-QTL Mapping (MQM) Analysis for R/qtl }
\author { Danny Arends, Pjotr Prins, Karl W. Broman and Ritsert C. Jansen }
\begin {document}
\maketitle
\clearpage

\SweaveOpts{prefix.string=Figs/fig,eps=TRUE}
\setkeys{Gin}{width=6.25in} %% <- change width of figures

\section{Introduction}

\input{mqm/description.txt}

\vspace{0.3in}

\input{mqm/advantages_latex.txt}

\input{mqm/limitations.txt}

Despite these limitations, \mqm\footnote{MQM should not be confused with
composite interval mapping (CIM) \cite{CIMa,CIMb}.  The advantage of MQM
over CIM is reduction of type I error (a QTL is indicated at a location where there
is no QTL present) and type II error (a QTL is not detected) for QTL detection
\cite{jansen94b}.} is a valuable addition to the \qtl\ mapper's toolbox. It
is able to deal with QTL in coupling phase and QTL in repulsion phase. \mqm\
handles missing data and has higher power to detect QTL (linked and unlinked)
than other methods.  R/qtl's \mqm\ is faster than other implementations and
scales on multi-CPU systems and computer clusters.  In this tutorial we will
show you how to use \mqm\ for \qtl\ mapping.

\mqm\ is an integral part of the free \rqtl\
package \cite{rqtlbook,broman09,broman03} for the R statistical
language\footnote{We assume the reader knows how to load his data into R using
the R/qtl \code{read.cross} function; see also the R/qtl tutorials \cite{broman09}
and book \cite{rqtlbook}.}.

\section{A quick overview of \mqm}

These are the typical steps in an \mqm\ \qtl\ analysis:

\begin{itemize}
\item Load data into R
\item Fill in missing data, using either \code{mqmaugmentdata} or \code{fill.geno}
\item Unsupervised backward elimination to analyse \emph{cofactors}, using \code{mqmscan}
\item Optionally select \emph{cofactors\/} at markers that are thought to influence \qtl\ at, or near, the location
\item Permutation or simulation analysis to get estimates of significance, using \code{mqmpermutation} or \code{mqmscanfdr}
\end{itemize}

Using maximum likelihood (ML), or restricted maximum likelihood (REML), the
algorithm employs a backward elimination strategy to identify \qtl\ underlying
the trait. The algorithm passes through the following stages:

\begin{itemize}
\item Likelihood-based estimation of the full model using all cofactors
\item Backward elimination of cofactors, followed by a
      genome scan for \qtl
\item If there are no \emph{cofactors\/} defined, the backward elimination of cofactors
      is skipped and a genome scan for \qtl\ is performed, testing each genetic (interval)
      location individually. In this case REML and ML will result in the same \qtl\ profile
      because there is no full model.
\end{itemize}

The results created during the genome scan and the \qtl\ model are
returned as an (extended) R/qtl \code{scanone} object. Several special
plotting routines are available for \mqm\ results.

%\clearpage

\section{Data augmentation}
\label{augmentation}

In an ideal world all datasets would be complete (with the genotype
for every
individual at every marker determined), however in the real world datasets are often incomplete. That is, genotype
information is missing, or can have multiple plausible values. \mqm\
automatically expands the
dataset by adding all potential variants and attaching a probability to each.  For
example, information is missing (unknown) at a marker location for one
individual. Based on the values of the neighbouring markers, and the
(estimated) recombination rate, a probability is attached to all possible
genotypes.  With \mqm\ all possible genotypes with a probability above
the
parameter \code{minprob} are considered.

When encountering a missing marker genotype (possible genotypes {\bf A} and {\bf B} in a
RIL), all possible genotypes at the missing location are created.  Thus at
the missing location two `individuals' are created in the \emph{augmentation} step,
one with genotype {\bf A}, and one with genotype {\bf B}. A probability is
attached to both \emph{augmented} individuals.  The combined probability of all
missing marker locations tells whether a genotype is likely, or unlikely,
which allows for weighted analysis later.

To see an example of missing data with an F$_2$ intercross, we can
visualize the genotypes of the individuals using \code{geno.image}. In
Figure~\ref{missing data} there are 2\% missing values in white. The
other colors are genotypes at a certain position, for a certain
individual. Simulate an F$_2$ dataset with 2\% missing genotypes as
follows:

\intro{Simulate a dataset with missing data}
% set seed so that everything comes out exactly the same
<<setseed,echo=FALSE>>=
set.seed(19696527)
@
<<>>=
library(qtl)
data(map10)
simcross <- sim.cross(map10, type="f2", n.ind=100, missing.prob=0.02)
@

and plot the genotype data using \code{geno.image} (Figure~\ref{missing data}):


<<missingdata,eval=FALSE>>=
geno.image(simcross)
@

\begin{figure}
<<missingdataplot,fig=TRUE,echo=FALSE>>=
<<missingdata>>
@
\caption{Genotype data for a simulated F$_2$ intercross generated with \code{sim.cross}, with 100
  individuals and 2\% missing data.  White pixels indicate missing genotypes.\label{missing data}}
\end{figure}

Before going to the next step (the \qtl\ genome scan), the data has to be
completed (i.e. no more missing data).  There are two possibilities: use (1)
the \mqm\ data augmentation routine \code{mqmaugment} or (2) the imputation
routine \code{fill.geno}. Augmentation tries to analyse
all possible genotypes of interest by leaving them in the solution space. In contrast,
the imputation method \emph{selects} the most likely genotype, and uses that
single individual for further analysis.

The downside of augmentation is that the addition of many possible genotypes
can exceed available computer memory.  Currently, augmentation moves an
individual to a second augmentation round when it has too many possible
genotypes (above the maximum number of augmented individuals \code{maxaugind}). In this second augmentation round the
user can specify what needs to be done with these individuals: (1) Only use the
most likely genotype, (2) use multiple imputation to create multiple possible
genotypes (up to \code{maxaugind}) or (3) remove the original genotype/individual
from the analysis.  Note that you can opt to use \code{fill.geno}'s imputation method on your dataset,
instead of augmentation, when too many individuals are dropped because of missing data.

The function \code{mqmaugment} is specific to \mqm\ and the recommended
procedure\footnote{Note that after augmentation the resulting object is
no longer suitable for the use with other R/qtl mapping functions, like \code{scanone} and \code{cim}, because they can not account for
duplicated or dropped individuals.}.
In this tutorial we focus on \mqm's augmentation. The function \code{mqmaugment} fills in
missing genotypes for us. For each missing genotype data, at a marker, it fills
in all possible genotypes and calculates the probability. When the total
probability is higher than the \code{minprob} parameter the \emph{augmented}
individual is stored in the new cross object, ready for QTL mapping.

The important parameters are: \code{cross}, \code{pheno.col}, \code{maxaugind}, \code{minprob} and
\code{verbose} (see also the \code{mqmaugment} help page). \code{maxaugind} sets the
maximum number of \emph{augmented} genotypes per individual in a dataset. The
default of 82 allows six missing markers per individual in a BC, and four in an
F$_2$. As a result the user has to increase the
\code{maxaugind} parameter when there are more missing markers.

The \code{minprob} parameter sets the minimum probability of a genotype for
inclusion in the augmented dataset. This genotype probability is calculated for
every marker \emph{relative} to the most likely genotype of this individual.  Note that setting
this value too low may result in moving a lot of individuals to the second
augmentation round as the maximum of augmented individuals (the parameter \code{maxaugind}) is quickly reached.
Increasing \code{minprob} (towards a value of 1.0) can keep individuals with more
missing data inside the first augmentation round; a possible rule of thumb may be
to set \code{minprob} to the percentage of data missing. A value of
\code{minprob=1.0} makes augmentation behave similar to \code{fill.geno}'s
imputation method, though with different resulting genotypes.  Use
\code{verbose=TRUE} to get more feedback on the augmentation routine and to
check how many individuals are moved to the second stage, for imputation or
removal\footnote{Augmentation is not always suitable with a lot of missing
data, like in the case of selective genotyped datasets (for example the mouse
\code{hyper} dataset that comes with R/qtl); these will always be handled with
\code{minprob=1.0} (and a warning will be issued).}

To start with an example, first run \code{mqmaugment} with \code{minprob=1.0}
(Figure~\ref{augment1}):

\intro{Plot augmented data using \code{geno.image}}
<<>>=
# displays warning because MQM ignores the X chromosome in an F2
augmentedcross <- mqmaugment(simcross, minprob=1.0)
@
Plot the genotype data as follows:
<<augment1,eval=FALSE>>=
geno.image(augmentedcross)
@
\begin{figure}
<<augment1plot,fig=TRUE,echo=FALSE>>=
<<augment1>>
@
\caption{Genotypes, as visualized with \code{geno.image}, of 100 filled individuals
(\code{mqmaugment} with \code{minprob=1.0}. With missing data only a `most
likely' individual is used and no real expansion of the dataset takes place,
with similar results as \code{fill.geno}'s imputation method).\label{augment1}}
\end{figure}

With a lower \code{minprob}, more augmented individuals are kept, and the
resulting \emph{augmented} dataset will be larger. Adding (weighted) augmented
individuals with all possible genotypes theoretically leads to a more accurate
mapping when dealing with missing values \cite{jansen93}\footnote{Note again that
the augmented dataset can only be used with pure \mqm\ functions.  \mqm\ functions
recognise expanded individuals as single entities.  Other R/qtl functions, like
\code{scanone}, assume the augmented individuals are \emph{real} individuals.}.

\intro{Try augmentation with \code{minprob=0.1} (Figure~\ref{augment2})}

<<>>=
augmentedcross <- mqmaugment(simcross, minprob=0.1)
@

\intro{Plot the genotype data}

<<augment2,eval=FALSE>>=
geno.image(augmentedcross)
@

\begin{figure}
<<augment2plot,fig=TRUE,echo=FALSE>>=
<<augment2>>
@
\caption{Genotypes, as visualized with \code{geno.image} of the \emph{augmented}
genotypes of 100 individuals. There are a total of \Sexpr{nind(augmentedcross)}
`expanded' individuals in this plot, because \mqm\ fills in missing markers with
all likely genotypes (an average expansion of \Sexpr{round(nind(augmentedcross)/nind(simcross),1)} per
individual).\label{augment2}}
\end{figure}

\label{multitrait}
An \mqtl\ dataset (\code{multitrait}), which contains 24 metabolite
traits from a RIL population of \At, is now distributed with
R/qtl (load the data with \code{data(multitrait)}).  This is part of the \At\ RIL selfing experiment
with Landsberg erecta (Ler) and Cape Verde Islands (Cvi) with 162 individuals
scored 117 markers \cite{Koornneef1998}. The experiment
concerned empirical untargeted metabolomics using liquid chromatography time of
flight mass spectrometry (LC-QTOF MS). This uncovered many qualitative and
quantitative differences in metabolite accumulation between \At\ accessions
\cite{Keurentjes2006}.

\intro{Simulate missing data by removing some genotype data (5\%, 10\%
  and 80\%) from the cross object}

<<augment3>>=
data(multitrait)
msim5 <- simulatemissingdata(multitrait, 5)
msim10 <- simulatemissingdata(multitrait, 10)
msim80 <- simulatemissingdata(multitrait, 80)
@

Next use augmentation to fill in the missing genotypes; with more missing data
increase the \code{minprob} parameter.  When the \code{minprob} parameter is
set too low it is possible that an individual cannot be augmented,
and is moved to the second round of augmentation (see the description above).

<<augment4>>=
maug5 <- mqmaugment(msim5)
maug10 <- mqmaugment(msim10, minprob=0.25)
maug80 <- mqmaugment(msim80, minprob=0.80)
@

Taking the 10\% missing set, we can try a lower \code{minprob=0.001}. The
output below shows that ten augmented individuals miss too many markers to be
augmented. By using the imputation strategy these individuals are kept in the
set with a single `most likely' genotype.

\intro{Augment with an imputation strategy}

<<augmentMinProb>>=
maug10minprob <- mqmaugment(msim10, minprob=0.001, verbose=TRUE)
maug10minprobImpute <- mqmaugment(msim10, minprob=0.001, strategy="impute",
                                  verbose=TRUE)
# check how many individuals are expanded:
nind(maug10minprob)
nind(maug10minprobImpute)
@


Next,
scan for QTL inside the cross objects with \code{mqmscan} and the single-QTL
mapping function \code{scanone} (for reference).  The effect of increasing the
amount of missing data on QTL mapping, using default values, can be seen in Figure~\ref{augment6}.


<<augment5>>=
mqm5 <- mqmscan(maug5)
mqm10 <- mqmscan(maug10)
mqm80 <- mqmscan(maug80)
@

<<augment5b>>=
msim5 <- calc.genoprob(msim5)
one5 <- scanone(msim5)
msim10 <- calc.genoprob(msim10)
one10 <- scanone(msim10)
msim80 <- calc.genoprob(msim80)
one80 <- scanone(msim80)
@

<<augment6,eval=FALSE,echo=FALSE>>=
op <- par(mfrow = c(2,2))
plot(mqm5, mqm10, mqm80, col=c("green","blue","red"), main="MQM missing data")
legend("topleft", c("MQM 5%","MQM 10%","MQM 80%"), col=c("green","blue","red"), lwd=1)
plot(one5, mqm5, main="5% missing", col=c("black","green"))
legend("topleft", c("scanone","MQM"), col=c("black","green"), lwd=1)
plot(one10, mqm10, main="10% missing", col=c("black","blue"))
legend("topleft", c("scanone","MQM"), col=c("black","blue"), lwd=1)
plot(one80, mqm80, main="80% missing", col=c("black","red"))
legend("topleft", c("scanone","MQM"), col=c("black","red"), lwd=1)
@

\begin{figure}
<<fig=TRUE,echo=FALSE>>=
<<augment6>>
@
\caption{\Atintro. Effect of missing data on \code{mqmscan} after augmentation
(green=5\%, blue=10\%, red=80\%) and \code{scanone} (black), after \code{fill.geno}
imputation. \label{augment6}}
\end{figure}
\clearpage

\section{Multiple-QTL Mapping (MQM)}

\label{QTL modelling}

The \code{multitrait} dataset, distributed with R/qtl, contains 24 metabolite
traits from a RIL population of \At \cite{Keurentjes2006} (see also section
\ref{multitrait} and \code{help(multitrait)} in R).

Here we analyse the \code{multitrait} dataset using both \code{scanone}
(single-\qtl\ analysis) and \code{mqmscan} (Multiple-QTL Mapping).
First augment the data using the \code{mqmaugment} function with
\code{minprob=1.0}, to compare against \code{scanone} with imputation (see also
section \ref{augmentation}).

\intro{Scan for \qtl\ with \code{mqmscan}, after filling missing data with
\code{mqmaugment minprob=1.0}}

<<>>=
data(multitrait)
maug_min1 <- mqmaugment(multitrait, minprob=1.0)
mqm_min1 <- mqmscan(maug_min1)
@

We compare \code{mqmscan} with \code{scanone}. For \code{scanone}
one first calculates conditional \qtl\ genotype probabilities via
\code{calc.genoprob}.

<<>>=
mgenop <- calc.genoprob(multitrait, step=5)
m_one <- scanone(mgenop)
@

Figure~\ref{Atminprob} shows that, without augmentation, the results from
\mqm\ are similar to \code{scanone}.

\intro{\code{mqmscan} after augmentation, without cofactor selection}

<<>>=
maug <- mqmaugment(multitrait)
mqm <- mqmscan(maug)
@

\begin{figure}
<<MinprobMulti,fig=TRUE,echo=FALSE,height=3>>=
plot(m_one, mqm_min1, col=c("black","green"), lty=1:2)
legend("topleft", c("scanone","MQM"), col=c("black","green"), lwd=1)
@
\caption{\Atintrocolors.
\mqm\ shows similar results as single \qtl\ mapping,
when used without \emph{augmentation} (\code{minprob} is 1.0), and with default parameters.\label{Atminprob}}
\end{figure}

By default \MQM\ introduces fictional markers, or `pseudo markers', at fixed intervals.
A pseudo marker has a name like \code{c7.loc25}, which is the
pseudo marker at 25 \cM\ on chromosome 7. (Note that this reflects the standard naming
used in R/qtl.)
Each
chromosome is divided into evenly spaced pseudo markers, \code{step.size}
\cM\ apart. A \lod\ score for underlying \qtl\ is calculated at these
pseudo markers. A small \code{step.size} allows for smoother profiles compared
with a pure marker-based mapping approach. The real markers are listed between
the pseudo markers. In the result you can remove the pseudo markers by using
the function \code{mqmextractmarkers}, as follows:

<<>>=
real_markers <- mqmextractmarkers(mqm)
@

For model selection in \mqm, first supply the
algorithm with an initial model. This initial model can be produced in two
ways: by (1) building a model by hand (forward stepwise), or (2) by unsupervised
backward elimination on a large number of markers (discussed in Section~\ref{backelim}).

% @@
First build this initial model by hand using a forward
stepwise approach. (Note that the automated procedure is preferred, both for
theoretical and practical reasons.) A model consists of a set of markers we
want to account for. We can start building the initial model by adding
cofactors at markers with high LOD scores scored by using \code{mqmscan} with
default values. Figure~\ref{Atminprob} displayed a large QTL peak on chromosome 5
at 35 \cM. So we account for that by setting a cofactor at the marker
nearest to the peak on chromosome 5 and running \code{mqmscan} again.  (See
Figures~\ref{Cofactor4} and \ref{Cofactor4b}.)

\intro{Add marker GH.117C (chromosome 5, at 35 \cM) as a cofactor}

<<>>=
max(mqm)
find.marker(maug, chr=5, pos=35)
multitoset <- find.markerindex(maug, "GH.117C")
setcofactors <- mqmsetcofactors(maug, cofactors=multitoset)
mqm_co1 <- mqmscan(maug, setcofactors)
@

The function \code{find.marker} identifies the name of the marker
closest to 35 \cM.  The function \code{find.markerindex} translates the
marker name into a cofactor number.  The function \code{mqmsetcofactors}
sets up a cofactor list for use with \code{mqmscan}.

\intro{Plot the results of the genome scan after adding a single cofactor (Figure~\ref{Cofactor4})}

<<Cofactor4multi,eval=FALSE>>=
par(mfrow = c(2,1))
plot(mqmgetmodel(mqm_co1))
plot(mqm_co1)
@

\begin{figure}
<<fig=TRUE,echo=FALSE>>=
# plot after adding first cofactor
<<Cofactor4multi>>
@
\caption{\Atintro. \code{mqmscan} after a cofactor is added at the top scoring marker of chromosome 5. During the analysis it is kept in the
model. \label{Cofactor4}}
\end{figure}


% --------------

\intro{Plot the \code{mqmscan} results with \code{scanone} results as follows (Figure~\ref{Cofactor4b})}

<<Cofactor4bMULTI,eval=FALSE>>=
plot(m_one, mqm_co1, col=c("black","green"), lty=1:2)
legend("topleft", c("scanone","MQM"), col=c("black","green"), lwd=1)
@

\begin{figure}
<<fig=TRUE,echo=FALSE,height=3>>=
<<Cofactor4bMULTI>>
@
\caption{\Atintro\ after introducing a cofactor on chromosome 5 (GH.117C).
\code{mqmscan} (green, dashed) differs from \code{scanone} (black). \label{Cofactor4b}}
\end{figure}

% ---------------------------

Figures~\ref{Cofactor4} and \ref{Cofactor4b} show the effect of setting a single marker as a
cofactor related to the \qtl\ on chromosome 5, followed by an \mqm\ scan.  The
marker is not dropped and it passes initial thresholding to account for
the \code{cofactor.significance} level. LOD scores are expected to change slightly,
because of variation already explained by the \qtl\ on
chromosome 5 (Figure~\ref{Cofactor4b}).

Figure~\ref{Cofactor4b} shows the second peak on chromosome 4 at 10
\cM\ increases. Add a cofactor to the model and check if the model with both
cofactors changes the \qtl. Combining \code{find.markerindex} with \code{find.marker},
adds the new cofactor to the cofactor already in \code{multitoset} (see Figure~\ref{twowaycomparison}):

<<>>=
# summary(mqm_co1)
multitoset <- c(multitoset, find.markerindex(maug, find.marker(maug,4,10)))
setcofactors <- mqmsetcofactors(maug,cofactors=multitoset)
mqm_co2 <- mqmscan(maug, setcofactors)
@
% -----------

\intro{Plot after adding second cofactor on chromosome 4 at 10 \cM}

<<twowaycomparison,eval=FALSE>>=
par(mfrow = c(2,1))
plot(mqmgetmodel(mqm_co2))
plot(mqm_co1, mqm_co2, col=c("blue","green"), lty=1:2)
legend("topleft", c("one cofactor","two cofactors"), col=c("blue","green"),
       lwd=1)
@)

\begin{figure}
<<fig=TRUE,echo=FALSE>>=
<<twowaycomparison>>
@
\caption{\Atintro\ using an added cofactor on chromosome 5 (blue), versus two
cofactors, using an additional cofactor on chromosome 4 (green).  \label{twowaycomparison}} \end{figure}

% ---------------------------




\intro{Plot the results with 0, 1 and 2 cofactors as follows}

<<threewaycomparisonmulti,eval=FALSE>>=
plot(mqm, mqm_co1, mqm_co2, col=c("green","red","blue"), lty=1:3)
legend("topleft", c("no cofactors","one cofactor","two cofactors"),
       col=c("green","red","blue"), lwd=1)
@

\begin{figure}
<<fig=TRUE,echo=FALSE,height=3>>=
# plot closeup of threeway comparison
<<threewaycomparisonmulti>>
@
\caption{\Atintro. Comparison of \mqm\ adding 0 (green), 1 (red) and 2 (blue)
cofactor(s) (note that adding more cofactors
does not improve the two QTL
model).\label{threewaycomparison}}
\end{figure}


When using the functions \code{mqmsetcofactors}, or the automated
\code{mqmautocofactor} (described in the next section), the number of cofactors is compared against the number
of individuals inside the cross object. If there is a danger of setting too
many cofactors, an error message is shown.

\mqm\ also verifies the \code{cofactor.significance} level specified by the
user. In the example the marker on chromosome 1 was informative enough, and
included into the model.  This way a new initial model consisting of cofactors
on chromosome 4 and 5 was created. This (forward) selection of cofactors can
continue until there are no more informative markers.

Manually determining the markers to set a cofactor can be very time consuming
in the case of many \qtl\ underlying a trait. It is also prone to overfitting.
Furthermore, manual fitting is generally not feasible for a large number of
traits. Fortunately \mqm\ provides unsupervised backward elimination, which is
described in the next section.

\clearpage

\section{Unsupervised cofactor selection through backward elimination\label{backelim}}

\mqm\ provides unsupervised backward elimination on a large number of markers
by selecting cofactors automatically. Normally the number of markers in a
dataset is much larger than the number of individuals. \mqm\ allows using any number of
cofactors simultaneously. This can be as low as 0 cofactors up to a maximum of
the number of individuals minus 12 ($Inds - 12$), as described in the Handbook of Statistical Genetics\cite{jansen07}.

The functions: ``\code{mqmsetcofactor}'' and ``\code{mqmautocofactors}'' both
create lists of cofactors that can be used for backward elimination.
\code{mqmautocofactor} accounts for the underlying marker density and is
therefore suitable for datasets with few individuals.  See
Figure~\ref{ManualAuto} for a comparison on the \code{multitrait} dataset, using the
\code{mqmsetcofactors} function to set cofactors every 5th marker and
\code{mqmautocofactor} to set 50 cofactors across the genome. After cofactor
selection \mqm\ analyses and drops the least informative cofactor from the
model. This step is repeated until a limited number of informative cofactors
remain.  When taking marker density into account, an extra cofactor is
introduced on chromosome 1 (see Figure~\ref{ManualAuto}).

After unsupervised backward elimination \code{mqmscan} scans each chromosome
using the model with the remaining set of cofactors. For example, starting with 50
cofactors using \code{mqmautocofactor} and \code{mqmsetcofactors}, map \qtl\ for
the various traits in \code{multitrait}, which contains 24 metabolite traits
from a RIL population of \At\, as described in section \ref{multitrait}. The
\qtl\ LOD scores differ between \mqm\ and single \qtl\ mapping with
\code{scanone} (see Figures~\ref{Backward1} and \ref{Backward2}).

\intro{Unsupervised cofactor selection through backward elimination}

<<eval=FALSE>>=
autocofactors <- mqmautocofactors(maug, 50)
mqm_auto <- mqmscan(maug, autocofactors)
setcofactors <- mqmsetcofactors(maug, 5)
mqm_backw <- mqmscan(maug, setcofactors)
@

<<echo=FALSE>>=
autocofactors <- mqmautocofactors(maug, 50)
mqm_auto <- mqmscan(maug, autocofactors)
setcofactors <- mqmsetcofactors(maug, 5)
mqm_backw <- mqmscan(maug, setcofactors)
@

\intro{Visual inspection of the initial models}

<<ManualAutoStart,eval=FALSE>>=
par(mfrow = c(2,1))
mqmplot.cofactors(maug, autocofactors, justdots=TRUE)
mqmplot.cofactors(maug, setcofactors, justdots=TRUE)
@

\begin{figure}
<<fig=TRUE,echo=FALSE>>=
# plot result of cofactor selection
<<ManualAutoStart>>
@
\caption{\Atintro. \code{mqmsetcofactor} after introducing cofactors at every fifth marker (top) and
\code{mqmautocofactor} automatic marker selection (bottom). Automatic selection takes the underlying
marker density into consideration.\label{ManualAutoStart}}
\end{figure}

\intro{Plot results}

<<ManualAuto,eval=FALSE>>=
par(mfrow = c(2,1))
plot(mqmgetmodel(mqm_backw))
plot(mqmgetmodel(mqm_auto))
@

\begin{figure}
<<fig=TRUE,echo=FALSE>>=
# plot result of cofactor backward elimination
<<ManualAuto>>
@
\caption{\Atintro. \code{mqmsetcofactor} after introducing cofactors at every
fifth marker (top) and \code{mqmautocofactor} automatic marker selection
(bottom). \code{mqmautocofactor} places an additional cofactor at chromosome 1
(see also Figure~\ref{ManualAutoStart}). After backward elimination this extra
marker remains informative.\label{ManualAuto}} \end{figure}

<<Backward1multi,eval=FALSE>>=
par(mfrow = c(2,1))
plot(mqmgetmodel(mqm_backw))
plot(mqm_backw)
@

\begin{figure}
<<fig=TRUE,echo=FALSE>>=
# plot result of cofactor backward elimination
<<Backward1multi>>
@
\caption{\Atintro. Unsupervised cofactor selection through backward elimination using \code{mqmsetcofactor} after introducing cofactors at every
fifth marker. \qtl\ mapped for trait X3.Hydroxypropyl on chromosome 4 and 5.\label{Backward1}}
\end{figure}

The \code{mqmgetmodel} function returns the final model from the output of
\code{mqmscan}.  This model can be further investigated using the \code{fitqtl}
and \code{fitqtl} routines from R/qtl. \code{mqmgetmodel} can only be used
after backward elimination produces a significant model. The resulting model
can also be used to obtain the location and name of the significant cofactors.

\intro{Plot result of \mqm, using unsupervised backward elimination, against that of \code{scanone}}

<<Backward2,eval=FALSE>>=
plot(m_one, mqm_backw, col=c("black","green"), lty=1:2)
legend("topleft", c("scanone","MQM"), col=c("black","green"), lwd=1)
@

% ------------------------------
<<Backward2multi,eval=FALSE>>=
plot(m_one, mqm_backw, col=c("black","green"), lty=1:2)
legend("topleft", c("scanone","MQM"), col=c("black","green"), lwd=1)
@

\begin{figure}
<<fig=TRUE,echo=FALSE,height=3>>=
<<Backward2multi>>
@
\caption{\Atintro. Compare \qtl\ mapping of \mqm\, after introducing cofactors
at every fifth marker and unsupervised backward elimination of cofactors
(green, dashed), and \code{scanone} (black).\label{Backward2}}
\end{figure}

\mqm\ \qtl\ mapping may result in many significant (informative) cofactors.
Figure~\ref{Backward2} shows at \code{cofactor.significance=0.02} chromosomes 4
and 5 are involved. Lowering the significance level from 0.02 to 0.002 may
yield a smaller model.  In biology extensive models are sometimes preferred,
but in general a simpler model is easier to understand and, perhaps, validated.
Depending on the trait, and the sample size, increasing cofactor.significance
can reduce the number of significant QTL in the model. In this example we have
already have a small model, so we don't really expect to lose the two QTL on
chromosome 4 and chromosome 5. When decreasing the \code{cofactor.significance}
no additional cofactors are dropped from the model (See Figure~\ref{lowalpha})

\clearpage

\intro{Plot with lowered \code{cofactor.significance}}

<<FigLowAlpha,eval=FALSE>>=
mqm_backw_low <- mqmscan(maug, setcofactors, cofactor.significance=0.002)
par(mfrow = c(2,1))
plot(mqmgetmodel(mqm_backw_low))
plot(mqm_backw,mqm_backw_low, col=c("blue","green"), lty=1:2)
legend("topleft", c("Significance=0.02","Significance=0.002"),
       col=c("blue","green"), lwd=1)
@

\intro{\qtl\ mapped with different \code{cofactor.significance=0.002}, using the
same starting markers as Figure~\ref{Backward1}. As can be seen from the plot the
models selected are similar. This means the QTL found significant at 0.02 are still
significant at a more restrictive cutoff.\label{lowalpha}}

\begin{figure}
<<fig=TRUE,echo=FALSE>>=
<<FigLowAlpha>>
@
\caption{\Atintro.
\qtl\ mapped with different \code{cofactor.significance}.
With the lower \code{cofactor.significance=0.002} the model does not change.\label{FigLowAlpha}}
\end{figure}

When comparing the \mqm\ scan in Figure~\ref{FigLowAlpha} with the original
\code{scanone} result in Figure~\ref{Backward2} there are some notable
differences.  Some \qtl\ show higher significance (LOD scores) and some others
show lower significance and are, therefore, estimated to be less likely
involved in this trait.

Figures can be reconstructed from the result of \code{mqmscan} using the
\code{mqmplot.singletrait} function (see, for example,
Figure~\ref{AutoCofactor}). Here the model and \qtl\ profile are retrieved.
These functions can only be used with \code{mqmscan} functions, as they require
the additional information about the inferred \qtl\ model.  The results also
contain the \emph{estimated} information content per marker.

% <<AutoCofactor,eval=FALSE>>=
% mqmplot.singletrait(result, extended=TRUE)
% @

<<AutoCofactor,eval=FALSE>>=
mqmplot.singletrait(mqm_backw_low, extended=TRUE)
@

\begin{figure}
<<fig=TRUE>>=
<<AutoCofactor>>
@
\caption{\Atintro. Plot using \code{mqmplot.singletrait} of the first metabolic trait. The information
content per marker (red) and the mapped \qtl\ (black).\label{AutoCofactor}}
\end{figure}

The information content \code{info} in the result is calculated from the deviation of
the `ideal marker distribution'. For example, with a dataset of 100
individuals, when comparing two distinct phenotypes at a marker location, we
have most power when both groups are equally divided 50/50. A marker has
virtually no power when one group containing 1 individual versus a group of 99.
We can multiply the estimated \qtl\ effect by this information content to
`clean' the QTL profile by giving less weight to less informative markers.
Please note that the sample size already plays a role in calculating \qtl.
Meanwhile it allows (informal) further weighting/exploring \emph{information}
content (Figure~\ref{AutoCofactor}).

\section{MQM effect plots}
\label{effectplots}

The function \code{mqmplot.directedqtl} is used to plot \lod\ curves with an
indication of the sign of the estimated QTL effects. This function because
it uses internal R/qtl functions cannot handle augmented cross objects. An
error will occur when the object supplied is augmented using \code{mqmaugment}.
This requires using \code{mqmscan} with parameter \code{outputmarkers=TRUE} (default).

\intro{Create a directed QTL plot (Figure~\ref{QTLeffects})}

<<QTLeffects,eval=FALSE>>=
dirresults <- mqmplot.directedqtl(multitrait, mqm_backw_low)
@

\begin{figure}
<<fig=TRUE,echo=FALSE,height=3.5>>=
<<QTLeffects>>
@
\caption{\Atintro. Like Figure~\ref{AutoCofactor}, but with LOD scores
multiplied by $\pm$1 according to the sign of the estimated \qtl\ effect.\label{QTLeffects}}
\end{figure}

The results in Figure~\ref{FigLowAlpha} imply that \qtl\ on chromosomes 4 and 5 are
associated with the metabolite X3.Hydroxypropyl. If we want to investigate the effects
of the QTL, we can use the functions \code{plotPXG} and \code{effectplot}.
The following plots show these for markers GH.117C (main effect,
Figure~\ref{MainEffectsD1}) and the interaction between GH.117C and
GA1 (Figure~\ref{epistatic1}).

<<MainEffectsD1,echo=FALSE,eval=FALSE>>=
plotPXG(multitrait, marker="GH.117C")
@

\begin{figure}
<<fig=TRUE, echo=FALSE,height=3>>=
<<MainEffectsD1>>
@
\caption{\Atintro. Main effect plot, with \code{plotPXG}, of marker GH.117C on
trait X3.Hydroxypropyl. For each marker genotype the individual phenotype is
plotted, with the mean of genotype AA (red) and BB (blue).\label{MainEffectsD1}} \end{figure}

%----------------------------------------------

The initial scans for X3.Hydroxypropyl (Figure~\ref{Backward1}) show
two possible \qtl\ on chromosome 4 and 5.  We can investigate
interactions between these main effect QTL using the \code{effectplot}
function. To investigate the possible epistatic interaction, select markers GA1
(significant in Figure~\ref{Backward1} and Figure~\ref{Backward2}) and GH.117C
(significant in Figure~\ref{Backward2} and \ref{AutoCofactor}). See
Figure~\ref{epistatic1}.

<<epistatic1,eval=FALSE>>=
effectplot(multitrait, mname1="GH.117C", mname2="GA1")
@

\begin{figure}
<<fig=TRUE,echo=FALSE,height=3.5>>=
<<epistatic1>>
@
\caption{\Atintro. Explore epistatic interaction, using \code{effectplot},
between markers GH.117C and GA1.  GA1 appears to obscure the effect of GH.117C.
An individual that has BB at GA1 has no difference in expression between being
AA or BB at GH.117C. However when an individual is AA at GA1 there is clear
difference between the two genotype means (~1.500 BB versus 12000 when AA) at
GH.117C. \label{epistatic1}}
\end{figure}

%------------------------------------------------


Likewise, in case we are interested in the interactions between the first small
hump on chromosomes 1 (marker: PVV4 not significant) and the main efect on 5
(GH.117C), we could make interaction plots between these two markers with a
high LOD score on those chromosomes.  See Figure~\ref{epistatic2}.

<<epistatic2,eval=FALSE>>=
effectplot(multitrait, mname1="PVV4", mname2="GH.117C")
@

Meanwhile, Figure~\ref{epistatic2} shows no evidence for an interaction between
the two markers GH.117C and PVV4, as the lines are close to parallel.

\begin{figure}
<<fig=TRUE,echo=FALSE,height=3.5>>=
<<epistatic2>>
@
\caption{\Atintro. \code{effectplot} shows no epistatic effects between
markers GH.117C and PVV4, ths can be seen because the two lines run in parallel,
the genotype on one location (PVV4) does not affect the effect of the expression
on GH.117C other location. \label{epistatic2}}
\end{figure}

%------------------------------------------------

\clearpage

\section{QTL significance}

\label{significance}

To estimate the significance of \qtl\, and perhaps further exclude markers from a
model, permutation testing is provided by the function \code{mqmpermutation}. This step
is computationally expensive\footnote{In the tutorial, for all examples, 25
permutations are used. A real experiment should use over 1000 permutation tests.}
as the same test is repeated many times on shuffled data. Each test calculates
\lod\ scores for non associated (randomly ordered) data.

\mqm\ provides parametric and non-parametric bootstrapping to estimate \qtl\
significance. Select the type with the \code{bootmethod} parameter.  If you
have are lucky enough to have multiple CPUs on your computer you can use the
SNOW package \cite{tierney03,tierney04}, which allows parallel computations on
multiple CPU/cores.

SNOW is available through the Internet R archive CRAN. For example, with Rgui
SNOW can be installed by selecting from the menu: `Packages' and `Install
Package(s)` from the drop down menu. Select a CRAN mirror near you and select
the SNOW package. Rgui will start downloading the package, and install any
dependencies needed. Linux users can download a copy of SNOW from
\url{https://cran.r-project.org/package=snow}. Once the package
has finished downloading the tar.gz file can be installed using \code{R CMD
INSTALL snow.tar.gz}

To summarize results from \code{mqmpermutation}, \code{mqmprocespermutation}
makes the output comparable to \code{scanone} when using the \code{n.perm}
parameter for permutation.

\intro{Calculate significance - using SNOW parallelization parameters}
<<>>=
require(snow)
results <- mqmpermutation(maug, scanfunction=mqmscan, cofactors=setcofactors,
                          n.cluster=2, n.perm=25, batchsize=25)
@
\begin{figure}
<<fig=TRUE>>=
mqmplot.permutations(results)
@
\caption{\Atintro. \qtl\ significance calculated through permutation using
\code{mqmpermutation}.  Estimate by permuting a single trait (X3.Hydroxypropyl)
with randomly distributing trait values amongst individuals, which gives an
indication of \lod\ scores found by chance. \qtl\ with a \lod\ higher than 2.5
can be considered significant (at \code{cofactor.significance=0.05} (green) or
\code{cofactor.significance=0.10} (blue)).  Chromosome are marked by the gray
vertical grid lines.}
\end{figure}

<<>>=
resultsrqtl <- mqmprocesspermutation(results)
summary(resultsrqtl)
@

For small datasets, with a limited amount of classical traits,
\code{mqmpermutation} is nice.  However, for large expression studies (eQTL)
using microarrays, use \code{mqmscanfdr} instead, which estimates false discovery
rates (FDR) across the entire dataset at \lod\ cutoff, as described by
Breitling \etal\cite{breitling08}.

To estimate the FDR, \code{mqmscanfdr} permutes whole genome information,
taking correlation between traits into account and giving an unbiased estimate
of FDR at different (user specified) thresholds. The function scans the traits
and counts observed \qtl\ with a LOD above \code{x}, setting a certain
threshold. It permutes all the data leaving the correlation structure between
traits intact. Below, very high FDR estimates are calculated because of a small
amount of permutations and high correlation between traits.  We discover many
QTL that map to the same location. This can normally only happen with
information sparse marker(s), or correlated traits, as seen in microarray
experiments.

\intro{Calculate FDR}
<<>>=
data(multitrait)
m_imp <- fill.geno(multitrait)
mqmscanfdr(m_imp, mqmscanall, cofactors=setcofactors, n.cluster=2)
@

In contrast, the function \code{mqmpermutation} does single trait permutations,
and does not take correlation between the traits into account. The advantage is
that a permutation threshold is determined for each trait. This leads to different
significance levels per trait and could lead to certain \qtl\ being significant at
their trait cut-off, which are not significant when a single cut-off.  The \mqm\
output needs to be converted to the standard R/qtl format using the
\code{mqmprocesspermutation} function. The resulting object is of class
\code{scanoneperm} and can be used by the standard R/qtl functions for further
analysis.

To parallelize calculations \code{n.cluster} sets the number of CPU cores to
use. A batch consists of a number of traits to analyze on one core.  A large(r)
\code{batchsize} (default 10) can also be set to improve efficiency. Every time
a batch is sent a new instance of R is started, so it pays to have as few
batches as possible.

\section{Parallelized \xqtl\ analysis}

\mqm\ can handle high throughput \xqtl\ data - the name coined for the family
of expression QTL, or eQTL \cite{jansen01}, metabolite QTL (mQTL) and pQTL
(protein QTL), where measurements like gene expression on microrray probes are
treated as phenotypes.  \mqm\ analyses traits simultaneously using parallel
computing on multiple CPU/cores, and even computer clusters. \xqtl datasets
(expression eQTL, metabolite mQTL) usually contain a large amount of phenotypes
with known locations on the genome. These locations can be used for detecting
cis/trans regulation, for example. For QTL mapping every phenotype requires one
or more calls to \code{mqmscan}. In addition special plots are presented for
\xqtl studies.

Our example, the mQTL dataset \code{multitrait}, an \At\ RIL cross, containing
24 metabolites measured as phenotypes. Of these 24 phenotypes we will only scan
the first five phenotypes by setting the \code{pheno.col} parameter. To map back
the regulatory locations of these metabolites one can use plain scanning of all
metabolites (initially without cofactors). Next, we plot all the profiles in a
heatmap (see Figure~\ref{At.heatmap1}). In this heatmap the colors represent
the LOD score, on the x-axis the marker number and on the y-axis the
metabolite. The traits are numbered in the plot. Plot heatmap without cofactors
and then the heatmap with cofactors and backward elimination.
Figure~\ref{At.heatmap2} shows improvement over Figure~\ref{At.heatmap1}
because of an improved signal to noise ratio.

<<>>=
data(multitrait)
m_imp <- fill.geno(multitrait)
mqm_imp5 <- mqmscan(m_imp, pheno.col=1:5, n.cluster=2)
@
\begin{figure}
<<fig=TRUE>>=
mqmplot.multitrait(mqm_imp5, type="image")
@
\caption{\Atintro. Heatmap of five metabolite expression traits, with profiles
created using \mqm\ without preselected cofactors. The colors represent the LOD
score, on the x-axis the marker number and on the y-axis the
metabolite.\label{At.heatmap1}} \end{figure}


<<>>=
cofactorlist <- mqmsetcofactors(m_imp, 3)
mqm_imp5 <- mqmscan(m_imp, pheno.col=1:5 , cofactors=cofactorlist,
                    n.cluster=2)
@
\begin{figure}
<<fig=TRUE>>=
mqmplot.multitrait(mqm_imp5, type="image")
@
\caption{\Atintro. Heatmap of metabolite expression traits, with profiles
created using \mqm\ with cofactors at each third marker. The colors represent
the LOD score, on the x-axis the marker number and on the y-axis the
metabolite.\label{At.heatmap2}} \end{figure}

Use \code{mqmplot.multitrait} for more graphical output. (Unfortunately this does not
show in the generated PDF, but in R it shows the trait profiles)

<<>>=
mqmplot.multitrait(mqm_imp5, type="lines")
@

Next is \code{mqmplot.circle}. The circle plot shows a circular representation
of the genome. After using automatic backward selection certain
marker/cofactors are found to be significant. These are highlighted and
colored. The cofactor size can be scaled, based on significance (see
Figures~\ref{circle1} and \ref{circle2}).

The plot can be tweaked. For example, \code{highlight} a specific trait, and
calculate interactions between the significant cofactors. All other traits are
grayed out, but remain partly visible, in this way it is possible to see if
significant QTL for this trait are also colocated with other traits. Parameter
\code{interactstrength}: highlights interactions between significant markers.
However they are only drawn (and reported in the output) if the effect change
is larger than \code{interactstrength} multiplied by the summed standard
deviation. Parameter \code{spacing} sets space between the chromosomes in Cm.


\begin{figure}
<<fig=TRUE>>=
mqmplot.circle(m_imp, mqm_imp5)
@
\caption{\Atintro. Circle plot 1 - Multiple metabolic traits without known
locations. Four traits are in the centre connected by a colored spline to their
QTL. Significant QTL locations are depicted as solid square circles. A lower
\lod\ score is closer to the center. A 'hotspot' of \qtl\ is visible on
chromosome 5. \label{circle1}} \end{figure}

\begin{figure}
<<fig=TRUE>>=
mqmplot.circle(m_imp, mqm_imp5, highlight=2)
@
\caption{\Atintro. Circle plot 2 - Multiple metabolic traits without known
locations. Highlight the second trait. The significant \qtl\ locations are
depicted as solid red square circles. The splines show epistatic interactions
(see also Figure~\ref{epistatic1}).  The blue lines are locations which are
modulating expression (higher or lower), the green lines show a flip in effect.
To explain this: with two markers, having AA at marker one shows trait mean AA
> trait mean BB at marker two however when the individual has BB at marker one
the effect at marker two is reversed AA < BB.  \label{circle2}} \end{figure}

Next a cis-trans plot with \code{mqmplot.cistrans}. This plot is only available
when genomic locations of the traits are known, e.g. the genomic probe
locations in microarray eQTL studies. By default the R/qtl cross object does
not store this data. So the user has to add this information to the cross
object using the \code{addloctocross} function. After this operation the
cis-trans plot can be created for \qtl\ with associated genome locations.

The two axis of the cis-trans plot both show the genetic location. The X-axis
is, normally, the \qtl\ location and the Y-axis the locations of the trait.

\begin{figure}
<<fig=TRUE>>=
data(locations)
multiloc <- addloctocross(m_imp, locations)
mqmplot.cistrans(mqm_imp5, multiloc, 5, FALSE, TRUE)
@
\caption{\Atintro. \code{mqmplot.cistrans} can be drawn when \qtl\
have associated genome locations. \qtl\ are plotted against the position on the
genome they were measured (here mQTL for \At), cutoff is at \lod=5. Normally
these plots are created using 10.000 + traits. However because this tutorial
is automatically generated we only use 5 traits to illustrate.}
\end{figure}

\intro{When having locations we can, again, use the \code{mqmplot.circle} function, now with
the extra information}

\begin{figure}
<<fig=TRUE>>=
mqmplot.circle(multiloc, mqm_imp5, highlight=2)
@
\caption{\Atintro. Circle plot 3 - Multiple metabolic traits with known
locations. Highlight the second trait. The significant \qtl\ locations are
depicted as solid red square circles. The known location of the trait is a
red triangle. The splines show epistatic interactions
(see also Figure~\ref{epistatic1}).  The blue lines are locations which are
modulating expression (higher or lower), the green lines show a flip in effect.\label{circle3}} \end{figure}

\clearpage

\section{Overview of all \mqm\ functions}
\begin{table}[ht]
    \caption{Added functionality}
    \centering
    \begin{tabular}{| l | l | }
    \hline
    mqmaugment:& data augmentation \\
    mqmscan:& \mqm\ modelling and scanning \\
    mqmsetcofactors:& Set cofactors at markers (or at fixed locations) \\
    find.markerindex:& Change marker numbering into mqmformat \\
    mqmscanall:& mqmscanall scans all traits using \mqm\\
    mqmpermutation:& Single trait permutation \\
    mqmscanfdr:& Genome wide False Discovery Rates (FDR) \\
    mqmprocesspermutation:& Creates an R/qtl permutationobject \\
                & \hspace{0.25in} from the output of the \code{mqmpermutation} function \\
    mqmplot.multitrait:& plot multiple traits (MQMmulti object) \\
  mqmplot.directedqtl:& plot of single trait with added \qtl\ effect\\
    mqmplot.permutations:& plot to show single trait permutations \\
    mqmplot.singletrait:& plot of single trait analysis with information content \\
    mqmplot.circle:& Genome plot of \qtl\ in a circle (optional: Use of location information)\\
    mqmplot.cistrans:& Genomewide plot of cis- and trans-\qtl, above a threshold \\
  addloctocross:& Adding genetic locations for traits \\
    mqmtestnormal:& Test normality of a trait \\
    \hline
    \end{tabular}
    \label{tbl:tabel1}
\end{table}

\clearpage

\begin{thebibliography}{9}
    \bibitem{broman09}
        Broman, K.W.; 2009
        \emph{A brief tour of R/qtl.}
        https://rqtl.org/tutorials/rqtltour.pdf.
  \bibitem{rqtlbook}
    Broman, K. W.; Sen, \'S; 2009
      \emph{A Guide to QTL Mapping with R/qtl.}
    Springer.
    \bibitem{broman03}
        Broman, K.W.; Wu, H.; Sen, S.; Churchill, G.A.; 2003
        \emph{R/qtl: QTL mapping in experimental crosses.}
        Bioinformatics, 19:889--890.
    \bibitem{jansen07}
        Jansen R. C.; 2007
        \emph{Chapter 18 - Quantitative trait loci in inbred lines.}
        Handbook of Statistical Genetics, 3rd edition.  Wiley.
    \bibitem{tierney04}
        Tierney, L.; Rossini, A.; Li, N.; and Sevcikova, H.; 2004
        \emph{The snow Package: Simple Network of Workstations.} Version 0.2-1.
    \bibitem{tierney03}
        Rossini, A.; Tierney, L.; and Li, N.; 2003
        \emph{Simple parallel statistical computing.}
        R. University of Washington Biostatistics working paper series, 193.
    \bibitem{jansen01}
        Jansen R. C.; Nap J.P.; 2001
        \emph{Genetical genomics: the added value from segregation.}
        Trends in Genetics, 17, 388--391.
    \bibitem{jansen94}
        Jansen R. C.; Stam P.; 1994
        \emph{High resolution of quantitative traits into multiple loci via interval mapping.}
        Genetics, 136, 1447--1455.
  \bibitem{jansen94b}
      Jansen R.C.; 1994
        \emph{Controlling the Type I and Type II Errors in Mapping Quantitative Trait Loci.}
    Genetics, Vol 138, 871--881.
    \bibitem{Churchill94}
        Churchill, G. A.; and Doerge, R. W.; 1994
        \emph{Empirical threshold values for quantitative trait mapping.}
        Genetics 138, 963--971.
    \bibitem{jansen93}
        Jansen R. C.; 1993
        \emph{Interval mapping of multiple quantitative trait loci.}
        Genetics, 135, 205--211.
    \bibitem{Dempster77}
        Dempster, A. P.; Laird, N. M. and Rubin, D. B.; 1977
        \emph{Maximum likelihood from incomplete data via the EM algorithm.}
        J. Roy. Statist. Soc. B, 39, 1--38.
  \bibitem{CIMa}
         Zeng, Z. B.; 1993 \emph{Theoretical basis for separation of
           multiple linked gene effects in mapping quantitative trait
           loci.} Proc. Natl. Acad. Sci. USA, 90, 10972--10976.
  \bibitem{CIMb}
         Zeng, Z. B.; 1994 \emph{Precision mapping of quantitative
           trait loci.}
         Genetics, 136, 1457--1468
  \bibitem{sugiyama}
    Sugiyama, F.; Churchill, G.A.; Higgins, D.C.; Johns, C.;
    Makaritsis, K.P.; Gavras, H.; Paigen, B.; 2001 \emph{Concordance
      of murine quantitative trait loci for salt-induced hypertension
      with rat and human loci.} Genomics, 71, 70--77.
  \bibitem{Keurentjes2006}
Keurentjes, J. J.; Fu, J.; de Vos, C. H.; Lommen, A.; Hall, R. D.;
Bino, R. J.; van der Plas, L. H.; Jansen, R. C.; Vreugdenhil, D.;
Koornneef, M.;
     2006 \emph{The genetics of plant metabolism.} Nature Genetics.
     38, 842--849.
  \bibitem{Koornneef1998}
Alonso-Blanco, C.; Peeters, A. J.; Koornneef, M.; Lister, C.; Dean, C.; van den Bosch, N.; Pot, J.; Kuiper, M. T.; 1998
     \emph{
Development of an AFLP based linkage map of Ler, Col; Cvi
     Arabidopsis thaliana ecotypes; construction of a Ler/Cvi recombinant
     inbred line population.} Plant J. 14, 259--271.
  \bibitem{breitling08}
Breitling, R.; Li, Y.; Tesson, B. M.; Fu, J.; Wu, C.; Wiltshire, T.; Gerrits, A.; Bystrykh, L. V.; de Haan, G.; Su, A. I.; Jansen, R. C.; 2008
\emph{Genetical genomics: spotlight on QTL hotspots.} PLoS Genet. 4/10.
\end{thebibliography}
\end{document}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Karl W. Broman
% The new summary.scantwo and plot.scantwo
%
% This is an "Sweave" document; see the corresponding PDF.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\documentclass[12pt]{article}

%\usepackage{times}
\usepackage{amsmath}
\usepackage{color}
\usepackage{times}

% revise margins
\setlength{\headheight}{0.0in}
\setlength{\topmargin}{-0.25in}
\setlength{\headsep}{0.0in}
\setlength{\textheight}{9.00in}
\setlength{\footskip}{0.5in}
\setlength{\oddsidemargin}{0in}
\setlength{\evensidemargin}{0in}
\setlength{\textwidth}{6.5in}

\setlength{\parskip}{6pt}
\setlength{\parindent}{0pt}

\newcommand{\code}{\texttt}
\newcommand{\lod}{\text{LOD}}



\begin{document}

\SweaveOpts{prefix.string=Figs/scantwo,eps=TRUE}
\setkeys{Gin}{width=\textwidth} %% <- change width of figures
% Try to get the R code from running into the margin
<<echo=FALSE>>=
options(width=77)
@ 

% Change S input/output font size
\DefineVerbatimEnvironment{Sinput}{Verbatim}{fontsize=\footnotesize, baselinestretch=0.75, formatcom = {\color[rgb]{0, 0, 0.56}}}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontsize=\footnotesize, baselinestretch=0.75, formatcom = {\color[rgb]{0.56, 0, 0}}}



\textbf{The new \code{summary.scantwo} and \code{plot.scantwo}} \\
Karl W Broman, 27 Oct 2006 \\
(Added color 26 Oct 2010; slight change 21 Mar 2012: \verb|summary.scantwo.old| is now \verb|summaryScantwoOld|)

\bigskip

In R/qtl version 1.04, the functions \code{summary.scantwo} and
\code{plot.scantwo} have been changed quite substantially.  Also, the
permutations with \code{scantwo} have been changed to match the new
format for \code{summary.scantwo}.

In this document, I describe the revisions and how to use the new
functions.   We'll look at the \code{hyper} data as an example.  

First we need to load the package and the data.
<<loaddata,eval=FALSE>>=
library(qtl)
data(hyper)
@ 

<<loadresults,echo=FALSE>>=
load("hyper_results.RData")
@ 

I'm going to use \code{scantwo} with \code{method="em"}.  First I run
\code{calc.genoprob}, and then \code{scantwo} as before.

<<scantwo,eval=FALSE>>=
hyper <- calc.genoprob(hyper, step=2.5)
out2 <- scantwo(hyper)
@ 

The output, in this new version of R/qtl, is slightly different.
The LOD scores for the full model (two QTL plus interaction) are there
as before, but in place of the epistasis LOD scores, I save the LOD
scores for the additive QTL model.  Also, we now always run the
single-QTL analysis with \code{scanone}, since the results are
necessary to make sense of the output of \code{scantwo}.

The big change is in \code{summary.scantwo}.  Consider a pair of
positions in the genome, $s$ and $t$.  We consider four models.
\begin{eqnarray*}
\text{Full: } && y = \mu + \beta_1 q_1 + \beta_2 q_2 + \beta_3 (q_1
\times q_2) + \epsilon \\
\text{Add: } && y = \mu + \beta_1 q_1 + \beta_2 q_2 + \epsilon \\
\text{One: } && y = \mu + \beta_1 q_1 + \epsilon \\
\text{Null: } && y = \mu + \epsilon
\end{eqnarray*}

Let $l_f(s,t)$ be the log$_{10}$ likelihood for the full model with QTL
at $s$ and $t$, $l_a(s,t)$ be the log$_{10}$ likelihood for the additive
model with QTL at $s$ and $t$, $l_1(s)$ be the log$_{10}$ likelihood
for the single-QTL model with the QTL at $s$, and $l_0$ be the
log$_{10}$ likelihood under the null (with no QTL).  

Define the LOD scores as follows.
\begin{eqnarray*}
\lod_f(s,t) & = & l_f(s,t) - l_0 \\
\lod_a(s,t) & = & l_a(s,t) - l_0 \\
\lod_1(s) & = & l_1(s) - l_0 
\end{eqnarray*}

Now for the new part.  Following a suggestion from Gary Churchill, we
consider a pair of chromosomes $j$ and $k$.  (We include the case
$j=k$.)  Let $c(s)$ denote the chromosome for position $s$.  We now
consider the maximum LOD scores over that pair of chromosomes.
\begin{eqnarray*}
M_f(j,k) & = & \max_{c(s)=j, c(t)=k} \lod_f(s,t) \\
M_a(j,k) & = & \max_{c(s)=j, c(t)=k} \lod_a(s,t) \\
M_1(j,k) & = & \max_{c(s)=j \text{ or } k} \lod_1(s)
\end{eqnarray*}

So $M_f(j,k)$ is the log$_{10}$ likelihood ratio comparing the full
model with QTL on chromosomes $j$ and $k$ to the null model, and
$M_a(j,k)$ is the analogous thing for the additive model.  Note that
the pair of positions at which the full model is maximized may be
different from the pair of positions at which the additive model is
maximized.  $M_1(j,k)$ is the log$_{10}$ likelihood ratio comparing
the model with a single QTL on either chromosomes $j$ or $k$ to the
null model.

We derive three further LOD scores from the above.
\begin{eqnarray*}
M_i(j,k) & = & M_f(j,k) - M_a(j,k) \\
M_{fv1}(j,k) & = & M_f(j,k) - M_1(j,k) \\
M_{av1}(j,k) & = & M_a(j,k) - M_1(j,k) 
\end{eqnarray*}

$M_i(j,k)$ is the log$_{10}$ likelihood ratio comparing the full model
with QTL on chromosomes $j$ and $k$ to the additive model with QTL on
chromosomes $j$ and $k$, and so indicates evidence for an interaction
between QTL on chromosomes $j$ and $k$, assuming that there is precisely
one QTL on each chromosome (or, for $j=k$, that there are two QTL
on the chromosome).

$M_{fv1}(j,k)$ is the log$_{10}$ likelihood ratio comparing the full model
with QTL on chromosomes $j$ and $k$ to the single-QTL model, with a
single QTL on either chromosome $j$ or $k$.  Thus, it indicates
evidence for a second QTL, allowing for the possibility of epistasis.

$M_{av1}(j,k)$ is the log$_{10}$ likelihood ratio comparing the additive model
with QTL on chromosomes $j$ and $k$ to the single-QTL model, with a
single QTL on either chromosome $j$ or $k$.  Thus, it indicates
evidence for a second QTL, assuming no epistasis.

In \code{summary.scantwo}, we must provide thresholds for each of the
five LOD scores, $M_f(j,k)$, $M_{fv1}(j,k)$, $M_i(j,k)$, $M_a(j,k)$
and $M_{av1}(j,k)$.  A pair of chromosomes $(j,k)$ is reported as
interesting if either of the following holds:
\begin{itemize}
\item $M_f(j,k) \ge T_f$ and [$M_{fv1}(j,k) \ge
  T_{fv1}$ or $M_i(j,k) \ge T_i$]
\item $M_a(j,k) \ge T_a$ and $M_{av1}(j,k) \ge T_{av1}$
\end{itemize}

I'm inclined towards ignoring $M_i(j,k)$ in this rule (i.e. setting
$T_i = \infty$), and using a common significance level ($\alpha$ = 5 or
10\%) for the four remaining thresholds.

By default, \code{summary.scantwo} now calculates the five LOD scores
above, keeping track of the positions at which $M_f(j,k)$ and
$M_a(j,k)$ were maximized.  It either prints the best results on all
pairs of chromosomes, or we must provide five thresholds ($T_f$,
$T_{fv1}$, $T_i$, $T_a$ and $T_{av1}$, in that order). 

The thresholds can be obtained by a permutation test (see below), but
this is extremely time-consuming.  For a mouse backcross, we suggest
the thresholds (6.0, 4.7, 4.4, 4.7, 2.6) for the full,
conditional-interactive, interaction, additive, and
conditional-additive LOD scores, respectively.  For a mouse
intercross, we suggest the thresholds (9.1, 7.1, 6.3, 6.3, 3.3) for the
full, conditional-interactive, interaction, additive, and
conditional-additive LOD scores, respectively.  These were obtained by
10,000 simulations of crosses with 250 individuals, markers at a 10 cM
spacing, and analysis by Haley-Knott regression.

<<summaryscantwoA>>=
summary(out2, thresholds=c(6.0, 4.7, 4.4, 4.7, 2.6))
@ 

$M_f$, $M_{fv1}$ and $M_i$ are \code{lod.full}, \code{lod.fv1} and
\code{lod.int}, respectively, and correspond to positions \code{pos1f}
and \code{pos2f}.

$M_a$ and $M_{av1}$ are \code{lod.add} and \code{lod.av1},
respectively, and correspond to positions \code{pos1a} and
\code{pos2a}. 

The above is the default output, with \code{what="best"}.  The
argument \code{what} may also be given as \code{"full"}, \code{"add"},
or \code{"int"}, in which case, for each pair of chromosomes, we pull
out the pair of positions with maximum full, additive, or interactive
LOD score, respectively, and calculate, for example, the interaction
LOD score as the difference between the full and additive LOD scores
\emph{for that fixed pair of positions}, rather than allow the full
and additive models to be maximized at different positions.  (This is
more like what we did before, and is included just for completeness.)
The same set of five thresholds is required.

<<summaryscantwoB>>=
summary(out2, thresholds=c(6.0, 4.7, 4.4, 4.7, 2.6), what="full")
summary(out2, thresholds=c(6.0, 4.7, 4.4, 4.7, 2.6), what="add")
summary(out2, thresholds=c(6.0, 4.7, 4.4, 4.7, 2.6), what="int")
@ 

One may also restrict the summary to just the case of $j=k$, to look
at evidence for linked QTL on each chromosome, by using
\code{allpairs=FALSE}.  

<<summaryscantwoC>>=
summary(out2, allpairs=FALSE)
@ 

Note also that the degrees of freedom associated with each LOD score
may be displayed, via \code{df=TRUE}:

<<summaryscantwoD>>=
summary(out2, thresholds=c(6.0, 4.7, 4.4, 4.7, 2.6), df=TRUE)
@ 

The old version of \code{summary.scantwo} is still available, though
it is now called \code{summaryScantwoOld}.

<<oldsummaryscantwo>>=
summaryScantwoOld(out2, thresholds=c(6, 4, 4))
@ 

The permutation test with \code{scantwo} has also changed.  At each
permutation replicate, we record the maxima for each of the
$M_f(j,k)$, $M_{fv1}(j,k)$, $M_i(j,k)$, $M_a(j,k)$ and
$M_{av1}(j,k)$.  The output is given class \code{"scantwoperm"}, and
there is a \code{summary.scantwoperm} function for getting LOD
thresholds.  These permutations can take a very long time, and so one
would generally use a multi-processor computer or cluster and do
multiple shorter runs in parallel.  And so we have added a function
\code{c.scantwoperm} for combining such runs together.

%In addition, we have added an argument \code{perm.strata} for doing
%stratified permutation tests.  In the \code{hyper} data, the extremes
%were genotyped at most markers, and the rest of the individuals were
%genotyped only in selected regions.  This suggests that one should do
%a stratified permutation test, permuting individuals within
%the more completely genotyped group and separately permuting those
%within the less completely genotyped group.  

We could perform the \code{scantwo} permutations in five batches, as
follows.  
<<scantwoperm,eval=FALSE>>=
operm2A <- scantwo(hyper, n.perm=200)
operm2B <- scantwo(hyper, n.perm=200)
operm2C <- scantwo(hyper, n.perm=200)
operm2D <- scantwo(hyper, n.perm=200)
operm2E <- scantwo(hyper, n.perm=200)
operm2 <- c(operm2A, operm2B, operm2C, operm2D, operm2E)
@ 

The 5 and 20\% thresholds could be calculated as follows.
<<summaryscantwoperm>>=
summary(operm2, alpha=c(0.05,0.20))
@ 

The permutation results may also be used within the
\code{summary.scantwo} function, to automatically calculate the
thresholds for desired significance levels.  In this case, rather than
provide \code{thresholds}, one provides \code{alphas}, which again
should be a vector of length 5, giving the significance levels for 
$M_f(j,k)$, $M_{fv1}(j,k)$, $M_i(j,k)$, $M_a(j,k)$ and
$M_{av1}(j,k)$, in that order.
<<summaryscantwopermB>>=
summary(out2, perms=operm2, alphas=rep(0.05, 5))
@ 

My version of the decision rule, in which $M_{i}(j,k)$
is ignored, could be obtained as follows:
<<summaryscantwopermC>>=
summary(out2, perms=operm2, alphas=c(0.05, 0.05, 0, 0.05, 0.05))
@ 

In the case that permutation results are provided,
genome-scan-adjusted p-values may also be displayed, via 
\code{pvalues=TRUE}:

<<summaryscantwoD>>=
summary(out2, perms=operm2, alphas=c(0.05, 0.05, 0, 0.05, 0.05), 
        pvalues=TRUE)
@ 

I have also made an important change in \code{plot.scantwo}.  The
arguments \code{upper} and \code{lower} control what is plotted in the
upper-left and lower-right triangles, respectively.
The options are \code{"full"}, \code{"add"}, \code{"cond-int"}, 
\code{"cond-add"}, and \code{"int"}.  The case \code{"full"} is what
was previously called \code{"joint"}, but this and the case
\code{"add"} are not changed; the LOD scores for the full model (two
QTLs plus interaction) and the additive model are displayed.

The other two cases, \code{"cond-int"}, and
\code{"cond-add"}, are quite different.  
We now plot the following LOD scores:
\begin{eqnarray*}
\text{\code{"cond-int"}: } && \lod_{fv1}(s,t) = \lod_f(s,t) - M_1[c(s), c(t)] \\
\text{\code{"cond-add"}: } && \lod_{av1}(s,t) = \lod_a(s,t) - M_1[c(s), c(t)] 
\end{eqnarray*}
When these values are negative, they are replaced with 0.  
Before, we had subtracted off the maximum of the single-QTL LOD scores
at the points $s$ and $t$.  Now we subtract off the maximum of the
single-QTL LOD scores for the chromosomes containing $s$ and $t$.  

Note that these LOD scores will be maximized at the same positions as
$\lod_f$ and $\lod_a$.  Indeed, except for the negative values being
changed to 0's, they will have the same shape as $\lod_f$ and
$\lod_a$.  

In the following code, we plot $\lod_f$ in the lower triangle and
$\lod_i$ in the upper triangle, for
chromosomes 1, 4, 6, and 15.  The result appears in Figure~\ref{scantwofull}.
<<plotscantwoA,eval=FALSE>>=
plot(out2, chr=c(1,4,6,15))
@ 

\begin{figure}
\centering
<<plotscantwoAplot,fig=TRUE,echo=FALSE,height=5>>=
plot(out2, chr=c(1,4,6,15),layout=list(cbind(1,2),c(5,1)),
     mar1=c(4,4,0,0)+0.1, mar2=c(4,2,0,2)+0.1)
@ 
\caption{LOD scores for selected chromosomes for a two-dimensional
  scan with the \code{hyper} data.  Epistasis LOD scores are in the
  upper triangle and $\lod_f$ is in the lower triangle.\label{scantwofull}}
\end{figure}

The same plot, but with the $fv1$-type LOD scores in the upper
triangle, would be obtained as follows.  The result appears in
Figure~\ref{scantwocondint}. 
<<plotscantwoB,eval=FALSE>>=
plot(out2, chr=c(1,4,6,15), upper="cond-int")
@ 

\begin{figure}
\centering
<<plotscantwoBplot,fig=TRUE,echo=FALSE,height=5>>=
plot(out2, chr=c(1,4,6,15), upper="cond-int", 
     layout=list(cbind(1,2),c(5,1)),
     mar1=c(4,4,0,0)+0.1, mar2=c(4,2,0,2)+0.1)
@
\caption{LOD scores for selected chromosomes for a two-dimensional
  scan with the \code{hyper} data.  $\lod_{fv1}$ is in the upper
  triangle and $\lod_f$ is in the lower
  triangle.\label{scantwocondint}}  
\end{figure}


\end{document}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Karl W. Broman
% The new multiple-qtl mapping functions
%
% This is an "Sweave" document; see the corresponding PDF.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\documentclass[12pt]{article}

\usepackage{times}
\usepackage{amsmath}
\usepackage{url}
\usepackage{color}

% revise margins
\setlength{\headheight}{0.0in}
\setlength{\topmargin}{-0.25in}
\setlength{\headsep}{0.0in}
\setlength{\textheight}{9.00in}
\setlength{\footskip}{0.5in}
\setlength{\oddsidemargin}{0in}
\setlength{\evensidemargin}{0in}
\setlength{\textwidth}{6.5in}

\setlength{\parskip}{6pt}
\setlength{\parindent}{0pt}

\newcommand{\code}{\texttt}
\newcommand{\lod}{\text{LOD}}

% make "Figure" within figure captions a small font
\renewcommand{\figurename}{\small Figure}

\begin{document}

\SweaveOpts{prefix.string=Figs/fig, eps=TRUE}
\setkeys{Gin}{width=\textwidth} %% <- change width of figures
% Try to get the R code from running into the margin
<<echo=FALSE>>=
options(width=87, digits=3, scipen=4)
set.seed(61777369)
@


% function (to be used later) to round numbers in a nicer way.
<<myround,echo=FALSE,results=hide>>=
library(broman) # includes myround() and holmans triangle stuff
@

% Change S input/output font size
\DefineVerbatimEnvironment{Sinput}{Verbatim}{fontsize=\footnotesize, baselinestretch=0.75, formatcom = {\color[rgb]{0, 0, 0.56}}}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontsize=\footnotesize, baselinestretch=0.75, formatcom = {\color[rgb]{0.56, 0, 0}}}




\begin{center}

\vspace*{1in}

\textbf{\Large Genetic map construction with R/qtl}

\bigskip \bigskip



\bigskip \bigskip

{\large Karl W. Broman}

\bigskip \bigskip

{\color{blue} \tt \bfseries University of Wisconsin-Madison \\
Department of Biostatistics \& Medical Informatics \\
Technical Report \# 214 \\[24pt]
4 November 2010

\color{red} Revised 21 March 2012: \color{black} some function names
have changed \\
(e.g., \verb|plot.geno| is now \verb|plotGeno|)
}

\end{center}

\bigskip \bigskip

\textbf{\sffamily Abstract}: Genetic map construction remains an important
prerequisite for quantitative trait loci analysis in
organisms for which genomic sequence is not available.  Surprisingly
little has been written about the construction of genetic maps,
particularly regarding the many practical issues involved.  We provide
an overview of the process, including a description of the key
facilities in the R/qtl software.  The process is illustrated through
simulated data on an F$_2$ intercross derived from two inbred strains.


\vfill

\noindent \emph{email}: \verb|broman@wisc.edu|



\newpage

\textbf{\sffamily Introduction}

Mouse geneticists no longer have to be concerned with genetic map
construction, as the available genomic sequence provides the physical
locations and order of genetic markers.  Scientists interested in
mapping quantitative trait loci (QTL) in other organisms, however,
often must first spend considerable time identifying polymorphic
markers and then constructing a genetic map specifying the chromosomal
locations of the markers.

Surprisingly little has been written about the construction of genetic
maps.  Certainly, papers describing seminal genetic maps include some
description of the methods used, but students desire a general
overview of the process along with a discussion of key issues that
arise and how to overcome them.  I attempt to provide such an overview
here, along with a description of the key facilities in R/qtl
(Broman et al., \emph{Bioinformatics}
18:889--890, 2003; see \url{https://rqtl.org}) for genetic map construction.  While I
describe the use of R/qtl for this purpose in considerable detail, I
hope that my general comments are useful for, and that the R code
is not an obstacle to, readers interested in the process generally or
in other software.

R/qtl is principally aimed at the analysis of simple crosses
(particularly backcrosses and intercrosses) derived from a pair of
inbred strains.  R/qtl includes the ability to analyze doubled
haploids or a panel of recombinant inbred lines or even a phase-known
four-way cross, but here I will focus on the simple case of an
intercross between two inbred strains.

\bigskip
\textbf{\sffamily Experimental design issues}
\nopagebreak


Before proceeding with our discussion of genetic map construction, let
us first describe some of the design issues that a scientist should
consider before embarking on such a project.

One must first ask: what sort of cross should be performed?  As I
have in mind a QTL mapping application, I would suggest using the
same cross (that is, the same material) for both constructing a
genetic map and for the subsequent QTL mapping.  It is likely the case
that one will need a larger cross for the QTL mapping than for
map construction, and also one probably should use a larger
set of markers for map construction than for QTL mapping.  Thus the
ideal strategy may be to produce a large cross, for QTL mapping, but
genotype only a portion of that cross on the full set of markers.
Once the genetic map is constructed, one may then identify a smaller
set of markers, which cover the genome as evenly as possible, to
genotype on the remainder of the cross.

In spite of the fact that map construction often requires fewer
individuals than QTL mapping, as a general rule one should use more
than the minimal number of individuals for map construction.  Issues
such as genotyping errors and segregation distortion are more easily
overcome with a larger cross population.

Similarly, one should aim for a much larger set of genetic markers
than simple calculations about genome size might indicate to be
sufficient.  In organisms with a mature complement of genomic
resources, one may choose an ideal set of markers that evenly cover
the genome and that are all well behaved and easily genotyped.  In the
\emph{de novo\/} construction of a genetic map, however, markers will
be developed at random, and so some regions will be densely covered
with markers and other regions will be quite thin with markers.
Variation in recombination rate (as well as in marker density and
polymorphism) will exacerbate this issue.  Moreover, one often finds
that some markers are very difficult to genotype, and rather than
spend considerable time optimizing such markers (such as by
redesigning primers or changing PCR conditions), it would be most
expedient if such poorly behaved markers could simply be omitted.  If
one has at hand more markers than are really necessary, it is less of
a concern to be dropping 10\% of them.

In the actual genotyping, one should always include DNA from the
parental lines and F$_1$ individuals as controls, preferably on each
plate.  Ideally, include the \emph{actual\/} parents and F$_1$
individuals; if residual heterozygosity is identified, the genotypes
of these progenitors will be extremely useful for making sense of the
data.  Additional blind duplicates are useful to give some sense of
the overall genotyping error rate.

\bigskip
\textbf{\sffamily Load the data}
\nopagebreak

I will consider, as an illustration, a set of simulated data.  Real
data might be preferred, but by considering simulated data, I can be
sure that they feature all of the various issues that I wish to
illustrate.  Moreover, these features can be made quite striking, so
that there will be little question here of what to do.  In practice,
aberrant features in the data will often be less than clear and may
require additional genotyping, or at least a reconsideration of the
raw genotyping results, before the appropriate action is clear.

The data set is called \code{mapthis} and is included in
R/qtl version 1.19-10 and above.  This is an intercross between two
inbred lines, with 300 individuals genotyped at 100 markers.  There is
just one ``phenotype,'' which contains individual identifiers.  The
100 markers were chosen at random from five chromosomes of length 200,
150, 100, 75 and 50~cM, respectively.  The chromosomal locations of
the markers are to be treated as unknown, though the marker names
do contain this information: marker \code{C2M4} is the fourth marker on
chromosome~2.   (Note that the chromosomes are all autosomes.  I'm
not considering the X chromosome in this tutorial.)

To load the data, in R, one first needs to load the R/qtl package,
using the function \code{library()}.  The data set is then loaded via
\code{data()}.

% load the data
<<loaddata>>=
library(qtl)
data(mapthis)
@

In practice, one would use the function \code{read.cross()} to load a
data set into R.  See the help file (type \code{?read.cross}),
look at the sample data sets at \url{https://rqtl.org/sampledata},
and consider Chapter~2 of Broman and Sen (2009) \emph{A Guide to
QTL Mapping with R/qtl} (\url{https://rqtl.org/book}).

In assembling the sort of comma-delimited file that R/qtl can read,
one should assign all markers to the same chromosome (which can be
named whatever is convenient, ``\code{1}'' or ``\code{un}'' or
whatever).  There is no need to assign map positions to markers.

A data file for the \code{mapthis} data is at
\url{https://rqtl.org/tutorials/mapthis.csv}.  It can be read into
R as follows.  (Note the use of \code{estimate.map=FALSE}.  Markers
will be assigned dummy locations, with no attempt to estimate
inter-marker distances.)

% read the data (illustration)
<<readdata,eval=FALSE>>=
mapthis <- read.cross("csv", "https://rqtl.org/tutorials", "mapthis.csv",
                      estimate.map=FALSE)
@


\bigskip
\textbf{\sffamily Omit individuals and markers with lots of missing data}
\nopagebreak

After importing a new data set, the first thing I look at is a summary
of the cross.

% summary
<<summarycross>>=
summary(mapthis)
@
I look to make sure that the cross type (in this case, an F$_2$
intercross) was determined correctly and that the numbers of
individuals and markers is as expected.  The function
\code{summary.cross()} also performs a variety of basic checks of the
integrity of the data, and so I'd pay attention to any warning or
error messages.

Next, I look at the pattern of missing data, through the function
\code{plotMissing()}.

% plot missing data pattern
<<plotmissing,eval=FALSE>>=
plotMissing(mapthis)
@

\begin{figure}
\centering
<<plotmissingplot,fig=TRUE,echo=FALSE,height=3.5>>=
par(mar=c(4.1,4.1,0.6,1.1))
plotMissing(mapthis, main="")
@
\caption{Pattern of missing genotype data in the \code{mapthis} dataset.  Black
  pixels indicate missing genotypes.\label{fig:plotmissing}}
\end{figure}

The result (in Fig.~\ref{fig:plotmissing}) indicates several
individuals with a great deal of missing data (horizontal lines), as well as several
markers with a great deal of missing data (vertical lines).

The function \code{ntyped()} provides the numbers of genotyped markers
for each individual (or the number of genotyped individuals for each
marker).  Let us plot these.  (And note that there is a related
function, \code{nmissing()}, which provides the number of missing
genotypes for each individual or marker.)

% plot of number of genotypes
<<plotntyped,eval=FALSE>>=
par(mfrow=c(1,2), las=1)
plot(ntyped(mapthis), ylab="No. typed markers", main="No. genotypes by individual")
plot(ntyped(mapthis, "mar"), ylab="No. typed individuals",
     main="No. genotypes by marker")
@

\begin{figure}
\centering
<<plotntypedplot,fig=TRUE,echo=FALSE,height=3>>=
par(mfrow=c(1,2), las=1, cex=0.8)
plot(ntyped(mapthis), ylab="No. typed markers", main="No. genotypes by individual")
plot(ntyped(mapthis, "mar"), ylab="No. typed individuals",
     main="No. genotypes by marker")
@
\caption{Plot of number of genotyped markers for each individual (left
  panel) and number of genotyped individuals for each marker (right
  panel).\label{fig:plotntyped}}
\end{figure}

As seen in Fig.~\ref{fig:plotntyped}, there are six individuals
missing almost all genotypes, and there are four markers that are
missing genotypes at about half of the individuals.  Such appreciable
missing data often indicates a problem (either bad DNAs or
difficult-to-call markers) and can lead to difficulties in the genetic
map construction, and so it is best, at this stage, to omit them,
though one might consider adding them back in later.

To omit the individuals with lots of missing genotype data, we may use
the function \code{subset.cross()}, as follows.

% drop individuals with missing data
<<dropind>>=
mapthis <- subset(mapthis, ind=(ntyped(mapthis)>50))
@

To omit the markers with lots of missing data, we first need to
identify the names of the markers.  We then use \code{drop.markers()}.

% drop markers with missing data
<<dropmarkers>>=
nt.bymar <- ntyped(mapthis, "mar")
todrop <- names(nt.bymar[nt.bymar < 200])
mapthis <- drop.markers(mapthis, todrop)
@

%We now have 294 individuals and 96 markers:
%% nind; totmar
%<<nindtotmar>>=
%nind(mapthis)
%totmar(mapthis)
%@


\bigskip
\textbf{\sffamily Identify duplicate individuals}
\nopagebreak

I find it useful to compare the genotypes between all pairs of
individuals, in order to reveal pairs with unusually similar
genotypes, which may indicate sample duplications or monozygotic
twins.  In either case, we will want to remove one individual from
each pair.  Such duplicates are not common, but they are also not
rare.

We use the function \code{comparegeno} to compare the genotypes for
all pairs of individuals.  The output is a matrix, whose contents are
the proportions of markers at which the pairs have matching
genotypes.

% compare individuals' genotypes
<<comparegeno, eval=FALSE>>=
cg <- comparegeno(mapthis)
hist(cg[lower.tri(cg)], breaks=seq(0, 1, len=101), xlab="No. matching genotypes")
rug(cg[lower.tri(cg)])
@

\begin{figure}
\centering
<<comparegenoplot,fig=TRUE,echo=FALSE,height=3>>=
cg <- comparegeno(mapthis)
par(mar=c(4.1,4.1,0.1,0.6),las=1)
hist(cg[lower.tri(cg)], breaks=seq(0, 1, len=101), xlab="No. matching genotypes",
     main="")
rug(cg[lower.tri(cg)])
@
\caption{Histogram of the proportion of markers for which pairs of
  individuals have matching genotypes.\label{fig:comparegenoplot}}
\end{figure}

As seen in Fig.~\ref{fig:comparegenoplot}, a pair of F$_2$ siblings
typically share genotypes at
\Sexpr{round(mean(cg[lower.tri(cg)])*100,-1)}\% of the markers.  But
there are some pairs with well over 90\% matching genotypes.  We may
identify these pairs as follows.

% identify matching pairs
<<matchingpairs>>=
wh <- which(cg > 0.9, arr=TRUE)
wh <- wh[wh[,1] < wh[,2],]
wh
@

We may inspect the genotype matches for these pairs with the following.

% matching genotypes
<<matchinggenotypes>>=
g <- pull.geno(mapthis)
table(g[144,], g[292,])
table(g[214,], g[216,])
table(g[238,], g[288,])
@

As seen above, in each case, the pairs have matching genotypes at all
but 2 or 3 markers.  Ideally, one would go back to the records to try
to assess the source of these problems (e.g., are the pairs from the
same litters?).  Here, we will simply omit one individual from each
pair.  But first we will omit the genotypes that mismatch, as these are
indicated to be errors in one or the other individual (or both).  The
R code is a bit complicated.

% zero out the mismatches
<<dropmismatches>>=
for(i in 1:nrow(wh)) {
  tozero <- !is.na(g[wh[i,1],]) & !is.na(g[wh[i,2],]) & g[wh[i,1],] != g[wh[i,2],]
  mapthis$geno[[1]]$data[wh[i,1],tozero] <- NA
}
@

Now, we omit one individual from each pair.

% omit individuals
<<omitdup>>=
mapthis <- subset(mapthis, ind=-wh[,2])
@

It's also useful to look for duplicate markers (that is, markers with
identical genotypes).  This is particularly true for the case of very
large sets of markers; multiple markers with identical genotypes will
invariably map to the same location, and so one might as well thin out
the markers so that there are no such duplicates, as the extra markers
simply slow down all of our analyses.  The function
\code{findDupMarkers()} may be used to identify markers with matching
genotypes.  (Note that the function \code{drop.dupmarkers()} is for
dropping markers with matching \emph{names}, and considers the
genotypes only in order to establish consensus genotypes across the
multiple markers with the same name).  Here, though, there are no
markers with matching genotypes.

% look for duplicate markers
<<finddupmar>>=
print(dup <- findDupMarkers(mapthis, exact.only=FALSE))
@


\bigskip
\textbf{\sffamily Look for markers with distorted segregation patterns}
\nopagebreak

We next study the segregation patterns of the markers.  We expect the
genotypes to appear with the frequencies 1:2:1.  Moderate departures
from these frequencies are not unusual and may indicate the presence
of partially lethal alleles.  Gross departures from these frequencies often
indicate problematic markers that should be omitted, at least
initially: monomorphic markers (that is, where the two parental lines
actually have the same allele), or markers that are especially
difficult to call (e.g., AA are often called as AB).  We use the
function \code{geno.table} to inspect the segregation patterns.
It calculates the genotype frequencies and also a P-value for a test
of departure from the 1:2:1 expected ratios.  We will focus on those
markers that show significant distortion at the 5\% level, after a Bonferroni
correction for the multiple tests.

% inspect segregation patterns
<<genotable>>=
gt <- geno.table(mapthis)
gt[gt$P.value < 0.05/totmar(mapthis),]
@
% $

The first of these markers, \code{C4M2}, is not terrible.  The others
appear to be monomorphic with a few errors (\code{C2M9} and
\code{C2M15}) or have one genotype that is quite rare, which likely
indicates difficulties in genotyping.  It would be best to omit the
worst of these markers.

% drop bad markers
<<dropbadmarkers>>=
todrop <- rownames(gt[gt$P.value < 1e-10,])
mapthis <- drop.markers(mapthis, todrop)
@
% $


\bigskip
\textbf{\sffamily Study individuals' genotype frequencies}
\nopagebreak

Just as we expect the markers to segregate 1:2:1, we expect the
individuals to have genotype frequencies that are in approximately
those proportions.  Studying such genotype frequencies may help to
identify individuals with high genotyping error rates or some other
labeling or breeding mistake.

There's no R/qtl function to to pull out the genotype frequencies by
individual, but we can write a bit of R code to do so.  It is a bit
nasty, with calls to \code{apply()}, \code{table()}, \code{factor()},
\code{colSums()} and \code{t()}, but hopefully the reader can figure
this out after a bit of exploration and introspection.

<<genofreqbyind,eval=FALSE>>=
g <- pull.geno(mapthis)
gfreq <- apply(g, 1, function(a) table(factor(a, levels=1:3)))
gfreq <- t(t(gfreq) / colSums(gfreq))
par(mfrow=c(1,3), las=1)
for(i in 1:3)
  plot(gfreq[i,], ylab="Genotype frequency", main=c("AA", "AB", "BB")[i],
       ylim=c(0,1))
@

\begin{figure}
\centering
<<plotgenofreqbyind,fig=TRUE,echo=FALSE,height=3.5>>=
g <- pull.geno(mapthis)
gfreq <- apply(g, 1, function(a) table(factor(a, levels=1:3)))
gfreq <- t(t(gfreq) / colSums(gfreq))
par(mfrow=c(1,3), las=1)
for(i in 1:3)
  plot(gfreq[i,], ylab="Genotype frequency", main=c("AA", "AB", "BB")[i],
       ylim=c(0,1))
@
\caption{Genotype frequencies by individual.\label{fig:plotgenofreqbyind}}
\end{figure}

The results in Fig.~\ref{fig:plotgenofreqbyind} do not indicate any
particular problems, though the small number of short chromosomes result
in considerable variability, including one individual with
no BB genotypes, and the frequencies of
AB genotypes varies from
\Sexpr{round(min(gfreq[2,])*100)}--\Sexpr{round(max(gfreq[2,])*100)}\%.
However, if there were an individual with $\sim$ 100\% AA or BB
genotypes (like one of the parental strains), we would see it here.

A more fancy, and potentially more clear view of these genotype
frequencies is obtained by representing them as points in an
equilateral triangle (see Fig.~\ref{fig:triangleplot}).  For any point
within an equilateral triangle, the sum of the distances to the three
sides is constant, and so one may represent a trinomial distribution as
a point within the triangle.  I won't show the code, but note that the
red X in the center of the figure corresponds to the expected genotype
frequencies (1/4, 1/2, 1/4).  Each black dot corresponds to an
individual's genotype frequencies.  There's one point along the right
edge; this corresponds to the individual with no BB genotypes.  Again,
the small size of the genome results in enormous variation, and so no
one individual stands out as unreasonable.

\begin{figure}
\centering
<<triangleplot,fig=TRUE,echo=FALSE,height=3,results=hide>>=
par(mar=rep(0.1,4), pty="s")
triplot(labels=c("AA","AB","BB"))
tripoints(gfreq, cex=0.8)
tripoints(c(0.25, 0.5, 0.25), col="red", lwd=2, cex=1, pch=4)
@
\caption{Genotype frequencies by individual, represented on an
  equilateral triangle.  The red X indicates the expected frequencies
  of 1:2:1.\label{fig:triangleplot}}
\end{figure}





\bigskip
\textbf{\sffamily Study pairwise marker linkages; look for switched alleles}
\nopagebreak

We are now in a position to begin the genetic map construction.  We
start by assessing the linkage between all pairs of markers.  The
function \code{est.rf()} is used to
estimate the recombination fraction between each pair and to calculate
a LOD score for a test of rf = 1/2.  But first note that, in the presence
of appreciable segregation distortion (which is not the case for these
data), unlinked markers can appear to be linked.  For example,
consider the following 2$\times$2 table of two-locus genotypes in a backcross.

\begin{center}
  \begin{tabular}{cccc} \hline
     & \multicolumn{2}{c}{\textbf{Marker 2}} \\ \cline{2-3}
    \textbf{Marker 1} & AA & AB & overall\\ \hline
    AA    &  243 & 27 & 270 \\
    AB    &   27 &  3 &  30 \\ \hline
  overall &  270 & 30 & 300 \\ \hline
  \end{tabular} \end{center}

In this case, the two markers are segregating independently but show
severe segregation distortion. (They segregate 9:1 rather than  1:1.)
The usual estimate of the recombination fraction is (27+27)/300 =
0.18, and the LOD score for the test of rf = 1/2 is $\sim$28.9.

If segregation distortion is rampant in a dataset (and such things do
happen), the usual tests of pairwise linkage will give distorted
results, and so it is best to instead use a simple chi-square or
likelihood ratio test, to assess the association between markers.  The
function \code{markerlrt()} behaves just like \code{est.rf()}, but
uses a general likelihood ratio test in place of the usual test of
pairwise linkage.

Now back to the data, which are not so badly behaved.

% pairwise linkages
<<pairwiselinkage>>=
mapthis <- est.rf(mapthis)
@
\begin{Schunk}
\begin{Soutput}
Warning message:
In est.rf(mapthis) : Alleles potentially switched at markers
  C3M16 C2M16 C1M2 C3M9 C2M14 C1M24 C1M1 C2M12 C1M36 C3M1 C2M25 C1M22 C5M5 C5M7 C1M17 C5M1
  C3M5 C1M15 C2M24 C2M17 C1M23 C5M6 C1M16 C3M2 C3M10 C3M6 C2M13
\end{Soutput}
\end{Schunk}

Note the warning message, which indicates that there are numerous
markers with likely switched alleles (A $\leftrightarrow$ B).  This is
indicated through pairs of markers that are strongly indicated to be
associated but have estimated recombination fractions $\gg$ 1/2.
The \code{checkAlleles()} function gives more detailed information on
this issue.

% checkAlleles
<<checkAlleles>>=
checkAlleles(mapthis, threshold=5)
@

The final column in the output for a marker, \code{diff.in.max.LOD}, is the
difference between the maximum LOD score for the cases where the
estimated recombination fraction is $>$ 1/2 and the maximum LOD score
for the cases where the estimated recombination fraction is $<$ 1/2.
There are a large number of markers that are tightly associated with other
markers, but with recombination fractions well above 1/2, which
indicates that some markers likely have their alleles switched.

A plot of the LOD scores against the estimated recombination fractions
for all marker pairs will give another good view of this problem.  We
use the function \code{pull.rf()} to pull out the recombination
fractions and LOD scores as matrices.

<<lodvrf,eval=FALSE>>=
rf <- pull.rf(mapthis)
lod <- pull.rf(mapthis, what="lod")
plot(as.numeric(rf), as.numeric(lod), xlab="Recombination fraction", ylab="LOD score")
@

\begin{figure}
\centering
<<lodvrfplot,fig=TRUE,echo=FALSE,height=3>>=
rf <- pull.rf(mapthis)
lod <- pull.rf(mapthis, what="lod")
par(mar=c(4.1,4.1,0.6,0.6), las=1, cex=0.8)
plot(as.numeric(rf), as.numeric(lod), xlab="Recombination fraction", ylab="LOD score")
@
\caption{Plot of LOD scores versus estimated recombination fractions
  for all marker pairs.\label{fig:lodvrf}}
\end{figure}

As seen in Fig.~\ref{fig:lodvrf}, there are many marker pairs with
large LOD scores but estimated recombination fractions $\gg$ 1/2.

One solution to this problem is to form initial linkage groups,
ensuring that markers with rf $>$ 1/2 are placed in different groups.
If all goes well, each chromosome will come out as a pair of linkage
groups: one containing markers with correct alleles and another
containing markers with switched alleles.

We use the function \code{formLinkageGroups()} to infer linkage
groups.  It uses the pairwise linkage results from \code{est.rf()}
(or, pairwise association information from \code{markerlrt()}).
The function has two main arguments: \code{max.rf} and
\code{min.lod}.  Two markers will be placed in the same linkage groups
if they have estimated recombination fraction $\le$ \code{max.rf} and
LOD score $\ge$ \code{min.lod}.  The linkage groups are closed via the
transitive property.  That is, if markers a and b are linked and
markers b and c are linked, then all three are placed in the same
linkage group.  I generally start with \code{min.lod} relatively large
(say 6 or 8 or even 10).  The appropriate value depends on the number
of markers and chromosomes (and individuals).  The aim is to get as
many truly linked markers together, but avoid completely putting
unlinked markers in the same linkage group.  It is usually easier to
combine linkage groups after the fact rather than to have to split
linkage groups apart.

<<forminitialgroups>>=
lg <- formLinkageGroups(mapthis, max.rf=0.35, min.lod=6)
table(lg[,2])
@

The output of \code{formLinkageGroups()} is a matrix with two columns:
the initial linkage group or chromosome for each marker, and then the
assigned linkage group, as inferred from the pairwise linkage
information.  The inferred linkage groups are numbered in decreasing
order of size (so that linkage group 1 has the largest number of
markers).  Here we see that \Sexpr{length(table(lg[,2]))} linkage
groups were inferred, with the last \Sexpr{sum(table(lg[,2])==1)}
groups having just one marker.  One may play with \code{max.rf} and
\code{min.lod} until the result is about as expected.  Here, I was
hoping for about 10 linkage groups (since there are five chromosomes but
each will likely be split in two due to the allele switching problem),
and so the results seem okay.

Since we are happy with the results, we can reorganize the markers
into these inferred linkage groups.  We do so with the same function,
\code{formLinkageGroups()}, via the argument \code{reorgMarkers}.

<<reorganizemarkers>>=
mapthis <- formLinkageGroups(mapthis, max.rf=0.35, min.lod=6, reorgMarkers=TRUE)
@

A plot of the pairwise recombination fractions and LOD scores may
indicate how well this worked.

<<plotrf,eval=FALSE>>=
plotRF(mapthis, alternate.chrid=TRUE)
@

\begin{figure}
\centering
<<plotrfplot,fig=TRUE,echo=FALSE,height=5.5>>=
par(mar=c(4.1,4.1,2.1,2.1), las=1)
plotRF(mapthis, main="", alternate.chrid=TRUE)
@
\caption{Plot of estimated recombination fractions (upper-left
  triangle) and LOD scores (lower-right triangle) for all pairs of
  markers.  Red indicates linked (large LOD score or small
  recombination fraction) and blue indicates not linked (small LOD
  score or large recombination fraction).\label{fig:plotrf}}
\end{figure}

The result, in Fig.~\ref{fig:plotrf}, looks about as expected.  The
markers within linkage group 1 are linked to each other.  The pattern
within the group is a bit random, but that is because we have not yet
ordered the markers in any way.

Note that linkage groups 4 and 5 are
associated with each other, in that they have large LOD scores (the
lower-right rectangle for groups 4 and 5 is largely red), but
their recombination fractions are not small (the upper-left rectangle
for groups 4 and 5 is strongly blue).  I would infer from this that
these markers belong to the same chromosome, but the alleles in one or
the other group are switched.  We can't know which is correct, and
ideally one would have parental genotype data to help fix these
problems, but it is not unreasonable, for now, to assume that the
larger group of markers corresponds to the ones with the correct
alleles.

Similarly, groups 6 and 7 belong together, group 8 belongs to
group 2, and groups 9--11 belong to group 1.

We can look at this more clearly by picking out a marker from one
group and studying the recombination fractions and LOD scores for that
marker against all others.  Let us pick the third marker in linkage
group 4.  We create the objects \code{rf} and \code{lod} again, using
\code{pull.rf()}.  These objects have class \code{"rfmatrix"}, and so \code{plot}
below actually goes to the function \code{plot.rfmatrix()}, which
plots the values for a single marker in a display similar to a set of
LOD curves.

<<plotrfonemarker,eval=FALSE>>=
rf <- pull.rf(mapthis)
lod <- pull.rf(mapthis, what="lod")
mn4 <- markernames(mapthis, chr=4)
par(mfrow=c(2,1))
plot(rf, mn4[3], bandcol="gray70", ylim=c(0,1), alternate.chrid=TRUE)
abline(h=0.5, lty=2)
plot(lod, mn4[3], bandcol="gray70", alternate.chrid=TRUE)
@

\begin{figure}
\centering
<<plotrfonemarkerplot,fig=TRUE,echo=FALSE,height=4>>=
par(mar=c(4.1,4.1,1.1,0.6), las=1)
<<plotrfonemarker>>
@
\caption{Plot of estimated recombination fractions (top panel) and
  LOD scores (bottom panel) for the marker \code{\Sexpr{mn4[3]}} against all
  other markers.\label{fig:plotrfonemarker}}
\end{figure}

As seen in Fig.~\ref{fig:plotrfonemarker}, the marker
\code{\Sexpr{mn4[3]}} is strongly associated with markers in both
linkage groups 4 and 5, but the estimated recombination fractions
between \code{\Sexpr{mn4[3]}} and the markers on linkage group 4 are
all $<$ 1/2, while the estimated recombination fractions
between \code{\Sexpr{mn4[3]}} and the markers on linkage group 5 are
all $>$ 1/2.  We can see the problem even more clearly by inspecting
a few tables of two-locus genotypes, produced by
\code{geno.crosstab()}.

<<genocrosstab>>=
geno.crosstab(mapthis, mn4[3], mn4[1])
mn5 <- markernames(mapthis, chr=5)
geno.crosstab(mapthis, mn4[3], mn5[1])
@

It is clear that, if the genotypes for \code{\Sexpr{mn4[3]}} are
correct, then the A and B alleles for \code{\Sexpr{mn5[1]}} are
switched.

In practice, I will look at enormous numbers of these sorts of
two-locus genotype tables in order to figure out what is going on.
But these data are (by design) quite clean, and so we will simply
trust that the alleles need to be switched for all of the markers on
linkage groups 5 and 7--11.  The function \code{switchAlleles()} is
convenient for performing the allele switching.

<<switchalleles>>=
toswitch <- markernames(mapthis, chr=c(5, 7:11))
mapthis <- switchAlleles(mapthis, toswitch)
@

Now when we revisit the plot of recombination fractions and LOD
scores, we will see a quite different picture.  (Note that we need to
re-run \code{est.rf()} after having run \code{switchAlleles()}.)

<<plotrfagain,eval=FALSE>>=
mapthis <- est.rf(mapthis)
plotRF(mapthis, alternate.chrid=TRUE)
@

\begin{figure}
\centering
<<plotrfagainplot,fig=TRUE,echo=FALSE,height=5.5>>=
mapthis <- est.rf(mapthis)
par(mar=c(4.1,4.1,2.1,2.1), las=1)
plotRF(mapthis, main="", alternate.chrid=TRUE)
@
\caption{Plot of estimated recombination fractions (upper-left
  triangle) and LOD scores (lower-right triangle) for all pairs of
  markers, after having switched alleles for many markers.  Red
  indicates linked (large LOD score or small recombination fraction)
  and blue indicates not linked (small LOD score or large
  recombination fraction).\label{fig:plotrfagain}}
\end{figure}

As seen in Fig.~\ref{fig:plotrfagain}, the LOD scores between marker pairs
are unchanged, but now the recombination fractions are small
(indicated in red) for marker pairs with evidence of association
(large LOD scores, also in red).

It is useful to revisit the plot of LOD scores versus recombination
fractions for all pairs.

<<lodvrfagain,eval=FALSE>>=
rf <- pull.rf(mapthis)
lod <- pull.rf(mapthis, what="lod")
plot(as.numeric(rf), as.numeric(lod), xlab="Recombination fraction", ylab="LOD score")
@

\begin{figure}
\centering
<<lodvrfagainplot,fig=TRUE,echo=FALSE,height=3>>=
rf <- pull.rf(mapthis)
lod <- pull.rf(mapthis, what="lod")
par(mar=c(4.1,4.1,0.6,0.6), las=1, cex=0.8)
plot(as.numeric(rf), as.numeric(lod), xlab="Recombination fraction", ylab="LOD score")
@
\caption{Plot of LOD scores versus estimated recombination fractions
  for all marker pairs, after alleles at many markers have been
  switched.\label{fig:lodvrfagain}}
\end{figure}

Now we see (in Fig.~\ref{fig:lodvrfagain}) no estimated recombination
fractions that are much above 1/2, and certainly no large
recombination fractions with large LOD scores.  In fact, the largest
LOD score for marker pairs with estimated recombination fraction $>$
1/2 is \Sexpr{round(max(lod[!is.na(rf) & rf>0.5]), 2)}.


\bigskip
\textbf{\sffamily Form linkage groups}
\nopagebreak

We now should have the genotype data in good shape and can finally
proceed to the actual map construction.  First, we again attempt to
infer the linkage groups, with the hope that we'll come away with
exactly five.

<<formgroupsagain>>=
lg <- formLinkageGroups(mapthis, max.rf=0.35, min.lod=6)
table(lg[,2])
@

Right; we've got five groups.  Now we reorganize the markers again.

<<reorganizemarkersagain>>=
mapthis <- formLinkageGroups(mapthis, max.rf=0.35, min.lod=6, reorgMarkers=TRUE)
@

It is useful to plot the pairwise recombination fractions and LOD
scores again.

<<plotrfyetagain,eval=FALSE>>=
plotRF(mapthis)
@

\begin{figure}
\centering
<<plotrfyetagainplot,fig=TRUE,echo=FALSE,height=5.5>>=
mapthis <- est.rf(mapthis)
par(mar=c(4.1,4.1,1.6,1.6), las=1)
plotRF(mapthis, main="")
@
\caption{Plot of estimated recombination fractions (upper-left
  triangle) and LOD scores (lower-right triangle) for all pairs of
  markers, after markers have been placed in their final linkage
  groups.  Red indicates linked (large LOD score or small
  recombination fraction) and blue indicates not linked (small LOD
  score or large recombination fraction).\label{fig:plotrfyetagain}}
\end{figure}

As seen in Fig.~\ref{fig:plotrfyetagain}, we have five clear linkage
groups, with markers within a group linked to one another, and markers
in distinct groups showing no evidence of linkage.  The results
couldn't possibly be cleaner (and they wouldn't be, if these were real
data).

\bigskip
\textbf{\sffamily Order markers on chromosome~5}
\nopagebreak

We now turn to the problem of ordering markers within each linkage
group.  (We could go ahead and call them chromosomes at this point.)
I always start with the chromosome with the fewest markers, as there
are fewer possible orders and so the process is quicker.  I like to
see some progress before I move to the larger chromosomes.

The function \code{orderMarkers()} may be used to establish an initial
order.  It picks an arbitrary pair of markers, and then adds an
additional marker (chosen at random), one at a time, in the best
supported position.  With the argument \code{use.ripple=TRUE}, the
function \code{ripple()} is used after the addition of each marker, to
consider all possible orders in a sliding window of markers, to
attempt to improve marker order.  The argument \code{window} defines
the length of the window.  Larger values will explore more possible
orders but will require considerably more computation time.  The value
\code{window=7} is usually about the largest one would ever consider;
use of \code{window=8} will take so long that one would not want to
sit and wait.  Note that the other arguments to \code{orderMarkers}
(e.g., \code{error.prob} and \code{map.function}) are passed to
the function \code{est.map()} for estimation of the genetic map with
the final order that is chosen.  Also note that \code{ripple()}, in
this case, chooses among orders in order to minimize the number of
obligate crossovers.  More on this below.

<<orderchrfive,eval=FALSE>>=
mapthis <- orderMarkers(mapthis, chr=5)
@
<<orderchrfiverun,echo=FALSE>>=
file <- "Rcache/order5.RData"
if(file.exists(file)) {
  load(file)
} else {
<<orderchrfive>>
  save(mapthis, file=file)
}
@

We may use \code{pull.map()} to inspect the result.

<<chrfivemap>>=
pull.map(mapthis, chr=5)
@

Since the marker names were chosen to indicate the true marker order,
we can see that we got the order exactly right.  (Though note that the
second marker, \code{C5M2} was omitted at some point during the course
of our analysis.)  Of course, we wouldn't know this, and so we should
make an attempt to explore alternate orders, in case another order
might be seen to be an improvement.

We use \code{ripple()} to explore alternate orders.  As mentioned
above, it considers all possible orders of markers in a sliding window
of fixed length, with the argument \code{window} defining the width of
the window.  If \code{window} is equal to the number of markers on the
chromosome, than all possible orders are considered.

The quickest approach is to count the number of obligate crossovers
for each order.  The good orders generally are those that result in
the smallest numbers of crossovers.  The more refined, but
considerably slower approach, is to compare the likelihoods of the
different orders.  (The likelihood for a given marker order is the
probability of the data assuming that order is correct and plugging in
estimates of the inter-marker distances.)  We do this with
\code{method="likelihood"} in \code{ripple()}.  We may also indicate a
genotyping error probability (through \code{error.prob}).

While we could start by comparing orders to minimize the number of
obligate crossovers (with \code{method="countxo"}, the default, in
\code{ripple()}), this was already done when we called
\code{orderMarkers()}, and it is not necessary to run it again.
Nevertheless, the results will indicate how close, in terms of number
of crossovers, the next-best marker order is to the inferred one.

<<ripplechr5run,echo=FALSE,results=hide>>=
file <- "Rcache/rip5.RData"
if(file.exists(file)) {
  load(file)
} else {
rip5 <- ripple(mapthis, chr=5, window=7)
save(rip5, file=file)
}
@
<<ripplechr5,eval=FALSE>>=
rip5 <- ripple(mapthis, chr=5, window=7)
@
\begin{Schunk}
\begin{Soutput}
   13680 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple5>>=
summary(rip5)
@

As seen above, 13,680 marker orders were considered.  (There are
$9!/2$ = 181,440 total marker orders.)  On my computer, the code above
took about 1.5 seconds.  If we'd looked at all possible orders (that
is, with \code{window=9}), it would take about 14 seconds, but the
results---I checked---are the same.)  Switching markers 8 and 9
results in one additional obligate crossover.  If, instead, one switches
markers 5 and 6, there are two additional obligate crossovers.

It is good to also study the likelihood of different orders, though we
will want to greatly reduce the \code{window} argument, so that it can
be accomplished in a reasonable amount of time.

<<ripplechr5likrun,echo=FALSE,results=hide>>=
file <- "Rcache/rip5lik.RData"
if(file.exists(file)) {
  load(file)
} else {
rip5lik <- ripple(mapthis, chr=5, window=4, method="likelihood",
                  error.prob=0.005)
save(rip5lik, file=file)
}
@
<<ripplechr5lik,eval=FALSE>>=
rip5lik <- ripple(mapthis, chr=5, window=4, method="likelihood",
                  error.prob=0.005)
@
\begin{Schunk}
\begin{Soutput}
   114 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple5lik>>=
summary(rip5lik)
@

The result (on my computer) took about two minutes.  We see that
switching markers 8 and 9 has a LOD score (that is, log$_{10}$
likelihood, relative to the initial order) of
\Sexpr{round(rip5lik[2,"LOD"], 1)}, which indicates that it is slightly
preferred.  However, the estimated chromosome length is slightly
longer (\Sexpr{round(rip5lik[2,"chrlen"], 1)} versus
\Sexpr{round(rip5lik[1,"chrlen"], 1)}~cM).

We know, from the marker names, that the initial order was the true
order, but in practice we would not have such information, and we
would probably want to switch markers 8 and 9.  Usually we are looking
to have the estimated chromosome length be as short as possible, but
if we trust the likelihood calculation, we should go with the
alternate order.  Of course, the two orders are not really
distinguishable; we can't really say, on the basis of these data,
whether the correct order is the first or the second.

Note that the results here can be sensitive to the assumed genotyping
error rate.  (I chose \code{error.prob=0.005} above, because the
data were simulated with this error rate.)  The function
\code{compareorder()} can be used to compare an initial order to a
fixed alternative order.  We can use this to quickly inspect how
sensitive the results are to the assumed error rate.

<<compareorder>>=
compareorder(mapthis, chr=5, c(1:7,9,8), error.prob=0.01)
compareorder(mapthis, chr=5, c(1:7,9,8), error.prob=0.001)
compareorder(mapthis, chr=5, c(1:7,9,8), error.prob=0)
@

These results indicate that with smaller assumed genotyping error rates, the evidence in
favor of switching markers 8 and 9 increases somewhat.  Note also that
the map length increases quite a bit.

If we were looking at these data blindly, I would likely go with
switching markers 8 and 9, so let's go ahead and do that here.  We use
\code{switch.order()} to do so.  It takes the same sort of arguments
as \code{est.map()}, as after the marker order is switched,
\code{est.map()} is called to revise the estimated map for the chromosome.

<<switchorder>>=
mapthis <- switch.order(mapthis, chr=5, c(1:7,9,8), error.prob=0.005)
pull.map(mapthis, chr=5)
@

Note that the map is slightly smaller than what we had seen above,
after running \code{orderMarkers()}, as we had used the default
value for \code{error.prob} in that function, and now we are using
\code{error.prob=0.005}.  Also note that markers \code{C5M10} and
\code{C5M9} are quite close together and are separated from the next
marker by \Sexpr{round(diff(pull.map(mapthis, chr=5)[[1]][7:8]),1)}~cM.
This explains why it is difficult to assess the appropriate order for
these two markers.


\bigskip
\textbf{\sffamily Order markers on chromosome~4}
\nopagebreak

We now turn to chromosome~4.  First, we run \code{orderMarkers()} and print out the estimated map.

<<orderchrfour,eval=FALSE>>=
mapthis <- orderMarkers(mapthis, chr=4)
pull.map(mapthis, chr=4)
@
<<orderchrfourrun,echo=FALSE>>=
file <- "Rcache/order4.RData"
if(file.exists(file)) {
  load(file)
} else {
<<orderchrfour>>
  save(mapthis, file=file)
}
pull.map(mapthis, chr=4)
@


The marker names tell us the true order, and so we see that the
inferred order is correct except that markers \code{C4M10} and
\code{C4M9} are switched.  Let us run \code{ripple()}, to see which
orders have similar numbers of obligate crossovers.

<<ripplechr4run,echo=FALSE,results=hide>>=
file <- "Rcache/rip4.RData"
if(file.exists(file)) {
  load(file)
} else {
rip4 <- ripple(mapthis, chr=4, window=7)
save(rip4, file=file)
}
@
<<ripplechr4,eval=FALSE>>=
rip4 <- ripple(mapthis, chr=4, window=7)
@
\begin{Schunk}
\begin{Soutput}
   18000 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple4>>=
summary(rip4)
@

The order with markers 9 and 10 switched gives the same number of
obligate crossovers, and so we can not distinguish between these two
marker orders.  We turn to the likelihood comparison.

<<ripplechr4likrun,echo=FALSE,results=hide>>=
file <- "Rcache/rip4lik.RData"
if(file.exists(file)) {
  load(file)
} else {
rip4lik <- ripple(mapthis, chr=4, window=4, method="likelihood",
                  error.prob=0.005)
save(rip4lik, file=file)
}
@
<<ripplechr4lik,eval=FALSE>>=
rip4lik <- ripple(mapthis, chr=4, window=4, method="likelihood",
                  error.prob=0.005)
@
\begin{Schunk}
\begin{Soutput}
   132 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple4lik>>=
summary(rip4lik)
@

There is reasonably good evidence to switch markers 9 and 10 (which is
nice, as this results in the true marker order).

<<switchmarkers4>>=
mapthis <- switch.order(mapthis, chr=4, c(1:8,10,9), error.prob=0.005)
pull.map(mapthis, chr=4)
@



\bigskip
\textbf{\sffamily Order markers on chromosome~3}
\nopagebreak

Turning to chromosome~3, we again run \code{orderMarkers()}.  These
calculations are starting to take quite a bit of time.  It would all
be faster if we reduced the argument \code{window} to a smaller value
(say 4 rather than 7), but then not as many alternate orders will be
explored and so we may not identify the best order.

<<orderchrthree,eval=FALSE>>=
mapthis <- orderMarkers(mapthis, chr=3)
pull.map(mapthis, chr=3)
@
<<orderchrthreerun,echo=FALSE>>=
file <- "Rcache/order3.RData"
if(file.exists(file)) {
  load(file)
} else {
<<orderchrthree>>
  save(mapthis, file=file)
}
pull.map(mapthis, chr=3)
@

Note, from the marker names, that the marker order is the true order.
The whole chromosome is flipped, but we have no information, from the
genotype data, to orient the chromosome.  Let us again use
\code{ripple()} to study alternate orders.

<<ripplechr3run,echo=FALSE,results=hide>>=
file <- "Rcache/rip3.RData"
if(file.exists(file)) {
  load(file)
} else {
rip3 <- ripple(mapthis, chr=3, window=7)
save(rip3, file=file)
}
@
<<ripplechr3,eval=FALSE>>=
rip3 <- ripple(mapthis, chr=3, window=7)
@
\begin{Schunk}
\begin{Soutput}
   39600 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple3>>=
summary(rip3)
@

The next-best order, with markers 2 and 3 switched, results in an
additional \Sexpr{diff(rip3[1:2,"obligXO"])} obligate crossovers.
We turn to the likelihood comparison.

<<ripplechr3likrun,echo=FALSE,results=hide>>=
file <- "Rcache/rip3lik.RData"
if(file.exists(file)) {
  load(file)
} else {
rip3lik <- ripple(mapthis, chr=3, window=4, method="likelihood",
                  error.prob=0.005)
save(rip3lik, file=file)
}
@
<<ripplechr3lik,eval=FALSE>>=
rip3lik <- ripple(mapthis, chr=3, window=4, method="likelihood",
                  error.prob=0.005)
@
\begin{Schunk}
\begin{Soutput}
   222 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple3lik>>=
summary(rip3lik)
@

The next-best order (with markers 2 and 3 switched) is considerably
worse than our initial order.


\bigskip
\textbf{\sffamily Order markers on chromosome~2}
\nopagebreak

We turn to chromosome~2, beginning with \code{orderMarkers()}.

<<orderchrtwo,eval=FALSE>>=
mapthis <- orderMarkers(mapthis, chr=2)
pull.map(mapthis, chr=2)
@
<<orderchrtworun,echo=FALSE>>=
file <- "Rcache/order2.RData"
if(file.exists(file)) {
  load(file)
} else {
<<orderchrtwo>>
  save(mapthis, file=file)
}
pull.map(mapthis, chr=2)
@

Again, as seen from the marker names, the inferred order is the true
one.  Let us run \code{ripple()} to inspect alternate orders.

<<ripplechr2run,echo=FALSE,results=hide>>=
file <- "Rcache/rip2.RData"
if(file.exists(file)) {
  load(file)
} else {
rip2 <- ripple(mapthis, chr=2, window=7)
save(rip2, file=file)
}
@
<<ripplechr2,eval=FALSE>>=
rip2 <- ripple(mapthis, chr=2, window=7)
@
\begin{Schunk}
\begin{Soutput}
   78480 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple2>>=
summary(rip2)
@

The next-best order, with markers 3 and 4 switched, has
\Sexpr{diff(rip2[1:2,"obligXO"])} additional obligate crossovers. We
turn to the likelihood comparison.

<<ripplechr2likrun,echo=FALSE,results=hide>>=
file <- "Rcache/rip2lik.RData"
if(file.exists(file)) {
  load(file)
} else {
rip2lik <- ripple(mapthis, chr=2, window=4, method="likelihood",
                  error.prob=0.005)
save(rip2lik, file=file)
}
@
<<ripplechr2lik,eval=FALSE>>=
rip2lik <- ripple(mapthis, chr=2, window=4, method="likelihood",
                  error.prob=0.005)
@
\begin{Schunk}
\begin{Soutput}
   384 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple2lik>>=
summary(rip2lik)
@

The next-best order, in terms of likelihood, has markers 21 and 22
switched, and is considerably worse than our initial order.


It is interesting to compare the numbers of obligate crossovers for
different orders to the LOD scores.  We have LOD scores for a much
smaller number of orders (\Sexpr{nrow(rip2lik)} versus
\Sexpr{nrow(rip2) %/% 1000},\Sexpr{nrow(rip2) %% 1000}), since we had used a smaller value for
\code{window}, but we can pull out the obligate crossover counts
for those orders that we evaluated in terms of likelihood.  It is a
bit tricky to line up the orders.

<<comparexo2lik,eval=FALSE>>=
pat2 <- apply(rip2[,1:24], 1, paste, collapse=":")
pat2lik <- apply(rip2lik[,1:24], 1, paste, collapse=":")
rip2 <- rip2[match(pat2lik, pat2),]
plot(rip2[,"obligXO"], rip2lik[,"LOD"], xlab="obligate crossover count",
     ylab="LOD score")
@

\begin{figure}
\centering
<<comparexo2likplot,fig=TRUE,echo=FALSE,height=3>>=
par(las=1, mar=c(4.1,4.1,1.1,0.1), cex=0.8)
<<comparexo2lik>>
@
\caption{Comparison of the number of obligate crossovers to the LOD
  score (relative to our inferred order), for chromosome~2 marker orders explored
  via \code{ripple()}.\label{fig:comparexo2lik}}
\end{figure}

As seen in Fig.~\ref{fig:comparexo2lik}, there is a clear negative relationship
between the crossover counts and the likelihoods, though the relationship is
not perfect, particularly for the orders that are not well supported.



\bigskip
\textbf{\sffamily Order markers on chromosome~1}
\nopagebreak

Finally, we turn to chromosome~1, first running \code{orderMarkers()}.

<<orderchrone,eval=FALSE>>=
mapthis <- orderMarkers(mapthis, chr=1)
pull.map(mapthis, chr=1)
@
<<orderchronerun,echo=FALSE>>=
file <- "Rcache/order1.RData"
if(file.exists(file)) {
  load(file)
} else {
<<orderchrone>>
  save(mapthis, file=file)
}
pull.map(mapthis, chr=1)
@

Again, the inferred order is precisely the true order.  I was quite
surprised to see how well \code{orderMarkers()} performed with these
data.  Of course, the data are quite clean (by design) and comprise quite a
large number of individuals.  In practice, one can't expect
\code{orderMarkers()} to perform so well,
and it is worthwhile to study the estimated map and the
pairwise linkage information closely.  We will do so in a moment, but
first let's complete our analysis of chromosome~1 by studying the results of
\code{ripple}.

<<ripplechr1run,echo=FALSE,results=hide>>=
file <- "Rcache/rip1.RData"
if(file.exists(file)) {
  load(file)
} else {
rip1 <- ripple(mapthis, chr=1, window=7)
save(rip1, file=file)
}
@
<<ripplechr1,eval=FALSE>>=
rip1 <- ripple(mapthis, chr=1, window=7)
@
\begin{Schunk}
\begin{Soutput}
   117360 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple1>>=
summary(rip1)
@

Two alternate orders (switching markers 28 and 29, or switching
markers 31 and 32) have 2 additional obligate crossovers, compared
to our initial order.  We turn to the likelihood comparison.

<<ripplechr1likrun,echo=FALSE,results=hide>>=
file <- "Rcache/rip1lik.RData"
if(file.exists(file)) {
  load(file)
} else {
rip1lik <- ripple(mapthis, chr=1, window=4, method="likelihood",
                  error.prob=0.005)
save(rip1lik, file=file)
}
@
<<ripplechr1lik,eval=FALSE>>=
rip1lik <- ripple(mapthis, chr=1, window=4, method="likelihood",
                  error.prob=0.005)
@
\begin{Schunk}
\begin{Soutput}
   546 total orders
\end{Soutput}
\end{Schunk}
<<summaryripple1lik>>=
summary(rip1lik)
@

The next best order (switching markers 28 and 29) results in a
moderately large decrease in likelihood, and so we stick with the
initial order produced by \code{orderMarkers()}.

Let us return to the question of how to assess the quality of the
results of \code{orderMarkers()}, aside from our investigations with
\code{ripple()}.  The \code{ripple()} analysis is excellent for
comparing nearby marker orders.  And for chromosomes with a modest
number of markers (like chromosome~5, here), one can consider all
possible orders exhaustively.  But with many markers (as with
chromosome~1), we can investigate only a small proportion of the possible
orders.  For example, with 33 markers (as on chromosome~1), there are
$33!/2 \approx 10^{36}$ possible marker orders, and we investigated
only 117,360 of them using \code{ripple} with \code{window=7}.

There are several things to look it, in assessing whether gross
changes in marker order are necessary.  (Ideally, R/qtl would include
a function providing a more complete investigation of marker order,
perhaps via simulated annealing or another randomized optimization
algorithm, and we do hope to implement such a feature in the future.)
First, we should look at the actual map: are there large gaps between
markers, indicating adjacent markers that are only weakly linked?
Studying the map locations with \code{pull.map()}, as above, is hard
when there are lots of markers.  The function \code{summaryMap()}
provides a summary of the average inter-marker distance and the
largest gap on each chromosome.

<<summarymap>>=
summaryMap(mapthis)
@
<<savesummarymap,echo=FALSE,results=hide>>=
firstsummary <- summaryMap(mapthis)
@

We see that chromosome~1 has a
\Sexpr{round(summaryMap(mapthis)[1,"max.spacing"],1)}~cM gap; the
other chromosomes have gaps no larger than 25~cM.  The large gap on
chromosome~1 is suspicious, but it is not terrible.

In addition to this summary, it is good to also plot the map.
We use the argument \code{show.marker.names=TRUE} so that marker names
are included.  Most of them are unreadable, because the markers are
densely spaced, but at least we learn the identity of markers that are
surrounded by large gaps.

<<plotmap,eval=FALSE>>=
plotMap(mapthis, show.marker.names=TRUE)
@

\begin{figure}
\centering
<<plotmapplot,fig=TRUE,echo=FALSE,height=3>>=
par(las=1, mar=c(4.1,4.1,1.1,0.1), cex=0.8)
plotMap(mapthis, main="", show.marker.names=TRUE)
@
\caption{Plot of the estimated genetic map, after the markers on each
  chromosome have been ordered.\label{fig:plotmap}}
\end{figure}

As seen in Fig.~\ref{fig:plotmap}, there are some large gaps on
chromosome~1 and some smaller gaps on the other chromosomes.  They
deserve further investigation (and we will do so in the next section),
but they aren't particularly troubling.  With gross mistakes in marker
order, one will often see much larger gaps, say $>$ 100~cM.

Finally, we inspect the pairwise linkage information again.

<<plotrfonemoretime,eval=FALSE>>=
plotRF(mapthis)
@

\begin{figure}
\centering
<<plotrfonemoretimeplot,fig=TRUE,echo=FALSE,height=5.5>>=
par(mar=c(4.1,4.1,1.6,1.6), las=1)
plotRF(mapthis, main="")
@
\caption{Plot of estimated recombination fractions (upper-left
  triangle) and LOD scores (lower-right triangle) for all pairs of
  markers, after ordering the markers on each chromosome.
  Red indicates linked (large LOD score or small
  recombination fraction) and blue indicates not linked (small LOD
  score or large recombination fraction).\label{fig:plotrfonemoretime}}
\end{figure}


The pairwise linkage information in Fig.~\ref{fig:plotrfonemoretime}
is close to what we want to see: nearby markers show clear association
and no distant markers show any association: red along the diagonal,
dissipating to blue away from the diagonal.

If there were gross problems with marker order, we might see groups of
distantly placed markers that are more highly associated than more closely
placed markers.  Just as an example, suppose we split chromosome~1
into three pieces and moved the latter piece into the middle.  Let's
look at the pairwise linkage information following such a
re-ordering.

<<plotrfafterreorder,eval=FALSE>>=
messedup <- switch.order(mapthis, chr=1, c(1:11,23:33,12:22),
                         error.prob=0.005)
plotRF(messedup, chr=1)
@

\begin{figure}
\centering
<<plotrfafterreorderplot,fig=TRUE,echo=FALSE,height=3>>=
par(mar=c(4.1,4.1,1.6,1.6), las=1, pty="s", cex=0.8)
messedup <- switch.order(mapthis, chr=1, c(1:11,23:33,12:22),
                         error.prob=0.005)
plotRF(messedup, chr=1, main="")
@
\caption{Plot of estimated recombination fractions (upper-left
  triangle) and LOD scores (lower-right triangle) for all pairs of
  markers on chromosome~1, after messing up the order of the markers.
  Red indicates linked (large LOD score or small
  recombination fraction) and blue indicates not linked (small LOD
  score or large recombination fraction).\label{fig:plotrfafterreorder}}
\end{figure}

Fig.~\ref{fig:plotrfafterreorder} (on
page~\pageref{fig:plotrfafterreorder}) displays the sort of pattern one should
expect with gross problems in marker order: the markers in the first and last
segments are more tightly associated to each other than they are to
the markers in the middle segment.

A plot of the genetic map (see Fig.~\ref{fig:plotmapmessedup}, page~\pageref{fig:plotmapmessedup}) shows a
50~cM gap between the first and middle segments and an 80~cM gap
between the middle and last segments.

<<plotmapmessedup,eval=FALSE>>=
plotMap(messedup, show.marker.names=TRUE)
@

\begin{figure}
\centering
<<plotmapmessedupplot,fig=TRUE,echo=FALSE,height=3>>=
par(las=1, mar=c(4.1,4.1,1.1,0.1), cex=0.8)
plotMap(messedup, main="", show.marker.names=TRUE)
@
\caption{Plot of the estimated genetic map, after messing up the order
  of markers on chromosome~1.\label{fig:plotmapmessedup}}
\end{figure}

The map is not as telling as the pairwise linkage information, but
these are the sorts of things to look at, in trying to decide whether
there are gross problems that need to be corrected.

If features such as those in Fig.~\ref{fig:plotrfafterreorder} and
\ref{fig:plotmapmessedup} were seen, one should identify the segments
of markers that need to be moved around, and then use
\code{switch.order()} to reorganize the markers.  One might first use
\code{compareorder()} to compare the current order to the reorganized
one, to see that the new order gave a clear improvement in likelihood.
In addition, one will often need to alternate between \code{ripple()}
and \code{switch.order()} until the final marker order is established.


\bigskip
\textbf{\sffamily Drop one marker at a time}
\nopagebreak

The large gaps in the genetic map on chromosome~1 remain a concern.
While such gaps may indicate problems with the order of markers, they
might also indicate a high genotyping error rate at certain markers.
If an individual marker is more prone to genotyping errors than
others, it would often (in the sort of analyses performed above) be
placed at one end of the chromosome or the other, but in some cases
(particularly if the genotyping error rate is not high and there are a
large number of individuals in the cross) it may be placed at
approximately the correct position but result in reasonably large gaps
surrounding the marker.

One approach for identifying such problematic markers is to drop one
marker at a time and investigate the change in chromosome length and
the change in log likelihood.  This analysis may be accomplished with
the function \code{droponemarker()}.

<<droponemarker,eval=FALSE>>=
dropone <- droponemarker(mapthis, error.prob=0.005)
@
<<droponemarkerrun,echo=FALSE,results=hide>>=
file <- "Rcache/dropone.RData"
if(file.exists(file)) {
  load(file)
} else {
  dropone <- droponemarker(mapthis, error.prob=0.005)
  save(dropone, file=file)
}
@

The results are of the same form as a genome scan by QTL mapping.  (In
particular, they have class \code{"scanone"}, like an object produced
by the \code{scanone()} function.)  We may plot the results as follows.

<<plotdropone,eval=FALSE>>=
par(mfrow=c(2,1))
plot(dropone, lod=1, ylim=c(-100,0))
plot(dropone, lod=2, ylab="Change in chromosome length")
@

\begin{figure}
\centering
<<plotdroponeplot,fig=TRUE,echo=FALSE,height=4>>=
par(mar=c(4.1,4.1,1.6,0.1), mfrow=c(2,1), cex=0.8)
plot(dropone, lod=1, ylim=c(-100,0))
plot(dropone, lod=2, ylab="Change in chr length (cM)")
@
\caption{Results of dropping one marker at a time.  The top panel
  contains LOD scores; positive values would indicate that dropping a
  marker improves the likelihood.  The bottom panel indicates the
  decrease in estimated chromosome length (in cM) following dropping
  a marker.\label{fig:plotdropone}}
\end{figure}

As seen in top panel of Fig.~\ref{fig:plotdropone}, there is no one
marker for which its omission results in an increase in likelihood,
but as seen in the bottom panel, there are a number of markers that
give an appreciable decrease in chromosome length, of 15--25 cM, when
omitted.  Markers at the ends of chromosomes will often result in a
smaller estimated chromosome length, but that is just because such
terminal markers hang off some distance from the rest of the
markers, and so these changes can often be discounted.  Interior markers
that result in a big change, and there appears to be one on each of
chromosomes 1, 2 and 3, might indicate error-prone markers that are
best omitted.

One may identify the marker on each chromosome whose omission results
in the largest decreases in chromosome length as follows.

<<worstmarkers>>=
summary(dropone, lod.column=2)
@

For chromosomes 4 and 5, these are terminal markers.  The markers on
chromosomes 1, 2 and 3 are all interior markers.  One should probably
study the pairwise linkage between these markers and
surrounding markers before proceeding, but we will go ahead and remove
these markers without any further investigations.

<<dropbadmarkers>>=
badmar <- rownames(summary(dropone, lod.column=2))[1:3]
mapthis <- drop.markers(mapthis, badmar)
@

One should re-estimate the genetic map.  We use \code{replace.map()}
to insert it into the cross object.

<<reestimatemap>>=
newmap <- est.map(mapthis, error.prob=0.005)
mapthis <- replace.map(mapthis, newmap)
summaryMap(mapthis)
@
<<savenewsummary,echo=FALSE,results=hide>>=
secondsummary <- summaryMap(mapthis)
@

Removing those three markers resulted in a decrease in the overall map
length from \Sexpr{round(firstsummary["overall","length"])}~cM to
\Sexpr{round(secondsummary["overall","length"])}~cM.


\bigskip
\textbf{\sffamily Look for problem individuals}
\nopagebreak

Now that we have the markers in their appropriate order, it is a good
idea to return to the question of whether there are particular
individuals whose data are problematic.  One should ask: if a
particular individual showed considerable genotyping errors or did not
actually belong to the cross under investigation (e.g., through some
sort of labeling or breeding error), what sort of aberrations might be
seen in the data?  Above, we studied the amount of missing genotype
data for each individual as well as the individuals' genotype
frequencies.  Another feature to investigate is the observed number of
crossovers in each individual.  These may be counted with the function
\code{countXO()}.  (A related function \code{locateXO()} will return
the estimated locations of all crossovers.)

<<countxo,eval=FALSE>>=
plot(countXO(mapthis), ylab="Number of crossovers")
@

\begin{figure}
\centering
<<countxoplot,fig=TRUE,echo=FALSE,height=3>>=
par(mar=c(4.1,4.1,0.6,0.6), cex=0.8)
<<countxo>>
thecounts <- countXO(mapthis)
worst <- rev(sort(thecounts, decreasing=TRUE)[1:2])
@
\caption{Numbers of observed crossovers in each individual.\label{fig:countxo}}
\end{figure}

The crossover counts in Fig.~\ref{fig:countxo} clearly indicate two
problematic individuals, with \Sexpr{worst[1]} and \Sexpr{worst[2]}
crossovers; the other individuals have
\Sexpr{min(thecounts)}--\Sexpr{max(thecounts[thecounts < worst[1]])}
crossovers.  We should remove these individuals.

<<drophighxoind>>=
mapthis <- subset(mapthis, ind=(countXO(mapthis) < 50))
@

Ideally, we would now revisit the entire process again; in particular,
after removing these problematic individuals, is there evidence that
marker order needs to be changed?  Let us at least look at chromosome 5.

<<rip5again>>=
summary(rip <- ripple(mapthis, chr=5, window=7))
summary(rip <- ripple(mapthis, chr=5, window=2, method="likelihood",
                      error.prob=0.005))
@

There is good evidence for switching markers 8 and 9, which actually
brings us back to the true order of the markers.

<<switchchr5again>>=
mapthis <- switch.order(mapthis, chr=5, c(1:7,9,8), error.prob=0.005)
pull.map(mapthis, chr=5)
@

I investigated the other four chromosomes similarly, and there was no
evidence for further changes in marker order, so I won't present the
results here.  We should, finally, re-estimate the genetic map.

<<reestmapagain>>=
newmap <- est.map(mapthis, error.prob=0.005)
mapthis <- replace.map(mapthis, newmap)
summaryMap(mapthis)
@
<<savethirdsummary,echo=FALSE,results=hide>>=
thirdsummary <- summaryMap(mapthis)
@

The overall map length has decreased further, from
\Sexpr{round(secondsummary["overall","length"])}~cM to
\Sexpr{round(thirdsummary["overall","length"])}~cM.



\bigskip
\textbf{\sffamily Estimate genotyping error rate}
\nopagebreak

Above, I had generally assumed a genotyping error rate of 5/1000 (in
estimating map distances and in likelihood calculations comparing
different marker orders); but I cheated somewhat in using this value,
as the genotype data were simulated with this error rate.

One can actually estimate the genotyping error rate from the data, as
the function \code{est.map()} not only estimates the inter-marker
distances, but also calculates the log likelihood for each
chromosome.  Thus, if we run \code{est.map()} with different assumed
values for the genotyping error rate (specified with the
\code{error.prob} argument), one can identify the maximum likelihood
estimate of the error rate.

<<studyerrorrate,eval=FALSE>>=
loglik <- err <- c(0.001, 0.0025, 0.005, 0.0075, 0.01, 0.0125, 0.015, 0.0175, 0.02)
for(i in seq(along=err)) {
  cat(i, "of", length(err), "\n")
  tempmap <- est.map(mapthis, error.prob=err[i])
  loglik[i] <- sum(sapply(tempmap, attr, "loglik"))
}
lod <- (loglik - max(loglik))/log(10)
@
<<runstudyerrorrate,echo=FALSE,results=hide>>=
file <- "Rcache/errorrate.RData"
if(file.exists(file)) {
  load(file)
} else {
<<studyerrorrate>>
  save(err, lod, file=file)
}
@

The code is a bit tricky, mostly because the log likelihoods (and
note that they are on the natural log scale) are included as
``attributes'' to each chromosome component in the output from
\code{est.map()}.  We use \code{sapply()} to pull those out, and then
we add them up.  We finally convert them to the log$_{10}$ scale and
re-center them so that the maximum is 0.

We may plot the log$_{10}$ likelihood as follows.

<<ploterrorratelik,eval=FALSE>>=
plot(err, lod, xlab="Genotyping error rate", xlim=c(0,0.02),
     ylab=expression(paste(log[10], " likelihood")))
@

\begin{figure}
\centering
<<ploterrorratelikplot,fig=TRUE,echo=FALSE,height=3>>=
par(mar=c(4.1,4.1,0.6,0.6), las=1)
<<ploterrorratelik>>
@
\caption{The log$_{10}$ likelihood for the genotyping error
  rate.\label{fig:errorratelik}}
\end{figure}

The log$_{10}$ likelihood in Fig.~\ref{fig:errorratelik} indicates
that the MLE is approximately 0.005.  Error rates of 0.0025 and 0.0075
have log$_{10}$ likelihoods that are 3 less than that of the MLE.  We
might investigate additional assumed error rates, or even use the R
function \code{optimize()} to refine our estimate, but we won't pursue
that here.

\bigskip
\textbf{\sffamily Look for genotyping errors}
\nopagebreak

While the methods for estimating inter-marker distances allow for the
presence of genotyping errors at a fixed rate, it is nevertheless
worthwhile to look for, and ideally correct, potential genotyping
errors in the data.  Such errors may be identified through apparent
tight double-crossovers, with a single marker being out of phase with
its adjacent markers.

The most convenient approach for identifying such double-crossovers is
to calculate genotyping error LOD scores, first developed by Lincoln
and Lander (\emph{Genomics\/} 14:604--610, 1992).  The LOD score
compares the likelihood for a genotype being in error versus it not
being in error.   R/qtl uses a modified calculation of such
genotyping error LOD scores, with all genotypes
except that being considered assumed to be strictly correct.

The error LOD scores are calculated with \code{calc.errorlod()}.  One
must assume a genotyping error rate, but the results are almost
identical for a wide range of values.

<<errorlod,eval=FALSE>>=
mapthis <- calc.errorlod(mapthis, error.prob=0.005)
@
<<runerrorlod,echo=FALSE,results=hide>>=
file <- "Rcache/errorlod.RData"
if(file.exists(file)) {
  load(file)
} else {
<<errorlod>>
  save(mapthis, file=file)
}
@

The function \code{top.errorlod()} produces a list of the genotypes
with the largest error LOD scores.  One may generally focus on those
with quite large values, say at least 4--5.  Here will we look at just
those genotypes with error LOD $\ge$ 6; this is indicated with the
argument \code{cutoff}.

<<toperrorlod>>=
print(toperr <- top.errorlod(mapthis, cutoff=6))
@

There are \Sexpr{nrow(toperr)} genotypes that meet this criterion.
Let us look at the genotypes that were flagged on chromosome 1, using
the function \code{plotGeno()}.  The argument \code{cutoff} indicates
a threshold for flagging genotypes as potential errors, based on their
error LOD scores.  If we had used the argument \code{include.xo=TRUE}
(which is the default), inferred locations of crossovers would be
displayed; we suppress that here to get a more clean figure.

<<plotgeno,eval=FALSE>>=
plotGeno(mapthis, chr=1, ind=toperr$id[toperr$chr==1],
          cutoff=6, include.xo=FALSE)
@

\begin{figure}
\centering
<<plotgenoplot,fig=TRUE,echo=FALSE,height=3>>=
par(mar=c(4.1,4.1,0.6,0.6), las=1, cex.axis=0.9)
plotGeno(mapthis, chr=1, ind=toperr$id[toperr$chr==1], main="", cex=0.8,
          include.xo=FALSE, cutoff=6)
@
\caption{Genotypes on chromosome 1 for individuals with some potential
  errors flagged by red squares.  White, gray and black circles correspond to AA, AB
  and BB genotypes, respectively.\label{fig:plotgeno}}
\end{figure}

The results are in Fig.~\ref{fig:plotgeno}.  All of the flagged
genotypes are cases with an exchange from one homozygote to the other
and then back again (thus, two crossovers in each interval flanking a
single marker).

One might zero out these suspicious genotypes (that is, make them
missing).  Even better would be to revisit the raw genotyping
information, or even re-genotype these instances.  But we are talking
about just \Sexpr{nrow(toperr)} genotypes out of
\Sexpr{sum(ntyped(mapthis)) %/% 1000},\Sexpr{sum(ntyped(mapthis)) %% 1000},
and with our allowance for genotyping
errors in the map estimation, they have little influence on the
results.

If we did wish to delete these genotypes, we could do so as follows.

<<dropgenotypes>>=
mapthis.clean <- mapthis
for(i in 1:nrow(toperr)) {
  chr <- toperr$chr[i]
  id <- toperr$id[i]
  mar <- toperr$marker[i]
  mapthis.clean$geno[[chr]]$data[mapthis$pheno$id==id, mar] <- NA
}
@
%$



\bigskip
\textbf{\sffamily Revisit segregation distortion}
\nopagebreak

Finally, let us return to an investigation of segregation distortion in these data.

<<segdis,eval=FALSE>>=
gt <- geno.table(mapthis, scanone.output=TRUE)
par(mfrow=c(2,1))
plot(gt, ylab=expression(paste(-log[10], " P-value")))
plot(gt, lod=3:5, ylab="Genotype frequency")
abline(h=c(0.25, 0.5), lty=2, col="gray")
@

\begin{figure}
\centering
<<plotsegdis,fig=TRUE,echo=FALSE,height=5>>=
gt <- geno.table(mapthis, scanone.output=TRUE)
par(mar=c(4.1,4.1,0.6,0.6), las=1, mfrow=c(2,1), cex=0.8)
plot(gt, ylab=expression(paste(-log[10], " P-value")))
plot(gt, lod=3:5, ylab="Genotype frequency")
abline(h=c(0.25, 0.5), lty=2, col="gray")
@
\caption{Evidence for segregation distortion: $-$log$_{10}$ P-values from
  tests of 1:2:1 segregation at each marker (top panel) and the
  genotype frequencies at each marker (bottom panel, with black, blue
  and red denoting AA, AB and BB genotypes, respectively).\label{fig:segdis}}
\end{figure}

The top panel of Fig.~\ref{fig:segdis} contains $-$log$_{10}$ P-values
from tests of 1:2:1 segregation at each marker.  The bottom panel in
Fig.~\ref{fig:segdis} contains the observed genotype frequencies at
each marker (with black, blue and red corresponding to AA, AB and BB
genotypes, respectively).

The greatest departure from 1:2:1
segregation is on chromosome 4, with somewhat more AA genotypes and
somewhat fewer BB genotypes.   If we apply a Bonferroni correction for
the \Sexpr{totmar(mapthis)} tests (\Sexpr{totmar(mapthis)} is the
total number of markers we have retained in the data), we would look
for P $\ge$ 0.05/\Sexpr{totmar(mapthis)} which corresponds to
$-$log$_{10}$ P $\ge$ \Sexpr{round(-log10(0.05/totmar(mapthis)),2)},
and there is one marker on chromosome 4 that exceeds this.

There are also some departures from 1:2:1 segregation on chromosomes 2
and 3, but these appear to be within the range of what would be
expected by chance; the evidence for a real departure from normal
segregation is not strong.

The aberrant segregation pattern on chromosome 4 is not too worrisome.
Multipoint estimates of genetic map distances are little affected by
segregation distortion, and the pattern of distortion on the
chromosome indicates rather smooth changes in genotype frequency.
More worrisome would be a single distorted marker in the midst of other
markers with normal segregation, which would indicate genotyping
errors rather than, for example, the presence of partially lethal
alleles.

And so, finally, we're done.  Let us plot the final map.

<<plotfinalmap,eval=FALSE>>=
plotMap(mapthis, show.marker.names=TRUE)
@

\begin{figure}
\centering
<<plotfinalmapplot,fig=TRUE,echo=FALSE,height=3>>=
par(las=1, mar=c(4.6,4.6,0.6,0.6), cex=0.8)
plotMap(mapthis, main="", show.marker.names=TRUE)
@
\caption{Plot of the final estimated genetic map\label{fig:plotfinalmap}}
\end{figure}

The map in
Fig.~\ref{fig:plotfinalmap} is not pretty, as most of the marker names
are obscured.  R/qtl does not produce production-quality figures
automatically; I always go through quite a few extra contortions within R to
produce a figure suitable for a paper.



\bigskip
\textbf{Discussion}
\nopagebreak

The process of genetic map construction seems to be at least 90\% data
diagnostics.  While many might find that frustrating, for me that is a
large part of what makes it fun: it is interesting detective work.  I
have been involved in the construction of genetic maps for humans,
mice, dogs, zebrafish, mosquitoes, and sea squirts.  Each project was
different, with its own special issues that needed to be overcome, and
such issues can seldom be anticipated in advance.

The general strategy is to think about what sorts of things might be
going wrong with data, and then what sorts of features of the data
(summary statistics or plots) might indicate the presence of such
problems.  There are a variety of things to check routinely, and note
that the particular order in which these checks are performed is often
important.

In the procedures described above, and in forming these simulated
data, I attempted to cover most of the possible problems that might be
expected to arise.  With the simple intercross considered here, and
particularly as the data were simulated so that the problems would
generally be quite clear, the decisions about how to proceed were easy
to make.  In practice, potential problems in data will often be more
murky, and careful judgment calls will need to be made and frequently
revisited, ideally with careful consideration of raw genotyping data
and other records, and perhaps even with some additional rounds of
genotyping or even the redesign of genotyping assays.  Moreover, with
more complex experiments, such as an outcross or the combined analysis
of multiple crosses, additional issues will arise that we have not
touched on here.  But the overall strategy, laid out above, can be
applied quite generally.




\newpage
{\small
\noindent \textbf{\sffamily R/qtl functions useful for genetic map construction} \\
\noindent \begin{tabular}{ll}
\hspace*{25mm} & \hspace*{103mm} \\ \hline
plotMissing & Plot pattern of missing genotypes \\
ntyped & Count number of typed markers for each individual \\
nmissing & Count number of missing genotypes for each individual \\
subset.cross & Pull out a specified set of chromosomes and/or
individuals from a cross \\
drop.markers & Remove a list of markers \\
pull.markers & Drop all but a selected set of markers \\
drop.nullmarkers & Remove markers without data \\
comparegeno & Count proportion of matching genotypes between all pairs
of individuals \\
findDupMarkers & Find markers with identical genotype data \\
drop.dupmarkers & Drop duplicate markers \\
geno.table & Create table of genotype distributions \\
est.rf & Estimate pairwise recombination fractions \\
markerlrt & General likelihood ratio test for association between
marker pairs \\
checkAlleles & Identify markers with potentially switched alleles \\
pull.rf & Pull out the pairwise recombination fractions or LOD scores
as a matrix \\
formLinkageGroups & Partition markers into linkage groups \\
plotRF & Plot recombination fractions \\
markernames & Pull out the marker names from a cross \\
plot.rfmatrix & Plot a slice through the pairwise recombination
fractions or LOD scores \\
geno.crosstab & Create cross-tabulation of genotypes at two markers \\
switchAlleles & Switch alleles at selected markers \\
orderMarkers & Find an initial order for markers within chromosomes \\
ripple & Assess marker order by permuting groups of adjacent
markers \\
summary.ripple & Print summary of ripple output \\
est.map & Estimate genetic map \\
pull.map & Pull out the genetic map from a cross \\
compareorder & Compare two orderings of markers on a chromosome \\
switch.order & Switch the order of markers on a chromosome \\
summaryMap & Print summary of a genetic map \\
plotMap & Plot genetic map(s) \\
droponemarker & Drop one marker at a time from a genetic map \\
replace.map & Replace the genetic map of a cross \\
countXO & Count number of obligate crossovers for each individual \\
locateXO & Estimate locations of crossovers \\
calc.errorlod & Calculate Lincoln \& Lander (1992) error LOD scores \\
top.errorlod & List genotypes with highest error LOD values \\
plotGeno & Plot genotypes on a particular chromosomes for selected
individuals \\ \hline
%
tryallpositions & Test all possible positions for a marker \\
allchrsplits & Test all possible splits of a chromosome into two
pieces \\
movemarker & Move a marker from one chromosome to another \\
convert.map & Change map function for a genetic map \\
shiftmap & Shift starting points in genetic maps \\
rescalemap & Rescale genetic map \\
\hline
\end{tabular}
}


\end{document}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Karl W. Broman
% The new multiple-qtl mapping functions
%
% This is an "Sweave" document; see the corresponding PDF.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\documentclass[12pt]{article}

\usepackage{times}
\usepackage{amsmath}
\usepackage{color}


% revise margins
\setlength{\headheight}{0.0in}
\setlength{\topmargin}{-0.25in}
\setlength{\headsep}{0.0in}
\setlength{\textheight}{9.00in}
\setlength{\footskip}{0.5in}
\setlength{\oddsidemargin}{0in}
\setlength{\evensidemargin}{0in}
\setlength{\textwidth}{6.5in}

\setlength{\parskip}{6pt}
\setlength{\parindent}{0pt}

\newcommand{\code}{\texttt}
\newcommand{\lod}{\text{LOD}}

% make "Figure" within figure captions a small font
\renewcommand{\figurename}{\small Figure}


\begin{document}

\SweaveOpts{prefix.string=Figs/scantwo,eps=TRUE}
\setkeys{Gin}{width=5in} %% <- change width of figures
% Try to get the R code from running into the margin
<<echo=FALSE>>=
options(width=87, digits=3, scipen=4)
@ 

% function (to be used later) to round numbers in a nicer way.
<<myround,echo=FALSE,results=hide>>=
source("myround.R")
@

% Change S input/output font size
\DefineVerbatimEnvironment{Sinput}{Verbatim}{fontsize=\footnotesize, baselinestretch=0.75, formatcom = {\color[rgb]{0, 0, 0.56}}}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontsize=\footnotesize, baselinestretch=0.75, formatcom = {\color[rgb]{0.56, 0, 0}}}

\textbf{\large New functions for exploring multiple-QTL models} \\
Karl W Broman, 7 February 2008 \\
(minor revisions 28 May 2008; further revised
18 July 2008 to discuss \code{stepwiseqtl}; added color 26 Oct 2010)

\bigskip

R/qtl version 1.08 included a number of new functions to simplify the
exploration of multiple-QTL models.  In R/qtl version 1.09, the
function \code{stepwiseqtl} was added to perform a stepwise selection
to identify the multiple-QTL model optimizing a penalized LOD score.

The ``Brief tour of R/qtl'' document contains a brief description of
these functions (see Example 5 in that document); here, we provide a
more thorough explanation.

\bigskip
\textbf{\large Introduction}
\nopagebreak

Let us begin with a brief overview of the changes.
The basic functions remain \code{makeqtl} (for creating a QTL object),
\code{fitqtl} (for fitting a defined multiple-QTL model) and
\code{scanqtl} (for multi-dimensional scans with a multiple-QTL
model). 

Previously, \code{fitqtl} and \code{scanqtl} used multiple imputation
exclusively.  We have now implemented Haley-Knott regression as well.
To use Haley-Knott regression, use the argument \code{what="prob"} in
a call to \code{makeqtl} and then \code{method="hk"} in calls to
\code{fitqtl} and \code{scanqtl} (and the other functions, to be
described shortly).

The \code{scanqtl} function, while completely flexible and so suitable
for most purposes, is rather cumbersome to use.  Our most important
additions are the functions \code{addqtl}, to scan for a single QTL to
be added to a multiple-QTL model, and \code{addpair}, to scan for a
pair of QTL to be added to a multiple-QTL model.  The output of these
functions is of the forms produced by \code{scanone} and
\code{scantwo}, respectively, and so one may use the corresponding
plot and summary functions, making the results easier to deal with.
For most purposes, these functions will be sufficient and direct calls
to \code{scanqtl} will no longer be needed.  Thus, we will not discuss
the use of \code{scanqtl} further here.  

The next important addition is \code{refineqtl}, which uses an
iterative algorithm to refine the locations of QTL in a multiple-QTL
model, with the aim of obtaining the maximum likelihood estimates of
the QTL positions.  If the function is called with 
\code{keeplodprofile=TRUE} (which is the default), one may use another new function,
\code{plotLodProfile}, to plot the LOD profiles for each QTL, in the
context of the multiple-QTL model, as is commonly used in multiple
interval mapping.  

The function \code{addint} may be used to test all possible pairwise
interactions among the QTL in a multiple-QTL model.

We added several functions for manipulating a QTL object
(created by \code{makeqtl}).  
The function \code{addtoqtl} is used to add additional QTL to an
object, \code{dropfromqtl} is used to remove QTL from an object, 
\code{replaceqtl} is used to move a QTL to a new position, and
\code{reorderqtl} is used to change the order the loci within the QTL
object. 

Finally, the function \code{stepwiseqtl} may be used to perform a
stepwise search for the multiple-QTL model optimizing a penalized LOD
score criterion.  The function \code{calc.penalties} can be used to
calculate the penalties for \code{stepwise}, using the output of a
permutation test with \code{scantwo}.

\bigskip
\textbf{\large \code{makeqtl} and \code{fitqtl}}
\nopagebreak

We'll look at the \code{hyper} data as an example.  These data are
from Sugiyama et al. (Genomics 71:70--71, 2001), and concern blood
pressure in 250 male backcross mice.  We'll use
multiple imputation (the default), as Haley-Knott regression performs
poorly in the case of selectively genotyping, which was used for the
\code{hyper} data.

First we need to load the package and the data.

<<loaddata,eval=TRUE>>=
library(qtl)
data(hyper)
@ 

We will use the multiple imputation approach, and so we first run
\code{sim.geno} to perform the imputations.  We'll use 128
imputations; this is insufficient for the current data, which has
extensive missing genotype information, but suffices to illustrate the
methods.  In practice, it is a good idea to repeat the analysis with
independent imputations.  If the results are much changed, increase
the number of imputations.

<<simgeno,eval=FALSE>>=
hyper <- sim.geno(hyper, step=2, n.draws=128, err=0.001)
@ 
<<runsimgeno,echo=FALSE,results=hide>>=
file <- "Rcache/simgeno.RData"
if(file.exists(file)) {
  load(file)
} else {
  set.seed(94743379)
<<simgeno>>
  save(hyper, file=file)
}
@ 

Results of \code{scanone} and \code{scantwo}, which we won't revisit,
indicated QTL on chromosomes 1, 4, 6  and 15, with an interaction
between the QTL on chr 6 and 15, and the possibility of a second QTL
on chr 1.  We will begin by fitting this 4-QTL model.  The function
\code{makeqtl} is used to create a QTL object; it pulls out the
imputated genotypes at the selected locations.

<<makeqtl>>=
qtl <- makeqtl(hyper, chr=c(1, 4, 6, 15), pos=c(67.3, 30, 60, 17.5))
@ 

Note that if you type the name of the QTL object, you get a brief
summary.  

<<print.qtl>>=
qtl
@ 

Also, there is a plot function for displaying the locations of the QTL
on the genetic map.  See Fig.~\ref{fig:plotqtl}.

<<plotqtl,eval=FALSE>>=
plot(qtl)
@ 

\begin{figure}
\centering
<<plotqtlplot,fig=TRUE,echo=FALSE,height=4>>=
par(mar=c(4.1,4.1,4.1,0.1), cex.axis=0.8, cex.main=1, cex=0.7)
<<plotqtl>>
@ 
\caption{\small Locations of the QTL object \code{qtl} on the genetic map for
  the \code{hyper} data.\label{fig:plotqtl}}
\end{figure}

We may now use \code{fitqtl} to fit the model (with QTL in fixed
positions). (In previous versions of R/qtl, \code{fitqtl} required a
column of phenotypes, rather than a cross object, but this has been
revised to make calls to \code{fitqtl} more like those to
\code{scanone}, \code{scantwo}, and \code{scanqtl}.)  Note that we use
\code{Q3*Q4} to indicate that QTL 3 and 4 should interact.  We could
also have written the formula as \verb|y~Q1+Q2+Q3+Q4+Q3:Q4|.  See the
help file for \code{formula} for more information.

<<fitqtl1>>=
out.fq <- fitqtl(hyper, qtl=qtl, formula=y~Q1+Q2+Q3*Q4)
summary(out.fq)
@ 

The initial table indicates the overall fit of the model; the LOD
score of 21.8 is relative to the null model (with no QTL).  In the
second table, each locus is dropped from the model, one at a time, and
a comparison is made between the full model and the model with the
term omitted.  If a QTL is dropped, any interactions it is involved in
are also dropped, and so the loci on chr 6 and 15 are associated with
2 degrees of freedom, as the 6$\times$15 interaction is dropped when
either of these QTL is dropped.  The results indicate strong evidence
for all of these loci as well as the interaction.

\bigskip
\textbf{\large \code{refineqtl} and \code{plotLodProfile}}
\nopagebreak

Let us explore the use of \code{refineqtl}, which allows us to
get improved estimates of the locations of the QTL.  We use
\code{verbose=FALSE} to suppress the display of tracing information.

<<refineqtl,eval=FALSE>>=
rqtl <- refineqtl(hyper, qtl=qtl, formula=y~Q1+Q2+Q3*Q4, verbose=FALSE)
@ 
<<runrefineqtl,echo=FALSE,results=hide>>=
file <- "Rcache/refineqtl.RData"
if(file.exists(file)) {
  load(file)
} else {
<<refineqtl>>
  save(rqtl, file=file)
}
@ 

The output is a modified QTL object, with loci in new positions. 
We can type the name of the new QTL object to see the new locations.

<<printrefineqtl>>=
rqtl
@ 

The loci on chr 1 and 4 changed position very slightly.  Let us use
\code{fitqtl} to assess the improvement in fit; we'll skip the
drop-one analysis.

<<fitqtl2>>=
out.fq2 <- fitqtl(hyper, qtl=rqtl, formula=y~Q1+Q2+Q3*Q4, dropone=FALSE)
summary(out.fq2)
@ 

The LOD score comparing the full model to the null model has increased
by \Sexpr{myround(out.fq2[[1]][1,4] - out.fq[[1]][1,4],1)}, from
\Sexpr{myround(out.fq[[1]][1,4],1)} to \Sexpr{myround(out.fq2[[1]][1,4],1)}.  

If \code{refineqtl} is run with the argument
\code{keeplodprofile=TRUE} (which is the default), the LOD traces at
that last iteration are saved, which can then be plotted with
\code{plotLodProfile}, as follows. See Fig.~\ref{fig:lodprofile}.

<<lodprofile,eval=FALSE>>=
plotLodProfile(rqtl)
@ 


\begin{figure}
\centering
<<plotlodprofile,fig=TRUE,echo=FALSE,height=4>>=
par(mar=c(4.1,4.1,4.1,0.1), cex=0.8)
plotLodProfile(rqtl)
@ 
\caption{\small LOD profiles for a 4-QTL model with the \code{hyper}
  data.\label{fig:lodprofile}}  
\end{figure}

The LOD profiles in Fig.~\ref{fig:lodprofile} are similar to the usual
LOD curves, but instead of comparing a model with a single QTL at a
particular position to the null model, we compare, at each position
for a given QTL, the model with the QTL of interest at that particular
position (and with the positions of all other QTL fixed at their
maximum likelihood estimates) to the model with the QTL of interest
omitted (and, again, with the positions of all other QTL fixed at
their maximum likelihood estimates).  For the loci on chromosomes 6
and 15, the 6$\times$15 interaction is omitted when either of the two
loci is omitted.  Note that maximum LOD for each of the LOD profiles
should be the value observed in the drop-one analysis from
\code{fitqtl}. 

These profile LOD curves are useful for the assessment of both the
evidence for the individual QTL and the precision of localization of
each QTL, but note that they fail to take account of the uncertainty in the
location of the other QTL in the model.  

\bigskip
\textbf{\large \code{addint}}
\nopagebreak

The function \code{addint} is used to test, one at a time, all
possible pairwise interactions between QTL that are not already
included in a model.  For our model with loci on chr 1, 4, 6 and 15,
and with a 6$\times$15 interaction, we will consider each of the 5
other possible pairwise interactions, and will compare the base model
(with the four QTL and just the 6$\times$15 interaction) to the model
with the additional interaction included.

The syntax of the function is similar to that of \code{fitqtl}.  The
output is a table of results similar to that provided by the drop-one
analysis of \code{fitqtl}.

<<addint>>=
addint(hyper, qtl=rqtl, formula=y~Q1+Q2+Q3*Q4)
@ 

None of the interactions is particularly interesting.

Note the difference in the results if we use as the formula
\code{y~Q1+Q2+Q3+Q4} (that is, omitting the 6$\times$15 interaction).

<<addint2>>=
addint(hyper, qtl=rqtl, formula=y~Q1+Q2+Q3+Q4)
@ 

The 6$\times$15 interaction is also tested, and the LOD scores for the
other interactions are somewhat different, as they concern comparisons
between the 4-locus additive model and the model with that one
interaction added.

\bigskip
\textbf{\large \code{addqtl}}
\nopagebreak

We may use the \code{addqtl} function to scan for an additional QTL,
to be added to the model.  By default, the new QTL is strictly
additive.  

<<addqtl,eval=FALSE>>=
out.aq <- addqtl(hyper, qtl=rqtl, formula=y~Q1+Q2+Q3*Q4)
@ 
<<runnaddqtl,echo=FALSE,results=hide>>=
file <- "Rcache/addqtl.RData"
if(file.exists(file)) {
  load(file)
} else {
<<addqtl>>
  save(out.aq, file=file)
}
@ 

The output of \code{addqtl} has the same form as that from
\code{scanone}, and so we may use the same summary and plot
functions.  For example, we can identify the genome-wide maximum LOD
score with \code{max.scanone}.

<<maxaddqtl>>=
max(out.aq)
@ 

And we may plot the results with \code{plot.scanone}; see Fig.~\ref{fig:addqtl}.

<<plotaddqtl,eval=FALSE>>=
plot(out.aq)
@ 

\begin{figure}
\centering
<<plotaddqtlplot,fig=TRUE,echo=FALSE,height=4>>=
par(mar=c(4.1,4.1,4.1,0.1))
<<plotaddqtl>>
@ 
\caption{\small LOD curves for adding one QTL to the 4-QTL model, with
  the \code{hyper} data.\label{fig:addqtl}}  
\end{figure}

We may also use \code{addqtl} to scan for an additional QTL,
interacting with one of the current loci.  This is done by including
the additional QTL in the model formula, with the relevant interaction
term.  For example, let's scan for
an additional QTL interacting with the chr 15 locus.

<<addqtlint,eval=FALSE>>=
out.aqi <- addqtl(hyper, qtl=rqtl, formula=y~Q1+Q2+Q3*Q4+Q4*Q5)
@ 

<<runaddqtlint,echo=FALSE,results=hide>>=
file <- "Rcache/addqtlint.RData"
if(file.exists(file)) {
  load(file)
} else {
<<addqtlint>>
  save(out.aqi, file=file)
}
@ 

We plot the results as follows; see Fig.~\ref{fig:addqtlint}.

<<plotaddqtlint,eval=FALSE>>=
plot(out.aqi)
@ 

\begin{figure}
\centering
<<plotaddqtlintplot,fig=TRUE,echo=FALSE,height=4>>=
par(mar=c(4.1,4.1,4.1,0.1))
<<plotaddqtlint>>
@ 
\caption{\small LOD curves for adding one QTL, interacting with the
  chromosome 15 locus, to the 4-QTL model, with
  the \code{hyper} data.\label{fig:addqtlint}}  
\end{figure}

Also of interest are the LOD scores for the interaction between the
chr 15 locus and the new locus being scanned, which are the
differences between the LOD scores in \code{out.aqi} and \code{out.aq}.
See Fig.~\ref{fig:addqtlint2}.

<<plotaddqtlint2,eval=FALSE>>=
plot(out.aqi - out.aq)
@ 

\begin{figure}
\centering
<<plotaddqtlint2plot,fig=TRUE,echo=FALSE,height=4>>=
par(mar=c(4.1,4.1,4.1,0.1))
<<plotaddqtlint2>>
@ 
\caption{\small Interaction LOD curves in the scan for an additional
  QTL, interacting with the chromosome 15 locus, to the 4-QTL model,
  with the \code{hyper} data.\label{fig:addqtlint2}}
\end{figure}

There is nothing particularly exciting in either of these plots.


\bigskip
\textbf{\large \code{addpair}}
\nopagebreak

The function \code{addpair} is similar to \code{addqtl}, but performs
a two-dimensional scan to seek a pair of QTL to add.  By default,
\code{addpair} performs a two-dimensional scan analogous to that of
\code{scantwo}: for each pair of positions for the two putative QTL,
we fit both an additive model and a model including an interaction
between the two QTL.

Recall that in the single-QTL analysis with the \code{hyper} data,
there were two peaks in the LOD curve on chromosome 1, which indicates
that there may be two QTL on that chromosome.  In the context of our
multiple-QTL model, the LOD profile on chromosome 1 (see
Fig.~\ref{fig:lodprofile}) still shows two peaks, though the distal
one is more prominent.   

We may use \code{addpair} to investigate the
possibility of a second QTL on chromosome 1.  To do so, we omit the
chr 1 locus from our formula, and perform a two-dimensional scan just
on chromosome 1.

<<addpair,eval=FALSE>>=
out.ap <- addpair(hyper, qtl=rqtl, chr=1, formula=y~Q2+Q3*Q4, verbose=FALSE)
@ 
<<runaddpair,echo=FALSE,results=hide>>=
file <- "Rcache/addpair.RData"
if(file.exists(file)) {
  load(file)
} else {
<<addpair>>
  save(out.ap, file=file)
}
@ 

The output is of the same form as that produced by \code{scantwo}, and
so we may use the same summary and plot functions.

<<summaddpair>>=
summary(out.ap)
@ 

There is little evidence for an interaction, and the LOD score comparing
the model with two additive QTL on chr 1 to that with a single QTL on chr 1 
is \Sexpr{myround(summary(out.ap)[[11]],2)}, indicating relatively weak
evidence for a second QTL on chr 1. 

Let us also plot the results.  We'll focus on the evidence for a 
second QTL on the chromosome, displaying $\lod_{fv1}$ (evidence for a 
second QTL, allowing for an interaction) and $\lod_{av1}$ (evidence for
a second QTL, assuming to two are additive).  See Fig~\ref{fig:addpair}.

<<plotaddpair, eval=FALSE>>=
plot(out.ap, lower="cond-int", upper="cond-add")
@ 

\begin{figure}
\centering
<<plotaddpairplot,fig=TRUE,echo=FALSE,height=4.6>>=
plot(out.ap, lower="cond-int", upper="cond-add", 
     layout=list(cbind(1,2),c(5,1)),
     mar1=c(4,4,0,0)+0.1, mar2=c(4,2,0,2)+0.1)
@ 
\caption{\small Results of a two-dimensional, two-QTL scan on
  chromosome 1, in the context of a model with additional QTL on chr
  4, 6 and 15, and a 6$\times$15 interaction, 
  with the \code{hyper} data. $\lod_{fv1}$ is in the lower-right
  triangle, and $\lod_{av1}$ is in the upper-left triangle.  \label{fig:addpair}}
\end{figure}

There is a good deal of flexibility in the way that \code{addpair} may
be used.  As in \code{addqtl}, where one can scan for loci that
interact with a particular locus in the model, we can use
\code{addpair} to scan for an additional pair, with any prespecified
set of interactions.

For example, we may retain the loci on chromosomes 1, 4 and 6, and
scan for an additional pair of interacting loci, one of which also
interacts with chromosome 6.  This would be useful for assessing
evidence for an additional QTL interacting with the chromosome 15
locus, but allowing the location of the locus on chromosome 15 to
vary.  We use the formula \verb|y~Q1+Q2+Q3+Q5*Q6+Q3:Q5|, as we will omit
the chr 15 locus (\code{Q4}), scan for an additional interacting pair
(\code{Q5*Q6}), and allow the first QTL in the additional pair to
interact with the chr 6 locus (\code{Q3}).  Note that the positions of
the chr 1, 4 and 6 loci are assumed known.  A three-dimensional scan
could be performed with the \code{scanqtl} function, but we won't try
that here.

To save time, we will focus just on chromosomes 7 and 15.

<<addpair2, eval=FALSE>>=
out.ap2 <- addpair(hyper, qtl=rqtl, formula=y~Q1+Q2+Q3+Q5*Q6+Q3:Q5, chr=c(7,15),
                   verbose=FALSE)
@ 
<<runaddpair2,echo=FALSE,results=hide>>=
file <- "Rcache/addpair2.RData"
if(file.exists(file)) {
  load(file)
} else {
<<addpair2>>
  save(out.ap2, file=file)
}
@ 

Because we are using a special formula here, with one of the new QTL
interacting with the chr 6 locus, the results are similar to, but not
quite the same as, those from \code{scantwo}.  Rather than fitting an
additive and an interactive model at each pair of positions, we fit
just the single model specified in the formula.  And note that as the
formula is not symmetric in \code{Q5} and \code{Q6}, we must do a full
2-dimensional scan, and not just on the triangle.  (That is, we need
\code{Q5} and \code{Q6} assigned to chromosomes (7,15) as well as
(15,7).) 

The summary of the results are somewhat different here.  For each pair
of chromosomes, a set of three LOD scores are presented.
\code{lod.2v0} compares the full model to the model with neither of
the two new QTL included,  \code{lod.2v1b} compares the full model to
the model with the first of the two new QTL omitted, and
\code{lod.2v1a} compares the full model to the model with the second
QTL omitted.  When a QTL is omitted, any interactions involving that
QTL is also omitted.  

<<summaddpair2>>=
summary(out.ap2)
@ 

Note that, because of the lack of symmetry in the formula we used in
\code{addpair}, separate results are provided for the two cases
\code{c7:c15} (in which the chr 7 locus interacts with the chr 6
locus) and \code{c15:c7} (in which the chr 15 locus interacts with the
chr 6 locus).  The \code{c15:c7} row is most interesting, but
\code{lod.2v1a} is \Sexpr{myround(summary(out.ap2)[3,7],2)}, indicating little
evidence for a chr 7 locus.  (Note that \code{lod.2v1a} here concerns
both the chr 7 locus and the 7$\times$15 interaction.)

With this sort of \code{addpair} output, the \code{thresholds}
argument should have length just 1 or 2 (which is different from the
usual case for \code{summary.scantwo}). Rows will be retained if
\code{lod.2v0} is greater than \code{thresholds[1]} and either 
of \code{lod.2v1a} or \code{lod.2v1b} is greater than
\code{thresholds[2]}.  (If a single thresholds is given, we assume
that \code{thresholds[2]==0}.)  Note that, of the other arguments to
\code{summary.scantwo}, all but \code{allpairs} is ignored.

The plot of the output from \code{addpair}, in the case of a special
formula, is also different from the usual \code{scantwo} plot.  

<<plotaddpair2, eval=FALSE>>=
plot(out.ap2)
@ 

\begin{figure}
\centering
<<plotaddpair2plot,fig=TRUE,echo=FALSE,height=4.6>>=
plot(out.ap2, 
     layout=list(cbind(1,2),c(5,1)),
     mar1=c(4,4,0,0)+0.1, mar2=c(4,2,0,2)+0.1)
@ 
\caption{\small Results of a two-dimensional, two-QTL scan on chr 7
  and 15, in the context of a model with additional QTL on chr 1, 4,
  and 6, with the \code{hyper} data.  The two QTL being scanned were
  allowed to interact, and the first of them interacts with the chr 6
  locus.  The LOD scores displayed are for the 5-QTL model relative to
  the 3-QTL model.  The x-axis corresponds to the first of the new QTL (which interacts with the
  chr 6 locus); the y-axis corresponds to the second of the new
  QTL.\label{fig:addpair2}}
\end{figure}

The plot, shown in Fig.~\ref{fig:addpair2}, contains LOD scores
comparing the full 5-QTL model to the 3-QTL model (having loci on chr
1, 4 and 6).  The x-axis corresponds to the first of the new QTL
(\code{Q5}), which is the one that interacts with the chr 6 locus.
The y-axis corresponds to the second of the new QTL (\code{Q6}).  
Clearly, the QTL interacting with the chr 6 locus wants to be on chr
15.  

Note that the \code{lower} and \code{upper} arguments to
\code{plot.scantwo} are ignored in this case.



\bigskip
\textbf{\large \code{addtoqtl}, \code{dropfromqtl}, and \code{replaceqtl}}
\nopagebreak

Our analysis of the \code{hyper} data, above, did not indicate much
evidence for any further QTL.  If we had seen evidence for additional
loci, we would want to add them to the QTL object and repeat our
explorations with \code{fitqtl}, \code{addint}, \code{addqtl}, and
\code{addpair}.   

The functions \code{addtoqtl}, \code{dropfromqtl} and
\code{replaceqtl} can be used to facilitate such analyses.  Rather
than re-creating the QTL object from scratch with \code{makeqtl}, one
can use \code{addtoqtl} to add an additional locus to a QTL object
that was previously created.  For example, if we were satisfied with the evidence for
an additional QTL on chr 1, it could be added to the QTL object
\code{rqtl} as follows.

<<addtoqtl>>=
rqtl <- addtoqtl(hyper, rqtl, 1, 43.3)
rqtl
@ 

The syntax of \code{addtoqtl} is much like that of \code{makeqtl},
though one also provides the QTL object to which additional QTL are to
be added.

If we want to move the first QTL on chromosome 1 to a different
position (say to 73.3 cM rather than 67.8 cM), we may use
\code{replaceqtl}, as follows.

<<replaceqtl>>=
rqtl <- replaceqtl(hyper, rqtl, 1, 1, 73.3)
rqtl
@ 

If we wish to reorder the QTL (e.g., according to their map positions), 
we may use \code{reorderqtl}, as follows.  The argument
\code{neworder} is to indicate the new order for the QTL.  If missing,
the QTL will be ordered by chromosome and position within a chromosome.

<<reorderqtl>>=
rqtl <- reorderqtl(rqtl, c(5,1:4))
rqtl
@ 

Finally, \code{dropfromqtl} is used to drop a locus from a QTL object.

<<dropfromqtl>>=
rqtl <- dropfromqtl(rqtl, 2)
rqtl
@ 

\bigskip
\textbf{\large \code{stepwiseqtl}}
\nopagebreak

With the function \code{stepwiseqtl}, one may perform a
forward/backward stepwise search algorithm find the multiple-QTL model
optimizing a penalized LOD score criterion.  The penalized LOD score
for a model is the LOD score comparing the model to the null model
(with no QTL), with a penalty subtracted for each main effect and 
separate penalties subtracted for each pairwise interactions among
QTL. 

We consider models with possible pairwise interactions among QTL but
no higher-order interactions allowed.  A hierarchy is enforced in
which the inclusion of an interaction requires the inclusion of each
of the corresponding main effects.  Such a model may be represented by
a graph in which vertices (dots) represent QTL and edges (line
segments between the dots) represent interactions between QTL.  

In the penalized LOD score considered by \code{stepwiseqtl}, we allow
two penalties on interactions, a light penalty and a heavy penalty.
Each disconnected component of a model is allowed one light
interaction penalty; all other interactions are assigned the heavy
penalty.

The three penalties may be calculed from permutation results with
\code{scantwo}, using the function \code{calc.penalties}.  We will use
default penalties derived by computer simulation: (2.69, 2.62, 1.19)
for a mouse backcross, or (3.52, 4.28, 2.69) for a mouse intercross.
(The penalties are in the order (main, heavy interaction, light
interaction).)

First, let us apply \code{stepwiseqtl}, considering only additive QTL
models (with \code{additive.only=TRUE}.  The algorithm performs
forward selection up to a model with a given number of QTL (specified
by the argument \code{max.qtl}; we'll use 6), followed by backward
elimination.

<<stepqtl1, eval=FALSE>>=
stepout1 <- stepwiseqtl(hyper, additive.only=TRUE, max.qtl=6,
                        verbose=FALSE)
@ 
<<runstepqtl1,echo=FALSE,results=hide>>=
file <- "Rcache/stepqtl1.RData"
if(file.exists(file)) {
  load(file)
} else {
<<stepqtl1>>
  save(stepout1, file=file)
}
@ 

The output is a QTL object; type the name to view the chosen model.

<<printstepqtl1>>=
stepout1
@ 

We obtain a model with two QTL, with one on each of chr 1 and 4.

Now let's re-run the analysis, allowing for the possibility of
interactions among the QTL.  If \code{stepwiseqtl} is called with
\code{keeptrace=TRUE}, the sequence of models from the stepwise
selection is retained as an attribute.

<<stepqtl2, eval=FALSE>>=
stepout2 <- stepwiseqtl(hyper, max.qtl=6, keeptrace=TRUE,
                        verbose=FALSE)
@ 
<<runstepqtl2,echo=FALSE,results=hide>>=
file <- "Rcache/stepqtl2.RData"
if(file.exists(file)) {
  load(file)
} else {
<<stepqtl2>>
  save(stepout2, file=file)
}
@ 

The chosen model contains QTL on chr 1, 4, 6 and 15, with the QTL on 6
and 15 interacting.

<<printstepqtl2>>=
stepout2
@ 

Since we called \code{stepwiseqtl} with the argument
\code{keeptrace=TRUE}, the sequence of models visited by
\code{stepwiseqtl} are retained as an \emph{attribute} (called
\code{"trace"}) of the output, \code{stepout2}.  Attributes are a way
of hiding additional information within an object.  The entire set of
attributes for an object may be obtained with the \code{attributes}
function.  It is often useful to just look at the names of the
attributes.

<<attributenames>>=
names(attributes(stepout2))
@ 

Individual attributes may be obtained with the \code{attr} function.
So we can pull out the trace of models with the
following.  This is a long list, with each component being a compact
representation of a QTL model, and so we will print just the first of
them.

<<thetrace>>=
thetrace <- attr(stepout2, "trace")
thetrace[[1]]
@ 

It is nicer to look at a sequence of pictures rather than a long list
of models.  The function \code{plotModel} may be used to plot a
graphical representation of a model, with nodes (i.e., dots)
representing QTL and edges (i.e., line segments connecting two nodes)
representing pairwise interactions among QTL.  The argument
\code{chronly} is used to print just the chromosome ID for each QTL
(rather than chromosome and position).  The penalized LOD score for
each model is saved as an attribute, \code{"pLOD"}; we include them in
the title of each subplot, but this requires another call to
\code{attr}.

<<traceplot,eval=FALSE>>=
par(mfrow=c(4,3))
for(i in seq(along=thetrace))
  plotModel(thetrace[[i]], chronly=TRUE,
            main=paste(i, ": pLOD =", 
              round(attr(thetrace[[i]], "pLOD"), 2)))
@ 

\begin{figure}
\centering
<<plottraceplot,fig=TRUE,echo=FALSE,height=9>>=
par(mar=c(0.6,0.1,2.1,0.6))
<<traceplot>>
@ 
\caption{The sequence of models visited by the forward/backward search
  of \code{stepwiseqtl}, with the \code{hyper}
  data.\label{stepwiseqtltrace}} 
\end{figure}

As seen in Fig.~\ref{stepwiseqtltrace}, our chosen model is
identified immediately (at step 4).  Note that the model at step 3
(with additive QTL on chr 1, 4 and 6) has a lower penalized LOD score
than the model at step 2 (with just the chr 1 and 4 QTL), but then the
inclusion of the chr 15 QTL and the 6 $\times$ 15 interaction gave the
largest penalized LOD score, among all models visited.  With the addition
of a QTL on chr 5 (at step 5), the pLOD decreased somewhat; the LOD
score for the model increased, but not as much as the main effect
penalty.



\end{document}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Karl W. Broman
% The new summary.scanone
%
% This is an "Sweave" document; see the corresponding PDF.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\documentclass[12pt]{article}

%\usepackage{times}
\usepackage{amsmath}
\usepackage{color}
\usepackage{times}

% revise margins
\setlength{\headheight}{0.0in}
\setlength{\topmargin}{-0.25in}
\setlength{\headsep}{0.0in}
\setlength{\textheight}{9.00in}
\setlength{\footskip}{0.5in}
\setlength{\oddsidemargin}{0in}
\setlength{\evensidemargin}{0in}
\setlength{\textwidth}{6.5in}

\setlength{\parskip}{6pt}
\setlength{\parindent}{0pt}

\newcommand{\code}{\texttt}
\newcommand{\lod}{\text{LOD}}



\begin{document}

\SweaveOpts{prefix.string=Figs/scanone}
\setkeys{Gin}{width=\textwidth} %% <- change width of figures
% Try to get the R code from running into the margin
<<echo=FALSE>>=
options(width=77)
@ 


% Change S input/output font size
\DefineVerbatimEnvironment{Sinput}{Verbatim}{fontsize=\footnotesize, baselinestretch=0.75, formatcom = {\color[rgb]{0, 0, 0.56}}}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontsize=\footnotesize, baselinestretch=0.75, formatcom = {\color[rgb]{0.56, 0, 0}}}



\textbf{The new \code{summary.scanone}} \\
Karl W Broman, 27 Oct 2006 \\
(Added color 26 Oct 2010)

\bigskip

In R/qtl version 1.04, the function \code{summary.scanone}
has been changed quite substantially.  Also, the
permutations with \code{scanone} have changed to allow the calculation
of autosome- and X-chromosome-specific LOD thresholds, and to enable
stratified permutation tests.

In this document, I describe the revisions and how to use the new
functions.   We'll first look at the \code{fake.f2} data as an
example.   

First we need to load the package and the data.
<<loaddata>>=
library(qtl)
data(fake.f2)
@ 

<<loadresults,echo=FALSE>>=
load("fakef2_results.RData")
@ 

I'm going to use \code{scanone} with \code{method="hk"}.  First I run
\code{calc.genoprob}, and then \code{scanone} as before.
<<scanoneA,eval=FALSE>>=
fake.f2 <- calc.genoprob(fake.f2, step=2.5)
out.f2 <- scanone(fake.f2, method="hk")
@ 

In \code{summary.scanone}, we can now get an indication of the number
of degrees of freedom associated with the LOD scores.  We use
\code{df=TRUE}, as follows.
<<summaryscanoneA>>=
summary(out.f2, threshold=3, df=TRUE)
@ 

There are a couple of improvements in the permutations performed
by \code{scanone}.  First, we can calculate autosome- and
X-chromosome-specific LOD thresholds; this is important in this case,
as the number of degrees of freedom is different for the X
chromosome.  Separate autosome and X chromosome permutations may be
performed in \code{scanone} via \code{perm.Xsp=TRUE}.  The
X-chromosome-specific thresholds requires many more 
permutation replicates to get a threshold of equivalent precision.
An increased number of permutations is chosen automatically.

Permutations can take a very long time, and so one
might want to use a multi-processor computer or cluster and do
multiple shorter runs in parallel.  And so we have added a function
\code{c.scanoneperm} for combining such runs together.
<<scanonepermA,eval=FALSE>>=
operm1.f2 <- scanone(fake.f2, method="hk", n.perm=500, perm.Xsp=TRUE)
operm2.f2 <- scanone(fake.f2, method="hk", n.perm=500, perm.Xsp=TRUE)
operm.f2 <- c(operm1.f2, operm2.f2)
@ 

Getting the autosome- and X-chromosome-specific thresholds is a bit
tricky, and so another improvement is the addition of the function
\code{summary.scanoneperm} for calculating such thresholds.  The
argument \code{alpha} indicates the significance levels.
<<summaryscanonepermA>>=
summary(operm.f2, alpha=c(0.05, 0.20))
@ 

Further, we may include the permutation results in the call to
\code{summary.scanone} to automatically calculate thresholds and to
have genome-scan-adjusted p-values displayed.
<<summaryscaononeB>>=
summary(out.f2, perms=operm.f2, alpha=0.05, pvalues=TRUE)
@ 

Finally, one may prefer to do a stratified permutation test, permuting
the phenotypes separately within each of the groups defined by sex and
cross direction.  This may be done in \code{scanone} with the argument
\code{perm.strata}, which should be a numeric vector whose unique
values define the separate strata.  

We set of the strata as follows.
<<thestrata>>=
sex <- fake.f2$pheno$sex
pgm <- fake.f2$pheno$pgm
strata <- sex + 2*pgm
table(strata)
@ 

We then perform the permutation test in four pieces, and combine the
results together, as follows.
<<scanonepermB,eval=FALSE>>=
operm1.f2strat <- scanone(fake.f2, method="hk", n.perm=250,
                         perm.Xsp=TRUE, perm.strata=strata)
operm2.f2strat <- scanone(fake.f2, method="hk", n.perm=250,
                         perm.Xsp=TRUE, perm.strata=strata)
operm3.f2strat <- scanone(fake.f2, method="hk", n.perm=250,
                         perm.Xsp=TRUE, perm.strata=strata)
operm4.f2strat <- scanone(fake.f2, method="hk", n.perm=250,
                         perm.Xsp=TRUE, perm.strata=strata)
operm.f2strat <- c(operm1.f2strat, operm2.f2strat, operm3.f2strat,
                   operm4.f2strat)
@ 

The new thresholds are as follows.
<<summaryscanonepermB>>=
summary(operm.f2strat, alpha=c(0.05, 0.20))
@ 

The big changes to the \code{summary.scanone} function concern the
case of results for multiple phenotypes.  To illustrate this, we will
look at the \code{fake.bc} data, which has two phenotypes.  First we
load the data.
<<fakebc>>=
data(fake.bc)
@ 

<<loaddataB,echo=FALSE>>=
load("fakebc_results.RData")
@ 

Now let's run \code{calc.genoprob} and do a genome scan on the two
phenotypes.  Again, we use \code{method="hk"} for the sake of speed.
<<scanoneB,eval=FALSE>>=
fake.bc <- calc.genoprob(fake.bc, step=2.5)
out.bc <- scanone(fake.bc, pheno.col=1:2, method="hk")
@ 

The results contain LOD scores for each of the phenotypes.  By
default, \code{summary.scanone} looks at the first of these, though it
also shows the LOD score for the second phenotype at the locations of
the LOD peaks for the first phenotype.
<<summaryscanoneC>>=
summary(out.bc, threshold=3)
@ 

If we use \code{lodcolumn=2}, we get the analogous results, looking at
the second phenotype.
<<summaryscanoneD>>=
summary(out.bc, threshold=3, lodcolumn=2)
@ 

If we use \code{format="allpheno"}, we get separate rows for the peaks
of each of the phenotypes.  
<<summaryscanoneE>>=
summary(out.bc, threshold=3, format="allpheno")
@ 

Perhaps the most convenient output is obtained with
\code{format="allpeaks"}, which gives a single row for each
chromosome, with the maximum LOD score and its position for each of
the phenotypes.  A chromosome is displayed if the LOD score for at
least one of the phenotypes exceeds its threshold.  The
\code{threshold} argument can be a single threshold, applied to all
phenotypes, or we can give a vector with separate thresholds for each
of the LOD score columns.
<<summaryscanoneF>>=
summary(out.bc, threshold=c(3,2.5), format="allpeaks")
@ 

A permutation test may be performed as before.  Since the
\code{fake.bc} data has only autosomal data, use of
\code{perm.Xsp=TRUE} would be ignored.
<<scanonepermC,eval=FALSE>>=
operm.bc <- scanone(out.bc, pheno.col=1:2, method="hk", n.perm=1000)
@ 

We can again use \code{summary} to get LOD thresholds
<<summaryscanonepermC>>=
summary(operm.bc, alpha=0.05)
@ 

And again these can be used in \code{summary.scanone} to calculate
thresholds and get genome-scan-adjusted p-values.
<<summaryscanoneG>>=
summary(out.bc, perms=operm.bc, alpha=0.05, format="allpeaks", 
        pvalues=TRUE)
@ 


\end{document}

\documentclass[12pt,fullpage]{article}
%\VignetteIndexEntry{Users Guide for New BCsFt Tools for R/qtl}

\usepackage{Sweave}
\usepackage{fullpage}
\usepackage{fancyhdr}
\usepackage{amsmath}
\usepackage{graphicx}
\usepackage{cite}

\bibliographystyle{plos}

\raggedbottom
\lhead{\bf R/qtl bcsft vignette}
\chead{}
\rhead{}
\lfoot{}
\cfoot{}
\rfoot{}

\setlength{\marginparwidth}{0pt}
\setlength{\oddsidemargin}{0pt}
\setlength{\evensidemargin}{0pt}
\setlength{\topmargin}{0pt}
\setlength{\textwidth}{16cm}
\setlength{\textheight}{21cm}
\setlength{\parindent}{0cm}
\addtolength{\parskip}{\baselineskip}

\begin{document}

\title{Users Guide for New $BC_sF_t$ Tools for R/qtl}
\author{Laura M. Shannon \and Brian S. Yandell \and Karl Broman}
\date{29 January 2013}
\maketitle

\section*{Introduction}

Historically QTL mapping studies have employed a variety of crossing schemes including: backcrosses \cite{Tanks}, sib-mating \cite{Sib}, selfing \cite{Collard}, RI lines \cite{fly}, and generations of random mating within mapping populations \cite{Darvasi}.  Different cross designs offer different advantages.  Backcrossing allows for the isolation of limited regions of the donor parent genome in an otherwise recurrent parent background. Selfing and sib-mating in an intercross provide the opportunity to examine all genotype combinations and observe dominance.  RI lines allow for multiple phenotype measures on a single line.  Random mating increases recombination frequency.  In order to use a combination of these cross types and access their various benefits, a more flexible analysis approach is needed.

\begin{figure}
\includegraphics{why_we_need_a_new_program.pdf}
  \caption{An illustration of QTL geneotype inference in populations created through different crossing structures.  All images are of a chromosome section including 2 markers (A and B) and a putative QTL (Q).  Chromosomal segments are pink when they share a genotype with the lower case parent and black when they share a genotype with the capital parent.  Regions where the genotype cannot be observed are dashed.  Regions where the genotype is unknown are gray.}
  \label{Crossovers}
\end{figure}

This guide develops methods to analyze advanced backcrosses and lines created by repeated selfing by extending features of R/qtl \cite{Broman}.  Interval mapping requires estimating the probable genotype of a putative QTL based on the neighboring markers \cite{Lander}.  The probability that a loci between two genotyped markers is of a given genotype depends on the recombination history of the population, which depends on the type of cross.  In Figure \ref{Crossovers} we have two markers, A and B, each with two possible allele genotypes, capital or lower case.  Let us assume that markers A and B are spaced such that double crossovers in a single generation are unlikely.  Let Q be the position of the putative QTL between A and B.  When the observed genotypes at A and B are both homozygous capital in an $F_2$ or $BC_1$ the genotype at Q is most likely homozygous capital (Figure \ref{Crossovers} part B).  However, in an $F_3$, Q might be heterozygous or homozygous for either parent (Figure \ref{Crossovers} part C).  Similarly, in a $BC_2$, Q might be homozygous capital or heterozygous, when the observered genotype is AB/AB. Each generation brings an additional opportunity for crossing over within the interval, increasing the likelihood that Q will not share a genotype with A and B.  This has real consequences when determining genotype probabilities (Figure \ref{Probs}).

\begin{figure}
\includegraphics{genotypeprobabilities.pdf}
  \caption{ The probability that a pair of loci is of a given genotype based on the transition probabilities from the known genotype of marker one (As) to the unknown genotype of the putative QTL (Bs).}
  \label{Probs}
\end{figure}

Genetic map creation is also based on recombination history.  Assuming an $F_2$ or $BC_1$ and sufficiently close markers to make double crossovers in a single generation improbable, individuals which are homozygous for the recurrent parent allele at two adjacent markers exhibit no recombination events between those two markers (figure \ref{Crossovers} part B). However, in an $F_3$ the state of being homozygous for the recurrent parent allele at neighboring markers can be accomplished with 0, 1, or 4 recombination events (figure \ref{Crossovers} part C).  If an $F_3$ is treated as an $F_2$, an individual with 2 adjacent markers homozygous for the same parent will be counted as having undergone 0 recombination events.  However, the actual expected number of recombination events for the described individual is:
\[r^4+\frac{r(1-r)}{8}\approx r/8~,\]

where $r$ is the recombination frequency.

Therefore, treating an $F_3$ as an $F_2$ would artificially shorten the map length (Figure \ref{recomb}).  The number of recombination events between two markers depends on the recombination frequency and cross history, and the number of recombination events in agregate determines the map length.

\begin{figure}
\includegraphics{recombinationcount.pdf}
 \caption{Estimated recombination counts between pairs of markers with observed genotypes.}
  \label{recomb}
\end{figure}

In this guide we present our method for analyzing mapping populations with advanced cross histories while avoiding the pitfalls described above.  Specifically, we address populations resulting from repeated backcrossing ($BC_s$), repeated selfing ($F_t$), and backcrossing followed by selfing ($BC_sF_t$).  The first section is a tutorial on how to use the new tools.  The second section lays out the way we derived the equations for probabilities and recombination counts, which allow for the analysis of advanced cross histories.  The third section contains a technical description of the modifications to the code of the previous release of R/qtl \cite{Broman}.

\section*{Tutorial}

These changes to R/qtl are mostly internal.  The one thing that does change for the user is reading in the data.  Data can be read in using {\em read.cross()} as for all other crosses.  We will use the listeria sample data from R/qtl below.

<<>>=
library(qtl)
listeria.bc2s3<-read.cross(format="csv",
  file=system.file(file.path("sampledata", "listeria.csv"), package = "qtl"),
      BC.gen=2, F.gen=3)
@

Here's another way to convert a cross. Suppose the R/qtl hyper data was really a $BC_3$ (or $BC_3F_0$). You can convert it as follows:
<<>>=
data(hyper)
hyper3 <- convert2bcsft(hyper, BC.gen = 3)
@

We will briefly highlight the difference in results between crosses analyzed using the traditional program and those analyzed using our new tools.  However, we do not discuss the entire process of QTL mapping.  Please refer to the tutorials available through rqtl.org or A Guide to QTL Mapping with R/qtl by Karl Broman \cite{bromanbook} for guidence on complete analysis.

First we compare the maps for the listeria data set (figure \ref{lismap}).

% est.map(listeria.bc2s3) takes time
<<>>=
listeria.f2<-read.cross(format="csv",
   file=system.file(file.path("sampledata", "listeria.csv"), package = "qtl"))
map.bc2s3 <- est.map(listeria.bc2s3)
map.f2<-est.map(listeria.f2)
@

Now, we will compare the maps for the hyper data (figure \ref{hypmap}).

<<>>=
map.bc1 <- est.map(hyper)
map.bc3<-est.map(hyper3)
@

\begin{figure}
<<fig=TRUE,echo=TRUE,height=3>>=
plot(map.f2, map.bc2s3, label=FALSE, main="")
@
  \caption{A comaprison of genetic maps of the listeria data set analyzed as though it were a $F_2$ (left) and as though it were a $BC_2F_3$ (right).}
\label{lismap}
\end{figure}

\begin{figure}
<<fig=TRUE,echo=TRUE,height=3>>=
plot(map.bc1, map.bc3, label=FALSE, main="")
@
  \caption{ A comparison of genetic maps of the hyper data set analyzed as though it were a $BC_1$ (left) and as though it were a $BC_3$(right).}
  \label{hypmap}
\end{figure}

% calc.genoprob takes time.

In both cases the map length is smaller when the cross is analyzed as a $BC_sF_t$ because the same number of recombination events are attributed to multiple generations.  In order to demonstrate that the cross history makes a real difference in outcome of a QTL analysis, we asign the same map to both cross objects regardless of cross history for direct comparisson.  Comparing identical data sets with identical maps using the {\em scanone} command illustraits that position-wise LOD score also depends on cross history (figures \ref{lisscan} and \ref{hypscan}) .

<<>>=
listeria.bc2s3<-replace.map(listeria.bc2s3, map.f2)
listeria.f2<-replace.map(listeria.f2, map.f2)

listeria.f2<-calc.genoprob(listeria.f2, step=1 )
one.f2<-scanone(listeria.f2, method="em",pheno.col=1)

listeria.bc2s3<-calc.genoprob(listeria.bc2s3, step=1 )
one.bc2s3<-scanone(listeria.bc2s3, method="em",pheno.col=1)
@

\begin{figure}
<<fig=TRUE,echo=TRUE,height=3>>=
plot(one.f2, one.bc2s3, col=c("red", "purple"))
@
  \caption{LOD plots for simple interval mapping with the listeria data set.  The red curves are from analysis as though the population were a $F_2$.  The purple curves are from analysis as though the population were a $BC_2F_3$.  Both were analyzed using the same map distances to facilitate comparison}
\label{lisscan}
\end{figure}

<<>>=
hyper3<-replace.map(hyper3, map.bc1)
hyper<-replace.map(hyper, map.bc1)

hyper<-calc.genoprob(hyper, step=1 )
one.hyp<-scanone(hyper, method="em",pheno.col=1)

hyper3<-calc.genoprob(hyper3, step=1 )
one.hyp3<-scanone(hyper3, method="em",pheno.col=1)
@

\begin{figure}
<<fig=TRUE,echo=TRUE,height=3>>=
plot(one.hyp, one.hyp3, col=c("red", "purple"))
@
\caption{LOD plots for simple interval mapping with the hyper data set.  The red curves are from analysis as though the population were a $BC_1$.  The purple curves are from analysis as though the population were a $BC_3$.  Both were analyzed using the same map distances to facilitate comparison}
\label{hypscan}
\end{figure}

\section*{Calculations}

Allowing for the analysis of $BC_SF_T$ crosses in R/qtl required two new sets of calculations: genotype probabilities for different cross histories and recombination counts for these cross histories.  The genotype probabilities were derived based on Jiang and Zeng's \cite{jiang} calculations and the recombination counts are estimated using a golden section search.

\subsection*{Genotype Probabilities}

Jiang and Zeng \cite{jiang} provide a guide for calculating genotype frequencies resulting from several types of crosses of inbred lines.  Although they examine many cases ($F_2$, selfed $F_t$, random mating $F_t$, backcross from selfed $F_t$, and $BC_s$) they do not address all possible cross structures.  Most notably, they do not discuss $BC_sF_t$ crosses. In this section we derive the equations for calculating genotype probabilities for a $BC_sF_t$ cross. The equations we arrived at are heavily based on those of Jiang and Zeng.  However they have been modified both to address $BC_sF_t$ cross histories and to function within  the context of the existing R/qtl program.   We include all the implemented equations, both new and modified, below.

QTL mapping requires estimating the putative QTL genotype based on the observed genotypes of flanking markers.  In all cases there are 2 parental inbred lines.  Line 1 will be indicated by capital letters, while line 2 will be indicated by lower case letters.  A particular descendant of these lines has a known genotype at locus A (indicated with $A$ or $a$), however the genotype at locus B (indicated with $B$ or $b$), the putative QTL, has not been observed.  The genotype at locus B is dependent on the genotype at locus A, the recombination rate between locus A and locus B ($r$), and the cross history.

\subsubsection*{Backcross $BC_S$}

The simplest case is a $BC_1$ with line 1 as the reccurrent parent. Let $q$ be a vector of the frequency of all possible genotypes of loci A and B

\[q = \left[\begin{array}{cccc}freq(\frac{AB}{AB}) & freq(\frac{Ab}{AB}) & freq(\frac{aB}{AB}) & freq(\frac{ab}{AB})\end{array}\right] = \left[\begin{array}{cccc}\frac{w}{2} & \frac{r}{2} & \frac{r}{2} & \frac{w}{2}\end{array}\right]   \]

  where $w=1-r$.

After a subsequent generation of backcrossing the genotype frequencies will change based on the probability that a pair of loci with a particular genotype will produce offspring of each genotype when backcrossed to the recurrent parent.  We will call this the transition probability.  In order to calculate $q$ for a $BC_2$ we will need transition probabilities for all possible genotype combinations. Let $M$ be the matrix of transition probabilities.


\[M=\left[\begin{array}{llll}
    P \left( \frac{AB}{AB}| \frac{AB}{AB}\right) &  P \left(\frac{AB}{AB}|\frac{AB}{Ab}\right) &  P \left( \frac{AB}{AB} |\frac{AB}{aB}\right) &  P \left(  \frac{AB}{AB}|\frac{AB}{ab}\right)\\[5pt]
 P \left(\frac{AB}{Ab} |\frac{AB}{AB} \right) &  P \left((\frac{AB}{Ab} | \frac{AB}{Ab}\right) &  P \left( \frac{AB}{Ab} |\frac{AB}{aB}\right) &  P \left( \frac{AB}{Ab} |\frac{AB}{ab}\right)\\[5pt]
  P \left( \frac{AB}{aB}|\frac{AB}{AB} \right) &  P \left(  \frac{AB}{aB}|\frac{AB}{Ab}\right) &  P \left( \frac{AB}{aB}| \frac{AB}{aB}\right) &  P \left(  \frac{AB}{aB}|\frac{AB}{ab}\right)\\[5pt]
   P \left( \frac{AB}{ab}| \frac{AB}{AB}\right) &  P \left( \frac{AB}{ab}| \frac{AB}{Ab}\right) &  P \left( \frac{AB}{ab}| \frac{AB}{aB}\right) &  P \left( \frac{AB}{ab}| \frac{AB}{ab}\right)\end{array} \right] = \left[\begin{array}{cccc}
   \frac{w}{2} & \frac{r}{2} & \frac{r}{2} & \frac {w}{2}\\[5pt]
   0 & \frac{1}{2} & 0 & \frac{1}{2}\\[5pt]
   0 & 0 & \frac{1}{2} & \frac{1}{2}\\[5pt]
   0 & 0 & 0 & 1 \end{array}\right]
   \]

The frequency vector from the $BC_1$ can then be multiplied by the transition matrix to arrive at a frequency vector for a $BC_2$:

 \[q_{BC_2} = qM~. \]

 With each subsequent generation of backcrossing it is necessary to multiply by the transtion matrix again.  The equation for determining genotype frequencies based on any number of backcross generations ($s$) is:

 \[q_{BC_s}=qM^{s-1}~.\]

This can be further simplified \cite{Bulmer}.  $P(s,0)$ is a set of probabilities for all genotype combinations at two loci in a $BC_s$ population.  It is equivalent to $q_{BC_s}$, but organized differently to make it easier to read.

\[P(s,0) =\begin{array}{ccc}
  & BB & Bb\\[5pt]
  AA & A_{11} & A_{12}\\[5pt]
  Aa & A_{12} & A_{22}
  \end{array}
  \]

  When $s=1$

  \[A_{11}=A_{12}=\frac{w}{2}\]
  \[A_{12} = \frac{r}{2} \]

  For any value of $s$

  \[A_{11}= \frac{2^s-2+w^s}{2^s} \]
  \[A_{12} = \frac{1-w^s}{2^s} \]
  \[A_{22}=  \frac{w^s}{2^s} \]

Note the symmetry on the diagonal of recombinant alleles (Ab/AB and aB/AB) but not on the diagonal with only non-recombinant alleles (AB/AB and ab/AB).  This asymmetry is due to the fact that $ab$ alleles are only introduced in the $F_1$ and therefore all such alleles remaining in the population have never recombined where as $AB$ alleles are introduced every generation. Genotype frequencies can be calculated for all types of crosses using a vector of initial frequencies and a transition matrix.

\subsubsection*{Repeated Selfing $F_t$}

Next, we will discuss the calculations for genotype frequencies from an $F_t$ population resulting from repeated selfing. This crossing structure is also sometimes refered to as an $S_t$, but we are using $F_t$ to be consistant with the notation used by R/qtl. The major difference between the calculations for an $F_t$ and a $BC_s$  is that while in a backcross one allele is always AB, so there are only 4 genotype possibilities, in an $F_t$ there are 10 genotype possibilities.

\[q_{F_1}=\left[\begin{array}{cccccccccc}\frac{AB}{AB} & \frac{AB}{Ab} & \frac{Ab}{Ab} & \frac{AB}{aB} & \frac{AB}{ab} & \frac{Ab}{aB} & \frac{Ab}{ab} & \frac{aB}{aB} & \frac{aB}{ab} & \frac{ab}{ab}\end{array}\right] =\left[\begin{array}{cccccccccc}0&0&0&0&1&0&0&0&0&0\end{array}\right]  \]

The transition matrix for an $F_t$ is the same as Jiang and Zeng \cite{jiang}:

\[N=\left[\begin{array}{cccccccccc}
    1 & 0 & 0 & 0 & 0 & 0 & 0 & 0 & 0 & 0 \\[5pt]
    \frac{1}{4} & \frac{1}{2} & \frac{1}{4} & 0 & 0 & 0 & 0 & 0 & 0 & 0 \\[5pt]
    0 & 0 & 1 & 0 & 0 & 0 & 0 & 0 & 0 & 0 \\[5pt]
    \frac{1}{4} & 0 & 0 & \frac{1}{2} & 0 & 0 & 0 & \frac{1}{4} & 0 & 0 \\[5pt]
    \frac{w^2}{4} & \frac{rw}{2} & \frac{r^2}{4} & \frac{rw}{2}& \frac{w^2}{2} & \frac{r^2}{2} & \frac{rw}{2} & \frac{r^2}{4} & \frac{rw}{2} & \frac{w^2}{4}\\[5pt]
    \frac{r^2}{4} & \frac{rw}{2} & \frac{w^2}{4} & \frac{rw}{2}& \frac{r^2}{2} & \frac{w^2}{2} & \frac{rw}{2} & \frac{w^2}{4} & \frac{rw}{2} & \frac{r^2}{4}\\[5pt]
    0 & 0 & \frac{1}{4} & 0  & 0 & 0 & \frac{1}{2} &  0 & 0 & \frac{1}{4} \\[5pt]
    0 & 0 & 0 & 0 & 0 & 0 & 0 & 1 & 0 & 0 \\[5pt]
    0 & 0 & 0 & 0 & 0 & 0 & 0 &  \frac{1}{4} & \frac{1}{2} & \frac{1}{4} \\[5pt]
    0 & 0 & 0 & 0 & 0 & 0 & 0 & 0 & 0 & 1\end{array} \right]
    \]

Again, these can be multiplied to arrive at the probability of all genotypes in the $F_t$.

 \[q_{F_t}=q_{F_1}N^{t-1}\]

This can be simplified. $P(0,t)$ contains the probabilities for all genotype combinations for two loci in an $F_t$ population (once again it is equivalent to $q_{F_t}$ reorganizes).

\[P(0,t) = \begin{array}{ccccc}
   & BB & Bb & bB & bb\\[5pt]
   AA & B_{11} & B_{12} & B_{12} & B_{14}\\[5pt]
   Aa & B_{12} & B_{22} & B_{23} & B_{12}\\[5pt]
   aA & B_{12} & B_{23} & B_{22} & B_{12}\\[5pt]
   aa & B_{14} & B_{12} & B_{12} & B_{11}
 \end{array} \]

The probabilities $B_{ij}$ of ending up in a particular genotype after $t$ generations can be modeled in terms of generations spent in the double heterozygous stage (at least 1, as $F_1$ is a double heterozygote), the probability of moving from that genotype to either one of the intermediate stages or to a double homozygote, the time spent at an intermediate stage (could be 0), and the probability of moving from an intermediate stage to a double homozygote. There are four transient states (double heterozygotes), 8 intermediate states (single heterozygotes) and 4 absorbing states (double heterozygotes).

The only genotypes which can produce all other genotypes are the transient double heterozygotes ($B_{22}$ and $B_{23}$). Therefore with each generation there is an exponential decay in the probability of remaining in the double heterozygous state.  In order to remain in the double heterozygous state there either has to be no recombination ($w^2$) or a double recombination event ($r^2$) in every generation. In order to model this we reparameterize $w^2$ and $r^2$ as $\beta$ and $\gamma$, specifically $\beta+\gamma=w^2$ while $\beta-\gamma = r^2$.  $\beta$ is also the probability of remaining in a double heterozygous state given that the line started in one of the two double heterozygous states in a single generation.

  \[B_{22}= \frac{\beta^{t-1}+\gamma^{t-1}}{2} \]
  \[B_{23}=  \frac{\beta^{t-1}-\gamma^{t-1}}{2} \]
  \[ \beta=\frac{w^2+r^2}{2} \]
  \[\gamma= \frac{w^2 - r^2}{2}\]

The 8 intermediate states, with one locus homozygous and one heterozygous. During one of the previous generations, one locus was fixed while the other remained heterozygous. There are two exponential decays, with the transition point unknown. After some simplification, this can be expressed as$B_{12}$:

  \[B_{12}=\frac{rw\left(\frac{1}{2^{t-1}}-\beta^{t-1}\right)}{1-2\beta} \]

Finally, the four absorbing states, heterozygous at both loci, can be reached from a number of paths, involving simultaneous or separate fixation of both loci. The calculations are more involved, but simplify ty $B_{11}$ or $B_{14}$:

  \[B_{11}= f(w,r)= \frac{1}{8} \left[ w^2 \left(g \left(\beta, t \right) + g \left( \gamma, t \right) \right) + r^2 \left(g \left( \beta, t \right) - g \left( \gamma, t \right) \right) \right] + \frac{rw}{5} \left[g \left( \beta ,t \right) + g \left(2\beta , t-1 \right) \right]    \]
  \[B_{14}=f(r,w)  \]

 With:

  \[g\left( \beta, t \right) =  (1-\beta^{t-1}) / (1-\beta)~.\]

  Unlike P(s,0), P(0,t) is symmetric on both diagonals because both parental alleles are equally present in the $F_1$ and never introgressed again.

One major difference between working with a backcross and an $F_t$ is that while in a backcross phase is always known, in an $F_t$ phase cannot be observed.  When dealing with phase unknown data the two heterozygote cases can be collapsed as follows:

 \[ \begin{array}{cccc}
      & BB & Bb  & bb\\[5pt]
   AA & B_{11} & 2B_{12} & B_{14}\\[5pt]
   Aa & 2B_{12} & 2\left(B_{22}+ B_{23}\right) & 2B_{12}\\[5pt]
   aa & B_{14} & 2B_{12} & B_{11}
  \end{array} \]

  Since we cannot distinguish between the two heterozygote classes we add them and report the frequency of both.

The final difference between backcross and $F_t$ calculations is that for $F_t$ populations it is possible to have partially informative markers.  Partially informative markers can only be interpreted as not belonging to a particular homozygous class.  For instance if a marker were measured using the presence or absence of a band on a gel, heterozygotes would be indistinguishable from the homozygous present class.  We will refer to partially informative markers as either "not AA" or "not aa".  In order to calculate the probability of partially informative markers we add the probabilities of the genotypes we cannot distinguish between, much like the phase unknown case above.  For example, $not\; AA/BB$ could be $Aa/BB$, $aA/BB$, or $aa/BB$ so we sum all of those probabilties to get $2B_{12}+B_{14}$.  All other genotypes with partially informative markers can be calculated similarly.

 \subsubsection*{Backcrossing followed by selfing $BC_sF_t$}

The described equations for the $BC_s$ and the $F_t$ form the basis for the $BC_sF_t$.  The two types of crosses can be thought of sequentially.  The $BC_s$ that forms the first steps of the $BC_SF_t$ is exactly the same as the $BC_S$ on it's own.  The difference between calculating an $F_t$ which follows several genetations of backcrossing and one which follows an $F_1$ is the vector of starting genotype frequencies. In this case the starting genotype frequencies can be supplied by $q_{BC_S}=qM^{s-1}$. The six genotypes not represented all have starting frequency 0.

\[q_{BC_SF_0} =\left[\begin{array}{cccccccccc}  \frac{2^s-2+w^s}{2^s} & \frac{1-w^s}{2^s} & 0 &  \frac{1-w^s}{2^s} & \frac{w^s}{2^s} & 0 & 0 & 0 & 0 & 0\end{array}\right]   \]


The $q$ resulting from this modification of $q_{BC_S}$ can be multiplied by the $F_t$ transition matrix. Much like in the previous cases this can be simplified.  $P(s,t)$ contains the probabilities of all possible genotype combinations at two loci for a $BC_sF_t$. Below, we explicitly identify the parts of the equations from the backcross (A) and selfing (B) probablities.

\[P(s,t) = \begin{array}{ccccc}
   & BB & Bb & bB & bb\\[5pt]
   AA & C_{11} & C_{12} & C_{12} & C_{14}\\[5pt]
   Aa & C_{12} & C_{22} & C_{23} & C_{24}\\[5pt]
   aA & C_{12} & C_{23} & C_{22} & C_{24}\\[5pt]
   aa & C_{14} & C_{24} & C_{24} & C_{44}
  \end{array}
   \]

   Where:

  \[C_{22}= A_{22}(s)B_{22}(t) \]
  \[C_{23}= A_{22}(s)B_{23}(t) \]
 \[C_{12}= A_{22}(s)B_{12}(t) + A_{12}(s)\left(\frac{1}{2}\right)^t \]
 \[C_{24}=A_{22}(s)B_{12}(t)  \]
  \[C_{11}= A_{22}(s)B_{11}(t)+A_{12}(s)\left(1-\left(\frac{1}{2}\right)^t\right)+A_{11}(s) \]
   \[C_{14}= A_{22}(s)B_{11}(t)+A_{12}(s)\left(1-\left(\frac{1}{2}\right)^t\right)  \]
  \[C_{44}=A_{22}(s)B_{11}(t)  \]

 Because these probabilities depend on the backcross probabilities there is only symmetry on one diagonal when $s>0$. Partially informative markers and phase unknown data can be treated the same way as an $F_t$.


\subsection*{Recombination Counts}

In the previous implementation of R/qtl recombination counts were calculated, however for advanced crossing schemes there is no direct analytic solution.  Instead we implemented a hill climbing algorithm using a golden section search \cite{Kiefer} which determines the most probable recombination frequency, rather than calculating an actual value.  The search space starts between 0 and 0.5 (all possible recombination frequencies).  The golden section search relies on comparing three points (figure \ref{search}).  To start with the points are $r=0$, $r=0.5$, and $r=r_1$, where the value of $r_1$ is determined so that the ratio of a to a+b is equal to the ratio of a to b.  Then a new point ($r=r_2$) is added in the larger interval so that the ratio of d to a is equal to the ratio of c to d.  The set of 3 $r$ values containing the highest maximum likelihood (as compared to the null model of unlinked markers $r=0.5$) are kept, and the remaining value is dropped (in this case $r=0.5$).  The search algorithm starts again with 0, $r_1$, and $r_2$ as the three points.  This process repeats until tolerance for the minimum improvement in likelihood is reached, then the $r$ value with the highest likelihood is reported as the maximum likelihodd estimate used in the map.  This provides an accurate estimate of recombination frequency.

\begin{figure}
\includegraphics{goldensectionsearch.pdf}
\caption{An illustration of the golden section search}
\label{search}
\end{figure}

\subsection*{A Note on Intercrosses and Random Matings}

These equations are accurate for $BC_sF_t$ when $F_t$ refers to any number of selfed generations.  We have not implemented code to address advanced intercross lines resulting from sib mating or random mating within an advanced cross. Below we sketch ideas to develop these algorithms.

In an $F_2$, selfing and sib-mating are interchangeable in terms of calculations because the entire population has an identical $F_1$ genotype. However after the $F_2$, calculations get more complicated for sib-mated populations. Each $F_2$ is sib-mated to create an $F_3$. Sib-mating brings the added complication that we need to think about families instead of individuals. There are 10 possible $F_2$ genotypes leading to 55 possible combinations of cross parents and their next generation families. A transition matrix ($L$) analagous to the one for selfing ($N$) would have to be 55 x 55 and account for the probability that a family that resulted from a cross between a particular set of parents in the $F_{t-2}$ would yield a cross between another set of parents in the $F_{t-1}$. This would be multiplied by a vector of 55 starting probabilities for the $F_1$ ($q_{F_1}^*$), these being probabilities of genotypes for specific crosses rather than for individuals.  Of course, for the $F_1$, there is only one type of cross $AB/ab X AB/ab$ which can be the parents of the $F_2$, and the probability of the other 54 types of crosses is 0.  Multiplying these successively will result in the probabilities of the crosses that produce the $F_t$, since our actual question is the probability of the genotypes of the $F_t$ where one individual is selected from each family, we will need a second matrix, K.  This matrix will give the probability for each of the 10 possible genotypes results based on the 55 possible crosses.  The final result will be the probability of the 10 genotypes ($q_{F_t}$). The equation for the genotype probabilities after t generations of intercrossing, then is:

  \[q_{F_t}=(q_{F_1}L^{t-2})K~.\]

Note that the selfed $F_t$ is a special case with only the 10 selfings of the 55 possible crosses being non-zero. In general, this formal equation can be simplified substantially by using symmetry arguments, and implemented in a similar manner to the selfing case. Transient and absorbing states can be handled in an analogous but somewhat more complicated manner to the selfed case. However, the devil is in the details!

We consider the case of random mating after $s$ generations of backcross and $t$ generations of selfing. We begin with the $q_{BC_s F_t}$ 10-vector of phase-known genotype frequencies, and multiply by a 10$\times$4 matrix ($J$) to convert these frequencies into the four possible two-locus alleles ($u$). These allele frequencies are cross multiplied ($u_Tu$) to create a 4$\times$4 matrix of random mating frequencis of genotypes, which are then reduced to the 10-vector format of phase-known genotype frequencies ($q_{BC_s F_t R}$). Eight of the rows of the matrix $J$ are simple (0, 0.5 or 1 values), while the middle two involve the possible recombinants and non-recombinants:

\[J=  \begin{array}{ccccc}
  & AB & Ab & aB & ab \\[5pt]
\frac{AB}{AB} & 1 & 0 & 0 & 0\\[5pt]
\frac{AB}{Ab} & \frac{1}{2} & \frac{1}{2} & 0 & 0\\[5pt]
\frac{Ab}{Ab} & 0 & 1 & 0 & 0 \\[5pt]
\frac{AB}{aB} & \frac{1}{2} & 0 & \frac{1}{2} & 0 \\[5pt]
\frac{AB}{ab} & \frac{w}{2} & \frac{r}{2} & \frac{r}{2} & \frac{w}{2} \\[5pt]
\frac{aB}{Ab} & \frac{r}{2} & \frac{w}{2} & \frac{w}{2} & \frac{r}{2}\\[5pt]
\frac{ab}{Ab} & 0 & \frac{1}{2} & 0 & \frac{1}{2} \\[5pt]
\frac{aB}{aB} & 0 & 0 & 1 & 0 \\[5pt]
\frac{ab}{aB} & 0 & 0 & \frac{1}{2} & \frac{1}{2} \\[5pt]
\frac{ab}{ab} & 0 & 0 & 0 & 1
\end{array}  \]

Extension to sib-mating instead of selfing follows directly. Extension to selfing or sib-mating after random mating follows as for the $F_t$ approach outlined earlier. Again, the devil is in the details.

\section*{Code modifications in R/qtl}

Our goal was to make R/qtl fully functional for a wider variety of cross types. Ideally these changes will be minimally visible to the user after inputing the cross type.  In order to create an identical user experience we had to modify all R routines so that they would recognize the \texttt{bcsft} cross type.  Cross objects in R/qtl all have the attribute "class" consisting of 2 parts: one which identifies it as a cross object and one which specifies the cross type (\texttt{bc}, \texttt{f2}, \texttt{riself}, \texttt{risib}, etc.).  We added an additional option, \texttt{bcsft}.  A major difference between the previous cross types and \texttt{bcsft}, all other cross types are specific.  In that there are no options for types of backcrosses, it's just a backcross.  With the $BC_sF_t$ we intentionally created a more flexible cross type, where the generation number can be set by the user.  This means that we don't have to go back and add a cross type every time we want to analyze a population with a different history.  The way we have created this flexibity is by adding the attribute \texttt{cross.scheme} to cross objects.  The \texttt{cross.scheme} consists of two numbers, the first is the generations of backcrossing (s), the second is the generations of selfing (t). The addition of a cross type and an attribute allow all R routines to recognize all types of $BC_sF_t$ crosses.

The previous sections detailed the way genotype probabilities and recombination counts are calculated for $BC_sF_t$ crosses.  These calculations are contained within the specific \texttt{init}, \texttt{emit}, and \texttt{step} functions for \texttt{bcsft} within the C code.  All three of these functions are used in the Hidden Markov Model (HMM).  The \texttt{init} function determines the probability of true genotypes.  The \texttt{emit} function determines the probability of observed genotypes given the true genotypes, while the \texttt{step} function determines the probability of a genotype at a particular locus given the genotype at a linked locus as described in the previous section.  We created $BC_sF_t$ versions of all of these functions which follow the same format and work the same way as the existing versions, except for when they are called by \texttt{ est.map}. We did not find a closed form solution for calculating the number of recombination events between pairs of markers in a $BC_sF_t$ and so we implemented a golden section search as part of \texttt{est.map} instead.

There is a second difference between the way the HMM is implemented for $BC_sF_t$ and all other types of crosses, leading to an improvement in efficiency with no effect on the estimates.  Previously the probabilities for each pair of markers for each individual were calculated independently given the recombination rate between those markers in the entire data set.  For the $BC_sF_t$ the entire set of probabilities is calculated once for a set of markers given the recombination rate and then applied to all individuals.  In a population with 100 lines, this is the difference between 10 calculations (1 for each possible genotype combination) and 100 calculations. This method could be readily expanded to analyze populations with mixed cross histories (where some lines have undergone more generations of selfing or backcrossing than others).  Recombination rates could be calculated across all individuals and then probabilities would be calculated separately for each cross history and applied to pairs of markers in an individual according to cross history.   However, record keeping about cross histories for each individual line would need to be implemented in the package.

Genotype probabilities differ for the autosomes and sex chromosomes.  While this is not an issue for selfed populations it could be an issue in an advanced backcross or advanced intercross populations.   We have arranged for proper handling of the X chromosome.  Basically in an $F_t$ the X chromosome is treated as though it were the product of a $BC_t$.  The only real change here is that we created the capacity to keep track of $t$.  All changes to the program have been unit tested and that code is included in the package.


%\section*{References}
% The bibtex filename

\bibliography{vignette}

\end{document}

\name{max.scanPhyloQTL}
\alias{max.scanPhyloQTL}

\title{Maximum peak in genome scan to map a QTL to a phylogenetic tree}

\description{
  Print the chromosome with the maximum LOD score across partitions,
  from the results of \code{\link{scanPhyloQTL}}.
}

\usage{
\method{max}{scanPhyloQTL}(object, chr, format=c("postprob", "lod"),
        \dots)
}

\arguments{
 \item{object}{An object output by the function
   \code{\link{scanPhyloQTL}}.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{format}{Indicates whether to provide LOD scores or approximate
   posterior probabilities; see the help file for \code{\link{summary.scanPhyloQTL}}.}
 \item{\dots}{Ignored at this point.}
}

\details{
  The output, and the use of the argument \code{format}, is as in
  \code{\link{summary.scanPhyloQTL}}.
}

\value{
  An object of class \code{summary.scanPhyloQTL}, to be printed by
  \code{print.summary.scanPhyloQTL}.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
\dontrun{
# example map; drop X chromosome
data(map10)
map10 <- map10[1:19]

# simulate data
x <- simPhyloQTL(4, partition="AB|CD", crosses=c("AB", "AC", "AD"),
                 map=map10, n.ind=150,
                 model=c(1, 50, 0.5, 0))

# run calc.genoprob on each cross
x <- lapply(x, calc.genoprob, step=2)

# scan genome, at each position trying all possible partitions
out <- scanPhyloQTL(x, method="hk")

# maximum peak
max(out, format="lod")

# approximate posterior probabilities at peak
max(out, format="postprob")

# all peaks above a threshold for LOD(best) - LOD(2nd best)
summary(out, threshold=1, format="lod")

# all peaks above a threshold for LOD(best), showing approx post'r prob
summary(out, format="postprob", threshold=3)

# plot of results
plot(out)
}
}

\seealso{ \code{\link{scanPhyloQTL}}, \code{\link{plot.scanPhyloQTL}},
  \code{\link{summary.scanPhyloQTL}}, \code{\link{max.scanone}},
  \code{\link{inferredpartitions}},
  \code{\link{simPhyloQTL}}
}

\references{
  Broman, K. W., Kim, S., An\'e, C. and Payseur, B. A.  Mapping
  quantitative trait loci to a phylogenetic tree.  In preparation.
}


\keyword{print}

\name{scanonevar}
\alias{scanonevar}

\title{Genome scan for QTL affecting mean and/or variance}

\description{Genome scan with a single QTL model for loci that can
  affect the variance as well as the mean.}

\usage{
scanonevar(cross, pheno.col=1, mean_covar=NULL, var_covar=NULL,
           maxit=25, tol=1e-6, quiet=TRUE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  This must be a single value (integer index or
   phenotype name) or a numeric vector of
   phenotype values, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
  \item{mean_covar}{Numeric matrix with covariates affecting the mean.}
  \item{var_covar}{Numeric matrix with covariates affecting the variances.}
  \item{maxit}{Maximum number of iterations in the algorithm to fit the
    model at a given position.}
  \item{tol}{Tolerance for convergence.}
  \item{quiet}{If \code{FALSE}, print some information about the course
    of the calculations.}
}


\value{
  A data frame (with class \code{"scanone"}, in the form output by
  \code{\link{scanone}}), with four columns: chromosome, position, the -log P-value for
  the mean effect, and the -log P-value for the effect on the variance.
  The result is given class \code{"scanone"}
}

\references{
  Ronnegard, L. and Valdar W. (2011) Detecting major genetic loci
  controlling phenotypic variability in experimental crosses. Genetics
  188:435-447

  Ronnegard, L. and Valdar W. (2012) Recent developments in statistical
  methods for detecting genetic loci affecting phenotypic
  variability. BMC Genetics 13:63
}

\author{Lars Ronnegard and Karl Broman}

\examples{
data(fake.bc)
\dontshow{fake.bc <- fake.bc[18:19,]}
fake.bc <- calc.genoprob(fake.bc, step=2.5)
out <- scanonevar(fake.bc)
color <- c("slateblue", "violetred")
plot(out, lod=1:2, col=color, bandcol="gray80")
legend("topright", lwd=2, c("mean", "variance"), col=color)

# use format="allpeaks" to get summary for each of mean and variance
#  also consider format="tabByCol" or format="tabByChr"
summary(out, format="allpeaks")

# with sex and age as covariates
covar <- fake.bc$pheno[,c("sex", "age")]
out.cov <- scanonevar(fake.bc, mean_covar=covar, var_covar=covar)
}

\seealso{ \code{\link{scanone}},
  \code{\link{summary.scanone}}, \code{\link{calc.genoprob}},
  \code{\link{summary.scanoneperm}} }

\keyword{models}

\name{subset.cross}
\alias{subset.cross}
\alias{[.cross}

\title{Subsetting data for QTL experiment}

\description{
  Pull out a specified set of chromosomes and/or individuals from a
  \code{cross} object.
}

\usage{
\method{subset}{cross}(x, chr, ind, \dots)
\method{[}{cross}(x, chr, ind)
}

\arguments{
 \item{x}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector specifying which chromosomes to keep or discard.
   This may be a logical, numeric, or character string vector.  See
 Details, below.}
 \item{ind}{Optional vector specifying which individuals to keep
   discard. This may be a logical, numeric or chacter string vector.
 See Details, below.}
 \item{\dots}{Ignored at this point.}
}

\value{The input \code{cross} object, but with only the specified subset
  of the data.}

\details{
  The \code{chr} argument may be a logical vector with length equal to
  the number of chromosomes in the input cross \code{x}.  Alternatively, it
  should be a vector of character strings referring to chromosomes by
  name.  Numeric values are converted to strings.  Refer to chromosomes
  with a preceding \code{-} to have all chromosomes but those
  considered.

  If the \code{ind} argument is a logical vector
  (\code{TRUE}/\code{FALSE}), it should have length equal to the number
  of individuals in the input cross \code{x}.  The individuals with
  corresponding \code{TRUE} values are retained.

  If the \code{ind} argument is numeric, it should have values either
  between 1 and the number of individuals in the input cross \code{x}
  (in which case these individuals will be retained),
  or it should have values between \code{-1} and \code{-n}, where
  \code{n} is the number of individuals in the input cross \code{x}, in
  which case all \emph{except} these individuals will be retained.

  If the input cross object \code{x} contains individual identifiers (a
  phenotype column labeled \code{"id"} or \code{"ID"}), and if the
  \code{ind} argument contains character strings, then these will be
  matched against the individual identifiers.
  If all values in \code{ind} are
  preceded by a \code{-}), we omit those individuals whose IDs match
  those in \code{ind}.  Otherwise, we retain those individuals whose IDs
  match those in \code{ind}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
fake.f2.A <- subset(fake.f2, chr=c("5","13"))
fake.f2.B <- subset(fake.f2, ind = -c(1,5,10))
fake.f2.C <- subset(fake.f2, chr=1:5, ind=1:50)

data(listeria)
y <- pull.pheno(listeria, 1)
listeriaB <- subset(listeria, ind = (!is.na(y) & y < 264))

# individual identifiers
listeria$pheno$ID <- paste("mouse", 1:nind(listeria), sep="")
listeriaC <- subset(listeria, ind=c("mouse1","mouse11","mouse21"))
listeriaD <- subset(listeria, ind=c("-mouse1","-mouse11","-mouse21"))

# you can also use brackets (like matrix with rows=chromosomes and columns=individuals)
temp <- listeria[c("5","13"),]  # chr 5 and 13
temp <- listeria[ , 1:10]       # first ten individuals
temp <- listeria[5, 1:10]       # chr 5 for first ten individuals
}

\seealso{ \code{\link{pull.map}}, \code{\link{drop.markers}}, \code{\link{subset.map}} }

\keyword{manip}

\name{plotErrorlod}
\alias{plotErrorlod}

\title{Plot grid of error LOD values}

\description{
  Plot a grid of the LOD scores indicating which genotypes are
  likely to be in error.
}

\usage{
plotErrorlod(x, chr, ind, breaks=c(-Inf,2,3,4.5,Inf),
              col=c("white","gray85","hotpink","purple3"),
              alternate.chrid=FALSE, \dots)
}

\arguments{
  \item{x}{An object of class \code{cross}.  See
    \code{\link{read.cross}} for details.}
  \item{chr}{Optional vector indicating the chromosomes to be drawn in
  the plot. This should be a vector of character strings referring to
  chromosomes by name; numeric values are converted to strings.  Refer
  to chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{ind}{Indicates the individuals for which the error LOD scores
    should be plotted (passed to \code{\link{subset.cross}}).}
  \item{breaks}{A set of breakpoints for the colors; must give one more
    breakpoint than color.  Intervals are open on the left and closed on
    the right, except for the lowest interval.}
  \item{col}{A vector of colors to appear in the image.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{\dots}{Ignored at this point.}
}

\details{
  Uses \code{\link[graphics]{image}} to plot a grid with different shades
  of pixels to indicate which genotypes are likely to be in error.

  Darker pixels have higher error LOD scores:
  \eqn{LOD \le 2} in white;
  \eqn{2 < LOD \le 3}{2 < LOD <= 3} in gray;
  \eqn{3 < LOD \le 4.5}{3 < LOD <= 4.5} in pink;
  \eqn{LOD > 4.5} in purple.
}

\value{None.}

\examples{
data(hyper)
\dontshow{hyper <- subset(hyper,chr=c(1,19))}
# Calculate error LOD scores
hyper <- calc.errorlod(hyper,error.prob=0.01)

# plot the error LOD scores; print those above a specified cutoff
plotErrorlod(hyper)
plotErrorlod(hyper,chr=1)
}

\seealso{ \code{\link{calc.errorlod}},
  \code{\link{top.errorlod}}, \code{\link[graphics]{image}},
  \code{\link{subset.cross}}, \code{\link{plotGeno}} }

\references{
  Lincoln, S. E. and Lander, E. S. (1992) Systematic detection of
  errors in genetic linkage data.  \emph{Genomics} \bold{14}, 604--610.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{fake.4way}
\alias{fake.4way}
\docType{data}

\title{Simulated data for a 4-way cross}

\description{
  Simulated data for a phase-known 4-way cross, obtained using
  \code{\link{sim.cross}}.
}

\usage{data(fake.4way)}

\format{
  An object of class \code{cross}.  See \code{\link{read.cross}}
  for details.
}

\details{
There are 250 individuals typed at 157 markers, including 8 on the X
chromosome.

There are two phenotypes (including sex, for which 0=female and 1=male).
The quantitative phenotype is affected by three QTLs: two on chromosome
2 at positions 10 and 25 cM on the female genetic map, and one on
chromosome 7 at position 40 cM on the female map.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.4way)
\dontshow{fake.4way <- subset(fake.4way,chr=c(2,7))}
plot(fake.4way)
summary(fake.4way)

# estimate recombination fractions
fake.4way <- est.rf(fake.4way)
plotRF(fake.4way)

# estimate genetic maps
ssmap <- est.map(fake.4way, verbose=TRUE)
samap <- est.map(fake.4way, sex.sp=FALSE, verbose=TRUE)
plot(ssmap, samap)

# error lod scores
fake.4way <- calc.genoprob(fake.4way, err=0.01)
fake.4way <- calc.errorlod(fake.4way, err=0.01)
top.errorlod(fake.4way, cutoff=2.5)

# genome scan
fake.4way <- calc.genoprob(fake.4way, step=2.5)
out.hk <- scanone(fake.4way, method="hk")
out.em <- scanone(fake.4way, method="em")
plot(out.em,out.hk,chr=c(2,7))
}

\seealso{ \code{\link{sim.cross}}, \code{\link{fake.bc}},
  \code{\link{fake.f2}}, \code{\link{listeria}},
  \code{\link{hyper}},
  \code{\link{bristle3}}, \code{\link{bristleX}} }

\keyword{datasets}

\name{pickMarkerSubset}
\alias{pickMarkerSubset}

\title{Identify the largest subset of markers that are some distance apart}

\description{
  Identify the largest subset of markers for which no two adjacent
  markers are separated by less than some specified distance; if weights
  are provided, find the marker subset for which the sum of the weights
  is maximized.
}

\usage{
pickMarkerSubset(locations, min.distance, weights)
}

\arguments{
 \item{locations}{A vector of marker locations.}
 \item{min.distance}{Minimum distance between adjacent markers in the
 chosen subset.}
 \item{weights}{(Optional) vector of weights for the markers.  If
 missing, we take \code{weights == 1}.}
}

\value{A vector of marker names.}


\details{
Let \eqn{d_i}{d[i]} be
the location of marker \eqn{i}, for \eqn{i \in 1, \dots, M}{i in 1,
\dots, M}.  We use the dynamic programming algorithm of Broman and Weber
(1999) to identify the subset of markers \eqn{i_1, \dots, i_k}{i[1], \dots,
i[k]} for which \eqn{d_{i_{j+1}} - d_{i_j} \le}{d(i[j+1]) - d(i[j]) <=}
\code{min.distance} and \eqn{\sum w_{i_j}}{sum w(i[j])} is maximized.

If there are multiple optimal subsets, we pick one at random.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\references{
  Broman,  K. W. and Weber,  J. L. (1999) Method for constructing
  confidently ordered linkage maps. \emph{Genet. Epidemiol.}, \bold{16},
  337--343.
}

\examples{
data(hyper)

# subset of markers on chr 4 spaced >= 5 cM
pickMarkerSubset(pull.map(hyper)[[4]], 5)

# no. missing genotypes at each chr 4 marker
n.missing <- nmissing(subset(hyper, chr=4), what="mar")

# weight by -log(prop'n missing), but don't let 0 missing go to +Inf
wts <- -log( (n.missing+1) / (nind(hyper)+1) )

# subset of markers on chr 4 spaced >= 5 cM, with weights = -log(prop'n missing)
pickMarkerSubset(pull.map(hyper)[[4]], 5, wts)
}

\seealso{ \code{\link{drop.markers}}, \code{\link{pull.markers}},
          \code{\link{findDupMarkers}} }

\keyword{manip}

\name{mqmprocesspermutation}
\alias{mqmprocesspermutation}

\title{Convert mqmmulti objects into a scanoneperm object}

\description{
    Function to convert \code{mqmmulti} objects into a \code{scanoneperm}
  object, this allows the use of R/qtl methods for permutation analysis
  that do not support the output of a multiple QTL scan using mqm's
  outputstructure.
}

\usage{
mqmprocesspermutation(mqmpermutationresult = NULL)
}

\arguments{
  \item{mqmpermutationresult}{ \code{mqmmulti} object obtained
    after performing permutations on a single trait.using the
    function \code{\link{mqmpermutation}}
  }
}

\value{
    Output of the algorithm is a \code{scanoneperm} object. See also: \code{\link{summary.scanoneperm}}
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
  \itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
# QTL at chromosome 3
f2qtl <- c(3,15,1,0)

# Mouse genetic map
data(map10)

\dontshow{map10 <- map10[1:3]}
# Simulate a F2 Cross
f2cross <- sim.cross(map10,f2qtl,n=100,type="f2")
\dontrun{# Bootstrap MQM mapping on the f2cross
f2result <- mqmpermutation(f2cross,scanfunction=mqmscan)
}\dontshow{f2result <- mqmpermutation(f2cross,scanfunction=mqmscan, n.perm=2, batchsize=2)}

# Create a permutation object
f2perm <- mqmprocesspermutation(f2result)

# What LOD score is considered significant?
summary(f2perm)
}

\keyword{ models }

\name{pull.draws}
\alias{pull.draws}

\title{Pull out the genotype imputations from a cross}

\description{
  Pull out the results of \code{\link{sim.geno}} from a cross as an array.
}

\usage{pull.draws(cross, chr)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
}

\value{
  An array containing numeric indicators of the imputed genotypes.
  Multiple chromosomes are pasted together.  The dimensions are
  individuals by positions by imputations
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
listeria <- sim.geno(listeria, step=5, stepwidth="max", n.draws=8)
dr <- pull.draws(listeria, chr=c(5,13))
dr[1:20,1:10,1]
}

\seealso{ \code{\link{pull.geno}}, \code{\link{pull.genoprob}},
  \code{\link{pull.argmaxgeno}}, \code{\link{sim.geno}} }

\keyword{utilities}

\name{summaryScantwoOld}
\alias{summaryScantwoOld}

\title{Summarize the results of a two-dimensional genome scan}

\description{
  Summarize the interesting aspects of the results of
  \code{\link{scantwo}}; this is the version of
  \code{\link{summary.scantwo}} that was included in R/qtl version
  1.03 and earlier.
}

\usage{
summaryScantwoOld(object, thresholds = c(0, 0, 0), lodcolumn=1,
                type = c("joint","interaction"), \dots)
}

\arguments{
 \item{object}{An object of class \code{scantwo}, the output of the
   function \code{\link{scantwo}}.}
 \item{thresholds}{A vector of length three, giving LOD thresholds
   for the joint LOD, interaction LOD and single-QTL conditional
   LOD. Negative threshold values are taken relative to the maximum
   joint, interaction, or individual QTL LOD, respectively.}
  \item{lodcolumn}{If the scantwo results contain LOD scores for
    multiple phenotypes, this argument indicates which to use in the
    summary.}
 \item{type}{Indicates whether to pick peaks with maximal joint or
   interaction LOD.}
 \item{\dots}{Ignored at this point.}
}

\value{
  An object of class \code{summary.scantwo.old}, to be printed by
  \code{print.summary.scantwo.old}.  Pairs of loci meeting
  the specified criteria are printed, with their joint LOD, interaction
  LOD, and the conditional LOD for each locus, along with single-point
  P-values calculated by the \eqn{\chi^2}{chi-square} approximation.
  P-values are printed as \eqn{-\log_{10}(P)}{-log10(P)}. \cr

  If the input \code{scantwo} object does not include the results of
  \code{scanone}, the interaction and conditional LOD thresholds are
  ignored, and all pairs of loci for which the joint LOD exceeds its
  threshold are printed, though without their conditional LOD scores.
}

\details{
  For each pair of chromosomes, the pair of loci for which the
  LOD score (either joint or interaction LOD, according to the argument
  \code{type}) is a maximum is considered.  The pair is printed only if
  its joint LOD score exceeds the joint threshold and either (a) the
  interaction LOD score exceeds its threshold or (b) both of the loci have
  conditional LOD scores that are above the conditional LOD threshold,
  where the conditional LOD score for locus \eqn{q_1}{q1}, \eqn{LOD(q_1
    | q_2)}{LOD(q1 | q2)}, is the \eqn{\log_{10}}{log10} likelihood ratio
  comparing the model with \eqn{q_1}{q1} and \eqn{q_2}{q2} acting
  additively to the model with \eqn{q_2}{q2} alone.

  In the case the results of \code{\link{scanone}} are not
  available, the maximum locus pair for each chromosome is printed
  whenever its joint LOD exceeds the joint LOD threshold.

  The criterion used in this summary is due to Gary Churchill and \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}aunak
  Sen, and deserves careful consideration and possible revision.
}

\author{Hao Wu; Karl W Broman,
  \email{broman@wisc.edu}; Brian Yandell }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(1,6,13))}
fake.f2 <- calc.genoprob(fake.f2, step=5)
out.2dim <- scantwo(fake.f2, method="hk")

# All pairs of loci
summaryScantwoOld(out.2dim)

# Pairs meeting specified criteria
summaryScantwoOld(out.2dim, c(7, 3, 3))

# Pairs with both conditional LODs > 2
summaryScantwoOld(out.2dim,c(0,1000,2))

# Pairs with interaction LOD is above 3
summaryScantwoOld(out.2dim,c(0,3,1000))
}

\seealso{ \code{\link{summary.scantwo}},
  \code{\link{scantwo}}, \code{\link{plot.scantwo}},
          \code{\link{max.scantwo}} }

\keyword{print}

\name{stepwiseqtl}
\alias{stepwiseqtl}

\title{Stepwise selection for multiple QTL}

\description{
  Performs forward/backward selection to identify a multiple QTL model,
  with model choice made via a penalized LOD score, with separate
  penalties on main effects and interactions.
}

\usage{
stepwiseqtl(cross, chr, pheno.col=1, qtl, formula, max.qtl=10, covar=NULL,
            method=c("imp", "hk"), model=c("normal", "binary"),
            incl.markers=TRUE, refine.locations=TRUE,
            additive.only=FALSE, scan.pairs=FALSE, penalties,
            keeplodprofile=TRUE, keeptrace=FALSE, verbose=TRUE,
            tol=1e-4, maxit=1000, require.fullrank=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider in
   search for QTL.  This should be a vector of character strings
   referring to chromosomes by name; numeric values are converted to
   strings.  Refer to chromosomes with a preceding \code{-} to have all
   chromosomes but those considered.  A logical (TRUE/FALSE) vector may
   also be used.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  One may also give character strings matching
   the phenotype names.  Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{qtl}{Optional QTL object (of class \code{"qtl"}, as created by
   \code{\link{makeqtl}}) to use as a starting point.}
 \item{formula}{Optional formula to define the QTL model to be used as a
   starting point.}
 \item{max.qtl}{Maximum number of QTL to which forward selection should
   proceed.}
 \item{covar}{Data frame of additive covariates.}
 \item{method}{Indicates whether to use multiple imputation or
   Haley-Knott regression.}
 \item{model}{The phenotype model: the usual model or a model for binary
  traits}
 \item{incl.markers}{If FALSE, do calculations only at points on an
   evenly spaced grid.}
 \item{refine.locations}{If TRUE, use \code{\link{refineqtl}} to
   refine the QTL locations after each step of forward and backward
   selection.}
 \item{additive.only}{If TRUE, allow only additive QTL models; if FALSE,
 consider also pairwise interactions among QTL.}
 \item{scan.pairs}{If TRUE, perform a two-dimensional, two-QTL scan at
   each step of forward selection.}
 \item{penalties}{Vector of three (or six) values indicating the penalty
   on the number of QTL terms. If three values, these are the penalties
   on main effects and heavy and light penalties on interactions.  If
   six values, these include X-chr-specific penalties, and the values
   are: main effect for autosomes, main effect for X chr, heavy penalty
   on A:A interactions, light penalty on A:A interactions, penalty on
   A:X interactions, and penalty on X:X interactions. See the Details
   below. If missing, default values are used that are based on
   simulations of backcrosses and intercrosses with genomes modeled
   after that of the mouse.}
 \item{keeplodprofile}{If TRUE, keep the LOD profiles from the last
   iteration as attributes to the output.}
 \item{keeptrace}{If TRUE, keep information on the sequence of models
   visited through the course of forward and backward selection as an
   attribute to the output.}
 \item{verbose}{If TRUE, give feedback about progress.  If
   \code{verbose} is an integer > 1, even more information is printed.}
 \item{tol}{Tolerance for convergence for the binary trait model.}
 \item{maxit}{Maximum number of iterations for fitting the binary trait
   model.}
 \item{require.fullrank}{If TRUE, give LOD=0 when covariate matrix in
   the linear regression is not of full rank.}
}

\section{Methods}{
  \bold{\code{imp}}: multiple imputation is used, as described by Sen
  and Churchill (2001).

  \bold{\code{hk}}: Haley-Knott regression is used (regression of the
  phenotypes on the multipoint QTL genotype probabilities), as described
  by Haley and Knott (1992).
}

\details{
  We seek to identify the model with maximal penalized LOD score.  The
  penalized LOD score, defined in Manichaikul et al. (2009),
  is the LOD score for the model (the \eqn{\log_{10}}{log10} likelihood
  ratio comparing the model to the null model with no QTL) with
  penalties on the number of QTL and QTL:QTL interactions.

  We consider QTL models allowing pairwise interactions among QTL but
  with an enforced hierarchy in which inclusion of a pairwise
  interaction requires the inclusion of both of the corresponding main
  effects.  Additive covariates may be included, but currently we do not
  explore QTL:covariate interactions.  Also, the penalized LOD score
  criterion is currently defined only for autosomal loci, and results
  with the X chromosome should be considered with caution.

  The penalized LOD score is of the form \eqn{pLOD(\gamma) = LOD(\gamma)
  - T_m p_m - T_h p_h - T_l p_l}{pLOD(g) = LOD(g)
  - Tm pm - Th ph - Tl pl} where \eqn{\gamma}{g} denotes a model,
  \eqn{p_m}{pm} is the number of QTL in the model ("main effects"),
  \eqn{p_h}{ph} is the number of pairwise interactions that will be
  given a heavy interaction penalty, \eqn{p_l}{pl} is the number of pairwise
  interactions that will be given a light interaction penalty,
  \eqn{T_m}{Tm} is the penalty on main effects, \eqn{T_h}{Th} is the heavy
  interaction penalty, and \eqn{T_l}{Tl} is the light interaction
  penalty.  The \code{penalties} argument is the vector \eqn{(T_m, T_h,
  T_l)}{(Tm, Th, Tl)}.  If \eqn{T_l}{Tl} is missing (\code{penalties}
  has a vector of length 2), we assume \eqn{T_l = T_h}{Tl = Th}, and so
  all pairwise interactions are assigned the same penalty.

  The "heavy" and "light" interaction penalties can be a bit
  confusing.  Consider the clusters of QTL that are connected via one or
  more pairwise interactions.  To each such cluster, we assign at most
  one "light" interaction penalty, and give all other pairwise
  interactions the heavy interaction penalty.  In other words, if
  \eqn{p_i}{pi} is the total number of pairwise interactions for a QTL
  model, we let \eqn{p_l}{pl} be the number of clusters of connected QTL
  with at least one pairwise interaction, and then let \eqn{p_h - p_i -
    p_l}{ph = pi - pl}.

  Let us give an explicit example.  Consider a model with 6 QTL, and
  with interactions between QTL 2 and 3, QTL 4 and 5 and QTL 4 and 6
  (so we have the model formula
  \code{y ~ Q1 + Q2 + Q3 + Q4 + Q5 + Q6 + Q2:Q3 + Q4:Q5 + Q4:Q6}).
  There are three clusters of connected QTL: (1), (2,3) and (4,5,6).  We
  would assign 6 main effect penalties (\eqn{T_m}{Tm}), 2 light
  interaction penalties (\eqn{T_l}{Tl}), and 1 heavy interaction penalty
  (\eqn{T_h}{Th}).

  Manichaikul et al. (2009) described a system for deriving the
  three penalties on the basis of permutation results from a
  two-dimensional, two-QTL genome scan (as calculated with
  \code{\link{scantwo}}).  These may be calculated with the
  function \code{\link{calc.penalties}}.

  A forward/backward search method is used, with the aim to optimize the
  penalized LOD score criterion.  That is, we seek to identify the
  model with maximal the penalized LOD score.  The search algorithm was
  based closely on an algorithm described by Zeng et al. (1999).

  We use forward selection to a model of moderate size (say 10 QTL),
  followed by backward elimination all the way to the null model.  The
  chosen model is that which optimizes the penalized LOD score
  criterion, among all models visited.  The detailed algorithm is as
  follows.  Note that if \code{additive.only=TRUE}, no pairwise
  interactions are considered.

  \enumerate{
    \item Start at the null model, and perform a single-QTL genome scan,
    and choose the position giving the largest LOD score.  If
    \code{scan.pairs=TRUE}, start with a two-dimensional, two-QTL genome
    scan instead.  If an initial QTL model were defined through the
    arguments \code{qtl} and \code{formula}, start with this model and
    jump immediately to step 2.

    \item With a fixed QTL model in hand:
    \enumerate{
      \item Scan for an additional additive QTL.

      \item For each QTL in the current model, scan for an additional
      interacting QTL.

      \item If there are \eqn{\ge}{>=} 2 QTL in the current model,
      consider adding one of the possible pairwise interactions.

      \item If \code{scan.pairs=TRUE} perform a two-dimensional, two-QTL
      scan, seeking to add a pair of novel QTL, either additive or
      interacting.

      \item Step to the model that gives the largest value for the model
      comparison criterion, among those considered at the current step.
      }

      \item Refine the locations of the QTL in the current model (if
      \code{refine.locations=TRUE}).

      \item Repeat steps 2 and 3 up to a model with some pre-determined
      number of loci.

      \item Perform backward elimination, all the way back to the null
      model.  At each step, consider dropping one of the current main
      effects or interactions; move to the model that maximizes the model
      comparison criterion, among those considered at this step.  Follow
      this with a refinement of the locations of the QTL.

      \item Finally, choose the model having the largest model comparison
      criterion, among all models visited.
  }

  In this forward/backward algorithm, it is likely best to build up to
  an overly large model and then prune it back.  Note that there is no
  "stopping rule"; the chosen model is that which optimizes the model
  comparison criterion, among all models visited.  The search can be
  time consuming, particularly if a two-dimensional scan is performed at
  each forward step.  Such two-dimensional scans may be useful for
  identifying QTL linked in repulsion (having effects of opposite sign)
  or interacting QTL with limited marginal effects, but our limited
  experience suggests that they are not necessary; important linked or
  interacting QTL pairs can be picked up in the forward selection to a
  large model, and will be retained in the backward elimination phase.
}

\value{
  The output is a representation of the best model, as measured by the
  penalized LOD score (see Details), among all models visited.
  This is QTL object (of class \code{"qtl"}, as produced by
  \code{\link{makeqtl}}), with attributes \code{"formula"},
  indicating the model formula, and \code{"pLOD"} indicating the
  penalized LOD score.

  If \code{keeplodprofile=TRUE}, LOD profiles from the last pass through
  the refinement algorithm are retained as an attribute,
  \code{"lodprofile"}, to the object.  These may be plotted with
  \code{\link{plotLodProfile}}.

  If \code{keeptrace=TRUE}, the output will contain an attribute
  \code{"trace"} containing information on the best model at each step
  of forward and backward elimination.  This is a list of objects of
  class \code{"compactqtl"}, which is similar to a QTL object (as
  produced by \code{\link{makeqtl}}) but containing just
  a vector of chromosome IDs and positions for the QTL.  Each will also
  have attributes \code{"formula"} (containing the model formula) and
  \code{"pLOD"} (containing the penalized LOD score.
}

\references{
  Manichaikul, A., Moon, J. Y., Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}, Yandell, B. S. and Broman,
  K. W. (2009) A model selection approach for the identification of
  quantitative trait loci in experimental crosses, allowing
  epistasis. \emph{Genetics}, \bold{181}, 1077--1086.

  Broman,  K. W. and Speed, T. P. (2002) A model selection approach for the
  identification of quantitative trait loci in experimental crosses (with
  discussion). \emph{J Roy Stat Soc B} \bold{64}, 641--656, 731--775.

  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.

  Zeng, Z.-B., Kao, C.-H. and Basten, C. J. (1999) Estimating the
  genetic architecture of quantitative traits.  \emph{Genetical
  Research}, \bold{74}, 279--289.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=c(1,2,3,5))}
\dontrun{fake.bc <- calc.genoprob(fake.bc, step=2.5)
}\dontshow{fake.bc <- calc.genoprob(fake.bc, step=0)}
outsw <- stepwiseqtl(fake.bc, max.qtl=3, method="hk", keeptrace=TRUE)

# best model
outsw
plotModel(outsw)

# path through model space
thetrace <- attr(outsw, "trace")

# plot of these
par(mfrow=c(3,3))
for(i in seq(along=thetrace))
  plotModel(thetrace[[i]], main=paste("pLOD =",round(attr(thetrace[[i]],"pLOD"), 2)))
}

\seealso{ \code{\link{calc.penalties}},
  \code{\link{plotModel}}, \code{\link{makeqtl}},
  \code{\link{fitqtl}}, \code{\link{refineqtl}},
  \code{\link{addqtl}}, \code{\link{addpair}} }

\keyword{models}

\name{mqmscanfdr}
\alias{mqmscanfdr}

\title{ Estimate FDR for multiple trait QTL analysis }

\description{
  Estimate the false discovery rate (FDR) for multiple trait analysis
}

\usage{
mqmscanfdr(cross, scanfunction=mqmscanall,
  thresholds=c(1,2,3,4,5,7,10,15,20), n.perm=10,
  verbose=FALSE, \dots
  )
}

\arguments{
    \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
    \item{scanfunction}{ QTL mapping function, Note: Must use scanall or mqmscanall. Otherwise this will
  not produce usefull results. Reason: We need a function that maps all traits ecause of the correlation
  structure which is not changed (between traits) during permutation (Valis options: scanall or mqmscanall)
  }
    \item{thresholds}{ False discovery rate (FDR) is calculated for peaks
    above these LOD thresholds (DEFAULT=Range from 1 to 20, using 10 thresholds)
    Parameter is a list of LOD scores at which FDR is calculated.
  }
    \item{n.perm}{ Number of permutations (DEFAULT=10 for quick analysis, however
    for publications use 1000, or higher)
  }
    \item{verbose}{ verbose output }
    \item{\dots}{ Parameters passed to the mapping function
  }
}

\details{
  This function wraps the analysis of \code{\link{scanone}}, \code{\link{cim}}
  and \code{\link{mqmscan}} to scan for QTL in shuffled/randomized data. It is
  recommended to also install the \code{snow} library for parallelization of
  calculations. The \code{snow} library allows
  calculations to run on multiple cores or even scale it up to an entire cluster,
  thus speeding up calculation by the number of computers used.
}

\value{
    Returns a data.frame with 3 columns: FalsePositives, FalseNegatives and False Discovery Rates.
  In the rows the userspecified thresholds are with scores for the 3 columns.
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
  }

\seealso{
    \itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
    }
}

\examples{
data(multitrait)

# impute missing genotype data
multitrait <- fill.geno(multitrait)
\dontrun{# Calculate the thresholds
result <- mqmscanfdr(multitrait, threshold=10.0, n.perm=1000)
}\dontshow{
  multitrait <- subset(multitrait, chr=1, ind=!apply(multitrait$pheno, 1, function(a) any(is.na(a))))
  multitrait$pheno <- multitrait$pheno[,1:3]
  result <- mqmscanfdr(multitrait, threshold=10.0, n.perm=2)
}
}

\references{
\itemize{
% \input{"inst/doc/Sources/MQM/mqm/significance_references.txt"}
    \item Bruno M. Tesson, Ritsert C. Jansen (2009) Chapter 3.7. Determining the significance threshold \emph{eQTL Analysis in Mice and Rats} \bold{1}, 20--25
        \item Churchill, G. A. and Doerge, R. W. (1994) Empirical threshold values for quantitative trait mapping. \emph{Genetics} \bold{138}, 963--971.
% -----^^ inst/doc/Sources/MQM/mqm/significance_references.txt ^^-----
% \input{"inst/doc/Sources/MQM/mqm/parallelisation_references.txt"}
        \item Rossini, A., Tierney, L., and Li, N. (2003), Simple parallel statistical computing.  \emph{R. UW Biostatistics working paper series} University of Washington. \bold{193}
        \item Tierney, L., Rossini, A., Li, N., and Sevcikova, H. (2004), The snow Package: Simple Network of Workstations. Version 0.2-1.
% -----^^ inst/doc/Sources/MQM/mqm/parallelisation_references.txt ^^-----
}
}

\keyword{ models }

\name{add.cim.covar}
\alias{add.cim.covar}

\title{Indicate marker covariates from composite interval mapping}

\description{
  Add dots at the locations of the selected marker covariates, for a
  plot of composite interval mapping results.
}

\usage{
add.cim.covar(cimresult, chr, gap=25, \dots)
}

\arguments{
 \item{cimresult}{Composite interval mapping results, as output from
   \code{\link{cim}}.}
  \item{chr}{Optional vector specifying which chromosomes to plot. (The
    chromosomes must be specified by name.)  This should be identical to
  that used in the call to \code{\link{plot.scanone}}.}
  \item{gap}{Gap separating chromosomes (in cM). This should be identical to
    that used in the call to \code{\link{plot.scanone}}.}
  \item{\dots}{Additional plot arguments, passed to the function
    \code{\link[graphics]{points}}.}
}


\details{
  One must first have used the function \code{\link{plot.scanone}}
  to plot the composite interval mapping results.

  The arguments \code{chr} and \code{gap} must be identical to the
  values used in the call to \code{\link{plot.scanone}}.

  Dots indicating the locations of the selected marker covariates are
  displayed on the x-axis.  (By default, solid red circles are plotted;
  this may be modified by specifying the graphics parameters \code{pch} and
  \code{col}.)
}

\value{
  A data frame indicating the marker covariates that were plotted.
}


\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
\dontrun{data(hyper)
hyper <- calc.genoprob(hyper, step=2.5)

out <- scanone(hyper)
out.cim <- cim(hyper, n.marcovar=3)
plot(out, out.cim, chr=c(1,4,6,15), col=c("blue", "red"))

add.cim.covar(out.cim, chr=c(1,4,6,15))}
}

\seealso{ \code{\link{cim}}, \code{\link{plot.scanone}} }

\keyword{hplot}

\name{geno.table}
\alias{geno.table}

\title{Create table of genotype distributions}

\description{
  Create table showing the observed numbers of individuals with each
  genotype at each marker, including P-values from chi-square tests
  for Mendelian segregation.
}

\usage{
geno.table(cross, chr, scanone.output=FALSE)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{scanone.output}{If TRUE, give result in the form output by
 \code{\link{scanone}}, so that one may use \code{\link{plot.scanone}}, etc.}
}

\value{
  If \code{scanone.output=FALSE}, the output is a matrix containing, for
  each marker, the number of individuals with each possible genotype, as
  well as the number that were not typed.  The first column gives the
  chromosome ID, and the last column gives P-values from chi-square
  tests of Mendelian segregation.

  If \code{scanone.output=TRUE}, the output is of the form produced by
  \code{\link{scanone}}, with the first two columns being chromosome IDs
  and cM positions of the markers.  The third column is
  \eqn{-\log_{10}(P)}{-log10P} from chi-square tests of Mendelian
  segregation.  The fourth column is the proportion of missing data.
  The remaining columns are the proportions of the different genotypes
  (among typed individuals).
}

\details{
  The P-values are obtained from chi-square tests of Mendelian
  segregation.  In the case of the X chromosome, the sexes and cross
  directions are tested separately, and the chi-square statistics
  combined, and so the test is of whether any of the groups show
  deviation from Mendel's rules.
}

\examples{
data(listeria)
geno.table(listeria)

geno.table(listeria, chr=13)

gt <- geno.table(listeria)
gt[gt$P.value < 0.01,]

out <- geno.table(listeria, scanone.output=TRUE)
plot(out)
plot(out, lod=2)
}

\seealso{ \code{\link{summary.cross}},
  \code{\link{drop.markers}}, \code{\link{drop.nullmarkers}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{utilities}

\name{plotRF}
\alias{plotRF}

\title{Plot recombination fractions}

\description{
  Plot a grid showing the recombination fractions for all pairs of
  markers, and/or the LOD scores for tests of linkage between pairs of
  markers.
}

\usage{
plotRF(x, chr, what=c("both","lod","rf"), alternate.chrid=FALSE,
        zmax=12, mark.diagonal=FALSE,
        col.scheme=c("viridis", "redblue"), \dots)
}

\arguments{
  \item{x}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
  \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{what}{Indicate whether to plot LOD scores, recombination
               fractions or both.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{zmax}{Maximum LOD score plotted; values above this are all
    thresholded at this value.}
  \item{mark.diagonal}{If TRUE, include black line segments around the
  pixels along the diagonal, to better separate the upper left triangle
  from the lower right triangle.}
  \item{col.scheme}{The color palette. The default is "viridis"; see
  Option D at \url{https://bids.github.io/colormap/}}
  \item{\dots}{Generally ignored, but you can include \code{main} to
  change or omit the title of the figure.}
}

\value{None.}

\details{
  Uses \code{\link[graphics]{image}} to plot a grid showing the
  recombination fractions and/or LOD scores for all pairs of markers.
  (The LOD scores are for a test of \eqn{r = 1/2}.)
  If both are plotted, the recombination fractions are in the upper left
  triangle while the LOD scores are in the lower right triangle.

  With \code{col.scheme="viridis"} (the default), purple corresponds to
  a large LOD score or a small recombination fraction, while yellow is
  the reverse. With \code{col.scheme="redblue"}, red corresponds to a
  large LOD or a small recombination fraction, while blue is the
  reverse.  Note that missing values appear in light gray.

  Recombination fractions are transformed by \eqn{-4(\log_2{r}+1)}{%
    -4(log2(r)+1)} to make them on the same sort of scale as LOD
  scores.  Values of LOD or the transformed recombination fraction that
  are above 12 are set to 12.
}


\examples{
data(badorder)
badorder <- est.rf(badorder)
plotRF(badorder)

# plot just chr 1
plotRF(badorder, chr=1)

# plot just the recombination fractions
plotRF(badorder, what="rf")

# plot just the LOD scores, and just for chr 2 and 3
plotRF(badorder, chr=2:3, what="lod")
}

\seealso{ \code{\link{est.rf}}, \code{\link{pull.rf}}, \code{\link{plot.rfmatrix}},
  \code{\link[graphics]{image}},
  \code{\link{badorder}}, \code{\link{ripple}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{calc.errorlod}
\alias{calc.errorlod}

\title{Identify likely genotyping errors}

\description{
  Calculates a LOD score for each genotype, measuring the evidence for
  genotyping errors.
}

\usage{
calc.errorlod(cross, error.prob=0.01,
              map.function=c("haldane","kosambi","c-f","morgan"),
              version=c("new","old"))
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype)}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions.}
 \item{version}{Specifies whether to use the original version of this
   function or the current (preferred) version.}
}

\details{
  Calculates, for each individual at each marker, a LOD score
  measuring the strength of evidence for a genotyping error, as
  described by Lincoln and Lander (1992).

  In the latest version, evidence for a genotype being in
  error is considered assuming that all other genotypes (for that
  individual, on that chromosome) are correct.  The argument
  \code{version} allows one to specify whether this new version is used,
  or whether the original (old) version of the calculation is
  performed.

  Note that values below 4 are generally not interesting.  Also note
  that if markers are extremely tightly linked, \emph{recombination
  events} can give large error LOD scores.  The error LOD scores should
  not be trusted blindly, but should be viewed as a tool for identifying
  genotypes deserving further study.

  Use \code{\link{top.errorlod}} to print all genotypes with error
  LOD scores above a specified threshold,
  \code{\link{plotErrorlod}} to plot the error LOD scores for
  specified chromosomes, and \code{\link{plotGeno}} to view the
  observed genotype data with likely errors flagged.
}

\value{
  The input \code{cross} object is returned with a component,
  \code{errorlod}, added to each component of \code{cross$geno}.  The
  \code{errorlod} component is a matrix of size (n.ind x n.mar).  An
  attribute \code{"error.prob"} is set to the value of the corresponding
  argument, for later reference.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\references{
  Lincoln, S. E. and Lander, E. S. (1992) Systematic detection of
  errors in genetic linkage data.  \emph{Genomics} \bold{14}, 604--610.
}

\examples{
data(hyper)
\dontshow{hyper <- subset(hyper,chr=c(1,19))}
hyper <- calc.errorlod(hyper,error.prob=0.01)

# print those above a specified cutoff
top.errorlod(hyper, cutoff=4)

# plot genotype data, flagging genotypes with error LOD > cutoff
plotGeno(hyper, chr=1, ind=160:200, cutoff=7, min.sep=2)
}

\seealso{ \code{\link{plotErrorlod}},
  \code{\link{top.errorlod}}, \code{\link{cleanGeno}} }


\keyword{utilities}

\name{est.map}
\alias{est.map}

\title{Estimate genetic maps}

\description{
  Uses the Lander-Green algorithm (i.e., the hidden Markov model
  technology) to re-estimate the genetic map for an experimental cross.
}

\usage{
est.map(cross, chr, error.prob=0.0001,
        map.function=c("haldane","kosambi","c-f","morgan"),
        m=0, p=0, maxit=10000, tol=1e-6, sex.sp=TRUE,
        verbose=FALSE, omit.noninformative=TRUE, offset, n.cluster=1)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
   This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions. (Ignored if m > 0.)}
 \item{m}{Interference parameter for the chi-square model for
   interference; a non-negative integer, with m=0 corresponding to no
   interference. This may be used only for a backcross or intercross.}
 \item{p}{Proportion of chiasmata from the NI mechanism, in the Stahl
   model; p=0 gives a pure chi-square model.  This may be used only for
   a backcross or intercross.}
 \item{maxit}{Maximum number of EM iterations to perform.}
 \item{tol}{Tolerance for determining convergence.}
 \item{sex.sp}{Indicates whether to estimate sex-specific maps; this is
 used only for the 4-way cross.}
 \item{verbose}{If TRUE, print tracing information.}
 \item{omit.noninformative}{If TRUE, on each chromosome, omit individuals
   with fewer than two typed markers, since they are not informative for
   linkage.}
 \item{offset}{Defines the starting position
  for each chromosome.  If missing, we use the starting positions that
  are currently present in the input cross object.  This should be a
  single value (to be used for all chromosomes) or a vector with length
  equal to the number of chromosomes, defining individual starting
  positions for each chromosome.  For a sex-specific map (as in a 4-way
  cross), we use the same offset for both the male and female maps.}
 \item{n.cluster}{If the package \code{snow} is available
   calculations for multiple chromosomes are run in parallel using this
   number of nodes.}
}

\details{
  By default, the map is estimated assuming no crossover interference,
  but a map function is used to derive the genetic distances (though, by
  default, the Haldane map function is used).

  For a backcross or intercross, inter-marker distances may be estimated
  using the Stahl model for crossover interference, of which the
  chi-square model is a special case.

  In the chi-square model, points are tossed down onto the four-strand
  bundle according to a Poisson process, and every \eqn{(m+1)}st point is a
  chiasma.  With the assumption of no chromatid interference, crossover
  locations on a random meiotic product are obtained by thinning the
  chiasma process.  The parameter \eqn{m} (a non-negative integer)
  governs the strength of crossover interference, with \eqn{m=0}
  corresponding to no interference.

  In the Stahl model, chiasmata on the four-strand bundle are a
  superposition of chiasmata from two mechanisms, one following a
  chi-square model and one exhibiting no interference.  An additional
  parameter, \eqn{p}, gives the proportion of chiasmata from the no
  interference mechanism.
}

\value{
  A \code{map} object; a list whose components (corresponding to
  chromosomes) are either vectors of marker positions (in cM) or
  matrices with two rows of sex-specific marker positions.
  The maximized log likelihood for each chromosome is saved as an
  attribute named \code{loglik}.  In the case that estimation was under
  an interference model (with m > 0), allowed only for a backcross, m
  and p are also included as attributes.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2,chr=18:19)}
newmap <- est.map(fake.f2)
logliks <- sapply(newmap, attr, "loglik")
plotMap(fake.f2, newmap)
fake.f2 <- replace.map(fake.f2, newmap)
}

\references{
  Armstrong, N. J., McPeek, M. J. and Speed, T. P. (2006) Incorporating
  interference into linkage analysis for experimental crosses.
  \emph{Biostatistics} \bold{7}, 374--386.

  Lander, E. S. and Green, P. (1987) Construction of multilocus genetic linkage
  maps in humans.  \emph{Proc. Natl. Acad. Sci. USA} \bold{84}, 2363--2367.

  Lange, K. (1999) \emph{Numerical analysis for statisticians}.
  Springer-Verlag. Sec 23.3.

  Rabiner, L. R. (1989) A tutorial on hidden Markov models and selected
  applications in speech recognition.  \emph{Proceedings of the IEEE}
  \bold{77}, 257--286.

  Zhao, H., Speed, T. P. and McPeek, M. S. (1995) Statistical analysis of
  crossover interference using the chi-square model.  \emph{Genetics}
  \bold{139}, 1045--1056.
}

\seealso{ \code{\link{map2table}}, \code{\link{plotMap}}, \code{\link{replace.map}},
  \code{\link{est.rf}}, \code{\link{fitstahl}} }

\keyword{utilities}

\name{addpair}
\alias{addpair}

\title{Scan for an additional pair of QTL in a multiple-QTL model}

\description{
  Scan for an additional pair of QTL in the context of a
  multiple QTL model.
}

\usage{
addpair(cross, chr, pheno.col=1, qtl, covar=NULL, formula,
        method=c("imp","hk"), model=c("normal", "binary"),
        incl.markers=FALSE, verbose=TRUE, tol=1e-4, maxit=1000,
        forceXcovar=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to be scanned. If
   missing, all chromosomes are scanned. Refer to chromosomes by
   name. Refer to chromosomes with a preceding \code{-} to have all
   chromosomes but those considered.  A logical (TRUE/FALSE) vector may
   also be used.}
 \item{pheno.col}{Column number in the phenotype matrix to be
   used as the phenotype.  One may also give a character string matching
   a phenotype name. Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{qtl}{An object of class \code{qtl}, as output from
   \code{\link{makeqtl}}.}
 \item{covar}{A matrix or data.frame of covariates.  These must be
   strictly numeric.}
 \item{formula}{An object of class \code{\link[stats]{formula}}
   indicating the model to be fitted.  (It can also be the character
   string representation of a formula.)  QTLs are referred to as
   \code{Q1}, \code{Q2}, etc.  Covariates are referred to by their names
   in the data frame \code{covar}.  If the new QTL are not included in
   the formula, a two-dimensional scan as in \code{\link{scantwo}}
   is performed.}
 \item{method}{Indicates whether to use multiple imputation or
   Haley-Knott regression.}
 \item{model}{The phenotype model: the usual model or a model for binary
  traits}
 \item{incl.markers}{If FALSE, do calculations only at points on an
   evenly spaced grid.  If \code{\link{calc.genoprob}} or
   \code{\link{sim.geno}} were run with
   \code{stepwidth="variable"} or \code{stepwidth="max"}, we force \code{incl.markers=TRUE}.}
 \item{verbose}{If TRUE, display information about the progress of
   calculations.  If \code{verbose} is an integer > 1, further messages
   from \code{\link{scanqtl}} are also displayed.}
 \item{tol}{Tolerance for convergence for the binary trait model.}
 \item{maxit}{Maximum number of iterations for fitting the binary trait
   model.}
 \item{forceXcovar}{If TRUE, force inclusion of X-chr-related covariates
   (like sex and cross direction).}
}

\details{
  The formula is used to specified the model to be fit. In the
  formula, use \code{Q1}, \code{Q2}, etc., or \code{q1},
  \code{q2}, etc., to represent the QTLs, and the column names in the
  covariate data frame to represent the covariates.

  We enforce a hierarchical structure on the model formula: if a QTL or
  covariate is in involved in an interaction, its main effect must also
  be included.

  If neither of the two new QTL are indicated in the \code{formula}, we
  perform a two-dimensional scan as in \code{\link{scantwo}}.  That
  is, for each pair of QTL positions, we fit two models: two
  additive QTL added to the \code{formula}, and two interacting QTL
  added to the \code{formula}.

  If the both of the new QTL are indicated in the \code{formula}, that
  particular model is fit, with the positions of the new QTL allowed to
  vary across the genome.  If just one of the QTL is indicated in the
  \code{formula}, a main effect for the other is added, and that
  particular model is fit, again with the positions of both QTL varying.
  Note that in this case the LOD scores are not analogous to those
  produced by \code{\link{scantwo}}.  Thus, there slightly modified forms
  for the plots (produced by \code{\link{plot.scantwo}}) and
  summaries (produced by \code{\link{summary.scantwo}} and
  \code{\link{max.scantwo}}).  In the plot, the x-axis is to be
  interpreted as the position of the first of the new QTL, and the
  y-axis is to be interpreted as the position of the second of the new
  QTL.  In the summaries, we give the single best pair of positions on
  each pair of chromosomes, and give LOD scores comparing that pair of
  positions to the base model (without each of these QTL), and to the
  base model plus one additional QTL on one or the other of the chromosomes.
}

\section{Value}{
  An object of class \code{scantwo}, as produced by
  \code{\link{scantwo}}.

  If neither of the new QTL were indicated
  in the \code{formula}, the result is just as in
  \code{\link{scantwo}}, though with LOD scores relative to the
  base model (omitting the new QTL).

  Otherwise, the results are contained in what would ordinarily be in
  the full and additive LOD scores, with the additive LOD scores
  corresponding to the case that the first of the new QTL is to the left
  of the second of the new QTL, and the full LOD scores corresponding to
  the case that the first of the new QTL is to the right of the second
  of the new QTL.  Because the structure of the LOD scores in this
  case is different from those output by \code{\link{scantwo}}, we
  include, in this case, an attribute \code{"addpair"=TRUE}.  (We also
  require results of single-dimensional scans, omitting each of the two
  new QTL from the formula, one at a time; these are included as
  attributes \code{"lod.minus1"} and \code{"lod.minus2"}.)  The
  results are then treated somewhat differently by
  \code{\link{summary.scantwo}}, \code{\link{max.scantwo}},
  and \code{\link{plot.scantwo}}.  See the Details section.
}

\references{
  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
# A totally contrived example to show some of what you can do

# simulate backcross data with 3 chromosomes (names "17", "18", "19")
#   one QTL on chr 17 at 40 cM
#   one QTL on chr 18 at 30 cM
#   two QTL on chr 19, at 10 and 40 cM
data(map10)
model <- rbind(c(1,40,0), c(2,30,0), c(3,10,0), c(3,40,0))
\dontrun{fakebc <- sim.cross(map10[17:19], model=model, type="bc", n.ind=250)}
\dontshow{fakebc <- sim.cross(map10[17:19], model=model, type="bc", n.ind=25)}

# het at QTL on 17 and 1st QTL on 19 increases phenotype by 1 unit
# het at QTL on 18 and 2nd QTL on 19 decreases phenotype by 1 unit
qtlgeno <- fakebc$qtlgeno
phe <- rnorm(nind(fakebc))
w <- qtlgeno[,1]==2 & qtlgeno[,3]==2
phe[w] <- phe[w] + 1
w <- qtlgeno[,2]==2 & qtlgeno[,4]==2
phe[w] <- phe[w] - 1
fakebc$pheno[,1] <- phe

\dontrun{fakebc <- calc.genoprob(fakebc, step=2, err=0.001)
}\dontshow{fakebc <- calc.genoprob(fakebc, step=0, err=0.001)}

# base model has QTLs on chr 17 and 18
qtl <- makeqtl(fakebc, chr=c("17", "18"), pos=c(40,30), what="prob")

# scan for an additional pair of QTL, one interacting with the locus
#     on 17 and one interacting with the locus on 18
out.ap <- addpair(fakebc, qtl=qtl, formula = y~Q1*Q3 + Q2*Q4, method="hk")

max(out.ap)
summary(out.ap)
plot(out.ap)
}

\seealso{ \code{\link{addint}}, \code{\link{addqtl}},
  \code{\link{fitqtl}}, \code{\link{makeqtl}},
  \code{\link{scanqtl}}, \code{\link{refineqtl}},
  \code{\link{makeqtl}}, \code{\link{scantwo}},
  \code{\link{addtoqtl}} }

\keyword{models}

\name{clean.scantwo}
\alias{clean.scantwo}

\title{Clean up scantwo output}

\description{
  In an object output from \code{\link{scantwo}}, replaces negative
  and missing LOD scores with 0, and replaces LOD scores for pairs of
  positions that are not separated by \code{n.mar} markers, or that are
  less than \code{distance} cM apart, with 0.  Further, if the LOD
  for full model is less than the LOD for the additive model, the
  additive LOD is pasted over the full LOD.
}

\usage{
\method{clean}{scantwo}(object, n.mar=1, distance=0, \dots)
}

\arguments{
 \item{object}{An object of class \code{scantwo}.  See
   \code{\link{scantwo}} for details.}
 \item{n.mar}{Pairs of positions not separated by at least this many
 markers have LOD scores set to 0.}
 \item{distance}{Pairs of positions not separated by at least this
 distance have LOD scores set to 0.}
 \item{\dots}{Ignored at this point.}
}

\value{
  The input scantwo object, with any negative or missing LOD scores
  replaced by 0, and LOD scores for pairs of positions separated by
  fewer than \code{n.mar} markers, or less than \code{distance} cM, are
  set to 0.
  Also, if the LOD for the full model is less than the LOD for the
  additive model, the additive LOD is used in place of the full LOD.
}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
fake.f2 <- calc.genoprob(fake.f2, step=5)
out2 <- scantwo(fake.f2, method="hk")
out2 <- clean(out2)
out2cl2 <- clean(out2, n.mar=2, distance=5)
}

\seealso{ \code{\link{scantwo}},
  \code{\link{summary.scantwo}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{manip}

\name{jittermap}
\alias{jittermap}

\title{Jitter marker positions in a genetic map}

\description{
  Jitter the marker positions in a genetic map so that no two markers
  are on top of each other.
}

\usage{jittermap(object, amount=1e-6)}

\arguments{
 \item{object}{Either a cross (an object of class \code{cross}; see
   \code{\link{read.cross}} for details) or a map (an object of
   class \code{map}; see \code{\link{pull.map}} for details).}
 \item{amount}{The amount by which markers should be moved.}
}

\value{
  Either the input cross object or the input map, but with marker
  positions slightly jittered.  If the input was a cross, the function
  \code{\link[=clean.cross]{clean}} is run to strip off any intermediate
  calculations.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
hyper <- jittermap(hyper)
}

\seealso{ \code{\link{pull.map}}, \code{\link{replace.map}},
  \code{\link{summary.cross}} }

\keyword{manip}

\name{drop.dupmarkers}
\alias{drop.dupmarkers}

\title{Drop duplicate markers}

\description{
  Drop markers with duplicate names; retaining the first of each set,
  with consensus genotyps
}

\usage{
drop.dupmarkers(cross, verbose=TRUE)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{verbose}{If TRUE, print information on the numbers of genotypes
 and markers omitted.  If > 1, give more detailed information
 on genotypes omitted.}
}

\value{
  The input \code{cross} object, with any duplicate markers omitted
  (except for one).  The marker retained will have consensus genotypes;
  if multiple versions of a marker have different genotypes for an
  individual, they will be replaced by \code{NA}.

  Any derived data (such as produced by \code{\link{calc.genoprob}})
  will be stripped off.
}

\examples{
data(listeria)

listeria <- drop.dupmarkers(listeria)
}

\seealso{ \code{\link{drop.nullmarkers}}, \code{\link{pull.markers}}, \code{\link{drop.markers}},
  \code{\link{summary.cross}}, \code{\link{clean.cross}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{manip}

\name{summaryMap}
\alias{summary.map}
\alias{summaryMap}

\title{Print summary of a genetic map}

\description{
  Print summary information about a \code{map} object.
}

\usage{
\method{summary}{map}(object, \dots)
summaryMap(object, \dots)
}

\arguments{
 \item{object}{An object of class \code{map}, which is a list of
   vectors (or, for a sex-specific map, 2-row matrices), each specifying
   the locations of the markers.  The object can also be of class
   \code{cross}, in which case the function \code{\link{pull.map}}
   is used to extract the genetic map from the object.}
 \item{\dots}{Ignored at this point.}
}

\value{
  An object of class \code{summary.map}, which is just a data.frame
  containing the number of markers, length, the average inter-marker
  spacing, and the maximum distance between markers, for each chromosome
  and overall.  An attribute \code{sexsp} indicates whether the map was
  sex-specific.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(map10)
summary(map10)
}

\seealso{ \code{\link{chrlen}}, \code{\link{pull.map}},
  \code{\link{summary.cross}} }

\keyword{print}

\name{fitqtl}
\alias{fitqtl}

\title{Fit a multiple-QTL model}

\description{
  Fits a user-specified multiple-QTL model.  If specified,
  a drop-one-term analysis will be performed.
}

\usage{
fitqtl(cross, pheno.col=1, qtl, covar=NULL, formula, method=c("imp", "hk"),
       model=c("normal", "binary"), dropone=TRUE, get.ests=FALSE,
       run.checks=TRUE, tol=1e-4, maxit=1000, forceXcovar=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  One may also give a character string matching
   a phenotype name. Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{qtl}{An object of class \code{qtl}, as output from
   \code{\link{makeqtl}}.}
 \item{covar}{A matrix or data.frame of covariates.  These must be
   strictly numeric.}
 \item{formula}{An object of class \code{\link[stats]{formula}}
   indicating the model to be fitted.  (It can also be the character
   string representation of a formula.)  QTLs are referred to as
   \code{Q1}, \code{Q2}, etc.  Covariates are referred to by their names
   in the data frame \code{covar}.  }
 \item{method}{Indicates whether to use multiple imputation or
   Haley-Knott regression.}
 \item{model}{The phenotype model: the usual model or a model for binary
  traits}
 \item{dropone}{If TRUE, do drop-one-term analysis.}
 \item{get.ests}{If TRUE, return estimated QTL effects and their
   estimated variance-covariance matrix.}
 \item{run.checks}{If TRUE, check the input formula and check for
   individuals with missing phenotypes or covariates.}
 \item{tol}{Tolerance for convergence for the binary trait model.}
 \item{maxit}{Maximum number of iterations for fitting the binary trait
   model.}
 \item{forceXcovar}{If TRUE, force inclusion of X-chr-related covariates
   (like sex and cross direction).}
}

\details{
  The formula is used to specified the model to be fit. In the
  formula, use \code{Q1}, \code{Q2}, etc., or \code{q1},
  \code{q2}, etc., to represent the QTLs, and the column names in the
  covariate data frame to represent the covariates.

  We enforce a hierarchical structure on the model formula: if a QTL or
  covariate is in involved in an interaction, its main effect must also
  be included.

  In the drop-one-term analysis, for a given QTL/covariate model, all
  submodels will be analyzed.  For each term in the input formula, when
  it is dropped, all higher order terms that contain it will also be
  dropped.  The comparison between the new model and the full (input)
  model will be output.

  The estimated percent variances explained for the QTL are simplify
  transformations of the conditional LOD scores by the formula \eqn{h^2
  = 1 - 10^{-(2/n) {\rm LOD}}}.  While these may be reasonable for
  unlinked, additive QTL, \bold{they can be completely wrong in the case
  of linked QTL}, but we don't currently have any alternative.

  For \code{model="binary"}, a logistic regression model is used.

  \bold{The part to get estimated QTL effects is not complete for the
    case of the X chromosome and 4-way crosses.  The values returned in
    these cases are based on a design matrix that is convenient for
    calculations but not easily interpreted.}

  The estimated QTL effects for a backcross are derived by the coding
  scheme \eqn{\pm}{+/-} 1/2 for AA and AB, so that the additive
  effect corresponds to the difference between phenotype averages for
  the two genotypes.  For doubled haploids and RIL, the coding scheme is
  \eqn{\pm}{+/-} 1 for AA and BB, so that the additive effect
  corresponds to half the difference between the phenotype averages for
  the two homozygotes.

  For an intercross, the additive effect is derived from the coding
  scheme -1/0/+1 for genotypes AA/AB/BB, and so is half the difference
  between the phenotype averages for the two homozygotes.  The dominance
  deviation is derived from the coding scheme 0/+1/0 for genotypes
  AA/AB/BB, and so is the difference between the phenotype average for
  the heterozygotes and the midpoint between the phenotype averages for
  the two homozygotes.

  Epistatic effects and QTL \eqn{\times}{x} covariate interaction
  effects are obtained through the products of the corresponding
  additive/dominant effect columns.
}

\value{
  An object of class \code{fitqtl}. It may contains as many as four components:
  \itemize{
    \item \code{result.full} is the ANOVA table as a matrix for the full model
    result. It contains the degree of freedom (df), Sum of squares (SS),
    mean square (MS), LOD score (LOD), percentage of variance explained
    (\%var) and P value (Pvalue).

    \item \code{lod} is the LOD score from the fit of the full model.

    \item \code{result.drop} is a drop-one-term ANOVA table as a
    matrix. It contains degrees of freedom (df), Type III sum of squares
    (Type III SS), LOD score(LOD), percentage of variance explained
    (\%var), F statistics (F value), and P values for chi square
    (Pvalue(chi2)) and F distribution (Pvalue(F)).  Note that the degree
    of freedom, Type III sum of squares, the LOD score and the
    percentage of variance explained are the values comparing the full
    to the sub-model with the term dropped. Also note that for
    imputation method, the percentage of variance explained, the the F
    values and the P values are approximations calculated from the LOD
    score.

    \item \code{ests} contains the estimated QTL effects and standard errors.
  }

  When \code{method="normal"}, residuals are saved as an attribute of
  the output, named \code{"residuals"} and accessible via the
  \code{\link[base]{attr}} function.


  \bold{The part to get estimated QTL effects is fully working only for
    the case of autosomes in a backcross, intercross, RIL or doubled haploids.
    In other cases the values returned are based on a
    design matrix that is convenient for calculations but not easily
    interpreted.}
}


\references{
  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.
}

\author{Hao Wu; Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 8, 13)
qp <- c(26, 56, 28)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

# fit model with 3 interacting QTLs interacting
# (performing a drop-one-term analysis)
lod <- fitqtl(fake.f2, pheno.col=1, qtl, formula=y~Q1*Q2*Q3, method="hk")
summary(lod)

\dontrun{
# fit an additive QTL model
lod.add <- fitqtl(fake.f2, pheno.col=1, qtl, formula=y~Q1+Q2+Q3, method="hk")
summary(lod.add)

# fit the model including sex as an interacting covariate
Sex <- data.frame(Sex=pull.pheno(fake.f2, "sex"))
lod.sex <- fitqtl(fake.f2, pheno.col=1, qtl, formula=y~Q1*Q2*Q3*Sex,
                  cov=Sex, method="hk")
summary(lod.sex)

# fit the same with an additive model
lod.sex.add <- fitqtl(fake.f2, pheno.col=1, qtl, formula=y~Q1+Q2+Q3+Sex,
                      cov=Sex, method="hk")
summary(lod.sex.add)

# residuals
residuals <- attr(lod.sex.add, "residuals")
plot(residuals)
}}

\seealso{ \code{\link{summary.fitqtl}}, \code{\link{makeqtl}},
\code{\link{scanqtl}}, \code{\link{refineqtl}},
  \code{\link{addtoqtl}} ,
  \code{\link{dropfromqtl}},
  \code{\link{replaceqtl}},
  \code{\link{reorderqtl}}}

\keyword{models}

\name{est.rf}
\alias{est.rf}

\title{Estimate pairwise recombination fractions}

\description{
  Estimate the sex-averaged recombination fraction between all pairs of
  genetic markers.
}

\usage{
est.rf(cross, maxit=10000, tol=1e-6)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{maxit}{Maximum number of iterations for the EM algorithm (not
   used with backcrosses).}
 \item{tol}{Tolerance for determining convergence (not used with
   backcrosses).}
}

\value{
  The input \code{cross} object is returned with a component, \code{rf},
  added.  This is a matrix of size (tot.mar x tot.mar).  The diagonal
  contains the number of typed meioses per marker, the lower triangle
  contains the estimated recombination fractions, and the upper triangle
  contains the LOD scores (testing rf = 0.5).
}

\details{
  For a backcross, one can simply count recombination events.  For an
  intercross or 4-way cross, a version of the EM algorithm must be used
  to estimate recombination fractions. (Since, for example, in an
  intercross individual that is heterozygous at two loci, it is not
  known whether there were 0 or 2 recombination events.)  Note that, for
  the 4-way cross, we estimate sex-averaged recombination fractions.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(badorder)
badorder <- est.rf(badorder)
plotRF(badorder)
}

\seealso{ \code{\link{plotRF}}, \code{\link{pull.rf}}, \code{\link{plot.rfmatrix}},
  \code{\link{est.map}},
  \code{\link{badorder}}, \code{\link{checkAlleles}} }

\keyword{utilities}

\name{replaceqtl}
\alias{replaceqtl}

\title{Replace a QTL in a qtl object with a different position}

\description{
  This function replaces a QTL or QTLs in a qtl object with a different
  position.
}

\usage{
replaceqtl(cross, qtl, index, chr, pos, qtl.name, drop.lod.profile=TRUE)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{qtl}{A qtl object, as created by \code{\link{makeqtl}}.}
 \item{index}{Numeric index indicating the QTL to be replaced.}
 \item{chr}{Vector (of same length as \code{index}) indicating the
   chromosomes for the new QTL.}
 \item{pos}{Vector (of same length as \code{index}) indicating the
   positions for the new QTL. If there is no marker or pseudomarker at a
 position, the nearest position is used.}
 \item{qtl.name}{Optional vector (of same length as \code{index})
   of user-specified names for each new QTL, used in the
   drop-one-term ANOVA table in \code{\link{fitqtl}}.
   If unspecified, the names will be of the form \code{"Chr1@10"} for a
   QTL on Chromsome 1 at 10 cM.}
 \item{drop.lod.profile}{If TRUE, remove any LOD profiles from the object.}
}


\value{
  The input \code{qtl} object,
  but with some QTL replaced by new ones.  See \code{\link{makeqtl}} for
  details on the format.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 6, 13)
qp <- c(25.8, 33.6, 18.63)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

qtl <- replaceqtl(fake.f2, qtl, 2, 6, 48.1)
}

\seealso{ \code{\link{makeqtl}}, \code{\link{fitqtl}},
  \code{\link{dropfromqtl}}, \code{\link{addtoqtl}},
  \code{\link{reorderqtl}} }

\keyword{utilities}

\name{nmissing}
\alias{nmissing}

\title{Number of missing genotypes}

\description{
  Count the number of missing genotypes for each individual or each marker
  in a cross.
}

\usage{nmissing(cross, what=c("ind","mar"))}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{what}{Indicates whether to count missing genotypes for each
   individual or each marker.}
}

\value{
  A vector containing the number of missing genotypes for each
  individual or for each marker.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)

# plot number of missing genotypes for each individual
plot(nmissing(listeria))

# plot number of missing genotypes for each marker
plot(nmissing(listeria, what="mar"))
}

\seealso{ \code{\link{ntyped}}, \code{\link{summary.cross}},
          \code{\link{nind}}, \code{\link{totmar}} }

\keyword{utilities}

\name{plot.cross}
\alias{plot.cross}

\title{Plot various features of a cross object}

\description{
  Plots grid of the missing genotypes, genetic map, and histograms or
  barplots of phenotypes for the data from an experimental cross.
}

\usage{
\method{plot}{cross}(x, auto.layout=TRUE, pheno.col,
          alternate.chrid=TRUE, \dots)
}

\arguments{
 \item{x}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{auto.layout}{If TRUE, \code{par(mfrow)} is set so that all plots
   fit within one figure.}
 \item{pheno.col}{Vector of numbers or character strings corresponding to
   phenotypes that should be plotted. If unspecified, all phenotypes are
   plotted.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
 \item{\dots}{Ignored at this point.}
}

\value{None.}

\details{
  Calls \code{\link{plotMissing}}, \code{\link{plotMap}} and
  \code{\link{plotPheno}} to plot the missing genotypes, genetic
  map, and histograms or barplots of all phenotypes.

  If \code{auto.format=TRUE}, \code{par(mfrow)} is used with
  \code{ceiling(sqrt(n.phe+2))} rows and the minimum number of columns
  so that all plots fit on the plotting device.

  Numeric phenotypes are displayed as histograms or barplots by calling
  \code{\link{plotPheno}}.
}

\examples{
data(fake.bc)
plot(fake.bc)
}

\seealso{ \code{\link{plotMissing}}, \code{\link{plotMap}},
  \code{\link{plotPheno}} }


\author{Karl W Broman, \email{broman@wisc.edu}; Brian Yandell }

\keyword{hplot}

\name{map2table}
\alias{map2table}

\title{Convert genetic map from list to table.}

\description{
  Convert a map object (as a list) to a table (as a data frame).
}

\usage{map2table(map, chr)}

\arguments{
 \item{map}{A \code{map} object: a list whose components (corresponding
 to chromosomes) are either vectors of marker positions or matrices with
 two rows of sex-specific marker positions.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
}

\value{
  A data frame with two or three columns: chromosome and sex-averaged
  position, or chromosome, female position, and male position.

  The row names are the marker names.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
map <- pull.map(fake.f2)
map_as_tab <- map2table(map)
}

\seealso{ \code{\link{table2map}}, \code{\link{pull.map}}, \code{\link{est.map}} }


\keyword{utilities}

\name{pull.map}
\alias{pull.map}

\title{Pull out the genetic map from a cross}

\description{
  Pull out the map portion of a cross object.
}

\usage{pull.map(cross, chr, as.table=FALSE)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{as.table}{If TRUE, return the genetic map as a table with
 chromosome assignments and marker names.  If FALSE, return the map as a
 \code{"map"} object.}
}

\value{
  The genetic map: a list with each component containing the marker
  positions (in cM) for a chromosome.  Each component has class
  \code{A} or \code{X} according to whether it is an autosome or the X
  chromosome.  The components are either vectors of marker positions or,
  for a sex-specific map, 2-row matrices containing the female and male
  marker locations.  The map itself is given class \code{map}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
map <- pull.map(fake.f2)
plot(map)
}

\seealso{ \code{\link{replace.map}}, \code{\link{plotMap}}, \code{\link{map2table}} }


\keyword{utilities}

\name{summary.scanone}
\alias{summary.scanone}

\title{Summarize the results of a genome scans}

\description{
  Print the rows of the output from \code{\link{scanone}} that
  correspond to the maximum LOD for each chromosome, provided that they
  exceed some specified thresholds.
}

\usage{
\method{summary}{scanone}(object, threshold,
        format=c("onepheno", "allpheno", "allpeaks", "tabByCol", "tabByChr"),
        perms, alpha, lodcolumn=1, pvalues=FALSE,
        ci.function=c("lodint", "bayesint"), \dots)
}

\arguments{
 \item{object}{An object output by the function
   \code{\link{scanone}}.}
 \item{threshold}{LOD score thresholds. Only peaks with LOD score above
   this value will be returned. This could be a single number or (for
   formats other than \code{"onepheno"}) a threshold for each LOD score
   column.  If \code{alpha} is specified, \code{threshold} should not be.}
 \item{format}{Format for the output.  See Details, below.}
 \item{perms}{Optional permutation results used to derive thresholds or
   to calculate genome-scan-adjusted p-values.  This must be consistent
   with the \code{object} input, in that it must have the same number of
   LOD score columns, though it can have just one column of permutation
   results, in which case they are reused for all LOD score columns in
   the \code{\link{scanone}} output, \code{object}.
   (These can also be permutation results from
   \code{\link{scantwo}}, which permutations for a one-dimensional scan.)}
 \item{alpha}{If perms are included, this is the significance level used
   to calculate thresholds for determining which peaks to pull out.
   If \code{threshold} is specified, \code{alpha} should not be.}
 \item{lodcolumn}{If \code{format="onepheno"}, this indicates the LOD
   score column to focus on.  This should be a single number between 1
   and the number of LOD columns in the object input.}
 \item{pvalues}{If TRUE, include columns with genome-scan-adjusted
   p-values in the results.  This requires that \code{perms} be
   provided.}
 \item{ci.function}{For formats \code{"tabByCol"} and \code{"tabByChr"},
   indicates the function to use to get approximate confidence intervals
   for QTL location.}
 \item{\dots}{For formats \code{"tabByCol"} and \code{"tabByChr"},
   additional arguments are passed to the function indicated by
   \code{ci.function} (for example, \code{drop} for \code{\link{lodint}}
   or \code{prob} for \code{\link{bayesint}}, or \code{expandtomarkers}
   for either).}
}

\details{
  This function is used to report loci deemed interesting from a one-QTL
  genome scan (by \code{\link{scanone}}).

  For \code{format="onepheno"}, we focus on a single LOD score column,
  indicated by \code{lodcolumn}.  The single largest LOD score peak on
  each chromosome is extracted.  If \code{threshold} is specified, only
  those peaks with LOD meeting the threshold will be
  returned.  If \code{perms} and \code{alpha} are specified, a threshold
  is calculated based on the permutation results in \code{perms} for the
  significance level \code{alpha}.  If neither \code{threshold} nor
  \code{alpha} are specified, the peak on each chromosome is returned.
  Again note that with this format, only the LOD score column indicated
  by \code{lodcolumn} is considered in deciding which chromosomes to
  return, but the LOD scores from other columns, at the position with
  maximum LOD score in the \code{lodcolumn} column, are also returned.

  For \code{format="allpheno"}, we consider all LOD score columns, and
  pull out the position, on each chromosome, showing the largest LOD
  score.  The output thus may contain multiple rows for a chromosome.
  Here \code{threshold} may be a vector of LOD score thresholds, one for
  each LOD score column, in which case only those positions for which a
  LOD score column exceeded its threshold are given.  If
  \code{threshold} is a single number, it is applied to all of the LOD
  score columns.  If \code{alpha} is specified, it must   be a single
  significance level, applied for all LOD score columns, and again
  \code{perms} must be specified, and these are used to calculate the
  LOD score threshold for the significance level \code{alpha}.

  For \code{format="allpeaks"}, the output will contain, for each
  chromosome, the maximum LOD score for each LOD score column, at the
  position at which it achieved its maximum.  Thus, the output will
  contain no more than one row per chromosome, but will contain the
  position and maximum LOD score for each of the LOD score columns.
  The arguments \code{threshold} and \code{alpha} may be specified as
  for the \code{"allpheno"} format.  The results for a chromosome are
  returned if at least one of the LOD score columns exceeded its
  threshold.

  For \code{format="tabByCol"}, there will be a separate table for each
  LOD score column, with a single peak per chromosome.  Included are
  columns indicating chromosome, peak position, lower and upper limits
  of the confidence interval calculated via \code{\link{lodint}} or
  \code{\link{bayesint}}, and lod score.

  The output for \code{format="tabByChr"}, is similar to that of
  \code{format="tabByCol"}, but with results organized by chromosome
  rather than by LOD score column.

  If \code{pvalues=TRUE}, and \code{perms} is specified,
  genome-scan-adjusted p-values are calculated for each LOD score
  column, and there are additional columns in the output containing
  these p-values.

  In the case that X-chromosome specific permutations were performed
  (with \code{perm.Xsp=TRUE} in \code{\link{scanone}}), autosome-
  and X-chromosome specific thresholds and p-values are calculated by
  the method in Broman et al. (2006).
}

\value{
  An object of class \code{summary.scanone}, to be printed by
  \code{print.summary.scanone}.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
\dontshow{set.seed(70377314)}
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=c(2,5,6))}
fake.bc <- calc.genoprob(fake.bc, step=5)

# genome scan by Haley-Knott regression
out <- scanone(fake.bc, method="hk")

# permutation tests
\dontrun{operm <- scanone(fake.bc, method="hk", n.perm=1000)
}\dontshow{operm <- scanone(fake.bc, method="hk", n.perm=100)
}

# peaks for all chromosomes
summary(out)

# results with LOD >= 3
summary(out, threshold=3)

# the same, but also showing the p-values
summary(out, threshold=3, perms=operm, pvalues=TRUE)

# results with LOD meeting the 0.05 threshold from the permutation results
summary(out, perms=operm, alpha=0.05)

# the same, also showing the p-values
summary(out, perms=operm, alpha=0.05, pvalues=TRUE)

##### summary with multiple phenotype results
out2 <- scanone(fake.bc, pheno.col=1:2, method="hk")

# permutations
\dontrun{operm2 <- scanone(fake.bc, pheno.col=1:2, method="hk", n.perm=1000)
}\dontshow{operm2 <- scanone(fake.bc, pheno.col=1:2, method="hk", n.perm=100)
}

# results with LOD >= 2 for the 1st phenotype and >= 1 for the 2nd phenotype
#     using format="allpheno"
summary(out2, thr=c(2, 1), format="allpheno")

# The same with format="allpeaks"
summary(out2, thr=c(2, 1), format="allpeaks")

# The same with p-values
summary(out2, thr=c(2, 1), format="allpeaks", perms=operm2, pvalues=TRUE)

# results with LOD meeting the 0.05 significance level by the permutations
#     using format="allpheno"
summary(out2, format="allpheno", perms=operm2, alpha=0.05)

# The same with p-values
summary(out2, format="allpheno", perms=operm2, alpha=0.05, pvalues=TRUE)

# The same with format="allpeaks"
summary(out2, format="allpeaks", perms=operm2, alpha=0.05, pvalues=TRUE)

# format="tabByCol"
summary(out2, format="tabByCol", perms=operm2, alpha=0.05, pvalues=TRUE)

# format="tabByChr", but using bayes intervals
summary(out2, format="tabByChr", perms=operm2, alpha=0.05, pvalues=TRUE,
        ci.function="bayesint")

# format="tabByChr", but using 99% bayes intervals
summary(out2, format="tabByChr", perms=operm2, alpha=0.05, pvalues=TRUE,
        ci.function="bayesint", prob=0.99)
}

\seealso{ \code{\link{scanone}}, \code{\link{plot.scanone}},
  \code{\link{max.scanone}}, \code{\link{subset.scanone}},
  \code{\link{c.scanone}}, \code{\link{summary.scanoneperm}}
  \code{\link{c.scanoneperm}} }

\references{
  Broman, K. W., Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}, Owens, S. E., Manichaikul, A.,
  Southard-Smith, E. M. and Churchill G. A. (2006)  The X chromosome in
  quantitative trait locus mapping.  \emph{Genetics}, \bold{174}, 2151--2158.
}


\keyword{print}

\name{convert2risib}
\alias{convert2risib}

\title{Convert a cross to RIL by sib mating}

\description{
  Convert a cross to type \code{"risib"} (RIL by sib mating).
}

\usage{
convert2risib(cross)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
}

\value{
  The input cross object, with genotype codes possibly changed and cross
  type changed to \code{"risib"}.
}

\details{
  If there are more genotypes with code 3 (BB) than code 2 (AB), we omit
  the genotypes with code==2 and call those with code==3 the BB
  genotypes.

  If, instead, there are more genotypes with code 2 than code 3, we omit
  the genotypes with code==3 and call those with code==2 the BB genotypes.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
hyper.as.risib <- convert2risib(hyper)
}

\seealso{ \code{\link{convert2riself}} }

\keyword{manip}

\name{cleanGeno}
\alias{cleanGeno}

\title{Delete genotypes that are possibly in error}

\description{
  Delete genotypes from a cross that are indicated to be possibly in
  error, as they result in apparent tight double-crossovers.
}

\usage{
cleanGeno(cross, chr, maxdist=2.5, maxmark=2, verbose=TRUE)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
   This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
 \item{maxdist}{A vector specifying the maximum distance between two
 crossovers.}
 \item{maxmark}{A vector specifying the maximum number of typed markers
 between two crossovers.}
 \item{verbose}{If TRUE, print information on the numbers of genotypes
 omitted from each chromosome.}

}

\details{
  We first use \code{\link{locateXO}} to identify crossover locations.
  If a pair of adjacted crossovers are separated by no more than
  \code{maxdist} and contain no more than \code{maxmark} genotyped
  markers, the intervening genotypes are omitted (that is, changed to
  \code{NA}).

  The arguments \code{maxdist} and \code{maxmark} may be vectors.  (If
  both have length greater than 1, they must have the same length.)  If
  they are vectors, genotypes are omitted if they satisify any one of
  the (\code{maxdist}, \code{maxmark}) pairs.
}

\value{
  The input \code{cross} object with suspect genotypes omitted.
}

\examples{
data(hyper)
sum(ntyped(hyper))
hyperc <- cleanGeno(hyper, chr=4, maxdist=c(2.5, 10), maxmark=c(2, 1))
sum(ntyped(hyperc))
}

\seealso{ \code{\link{locateXO}},
          \code{\link{countXO}}, \code{\link{calc.errorlod}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{utilities}

\name{find.pheno}
\alias{find.pheno}

\title{Find column number for a particular phenotype}

\description{
  Find the column number corresponding to a particular
  phenotype name.
}

\usage{find.pheno(cross, pheno)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{pheno}{Vector of phenotype names (as character strings).}
}

\value{
  A vector of numbers, corresponding to the column numbers of the
  phenotype in the input cross with the specified names.
}

%\details{
%}

\author{Brian Yandell}

\examples{
data(fake.bc)
find.pheno(fake.bc, "sex")
}

%\seealso{ }

\keyword{utilities}

\name{sim.map}
\alias{sim.map}

\title{Simulate a genetic map}

\description{
  Simulate the positions of markers on a genetic map.
}

\usage{
sim.map(len=rep(100,20), n.mar=10, anchor.tel=TRUE,
        include.x=TRUE, sex.sp=FALSE, eq.spacing=FALSE)
}

\arguments{
 \item{len}{A vector specifying the chromosome lengths (in cM)}
 \item{n.mar}{A vector specifying the number of markers per chromosome.}
 \item{anchor.tel}{If true, markers at the two telomeres will always be
   included, so if \code{n.mar} = 1 or 2, we'll give just the two
   telomeric markers.}
 \item{include.x}{Indicates whether the last chromosome should be
   considered the X chromosome.}
 \item{sex.sp}{Indicates whether to create sex-specific maps, in which
   case the output will be a vector of 2-row matrices, with rows
   corresponding to the maps for the two sexes.}
 \item{eq.spacing}{If TRUE, markers will be equally spaced.}
}

\details{
  Aside from the telomeric markers, marker positions are simulated as
  iid Uniform(\eqn{0,L}).  If \code{len} or \code{n.mar} has just one element,
  it is expanded to the length of the other argument.  If they both have
  just one element, only one chromosome is simulated.

  If \code{eq.spacing} is TRUE, markers are equally spaced between 0 and
  \eqn{L}.  If \code{anchor.tel} is FALSE, telomeric markers are not
  included.
}

\value{
  A list of vectors, each specifying the locations of the markers.  Each
  component of the list is given class \code{A} or \code{X}, according
  to whether it is autosomal or the X chromosome.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
# simulate 4 autosomes, each with 10 markers
map <- sim.map(c(100,90,80,40), 10, include.x=FALSE)
plotMap(map)

# equally spaced markers
map2 <- sim.map(c(100,90,80,40), 10, include.x=FALSE, eq.spacing=TRUE)
plot(map2)
}

\seealso{ \code{\link{sim.cross}}, \code{\link{plotMap}},
  \code{\link{replace.map}}, \code{\link{pull.map}} }

\keyword{datagen}

\name{movemarker}
\alias{movemarker}
\title{Move a marker to a new chromosome}

\description{
  Move a specified marker to a different chromosome.
}

\usage{movemarker(cross, marker, newchr, newpos)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{marker}{The name of the marker to be moved (a character string).}
 \item{newchr}{The chromosome to which the marker should be moved.}
 \item{newpos}{The position (in cM) at which the marker should be
   placed.  If missing, the marker is placed at the end of the
   chromosome.}
}

\value{
  The input \code{cross} object, but with the specified marker moved to
  the specified chromosome.

  All intermediate calculations (such as the results of
  \code{\link{calc.genoprob}} and \code{\link{est.rf}}) are
  removed.
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(badorder)
badorder <- movemarker(badorder, "D2M937", 3, 48.15)
badorder <- movemarker(badorder, "D3M160", 2, 28.83)
}

\seealso{ \code{\link{switch.order}} }

\keyword{manip}

\name{effectscan}
\alias{effectscan}

\title{Plot estimated QTL effects across the whole genome}

\description{
  This function is used to plot the estimated QTL effects along
  selected chromosomes. For a backcross, there will be only one line,
  representing the additive effect. For an intercross, there will be two
  lines, representing the additive and dominance effects.
}

\usage{
effectscan(cross, pheno.col=1, chr, get.se=FALSE, draw=TRUE,
           gap=25, ylim, mtick=c("line","triangle"),
           add.legend=TRUE, alternate.chrid=FALSE, \dots)
}

\arguments{
  \item{cross}{An object of class \code{cross}.}
  \item{pheno.col}{Column number in the phenotype matrix which to be
    drawn in the plot.  One may also give a character string matching
   a phenotype name.}
  \item{chr}{Optional vector indicating the chromosomes to be drawn in
  the plot. This should be a vector of character strings referring to
  chromosomes by name; numeric values are converted to strings.  Refer
  to chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{get.se}{If TRUE, estimated standard errors are calculated.}
  \item{draw}{If TRUE, draw the figure.}
  \item{gap}{Gap separating chromosomes (in cM).}
  \item{ylim}{Y-axis limits (optional).}
  \item{mtick}{Tick mark type for markers.}
  \item{add.legend}{If TRUE, add a legend.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{\dots}{Passed to the function \code{plot} when it
    is called.}
}
\value{
  The results are returned silently, as an object of class
  \code{"effectscan"}, which is the same as the form returned by the
  function \code{\link{scanone}}, though with estimated effects
  where LOD scores might be.  That is, it is a data frame with the first
  two columns being chromosome ID and position (in cM), and subsequent
  columns being estimated effects, and (if \code{get.se=TRUE}) standard
  errors.
}

\details{
  The results of \code{\link{sim.geno}} are required for taking
  account of missing genotype information.

  For a backcross, the additive effect is estimated as the difference
  between the phenotypic averages for heterozygotes and homozygotes.

  For recombinant inbred lines, the additive effect is estimated as half
  the difference between the phenotypic averages for the two homozygotes.

  For an intercross, the additive and dominance effects are estimated
  from linear regression on \eqn{a} and \eqn{d} with \eqn{a} = -1, 0, 1,
  for the AA, AB and BB genotypes, respectively, and \eqn{d} = 0, 1, 0,
  for the AA, AB and BB genotypes, respectively.

  As usual, the X chromosome is a bit more complicated.  We estimate
  separate additive effects for the two sexes, and for the two
  directions within females.

  There is an internal function \code{plot.effectscan} that creates
  the actual plot by calling \code{plot.scanone}.  In the case
  \code{get.se=TRUE}, colored regions indicate \eqn{\pm}{+/-} 1 SE.
}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(1,13,"X"))}
fake.f2 <- sim.geno(fake.f2, step=2.5, n.draws=16)

# allelic effect on whole genome
effectscan(fake.f2)

# on chromosome 13, include standard errors
effectscan(fake.f2, chr="13", mtick="triangle", get.se=TRUE)
}

\references{
  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.
}

\author{Karl W. Broman, \email{broman@wisc.edu}}

\seealso{ \code{\link{effectplot}}, \code{\link{plotPXG}},
  \code{\link{sim.geno}} }

\keyword{hplot}

\name{find.flanking}
\alias{find.flanking}

\title{Find flanking markers for a specified position}

\description{
  Find the genetic markers flanking a specified position on a
  chromosome, as well as the marker that is closest to the specified
  position.
}

\usage{find.flanking(cross, chr, pos)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{A vector of chromosome identifiers, or a single such.}
 \item{pos}{A vector of cM positions.}
}

\value{
  A data.frame, each row corresponding to one of the input positions.
  The first column contains the left-flanking markers, the second column
  contains the right-flanking markers, and the third column contains the
  markers closest to the specified positions.
}

%\details{
%}

\author{Brian Yandell}

\examples{
data(listeria)
find.flanking(listeria, 5, 28)
find.flanking(listeria, c(1, 5, 13), c(81, 28, 26))
}

\seealso{ \code{\link{find.marker}}, \code{\link{plotPXG}},
\code{\link{find.markerpos}}, \code{\link{find.pseudomarker}} }

\keyword{utilities}

\name{addint}
\alias{addint}

\title{Add pairwise interaction to a multiple-QTL model}

\description{
  Try adding all possible pairwise interactions, one at a time, to a
  multiple QTL model.
}

\usage{
addint(cross, pheno.col=1, qtl, covar=NULL, formula, method=c("imp","hk"),
       model=c("normal", "binary"), qtl.only=FALSE, verbose=TRUE,
       pvalues=TRUE, simple=FALSE, tol=1e-4, maxit=1000, require.fullrank=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{Column number in the phenotype matrix to be
   used as the phenotype.  One may also give a character string matching
   a phenotype name. Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{qtl}{An object of class \code{qtl}, as output from
   \code{\link{makeqtl}}.}
 \item{covar}{A matrix or data.frame of covariates.  These must be
   strictly numeric.}
 \item{formula}{An object of class \code{\link[stats]{formula}}
   indicating the model to be fitted.  (It can also be the character
   string representation of a formula.)  QTLs are referred to as
   \code{Q1}, \code{Q2}, etc.  Covariates are referred to by their names
   in the data frame \code{covar}.  If the new QTL is not included in
   the formula, its main effect is added.}
 \item{method}{Indicates whether to use multiple imputation or
   Haley-Knott regression.}
 \item{model}{The phenotype model: the usual model or a model for binary
  traits}
 \item{qtl.only}{If TRUE, only test QTL:QTL interactions (and not
   interactions with covariates).}
 \item{verbose}{If TRUE, will print a message if there are no
   interactions to test.}
 \item{pvalues}{If FALSE, p-values will not be included in the results.}
 \item{simple}{If TRUE, don't include p-values or sums of squares in the summary.}
 \item{tol}{Tolerance for convergence for the binary trait model.}
 \item{maxit}{Maximum number of iterations for fitting the binary trait
   model.}
 \item{require.fullrank}{If TRUE, give LOD=0 when covariate matrix in
   the linear regression is not of full rank.}
}

\details{
  The formula is used to specified the model to be fit. In the
  formula, use \code{Q1}, \code{Q2}, etc., or \code{q1},
  \code{q2}, etc., to represent the QTLs, and the column names in the
  covariate data frame to represent the covariates.

  We enforce a hierarchical structure on the model formula: if a QTL or
  covariate is in involved in an interaction, its main effect must also
  be included.

}

\section{Value}{
  An object of class \code{addint}, with results as in the
  drop-one-term analysis from \code{\link{fitqtl}}.  This is a data
  frame (given class \code{"addint"}, with the following columns:
  degrees of freedom (df), Type III sum of squares (Type III
  SS), LOD score(LOD), percentage of variance explained (\%var), F
  statistics (F value),  and P values for chi square (Pvalue(chi2))
  and F distribution (Pvalue(F)).

  Note that the degree of freedom, Type III sum of squares, the LOD
  score and the percentage of variance explained are the values
  comparing the full to the sub-model with the term dropped. Also note
  that for imputation method, the percentage of variance explained, the
  the F values and the P values are approximations calculated from the
  LOD score.

  Pairwise interactions already included in the input \code{formula} are
  not tested.
}

\references{
  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 8, 13)
qp <- c(26, 56, 28)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

# try all possible pairwise interactions, one at a time
addint(fake.f2, pheno.col=1, qtl, formula=y~Q1+Q2+Q3, method="hk")
}

\seealso{ \code{\link{addcovarint}}, \code{\link{fitqtl}}, \code{\link{makeqtl}},
  \code{\link{scanqtl}}, \code{\link{refineqtl}},
  \code{\link{addqtl}}, \code{\link{addpair}} }

\keyword{models}

\name{refineqtl}
\alias{refineqtl}

\title{Refine the positions of QTL}

\description{
  Iteratively scan the positions for QTL in the context of a multiple
  QTL model, to try to identify the positions with maximum likelihood,
  for a fixed QTL model.
}

\usage{
refineqtl(cross, pheno.col=1, qtl, chr, pos, qtl.name, covar=NULL, formula,
          method=c("imp","hk"), model=c("normal", "binary"), verbose=TRUE, maxit=10,
          incl.markers=TRUE, keeplodprofile=TRUE, tol=1e-4,
          maxit.fitqtl=1000, forceXcovar=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{Column number in the phenotype matrix to be
   used as the phenotype.  One may also give a character string matching
   the phenotype name.  Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{qtl}{A QTL object, as produced by \code{\link{makeqtl}},
   containing the positions of the QTL.  Provide either \code{qtl} or
   the pair \code{chr} and \code{pos}.}
 \item{chr}{Vector indicating the chromosome for each QTL; if \code{qtl}
 is provided, this should not be.}
 \item{pos}{Vector indicating the positions for each QTL; if \code{qtl}
   is provided, this should not be.}
 \item{qtl.name}{Optional user-specified name for each QTL.  If
   \code{qtl} is provided, this should not be.}
 \item{covar}{A matrix or data.frame of covariates.  These must be
   strictly numeric.}
 \item{formula}{An object of class \code{\link[stats]{formula}}
   indicating the model to be fitted.  (It can also be the character
   string representation of a formula.)  QTLs are indicated as \code{Q1},
   \code{Q2}, etc.  Covariates are indicated by their names in \code{covar}.}
 \item{method}{Indicates whether to use multiple imputation or
   Haley-Knott regression.}
 \item{model}{The phenotype model: the usual model or a model for binary
  traits}
 \item{verbose}{If TRUE, give feedback about progress.  If
   \code{verbose} is an integer > 1, further messages from
   \code{\link{scanqtl}} are also displayed.}
 \item{maxit}{Maximum number of iterations.}
 \item{incl.markers}{If FALSE, do calculations only at points on an
   evenly spaced grid.}
 \item{keeplodprofile}{If TRUE, keep the LOD profiles from the last
   iteration as attributes to the output.}
 \item{tol}{Tolerance for convergence for the binary trait model.}
 \item{maxit.fitqtl}{Maximum number of iterations for fitting the binary trait
   model.}
 \item{forceXcovar}{If TRUE, force inclusion of X-chr-related covariates
   (like sex and cross direction).}
}

\details{
  QTL positions are optimized, within the context of a fixed QTL model,
  by a scheme described in Zeng et al. (1999).  Each QTL is considered
  one at a time (in a random order), and a scan is performed, allowing
  the QTL to vary across its chromosome, keeping the positions of all
  other QTL fixed.  If there is another QTL on the chromosome, the
  position of the floating QTL is scanned from the end of the chromosome
  to the position of the flanking QTL.  If the floating QTL is between
  two QTL on a chromosome, its position is scanned between those two QTL
  positions.  Each QTL is moved to the position giving the highest
  likelihood, and the entire process is repeated until no further
  improvement in likelihood can be obtained.

  One may provide either a \code{qtl} object (as produced by
  \code{\link{makeqtl}}), or vectors \code{chr} and \code{pos}
  (and, optionally, \code{qtl.name}) indicating the positions of the
  QTL.

  If a \code{qtl} object is provided, QTL that do not appear in
  the model \code{formula} are ignored, but they remain part of the QTL
  object that is output.
}

\value{
  An object of class \code{qtl}, with QTL placed in their new positions.

  If \code{keeplodprofile=TRUE}, LOD profiles from the last pass through
  the refinement algorithm are retained as an attribute,
  \code{"lodprofile"}, to the object.  These may be plotted with
  \code{\link{plotLodProfile}}.
}

\references{
  Zeng, Z.-B., Kao, C.-H., and Basten, C. J. (1999) Estimating the
  genetic architecture of quantitative traits.  \emph{Genet. Res.}
  \bold{74}, 279--289.

  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=c(2,5))}
fake.bc <- calc.genoprob(fake.bc, step=2)
qtl <- makeqtl(fake.bc, chr=c(2,5), pos=c(32.5, 17.5), what="prob")
rqtl <- refineqtl(fake.bc, qtl=qtl, method="hk")
}

\seealso{ \code{\link{fitqtl}}, \code{\link{makeqtl}},
  \code{\link{scanqtl}}, \code{\link{addtoqtl}},
  \code{\link{dropfromqtl}}, \code{\link{replaceqtl}},
  \code{\link{plotLodProfile}} }

\keyword{utilities}

\name{makeqtl}
\alias{makeqtl}

\title{Make a qtl object}

\description{
  This function takes a cross object and specified chromosome numbers
  and positions and pulls out the genotype probabilities or imputed
  genotypes at the nearest pseudomarkers, for later use by the function
  \code{\link{fitqtl}}.
}

\usage{
makeqtl(cross, chr, pos, qtl.name, what=c("draws","prob"))
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Vector indicating the chromosome for each QTL. (These should
   be character strings referring to the chromosomes by name.)}
 \item{pos}{Vector (of same length as \code{chr}) indicating the
   positions on the chromosome to be taken. If there is no marker or
   pseudomarker at a position, the nearest position is used.}
 \item{qtl.name}{Optional user-specified name for each QTL, used in the
   drop-one-term ANOVA table in \code{\link{fitqtl}}.
   If unspecified, the names will be of the form \code{"Chr1@10"} for a
   QTL on Chromsome 1 at 10 cM.}
 \item{what}{Indicates whether to pull out the imputed genotypes or the
   genotype probabilities.}
}

\details{
  This function will take out the genotype probabilities and imputed
  genotypes if they are present in the input \code{cross} object. If both
  fields are missing in the input object, the function will report an
  error. Before running this function, the user must have first run either
  \code{\link{sim.geno}} (for \code{what="draws"}) or
  \code{\link{calc.genoprob}} (for \code{what="prob"}).
}

\value{
  An object of class \code{qtl} with the following elements (though only
  one of \code{geno} and \code{prob} will be included, according to
  whether \code{what} is given as \code{"draws"} or \code{"prob"}):
  \item{\code{geno}}{Imputed genotypes.}
  \item{\code{prob}}{Genotype probabilities.}
  \item{\code{name}}{User-defined name for each QTL, or a name of the
    form \code{"Chr1@10"}.}
  \item{\code{altname}}{QTL names of the form \code{"Q1"}, \code{"Q2"},
    etc.}
  \item{\code{chr}}{Input vector of chromosome numbers.}
  \item{\code{pos}}{Input vector of chromosome positions.}
  \item{\code{n.qtl}}{Number of QTLs.}
  \item{\code{n.ind}}{Number of individuals.}
  \item{\code{n.gen}}{A vector indicating the number of genotypes for each QTL.}
}

\author{Hao Wu; Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c("1", "6", "13")
qp <- c(25.8, 33.6, 18.63)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- sim.geno(fake.f2, n.draws=8, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="draws")
summary(qtl)
}

\seealso{ \code{\link{fitqtl}}, \code{\link{calc.genoprob}},
  \code{\link{sim.geno}}, \code{\link{dropfromqtl}},
  \code{\link{replaceqtl}}, \code{\link{addtoqtl}}, \code{\link{summary.qtl}},
  \code{\link{reorderqtl}}  }

\keyword{utilities}

\name{locateXO}
\alias{locateXO}

\title{Estimate locations of crossovers}

\description{
  Estimate the locations of crossovers for each individual on a given
  chromosome.
}

\usage{
locateXO(cross, chr, full.info=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Chromosome to investigate (if unspecified, the first
   chromosome is considered).
   This should be a character string referring to a chromosome
   by name; numeric values are converted to strings.}
 \item{full.info}{If TRUE, output will include information on the left
   and right endpoints of the intervals to which recombination events
   are known, as well as the corresponding marker indices.}
}

\details{
  For each individual we detemine the locations of obligate crossovers,
  and estimate their location to be at the midpoint between the nearest
  flanking typed markers.

  The function currently only works for a backcross, intercross, or
  recombinant inbred line.
}

\value{
  A list with one component per individual.  Each component is either
  NULL or is a numeric vector with the estimated crossover locations.

  If \code{full.info=TRUE}, in place of a numeric vector with estimated
  locations, there is a matrix that includes those locations, the left
  and right endpoints of the intervals to which crossovers can be
  placed, the marker indices corresponding to those endpoint, and
  genotype codes for the genotypes to the left and right of each crossover.  The
  final column indicates the number of typed markers between the current
  crossover and the next one (useful for identifying potential
  genotyping errors).
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
xoloc <- locateXO(hyper, chr=4)
table(sapply(xoloc, length))
}

\seealso{ \code{\link{countXO}}, \code{\link{cleanGeno}} }

\keyword{utilities}

\name{simPhyloQTL}
\alias{simPhyloQTL}

\title{Simulate a set of intercrosses for a single diallelic QTL}

\description{
  Simulate a set of intercrosses with a single diallelic QTL.
}

\usage{
simPhyloQTL(n.taxa=3, partition, crosses, map, n.ind=100, model,
            error.prob=0, missing.prob=0, partial.missing.prob=0,
            keep.qtlgeno=FALSE, keep.errorind=TRUE, m=0, p=0,
        map.function=c("haldane","kosambi","c-f","morgan"))
}

\arguments{
 \item{n.taxa}{Number of taxa (i.e., strains).}
 \item{partition}{A vector of character strings of the form "AB|CD" or "A|BCD"
 indicating, for each QTL, which taxa have which allele.  If missing,
 simulate under the null hypothesis of no QTL.}
 \item{crosses}{A vector of character strings indicating the crosses to
 do (for the form "AB", "AC", etc.).  These will be sorted and then only
 unique ones used.  If missing, all crosses will be simulated.}
 \item{map}{A list whose components are vectors containing the marker
   locations on each of the chromosomes.}
 \item{n.ind}{The number of individuals in each cross.  If length 1, all
 crosses will have the same number of individuals; otherwise the length
 should be the same as \code{crosses}.}
 \item{model}{A matrix where each row corresponds to a
 different QTL, and gives the chromosome number, cM position and
 effects of the QTL (assumed to be the same in each cross in which the
 QTL is segregating).}
 \item{error.prob}{The genotyping error rate.}
 \item{missing.prob}{The rate of missing genotypes.}
 \item{partial.missing.prob}{When simulating an intercross or 4-way
   cross, this gives the rate at which markers will be incompletely
   informative (i.e., dominant or recessive).}
 \item{keep.qtlgeno}{If TRUE, genotypes for the simulated QTLs will be
   included in the output.}
 \item{keep.errorind}{If TRUE, and if \code{error.prob > 0}, the
   identity of genotyping errors will be included in the output.}
 \item{m}{Interference parameter; a non-negative integer.  0 corresponds
   to no interference.}
 \item{p}{Probability that a chiasma comes from the no-interference
   mechanism}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions.}
}

\details{
  Meiosis is assumed to follow the Stahl model for crossover
  interference (see the references, below), of which the no interference
  model and the chi-square model are special cases.  Chiasmata on the
  four-strand bundle are a superposition of chiasmata from two different
  mechanisms.  With probability \code{p}, they arise by a mechanism
  exhibiting no interference; the remainder come from a chi-square model
  with inteference parameter \code{m}.  Note that \code{m=0} corresponds
  to no interference, and with \code{p=0}, one gets a pure chi-square
  model.

  QTLs are assumed to act additively, and the residual phenotypic
  variation is assumed to be normally distributed with variance 1.

  The effect of a QTL is a pair of numbers,
  (\eqn{a,d}), where \eqn{a} is the additive effect (half the difference
  between the homozygotes) and \eqn{d} is the dominance deviation (the
  difference between the heterozygote and the midpoint between the
  homozygotes).

}

\value{
  A list with each component being an object of class \code{cross}.  See \code{\link{read.cross}} for
  details.  The names (e.g. "AB", "AC", "BC") indicate the crosses.

  If \code{keep.qtlgeno} is TRUE, each cross object will contain a
  component \code{qtlgeno} which is a matrix containing the QTL
  genotypes (with complete data and no errors), coded as in the genotype
  data.

  If \code{keep.errorind} is TRUE and errors were simulated, each
  component of \code{geno} in each cross will each contain a matrix \code{errors},
  with 1's indicating simulated genotyping errors.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\seealso{ \code{\link{scanPhyloQTL}},   \code{\link{inferredpartitions}},
  \code{\link{summary.scanPhyloQTL}}, \code{\link{max.scanPhyloQTL}},
  \code{\link{plot.scanPhyloQTL}},
  \code{\link{sim.cross}}, \code{\link{read.cross}} }

\examples{
\dontrun{
# example map; drop X chromosome
data(map10)
map10 <- map10[1:19]

# simulate data
x <- simPhyloQTL(4, partition="AB|CD", crosses=c("AB", "AC", "AD"),
                 map=map10, n.ind=150,
                 model=c(1, 50, 0.5, 0))

# run calc.genoprob on each cross
x <- lapply(x, calc.genoprob, step=2)

# scan genome, at each position trying all possible partitions
out <- scanPhyloQTL(x, method="hk")

# maximum peak
max(out, format="lod")

# approximate posterior probabilities at peak
max(out, format="postprob")

# all peaks above a threshold for LOD(best) - LOD(2nd best)
summary(out, threshold=1, format="lod")

# all peaks above a threshold for LOD(best), showing approx post'r prob
summary(out, format="postprob", threshold=3)

# plot of results
plot(out)
}
}

\references{
  Broman, K. W., Kim, S., An\'e, C. and Payseur, B. A.  Mapping
  quantitative trait loci to a phylogenetic tree.  In preparation.
}

\keyword{datagen}

\name{orderMarkers}
\alias{orderMarkers}

\title{Find an initial order for markers within chromosomes}

\description{
  Establish initial orders for markers within chromosomes by a greedy
  algorithm, adding one marker at a time with locations of previous
  markers fixed, in the position giving the miniminum number of obligate
  crossovers.
}

\usage{
orderMarkers(cross, chr, window=7, use.ripple=TRUE, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             maxit=4000, tol=1e-4, sex.sp=TRUE, verbose=FALSE)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
   This should be a vector of character strings referring to chromosomes
   by name; numeric values are converted to strings.  Refer to
   chromosomes with a preceding \code{-} to have all chromosomes but
   those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{window}{If \code{use.ripple=TRUE}, this indicates the number of
   markers to include in the sliding window of permuted markers.  Larger
   numbers result in the comparison of a greater number of marker
   orders, but will require a considerable increase in computation
   time.}
 \item{use.ripple}{If TRUE, the initial order is refined by a call to
 the function \code{\link{ripple}}.}
 \item{error.prob}{Assumed genotyping error rate used in the final
   estimated map.}
 \item{map.function}{Indicates the map function to use in the final
   estimated map.}
 \item{maxit}{Maximum number of EM iterations to perform in the final
   estimated map.}
 \item{tol}{Tolerance for determining convergence in the final estimated
   map.}
 \item{sex.sp}{Indicates whether to estimate sex-specific maps in the
   final estimated map; this is used only for the 4-way cross.}
 \item{verbose}{If TRUE, information about the progress of the
   calculations is displayed; if > 1, even more information is given.}
}

\value{
  The output is a cross object, as in the input, with orders of markers
  on selected chromosomes revised.
}

\details{
  Markers within a linkage group are considered in order of decreasing
  number of genotyped individuals.  The first two markers are placed in
  an arbitrary order.  Additional markers are considered one at a time,
  and each possible placement of a marker is compared (with the order of
  the previously placed markers taken as fixed) via the number of
  obligate crossovers (that is, the minimal number of crossovers that
  would explain the observed data).  The marker is placed in the
  position giving the minimal number of obligate crossovers.  If
  multiple positions give the same number of obligate crossovers, a
  single location (among those positions) is chosen at random.

  If \code{use.ripple=TRUE}, the final order is passed to
  \code{\link{ripple}} with \code{method="countxo"} to refine the
  marker order.  If \code{use.ripple=TRUE} and the number of markers on
  a chromosome is \eqn{\le}{<=} the argument \code{window}, the initial
  greedy algorithm is skipped and all possible marker orders are
  compared via \code{\link{ripple}}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
pull.map(listeria, chr=3)
revcross <- orderMarkers(listeria, chr=3, use.ripple=FALSE)
pull.map(revcross, chr=3)
}

\seealso{ \code{\link{formLinkageGroups}},
  \code{\link{ripple}}, \code{\link{est.map}}, \code{\link{countXO}} }

\keyword{utilities}

\name{find.markerindex}
\alias{find.markerindex}

\title{Determine the numeric index for a marker}

\description{
  Determine the numeric index for a marker in a cross object, when all
  markers on all chromosomes are pasted together.
}

\usage{find.markerindex(cross, name)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{name}{A vector of marker names.}
}

\value{
  A vector of numeric indices, from 1, 2, \dots, \code{totmar(cross)},
  with \code{NA} for markers not found.
}


\author{Danny Arends; Karl W Broman \email{broman@wisc.edu} }

\examples{
data(hyper)
mar <- find.marker(hyper, 4, 30)
find.markerindex(hyper, mar)
}

\seealso{ \code{\link{find.markerpos}} }

\keyword{utilities}

\name{locations}
\alias{locations}

\docType{data}

\title{
    Genetic locations of traits for the multitrait dataset
}

\description{
    A table with genetic locations of the traits in the \code{\link{multitrait}} dataset
}
\usage{data(locations)}
\format{
    Each row is a trait with the following information:
    \code{Name},	Name of the trait (will be checked against the name in the cross object
    \code{Chr},		Chromosome of the trait
    \code{cM},		Location in cM from the start of the chromosome
}
%\details{
%
%}
\source{
  Additional information from the Arabidopsis RIL selfing experiment with Landsberg erecta (Ler) and Cape Verde Islands (Cvi) with 162 individuals scored (with errors at) 117 markers.
  Dataset obtained from GBIC - Groningen BioInformatics Centre
}
\references{
\itemize{

 \item Keurentijes JJB, Fu J, de Vos CHR,Lommen A, Jansen RC et al
 (2006), The genetics of plant metabolism. \emph{Nature Genetics}
 \bold{38}, 842--849.

 \item Alonso-Blanco C., Peeters, A. J. and Koornneef, M. (2006)
 Development of an AFLP based linkage map of Ler, Col and Cvi
 Arabidopsis thaliana ecotypes and construction of a Ler/Cvi recombinant
 inbred line population. \emph{Plant J.} \bold{14}(2), 259--271.
}

}

\examples{
% See the help file for mqmplot.cistrans
\dontrun{
    data(multitrait)
    data(locations)
    multiloc <- addloctocross(multitrait,locations)
    results <- scanall(multiloc)
    mqmplot.cistrans(results,multiloc, 5, FALSE, TRUE)
}
}

\seealso{\code{\link{multitrait}} }

\keyword{datasets}

\name{fitstahl}
\alias{fitstahl}

\title{Fit Stahl interference model}

\description{
  Fit the Stahl model for crossover inference (or the chi-square model,
  which is a special case).
}

\usage{
fitstahl(cross, chr, m, p, error.prob=0.0001, maxit=4000, tol=1e-4,
         maxm=15, verbose=TRUE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
\item{m}{Interference parameter (a non-negative integer); if
   unspecified, this is estimated.}
 \item{p}{The proportion of chiasmata coming from the no interference
   mechanism in the Stahl model (0 <= p <= 1).  p=0 gives the chi-square
   model.  If unspecified, this is estimated.}
 \item{error.prob}{The genotyping error probability.  If = NULL, it is
   estimated.}
 \item{maxit}{Maximum number of iterations to perform.}
 \item{tol}{Tolerance for determining convergence.}
 \item{maxm}{Maximum value of m to consider, if m is unspecified.}
 \item{verbose}{Logical; indicates whether to print tracing information.}
}

\details{
  This function is currently only available for backcrosses and
  intercrosses.

  The Stahl model of crossover interference (of which the chi-square
  model is a special case) is fit.      In the chi-square model, points
  are tossed down onto the four-strand bundle according to a Poisson
  process, and every \eqn{(m+1)}st point is a chiasma.  With the
  assumption of no chromatid interference, crossover locations on a
  random meiotic product are obtained by thinning the chiasma process.
  The parameter \eqn{m} (a non-negative integer) governs the strength of
  crossover interference, with \eqn{m=0} corresponding to no
  interference.

  In the Stahl model, chiasmata on the four-strand bundle are a
  superposition of chiasmata from two mechanisms, one following a
  chi-square model and one exhibiting no interference.  An additional
  parameter, \eqn{p}, gives the proportion of chiasmata from the no
  interference mechanism.

  If all of \code{m}, \code{p}, and \code{error.prob} are specified, any
  of them with length > 1 must all have the same length.

  If \code{m} is unspecified, we do a grid search starting at 0 and stop
  when the likelihood decreases (thus assuming a single mode), or
  \code{maxm} is reached.
}

\value{
  A matrix with four columns: m, p, error.prob, and the log likelihood.

  If specific values for m, p, error.prob are provided, the log
  likelihood for each set are given.

  If some are left unspecified, the maximum likelihood estimates are
  provided in the results.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
\dontshow{set.seed(12201969)}
# Simulate genetic map: one chromosome of length 200 cM with
# a 2 cM marker spacing
mymap <- sim.map(200, 51, anchor.tel=TRUE, include.x=FALSE,
                 sex.sp=FALSE, eq.spacing=TRUE)

# Simulate data under the chi-square model, no errors
mydata <- sim.cross(mymap, n.ind=250, type="bc",
                    error.prob=0, m=3, p=0)

# Fit the chi-square model for specified m's
\dontrun{output <- fitstahl(mydata, m=1:5, p=0, error.prob=0)
}\dontshow{output <- fitstahl(mydata, m=1, p=0, error.prob=0,tol=0.001)}
plot(output$m, output$loglik, lwd=2, type="b")

# Find the MLE of m in the chi-square model
\dontrun{mle <- fitstahl(mydata, p=0, error.prob=0)}

\dontrun{
# Simulate data under the Stahl model, no errors
mydata <- sim.cross(mymap, n.ind=250, type="bc",
                    error.prob=0, m=3, p=0.1)

# Find MLE of m for the Stahl model with known p
mle.stahl <- fitstahl(mydata, p=0.1, error.prob=0)

# Fit the Stahl model with unknown p and m,
# get results for m=0, 1, 2, ..., 8
output <- fitstahl(mydata, m=0:8, error.prob=0)
plot(output$m, output$loglik, type="b", lwd=2)}
}

\references{
  Armstrong, N. J., McPeek, M. J. and Speed, T. P. (2006) Incorporating
  interference into linkage analysis for experimental crosses.
  \emph{Biostatistics} \bold{7}, 374--386.

  Zhao, H., Speed, T. P. and McPeek, M. S. (1995) Statistical analysis of
  crossover interference using the chi-square model.  \emph{Genetics}
  \bold{139}, 1045--1056.
}

\seealso{ \code{\link{est.map}}, \code{\link{sim.cross}}}

\keyword{models}

\name{subset.map}
\alias{subset.map}
\alias{[.map}

\title{Subsetting chromosomes for a genetic map}

\description{
  Pull out a specified set of chromosomes from a
  \code{map} object.
}

\usage{
\method{subset}{map}(x, \dots)
\method{[}{map}(x, \dots)
}

\arguments{
 \item{x}{A list whose components are vectors of marker locations.}
 \item{\dots}{Vector of chromosome indices.}
}

\value{The input \code{map} object, but with only the specified subset
  of chromosomes.}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(map10)
map10 <- subset(map10, chr=1:5)

# you can also use brackets
map10 <- map10[2:3]
}

\seealso{ \code{\link{subset.cross}} }

\keyword{manip}

\name{c.scantwoperm}
\alias{c.scantwoperm}
\alias{rbind.scantwoperm}

\title{Combine data from scantwo permutations}

\description{
  Concatenate the data for multiple runs of \code{\link{scantwo}}
  with \code{n.perm > 0}.
}

\usage{
\method{c}{scantwoperm}(\dots)
\method{rbind}{scantwoperm}(\dots)
}

\arguments{
 \item{\dots}{A set of objects of class \code{scantwoperm}. (This can
 also be a list of \code{scantwoperm} objects.)  These are
   the permutation results from \code{\link{scantwo}} (that is,
   when \code{n.perm > 0}).  These must all concern the same number of
   LOD columns. (That is, they must have been created with the same number
   of phenotypes, and it is assumed that they were generated in
   precisely the same way.)}
}

\value{The concatenated input, as a \code{scantwoperm} object.}

\details{
  The aim of this function is to concatenate the results from multiple
  runs of a permutation test \code{\link{scantwo}}, to assist with
  the case that such permutations are done on multiple processors in
  parallel.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(18:19,"X"), ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2)
\dontrun{operm1 <- scantwo(fake.f2, method="hk", n.perm=50)
operm2 <- scantwo(fake.f2, method="hk", n.perm=50)}\dontshow{
operm1 <- scantwo(fake.f2, method="hk", n.perm=3)
operm2 <- scantwo(fake.f2, method="hk", n.perm=3)}

operm <- c(operm1, operm2)
}

\seealso{ \code{\link{summary.scantwoperm}},
  \code{\link{scantwo}}, \code{\link{cbind.scantwoperm}} }

\keyword{manip}

\name{scanonevar.varperm}
\alias{scanonevar.varperm}

\title{Permutation test for variance effect in scanonevar}

\description{Executes permutations of the genotypes in the variance-effect part of scanonevar}

\usage{
scanonevar.varperm(cross, pheno.col=1, mean_covar=NULL, var_covar=NULL,
                   maxit=25, tol=1e-6, n.var.perm = 2, seed = 27517, quiet=TRUE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  This must be a single value (integer index or
   phenotype name) or a numeric vector of
   phenotype values, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
  \item{mean_covar}{Numeric matrix with covariates affecting the mean.}
  \item{var_covar}{Numeric matrix with covariates affecting the variances.}
  \item{maxit}{Maximum number of iterations in the algorithm to fit the
    model at a given position.}
  \item{tol}{Tolerance for convergence.}
  \item{n.var.perm}{Numeric vector of length one indicates the number of permutations to execute.}
  \item{seed}{Numeric vector of length one indicates the random seed to start the permutations.}
  \item{quiet}{If \code{FALSE}, print some information about the course
    of the calculations.}
}


\value{
  A vector of length \code{n.var.perm} of the maximum negative log10 p-value that resulted from each permutation.
}

\name{plotGeno}
\alias{plotGeno}

\title{Plot observed genotypes, flagging likely errors}

\description{
  Plot the genotypes on a particular chromosome for a set of
  individuals, flagging likely errors.
}

\usage{
plotGeno(x, chr, ind, include.xo=TRUE, horizontal=TRUE,
          cutoff=4, min.sep=2, cex=1.2, \dots)
}

\arguments{
  \item{x}{An object of class \code{cross}.  See
    \code{\link{read.cross}} for details. }
  \item{chr}{The chromosome to plot.  Only one chromosome is
  allowed. (This should be a character string referring to the
  chromosomes by name.)}
  \item{ind}{Vector of individuals to plot (passed to \code{\link{subset.cross}}).  If missing, all individuals
    are plotted.}
  \item{include.xo}{If TRUE, plot X's in intervals having a
    crossover. Not available for a 4-way cross.}
  \item{horizontal}{If TRUE, chromosomes are plotted horizontally.}
  \item{cutoff}{Genotypes with error LOD scores above this value are
    flagged as possible errors.}
  \item{min.sep}{Markers separated by less than this value (as a
    percent of the chromosome length) are pulled apart, so that they may
    be distinguished in the picture.}
  \item{cex}{Character expansion for the size of points in the plot.
    Larger numbers give larger points; see \code{\link[graphics]{par}}.}
  \item{\dots}{Ignored at this point.}
}

\details{
  Plots the genotypes for a set of individuals.  Likely errors are
  indicated by red squares.  In a backcross, genotypes AA and AB are
  indicated by white and black circles, respectively.  In an intercross,
  genotypes AA, AB and BB are indicated by white, gray, and black
  circles, respectively, and the partially missing genotypes "not BB"
  (D in mapmaker) and "not AA" (C in mapmaker) are indicated by green
  and orange circles, respectively.

  For the X chromosome in a backcross or intercross, hemizygous males
  are plotted as if they were homozygous (that is, with white and black
  circles).

  For a 4-way cross, two lines are plotted for each individual.  The
  left or upper line indicates the allele A (white) or B (black); the
  right or lower line indicates the allele C (white) or D (black).  For
  the case that genotype is known to be only AC/BD or AD/BC, we use
  green and orange, respectively.

}

\value{None.}

\examples{
data(hyper)
\dontshow{hyper <- subset(hyper, chr=c(1,19))}
# Calculate error LOD scores
hyper <- calc.errorlod(hyper,error.prob=0.01)

# print those above a specified cutoff
top.errorlod(hyper,cutoff=4)

# plot genotype data, flagging genotypes with error LOD > cutoff
plotGeno(hyper, chr=1, ind=160:200, cutoff=7, min.sep=2)
}

\seealso{ \code{\link{calc.errorlod}},
  \code{\link{top.errorlod}}, \code{\link{subset.cross}} }


\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{inferFounderHap}
\alias{inferFounderHap}

\title{Crude reconstruction of founder haplotypes in multi-parent RIL}

\description{
  Uses groups of adjacent markers to infer the founder haplotypes in SNP
  data on multi-parent recombinant inbred lines.
}

\usage{
inferFounderHap(cross, chr, max.n.markers=15)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Indicator of chromosome to consider.  If multiple
 chromosomes are selected, only the first is used.}
 \item{max.n.markers}{Maximum number of adjacent markers to consider.}
}

\details{
  We omit SNPs for which any of the founders are missing.

  We then consider groups of adjacent SNPs, looking for founder
  haplotypes that are unique; RIL sharing such a unique haplotype are
  then inferred to have that founder's DNA.

  We consider each marker as the center of a haplotype, and consider
  haplotypes of size 1, 3, 5, \dots, \code{max.n.markers}.  We end the
  extension of the haplotypes when all founders have a unique haplotype.
}

\value{
  A matrix of dimension \code{nind(cross)} \eqn{\times}{x} no. markers,
  with the inferred founder origin for each line at each marker.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
map <- sim.map(100, n.mar=101, include.x=FALSE, eq.spacing=TRUE)
founderGeno <- simFounderSnps(map, "8")
ril <- sim.cross(map, n.ind=10, type="ri8sib", founderGeno=founderGeno)

h <- inferFounderHap(ril, max.n.markers=11)
mean(!is.na(h)) # proportion inferred
plot(map[[1]], h[1,], ylim=c(0.5, 8.5), xlab="Position", ylab="Genotype")
}

\seealso{ \code{\link{sim.geno}}, \code{\link{calc.genoprob}},
          \code{\link{fill.geno}}, \code{\link{argmax.geno}} }

\keyword{utilities}

\name{convert2sa}
\alias{convert2sa}

\title{Convert a sex-specific map to a sex-averaged one}

\description{
  Convert a sex-specific map to a sex-averaged one, assuming that the
  female and male maps are actually the same (that is, that the map was
  estimated assuming a common recombination rate in females and males).
}

\usage{
convert2sa(map, tol=1e-4)
}

\arguments{
 \item{map}{A map object with sex-specific locations (but assuming that
 the female and male maps are the same), as output by the function
 \code{\link{est.map}} for a 4-way cross, with argument
 \code{sex.sp=FALSE}.}
  \item{tol}{Tolerance value for inspecting the differences between the
 female and male maps; if they differ by more than this tolerance, a
 warning is issued.}
}

\value{
  A map object, with sex-averaged distances.
}

\details{
  We pull out just the female marker locations, and give a warning if
  there are large differences between the female and male maps.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.4way)
\dontrun{fake.4way <- subset(fake.4way, chr="-X")}
\dontshow{fake.4way <- subset(fake.4way, chr=18:19)}
nm <- est.map(fake.4way, sex.sp=FALSE)
plot(convert2sa(nm))
}

\seealso{ \code{\link{est.map}}, \code{\link{plotMap}} }

\keyword{manip}

\name{summary.scantwo}
\alias{summary.scantwo}

\title{Summarize the results of a two-dimensional genome scan}

\description{
  Summarize the interesting aspects of the results of \code{\link{scantwo}}.
}

\usage{
\method{summary}{scantwo}(object, thresholds,
        what=c("best", "full", "add", "int"),
        perms, alphas, lodcolumn=1, pvalues=FALSE,
        allpairs=TRUE, \dots)
}

\arguments{
 \item{object}{An object of class \code{scantwo}, the output of the
   function \code{\link{scantwo}}.}
 \item{thresholds}{A vector of length 5, giving LOD thresholds
   for the full, conditional-interactive, interaction, additive, and
   conditional-additive LOD scores. See Details, below.}
 \item{what}{Indicates for which LOD score the maximum should be
   reported. See Details, below.}
 \item{perms}{Optional permutation results used to derive thresholds or
   to calculate genome-scan-adjusted p-values.  This must be consistent
   with the \code{object} input, in that it must have the same number of
   LOD score columns, though it can have just one column of permutation
   results, in which case they are assumed to apply to any chosen LOD
   score column.}
 \item{alphas}{If perms are included, these are the significance levels used
   to calculate thresholds for determining which peaks to pull out.  It
   should be a vector of length 5, giving significance levels
   for the full, conditional-interactive, interaction, additive, and
   conditional-additive LOD scores.  (It can also be a single number, in
   which case it is assumed that the same value is used for all five LOD
   scores.)  If \code{thresholds} is specified, \code{alphas} should not
   be.}
 \item{lodcolumn}{If the scantwo results contain LOD scores for
    multiple phenotypes, this argument indicates which to use in the
    summary.  Only one LOD score column may be considered at a time.}
 \item{pvalues}{If TRUE, include columns with genome-scan-adjusted
   p-values in the results.  This requires that \code{perms} be
   provided.}
 \item{allpairs}{If TRUE, all pairs of chromosomes are considered.  If
   FALSE, only self-self pairs are considered, so that one may more
   conveniently check for possible linked QTL.}
 \item{\dots}{Ignored at this point.}
}

\value{
  An object of class \code{summary.scantwo}, to be printed by
  \code{print.summary.scantwo};
}

\details{
  If \code{what="best"}, we calculate, for each pair of chromosomes, the
  maximum LOD score for the full model (two QTL plus interaction) and
  the maximum LOD score for the additive model.  The difference between
  these is a LOD score for a test for interaction.  We also calculate
  the difference between the maximum full LOD and the maximum single-QTL LOD
  score for the two chromosomes; this is the LOD score for a test for a
  second QTL, allowing for epistasis, which we call either the
  conditional-interactive or "fv1" LOD score.  Finally,
  we calculate the difference between the maximum additive LOD score and
  the maximum single-QTL LOD score for the two chromosomes; this is the
  LOD score for a test for a second QTL, assuming that the two QTL act
  additively, which we call either the conditional-additive or "av1" LOD
  score.  Note that the maximum full LOD and additive LOD are allowed to
  occur in different places.

  If \code{what="full"}, we find the maximum full LOD and extract the
  additive LOD at the corresponding pair of positions; we derive
  the other three LOD scores for that fixed pair of positions.

  If \code{what="add"}, we find the maximum additive LOD and extract the
  full LOD  at the corresponding pair of positions; we derive
  the other three LOD scores for that fixed pair of positions.

  If \code{what="int"}, we find the pair of positions for which the
  difference between the full and additive LOD scores is largest, and
  then calculate the five LOD scores at that pair of positions.

  If \code{thresholds} or \code{alphas} is provided (and note that when
  \code{alphas} is provided, \code{perms} must also), we extract just
  those pairs of chromosomes for which either (a) the full LOD score
  exceeds its thresholds and either the conditional-interactive LOD or
  the interaction LOD exceed their threshold, or (b) the additive LOD
  score exceeds its threshold and the conditional-additive LOD exceeds
  its threshold.  The thresholds or alphas must be given in the order
  full, cond-int, int, add, cond-add.

  Thresholds may be obtained by a permutation test with
  \code{\link{scantwo}}, but these are extremely time-consuming.
  For a mouse backcross, we suggest the thresholds (6.0, 4.7, 4.4, 4.7,
  2.6) for the full, conditional-interactive, interaction, additive, and
  conditional-additive LOD scores, respectively.
  For a mouse intercross, we suggest the thresholds (9.1, 7.1, 6.3, 6.3,
  3.3) for the full, conditional-interactive, interaction, additive, and
  conditional-additive LOD scores, respectively.  These were obtained by
  10,000 simulations of crosses with 250 individuals, markers at a 10 cM
  spacing, and analysis by Haley-Knott regression.
}

\section{Output of addpair}{

  \bold{Note} that, for output from \code{\link{addpair}} in which the
  new loci are indicated explicitly in the formula, the summary provided
  by \code{summary.scantwo} is somewhat special.

  All arguments except \code{allpairs} and \code{thresholds} (and, of
  course, the input \code{object}) are ignored.

  If the formula is symmetric in the two new QTL, the output has just two LOD
  score columns: \code{lod.2v0} comparing the full model to the model
  with neither of the new QTL, and \code{lod.2v1} comparing the full
  model to the model with just one new QTL.

  If the formula is \emph{not} symmetric in the two new QTL, the output
  has three LOD score columns: \code{lod.2v0} comparing the full model
  to the model with neither of the new QTL, \code{lod.2v1b} comparing
  the full model to the model in which the first of the new QTL is
  omitted, and \code{lod.2v1a} comparing the full model to the model
  with the second of the new QTL omitted.

  The \code{thresholds} argument should have length 1 or 2, rather than
  the usual 5.  Rows will be retained if \code{lod.2v0} is greater than
  \code{thresholds[1]} and \code{lod.2v1} (or either of \code{lod.2v1a}
  or \code{lod.2v1b}) is greater than \code{thresholds[2]}.  (If a
  single thresholds is given, we assume that \code{thresholds[2]==0}.)
}

\section{The older version}{
  The previous version of this function is still available, though it is
  now named \code{\link{summaryScantwoOld}}.

  We much prefer the revised function.  However, while we are confident
  that this function (and the permutations in
  \code{\link{scantwo}}) are calculating the relevant statistics,
  the appropriate significance levels for these relatively complex
  series of statistical tests is not yet completely clear.
}


\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(1,8,13,"X"))}
fake.f2 <- calc.genoprob(fake.f2, step=5)
out.2dim <- scantwo(fake.f2, method="hk")

# All pairs of chromosomes
summary(out.2dim)

# Chromosome pairs meeting specified criteria
summary(out.2dim, thresholds=c(9.1, 7.1, 6.3, 6.3, 3.3))

# Similar, but ignoring the interaction LOD score in the rule
summary(out.2dim, thresholds=c(9.1, 7.1, Inf, 6.3, 3.3))

# Pairs having largest interaction LOD score, if it's > 4
summary(out.2dim, thresholds=c(0, Inf, 4, Inf, Inf), what="int")

# permutation test to get thresholds; run in two batches
#     and then combined with c.scantwoperm
\dontrun{operm.2dimA <- scantwo(fake.f2, method="hk", n.perm=500)
operm.2dimB <- scantwo(fake.f2, method="hk", n.perm=500)
operm.2dim <- c(operm.2dimA, operm.2dimB)}
\dontshow{strata <- pull.pheno(fake.f2, "sex") + pull.pheno(fake.f2, "pgm")*2
operm.2dim <- scantwo(fake.f2, method="hk", n.perm=100, perm.strata=strata)}

# estimated LOD thresholds
summary(operm.2dim)

# Summary, citing significance levels and so estimating thresholds
#     from the permutation results
summary(out.2dim, perms=operm.2dim, alpha=rep(0.05, 5))

# Similar, but ignoring the interaction LOD score in the rule
summary(out.2dim, perms=operm.2dim, alpha=c(0.05, 0.05, 0, 0.05, 0.05))

# Similar, but also getting genome-scan-adjusted p-values
summary(out.2dim, perms=operm.2dim, alpha=c(0.05, 0.05, 0, 0.05, 0.05),
        pvalues=TRUE)
}

\seealso{ \code{\link{scantwo}}, \code{\link{plot.scantwo}},
          \code{\link{max.scantwo}}, \code{\link{condense.scantwo}} }

\keyword{print}

\name{plot.comparegeno}
\alias{plot.comparegeno}

\title{Plot genotype comparison}

\description{
  Plot the results of the comparison of all pairs of individuals'
  genotypes. A histogram of the proportion of matching genotypes, with
  tick marks at individual values below, via \code{\link[graphics]{rug}}.
}

\usage{
\method{plot}{comparegeno}(x, breaks=NULL, main="",
     xlab="Proportion matching genotypes", \dots)
}

\arguments{
  \item{x}{An object of class \code{"comparegeno"}, as produced by
    \code{\link{comparegeno}}.}
  \item{breaks}{Passed to \code{\link[graphics]{hist}}, with the default
    \eqn{2 \sqrt{n}} where \eqn{n} is the number of pairs of individuals.}
  \item{main}{Title for the plot.}
  \item{xlab}{x-axis label for the plot.}
  \item{\dots}{Passed to \code{\link{hist}}.}
}

\value{None.}

\details{
  Creates a histogram with \code{\link[graphics]{hist}} with ticks at
  individual values using \code{\link[graphics]{rug}}.
}

\examples{
data(fake.f2)
cg <- comparegeno(fake.f2)
plot(cg)
}

\seealso{ \code{\link{comparegeno}}, \code{\link{summary.comparegeno}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{nullmarkers}
\alias{nullmarkers}

\title{Identify markers without any genotype data}

\description{
  Identify markers in a cross that have no genotype data.
}

\usage{
nullmarkers(cross)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
}

\value{
  Marker names (a vector of character strings) with no genotype data.
}

\examples{
# one marker with no data
data(hyper)
nullmarkers(hyper)

# nothing in listeria
data(listeria)
nullmarkers(listeria)
}

\seealso{ \code{\link{drop.nullmarkers}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{utilities}

\name{droponemarker}
\alias{droponemarker}

\title{Drop one marker at a time and determine effect on genetic map}

\description{
  Drop one marker at a time from a genetic map and calculate the change
  in log likelihood and in the chromosome length, in order to identify
  problematic markers.
}

\usage{
droponemarker(cross, chr, error.prob=0.0001,
                map.function=c("haldane","kosambi","c-f","morgan"),
                m=0, p=0, maxit=4000, tol=1e-6, sex.sp=TRUE,
                verbose=TRUE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{A vector specifying which chromosomes to test for the
   position of the marker.    This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions. (Ignored if m > 0.)}
 \item{m}{Interference parameter for the chi-square model for
   interference; a non-negative integer, with m=0 corresponding to no
   interference. This may be used only for a backcross or intercross.}
 \item{p}{Proportion of chiasmata from the NI mechanism, in the Stahl
   model; p=0 gives a pure chi-square model.  This may be used only for
   a backcross or intercross.}
 \item{maxit}{Maximum number of EM iterations to perform.}
 \item{tol}{Tolerance for determining convergence.}
 \item{sex.sp}{Indicates whether to estimate sex-specific maps; this is
 used only for the 4-way cross.}
 \item{verbose}{If TRUE, print information on progress; if > 1, print
   even more information.}
}

\value{
  A data frame (actually, an object of class \code{"scanone"}, so that
  one may use \code{\link{plot.scanone}},
  \code{\link{summary.scanone}}, etc.) with each row being a marker.
  The first two columns are the chromosome ID and position.  The third
  column is a LOD score comparing the hypothesis that the marker is not
  linked to the hypothesis that it belongs at that position.

  In the case of a 4-way cross, with \code{sex.sp=TRUE}, there are two
  additional columns with the change in the estimated female and male genetic lengths
  of the respective chromosome, upon deleting that marker.
  With \code{sex.sp=FALSE}, or for other types of crosses, there is one
  additional column, with the change in estimated genetic length of the respective
  chromosome, when the marker is omitted.

  A well behaved marker will have a negative LOD score and a small
  change in estimated genetic length.  A poorly behaved marker will have a large
  positive LOD score and a large change in estimated genetic length.  But note
  that dropping the first or last marker on a chromosome could result in
  a large change in estimated length, even if they are not badly
  behaved; for these markers one should focus on the LOD scores, with a large
  positive LOD score being bad.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.bc)
droponemarker(fake.bc, 7, error.prob=0, verbose=FALSE)
}


\seealso{ \code{tryallpositions}, \code{\link{est.map}}, \code{\link{ripple}},
  \code{\link{est.rf}}, \code{\link{switch.order}},
  \code{\link{movemarker}}, \code{\link{drop.markers}} }

\keyword{utilities}

\name{nind}
\alias{nind}

\title{Determine the number of individuals QTL experiment}

\description{
  Determine the number of individuals in cross object.
}

\usage{
nind(object)
}

\arguments{
 \item{object}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
}

\value{
  The number of individuals in the input cross object.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
nind(fake.f2)
}

\seealso{ \code{\link{read.cross}}, \code{\link{plot.cross}},
  \code{\link{summary.cross}},
  \code{\link{nmar}},
  \code{\link{nchr}},
  \code{\link{totmar}},
  \code{\link{nphe}} }

\keyword{print}

\name{simulatemissingdata}
\alias{simulatemissingdata}

\title{Simulates missing genotype data}

\description{
  Simulate missing genotype data by removing some genotype data from the cross object
}

\usage{
  simulatemissingdata(cross, percentage = 5)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{percentage}{ How much of the genotype data do we need to randomly drop? }
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\value{
  An object of class \code{cross} with \code{percentage} % of (randomly) missing genotype data. See \code{\link{read.cross}} for details
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
data(multitrait)
multitrait <- fill.geno(multitrait)
multimissing5 <- simulatemissingdata(multitrait,perc=5)
perc <- (sum(nmissing(multimissing5))/sum(ntyped(multimissing5)))
}

\keyword{utilities}

\name{scanPhyloQTL}
\alias{scanPhyloQTL}

\title{Single-QTL genome scan to map QTL to a phylogenetic tree}

\description{
  Jointly consider multiple intercrosses with a single diallelic QTL
  model, considering all possible partitions of the strains into the two
  QTL allele groups.
}

\usage{
scanPhyloQTL(crosses, partitions, chr, pheno.col=1,
             model=c("normal", "binary"), method=c("em", "imp", "hk"),
             addcovar, maxit=4000, tol=0.0001, useAllCrosses=TRUE,
             verbose=FALSE)
}
\arguments{
 \item{crosses}{A list with each component being an intercross, as an object of class
   \code{cross} (see \code{\link{read.cross}} for details).  The names
   (of the form "AB") indicate the strains in the cross.}
 \item{partitions}{A vector of character strings of the form "AB|CD" or
   "A|BCD" indicating the set of paritions of the strains into two
   allele groups.  If missing, all partitions should be considered.}
 \item{chr}{Optional vector indicating the chromosomes for which LOD
   scores should be calculated.  This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  This can be a vector of integers; for methods
   \code{"hk"} and \code{"imp"} this can be considerably faster than doing
   them one at a time.  One may also give a character strings matching
   the phenotype names.  Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{model}{The phenotype model: the usual normal model or a model for
   binary traits}
 \item{method}{Indicates whether to use the EM algorithm,
   imputation, or Haley-Knott regression.}
 \item{addcovar}{Optional set of additive covariates to include in the
   analysis, as a list with the same length as \code{crosses}.  They
   must be numeric vectors or matrices, as for \code{\link{scanone}}.}
 \item{maxit}{Maximum number of iterations for method \code{"em"}.}
 \item{tol}{Tolerance value for determining convergence for method
   \code{"em"}.}
 \item{useAllCrosses}{If TRUE, use all crosses in the analysis of all
   partitions, with crosses not segregating the QTL included in the
   estimation of the residual variance.}
 \item{verbose}{If TRUE, print information about progress.}
}


\details{
  The aim is to jointly consider multiple intercrosses to not just map
  QTL but to also, under the assumption of a single diallelic QTL,
  identify the set of strains with each QTL allele.

  For each partition (of the strains into two groups) that is under
  consideration, we pull out the set of crosses that are segregating the
  QTL, re-code the alleles, and combine the crosses into one large cross.
  Crosses not segregating the QTL are also used, though with no QTL
  effects.

  Additive covariate indicators for the crosses are included in the
  analysis, to allow for the possibility that there are overall shifts
  in the phenotypes between crosses.
}

\value{
  A data frame, as for the output of \code{\link{scanone}}, though with
  LOD score columns for each partition that is considered.  The result
  is given class \code{"scanPhyloQTL"}.
}

\references{
  Broman, K. W., Kim, S., An\'e, C. and Payseur, B. A.  Mapping
  quantitative trait loci to a phylogenetic tree.  In preparation.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
\dontshow{set.seed(66787403)}
# example map; drop X chromosome
data(map10)
map10 <- map10[1:19]
\dontshow{map10 <- map10[1:4]}

# simulate data
x <- simPhyloQTL(4, partition="AB|CD", crosses=c("AB", "AC", "AD"),
                 map=map10, n.ind=150,
                 model=c(1, 50, 0.5, 0))

# run calc.genoprob on each cross
\dontrun{x <- lapply(x, calc.genoprob, step=2)
}\dontshow{x <- lapply(x, calc.genoprob, step=0)}

# scan genome, at each position trying all possible partitions
out <- scanPhyloQTL(x, method="hk")

# maximum peak
max(out, format="lod")

# approximate posterior probabilities at peak
max(out, format="postprob")

# all peaks above a threshold for LOD(best) - LOD(2nd best)
summary(out, threshold=1, format="lod")

# all peaks above a threshold for LOD(best), showing approx post'r prob
summary(out, format="postprob", threshold=3)

# plot results
plot(out)
}

\seealso{ \code{\link{plot.scanPhyloQTL}},
  \code{\link{summary.scanPhyloQTL}}, \code{\link{max.scanPhyloQTL}},
  \code{\link{inferredpartitions}},
  \code{\link{simPhyloQTL}} }

\keyword{models}

\name{plot.scanone}
\alias{plot.scanone}

\title{Plot LOD curves}

\description{
  Plot the LOD curve for a genome scan with a single-QTL model (the
  output of \code{\link{scanone}}).
}

\usage{
\method{plot}{scanone}(x, x2, x3, chr, lodcolumn=1, incl.markers=TRUE,
     xlim, ylim, lty=1, col=c("black","blue","red"), lwd=2,
     add=FALSE, gap=25, mtick = c("line", "triangle"),
     show.marker.names=FALSE, alternate.chrid=FALSE,
     bandcol=NULL, type="l", cex=1, pch=1, bg="transparent",
     bgrect=NULL, \dots)
}

\arguments{
  \item{x}{An object of class \code{"scanone"}, as output by
    \code{\link{scanone}}.}
  \item{x2}{Optional second \code{scanone} object.}
  \item{x3}{Optional third \code{scanone} object.}
  \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{lodcolumn}{An integer, or vector of 3 integers, indicating which
    of the LOD score columns should be plotted (generally this is 1).}
  \item{incl.markers}{Indicate whether to plot line segments at the
  marker locations.}
  \item{xlim}{Limits for x-axis (optional).}
  \item{ylim}{Limits for y-axis (optional).}
  \item{lty}{Line types; a vector of length 1 or 3.}
  \item{col}{Line colors; a vector of length 1 or 3.}
  \item{lwd}{Line widths; a vector of length 1 or 3.}
  \item{add}{If TRUE, add to a current plot.}
  \item{gap}{Gap separating chromosomes (in cM).}
  \item{mtick}{Tick mark type for markers (line segments or
    upward-pointing triangels).}
  \item{show.marker.names}{If TRUE, show the marker names along the x axis.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{bandcol}{Optional color for alternating bands to indicate
  chromosomes.  If NULL (the default), no bands are plotted.  A good
  choice might be \code{bandcol="gray70"}.}
  \item{type}{Type of plot (see \code{plot}): for
  example, \code{type="l"} for lines or \code{type="p"} for points only,
  may be of length 1 or 3.}
  \item{cex}{Point size expansion, for example if \code{type="p"} is used.
  May be of length 1 or 3.}
  \item{pch}{Point type, for example if \code{type="p"} is used. See \code{\link[graphics]{points}}.
  May be of length 1 or 3.}
  \item{bg}{Background color for points, for example if \code{type="p"}
  and \code{pch=21} are used. See \code{\link[graphics]{points}}.
  May be of length 1 or 3.}
  \item{bgrect}{Optional background color for the rectangular plotting region.}
  \item{\dots}{Passed to the function \code{plot} when it
    is called.}
}
\value{None.}

\details{
  This function allows you to plot the results of up to three genome
  scans against one another.  Such objects must conform with each
  other.

  One may alternatively use the argument \code{add} to add the plot of
  an additional genome scan to the current figure, but some care is
  required: the same chromosomes should be selected, and the results
  must concern crosses with the same genetic maps.

  If a single \code{scanone} object containing multiple LOD score
  columns (for example, from different phenotypes) is input, up to three
  LOD curves may be plotted, by providing a vector in the argument
  \code{lodcolumn}.  If multiple \code{scanone} objects are input (via
  \code{x}, \code{x2} and \code{x3}), the LOD score columns to be
  plotted are chosen from the corresponding element of the
  \code{lodcolumn} argument.
}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(1,13))}
fake.f2 <- calc.genoprob(fake.f2,step=2.5)
out.mr <- scanone(fake.f2, method="mr")
out.em <- scanone(fake.f2, method="em")
plot(out.mr)
plot(out.mr, out.em, chr=c(1,13), lty=1, col=c("violetred","black"))
out.hk <- scanone(fake.f2, method="hk")
plot(out.hk, chr=c(1,13), add=TRUE, col="slateblue")

plot(out.hk, chr=13, show.marker.names=TRUE)

plot(out.hk, bandcol="gray70")

# plot points rather than lines
plot(out.hk, bandcol="gray70", type="p", cex=0.3, pch=21, bg="slateblue")
}

\seealso{ \code{\link{scanone}},
  \code{\link{summary.scanone}}, \code{\link[graphics]{par}},
  \code{\link[grDevices]{colors}}, \code{\link{add.threshold}}, \code{\link{xaxisloc.scanone}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{scanqtl}
\alias{scanqtl}

\title{General QTL scan}

\description{
  Performs a multiple QTL scan for specified chromosomes and positions
  or intervals, with the possible inclusion of QTL-QTL interactions and/or
  covariates.
}

\usage{
scanqtl(cross, pheno.col=1, chr, pos, covar=NULL, formula,
        method=c("imp","hk"), model=c("normal", "binary"),
        incl.markers=FALSE, verbose=TRUE, tol=1e-4, maxit=1000,
        forceXcovar=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{Column number in the phenotype matrix to be
   used as the phenotype.  One may also give a character string matching
   a phenotype name.  Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{chr}{Vector indicating the chromosome for each QTL. (These should
   be character strings referring to the chromosomes by name.)}
 \item{pos}{List indicating the positions or intervals on the chromosome
   to be scanned.  Each element should be either a single number (for a
   specific position) or a pair of numbers (for an interval).}
 \item{covar}{A matrix or data.frame of covariates.  These must be
   strictly numeric.}
 \item{formula}{An object of class \code{\link[stats]{formula}}
   indicating the model to be fitted.  (It can also be the character
   string representation of a formula.)  QTLs are indicated as \code{Q1},
   \code{Q2}, etc.  Covariates are indicated by their names in \code{covar}.}
 \item{method}{Indicates whether to use multiple imputation or
   Haley-Knott regression.}
 \item{model}{The phenotype model: the usual model or a model for binary
  traits}
 \item{incl.markers}{If FALSE, do calculations only at points on an
   evenly spaced grid.  If \code{\link{calc.genoprob}} or
   \code{\link{sim.geno}} were run with
   \code{stepwidth="variable"} or \code{stepwidth="max"}, we force \code{incl.markers=TRUE}.}
 \item{verbose}{If TRUE, give feedback about progress.}
 \item{tol}{Tolerance for convergence for the binary trait model.}
 \item{maxit}{Maximum number of iterations for fitting the binary trait
   model.}
 \item{forceXcovar}{If TRUE, force inclusion of X-chr-related covariates
   (like sex and cross direction).}
}

\details{
  The formula is used to specified the model to be fit. In the
  formula, use \code{Q1}, \code{Q2}, etc., or \code{q1},
  \code{q2}, etc., to represent the QTLs, and the column names in the
  covariate data frame to represent the covariates.

  We enforce a hierarchical structure on the model formula: if a QTL or
  covariate is in involved in an interaction, its main effect are also
  be included.

  Only the interaction terms need to be specifed in the formula. The
  main effects of all input QTLs (as specified by chr and pos) and
  covariates (as specifed by covar) will be included by default. For
  example, if the formula is \code{y~Q1*Q2*Sex}, and there are three
  elements in input \code{chr} and \code{pos} and Sex is one of the
  column names for
  input covariates, the formula used in genome scan will be
  \code{y ~ Q1 + Q2 + Q3 + Sex + Q1:Q2 + Q1:Sex + Q2:Sex + Q1:Q2:Sex}.

  The input \code{pos} is a list or vector to specify the position/range
  of the input chromosomes to be scanned. If it is a vector, it gives the
  precise positions of the QTL on the chromosomes. If it is a list, it will
  contain either the precise positions or a range on the chromosomes. For
  example, consider the case that the input \code{chr = c(1, 6,
    13)}. If \code{pos = c(9.8, 34.0, 18.6)},
  it means to fit a model with QTL on chromosome 1 at 9.8cM, chromosome
  6 at 34cM and  chromosome 13 at 18.6cM.
  If \code{pos = list(c(5,15), c(30,36), 18)}, it
  means to scan chromosome 1 from 5cM to 15cM, chromosome 6 from 30cM to
  36cM, fix the QTL on chromosome 13 at 18cM.
}

\value{
  An object of class \code{scanqtl}. It is a multi-dimensional
  array of LOD scores, with the number of dimension equal to the number
  of QTLs specifed.}

\references{
  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.
}

\author{Hao Wu}

\examples{
data(fake.f2)

# take out several QTLs
qc <- c(1, 8, 13)
fake.f2 <- subset(fake.f2, chr=qc)

# imputate genotypes
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=5, err=0.001)

# 2-dimensional genome scan with additive 3-QTL model
pos <- list(c(15,35), c(45,65), 28)
result <- scanqtl(fake.f2, pheno.col=1, chr=qc, pos=pos,
                  formula=y~Q1+Q2+Q3, method="hk")

# image of the results
# chr locations
chr1 <- as.numeric(matrix(unlist(strsplit(colnames(result),"@")),
                   ncol=2,byrow=TRUE)[,2])
chr8 <- as.numeric(matrix(unlist(strsplit(rownames(result),"@")),
                   ncol=2,byrow=TRUE)[,2])
# image plot
image(chr1, chr8, t(result), las=1, col=rev(rainbow(256,start=0,end=2/3)))

# do the same, allowing the QTLs on chr 1 and 13 to interact
result2 <- scanqtl(fake.f2, pheno.col=1, chr=qc, pos=pos,
                   formula=y~Q1+Q2+Q3+Q1:Q3, method="hk")
# image plot
image(chr1, chr8, t(result2), las=1, col=rev(rainbow(256,start=0,end=2/3)))
}

\seealso{ \code{\link{fitqtl}}, \code{\link{makeqtl}},
  \code{\link{refineqtl}} }

\keyword{models}

\name{effectplot}
\alias{effectplot}

\title{Plot phenotype means against genotypes at one or two markers}

\description{
  Plot the phenotype means for each group defined by the genotypes at
  one or two markers (or the values at a discrete covariate).
}

\usage{
effectplot(cross, pheno.col=1, mname1, mark1, geno1, mname2, mark2,
           geno2, main, ylim, xlab, ylab, col, add.legend=TRUE,
           legend.lab, draw=TRUE, var.flag=c("pooled","group"))
}

\arguments{
  \item{cross}{An object of class \code{cross}.}
  \item{pheno.col}{Column number in the phenotype matrix to be
    drawn in the plot.  One may also give a character string matching
   a phenotype name. Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
  \item{mname1}{Name for the first marker or pseudomarker.
  Pseudomarkers (that is, non-marker positions on the imputation grid)
  may be referred to in a form like \code{"5@30.3"}, for position 30.3
  on chromosome 5.}
  \item{mark1}{Genotype data for the first marker.  If unspecified,
    genotypes will be taken from the data in the input cross object,
    using the name specified in \code{mname1}.}
  \item{geno1}{Optional labels for the genotypes (or classes in a covariate).}
  \item{mname2}{Name for the second marker or pseudomarker (optional).}
  \item{mark2}{Like \code{mark1} (optional).}
  \item{geno2}{Optional labels for the genotypes (or classes in a covariate).}
  \item{main}{Optional figure title.}
  \item{ylim}{Optional y-axis limits.}
  \item{xlab}{Optional x-axis label.}
  \item{ylab}{Optional y-axis label.}
  \item{col}{Optional vector of colors for the different line segments.}
  \item{add.legend}{A logical value to indicate whether to add a legend.}
  \item{legend.lab}{Optional title for the legend.}
  \item{draw}{A logical value to indicate generate the plot or not. If
    FALSE, no figure will be plotted and this function can be used to
    calculate the group means and standard errors.}
  \item{var.flag}{The method to calculate the group variance. "pooled"
    means to use the pooled variance and "group" means to calculate from
    individual group.}
}

\value{
  A data.frame containing the phenotype means and standard errors
  for each group.
}

\details{
  In the plot, the y-axis is the phenotype.  In the case of one marker,
  the x-axis is the genotype for that marker. In the case of two
  markers, the x-axis is for different genotypes of the second marker,
  and the genotypes of first marker are represented by lines in
  different colors.  Error bars are plotted at \eqn{\pm}{+/-} 1 SE.

  The results of \code{sim.geno} are used; if they are not available,
  \code{sim.geno} is run with \code{n.draws=16}.  The average phenotype
  for each genotype group takes account of missing genotype data by
  averaging across the imputations.  The SEs take account of both the
  residual phenotype variation and the imputation error.
}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(1, 13, "X"))}

# impute genotype data
\dontrun{fake.f2 <- sim.geno(fake.f2, step=5, n.draws=64)
}\dontshow{fake.f2 <- sim.geno(fake.f2, step=5, n.draws=8)
}

########################################
# one marker plots
########################################
### plot of genotype-specific phenotype means for 1 marker
mname <- find.marker(fake.f2, 1, 37) # marker D1M437
effectplot(fake.f2, pheno.col=1, mname1=mname)

### output of the function contains the means and SEs
output <- effectplot(fake.f2, mname1=mname)
output

### plot a phenotype
# Plot of sex-specific phenotype means,
# note that "sex" must be a phenotype name here
effectplot(fake.f2, mname1="sex", geno1=c("F","M"))
# alternatively:
sex <- pull.pheno(fake.f2, "sex")
effectplot(fake.f2, mname1="Sex", mark1=sex, geno1=c("F","M"))

########################################
# two markers plots
########################################

### plot two markers
# plot of genotype-specific phenotype means for 2 markers
mname1 <- find.marker(fake.f2, 1, 37) # marker D1M437
mname2 <- find.marker(fake.f2, 13, 24) # marker D13M254
effectplot(fake.f2, mname1=mname1, mname2=mname2)

### plot two pseudomarkers
#####  refer to pseudomarkers by their positions
effectplot(fake.f2, mname1="1@35", mname2="13@25")

#####  alternatively, find their names via find.pseudomarker
pmnames <- find.pseudomarker(fake.f2, chr=c(1, 13), c(35, 25))
effectplot(fake.f2, mname1=pmnames[1], mname2=pmnames[2])

### Plot of sex- and genotype-specific phenotype means
mname <- find.marker(fake.f2, 13, 24) # marker D13M254
# sex and a marker
effectplot(fake.f2, mname1=mname, mname2="Sex",
           mark2=sex, geno2=c("F","M"))

# Same as above, switch role of sex and the marker
# sex and marker
effectplot(fake.f2, mname1="Sex", mark1=sex,
           geno1=c("F","M"), mname2=mname)

# X chromosome marker
mname <- find.marker(fake.f2, "X", 14) # marker DXM66
effectplot(fake.f2, mname1=mname)

# Two markers, including one on the X
mnames <- find.marker(fake.f2, c(13, "X"), c(24, 14))
effectplot(fake.f2, mname1=mnames[1], mname2=mnames[2])
}

\author{Hao Wu; Karl W Broman, \email{broman@wisc.edu} }

\seealso{ \code{\link{plotPXG}}, \code{\link{find.marker}},
  \code{\link{effectscan}}, \code{\link{find.pseudomarker}} }

\keyword{hplot}

\name{cbind.scanoneperm}
\alias{cbind.scanoneperm}

\title{Combine columns from multiple scanone permutation results}

\description{
  Concatenate the columns from different runs of
  \code{\link{scanone}} with \code{n.perm > 0}.
}

\usage{
\method{cbind}{scanoneperm}(\dots, labels)
}

\arguments{
 \item{\dots}{A set of objects of class \code{scanoneperm}. These are
   the permutation results from \code{\link{scanone}} (that is,
   when \code{n.perm > 0}), generally run with different phenotypes or
   methods.}
 \item{labels}{A vector of character strings, of length 1 or of the same
   length as the input \code{\dots}, to be appended to the column names
   in the output.}
}

\value{The concatenated input, as a \code{scanoneperm} object. If
  different numbers of permutation replicates were used, those columns
  with fewer replicates are padded with missing values (\code{NA}).}

\details{
  The aim of this function is to concatenate the results from multiple
  runs of a permutation test \code{\link{scanone}}, generally for
  different phenotypes and/or methods, to be used in parallel with
  \code{\link{c.scanone}}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
fake.f2 <- calc.genoprob(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(18:19,"X"))}
operm1 <- scanone(fake.f2, method="hk", n.perm=10, perm.Xsp=TRUE)
operm2 <- scanone(fake.f2, method="em", n.perm=5, perm.Xsp=TRUE)

operm <- cbind(operm1, operm2, labels=c("hk","em"))
summary(operm)
}

\seealso{ \code{\link{summary.scanoneperm}},
  \code{\link{scanone}}, \code{\link{c.scanoneperm}},
  \code{\link{c.scanone}} }

\keyword{manip}

\name{getid}
\alias{getid}

\title{Pull out the individual identifiers from a cross}

\description{
  Pull out the individual identifiers from a cross object.
}

\usage{getid(cross)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
}

\value{
  A vector of individual identifiers, pulled from the phenotype data (a
  column named \code{id} or \code{ID}).

  If there are no such identifiers in the cross, the function returns
  \code{NULL}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)

# create an ID column
fake.f2$pheno$id <- paste("ind", sample(nind(fake.f2)), sep="")

getid(fake.f2)
}

\seealso{ \code{\link{subset.cross}}, \code{\link{top.errorlod}} }


\keyword{utilities}

\name{mqmtestnormal}
\alias{mqmtestnormal}

\title{ Shapiro normality test used for MQM }

\description{
  Wraps a shapiro's normality test from the nortest package.
  This function is used in MQM to test the normality of the
  trait under investigation
}

\usage{
  mqmtestnormal(cross, pheno.col = 1,significance=0.05, verbose=FALSE)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{pheno.col}{
Column number in the phenotype matrix which should be used as the phenotype. This can be a vector of integers. % \phenocol
  }
  \item{significance}{
    Significance level used in the normality test. Lower significance levels will accept larger deviations from normality.
  }
  \item{verbose}{
    If TRUE, print result as well as return it.
  }
}

\value{
    Boolean indicating normality of the trait in pheno.col. (FALSE
    when not normally distributed.)
}

\details{
        For augmented data (as from \code{\link{mqmaugment}}), the cross
        is first reduced to distinct individuals. Furthermore the shapiro used to
        test normality works only for 3 <= nind(cross) <= 5000
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
    \itemize{
    \item \code{\link[stats]{shapiro.test}} - Function wrapped by our mqmtestnormal
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
data(multitrait)

# test normality of 7th phenotype
mqmtestnormal(multitrait, pheno.col=7)

# take log
multitrait <- transformPheno(multitrait, pheno.col=7, transf=log)

# test again
mqmtestnormal(multitrait, pheno.col=7)
}

\keyword{ utilities }

\name{convert.scanone}
\alias{convert.scanone}

\title{Convert output from scanone for R/qtl version 0.98}

\description{
  Convert the output from scanone from the format used in R/qtl version
  0.97 and earlier to that used in version 0.98 and later.
}

\usage{
\method{convert}{scanone}(object, \dots)
}

\arguments{
 \item{object}{Output from the function \code{\link{scanone}}, for
   R/qtl version 0.97 and earlier.}
 \item{\dots}{Ignored at this point.}
}

\value{
  The same scanone output, but revised for use with R/qtl version 0.98
  and later.
}

\details{
  Previously, inter-marker locations were named as, for example,
  \code{loc7.5.c3}; these were changed to \code{c3.loc7.5}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
\dontrun{out.new <- convert(out.old)}
}

\seealso{ \code{\link{scanone}}, \code{\link{convert.scantwo}} }

\keyword{manip}

\name{mqmaugment}
\alias{mqmaugment}

\title{MQM augmentation}

\description{
Fill in missing genotypes for MQM mapping. For each missing or incomplete
marker it fills in (or `augments') all possible genotypes, thus creating new
candidate `individuals'. The probability of each indidual is calculated using
information on neighbouring markers and recombination frequencies. When a
genotype of an augmented genotype is less likely than the \code{minprob}
parameter it is dropped from the dataset. The \emph{augmented} list of
individuals is returned in a new cross object. For a full discussion on
augmentation see the MQM tutorial online.
}

\usage{
mqmaugment(cross, maxaugind=82, minprob=0.1,
           strategy=c("default","impute","drop"),
           verbose=FALSE)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{maxaugind}{
    Maximum number of augmentations per individual.  The default of 82
    allows for six missing markers for an individual in a BC cross
    (\eqn{2^6=64}) and four missing markers in an F2 (\eqn{3^4=81}). When a
    large number of markers are missing this default number is quickly
    reached.
  }
  \item{minprob}{
    Return individuals with augmented genotypes that have at least this probability of
    occurring. \code{minprob} is a value between 0 and 1. For example a value of 0.5
    will drop all genotypes that are half as likely as the most likely
    genotype (candidate of the individual). The default value of 0.1 will drop
    all genotypes that are less likely of ocurring than 1 in 10, compared
    against the most likely genotype. Use a value of 1.0 to return a single
    filled in genotype for each individual.
 }
  \item{strategy}{
    When individuals have too much missing data and augmentation fails three
    options are provided:
    1. \code{"default"}: Calculate genotypes at missing marker positions,
    accounting for \code{minprob}, and add this individual to the set.
    2. \code{"impute"}: Calculate the most likely genotypes at missing marker
    positions and impute \code{maxaugind} individual-variants around the most
    likely genotype.
    3. \code{"drop"}: Drop individuals that cannot be augmented from the
    dataset, this option is not advised because information from the dropped
    individuals will be lost.
 }
  \item{verbose}{ If TRUE, give verbose output }
}

\value{
  Returns the cross object with augmented individuals (many individuals from
  the data set will be repeated multiple times). Some individuals may have been
  dropped completely when the probability falls below \code{minprob}. An added
  component to the cross object named \code{mqm} contains information on
  exactly which individuals are retained and repeated.
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\note{
The sex chromosome 'X' is treated like autosomes during augmentation.
With an F2 the sex chromosome is not considered. This will change in
a future version of MQM.
Run with \code{verbose=TRUE} to verify how many individuals are augmented
versus moved to the second augmentation round. This could have an effect
on the resulting dataset or check the return \code{cross$mqm} values. Compare
results by using \code{minprob=1}.
}

\seealso{
  \itemize{
    \item \code{\link{fill.geno}} - Alternative routine for estimating missing data
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
% \input{"inst/doc/Sources/MQM/mqm/standard_example.txt"}
data(map10)                    # Genetic map modeled after mouse

# simulate a cross (autosomes 1-10)
qtl <- c(3,15,1,0)             # QTL model: chr, pos'n, add've & dom effects
cross <- sim.cross(map10[1:10],qtl,n=100,missing.prob=0.01)

# MQM
crossaug <- mqmaugment(cross)  # Augmentation
cat(crossaug$mqm$Nind,'real individuals retained in dataset',
    crossaug$mqm$Naug,'individuals augmented\n')

result <- mqmscan(crossaug)    # Scan

# show LOD interval of the QTL on chr 3
lodint(result,chr=3)
% -----^^ inst/doc/Sources/MQM/mqm/standard_example.txt ^^-----
}

\keyword{utilities}

\name{ntyped}
\alias{ntyped}

\title{Number of genotypes}

\description{
  Count the number of genotypes for each individual or each marker
  in a cross.
}

\usage{ntyped(cross, what=c("ind","mar"))}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{what}{Indicates whether to count genotypes for each
   individual or each marker.}
}

\value{
  A vector containing the number of genotypes for each
  individual or for each marker.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)

# plot number of genotypes for each individual
plot(ntyped(listeria))

# plot number of genotypes for each marker
plot(ntyped(listeria, what="mar"))
}

\seealso{ \code{\link{nmissing}},  \code{\link{summary.cross}},
          \code{\link{nind}}, \code{\link{totmar}} }

\keyword{utilities}

\name{xaxisloc.scanone}
\alias{xaxisloc.scanone}

\title{Get x-axis locations in scanone plot}

\description{
  Get x-axis locations for given cM positions on given chromosomes in a
  plot from \code{\link{plot.scanone}})
}

\usage{
xaxisloc.scanone(out, thechr, thepos, chr, gap=25)
}

\arguments{
  \item{out}{An object of class \code{"scanone"}, as output by
    \code{\link{scanone}}.  This must be identical to what was used
  in the call to \code{\link{plot.scanone}}.}
  \item{thechr}{Chromosome IDs at which x-axis locations are to be determined.}
  \item{thepos}{Chromosome positions at which x-axis locations are to be
    determined.}
  \item{chr}{Optional vector specifying which chromosomes were plotted.
  This must be identical to what was used in the call to
  \code{\link{plot.scanone}}.}
  \item{gap}{Gap separating chromosomes (in cM).  This must be identical
  to what was used in the call to \code{\link{plot.scanone}}.}
}

\value{A numeric vector of x-axis locations.}

\details{
  This function allows you to identify the x-axis locations in a plot of
  genome scan results, produced by
  \code{\link{plot.scanone}}.  This is useful for adding
  annotations, such as text or arrows.

  The arguments \code{out}, \code{chr}, and \code{gap} must match what
  was used in the call to \code{\link{plot.scanone}}.

  The arguments \code{thechr} and \code{thepos} indicate the genomic
  positions for which x-axis locations are desired.  If they both have
  length > 1, they must have the same length.  If one has length > 1 and
  one has length 1, the one with length 1 is expanded to match.
}


\examples{
data(hyper)
\dontshow{hyper <- subset(hyper, c(1,4,6,15))}
hyper <- calc.genoprob(hyper)
out <- scanone(hyper, method="hk")
plot(out, chr=c(1, 4, 6, 15))

# add arrow and text to indicate peak LOD score
mxout <- max(out)
x <- xaxisloc.scanone(out, mxout$chr, mxout$pos, chr=c(1,4,6,15))
arrows(x+30, mxout$lod, x+5, mxout$lod, len=0.1, col="blue")
text(x+35, mxout$lod, "the peak", col="blue", adj=c(0, 0.5))
}

\seealso{
  \code{\link{plot.scanone}},
  \code{\link{add.threshold}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{hyper}
\alias{hyper}
\docType{data}

\title{Data on hypertension}

\description{
  Data from an experiment on hypertension in the mouse.
}

\usage{data(hyper)}

\format{
  An object of class \code{cross}.  See \code{\link{read.cross}}
  for details.
}

\details{
There are 250 male backcross individuals typed at 174 markers (actually one
contains only missing values), including 4 on the X chromosome, with one
phenotype.

The phenotype is the blood pressure.  See the reference below.  Note
that, for most markers, genotypes are available on only the individuals
with extreme phenotypes.  At many markers, only recombinant individuals
were typed.
}


\references{
  Sugiyama, F., Churchill, G. A., Higgens, D. C., Johns, C.,
  Makaritsis, K. P., Gavras, H. and Paigen, B. (2001) Concordance of
  murine quantitative trait loci for salt-induced hypertension with rat
  and human loci. \emph{Genomics} \bold{71}, 70--77.
}

\source{
  Bev Paigen and Gary Churchill (The Jackson Laboratory, Bar Harbor,
  Maine)
  \url{https://phenome.jax.org/projects/Sugiyama2}
}

\examples{
data(hyper)
summary(hyper)
plot(hyper)

# Note the selective genotyping
\dontrun{plotMissing(hyper, reorder=TRUE)}

# A marker on c14 has no data; remove it
hyper <- drop.nullmarkers(hyper)
}

\seealso{ \code{\link{fake.bc}}, \code{\link{fake.f2}},
  \code{\link{fake.4way}}, \code{\link{listeria}},
  \code{\link{bristle3}}, \code{\link{bristleX}} }

\keyword{datasets}

\name{max.scantwo}
\alias{max.scantwo}

\title{Maximum peak in two-dimensional genome scan}

\description{
  Print the pair of loci with the largest LOD score in the results of
  \code{\link{scantwo}}.
}

\usage{
\method{max}{scantwo}(object, lodcolumn=1,
    what=c("best", "full", "add", "int"),
    na.rm=TRUE, \dots)
}

\arguments{
 \item{object}{An object of class \code{scantwo}, the output of the
   function \code{\link{scantwo}}.}
 \item{lodcolumn}{If the scantwo results contain LOD scores for
   multiple phenotypes, this argument indicates which to use.}
 \item{what}{Indicates for which LOD score the maximum should be reported.}
 \item{na.rm}{Ignored.}
 \item{\dots}{Ignored.}
}

\value{
  An object of class \code{summary.scantwo}, to be printed by
  \code{print.summary.scantwo}, with the pair of positions with the
  maximum LOD score. (Which LOD score is considered is indicated by the
  \code{what} argument.)
}

\details{
  This is very similar to the \code{\link{summary.scantwo}}
  function, though this pulls out one pair of positions.

  If \code{what="best"}, we find the pair of positions at which the LOD
  score for the full model (2 QTL + interaction) is maximized, and then
  also print the positions on that same pair of chromosomes at which the
  additive LOD score is maximized.

  In the other cases, we pull out the pair of positions with the largest
  LOD score; which LOD score is considered is indicated by the
  \code{what} argument.
}

\section{Output of addpair}{

  \bold{Note} that, for output from \code{\link{addpair}} in which the
  new loci are indicated explicitly in the formula, the summary provided
  by \code{max.scantwo} is somewhat special.

  All arguments (except, of course, the input
  \code{object}) are ignored.

  If the formula is symmetric in the two new QTL, the output has just two LOD
  score columns: \code{lod.2v0} comparing the full model to the model
  with neither of the new QTL, and \code{lod.2v1} comparing the full
  model to the model with just one new QTL.

  If the formula is \emph{not} symmetric in the two new QTL, the output
  has three LOD score columns: \code{lod.2v0} comparing the full model
  to the model with neither of the new QTL, \code{lod.2v1b} comparing
  the full model to the model in which the first of the new QTL is
  omitted, and \code{lod.2v1a} comparing the full model to the model
  with the second of the new QTL omitted.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
fake.f2 <- calc.genoprob(fake.f2, step=10)
out.2dim <- scantwo(fake.f2, method="hk")
max(out.2dim)
}

\seealso{ \code{\link{scantwo}}, \code{\link{plot.scantwo}},
          \code{\link{summary.scantwo}} }

\keyword{print}

\name{mqmplot.heatmap}
\alias{mqmplot.heatmap}

\title{Heatmap of a genome of MQM scan on multiple phenotypes}

\description{
  Plotting routine to display a heatmap of results obtained from a multiple-QTL model on multiple phenotypes (the
  output of \code{\link{mqmscanall}})
}

\usage{
mqmplot.heatmap(cross, result, directed=TRUE, legend=FALSE, breaks =
c(-100,-10,-3,0,3,10,100), col =
c("darkblue","blue","lightblue","yellow","orange","red"), \dots)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{result}{ Result object from mqmscanall, the object needs to be of class \code{mqmmulti} }
  \item{directed}{ Take direction of QTLs into account (takes more time because of QTL direction calculations }
  \item{legend}{ If TRUE, add a legend to the plot }
  \item{breaks}{ Color break points for the LOD scores }
  \item{col}{ Colors used between breaks }
  \item{\dots}{Additional arguments passed to the \code{\link{image}} function }
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
data(multitrait)
\dontshow{multitrait <- subset(multitrait, chr=1:2, ind=!apply(multitrait$pheno, 1, function(a) any(is.na(a))))
multitrait$pheno <- multitrait$pheno[,1:3]}
multitrait <- fill.geno(multitrait) # impute missing genotype data
result <- mqmscanall(multitrait, logtransform=TRUE)
mqmplot.heatmap(multitrait,result)
}

\keyword{ hplot }

\name{mqmautocofactors}
\alias{mqmautocofactors}

\title{Automatic setting of cofactors, taking marker density into account}

\description{
  Sets cofactors, taking underlying marker density into account. Together
  with \code{mqmscan} cofactors are selected through backward elimination.
}

\usage{
mqmautocofactors(cross, num=50, distance=5, dominance=FALSE, plot=FALSE, verbose=FALSE)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{num}{ Number of cofactors to set (warns when setting too many cofactors).}
  \item{distance}{ Minimal distance between two cofactors, in cM.}
  \item{dominance}{If TRUE, create a cofactor list that is safe to use
  with the dominance scan mode of MQM. See \code{\link{mqmscan}} for details.}
  \item{plot}{If TRUE, plots a genetic map displaying the selected markers as cofactors.}
  \item{verbose}{If TRUE, give verbose output.}
}

\value{
    A list of cofactors to be used with \code{\link{mqmscan}}.
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
    data(hyper)                     # hyper dataset
    \dontshow{hyper <- hyper[1:5]}
    hyperfilled <- fill.geno(hyper)
    cofactors <- mqmautocofactors(hyperfilled,15)	# Set 15 Cofactors
    result <- mqmscan(hyperfilled,cofactors)	# Backward model selection
    mqmgetmodel(result)
}

\keyword{ models }

\name{readMWril}
\alias{readMWril}

\title{Read data for 4- or 8-way RIL}

\description{
  Data for a set of 4- or 8-way recombinant inbred lines (RIL) is read
  from a pair of comma-delimited files and converted
  into an object of class \code{cross}.
  We require chromosome
  assignments for the genetic markers, and assume that markers are in
  their correct order.
}

\usage{
readMWril(dir="", rilfile, founderfile,
          type=c("ri4self", "ri4sib", "ri8self", "ri8selfIRIP1", "ri8sib", "bgmagic16"),
          na.strings=c("-","NA"), rotate=FALSE, \dots)
}

\arguments{
  \item{dir}{Directory in which the data files will be found.  In
    Windows, use forward slashes (\code{"/"}) or double backslashes
    (\code{"\\\\"}) to specify directory trees.}
  \item{rilfile}{Comma-delimited file for the RIL, in the \code{"csv"}
    format described in the help file for \code{\link{read.cross}}.}
  \item{founderfile}{File with founder strains' genotypes, in the same
  orientation as the \code{rilfile}, but with just marker names and the
  founders' marker genotypes.}
  \item{type}{The type of RIL.}
  \item{na.strings}{A vector of strings which are to be interpreted as
    missing values.  For the
    \code{csv} formats, these are interpreted globally
    for the entire
    file, so missing value codes in phenotypes must not be valid
    genotypes, and vice versa. For the \code{gary} format, these are
    used only for the phenotype data.}
  \item{rotate}{If TRUE, the \code{rilfile} and \code{founderfile} are
  rotated (really transposed), with rows corresponding to markers
  and columns corresponding to individuals.}
  \item{\dots}{Additional arguments, passed to the function
    \code{\link[utils]{read.table}} in the case of
    \code{csv} and \code{csvr} formats.  In particular, one may use the
    argument
    \code{sep} to specify the field separator (the default is a comma)
    and \code{dec} to specify the character used for the decimal point
    (the default is a period).}
}

\value{
  An object of class \code{cross}; see the help file for
  \code{\link{read.cross}} for details.

  An additional component \code{crosses} is included; this is a matrix
  indicating the crosses used to generate the RIL.
}

\details{
  The \code{rilfile} should include a phenotype \code{cross} containing
  character strings of the form \code{ABCDEFGH}, indicating the cross
  used to generate each RIL.  The genotypes should be coded as
  \bold{integers} (e.g., 1 and 2).

  The founder strains in the \code{founderfile} should be the strains
  \code{A}, \code{B}, \code{C}, \dots, as indicated in the \code{cross}
  phenotype.

  The default arrangement of the files is to have markers as columns and
  individuals/founders as rows. If \code{rotate=TRUE}, do the opposite:
  markers as rows and individuals/founders as columns.
}



\examples{
\dontrun{
ril <- read.cross("../Data", "ril_data.csv", "founder_geno.csv", "ri4self",
                  rotate=TRUE)}
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\seealso{ \code{\link{read.cross}}, \code{\link{sim.cross}} }

\keyword{IO}

\name{cim}
\alias{cim}

\title{Composite interval mapping}

\description{
  Composite interval mapping by a scheme from QTL Cartographer:
  forward selection at the markers (here, with filled-in genotype data)
  to a fixed number, followed by interval mapping with the selected
  markers as covariates, dropping marker covariates if they are within
  some fixed window size of the location under test.
}

\usage{
cim(cross, pheno.col=1, n.marcovar=3, window=10,
    method=c("em", "imp", "hk", "ehk"),
    imp.method=c("imp", "argmax"), error.prob=0.0001,
    map.function=c("haldane", "kosambi", "c-v", "morgan"),
    addcovar=NULL, n.perm)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  One may also give a character string matching
   a phenotype name. Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{n.marcovar}{Number of marker covariates to use.}
 \item{window}{Window size, in cM.}
 \item{method}{Indicates whether to use the EM algorithm,
   imputation, Haley-Knott regression, or the extended Haley-Knott
   method.}
 \item{imp.method}{Method used to impute any missing marker genotype
   data.}
 \item{error.prob}{Genotyping error probability assumed when imputing
   the missing marker genotype data.}
 \item{map.function}{Map function used when imputing the missing marker
   genotype data.}
 \item{addcovar}{Optional numeric matrix of additional covariates to include.}
 \item{n.perm}{If specified, a permutation test is performed rather than
   an analysis of the observed data.  This argument defines the number
   of permutation replicates.}
}


\details{
  We first use \code{\link{fill.geno}} to impute any missing marker
  genotype data, either via a simple random imputation or using the
  Viterbi algorithm.

  We then perform forward selection to a fixed number of markers.  These
  will be used (again, with any missing data filled in) as covariates in
  the subsequent genome scan.
}

\value{
  The function returns an object of the same form as the function
  \code{\link{scanone}}:

  If \code{n.perm} is missing, the function returns the scan results as
  a data.frame with three columns: chromosome, position, LOD score.
  Attributes indicate the names and positions of the chosen marker
  covariates.

  If \code{n.perm} > 0, the function results the results of a
  permutation test: a vector giving the genome-wide maximum LOD score in
  each of the permutations.
}

\references{
  Jansen, R. C. (1993) Interval mapping of multiple quantitative trait
  loci.  \emph{Genetics}, \bold{135}, 205--211.

  Jansen, R. C. and Stam, P. (1994) High resolution of quantitative
  traits into multiple loci via interval mapping.  \emph{Genetics},
  \bold{136}, 1447-1455.

  Zeng, Z. B. (1993) Theoretical basis for separation of multiple linked
  gene effects in mapping quantitative trait loci.
  \emph{Proc. Natl. Acad. Sci. USA}, \bold{90}, 10972--10976.

  Zeng, Z. B. (1994) Precision mapping of quantitative trait loci.
  \emph{Genetics}, \bold{136}, 1457--1468.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(hyper)
hyper <- calc.genoprob(hyper, step=2.5)
\dontshow{hyper <- subset(hyper, chr=c(1,4,6,15))}

out <- scanone(hyper)
out.cim <- cim(hyper, n.marcovar=3)
plot(out, out.cim, chr=c(1,4,6,15), col=c("blue", "red"))

add.cim.covar(out.cim, chr=c(1,4,6,15))
}

\seealso{ \code{\link{add.cim.covar}}, \code{\link{scanone}},
  \code{\link{summary.scanone}}, \code{\link{plot.scanone}},
  \code{\link{fill.geno}}}

\keyword{models}

\name{geno.crosstab}
\alias{geno.crosstab}

\title{Create table of two-locus genotypes}

\description{
  Create a cross tabulation of the genotypes at a pair of markers.
}

\usage{
geno.crosstab(cross, mname1, mname2, eliminate.zeros=TRUE)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{mname1}{The name of the first marker (as a character
 string). (Alternatively, a vector with the two character strings, in
 which case \code{mname2} should not be given.)}
 \item{mname2}{The name of the second marker (as a character string).}
 \item{eliminate.zeros}{If TRUE, don't show the rows and columns that
 have no data.}
}

\value{
  A matrix containing the number of individuals having each possible
  pair of genotypes.  Genotypes for the first marker are in the rows;
  genotypes for the second marker are in the columns.
}


\examples{
data(hyper)
geno.crosstab(hyper, "D1Mit123", "D1Mit156")
geno.crosstab(hyper, "DXMit22", "DXMit16")
geno.crosstab(hyper, c("DXMit22", "DXMit16"))
}

\seealso{ \code{\link{geno.table}},
  \code{\link{find.marker}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{utilities}

\name{summary.comparegeno}
\alias{summary.comparegeno}

\title{Print pairs of individuals with similar genotype data.}

\description{
  Prints a summary the output from \code{\link{comparegeno}} that
  includes pairs of individuals whose proportion of matching genotypes
  is above a chosen threshold.
}

\usage{
\method{summary}{comparegeno}(object, thresh=0.9, \dots)
}

\arguments{
 \item{object}{An object of class \code{comparegeno}, the output of the
   function \code{\link{comparegeno}}.}
 \item{thresh}{Threshold on the proportion of matching genotypes.}
 \item{\dots}{Ignored at this point.}
}

\value{
  A data frame with each row being a pair of individuals and columns
  including the individual identifiers (via \code{getid}, or just as
  numeric indexes) along with the proportion of matching genotypes.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
cg <- comparegeno(fake.f2)
summary(cg, 0.7)
}

\seealso{ \code{\link{comparegeno}}, \code{\link{plot.comparegeno}} }

\keyword{print}

\name{drop.markers}
\alias{drop.markers}

\title{Drop a set of markers}

\description{
  Drop a vector of markers from the data matrices and genetic maps.
}

\usage{
drop.markers(cross, markers)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{markers}{A character vector of marker names.}
}

\value{
  The input object, with any markers in the vector \code{markers} removed
  from the genotype data matrices, genetic maps, and, if applicable, any
  derived data (such as produced by \code{\link{calc.genoprob}}).
  (It might be a good idea to re-derive such things after using this
  function.)
}

\examples{
data(listeria)
listeria2 <- drop.markers(listeria, c("D10M44","D1M3","D1M75"))
}

\seealso{ \code{\link{drop.nullmarkers}}, \code{\link{pull.markers}}, \code{\link{geno.table}},
  \code{\link{clean.cross}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{manip}

\name{pull.rf}
\alias{pull.rf}

\title{Pull out recombination fractions or LOD scores from a cross object}

\description{
  Pull out either the pairwise recombination fractions or the LOD
  scores, as calculated by \code{\link{est.rf}}, from a cross object.
}

\usage{pull.rf(cross, what=c("rf", "lod"), chr)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{what}{Indicates whether to pull out a matrix of estimated
 recombination fractions or a matrix of LOD scores.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
}

\value{
An object of class \code{"rfmatrix"}, which is a matrix of either
estimated recombination fractions between all marker pairs or of LOD
scores (for the test of rf=1/2) for all marker pairs.

The genetic map is included as an attribute.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=5)}
fake.f2 <- est.rf(fake.f2)
rf <- pull.rf(fake.f2)
lod <- pull.rf(fake.f2, "lod")
plot(rf[1,], lod[1,], xlab="rec frac", ylab="LOD score")
marker <- markernames(fake.f2, chr=5)[6]
par(mfrow=c(2,1))
plot(rf, marker, bandcol="gray70")
plot(lod, marker, bandcol="gray70")
}

\seealso{ \code{\link{est.rf}}, \code{\link{plot.rfmatrix}}, \code{\link{plotRF}} }


\keyword{utilities}

\name{markerlrt}
\alias{markerlrt}

\title{General likelihood ratio test for association between marker pairs}

\description{
  Calculate a LOD score for a general likelihood ratio test for each
  pair of markers, to assess their association.
}

\usage{
markerlrt(cross)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
}

\value{
  The input \code{cross} object is returned with a component, \code{rf},
  added.  This is a matrix of size (tot.mar x tot.mar).  The diagonal
  contains the number of typed meioses per marker, the upper and lower triangles
  each contain the LOD scores.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(badorder)
badorder <- markerlrt(badorder)
plotRF(badorder)
}

\seealso{ \code{\link{plotRF}}, \code{\link{est.rf}},
  \code{\link{badorder}} }

\keyword{utilities}

\name{groupclusteredheatmap}
\alias{groupclusteredheatmap}

\title{Retrieving groups of traits after clustering}

\description{
  Retrieving groups of clustered traits from the output of mqmplot.clusteredheatmap.
}

\usage{
  groupclusteredheatmap(cross, clusteredheatmapresult, height)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{clusteredheatmapresult}{ Resultint dendrogram object from \code{\link{mqmplot.clusteredheatmap}} }
  \item{height}{ Height at which to 'cut' the dendrogram, a higher cut-off gives less but larger groups.
  Height represents the maximum distance between two traits clustered together using hclust. the 'normal'
  behaviour of bigger groups when using a higher heigh cut-off depends on the tree stucture and the amount
  of traits clustered using \code{\link{mqmplot.clusteredheatmap}} }
}

\value{
  A list containing groups of traits which were clustered together with a distance less that \code{height}
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
data(multitrait)
\dontshow{multitrait <- subset(multitrait, chr=1:2, ind=!apply(multitrait$pheno, 1, function(a) any(is.na(a))))
multitrait$pheno <- multitrait$pheno[,1:3]}
multitrait <- fill.geno(multitrait) # impute missing genotype data
result <- mqmscanall(multitrait, logtransform=TRUE)
cresults <- mqmplot.clusteredheatmap(multitrait,result)
groupclusteredheatmap(multitrait,cresults,10)
}

\keyword{utilities}

\name{c.cross}
\alias{c.cross}

\title{Combine data for QTL experiments}

\description{
  Concatenate the data for multiple QTL experiments.
}

\usage{
\method{c}{cross}(\dots)
}

\arguments{
 \item{\dots}{A set of objects of class \code{cross}.  See
   \code{\link{read.cross}} for details.  These must all either be of the
   same cross type or be a combination of backcrosses and intercrosses.
   All crosses must have the same number of chromosomes and chromosome
   names, and the same marker orders and positions, though the set of
   markers need not be precisely the same.}
}

\value{
  The concatenated input, as a \code{cross} object. Additional
  columns are added to the phenotype data indicating which cross an
  individual comes from; another column indicates cross type (0=BC,
  1=intercross), if there are crosses of different types.  The crosses
  are not required to have exactly the same set of phenotypes;
  phenotypes with the same names are assumed to be the same.

  If the crosses have different sets of markers, we
  interpolate marker order, but the cM positions of markers that are in
  common between crosses must be precisely the same in the different crosses.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
junk <- fake.f2
junk <- c(fake.f2,junk)
}

\seealso{ \code{\link{subset.cross}} }

\keyword{manip}

\name{mqmsetcofactors}
\alias{mqmsetcofactors}

\title{Set cofactors at fixed intervals, to be used with MQM}

\description{
  Set cofactors, at fixed marker intervals. Together
  with \code{mqmscan} cofactors are selected through backward elimination.
}

\usage{
mqmsetcofactors(cross, each = NULL, cofactors=NULL, sexfactors=NULL, verbose=FALSE)
}

\arguments{
  \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
  \item{each}{ Every 'each' marker will be used as a cofactor, when each is used the \code{cofactors} and \code{sexfactors} parameter is ignored }
  \item{cofactors}{
List of cofactors to be analysed in the QTL model. To set cofactors use \code{\link{mqmautocofactors}} or \code{mqmsetcofactors}; when \code{each} is set, this parameter is ignored }
  \item{sexfactors}{ list of markers which should be treated as dominant cofactors (sexfactors), when \code{each} is set, this parameter is ignored }
  \item{verbose}{If TRUE, print tracing information.}
}

\value{
    An list of cofactors to be passed into \code{\link{mqmscan}}.
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
    data(hyper)                                 # Hyper dataset
    \dontshow{hyper <- hyper[1:5]}
    hyperfilled <- fill.geno(hyper)
  # Automatic cofactors every third marker
    cofactors <- mqmsetcofactors(hyperfilled,3)
    result <- mqmscan(hyperfilled,cofactors)	# Backward model selection
    mqmgetmodel(result)
  #Manual cofactors at markers 3,6,9,12,40 and 60
  cofactors <- mqmsetcofactors(hyperfilled,cofactors=c(3,6,9,12,40,60))
    result <- mqmscan(hyperfilled,cofactors)	# Backward model selection
    mqmgetmodel(result)
}

\keyword{ models }

\name{plotMissing}
\alias{plotMissing}

\title{Plot grid of missing genotypes}

\description{
  Plot a grid showing which genotypes are missing.
}

\usage{
plotMissing(x, chr, reorder=FALSE, main="Missing genotypes",
             alternate.chrid=FALSE, \dots)
}

\arguments{
  \item{x}{An object of class \code{cross}.  See
    \code{\link{read.cross}} for details. }
  \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{reorder}{Specify whether to reorder individuals according to
  their phenotypes.
  \tabular{ll}{
    FALSE \tab Don't reorder \cr
    TRUE  \tab Reorder according to the sum of the phenotypes \cr
    n \tab Reorder according to phenotype n}}
  \item{main}{Title to place on plot.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{\dots}{Ignored at this point.}
}

\details{
  Uses \code{\link[graphics]{image}} to plot a grid with black pixels where the
  genotypes are missing.  For intercross and 4-way cross data, gray
  pixels are plotted for the partially missing genotypes (for example,
  "not AA").
}

\value{None.}

\examples{
data(fake.f2)
plotMissing(fake.f2)
}

\seealso{ \code{\link{plot.cross}},  \code{\link{geno.image}},
  \code{\link[graphics]{image}} }


\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{mqmplot.cofactors}
\alias{mqmplot.cofactors}

\title{Plot cofactors on the genetic map}

\description{
  Plots cofactors as created by \code{\link{mqmsetcofactors}}
  or \code{\link{mqmautocofactors}} on the genetic map.
}

\usage{
mqmplot.cofactors(cross,cofactors, \dots)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{cofactors}{
List of cofactors to be analysed in the QTL model. To set cofactors use \code{\link{mqmautocofactors}} or \code{mqmsetcofactors}}. % \mqmcofactors
  \item{\dots}{Passed to \code{\link{plot.qtl}}}
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
data(multitrait)
cof1 <- mqmsetcofactors(multitrait,20)
cof2 <- mqmsetcofactors(multitrait,10)
op <- par(mfrow=c(2,1))
mqmplot.cofactors(multitrait,cof1,col="blue")
mqmplot.cofactors(multitrait,cof2,col="blue")
op <- par(mfrow=c(1,1))
}

\keyword{ hplot }

\name{shiftmap}
\alias{shiftmap}

\title{Shift starting points in genetic maps}

\description{
  Shift starting points in a genetic map to a set of defined positions
}

\usage{
shiftmap(object, offset=0)
}
\arguments{
 \item{object}{An object of class \code{cross} (see
   \code{\link{read.cross}} for details) or \code{map} (see
   \code{\link{sim.map}} for details).}
 \item{offset}{Defines the starting position
  for each chromosome.  This should be a
  single value (to be used for all chromosomes) or a vector with length
  equal to the number of chromosomes, defining individual starting
  positions for each chromosome.  For a sex-specific map (as in a 4-way
  cross), we use the same offset for both the male and female maps.}
}


\value{
If the input is a \code{map} object, a \code{map} object is returned; if
the input is a \code{cross} object, a \code{cross} object is returned.
In either case, the positions of markers are shifted so that the
starting positions are as in \code{offset}.

}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
shiftedhyper <- shiftmap(hyper, offset=0)
par(mfrow=c(1,2))
plotMap(hyper, shift=FALSE, alternate.chrid=TRUE)
plotMap(shiftedhyper, shift=FALSE, alternate.chrid=TRUE)
}


\seealso{ \code{\link{replace.map}},
  \code{\link{est.map}} }

\keyword{utilities}

\name{mqmscanall}
\alias{mqmscanall}

\title{Parallelized MQM on multiple phenotypes in a cross object}

\description{
  Parallelized QTL analysis using MQM on multiple phenotypes in a cross object (uses SNOW)
}

\usage{
    mqmscanall(cross, multicore=TRUE, n.clusters = 1,batchsize=10,cofactors=NULL, \dots)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{multicore}{ Use multiple cores (only if the package SNOW is available, otherwise this setting will be ignored)
  }
  \item{n.clusters}{ Number of parallel processes to spawn, recommended is setting this lower than the number of cores in the computer
  }
  \item{batchsize}{ Batch size. The entire set is split in
    jobs to reduce memory load per core. Each job contains batchsize number of traits per job.
  }
  \item{cofactors}{cofactors, only used when scanfunction is mqmscan.
List of cofactors to be analysed in the QTL model. To set cofactors use \code{\link{mqmautocofactors}} or \code{mqmsetcofactors}}. % \mqmcofactors
  \item{\dots}{ Parameters passed through to the \code{\link{mqmscan}} function used in
    scanning for QTLs
  }
}

\details{
  Uses \code{mqmscan} to scan for QTL's for each phenotype in the cross object.
  It is recomended that the package SNOW is installed before using this function
  on large numbers of phenotypes.
}

\value{
  Returns a MQMmulti object. This object is a list of \code{\link{scanone}}
  objects that can be plotted using \code{plot.scanone(result[[trait]])} or using \code{mqmplot.multitrait(result)}
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
    \itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
    }
}

\examples{
#Doing a multitrait analysis
data(multitrait)
\dontshow{multitrait <- subset(multitrait, chr=1:2, ind=!apply(multitrait$pheno, 1, function(a) any(is.na(a))))
multitrait$pheno <- multitrait$pheno[,1:3]}
multitrait <- calc.genoprob(multitrait)
cof <- mqmsetcofactors(multitrait,3)
multitrait <- fill.geno(multitrait)
result <- mqmscanall(multitrait,cofactors=cof,batchsize=5)
mqmplot.multitrait(result,"lines")
}

\references{
    \itemize{
% \input{"inst/doc/Sources/MQM/mqm/parallelisation_references.txt"}
        \item Rossini, A., Tierney, L., and Li, N. (2003), Simple parallel statistical computing.  \emph{R. UW Biostatistics working paper series} University of Washington. \bold{193}
        \item Tierney, L., Rossini, A., Li, N., and Sevcikova, H. (2004), The snow Package: Simple Network of Workstations. Version 0.2-1.
% -----^^ inst/doc/Sources/MQM/mqm/parallelisation_references.txt ^^-----
    }
}

\keyword{ models }

\name{chrlen}
\alias{chrlen}

\title{Chromosome lengths in QTL experiment}

\description{
  Obtain the chromosome lengths in a \code{cross} or \code{map} object.
}

\usage{
chrlen(object)
}

\arguments{
 \item{object}{An object of class \code{map} or of class \code{cross}.}
}

\value{
  Returns a vector of chromosome lengths.  If the cross has sex-specific
  maps, it returns a 2-row matrix with the two lengths for each chromosome.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
chrlen(fake.f2)

map <- pull.map(fake.f2)
chrlen(map)
}

\seealso{ \code{\link{summaryMap}}, \code{\link{pull.map}},
  \code{\link{summary.cross}} }

\keyword{print}

\name{bristle3}
\alias{bristle3}
\docType{data}

\title{Data on bristle number in Drosophila}

\description{
  Data from bristle number in chromosome 3 recombinant isogenic lines of
  \emph{Drosophila melanogaster}.
}

\usage{data(bristle3)}

\format{
  An object of class \code{cross}.  See \code{\link{read.cross}}
  for details.
}

\details{
  There are 66 chromosome 3 recombinant isogenic lines, derived from
  inbred lines that were selected for low (A) and high (B) abdominal
  bristle numbers.  A recombinant chromosome 3 was placed in an isogenic
  low background.

  There are eight phenotypes: the average and SD of the number of
  abdominal and sternopleural bristles in males and females for each
  line.

  Each line is typed at 29 genetic markers on chromosome 3.
}

\references{
  Long, A. D., Mullaney, S. L., Reid, L. A., Fry, J. D., Langley,
  C. H. and MacKay, T. F. C. (1995) High resolution mapping of genetic
  factors affecting abdominal bristle number in \emph{Drosophila
    melanogaster}.  \emph{Genetics} \bold{139}, 1273--1291.
}

%\source{
%}

\examples{
data(bristle3)
# Summaries
summary(bristle3)
plot(bristle3)

# genome scan for each of the average phenotypes
bristle3 <- calc.genoprob(bristle3, step=2)
out <- scanone(bristle3, pheno.col=c(1,3,5,7))

# Plot the results
    # maximum LOD score among four phenotypes
ym <- max(apply(out[,-(1:2)], 2, max))
plot(out, lod=1:3, ylim=c(0,ym))
plot(out, lod=4, add=TRUE, col="green")
}

\seealso{ \code{\link{bristleX}}, \code{\link{listeria}},
  \code{\link{fake.bc}}, \code{\link{fake.f2}},
  \code{\link{fake.4way}}, \code{\link{hyper}} }

\keyword{datasets}

\name{replace.map}
\alias{replace.map}
\alias{replacemap.cross}

\title{Replace the genetic map of a cross}

\description{
  Replace the map portion of a cross object.
}

\usage{
replace.map(cross, map)
\method{replacemap}{cross}(object, map)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{object}{Same as \code{cross}.}
 \item{map}{A list containing the new genetic map.  This must be the
   same length and with the same marker names as that contained in
   \code{cross}. }
}

\value{The input \code{cross} object with the genetic map replaced by
  the input \code{map}.  Maps for results from
  \code{\link{calc.genoprob}}, \code{\link{sim.geno}} and
  \code{\link{argmax.geno}} are also replaced, using interpolation if
  necessary.}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
newmap <- est.map(fake.f2)
plotMap(fake.f2, newmap)
fake.f2 <- replace.map(fake.f2, newmap)
}

\seealso{ \code{\link{pull.map}}, \code{\link{est.map}} }

\keyword{manip}

\name{c.scanoneperm}
\alias{c.scanoneperm}
\alias{rbind.scanoneperm}

\title{Combine data from scanone permutations}

\description{
  Concatenate the data for multiple runs of \code{\link{scanone}}
  with \code{n.perm > 0}.
}

\usage{
\method{c}{scanoneperm}(\dots)
\method{rbind}{scanoneperm}(\dots)
}

\arguments{
 \item{\dots}{A set of objects of class \code{scanoneperm}. (This can
 also be a list of \code{scanoneperm} objects.) These are
   the permutation results from \code{\link{scanone}} (that is,
   when \code{n.perm > 0}).  These must all have the same number of
   columns. (That is, they must have been created with the same number
   of phenotypes, and it is assumed that they were generated in
   precisely the same way.)}
}

\value{The concatenated input, as a \code{scanoneperm} object.}

\details{
  The aim of this function is to concatenate the results from multiple
  runs of a permutation test \code{\link{scanone}}, to assist with
  the case that such permutations are done on multiple processors in
  parallel.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(18:19,"X"))}
fake.f2 <- calc.genoprob(fake.f2)
operm1 <- scanone(fake.f2, method="hk", n.perm=100, perm.Xsp=TRUE)
operm2 <- scanone(fake.f2, method="hk", n.perm=50, perm.Xsp=TRUE)

operm <- c(operm1, operm2)
}

\seealso{ \code{\link{summary.scanoneperm}},
  \code{\link{scanone}}, \code{\link{cbind.scanoneperm}},
  \code{\link{c.scantwoperm}} }

\keyword{manip}

\name{inferredpartitions}
\alias{inferredpartitions}

\title{Identify inferred partitions in mapping QTL to a phylogenetic tree}

\description{
  Identify the inferred partitions for a chromosome from the results of scanPhyloQTL.
}

\usage{
inferredpartitions(output, chr, lodthreshold, probthreshold=0.9)
}
\arguments{
 \item{output}{An object output by the function
   \code{\link{scanPhyloQTL}}.}
 \item{chr}{A character string indicating the chromosome to
   consider. (It can also be a number, but it's then converted to a
   character string.)}
 \item{lodthreshold}{LOD threshold; if maximum LOD score is less than
   this, the null model is considered.}
 \item{probthreshold}{Threshold on posterior probabilities.  See Details
   below.}
}


\details{
  We consider a single chromosome, and take the maximum LOD score for
  each partition on that chromosome.  The presence of a QTL is inferred
  if at least one partition has LOD score greater than
  \code{lodthreshold}.  In this case, we then convert the LOD scores for
  the partitions to approximate posterior probabilities by taking
  \eqn{10^{LOD}}{10^LOD} and then rescaling them to sum to 1.
  These are sorted from largest to smallest, and we
  then take as the inferred partitions the smallest set whose posterior
  probabilities cumulatively add up to at least \code{probthreshold}.
}

\value{
  A vector of character strings.  If the null model (no QTL) is
  inferred, the output is \code{"null"}.  Otherwise, it is the set of
  inferred partitions.
}

\references{
  Broman, K. W., Kim, S., An\'e, C. and Payseur, B. A.  Mapping
  quantitative trait loci to a phylogenetic tree.  In preparation.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
\dontshow{set.seed(66787403)}
# example map; drop X chromosome
data(map10)
map10 <- map10[1:19]
\dontshow{map10 <- map10[1:4]}

# simulate data
x <- simPhyloQTL(4, partition="AB|CD", crosses=c("AB", "AC", "AD"),
                 map=map10, n.ind=150,
                 model=c(1, 50, 0.5, 0))

# run calc.genoprob on each cross
\dontrun{x <- lapply(x, calc.genoprob, step=2)
}\dontshow{x <- lapply(x, calc.genoprob, step=0)}

# scan genome, at each position trying all possible partitions
out <- scanPhyloQTL(x, method="hk")

# inferred partitions
inferredpartitions(out, chr=3, lodthreshold=3)

# inferred partitions with prob'y threshold = 0.95
inferredpartitions(out, chr=3, lodthreshold=3, probthreshold=0.95)
}

\seealso{ \code{\link{scanPhyloQTL}}, \code{\link{plot.scanPhyloQTL}},
  \code{\link{summary.scanPhyloQTL}}, \code{\link{max.scanPhyloQTL}},
  \code{\link{simPhyloQTL}} }

\keyword{print}

\name{summary.qtl}
\alias{summary.qtl}

\title{Print summary of a QTL object}

\description{
  Print summary information about a \code{qtl} object.
}

\usage{
\method{summary}{qtl}(object, \dots)
}

\arguments{
 \item{object}{An object of class \code{qtl}, created by \code{\link{makeqtl}}.}
 \item{\dots}{Ignored at this point.}
}

\value{
  An object of class \code{summary.qtl}, which is just a data.frame
  containing the chromosomes, positions, and number of possible
  genotypes for each QTL.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 6, 13)
qp <- c(25.8, 33.6, 18.63)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

summary(qtl)
}

\seealso{ \code{\link{makeqtl}} }

\keyword{print}

\name{fill.geno}
\alias{fill.geno}

\title{Fill holes in genotype data}

\description{
  Replace the genotype data for a cross with a version imputed either
  by simulation with \code{\link{sim.geno}}, by the Viterbi
  algorithm with \code{\link{argmax.geno}}, or simply filling in
  genotypes between markers that have matching genotypes.
}

\usage{
fill.geno(cross, method=c("imp","argmax", "no_dbl_XO", "maxmarginal"),
          error.prob=0.0001,
          map.function=c("haldane","kosambi","c-f","morgan"),
          min.prob=0.95)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{method}{Indicates whether to impute using a single simulation
   replicate from \code{\link{sim.geno}}, using the Viterbi
   algorithm, as implemented in \code{\link{argmax.geno}}, by simply
   filling in missing genotypes between markers with matching genotypes,
   or by choosing (at each marker) the genotype with maximal marginal probability.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
 of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi or
   Carter-Falconer map function when converting genetic distances into
   recombination fractions.}
 \item{min.prob}{For \code{method="maxmarginal"}, genotypes with
 probability greater than this value will be imputed; those less than
 this value will be made missing.}
}

\details{
  This function is written so that one may perform rough genome scans by
  marker regression without having to drop individuals with missing
  genotype data.  \bold{We must caution the user that little trust
    should be placed in the results}.

  With \code{method="imp"}, a single random imputation is performed,
  using \code{\link{sim.geno}}.

  With \code{method="argmax"}, for each individual the most probable
  sequence of genotypes, given the observed data (via
  \code{argmax.geno}), is used.

  With \code{method="no_dbl_XO"}, non-recombinant intervals are filled
  in; recombinant intervals are left missing. For example, a sequence of
  genotypes like \code{A---A---H---H---A} (with \code{A} and \code{H}
  corresponding to genotypes AA and AB, respectively, and with \code{-}
  being a missing value) will be filled in as
  \code{AAAAA---HHHHH---A}.

  With \code{method="maxmarginal"}, the conditional genotype
  probabilities are calculated with \code{calc.genoprob}, and then at
  each marker, the most probable genotype is determined. This is taken
  as the imputed genotype if it has probability greater than
  \code{min.prob}; otherwise it is made missing.

  With \code{method="no_dbl_XO"} and \code{method="maxmarginal"},
  some missing genotypes likely remain. With
  \code{method="maxmarginal"}, some observed genotypes may be made
  missing.
}

\value{
  The input \code{cross} object with the genotype data replaced by an
  imputed version.  Any intermediate calculations (such as is produced
  by \code{\link{calc.genoprob}}, \code{\link{argmax.geno}}
  and \code{\link{sim.geno}}) are removed.
}

\examples{
data(hyper)
\dontshow{hyper <- subset(hyper,chr=18:19)}
out.mr <- scantwo(fill.geno(hyper,method="argmax"), method="mr")
plot(out.mr)
}

\seealso{ \code{\link{sim.geno}},
          \code{\link{argmax.geno}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{utilities}

\name{plot.scanPhyloQTL}
\alias{plot.scanPhyloQTL}

\title{Plot LOD curves from single-QTL scan to map QTL to a phylogenetic tree}

\description{
  Plot the LOD curves for each partition for a genome scan with a single
  diallelic QTL (the
  output of \code{\link{scanPhyloQTL}}).
}

\usage{
\method{plot}{scanPhyloQTL}(x, chr, incl.markers=TRUE,
     col, xlim, ylim, lwd=2, gap=25, mtick=c("line", "triangle"),
     show.marker.names=FALSE, alternate.chrid=FALSE, legend=TRUE, \dots)
}

\arguments{
  \item{x}{An object of class \code{"scanPhyloQTL"}, as output by
    \code{\link{scanPhyloQTL}}.}
  \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{incl.markers}{Indicate whether to plot line segments at the
  marker locations.}
  \item{col}{Optional vector of colors to use for each partition.}
  \item{xlim}{Limits for x-axis (optional).}
  \item{ylim}{Limits for y-axis (optional).}
  \item{lwd}{Line width.}
  \item{gap}{Gap separating chromosomes (in cM).}
  \item{mtick}{Tick mark type for markers (line segments or
    upward-pointing triangels).}
  \item{show.marker.names}{If TRUE, show the marker names along the x axis.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{legend}{Indicates whether to include a legend in the plot.}
  \item{\dots}{Passed to the function \code{\link{plot.scanone}} when it
    is called.}
}
\value{None.}

%\details{
%}

\examples{
\dontrun{
# example map; drop X chromosome
data(map10)
map10 <- map10[1:19]

# simulate data
x <- simPhyloQTL(4, partition="AB|CD", crosses=c("AB", "AC", "AD"),
                 map=map10, n.ind=150,
                 model=c(1, 50, 0.5, 0))

# run calc.genoprob on each cross
x <- lapply(x, calc.genoprob, step=2)

# scan genome, at each position trying all possible partitions
out <- scanPhyloQTL(x, method="hk")

# maximum peak
max(out, format="lod")

# approximate posterior probabilities at peak
max(out, format="postprob")

# all peaks above a threshold for LOD(best) - LOD(2nd best)
summary(out, threshold=1, format="lod")

# all peaks above a threshold for LOD(best), showing approx post'r prob
summary(out, format="postprob", threshold=3)

# plot of results
plot(out)
}
}

\seealso{ \code{\link{scanPhyloQTL}}, \code{\link{max.scanPhyloQTL}},
  \code{\link{summary.scanPhyloQTL}}, \code{\link{plot.scanone}},
  \code{\link{inferredpartitions}},
  \code{\link{simPhyloQTL}},
  \code{\link[graphics]{par}}, \code{\link[grDevices]{colors}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\references{
  Broman, K. W., Kim, S., An\'e, C. and Payseur, B. A.  Mapping
  quantitative trait loci to a phylogenetic tree.  In preparation.
}

\keyword{hplot}

\name{summary.scantwoperm}
\alias{summary.scantwoperm}

\title{LOD thresholds from scantwo permutation results}

\description{
  Print the estimated genome-wide LOD thresholds on the basis of
  permutation results from \code{\link{scantwo}} (with
  \code{n.perm} > 0).
}

\usage{
\method{summary}{scantwoperm}(object, alpha=c(0.05, 0.10), \dots)
}

\arguments{
 \item{object}{Output from the function \code{\link{scantwo}}
   with \code{n.perm} > 0.}
 \item{alpha}{Genome-wide significance levels.}
 \item{\dots}{Ignored at this point.}
}

\details{
  We take the \eqn{1-\alpha}{1-alpha} quantiles of the individual LOD
  scores.

  In the case of X-chr-specific permutations, we use the combined length
  of the autosomes, \eqn{L_A}, and the length of the X chromosome,
  \eqn{L_X}, and calculate the area of the A:A, A:X, and X:X regions as
  \eqn{L_A^2/2}, \eqn{L_A L_X}, and \eqn{L_X^2/2}, and then use the
  nominal significance levels of \eqn{1 - (1-\alpha)^p}{1 - (1-alpha)^p},
  where \eqn{p} is the proportional area for that region.
}

\value{
  An object of class \code{summary.scantwoperm}, to be printed by
  \code{print.summary.scantwoperm}.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(18:19,"X"), ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=0)

\dontrun{operm <- scantwo(fake.f2, n.perm=100, method="hk")}\dontshow{
operm <- scantwo(fake.f2, n.perm=3, method="hk")}
summary(operm)
}

\seealso{ \code{\link{scantwo}},
  \code{\link{summary.scantwo}},
  \code{\link{plot.scantwoperm}} }

\references{
  Churchill, G. A. and Doerge, R. W. (1994) Empirical threshold values for
  quantitative trait mapping.  \emph{Genetics} \bold{138}, 963--971.
}


\keyword{print}

\name{sim.cross}
\alias{sim.cross}

\title{Simulate a QTL experiment}

\description{
  Simulates data for a QTL experiment using a model in which QTLs act additively.
}

\usage{
sim.cross(map, model=NULL, n.ind=100,
         type=c("f2", "bc", "4way", "risib", "riself",
           "ri4sib", "ri4self", "ri8sib", "ri8self", "bcsft"),
          error.prob=0, missing.prob=0, partial.missing.prob=0,
          keep.qtlgeno=TRUE, keep.errorind=TRUE, m=0, p=0,
      map.function=c("haldane","kosambi","c-f","morgan"),
          founderGeno, random.cross=TRUE, \dots)
}

\arguments{
 \item{map}{A list whose components are vectors containing the marker
   locations on each of the chromosomes.}
 \item{model}{A matrix where each row corresponds to a
 different QTL, and gives the chromosome number, cM position and
 effects of the QTL.}
 \item{n.ind}{Number of individuals to simulate.}
 \item{type}{Indicates whether to simulate an intercross (\code{f2}),
   a backcross (\code{bc}), a phase-known 4-way cross (\code{4way}),
   or recombinant inbred lines (by selfing or by sib-mating, and with
   the usual 2 founder strains or with 4 or 8 founder strains).}
 \item{error.prob}{The genotyping error rate.}
 \item{missing.prob}{The rate of missing genotypes.}
 \item{partial.missing.prob}{When simulating an intercross or 4-way
   cross, this gives the rate at which markers will be incompletely
   informative (i.e., dominant or recessive).}
 \item{keep.qtlgeno}{If TRUE, genotypes for the simulated QTLs will be
   included in the output.}
 \item{keep.errorind}{If TRUE, and if \code{error.prob > 0}, the
   identity of genotyping errors will be included in the output.}
 \item{m}{Interference parameter; a non-negative integer.  0 corresponds
   to no interference.}
 \item{p}{Probability that a chiasma comes from the no-interference
   mechanism}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions.}
 \item{founderGeno}{For 4- or 8-way RIL, the genotype data of the
   founder strains, as a list whose components are numeric matrices
   (no. markers x no. founders), one for each chromosome.}
 \item{random.cross}{For 4- or 8-way RIL, indicates whether the order of the
    founder strains should be randomized, independently for each RIL, or
    whether all RIL be derived from a common cross.  In the latter case,
    for a 4-way RIL, the cross would be (AxB)x(CxD).}
 \item{\dots}{For \code{type = "bcsft"}, additional arguments passed to \code{sim.cross.bcsft}.}
}

\details{
  Meiosis is assumed to follow the Stahl model for crossover
  interference (see the references, below), of which the no interference
  model and the chi-square model are special cases.  Chiasmata on the
  four-strand bundle are a superposition of chiasmata from two different
  mechanisms.  With probability \code{p}, they arise by a mechanism
  exhibiting no interference; the remainder come from a chi-square model
  with inteference parameter \code{m}.  Note that \code{m=0} corresponds
  to no interference, and with \code{p=0}, one gets a pure chi-square
  model.

  If a chromosomes has class \code{X}, it is assumed to be the X
  chromosome, and is assumed to be segregating in the cross.  Thus, in
  an intercross, it is segregating like a backcross chromosome.  In a
  4-way cross, a second phenotype, \code{sex}, will be generated.

  QTLs are assumed to act additively, and the residual phenotypic
  variation is assumed to be normally distributed with variance 1.

  For a backcross, the effect of a QTL is a single number corresponding
  to the difference between the homozygote and the heterozygote.

  For an intercross, the effect of a QTL is a pair of numbers,
  (\eqn{a,d}), where \eqn{a} is the additive effect (half the difference
  between the homozygotes) and \eqn{d} is the dominance deviation (the
  difference between the heterozygote and the midpoint between the
  homozygotes).

  For a four-way cross, the effect of a QTL is a set of three numbers,
  (\eqn{a,b,c}), where, in the case of one QTL, the mean phenotype,
  conditional on the QTL genotyping being AC, BC, AD or BD, is  \eqn{a},
  \eqn{b}, \eqn{c} or 0, respectively.
}

\section{Recombinant inbred lines}{
  In the simulation of recombinant inbred lines (RIL), we simulate a
  single individual from each line, and no phenotypes are simulated (so the
  argument \code{model} is ignored).

  The types \code{riself} and \code{risib} are the usual two-way RIL.

  The types \code{ri4self}, \code{ri4sib}, \code{ri8self}, and
  \code{ri8sib} are RIL by selfing or sib-mating derived from four or
  eight founding parental strains.

  For the 4- and 8-way RIL, one must include the genotypes of the
  founding individuals; these may be simulated with
  \code{\link{simFounderSnps}}.  Also, the output cross will
  contain a component \code{cross}, which is a matrix with rows
  corresponding to RIL and columns corresponding to the founders,
  indicating order of the founder strains in the crosses used to
  generate the RIL.

  The coding of genotypes in 4- and 8-way RIL is rather complicated.  It
  is a binary encoding of which founder strains' genotypes match the
  RIL's genotype at a marker, and not that this is specific to the order
  of the founders in the crosses used to generate the RIL.  For example,
  if an RIL generated from 4 founders has the 1 allele at a SNP, and the
  four founders have SNP alleles 0, 1, 0, 1, then the RIL allele matches
  that of founders B and D.  If the RIL was derived by the cross (AxB)x(CxD),
  then the RIL genotype would be encoded \eqn{2^{2-1} + 2^{3-1} = 6}{2^(2-1) + 2^(3-1) = 6}.
  If the cross was derived by the cross (DxA)x(CxB), then the RIL
  genotype would be encoded \eqn{2^{1-1} + 2^{4-1} = 9}{2^(1-1) + 2^(4-1) = 6}.
  These get reorganized after calls to \code{\link{calc.genoprob}},
  \code{\link{sim.geno}}, or \code{\link{argmax.geno}}, and
  this approach simplifies the hidden Markov model (HMM) code.

  For the 4- and 8-way RIL, genotyping errors are simulated only if the
  founder genotypes are 0/1 SNPs.
}

\value{
  An object of class \code{cross}.  See \code{\link{read.cross}} for
  details.

  If \code{keep.qtlgeno} is TRUE, the cross object will contain a
  component \code{qtlgeno} which is a matrix containing the QTL
  genotypes (with complete data and no errors), coded as in the genotype
  data.

  If \code{keep.errorind} is TRUE and errors were simulated, each
  component of \code{geno} will each contain a matrix \code{errors},
  with 1's indicating simulated genotyping errors.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\seealso{ \code{\link{sim.map}}, \code{\link{read.cross}},
    \code{\link{fake.f2}}, \code{\link{fake.bc}}
    \code{\link{fake.4way}}, \code{\link{simFounderSnps}} }

\examples{
# simulate a genetic map
map <- sim.map()


### simulate 250 intercross individuals with 2 QTLs
fake <- sim.cross(map, type="f2", n.ind=250,
                  model = rbind(c(1,45,1,1),c(5,20,0.5,-0.5)))


### simulate 100 backcross individuals with 3 QTL
# a 10-cM map model after the mouse
data(map10)

fakebc <- sim.cross(map10, type="bc", n.ind=100,
                    model=rbind(c(1,45,1), c(5,20,1), c(5,50,1)))


### simulate 8-way RIL by sibling mating
# get lengths from the above 10-cM map
L <- ceiling(sapply(map10, max))

# simulate a 1 cM map
themap <- sim.map(L, n.mar=L+1, eq.spacing=TRUE)

# simulate founder genotypes
pg <- simFounderSnps(themap, "8")

# simulate the 8-way RIL by sib mating (256 lines)
ril <- sim.cross(themap, n.ind=256, type="ri8sib", founderGeno=pg)
}

\references{
  Copenhaver, G. P., Housworth, E. A. and Stahl, F. W. (2002) Crossover
  interference in arabidopsis.  \emph{Genetics} \bold{160}, 1631--1639.

  Foss, E., Lande, R., Stahl, F. W. and Steinberg, C. M. (1993) Chiasma
  interference as a function of genetic distance. \emph{Genetics}
  \bold{133}, 681--691.

  Zhao, H., Speed, T. P. and McPeek, M. S. (1995) Statistical analysis
  of crossover interference using the chi-square model.  \emph{Genetics}
  \bold{139}, 1045--1056.

  Broman, K. W. (2005) The genomes of recombinant inbred lines
  \emph{Genetics} \bold{169}, 1133--1146.

  Teuscher, F. and Broman, K. W. (2007) Haplotype probabilities for
  multiple-strain recombinant inbred lines.  \emph{Genetics} \bold{175},
  1267--1274.
}

\keyword{datagen}

\name{scanoneboot}
\alias{scanoneboot}

\title{Bootstrap to get interval estimate of QTL location}

\description{
  Nonparametric bootstrap to get an estimated confidence interval for
  the location of a QTL, in the context of a single-QTL model.
}

\usage{
scanoneboot(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
            method=c("em","imp","hk","ehk","mr","mr-imp","mr-argmax"),
            addcovar=NULL, intcovar=NULL, weights=NULL,
            use=c("all.obs", "complete.obs"), upper=FALSE,
            ties.random=FALSE, start=NULL, maxit=4000,
            tol=1e-4, n.boot=1000, verbose=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{The chromosome to investigate.  Only one chromosome is
  allowed. (This should be a character string referring to the
  chromosomes by name.)}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  One may also give a character string matching
   a phenotype name.  Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{model}{The phenotypic model: the usual normal model, a model for
   binary traits, a two-part model or non-parametric analysis}
 \item{method}{Indicates whether to use the EM algorithm,
   imputation, Haley-Knott regression, the extended Haley-Knott method,
   or marker regression.  Not all methods are available for all models.
   Marker regression is performed either by dropping individuals with
   missing genotypes (\code{"mr"}), or by first filling in missing data
   using a single imputation (\code{"mr-imp"}) or by the Viterbi
   algorithm (\code{"mr-argmax"}).}
 \item{addcovar}{Additive covariates;
   allowed only for the normal and binary models.}
 \item{intcovar}{Interactive covariates (interact with QTL genotype);
   allowed only for the normal and binary models.}
 \item{weights}{Optional weights of individuals.  Should be either NULL
   or a vector of length n.ind containing positive weights.  Used only
   in the case \code{model="normal"}.}
 \item{use}{In the case that multiple phenotypes are selected to be
   scanned, this argument indicates whether to use all individuals,
   including those missing some phenotypes, or just those individuals
   that have data on all selected phenotypes.}
 \item{upper}{Used only for the two-part model; if true, the
   "undefined" phenotype is the maximum observed phenotype; otherwise,
   it is the smallest observed phenotype.}
 \item{ties.random}{Used only for the non-parametric "model"; if TRUE,
   ties in the phenotypes are ranked at random.  If FALSE, average ranks
   are used and a corrected LOD score is calculated.}
 \item{start}{Used only for the EM algorithm with the normal model and
   no covariates.  If \code{NULL}, use the usual starting values; if
   length 1, use random initial weights for EM; otherwise, this should
   be a vector of length n+1 (where n is the number of possible
   genotypes for the cross), giving the initial values for EM.}
 \item{maxit}{Maximum number of iterations for methods \code{"em"} and
   \code{"ehk"}.}
 \item{tol}{Tolerance value for determining convergence for methods
   \code{"em"} and \code{"ehk"}.}
 \item{n.boot}{Number of bootstrap replicates.}
 \item{verbose}{If TRUE, display information about the progress of the
   bootstrap.}
}

\details{
  We recommend against the use of the bootstrap to derive a confidence
  interval for the location of a QTL; see Manichaikul et al. (2006).
  Use \code{\link{lodint}} or \code{\link{bayesint}} instead.

  The bulk of the arguments are the same as for the
  \code{\link{scanone}} function.  A single chromosome should be
  indicated with the \code{chr} argument; otherwise, we focus on the
  first chromosome in the input \code{cross} object.

  A single-dimensional scan on the relevant chromosome is performed.  We
  further perform a nonparametric bootstrap (sampling individuals \emph{with
  replacement} from the available data, to create a new data set with
  the same size as the input cross; some individuals with be duplicated
  and some omitted).  The same scan is performed with the resampled data;
  for each bootstrap replicate, we store only the location with maximum
  LOD score.

  Use \code{\link{summary.scanoneboot}} to obtain the desired
  confidence interval.
}

\value{
  A vector of length \code{n.boot}, giving the estimated QTL locations
  in the bootstrap replicates.  The results for the original data are
  included as an attribute, \code{"results"}.
}

\references{
  Manichaikul, A., Dupuis, J., Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S} and Broman, K. W. (2006) Poor
  performance of bootstrap confidence intervals for the location of a
  quantitative trait locus. \emph{Genetics} \bold{174}, 481--489.

  Visscher, P. M., Thompson, R. and Haley, C. S. (1996) Confidence
  intervals in QTL mapping by bootstrap.  \emph{Genetics} \bold{143},
  1013--1020.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)
fake.f2 <- calc.genoprob(fake.f2, step=1, err=0.001)
\dontrun{bootoutput <- scanoneboot(fake.f2, chr=13, method="hk")}
\dontshow{bootoutput <- scanoneboot(fake.f2, chr=13, method="hk", n.boot=50)}

plot(bootoutput)
summary(bootoutput)
}

\seealso{ \code{\link{scanone}}, \code{\link{summary.scanoneboot}},
  \code{\link{plot.scanoneboot}},
  \code{\link{lodint}}, \code{\link{bayesint}} }

\keyword{utilities}

\name{table2map}
\alias{table2map}

\title{Convert a table of marker positions to a map object.}

\description{
  Convert a data frame with marker positions to a map object.
}

\usage{table2map(tab)}

\arguments{
 \item{tab}{A data frame with two columns: chromosome and position.
  The row names are the marker names.}
}

\value{
 A \code{map} object: a list whose components (corresponding
 to chromosomes) are vectors of marker positions.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
tab <- data.frame(chr=c(1,1,1,1,2,2,2,2,3,3,3,3),
                  pos=c(0,2,4,8,0,2,4,8,0,2,4,8))
rownames(tab) <- paste0("marker", 1:nrow(tab))

map <- table2map(tab)
}

\seealso{ \code{\link{map2table}}, \code{\link{pull.map}}, \code{\link{est.map}} }


\keyword{utilities}

\name{scantwopermhk}
\alias{scantwopermhk}

\title{Permutation test for 2d genome scan by Haley-Knott regression}

\description{
  Perform a permutation test with a two-dimensional genome scan with a
  two-QTL model, with possible allowance for additive covariates, by
  Haley-Knott regression.
}

\usage{
scantwopermhk(cross, chr, pheno.col=1,
              addcovar=NULL, weights=NULL, n.perm=1,
              batchsize=1000,
              perm.strata=NULL, perm.Xsp=NULL,
              verbose=FALSE, assumeCondIndep=FALSE)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes for which LOD
   scores should be calculated.  This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  This should be a single value (numeric index
   or character string for a phenotype name), but it may also be a
   vector of numeric values with length equal to the number
   of individuals in the cross, in which case it is taken to be a vector
   of individuals' phenotypes.}
 \item{addcovar}{Additive covariates.}
 \item{weights}{Optional weights of individuals.  Should be either NULL
   or a vector of length n.ind containing positive weights. Used only
   in the case \code{model="normal"}.}
 \item{n.perm}{Number of permutation replicates.}
 \item{batchsize}{If \code{n.perm > batchsize}, permutations will be
   run in batches of no more than \code{batchsize} permutations.}
 \item{perm.strata}{Used to perform a
   stratified permutation test.  This should be a vector with the same
   number of individuals as in the cross data.  Unique values indicate
   the individual strata, and permutations will be performed within the
   strata.}
 \item{perm.Xsp}{If TRUE, run separate permutations for A:A, A:X, and
 X:X. In this case, \code{n.perm} refers to the number of permutations
 for the A:A part; more permutations are used for the A:X and X:X parts,
 as estimates of quantiles farther out into the tails are needed.}
 \item{verbose}{If TRUE, display information about the progress of
   calculations.}
 \item{assumeCondIndep}{If TRUE, assume conditional independence of QTL
   genotypes given marker genotypes.  This is an approximation, but it
   may speed things up.}
}

\details{
  This is a scaled-back version of the permutation test provided by
  \code{\link{scantwo}}: only for a normal model with Haley-Knott
  regression, and not allowing interactive covariates.

  This is an attempt to speed things up and attentuate the memory usage
  problems in \code{\link{scantwo}}.

  In the case of \code{perm.Xsp=TRUE} (X-chr-specific thresholds), we
  use a stratified permutation test, stratified by sex and
  cross-direction.
}

\value{
  A list with six
  different LOD scores from each of the permutation replicates.
  First, the maximum LOD score for the full model (two QTLs plus an
  interaction).  Second, for each pair of
  chromosomes, we take the difference between the full LOD and the
  maximum single-QTL LOD for those two chromosomes, and then maximize
  this across chromosome pairs.  Third, for each pair of chromosomes we
  take the difference between the maximum full LOD and the maximum
  additive LOD, and then maximize this across chromosome pairs.  Fourth,
  the maximum LOD score for the additive QTL model.  Fifth, for each
  pair of chromosomes, we take the difference between the additive LOD
  and the maximum single-QTL LOD for those two chromosomes, and then
  maximize this across chromosome pairs.  Finally, the maximum
  single-QTL LOD score (that is, from a single-QTL scan).  The latter is
  not used in \code{\link{summary.scantwoperm}}, but does get
  calculated at each permutation, so we include it for the sake of
  completeness.

  If \code{perm.Xsp=TRUE}, this is a list of lists, for the A:A, A:X,
  and X:X sections, each being a list as described above.
}

\references{
  Churchill, G. A. and Doerge, R. W. (1994) Empirical threshold values for
  quantitative trait mapping.  \emph{Genetics} \bold{138}, 963--971.

  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.
}

\author{Karl W Broman, \email{broman@wisc.edu}; Hao Wu}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
fake.f2 <- calc.genoprob(fake.f2, step=5)
operm <- scantwopermhk(fake.f2, n.perm=2)
summary(operm, alpha=0.05)
}

\seealso{ \code{\link{scantwo}}, \code{\link{plot.scantwoperm}},
  \code{\link{summary.scantwoperm}},
  \code{\link{c.scantwoperm}} }

\keyword{models}

\name{multitrait}
\alias{multitrait}
\docType{data}

\title{
    Example Cross object from R/QTL with multiple traits
}

\description{
    Cross object from R/QTL, an object of class \code{cross} from R/QTL. See \code{\link{read.cross}} for details.
}
\usage{data(multitrait)}
\format{
    Cross object from R/QTL
}
\details{
    Arabidopsis recombinant inbred lines by selfing.  There are 162
    lines, 24 phenotypes, and 117 markers on 5 chromosomes.
}
\source{
  Part of the Arabidopsis RIL selfing experiment with Landsberg erecta (Ler) and Cape Verde Islands (Cvi) with 162 individuals scored (with errors at) 117 markers.
  Dataset obtained from GBIC - Groningen BioInformatics Centre
}
\references{
\itemize{

 \item Keurentjes, J. J. and Fu, J. and de Vos, C. H. and Lommen, A. and Hall, R. D. and Bino, R. J. and van der Plas, L. H. and Jansen, R. C. and Vreugdenhil, D. and Koornneef, M.
 (2006), The genetics of plant metabolism. \emph{Nature Genetics.}
 \bold{38}-7, 842--849.

 \item Alonso-Blanco, C. and Peeters, A. J. and Koornneef, M. and Lister, C. and Dean, C. and van den Bosch, N. and Pot, J. and Kuiper, M. T. (1998),
 Development of an AFLP based linkage map of Ler, Col and Cvi
 Arabidopsis thaliana ecotypes and construction of a Ler/Cvi recombinant
 inbred line population}. \emph{Plant J.} \bold{14}(2), 259--271.

}

\examples{
data(multitrait)					# Load dataset
multitrait <- fill.geno(multitrait)                     # impute missing genotype data
\dontshow{multitrait <- subset(multitrait, chr=1)
multitrait$pheno <- multitrait$pheno[,1:3]}
result <- mqmscanall(multitrait, logtransform=TRUE)	# Analyse all 24 traits
}

\keyword{datasets}

\name{findDupMarkers}
\alias{findDupMarkers}

\title{Find markers with identical genotype data}

\description{
  Identify sets of markers with identical genotype data.
}

\usage{
findDupMarkers(cross, chr, exact.only=TRUE, adjacent.only=FALSE)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector specifying which chromosomes to consider.
   This may be a logical, numeric, or character string vector.}
 \item{exact.only}{If TRUE, look only for markers that have matching
 genotypes and the same pattern of missing data; if FALSE, also look for
 cases where the observed genotypes at one marker match those at
 another, and where the first marker has missing genotype whenever the
 genotype for the second marker is missing.}
 \item{adjacent.only}{If TRUE, look only for sets of markers that are
  adjacent to each other.}
}

\value{A list of marker names; each component is a set of markers whose
  genotypes match one other marker, and the name of the component is the
  name of the marker that they match.}


\details{
  If \code{exact.only=TRUE}, we look only for groups of markers whose
  pattern of missing data and observed genotypes match exactly.  One
  marker (chosen at random) is selected as the name of the group (in the
  output of the function).

  If \code{exact.only=FALSE}, we look also for markers whose observed genotypes
  are contained in the observed genotypes of another marker.  We use a
  pair of nested loops, working from the markers with the most observed
  genotypes to the markers with the fewest observed genotypes.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)

hyper <- drop.nullmarkers(hyper)

dupmar <- findDupMarkers(hyper) # finds 4 pairs
dupmar.adjonly <- findDupMarkers(hyper, adjacent.only=TRUE) # finds 4 pairs

dupmar.nexact <- findDupMarkers(hyper, exact.only=FALSE, adjacent.only=TRUE) # finds 6 pairs

# one might consider dropping the extra markers
totmar(hyper) # 173 markers
hyper <- drop.markers(hyper, unlist(dupmar.adjonly))
totmar(hyper) # 169 markers
}

\seealso{ \code{\link{drop.nullmarkers}},
  \code{\link{drop.markers}}, \code{\link{pickMarkerSubset}} }

\keyword{manip}

\name{subset.scantwoperm}
\alias{subset.scantwoperm}
\alias{[.scantwoperm}

\title{Subsetting two-dimensional permutation test results}

\description{
  Pull out results for a specified set LOD columns from
  permutation results from \code{\link{scantwo}}.
}

\usage{
\method{subset}{scantwoperm}(x, repl, lodcolumn, \dots)
\method{[}{scantwoperm}(x, repl, lodcolumn)
}

\arguments{
 \item{x}{Permutation results from
   \code{\link{scantwo}}, run with \code{n.perm}>0.}
 \item{repl}{A vector specifying which permutation replicates to keep or (if
 negative) omit. Ignored in case of X-chr specific permutations}
 \item{lodcolumn}{A vector specifying which LOD columns to keep or (if
   negative) omit.  These should be between 1 and the number of LOD
   columns in the input \code{x}.}
 \item{\dots}{Ignored at this point.}
}

\value{The input \code{scantwo} permutation results, but with only the specified
  subset of the data.}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=18:19)}
fake.bc <- calc.genoprob(fake.bc, step=0)
operm <- scantwo(fake.bc, method="hk", pheno.col=1:2, n.perm=5)
operm2 <- subset(operm, lodcolumn=2)

# alternatively
operm2alt <- operm[,2]
}

\seealso{ \code{\link{summary.scantwoperm}},
  \code{\link{scantwo}}, \code{\link{c.scantwoperm}},
  \code{\link[=c.scantwoperm]{rbind.scantwoperm}} }

\keyword{manip}

\name{subset.scanoneperm}
\alias{subset.scanoneperm}
\alias{[.scanoneperm}

\title{Subsetting permutation test results}

\description{
  Pull out results for a specified set LOD columns from
  permutation results from \code{\link{scanone}}.
}

\usage{
\method{subset}{scanoneperm}(x, repl, lodcolumn, \dots)
\method{[}{scanoneperm}(x, repl, lodcolumn)
}

\arguments{
 \item{x}{Permutation results from
   \code{\link{scanone}}, run with \code{n.perm}>0.}
 \item{repl}{A vector specifying which permutation replicates to keep or (if
 negative) omit.}
 \item{lodcolumn}{A vector specifying which LOD columns to keep or (if
   negative) omit.  These should be between 1 and the number of LOD
   columns in the input \code{x}.}
 \item{\dots}{Ignored at this point.}
}

\value{The input \code{scanone} permutation results, but with only the specified
  subset of the data.}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=16:19)}
fake.bc <- calc.genoprob(fake.bc, step=5)
operm <- scanone(fake.bc, method="hk", pheno.col=1:2, n.perm=25)
operm2 <- subset(operm, lodcolumn=2)

# alternatively
operm2alt <- operm[,2]
}

\seealso{ \code{\link{summary.scanoneperm}},
  \code{\link{scanone}}, \code{\link{c.scanoneperm}},
  \code{\link{cbind.scanoneperm}},
  \code{\link[=c.scanoneperm]{rbind.scanoneperm}} }

\keyword{manip}

\name{cbind.scantwoperm}
\alias{cbind.scantwoperm}

\title{Combine scantwo permutations by column}

\description{
  Column-bind permutations results from \code{\link{scantwo}}
  for multiple phenotypes or models.
}

\usage{
\method{cbind}{scantwoperm}(\dots)
}

\arguments{
 \item{\dots}{A set of objects of class \code{scantwoperm}. (This can
   also be a list of \code{scantwoperm} objects.)  These are
   the permutation results from \code{\link{scantwo}} (that is,
   when \code{n.perm > 0}).  These must all concern the same number of
   permutations.}
}

\value{The column-binded input, as a \code{scantwoperm} object.}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=c(18:19), ind=1:50)}
fake.bc <- calc.genoprob(fake.bc)
\dontrun{operm1 <- scantwo(fake.bc, pheno.col=1, method="hk", n.perm=50)
operm2 <- scantwo(fake.bc, pheno.col=2, method="hk", n.perm=50)}\dontshow{
operm1 <- scantwo(fake.bc, pheno.col=1, method="hk", n.perm=3)
operm2 <- scantwo(fake.bc, pheno.col=2, method="hk", n.perm=3)}

operm <- cbind(operm1, operm2)
}

\seealso{ \code{\link{scantwo}}, \code{\link{c.scantwoperm}},
  \code{\link{summary.scantwoperm}} }

\keyword{manip}

\name{allchrsplits}
\alias{allchrsplits}

\title{Test all possible splits of a chromosome into two pieces}

\description{
  In order to assess the support for a linkage group, this function
  splits the linkage groups into two pieces at each interval and in each
  case calculates a LOD score comparing the combined linkage group to
  the two pieces.
}

\usage{
allchrsplits(cross, chr, error.prob=0.0001,
                map.function=c("haldane","kosambi","c-f","morgan"),
                m=0, p=0, maxit=4000, tol=1e-6, sex.sp=TRUE,
                verbose=TRUE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{A vector specifying which chromosomes to study.
   This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions. (Ignored if m > 0.)}
 \item{m}{Interference parameter for the chi-square model for
   interference; a non-negative integer, with m=0 corresponding to no
   interference. This may be used only for a backcross or intercross.}
 \item{p}{Proportion of chiasmata from the NI mechanism, in the Stahl
   model; p=0 gives a pure chi-square model.  This may be used only for
   a backcross or intercross.}
 \item{maxit}{Maximum number of EM iterations to perform.}
 \item{tol}{Tolerance for determining convergence.}
 \item{sex.sp}{Indicates whether to estimate sex-specific maps; this is
 used only for the 4-way cross.}
 \item{verbose}{If TRUE, print information on progress.}
}

\value{
  A data frame (actually, an object of class \code{"scanone"}, so that
  one may use \code{\link{plot.scanone}},
  \code{\link{summary.scanone}}, etc.) with each row being an interval
  at which a split is made.
  The first two columns are the chromosome ID and midpoint of the interval.  The third
  column is a LOD score comparing the combined linkage group to the
  split into two linkage groups.  A fourth column (\code{gap}) indicates the length of
  each interval.

  The row names indicate the flanking markers for each interval.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.bc)
allchrsplits(fake.bc, 7, error.prob=0, verbose=FALSE)
}


\seealso{ \code{\link{est.map}}, \code{\link{ripple}},
  \code{\link{est.rf}}, \code{\link{switch.order}},
  \code{\link{movemarker}} }

\keyword{utilities}

\name{lodint}
\alias{lodint}

\title{LOD support interval}

\description{
  Calculate a LOD support interval for a particular chromosome,
  using output from scanone.
}

\usage{lodint(results, chr, qtl.index, drop=1.5, lodcolumn=1, expandtomarkers=FALSE)}

\arguments{
 \item{results}{Output from \code{\link{scanone}}, or a qtl object
 as output from \code{\link{refineqtl}}.}
 \item{chr}{A chromosome ID (if input \code{results} are from
 \code{\link{scanone}} (should have length 1).}
 \item{qtl.index}{Numeric index for a QTL (if input \code{results} are
 from \code{\link{refineqtl}} (should have length 1).}
 \item{drop}{LOD units to drop to form the interval.}
 \item{lodcolumn}{An integer indicating which
   of the LOD score columns should be considered (if input
 \code{results} are from \code{\link{scanone}}).}
 \item{expandtomarkers}{If TRUE, the interval is expanded to the nearest
   flanking markers.}
}

\value{
  An object of class \code{scanone} indicating the
  estimated QTL position and the approximate endpoints
  for the LOD support interval.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
\dontshow{hyper <- subset(hyper, chr=c(1,4))}
hyper <- calc.genoprob(hyper, step=0.5)
out <- scanone(hyper, method="hk")
lodint(out, chr=1)
lodint(out, chr=4)
lodint(out, chr=4, drop=2)
lodint(out, chr=4, expandtomarkers=TRUE)
}

\seealso{ \code{\link{scanone}}, \code{\link{bayesint}} }

\keyword{utilities}

\name{summary.fitqtl}
\alias{summary.fitqtl}

\title{Summary of fit of qtl model}

\description{
  Print summary information about the results of \code{\link{fitqtl}}.
}

\usage{
\method{summary}{fitqtl}(object, pvalues=TRUE, simple=FALSE, \dots)
}
\arguments{
 \item{object}{Output from \code{\link{fitqtl}}.}
 \item{pvalues}{If FALSE, don't include p-values in the summary.}
 \item{simple}{If TRUE, don't include p-values or sums of squares in the summary.}
 \item{\dots}{Ignored at this point.}
}

\value{
  An object of class \code{summary.fitqtl}, which is not all that
  different than the input, but when printed gives summary information
  about the results.
}

\author{Hao Wu; Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 8, 13)
qp <- c(26, 56, 28)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

# fit model with 3 interacting QTLs interacting
# (performing a drop-one-term analysis)
lod <- fitqtl(fake.f2, pheno.col=1, qtl, formula=y~Q1*Q2*Q3,
              method="hk")
summary(lod)
}

\seealso{ \code{\link{fitqtl}}, \code{\link{makeqtl}},
\code{\link{scanqtl}} }

\keyword{print}

\name{mqmgetmodel}
\alias{mqmgetmodel}
\alias{mqmgetmodel}

\title{ Retrieve the QTL model used in mapping from the results of an MQM scan }

\description{
  Retrieves the QTL model used for scanning from the output of an MQM scan. The
  model only contains the selected cofactors significant at the specified
  cofactor.significance from the results of an mqm scan
}

\usage{
    mqmgetmodel(scanresult)
}

\arguments{
  \item{scanresult}{
An object returned by \code{mqmscan}, including cofactors and QTL model. % \mqmscanobject
  }
}

\value{
  The function returns the multiple QTL model created, which consists of the cofactors selected during the modeling phase of the algorithm.
  This model was used when scanning for additional QTL in the mqmscan function. The format of the model is compatible with the
  \code{\link{makeqtl}} function. For more information about the format of the model see the \code{\link{makeqtl}} page.
  When no cofactor was selected in the modeling phase no model was created, then this function will return a NULL value.
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
    \itemize{
     \item \code{\link{mqmsetcofactors}} - Setting multiple cofactors for backward elimination
    \item \code{\link{makeqtl}} - Make a qtl object
   % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
    }
}

\examples{
    data(hyper)
        \dontshow{hyper <- subset(hyper, chr=1:4)}
    hyperfilled <- fill.geno(hyper)
    cofactors <- mqmsetcofactors(hyperfilled,4)
    result <- mqmscan(hyperfilled,cofactors)
    mqmgetmodel(result)
    plot(mqmgetmodel(result))
}

\keyword{ utilities }

\name{comparegeno}
\alias{comparegeno}

\title{Compare individuals' genotype data}

\description{
  Count proportion of matching genotypes between all pairs of
  individuals, to look for unusually closely related individuals.
}

\usage{comparegeno(cross, what=c("proportion","number","both"))}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{what}{Indicates whether to return the proportion or number of matching
genotypes (or both).}
}

\value{
  A matrix whose (i,j)th element is the proportion or number of matching
  genotypes for individuals i and j.

  If called with \code{what="both"}, the lower triangle contains the
  proportion and the upper triangle contains the number.

  If called with \code{what="proportion"}, the diagonal contains missing
  values.  Otherwise, the diagonal contains the number of typed markers
  for each individual.

  The output is given class \code{"comparegeno"} so that appropriate
  \code{summary} and \code{plot} functions may be used.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)

cg <- comparegeno(listeria)

summary(cg, 0.7)
plot(cg)
}

\seealso{ \code{\link{nmissing}}, \code{\link{summary.comparegeno}},
          \code{\link{plot.comparegeno}} }

\keyword{utilities}

\name{summary.scanoneperm}
\alias{summary.scanoneperm}

\title{LOD thresholds from scanone permutation results}

\description{
  Print the estimated genome-wide LOD thresholds on the basis of
  permutation results from \code{\link{scanone}} (with
  \code{n.perm} > 0).
}

\usage{
\method{summary}{scanoneperm}(object, alpha=c(0.05, 0.10),
                    controlAcrossCol=FALSE, \dots)
}

\arguments{
 \item{object}{Output from the function \code{\link{scanone}}
   with \code{n.perm} > 0.}
 \item{alpha}{Genome-wide significance levels.}
 \item{controlAcrossCol}{If TRUE, control error rate not just across
   the genome but also across the columns of LOD scores.}
 \item{\dots}{Ignored at this point.}
}

\details{
  If there were autosomal data only or \code{\link{scanone}} was
  run with \code{perm.Xsp=FALSE}, genome-wide LOD thresholds are given;
  these are the 1-\eqn{\alpha}{alpha} quantiles of the genome-wide maximum LOD
  scores from the permutations.

  If there were autosomal and X chromosome data and
  \code{\link{scanone}} was run with \code{perm.Xsp=TRUE},
  autosome- and X-chromsome-specific LOD thresholds are given, by the
  method described in Broman et al. (2006).  Let \eqn{L_A} and
  \eqn{L_X} be total the genetic lengths of the autosomes and X
  chromosome, respectively, and let \eqn{L_T = L_A + L_X}
  Then in place of \eqn{\alpha}{alpha}, we use
  \deqn{\alpha_A = 1 - (1-\alpha)^{L_A/L_T}}{alpha_A = 1 - (1 -
    alpha)^(L_A/L_T)}
  as the significance level for the autosomes and
  \deqn{\alpha_X = 1 - (1-\alpha)^{L_X/L_T}}{alpha_x = 1 - (1 -
    alpha)^(LX/LT)}
  as the significance level for the X chromosome.  The result is a list
  with two matrices, one for the autosomes and one for the X chromosome.

  If \code{controlAcrossCol=TRUE}, we use a trick to control the error
  rate not just across the genome but also across the LOD score
  columns.  Namely, we convert each column of permutation results to
  ranks, and then for each permutation replicate we find the maximum
  rank across the columns.  We then find the appropriate quantile of the
  maximized ranks, and then backtrack to the corresponding LOD score
  within each of the columns. See Burrage et al. (2010),
  right column on page 118.
}

\value{
  An object of class \code{summary.scanoneperm}, to be printed by
  \code{print.summary.scanoneperm}.  If there were X chromosome data and
  \code{scanone} was run with \code{perm.Xsp=TRUE}, there are two
  matrices in the results, for the autosome and X-chromosome LOD
  thresholds.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(18:19,"X"))}
fake.f2 <- calc.genoprob(fake.f2, step=2.5)

operm1 <- scanone(fake.f2, n.perm=100, method="hk")
summary(operm1)

operm2 <- scanone(fake.f2, n.perm=100, method="hk", perm.Xsp=TRUE)
summary(operm2)

# Add noise column
fake.f2$pheno$noise <- rnorm(nind(fake.f2))
operm3 <- scanone(fake.f2, pheno.col=c("phenotype", "noise"), n.perm=10, method="hk")
summary(operm3)
summary(operm3, controlAcrossCol=TRUE, alpha=c(0.05, 0.36))
}

\seealso{ \code{\link{scanone}},
  \code{\link{summary.scanone}},
  \code{\link{plot.scanoneperm}} }

\references{
  Broman KW, Sen \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}, Owens SE, Manichaikul A,
  Southard-Smith EM, Churchill GA (2006)  The X chromosome in
  quantitative trait locus mapping.  \emph{Genetics}, \bold{174}, 2151--2158.

  Burrage LC, Baskin-Hill AE, Sinasac DS, Singer JB, Croniger CM, Kirby A,
  Kulbokas EJ, Daly MJ, Lander ES, Broman KW, Nadeau JH (2010) Genetic
  resistance to diet-induced obesity in chromosome substitution strains of
  mice. \emph{Mamm Genome}, \bold{21}, 115--129.

  Churchill GA, Doerge RW (1994) Empirical threshold values for
  quantitative trait mapping.  \emph{Genetics} \bold{138}, 963--971.
}


\keyword{print}

\name{fake.bc}
\alias{fake.bc}
\docType{data}

\title{Simulated data for a backcross}

\description{
  Simulated data for a backcross, obtained using
  \code{\link{sim.cross}}.
}

\usage{data(fake.bc)}

\format{
  An object of class \code{cross}.  See \code{\link{read.cross}}
  for details.
}


\details{
  There are 400 backcross individuals typed at 91 markers and with two
  phenotypes and two covariates (sex and age).

  The two phenotypes are due to four QTLs, with no epistasis.  There is
  one on chromosome 2 (at 30 cM), two on chromosome 5 (at 10 and 50 cM),
  and one on chromosome 10 (at 30 cM).  The QTL on chromosome 2 has an
  effect only in the males (sex=1); the two QTLs on chromosome 5 have
  effect in coupling for the first phenotype and in repulsion for the
  second phenotype. Age has an effect of increasing the phenotypes.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc,chr=c(2,5,10))}
summary(fake.bc)
plot(fake.bc)

# genome scans without covariates
fake.bc <- calc.genoprob(fake.bc, step=2.5)
out.nocovar <- scanone(fake.bc, pheno.col=1:2)

# genome scans with covariates
ac <- pull.pheno(fake.bc, c("sex","age"))
ic <- pull.pheno(fake.bc, "sex")
out.covar <- scanone(fake.bc, pheno.col=1:2,
                      addcovar=ac, intcovar=ic)

# summaries
summary(out.nocovar, thr=3, format="allpeaks")
summary(out.covar, thr=3, format="allpeaks")

# plots
plot(out.nocovar, out.covar, chr=c(2,5,10), lod=1, col="blue",
     lty=1:2, ylim=c(0,13))
plot(out.nocovar, out.covar, chr=c(2,5,10), lod=2, col="red",
     lty=1:2, add=TRUE)
}

\seealso{ \code{\link{sim.cross}}, \code{\link{fake.4way}},
  \code{\link{fake.f2}}, \code{\link{listeria}},
  \code{\link{hyper}},
  \code{\link{bristle3}}, \code{\link{bristleX}} }

\keyword{datasets}

\name{plot.scanoneperm}
\alias{plot.scanoneperm}

\title{Plot permutation results for a single-QTL genome scan}

\description{
  Plot a histogram of the permutation results from a single-QTL genome
  scan.
}

\usage{
\method{plot}{scanoneperm}(x, lodcolumn=1, \dots)
}

\arguments{
  \item{x}{An object of class \code{"scanoneperm"}, as output by
    \code{\link{scanone}} when \code{n.perm} is specified.}
  \item{lodcolumn}{This indicates the LOD score column to plot.  This
    should be a single number between 1 and the number of LOD columns in
    the object input.}
  \item{\dots}{Passed to the function \code{\link[graphics]{hist}} when it
    is called.}
}
\value{None.}

\details{
  The function plots a histogram of the permutation results obtained by
  \code{\link{scanone}} when \code{n.perm} is specified. If
  separate permutations were performed for the autosomes and the X
  chromosome (using \code{perm.Xsp=TRUE}), separate histograms are given.
}

\examples{
data(fake.bc)
fake.bc <- calc.genoprob(fake.bc)

operm <- scanone(fake.bc, method="hk", n.perm=100)
plot(operm)
}

\seealso{ \code{\link{scanone}},
  \code{\link{summary.scanoneperm}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{qtl-internal}
\alias{reviseXdata}
\alias{calc.plod}
\alias{checkcovar}
\alias{clean}
\alias{condense}
\alias{convert}
\alias{convert2bcsft}
\alias{create.map}
\alias{find.pseudomarkerpos}
\alias{flipcross}
\alias{genAllPartitions}
\alias{getsex}
\alias{addmarkerstointervalmap}
\alias{mqm_version}
\alias{scanall}
\alias{getgenonames}
\alias{imf.cf}
\alias{imf.h}
\alias{imf.k}
\alias{imf.m}
\alias{imf.stahl}
\alias{locatemarker}
\alias{max.scantwocondensed}
\alias{mf.cf}
\alias{mf.h}
\alias{mf.k}
\alias{mf.m}
\alias{mf.stahl}
\alias{polyplot}
\alias{print.addcovarint}
\alias{print.addint}
\alias{print.compactqtl}
\alias{print.cross}
\alias{print.map}
\alias{print.qtl}
\alias{print.scanoneboot}
\alias{print.scantwo}
\alias{print.summary.addpair}
\alias{print.summary.compactqtl}
\alias{print.summary.comparegeno}
\alias{print.summary.cross}
\alias{print.summary.fitqtl}
\alias{print.summary.map}
\alias{print.summary.qtl}
\alias{print.summary.ripple}
\alias{print.summary.scanone}
\alias{print.summary.scanoneperm}
\alias{print.summary.scantwo}
\alias{print.summary.scantwo.old}
\alias{print.summary.scantwoperm}
\alias{replacemap}
\alias{subrousummaryscantwo}
\alias{summary.scantwocondensed}
\alias{summary.addcovarint}
\alias{summary.addint}
\alias{summary.compactqtl}
\alias{checkStepwiseqtlStart}
\alias{checkformula}
\alias{countqtlterms}
\alias{deparseQTLformula}
\alias{fitqtlengine}
\alias{matchchr}
\alias{parseformula}
\alias{reviseqtlnuminformula}

\title{Internal qtl functions}

\description{
  Internal qtl functions. These are generally not to be called by the
  user.
}

\usage{
reviseXdata(type=c("f2","bc","bcsft"), expandX=c("simple","standard","full"),
            sexpgm, geno, prob, draws, pairprob, cross.attr, force=FALSE)
calc.plod(lod, nterms, type=c("f2","bc"), penalties)
checkcovar(cross, pheno.col, addcovar, intcovar, perm.strata,
           ind.noqtl=NULL, weights=NULL, verbose=TRUE)
clean(object, \dots)
condense(object)
convert(object, \dots)
convert2bcsft(cross, BC.gen = 0, F.gen = 0, estimate.map = TRUE,
              error.prob=0.0001, map.function=c("haldane","kosambi","c-f","morgan"),
              verbose=FALSE)
create.map(map, step, off.end, stepwidth=c("fixed", "variable", "max"))
find.pseudomarkerpos(cross, marker, where=c("draws", "prob"))
flipcross(cross)
genAllPartitions(n.taxa, taxa)
getsex(cross)
addmarkerstointervalmap(cross,intervalresult,verbose=FALSE)
mqm_version()
scanall(cross, scanfunction=scanone, multicore=TRUE, n.clusters=1,
        batchsize=10, FF=0,cofactors=NULL, \dots, plot=FALSE, verbose=FALSE)
getgenonames(type=c("f2","bc","riself","risib","4way","dh","haploid","special","bcsft"),
             chrtype=c("A","X"), expandX=c("simple","standard","full"),
             sexpgm, cross.attr)
imf.cf(r)
imf.h(r)
imf.k(r)
imf.m(r)
imf.stahl(r, m=0, p=0, tol=1e-12, maxit=1000)
locatemarker(map, pos, chr, flag)
\method{max}{scantwocondensed}(object, lodcolumn=1,
    what=c("best", "full", "add", "int"),
    na.rm=TRUE, \dots)
mf.cf(d)
mf.h(d)
mf.k(d)
mf.m(d)
mf.stahl(d, m=0, p=0)
polyplot(x, type='b', legend=TRUE,legendloc=0, labels=NULL, cex =
         par("cex"), pch = 19, gpch = 21, bg = par("bg"), color = par("fg"),
         col=NULL, ylim=range(x[is.finite(x)]), xlim = NULL, main = NULL, xlab =
         NULL, ylab = NULL, add=FALSE, \dots)
\method{print}{addcovarint}(x, \dots)
\method{print}{addint}(x, \dots)
\method{print}{compactqtl}(x, \dots)
\method{print}{summary.comparegeno}(x, \dots)
\method{print}{cross}(x, \dots)
\method{print}{map}(x, \dots)
\method{print}{qtl}(x, \dots)
\method{print}{scanoneboot}(x, \dots)
\method{print}{scantwo}(x, \dots)
\method{print}{summary.addpair}(x, \dots)
\method{print}{summary.compactqtl}(x, \dots)
\method{print}{summary.cross}(x, \dots)
\method{print}{summary.fitqtl}(x, \dots)
\method{print}{summary.map}(x, \dots)
\method{print}{summary.qtl}(x, \dots)
\method{print}{summary.ripple}(x, \dots)
\method{print}{summary.scanone}(x, \dots)
\method{print}{summary.scanoneperm}(x, \dots)
\method{print}{summary.scantwo}(x, \dots)
\method{print}{summary.scantwo.old}(x, \dots)
\method{print}{summary.scantwoperm}(x, \dots)
replacemap(object, map)
subrousummaryscantwo(object, for.perm=FALSE)
\method{summary}{scantwocondensed}(object, thresholds,
         what=c("best", "full", "add", "int"),
         perms, alphas, lodcolumn=1, pvalues=FALSE,
         allpairs=TRUE, \dots)
\method{summary}{addcovarint}(object, \dots)
\method{summary}{addint}(object, \dots)
\method{summary}{compactqtl}(object, \dots)
checkStepwiseqtlStart(qtl, formula, covar=NULL)
checkformula(formula, qtl.name, covar.name)
countqtlterms(formula, ignore.covar=TRUE)
deparseQTLformula(formula, reorderterms=FALSE)
fitqtlengine(pheno, qtl, covar=NULL, formula, method=c("imp", "hk"),
         model=c("normal", "binary"),
         dropone=TRUE, get.ests=FALSE, run.checks=TRUE, cross.attr,
         crosstype, sexpgm, tol, maxit, forceXcovar=FALSE)
matchchr(selection, thechr)
parseformula(formula, qtl.dimname, covar.dimname)
reviseqtlnuminformula(formula, oldnum, newnum)
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{internal}

\name{mqmfind.marker}
\alias{mqmfind.marker}

\title{ Fetch significant markers after permutation analysis }

\description{
 Fetch significant makers after permutation analysis.  These markers can be
 used as cofactors for model selection in a forward stepwise approach.
}

\usage{
mqmfind.marker(cross, mqmscan = NULL, perm = NULL, alpha = 0.05, verbose=FALSE)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{mqmscan}{ Results from either \code{\link{scanone}} or \code{\link{mqmscan}} }
  \item{perm}{ a \code{scanoneperm} object }
  \item{alpha}{ Threshold value, everything with significance < alpha is reported }
  \item{verbose}{
Display more output on verbose=TRUE % \verbose
  }
}

\value{
  returns a matrix with at each row a significant marker (determined from the
  \code{scanoneperm} object) and with columns: markername, chr and pos (cM)
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
\itemize{
    \item \code{\link{mqmprocesspermutation}} - Function called to convert results from an mqmpermutation into an scanoneperm object
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
# Use the multitrait dataset
data(multitrait)

# Set cofactors at each 3th marker
cof <- mqmsetcofactors(multitrait,3)

# impute missing genotypes
multitrait <- fill.geno(multitrait)

# log transform the 7th phenotype
multitrait <- transformPheno(multitrait, 7)

# Bootstrap 50 runs in batches of 10
\dontrun{result <- mqmpermutation(multitrait,scanfunction=mqmscan,cofactors=cof,
                         pheno.col=7,n.perm=50,batchsize=10)
}\dontshow{result <- mqmpermutation(multitrait,scanfunction=mqmscan,cofactors=cof,
                         pheno.col=7,n.perm=2,batchsize=2)}

# Create a permutation object
f2perm <- mqmprocesspermutation(result)

# What LOD score is considered significant ?
summary(f2perm)

# Find markers with a significant QTL effect (First run is original phenotype data)
marker <- mqmfind.marker(multitrait,result[[1]],f2perm)

# Print it to the screen
marker
}

\keyword{ models }

\name{mqmplot.directedqtl}
\alias{mqmplot.directedqtl}

\title{Plot LOD*Effect curves of a multiple-QTL model }

\description{
  Plot the LOD*Effect curve for a genome scan with a multiple-QTL model (the
  output of \code{\link{mqmscan}}).
}

\usage{
mqmplot.directedqtl(cross, mqmresult, pheno.col=1, draw = TRUE)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{mqmresult}{ Results from mqmscan of type \code{scanone} }
  \item{pheno.col}{ From which phenotype in the crossobject are the result calculated }
  \item{draw}{ If TRUE, draw the figure. }
}

\value{
  Returns a scanone object, with added the effectsign calculated
  internally by the function effect.scan. For more info on the
  scanone object see: \code{\link{scanone}}
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
#Simulated F2 Population
f2qtl <- c(3,15,1,0)                                    # QTL at chromosome 3
data(map10)                                             # Mouse genetic map
\dontshow{map10 <- map10[1:3]}
f2cross <- sim.cross(map10,f2qtl,n=100,type="f2")       # Simulate a F2 Cross
f2cross <- fill.geno(f2cross)                           # Fill in missing genotypes
f2result <- mqmscan(f2cross)                        # Do a MQM scan of the genome
mqmplot.directedqtl(f2cross,f2result)
}

\keyword{ hplot }

\name{plotPheno}
\alias{plotPheno}

\title{Plot a phenotype distribution}

\description{
  Plots a histogram or barplot of the data for a phenotype from an
  experimental cross.
}

\usage{
plotPheno(x, pheno.col=1, \dots)
}

\arguments{
 \item{x}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{The phenotype column to plot: a numeric index, or the
   phenotype name as a character string. Alternatively, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{\dots}{Passed to \code{\link[graphics]{hist}} or
   \code{\link[graphics]{barplot}}.}
}

\value{None.}

\details{
  Numeric phenotypes are displayed as histograms with approximately
  \eqn{2\sqrt{n}}{2 sqrt(n)} bins.  Phenotypes that are factors or that
  have very few unique values are displayed as barplots.
}

\examples{
data(fake.bc)
plotPheno(fake.bc, pheno.col=1)
plotPheno(fake.bc, pheno.col=3)
plotPheno(fake.bc, pheno.col="age")
}

\seealso{ \code{\link{plot.cross}}, \code{\link{plotMap}},
  \code{\link{plotMissing}},
  \code{\link[graphics]{hist}}, \code{\link[graphics]{barplot}} }


\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{dropfromqtl}
\alias{dropfromqtl}

\title{Drop a QTL from a qtl object}

\description{
  Drop a QTL or multiple QTL from a QTL object
}

\usage{
dropfromqtl(qtl, index, chr, pos, qtl.name, drop.lod.profile=TRUE)
}

\arguments{
 \item{qtl}{A qtl object, as created by \code{\link{makeqtl}}.}
 \item{index}{Vector specifying the numeric indices of the QTL to be dropped.}
 \item{chr}{Vector indicating the chromosome for each QTL to drop.}
 \item{pos}{Vector (of same length as \code{chr}) indicating the
   positions of the QTL to be dropped.}
 \item{qtl.name}{Vector specifying the names of the QTL to be dropped.}
 \item{drop.lod.profile}{If TRUE, remove any LOD profiles from the object.}
}

\details{
  Provide either \code{chr} and \code{pos}, or one of \code{qtl.name} or
  \code{index}.
}

\value{
  The input \code{qtl} object with the specified QTL omitted.   See \code{\link{makeqtl}} for
  details on the format.
}

\author{Karl W Broman, \code{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 6, 13)
qp <- c(25.8, 33.6, 18.63)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

newqtl <- dropfromqtl(qtl, chr=1, pos=25.8)
altqtl <- dropfromqtl(qtl, index=1)
}

\seealso{ \code{\link{makeqtl}}, \code{\link{fitqtl}},
  \code{\link{addtoqtl}}, \code{\link{replaceqtl}} ,
  \code{\link{reorderqtl}}}

\keyword{utilities}

\name{MQM}
\alias{MQM}

\title{Introduction to Multiple QTL Model (MQM) mapping}

\description{
Overview of the MQM mapping functions
}

%\usage{
%}

\section{Introduction}{
% \input{"inst/doc/Sources/MQM/mqm/description.txt"}
Multiple QTL Mapping (MQM) provides a sensitive approach for
mapping quantititive trait loci (QTL) in experimental populations. MQM
adds higher statistical power compared to many other methods.  The
theoretical framework of MQM was introduced and explored by
Ritsert Jansen, explained in the `Handbook of Statistical Genetics'
(see references), and used effectively in practical research, with the
commercial `mapqtl' software package. Here we present the first free
and open source implementation of MQM, with extra features like high
performance parallelization on multi-CPU computers, new plots and
significance testing.

MQM is an automatic three-stage procedure in which, in the first
stage, missing data is `augmented'. In other words, rather than guessing one
likely genotype, multiple
genotypes are modeled with their estimated
probabilities.  In the second stage important markers are selected by
multiple regression and backward elimination. In the third stage a QTL is moved
along the chromosomes using these pre-selected markers as cofactors,
except for the markers in the window around the interval under study. QTL are
(interval) mapped using the most `informative' model through maximum
likelihood.  A refined and automated procedure for cases with large
numbers of marker cofactors is included.  The method internally
controls false discovery rates (FDR) and lets users test different QTL
models by elimination of non-significant cofactors.

% -----^^ inst/doc/Sources/MQM/mqm/description.txt ^^-----
% \input{"inst/doc/Sources/MQM/mqm/advantages_Rd.txt"}
R/qtl-MQM has the following advantages:

\itemize{
  \item Higher power to detect linked as well as unlinked QTL, as long as the QTL explain a reasonable amount of variation
  \item Protection against overfitting, because it fixes the residual variance from the full model. For this reason more parameters (cofactors) can be used compared to, for example, CIM
  \item Prevention of ghost QTL (between two QTL in coupling phase)
  \item Detection of negating QTL (QTL in repulsion phase)}
% -----^^ inst/doc/Sources/MQM/mqm/advantages_Rd.txt ^^-----
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\note{
% \input{"inst/doc/Sources/MQM/mqm/limitations.txt"}

The current implementation of R/qtl-MQM has the following
limitations: (1) MQM is limited to experimental crosses F2,
BC, and selfed RIL, (2) MQM does not treat sex
chromosomes differently from autosomal chromosomes - though one can
introduce sex as a cofactor. Future versions of R/qtl-MQM may improve on
these points. Check the website and change log
(\url{https://github.com/kbroman/qtl/blob/main/NEWS.md}) for updates.

% -----^^ inst/doc/Sources/MQM/mqm/limitations.txt ^^-----
}

\seealso{
    \itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
    }
}

\examples{
% \input{"inst/doc/Sources/MQM/mqm/standard_example.txt"}
data(map10)                    # Genetic map modeled after mouse

# simulate a cross (autosomes 1-10)
qtl <- c(3,15,1,0)             # QTL model: chr, pos'n, add've & dom effects
cross <- sim.cross(map10[1:10],qtl,n=100,missing.prob=0.01)

# MQM
crossaug <- mqmaugment(cross)  # Augmentation
cat(crossaug$mqm$Nind,'real individuals retained in dataset',
    crossaug$mqm$Naug,'individuals augmented\n')

result <- mqmscan(crossaug)    # Scan

# show LOD interval of the QTL on chr 3
lodint(result,chr=3)
% -----^^ inst/doc/Sources/MQM/mqm/standard_example.txt ^^-----
}

\references{
\itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_references.txt"}
        \item Arends D, Prins P, Jansen RC.  R/qtl: High-throughput multiple QTL mapping. \emph{Bioinformatics}, to appear
        \item Jansen RC, (2007) Quantitative trait loci in inbred lines. Chapter 18 of \emph{Handbook of Stat. Genetics} 3rd edition. John Wiley & Sons, Ltd.
        \item Jansen RC, Nap JP (2001), Genetical genomics: the added value from segregation. \emph{Trends in Genetics}, \bold{17}, 388--391.
        \item Jansen RC, Stam P (1994), High resolution of quantitative traits into multiple loci via interval mapping. \emph{Genetics}, \bold{136}, 1447--1455.
        \item Jansen RC (1993), Interval mapping of multiple quantitative trait loci. \emph{Genetics}, \bold{135}, 205--211.
        \item Swertz MA, Jansen RC. (2007), Beyond standardization: dynamic software infrastructures for systems biology. \emph{Nat Rev Genet.} \bold{3}, 235--243.
        \item Dempster, A. P., Laird, N. M. and Rubin, D. B. (1977) Maximum likelihood from incomplete data via the EM algorithm.  \emph{J. Roy. Statist. Soc.} B, \bold{39}, 1--38.

% -----^^ inst/doc/Sources/MQM/mqm/standard_references.txt ^^-----
}
}

\keyword{models}

\name{markernames}
\alias{markernames}

\title{Pull out the marker names from a cross}

\description{
  Pull out the marker names from a cross object as one big vector.
}

\usage{markernames(cross, chr)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
}

\value{
  A vector of character strings (the marker names).
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
markernames(listeria, chr=5)
}

\seealso{ \code{\link{pull.map}}, \code{\link{phenames}}, \code{\link{chrnames}} }


\keyword{utilities}

\name{condense.scantwo}
\alias{condense.scantwo}

\title{Condense the output from a 2-d genome scan}

\description{
  Produces a very condensed version of the output of \code{\link{scantwo}}.
}

\usage{
\method{condense}{scantwo}(object)
}

\arguments{
 \item{object}{An object of class \code{scantwo}, the output of the
   function \code{\link{scantwo}}.}
}

\value{
  An object of class \code{scantwocondensed}, containing just the
  maximum full, additive and interactive LOD scores, and the positions
  where they occured, on each pair of chromosomes.
}

\details{
  This produces a very reduced version of the output of
  \code{\link{scantwo}}, for which a summary may still be created
  via \code{\link{summary.scantwo}}, though plots can no longer be
  made.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(18:19,"X"))}
fake.f2 <- calc.genoprob(fake.f2)

out2 <- scantwo(fake.f2, method="hk")

out2c <- condense(out2)
summary(out2c, allpairs=FALSE)
max(out2c)
}

\seealso{ \code{\link{scantwo}}, \code{\link{summary.scantwo}},
  \code{\link{max.scantwo}} }

\keyword{print}

\name{mqmextractmarkers}
\alias{mqmextractmarkers}

\title{MQM marker extraction}

\description{
  Extract the real markers from a cross object that includes pseudo markers
}

\usage{
mqmextractmarkers(mqmresult)
}

\arguments{
  \item{mqmresult}{result from \code{mqmscan}, including pseudo markers}
}

\value{
  Returns a scanone object with the pseudo markers removed
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
  \itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
data(multitrait)

multitrait <- fill.geno(multitrait)

result <- mqmscan(multitrait)
newresult <- mqmextractmarkers(result)
}

\keyword{utilities}

\name{plot.scantwo}
\alias{plot.scantwo}

\title{Plot LOD scores for a two-dimensional genome scan}

\description{
  Plot the results of a two-dimensional, two-QTL genome scan.
}

\usage{
\method{plot}{scantwo}(x, chr, incl.markers=FALSE, zlim, lodcolumn=1,
     lower = c("full", "add", "cond-int", "cond-add", "int"),
     upper = c("int", "cond-add", "cond-int", "add", "full"),
     nodiag=TRUE, contours=FALSE, main, zscale=TRUE, point.at.max=FALSE,
     col.scheme = c("viridis", "redblue","cm","gray","heat","terrain","topo"),
     gamma=0.6, allow.neg=FALSE, alternate.chrid=FALSE, \dots)
}

\arguments{
  \item{x}{An object of class \code{"scantwo"}, as output by
    \code{\link{scantwo}}.}
  \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{incl.markers}{If FALSE, plot LOD scores on an evenly
    spaced grid (not including the results at the markers).}
  \item{zlim}{A vector of length 2 (optional), indicating the z limits
    for the lower-right and upper-left triangles, respectively.  If one
    number is given, the same limits are used for both triangles.  If
    \code{zlim} is missing, the maximum limits are used for each.}
  \item{lodcolumn}{If the scantwo results contain LOD scores for
    multiple phenotypes, this argument indicates which to use in the
    plot.}
  \item{lower}{Indicates which LOD scores should be plotted in the lower
    triangle.  See the details below.}
  \item{upper}{Indicates which LOD scores should be plotted in the upper
    triangle.  See the details below.}
  \item{nodiag}{If TRUE, suppress the plot of the scanone output
    (which is normally along the diagonal.)}
  \item{contours}{If TRUE, add a contour to the plot at 1.5-LOD below
    its maximum, using a call to \code{\link[graphics]{contour}}.  If a
    numeric vector, contours are drawn at these values below the maximum
    LOD.}
  \item{main}{An optional title for the plot.}
  \item{zscale}{If TRUE, a color scale is plotted at the right.}
  \item{point.at.max}{If TRUE, plot an X at the maximum LOD.}
  \item{col.scheme}{Name of color pallet. The default is "viridis"; see
  Option D at \url{https://bids.github.io/colormap/}}
  \item{gamma}{Parameter affecting range of colors when
    \code{col.scheme="gray"} or \code{="redblue"}.}
  \item{allow.neg}{If TRUE, allow the plot of negative LOD scores; in
    this case, the z-limits are symmetric about 0.  This option is
    chiefly to allow a plot of difference between LOD scores from
    different methods, calculated via \code{\link[=-.scantwo]{-.scantwo}}.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{\dots}{Ignored at this point.}
}

\value{None.}

\details{
  Uses \code{\link[graphics]{image}} to plot a grid of LOD scores.  The
  particular LOD scores plotted in the upper-left and lower-right
  triangles are selected via \code{upper} and \code{lower},
  respectively.  By default, the upper-left triangle contains the
  epistasis LOD
  scores (\code{"int"}), and the lower-right triangle contains the LOD
  scores for the full model (\code{"full"}).
  The diagonal contains either all zeros or the main effects LOD scores
  (from \code{\link{scanone}}).

  The \code{\link{scantwo}} function calculates, for each pair of
  putative QTLs, \eqn{(q_1,q_2)}{(q1,q2)}, the likelihood under the
  null model \eqn{L_0}{L0}, the likelihood under each of the single-QTL
  models, \eqn{L(q_1)}{L(q1)} and \eqn{L(q_2)}{L(q2)}, the likelihood
  under an additive QTL model, \eqn{L_a(q_1,q_2)}{La(q1,q2)}, and the
  likelihood under a full QTL model (including QTL-QTL interaction),
  \eqn{L_f(q_1,q_2)}{Lf(q1,q2)}.

  The five possible LOD scores that may be plotted are the following.
  The epistasis LOD scores (\code{"int"}) are \eqn{LOD_i = \log_{10}
    L_f(q_1,q_2) - \log_{10} L_a(q_1,q_2)}{LODi = log10 Lf(q1,q2) -
    log10 La(q1,q2)}.

  The full LOD scores (\code{"full"}) are
  \eqn{LOD_f = \log_{10} L_f(q_1,q_2) - \log_{10} L_0}{%
    LODj = log10 Lf(q1,q2) - log10 L0}.

  The additive LOD scores (\code{"add"}) are
  \eqn{LOD_a = \log_{10} L_a(q_1,q_2) - \log_{10} L_0}{%
    LODa = log10 La(q1,q2) - log10 L0}.

  In addition, we may calculate, for each pair of
  chromosomes, the difference between the full LOD score and the
  maximum single-QTL LOD scores for that pair of chromosomes
  (\code{"cond-int"}).

  Finally, we may calculate, for each pair of
  chromosomes, the difference between the additive LOD score and the
  maximum single-QTL LOD scores for that pair of chromosomes
  (\code{"cond-add"}).

  If a color scale is plotted (\code{zscale=TRUE}), the axis on the
  left indicates the scale for the upper-left triangle,
  while the axis on the right indicates the scale for the
  lower-right triangle.  Note that the axis labels can get screwed up
  if you change the size of the figure window; you'll need to redo the
  plot.
}

\section{Output of addpair}{

  \bold{Note} that, for output from \code{\link{addpair}} in which the
  new loci are indicated explicitly in the formula, the summary provided
  by \code{plot.scantwo} is somewhat special.  In particular, the
  \code{lower} and \code{upper} arguments are ignored.

  In the case that the formula used in \code{\link{addpair}} was
  not symmetric in the two new QTL, the x-axis in the plot corresponds
  to the first of the new QTL and the y-axis corresponds to the second
  of the new QTL.
}


\examples{
data(hyper)
\dontshow{hyper <- subset(hyper, chr=c(1,4,6,15))}
hyper <- calc.genoprob(hyper, step=5)
\dontshow{hyper <- calc.genoprob(hyper)}

# 2-d scan by EM and by Haley-Knott regression
out2.em <- scantwo(hyper, method="em")
out2.hk <- scantwo(hyper, method="hk")

# plot epistasis and full LOD scores
plot(out2.em)

# plot cond-int in upper triangle and full in lower triangle
#     for chromosomes 1, 4, 6, 15
plot(out2.em, upper="cond-int", chr=c(1,4,6,15))

# plot cond-add in upper triangle and add in lower triangle
#     for chromosomes 1, 4
plot(out2.em, upper="cond-add", lower="add", chr=c(1,4))

# plot the differences between the LOD scores from Haley-Knott
#     regression and the EM algorithm
plot(out2.hk - out2.em, allow.neg=TRUE)
}

\seealso{ \code{\link{scantwo}},
  \code{\link{summary.scantwo}}, \code{\link{plot.scanone}},
  \code{\link[=-.scantwo]{-.scantwo}} }

\author{Hao Wu; Karl W Broman,
  \email{broman@wisc.edu}; Brian Yandell }

\keyword{hplot}

\name{nqtl}
\alias{nqtl}

\title{Determine the number of QTL in a QTL object}

\description{
  Determine the number of QTL in a QTL object.
}

\usage{
nqtl(qtl)
}

\arguments{
 \item{qtl}{An object of class \code{qtl}.  See
   \code{\link{makeqtl}} for details.}
}

\value{
  The number of QTL in the input QTL object.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c("1", "6", "13")
qp <- c(25.8, 33.6, 18.63)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

nqtl(qtl)
}

\seealso{ \code{\link{makeqtl}}, \code{\link{fitqtl}},
  \code{\link{dropfromqtl}}, \code{\link{replaceqtl}},
  \code{\link{addtoqtl}}, \code{\link{summary.qtl}},
  \code{\link{reorderqtl}} }

\keyword{print}

\name{plotLodProfile}
\alias{plotLodProfile}

\title{Plot 1-d LOD profiles for a multiple QTL model}

\description{
  Use the results of \code{\link{refineqtl}} to plot
  one-dimensional LOD profiles for each QTL.
}

\usage{
plotLodProfile(qtl, chr, incl.markers=TRUE, gap=25, lwd=2, lty=1, col="black",
               qtl.labels=TRUE, mtick=c("line", "triangle"),
               show.marker.names=FALSE, alternate.chrid=FALSE,
               add=FALSE, showallchr=FALSE, labelsep=5, \dots)

}
\arguments{
  \item{qtl}{An object of class \code{"qtl"}; must have been produced by
    \code{\link{refineqtl}} using \code{keeplodprofiles=TRUE}.}
  \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{incl.markers}{Indicate whether to plot line segments at the
  marker locations.}
  \item{gap}{Gap separating chromosomes (in cM).}
  \item{lwd}{Line widths for each QTL trace (length 1 or the number of QTL).}
  \item{lty}{Line types for each QTL trace (length 1 or the number of QTL).}
  \item{col}{Line col for each QTL trace (length 1 or the number of QTL).}
  \item{qtl.labels}{If TRUE, place a label on each QTL trace.}
  \item{mtick}{Tick mark type for markers (line segments or
    upward-pointing triangels).}
  \item{show.marker.names}{If TRUE, show the marker names along the x axis.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{add}{If TRUE, add curves to a current plot.}
  \item{showallchr}{If FALSE (the default), only show the chr with a QTL}
  \item{labelsep}{If \code{qtl.labels=TRUE}, separation between peak LOD
    and QTL label, as percent of the height of the plot.}
  \item{\dots}{Passed to the function \code{plot} when it
    is called.}
}
\value{None.}

\details{
  The function plots LOD profiles in the context of a multiple QTL
  model, using a scheme best described in Zeng et al. (2000).  The
  position of each QTL is varied, keeping all other loci fixed.  If a
  QTL is isolated on a chromosome, the entire chromosome is scanned; if
  there are additional linked QTL, the position of a QTL is scanned over
  the largest interval possible without allowing the order of QTLs along
  a chromosome to change.  At each position for the QTL being scanned,
  we calculate a LOD score comparing the full model, with the QTL of
  interest at that particular position (and all others at their fixed
  positions) to the model with the QTL of interest (and any
  interactions that include that QTL) omitted.

  Care should be take regarding the arguments \code{lwd}, \code{lty},
  and \code{col}; if vectors are given, they should be in the order of
  the QTL within the object, which may be different than the order in
  which they are plotted.  (The LOD profiles are sorted by chromosome
  and position.)
}

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=c(2,5))}
fake.bc <- calc.genoprob(fake.bc, step=2)
qtl <- makeqtl(fake.bc, chr=c(2,5), pos=c(32.5, 17.5), what="prob")

out <- scanone(fake.bc, method="hk")

# refine QTL positions and keep LOD profiles
rqtl <- refineqtl(fake.bc, qtl=qtl, method="hk", keeplodprofile=TRUE)

# plot the LOD profiles
plotLodProfile(rqtl)

# add the initial scan results, for comparison
plot(out, add=TRUE, chr=c(2,5), col="red")
}

\seealso{ \code{\link{refineqtl}},
  \code{\link{makeqtl}}, \code{\link{scanqtl}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\references{
  Zeng  Z.-B., Liu, J., Stam, L. F., Kao, C.-H., Mercer, J. M. and
  Laurie, C. C. (2000) Genetic architecture of a morphological shape
  difference between two Drosophila species. \emph{Genetics} \bold{154},
  299--310.
}

\keyword{hplot}

\name{plotPXG}
\alias{plotPXG}

\title{Plot phenotypes versus marker genotypes}

\description{
  Plot the phenotype values versus the genotypes at a marker or markers.
}

\usage{
plotPXG(x, marker, pheno.col=1, jitter=1, infer=TRUE,
         pch, ylab, main, col, \dots)
}

\arguments{
  \item{x}{An object of class \code{cross}.  See
    \code{\link{read.cross}} for details. }
  \item{marker}{Marker name (a character string; can be a vector).}
  \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  One may also give a character string matching
   a phenotype name. Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
  \item{jitter}{A positive number indicating how much to spread out the
    points horizontally.  (Larger numbers correspond to greater spread.)}
  \item{infer}{If TRUE, missing genotypes are filled in with a single
    random imputation and plotted in red; if FALSE, only individuals
    typed at the specified marker are plotted.}
  \item{pch}{Plot symbol.}
  \item{ylab}{Label for y-axis.}
  \item{main}{Main title for the plot.  If missing, the names of the
    markers are used.}
  \item{col}{A vector of colors to use for the confidence intervals
    (optional).}
  \item{\dots}{Passed to \code{plot}.}
}

\details{
  Plots the phenotype data against the genotypes at the specified
  marker.  If infer=TRUE, the genotypes of individuals that were not
  typed is inferred based the genotypes at linked markers via a single
  imputation from \code{\link{sim.geno}}; these points are plotted
  in red.  For each genotype, the phenotypic mean is plotted, with error
  bars at \eqn{\pm}{+/-} 1 SE.
}

\value{
A data.frame with initial columns the marker genotypes, then the
phenotype data, then a column indicating whether any of the marker
genotypes were inferred (1=at least one genotype inferred, 0=none were
inferred).
}

\examples{
data(listeria)
mname <- find.marker(listeria, 5, 28) # marker D5M357
plotPXG(listeria, mname)

mname2 <- find.marker(listeria, 13, 26) # marker D13Mit147
plotPXG(listeria, c(mname, mname2))
plotPXG(listeria, c(mname2, mname))

# output of the function contains the raw data
output <- plotPXG(listeria, mname)
head(output)

# another example
data(fake.f2)
mname <- find.marker(fake.f2, 1, 37) # marker D1M437
plotPXG(fake.f2, mname)

mname2 <- find.marker(fake.f2, "X", 14) # marker DXM66
plotPXG(fake.f2, mname2)

plotPXG(fake.f2, c(mname,mname2))
plotPXG(fake.f2, c(mname2,mname))

}

\seealso{ \code{\link{find.marker}}, \code{\link{effectplot}},
  \code{\link{find.flanking}}, \code{\link{effectscan}} }


\author{Karl W Broman, \email{broman@wisc.edu}; Brian Yandell }

\keyword{hplot}

\name{plot.scanoneboot}
\alias{plot.scanoneboot}

\title{Plot results of bootstrap for QTL position}

\description{
  Plot a histogram of the results of a nonparametric bootstrap to assess
  uncertainty in QTL position.
}

\usage{
\method{plot}{scanoneboot}(x, \dots)
}

\arguments{
  \item{x}{An object of class \code{"scanoneboot"}, as output by
    \code{\link{scanoneboot}}.}
  \item{\dots}{Passed to the function \code{\link[graphics]{hist}} when it
    is called.}
}
\value{None.}

\details{
  The function plots a histogram of the bootstrap results obtained by
  \code{\link{scanoneboot}}.  Genetic marker locations are
  displayed by vertical lines at the bottom of the plot.
}

\examples{
data(fake.f2)
fake.f2 <- calc.genoprob(fake.f2, step=1)

\dontrun{out.boot <- scanoneboot(fake.f2, chr=13, method="hk")}
\dontshow{out.boot <- scanoneboot(fake.f2, chr=13, method="hk", n.boot=50)}

summary(out.boot)
plot(out.boot)
}

\seealso{ \code{\link{scanone}},
  \code{\link{summary.scanoneboot}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{pull.argmaxgeno}
\alias{pull.argmaxgeno}

\title{Pull out the results of the Viterbi algorithm from a cross}

\description{
  Pull out the results of \code{\link{argmax.geno}} from a cross as a matrix.
}

\usage{pull.argmaxgeno(cross, chr, include.pos.info=FALSE, rotate=FALSE)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{include.pos.info}{If TRUE, include columns with marker name,
 chromosmoe ID, and cM position.  (If \code{include.pos.info=TRUE}, we
 take \code{rotate=TRUE}.)}
 \item{rotate}{If TRUE, return matrix with individuals as columns and
 positions as rows.  If FALSE, rows correspond to individuals.}
}

\value{
  A matrix containing numeric indicators of the inferred genotypes.
  Multiple chromosomes are pasted together.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
listeria <- argmax.geno(listeria, step=1, stepwidth="max")
amg <- pull.argmaxgeno(listeria, chr=c(5,13), include.pos.info=TRUE, rotate=TRUE)
amg[1:5,1:10]
}

\seealso{ \code{\link{pull.geno}}, \code{\link{pull.genoprob}},
  \code{\link{pull.draws}}, \code{\link{argmax.geno}} }

\keyword{utilities}

\name{transformPheno}
\alias{transformPheno}

\title{Transformation of the phenotypes in a cross object}

\description{
  Transform phenotypes in a cross object; by default use a logarithmic
  transformation, though any function may be used.
}

\usage{
transformPheno(cross, pheno.col=1, transf=log, \dots)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
 \item{pheno.col}{
   A vector of numeric indices or character strings (indicating phenotypes by
   name) of phenotypes to be transformed.
 }
 \item{transf}{The function to use in the transformation.}
 \item{\dots}{Additional arguments, to be passed to \code{transf}.}
}

\value{
  The input cross object with the transformed phenotypes
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{ \code{\link{mqmscan}}, \code{\link{scanone}} }

\examples{
data(multitrait)

# Log transformation of all phenotypes
multitrait.log <- transformPheno(multitrait, pheno.col=1:nphe(multitrait))

# Square-root transformation of all phenotypes
multitrait.sqrt <- transformPheno(multitrait, pheno.col=1:nphe(multitrait),
                                  transf=sqrt)
}

\keyword{ utilities }

\name{pull.geno}
\alias{pull.geno}

\title{Pull out the genotype data from a cross}

\description{
  Pull out the genotype data from a cross object, as a single big matrix.
}

\usage{pull.geno(cross, chr)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
}

\value{
  A matrix of size n.ind x tot.mar.  The raw genotype data in the input
  cross object, with the chromosomes pasted together.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
dat <- pull.geno(listeria)

# image of the genotype data
image(1:ncol(dat),1:nrow(dat),t(dat),ylab="Individuals",xlab="Markers",
      col=c("red","yellow","blue","green","violet"))
abline(v=cumsum(c(0,nmar(listeria)))+0.5)
abline(h=nrow(dat)+0.5)
}

\seealso{ \code{\link{pull.pheno}}, \code{\link{pull.map}}
  \code{\link{pull.draws}}, \code{\link{pull.genoprob}},
  \code{\link{pull.argmaxgeno}} }

\keyword{utilities}

\name{summary.ripple}
\alias{summary.ripple}

\title{Print summary of ripple results}

\description{
  Print marker orders, from the output of the function \code{ripple},
  for which the log10 likelihood relative to the initial order is above
  a specified cutoff.
}

\usage{
\method{summary}{ripple}(object, lod.cutoff = -1, \dots)
}

\arguments{
 \item{object}{An object of class \code{ripple}, the output of the
   function \code{\link{ripple}}.}
 \item{lod.cutoff}{Only marker orders with LOD score (relative to the
   initial order) above this cutoff will be displayed.  For output of
   \code{\link{ripple}} in the case of minimization of the number
   of obligate crossovers, we double this argument and treat it as a
   cutoff for the number of obligate crossovers.}
 \item{\dots}{Ignored at this point.}
}

\value{
  An object of class \code{summary.ripple}, whose rows correspond to
  marker orders with likelihood (or number of obligate crossovers)
  within some cutoff of the initial order. If no marker order, other
  than the initial one, has likelihood within the specified range, the
  initial and next-best orders are returned.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{ % I don't run the following, since they are run in ripple.Rd
\dontrun{data(badorder)
rip1 <- ripple(badorder, 1, 7)
summary(rip1)

rip2 <- ripple(badorder, 1, 2, method="likelihood")
summary(rip2)

badorder <- switch.order(badorder, 1, rip2[2,])
}}

\seealso{ \code{\link{ripple}}, \code{\link{est.map}},
  \code{\link{est.rf}} }

\keyword{print}

\name{typingGap}
\alias{typingGap}

\title{Maximum distance between genotyped markers}

\description{
  Calculates, for each individual on each chromosome, the maximum
  distance between genotyped markers.
}

\usage{
typingGap(cross, chr, terminal=FALSE)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
   This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
\item{terminal}{If TRUE, just look at terminal typing gaps (from the
 terminal markers to the first typed marker).}
}

\details{
  We consider not just the distances between internal genotypes, but
  also distances from the beginning of the chromosome to the first typed
  marker, and similarly for the end of the chromosome.  (The start and end
  of a chromosome are taken to be the locations of the initial and final
  markers.)  If \code{terminal=TRUE}, we look only at those beginning
  and end distances.
}

\value{
  A matrix with rows corresponding to individuals and columns
  corresponding to chromosomes.  (If there is just one chromosome, it is
  a numeric vector rather than a matrix.)
}

\examples{
data(hyper)
plot(typingGap(hyper, chr=5),
     ylab="Maximum gap between typed markers (cM)",
     ylim=c(0, diff(range(pull.map(hyper,chr=5)[[1]]))))

plot(typingGap(hyper, chr=4),
     ylab="Maximum gap between typed markers (cM)",
     ylim=c(0, diff(range(pull.map(hyper,chr=4)[[1]]))))

plot(typingGap(hyper, chr=4, terminal=TRUE),
     ylab="Maximum gap between chr end and typed marker (cM)",
     ylim=c(0, diff(range(pull.map(hyper,chr=4)[[1]]))))
}

\seealso{ \code{\link{ntyped}},
          \code{\link{nmissing}}, \code{\link{locateXO}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{utilities}

\name{convert.scantwo}
\alias{convert.scantwo}

\title{Convert output from scantwo for R/qtl version 1.03 and earlier}

\description{
  Convert the output from scantwo from the format used in R/qtl version
  1.03 and earlier to that used in version 1.04 and later.
}

\usage{
\method{convert}{scantwo}(object, \dots)
}

\arguments{
 \item{object}{Output from the function \code{\link{scantwo}}, for
   R/qtl version 1.03 and earlier.}
 \item{\dots}{Ignored at this point.}
}

\value{
  The same scanone output, but revised for use with R/qtl version 1.03
  and later.
}

\details{
  Previously, the output from \code{\link{scantwo}} contained the
  full and interaction LOD scores.  In R/qtl version 1.04 and later,
  the output contains the LOD scores from the full and
  additive QTL models.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
\dontrun{out2.new <- convert(out2.old)}
}

\seealso{ \code{\link{scantwo}}, \code{\link{convert.scanone}} }

\keyword{manip}

\name{mqmplot.cistrans}
\alias{mqmplot.cistrans}

\title{cis-trans plot}

\description{
Plot results for a genomescan using a multiple-QTL model. With genetic location
for the traits it is possible to show cis- and trans- locations, and detect
trans-bands
}

\usage{
mqmplot.cistrans(result, cross, threshold=5, onlyPEAK=TRUE,
                 highPEAK=FALSE, cisarea=10, pch=22, cex=0.5,
                 verbose=FALSE, \dots)
}

\arguments{
  \item{result}{An object of class \code{mqmmulti}. See \code{\link{mqmscanall}} for details.}
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{threshold}{ Threshold value in LOD, Markers that have a
        LOD score above this threshold are plotted as small squares
        (see \code{pch} parameter). The markers with LODscores below
        this threshold are not visible
  }
  \item{onlyPEAK}{ Plot only the peak markers ? (TRUE/FALSE)
        (Peak markers are markers that have a QTL likelihood above
        \code{threshold} and higher than other markers in the same region)
  }
  \item{highPEAK}{ Highlight peak markers ? (TRUE/FALSE).
        When using this option peak markers (the marker with the
        highest LOD score in a region above the threshold gets
        an 25\% increase in size and is displayed in red)
  }
  \item{cisarea}{ Adjust the two green lines around the line y=x }
  \item{pch}{ What kind of character is used in plotting of the figure (Default: 22, small square) }
  \item{cex}{ Size of the points plotted (default to 0.5 half of the original size)}
  \item{verbose}{If TRUE, give verbose output}
  \item{\dots}{ Extra parameters will be passed to points }
}

%\details{
%
%}

\value{
        Plotting routine, so no return
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
        \itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
        }
}

\examples{
data(multitrait)
\dontshow{multitrait <- subset(multitrait, chr=1,
                               ind=!apply(multitrait$pheno,
                                          1, function(a) any(is.na(a))))}
data(locations)
multiloc <- addloctocross(multitrait,locations)
multiloc <- calc.genoprob(multiloc)
results <- scanall(multiloc, method="hk")
mqmplot.cistrans(results, multiloc, 5, FALSE, TRUE)
}

\keyword{ hplot }

\name{mqmplot.circle}
\alias{mqmplot.circle}
\title{Circular genome plot for MQM}

\description{
  Circular genome plot - shows QTL locations and relations.
}

\usage{
mqmplot.circle(cross,result,highlight=0,spacing=25, interactstrength=2,
               axis.legend=TRUE, col.legend=FALSE, verbose=FALSE, transparency=FALSE)
}

\arguments{
  \item{cross}{An object of class \code{cross} with optionally phenotype locations. See
    \code{\link{read.cross}} for details on reading in cross objects, and optionally
    \code{\link{addloctocross}} for adding phenotype locations.
  }
  \item{result}{An object of class \code{mqmmulti} or \code{scanone}.
    See \code{\link{mqmscanall}} \code{\link{scanone}} for details.
  }
  \item{highlight}{ With a mqmmulti object, highlight this phenotype (value between
    one and the number of results in the mqmmultiobject)
  }
  \item{interactstrength}{ When highlighting a trait, consider interactions
  significant they have a change of more than interactstrength*SEs. A higher value
  will show less interactions.  However the interactions reported at higher
  interactstrength values will generaty be more reliable.
  }
  \item{spacing}{ User defined spacing between chromosomes in cM }
  \item{axis.legend}{ When set to FALSE, suppresses the legends. (defaults to plotting legends besides the axis.  }
  \item{col.legend}{ With a mqmmulti object, plots a legend for the non-highlighed version }
  \item{transparency}{ Use transparency when drawing the plots (defaults to no transparency) }
  \item{verbose}{ Be verbose }
}

\details{
  Depending on the input of the result being either \code{\link{scanone}} or \code{mqmmulti} a different plot is drawn.
  If model information is present from \code{\link{mqmscan}} (by setting cofactors) This will be highlighted in
  red (see example).
  If phenotypes have genetic locations (e.g. eQTL) they will be plotted on the genome otherwise
  phenotypes will be plotted in the middle of the circle (with a small offset)
  Locations can be added by using the \code{\link{addloctocross}} function.
}

\value{
    Plotting routine, no return
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
    \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
    }
}

\examples{
data(multitrait)
\dontshow{multitrait$pheno <- multitrait$pheno[,1:3]}
data(locations)
\dontshow{locations <- locations[1:3,]}

multifilled <- fill.geno(multitrait)                    # impute missing genotypes
multicof <- mqmsetcofactors(multitrait,10)              # create cofactors
multiloc <- addloctocross(multifilled,locations)        # add phenotype information to cross
multires <- mqmscanall(multifilled,cofactors=multicof)  # run mqmscan for all phenotypes

#Basic mqmmulti, color = trait, round circle = significant
mqmplot.circle(multifilled,multires)

#mqmmulti with locations of traits in multiloc
mqmplot.circle(multiloc,multires)

#mqmmulti with highlighting
mqmplot.circle(multitrait,multires,highlight=3)

#mqmmulti with locations of traits in multiloc and highlighting
mqmplot.circle(multiloc,multires,highlight=3)
}

\keyword{ utilities }

\name{listeria}
\alias{listeria}
\docType{data}

\title{Data on Listeria monocytogenes susceptibility}

\description{
  Data from an experiment on susceptibility to \emph{Listeria
  monocytogenes} infection in the mouse.
}

\usage{data(listeria)}

\format{
  An object of class \code{cross}.  See \code{\link{read.cross}}
  for details.
}

\details{
  There are 120 F2 individuals typed at 133 markers, including 2 on the
  X chromosome, with one phenotype.

  The phenotype is the survival time (in hours) following infection.
  Mice with phenotype 264 hours may be considered to have recovered
  from the infection. See the references below.
}

\references{
  Boyartchuk, V. L., Broman, K. W., Mosher, R. E., D'Orazio
  S. E. F., Starnbach, M. N. and Dietrich, W. F. (2001) Multigenic
  control of \emph{Listeria monocytogenes} susceptibility in
  mice. \emph{Nature Genetics} \bold{27}, 259--260.

  Broman,  K. W. (2003) Mapping quantitative trait loci in the case of a
  spike in the phenotype distribution. \emph{Genetics} \bold{163},
  1169--1175.
}

\source{
  Victor Boyartchuk and William Dietrich (Department of Genetics,
  Harvard Medical School and Howard Hughes Medical Institute)
}

\examples{
data(listeria)
\dontshow{listeria <- subset(listeria,chr=c(1,5,6,13,15), ind=!is.na(listeria$pheno[,1]))}
# Summaries
summary(listeria)
plot(listeria)

# Take log of phenotype
listeria$pheno[,1] <- log2(listeria$pheno[,1])
plot(listeria)

# Genome scan with a two-part model, using log survival
listeria <- calc.genoprob(listeria, step=2)
out <- scanone(listeria, model="2part", method="em",
               upper=TRUE)

# Summary of the results
summary(out, thr=c(5,3,3), format="allpeaks")

# Plot LOD curves for interesting chromosomes
#     (The two-part model gives three LOD scores)
plot(out, chr=c(1,5,6,13,15), lodcolumn=1:3,
     lty=1, col=c("black","red","blue"))
}

\seealso{ \code{\link{fake.bc}}, \code{\link{fake.f2}},
  \code{\link{fake.4way}}, \code{\link{hyper}},
  \code{\link{bristle3}}, \code{\link{bristleX}} }

\keyword{datasets}

\name{max.scanone}
\alias{max.scanone}

\title{Maximum peak in genome scan}

\description{
  Print the row of the output from \code{\link{scanone}} that
  corresponds to the maximum LOD, genome-wide.
}

\usage{
\method{max}{scanone}(object, chr, lodcolumn=1, na.rm=TRUE, \dots)
}

\arguments{
 \item{object}{An object of the form output by the function
   \code{\link{scanone}}: a data.frame whose third column is the
   LOD score.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{lodcolumn}{An integer, indicating which of the LOD score columns
   should be considered in pulling out the peak (these are indexed 1,
   2, \dots).}
 \item{na.rm}{A logical indicating whether missing values should be
   removed.}
 \item{\dots}{Ignored.}
}

\value{
  An object of class \code{summary.scanone}, to be printed by
  \code{print.summary.scanone}.  This is a data.frame with one row,
  corresponding to the maximum LOD peak either genome-wide or for the
  particular chromosome specified.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
\dontshow{listeria <- subset(listeria,chr=c(5,13),ind=!is.na(listeria$pheno[,1]))}
listeria <- calc.genoprob(listeria, step=2.5)
out <- scanone(listeria, model="2part", upper=TRUE)
# Maximum peak for LOD(p,mu)
max(out)

# Maximum peak for LOD(p,mu) on chr 5
max(out,chr=5)

# Maximum peak for LOD(p,mu) on chromosomes other than chr 13
max(out,chr="-13")

# Maximum peak for LOD(p)
max(out, lodcolumn=2)

# Maximum peak for LOD(mu)
max(out, lodcolumn=3)
}

\seealso{ \code{\link{scanone}}, \code{\link{plot.scanone}},
          \code{\link{summary.scanone}} }

\keyword{print}

\name{arithscanperm}
\alias{+.scanoneperm}
\alias{-.scanoneperm}
\alias{+.scantwoperm}
\alias{-.scantwoperm}

\title{Arithmetic Operators for permutation results}

\description{
  Add or subtract LOD scores in permutation results from
  \code{\link{scanone}} or \code{\link{scantwo}}.
}

\usage{
\special{perm1+perm2}
\special{perm1-perm2}
}

\arguments{
  \item{perm1, perm2}{Permutation results from
    \code{\link{scanone}} or \code{\link{scantwo}}, on the
    same set of chromosomes and markers.}
}

\details{
  This is used to calculate the sum or difference of LOD scores of two
  sets of permutation results from \code{\link{scanone}} or
  \code{\link{scantwo}}.  One must be careful to ensure that the
  permutations are perfectly linked, which
  will require the use of \code{\link[base:Random]{set.seed}}.
}

\value{
  The same data structure as the input objects, with LOD scores
  added or subtracted.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=1:2)}
fake.bc <- calc.genoprob(fake.bc, step=2.5)

# covariates
ac <- pull.pheno(fake.bc, c("sex","age"))
ic <- pull.pheno(fake.bc, "sex")

# set seed
theseed <- round(runif(1, 1, 10^8))
set.seed(theseed)

# permutations with additive but not the interactive covariate
\dontrun{operm.acovar <- scanone(fake.bc, addcovar=ac, n.perm=1000)
}\dontshow{operm.acovar <- scanone(fake.bc, pheno.col=1, method="hk", addcovar=ac, n.perm=100)
}

# re-set the seed
set.seed(theseed)

# permutations with interactive covariate
\dontrun{operm.icovar <- scanone(fake.bc, addcovar=ac, intcovar=ic,
                      n.perm=1000)
}\dontshow{operm.icovar <- scanone(fake.bc, pheno.col=1, addcovar=ac,
                      intcovar=ic, method="hk", n.perm=100)
}

# permutation results for the QTL x covariate interaction
operm.gxc <- operm.icovar - operm.acovar

# LOD thresholds
summary(operm.gxc)
}

\keyword{arith}

\name{c.scanone}
\alias{c.scanone}
\alias{cbind.scanone}

\title{Combine columns from multiple scanone results}

\description{
  Concatenate the columns from different runs of
  \code{\link{scanone}}.
}

\usage{
\method{c}{scanone}(\dots, labels)
\method{cbind}{scanone}(\dots, labels)
}

\arguments{
 \item{\dots}{A set of objects of class \code{scanone}. (This can also
 be a list of \code{scanone} objects.) These are
   the results from  \code{\link{scanone}} (with \code{n.perm=0}),
   generally run with different phenotypes or methods.  All must
   conform with each other, meaning that
   \code{\link{calc.genoprob}} and/or \code{\link{sim.geno}}
   were run with the same values for \code{step} and \code{off.end} and
   with data having the same genetic map.}
 \item{labels}{A vector of character strings, of length 1 or of the same
   length as the input, to be appended to the column names
   in the output.}
}

\value{The concatenated input, as a \code{scanone} object.}

\details{
  The aim of this function is to concatenate the results from multiple
  runs \code{\link{scanone}}, generally for
  different phenotypes and/or methods, to be used in parallel with
  \code{\link{summary.scanone}}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
fake.f2 <- calc.genoprob(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
out.hk <- scanone(fake.f2, method="hk")
out.np <- scanone(fake.f2, model="np")

out <- c(out.hk, out.np, labels=c("hk","np"))
plot(out, lod=1:2, col=c("blue", "red"))
}

\seealso{ \code{\link{summary.scanone}},
  \code{\link{scanone}}, \code{\link{cbind.scanoneperm}} }

\keyword{manip}

\name{plot.rfmatrix}
\alias{plot.rfmatrix}

\title{Plot recombination fractions or LOD scores for a single marker}

\description{
  Plot a slice (corresponding to a single marker) through the pairwise
  recombination fractions or LOD scores calculated by
  \code{\link{est.rf}} and extracted with \code{\link{pull.rf}}.
}

\usage{
\method{plot}{rfmatrix}(x, marker, \dots)
}

\arguments{
 \item{x}{An object of class \code{rfmatrix}, as output by \code{\link{pull.rf}}.}
 \item{marker}{A single marker name, as a character string.}
 \item{\dots}{Optional arguments passed to \code{\link{plot.scanone}}.}
}

\value{
An object of class \code{"scanone"} (as output by \code{\link{scanone}},
and which may be summarized by \code{\link{summary.scanone}} or plotted
with \code{\link{plot.scanone}}), containing the estimated recombination
fractions or LOD scores for the input marker against all others.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=5)}
fake.f2 <- est.rf(fake.f2)
marker <- markernames(fake.f2, chr=5)[6]
lod <- pull.rf(fake.f2, "lod")
plot(lod, marker, bandcol="gray70")
}

\seealso{ \code{\link{pull.rf}}, \code{\link{est.rf}}, \code{\link{plotRF}} }


\keyword{hplot}

\name{rescalemap}
\alias{rescalemap}

\title{Rescale genetic maps}

\description{
  Rescale a genetic map by multiplying all positions by a constant
}

\usage{
rescalemap(object, scale=1e-6)
}
\arguments{
 \item{object}{An object of class \code{cross} (see
   \code{\link{read.cross}} for details) or \code{map} (see
   \code{\link{sim.map}} for details).}
 \item{scale}{Scale factor by which all positions will be multiplied.}
}

\details{
  This function is included particularly for the case that map positions
  in a cross object were provided in basepairs and one wishes to quickly
  convert them to Mbp or some other approximation of cM distances.  (In
  the mouse, 1 cM is approximation 2 Mbp, so one might use
  \code{scale=5e-7} in this function.)
}

\value{
If the input is a \code{map} object, a \code{map} object is returned; if
the input is a \code{cross} object, a \code{cross} object is returned.
In either case, the positions of markers are simply multiplied by
\code{scale}.

}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
rescaled <- rescalemap(hyper, scale=2)
plotMap(hyper, rescaled)
}


\seealso{ \code{\link{replace.map}},
  \code{\link{est.map}} }

\keyword{utilities}

\name{replacemap.scanone}
\alias{replacemap.scanone}

\title{Replace the genetic map in QTL mapping results with an alternate map}

\description{
  Replace the positions of LOD scores in output from
  \code{\link{scanone}} with values
  based on an alternative map (such as a physical map), with
  pseudomarker locations determined by linear interpolation.
}

\usage{
\method{replacemap}{scanone}(object, map)
}

\arguments{
 \item{object}{An object of class \code{"scanone"}, as output by the
 function \code{\link{scanone}}.}
 \item{map}{A list containing the alternative genetic map.  All
 chromosomes in \code{object} should have corresponding chromosomes in
 \code{map}, and markers must be in the same order in the two maps.
 There must be at least two markers on each chromosome in \code{map}
 that appear in \code{object}.}
}

\value{The input \code{object} with the positions of LOD scores
  revised to match those in the input \code{map}.}

\details{
  The positions of pseudomarkers are determined by linear interpolation
  between markers.  In the case of pseudomarkers beyond the ends of the
  terminal markers on chromosomes, we use the overall lengths of the
  chromosome in \code{object} and \code{map} to determine the new
  spacing.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
origmap <- pull.map(fake.f2)
newmap <- est.map(fake.f2)
fake.f2 <- replacemap(fake.f2, newmap)
fake.f2 <- calc.genoprob(fake.f2, step=2.5)
out <- scanone(fake.f2, method="hk")
out.rev <- replacemap(out, origmap)
}

\seealso{ \code{\link[=replace.map]{replacemap.cross}},
  \code{\link{est.map}}, \code{\link{replacemap.scantwo}} }

\keyword{manip}

\name{nmar}
\alias{nmar}

\title{Determine the numbers of markers on each chromosome}

\description{
  Determine the number of markers on each chromosome in a cross
  or map object.
}

\usage{
nmar(object)
}

\arguments{
 \item{object}{An object of class \code{cross} (see
   \code{\link{read.cross}} for details) or \code{map} (see
 \code{\link{sim.map}} for details).}
}

\value{
  A vector with the numbers of markers on each chromosome in the input.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
nmar(fake.f2)
map <- pull.map(fake.f2)
nmar(map)
}

\seealso{ \code{\link{read.cross}}, \code{\link{plot.cross}},
  \code{\link{summary.cross}},
  \code{\link{nind}},
  \code{\link{nchr}},
  \code{\link{totmar}},
  \code{\link{nphe}} }

\keyword{print}

\name{pull.pheno}
\alias{pull.pheno}

\title{Pull out phenotype data from a cross}

\description{
  Pull out selected phenotype data from a cross object, as a data frame
  or vector.
}

\usage{pull.pheno(cross, pheno.col)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{pheno.col}{A vector specifying which phenotypes to keep or discard.
   This may be a logical vector, a numeric vector, or a vector
   of character strings (for the phenotype names). If missing, the
   entire set of phenotypes is output.}
}

\value{
  A data.frame with columns specifying phenotypes and rows specifying individuals.
  If there is just one phenotype, a vector (rather than a data.frame) is returned.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
pull.pheno(listeria, "sex")
}

\seealso{ \code{\link{pull.geno}}, \code{\link{pull.map}} }

\keyword{utilities}

\name{mqmscan}
\alias{mqmscan}

\title{Genome scan with a multiple QTL model (MQM)}

\description{
  Genome scan with a multiple QTL model.
}

\usage{
mqmscan(cross, cofactors=NULL, pheno.col = 1,
  model=c("additive","dominance"), forceML=FALSE,
  cofactor.significance=0.02, em.iter=1000,
  window.size=25.0, step.size=5.0,
  logtransform = FALSE, estimate.map = FALSE,
  plot=FALSE, verbose=FALSE, outputmarkers=TRUE,
  multicore=TRUE, batchsize=10, n.clusters=1, test.normality=FALSE,off.end=0
  )
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{cofactors}{ List of cofactors to be analysed as cofactors in backward elimination
    procedure when building the QTL model. See \code{\link{mqmsetcofactors}} on how-to manually set cofactors
    for backward elimination. Or use \code{\link{mqmautocofactors}} for automatic selection of cofactors. Only
    three kind of (integer) values are allowed in the cofactor list. (0: no cofactor at this marker, 1: Use
    this marker as an additive cofactor, 2: Use this marker as an sexfactor (Dominant cofactor))
  }
  \item{pheno.col}{ Column number in the phenotype matrix which should be
    used as the phenotype.  This can be a vector of integers; One may also
    give a character strings matching the phenotype names.  Finally, one
    may give a numeric vector of phenotypeIDs. This should consist of
    integers with  0 < value < no. phenotypes.
  }
  \item{model}{ When scanning for QTLs should haplotype dominance be
    considered in an F2 intercross. Using the dominance model we scan
    for additive effects but also allow an additional effect where AA+AB
    versus BB and AA versus AB+BB. This setting is ignored for BC and RIL
    populations
  }
  \item{forceML}{ Specify which statistical method to use to estimate
    variance components to use when QTL modeling and mapping. Default
    usage is the Restricted maximum likelihood approach (REML). With this
    option a user can disable REML and use maximum likelihood.
  }
  \item{cofactor.significance}{ Significance level at which a cofactor is
    considered significant. This is estimated using an analysis of
    deviance, and compared to the level specified by the user. The
    cofactors that dont reach this level of statistical significance
    are NOT used in the mapping stage. Value between 0 and 1 }
  \item{em.iter}{ Maximum number of iterations for the EM algorithm to converge }
  \item{window.size}{ Window size for mapping QTL locations,
    this parameter is used in the interval mapping stage. When
    calculating LOD scores at a genomic position all cofactors
    within window.size are dropped to estimate the (unbiased)
    effect of the location under interest.
  }
  \item{step.size}{ Step size used in interval mapping. A lower
    step.size parameter increases the number of output points,
    this creates a smoother QTL profile
  }
  \item{off.end}{Distance (in cM) past the terminal markers on each
    chromosome to which the genotype simulations will be carried.
  }
  \item{logtransform}{ Indicate if the algorithm should do a log
    transformation on the trait data in the pheno.col
  }
  \item{estimate.map}{ Should Re-estimation of the marker locations
    on the genetic map occur before mapping QTLs. This method is
    deprecated rather use the \code{\link{est.map}} function in R/qtl.
    This is because no map is returned into the crossobject.
    The old map remains in the cross object.
    }
  \item{plot}{ plot the results (default FALSE)}
  \item{verbose}{ verbose output}
  \item{outputmarkers}{If TRUE (the default), the results include the
  marker locations as well as along a grid of pseudomarkers; if FALSE,
  the results include only the grid positions.}
  \item{multicore}{Use multicore (if available)}
  \item{batchsize}{Number of traits being analyzed as a batch.}
  \item{n.clusters}{Number of child processes to split the job into.}
  \item{test.normality}{If TRUE, test whether the phenotype follows a
    normal distribution via \code{\link{mqmtestnormal}}.
  }
}

%\details{
%}

\value{
When scanning a single phenotype the function returns a \code{\link{scanone}}
object.

The object contains a matrix of three columns for LOD scores, information content
and LOD*information content with pseudo markers sorted in increasing
order. For more information on the scanone object see: \code{\link{scanone}}
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\note{
The resulting scanone object itself can be visualized using the standard R/qtl
plotting routines (\code{\link{plot.scanone}}) or specialized function to show
the mqm model (\code{\link{mqmplot.singletrait}}) and QTL profile. If cofactors
were specified the QTL model used in scanning is also returned as a named
attribute of the scanone object called mqmmodel. It can be extracted from the
resulting scanone object by using the \code{\link{mqmgetmodel}} function or the
\code{\link{attr}} function.

Also note the \code{estimate.map} parameter does not return
its re-estimated genetic map, altough it is used internally.  When scanning
multiple genotypes a \code{mqmmulti} object is created.  This object is just a
list composed of scanone objects. The results for a single trait can be
obtained from the \code{mqmmulti} object, in scanone format.
}

\seealso{
  \itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
% \input{"inst/doc/Sources/MQM/mqm/standard_example.txt"}
data(map10)                    # Genetic map modeled after mouse

# simulate a cross (autosomes 1-10)
qtl <- c(3,15,1,0)             # QTL model: chr, pos'n, add've & dom effects
cross <- sim.cross(map10[1:10],qtl,n=100,missing.prob=0.01)

# MQM
crossaug <- mqmaugment(cross)  # Augmentation
cat(crossaug$mqm$Nind,'real individuals retained in dataset',
    crossaug$mqm$Naug,'individuals augmented\n')

result <- mqmscan(crossaug)    # Scan

# show LOD interval of the QTL on chr 3
lodint(result,chr=3)
% -----^^ inst/doc/Sources/MQM/mqm/standard_example.txt ^^-----
}

\keyword{ models }

\name{calc.penalties}
\alias{calc.penalties}

\title{Calculate LOD penalties}

\description{
  Derive penalties for the penalized LOD scores (used by
  \code{\link{stepwiseqtl}}) on the basis of permutation results
  from a two-dimensional, two-QTL scan (obtained by \code{\link{scantwo}}).
}

\usage{calc.penalties(perms, alpha=0.05, lodcolumn)}

\arguments{
 \item{perms}{Permutation results from \code{\link{scantwo}}.}
 \item{alpha}{Significance level.}
 \item{lodcolumn}{If the scantwo permutation results contain LOD scores for
    multiple phenotypes, this argument indicates which to use in the
    summary.  This may be a vector.  If missing, penalties for all
 phenotypes are calculated.}
}

\details{
  Thresholds derived from \code{\link{scantwo}} permutations (that
  is, for a two-dimensional, two-QTL genome scan) are used to calculate
  penalties on main effects and interactions.

  The main effect penalty is the 1-\code{alpha} quantile of the null
  distribution of the genome-wide maximum LOD score from a single-QTL
  genome scan (as with \code{\link{scanone}}).

  The "heavy" interaction penalty is the 1-\code{alpha} quantile of
  the null distribution of the maximum interaction LOD score (that is,
  the \eqn{\log_{10}}{log10} likelihood ratio comparing the best model
  with two interacting QTL to the best model with two additive QTL) from
  a two-dimensional, two-QTL genome scan (as with
  \code{\link{scantwo}}).

  The "light" interaction penality is the difference between the
  \code{"fv1"} threshold from the \code{\link{scantwo}}
  permutations (that is, the 1-\code{alpha} quantile of the LOD score
  comparing the best model with two interacting QTL to the best
  single-QTL model) and the main effect penalty.

  If the permutations results were obtained with \code{perm.Xsp=TRUE},
  to give X-chr-specific results, six penalties are calculated: main
  effect for autosomes, main effect for X chr, heavy penalty on A:A
  interactions, light penalty on A:A interactions, penalty on A:X
  interactions, and penalty on X:X interactions.
}

\value{
  Vector of three values indicating the penalty on main
  effects and heavy and light penalties on interactions,
  or a matrix of such results, with each row corresponding to a
  different phenotype.

  If the input permutations are X-chromosome-specific, the result has
  six values: main effect for autosomes, main effect for X chr, heavy
  penalty on A:A interactions, light penalty on A:A interactions,
  penalty on A:X interactions, and penalty on X:X interactions.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
fake.f2 <- calc.genoprob(fake.f2, step=5)
out.2dim <- scantwo(fake.f2, method="hk")

# permutations
\dontshow{permo.2dim <- scantwo(fake.f2, method="hk", n.perm=2)}
\dontrun{permo.2dim <- scantwo(fake.f2, method="hk", n.perm=1000)}
summary(permo.2dim, alpha=0.05)

# penalties
calc.penalties(permo.2dim)
}

\references{
  Manichaikul, A., Moon, J. Y., Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}, Yandell, B. S. and Broman,
  K. W. (2009) A model selection approach for the identification of
  quantitative trait loci in experimental crosses, allowing
  epistasis. \emph{Genetics}, \bold{181}, 1077--1086.
}

\seealso{ \code{\link{scantwo}}, \code{\link{stepwiseqtl}} }

\keyword{utilities}

\name{scantwo}
\alias{scantwo}

\title{Two-dimensional genome scan with a two-QTL model}

\description{
  Perform a two-dimensional genome scan with a two-QTL model, with
  possible allowance for covariates.
}

\usage{
scantwo(cross, chr, pheno.col=1, model=c("normal","binary"),
        method=c("em","imp","hk","mr","mr-imp","mr-argmax"),
        addcovar=NULL, intcovar=NULL, weights=NULL,
        use=c("all.obs", "complete.obs"),
        incl.markers=FALSE, clean.output=FALSE,
        clean.nmar=1, clean.distance=0,
        maxit=4000, tol=1e-4,
        verbose=TRUE, n.perm, perm.Xsp=FALSE, perm.strata=NULL,
        assumeCondIndep=FALSE, batchsize=250, n.cluster=1)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes for which LOD
   scores should be calculated.  This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  This can be a vector of integers; for methods
   \code{"hk"} and \code{"imp"} this can be considerably faster than doing
   them one at a time.  One may also give character strings matching
   the phenotype names.  Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{model}{The phenotype model: the usual normal model or a model for
   binary traits.}
 \item{method}{Indicates whether to use the
   the EM algorithm, imputation, Haley-Knott regression, or marker
   regression.  Marker regression is performed either by dropping
   individuals with missing genotypes (\code{"mr"}), or by first filling
   in missing data using a single imputation (\code{"mr-imp"}) or by the
   Viterbi algorithm (\code{"mr-argmax"}).}
 \item{addcovar}{Additive covariates.}
 \item{intcovar}{Interactive covariates (interact with QTL genotype).}
 \item{weights}{Optional weights of individuals.  Should be either NULL
   or a vector of length n.ind containing positive weights. Used only
   in the case \code{model="normal"}.}
 \item{use}{In the case that multiple phenotypes are selected to be
   scanned, this argument indicates whether to use all individuals,
   including those missing some phenotypes, or just those individuals
   that have data on all selected phenotypes.}
 \item{incl.markers}{If FALSE, do calculations only at points on an
   evenly spaced grid.  If \code{\link{calc.genoprob}} or
   \code{\link{sim.geno}} were run with
   \code{stepwidth="variable"} or \code{stepwidth="max"}, we force \code{incl.markers=TRUE}.}
 \item{clean.output}{If TRUE, clean the output with
   \code{\link{clean.scantwo}}, replacing LOD scores for pairs of
   positions that are not well separated with 0.  In permutations, this
   will be done for each permutation replicate.  This can be important
   for the case of \code{method="em"}, as there can be difficulty with
   algorithm convergence in these regions.}
 \item{clean.nmar}{If \code{clean.output=TRUE}, this is the number of
 markers that must separate two positions.}
 \item{clean.distance}{If \code{clean.output=TRUE}, this is the cM distance
 that must separate two positions.}
 \item{maxit}{Maximum number of iterations; used
   only with method \code{"em"}.}
 \item{tol}{Tolerance value for determining convergence; used only with
   method \code{"em"}.}
 \item{verbose}{If TRUE, display information about the progress of
   calculations.  For method \code{"em"}, if \code{verbose} is an integer
   above 1, further details on the progress of the algorithm will be
   displayed.}
 \item{n.perm}{If specified, a permutation test is performed rather than
   an analysis of the observed data.  This argument defines the number
   of permutation replicates.}
 \item{perm.Xsp}{If \code{n.perm} > 0, so that a permutation test will
   be performed, this indicates whether separate permutations should be
   performed for the autosomes and the X chromosome, in order to get an
   X-chromosome-specific LOD threshold.  In this case, additional
   permutations are performed for the X chromosome.}
 \item{perm.strata}{If \code{n.perm} > 0, this may be used to perform a
   stratified permutation test.  This should be a vector with the same
   number of individuals as in the cross data.  Unique values indicate
   the individual strata, and permutations will be performed within the
   strata.}
 \item{assumeCondIndep}{If TRUE, assume conditional independence of QTL
   genotypes given marker genotypes.  This is an approximation, but it
   may speed things up.}
 \item{batchsize}{The number of phenotypes (or permutations) to be run
   as a batch; used only for methods \code{"hk"} and \code{"imp"}.}
 \item{n.cluster}{If the package \code{snow} is available and
   \code{n.perm} > 0, permutations are run in parallel using this number
   of nodes.}
}

\details{
  Standard interval mapping (\code{method="em"}) and Haley-Knott
  regression (\code{method="hk"}) require that multipoint genotype probabilities are
  first calculated using \code{\link{calc.genoprob}}.  The
  imputation method uses the results of \code{\link{sim.geno}}.

  The method \code{"em"} is standard interval mapping by the EM algorithm
  (Dempster et al. 1977; Lander and Botstein 1989).  Marker regression
  (\code{method="mr"}) is simply linear regression of phenotypes on
  marker genotypes  (individuals with missing genotypes are
  discarded). Haley-Knott regression (\code{method="hk"}) uses the
  regression of phenotypes on multipoint genotype probabilities.  The
  imputation method (\code{method="imp"}) uses the pseudomarker
  algorithm described by Sen and Churchill (2001).

  Individuals with missing phenotypes are dropped.

  In the presence of covariates, the full model is
  \deqn{y = \mu + \beta_{q_1} + \beta_{q_2} + \beta_{q_1 \times q_2} +
    A \gamma + Z \delta_{q_1} + Z \delta_{q_2} + Z \delta_{q_1 \times q_2} +
    \epsilon}{y = m + b[q1] + b[q2] + b[q1 x q2] + A g + Z d[q1] + Z
    d[q2] + Z d[q1 x q2] + e}
  where \eqn{q_1}{q1} and \eqn{q_2}{q2} are the unknown QTL genotypes at two
  locations, \emph{A} is a matrix of covariates, and \emph{Z} is a
  matrix of covariates that interact with QTL genotypes.  The columns of
  \emph{Z} are forced to be contained in the matrix \emph{A}.

  The above full model is compared to the additive QTL model,
  \deqn{y = \mu + \beta_{q_1} + \beta_{q_2} + A \gamma + Z \delta_{q_1} + Z
    \delta_{q_2} + \epsilon}{y = m + b[q1] + b[q2] + A g + Z d[q1] + Z
    d[q2] + e}
  and also to the null model, with no QTL,
  \deqn{y = \mu + A \gamma + \epsilon}{y = m + A g + e}

  In the case that \code{n.perm} is specified, the R function
  \code{scantwo} is called repeatedly.

  For \code{model="binary"}, a logistic regression model is used.
}

\section{X chromosome}{
The X chromosome must be treated specially in QTL mapping.

As in \code{\link{scanone}}, if both males and females are
included, male hemizygotes are allowed to be different from female
homozygotes, and the null hypothesis must be changed in order to ensure
that sex- or pgm-differences in the phenotype do not results in spurious
linkage to the X chromosome.  (See the help file for
\code{\link{scanone}}.)

If \code{n.perm} is specified and \code{perm.Xsp=TRUE},
X-chromosome-specific permutations are performed, to obtain separate
thresholds for the regions A:A, A:X, and X:X.
}

\value{
  If \code{n.perm} is missing, the function returns a list with class
  \code{"scantwo"} and containing three components.  The first component
  is a matrix of dimension [tot.pos x tot.pos]; the upper triangle
  contains the LOD scores for the additive model, and the lower triangle
  contains the LOD scores for the full model.  The diagonal contains the
  results of \code{\link{scanone}}. The second component of the
  output is a data.frame indicating the locations at which the two-QTL
  LOD scores were calculated.  The first column is the chromosome
  identifier, the second column is the position in cM, the third column
  is a 1/0 indicator for ease in later pulling out only the equally
  spaced positions, and the fourth column indicates whether the position
  is on the X chromosome or not.  The final component is a version of
  the results of \code{\link{scanone}} including sex and/or cross
  direction as additive covariates, which is needed for a proper
  calculation of conditional LOD scores.

  If \code{n.perm} is specified, the function returns a list with six
  different LOD scores from each of the permutation replicates.
  First, the maximum LOD score for the full model (two QTLs plus an
  interaction).  Second, for each pair of
  chromosomes, we take the difference between the full LOD and the
  maximum single-QTL LOD for those two chromosomes, and then maximize
  this across chromosome pairs.  Third, for each pair of chromosomes we
  take the difference between the maximum full LOD and the maximum
  additive LOD, and then maximize this across chromosome pairs.  Fourth,
  the maximum LOD score for the additive QTL model.  Fifth, for each
  pair of chromosomes, we take the difference between the additive LOD
  and the maximum single-QTL LOD for those two chromosomes, and then
  maximize this across chromosome pairs.  Finally, the maximum
  single-QTL LOD score (that is, from a single-QTL scan).  The latter is
  not used in \code{\link{summary.scantwo}}, but does get
  calculated at each permutation, so we include it for the sake of
  completeness.

  If \code{n.perm} is specified and \code{perm.Xsp=TRUE}, the result is
  a list with the permutation results for the regions A:A, A:X, and X:X,
  each of which is a list with the six different LOD scores. Independent
  permutations are performed in each region, \code{n.perm} is the number
  of permutations for the A:A region; additional permutations are are
  used for the A:X and X:X parts, as estimates of quantiles farther out
  into the tails are needed.
}

\references{
  Churchill, G. A. and Doerge, R. W. (1994) Empirical threshold values for
  quantitative trait mapping.  \emph{Genetics} \bold{138}, 963--971.

  Dempster, A. P., Laird, N. M. and Rubin, D. B. (1977) Maximum
  likelihood from incomplete data via the EM algorithm.  \emph{J. Roy.
    Statist. Soc.} B, \bold{39}, 1--38.

  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Lander, E. S. and Botstein, D. (1989) Mapping Mendelian factors underlying
  quantitative traits using RFLP linkage maps.  \emph{Genetics}
  \bold{121}, 185--199.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.

  Soller, M., Brody, T. and Genizi, A. (1976) On the power of experimental
  designs for the detection of linkage between marker loci and
  quantitative loci in crosses between inbred lines.
  \emph{Theor. Appl. Genet.} \bold{47}, 35--39.
}

\author{Karl W Broman, \email{broman@wisc.edu}; Hao Wu}

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
fake.f2 <- calc.genoprob(fake.f2, step=5)
out.2dim <- scantwo(fake.f2, method="hk")
plot(out.2dim)

# permutations
\dontshow{permo.2dim <- scantwo(fake.f2, method="hk", n.perm=2)}
\dontrun{permo.2dim <- scantwo(fake.f2, method="hk", n.perm=1000)}
summary(permo.2dim, alpha=0.05)

# summary with p-values
summary(out.2dim, perms=permo.2dim, pvalues=TRUE,
        alphas=c(0.05, 0.10, 0.10, 0.05, 0.10))

# covariates
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=16:17)}
fake.bc <- calc.genoprob(fake.bc, step=10)

ac <- pull.pheno(fake.bc, c("sex","age"))
ic <- pull.pheno(fake.bc, "sex")

out <- scantwo(fake.bc, method="hk", pheno.col=1,
               addcovar=ac, intcovar=ic)
plot(out)
}

\seealso{ \code{\link{plot.scantwo}}, \code{\link{summary.scantwo}},
  \code{\link{scanone}}, \code{\link{max.scantwo}},
  \code{\link{summary.scantwoperm}},
  \code{\link{c.scantwoperm}} }

\keyword{models}

\name{subset.scantwo}
\alias{subset.scantwo}

\title{Subsetting the results of a 2-d genome scan}

\description{
  Pull out a specified set of chromosomes and/or LOD columns from
  \code{\link{scantwo}} output.
}

\usage{
\method{subset}{scantwo}(x, chr, lodcolumn, \dots)
}

\arguments{
 \item{x}{An object of class \code{scantwo}, output from
   \code{\link{scantwo}}.}
 \item{chr}{Optional vector specifying which chromosomes to keep.
 This should be a vector of character strings referring to
 chromosomes by name; numeric values are converted to strings.  Refer to
 chromosomes with a preceding \code{-} to have all chromosomes but those
 considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{lodcolumn}{A vector specifying which LOD columns to keep (or, if
   negative), omit.  These should be between 1 and the number of LOD
   columns in the input \code{x}.}
 \item{\dots}{Ignored at this point.}
}

\value{The input \code{scantwo} object, but with only the specified
  subset of the data.}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=16:19)}
fake.bc <- calc.genoprob(fake.bc)
out <- scantwo(fake.bc, method="hk", pheno.col=1:2)

summary(subset(out, chr=18:19))
}

\seealso{ \code{\link{summary.scantwo}},
  \code{\link{scantwo}} }

\keyword{manip}

\name{convert2riself}
\alias{convert2riself}

\title{Convert a cross to RIL by selfing}

\description{
  Convert a cross to type \code{"riself"} (RIL by selfing).
}

\usage{
convert2riself(cross)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
}

\value{
  The input cross object, with genotype codes possibly changed and cross
  type changed to \code{"riself"}.
}

\details{
  If there are more genotypes with code 3 (BB) than code 2 (AB), we omit
  the genotypes with code==2 and call those with code==3 the BB
  genotypes.

  If, instead, there are more genotypes with code 2 than code 3, we omit
  the genotypes with code==3 and call those with code==2 the BB genotypes.

  Any chromosomes with class \code{"X"} (X chromosome) are changed to
  class \code{"A"} (autosomal).
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
hyper.as.riself <- convert2riself(hyper)
}

\seealso{ \code{\link{convert2risib}} }

\keyword{manip}

\name{comparecrosses}
\alias{comparecrosses}
\title{Compare two cross objects}

\description{
  Verify that two objects of class \code{cross} have identical classes,
  chromosomes, markers, genotypes, genetic maps, and phenotypes.
}

\usage{comparecrosses(cross1, cross2, tol=1e-5)}

\arguments{
 \item{cross1}{An object of class \code{cross} (must be an
   intercross). See \code{\link{read.cross}} for details.}
 \item{cross2}{An object of class \code{cross} (must be an
   intercross). See \code{\link{read.cross}} for details.}
 \item{tol}{Tolerance value for comparing genetic map positions and
   numeric phenotypes.}
}

\value{None.}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
comparecrosses(listeria, listeria)
}

\seealso{ \code{\link{summary.cross}} }

\keyword{utilities}

\name{mapthis}
\alias{mapthis}
\docType{data}

\title{Simulated data for illustrating genetic map construction}

\description{
  Simulated data for an F2 intercross, obtained using
  \code{\link{sim.cross}}, useful for illustrating the process of
  constructing a genetic map.
}

\usage{data(mapthis)}

\format{
  An object of class \code{cross}.  See \code{\link{read.cross}}
  for details.
}

\details{ These are simulated data, consisting of 300 F2 individuals
typed at 100 markers on five chromosomes. There are no real phenotypes,
just a set of individual identifiers.  The data were simulated for the
purpose of illustrating the process of constructing a genetic map.  The
markers are all assigned to a single chromosome and in a random order,
and there are a number of problematic markers and individuals.

See \url{https://rqtl.org/tutorials/geneticmaps.pdf} for a tutorial
on how to construct a genetic map with these data.}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(mapthis)
summary(mapthis)
plot(mapthis)
}

\references{
Broman, K. W. (2010) Genetic map construction with R/qtl.  Technical
report #214, Department of Biostatistics and Medical Informatics,
University of Wisconsin--Madison
}


\seealso{ \code{\link{fake.f2}}, \code{\link{est.rf}},
  \code{\link{est.map}}, \code{\link{formLinkageGroups}},
  \code{\link{orderMarkers}} }

\keyword{datasets}

\name{addloctocross}
\alias{addloctocross}

\title{Add phenotype location into a cross object}

\description{
    Add phenotype location(s) into a cross object
  (with eQTL/pQTL studies)
}

\usage{
    addloctocross(cross, locations=NULL, locfile="locations.txt", verbose=FALSE)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{locations}{R variable holding location information}
  \item{locfile}{ load from a file, see the details section for the layout of the file.}
  \item{verbose}{If TRUE, give verbose output}
}

\details{
inputfile layout:
Num	Name	Chr	cM
1	X3.Hydroxypropyl	4	50.0
Num is the number of the phenotype in the cross object
Name is the name of the phenotype (will be checked against the name already in the cross object at position num
Chr Chromosome
cM	position from start of chromosome in cM
}

\value{
  The input cross object, with the locations added as an additional component \code{locations}
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
    \itemize{
    \item \code{\link{mqmplot.cistrans}} - Cis/trans plot
% \input{"inst/docs/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
    }
}

\examples{
% See the help file for mqmplot.cistrans
\dontrun{
    data(multitrait)
    data(locations)
    multiloc <- addloctocross(multitrait,locations)
    results <- scanall(multiloc)
    mqmplot.cistrans(results, multiloc, 5, FALSE, TRUE)
  }
}

\keyword{ utilities }

\name{simFounderSnps}
\alias{simFounderSnps}

\title{Simulate founder SNPs for a multiple-strain RIL}

\description{
  Simulate genotype data for the founding strains for a panel of
  multiple-strain RIL.
}

\usage{
simFounderSnps(map, n.str=c("4","8"), pat.freq)
}

\arguments{
 \item{map}{A list whose components are vectors containing the marker
   locations on each of the chromosomes.}
 \item{n.str}{Number of founding strains (4 or 8).}
 \item{pat.freq}{Frequency of SNP genotype patterns in the founder (a
   vector of length \code{n.str}/2 + 1): (monoallelic, SNP unique to one
   founder, SNP present in 2 founders, [and, for the case of 8 founders,
   SNP in 3/8 founders, SNP in 4/8 founders].)}
}

\details{
  The SNPs are simulated to be in linkage equilibrium.
}

\value{
  A vector of the same length as there are chromosomes in \code{map},
  with each component being a matrix of 0's and 1's, of dim \code{n.str}
  x \code{n.mar}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\seealso{ \code{\link{sim.map}}, \code{\link{sim.cross}} }

\examples{
data(map10)
x <- simFounderSnps(map10, "8", c(0, 0.5, 0.2, 0.2, 0.1))
}

\keyword{datagen}

\name{fake.f2}
\alias{fake.f2}
\docType{data}

\title{Simulated data for an F2 intercross}

\description{
  Simulated data for an F2 intercross, obtained using
  \code{\link{sim.cross}}.
}

\usage{data(fake.f2)}

\format{
  An object of class \code{cross}.  See \code{\link{read.cross}}
  for details.
}

\details{
There are 200 F2 individuals typed at 94 markers, including 3 on the X
chromosome.  There is one quantitative phenotype, along with an
indication of sex (0=female, 1=male) and the direction of the cross (pgm
= paternal grandmother, 0=A, meaning the cross was (AxB)x(AxB), and
1=B, meaning the cross was (AxB)x(BxA)).

Note that the X chromosome genotypes are coded in a special way (see
\code{\link{read.cross}}).  For the individuals with pgm=0, sex=0,
1=AA and 2=AB; for individuals with pgm=0, sex=1, 1=A and 2=B
(hemizygous); for individuals with pgm=1, sex=0, 1=BB and 2=AB; for
individuals with pgm=1, sex=1, 1=A and 2=B.  \bold{This requires special
care!}

The data were simulated using an additive model with three QTLs on
chromosome 1 (at 30, 50 and 70 cM), one QTL on chromosome 13 (at 30
cM), and one QTL on the X chromosome (at 10 cM).
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
summary(fake.f2)
plot(fake.f2)
}

\seealso{ \code{\link{sim.cross}}, \code{\link{fake.bc}},
  \code{\link{fake.4way}}, \code{\link{listeria}},
  \code{\link{hyper}},
  \code{\link{bristle3}}, \code{\link{bristleX}} }

\keyword{datasets}

\name{arithscan}
\alias{+.scanone}
\alias{-.scanone}
\alias{+.scantwo}
\alias{-.scantwo}

\title{Arithmetic operators for scanone and scantwo results}

\description{
  Add or subtract LOD scores in results
  from \code{\link{scanone}} or \code{\link{scantwo}}.
}

\usage{
\special{scan1+scan2}
\special{scan1-scan2}
}

\arguments{
  \item{scan1, scan2}{Genome scan results on the same set of chromosomes
    and markers, as output by \code{\link{scanone}} or \code{\link{scantwo}}.}
}

\details{
  This is used to calculate the sum or difference of LOD scores of two
  genome scan results.  It is particularly useful for calculating the LOD
  scores for QTL-by-covariate interactions (see the example, below).
  Note that the degrees of freedom are also added or subtracted.
}

\value{
  The same type of data structure as the input objects, with LOD scores
  added or subtracted.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=1:2)}
fake.bc <- calc.genoprob(fake.bc, step=2.5)

# covariates
ac <- pull.pheno(fake.bc, c("sex","age"))
ic <- pull.pheno(fake.bc, "sex")

# scan with additive but not the interactive covariate
out.acovar <- scanone(fake.bc, addcovar=ac)

# scan with interactive covariate
out.icovar <- scanone(fake.bc, addcovar=ac, intcovar=ic)

# plot the difference of with and without the interactive covariate
#     This is a LOD score for a test of QTL x covariate interaction
plot(out.icovar-out.acovar)
}

\keyword{arith}

\name{scanone}
\alias{scanone}

\title{Genome scan with a single QTL model}

\description{
  Genome scan with a single QTL model, with possible allowance for
  covariates, using any of several possible models for the phenotype and
  any of several possible numerical methods.
}

\usage{
scanone(cross, chr, pheno.col=1, model=c("normal","binary","2part","np"),
        method=c("em","imp","hk","ehk","mr","mr-imp","mr-argmax"),
        addcovar=NULL, intcovar=NULL, weights=NULL,
        use=c("all.obs", "complete.obs"), upper=FALSE,
        ties.random=FALSE, start=NULL, maxit=4000,
        tol=1e-4, n.perm, perm.Xsp=FALSE, perm.strata=NULL, verbose,
        batchsize=250, n.cluster=1, ind.noqtl)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes for which LOD
   scores should be calculated.  This should be a vector of character
   strings referring to chromosomes by name; numeric values are
   converted to strings.  Refer to chromosomes with a preceding \code{-}
   to have all chromosomes but those considered.  A logical (TRUE/FALSE)
   vector may also be used.}
 \item{pheno.col}{Column number in the phenotype matrix which should be
   used as the phenotype.  This can be a vector of integers; for methods
   \code{"hk"} and \code{"imp"} this can be considerably faster than doing
   them one at a time.  One may also give a character strings matching
   the phenotype names.  Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{model}{The phenotype model: the usual normal model, a model for
   binary traits, a two-part model or non-parametric analysis}
 \item{method}{Indicates whether to use the EM algorithm,
   imputation, Haley-Knott regression, the extended Haley-Knott method,
   or marker regression.  Not all methods are available for all models.
   Marker regression is performed either by dropping individuals with
   missing genotypes (\code{"mr"}), or by first filling in missing data
   using a single imputation (\code{"mr-imp"}) or by the Viterbi
   algorithm (\code{"mr-argmax"}).}
 \item{addcovar}{Additive covariates;
   allowed only for the normal and binary models.}
 \item{intcovar}{Interactive covariates (interact with QTL genotype);
   allowed only for the normal and binary models.}
 \item{weights}{Optional weights of individuals.  Should be either NULL
   or a vector of length n.ind containing positive weights.  Used only
   in the case \code{model="normal"}.}
 \item{use}{In the case that multiple phenotypes are selected to be
   scanned, this argument indicates whether to use all individuals,
   including those missing some phenotypes, or just those individuals
   that have data on all selected phenotypes.}
 \item{upper}{Used only for the two-part model; if true, the
   "undefined" phenotype is the maximum observed phenotype; otherwise,
   it is the smallest observed phenotype.}
 \item{ties.random}{Used only for the non-parametric "model"; if TRUE,
   ties in the phenotypes are ranked at random.  If FALSE, average ranks
   are used and a corrected LOD score is calculated.}
 \item{start}{Used only for the EM algorithm with the normal model and
   no covariates.  If \code{NULL}, use the usual starting values; if
   length 1, use random initial weights for EM; otherwise, this should
   be a vector of length n+1 (where n is the number of possible
   genotypes for the cross), giving the initial values for EM.}
 \item{maxit}{Maximum number of iterations for methods \code{"em"} and
   \code{"ehk"}.}
 \item{tol}{Tolerance value for determining convergence for methods
   \code{"em"} and \code{"ehk"}.}
 \item{n.perm}{If specified, a permutation test is performed rather than
   an analysis of the observed data.  This argument defines the number
   of permutation replicates.}
 \item{perm.Xsp}{If \code{n.perm} > 0, so that a permutation test will
   be performed, this indicates whether separate permutations should be
   performed for the autosomes and the X chromosome, in order to get an
   X-chromosome-specific LOD threshold.  In this case, additional
   permutations are performed for the X chromosome.}
 \item{perm.strata}{If \code{n.perm} > 0, this may be used to perform a
   stratified permutation test.  This should be a vector with the same
   number of individuals as in the cross data.  Unique values indicate
   the individual strata, and permutations will be performed within the
   strata.}
 \item{verbose}{In the case \code{n.perm} is specified, display
   information about the progress of the permutation tests.}
 \item{batchsize}{The number of phenotypes (or permutations) to be run
   as a batch; used only for methods \code{"hk"} and \code{"imp"}.}
 \item{n.cluster}{If the package \code{snow} is available and
   \code{n.perm} > 0, permutations are run in parallel using this number
   of nodes.}
 \item{ind.noqtl}{Indicates individuals who should not be allowed a QTL
   effect (used rarely, if at all); this is a logical vector of same
   length as there are individuals in the cross.}
}

\section{Models}{
  \bold{The normal model} is the standard model for QTL mapping (see
  Lander and Botstein 1989).  The
  residual phenotypic variation is assumed to follow a normal
  distribution, and analysis is analogous to analysis of variance.

  \bold{The binary model} is for the case of a binary phenotype, which
  must have values 0 and 1.  The proportions of 1's in the different
  genotype groups are compared.  Currently only methods \code{em}, \code{hk}, and
  \code{mr} are available for this model.  See Xu and Atchley (1996) and
  Broman (2003).

  \bold{The two-part model} is appropriate for the case of a spike in
  the phenotype distribution (for example, metastatic density when many
  individuals show no metastasis, or survival time following an
  infection when individuals may recover from the infection and fail to
  die).  The two-part model was described by
  Boyartchuk et al. (2001) and Broman (2003).  Individuals with QTL
  genotype \eqn{g} have probability \eqn{p_g}{p[g]} of having an
  undefined phenotype (the spike), while if their phenotype is defined,
  it comes from a normal distribution with mean \eqn{\mu_g}{mu[g]} and
  common standard deviation \eqn{\sigma}{s}. Three LOD scores are
  calculated: LOD(\eqn{p,\mu}{p,mu}) is for the test of the hypothesis
  that \eqn{p_g = p}{p[g] = p} and \eqn{\mu_g = \mu}{mu[g] = mu}.
  LOD(\eqn{p}) is for the test that \eqn{p_g = p}{p[g] = p} while the
  \eqn{\mu_g}{mu[g]} may vary. LOD(\eqn{\mu}{mu}) is for the test that
  \eqn{\mu_g = \mu}{mu[g] = mu} while the \eqn{p_g}{p[g]} may vary.

  \bold{With the non-parametric "model"}, an extension of the
  Kruskal-Wallis test is used; this is similar to the method described
  by Kruglyak and Lander (1995).  In the case of incomplete genotype
  information (such as at locations between genetic markers), the
  Kruskal-Wallis statistic is modified so that the rank for each
  individual is weighted by the genotype probabilities, analogous to
  Haley-Knott regression.  For this method, if the argument
  \code{ties.random} is TRUE, ties in the phenotypes are assigned random
  ranks; if it is FALSE, average ranks are used and a corrected LOD
  score is calculate.  Currently the \code{method} argument is ignored
  for this model.
}

\section{Methods}{
  \bold{\code{em}}: maximum likelihood is performed via the
  EM algorithm (Dempster et al. 1977), first used in this context by
  Lander and Botstein (1989).

  \bold{\code{imp}}: multiple imputation is used, as described by Sen
  and Churchill (2001).

  \bold{\code{hk}}: Haley-Knott regression is used (regression of the
  phenotypes on the multipoint QTL genotype probabilities), as described
  by Haley and Knott (1992).

  \bold{\code{ehk}}: the extended Haley-Knott method is used (like H-K,
  but taking account of the variances), as described in Feenstra et
  al. (2006).

  \bold{\code{mr}}: Marker regression is used.  Analysis is performed
  only at the genetic markers, and individuals with missing genotypes
  are discarded.  See Soller et al. (1976).
}

\section{Covariates}{
Covariates are allowed only for the normal and binary models.  The
normal model is \eqn{y = \beta_q + A \gamma + Z \delta_q + \epsilon}{y = b[q] +
  A g + Z d[q] + e} where \emph{q} is the unknown QTL genotype, \emph{A}
is a matrix of additive covariates, and \emph{Z} is a matrix of
covariates that interact with the QTL genotype.  The columns of \emph{Z}
are forced to be contained in the matrix \emph{A}.  The binary model is
the logistic regression analog.

The LOD score is calculated comparing the likelihood of the above
model to that of the null model \eqn{y = \mu + A \gamma + \epsilon}{y =
  m + A g + e}.

Covariates must be numeric matrices.  Individuals with any missing
covariates are discarded.
}

\section{X chromosome}{
The X chromosome must be treated specially in QTL mapping.  See Broman
et al. (2006).

If both males and females are included, male hemizygotes are allowed
to be different from female homozygotes.  Thus, in a backcross, we will
fit separate means for the genotype classes AA, AB, AY, and BY.  In such
cases, sex differences in the phenotype could cause spurious linkage to
the X chromosome, and so the null hypothesis must be changed to allow
for a sex difference in the phenotype.

Numerous special cases must be considered, as detailed in the following
table.

\tabular{lllccc}{
\bold{BC}
 \tab \tab \bold{Sexes} \tab \bold{Null} \tab \bold{Alternative} \tab \bold{df} \cr
 \tab \tab both sexes   \tab sex         \tab AA/AB/AY/BY        \tab 2         \cr
 \tab \tab all female   \tab grand mean  \tab AA/AB              \tab 1         \cr
 \tab \tab all male     \tab grand mean  \tab AY/BY              \tab 1         \cr
 \tab \tab              \tab             \tab                    \tab           \cr

\bold{F2}
 \tab \bold{Direction} \tab \bold{Sexes}\tab \bold{Null}\tab \bold{Alternative} \tab \bold{df} \cr
 \tab Both             \tab both sexes  \tab femaleF/femaleR/male \tab AA/ABf/ABr/BB/AY/BY \tab 3         \cr
 \tab                  \tab all female  \tab pgm        \tab AA/ABf/ABr/BB      \tab 2         \cr
 \tab                  \tab all male    \tab grand mean \tab AY/BY              \tab 1         \cr
 \tab Forward          \tab both sexes  \tab sex        \tab AA/AB/AY/BY        \tab 2         \cr
 \tab                  \tab all female  \tab grand mean \tab AA/AB              \tab 1         \cr
 \tab                  \tab all male    \tab grand mean \tab AY/BY              \tab 1         \cr
 \tab Backward         \tab both sexes  \tab sex        \tab AB/BB/AY/BY        \tab 2         \cr
 \tab                  \tab all female  \tab grand mean \tab AB/BB              \tab 1         \cr
 \tab                  \tab all male    \tab grand mean \tab AY/BY              \tab 1         \cr
}

In the case that the number of degrees of freedom for the linkage test
for the X chromosome is different from that for autosomes, a separate
X-chromosome LOD threshold is recommended.  Autosome- and
X-chromosome-specific LOD thresholds may be estimated by permutation
tests with \code{scanone} by setting \code{n.perm}>0 and using
\code{perm.Xsp=TRUE}.

}

\details{
  Use of the EM algorithm, Haley-Knott regression, and the extended
  Haley-Knott method require that multipoint genotype probabilities are
  first calculated using \code{\link{calc.genoprob}}.  The
  imputation method uses the results of \code{\link{sim.geno}}.

  Individuals with missing phenotypes are dropped.

  In the case that \code{n.perm}>0, so that a permutation
  test is performed, the R function \code{scanone} is called repeatedly.
  If \code{perm.Xsp=TRUE}, separate permutations are performed for the
  autosomes and the X chromosome, so that an X-chromosome-specific
  threshold may be calculated.  In this case, \code{n.perm} specifies
  the number of permutations used for the autosomes; for the X
  chromosome, \code{n.perm} \eqn{\times \, L_A/L_X}{* L_A/L_X} permutations
  will be run, where \eqn{L_A} and \eqn{L_X} are the total genetic
  lengths of the autosomes and X chromosome, respectively.  More
  permutations are needed for the X chromosome in order to obtain
  thresholds of similar accuracy.

  For further details on the models, the methods and the use of
  covariates, see below.
}

\value{
  If \code{n.perm} is missing, the function returns a data.frame whose
  first two columns contain the chromosome IDs and cM positions.
  Subsequent columns contain the LOD scores for each phenotype.
  In the case of the two-part model, there are three LOD score columns
  for each phenotype: LOD(\eqn{p,\mu}{p,mu}), LOD(\eqn{p}) and
  LOD(\eqn{\mu}{mu}).   The result is given class \code{"scanone"} and
  has attributes  \code{"model"}, \code{"method"}, and
  \code{"type"} (the latter is the type of cross analyzed).

  If \code{n.perm} is specified, the function returns the results of a
  permutation test and the output has class \code{"scanoneperm"}.  If
  \code{perm.Xsp=FALSE}, the function returns a matrix with
  \code{n.perm} rows, each row containing the genome-wide
  maximum LOD score for each of the phenotypes.  In the case of the
  two-part model, there are three columns for each phenotype,
  corresponding to the three different LOD scores. If
  \code{perm.Xsp=TRUE}, the result contains separate permutation results
  for the autosomes and the X chromosome respectively, and an attribute
  indicates the lengths of the chromosomes and an indicator of which
  chromosome is X.
}

\references{
  Boyartchuk,  V. L., Broman, K. W., Mosher, R. E., D'Orazio
  S. E. F., Starnbach, M. N. and Dietrich, W. F. (2001) Multigenic
  control of \emph{Listeria monocytogenes} susceptibility in
  mice. \emph{Nature Genetics} \bold{27}, 259--260.

  Broman,  K. W. (2003) Mapping quantitative trait loci in the case of a
  spike in the phenotype distribution. \emph{Genetics} \bold{163},
  1169--1175.

  Broman, K. W., Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}, Owens, S. E., Manichaikul, A.,
  Southard-Smith, E. M. and Churchill G. A. (2006)  The X chromosome in
  quantitative trait locus mapping.  \emph{Genetics}, \bold{174}, 2151--2158.

  Churchill, G. A. and Doerge, R. W. (1994) Empirical threshold values for
  quantitative trait mapping.  \emph{Genetics} \bold{138}, 963--971.

  Dempster, A. P., Laird, N. M. and Rubin, D. B. (1977) Maximum
  likelihood from incomplete data via the EM algorithm.  \emph{J. Roy.
    Statist. Soc.} B, \bold{39}, 1--38.

  Feenstra, B., Skovgaard, I. M. and Broman, K. W. (2006) Mapping
  quantitative trait loci by an extension of the Haley-Knott regression
  method using estimating equations. \emph{Genetics}, \bold{173},
  2111--2119.

  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Kruglyak, L. and Lander, E. S. (1995) A nonparametric approach for
  mapping quantitative trait loci.  \emph{Genetics} \bold{139},
  1421--1428.

  Lander, E. S. and Botstein, D. (1989) Mapping Mendelian factors underlying
  quantitative traits using RFLP linkage maps.  \emph{Genetics}
  \bold{121}, 185--199.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.

  Soller, M., Brody, T. and Genizi, A. (1976) On the power of experimental
  designs for the detection of linkage between marker loci and
  quantitative loci in crosses between inbred lines.
  \emph{Theor. Appl. Genet.} \bold{47}, 35--39.

  Xu, S., and Atchley, W.R. (1996) Mapping quantitative trait loci for
  complex binary diseases using line crosses. \emph{Genetics}
  \bold{143}, 1417--1424.
}

\author{Karl W Broman, \email{broman@wisc.edu}; Hao Wu}

\examples{
###################
# Normal Model
###################
data(hyper)
\dontshow{hyper <- subset(hyper,chr=1:4,ind=1:100)}
# Genotype probabilities for EM and H-K
\dontrun{hyper <- calc.genoprob(hyper, step=2.5)
}\dontshow{hyper <- calc.genoprob(hyper)}
out.em <- scanone(hyper, method="em")
out.hk <- scanone(hyper, method="hk")

# Summarize results: peaks above 3
summary(out.em, thr=3)
summary(out.hk, thr=3)

# An alternate method of summarizing:
#     patch them together and then summarize
out <- c(out.em, out.hk)
summary(out, thr=3, format="allpeaks")

# Plot the results
plot(out.hk, out.em)
plot(out.hk, out.em, chr=c(1,4), lty=1, col=c("blue","black"))

# Imputation; first need to run sim.geno
# Do just chromosomes 1 and 4, to save time
\dontrun{hyper.c1n4 <- sim.geno(subset(hyper, chr=c(1,4)),
                       step=2.5, n.draws=8)
}\dontshow{hyper.c1n4 <- sim.geno(subset(hyper, chr=c(1,4)),
                       step=0, n.draws=4)}
out.imp <- scanone(hyper.c1n4, method="imp")
summary(out.imp, thr=3)

# Plot all three results
plot(out.imp, out.hk, out.em, chr=c(1,4), lty=1,
     col=c("red","blue","black"))

# extended Haley-Knott
out.ehk <- scanone(hyper, method="ehk")
plot(out.hk, out.em, out.ehk, chr=c(1,4))

# Permutation tests
\dontrun{permo <- scanone(hyper, method="hk", n.perm=1000)
}\dontshow{permo <- scanone(hyper, method="hk", n.perm=5)
}

# Threshold from the permutation test
summary(permo, alpha=c(0.05, 0.10))

# Results above the 0.05 threshold
summary(out.hk, perms=permo, alpha=0.05)


####################
# scan with square-root of phenotype
#   (Note that pheno.col can be a vector of phenotype values)
####################
out.sqrt <- scanone(hyper, pheno.col=sqrt(pull.pheno(hyper, 1)))
plot(out.em - out.sqrt, ylim=c(-0.1,0.1),
     ylab="Difference in LOD")
abline(h=0, lty=2, col="gray")


####################
# Stratified permutations
####################
extremes <- (nmissing(hyper)/totmar(hyper) < 0.5)

\dontrun{operm.strat <- scanone(hyper, method="hk", n.perm=1000,
                       perm.strata=extremes)
}\dontshow{operm.strat <- scanone(hyper, method="hk", n.perm=5,
                       perm.strata=extremes)
}

summary(operm.strat)



####################
# X-specific permutations
####################
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(18:19,"X"))}
\dontrun{fake.f2 <- calc.genoprob(fake.f2, step=2.5)
}\dontshow{fake.f2 <- calc.genoprob(fake.f2, step=0)}

# genome scan
out <- scanone(fake.f2, method="hk")

# X-chr-specific permutations
\dontrun{operm <- scanone(fake.f2, method="hk", n.perm=1000, perm.Xsp=TRUE)
}\dontshow{operm <- scanone(fake.f2, method="hk", n.perm=5, perm.Xsp=TRUE)
}

# thresholds
summary(operm)

# scanone summary with p-values
summary(out, perms=operm, alpha=0.05, pvalues=TRUE)



###################
# Non-parametric
###################
out.np <- scanone(hyper, model="np")
summary(out.np, thr=3)

# Plot with previous results
plot(out.np, chr=c(1,4), lty=1, col="green")
plot(out.imp, out.hk, out.em, chr=c(1,4), lty=1,
     col=c("red","blue","black"), add=TRUE)

###################
# Two-part Model
###################
data(listeria)
\dontshow{listeria <- subset(listeria,chr=c(1,5,13),ind=!is.na(listeria$pheno[,1]))}
\dontrun{listeria <- calc.genoprob(listeria,step=2.5)
}\dontshow{listeria <- calc.genoprob(listeria,step=0)}
out.2p <- scanone(listeria, model="2part", upper=TRUE)
summary(out.2p, thr=c(5,3,3), format="allpeaks")

# Plot all three LOD scores together
plot(out.2p, out.2p, out.2p, lodcolumn=c(2,3,1), lty=1, chr=c(1,5,13),
     col=c("red","blue","black"))

# Permutation test
\dontrun{permo <- scanone(listeria, model="2part", upper=TRUE,
                 n.perm=1000)
}\dontshow{permo <- scanone(listeria, model="2part", upper=TRUE,
                 n.perm=3)
}

# Thresholds
summary(permo)

###################
# Binary model
###################
binphe <- as.numeric(pull.pheno(listeria,1)==264)
out.bin <- scanone(listeria, pheno.col=binphe, model="binary")
summary(out.bin, thr=3)

# Plot LOD for binary model with LOD(p) from 2-part model
plot(out.bin, out.2p, lodcolumn=c(1,2), lty=1, col=c("black", "red"),
     chr=c(1,5,13))

# Permutation test
\dontrun{permo <- scanone(listeria, pheno.col=binphe, model="binary",
                 n.perm=1000)
}\dontshow{permo <- scanone(listeria, pheno.col=binphe, model="binary",
                 n.perm=3)
}

# Thresholds
summary(permo)

###################
# Covariates
###################
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=c(2,5,10))}
\dontrun{fake.bc <- calc.genoprob(fake.bc, step=2.5)
}\dontshow{fake.bc <- calc.genoprob(fake.bc, step=0)}

# genome scans without covariates
out.nocovar <- scanone(fake.bc)

# genome scans with covariates
ac <- pull.pheno(fake.bc, c("sex","age"))
ic <- pull.pheno(fake.bc, "sex")

out.covar <- scanone(fake.bc, pheno.col=1,
                     addcovar=ac, intcovar=ic)
summary(out.nocovar, thr=3)
summary(out.covar, thr=3)
plot(out.covar, out.nocovar, chr=c(2,5,10))
}

\seealso{ \code{\link{plot.scanone}},
  \code{\link{summary.scanone}}, \code{\link{scantwo}},
  \code{\link{calc.genoprob}}, \code{\link{sim.geno}},
  \code{\link{max.scanone}},
  \code{\link{summary.scanoneperm}},
  \code{\link[=-.scanone]{-.scanone}}, \code{\link[=+.scanone]{+.scanone}} }

\keyword{models}

\name{add.threshold}
\alias{add.threshold}

\title{Add significance threshold to plot}

\description{
  Add a significance threshold to a plot created by
  \code{\link{plot.scanone}}), using the permutation results.
}

\usage{
add.threshold(out, chr, perms, alpha=0.05, lodcolumn=1, gap=25, \dots)
}

\arguments{
  \item{out}{An object of class \code{"scanone"}, as output by
    \code{\link{scanone}}.  This must be identical to what was used
  in the call to \code{\link{plot.scanone}}.}
  \item{chr}{Optional vector specifying which chromosomes to plot. If a
    selected subset of chromosomes were plotted, they must be specified
    here.}
  \item{perms}{Permutation results from \code{\link{scanone}}, used
    to calculate the significance threshold.}
  \item{alpha}{Significance level of the threshold.}
  \item{lodcolumn}{An integer indicating which
    of column in the permutation results should be used.}
  \item{gap}{Gap separating chromosomes (in cM).  This must be identical
  to what was used in the call to \code{\link{plot.scanone}}.}
  \item{\dots}{Passed to the function \code{\link[graphics]{abline}} when it
    is called.}
}
\value{None.}

\details{
  This function allows you to add a horizontal line at the significance
  threshold to genome scan results plotted by
  \code{\link{plot.scanone}}.

  The arguments \code{out}, \code{chr}, and \code{gap} must match what
  was used in the call to \code{\link{plot.scanone}}.

  The argument \code{perms} must be specified.  If X-chromosome-specific
  permutations were performed (via the argument \code{perm.Xsp} in the
  call to \code{\link{scanone}}), separate thresholds will be
  plotted for the autosomes and the X chromosome.  These are calculated
  via the \code{\link{summary.scanoneperm}} function.
}

\examples{
data(hyper)
hyper <- calc.genoprob(hyper)
out <- scanone(hyper, method="hk")
operm <- scanone(hyper, method="hk", n.perm=100, perm.Xsp=TRUE)

plot(out, chr=c(1,4,6,15,"X"))
add.threshold(out, chr=c(1,4,6,15,"X"), perms=operm, alpha=0.05)
add.threshold(out, chr=c(1,4,6,15,"X"), perms=operm, alpha=0.1,
              col="green", lty=2)
}

\seealso{ \code{\link{scanone}},
  \code{\link{plot.scanone}},
  \code{\link{summary.scanoneperm}}, \code{\link{xaxisloc.scanone}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{sim.geno}
\alias{sim.geno}

\title{Simulate genotypes given observed marker data}

\description{
  Uses the hidden Markov model technology to simulate from the joint
  distribution Pr(g | O) where g is the underlying genotype vector and O
  is the observed multipoint marker data, with possible allowance for
  genotyping errors.
}

\usage{
sim.geno(cross, n.draws=16, step=0, off.end=0, error.prob=0.0001,
         map.function=c("haldane","kosambi","c-f","morgan"),
         stepwidth=c("fixed", "variable", "max"))
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{n.draws}{Number of simulation replicates to perform.}
 \item{step}{Maximum distance (in cM) between positions at which the
   simulated genotypes will be drawn, though for \code{step=0},
   genotypes are drawn only at the marker locations.}
 \item{off.end}{Distance (in cM) past the terminal markers on each
   chromosome to which the genotype simulations will be carried.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions.}
 \item{stepwidth}{Indicates whether the intermediate points should with
   fixed or variable step sizes.  We recommend using
   \code{"fixed"}; \code{"variable"} was included for the qtlbim
   package (\url{https://cran.r-project.org/src/contrib/Archive/qtlbim/}).  The \code{"max"}
   option inserts the minimal number of intermediate points so that the
   maximum distance between points is \code{step}.}
}

\details{
  After performing the forward-backward equations, we draw from
  \eqn{Pr(g_1 = v | O)}{Pr(g[1] = v | O)} and then \eqn{Pr(g_{k+1} = v |
    O, g_k = u)}{Pr(g[k+1] = v | O, g[k] = u)}.

  In the case of the 4-way cross, with a sex-specific map, we assume a
  constant ratio of female:male recombination rates within the
  inter-marker intervals.
}

\value{
  The input \code{cross} object is returned with a component,
  \code{draws}, added to each component of \code{cross$geno}.
  This is an array of size [n.ind x n.pos x n.draws] where n.pos is
  the number of positions at which the simulations were performed and
  n.draws is the number of replicates.  Attributes \code{"error.prob"},
  \code{"step"}, and \code{"off.end"} are set to the values of the
  corresponding arguments, for later reference.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr=18:19)}
fake.f2 <- sim.geno(fake.f2, step=2, n.draws=8)
}

\seealso{ \code{\link{calc.genoprob}},
  \code{\link{argmax.geno}} }

\keyword{utilities}

\name{strip.partials}
\alias{strip.partials}

\title{Strip partially informative genotypes}

\description{
  Replace all partially informative genotypes (e.g., dominant markers in
  an intercross) with missing values.
}

\usage{strip.partials(cross, verbose=TRUE)}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{verbose}{If TRUE, print the number of genotypes removed.}
}

\value{
  The same class \code{cross} object as in the input, but with partially
  informative genotypes made missing.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
sum(nmissing(listeria))
listeria <- strip.partials(listeria)
sum(nmissing(listeria))
}

\seealso{ \code{\link{plotMissing}}, \code{\link{plotInfo}} }

\keyword{manip}

\name{find.pseudomarker}
\alias{find.pseudomarker}

\title{Find the pseudomarker closest to a specified position}

\description{
  Find the pseudomarker closest to a specified position on a
  chromosome.
}

\usage{find.pseudomarker(cross, chr, pos, where=c("draws", "prob"), addchr=TRUE)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{A vector of chromosome identifiers, or a single such.}
 \item{pos}{A vector of cM positions.}
 \item{where}{Indicates whether to look in the \code{draws} or
 \code{prob} components of the input cross.}
 \item{addchr}{If TRUE, include something like \code{"c5."} at the
 beginning of the names of non-pseudomarker locations, as in the output
 of \code{\link{scanone}}; if FALSE, don't include this sort of string,
 as in the genotype probabilities from \code{\link{calc.genoprob}}.}
}

\value{
  A vector of pseudomarker names (of the same length as the input \code{pos}),
  corresponding to the markers nearest to the specified chromosomes/positions.
}

\details{
  If the input \code{chr} has length one, it is expanded to the same
  length as the input \code{pos}.

  If multiple markers are exactly the same distance from the specified
  position, one is chosen at random from among those with the most
  genotype data.

  For a cross with sex-specific maps, the input positions are assumed to
  correspond to the female genetic map.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
listeria <- calc.genoprob(listeria, step=2.5)
find.pseudomarker(listeria, 5, 28, "prob")
find.pseudomarker(listeria, c(1, 5, 13), c(81, 28, 26), "prob")
}

\seealso{ \code{\link{find.flanking}}, \code{\link{plotPXG}},
\code{\link{effectplot}}, \code{\link{find.marker}},
\code{\link{find.markerpos}}  }

\keyword{utilities}

\name{plotInfo}
\alias{plotInfo}

\title{Plot the proportion of missing genotype information}

\description{
  Plot a measure of the proportion of missing information in the
  genotype data.
}

\usage{
plotInfo(x, chr, method=c("entropy","variance","both"), step=1,
          off.end=0, error.prob=0.001,
          map.function=c("haldane","kosambi","c-f","morgan"),
          alternate.chrid=FALSE, fourwaycross=c("all", "AB", "CD"),
          include.genofreq=FALSE, \dots)
}

\arguments{
 \item{x}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{method}{Indicates whether to plot the entropy version of the
   information, the variance version, or both.}
 \item{step}{Maximum distance (in cM) between positions at which the
   missing information is calculated, though for \code{step=0},
   it is are calculated only at the marker locations.}
 \item{off.end}{Distance (in cM) past the terminal markers on each
   chromosome to which the genotype probability calculations will be
   carried.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi or
   Carter-Falconer map function when converting genetic distances into
   recombination fractions.}
 \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
    axis labels, so that they may be more easily distinguished.}
  \item{fourwaycross}{For a phase-known four-way cross, measure missing
 genotype information overall (\code{"all"}), or just for the alleles
 from the first parent (\code{"AB"}) or from the second parent (\code{"CD"}).}
  \item{include.genofreq}{If TRUE, estimated genotype frequencies (from
 the results of
 \code{\link{calc.genoprob}} averaged across the individuals) are
 included as additional columns in the output.}
 \item{\dots}{Passed to \code{\link{plot.scanone}}.}
}

\details{
  The entropy version of the missing information: for a single
  individual at a single genomic position, we measure the missing
  information as \eqn{H = \sum_g p_g \log p_g / \log n}{H = sum p[g] log
  p[g] / log n}, where \eqn{p_g}{p[g]} is the probability of the
  genotype \eqn{g}, and \eqn{n} is the number of possible genotypes,
  defining \eqn{0 \log 0 = 0}{0 log 0 = 0}.  This takes values between 0
  and 1, assuming the value 1 when the genotypes (given the marker data)
  are equally likely and 0 when the genotypes are completely determined.
  We calculate the missing information at a particular position as the
  average of \eqn{H} across individuals.  For an intercross, we don't
  scale by \eqn{\log n} but by the entropy in the case of genotype
  probabilities (1/4, 1/2, 1/4).

  The variance version of the missing information: we calculate the
  average, across individuals, of the variance of the genotype
  distribution (conditional on the observed marker data) at a particular
  locus, and scale by the maximum such variance.

  Calculations are done in C (for the sake of speed in the presence of
  little thought about programming efficiency) and the plot is created
  by a call to \code{\link{plot.scanone}}.

  Note that \code{\link{summary.scanone}} may be used to display
  the maximum missing information on each chromosome.
}

\value{
  An object with class \code{scanone}: a data.frame with columns the
  chromosome IDs and cM positions followed by the entropy and/or
  variance version of the missing information.
}

\examples{
data(hyper)
\dontshow{hyper <- subset(hyper,chr=1:4)}
plotInfo(hyper,chr=c(1,4))

# save the results and view maximum missing info on each chr
info <- plotInfo(hyper)
summary(info)

plotInfo(hyper, bandcol="gray70")
}

\seealso{ \code{\link{plot.scanone}},
  \code{\link{plotMissing}}, \code{\link{calc.genoprob}},
  \code{\link{geno.table}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}
\keyword{univar}

\name{read.cross}
\alias{read.cross}

\title{Read data for a QTL experiment}

\description{
  Data for a QTL experiment is read from a set of files and converted
  into an object of class \code{cross}.  The comma-delimited format
  (\code{csv}) is recommended.  All formats require chromosome
  assignments for the genetic markers, and assume that markers are in
  their correct order.
}

\usage{
read.cross(format=c("csv", "csvr", "csvs", "csvsr", "mm", "qtx",
                    "qtlcart", "gary", "karl", "mapqtl", "tidy"),
           dir="", file, genfile, mapfile, phefile, chridfile,
           mnamesfile, pnamesfile, na.strings=c("-","NA"),
           genotypes=c("A","H","B","D","C"), alleles=c("A","B"),
           estimate.map=FALSE, convertXdata=TRUE, error.prob=0.0001,
           map.function=c("haldane", "kosambi", "c-f", "morgan"),
           BC.gen=0, F.gen=0, crosstype, \dots)
}

\arguments{
  \item{format}{Specifies the format of the data file or files. Details
  on the various file formats are provided below.}
  \item{dir}{Directory in which the data files will be found.  In
    Windows, use forward slashes (\code{"/"}) or double backslashes
    (\code{"\\\\"}) to specify directory trees.}
  \item{file}{The main input file for formats \code{csv}, \code{csvr}
    and \code{mm}.}
  \item{genfile}{File with genotype data (formats \code{csvs},
    \code{csvsr}, \code{karl}, \code{gary} and \code{mapqtl} only).}
  \item{mapfile}{File with marker position information (all
    except the \code{csv} formats).}
  \item{phefile}{File with phenotype data (formats \code{csvs},
    \code{csvsr}, \code{karl}, \code{gary} and \code{mapqtl} only).}
  \item{chridfile}{File with chromosome ID for each marker (\code{gary}
    format only).}
  \item{mnamesfile}{File with marker names (\code{gary} format only).}
  \item{pnamesfile}{File with phenotype names (\code{gary} format
    only).}
  \item{na.strings}{A vector of strings which are to be interpreted as
    missing values (\code{csv} and \code{gary} formats only).  For the
    \code{csv} formats, these are interpreted globally
    for the entire
    file, so missing value codes in phenotypes must not be valid
    genotypes, and vice versa. For the \code{gary} format, these are
    used only for the phenotype data.}
  \item{genotypes}{A vector of character strings specifying the genotype
    codes (\code{csv} formats only).  Generally this is a vector of
    length 5, with the elements corresponding to AA, AB, BB, not BB
    (i.e., AA or AB), and not AA (i.e., AB or BB).  \bold{Note}: Pay
    careful attention to the third and fourth of these; the order of
    these can be confusing.

    If you are trying to read 4-way cross data, your file must have
    genotypes coded as described below, and you need to set
    \code{genotypes=NULL} so that no re-coding gets done.}
  \item{alleles}{A vector of two one-letter character strings (or four,
    for the four-way cross), to be used as labels for the two alleles.}
  \item{estimate.map}{For all formats but \code{qtlcart}, \code{mapqtl},
    and \code{karl}: if TRUE and marker positions are not included in
    the input files, the genetic map is estimated using the function
    \code{\link{est.map}}.}
  \item{convertXdata}{If TRUE, any X chromosome genotype data is
    converted to the internal standard, using columns \code{sex} and
    \code{pgm} in the phenotype data if they available or by inference
    if they are not.  If FALSE, the X chromsome data is read as is.}
  \item{error.prob}{In the case that the marker map must be estimated:
   Assumed genotyping error rate used in the calculation of the
   penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{In the case that the marker map must be estimated:
   Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions. (Ignored if m > 0.)}
  \item{BC.gen}{Used only for cross type \code{"bcsft"}.}
  \item{F.gen}{Used only for cross type \code{"bcsft"}.}
  \item{crosstype}{Optional character string to force a particular cross type.}
  \item{\dots}{Additional arguments, passed to the function
    \code{\link[utils]{read.table}} in the case of
    \code{csv} and \code{csvr} formats.  In particular, one may use the
    argument
    \code{sep} to specify the field separator (the default is a comma),
    \code{dec} to specify the character used for the decimal point
    (the default is a period), and \code{comment.char} to specify a
    character to indicate comment lines.}
}

\value{
  An object of class \code{cross}, which is a list with two components:

  \item{geno}{This is a list with elements corresponding to
    chromosomes.  \code{names(geno)} contains the names of the
    chromsomes.  Each chromosome is itself a list, and is given class
    \code{A} or \code{X} according to whether it is autosomal
    or the X chromosome.

    There are two components for each chromosome: \code{data}, a matrix
    whose rows are individuals and whose columns are markers, and
    \code{map}, either a vector of marker positions (in cM) or a matrix
    of dim (\code{2 x n.mar}) where the rows correspond to marker
    positions in female and male genetic distance, respectively.

    The genotype data gets converted into numeric codes, as follows.

    The genotype data for a backcross is coded as NA = missing,
    1 = AA, 2 = AB.

    For an F2 intercross, the coding is NA = missing, 1 = AA, 2 = AB, 3
    = BB, 4 = not BB (i.e. AA or AB; D in Mapmaker/qtl), 5 = not AA (i.e. AB
    or BB; C in Mapmaker/qtl).

    For a 4-way cross, the mother and father are assumed to have
    genotypes AB and CD, respectively.  The genotype data for the
    progeny is assumed to be phase-known, with the following coding
    scheme: NA = missing, 1 = AC, 2 = BC, 3 = AD, 4 = BD, 5 = A = AC or AD,
    6 = B = BC or BD, 7 = C = AC or BC, 8 = D = AD or BD, 9 = AC or BD,
    10 = AD or BC, 11 = not AC, 12 = not BC, 13 = not AD, 14 = not BD.
  }

  \item{pheno}{data.frame of size (\code{n.ind x n.phe}) containing the
    phenotypes.  If a phenotype with the name \code{id} or \code{ID} is
  included, these identifiers will be used in \code{\link{top.errorlod}},
  \code{\link{plotErrorlod}}, and \code{\link{plotGeno}} as
  identifiers for the individual.}

  While the data format is complicated, there are a number of functions,
  such as \code{\link{subset.cross}}, to
  assist in pulling out portions of the data.
}

\details{
  The available formats are comma-delimited (\code{csv}), rotated
  comma-delimited (\code{csvr}), comma-delimited with separate files for
  genotype and phenotype data (\code{csvs}), rotated comma-delimited
  with separate files for genotype and phenotype data (\code{csvsr}),
  Mapmaker (\code{mm}), Map Manager QTX (\code{qtx}), Gary Churchill's
  format (\code{gary}), Karl Broman's format (\code{karl}) and
  MapQTL/JoinMap (\code{mapqtl}).  The required files and their
  specification for each format appears below. The comma-delimited
  formats are recommended. Note that most of these formats work only
  for backcross and intercross data.

  The \code{sampledata} directory in the package distribution contains
  sample data files in multiple formats.    Also see
  \url{https://rqtl.org/sampledata/}.

  The \code{\dots} argument enables additional arguments to be passed to
  the function \code{\link[utils]{read.table}} in the case of \code{csv}
  and \code{csvr} formats.  In particular, one may use the argument
  \code{sep} to specify the field separator (the default is a comma),
  \code{dec} to specify the character used for the decimal point (the
  default is a period), and \code{comment.char} to specify a character
  to indicate comment lines.
}


\section{X chromosome}{
  \bold{The genotypes for the X chromosome require special care!}

  The X chromosome should be given chromosome identifier \code{X} or
  \code{x}.  If it is labeled by a number or by \code{Xchr}, it will be
  interpreted as an autosome.

  The phenotype data should contain a column named \code{"sex"} which
  indicates the sex of each individual, either coded as \code{0}=female and
  \code{1}=male, or as a factor with levels \code{female}/\code{male} or
  \code{f}/\code{m}.  Case will be
  ignored both in the name and in the factor levels.  If no such
  phenotype column is included, it will be assumed that all individuals
  are of the same sex.

  In the case of an intercross, the phenotype data may also contain a
  column named \code{"pgm"} (for "paternal grandmother") indicating the
  direction of the cross.  It should be coded as 0/1 with 0 indicating
  the cross (AxB)x(AxB) or (BxA)x(AxB) and 1 indicating the cross
  (AxB)x(BxA) or (BxA)x(BxA).  If no such phenotype column is included,
  it will be assumed that all individuals come from the same direction
  of cross.

  The internal storage of X chromosome data is quite different from that
  of autosomal data.  Males are coded 1=AA and 2=BB; females with pgm==0
  are coded 1=AA and 2=AB; and females with pgm==1 are coded 1=BB and
  2=AB.  If the argument \code{convertXdata} is TRUE, conversion to this
  format is made automatically; if FALSE, no conversion is done,
  \code{\link{summary.cross}} will likely return a warning, and
  most analyses will not work properly.

  Use of \code{convertXdata=FALSE} (in which case the X chromosome
  genotypes will not be converted to our internal standard) can be
  useful for diagnosing problems in the data, but will require some
  serious mucking about in the internal data structure.
}

\section{CSV format}{
  The input file is a comma-delimited text file.  A different field
  separator may be specified via the argument \code{sep}, which will be passed
  to the function \code{\link[utils]{read.table}}).  For example, in
  Europe, it is common to use a comma in place of the decimal point in
  numbers and so a semi-colon in place of a comma as the field
  separator; such data may be read by using \code{sep=";"} and
  \code{dec=","}.

  The first line should contain the phenotype names followed by the
  marker names.  \bold{At least one phenotype must be included}; for
  example, include a numerical index for each individual.

  The second line should contain blanks in the phenotype columns,
  followed by chromosome identifiers for each marker in all other
  columns. If a chromosome has the identifier \code{X} or \code{x}, it
  is assumed to be the X chromosome; otherwise, it is assumed to be an
  autosome.

  An optional third line should contain blanks in the phenotype
  columns, followed by marker positions, in cM.

  Marker order is taken from the cM positions, if provided; otherwise,
  it is taken from the column order.

  Subsequent lines should give the data, with one line for each
  individual, and with phenotypes followed by genotypes.  If possible,
  phenotypes are made numeric; otherwise they are converted to factors.

  The genotype codes must be the same across all markers.  For example,
  you can't have one marker coded AA/AB/BB and another coded A/H/B.
  This includes genotypes for the X chromosome, for which hemizygous
  individuals should be coded as if they were homoyzogous.

  The cross is determined to be a backcross if only the first two elements
  of the \code{genotypes} string are found; otherwise, it is assumed to
  be an intercross.
}

\section{CSVr format}{
  This is just like the \code{csv} format, but rotated (or really
  transposed), so that rows are columns and columns are rows.
}


\section{CSVs format}{
  This is like the \code{csv} format, but with separate files for the
  genotype and phenotype data.

  The first column in the genotype data must specify individuals'
  identifiers, and there must be a column in the phenotype data with
  precisely the same information (and with the same name).  These IDs
  will be included in the data as a phenotype.  If the name \code{id} or
  \code{ID} is used, these identifiers will be used in
  \code{\link{top.errorlod}}, \code{\link{plotErrorlod}}, and
  \code{\link{plotGeno}} as identifiers for the individual.

  The first row in each file contains the column names.  For the
  phenotype file, these are the names of the phenotypes.  For the
  genotype file, the first cell will be the name of the identifier
  column (\code{id} or \code{ID}) and the subsequent fields will be the
  marker names.

  In the genotype data file, the second row gives the chromosome IDs.
  The cell in the second row, first column, must be blank.  A third
  row giving cM positions of markers may be included, in which case the
  cell in the third row, first column, must be blank.

  There need be no blank rows in the phenotype data file.
}


\section{CSVsr format}{
  This is just like the \code{csvs} format, but with each file rotated
  (or really transposed), so that rows are columns and columns are rows.
}


\section{Mapmaker format}{
  This format requires two files.  The so-called rawfile, specified by
  the argument \code{file}, contains the genotype and phenotype
  data. Rows beginning with the symbol \code{#} are ignored.  The first
  line should be either \code{data type f2 intercross} or
  \code{data type f2 backcross}.  The second line should begin with
  three numbers indicating the numbers of individuals, markers and
  phenotypes in the file.  This line may include the word \code{symbols}
  followed by symbol assignments (see the documentation for mapmaker,
  and cross your fingers).  The rest of the lines give genotype data
  followed by phenotype data, with marker and phenotype names always
  beginning with the \code{*} symbol.

  A second file contains the genetic map information, specified with
  the argument \code{mapfile}.  The map file may be in
  one of two formats.  The function will determine which format of map
  file is presented.

  The simplest format for the map file is not standard for the Mapmaker
  software, but is easy to create.  The file contains two or three
  columns separated by white space and with no header row.  The first
  column gives the chromosome assignments.  The second column gives the
  marker names, with markers listed in the order along the chromosomes.
  An optional third column lists the map positions of the markers.

  Another possible format for the map file is the \code{.maps}
  format, which is produced by Mapmaker.  The code for reading this
  format was written by Brian Yandell.

  Marker order is taken from the map file, either by the order they are
  presented or by the cM positions, if specified.
}

\section{Map Manager QTX format}{
  This format requires a single file (that produced by the Map Manager
  QTX program).
}

\section{QTL Cartographer format}{
  This format requires two files: the \code{.cro} and \code{.map} files
  for QTL Cartographer (produced by the QTL Cartographer
  sub-program, Rmap and Rcross).

  Note that the QTL Cartographer cross types are converted as follows:
  RF1 to riself, RF2 to risib, RF0 (doubled haploids) to bc, B1 or B2 to
  bc, RF2 or SF2 to f2.
}

\section{Tidy format}{
  This format requires three simple CSV files, separating the genotype,
  phenotype, and marker map information so that each file may be of a
  simple form.
}

\section{Gary format}{
  This format requires the six files.  All files have default names, and
  so the file names need not be specified if the default names are used.

  \code{genfile} (default = \code{"geno.dat"}) contains the genotype
  data.  The file contains one line per individual, with genotypes for
  the set of markers separated by white space.  Missing values are
  coded as 9, and genotypes are coded as 0/1/2 for AA/AB/BB.

  \code{mapfile} (default = \code{"markerpos.txt"}) contains two
  columns with no header row: the marker names in the first column and
  their cM position in the second column.  If marker positions are not
  available, use \code{mapfile=NULL}, and a dummy map will be inserted.

  \code{phefile} (default = \code{"pheno.dat"}) contains the phenotype
  data, with one row for each mouse and one column for each phenotype.
  There should be no header row, and missing values are coded as
  \code{"-"}.

  \code{chridfile} (default = \code{"chrid.dat"}) contains the
  chromosome identifier for each marker.

  \code{mnamesfile} (default = \code{"mnames.txt"}) contains the marker
  names.

  \code{pnamesfile} (default = \code{"pnames.txt"}) contains the names
  of the phenotypes.  If phenotype names file is not available, use
  \code{pnamesfile=NULL}; arbitrary phenotype names will then be
  assigned.
}


\section{Karl format}{
  This format requires three files; all files have default names, and so
  need not be specified if the default name is used.

  \code{genfile} (default = \code{"gen.txt"}) contains the genotype
  data.  The file contains one line per individual, with genotypes
  separated by white space.  Missing values are coded 0; genotypes are
  coded as 1/2/3/4/5 for AA/AB/BB/not BB/not AA.

  \code{mapfile} (default = \code{"map.txt"}) contains the map
  information, in the following complicated format: \cr \cr
    \code{n.chr} \cr
    \code{n.mar(1) rf(1,1) rf(1,2) \ldots rf(1,n.mar(1)-1)}\cr
    \code{mar.name(1,1)}\cr
    \code{mar.name(1,2)}\cr
    \code{\ldots}\cr
    \code{mar.name(1,n.mar(1))}\cr
    \code{n.mar(2)}\cr
    \code{\ldots}\cr
    \code{etc.} \cr

  \code{phefile} (default = \code{"phe.txt"}) contains a matrix of
  phenotypes, with one individual per line.  The first line in the
  file should give the phenotype names.
}

\section{MapQTL format}{
  This format requires three files, described in the manual of the
  MapQTL program (same as JoinMap).

  \code{genfile} corresponds to the loc file containing the genotype
  data. Each marker and its genotypes should be on a single line.

  \code{mapfile} corresponds to the map file containing the linkage
  group assignment, marker names and their map positions.

  \code{phefile} corresponds to the qua file containing the phenotypes.

  For the moment, only 4-way crosses are supported (CP population type
  in MapQTL).
}

\examples{
\dontrun{
# CSV format
dat1 <- read.cross("csv", dir="Mydata", file="mydata.csv")

# CSVS format
dat2 <- read.cross("csvs", dir="Mydata", genfile="mydata_gen.csv",
                   phefile="mydata_phe.csv")

# you can read files directly from the internet
datweb <- read.cross("csv", "https://rqtl.org/sampledata",
                     "listeria.csv")

# Mapmaker format
dat3 <- read.cross("mm", dir="Mydata", file="mydata.raw",
                   mapfile="mydata.map")

# Map Manager QTX format
dat4 <- read.cross("qtx", dir="Mydata", file="mydata.qtx")

# QTL Cartographer format
dat5 <- read.cross("qtlcart", dir="Mydata", file="qtlcart.cro",
                   mapfile="qtlcart.map")

# Gary format
dat6 <- read.cross("gary", dir="Mydata", genfile="geno.dat",
                   mapfile="markerpos.txt", phefile="pheno.dat",
                   chridfile="chrid.dat", mnamesfile="mnames.txt",
                   pnamesfile="pnames.txt")

# Karl format
dat7 <- read.cross("karl", dir="Mydata", genfile="gen.txt",
                   phefile="phe.txt", mapfile="map.txt")}
}

\author{Karl W Broman, \email{broman@wisc.edu}; Brian
  S. Yandell; Aaron Wolen}

\references{
  Broman, K. W. and Sen,
  \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. (2009) \emph{A
  guide to QTL mapping with R/qtl.}  Springer.  \url{https://rqtl.org/book/}
}


\seealso{ \code{\link{subset.cross}}, \code{\link{summary.cross}},
  \code{\link{plot.cross}}, \code{\link{c.cross}}, \code{\link{clean.cross}},
  \code{\link{write.cross}}, \code{\link{sim.cross}}, \code{\link[utils]{read.table}}.
  The \code{sampledata} directory in the package distribution contains
  sample data files in multiple formats.  Also see
  \url{https://rqtl.org/sampledata/}.
}

\keyword{IO}

\name{mqmplot.permutations}
\alias{mqmplot.permutations}

\title{Plot results from mqmpermutation}

\description{
    Plotting routine to display the results from a permutation QTL scan. (the
  output of \code{\link{mqmpermutation}})
}

\usage{
mqmplot.permutations(permutationresult, \dots)
}

\arguments{
  \item{permutationresult}{ \code{mqmmulti} object returned by
  \code{\link{mqmpermutation}} permutation analysis.
  }
  \item{\dots}{ Extra arguments passed to \code{\link{polyplot}}}
  }

%\details{
%  ~~ If necessary, more details than the description above ~~
%}

\value{ No value returned (plotting routine) }

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
  , Rutger Brouwer
}

\seealso{
\itemize{
  % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
# Simulated F2 Population
# QTL at chromosome 3
f2qtl <- c(3,15,1,0)

# Mouse genetic map
data(map10)

\dontshow{map10 <- map10[1:3]}
# Simulate a F2 Cross
f2cross <- sim.cross(map10,f2qtl,n=100,type="f2")
f2cross <- calc.genoprob(f2cross)
\dontrun{# Permutations to obtain significance threshold
f2result <- mqmpermutation(f2cross, n.perm=1000, method="permutation")
}\dontshow{f2result <- mqmpermutation(f2cross, n.perm=2, batchsize=2, method="permutation")}

# Plot results
mqmplot.permutations(f2result)
}
\keyword{ hplot }

\name{argmax.geno}
\alias{argmax.geno}

\title{Reconstruct underlying genotypes}

\description{
  Uses the Viterbi algorithm to identify the most likely sequence
  of underlying genotypes, given the observed multipoint marker data,
  with possible allowance for genotyping errors.
}

\usage{
argmax.geno(cross, step=0, off.end=0, error.prob=0.0001,
            map.function=c("haldane","kosambi","c-f","morgan"),
            stepwidth=c("fixed", "variable", "max"))
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{step}{Maximum distance (in cM) between positions at which the
   genotypes are reconstructed, though for \code{step=0}, genotypes
   are reconstructed only at the marker locations.}
 \item{off.end}{Distance (in cM) past the terminal markers on each
   chromosome to which the genotype reconstructions will be carried.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer or Morgan map function when converting genetic
   distances into recombination fractions.}
 \item{stepwidth}{Indicates whether the intermediate points should with
   fixed or variable step sizes.  We recommend using
   \code{"fixed"}; \code{"variable"} was included for the qtlbim
   package (\url{https://cran.r-project.org/src/contrib/Archive/qtlbim/}).  The \code{"max"}
   option inserts the minimal number of intermediate points so that the
   maximum distance between points is \code{step}.}
}

\details{
  We use the Viterbi algorithm to calculate
  \eqn{\arg \max_v \Pr(g = v | O)}{arg max_v Pr(g = v | O)} where
  \eqn{g} is the underlying sequence of genotypes and \eqn{O} is the
  observed marker genotypes.

  This is done by calculating
  \eqn{\gamma_k(v_k) = \max_{v_1, \ldots, v_{k-1}} \Pr(g_1 = v_1,
    \ldots, g_k = v_k, O_1, \ldots, O_k)}{%
    Q[k](v[k]) = max{v[1], \ldots, v[k-1]} Pr(g[1] = v[1],
    \ldots, g[k] = v[k], O[1], \ldots, O[k])}
  for \eqn{k = 1, \ldots, n} and then tracing back through the
  sequence.
}

\value{
  The input \code{cross} object is returned with a component,
  \code{argmax}, added to each component of \code{cross$geno}.
  The \code{argmax} component is a matrix of size [n.ind x n.pos], where
  n.pos is the
  number of positions at which the reconstructed genotypes were obtained,
  containing the most likely sequences of underlying genotypes.
  Attributes \code{"error.prob"}, \code{"step"}, and \code{"off.end"}
  are set to the values of the corresponding arguments, for later
  reference.
}

\section{Warning}{
  The Viterbi algorithm can behave badly when \code{step} is small but
  positive. One may observe quite different results for different values
  of \code{step}.

  The problem is that, in the presence of data like \code{A----H},  the
  sequences \code{AAAAAA} and \code{HHHHHH} may be more likely than any
  one of the sequences \code{AAAAAH}, \code{AAAAHH}, \code{AAAHHH},
  \code{AAHHHH}, \code{AHHHHH}, \code{AAAAAH}.  The Viterbi algorithm
  produces a single "most likely" sequence of underlying genotypes.
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2,chr=18:19)
}fake.f2 <- argmax.geno(fake.f2, step=2, off.end=5, err=0.01)
}

\references{
  Lange, K. (1999) \emph{Numerical analysis for statisticians}.
  Springer-Verlag. Sec 23.3.

  Rabiner, L. R. (1989) A tutorial on hidden Markov models and selected
  applications in speech recognition.  \emph{Proceedings of the IEEE}
  \bold{77}, 257--286.
}


\seealso{ \code{\link{sim.geno}}, \code{\link{calc.genoprob}},
\code{\link{fill.geno}} }

\keyword{utilities}

\name{c.scantwo}
\alias{c.scantwo}
\alias{cbind.scantwo}

\title{Combine columns from multiple scantwo results}

\description{
  Concatenate the columns from different runs of
  \code{\link{scantwo}}.
}

\usage{
\method{c}{scantwo}(\dots)
\method{cbind}{scantwo}(\dots)
}

\arguments{
 \item{\dots}{A set of objects of class \code{scantwo}. (This can also
 be a list of \code{scantwo} objects.) These are
   the results from  \code{\link{scantwo}} (with \code{n.perm=0}),
   generally run with different phenotypes or methods.  All must
   conform with each other, meaning that
   \code{\link{calc.genoprob}} and/or \code{\link{sim.geno}}
   were run with the same values for \code{step} and \code{off.end} and
   with data having the same genetic map.}
}

\value{The concatenated input, as a \code{scantwo} object.}

\details{
  The aim of this function is to concatenate the results from multiple
  runs \code{\link{scantwo}}, generally for
  different phenotypes and/or methods.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.bc)
fake.bc <- calc.genoprob(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=18:19)}
out2a <- scantwo(fake.bc, method="hk")
out2b <- scantwo(fake.bc, pheno.col=2, method="hk")

out2 <- c(out2a, out2b)
}

\seealso{ \code{\link{summary.scantwo}},
  \code{\link{scantwo}}, \code{\link{c.scanone}} }

\keyword{manip}

\name{find.markerpos}
\alias{find.markerpos}

\title{Find position of a marker}

\description{
  Find the chromosome and cM position of a set of genetic markers.
}

\usage{find.markerpos(cross, marker)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{marker}{A vector of marker names.}
}

\value{
  A data frame with two columns: the chromosome and position of the
  markers.
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
find.markerpos(hyper, "D4Mit164")
find.markerpos(hyper, c("D4Mit164", "D1Mit94"))
}

\seealso{ \code{\link{find.flanking}}, \code{\link{find.marker}},
\code{\link{find.pseudomarker}} }

\keyword{utilities}

\name{reduce2grid}
\alias{reduce2grid}

\title{Reduce to a grid of pseudomarkers.}

\description{
  For high-density marker data, rather than run \code{\link{scanone}} at both the
  markers and at a set of pseudomarkers, we reduce to just
  a set of evenly-spaced pseudomarkers
}

\usage{
reduce2grid(cross)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
}

\details{
  Genotype probabilities (from \code{\link{calc.genoprob}}) and/or
  imputations (from \code{\link{sim.geno}}) are subset to a grid of
  pseudomarkers.

  This is so that, in the case of high-density markers, we can do the
  genome scan calculations at a smaller set of points (on an
  evenly-spaced grid, but not at the markers) to save computation time.

  You need to first have run \code{\link{calc.genoprob}} and/or
  \code{\link{sim.geno}}, and you must use \code{stepwidth="fixed"}.

  When plotting results with \code{\link{plot.scanone}}, use
  \code{incl.markers=FALSE}, as the output of \code{\link{scanone}}
  won't include information about the marker locations and so will plot
  tick marks only at the first marker on each chromosome.
}

\value{
  The input \code{cross} object with included genotype probabilities or
  imputations subset to an evenly-spaced grid.
}

\examples{
data(hyper)
hyper <- calc.genoprob(hyper, step=2)
hypersub <- reduce2grid(hyper)

\dontrun{out <- scanone(hypersub)
plot(out, incl.markers=FALSE)}
}

\seealso{ \code{\link{calc.genoprob}},
          \code{\link{sim.geno}}, \code{\link{scanone}}, \code{\link{plot.scanone}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{utilities}

\name{geno.image}
\alias{geno.image}

\title{Plot grid of genotype data}

\description{
  Plot a grid showing which the genotype data in a cross.
}

\usage{
geno.image(x, chr, reorder=FALSE, main="Genotype data",
           alternate.chrid=FALSE, col=NULL, \dots)
}

\arguments{
  \item{x}{An object of class \code{cross}.  See
    \code{\link{read.cross}} for details. }
  \item{chr}{Optional vector indicating the chromosomes to be drawn in
  the plot. This should be a vector of character strings referring to
  chromosomes by name; numeric values are converted to strings.  Refer
  to chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{reorder}{Specify whether to reorder individuals according to
  their phenotypes.
  \tabular{ll}{
    FALSE \tab Don't reorder \cr
    TRUE  \tab Reorder according to the sum of the phenotypes \cr
    n \tab Reorder according to phenotype n}}
  \item{main}{Title to place on plot.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{col}{Vector of colors. The first is for missing genotypes,
  followed by colors for each of the genotypes. If \code{NULL}, a
  default set of colors are used.}
  \item{\dots}{Passed to \code{\link[graphics]{image}}.}
}

\details{
  Uses \code{\link[graphics]{image}} to plot a grid with the genotype
  data.  The genotypes AA, AB, BB are displayed in the colors red, blue,
  and green, respectively.  In an intercross, if there are genotypes
  "not BB" and "not AA", these are displayed in purple and orange,
  respectively.  White pixels indicate missing data.
}

\value{None.}

\examples{
data(listeria)
geno.image(listeria)
}

\seealso{ \code{\link{plot.cross}},
  \code{\link{plotMissing}}, \code{\link{plotGeno}},
  \code{\link[graphics]{image}} }


\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{switchAlleles}
\alias{switchAlleles}

\title{Switch alleles at selected markers}

\description{
  Switch alleles at selected markers in a cross object.
}

\usage{switchAlleles(cross, markers, switch=c("AB", "CD", "ABCD", "parents"))}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{markers}{Names of markers whose alleles are to be switched.}
 \item{switch}{For a 4-way cross, indicates how to switch the alleles (A
 for B, C for D, both A for B and C for D), or both A for C and B for D (\code{parents}).}
}

\value{
  The input cross object, with alleles at selected markers switched.
}

\details{
  For a backcross, we exchange homozygotes (AA) and heterozygotes (AB).

  For doubled haploids and recombinant inbred lines, we
  exchange the two homozygotes.

  For an intercross, we exchange the two homozygotes, and exchange C
  (i.e., not AA) and D (i.e., not BB). (The heterozygotes in an intercross are
  left unchanged.)

  For a 4-way cross, we consider the argument \code{switch}, and the
  exchanges among the genotypes are more complicated.
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
geno.crosstab(fake.f2, "D5M391", "D5M81")

# switch homozygotes at marker D5M391
fake.f2 <- switchAlleles(fake.f2, "D5M391")

geno.crosstab(fake.f2, "D5M391", "D5M81")

\dontrun{fake.f2 <- est.rf(fake.f2)
checkAlleles(fake.f2)
}
}

\seealso{ \code{\link{checkAlleles}}, \code{\link{est.rf}}, \code{\link{geno.crosstab}} }

\keyword{utilities}

\name{countXO}
\alias{countXO}

\title{Count number of obligate crossovers for each individual}

\description{
  Count the number of obligate crossovers for each individual in a
  cross, either by chromosome or overall.
}

\usage{
countXO(cross, chr, bychr=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to investigate.
   This should be a vector of character strings referring to chromosomes
   by name; numeric values are converted to strings.  Refer to
   chromosomes with a preceding \code{-} to have all chromosomes but
   those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{bychr}{If TRUE, return counts for each individual chromosome; if
   FALSE, return the overall number across the selected chromosomes.}
}

\details{
  For each individual we count the minimal number of crossovers that
  explain the observed genotype data.
}

\value{
  If \code{bychr=TRUE}, a matrix of counts is returned, with rows
  corresponding to individuals and columns corresponding to
  chromosomes.

  If \code{bychr=FALSE}, a vector of counts (the total number of
  crossovers across all selected chromosomes) is returned.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
plot(countXO(hyper))
}

\seealso{ \code{\link{ripple}}, \code{\link{locateXO}}, \code{\link{cleanGeno}} }

\keyword{utilities}

\name{mqmplot.clusteredheatmap}
\alias{mqmplot.clusteredheatmap}

\title{Plot clustered heatmap
of MQM scan on multiple phenotypes}

\description{
  Plot the results from a MQM scan on multiple phenotypes.
}

\usage{
mqmplot.clusteredheatmap(cross, mqmresult, directed=TRUE, legend=FALSE,
                         Colv=NA, scale="none", verbose=FALSE,
                         breaks = c(-100,-10,-3,0,3,10,100),
                         col = c("darkblue","blue","lightblue","yellow",
                                 "orange","red"), \dots)
}

\arguments{
  \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
  \item{mqmresult}{ Result object from mqmscanall, the object needs to be of class \code{mqmmulti} }
  \item{directed}{ Take direction of QTLs into account (takes more time because of QTL direction calculations }
  \item{legend}{ If TRUE, add a legend to the plot }
  \item{Colv}{ Cluster only the Rows, the columns (Markers) should not be clustered }
  \item{scale}{ character indicating if the values should be centered and scaled in either
                the row direction or the column direction, or none. The default "none" }
  \item{verbose}{If TRUE, give verbose output.}
  \item{breaks}{ Color break points for the LOD scores }
  \item{col}{ Colors used between breaks }
  \item{\dots}{Additional arguments passed to \code{\link[stats]{heatmap}}.}
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
data(multitrait)
\dontshow{multitrait <- subset(multitrait, chr=1:2, ind=!apply(multitrait$pheno, 1, function(a) any(is.na(a))))
multitrait$pheno <- multitrait$pheno[,1:3]}
multitrait <- fill.geno(multitrait) # impute missing genotype data
result <- mqmscanall(multitrait, logtransform=TRUE)
cresults <- mqmplot.clusteredheatmap(multitrait,result)
groupclusteredheatmap(multitrait,cresults,10)
}

\keyword{ hplot }

\name{mqmplot.singletrait}
\alias{mqmplot.singletrait}

\title{Plot LOD curves of a multiple-QTL model}

\description{
  Plot the LOD curve for a genome scan for a single trait, with a multiple-QTL model (the
  output of \code{\link{mqmscan}}).
}

\usage{
mqmplot.singletrait(result, extended = 0 ,\dots)
}

\arguments{
  \item{result}{ \code{\link{mqmscan}} result.}
  \item{extended}{ Extended plotting of the information content }
  \item{\dots}{ Extra arguments passed to \code{\link{plot.scanone}} }
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
#Simulated F2 Population
f2qtl <- c(3,15,1,0)                                    # QTL at chromosome 3
data(map10)                                             # Mouse genetic map
\dontshow{map10 <- map10[1:3]}
f2cross <- sim.cross(map10,f2qtl,n=100,type="f2")       # Simulate a F2 Cross
f2cross <- mqmaugment(f2cross)
f2result <- mqmscan(f2cross)                        # Do a MQM scan of the genome
mqmplot.singletrait(f2result)					# Use our fancy plotting routine
}

\keyword{ hplot }

\name{reorderqtl}
\alias{reorderqtl}

\title{Reorder the QTL in a qtl object}

\description{
  This function changes the order of the QTL in a QTL object.
}

\usage{
reorderqtl(qtl, neworder)
}

\arguments{
 \item{qtl}{A qtl object, as created by \code{\link{makeqtl}}.}
 \item{neworder}{A vector containing the positive integers up to the
   number of QTL in \code{qtl}, indicating the new order for the QTL.
   If missing, the QTL are ordered by chromosome and then by their
   position within a chromosome.}
}


\value{
  The input \code{qtl} object, with the loci reordered.
}

\details{
  Everything in the input \code{qtl} is reordered except the
  \code{altname} component, which contains names of the form \code{Q1},
  \code{Q2}, etc.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 6, 13)
qp <- c(25.8, 33.6, 18.63)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

qtl <- reorderqtl(qtl, c(2,3,1))
qtl

qtl <- reorderqtl(qtl)
qtl
}

\seealso{ \code{\link{makeqtl}}, \code{\link{fitqtl}},
  \code{\link{dropfromqtl}}, \code{\link{addtoqtl}},
  \code{\link{replaceqtl}} }

\keyword{utilities}

\name{badorder}
\alias{badorder}
\docType{data}

\title{An intercross with misplaced markers}

\description{
  Simulated data for an intercross with some markers out of order.
}

\usage{data(badorder)}

\format{
  An object of class \code{cross}.  See \code{\link{read.cross}}
  for details.
}

\details{
  There are 250 F2 individuals typed at a total of 36 markers on four
  chromosomes.  The data were simulated with QTLs at the center of
  chromosomes 1 and 3.

  The order of several markers on chromosome 1 is incorrect.  Markers on
  chromosomes 2 and 3 are switched.
}

\examples{
data(badorder)
\dontshow{badorder <- subset(badorder,chr=1)
}
# estimate recombination fractions
badorder <- est.rf(badorder)
plotRF(badorder)

# re-estimate map
newmap <- est.map(badorder)
plotMap(badorder, newmap)

# assess marker order on chr 1
rip3 <- ripple(badorder, chr=1, window=3)
summary(rip3)
}

\seealso{ \code{\link{est.rf}}, \code{\link{ripple}},
  \code{\link{est.map}}, \code{\link{sim.cross}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{datasets}

\name{top.errorlod}
\alias{top.errorlod}

\title{List genotypes with large error LOD scores}

\description{
  Prints those genotypes with error LOD scores above a specified
  cutoff.
}

\usage{
top.errorlod(cross, chr, cutoff=4, msg=TRUE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{cutoff}{Only those genotypes with error LOD scores above this
   cutoff will be listed.}
 \item{msg}{If TRUE, print a message if there are no apparent errors.}
}

\value{
  A data.frame with 4 columns, whose rows correspond to the genotypes
  that are possibly in error.  The four columns give the chromosome
  number, individual number, marker name, and error LOD score.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
\dontshow{hyper <- subset(hyper, chr=c(1,19))}
# Calculate error LOD scores
hyper <- calc.errorlod(hyper,error.prob=0.01)

# Print those above a specified cutoff
top.errorlod(hyper,cutoff=4)
}

\seealso{ \code{\link{calc.errorlod}}, \code{\link{plotGeno}}, \code{\link{plotErrorlod}} }


\keyword{print}

\name{convert.map}
\alias{convert.map}

\title{Change map function for a genetic map}

\description{
  Convert a genetic map from using one map function to another.
}

\usage{
\method{convert}{map}(object, old.map.function=c("haldane", "kosambi", "c-f", "morgan"),
         new.map.function=c("haldane", "kosambi", "c-f", "morgan"), \dots)
}

\arguments{
 \item{object}{A genetic map object, of class \code{"map"}: A list whose
 components are vectors of marker locations.}
 \item{old.map.function}{The map function used in forming the map in
   \code{object}.}
 \item{new.map.function}{The new map function to be used.}
 \item{\dots}{Ignored at this point.}
}

\value{
  The same as the input, but with inter-marker distances changed to
  reflect a different map function.
}

\details{
  The location of the first marker on each chromosome is left
  unchanged.  Inter-marker distances are converted to recombination
  fractions with the inverse of the \code{old.map.function}, and then
  back to distances with the \code{new.map.function}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
map <- pull.map(listeria)
map <- convert(map, "haldane", "kosambi")
listeria <- replace.map(listeria, map)
}

\seealso{ \code{\link{est.map}}, \code{\link{replace.map}} }

\keyword{manip}

\name{nchr}
\alias{nchr}

\title{Determine the number of chromosomes}

\description{
  Determine the number of chromosomes in a cross
  or map object.
}

\usage{
nchr(object)
}

\arguments{
 \item{object}{An object of class \code{cross} (see
   \code{\link{read.cross}} for details) or \code{map} (see
 \code{\link{sim.map}} for details).}
}

\value{
  The number of chromosomes in the input.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
nchr(fake.f2)
map <- pull.map(fake.f2)
nchr(map)
}

\seealso{ \code{\link{read.cross}}, \code{\link{plot.cross}},
  \code{\link{summary.cross}},
  \code{\link{nind}},
  \code{\link{totmar}},
  \code{\link{nmar}},
  \code{\link{nphe}} }

\keyword{print}

\name{calc.genoprob}
\alias{calc.genoprob}

\title{Calculate conditional genotype probabilities}

\description{
  Uses the hidden Markov model technology to calculate the
  probabilities of the true underlying genotypes given the observed
  multipoint marker data, with possible allowance for genotyping
  errors.
}

\usage{
calc.genoprob(cross, step=0, off.end=0, error.prob=0.0001,
              map.function=c("haldane","kosambi","c-f","morgan"),
              stepwidth=c("fixed", "variable", "max"))
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{step}{Maximum distance (in cM) between positions at which the
   genotype probabilities are calculated, though for \code{step = 0},
   probabilities are calculated only at the marker locations.}
 \item{off.end}{Distance (in cM) past the terminal markers on each
   chromosome to which the genotype probability calculations will be
   carried.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi or
   Carter-Falconer map function when converting genetic distances into
   recombination fractions.}
 \item{stepwidth}{Indicates whether the intermediate points should with
   fixed or variable step sizes.  We recommend using
   \code{"fixed"}; \code{"variable"} was included for the qtlbim
   package (\url{https://cran.r-project.org/src/contrib/Archive/qtlbim/}).  The \code{"max"}
   option inserts the minimal number of intermediate points so that the
   maximum distance between points is \code{step}.}
}

\details{
  Let \eqn{O_k}{O[k]} denote the observed marker genotype at position
  \eqn{k}, and \eqn{g_k}{g[k]} denote the corresponding true underlying
  genotype.

  We use the forward-backward equations to calculate
  \eqn{\alpha_{kv} = \log Pr(O_1, \ldots, O_k, g_k = v)}{%
    a[k][v] = log Pr(O[1], \ldots, O[k], g[k] = v)}
  and
  \eqn{\beta_{kv} = \log Pr(O_{k+1}, \ldots, O_n | g_k = v)}{%
    b[k][v] = log Pr(O[k+1], \ldots, O[n] | g[k] = v)}

  We then obtain
  \eqn{Pr(g_k | O_1, \ldots, O_n) = \exp(\alpha_{kv} + \beta_{kv}) / s}{%
    Pr(g[k] | O[1], \ldots, O[n] = exp(a[k][v] + b[k][v]) / s}
  where
  \eqn{s = \sum_v \exp(\alpha_{kv} + \beta_{kv})}{%
    s = sum_v exp(a[k][v] + b[k][v])}

  In the case of the 4-way cross, with a sex-specific map, we assume a
  constant ratio of female:male recombination rates within the
  inter-marker intervals.
}

\value{
  The input \code{cross} object is returned with a component,
  \code{prob}, added to each component of \code{cross$geno}.
  \code{prob} is an array of size [n.ind x n.pos x n.gen] where n.pos is
  the number of positions at which the probabilities were calculated and
  n.gen = 3 for an intercross, = 2 for a backcross, and = 4 for a 4-way
  cross.  Attributes \code{"error.prob"}, \code{"step"},
  \code{"off.end"}, and \code{"map.function"} are set to the values of
  the corresponding arguments, for later reference (especially by the
  function \code{\link{calc.errorlod}}).
}

\references{
  Lange, K. (1999) \emph{Numerical analysis for statisticians}.
  Springer-Verlag. Sec 23.3.

  Rabiner, L. R. (1989) A tutorial on hidden Markov models and selected
  applications in speech recognition.  \emph{Proceedings of the IEEE}
  \bold{77}, 257--286.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2,chr=18:19)
}fake.f2 <- calc.genoprob(fake.f2, step=2, off.end=5)

data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc,chr=18:19)
}fake.bc <- calc.genoprob(fake.bc, step=0, off.end=0, err=0.01)
}

\seealso{ \code{\link{sim.geno}}, \code{\link{argmax.geno}},
         \code{\link{calc.errorlod}} }

\keyword{utilities}

\name{totmar}
\alias{totmar}

\title{Determine the total number of markers}

\description{
  Determine the total number of markers in a cross
  or map object.
}

\usage{
totmar(object)
}

\arguments{
 \item{object}{An object of class \code{cross} (see
   \code{\link{read.cross}} for details) or \code{map} (see
 \code{\link{sim.map}} for details).}
}

\value{
  The total number of markers in the input.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
totmar(fake.f2)
map <- pull.map(fake.f2)
totmar(map)
}

\seealso{ \code{\link{read.cross}}, \code{\link{plot.cross}},
  \code{\link{summary.cross}},
  \code{\link{nind}},
  \code{\link{nchr}},
  \code{\link{nmar}},
  \code{\link{nphe}} }

\keyword{print}

\name{plotMap}
\alias{plotMap}
\alias{plot.map}

\title{Plot genetic map}

\description{
  Plot genetic map of marker locations for all chromosomes.
}

\usage{
\method{plot}{map}(x, map2, chr, horizontal=FALSE, shift=TRUE,
     show.marker.names=FALSE, alternate.chrid=FALSE, \dots)
plotMap(x, map2, chr, horizontal=FALSE, shift=TRUE,
     show.marker.names=FALSE, alternate.chrid=FALSE, \dots)
}

\arguments{
  \item{x}{A list whose components are vectors of marker locations.  A
    \code{cross} object may be given instead, in which case the genetic
    map it contains is used.}
  \item{map2}{An optional second genetic map with the same number (and
    names) of chromosomes.  As with the first argument, a
    \code{cross} object may be given instead.  If this argument is
    given, a comparison of the two genetic maps is plotted.}
  \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{horizontal}{Specifies whether the chromosomes should be
    plotted horizontally.}
  \item{shift}{If TRUE, shift the first marker on each chromosome to be
    at 0 cM.}
  \item{show.marker.names}{If TRUE, marker names are included.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
     axis labels, so that they may be more easily distinguished.}
  \item{\dots}{Passed to \code{plot}.}
}

\value{None.}

\details{
  Plots the genetic map for each chromosome, or a comparison of the
  genetic maps if two maps are given.

  For a comparison of two maps, the first
  map is on the left (or, if \code{horizontal=TRUE}, on the top).  Lines
  are drawn to connect markers.  Markers that exist in just one map and
  not the other are indicated by short line segments, on one side or the
  other, that are not connected across.

  For a sex-specific map, female and male maps are plotted against one
  another.  For two sex-specific maps, the two female maps are plotted
  against one another and the two male maps are plotted against one
  another.
}

\examples{
data(fake.bc)
\dontshow{fake.bc <- subset(fake.bc, chr=18:19)}
plotMap(fake.bc)
plotMap(fake.bc,horizontal=TRUE)

newmap <- est.map(fake.bc)
plot(newmap)
plotMap(fake.bc, newmap)

plotMap(fake.bc, show.marker.names=TRUE)

}

\seealso{ \code{\link{est.map}}, \code{\link{plot.cross}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{addtoqtl}
\alias{addtoqtl}

\title{Add to a qtl object}

\description{
  Add a QTL or multiple QTL to a qtl object.
}

\usage{
addtoqtl(cross, qtl, chr, pos, qtl.name, drop.lod.profile=TRUE)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{qtl}{The qtl object to which additional QTL are to be added.}
 \item{chr}{Vector indicating the chromosome for each new QTL. (These
 should be character strings referring to the chromosomes by name.)}
 \item{pos}{Vector (of same length as \code{chr}) indicating the
   positions on the chromosome for each new QTL. If there is no marker or
   pseudomarker at a position, the nearest position is used.}
 \item{qtl.name}{Optional user-specified name for each new QTL, used in the
   drop-one-term ANOVA table in \code{\link{fitqtl}}.
   If unspecified, the names will be of the form \code{"Chr1@10"} for a
   QTL on Chromsome 1 at 10 cM.}
 \item{drop.lod.profile}{If TRUE, remove any LOD profiles from the object.}
}

\value{
  An object of class \code{qtl}, just like the input \code{qtl} object,
  but with additional QTL added.  See \code{\link{makeqtl}} for
  details.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 6, 13)
qp <- c(25.8, 33.6, 18.63)
\dontshow{fake.f2 <- subset(fake.f2, chr=c(qc, 14))}
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")
qtl <- addtoqtl(fake.f2, qtl, 14, 35)
}

\seealso{ \code{\link{makeqtl}}, \code{\link{fitqtl}},
  \code{\link{dropfromqtl}}, \code{\link{replaceqtl}},
  \code{\link{reorderqtl}}  }

\keyword{utilities}

\name{nphe}
\alias{nphe}

\title{Determine the number of phenotypes QTL experiment}

\description{
  Determine the number of phenotypes in cross object.
}

\usage{
nphe(object)
}

\arguments{
 \item{object}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
}

\value{
  The number of phenotypes in the input cross object.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
nphe(fake.f2)
}

\seealso{ \code{\link{read.cross}}, \code{\link{plot.cross}},
  \code{\link{summary.cross}},
  \code{\link{nmar}},
  \code{\link{nchr}},
  \code{\link{totmar}},
  \code{\link{nind}} }

\keyword{print}

\name{ripple}
\alias{ripple}

\title{Compare marker orders}

\description{
  Investigate different marker orders for a given chromosome, comparing
  all possible permutations of a sliding window of markers.
}

\usage{
ripple(cross, chr, window=4, method=c("countxo","likelihood"),
       error.prob=0.0001, map.function=c("haldane","kosambi","c-f","morgan"),
       maxit=4000, tol=1e-6, sex.sp=TRUE, verbose=TRUE, n.cluster=1)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{The chromosome to investigate.  Only one chromosome is
  allowed. (This should be a character string referring to the
  chromosomes by name.)}
 \item{window}{Number of markers to include in the sliding window of
   permuted markers.  Larger numbers result in the comparison of a
   greater number of marker orders, but will require a considerable
   increase in computation time.}
 \item{method}{Indicates whether to compare orders by counting the
   number of obligate crossovers, or by a likelihood analysis.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions.}
 \item{maxit}{Maximum number of EM iterations to perform.}
 \item{tol}{Tolerance for determining convergence.}
 \item{sex.sp}{Indicates whether to estimate sex-specific maps; this is
   used only for the 4-way cross.}
 \item{verbose}{If TRUE, information about the number of orders (and, if
   \code{method="likelihood"}, about progress) are printed.}
 \item{n.cluster}{If the package \code{snow} is available and
   \code{n.perm} > 0, permutations are run in parallel using this number
   of nodes.  This is really only useful with \code{method="likelihood"}.}
}

\details{
  For \code{method="likelihood"}, calculations are done by first
  constructing a matrix of marker orders and then making repeated calls
  to the R function \code{\link{est.map}}.  Of course, it would be
  faster to do everything within C, but this was a lot easier to code.

  For \code{method="countxo"}, calculations are done within C.
}

\value{
  A matrix, given class \code{"ripple"}; the first set of columns are
  marker indices describing the order.  In the case of
  \code{method="countxo"}, the last column is the number of obligate
  crossovers for each particular order.  In the case of
  \code{method="likelihood"}, the last two columns are LOD scores (log
  base 10 likelihood ratios) comparing each order to the initial order
  and the estimated chromosome length for the given order.  Positive LOD
  scores indicate that the alternate order has more support than the
  original.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(badorder)
rip1 <- ripple(badorder, chr=1, window=3)
summary(rip1)

\dontrun{
rip2 <- ripple(badorder, chr=1, window=2, method="likelihood")
summary(rip2)
}

badorder <- switch.order(badorder, 1, rip1[2,])
}

\seealso{ \code{\link{summary.ripple}}, \code{\link{switch.order}},
  \code{\link{est.map}}, \code{\link{est.rf}} }

\keyword{utilities}

\name{A starting point}
\docType{package}
\alias{qtl-package}

\title{Introductory comments on R/qtl}
\description{
  A brief introduction to the R/qtl package, with a walk-through of an
  analysis.
}

\section{New to R and/or R/qtl?}{
  \itemize{
    \item In order to use the R/qtl package, you must type (within R)
    \code{library(qtl)}.  You may wish to include this in a
    \code{\link[base:Startup]{.Rprofile}} file.

    \item Documention and several tutorials are available
    at the R archive (\url{https://cran.r-project.org}).

    \item Use the \code{\link[utils]{help.start}} function to start the
    html version of the R help.

    \item Type \code{library(help=qtl)} to get a list of the functions
    in R/qtl.

    \item Use the \code{\link[utils]{example}} function to run examples
    of the various functions in R/qtl.

    \item A tutorial on the use of R/qtl is distributed with the
    package and is also available at \url{https://rqtl.org/rqtltour.pdf}.

    \item Download the latest version of R/qtl from the R archive or
    from \url{https://rqtl.org}.
  }
}

\section{Walk-through of an analysis}{
Here we briefly describe the use of R/qtl to analyze an experimental
cross.  A more extensive tutorial on its use is distributed with the
package and is also available at \url{https://rqtl.org/rqtltour.pdf}.

A difficult first step in the use of most data analysis software is the
import of data.  With R/qtl, one may import data in several different
formats by use of the function \code{\link{read.cross}}.  The
internal data structure used by R/qtl is rather complicated, and is
described in the help file for \code{\link{read.cross}}.  We won't
discuss data import any further here, except to say that the
comma-delimited format (\code{"csv"}) is recommended.  If you have
trouble importing data, send an email to Karl Broman,
\email{broman@wisc.edu}, perhaps attaching examples of your data
files.  (Such data will be kept confidential.)  Also see the sample data
files and code at \url{https://rqtl.org/sampledata/}.

We consider the example data \code{\link{hyper}}, an experiment on
hypertension in the mouse, kindly provided
by Bev Paigen and Gary Churchill.  Use the \code{\link[utils]{data}}
function to load the data.

\code{data(hyper)}

The \code{\link{hyper}} data set has class \code{"cross"}.  The
function \code{\link{summary.cross}} gives summary information
on the data, and checks the data for internal consistency.  A number
of other utility functions are available; hopefully these are
self-explanatory.

\code{summary(hyper)} \cr
\code{nind(hyper)} \cr
\code{nphe(hyper)} \cr
\code{nchr(hyper)} \cr
\code{nmar(hyper)} \cr
\code{totmar(hyper)}

The function \code{\link{plot.cross}} gives a graphical summary of
the data; it calls \code{\link{plotMissing}} (to plot a matrix
displaying missing genotypes) and \code{\link{plotMap}} (to plot
the genetic maps), and also displays histograms or barplots of the
phenotypes.  The \code{\link{plotMissing}} function can plot
individuals ordered by their phenotypes; you can see that for most
markers, only individuals with extreme phenotypes were genotyped.

\code{plot(hyper)} \cr
\code{plotMissing(hyper)} \cr
\code{plotMissing(hyper, reorder=TRUE)} \cr
\code{plotMap(hyper)}

Note that one marker (on chromosome 14) has no genotype data.  The
function \code{\link{drop.nullmarkers}} removes such markers from
the data.

\code{hyper <- drop.nullmarkers(hyper)} \cr
\code{totmar(hyper)}

The function \code{\link{est.rf}} estimates the recombination
fraction between each pair of markers, and calculates a LOD score for
the test of \eqn{r} = 1/2.  This is useful for identifying markers that
are placed on the wrong chromosome.  Note that since, for these data,
many markers were typed only on recombinant individuals, the pairwise
recombination fractions show rather odd patterns.

\code{hyper <- est.rf(hyper)} \cr
\code{plotRF(hyper)} \cr
\code{plotRF(hyper, chr=c(1,4))}

To re-estimate the genetic map for an experimental cross, use the
function \code{\link{est.map}}.  The function
\code{\link{plotMap}}, in addition to plotting a single map, can
plot the comparison of two genetic maps (as long as they are composed of
the same numbers of chromosomes and markers per chromosome).  The
function \code{\link{replace.map}} map be used to replace the
genetic map in a cross with a new one.

\code{newmap <- est.map(hyper, error.prob=0.01, verbose=TRUE)} \cr
\code{plotMap(hyper, newmap)} \cr
\code{hyper <- replace.map(hyper, newmap)}

The function \code{\link{calc.errorlod}} may be used to assist in
identifying possible genotyping errors; it calculates the error LOD
scores described by Lincoln and Lander (1992).  The
\code{\link{calc.errorlod}} function return a modified version of
the input cross, with error LOD scores included.  The function
\code{\link{top.errorlod}} prints the genotypes with values above a
cutoff (by default, the cutoff is 4.0).

\code{hyper <- calc.errorlod(hyper, error.prob=0.01)} \cr
\code{top.errorlod(hyper)}

The function \code{\link{plotGeno}} may be used to inspect the
observed genotypes for a chromosome, with likely genotyping errors
flagged.

\code{plotGeno(hyper, chr=16, ind=c(24:34, 71:81))}

Before doing QTL analyses, some intermediate calculations need to be
performed.  The function \code{\link{calc.genoprob}} calculates
conditional genotype probabilities given the multipoint marker data.
\code{\link{sim.geno}} simulates sequences of genotypes from their
joint distribution, given the observed marker data.

As with \code{\link{calc.errorlod}}, these functions return a
modified version of the input cross, with the intermediate calculations
included.  The \code{step} argument indicates the density of the grid on
which the calculations will be performed, and determines the density at
which LOD scores will be calculated.

\code{hyper <- calc.genoprob(hyper, step=2.5, error.prob=0.01)} \cr
\code{hyper <- sim.geno(hyper, step=2.5, n.draws=64, error.prob=0.01)}

The function \code{\link{scanone}} performs a genome scan with a
single QTL model.  By default, it performs standard interval mapping
(Lander and Botstein 1989): use of a normal model and the EM algorithm.
If one specifies \code{method="hk"}, Haley-Knott regression is performed
(Haley and Knott 1992).  These two methods require the results from
\code{\link{calc.genoprob}}.

\code{out.em <- scanone(hyper)} \cr
\code{out.hk <- scanone(hyper, method="hk")}

If one specifies \code{method="imp"}, a genome scan is performed by the
multiple imputation method of Sen and Churchill (2001).  This method
requires the results from \code{\link{sim.geno}}.

\code{out.imp <- scanone(hyper, method="imp")}

The output of \code{\link{scanone}} is a data.frame with class
\code{"scanone"}.  The function \code{\link{plot.scanone}} may be
used to plot the results, and may plot up to three sets of results
against each other, as long as they conform appropriately.

\code{plot(out.em)} \cr
\code{plot(out.hk, col="blue", add=TRUE)} \cr
\code{plot(out.imp, col="red", add=TRUE)} \cr
\code{plot(out.hk, out.imp, out.em, chr=c(1,4), lty=1,} \cr
\code{     col=c("blue","red","black"))}

The function \code{\link{summary.scanone}} may be used to list
information on the peak LOD for each chromosome for which the LOD
exceeds a specified threshold.

\code{summary(out.em)} \cr
\code{summary(out.em, threshold=3)} \cr
\code{summary(out.hk, threshold=3)} \cr
\code{summary(out.imp, threshold=3)}

The function \code{\link{max.scanone}} returns the maximum LOD
score, genome-wide.

\code{max(out.em)} \cr
\code{max(out.hk)} \cr
\code{max(out.imp)}

One may also use \code{\link{scanone}} to perform a permutation
test to get a genome-wide LOD significance threshold.

\code{operm.hk <- scanone(hyper, method="hk", n.perm=1000)}

The result has class \code{"scanoneperm"}.  The
\code{\link{summary.scanoneperm}} function may be used to calculate
LOD thresholds.

\code{summary(operm.hk, alpha=0.05)}

The permutation results may also be used in the
\code{\link{summary.scanone}} function to calculate LOD thresholds
and genome-scan-adjusted p-values.

\code{summary(out.hk, perms=operm.hk, alpha=0.05, pvalues=TRUE)}

We should say at this point that the function
\code{\link[base:save]{save.image}} will save your workspace to disk.  You'll
wish you had used this if R crashes.

\code{save.image()}

The function \code{\link{scantwo}} performs a two-dimensional
genome scan with a two-QTL model.  Methods \code{"em"}, \code{"hk"} and
\code{"imp"} are all available.  \code{\link{scantwo}} is
considerably slower than \code{\link{scanone}}, and can require a
great deal of memory.  Thus, you may wish to re-run
\code{\link{calc.genoprob}} and/or \code{\link{sim.geno}} with
a more coarse grid.

\code{hyper <- calc.genoprob(hyper, step=10, err=0.01)} \cr
\code{hyper <- sim.geno(hyper, step=10, n.draws=64, err=0.01)} \cr
\cr
\code{out2.hk <- scantwo(hyper, method="hk")} \cr
\code{out2.em <- scantwo(hyper)} \cr
\code{out2.imp <- scantwo(hyper, method="imp")}

The output is an object with class \code{scantwo}.  The function
\code{\link{plot.scantwo}} may be used to plot the results.  The
upper triangle contains LOD scores for tests of epistasis, while the
lower triangle contains LOD scores for the full model.

\code{plot(out2.hk)} \cr
\code{plot(out2.em)} \cr
\code{plot(out2.imp)}

The function \code{\link{summary.scantwo}} lists the interesting
aspects of the output.  For each pair of chromosomes \eqn{(k,l)}, it
calculates the maximum LOD score for the full model, \eqn{M_f(k,l)}; a
LOD score indicating evidence for a second QTL, allowing for epistasis),
\eqn{M_{fv1}(k,l)}{M_fv1(k,l)}; a LOD score indicating evidence for
epistasis, \eqn{M_i(k,l)}; the LOD score for the additive QTL model,
\eqn{M_a(k,l)}; and a LOD score indicating evidence for a second QTL,
assuming no epistasis, \eqn{M_{av1}(k,l)}{M_av1(k,l)}.

You must provide five LOD thresholds, corresponding to the above five
LOD scores, and in that order.  A chromosome pair is printed if either
(a) \eqn{M_f(k,l) \ge T_f}{M_f(k,l) >= T_f} and (\eqn{M_{fv1}(k,l) \ge
  T_{fv1}}{M_fv1(k,l) >= T_fv1} or \eqn{M_i(k,l) \ge
  T_i}{M_i(k,l) >= T_i}), or (b)
\eqn{M_a(k,l) \ge T_a}{M_a(k,l) >= T_a} and \eqn{M_{av1}(k,l) \ge
  T_{av1}}{M_av1(k,l) >= T_av1}.

\code{summary(out2.em, thresholds=c(6.2, 5.0, 4.6, 4.5, 2.3))} \cr
\code{summary(out2.em, thresholds=c(6.2, 5.0, Inf, 4.5, 2.3))}

In the latter case, the interaction LOD score will be ignored.

The function \code{\link{max.scantwo}} returns the maximum joint
and additive LODs for a two-dimensional genome scan.

\code{max(out2.em)}

Permutation tests may also performed with \code{\link{scantwo}};
it may take a few days of CPU time.  The output is a list containing the
maxima of the above five LOD scores for each of the imputations.

\code{operm2 <- scantwo(hyper, method="hk", n.perm=100)} \cr
\code{summary(operm2, alpha=0.05)}
}

\section{Citing R/qtl}{
  To cite R/qtl in publications, use the Broman et al. (2003) reference
  listed below.
}

\references{
  Broman, K. W. and Sen,
  \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. (2009) \emph{A
  guide to QTL mapping with R/qtl.}  Springer.  \url{https://rqtl.org/book/}

  Broman, K. W., Wu, H., Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2003) R/qtl: QTL
  mapping in experimental crosses.  \emph{Bioinformatics} \bold{19},
  889--890.

  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Lander, E. S. and Botstein, D. (1989) Mapping Mendelian factors underlying
  quantitative traits using RFLP linkage maps.  \emph{Genetics}
  \bold{121}, 185--199.

  Lincoln, S. E. and Lander, E. S. (1992) Systematic detection of
  errors in genetic linkage data.  \emph{Genomics} \bold{14}, 604--610.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{models}

\name{compareorder}
\alias{compareorder}

\title{Compare two orderings of markers on a chromosome}

\description{
  Compare the likelihood of an alternative order for markers on a
  chromosome to the current order.
}

\usage{
compareorder(cross, chr, order, error.prob=0.0001,
             map.function=c("haldane","kosambi","c-f","morgan"),
             maxit=4000, tol=1e-6, sex.sp=TRUE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
\item{chr}{The chromosome to investigate.  Only one chromosome is
  allowed. (This should be a character string referring to the
  chromosomes by name.)}
\item{order}{The alternate order of markers on the chromosome: a numeric
             vector that is a permutation of the integers from 1 to the
             number of markers on the chromosome.}
 \item{error.prob}{Assumed genotyping error rate used in the calculation
   of the penetrance Pr(observed genotype | true genotype).}
 \item{map.function}{Indicates whether to use the Haldane, Kosambi,
   Carter-Falconer, or Morgan map function when converting genetic
   distances into recombination fractions.}
 \item{maxit}{Maximum number of EM iterations to perform.}
 \item{tol}{Tolerance for determining convergence.}
 \item{sex.sp}{Indicates whether to estimate sex-specific maps; this is
   used only for the 4-way cross.}
}

\value{
  A data frame with two rows: the current order in the input cross object, and the
  revised order.  The first column is the log10 likelihood of the new order relative
  to the original one (positive values indicate that the new order is better supported).
  The second column is the estimated genetic length of the chromosome for each order.
  In the case of sex-specific maps, there are separate columns for the female and male
  genetic lengths.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(badorder)
compareorder(badorder, chr=1, order=c(1:8,11,10,9,12))
}

\seealso{ \code{\link{ripple}}, \code{\link{switch.order}},
  \code{\link{movemarker}} }

\keyword{utilities}

\name{bayesint}
\alias{bayesint}

\title{Bayesian credible interval}

\description{
  Calculate an approximate Bayesian credible interval for a particular
  chromosome, using output from \code{\link{scanone}}.
}

\usage{bayesint(results, chr, qtl.index, prob=0.95, lodcolumn=1, expandtomarkers=FALSE)}

\arguments{
 \item{results}{Output from \code{\link{scanone}}, or a qtl object
 as output from \code{\link{refineqtl}}.}
 \item{chr}{A chromosome ID (if input \code{results} are from
 \code{\link{scanone}} (should have length 1).}
 \item{qtl.index}{Numeric index for a QTL (if input \code{results} are
 from \code{\link{refineqtl}} (should have length 1).}
 \item{prob}{Probability coverage of the interval.}
 \item{lodcolumn}{An integer indicating which
   of the LOD score columns should be considered (if input
 \code{results} are from \code{\link{scanone}}).}
 \item{expandtomarkers}{If TRUE, the interval is expanded to the nearest
   flanking markers.}
}

\value{
  An object of class \code{scanone} indicating the
  estimated QTL position and the approximate endpoints
  for the Bayesian credible interval.
}

\details{
  We take \eqn{10^{LOD}}{10^LOD}, rescale it to have area 1, and then
  calculate the connected interval with density above some threshold
  and having coverage matching the target probability.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
\dontshow{hyper <- subset(hyper, chr=c(1,4))}
hyper <- calc.genoprob(hyper, step=0.5)
out <- scanone(hyper, method="hk")
bayesint(out, chr=1)
bayesint(out, chr=4)
bayesint(out, chr=4, prob=0.99)
bayesint(out, chr=4, expandtomarkers=TRUE)
}

\seealso{ \code{\link{scanone}}, \code{\link{lodint}} }

\keyword{utilities}

\name{clean.cross}
\alias{clean.cross}

\title{Remove derived data}

\description{
  Remove any intermediate calculations from a cross object.
}

\usage{
\method{clean}{cross}(object, \dots)
}

\arguments{
 \item{object}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{\dots}{Ignored at this point.}
}

\value{
  The input object, with any intermediate calculations
  (such as is produced by \code{\link{calc.genoprob}},
  \code{\link{argmax.geno}} and \code{\link{sim.geno}})
  removed.
}

\examples{
data(fake.f2)
names(fake.f2$geno)
fake.f2 <- calc.genoprob(fake.f2)
names(fake.f2$geno)
fake.f2 <- clean(fake.f2)
names(fake.f2$geno)
}

\seealso{ \code{\link{drop.nullmarkers}},
  \code{\link{drop.markers}}, \code{\link{clean.scantwo}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{manip}

\name{flip.order}
\alias{flip.order}
\title{Flip the orders of markers on a set of chromosomes}

\description{
  Flip the orders of markers on a specified set of chromosome,
  so that the markers will be in the reverse order.
}

\usage{flip.order(cross, chr)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Vector indicating the chromosomes to flip. This should be a
 vector of character strings referring to chromosomes by name. A logical
 (TRUE/FALSE) vector may also be used.}
}

\value{
  The input \code{cross} object, but with the marker order on the
  specified chromosomes flipped.
}

\details{
  If the cross contains results from \code{\link{calc.genoprob}},
  \code{\link{sim.geno}}, \code{\link{argmax.geno}}, or
  \code{\link{calc.errorlod}}, those results are also updated.

  Results of \code{\link{est.rf}} and \code{\link{markerlrt}} are
  deleted.
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
fake.f2 <- flip.order(fake.f2, c(1, 5, 13))
}

\seealso{ \code{\link{switch.order}} }

\keyword{manip}

\name{phenames}
\alias{phenames}

\title{Pull out the phenotypes names from a cross}

\description{
  Pull out the phenotype names from a cross object as a vector.
}

\usage{phenames(cross)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
}

\value{
  A vector of character strings (the phenotype names).
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
phenames(listeria)
}

\seealso{ \code{\link{markernames}}, \code{\link{chrnames}} }


\keyword{utilities}

\name{find_large_intervals}
\alias{find_large_intervals}

\title{Find large intervals in a map}

\description{
  Find large inter-marker intervals in a map.
}

\usage{
find_large_intervals(map, min_length=35)
}

\arguments{
 \item{map}{A list of numeric vectors; each component is a chromosome
   with the positions of markers on that chromosome. Can also
   be an object of class \code{cross}, in which case
 \code{\link{pull.map}} is used.}
 \item{min_length}{Minimum length of interval to be flagged.}
}


\value{
  Data frame with chromosome, left and right markers and interval length.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

find_large_intervals(fake.f2, 30)
}

\seealso{ \code{\link{summaryMap}} }

\keyword{utilities}

\name{qtlversion}
\alias{qtlversion}

\title{Installed version of R/qtl}

\description{
  Print the version number of the currently installed version of R/qtl.
}

\usage{qtlversion()}

%\arguments{
%}

\value{
  A character string with the version number of the currently installed
  version of R/qtl.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
  qtlversion()
}


\keyword{print}

\name{plot.qtl}
\alias{plot.qtl}

\title{Plot QTL locations}

\description{
  Plot the locations of the QTL against a genetic map
}

\usage{
\method{plot}{qtl}(x, chr, horizontal=FALSE, shift=TRUE,
     show.marker.names=FALSE, alternate.chrid=FALSE, justdots=FALSE,
     col="red", \dots)
}

\arguments{
  \item{x}{An object of class \code{"qtl"}, as produced by
    \code{\link{makeqtl}}.}
  \item{chr}{Optional vector indicating the chromosomes to plot.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
  \item{horizontal}{Specifies whether the chromosomes should be
    plotted horizontally.}
  \item{shift}{If TRUE, shift the first marker on each chromosome to be
    at 0 cM.}
  \item{show.marker.names}{If TRUE, marker names are included.}
  \item{alternate.chrid}{If TRUE and more than one chromosome is
    plotted, alternate the placement of chromosome
    axis labels, so that they may be more easily distinguished.}
  \item{justdots}{If FALSE, just plot dots at the QTL, rather than
    arrows and QTL names.}
  \item{col}{Color used to plot indications of QTL}
  \item{\dots}{Passed to \code{\link{plotMap}}.}
}

\value{None.}

\details{
  Creates a plot, via \code{\link{plotMap}}, and indicates the
  locations of the QTL in the input QTL object, \code{x}.
}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c("1", "6", "13")
qp <- c(25.8, 33.6, 18.63)
fake.f2 <- subset(fake.f2, chr=qc)
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}
fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")
plot(qtl)
plot(qtl, justdots=TRUE, col="seagreen")
}

\seealso{ \code{\link{plotMap}}, \code{\link{makeqtl}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{chrnames}
\alias{chrnames}

\title{Pull out the chromosome names from a cross}

\description{
  Pull out the chromosome names from a cross object as one big vector.
}

\usage{chrnames(cross)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
}

\value{
  A vector of character strings (the chromosome names).
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
chrnames(listeria)
}

\seealso{ \code{\link{markernames}}, \code{\link{phenames}} }


\keyword{utilities}

\name{drop.nullmarkers}
\alias{drop.nullmarkers}

\title{Drop markers without any genotype data}

\description{
  Drop markers, from the data matrices and genetic maps, that have no
  genotype data.
}

\usage{
drop.nullmarkers(cross)
}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
}

\value{
  The input object, with any markers lacking genotype data removed from
  the genotype data matrices, genetic maps, and, if applicable, any
  derived data (such as produced by \code{\link{calc.genoprob}}).
  (It might be a good idea to re-derive such things after using this
  function.)
}

\examples{
# removes one marker from hyper
data(hyper)
hyper <- drop.nullmarkers(hyper)

# shouldn't do anything to listeria
data(listeria)
listeria <- drop.nullmarkers(listeria)
}

\seealso{ \code{\link{nullmarkers}}, \code{\link{drop.markers}}, \code{\link{clean.cross}},
  \code{\link{geno.table}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{manip}

\name{switch.order}
\alias{switch.order}
\title{Switch the order of markers on a chromosome}

\description{
  Switch the order of markers on a specified chromosome to a specified
  new order.
}

\usage{switch.order(cross, chr, order, error.prob=0.0001,
                    map.function=c("haldane","kosambi","c-f","morgan"),
                    maxit=4000, tol=1e-6, sex.sp=TRUE)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{The chromosome for which the marker order is to be
   switched. Only one chromosome is allowed. (This should be a character
 string referring to the chromosomes by name.)}
 \item{order}{A vector of numeric indices defining the new marker
   order. The vector may have length two more than the number of
   markers, for easy in use with the output of the function
   \code{\link{ripple}}.}
 \item{error.prob}{Assumed genotyping error rate (passed to
   \code{\link{est.map}}).}
 \item{map.function}{Map function to be used (passed to
   \code{\link{est.map}}).}
 \item{maxit}{Maximum number of EM iterations to perform.}
 \item{tol}{Tolerance for determining convergence.}
 \item{sex.sp}{Indicates whether to estimate sex-specific maps; this is
 used only for the 4-way cross.}
}

\value{
  The input \code{cross} object, but with the marker order on the
  specified chromosome updated, and with any derived data removed
  (except for recombination fractions, if present, which are not
  removed);  the genetic map for the relevant chromosome is
  re-estimated.
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
fake.f2 <- switch.order(fake.f2, 1, c(1,3,2,4:7))
}

\seealso{ \code{\link{flip.order}}, \code{\link{ripple}}, \code{\link{clean.cross}} }

\keyword{manip}

\name{formMarkerCovar}
\alias{formMarkerCovar}

\title{Create matrix of marker covariates for QTL analysis}

\description{
  Pull out a matrix of genotypes or genotype probabilities to use
  markers as covariates in QTL analysis.
}

\usage{formMarkerCovar(cross, markers, method=c("prob", "imp", "argmax"), \dots)}


\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{markers}{A vector of character strings of marker or pseudomarker
   names. Pseudomarker names may be of the form \code{"5@21.5"} (for chr 5
   at 21.5 cM), but then all names must be of this form.}
 \item{method}{If \code{method="prob"}, the genotype probabilities from
   \code{\link{calc.genoprob}} are used; otherwise we use
   \code{\link{fill.geno}} to impute missing data, with this method.}
 \item{\dots}{Passed to \code{\link{fill.geno}}, if necessary.}
}

\value{
  A matrix containing genotype probabilities or genotype indicators,
  suitable for use as covariates in \code{\link{scanone}}.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(hyper)
hyper <- calc.genoprob(hyper, step=0)
peakMarker <- "D4Mit164"
X <- formMarkerCovar(hyper, peakMarker)
\dontshow{hyper <- hyper[c(1,4),]}
out <- scanone(hyper, addcovar=X)
}

\seealso{ \code{\link{pull.geno}}, \code{\link{pull.genoprob}},
  \code{\link{fill.geno}}, \code{\link{scanone}} }

\keyword{utilities}

\name{plotModel}
\alias{plotModel}

\title{Plot a QTL model}

\description{
  Plot a graphical representation of a QTL model, with nodes
  representing QTL and line segments representing pairwise
  interactions.
}

\usage{
plotModel(qtl, formula, circrad.rel=0.25, circrad.abs,
         cex.name=1, chronly=FALSE, order, \dots)
}

\arguments{
  \item{qtl}{A QTL object (as created by \code{\link{makeqtl}}) or
    vector of character strings indicating the names for the QTL.  This
  is also allowed to be a list that contains a component named
  \code{"chr"} (and, optionally, components names \code{"pos"} and
  \code{"formula"}).}
  \item{formula}{Optional formula defining the QTL model.  If missing,
  we look for an attribute \code{"formula"} to the input QTL object or a
  item named \code{"formula"} within the QTL object.}
  \item{circrad.rel}{Radius of the circles that
  indicate the QTL, relative to the distance between the circles.}
  \item{circrad.abs}{Optional radius of the circles that
  indicate the QTL; note that the plotting region will have x- and
  y-axis limits spanning 3 units.}
  \item{cex.name}{Character expansion for the QTL names.}
  \item{chronly}{If TRUE and a formal QTL object is given, only the
  chromosome IDs are used to identify the QTL.}
  \item{order}{Optional vector indicating a permutation of the QTL to
  define where they are to appear in the plot.  QTL are placed around a
  circle, starting at the top and going clockwise.}
  \item{\dots}{Passed to the function \code{plot}.}
}

%\details{
%
%}

\value{None.}

\examples{
# plot a QTL model, using a vector of character strings to define the QTL
plotModel(c("1","4","6","15"), formula=y~Q1+Q2+Q3*Q4)

# plot an additive QTL model
data(hyper)
hyper <- calc.genoprob(hyper)
qtl <- makeqtl(hyper, chr=c(1,4,6,15), pos=c(68.3,30,60,18), what="prob")
plotModel(qtl)

# include an interaction
plotModel(qtl, formula=y~Q1+Q2+Q3*Q4)

# alternatively, include the formula as an attribute to the QTL object
attr(qtl, "formula") <- y~Q1+Q2+Q3*Q4
plotModel(qtl)

# if formula given, the attribute within the object is ignored
plotModel(qtl, y~Q1+Q2+Q3+Q4)

# NULL formula indicates additive QTL model
plotModel(qtl, NULL)

# reorder the QTL in the figure
plotModel(qtl, order=c(1,3,4,2))

# show just the chromosome numbers
plotModel(qtl, chronly=TRUE)
}

\seealso{ \code{\link{stepwiseqtl}}, \code{\link{makeqtl}} }


\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{interpPositions}
\alias{interpPositions}

\title{Interpolate positions from one map to another}

\description{
  On the basis of a pair of marker maps with common markers, take
  positions along one map and interpolate (or, past the terminal markers
  on a chromosome, extrapolate) their positions on the second map.
}

\usage{
interpPositions(oldpositions, oldmap, newmap)
}

\arguments{
 \item{oldpositions}{A data frame with two columns: \code{chr}
 (chromosome identifiers) and
 \code{pos} (positions, along \code{oldmap}).}
 \item{oldmap}{An object of class \code{"map"}; see
 \code{\link{sim.map}} for details.}
 \item{newmap}{An object of class \code{"map"}, with the same
 chromosomes and markers as \code{oldmap}.}
}

\value{The input data frame, \code{oldpositions}, with an additional
  column \code{newpos} with the interpolated positions along
  \code{newmap}.}


\details{
In this explanation, take \code{oldmap} and \code{newmap} to be the
physical and genetic maps, respectively.

We use linear interpolation within each interval, assuming a constant
recombination rate within the interval.  Past the terminal
markers, we use linear extrapolation, using the chromosome-wide average
recombination rate.
}

\author{Karl W Broman, \email{broman@wisc.edu} }


\examples{
data(hyper)

# hyper genetic map
gmap <- pull.map(hyper)

# a fake physical map, with each chromosome starting at 0.
pmap <- shiftmap(rescalemap(gmap, 2))

# positions on pmap to determine location on gmap
tofind <- data.frame(chr=c(1, 5, 17, "X"), pos=c(220, 20, 105, 10))
rownames(tofind) <- paste("loc", 1:nrow(tofind), sep="")

interpPositions(tofind, pmap, gmap)
}

\seealso{ \code{\link{shiftmap}}, \code{\link{rescalemap}}, \code{\link{pull.map}} }

\keyword{manip}

\name{nqrank}
\alias{nqrank}

\title{Transform a vector of quantitative values to the corresponding
  normal quantiles}

\description{
  Transform a vector of quantitative values to the corresponding normal
  quantiles (preserving the mean and SD).
}

\usage{
nqrank(x, jitter)
}

\arguments{
  \item{x}{A numeric vector}
  \item{jitter}{If TRUE, randomly jitter the values to break ties.}
}

\value{
  A numeric vector; the input \code{x} is converted to ranks and then to
  normal quantiles.
}

\author{
Karl W Broman, \email{broman@wisc.edu}
}

\seealso{ \code{\link{rank}}, \code{\link{qnorm}}, \code{transformPheno} }

\examples{
data(hyper)

hyper <- transformPheno(hyper, pheno.col=1, transf=nqrank)
}

\keyword{ utilities }

\name{summary.scanPhyloQTL}
\alias{summary.scanPhyloQTL}

\title{Summarize the results a genome scan to map a QTL to a phylogenetic tree}

\description{
  Print the maximum LOD scores for each partition on each chromosome,
  from the results of \code{\link{scanPhyloQTL}}.
}

\usage{
\method{summary}{scanPhyloQTL}(object, format=c("postprob", "lod"),
        threshold, \dots)
}

\arguments{
 \item{object}{An object output by the function
   \code{\link{scanPhyloQTL}}.}
 \item{format}{Indicates whether to provide LOD scores or approximate
   posterior probabilities; see Details below.}
 \item{threshold}{A threshold determining which chromosomes should be
   output; see Details below.}
 \item{\dots}{Ignored at this point.}
}

\details{
  This function is used to report chromosomes deemed interesting from a one-QTL
  genome scan to map QTL to a phylogenetic tree (by \code{\link{scanPhyloQTL}}).

  For \code{format="lod"}, the output contains the maximum LOD score for
  each partition on each chromosome (which do not necessarily occur at
  the same position).  The position corresponds to the peak location for
  the partition with the largest LOD score on that chromosome.  The
  last column is the overall maximum LOD (across partitions) on that
  chromosome.  The second-to-last column is the inferred partition
  (i.e., that with the largest LOD
  score.  The third-to-last column is the difference between the LOD score for
  the best partition and that for the second-best.

  For \code{format="postprob"}, the final column contains the maximum
  LOD score across partitions.  But instead of providing the LOD
  scores for each partition, these are converted to approximate
  posterior probabilities under the assumption of a single diallelic QTL
  on that chromosome:  on each chromosome, we take
  \eqn{10^{LOD}}{10^LOD} for the partitions and rescale them to sum to 1.

  The \code{threshold} argument is applied to the last column (the
  maximum LOD score across partitions).
}

\value{
  An object of class \code{summary.scanPhyloQTL}, to be printed by
  \code{print.summary.scanPhyloQTL}.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
\dontrun{
# example map; drop X chromosome
data(map10)
map10 <- map10[1:19]

# simulate data
x <- simPhyloQTL(4, partition="AB|CD", crosses=c("AB", "AC", "AD"),
                 map=map10, n.ind=150,
                 model=c(1, 50, 0.5, 0))

# run calc.genoprob on each cross
x <- lapply(x, calc.genoprob, step=2)

# scan genome, at each position trying all possible partitions
out <- scanPhyloQTL(x, method="hk")

# maximum peak
max(out, format="lod")

# approximate posterior probabilities at peak
max(out, format="postprob")

# all peaks above a threshold for LOD(best) - LOD(2nd best)
summary(out, threshold=1, format="lod")

# all peaks above a threshold for LOD(best), showing approx post'r prob
summary(out, format="postprob", threshold=3)

# plot of results
plot(out)
}
}

\seealso{ \code{\link{scanPhyloQTL}}, \code{\link{plot.scanPhyloQTL}},
  \code{\link{max.scanPhyloQTL}}, \code{\link{summary.scanone}},
  \code{\link{inferredpartitions}},
  \code{\link{simPhyloQTL}}
}

\references{
  Broman, K. W., Kim, S., An\'e, C. and Payseur, B. A.  Mapping
  quantitative trait loci to a phylogenetic tree.  In preparation.
}


\keyword{print}

\name{pull.genoprob}
\alias{pull.genoprob}

\title{Pull out the genotype probabilities from a cross}

\description{
  Pull out the results of \code{\link{calc.genoprob}} from a cross as a matrix.
}

\usage{pull.genoprob(cross, chr, omit.first.prob=FALSE,
              include.pos.info=FALSE, rotate=FALSE)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to consider.
  This should be a vector of character strings referring to chromosomes
  by name; numeric values are converted to strings.  Refer to
  chromosomes with a preceding \code{-} to have all chromosomes but
  those considered.  A logical (TRUE/FALSE) vector may also be used.}
 \item{omit.first.prob}{If TRUE, omit the probabilities for the first
 genotype at each position (since they sum to 1).}
 \item{include.pos.info}{If TRUE, include columns with marker name,
 genotype, chromosome ID, and cM position.  (If
 \code{include.pos.info=TRUE}, we take \code{rotate=TRUE}.)}
 \item{rotate}{If TRUE, return matrix with individuals as columns and
 positions/genotypes as rows.  If FALSE, rows correspond to individuals.}
}

\value{
  A matrix containing genotype probabilities.
  Multiple chromosomes and the multiple genotypes at each position are
  pasted together.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
listeria <- calc.genoprob(listeria, step=1, stepwidth="max")
pr <- pull.genoprob(listeria, chr=c(5,13), omit.first.prob=TRUE, include.pos.info=TRUE, rotate=TRUE)
pr[1:5,1:10]
}

\seealso{ \code{\link{pull.geno}}, \code{\link{pull.argmaxgeno}},
  \code{\link{pull.draws}}, \code{\link{calc.genoprob}} }

\keyword{utilities}

\name{summary.scanoneboot}
\alias{summary.scanoneboot}

\title{Bootstrap confidence interval for QTL location}

\description{
  Calculates a bootstrap confidence interval for QTL location, using the
  bootstrap results from \code{\link{scanoneboot}}.
}

\usage{
\method{summary}{scanoneboot}(object, prob=0.95, expandtomarkers=FALSE, \dots)
}

\arguments{
 \item{object}{Output from \code{\link{scanoneboot}}.}
 \item{prob}{Desired coverage.}
 \item{expandtomarkers}{If TRUE, the interval is expanded to the nearest
   flanking markers.}
 \item{\dots}{Ignored at this point.}
}

\value{
  An object of class \code{scanone}, indicating the
  position with the maximum LOD, and indicating endpoints
  for the estimated bootstrap confidence interval.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
\dontrun{data(fake.f2)
fake.f2 <- calc.genoprob(fake.f2, step=1, err=0.001)
bootoutput <- scanoneboot(fake.f2, chr=13, method="hk")

summary(bootoutput)}
}

\seealso{ \code{\link{scanoneboot}}, \code{\link{plot.scanoneboot}},
  \code{\link{lodint}}, \code{\link{bayesint}} }

\keyword{utilities}

\name{write.cross}
\alias{write.cross}

\title{Write data for a QTL experiment to a file}

\description{
  Data for a QTL experiment is written to a file (or files).
}

\usage{
write.cross(cross, format=c("csv", "csvr", "csvs", "csvsr",
                            "mm", "qtlcart", "gary", "qtab",
                            "mapqtl", "tidy"),
            filestem="data", chr, digits=NULL, descr)
}

\arguments{
  \item{cross}{An object of class \code{cross}. See
    \code{\link{read.cross}} for details.}
  \item{format}{Specifies whether to write the data in comma-delimited,
    rotated comma-delimited, Mapmaker, QTL Cartographer, Gary
    Churchill's, QTAB, MapQTL format.}
  \item{filestem}{A character string giving the first part of the output
    file names (the bit before the dot).  In Windows, use forward
    slashes (\code{"/"}) or double backslashes (\code{"\\\\"}) to
    specify directory trees.}
  \item{chr}{A vector specifying for which chromosomes genotype data
    should be written.  This should be a vector of character strings
  referring to chromosomes by name; numeric values are converted to
  strings.  Refer to chromosomes with a preceding \code{-} to have all
  chromosomes but those considered.  A logical (TRUE/FALSE) vector may
  also be used.}
  \item{digits}{Number of digits to which phenotype values and genetic
     map positions should be rounded. If NULL (the default), they are
     not rounded.}
  \item{descr}{Character string description; used only with \code{format="qtab"}.}
}

%\value{
%  Data is written to a single file (comma-delimited format) or two files
%  (Mapmaker format).
%}

\details{
  Comma-delimited formats: a single csv file is created in the formats
  \code{"csv"} or \code{"csvr"}. Two files are created (one for the
  genotype data and one for the phenotype data) for the formats
  \code{"csvs"} and \code{"csvsr"}; if \code{filestem="file"}, the two
  files will be names \code{"file_gen.csv"} and \code{"file_phe.csv"}.
  See the help file for \code{\link{read.cross}} for details on these formats.

  Mapmaker format: Data is written to two files.
  Suppose \code{filestem="file"}.  Then \code{"file.raw"} will contain
  the genotype and phenotype data, and \code{"file.prep"} will contain
  the necessary code for defining the chromosome assignments, marker
  order, and inter-marker distances.

  QTL Cartographer format: Data is written to two files.    Suppose
  \code{filestem="file"}.  Then \code{"file.cro"} will contain
  the genotype and phenotype data, and \code{"file.map"} will contain
  the genetic map information.  Note that cross types are converted to
  QTL Cartographer cross types as follows: riself to RF1, risib to RF2,
  bc to B1 and f2 to RF2.

  Gary's format: Data is written to six files. They are: \cr
  \code{"geno.data"} - genotype data; \cr
  \code{"pheno.data"} - phenotype data; \cr
  \code{"chrid.dat"} - the chromosome identifier for each marker; \cr
  \code{"mnames.txt"} - the marker names; \cr
  \code{"markerpos.txt"} - the marker positions; \cr
  \code{"pnames.txt"} - the phenotype names

  QTAB format: See
  \href{https://github.com/qtlHD/qtlHD/blob/master/doc/input/qtab.md}{documentation}.

  MapQTL format: See
  \href{https://www.kyazma.nl/docs/MQ6Manual.pdf}{documentation}.

  Tidy format: Data is written to three files, \code{"stem_gen.csv"},
  \code{"stem_phe.csv"}, and \code{"stem_map.csv"} (where \code{stem} is
  taken from the \code{filestem} argument.
}


\examples{
\dontrun{data(fake.bc)

# comma-delimited format
write.cross(fake.bc, "csv", "Data/fakebc", c(1,5,13))

# rotated comma-delimited format
write.cross(fake.bc, "csvr", "Data/fakebc", c(1,5,13))

# split comma-delimited format
write.cross(fake.bc, "csvs", "Data/fakebc", c(1,5,13))

# split and rotated comma-delimited format
write.cross(fake.bc, "csvsr", "Data/fakebc", c(1,5,13))

# Mapmaker format
write.cross(fake.bc, "mm", "Data/fakebc", c(1,5,13))

# QTL Cartographer format
write.cross(fake.bc, "qtlcart", "Data/fakebc", c(1,5,13))

# Gary's format
write.cross(fake.bc, "gary", c(1,5,13))} }

\author{Karl W Broman, \email{broman@wisc.edu}; Hao Wu; Brian
  S. Yandell; Danny Arends; Aaron Wolen}

\seealso{ \code{\link{read.cross}} }

\keyword{IO}

\name{map10}
\alias{map10}
\docType{data}

\title{An example genetic map}

\description{
  A genetic map corresponding approximately to the mouse genome with
  a 10 cM marker spacing.
}

\usage{data(map10)}

\format{
  An object of class \code{map}: a list whose components are vectors of
  marker locations.  This map approximates the mouse genome, with 20
  chromosomes (including the X chromosome) and 187 markers at an
  approximately 10 cM spacing.  The markers are equally spaced on each
  chromosome, but the spacings are a bit above or below 10 cM, so that
  the lengths match those in the Mouse Genome Database.}

\examples{
  data(map10)
  plot(map10)

  mycross <- sim.cross(map10, type="f2", n.ind=100)
}

\seealso{ \code{\link{sim.map}}, \code{\link{plotMap}},
  \code{\link{pull.map}} }

\keyword{datasets}

\name{formLinkageGroups}
\alias{formLinkageGroups}

\title{Partition markers into linkage groups}

\description{
  Use pairwise linkage information between markers (as calculated by
  \code{\link{est.rf}} to partition markers into linkage groups.
}

\usage{
formLinkageGroups(cross, max.rf=0.25, min.lod=3, reorgMarkers=FALSE,
                  verbose=FALSE)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{max.rf}{Maximum recombination fraction for placing two markers in
   the same linkage group (see Details).}
 \item{min.lod}{Minimum LOD score for placing two markers in the same
 linkage group (see Details).}
 \item{reorgMarkers}{If TRUE, the output is a cross object, like the
 input, but with the markers organized into the inferred linkage
 groups.  If FALSE, the output is a table indicating the initial
 chromosome assignments and the inferred linkage group partitions.}
 \item{verbose}{If TRUE, display information about the progress of the
 calculations.}
}

\value{
  If \code{reorgMarkers=FALSE} (the default), the output is a data frame
  with rows corresponding to the markers and with two columns: the
  initial chromosome assignment and the inferred linkage group.  Linkage
  groups are ordered by the number of markers they contain (from largest
  to smallest).

  If \code{reorgMarkers=TRUE}, the output is a cross object, like the
  input, but with the markers reorganized into the inferred linkage
  groups.  The marker order and marker positions within the linkage
  groups are arbitrary.
}

\details{
  Two markers are placed in the same linkage group if the estimated
  recombination fraction between them is \eqn{\le}{<=} \code{max.rf} and
  the LOD score (for the test of the rec. frac. = 1/2) is \eqn{\ge}{>=}
  \code{min.lod}.  The transitive property (if A is linked to B and B is
  linked to C then A is linked to C) is used to close the groups.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
listeria <- est.rf(listeria)
result <- formLinkageGroups(listeria)
tab <- table(result[,1], result[,2])
apply(tab, 1, function(a) sum(a!=0))
apply(tab, 2, function(a) sum(a!=0))
}

\seealso{ \code{\link{est.rf}}, \code{\link{orderMarkers}} }


\keyword{utilities}

\name{mqmplot.multitrait}
\alias{mqmplot.multitrait}

\title{Plot the results from a genomescan using a multiple-QTL model on multiple phenotypes}

\description{
  Plotting routine to display the results from a  multiple-QTL model on
  multiple phenotypes.  It supports four different visualizations: a
  contourmap, heatmap, 3D graph or a multiple QTL plot created by using
  \code{\link{plot.scanone}} on the \code{mqmmulti} object
}

\usage{
mqmplot.multitrait(result, type=c("lines","image","contour","3Dplot"),
                   group=NULL, meanprofile=c("none","mean","median"),
                   theta=30, phi=15, \dots)
}

\arguments{
  \item{result}{ Result object from \code{\link{mqmscanall}} }
  \item{type}{ Selection of the plot method to visualize the data: "lines" (defaut plotting option), "image", "contour" and "3Dplot" }
  \item{group}{ A numeric vector indicating which traits to plot. NULL means no grouping }
  \item{meanprofile}{ Plot a mean/median profile from the group selected }
  \item{theta}{ Horizontal axis rotation in a 3D plot }
  \item{phi}{ Vertical axis rotation in a 3D plot }
  \item{\dots}{Additional arguments passed to \code{plot}.}
}

\author{
Danny Arends \email{danny.arends@gmail.com} % \dannyauthor
}

\seealso{
  \itemize{
    % \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
  }
}

\examples{
data(multitrait)
\dontshow{multitrait <- subset(multitrait, chr=1:2, ind=!apply(multitrait$pheno, 1, function(a) any(is.na(a))))
multitrait$pheno <- multitrait$pheno[,1:3]}
multitrait <- fill.geno(multitrait) # impute missing genotype data
result <- mqmscanall(multitrait, logtransform=TRUE)
mqmplot.multitrait(result,"lines")
mqmplot.multitrait(result,"contour")
mqmplot.multitrait(result,"image")
mqmplot.multitrait(result,"3Dplot")
}

\keyword{ hplot }

\name{checkAlleles}
\alias{checkAlleles}

\title{Identify markers with switched alleles}

\description{
  Identify markers whose alleles might have been switched by comparing
  the LOD score for linkage to all other autosomal markers with the
  original data to that when the alleles have been switched.
}

\usage{checkAlleles(cross, threshold=3, verbose)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{threshold}{Only an increase in maximum 2-point LOD of at least
   this amount will lead to a marker being flagged.}
 \item{verbose}{If TRUE and there are no markers above the threshold,
   print a message.}
}

\value{
  A data frame containing the flagged markers, having four columns: the
  marker name, chromosome ID, numeric index within chromosome, and the
  difference between the maximum two-point LOD score with the alleles
  switched to that from the original data.
}

\details{
  For each marker, we compare the maximum LOD score for the cases where
  the estimated recombination fraction > 0.5 to those where r.f. < 0.5.
  The function \code{\link{est.rf}} must first be run.

  \bold{Note}: Markers that are tightly linked to a marker whose alleles
  are switched are likely to also be flagged by this method.  The real
  problem markers are likely those with the biggest difference in LOD
  scores.
}


\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
\dontshow{fake.f2 <- subset(fake.f2, chr="5")}

# switch homozygotes at marker D5M391
fake.f2 <- switchAlleles(fake.f2, "D5M391")

fake.f2 <- est.rf(fake.f2)
checkAlleles(fake.f2)
}

\seealso{ \code{\link{est.rf}}, \code{\link{geno.crosstab}}, \code{\link{switchAlleles}} }

\keyword{utilities}

\name{addqtl}
\alias{addqtl}

\title{Scan for an additional QTL in a multiple-QTL model}

\description{
  Scan for an additional QTL in the context of a
  multiple QTL model.
}

\usage{
addqtl(cross, chr, pheno.col=1, qtl, covar=NULL, formula,
       method=c("imp","hk"), model=c("normal", "binary"),
       incl.markers=TRUE, verbose=FALSE, tol=1e-4, maxit=1000,
       forceXcovar=FALSE, require.fullrank=FALSE)
}
\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{chr}{Optional vector indicating the chromosomes to be scanned. If
   missing, all chromosomes are scanned. Refer to chromosomes by
   name. Refer to chromosomes with a preceding \code{-} to have all
   chromosomes but those considered.  A logical (TRUE/FALSE) vector may
   also be used.}
 \item{pheno.col}{Column number in the phenotype matrix to be
   used as the phenotype.  One may also give a character string matching
   a phenotype name. Finally, one may give a numeric vector of
   phenotypes, in which case it must have the length equal to the number
   of individuals in the cross, and there must be either non-integers or
   values < 1 or > no. phenotypes; this last case may be useful for studying
   transformations.}
 \item{qtl}{An object of class \code{qtl}, as output from
   \code{\link{makeqtl}}.}
 \item{covar}{A matrix or data.frame of covariates.  These must be
   strictly numeric.}
 \item{formula}{An object of class \code{\link[stats]{formula}}
   indicating the model to be fitted.  (It can also be the character
   string representation of a formula.)  QTLs are referred to as
   \code{Q1}, \code{Q2}, etc.  Covariates are referred to by their names
   in the data frame \code{covar}.  If the new QTL is not included in
   the formula, its main effect is added.}
 \item{method}{Indicates whether to use multiple imputation or
   Haley-Knott regression.}
 \item{model}{The phenotype model: the usual model or a model for binary
  traits}
 \item{incl.markers}{If FALSE, do calculations only at points on an
   evenly spaced grid.  If \code{\link{calc.genoprob}} or
   \code{\link{sim.geno}} were run with
   \code{stepwidth="variable"} or \code{stepwidth="max"}, we force \code{incl.markers=TRUE}.}
 \item{verbose}{If TRUE, display information about the progress of
   calculations.  If \code{verbose} is an integer > 1, further messages
   from \code{\link{scanqtl}} are also displayed.}
 \item{tol}{Tolerance for convergence for the binary trait model.}
 \item{maxit}{Maximum number of iterations for fitting the binary trait
   model.}
 \item{forceXcovar}{If TRUE, force inclusion of X-chr-related covariates
   (like sex and cross direction).}
 \item{require.fullrank}{If TRUE, give LOD=0 when covariate matrix in
   the linear regression is not of full rank.}
}

\details{
  The formula is used to specified the model to be fit. In the
  formula, use \code{Q1}, \code{Q2}, etc., or \code{q1},
  \code{q2}, etc., to represent the QTLs, and the column names in the
  covariate data frame to represent the covariates.

  We enforce a hierarchical structure on the model formula: if a QTL or
  covariate is in involved in an interaction, its main effect must also
  be included.

  If one wishes to scan for QTL that interact with another QTL, include
  it in the formula (with an index of one more than the number of QTL in
  the input \code{qtl} object).
}

\section{Value}{
  An object of class \code{scanone}, as produced by the
  \code{\link{scanone}} function.  LOD scores are relative to the
  base model (with any terms that include the new QTL omitted).
}

\references{
  Haley, C. S. and Knott, S. A. (1992) A simple regression method for mapping
  quantitative trait loci in line crosses using flanking markers.
  \emph{Heredity} \bold{69}, 315--324.

  Sen, \if{latex}{\out{\'S}}\if{html}{\out{&#346;}}\if{text}{S}. and Churchill, G. A. (2001) A statistical framework for quantitative
  trait mapping.  \emph{Genetics} \bold{159}, 371--387.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# take out several QTLs and make QTL object
qc <- c(1, 8, 13)
qp <- c(26, 56, 28)

fake.f2 <- subset(fake.f2, chr=c(1,2,3,8,13))
\dontshow{fake.f2 <- subset(fake.f2, ind=1:50)}

fake.f2 <- calc.genoprob(fake.f2, step=2, err=0.001)
qtl <- makeqtl(fake.f2, qc, qp, what="prob")

# scan for an additional QTL
out1 <- addqtl(fake.f2, qtl=qtl, formula=y~Q1+Q2+Q3, method="hk")
max(out1)

# scan for an additional QTL that interacts with the locus on chr 1
out2 <- addqtl(fake.f2, qtl=qtl, formula=y~Q1*Q4+Q2+Q3, method="hk")
max(out2)

# plot interaction LOD scores
plot(out2-out1)
}

\seealso{ \code{\link{scanone}}, \code{\link{fitqtl}},
  \code{\link{scanqtl}}, \code{\link{refineqtl}},
  \code{\link{makeqtl}}, \code{\link{addtoqtl}},
  \code{\link{addpair}}, \code{\link{addint}} }

\keyword{models}

\name{summary.cross}
\alias{summary.cross}

\title{Print summary of QTL experiment}

\description{
  Print summary information about a \code{cross} object.
}

\usage{
\method{summary}{cross}(object, \dots)
}

\arguments{
 \item{object}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{\dots}{Ignored at this point.}
}

\value{
  An object of class \code{summary.cross}
  containing a variety of summary information about the cross (this is
  generally printed automatically).
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(fake.f2)
summary(fake.f2)
}

\seealso{ \code{\link{read.cross}}, \code{\link{plot.cross}},
  \code{\link{nind}},
  \code{\link{nmar}},
  \code{\link{nchr}},
  \code{\link{totmar}},
  \code{\link{nphe}} }

\keyword{print}

\name{find.marker}
\alias{find.marker}

\title{Find marker closest to a specified position}

\description{
  Find the genetic marker closest to a specified position on a
  chromosome.
}

\usage{find.marker(cross, chr, pos, index)}

\arguments{
 \item{cross}{An object of class \code{cross}.  See
   \code{\link{read.cross}} for details.}
 \item{chr}{A vector of chromosome identifiers, or a single such.}
 \item{pos}{A vector of cM positions.}
 \item{index}{A vector of numeric indices of the markers within
   chromosomes.}
}

\value{
  A vector of marker names (of the same length as the input \code{pos}),
  corresponding to the markers nearest to the specified
  chromosomes/positions (if \code{pos} is specified) or to the input
  numeric indices (in \code{index} is specified).
}

\details{
  Provide one of \code{pos} or \code{index}.

  If the input \code{chr} has length one, it is expanded to the same
  length as the input \code{pos} or \code{index}.

  If \code{pos} is specified and multiple markers are exactly the same
  distance from the specified position, one is chosen at random from
  among those with the most genotype data.

  For a cross with sex-specific maps, positions specified by \code{pos}
  are assumed to correspond to the female genetic map.
}

\author{Karl W Broman, \email{broman@wisc.edu} }

\examples{
data(listeria)
find.marker(listeria, 5, 28)
find.marker(listeria, 5, index=6)
find.marker(listeria, c(1, 5, 13), c(81, 28, 26))
}

\seealso{ \code{\link{find.flanking}}, \code{\link{plotPXG}},
  \code{\link{find.pseudomarker}}, \code{\link{effectplot}},
\code{\link{find.markerpos}} }

\keyword{utilities}

\name{plot.scantwoperm}
\alias{plot.scantwoperm}

\title{Plot permutation results for a 2d, 2-QTL genome scan}

\description{
  Plot a histogram of the permutation results from a two-dimensional,
  two-QTL genome scan.
}

\usage{
\method{plot}{scantwoperm}(x, lodcolumn=1, include_rug=TRUE, \dots)
}

\arguments{
  \item{x}{An object of class \code{"scantwoperm"}, as output by
    \code{\link{scantwo}} when \code{n.perm} is specified.}
  \item{lodcolumn}{This indicates the LOD score column to plot.  This
    should be a single number between 1 and the number of LOD columns in
    the object input.}
  \item{include_rug}{If TRUE, include a call to \code{\link[graphics]{rug}}.}
  \item{\dots}{Passed to the function \code{\link[graphics]{hist}} when it
    is called.}
}
\value{None.}

\details{
  The function plots a histogram of the permutation results obtained by
  \code{\link{scantwo}} when \code{n.perm} is specified. Separate
  histograms are provided for the five LOD scores, \code{full},
  \code{fv1}, \code{int}, \code{add}, and \code{av1}.
}

\examples{
data(fake.bc)
fake.bc <- calc.genoprob(fake.bc)

operm2 <- scantwo(fake.bc, method="hk", n.perm=10)
plot(operm2)
}

\seealso{ \code{\link{scantwo}},
  \code{\link{summary.scantwoperm}} }

\author{Karl W Broman, \email{broman@wisc.edu} }

\keyword{hplot}

\name{mqmpermutation}
\alias{mqmpermutation}

\title{Estimate QTL LOD score significance using permutations or simulations }

\description{
   Two randomization approaches to obtain estimates of QTL significance:
   \itemize{
   \item Random redistribution of traits (method='permutation')
   \item Random redistribution of simulated trait values (method='simulation')
   }
   Calculations can be parallelized using the SNOW package.
}

\usage{
mqmpermutation(cross, scanfunction=scanone, pheno.col=1, multicore=TRUE,
               n.perm=10, file="MQM_output.txt",
               n.cluster=1, method=c("permutation","simulation"),
               cofactors=NULL, plot=FALSE, verbose=FALSE, \dots)
}

\arguments{
        \item{cross}{
An object of class \code{cross}. See \code{\link{read.cross}} for details. % \crossobject
  }
        \item{scanfunction}{ Function to use when mappingQTL's (either scanone,cim or mqm)}
        \item{pheno.col}{
Column number in the phenotype matrix which should be used as the phenotype. This can be a vector of integers. % \phenocol
  }
        \item{multicore}{ Use multicore (if available)}
        \item{n.perm}{ Number of permutations to perform (DEFAULT=10, should be 1000, or higher,
  for publications) }
        \item{file}{ Name of the intermediate output file used }
        \item{n.cluster}{ Number of child processes to split the job into }
        \item{method}{ What kind permutation should occur: permutation or simulation }
  \item{cofactors}{cofactors, only used when scanfunction is mqm.
List of cofactors to be analysed in the QTL model. To set cofactors use \code{\link{mqmautocofactors}} or \code{mqmsetcofactors}}. % \mqmcofactors
        \item{plot}{If TRUE, make a plot}
        \item{verbose}{If TRUE, print tracing information}
        \item{\dots}{Parameters passed through to the
          \code{\link{scanone}}, \code{\link{cim}} or
            \code{\link{mqmscan}} functions}
}

\details{
  Analysis of \code{\link{scanone}}, \code{\link{cim}} or
            \code{\link{mqmscan}} to scan for QTL in shuffled/randomized data. It is recommended to also install the \code{snow} library.
  The \code{snow} library allows calculations to run on multiple cores or even scale it up to an entire cluster, thus speeding up calculation.
}

\value{
        Returns a mqmmulti object. this object is a list of scanone objects that can be plotted using \code{plot.scanone(result[[trait]])}
}

\author{
Ritsert C Jansen; Danny Arends; Pjotr Prins; Karl W Broman \email{broman@wisc.edu} % \mqmauthors
}

\seealso{
        \itemize{
% \input{"inst/doc/Sources/MQM/mqm/standard_seealso.txt"}
    \item The MQM tutorial: \url{https://rqtl.org/tutorials/MQM-tour.pdf}
    \item \code{\link{MQM}} - MQM description and references
    \item \code{\link{mqmscan}} - Main MQM single trait analysis
    \item \code{\link{mqmscanall}} - Parallellized traits analysis
    \item \code{\link{mqmaugment}} - Augmentation routine for estimating missing data
    \item \code{\link{mqmautocofactors}} - Set cofactors using marker density
    \item \code{\link{mqmsetcofactors}} - Set cofactors at fixed locations
    \item \code{\link{mqmpermutation}} - Estimate significance levels
    \item \code{\link{scanone}} - Single QTL scanning
% -----^^ inst/doc/Sources/MQM/mqm/standard_seealso.txt ^^-----
        }
}

\examples{
# Use the multitrait dataset
data(multitrait)

\dontshow{multitrait <- subset(multitrait, chr=1:2,
                     ind=!apply(multitrait$pheno, 1, function(a) any(is.na(a))))
multitrait$pheno <- log(multitrait$pheno)}

multitrait <- calc.genoprob(multitrait)
result <- mqmpermutation(multitrait,pheno.col=7, n.perm=2, batchsize=2)

\dontrun{#Set 50 cofactors
cof <- mqmautocofactors(multitrait,50)
}\dontshow{cof <- mqmautocofactors(multitrait,5)}

multitrait <- fill.geno(multitrait)
result <- mqmpermutation(multitrait,scanfunction=mqmscan,cofactors=cof,
                         pheno.col=7, n.perm=2,batchsize=2,verbose=FALSE)

#Create a permutation object
f2perm <- mqmprocesspermutation(result)

#Get Significant LOD thresholds
summary(f2perm)
}

\references{
\itemize{
% \input{"inst/doc/Sources/MQM/mqm/significance_references.txt"}
    \item Bruno M. Tesson, Ritsert C. Jansen (2009) Chapter 3.7. Determining the significance threshold \emph{eQTL Analysis in Mice and Rats} \bold{1}, 20--25
                \item Churchill, G. A. and Doerge, R. W. (1994) Empirical threshold values for quantitative trait mapping. \emph{Genetics} \bold{138}, 963--971.
% -----^^ inst/doc/Sources/MQM/mqm/significance_references.txt ^^-----
% \input{"inst/doc/Sources/MQM/mqm/parallelisation_references.txt"}
                \item Rossini, A., Tierney, L., and Li, N. (2003), Simple parallel statistical computing.  \emph{R. UW Biostatistics working paper series} University of Washington. \bold{193}
                \item Tierney, L., Rossini, A., Li, N., and Sevcikova, H. (2004), The snow Package: Simple Network of Workstations. Version 0.2-1.
% -----^^ inst/doc/Sources/MQM/mqm/parallelisation_references.txt ^^-----
}
}

\keyword{ models }

\name{addmarker}
\alias{addmarker}

\title{Add a marker to a cross}

\description{
  Add a marker to a cross object.
}

\usage{
addmarker(cross, genotypes, markername, chr, pos)
}

\arguments{
 \item{cross}{An object of class \code{cross}. See
   \code{\link{read.cross}} for details.}
 \item{genotypes}{Vector of numeric genotypes.}
 \item{markername}{Marker name as character string.}
 \item{chr}{Chromosome ID as character string.}
 \item{pos}{Position of marker, as numeric value.}
}

\details{
  Use this function with caution.  It would be best to incorporate new
  data into a single file to be imported with \code{\link{read.cross}}.

  But if you have genotypes on one or two additional markers that you
  want to add, you might load them with \code{\link[utils]{read.csv}}
  and incorporate them with this function.
}

\value{
  The input \code{cross} object with the single marker added.
}

\author{Karl W Broman, \email{broman@wisc.edu}}

\examples{
data(fake.f2)

# genotypes for new marker
gi <- pull.geno(fill.geno(fake.f2))[,"D5M197"]

# add marker to cross
fake.f2 <- addmarker(fake.f2, gi, "D5M197imp", "5", 11)
}

\seealso{ \code{\link{pull.markers}}, \code{\link{drop.markers}} }

\keyword{utilities}
